/*******************************************************************************
* Copyright (C) 2016-2020 Advanced Micro Devices, Inc. All rights reserved.
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell cop-
* ies of the Software, and to permit persons to whom the Software is furnished
* to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IM-
* PLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
* FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
* COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
* IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNE-
* CTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*******************************************************************************/

/**************************************************
* This file was generated by Tensile:             *
* https://github.com/ROCmSoftwarePlatform/Tensile *
**************************************************/


#include "Kernels.h"


  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 819

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? static_cast<tensile_bfloat16>((ALPHA)*(REG) + (BETA) * static_cast<float>(SRC)) : static_cast<tensile_bfloat16>((ALPHA)*(REG));

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_BBH_MT64x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_bfloat16 *D,
  tensile_bfloat16 const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 7 */

    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? static_cast<tensile_bfloat16>((ALPHA)*(REG) + (BETA) * static_cast<float>(SRC)) : static_cast<tensile_bfloat16>((ALPHA)*(REG));

/* 2x8 micro-tile */
#define MAC_2x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \

#define MAC_2x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_BBH_MT32x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_8_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_bfloat16 *D,
  tensile_bfloat16 const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_1_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_1_3_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_2_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_3_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_2_1;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_3_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_1);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_1 + 0) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_1 + 0) = b_0_1_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x8


    /* iter 15 */

    MAC_2x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_1_2_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 1);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_1_3_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_1);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_2_1 + 0) = b_0_1_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_3_1 + 0) = b_0_1_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x8
#undef MAC_2x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 8

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? static_cast<tensile_bfloat16>((ALPHA)*(REG) + (BETA) * static_cast<float>(SRC)) : static_cast<tensile_bfloat16>((ALPHA)*(REG));

/* 2x8 micro-tile */
#define MAC_2x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \

#define MAC_2x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_BBH_MT16x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_8_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  tensile_bfloat16 *D,
  tensile_bfloat16 const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_1_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_1_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_1_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_1_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_1_6_0;
  DATA_TYPE b_0_0_7_0;
  DATA_TYPE b_0_1_7_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;
  unsigned int flattenedOffsetB_4_0 = globalReadOffsetB1J + 0 + 4*LSPB;
  flattenedOffsetB_4_0 = (flattenedOffsetB_4_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_4_0;
  unsigned int globalReadOffsetB1J_4_0 = flattenedOffsetB_4_0;
  unsigned int flattenedOffsetB_5_0 = globalReadOffsetB1J + 0 + 5*LSPB;
  flattenedOffsetB_5_0 = (flattenedOffsetB_5_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_5_0;
  unsigned int globalReadOffsetB1J_5_0 = flattenedOffsetB_5_0;
  unsigned int flattenedOffsetB_6_0 = globalReadOffsetB1J + 0 + 6*LSPB;
  flattenedOffsetB_6_0 = (flattenedOffsetB_6_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_6_0;
  unsigned int globalReadOffsetB1J_6_0 = flattenedOffsetB_6_0;
  unsigned int flattenedOffsetB_7_0 = globalReadOffsetB1J + 0 + 7*LSPB;
  flattenedOffsetB_7_0 = (flattenedOffsetB_7_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_7_0;
  unsigned int globalReadOffsetB1J_7_0 = flattenedOffsetB_7_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;
  globalReadOffsetB1J_4_0 = (  globalReadOffsetB1J_4_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_4_0;
  globalReadOffsetB1J_5_0 = (  globalReadOffsetB1J_5_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_5_0;
  globalReadOffsetB1J_6_0 = (  globalReadOffsetB1J_6_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_6_0;
  globalReadOffsetB1J_7_0 = (  globalReadOffsetB1J_7_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_7_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_7_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_2_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_3_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 4*LSPB);
  unsigned int localWriteOffsetB_0_0_4_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 4*LSPB);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 5*LSPB);
  unsigned int localWriteOffsetB_0_0_5_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 5*LSPB);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 6*LSPB);
  unsigned int localWriteOffsetB_0_0_6_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 6*LSPB);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 7*LSPB);
  unsigned int localWriteOffsetB_0_0_7_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 7*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_2_1;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_3_1;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_4_1;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_5_1;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_6_1;
  DATA_TYPE *localWriteB_0_0_7_0;
  DATA_TYPE *localWriteB_0_0_7_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_1);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_1);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_4_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_1);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_5_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_1);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_6_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_1);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);
  localWriteB_0_0_7_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_1_4_0 = *(globalReadB_0_0_4_0 + 1);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_1_5_0 = *(globalReadB_0_0_5_0 + 1);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_1_6_0 = *(globalReadB_0_0_6_0 + 1);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);
    b_0_1_7_0 = *(globalReadB_0_0_7_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_1 + 0) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_1 + 0) = b_0_1_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_4_1 + 0) = b_0_1_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_5_1 + 0) = b_0_1_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_6_1 + 0) = b_0_1_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
    *(localWriteB_0_0_7_1 + 0) = b_0_1_7_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x8


    /* iter 15 */

    MAC_2x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_1_2_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 1);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_1_3_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 1);
  b_0_0_4_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_1_4_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 1);
  b_0_0_5_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_1_5_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 1);
  b_0_0_6_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_1_6_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 1);
  b_0_0_7_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);
  b_0_1_7_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_1);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_1);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_4_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_1);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_5_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_1);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_6_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_1);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);
  localWriteB_0_0_7_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_2_1 + 0) = b_0_1_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_3_1 + 0) = b_0_1_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_4_1 + 0) = b_0_1_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_5_1 + 0) = b_0_1_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_6_1 + 0) = b_0_1_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
  *(localWriteB_0_0_7_1 + 0) = b_0_1_7_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x8
#undef MAC_2x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? static_cast<tensile_bfloat16>((ALPHA)*(REG) + (BETA) * static_cast<float>(SRC)) : static_cast<tensile_bfloat16>((ALPHA)*(REG));

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_BBH_MT128x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_bfloat16 *D,
  tensile_bfloat16 const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;
  unsigned int flattenedOffsetA_4_0 = globalReadOffsetA0I + 0 + 4*LSPA;
  flattenedOffsetA_4_0 = (flattenedOffsetA_4_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_4_0;
  unsigned int globalReadOffsetA0I_4_0 = flattenedOffsetA_4_0;
  unsigned int flattenedOffsetA_5_0 = globalReadOffsetA0I + 0 + 5*LSPA;
  flattenedOffsetA_5_0 = (flattenedOffsetA_5_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_5_0;
  unsigned int globalReadOffsetA0I_5_0 = flattenedOffsetA_5_0;
  unsigned int flattenedOffsetA_6_0 = globalReadOffsetA0I + 0 + 6*LSPA;
  flattenedOffsetA_6_0 = (flattenedOffsetA_6_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_6_0;
  unsigned int globalReadOffsetA0I_6_0 = flattenedOffsetA_6_0;
  unsigned int flattenedOffsetA_7_0 = globalReadOffsetA0I + 0 + 7*LSPA;
  flattenedOffsetA_7_0 = (flattenedOffsetA_7_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_7_0;
  unsigned int globalReadOffsetA0I_7_0 = flattenedOffsetA_7_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;
  globalReadOffsetA0I_4_0 = (  globalReadOffsetA0I_4_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_4_0;
  globalReadOffsetA0I_5_0 = (  globalReadOffsetA0I_5_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_5_0;
  globalReadOffsetA0I_6_0 = (  globalReadOffsetA0I_6_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_6_0;
  globalReadOffsetA0I_7_0 = (  globalReadOffsetA0I_7_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_7_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 4*LSPA);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 5*LSPA);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 6*LSPA);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 7*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 8

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? static_cast<tensile_bfloat16>((ALPHA)*(REG) + (BETA) * static_cast<float>(SRC)) : static_cast<tensile_bfloat16>((ALPHA)*(REG));

/* 2x8 micro-tile */
#define MAC_2x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \

#define MAC_2x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_BBH_MT32x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_8_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_bfloat16 *D,
  tensile_bfloat16 const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;
  unsigned int flattenedOffsetB_4_0 = globalReadOffsetB1J + 0 + 4*LSPB;
  flattenedOffsetB_4_0 = (flattenedOffsetB_4_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_4_0;
  unsigned int globalReadOffsetB1J_4_0 = flattenedOffsetB_4_0;
  unsigned int flattenedOffsetB_5_0 = globalReadOffsetB1J + 0 + 5*LSPB;
  flattenedOffsetB_5_0 = (flattenedOffsetB_5_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_5_0;
  unsigned int globalReadOffsetB1J_5_0 = flattenedOffsetB_5_0;
  unsigned int flattenedOffsetB_6_0 = globalReadOffsetB1J + 0 + 6*LSPB;
  flattenedOffsetB_6_0 = (flattenedOffsetB_6_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_6_0;
  unsigned int globalReadOffsetB1J_6_0 = flattenedOffsetB_6_0;
  unsigned int flattenedOffsetB_7_0 = globalReadOffsetB1J + 0 + 7*LSPB;
  flattenedOffsetB_7_0 = (flattenedOffsetB_7_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_7_0;
  unsigned int globalReadOffsetB1J_7_0 = flattenedOffsetB_7_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;
  globalReadOffsetB1J_4_0 = (  globalReadOffsetB1J_4_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_4_0;
  globalReadOffsetB1J_5_0 = (  globalReadOffsetB1J_5_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_5_0;
  globalReadOffsetB1J_6_0 = (  globalReadOffsetB1J_6_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_6_0;
  globalReadOffsetB1J_7_0 = (  globalReadOffsetB1J_7_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_7_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_7_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 4*LSPB);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 5*LSPB);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 6*LSPB);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 7*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x8


    /* iter 15 */

    MAC_2x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x8
#undef MAC_2x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 16
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? static_cast<tensile_bfloat16>((ALPHA)*(REG) + (BETA) * static_cast<float>(SRC)) : static_cast<tensile_bfloat16>((ALPHA)*(REG));

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_BBH_MT64x16x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  tensile_bfloat16 *D,
  tensile_bfloat16 const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;
  unsigned int flattenedOffsetA_4_0 = globalReadOffsetA0I + 0 + 4*LSPA;
  flattenedOffsetA_4_0 = (flattenedOffsetA_4_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_4_0;
  unsigned int globalReadOffsetA0I_4_0 = flattenedOffsetA_4_0;
  unsigned int flattenedOffsetA_5_0 = globalReadOffsetA0I + 0 + 5*LSPA;
  flattenedOffsetA_5_0 = (flattenedOffsetA_5_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_5_0;
  unsigned int globalReadOffsetA0I_5_0 = flattenedOffsetA_5_0;
  unsigned int flattenedOffsetA_6_0 = globalReadOffsetA0I + 0 + 6*LSPA;
  flattenedOffsetA_6_0 = (flattenedOffsetA_6_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_6_0;
  unsigned int globalReadOffsetA0I_6_0 = flattenedOffsetA_6_0;
  unsigned int flattenedOffsetA_7_0 = globalReadOffsetA0I + 0 + 7*LSPA;
  flattenedOffsetA_7_0 = (flattenedOffsetA_7_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_7_0;
  unsigned int globalReadOffsetA0I_7_0 = flattenedOffsetA_7_0;
  unsigned int flattenedOffsetA_8_0 = globalReadOffsetA0I + 0 + 8*LSPA;
  flattenedOffsetA_8_0 = (flattenedOffsetA_8_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_8_0;
  unsigned int globalReadOffsetA0I_8_0 = flattenedOffsetA_8_0;
  unsigned int flattenedOffsetA_9_0 = globalReadOffsetA0I + 0 + 9*LSPA;
  flattenedOffsetA_9_0 = (flattenedOffsetA_9_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_9_0;
  unsigned int globalReadOffsetA0I_9_0 = flattenedOffsetA_9_0;
  unsigned int flattenedOffsetA_10_0 = globalReadOffsetA0I + 0 + 10*LSPA;
  flattenedOffsetA_10_0 = (flattenedOffsetA_10_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_10_0;
  unsigned int globalReadOffsetA0I_10_0 = flattenedOffsetA_10_0;
  unsigned int flattenedOffsetA_11_0 = globalReadOffsetA0I + 0 + 11*LSPA;
  flattenedOffsetA_11_0 = (flattenedOffsetA_11_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_11_0;
  unsigned int globalReadOffsetA0I_11_0 = flattenedOffsetA_11_0;
  unsigned int flattenedOffsetA_12_0 = globalReadOffsetA0I + 0 + 12*LSPA;
  flattenedOffsetA_12_0 = (flattenedOffsetA_12_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_12_0;
  unsigned int globalReadOffsetA0I_12_0 = flattenedOffsetA_12_0;
  unsigned int flattenedOffsetA_13_0 = globalReadOffsetA0I + 0 + 13*LSPA;
  flattenedOffsetA_13_0 = (flattenedOffsetA_13_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_13_0;
  unsigned int globalReadOffsetA0I_13_0 = flattenedOffsetA_13_0;
  unsigned int flattenedOffsetA_14_0 = globalReadOffsetA0I + 0 + 14*LSPA;
  flattenedOffsetA_14_0 = (flattenedOffsetA_14_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_14_0;
  unsigned int globalReadOffsetA0I_14_0 = flattenedOffsetA_14_0;
  unsigned int flattenedOffsetA_15_0 = globalReadOffsetA0I + 0 + 15*LSPA;
  flattenedOffsetA_15_0 = (flattenedOffsetA_15_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_15_0;
  unsigned int globalReadOffsetA0I_15_0 = flattenedOffsetA_15_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;
  globalReadOffsetA0I_4_0 = (  globalReadOffsetA0I_4_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_4_0;
  globalReadOffsetA0I_5_0 = (  globalReadOffsetA0I_5_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_5_0;
  globalReadOffsetA0I_6_0 = (  globalReadOffsetA0I_6_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_6_0;
  globalReadOffsetA0I_7_0 = (  globalReadOffsetA0I_7_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_7_0;
  globalReadOffsetA0I_8_0 = (  globalReadOffsetA0I_8_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_8_0;
  globalReadOffsetA0I_9_0 = (  globalReadOffsetA0I_9_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_9_0;
  globalReadOffsetA0I_10_0 = (  globalReadOffsetA0I_10_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_10_0;
  globalReadOffsetA0I_11_0 = (  globalReadOffsetA0I_11_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_11_0;
  globalReadOffsetA0I_12_0 = (  globalReadOffsetA0I_12_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_12_0;
  globalReadOffsetA0I_13_0 = (  globalReadOffsetA0I_13_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_13_0;
  globalReadOffsetA0I_14_0 = (  globalReadOffsetA0I_14_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_14_0;
  globalReadOffsetA0I_15_0 = (  globalReadOffsetA0I_15_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_15_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_15_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 4*LSPA);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 5*LSPA);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 6*LSPA);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 7*LSPA);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 8*LSPA);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 9*LSPA);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 10*LSPA);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 11*LSPA);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 12*LSPA);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 13*LSPA);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 14*LSPA);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 15*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_0_0_15_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_8_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_9_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_10_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_11_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_12_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_13_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_14_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_15_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_8_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_9_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_10_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_11_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_12_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_13_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_14_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_15_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_0_8_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_0_0_9_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_0_0_10_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_0_0_11_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_0_0_12_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_0_0_13_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_0_0_14_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_0_0_15_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 16

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? static_cast<tensile_bfloat16>((ALPHA)*(REG) + (BETA) * static_cast<float>(SRC)) : static_cast<tensile_bfloat16>((ALPHA)*(REG));

/* 2x8 micro-tile */
#define MAC_2x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \

#define MAC_2x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_BBH_MT16x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_8_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  tensile_bfloat16 *D,
  tensile_bfloat16 const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;
  DATA_TYPE b_0_0_8_0;
  DATA_TYPE b_0_0_9_0;
  DATA_TYPE b_0_0_10_0;
  DATA_TYPE b_0_0_11_0;
  DATA_TYPE b_0_0_12_0;
  DATA_TYPE b_0_0_13_0;
  DATA_TYPE b_0_0_14_0;
  DATA_TYPE b_0_0_15_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;
  unsigned int flattenedOffsetB_4_0 = globalReadOffsetB1J + 0 + 4*LSPB;
  flattenedOffsetB_4_0 = (flattenedOffsetB_4_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_4_0;
  unsigned int globalReadOffsetB1J_4_0 = flattenedOffsetB_4_0;
  unsigned int flattenedOffsetB_5_0 = globalReadOffsetB1J + 0 + 5*LSPB;
  flattenedOffsetB_5_0 = (flattenedOffsetB_5_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_5_0;
  unsigned int globalReadOffsetB1J_5_0 = flattenedOffsetB_5_0;
  unsigned int flattenedOffsetB_6_0 = globalReadOffsetB1J + 0 + 6*LSPB;
  flattenedOffsetB_6_0 = (flattenedOffsetB_6_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_6_0;
  unsigned int globalReadOffsetB1J_6_0 = flattenedOffsetB_6_0;
  unsigned int flattenedOffsetB_7_0 = globalReadOffsetB1J + 0 + 7*LSPB;
  flattenedOffsetB_7_0 = (flattenedOffsetB_7_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_7_0;
  unsigned int globalReadOffsetB1J_7_0 = flattenedOffsetB_7_0;
  unsigned int flattenedOffsetB_8_0 = globalReadOffsetB1J + 0 + 8*LSPB;
  flattenedOffsetB_8_0 = (flattenedOffsetB_8_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_8_0;
  unsigned int globalReadOffsetB1J_8_0 = flattenedOffsetB_8_0;
  unsigned int flattenedOffsetB_9_0 = globalReadOffsetB1J + 0 + 9*LSPB;
  flattenedOffsetB_9_0 = (flattenedOffsetB_9_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_9_0;
  unsigned int globalReadOffsetB1J_9_0 = flattenedOffsetB_9_0;
  unsigned int flattenedOffsetB_10_0 = globalReadOffsetB1J + 0 + 10*LSPB;
  flattenedOffsetB_10_0 = (flattenedOffsetB_10_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_10_0;
  unsigned int globalReadOffsetB1J_10_0 = flattenedOffsetB_10_0;
  unsigned int flattenedOffsetB_11_0 = globalReadOffsetB1J + 0 + 11*LSPB;
  flattenedOffsetB_11_0 = (flattenedOffsetB_11_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_11_0;
  unsigned int globalReadOffsetB1J_11_0 = flattenedOffsetB_11_0;
  unsigned int flattenedOffsetB_12_0 = globalReadOffsetB1J + 0 + 12*LSPB;
  flattenedOffsetB_12_0 = (flattenedOffsetB_12_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_12_0;
  unsigned int globalReadOffsetB1J_12_0 = flattenedOffsetB_12_0;
  unsigned int flattenedOffsetB_13_0 = globalReadOffsetB1J + 0 + 13*LSPB;
  flattenedOffsetB_13_0 = (flattenedOffsetB_13_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_13_0;
  unsigned int globalReadOffsetB1J_13_0 = flattenedOffsetB_13_0;
  unsigned int flattenedOffsetB_14_0 = globalReadOffsetB1J + 0 + 14*LSPB;
  flattenedOffsetB_14_0 = (flattenedOffsetB_14_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_14_0;
  unsigned int globalReadOffsetB1J_14_0 = flattenedOffsetB_14_0;
  unsigned int flattenedOffsetB_15_0 = globalReadOffsetB1J + 0 + 15*LSPB;
  flattenedOffsetB_15_0 = (flattenedOffsetB_15_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_15_0;
  unsigned int globalReadOffsetB1J_15_0 = flattenedOffsetB_15_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;
  globalReadOffsetB1J_4_0 = (  globalReadOffsetB1J_4_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_4_0;
  globalReadOffsetB1J_5_0 = (  globalReadOffsetB1J_5_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_5_0;
  globalReadOffsetB1J_6_0 = (  globalReadOffsetB1J_6_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_6_0;
  globalReadOffsetB1J_7_0 = (  globalReadOffsetB1J_7_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_7_0;
  globalReadOffsetB1J_8_0 = (  globalReadOffsetB1J_8_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_8_0;
  globalReadOffsetB1J_9_0 = (  globalReadOffsetB1J_9_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_9_0;
  globalReadOffsetB1J_10_0 = (  globalReadOffsetB1J_10_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_10_0;
  globalReadOffsetB1J_11_0 = (  globalReadOffsetB1J_11_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_11_0;
  globalReadOffsetB1J_12_0 = (  globalReadOffsetB1J_12_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_12_0;
  globalReadOffsetB1J_13_0 = (  globalReadOffsetB1J_13_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_13_0;
  globalReadOffsetB1J_14_0 = (  globalReadOffsetB1J_14_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_14_0;
  globalReadOffsetB1J_15_0 = (  globalReadOffsetB1J_15_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_15_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_7_0), wgK );
  int64_t globalReadOffsetB_0_0_8_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_8_0), wgK );
  int64_t globalReadOffsetB_0_0_9_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_9_0), wgK );
  int64_t globalReadOffsetB_0_0_10_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_10_0), wgK );
  int64_t globalReadOffsetB_0_0_11_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_11_0), wgK );
  int64_t globalReadOffsetB_0_0_12_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_12_0), wgK );
  int64_t globalReadOffsetB_0_0_13_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_13_0), wgK );
  int64_t globalReadOffsetB_0_0_14_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_14_0), wgK );
  int64_t globalReadOffsetB_0_0_15_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_15_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;
  DATA_TYPE const *globalReadB_0_0_8_0 = B + globalReadOffsetB_0_0_8_0;
  DATA_TYPE const *globalReadB_0_0_9_0 = B + globalReadOffsetB_0_0_9_0;
  DATA_TYPE const *globalReadB_0_0_10_0 = B + globalReadOffsetB_0_0_10_0;
  DATA_TYPE const *globalReadB_0_0_11_0 = B + globalReadOffsetB_0_0_11_0;
  DATA_TYPE const *globalReadB_0_0_12_0 = B + globalReadOffsetB_0_0_12_0;
  DATA_TYPE const *globalReadB_0_0_13_0 = B + globalReadOffsetB_0_0_13_0;
  DATA_TYPE const *globalReadB_0_0_14_0 = B + globalReadOffsetB_0_0_14_0;
  DATA_TYPE const *globalReadB_0_0_15_0 = B + globalReadOffsetB_0_0_15_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 4*LSPB);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 5*LSPB);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 6*LSPB);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 7*LSPB);
  unsigned int localWriteOffsetB_0_0_8_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 8*LSPB);
  unsigned int localWriteOffsetB_0_0_9_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 9*LSPB);
  unsigned int localWriteOffsetB_0_0_10_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 10*LSPB);
  unsigned int localWriteOffsetB_0_0_11_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 11*LSPB);
  unsigned int localWriteOffsetB_0_0_12_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 12*LSPB);
  unsigned int localWriteOffsetB_0_0_13_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 13*LSPB);
  unsigned int localWriteOffsetB_0_0_14_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 14*LSPB);
  unsigned int localWriteOffsetB_0_0_15_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 15*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;
  DATA_TYPE *localWriteB_0_0_8_0;
  DATA_TYPE *localWriteB_0_0_9_0;
  DATA_TYPE *localWriteB_0_0_10_0;
  DATA_TYPE *localWriteB_0_0_11_0;
  DATA_TYPE *localWriteB_0_0_12_0;
  DATA_TYPE *localWriteB_0_0_13_0;
  DATA_TYPE *localWriteB_0_0_14_0;
  DATA_TYPE *localWriteB_0_0_15_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);
  localWriteB_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_8_0);
  localWriteB_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_9_0);
  localWriteB_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_10_0);
  localWriteB_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_11_0);
  localWriteB_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_12_0);
  localWriteB_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_13_0);
  localWriteB_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_14_0);
  localWriteB_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_15_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_8_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_9_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_10_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_11_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_12_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_13_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_14_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_15_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);
    b_0_0_8_0 = *(globalReadB_0_0_8_0 + 0);
    b_0_0_9_0 = *(globalReadB_0_0_9_0 + 0);
    b_0_0_10_0 = *(globalReadB_0_0_10_0 + 0);
    b_0_0_11_0 = *(globalReadB_0_0_11_0 + 0);
    b_0_0_12_0 = *(globalReadB_0_0_12_0 + 0);
    b_0_0_13_0 = *(globalReadB_0_0_13_0 + 0);
    b_0_0_14_0 = *(globalReadB_0_0_14_0 + 0);
    b_0_0_15_0 = *(globalReadB_0_0_15_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_8_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_8_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_9_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_9_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_10_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_10_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_11_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_11_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_12_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_12_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_13_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_13_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_14_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_14_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_15_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_15_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
    *(localWriteB_0_0_8_0 + 0) = b_0_0_8_0;
    *(localWriteB_0_0_9_0 + 0) = b_0_0_9_0;
    *(localWriteB_0_0_10_0 + 0) = b_0_0_10_0;
    *(localWriteB_0_0_11_0 + 0) = b_0_0_11_0;
    *(localWriteB_0_0_12_0 + 0) = b_0_0_12_0;
    *(localWriteB_0_0_13_0 + 0) = b_0_0_13_0;
    *(localWriteB_0_0_14_0 + 0) = b_0_0_14_0;
    *(localWriteB_0_0_15_0 + 0) = b_0_0_15_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x8


    /* iter 15 */

    MAC_2x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);
  b_0_0_8_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_8_0 + 0);
  b_0_0_9_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_9_0 + 0);
  b_0_0_10_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_10_0 + 0);
  b_0_0_11_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_11_0 + 0);
  b_0_0_12_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_12_0 + 0);
  b_0_0_13_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_13_0 + 0);
  b_0_0_14_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_14_0 + 0);
  b_0_0_15_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_15_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);
  localWriteB_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_8_0);
  localWriteB_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_9_0);
  localWriteB_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_10_0);
  localWriteB_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_11_0);
  localWriteB_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_12_0);
  localWriteB_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_13_0);
  localWriteB_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_14_0);
  localWriteB_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_15_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
  *(localWriteB_0_0_8_0 + 0) = b_0_0_8_0;
  *(localWriteB_0_0_9_0 + 0) = b_0_0_9_0;
  *(localWriteB_0_0_10_0 + 0) = b_0_0_10_0;
  *(localWriteB_0_0_11_0 + 0) = b_0_0_11_0;
  *(localWriteB_0_0_12_0 + 0) = b_0_0_12_0;
  *(localWriteB_0_0_13_0 + 0) = b_0_0_13_0;
  *(localWriteB_0_0_14_0 + 0) = b_0_0_14_0;
  *(localWriteB_0_0_15_0 + 0) = b_0_0_15_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x8
#undef MAC_2x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 819

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? static_cast<tensile_bfloat16>((ALPHA)*(REG) + (BETA) * static_cast<float>(SRC)) : static_cast<tensile_bfloat16>((ALPHA)*(REG));

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_BBH_MT64x32x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_bfloat16 *D,
  tensile_bfloat16 const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 7 */

    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? static_cast<tensile_bfloat16>((ALPHA)*(REG) + (BETA) * static_cast<float>(SRC)) : static_cast<tensile_bfloat16>((ALPHA)*(REG));

/* 2x8 micro-tile */
#define MAC_2x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \

#define MAC_2x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_BBH_MT32x128x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_8_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_bfloat16 *D,
  tensile_bfloat16 const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_1_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_1_3_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_2_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_3_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_2_1;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_3_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_1);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_1 + 0) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_1 + 0) = b_0_1_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x8


    /* iter 15 */

    MAC_2x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_1_2_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 1);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_1_3_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_1);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_2_1 + 0) = b_0_1_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_3_1 + 0) = b_0_1_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x8
#undef MAC_2x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 8

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? static_cast<tensile_bfloat16>((ALPHA)*(REG) + (BETA) * static_cast<float>(SRC)) : static_cast<tensile_bfloat16>((ALPHA)*(REG));

/* 2x8 micro-tile */
#define MAC_2x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \

#define MAC_2x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_BBH_MT16x64x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_8_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  tensile_bfloat16 *D,
  tensile_bfloat16 const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_1_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_1_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_1_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_1_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_1_6_0;
  DATA_TYPE b_0_0_7_0;
  DATA_TYPE b_0_1_7_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;
  unsigned int flattenedOffsetB_4_0 = globalReadOffsetB1J + 0 + 4*LSPB;
  flattenedOffsetB_4_0 = (flattenedOffsetB_4_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_4_0;
  unsigned int globalReadOffsetB1J_4_0 = flattenedOffsetB_4_0;
  unsigned int flattenedOffsetB_5_0 = globalReadOffsetB1J + 0 + 5*LSPB;
  flattenedOffsetB_5_0 = (flattenedOffsetB_5_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_5_0;
  unsigned int globalReadOffsetB1J_5_0 = flattenedOffsetB_5_0;
  unsigned int flattenedOffsetB_6_0 = globalReadOffsetB1J + 0 + 6*LSPB;
  flattenedOffsetB_6_0 = (flattenedOffsetB_6_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_6_0;
  unsigned int globalReadOffsetB1J_6_0 = flattenedOffsetB_6_0;
  unsigned int flattenedOffsetB_7_0 = globalReadOffsetB1J + 0 + 7*LSPB;
  flattenedOffsetB_7_0 = (flattenedOffsetB_7_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_7_0;
  unsigned int globalReadOffsetB1J_7_0 = flattenedOffsetB_7_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;
  globalReadOffsetB1J_4_0 = (  globalReadOffsetB1J_4_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_4_0;
  globalReadOffsetB1J_5_0 = (  globalReadOffsetB1J_5_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_5_0;
  globalReadOffsetB1J_6_0 = (  globalReadOffsetB1J_6_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_6_0;
  globalReadOffsetB1J_7_0 = (  globalReadOffsetB1J_7_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_7_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_7_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_2_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_3_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 4*LSPB);
  unsigned int localWriteOffsetB_0_0_4_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 4*LSPB);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 5*LSPB);
  unsigned int localWriteOffsetB_0_0_5_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 5*LSPB);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 6*LSPB);
  unsigned int localWriteOffsetB_0_0_6_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 6*LSPB);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 7*LSPB);
  unsigned int localWriteOffsetB_0_0_7_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 7*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_2_1;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_3_1;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_4_1;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_5_1;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_6_1;
  DATA_TYPE *localWriteB_0_0_7_0;
  DATA_TYPE *localWriteB_0_0_7_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_1);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_1);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_4_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_1);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_5_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_1);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_6_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_1);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);
  localWriteB_0_0_7_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_1_4_0 = *(globalReadB_0_0_4_0 + 1);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_1_5_0 = *(globalReadB_0_0_5_0 + 1);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_1_6_0 = *(globalReadB_0_0_6_0 + 1);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);
    b_0_1_7_0 = *(globalReadB_0_0_7_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_1 + 0) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_1 + 0) = b_0_1_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_4_1 + 0) = b_0_1_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_5_1 + 0) = b_0_1_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_6_1 + 0) = b_0_1_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
    *(localWriteB_0_0_7_1 + 0) = b_0_1_7_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x8


    /* iter 15 */

    MAC_2x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_1_2_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 1);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_1_3_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 1);
  b_0_0_4_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_1_4_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 1);
  b_0_0_5_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_1_5_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 1);
  b_0_0_6_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_1_6_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 1);
  b_0_0_7_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);
  b_0_1_7_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_1);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_1);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_4_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_1);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_5_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_1);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_6_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_1);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);
  localWriteB_0_0_7_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_2_1 + 0) = b_0_1_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_3_1 + 0) = b_0_1_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_4_1 + 0) = b_0_1_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_5_1 + 0) = b_0_1_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_6_1 + 0) = b_0_1_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
  *(localWriteB_0_0_7_1 + 0) = b_0_1_7_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x8
#undef MAC_2x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? static_cast<tensile_bfloat16>((ALPHA)*(REG) + (BETA) * static_cast<float>(SRC)) : static_cast<tensile_bfloat16>((ALPHA)*(REG));

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_BBH_MT128x32x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_bfloat16 *D,
  tensile_bfloat16 const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;
  unsigned int flattenedOffsetA_4_0 = globalReadOffsetA0I + 0 + 4*LSPA;
  flattenedOffsetA_4_0 = (flattenedOffsetA_4_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_4_0;
  unsigned int globalReadOffsetA0I_4_0 = flattenedOffsetA_4_0;
  unsigned int flattenedOffsetA_5_0 = globalReadOffsetA0I + 0 + 5*LSPA;
  flattenedOffsetA_5_0 = (flattenedOffsetA_5_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_5_0;
  unsigned int globalReadOffsetA0I_5_0 = flattenedOffsetA_5_0;
  unsigned int flattenedOffsetA_6_0 = globalReadOffsetA0I + 0 + 6*LSPA;
  flattenedOffsetA_6_0 = (flattenedOffsetA_6_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_6_0;
  unsigned int globalReadOffsetA0I_6_0 = flattenedOffsetA_6_0;
  unsigned int flattenedOffsetA_7_0 = globalReadOffsetA0I + 0 + 7*LSPA;
  flattenedOffsetA_7_0 = (flattenedOffsetA_7_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_7_0;
  unsigned int globalReadOffsetA0I_7_0 = flattenedOffsetA_7_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;
  globalReadOffsetA0I_4_0 = (  globalReadOffsetA0I_4_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_4_0;
  globalReadOffsetA0I_5_0 = (  globalReadOffsetA0I_5_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_5_0;
  globalReadOffsetA0I_6_0 = (  globalReadOffsetA0I_6_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_6_0;
  globalReadOffsetA0I_7_0 = (  globalReadOffsetA0I_7_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_7_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 4*LSPA);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 5*LSPA);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 6*LSPA);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 7*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 8

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? static_cast<tensile_bfloat16>((ALPHA)*(REG) + (BETA) * static_cast<float>(SRC)) : static_cast<tensile_bfloat16>((ALPHA)*(REG));

/* 2x8 micro-tile */
#define MAC_2x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \

#define MAC_2x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_BBH_MT32x128x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_8_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_bfloat16 *D,
  tensile_bfloat16 const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;
  unsigned int flattenedOffsetB_4_0 = globalReadOffsetB1J + 0 + 4*LSPB;
  flattenedOffsetB_4_0 = (flattenedOffsetB_4_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_4_0;
  unsigned int globalReadOffsetB1J_4_0 = flattenedOffsetB_4_0;
  unsigned int flattenedOffsetB_5_0 = globalReadOffsetB1J + 0 + 5*LSPB;
  flattenedOffsetB_5_0 = (flattenedOffsetB_5_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_5_0;
  unsigned int globalReadOffsetB1J_5_0 = flattenedOffsetB_5_0;
  unsigned int flattenedOffsetB_6_0 = globalReadOffsetB1J + 0 + 6*LSPB;
  flattenedOffsetB_6_0 = (flattenedOffsetB_6_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_6_0;
  unsigned int globalReadOffsetB1J_6_0 = flattenedOffsetB_6_0;
  unsigned int flattenedOffsetB_7_0 = globalReadOffsetB1J + 0 + 7*LSPB;
  flattenedOffsetB_7_0 = (flattenedOffsetB_7_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_7_0;
  unsigned int globalReadOffsetB1J_7_0 = flattenedOffsetB_7_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;
  globalReadOffsetB1J_4_0 = (  globalReadOffsetB1J_4_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_4_0;
  globalReadOffsetB1J_5_0 = (  globalReadOffsetB1J_5_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_5_0;
  globalReadOffsetB1J_6_0 = (  globalReadOffsetB1J_6_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_6_0;
  globalReadOffsetB1J_7_0 = (  globalReadOffsetB1J_7_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_7_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_7_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 4*LSPB);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 5*LSPB);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 6*LSPB);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 7*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x8


    /* iter 15 */

    MAC_2x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x8
#undef MAC_2x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 16
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? static_cast<tensile_bfloat16>((ALPHA)*(REG) + (BETA) * static_cast<float>(SRC)) : static_cast<tensile_bfloat16>((ALPHA)*(REG));

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_BBH_MT64x16x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  tensile_bfloat16 *D,
  tensile_bfloat16 const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;
  unsigned int flattenedOffsetA_4_0 = globalReadOffsetA0I + 0 + 4*LSPA;
  flattenedOffsetA_4_0 = (flattenedOffsetA_4_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_4_0;
  unsigned int globalReadOffsetA0I_4_0 = flattenedOffsetA_4_0;
  unsigned int flattenedOffsetA_5_0 = globalReadOffsetA0I + 0 + 5*LSPA;
  flattenedOffsetA_5_0 = (flattenedOffsetA_5_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_5_0;
  unsigned int globalReadOffsetA0I_5_0 = flattenedOffsetA_5_0;
  unsigned int flattenedOffsetA_6_0 = globalReadOffsetA0I + 0 + 6*LSPA;
  flattenedOffsetA_6_0 = (flattenedOffsetA_6_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_6_0;
  unsigned int globalReadOffsetA0I_6_0 = flattenedOffsetA_6_0;
  unsigned int flattenedOffsetA_7_0 = globalReadOffsetA0I + 0 + 7*LSPA;
  flattenedOffsetA_7_0 = (flattenedOffsetA_7_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_7_0;
  unsigned int globalReadOffsetA0I_7_0 = flattenedOffsetA_7_0;
  unsigned int flattenedOffsetA_8_0 = globalReadOffsetA0I + 0 + 8*LSPA;
  flattenedOffsetA_8_0 = (flattenedOffsetA_8_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_8_0;
  unsigned int globalReadOffsetA0I_8_0 = flattenedOffsetA_8_0;
  unsigned int flattenedOffsetA_9_0 = globalReadOffsetA0I + 0 + 9*LSPA;
  flattenedOffsetA_9_0 = (flattenedOffsetA_9_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_9_0;
  unsigned int globalReadOffsetA0I_9_0 = flattenedOffsetA_9_0;
  unsigned int flattenedOffsetA_10_0 = globalReadOffsetA0I + 0 + 10*LSPA;
  flattenedOffsetA_10_0 = (flattenedOffsetA_10_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_10_0;
  unsigned int globalReadOffsetA0I_10_0 = flattenedOffsetA_10_0;
  unsigned int flattenedOffsetA_11_0 = globalReadOffsetA0I + 0 + 11*LSPA;
  flattenedOffsetA_11_0 = (flattenedOffsetA_11_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_11_0;
  unsigned int globalReadOffsetA0I_11_0 = flattenedOffsetA_11_0;
  unsigned int flattenedOffsetA_12_0 = globalReadOffsetA0I + 0 + 12*LSPA;
  flattenedOffsetA_12_0 = (flattenedOffsetA_12_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_12_0;
  unsigned int globalReadOffsetA0I_12_0 = flattenedOffsetA_12_0;
  unsigned int flattenedOffsetA_13_0 = globalReadOffsetA0I + 0 + 13*LSPA;
  flattenedOffsetA_13_0 = (flattenedOffsetA_13_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_13_0;
  unsigned int globalReadOffsetA0I_13_0 = flattenedOffsetA_13_0;
  unsigned int flattenedOffsetA_14_0 = globalReadOffsetA0I + 0 + 14*LSPA;
  flattenedOffsetA_14_0 = (flattenedOffsetA_14_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_14_0;
  unsigned int globalReadOffsetA0I_14_0 = flattenedOffsetA_14_0;
  unsigned int flattenedOffsetA_15_0 = globalReadOffsetA0I + 0 + 15*LSPA;
  flattenedOffsetA_15_0 = (flattenedOffsetA_15_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_15_0;
  unsigned int globalReadOffsetA0I_15_0 = flattenedOffsetA_15_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;
  globalReadOffsetA0I_4_0 = (  globalReadOffsetA0I_4_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_4_0;
  globalReadOffsetA0I_5_0 = (  globalReadOffsetA0I_5_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_5_0;
  globalReadOffsetA0I_6_0 = (  globalReadOffsetA0I_6_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_6_0;
  globalReadOffsetA0I_7_0 = (  globalReadOffsetA0I_7_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_7_0;
  globalReadOffsetA0I_8_0 = (  globalReadOffsetA0I_8_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_8_0;
  globalReadOffsetA0I_9_0 = (  globalReadOffsetA0I_9_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_9_0;
  globalReadOffsetA0I_10_0 = (  globalReadOffsetA0I_10_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_10_0;
  globalReadOffsetA0I_11_0 = (  globalReadOffsetA0I_11_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_11_0;
  globalReadOffsetA0I_12_0 = (  globalReadOffsetA0I_12_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_12_0;
  globalReadOffsetA0I_13_0 = (  globalReadOffsetA0I_13_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_13_0;
  globalReadOffsetA0I_14_0 = (  globalReadOffsetA0I_14_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_14_0;
  globalReadOffsetA0I_15_0 = (  globalReadOffsetA0I_15_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_15_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_15_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 4*LSPA);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 5*LSPA);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 6*LSPA);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 7*LSPA);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 8*LSPA);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 9*LSPA);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 10*LSPA);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 11*LSPA);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 12*LSPA);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 13*LSPA);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 14*LSPA);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 15*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_0_0_15_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_8_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_9_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_10_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_11_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_12_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_13_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_14_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_15_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_8_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_9_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_10_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_11_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_12_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_13_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_14_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_15_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_0_8_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_0_0_9_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_0_0_10_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_0_0_11_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_0_0_12_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_0_0_13_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_0_0_14_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_0_0_15_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 16

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? static_cast<tensile_bfloat16>((ALPHA)*(REG) + (BETA) * static_cast<float>(SRC)) : static_cast<tensile_bfloat16>((ALPHA)*(REG));

/* 2x8 micro-tile */
#define MAC_2x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \

#define MAC_2x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_BBH_MT16x64x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_8_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  tensile_bfloat16 *D,
  tensile_bfloat16 const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;
  DATA_TYPE b_0_0_8_0;
  DATA_TYPE b_0_0_9_0;
  DATA_TYPE b_0_0_10_0;
  DATA_TYPE b_0_0_11_0;
  DATA_TYPE b_0_0_12_0;
  DATA_TYPE b_0_0_13_0;
  DATA_TYPE b_0_0_14_0;
  DATA_TYPE b_0_0_15_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;
  unsigned int flattenedOffsetB_4_0 = globalReadOffsetB1J + 0 + 4*LSPB;
  flattenedOffsetB_4_0 = (flattenedOffsetB_4_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_4_0;
  unsigned int globalReadOffsetB1J_4_0 = flattenedOffsetB_4_0;
  unsigned int flattenedOffsetB_5_0 = globalReadOffsetB1J + 0 + 5*LSPB;
  flattenedOffsetB_5_0 = (flattenedOffsetB_5_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_5_0;
  unsigned int globalReadOffsetB1J_5_0 = flattenedOffsetB_5_0;
  unsigned int flattenedOffsetB_6_0 = globalReadOffsetB1J + 0 + 6*LSPB;
  flattenedOffsetB_6_0 = (flattenedOffsetB_6_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_6_0;
  unsigned int globalReadOffsetB1J_6_0 = flattenedOffsetB_6_0;
  unsigned int flattenedOffsetB_7_0 = globalReadOffsetB1J + 0 + 7*LSPB;
  flattenedOffsetB_7_0 = (flattenedOffsetB_7_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_7_0;
  unsigned int globalReadOffsetB1J_7_0 = flattenedOffsetB_7_0;
  unsigned int flattenedOffsetB_8_0 = globalReadOffsetB1J + 0 + 8*LSPB;
  flattenedOffsetB_8_0 = (flattenedOffsetB_8_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_8_0;
  unsigned int globalReadOffsetB1J_8_0 = flattenedOffsetB_8_0;
  unsigned int flattenedOffsetB_9_0 = globalReadOffsetB1J + 0 + 9*LSPB;
  flattenedOffsetB_9_0 = (flattenedOffsetB_9_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_9_0;
  unsigned int globalReadOffsetB1J_9_0 = flattenedOffsetB_9_0;
  unsigned int flattenedOffsetB_10_0 = globalReadOffsetB1J + 0 + 10*LSPB;
  flattenedOffsetB_10_0 = (flattenedOffsetB_10_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_10_0;
  unsigned int globalReadOffsetB1J_10_0 = flattenedOffsetB_10_0;
  unsigned int flattenedOffsetB_11_0 = globalReadOffsetB1J + 0 + 11*LSPB;
  flattenedOffsetB_11_0 = (flattenedOffsetB_11_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_11_0;
  unsigned int globalReadOffsetB1J_11_0 = flattenedOffsetB_11_0;
  unsigned int flattenedOffsetB_12_0 = globalReadOffsetB1J + 0 + 12*LSPB;
  flattenedOffsetB_12_0 = (flattenedOffsetB_12_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_12_0;
  unsigned int globalReadOffsetB1J_12_0 = flattenedOffsetB_12_0;
  unsigned int flattenedOffsetB_13_0 = globalReadOffsetB1J + 0 + 13*LSPB;
  flattenedOffsetB_13_0 = (flattenedOffsetB_13_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_13_0;
  unsigned int globalReadOffsetB1J_13_0 = flattenedOffsetB_13_0;
  unsigned int flattenedOffsetB_14_0 = globalReadOffsetB1J + 0 + 14*LSPB;
  flattenedOffsetB_14_0 = (flattenedOffsetB_14_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_14_0;
  unsigned int globalReadOffsetB1J_14_0 = flattenedOffsetB_14_0;
  unsigned int flattenedOffsetB_15_0 = globalReadOffsetB1J + 0 + 15*LSPB;
  flattenedOffsetB_15_0 = (flattenedOffsetB_15_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_15_0;
  unsigned int globalReadOffsetB1J_15_0 = flattenedOffsetB_15_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;
  globalReadOffsetB1J_4_0 = (  globalReadOffsetB1J_4_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_4_0;
  globalReadOffsetB1J_5_0 = (  globalReadOffsetB1J_5_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_5_0;
  globalReadOffsetB1J_6_0 = (  globalReadOffsetB1J_6_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_6_0;
  globalReadOffsetB1J_7_0 = (  globalReadOffsetB1J_7_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_7_0;
  globalReadOffsetB1J_8_0 = (  globalReadOffsetB1J_8_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_8_0;
  globalReadOffsetB1J_9_0 = (  globalReadOffsetB1J_9_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_9_0;
  globalReadOffsetB1J_10_0 = (  globalReadOffsetB1J_10_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_10_0;
  globalReadOffsetB1J_11_0 = (  globalReadOffsetB1J_11_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_11_0;
  globalReadOffsetB1J_12_0 = (  globalReadOffsetB1J_12_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_12_0;
  globalReadOffsetB1J_13_0 = (  globalReadOffsetB1J_13_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_13_0;
  globalReadOffsetB1J_14_0 = (  globalReadOffsetB1J_14_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_14_0;
  globalReadOffsetB1J_15_0 = (  globalReadOffsetB1J_15_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_15_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_7_0), wgK );
  int64_t globalReadOffsetB_0_0_8_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_8_0), wgK );
  int64_t globalReadOffsetB_0_0_9_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_9_0), wgK );
  int64_t globalReadOffsetB_0_0_10_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_10_0), wgK );
  int64_t globalReadOffsetB_0_0_11_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_11_0), wgK );
  int64_t globalReadOffsetB_0_0_12_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_12_0), wgK );
  int64_t globalReadOffsetB_0_0_13_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_13_0), wgK );
  int64_t globalReadOffsetB_0_0_14_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_14_0), wgK );
  int64_t globalReadOffsetB_0_0_15_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_15_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;
  DATA_TYPE const *globalReadB_0_0_8_0 = B + globalReadOffsetB_0_0_8_0;
  DATA_TYPE const *globalReadB_0_0_9_0 = B + globalReadOffsetB_0_0_9_0;
  DATA_TYPE const *globalReadB_0_0_10_0 = B + globalReadOffsetB_0_0_10_0;
  DATA_TYPE const *globalReadB_0_0_11_0 = B + globalReadOffsetB_0_0_11_0;
  DATA_TYPE const *globalReadB_0_0_12_0 = B + globalReadOffsetB_0_0_12_0;
  DATA_TYPE const *globalReadB_0_0_13_0 = B + globalReadOffsetB_0_0_13_0;
  DATA_TYPE const *globalReadB_0_0_14_0 = B + globalReadOffsetB_0_0_14_0;
  DATA_TYPE const *globalReadB_0_0_15_0 = B + globalReadOffsetB_0_0_15_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 4*LSPB);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 5*LSPB);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 6*LSPB);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 7*LSPB);
  unsigned int localWriteOffsetB_0_0_8_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 8*LSPB);
  unsigned int localWriteOffsetB_0_0_9_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 9*LSPB);
  unsigned int localWriteOffsetB_0_0_10_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 10*LSPB);
  unsigned int localWriteOffsetB_0_0_11_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 11*LSPB);
  unsigned int localWriteOffsetB_0_0_12_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 12*LSPB);
  unsigned int localWriteOffsetB_0_0_13_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 13*LSPB);
  unsigned int localWriteOffsetB_0_0_14_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 14*LSPB);
  unsigned int localWriteOffsetB_0_0_15_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 15*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;
  DATA_TYPE *localWriteB_0_0_8_0;
  DATA_TYPE *localWriteB_0_0_9_0;
  DATA_TYPE *localWriteB_0_0_10_0;
  DATA_TYPE *localWriteB_0_0_11_0;
  DATA_TYPE *localWriteB_0_0_12_0;
  DATA_TYPE *localWriteB_0_0_13_0;
  DATA_TYPE *localWriteB_0_0_14_0;
  DATA_TYPE *localWriteB_0_0_15_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);
  localWriteB_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_8_0);
  localWriteB_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_9_0);
  localWriteB_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_10_0);
  localWriteB_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_11_0);
  localWriteB_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_12_0);
  localWriteB_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_13_0);
  localWriteB_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_14_0);
  localWriteB_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_15_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_8_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_9_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_10_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_11_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_12_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_13_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_14_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_15_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);
    b_0_0_8_0 = *(globalReadB_0_0_8_0 + 0);
    b_0_0_9_0 = *(globalReadB_0_0_9_0 + 0);
    b_0_0_10_0 = *(globalReadB_0_0_10_0 + 0);
    b_0_0_11_0 = *(globalReadB_0_0_11_0 + 0);
    b_0_0_12_0 = *(globalReadB_0_0_12_0 + 0);
    b_0_0_13_0 = *(globalReadB_0_0_13_0 + 0);
    b_0_0_14_0 = *(globalReadB_0_0_14_0 + 0);
    b_0_0_15_0 = *(globalReadB_0_0_15_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_8_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_8_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_9_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_9_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_10_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_10_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_11_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_11_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_12_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_12_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_13_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_13_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_14_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_14_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_15_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_15_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
    *(localWriteB_0_0_8_0 + 0) = b_0_0_8_0;
    *(localWriteB_0_0_9_0 + 0) = b_0_0_9_0;
    *(localWriteB_0_0_10_0 + 0) = b_0_0_10_0;
    *(localWriteB_0_0_11_0 + 0) = b_0_0_11_0;
    *(localWriteB_0_0_12_0 + 0) = b_0_0_12_0;
    *(localWriteB_0_0_13_0 + 0) = b_0_0_13_0;
    *(localWriteB_0_0_14_0 + 0) = b_0_0_14_0;
    *(localWriteB_0_0_15_0 + 0) = b_0_0_15_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x8


    /* iter 15 */

    MAC_2x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);
  b_0_0_8_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_8_0 + 0);
  b_0_0_9_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_9_0 + 0);
  b_0_0_10_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_10_0 + 0);
  b_0_0_11_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_11_0 + 0);
  b_0_0_12_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_12_0 + 0);
  b_0_0_13_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_13_0 + 0);
  b_0_0_14_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_14_0 + 0);
  b_0_0_15_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_15_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);
  localWriteB_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_8_0);
  localWriteB_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_9_0);
  localWriteB_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_10_0);
  localWriteB_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_11_0);
  localWriteB_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_12_0);
  localWriteB_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_13_0);
  localWriteB_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_14_0);
  localWriteB_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_15_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
  *(localWriteB_0_0_8_0 + 0) = b_0_0_8_0;
  *(localWriteB_0_0_9_0 + 0) = b_0_0_9_0;
  *(localWriteB_0_0_10_0 + 0) = b_0_0_10_0;
  *(localWriteB_0_0_11_0 + 0) = b_0_0_11_0;
  *(localWriteB_0_0_12_0 + 0) = b_0_0_12_0;
  *(localWriteB_0_0_13_0 + 0) = b_0_0_13_0;
  *(localWriteB_0_0_14_0 + 0) = b_0_0_14_0;
  *(localWriteB_0_0_15_0 + 0) = b_0_0_15_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x8
#undef MAC_2x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HBH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 3 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 3 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HBH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 7 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HBH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 819

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HBH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 3 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 3584
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HBH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HBH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 7 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HBH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 819

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HBH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 819

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HBH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 3 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 3584
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HBH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 128
#define LDS_NUM_ELEMENTS 512
#define LDS_OFFSET_BLK 256

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT16x16x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 8
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1792
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x4 micro-tile */
#define MAC_8x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7],rB[3],rC[7+3*TT0I]); \

#define MAC_8x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT64x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_2_1_0;
  DATA_TYPE a_0_3_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
    *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
    *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_8x4


    /* iter 7 (last unrolled loop) */

    MAC_8x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_3 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_2_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 2);
  a_0_3_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
  *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 2) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 3) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x4
#undef MAC_8x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 896
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT32x16x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1664
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT64x16x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x2


    /* iter 7 (last unrolled loop) */

    MAC_4x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 896
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT32x16x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x2


    /* iter 7 (last unrolled loop) */

    MAC_4x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT32x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 7 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_3 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 2) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 3) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 896
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT32x16x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_8_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1664
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT64x16x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_8_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x2


    /* iter 7 (last unrolled loop) */

    MAC_4x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT16x16x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 15 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 3328
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT64x16x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x2


    /* iter 15 (last unrolled loop) */

    MAC_4x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT16x16x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 15 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 3328
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT64x16x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_8_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x2


    /* iter 15 (last unrolled loop) */

    MAC_4x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 2) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 3) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_3 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 2) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 3) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 7168
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT64x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_2_1_0;
  DATA_TYPE b_0_3_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;
  DATA_TYPE *localWriteB_0_0_1_2;
  DATA_TYPE *localWriteB_0_0_1_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
  localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
    *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_2 = (localWriteOffsetB_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_3 = (localWriteOffsetB_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
    localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
    localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
    *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_2 = (localWriteOffsetB_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_3 = (localWriteOffsetB_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
    localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
    localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x8


    /* iter 15 (last unrolled loop) */

    MAC_4x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_3 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_3 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_2_1_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 2);
  b_0_3_1_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
  localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
  *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
  *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
    if (r0I == 2) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
    if (r0I == 3) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 6
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 3
#define NLCB 1
#define NLPA 1
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1536
#define LDS_NUM_ELEMENTS 3584

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 6x8 micro-tile */
#define MAC_6x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I]); \
  TYPE_MAC(rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I]); \
  TYPE_MAC(rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I]); \
  TYPE_MAC(rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I]); \
  TYPE_MAC(rA[5],rB[7],rC[5+7*TT0I]); \

#define MAC_6x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[4+TT1J],rC[4+4*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[4+TT1J],rC[5+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[5+TT1J],rC[4+5*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[5+TT1J],rC[5+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[6+TT1J],rC[4+6*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[6+TT1J],rC[5+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[7+TT1J],rC[4+7*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[7+TT1J],rC[5+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT96x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA3_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT6_8_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_1_0_0_0;
  DATA_TYPE a_1_1_0_0;
  DATA_TYPE a_2_0_0_0;
  DATA_TYPE a_2_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_1_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_1_3_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSCA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSCA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_2_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_1_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_1_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_2_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_2_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_1_0_0_0 = A + globalReadOffsetA_1_0_0_0;
  DATA_TYPE const *globalReadA_2_0_0_0 = A + globalReadOffsetA_2_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_1_0_0_0 = localWriteFirstOffsetA + (0 + 1*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_2_0_0_0 = localWriteFirstOffsetA + (0 + 2*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_2_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_3_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_1_0_0_0;
  DATA_TYPE *localWriteA_2_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_2_1;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_3_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_1_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_0_0);
  localWriteA_2_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_2_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_1);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_1_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_2_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_1_0_0_0 = *(globalReadA_1_0_0_0 + 0);
    a_1_1_0_0 = *(globalReadA_1_0_0_0 + 1);
    a_2_0_0_0 = *(globalReadA_2_0_0_0 + 0);
    a_2_1_0_0 = *(globalReadA_2_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_1_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_1_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_1_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_2_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_2_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_2_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_1_0_0_0 + 0) = a_1_0_0_0;
    *(localWriteA_1_0_0_0 + 1) = a_1_1_0_0;
    *(localWriteA_2_0_0_0 + 0) = a_2_0_0_0;
    *(localWriteA_2_0_0_0 + 1) = a_2_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_1 + 0) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_1 + 0) = b_0_1_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_6x8


    /* iter 15 */

    MAC_6x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_1_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_2_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_1_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_1_0_0_0 + 0);
  a_1_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_1_0_0_0 + 1);
  a_2_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_2_0_0_0 + 0);
  a_2_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_2_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_1_2_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 1);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_1_3_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_1_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_0_0);
  localWriteA_2_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_2_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_1);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_1_0_0_0 + 0) = a_1_0_0_0;
  *(localWriteA_1_0_0_0 + 1) = a_1_1_0_0;
  *(localWriteA_2_0_0_0 + 0) = a_2_0_0_0;
  *(localWriteA_2_0_0_0 + 1) = a_2_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_2_1 + 0) = b_0_1_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_3_1 + 0) = b_0_1_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_6x8

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else if (s0I == 1) {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else {
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_6x8
#undef MAC_6x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 2) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 3) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_3 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 2) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 3) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 7168
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT64x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_2_1_0;
  DATA_TYPE b_0_3_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;
  DATA_TYPE *localWriteB_0_0_1_2;
  DATA_TYPE *localWriteB_0_0_1_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
  localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
    *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_2 = (localWriteOffsetB_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_3 = (localWriteOffsetB_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
    localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
    localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
    *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_2 = (localWriteOffsetB_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_3 = (localWriteOffsetB_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
    localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
    localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x8


    /* iter 15 (last unrolled loop) */

    MAC_4x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_3 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_3 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_2_1_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 2);
  b_0_3_1_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
  localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
  *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
  *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
    if (r0I == 2) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
    if (r0I == 3) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 64
#define LDS_NUM_ELEMENTS 409
#define LDS_OFFSET_BLK 128

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT16x16x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 3 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 3 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 2
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 64
#define LDS_NUM_ELEMENTS 409
#define LDS_OFFSET_BLK 128

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT32x32x2_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 1 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 1 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 128
#define LDS_NUM_ELEMENTS 512
#define LDS_OFFSET_BLK 256

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT16x16x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 8
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1792
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x4 micro-tile */
#define MAC_8x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7],rB[3],rC[7+3*TT0I]); \

#define MAC_8x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT64x32x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_2_1_0;
  DATA_TYPE a_0_3_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
    *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
    *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_8x4


    /* iter 7 (last unrolled loop) */

    MAC_8x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_3 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_2_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 2);
  a_0_3_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
  *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 2) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 3) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x4
#undef MAC_8x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 896
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT32x16x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1664
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT64x16x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x2


    /* iter 7 (last unrolled loop) */

    MAC_4x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 896
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT32x16x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x2


    /* iter 7 (last unrolled loop) */

    MAC_4x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT32x32x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 7 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_3 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 2) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 3) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 896
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT32x16x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_8_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1664
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT64x16x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_8_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x2


    /* iter 7 (last unrolled loop) */

    MAC_4x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT16x16x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 15 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 3328
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT64x16x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x2


    /* iter 15 (last unrolled loop) */

    MAC_4x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT16x16x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 15 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 3328
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT64x16x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_8_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x2


    /* iter 15 (last unrolled loop) */

    MAC_4x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT64x64x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 2) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 3) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT64x64x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_3 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 2) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 3) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 7168
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT64x128x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_2_1_0;
  DATA_TYPE b_0_3_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;
  DATA_TYPE *localWriteB_0_0_1_2;
  DATA_TYPE *localWriteB_0_0_1_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
  localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
    *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_2 = (localWriteOffsetB_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_3 = (localWriteOffsetB_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
    localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
    localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
    *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_2 = (localWriteOffsetB_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_3 = (localWriteOffsetB_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
    localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
    localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x8


    /* iter 15 (last unrolled loop) */

    MAC_4x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_3 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_3 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_2_1_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 2);
  b_0_3_1_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
  localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
  *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
  *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
    if (r0I == 2) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
    if (r0I == 3) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 6
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 3
#define NLCB 1
#define NLPA 1
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1536
#define LDS_NUM_ELEMENTS 3584

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 6x8 micro-tile */
#define MAC_6x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I]); \
  TYPE_MAC(rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I]); \
  TYPE_MAC(rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I]); \
  TYPE_MAC(rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I]); \
  TYPE_MAC(rA[5],rB[7],rC[5+7*TT0I]); \

#define MAC_6x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[4+TT1J],rC[4+4*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[4+TT1J],rC[5+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[5+TT1J],rC[4+5*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[5+TT1J],rC[5+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[6+TT1J],rC[4+6*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[6+TT1J],rC[5+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[7+TT1J],rC[4+7*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[7+TT1J],rC[5+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT96x128x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA3_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT6_8_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_1_0_0_0;
  DATA_TYPE a_1_1_0_0;
  DATA_TYPE a_2_0_0_0;
  DATA_TYPE a_2_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_1_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_1_3_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSCA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSCA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_2_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_1_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_1_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_2_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_2_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_1_0_0_0 = A + globalReadOffsetA_1_0_0_0;
  DATA_TYPE const *globalReadA_2_0_0_0 = A + globalReadOffsetA_2_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_1_0_0_0 = localWriteFirstOffsetA + (0 + 1*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_2_0_0_0 = localWriteFirstOffsetA + (0 + 2*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_2_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_3_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_1_0_0_0;
  DATA_TYPE *localWriteA_2_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_2_1;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_3_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_1_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_0_0);
  localWriteA_2_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_2_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_1);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_1_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_2_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_1_0_0_0 = *(globalReadA_1_0_0_0 + 0);
    a_1_1_0_0 = *(globalReadA_1_0_0_0 + 1);
    a_2_0_0_0 = *(globalReadA_2_0_0_0 + 0);
    a_2_1_0_0 = *(globalReadA_2_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_1_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_1_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_1_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_2_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_2_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_2_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_1_0_0_0 + 0) = a_1_0_0_0;
    *(localWriteA_1_0_0_0 + 1) = a_1_1_0_0;
    *(localWriteA_2_0_0_0 + 0) = a_2_0_0_0;
    *(localWriteA_2_0_0_0 + 1) = a_2_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_1 + 0) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_1 + 0) = b_0_1_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_6x8


    /* iter 15 */

    MAC_6x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_1_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_2_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_1_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_1_0_0_0 + 0);
  a_1_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_1_0_0_0 + 1);
  a_2_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_2_0_0_0 + 0);
  a_2_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_2_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_1_2_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 1);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_1_3_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_1_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_0_0);
  localWriteA_2_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_2_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_1);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_1_0_0_0 + 0) = a_1_0_0_0;
  *(localWriteA_1_0_0_0 + 1) = a_1_1_0_0;
  *(localWriteA_2_0_0_0 + 0) = a_2_0_0_0;
  *(localWriteA_2_0_0_0 + 1) = a_2_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_2_1 + 0) = b_0_1_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_3_1 + 0) = b_0_1_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_6x8

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else if (s0I == 1) {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else {
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_6x8
#undef MAC_6x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT64x64x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 2) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 3) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT64x64x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_3 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 2) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 3) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 7168
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT64x128x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_2_1_0;
  DATA_TYPE b_0_3_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;
  DATA_TYPE *localWriteB_0_0_1_2;
  DATA_TYPE *localWriteB_0_0_1_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
  localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
    *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_2 = (localWriteOffsetB_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_3 = (localWriteOffsetB_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
    localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
    localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
    *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_2 = (localWriteOffsetB_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_3 = (localWriteOffsetB_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
    localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
    localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x8


    /* iter 15 (last unrolled loop) */

    MAC_4x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_3 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_3 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_2_1_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 2);
  b_0_3_1_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
  localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
  *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
  *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
    if (r0I == 2) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
    if (r0I == 3) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 64
#define LDS_NUM_ELEMENTS 409
#define LDS_OFFSET_BLK 128

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT16x16x4_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 3 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 3 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 2
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 64
#define LDS_NUM_ELEMENTS 409
#define LDS_OFFSET_BLK 128

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT32x32x2_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 1 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 1 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 819

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? static_cast<tensile_bfloat16>((ALPHA)*(REG) + (BETA) * static_cast<float>(SRC)) : static_cast<tensile_bfloat16>((ALPHA)*(REG));

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_BBH_MT64x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_bfloat16 *D,
  tensile_bfloat16 const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 7 */

    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? static_cast<tensile_bfloat16>((ALPHA)*(REG) + (BETA) * static_cast<float>(SRC)) : static_cast<tensile_bfloat16>((ALPHA)*(REG));

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_BBH_MT128x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_bfloat16 *D,
  tensile_bfloat16 const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 1) {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 2) {
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 8

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? static_cast<tensile_bfloat16>((ALPHA)*(REG) + (BETA) * static_cast<float>(SRC)) : static_cast<tensile_bfloat16>((ALPHA)*(REG));

/* 2x8 micro-tile */
#define MAC_2x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \

#define MAC_2x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_BBH_MT16x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_8_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  tensile_bfloat16 *D,
  tensile_bfloat16 const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_1_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_1_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_1_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_1_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_1_6_0;
  DATA_TYPE b_0_0_7_0;
  DATA_TYPE b_0_1_7_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;
  unsigned int flattenedOffsetB_4_0 = globalReadOffsetB1J + 0 + 4*LSPB;
  flattenedOffsetB_4_0 = (flattenedOffsetB_4_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_4_0;
  unsigned int globalReadOffsetB1J_4_0 = flattenedOffsetB_4_0;
  unsigned int flattenedOffsetB_5_0 = globalReadOffsetB1J + 0 + 5*LSPB;
  flattenedOffsetB_5_0 = (flattenedOffsetB_5_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_5_0;
  unsigned int globalReadOffsetB1J_5_0 = flattenedOffsetB_5_0;
  unsigned int flattenedOffsetB_6_0 = globalReadOffsetB1J + 0 + 6*LSPB;
  flattenedOffsetB_6_0 = (flattenedOffsetB_6_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_6_0;
  unsigned int globalReadOffsetB1J_6_0 = flattenedOffsetB_6_0;
  unsigned int flattenedOffsetB_7_0 = globalReadOffsetB1J + 0 + 7*LSPB;
  flattenedOffsetB_7_0 = (flattenedOffsetB_7_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_7_0;
  unsigned int globalReadOffsetB1J_7_0 = flattenedOffsetB_7_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;
  globalReadOffsetB1J_4_0 = (  globalReadOffsetB1J_4_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_4_0;
  globalReadOffsetB1J_5_0 = (  globalReadOffsetB1J_5_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_5_0;
  globalReadOffsetB1J_6_0 = (  globalReadOffsetB1J_6_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_6_0;
  globalReadOffsetB1J_7_0 = (  globalReadOffsetB1J_7_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_7_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_7_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_2_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_3_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 4*LSPB);
  unsigned int localWriteOffsetB_0_0_4_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 4*LSPB);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 5*LSPB);
  unsigned int localWriteOffsetB_0_0_5_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 5*LSPB);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 6*LSPB);
  unsigned int localWriteOffsetB_0_0_6_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 6*LSPB);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 7*LSPB);
  unsigned int localWriteOffsetB_0_0_7_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 7*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_2_1;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_3_1;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_4_1;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_5_1;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_6_1;
  DATA_TYPE *localWriteB_0_0_7_0;
  DATA_TYPE *localWriteB_0_0_7_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_1);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_1);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_4_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_1);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_5_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_1);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_6_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_1);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);
  localWriteB_0_0_7_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_1_4_0 = *(globalReadB_0_0_4_0 + 1);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_1_5_0 = *(globalReadB_0_0_5_0 + 1);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_1_6_0 = *(globalReadB_0_0_6_0 + 1);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);
    b_0_1_7_0 = *(globalReadB_0_0_7_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_1 + 0) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_1 + 0) = b_0_1_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_4_1 + 0) = b_0_1_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_5_1 + 0) = b_0_1_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_6_1 + 0) = b_0_1_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
    *(localWriteB_0_0_7_1 + 0) = b_0_1_7_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x8


    /* iter 15 */

    MAC_2x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_1_2_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 1);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_1_3_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 1);
  b_0_0_4_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_1_4_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 1);
  b_0_0_5_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_1_5_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 1);
  b_0_0_6_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_1_6_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 1);
  b_0_0_7_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);
  b_0_1_7_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_1);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_1);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_4_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_1);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_5_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_1);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_6_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_1);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);
  localWriteB_0_0_7_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_2_1 + 0) = b_0_1_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_3_1 + 0) = b_0_1_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_4_1 + 0) = b_0_1_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_5_1 + 0) = b_0_1_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_6_1 + 0) = b_0_1_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
  *(localWriteB_0_0_7_1 + 0) = b_0_1_7_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x8

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x8
#undef MAC_2x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? static_cast<tensile_bfloat16>((ALPHA)*(REG) + (BETA) * static_cast<float>(SRC)) : static_cast<tensile_bfloat16>((ALPHA)*(REG));

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_BBH_MT128x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_bfloat16 *D,
  tensile_bfloat16 const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 8

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? static_cast<tensile_bfloat16>((ALPHA)*(REG) + (BETA) * static_cast<float>(SRC)) : static_cast<tensile_bfloat16>((ALPHA)*(REG));

/* 2x8 micro-tile */
#define MAC_2x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \

#define MAC_2x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_BBH_MT32x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_8_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_bfloat16 *D,
  tensile_bfloat16 const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;
  unsigned int flattenedOffsetB_4_0 = globalReadOffsetB1J + 0 + 4*LSPB;
  flattenedOffsetB_4_0 = (flattenedOffsetB_4_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_4_0;
  unsigned int globalReadOffsetB1J_4_0 = flattenedOffsetB_4_0;
  unsigned int flattenedOffsetB_5_0 = globalReadOffsetB1J + 0 + 5*LSPB;
  flattenedOffsetB_5_0 = (flattenedOffsetB_5_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_5_0;
  unsigned int globalReadOffsetB1J_5_0 = flattenedOffsetB_5_0;
  unsigned int flattenedOffsetB_6_0 = globalReadOffsetB1J + 0 + 6*LSPB;
  flattenedOffsetB_6_0 = (flattenedOffsetB_6_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_6_0;
  unsigned int globalReadOffsetB1J_6_0 = flattenedOffsetB_6_0;
  unsigned int flattenedOffsetB_7_0 = globalReadOffsetB1J + 0 + 7*LSPB;
  flattenedOffsetB_7_0 = (flattenedOffsetB_7_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_7_0;
  unsigned int globalReadOffsetB1J_7_0 = flattenedOffsetB_7_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;
  globalReadOffsetB1J_4_0 = (  globalReadOffsetB1J_4_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_4_0;
  globalReadOffsetB1J_5_0 = (  globalReadOffsetB1J_5_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_5_0;
  globalReadOffsetB1J_6_0 = (  globalReadOffsetB1J_6_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_6_0;
  globalReadOffsetB1J_7_0 = (  globalReadOffsetB1J_7_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_7_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_7_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 4*LSPB);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 5*LSPB);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 6*LSPB);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 7*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x8


    /* iter 15 */

    MAC_2x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x8
#undef MAC_2x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 16
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? static_cast<tensile_bfloat16>((ALPHA)*(REG) + (BETA) * static_cast<float>(SRC)) : static_cast<tensile_bfloat16>((ALPHA)*(REG));

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_BBH_MT64x16x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  tensile_bfloat16 *D,
  tensile_bfloat16 const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;
  unsigned int globalReadOffsetAL_8_0 = globalReadOffsetAL + 0 + 8*LSPA;
  unsigned int globalReadOffsetAL_9_0 = globalReadOffsetAL + 0 + 9*LSPA;
  unsigned int globalReadOffsetAL_10_0 = globalReadOffsetAL + 0 + 10*LSPA;
  unsigned int globalReadOffsetAL_11_0 = globalReadOffsetAL + 0 + 11*LSPA;
  unsigned int globalReadOffsetAL_12_0 = globalReadOffsetAL + 0 + 12*LSPA;
  unsigned int globalReadOffsetAL_13_0 = globalReadOffsetAL + 0 + 13*LSPA;
  unsigned int globalReadOffsetAL_14_0 = globalReadOffsetAL + 0 + 14*LSPA;
  unsigned int globalReadOffsetAL_15_0 = globalReadOffsetAL + 0 + 15*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_15_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 8*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 9*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 10*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 11*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 12*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 13*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 14*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 15*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_0_0_15_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_8_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_9_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_10_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_11_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_12_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_13_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_14_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_15_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_8_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_9_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_10_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_11_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_12_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_13_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_14_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_15_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_0_8_0 = ( globalReadOffsetAL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_0_0_9_0 = ( globalReadOffsetAL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_0_0_10_0 = ( globalReadOffsetAL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_0_0_11_0 = ( globalReadOffsetAL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_0_0_12_0 = ( globalReadOffsetAL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_0_0_13_0 = ( globalReadOffsetAL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_0_0_14_0 = ( globalReadOffsetAL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_0_0_15_0 = ( globalReadOffsetAL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 16

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? static_cast<tensile_bfloat16>((ALPHA)*(REG) + (BETA) * static_cast<float>(SRC)) : static_cast<tensile_bfloat16>((ALPHA)*(REG));

/* 2x8 micro-tile */
#define MAC_2x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \

#define MAC_2x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_BBH_MT16x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_8_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  tensile_bfloat16 *D,
  tensile_bfloat16 const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;
  DATA_TYPE b_0_0_8_0;
  DATA_TYPE b_0_0_9_0;
  DATA_TYPE b_0_0_10_0;
  DATA_TYPE b_0_0_11_0;
  DATA_TYPE b_0_0_12_0;
  DATA_TYPE b_0_0_13_0;
  DATA_TYPE b_0_0_14_0;
  DATA_TYPE b_0_0_15_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;
  unsigned int flattenedOffsetB_4_0 = globalReadOffsetB1J + 0 + 4*LSPB;
  flattenedOffsetB_4_0 = (flattenedOffsetB_4_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_4_0;
  unsigned int globalReadOffsetB1J_4_0 = flattenedOffsetB_4_0;
  unsigned int flattenedOffsetB_5_0 = globalReadOffsetB1J + 0 + 5*LSPB;
  flattenedOffsetB_5_0 = (flattenedOffsetB_5_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_5_0;
  unsigned int globalReadOffsetB1J_5_0 = flattenedOffsetB_5_0;
  unsigned int flattenedOffsetB_6_0 = globalReadOffsetB1J + 0 + 6*LSPB;
  flattenedOffsetB_6_0 = (flattenedOffsetB_6_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_6_0;
  unsigned int globalReadOffsetB1J_6_0 = flattenedOffsetB_6_0;
  unsigned int flattenedOffsetB_7_0 = globalReadOffsetB1J + 0 + 7*LSPB;
  flattenedOffsetB_7_0 = (flattenedOffsetB_7_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_7_0;
  unsigned int globalReadOffsetB1J_7_0 = flattenedOffsetB_7_0;
  unsigned int flattenedOffsetB_8_0 = globalReadOffsetB1J + 0 + 8*LSPB;
  flattenedOffsetB_8_0 = (flattenedOffsetB_8_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_8_0;
  unsigned int globalReadOffsetB1J_8_0 = flattenedOffsetB_8_0;
  unsigned int flattenedOffsetB_9_0 = globalReadOffsetB1J + 0 + 9*LSPB;
  flattenedOffsetB_9_0 = (flattenedOffsetB_9_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_9_0;
  unsigned int globalReadOffsetB1J_9_0 = flattenedOffsetB_9_0;
  unsigned int flattenedOffsetB_10_0 = globalReadOffsetB1J + 0 + 10*LSPB;
  flattenedOffsetB_10_0 = (flattenedOffsetB_10_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_10_0;
  unsigned int globalReadOffsetB1J_10_0 = flattenedOffsetB_10_0;
  unsigned int flattenedOffsetB_11_0 = globalReadOffsetB1J + 0 + 11*LSPB;
  flattenedOffsetB_11_0 = (flattenedOffsetB_11_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_11_0;
  unsigned int globalReadOffsetB1J_11_0 = flattenedOffsetB_11_0;
  unsigned int flattenedOffsetB_12_0 = globalReadOffsetB1J + 0 + 12*LSPB;
  flattenedOffsetB_12_0 = (flattenedOffsetB_12_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_12_0;
  unsigned int globalReadOffsetB1J_12_0 = flattenedOffsetB_12_0;
  unsigned int flattenedOffsetB_13_0 = globalReadOffsetB1J + 0 + 13*LSPB;
  flattenedOffsetB_13_0 = (flattenedOffsetB_13_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_13_0;
  unsigned int globalReadOffsetB1J_13_0 = flattenedOffsetB_13_0;
  unsigned int flattenedOffsetB_14_0 = globalReadOffsetB1J + 0 + 14*LSPB;
  flattenedOffsetB_14_0 = (flattenedOffsetB_14_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_14_0;
  unsigned int globalReadOffsetB1J_14_0 = flattenedOffsetB_14_0;
  unsigned int flattenedOffsetB_15_0 = globalReadOffsetB1J + 0 + 15*LSPB;
  flattenedOffsetB_15_0 = (flattenedOffsetB_15_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_15_0;
  unsigned int globalReadOffsetB1J_15_0 = flattenedOffsetB_15_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;
  globalReadOffsetB1J_4_0 = (  globalReadOffsetB1J_4_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_4_0;
  globalReadOffsetB1J_5_0 = (  globalReadOffsetB1J_5_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_5_0;
  globalReadOffsetB1J_6_0 = (  globalReadOffsetB1J_6_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_6_0;
  globalReadOffsetB1J_7_0 = (  globalReadOffsetB1J_7_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_7_0;
  globalReadOffsetB1J_8_0 = (  globalReadOffsetB1J_8_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_8_0;
  globalReadOffsetB1J_9_0 = (  globalReadOffsetB1J_9_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_9_0;
  globalReadOffsetB1J_10_0 = (  globalReadOffsetB1J_10_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_10_0;
  globalReadOffsetB1J_11_0 = (  globalReadOffsetB1J_11_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_11_0;
  globalReadOffsetB1J_12_0 = (  globalReadOffsetB1J_12_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_12_0;
  globalReadOffsetB1J_13_0 = (  globalReadOffsetB1J_13_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_13_0;
  globalReadOffsetB1J_14_0 = (  globalReadOffsetB1J_14_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_14_0;
  globalReadOffsetB1J_15_0 = (  globalReadOffsetB1J_15_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_15_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_7_0), wgK );
  int64_t globalReadOffsetB_0_0_8_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_8_0), wgK );
  int64_t globalReadOffsetB_0_0_9_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_9_0), wgK );
  int64_t globalReadOffsetB_0_0_10_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_10_0), wgK );
  int64_t globalReadOffsetB_0_0_11_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_11_0), wgK );
  int64_t globalReadOffsetB_0_0_12_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_12_0), wgK );
  int64_t globalReadOffsetB_0_0_13_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_13_0), wgK );
  int64_t globalReadOffsetB_0_0_14_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_14_0), wgK );
  int64_t globalReadOffsetB_0_0_15_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_15_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;
  DATA_TYPE const *globalReadB_0_0_8_0 = B + globalReadOffsetB_0_0_8_0;
  DATA_TYPE const *globalReadB_0_0_9_0 = B + globalReadOffsetB_0_0_9_0;
  DATA_TYPE const *globalReadB_0_0_10_0 = B + globalReadOffsetB_0_0_10_0;
  DATA_TYPE const *globalReadB_0_0_11_0 = B + globalReadOffsetB_0_0_11_0;
  DATA_TYPE const *globalReadB_0_0_12_0 = B + globalReadOffsetB_0_0_12_0;
  DATA_TYPE const *globalReadB_0_0_13_0 = B + globalReadOffsetB_0_0_13_0;
  DATA_TYPE const *globalReadB_0_0_14_0 = B + globalReadOffsetB_0_0_14_0;
  DATA_TYPE const *globalReadB_0_0_15_0 = B + globalReadOffsetB_0_0_15_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 4*LSPB);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 5*LSPB);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 6*LSPB);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 7*LSPB);
  unsigned int localWriteOffsetB_0_0_8_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 8*LSPB);
  unsigned int localWriteOffsetB_0_0_9_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 9*LSPB);
  unsigned int localWriteOffsetB_0_0_10_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 10*LSPB);
  unsigned int localWriteOffsetB_0_0_11_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 11*LSPB);
  unsigned int localWriteOffsetB_0_0_12_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 12*LSPB);
  unsigned int localWriteOffsetB_0_0_13_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 13*LSPB);
  unsigned int localWriteOffsetB_0_0_14_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 14*LSPB);
  unsigned int localWriteOffsetB_0_0_15_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 15*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;
  DATA_TYPE *localWriteB_0_0_8_0;
  DATA_TYPE *localWriteB_0_0_9_0;
  DATA_TYPE *localWriteB_0_0_10_0;
  DATA_TYPE *localWriteB_0_0_11_0;
  DATA_TYPE *localWriteB_0_0_12_0;
  DATA_TYPE *localWriteB_0_0_13_0;
  DATA_TYPE *localWriteB_0_0_14_0;
  DATA_TYPE *localWriteB_0_0_15_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);
  localWriteB_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_8_0);
  localWriteB_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_9_0);
  localWriteB_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_10_0);
  localWriteB_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_11_0);
  localWriteB_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_12_0);
  localWriteB_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_13_0);
  localWriteB_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_14_0);
  localWriteB_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_15_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_8_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_9_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_10_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_11_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_12_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_13_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_14_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_15_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);
    b_0_0_8_0 = *(globalReadB_0_0_8_0 + 0);
    b_0_0_9_0 = *(globalReadB_0_0_9_0 + 0);
    b_0_0_10_0 = *(globalReadB_0_0_10_0 + 0);
    b_0_0_11_0 = *(globalReadB_0_0_11_0 + 0);
    b_0_0_12_0 = *(globalReadB_0_0_12_0 + 0);
    b_0_0_13_0 = *(globalReadB_0_0_13_0 + 0);
    b_0_0_14_0 = *(globalReadB_0_0_14_0 + 0);
    b_0_0_15_0 = *(globalReadB_0_0_15_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_8_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_8_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_9_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_9_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_10_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_10_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_11_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_11_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_12_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_12_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_13_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_13_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_14_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_14_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_15_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_15_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
    *(localWriteB_0_0_8_0 + 0) = b_0_0_8_0;
    *(localWriteB_0_0_9_0 + 0) = b_0_0_9_0;
    *(localWriteB_0_0_10_0 + 0) = b_0_0_10_0;
    *(localWriteB_0_0_11_0 + 0) = b_0_0_11_0;
    *(localWriteB_0_0_12_0 + 0) = b_0_0_12_0;
    *(localWriteB_0_0_13_0 + 0) = b_0_0_13_0;
    *(localWriteB_0_0_14_0 + 0) = b_0_0_14_0;
    *(localWriteB_0_0_15_0 + 0) = b_0_0_15_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x8


    /* iter 15 */

    MAC_2x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);
  b_0_0_8_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_8_0 + 0);
  b_0_0_9_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_9_0 + 0);
  b_0_0_10_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_10_0 + 0);
  b_0_0_11_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_11_0 + 0);
  b_0_0_12_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_12_0 + 0);
  b_0_0_13_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_13_0 + 0);
  b_0_0_14_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_14_0 + 0);
  b_0_0_15_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_15_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);
  localWriteB_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_8_0);
  localWriteB_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_9_0);
  localWriteB_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_10_0);
  localWriteB_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_11_0);
  localWriteB_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_12_0);
  localWriteB_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_13_0);
  localWriteB_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_14_0);
  localWriteB_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_15_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
  *(localWriteB_0_0_8_0 + 0) = b_0_0_8_0;
  *(localWriteB_0_0_9_0 + 0) = b_0_0_9_0;
  *(localWriteB_0_0_10_0 + 0) = b_0_0_10_0;
  *(localWriteB_0_0_11_0 + 0) = b_0_0_11_0;
  *(localWriteB_0_0_12_0 + 0) = b_0_0_12_0;
  *(localWriteB_0_0_13_0 + 0) = b_0_0_13_0;
  *(localWriteB_0_0_14_0 + 0) = b_0_0_14_0;
  *(localWriteB_0_0_15_0 + 0) = b_0_0_15_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x8
#undef MAC_2x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 819

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? static_cast<tensile_bfloat16>((ALPHA)*(REG) + (BETA) * static_cast<float>(SRC)) : static_cast<tensile_bfloat16>((ALPHA)*(REG));

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_BBH_MT64x32x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_bfloat16 *D,
  tensile_bfloat16 const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 7 */

    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? static_cast<tensile_bfloat16>((ALPHA)*(REG) + (BETA) * static_cast<float>(SRC)) : static_cast<tensile_bfloat16>((ALPHA)*(REG));

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_BBH_MT128x32x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_bfloat16 *D,
  tensile_bfloat16 const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 1) {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 2) {
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 8

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? static_cast<tensile_bfloat16>((ALPHA)*(REG) + (BETA) * static_cast<float>(SRC)) : static_cast<tensile_bfloat16>((ALPHA)*(REG));

/* 2x8 micro-tile */
#define MAC_2x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \

#define MAC_2x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_BBH_MT16x64x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_8_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  tensile_bfloat16 *D,
  tensile_bfloat16 const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_1_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_1_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_1_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_1_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_1_6_0;
  DATA_TYPE b_0_0_7_0;
  DATA_TYPE b_0_1_7_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;
  unsigned int flattenedOffsetB_4_0 = globalReadOffsetB1J + 0 + 4*LSPB;
  flattenedOffsetB_4_0 = (flattenedOffsetB_4_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_4_0;
  unsigned int globalReadOffsetB1J_4_0 = flattenedOffsetB_4_0;
  unsigned int flattenedOffsetB_5_0 = globalReadOffsetB1J + 0 + 5*LSPB;
  flattenedOffsetB_5_0 = (flattenedOffsetB_5_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_5_0;
  unsigned int globalReadOffsetB1J_5_0 = flattenedOffsetB_5_0;
  unsigned int flattenedOffsetB_6_0 = globalReadOffsetB1J + 0 + 6*LSPB;
  flattenedOffsetB_6_0 = (flattenedOffsetB_6_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_6_0;
  unsigned int globalReadOffsetB1J_6_0 = flattenedOffsetB_6_0;
  unsigned int flattenedOffsetB_7_0 = globalReadOffsetB1J + 0 + 7*LSPB;
  flattenedOffsetB_7_0 = (flattenedOffsetB_7_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_7_0;
  unsigned int globalReadOffsetB1J_7_0 = flattenedOffsetB_7_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;
  globalReadOffsetB1J_4_0 = (  globalReadOffsetB1J_4_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_4_0;
  globalReadOffsetB1J_5_0 = (  globalReadOffsetB1J_5_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_5_0;
  globalReadOffsetB1J_6_0 = (  globalReadOffsetB1J_6_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_6_0;
  globalReadOffsetB1J_7_0 = (  globalReadOffsetB1J_7_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_7_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_7_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_2_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_3_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 4*LSPB);
  unsigned int localWriteOffsetB_0_0_4_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 4*LSPB);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 5*LSPB);
  unsigned int localWriteOffsetB_0_0_5_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 5*LSPB);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 6*LSPB);
  unsigned int localWriteOffsetB_0_0_6_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 6*LSPB);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 7*LSPB);
  unsigned int localWriteOffsetB_0_0_7_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 7*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_2_1;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_3_1;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_4_1;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_5_1;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_6_1;
  DATA_TYPE *localWriteB_0_0_7_0;
  DATA_TYPE *localWriteB_0_0_7_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_1);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_1);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_4_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_1);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_5_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_1);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_6_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_1);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);
  localWriteB_0_0_7_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_1_4_0 = *(globalReadB_0_0_4_0 + 1);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_1_5_0 = *(globalReadB_0_0_5_0 + 1);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_1_6_0 = *(globalReadB_0_0_6_0 + 1);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);
    b_0_1_7_0 = *(globalReadB_0_0_7_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_1 + 0) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_1 + 0) = b_0_1_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_4_1 + 0) = b_0_1_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_5_1 + 0) = b_0_1_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_6_1 + 0) = b_0_1_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
    *(localWriteB_0_0_7_1 + 0) = b_0_1_7_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x8


    /* iter 15 */

    MAC_2x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_1_2_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 1);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_1_3_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 1);
  b_0_0_4_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_1_4_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 1);
  b_0_0_5_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_1_5_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 1);
  b_0_0_6_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_1_6_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 1);
  b_0_0_7_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);
  b_0_1_7_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_1);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_1);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_4_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_1);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_5_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_1);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_6_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_1);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);
  localWriteB_0_0_7_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_2_1 + 0) = b_0_1_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_3_1 + 0) = b_0_1_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_4_1 + 0) = b_0_1_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_5_1 + 0) = b_0_1_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_6_1 + 0) = b_0_1_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
  *(localWriteB_0_0_7_1 + 0) = b_0_1_7_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x8

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x8
#undef MAC_2x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? static_cast<tensile_bfloat16>((ALPHA)*(REG) + (BETA) * static_cast<float>(SRC)) : static_cast<tensile_bfloat16>((ALPHA)*(REG));

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_BBH_MT128x32x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_bfloat16 *D,
  tensile_bfloat16 const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 8

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? static_cast<tensile_bfloat16>((ALPHA)*(REG) + (BETA) * static_cast<float>(SRC)) : static_cast<tensile_bfloat16>((ALPHA)*(REG));

/* 2x8 micro-tile */
#define MAC_2x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \

#define MAC_2x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_BBH_MT32x128x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_8_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_bfloat16 *D,
  tensile_bfloat16 const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;
  unsigned int flattenedOffsetB_4_0 = globalReadOffsetB1J + 0 + 4*LSPB;
  flattenedOffsetB_4_0 = (flattenedOffsetB_4_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_4_0;
  unsigned int globalReadOffsetB1J_4_0 = flattenedOffsetB_4_0;
  unsigned int flattenedOffsetB_5_0 = globalReadOffsetB1J + 0 + 5*LSPB;
  flattenedOffsetB_5_0 = (flattenedOffsetB_5_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_5_0;
  unsigned int globalReadOffsetB1J_5_0 = flattenedOffsetB_5_0;
  unsigned int flattenedOffsetB_6_0 = globalReadOffsetB1J + 0 + 6*LSPB;
  flattenedOffsetB_6_0 = (flattenedOffsetB_6_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_6_0;
  unsigned int globalReadOffsetB1J_6_0 = flattenedOffsetB_6_0;
  unsigned int flattenedOffsetB_7_0 = globalReadOffsetB1J + 0 + 7*LSPB;
  flattenedOffsetB_7_0 = (flattenedOffsetB_7_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_7_0;
  unsigned int globalReadOffsetB1J_7_0 = flattenedOffsetB_7_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;
  globalReadOffsetB1J_4_0 = (  globalReadOffsetB1J_4_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_4_0;
  globalReadOffsetB1J_5_0 = (  globalReadOffsetB1J_5_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_5_0;
  globalReadOffsetB1J_6_0 = (  globalReadOffsetB1J_6_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_6_0;
  globalReadOffsetB1J_7_0 = (  globalReadOffsetB1J_7_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_7_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_7_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 4*LSPB);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 5*LSPB);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 6*LSPB);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 7*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x8


    /* iter 15 */

    MAC_2x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x8
#undef MAC_2x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 16
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? static_cast<tensile_bfloat16>((ALPHA)*(REG) + (BETA) * static_cast<float>(SRC)) : static_cast<tensile_bfloat16>((ALPHA)*(REG));

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_BBH_MT64x16x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  tensile_bfloat16 *D,
  tensile_bfloat16 const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;
  unsigned int globalReadOffsetAL_8_0 = globalReadOffsetAL + 0 + 8*LSPA;
  unsigned int globalReadOffsetAL_9_0 = globalReadOffsetAL + 0 + 9*LSPA;
  unsigned int globalReadOffsetAL_10_0 = globalReadOffsetAL + 0 + 10*LSPA;
  unsigned int globalReadOffsetAL_11_0 = globalReadOffsetAL + 0 + 11*LSPA;
  unsigned int globalReadOffsetAL_12_0 = globalReadOffsetAL + 0 + 12*LSPA;
  unsigned int globalReadOffsetAL_13_0 = globalReadOffsetAL + 0 + 13*LSPA;
  unsigned int globalReadOffsetAL_14_0 = globalReadOffsetAL + 0 + 14*LSPA;
  unsigned int globalReadOffsetAL_15_0 = globalReadOffsetAL + 0 + 15*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_15_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 8*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 9*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 10*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 11*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 12*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 13*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 14*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 15*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_0_0_15_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_8_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_9_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_10_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_11_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_12_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_13_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_14_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_15_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_8_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_9_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_10_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_11_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_12_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_13_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_14_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_15_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_0_8_0 = ( globalReadOffsetAL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_0_0_9_0 = ( globalReadOffsetAL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_0_0_10_0 = ( globalReadOffsetAL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_0_0_11_0 = ( globalReadOffsetAL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_0_0_12_0 = ( globalReadOffsetAL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_0_0_13_0 = ( globalReadOffsetAL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_0_0_14_0 = ( globalReadOffsetAL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_0_0_15_0 = ( globalReadOffsetAL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 16

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? static_cast<tensile_bfloat16>((ALPHA)*(REG) + (BETA) * static_cast<float>(SRC)) : static_cast<tensile_bfloat16>((ALPHA)*(REG));

/* 2x8 micro-tile */
#define MAC_2x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \

#define MAC_2x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_BBH_MT16x64x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_8_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  tensile_bfloat16 *D,
  tensile_bfloat16 const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;
  DATA_TYPE b_0_0_8_0;
  DATA_TYPE b_0_0_9_0;
  DATA_TYPE b_0_0_10_0;
  DATA_TYPE b_0_0_11_0;
  DATA_TYPE b_0_0_12_0;
  DATA_TYPE b_0_0_13_0;
  DATA_TYPE b_0_0_14_0;
  DATA_TYPE b_0_0_15_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;
  unsigned int flattenedOffsetB_4_0 = globalReadOffsetB1J + 0 + 4*LSPB;
  flattenedOffsetB_4_0 = (flattenedOffsetB_4_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_4_0;
  unsigned int globalReadOffsetB1J_4_0 = flattenedOffsetB_4_0;
  unsigned int flattenedOffsetB_5_0 = globalReadOffsetB1J + 0 + 5*LSPB;
  flattenedOffsetB_5_0 = (flattenedOffsetB_5_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_5_0;
  unsigned int globalReadOffsetB1J_5_0 = flattenedOffsetB_5_0;
  unsigned int flattenedOffsetB_6_0 = globalReadOffsetB1J + 0 + 6*LSPB;
  flattenedOffsetB_6_0 = (flattenedOffsetB_6_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_6_0;
  unsigned int globalReadOffsetB1J_6_0 = flattenedOffsetB_6_0;
  unsigned int flattenedOffsetB_7_0 = globalReadOffsetB1J + 0 + 7*LSPB;
  flattenedOffsetB_7_0 = (flattenedOffsetB_7_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_7_0;
  unsigned int globalReadOffsetB1J_7_0 = flattenedOffsetB_7_0;
  unsigned int flattenedOffsetB_8_0 = globalReadOffsetB1J + 0 + 8*LSPB;
  flattenedOffsetB_8_0 = (flattenedOffsetB_8_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_8_0;
  unsigned int globalReadOffsetB1J_8_0 = flattenedOffsetB_8_0;
  unsigned int flattenedOffsetB_9_0 = globalReadOffsetB1J + 0 + 9*LSPB;
  flattenedOffsetB_9_0 = (flattenedOffsetB_9_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_9_0;
  unsigned int globalReadOffsetB1J_9_0 = flattenedOffsetB_9_0;
  unsigned int flattenedOffsetB_10_0 = globalReadOffsetB1J + 0 + 10*LSPB;
  flattenedOffsetB_10_0 = (flattenedOffsetB_10_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_10_0;
  unsigned int globalReadOffsetB1J_10_0 = flattenedOffsetB_10_0;
  unsigned int flattenedOffsetB_11_0 = globalReadOffsetB1J + 0 + 11*LSPB;
  flattenedOffsetB_11_0 = (flattenedOffsetB_11_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_11_0;
  unsigned int globalReadOffsetB1J_11_0 = flattenedOffsetB_11_0;
  unsigned int flattenedOffsetB_12_0 = globalReadOffsetB1J + 0 + 12*LSPB;
  flattenedOffsetB_12_0 = (flattenedOffsetB_12_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_12_0;
  unsigned int globalReadOffsetB1J_12_0 = flattenedOffsetB_12_0;
  unsigned int flattenedOffsetB_13_0 = globalReadOffsetB1J + 0 + 13*LSPB;
  flattenedOffsetB_13_0 = (flattenedOffsetB_13_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_13_0;
  unsigned int globalReadOffsetB1J_13_0 = flattenedOffsetB_13_0;
  unsigned int flattenedOffsetB_14_0 = globalReadOffsetB1J + 0 + 14*LSPB;
  flattenedOffsetB_14_0 = (flattenedOffsetB_14_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_14_0;
  unsigned int globalReadOffsetB1J_14_0 = flattenedOffsetB_14_0;
  unsigned int flattenedOffsetB_15_0 = globalReadOffsetB1J + 0 + 15*LSPB;
  flattenedOffsetB_15_0 = (flattenedOffsetB_15_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_15_0;
  unsigned int globalReadOffsetB1J_15_0 = flattenedOffsetB_15_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;
  globalReadOffsetB1J_4_0 = (  globalReadOffsetB1J_4_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_4_0;
  globalReadOffsetB1J_5_0 = (  globalReadOffsetB1J_5_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_5_0;
  globalReadOffsetB1J_6_0 = (  globalReadOffsetB1J_6_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_6_0;
  globalReadOffsetB1J_7_0 = (  globalReadOffsetB1J_7_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_7_0;
  globalReadOffsetB1J_8_0 = (  globalReadOffsetB1J_8_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_8_0;
  globalReadOffsetB1J_9_0 = (  globalReadOffsetB1J_9_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_9_0;
  globalReadOffsetB1J_10_0 = (  globalReadOffsetB1J_10_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_10_0;
  globalReadOffsetB1J_11_0 = (  globalReadOffsetB1J_11_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_11_0;
  globalReadOffsetB1J_12_0 = (  globalReadOffsetB1J_12_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_12_0;
  globalReadOffsetB1J_13_0 = (  globalReadOffsetB1J_13_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_13_0;
  globalReadOffsetB1J_14_0 = (  globalReadOffsetB1J_14_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_14_0;
  globalReadOffsetB1J_15_0 = (  globalReadOffsetB1J_15_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_15_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_7_0), wgK );
  int64_t globalReadOffsetB_0_0_8_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_8_0), wgK );
  int64_t globalReadOffsetB_0_0_9_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_9_0), wgK );
  int64_t globalReadOffsetB_0_0_10_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_10_0), wgK );
  int64_t globalReadOffsetB_0_0_11_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_11_0), wgK );
  int64_t globalReadOffsetB_0_0_12_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_12_0), wgK );
  int64_t globalReadOffsetB_0_0_13_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_13_0), wgK );
  int64_t globalReadOffsetB_0_0_14_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_14_0), wgK );
  int64_t globalReadOffsetB_0_0_15_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_15_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;
  DATA_TYPE const *globalReadB_0_0_8_0 = B + globalReadOffsetB_0_0_8_0;
  DATA_TYPE const *globalReadB_0_0_9_0 = B + globalReadOffsetB_0_0_9_0;
  DATA_TYPE const *globalReadB_0_0_10_0 = B + globalReadOffsetB_0_0_10_0;
  DATA_TYPE const *globalReadB_0_0_11_0 = B + globalReadOffsetB_0_0_11_0;
  DATA_TYPE const *globalReadB_0_0_12_0 = B + globalReadOffsetB_0_0_12_0;
  DATA_TYPE const *globalReadB_0_0_13_0 = B + globalReadOffsetB_0_0_13_0;
  DATA_TYPE const *globalReadB_0_0_14_0 = B + globalReadOffsetB_0_0_14_0;
  DATA_TYPE const *globalReadB_0_0_15_0 = B + globalReadOffsetB_0_0_15_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 4*LSPB);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 5*LSPB);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 6*LSPB);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 7*LSPB);
  unsigned int localWriteOffsetB_0_0_8_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 8*LSPB);
  unsigned int localWriteOffsetB_0_0_9_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 9*LSPB);
  unsigned int localWriteOffsetB_0_0_10_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 10*LSPB);
  unsigned int localWriteOffsetB_0_0_11_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 11*LSPB);
  unsigned int localWriteOffsetB_0_0_12_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 12*LSPB);
  unsigned int localWriteOffsetB_0_0_13_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 13*LSPB);
  unsigned int localWriteOffsetB_0_0_14_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 14*LSPB);
  unsigned int localWriteOffsetB_0_0_15_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 15*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;
  DATA_TYPE *localWriteB_0_0_8_0;
  DATA_TYPE *localWriteB_0_0_9_0;
  DATA_TYPE *localWriteB_0_0_10_0;
  DATA_TYPE *localWriteB_0_0_11_0;
  DATA_TYPE *localWriteB_0_0_12_0;
  DATA_TYPE *localWriteB_0_0_13_0;
  DATA_TYPE *localWriteB_0_0_14_0;
  DATA_TYPE *localWriteB_0_0_15_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);
  localWriteB_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_8_0);
  localWriteB_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_9_0);
  localWriteB_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_10_0);
  localWriteB_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_11_0);
  localWriteB_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_12_0);
  localWriteB_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_13_0);
  localWriteB_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_14_0);
  localWriteB_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_15_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_8_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_9_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_10_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_11_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_12_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_13_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_14_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_15_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);
    b_0_0_8_0 = *(globalReadB_0_0_8_0 + 0);
    b_0_0_9_0 = *(globalReadB_0_0_9_0 + 0);
    b_0_0_10_0 = *(globalReadB_0_0_10_0 + 0);
    b_0_0_11_0 = *(globalReadB_0_0_11_0 + 0);
    b_0_0_12_0 = *(globalReadB_0_0_12_0 + 0);
    b_0_0_13_0 = *(globalReadB_0_0_13_0 + 0);
    b_0_0_14_0 = *(globalReadB_0_0_14_0 + 0);
    b_0_0_15_0 = *(globalReadB_0_0_15_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_8_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_8_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_9_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_9_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_10_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_10_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_11_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_11_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_12_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_12_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_13_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_13_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_14_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_14_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_15_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_15_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
    *(localWriteB_0_0_8_0 + 0) = b_0_0_8_0;
    *(localWriteB_0_0_9_0 + 0) = b_0_0_9_0;
    *(localWriteB_0_0_10_0 + 0) = b_0_0_10_0;
    *(localWriteB_0_0_11_0 + 0) = b_0_0_11_0;
    *(localWriteB_0_0_12_0 + 0) = b_0_0_12_0;
    *(localWriteB_0_0_13_0 + 0) = b_0_0_13_0;
    *(localWriteB_0_0_14_0 + 0) = b_0_0_14_0;
    *(localWriteB_0_0_15_0 + 0) = b_0_0_15_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x8


    /* iter 15 */

    MAC_2x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);
  b_0_0_8_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_8_0 + 0);
  b_0_0_9_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_9_0 + 0);
  b_0_0_10_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_10_0 + 0);
  b_0_0_11_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_11_0 + 0);
  b_0_0_12_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_12_0 + 0);
  b_0_0_13_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_13_0 + 0);
  b_0_0_14_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_14_0 + 0);
  b_0_0_15_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_15_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);
  localWriteB_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_8_0);
  localWriteB_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_9_0);
  localWriteB_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_10_0);
  localWriteB_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_11_0);
  localWriteB_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_12_0);
  localWriteB_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_13_0);
  localWriteB_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_14_0);
  localWriteB_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_15_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
  *(localWriteB_0_0_8_0 + 0) = b_0_0_8_0;
  *(localWriteB_0_0_9_0 + 0) = b_0_0_9_0;
  *(localWriteB_0_0_10_0 + 0) = b_0_0_10_0;
  *(localWriteB_0_0_11_0 + 0) = b_0_0_11_0;
  *(localWriteB_0_0_12_0 + 0) = b_0_0_12_0;
  *(localWriteB_0_0_13_0 + 0) = b_0_0_13_0;
  *(localWriteB_0_0_14_0 + 0) = b_0_0_14_0;
  *(localWriteB_0_0_15_0 + 0) = b_0_0_15_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x8
#undef MAC_2x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE TensileComplexFloat
#define DEST_DATA_TYPE TensileComplexFloat
#define COMPUTE_DATA_TYPE TensileComplexFloat
#define s0 x
#define s1 y
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) \
  DST.s0 = MAC(  MULA.s0,  MULB.s0, DST.s0 ); \
  DST.s0 = MAC(  MULA.s1, -MULB.s1, DST.s0 ); \
  DST.s1 = MAC(  MULA.s0, -MULB.s1, DST.s1 ); \
  DST.s1 = MAC( -MULA.s1,  MULB.s0, DST.s1 );
#define TYPE_MAC_WRITE( DST, SRC, ALPHA, REG, BETA ) \
  /* (1) */ \
  type_mac_tmp = REG.s0; \
  REG.s0 *= ALPHA.s0; \
  REG.s0 = MAC( -ALPHA.s1, REG.s1, REG.s0 ); \
  REG.s1 *= ALPHA.s0; \
  REG.s1 = MAC(  ALPHA.s1, type_mac_tmp, REG.s1 ); \
  /* (2) */ \
  if(BETA.s0 != 0) { \
  REG.s0 = MAC(  BETA.s0, SRC.s0, REG.s0 ); \
  REG.s1 = MAC(  BETA.s0, SRC.s1, REG.s1 ); \
  } \
  if (BETA.s1 != 0) { \
  REG.s0 = MAC( -BETA.s1, SRC.s1, REG.s0 ); \
  REG.s1 = MAC(  BETA.s1, SRC.s0, REG.s1 ); \
  } \
  /* (3) */ \
  DST = REG;

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_AlikC_BjlkC_CB_MT32x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW2_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  TensileComplexFloat *D,
  TensileComplexFloat const * __restrict__ C,
  TensileComplexFloat const * __restrict__ A,
  TensileComplexFloat const * __restrict__ B,
  TensileComplexFloat const alpha,
  TensileComplexFloat const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (TensileComplexFloat)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == tensile_complex<float>(0.0f)) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }



  float type_mac_tmp;


  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE TensileComplexFloat
#define DEST_DATA_TYPE TensileComplexFloat
#define COMPUTE_DATA_TYPE TensileComplexFloat
#define s0 x
#define s1 y
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) \
  DST.s0 = MAC(  MULA.s0,  MULB.s0, DST.s0 ); \
  DST.s0 = MAC(  MULA.s1, -MULB.s1, DST.s0 ); \
  DST.s1 = MAC(  MULA.s0, -MULB.s1, DST.s1 ); \
  DST.s1 = MAC( -MULA.s1,  MULB.s0, DST.s1 );
#define TYPE_MAC_WRITE( DST, SRC, ALPHA, REG, BETA ) \
  /* (1) */ \
  type_mac_tmp = REG.s0; \
  REG.s0 *= ALPHA.s0; \
  REG.s0 = MAC( -ALPHA.s1, REG.s1, REG.s0 ); \
  REG.s1 *= ALPHA.s0; \
  REG.s1 = MAC(  ALPHA.s1, type_mac_tmp, REG.s1 ); \
  /* (2) */ \
  if(BETA.s0 != 0) { \
  REG.s0 = MAC(  BETA.s0, SRC.s0, REG.s0 ); \
  REG.s1 = MAC(  BETA.s0, SRC.s1, REG.s1 ); \
  } \
  if (BETA.s1 != 0) { \
  REG.s0 = MAC( -BETA.s1, SRC.s1, REG.s0 ); \
  REG.s1 = MAC(  BETA.s1, SRC.s0, REG.s1 ); \
  } \
  /* (3) */ \
  DST = REG;

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_AlikC_BjlkC_CB_MT32x32x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW2_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  TensileComplexFloat *D,
  TensileComplexFloat const * __restrict__ C,
  TensileComplexFloat const * __restrict__ A,
  TensileComplexFloat const * __restrict__ B,
  TensileComplexFloat const alpha,
  TensileComplexFloat const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (TensileComplexFloat)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == tensile_complex<float>(0.0f)) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }



  float type_mac_tmp;


  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 819

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HBH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HBH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 819

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HBH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 3 */

    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 819

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HBH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 3 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HBH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 3 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 3 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HBH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HBH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HBH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 7 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HBH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 819

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HBH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 1) {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 2) {
        rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 819

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HBH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HBH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 819

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HBH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 3 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1792
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HBH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 3 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x8


    /* iter 3 (last unrolled loop) */

    MAC_4x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 1) {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 2) {
        rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1536

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HBH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 1) {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 2) {
        rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HBH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 3584
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HBH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 1) {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 2) {
        rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HBH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HBH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HBH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 7 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HBH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HBH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 7168
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HBH_MT64x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_1_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_1_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x8


    /* iter 15 (last unrolled loop) */

    MAC_4x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_1_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 1);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_1_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 1) {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 2) {
        rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HBH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 819

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HBH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1792
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HBH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I] , rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I] , rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I] , rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I] , rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I] , rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I] , rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I] , rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I] , rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I] , rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I] , rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I] , rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I] , rA[7],rB[7],rC[7+7*TT0I]); \

#define MAC_8x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I] , rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I] , rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I] , rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I] , rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[4+TT1J],rC[4+4*TT0I] , rA[5+TT0I],rB[4+TT1J],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[4+TT1J],rC[6+4*TT0I] , rA[7+TT0I],rB[4+TT1J],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[5+TT1J],rC[4+5*TT0I] , rA[5+TT0I],rB[5+TT1J],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[5+TT1J],rC[6+5*TT0I] , rA[7+TT0I],rB[5+TT1J],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[6+TT1J],rC[4+6*TT0I] , rA[5+TT0I],rB[6+TT1J],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[6+TT1J],rC[6+6*TT0I] , rA[7+TT0I],rB[6+TT1J],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[7+TT1J],rC[4+7*TT0I] , rA[5+TT0I],rB[7+TT1J],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[7+TT1J],rC[6+7*TT0I] , rA[7+TT0I],rB[7+TT1J],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HBH_MT128x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8


    /* iter 3 */

    MAC_8x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef MAC_8x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I] , rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I] , rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I] , rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I] , rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I] , rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I] , rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I] , rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I] , rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I] , rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I] , rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I] , rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I] , rA[7],rB[7],rC[7+7*TT0I]); \

#define MAC_8x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I] , rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I] , rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I] , rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I] , rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[4+TT1J],rC[4+4*TT0I] , rA[5+TT0I],rB[4+TT1J],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[4+TT1J],rC[6+4*TT0I] , rA[7+TT0I],rB[4+TT1J],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[5+TT1J],rC[4+5*TT0I] , rA[5+TT0I],rB[5+TT1J],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[5+TT1J],rC[6+5*TT0I] , rA[7+TT0I],rB[5+TT1J],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[6+TT1J],rC[4+6*TT0I] , rA[5+TT0I],rB[6+TT1J],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[6+TT1J],rC[6+6*TT0I] , rA[7+TT0I],rB[6+TT1J],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[7+TT1J],rC[4+7*TT0I] , rA[5+TT0I],rB[7+TT1J],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[7+TT1J],rC[6+7*TT0I] , rA[7+TT0I],rB[7+TT1J],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HBH_MT128x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8


    /* iter 3 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_8x8


    /* iter 3 (last unrolled loop) */

    MAC_8x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef MAC_8x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1792
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HBH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 3 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x8


    /* iter 3 (last unrolled loop) */

    MAC_4x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1536

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HBH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 3584
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HBH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1536

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HBH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 7 */

    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HBH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 8

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 3072

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HBH_MT64x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;
  unsigned int globalReadOffsetBL_4_0 = globalReadOffsetBL + 0 + 4*LSPB;
  unsigned int globalReadOffsetBL_5_0 = globalReadOffsetBL + 0 + 5*LSPB;
  unsigned int globalReadOffsetBL_6_0 = globalReadOffsetBL + 0 + 6*LSPB;
  unsigned int globalReadOffsetBL_7_0 = globalReadOffsetBL + 0 + 7*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_7_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 4*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 5*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 6*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 7*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HBH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HBH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HBH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_BSS_BH_MT32x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_bfloat16)(0)
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_BSS_BH_MT128x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_bfloat16)(0)
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;
  unsigned int flattenedOffsetA_4_0 = globalReadOffsetA0I + 0 + 4*LSPA;
  flattenedOffsetA_4_0 = (flattenedOffsetA_4_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_4_0;
  unsigned int globalReadOffsetA0I_4_0 = flattenedOffsetA_4_0;
  unsigned int flattenedOffsetA_5_0 = globalReadOffsetA0I + 0 + 5*LSPA;
  flattenedOffsetA_5_0 = (flattenedOffsetA_5_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_5_0;
  unsigned int globalReadOffsetA0I_5_0 = flattenedOffsetA_5_0;
  unsigned int flattenedOffsetA_6_0 = globalReadOffsetA0I + 0 + 6*LSPA;
  flattenedOffsetA_6_0 = (flattenedOffsetA_6_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_6_0;
  unsigned int globalReadOffsetA0I_6_0 = flattenedOffsetA_6_0;
  unsigned int flattenedOffsetA_7_0 = globalReadOffsetA0I + 0 + 7*LSPA;
  flattenedOffsetA_7_0 = (flattenedOffsetA_7_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_7_0;
  unsigned int globalReadOffsetA0I_7_0 = flattenedOffsetA_7_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 4*LSPA);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 5*LSPA);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 6*LSPA);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 7*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_BSS_BH_MT32x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_bfloat16)(0)
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_BSS_BH_MT16x16x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_bfloat16)(0)
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 819

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HBH_MT64x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 7 */

    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HBH_MT128x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_2_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_3_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_2_1;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_3_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_1 + 0) = a_0_1_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_1 + 0) = a_0_1_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HBH_MT64x16x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_1_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_1_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_1_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_1_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;
  unsigned int flattenedOffsetA_4_0 = globalReadOffsetA0I + 0 + 4*LSPA;
  flattenedOffsetA_4_0 = (flattenedOffsetA_4_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_4_0;
  unsigned int globalReadOffsetA0I_4_0 = flattenedOffsetA_4_0;
  unsigned int flattenedOffsetA_5_0 = globalReadOffsetA0I + 0 + 5*LSPA;
  flattenedOffsetA_5_0 = (flattenedOffsetA_5_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_5_0;
  unsigned int globalReadOffsetA0I_5_0 = flattenedOffsetA_5_0;
  unsigned int flattenedOffsetA_6_0 = globalReadOffsetA0I + 0 + 6*LSPA;
  flattenedOffsetA_6_0 = (flattenedOffsetA_6_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_6_0;
  unsigned int globalReadOffsetA0I_6_0 = flattenedOffsetA_6_0;
  unsigned int flattenedOffsetA_7_0 = globalReadOffsetA0I + 0 + 7*LSPA;
  flattenedOffsetA_7_0 = (flattenedOffsetA_7_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_7_0;
  unsigned int globalReadOffsetA0I_7_0 = flattenedOffsetA_7_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;
  globalReadOffsetA0I_4_0 = (  globalReadOffsetA0I_4_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_4_0;
  globalReadOffsetA0I_5_0 = (  globalReadOffsetA0I_5_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_5_0;
  globalReadOffsetA0I_6_0 = (  globalReadOffsetA0I_6_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_6_0;
  globalReadOffsetA0I_7_0 = (  globalReadOffsetA0I_7_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_7_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_2_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_3_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 4*LSPA);
  unsigned int localWriteOffsetA_0_0_4_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 4*LSPA);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 5*LSPA);
  unsigned int localWriteOffsetA_0_0_5_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 5*LSPA);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 6*LSPA);
  unsigned int localWriteOffsetA_0_0_6_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 6*LSPA);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 7*LSPA);
  unsigned int localWriteOffsetA_0_0_7_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 7*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_2_1;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_3_1;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_4_1;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_5_1;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_6_1;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_7_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_4_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_1);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_5_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_1);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_6_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_1);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_7_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_1_4_0 = *(globalReadA_0_0_4_0 + 1);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_1_5_0 = *(globalReadA_0_0_5_0 + 1);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_1_6_0 = *(globalReadA_0_0_6_0 + 1);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_1_7_0 = *(globalReadA_0_0_7_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_1 + 0) = a_0_1_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_4_1 + 0) = a_0_1_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_5_1 + 0) = a_0_1_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_6_1 + 0) = a_0_1_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_7_1 + 0) = a_0_1_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);
  a_0_0_4_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_1_4_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 1);
  a_0_0_5_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_1_5_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 1);
  a_0_0_6_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_1_6_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 1);
  a_0_0_7_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_1_7_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_4_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_1);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_5_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_1);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_6_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_1);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_7_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_1 + 0) = a_0_1_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_4_1 + 0) = a_0_1_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_5_1 + 0) = a_0_1_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_6_1 + 0) = a_0_1_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_7_1 + 0) = a_0_1_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HBH_MT128x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;
  unsigned int flattenedOffsetA_4_0 = globalReadOffsetA0I + 0 + 4*LSPA;
  flattenedOffsetA_4_0 = (flattenedOffsetA_4_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_4_0;
  unsigned int globalReadOffsetA0I_4_0 = flattenedOffsetA_4_0;
  unsigned int flattenedOffsetA_5_0 = globalReadOffsetA0I + 0 + 5*LSPA;
  flattenedOffsetA_5_0 = (flattenedOffsetA_5_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_5_0;
  unsigned int globalReadOffsetA0I_5_0 = flattenedOffsetA_5_0;
  unsigned int flattenedOffsetA_6_0 = globalReadOffsetA0I + 0 + 6*LSPA;
  flattenedOffsetA_6_0 = (flattenedOffsetA_6_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_6_0;
  unsigned int globalReadOffsetA0I_6_0 = flattenedOffsetA_6_0;
  unsigned int flattenedOffsetA_7_0 = globalReadOffsetA0I + 0 + 7*LSPA;
  flattenedOffsetA_7_0 = (flattenedOffsetA_7_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_7_0;
  unsigned int globalReadOffsetA0I_7_0 = flattenedOffsetA_7_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;
  globalReadOffsetA0I_4_0 = (  globalReadOffsetA0I_4_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_4_0;
  globalReadOffsetA0I_5_0 = (  globalReadOffsetA0I_5_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_5_0;
  globalReadOffsetA0I_6_0 = (  globalReadOffsetA0I_6_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_6_0;
  globalReadOffsetA0I_7_0 = (  globalReadOffsetA0I_7_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_7_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 4*LSPA);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 5*LSPA);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 6*LSPA);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 7*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 8

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x8 micro-tile */
#define MAC_2x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \

#define MAC_2x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HBH_MT32x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;
  unsigned int flattenedOffsetB_4_0 = globalReadOffsetB1J + 0 + 4*LSPB;
  flattenedOffsetB_4_0 = (flattenedOffsetB_4_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_4_0;
  unsigned int globalReadOffsetB1J_4_0 = flattenedOffsetB_4_0;
  unsigned int flattenedOffsetB_5_0 = globalReadOffsetB1J + 0 + 5*LSPB;
  flattenedOffsetB_5_0 = (flattenedOffsetB_5_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_5_0;
  unsigned int globalReadOffsetB1J_5_0 = flattenedOffsetB_5_0;
  unsigned int flattenedOffsetB_6_0 = globalReadOffsetB1J + 0 + 6*LSPB;
  flattenedOffsetB_6_0 = (flattenedOffsetB_6_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_6_0;
  unsigned int globalReadOffsetB1J_6_0 = flattenedOffsetB_6_0;
  unsigned int flattenedOffsetB_7_0 = globalReadOffsetB1J + 0 + 7*LSPB;
  flattenedOffsetB_7_0 = (flattenedOffsetB_7_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_7_0;
  unsigned int globalReadOffsetB1J_7_0 = flattenedOffsetB_7_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;
  globalReadOffsetB1J_4_0 = (  globalReadOffsetB1J_4_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_4_0;
  globalReadOffsetB1J_5_0 = (  globalReadOffsetB1J_5_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_5_0;
  globalReadOffsetB1J_6_0 = (  globalReadOffsetB1J_6_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_6_0;
  globalReadOffsetB1J_7_0 = (  globalReadOffsetB1J_7_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_7_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_7_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 4*LSPB);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 5*LSPB);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 6*LSPB);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 7*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x8


    /* iter 15 */

    MAC_2x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x8
#undef MAC_2x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 16
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HBH_MT64x16x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;
  unsigned int flattenedOffsetA_4_0 = globalReadOffsetA0I + 0 + 4*LSPA;
  flattenedOffsetA_4_0 = (flattenedOffsetA_4_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_4_0;
  unsigned int globalReadOffsetA0I_4_0 = flattenedOffsetA_4_0;
  unsigned int flattenedOffsetA_5_0 = globalReadOffsetA0I + 0 + 5*LSPA;
  flattenedOffsetA_5_0 = (flattenedOffsetA_5_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_5_0;
  unsigned int globalReadOffsetA0I_5_0 = flattenedOffsetA_5_0;
  unsigned int flattenedOffsetA_6_0 = globalReadOffsetA0I + 0 + 6*LSPA;
  flattenedOffsetA_6_0 = (flattenedOffsetA_6_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_6_0;
  unsigned int globalReadOffsetA0I_6_0 = flattenedOffsetA_6_0;
  unsigned int flattenedOffsetA_7_0 = globalReadOffsetA0I + 0 + 7*LSPA;
  flattenedOffsetA_7_0 = (flattenedOffsetA_7_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_7_0;
  unsigned int globalReadOffsetA0I_7_0 = flattenedOffsetA_7_0;
  unsigned int flattenedOffsetA_8_0 = globalReadOffsetA0I + 0 + 8*LSPA;
  flattenedOffsetA_8_0 = (flattenedOffsetA_8_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_8_0;
  unsigned int globalReadOffsetA0I_8_0 = flattenedOffsetA_8_0;
  unsigned int flattenedOffsetA_9_0 = globalReadOffsetA0I + 0 + 9*LSPA;
  flattenedOffsetA_9_0 = (flattenedOffsetA_9_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_9_0;
  unsigned int globalReadOffsetA0I_9_0 = flattenedOffsetA_9_0;
  unsigned int flattenedOffsetA_10_0 = globalReadOffsetA0I + 0 + 10*LSPA;
  flattenedOffsetA_10_0 = (flattenedOffsetA_10_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_10_0;
  unsigned int globalReadOffsetA0I_10_0 = flattenedOffsetA_10_0;
  unsigned int flattenedOffsetA_11_0 = globalReadOffsetA0I + 0 + 11*LSPA;
  flattenedOffsetA_11_0 = (flattenedOffsetA_11_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_11_0;
  unsigned int globalReadOffsetA0I_11_0 = flattenedOffsetA_11_0;
  unsigned int flattenedOffsetA_12_0 = globalReadOffsetA0I + 0 + 12*LSPA;
  flattenedOffsetA_12_0 = (flattenedOffsetA_12_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_12_0;
  unsigned int globalReadOffsetA0I_12_0 = flattenedOffsetA_12_0;
  unsigned int flattenedOffsetA_13_0 = globalReadOffsetA0I + 0 + 13*LSPA;
  flattenedOffsetA_13_0 = (flattenedOffsetA_13_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_13_0;
  unsigned int globalReadOffsetA0I_13_0 = flattenedOffsetA_13_0;
  unsigned int flattenedOffsetA_14_0 = globalReadOffsetA0I + 0 + 14*LSPA;
  flattenedOffsetA_14_0 = (flattenedOffsetA_14_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_14_0;
  unsigned int globalReadOffsetA0I_14_0 = flattenedOffsetA_14_0;
  unsigned int flattenedOffsetA_15_0 = globalReadOffsetA0I + 0 + 15*LSPA;
  flattenedOffsetA_15_0 = (flattenedOffsetA_15_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_15_0;
  unsigned int globalReadOffsetA0I_15_0 = flattenedOffsetA_15_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;
  globalReadOffsetA0I_4_0 = (  globalReadOffsetA0I_4_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_4_0;
  globalReadOffsetA0I_5_0 = (  globalReadOffsetA0I_5_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_5_0;
  globalReadOffsetA0I_6_0 = (  globalReadOffsetA0I_6_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_6_0;
  globalReadOffsetA0I_7_0 = (  globalReadOffsetA0I_7_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_7_0;
  globalReadOffsetA0I_8_0 = (  globalReadOffsetA0I_8_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_8_0;
  globalReadOffsetA0I_9_0 = (  globalReadOffsetA0I_9_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_9_0;
  globalReadOffsetA0I_10_0 = (  globalReadOffsetA0I_10_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_10_0;
  globalReadOffsetA0I_11_0 = (  globalReadOffsetA0I_11_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_11_0;
  globalReadOffsetA0I_12_0 = (  globalReadOffsetA0I_12_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_12_0;
  globalReadOffsetA0I_13_0 = (  globalReadOffsetA0I_13_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_13_0;
  globalReadOffsetA0I_14_0 = (  globalReadOffsetA0I_14_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_14_0;
  globalReadOffsetA0I_15_0 = (  globalReadOffsetA0I_15_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_15_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_15_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 4*LSPA);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 5*LSPA);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 6*LSPA);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 7*LSPA);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 8*LSPA);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 9*LSPA);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 10*LSPA);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 11*LSPA);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 12*LSPA);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 13*LSPA);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 14*LSPA);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 15*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_0_0_15_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_8_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_9_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_10_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_11_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_12_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_13_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_14_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_15_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_8_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_9_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_10_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_11_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_12_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_13_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_14_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_15_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_0_8_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_0_0_9_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_0_0_10_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_0_0_11_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_0_0_12_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_0_0_13_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_0_0_14_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_0_0_15_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 819

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HBH_MT64x32x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 7 */

    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HBH_MT128x32x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_2_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_3_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_2_1;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_3_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_1 + 0) = a_0_1_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_1 + 0) = a_0_1_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HBH_MT64x16x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_1_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_1_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_1_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_1_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;
  unsigned int flattenedOffsetA_4_0 = globalReadOffsetA0I + 0 + 4*LSPA;
  flattenedOffsetA_4_0 = (flattenedOffsetA_4_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_4_0;
  unsigned int globalReadOffsetA0I_4_0 = flattenedOffsetA_4_0;
  unsigned int flattenedOffsetA_5_0 = globalReadOffsetA0I + 0 + 5*LSPA;
  flattenedOffsetA_5_0 = (flattenedOffsetA_5_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_5_0;
  unsigned int globalReadOffsetA0I_5_0 = flattenedOffsetA_5_0;
  unsigned int flattenedOffsetA_6_0 = globalReadOffsetA0I + 0 + 6*LSPA;
  flattenedOffsetA_6_0 = (flattenedOffsetA_6_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_6_0;
  unsigned int globalReadOffsetA0I_6_0 = flattenedOffsetA_6_0;
  unsigned int flattenedOffsetA_7_0 = globalReadOffsetA0I + 0 + 7*LSPA;
  flattenedOffsetA_7_0 = (flattenedOffsetA_7_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_7_0;
  unsigned int globalReadOffsetA0I_7_0 = flattenedOffsetA_7_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;
  globalReadOffsetA0I_4_0 = (  globalReadOffsetA0I_4_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_4_0;
  globalReadOffsetA0I_5_0 = (  globalReadOffsetA0I_5_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_5_0;
  globalReadOffsetA0I_6_0 = (  globalReadOffsetA0I_6_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_6_0;
  globalReadOffsetA0I_7_0 = (  globalReadOffsetA0I_7_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_7_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_2_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_3_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 4*LSPA);
  unsigned int localWriteOffsetA_0_0_4_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 4*LSPA);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 5*LSPA);
  unsigned int localWriteOffsetA_0_0_5_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 5*LSPA);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 6*LSPA);
  unsigned int localWriteOffsetA_0_0_6_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 6*LSPA);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 7*LSPA);
  unsigned int localWriteOffsetA_0_0_7_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 7*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_2_1;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_3_1;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_4_1;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_5_1;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_6_1;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_7_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_4_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_1);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_5_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_1);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_6_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_1);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_7_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_1_4_0 = *(globalReadA_0_0_4_0 + 1);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_1_5_0 = *(globalReadA_0_0_5_0 + 1);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_1_6_0 = *(globalReadA_0_0_6_0 + 1);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_1_7_0 = *(globalReadA_0_0_7_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_1 + 0) = a_0_1_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_4_1 + 0) = a_0_1_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_5_1 + 0) = a_0_1_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_6_1 + 0) = a_0_1_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_7_1 + 0) = a_0_1_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);
  a_0_0_4_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_1_4_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 1);
  a_0_0_5_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_1_5_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 1);
  a_0_0_6_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_1_6_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 1);
  a_0_0_7_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_1_7_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_4_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_1);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_5_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_1);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_6_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_1);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_7_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_1 + 0) = a_0_1_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_4_1 + 0) = a_0_1_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_5_1 + 0) = a_0_1_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_6_1 + 0) = a_0_1_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_7_1 + 0) = a_0_1_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HBH_MT128x32x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;
  unsigned int flattenedOffsetA_4_0 = globalReadOffsetA0I + 0 + 4*LSPA;
  flattenedOffsetA_4_0 = (flattenedOffsetA_4_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_4_0;
  unsigned int globalReadOffsetA0I_4_0 = flattenedOffsetA_4_0;
  unsigned int flattenedOffsetA_5_0 = globalReadOffsetA0I + 0 + 5*LSPA;
  flattenedOffsetA_5_0 = (flattenedOffsetA_5_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_5_0;
  unsigned int globalReadOffsetA0I_5_0 = flattenedOffsetA_5_0;
  unsigned int flattenedOffsetA_6_0 = globalReadOffsetA0I + 0 + 6*LSPA;
  flattenedOffsetA_6_0 = (flattenedOffsetA_6_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_6_0;
  unsigned int globalReadOffsetA0I_6_0 = flattenedOffsetA_6_0;
  unsigned int flattenedOffsetA_7_0 = globalReadOffsetA0I + 0 + 7*LSPA;
  flattenedOffsetA_7_0 = (flattenedOffsetA_7_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_7_0;
  unsigned int globalReadOffsetA0I_7_0 = flattenedOffsetA_7_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;
  globalReadOffsetA0I_4_0 = (  globalReadOffsetA0I_4_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_4_0;
  globalReadOffsetA0I_5_0 = (  globalReadOffsetA0I_5_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_5_0;
  globalReadOffsetA0I_6_0 = (  globalReadOffsetA0I_6_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_6_0;
  globalReadOffsetA0I_7_0 = (  globalReadOffsetA0I_7_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_7_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 4*LSPA);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 5*LSPA);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 6*LSPA);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 7*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 8

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x8 micro-tile */
#define MAC_2x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \

#define MAC_2x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HBH_MT32x128x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;
  unsigned int flattenedOffsetB_4_0 = globalReadOffsetB1J + 0 + 4*LSPB;
  flattenedOffsetB_4_0 = (flattenedOffsetB_4_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_4_0;
  unsigned int globalReadOffsetB1J_4_0 = flattenedOffsetB_4_0;
  unsigned int flattenedOffsetB_5_0 = globalReadOffsetB1J + 0 + 5*LSPB;
  flattenedOffsetB_5_0 = (flattenedOffsetB_5_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_5_0;
  unsigned int globalReadOffsetB1J_5_0 = flattenedOffsetB_5_0;
  unsigned int flattenedOffsetB_6_0 = globalReadOffsetB1J + 0 + 6*LSPB;
  flattenedOffsetB_6_0 = (flattenedOffsetB_6_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_6_0;
  unsigned int globalReadOffsetB1J_6_0 = flattenedOffsetB_6_0;
  unsigned int flattenedOffsetB_7_0 = globalReadOffsetB1J + 0 + 7*LSPB;
  flattenedOffsetB_7_0 = (flattenedOffsetB_7_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_7_0;
  unsigned int globalReadOffsetB1J_7_0 = flattenedOffsetB_7_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;
  globalReadOffsetB1J_4_0 = (  globalReadOffsetB1J_4_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_4_0;
  globalReadOffsetB1J_5_0 = (  globalReadOffsetB1J_5_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_5_0;
  globalReadOffsetB1J_6_0 = (  globalReadOffsetB1J_6_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_6_0;
  globalReadOffsetB1J_7_0 = (  globalReadOffsetB1J_7_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_7_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_7_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 4*LSPB);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 5*LSPB);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 6*LSPB);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 7*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x8


    /* iter 15 */

    MAC_2x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x8
#undef MAC_2x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 16
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HBH_MT64x16x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;
  unsigned int flattenedOffsetA_4_0 = globalReadOffsetA0I + 0 + 4*LSPA;
  flattenedOffsetA_4_0 = (flattenedOffsetA_4_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_4_0;
  unsigned int globalReadOffsetA0I_4_0 = flattenedOffsetA_4_0;
  unsigned int flattenedOffsetA_5_0 = globalReadOffsetA0I + 0 + 5*LSPA;
  flattenedOffsetA_5_0 = (flattenedOffsetA_5_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_5_0;
  unsigned int globalReadOffsetA0I_5_0 = flattenedOffsetA_5_0;
  unsigned int flattenedOffsetA_6_0 = globalReadOffsetA0I + 0 + 6*LSPA;
  flattenedOffsetA_6_0 = (flattenedOffsetA_6_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_6_0;
  unsigned int globalReadOffsetA0I_6_0 = flattenedOffsetA_6_0;
  unsigned int flattenedOffsetA_7_0 = globalReadOffsetA0I + 0 + 7*LSPA;
  flattenedOffsetA_7_0 = (flattenedOffsetA_7_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_7_0;
  unsigned int globalReadOffsetA0I_7_0 = flattenedOffsetA_7_0;
  unsigned int flattenedOffsetA_8_0 = globalReadOffsetA0I + 0 + 8*LSPA;
  flattenedOffsetA_8_0 = (flattenedOffsetA_8_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_8_0;
  unsigned int globalReadOffsetA0I_8_0 = flattenedOffsetA_8_0;
  unsigned int flattenedOffsetA_9_0 = globalReadOffsetA0I + 0 + 9*LSPA;
  flattenedOffsetA_9_0 = (flattenedOffsetA_9_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_9_0;
  unsigned int globalReadOffsetA0I_9_0 = flattenedOffsetA_9_0;
  unsigned int flattenedOffsetA_10_0 = globalReadOffsetA0I + 0 + 10*LSPA;
  flattenedOffsetA_10_0 = (flattenedOffsetA_10_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_10_0;
  unsigned int globalReadOffsetA0I_10_0 = flattenedOffsetA_10_0;
  unsigned int flattenedOffsetA_11_0 = globalReadOffsetA0I + 0 + 11*LSPA;
  flattenedOffsetA_11_0 = (flattenedOffsetA_11_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_11_0;
  unsigned int globalReadOffsetA0I_11_0 = flattenedOffsetA_11_0;
  unsigned int flattenedOffsetA_12_0 = globalReadOffsetA0I + 0 + 12*LSPA;
  flattenedOffsetA_12_0 = (flattenedOffsetA_12_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_12_0;
  unsigned int globalReadOffsetA0I_12_0 = flattenedOffsetA_12_0;
  unsigned int flattenedOffsetA_13_0 = globalReadOffsetA0I + 0 + 13*LSPA;
  flattenedOffsetA_13_0 = (flattenedOffsetA_13_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_13_0;
  unsigned int globalReadOffsetA0I_13_0 = flattenedOffsetA_13_0;
  unsigned int flattenedOffsetA_14_0 = globalReadOffsetA0I + 0 + 14*LSPA;
  flattenedOffsetA_14_0 = (flattenedOffsetA_14_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_14_0;
  unsigned int globalReadOffsetA0I_14_0 = flattenedOffsetA_14_0;
  unsigned int flattenedOffsetA_15_0 = globalReadOffsetA0I + 0 + 15*LSPA;
  flattenedOffsetA_15_0 = (flattenedOffsetA_15_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_15_0;
  unsigned int globalReadOffsetA0I_15_0 = flattenedOffsetA_15_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;
  globalReadOffsetA0I_4_0 = (  globalReadOffsetA0I_4_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_4_0;
  globalReadOffsetA0I_5_0 = (  globalReadOffsetA0I_5_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_5_0;
  globalReadOffsetA0I_6_0 = (  globalReadOffsetA0I_6_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_6_0;
  globalReadOffsetA0I_7_0 = (  globalReadOffsetA0I_7_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_7_0;
  globalReadOffsetA0I_8_0 = (  globalReadOffsetA0I_8_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_8_0;
  globalReadOffsetA0I_9_0 = (  globalReadOffsetA0I_9_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_9_0;
  globalReadOffsetA0I_10_0 = (  globalReadOffsetA0I_10_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_10_0;
  globalReadOffsetA0I_11_0 = (  globalReadOffsetA0I_11_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_11_0;
  globalReadOffsetA0I_12_0 = (  globalReadOffsetA0I_12_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_12_0;
  globalReadOffsetA0I_13_0 = (  globalReadOffsetA0I_13_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_13_0;
  globalReadOffsetA0I_14_0 = (  globalReadOffsetA0I_14_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_14_0;
  globalReadOffsetA0I_15_0 = (  globalReadOffsetA0I_15_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_15_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_15_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 4*LSPA);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 5*LSPA);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 6*LSPA);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 7*LSPA);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 8*LSPA);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 9*LSPA);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 10*LSPA);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 11*LSPA);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 12*LSPA);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 13*LSPA);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 14*LSPA);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 15*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_0_0_15_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_8_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_9_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_10_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_11_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_12_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_13_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_14_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_15_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_8_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_9_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_10_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_11_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_12_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_13_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_14_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_15_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_0_8_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_0_0_9_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_0_0_10_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_0_0_11_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_0_0_12_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_0_0_13_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_0_0_14_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_0_0_15_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 768

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 7 */

    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 768

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 7 */

    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT128x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_2_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_3_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_2_1;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_3_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_1 + 0) = a_0_1_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_1 + 0) = a_0_1_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 16
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 16

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 2304

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 2x16 micro-tile */
#define MAC_2x16\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[0],rB[8],rC[0+8*TT0I] , rA[1],rB[8],rC[1+8*TT0I]); \
  TYPE_MAC(rA[0],rB[9],rC[0+9*TT0I] , rA[1],rB[9],rC[1+9*TT0I]); \
  TYPE_MAC(rA[0],rB[10],rC[0+10*TT0I] , rA[1],rB[10],rC[1+10*TT0I]); \
  TYPE_MAC(rA[0],rB[11],rC[0+11*TT0I] , rA[1],rB[11],rC[1+11*TT0I]); \
  TYPE_MAC(rA[0],rB[12],rC[0+12*TT0I] , rA[1],rB[12],rC[1+12*TT0I]); \
  TYPE_MAC(rA[0],rB[13],rC[0+13*TT0I] , rA[1],rB[13],rC[1+13*TT0I]); \
  TYPE_MAC(rA[0],rB[14],rC[0+14*TT0I] , rA[1],rB[14],rC[1+14*TT0I]); \
  TYPE_MAC(rA[0],rB[15],rC[0+15*TT0I] , rA[1],rB[15],rC[1+15*TT0I]); \

#define MAC_2x16_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[8+TT1J],rC[0+8*TT0I] , rA[1+TT0I],rB[8+TT1J],rC[1+8*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[9+TT1J],rC[0+9*TT0I] , rA[1+TT0I],rB[9+TT1J],rC[1+9*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[10+TT1J],rC[0+10*TT0I] , rA[1+TT0I],rB[10+TT1J],rC[1+10*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[11+TT1J],rC[0+11*TT0I] , rA[1+TT0I],rB[11+TT1J],rC[1+11*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[12+TT1J],rC[0+12*TT0I] , rA[1+TT0I],rB[12+TT1J],rC[1+12*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[13+TT1J],rC[0+13*TT0I] , rA[1+TT0I],rB[13+TT1J],rC[1+13*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[14+TT1J],rC[0+14*TT0I] , rA[1+TT0I],rB[14+TT1J],rC[1+14*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[15+TT1J],rC[0+15*TT0I] , rA[1+TT0I],rB[15+TT1J],rC[1+15*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT16x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_16_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_1_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_1_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_1_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_1_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_1_6_0;
  DATA_TYPE b_0_0_7_0;
  DATA_TYPE b_0_1_7_0;
  DATA_TYPE b_0_0_8_0;
  DATA_TYPE b_0_1_8_0;
  DATA_TYPE b_0_0_9_0;
  DATA_TYPE b_0_1_9_0;
  DATA_TYPE b_0_0_10_0;
  DATA_TYPE b_0_1_10_0;
  DATA_TYPE b_0_0_11_0;
  DATA_TYPE b_0_1_11_0;
  DATA_TYPE b_0_0_12_0;
  DATA_TYPE b_0_1_12_0;
  DATA_TYPE b_0_0_13_0;
  DATA_TYPE b_0_1_13_0;
  DATA_TYPE b_0_0_14_0;
  DATA_TYPE b_0_1_14_0;
  DATA_TYPE b_0_0_15_0;
  DATA_TYPE b_0_1_15_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;
  unsigned int flattenedOffsetB_4_0 = globalReadOffsetB1J + 0 + 4*LSPB;
  flattenedOffsetB_4_0 = (flattenedOffsetB_4_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_4_0;
  unsigned int globalReadOffsetB1J_4_0 = flattenedOffsetB_4_0;
  unsigned int flattenedOffsetB_5_0 = globalReadOffsetB1J + 0 + 5*LSPB;
  flattenedOffsetB_5_0 = (flattenedOffsetB_5_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_5_0;
  unsigned int globalReadOffsetB1J_5_0 = flattenedOffsetB_5_0;
  unsigned int flattenedOffsetB_6_0 = globalReadOffsetB1J + 0 + 6*LSPB;
  flattenedOffsetB_6_0 = (flattenedOffsetB_6_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_6_0;
  unsigned int globalReadOffsetB1J_6_0 = flattenedOffsetB_6_0;
  unsigned int flattenedOffsetB_7_0 = globalReadOffsetB1J + 0 + 7*LSPB;
  flattenedOffsetB_7_0 = (flattenedOffsetB_7_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_7_0;
  unsigned int globalReadOffsetB1J_7_0 = flattenedOffsetB_7_0;
  unsigned int flattenedOffsetB_8_0 = globalReadOffsetB1J + 0 + 8*LSPB;
  flattenedOffsetB_8_0 = (flattenedOffsetB_8_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_8_0;
  unsigned int globalReadOffsetB1J_8_0 = flattenedOffsetB_8_0;
  unsigned int flattenedOffsetB_9_0 = globalReadOffsetB1J + 0 + 9*LSPB;
  flattenedOffsetB_9_0 = (flattenedOffsetB_9_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_9_0;
  unsigned int globalReadOffsetB1J_9_0 = flattenedOffsetB_9_0;
  unsigned int flattenedOffsetB_10_0 = globalReadOffsetB1J + 0 + 10*LSPB;
  flattenedOffsetB_10_0 = (flattenedOffsetB_10_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_10_0;
  unsigned int globalReadOffsetB1J_10_0 = flattenedOffsetB_10_0;
  unsigned int flattenedOffsetB_11_0 = globalReadOffsetB1J + 0 + 11*LSPB;
  flattenedOffsetB_11_0 = (flattenedOffsetB_11_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_11_0;
  unsigned int globalReadOffsetB1J_11_0 = flattenedOffsetB_11_0;
  unsigned int flattenedOffsetB_12_0 = globalReadOffsetB1J + 0 + 12*LSPB;
  flattenedOffsetB_12_0 = (flattenedOffsetB_12_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_12_0;
  unsigned int globalReadOffsetB1J_12_0 = flattenedOffsetB_12_0;
  unsigned int flattenedOffsetB_13_0 = globalReadOffsetB1J + 0 + 13*LSPB;
  flattenedOffsetB_13_0 = (flattenedOffsetB_13_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_13_0;
  unsigned int globalReadOffsetB1J_13_0 = flattenedOffsetB_13_0;
  unsigned int flattenedOffsetB_14_0 = globalReadOffsetB1J + 0 + 14*LSPB;
  flattenedOffsetB_14_0 = (flattenedOffsetB_14_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_14_0;
  unsigned int globalReadOffsetB1J_14_0 = flattenedOffsetB_14_0;
  unsigned int flattenedOffsetB_15_0 = globalReadOffsetB1J + 0 + 15*LSPB;
  flattenedOffsetB_15_0 = (flattenedOffsetB_15_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_15_0;
  unsigned int globalReadOffsetB1J_15_0 = flattenedOffsetB_15_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_7_0), wgK );
  int64_t globalReadOffsetB_0_0_8_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_8_0), wgK );
  int64_t globalReadOffsetB_0_0_9_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_9_0), wgK );
  int64_t globalReadOffsetB_0_0_10_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_10_0), wgK );
  int64_t globalReadOffsetB_0_0_11_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_11_0), wgK );
  int64_t globalReadOffsetB_0_0_12_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_12_0), wgK );
  int64_t globalReadOffsetB_0_0_13_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_13_0), wgK );
  int64_t globalReadOffsetB_0_0_14_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_14_0), wgK );
  int64_t globalReadOffsetB_0_0_15_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_15_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;
  DATA_TYPE const *globalReadB_0_0_8_0 = B + globalReadOffsetB_0_0_8_0;
  DATA_TYPE const *globalReadB_0_0_9_0 = B + globalReadOffsetB_0_0_9_0;
  DATA_TYPE const *globalReadB_0_0_10_0 = B + globalReadOffsetB_0_0_10_0;
  DATA_TYPE const *globalReadB_0_0_11_0 = B + globalReadOffsetB_0_0_11_0;
  DATA_TYPE const *globalReadB_0_0_12_0 = B + globalReadOffsetB_0_0_12_0;
  DATA_TYPE const *globalReadB_0_0_13_0 = B + globalReadOffsetB_0_0_13_0;
  DATA_TYPE const *globalReadB_0_0_14_0 = B + globalReadOffsetB_0_0_14_0;
  DATA_TYPE const *globalReadB_0_0_15_0 = B + globalReadOffsetB_0_0_15_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_2_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_3_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 4*LSPB);
  unsigned int localWriteOffsetB_0_0_4_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 4*LSPB);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 5*LSPB);
  unsigned int localWriteOffsetB_0_0_5_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 5*LSPB);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 6*LSPB);
  unsigned int localWriteOffsetB_0_0_6_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 6*LSPB);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 7*LSPB);
  unsigned int localWriteOffsetB_0_0_7_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 7*LSPB);
  unsigned int localWriteOffsetB_0_0_8_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 8*LSPB);
  unsigned int localWriteOffsetB_0_0_8_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 8*LSPB);
  unsigned int localWriteOffsetB_0_0_9_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 9*LSPB);
  unsigned int localWriteOffsetB_0_0_9_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 9*LSPB);
  unsigned int localWriteOffsetB_0_0_10_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 10*LSPB);
  unsigned int localWriteOffsetB_0_0_10_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 10*LSPB);
  unsigned int localWriteOffsetB_0_0_11_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 11*LSPB);
  unsigned int localWriteOffsetB_0_0_11_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 11*LSPB);
  unsigned int localWriteOffsetB_0_0_12_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 12*LSPB);
  unsigned int localWriteOffsetB_0_0_12_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 12*LSPB);
  unsigned int localWriteOffsetB_0_0_13_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 13*LSPB);
  unsigned int localWriteOffsetB_0_0_13_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 13*LSPB);
  unsigned int localWriteOffsetB_0_0_14_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 14*LSPB);
  unsigned int localWriteOffsetB_0_0_14_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 14*LSPB);
  unsigned int localWriteOffsetB_0_0_15_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 15*LSPB);
  unsigned int localWriteOffsetB_0_0_15_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 15*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_2_1;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_3_1;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_4_1;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_5_1;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_6_1;
  DATA_TYPE *localWriteB_0_0_7_0;
  DATA_TYPE *localWriteB_0_0_7_1;
  DATA_TYPE *localWriteB_0_0_8_0;
  DATA_TYPE *localWriteB_0_0_8_1;
  DATA_TYPE *localWriteB_0_0_9_0;
  DATA_TYPE *localWriteB_0_0_9_1;
  DATA_TYPE *localWriteB_0_0_10_0;
  DATA_TYPE *localWriteB_0_0_10_1;
  DATA_TYPE *localWriteB_0_0_11_0;
  DATA_TYPE *localWriteB_0_0_11_1;
  DATA_TYPE *localWriteB_0_0_12_0;
  DATA_TYPE *localWriteB_0_0_12_1;
  DATA_TYPE *localWriteB_0_0_13_0;
  DATA_TYPE *localWriteB_0_0_13_1;
  DATA_TYPE *localWriteB_0_0_14_0;
  DATA_TYPE *localWriteB_0_0_14_1;
  DATA_TYPE *localWriteB_0_0_15_0;
  DATA_TYPE *localWriteB_0_0_15_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_1);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_1);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_4_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_1);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_5_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_1);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_6_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_1);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);
  localWriteB_0_0_7_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_1);
  localWriteB_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_8_0);
  localWriteB_0_0_8_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_8_1);
  localWriteB_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_9_0);
  localWriteB_0_0_9_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_9_1);
  localWriteB_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_10_0);
  localWriteB_0_0_10_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_10_1);
  localWriteB_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_11_0);
  localWriteB_0_0_11_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_11_1);
  localWriteB_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_12_0);
  localWriteB_0_0_12_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_12_1);
  localWriteB_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_13_0);
  localWriteB_0_0_13_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_13_1);
  localWriteB_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_14_0);
  localWriteB_0_0_14_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_14_1);
  localWriteB_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_15_0);
  localWriteB_0_0_15_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_15_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_8_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_9_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_10_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_11_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_12_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_13_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_14_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_15_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_1_4_0 = *(globalReadB_0_0_4_0 + 1);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_1_5_0 = *(globalReadB_0_0_5_0 + 1);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_1_6_0 = *(globalReadB_0_0_6_0 + 1);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);
    b_0_1_7_0 = *(globalReadB_0_0_7_0 + 1);
    b_0_0_8_0 = *(globalReadB_0_0_8_0 + 0);
    b_0_1_8_0 = *(globalReadB_0_0_8_0 + 1);
    b_0_0_9_0 = *(globalReadB_0_0_9_0 + 0);
    b_0_1_9_0 = *(globalReadB_0_0_9_0 + 1);
    b_0_0_10_0 = *(globalReadB_0_0_10_0 + 0);
    b_0_1_10_0 = *(globalReadB_0_0_10_0 + 1);
    b_0_0_11_0 = *(globalReadB_0_0_11_0 + 0);
    b_0_1_11_0 = *(globalReadB_0_0_11_0 + 1);
    b_0_0_12_0 = *(globalReadB_0_0_12_0 + 0);
    b_0_1_12_0 = *(globalReadB_0_0_12_0 + 1);
    b_0_0_13_0 = *(globalReadB_0_0_13_0 + 0);
    b_0_1_13_0 = *(globalReadB_0_0_13_0 + 1);
    b_0_0_14_0 = *(globalReadB_0_0_14_0 + 0);
    b_0_1_14_0 = *(globalReadB_0_0_14_0 + 1);
    b_0_0_15_0 = *(globalReadB_0_0_15_0 + 0);
    b_0_1_15_0 = *(globalReadB_0_0_15_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_8_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_8_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_9_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_9_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_10_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_10_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_11_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_11_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_12_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_12_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_13_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_13_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_14_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_14_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_15_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_15_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_1 + 0) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_1 + 0) = b_0_1_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_4_1 + 0) = b_0_1_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_5_1 + 0) = b_0_1_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_6_1 + 0) = b_0_1_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
    *(localWriteB_0_0_7_1 + 0) = b_0_1_7_0;
    *(localWriteB_0_0_8_0 + 0) = b_0_0_8_0;
    *(localWriteB_0_0_8_1 + 0) = b_0_1_8_0;
    *(localWriteB_0_0_9_0 + 0) = b_0_0_9_0;
    *(localWriteB_0_0_9_1 + 0) = b_0_1_9_0;
    *(localWriteB_0_0_10_0 + 0) = b_0_0_10_0;
    *(localWriteB_0_0_10_1 + 0) = b_0_1_10_0;
    *(localWriteB_0_0_11_0 + 0) = b_0_0_11_0;
    *(localWriteB_0_0_11_1 + 0) = b_0_1_11_0;
    *(localWriteB_0_0_12_0 + 0) = b_0_0_12_0;
    *(localWriteB_0_0_12_1 + 0) = b_0_1_12_0;
    *(localWriteB_0_0_13_0 + 0) = b_0_0_13_0;
    *(localWriteB_0_0_13_1 + 0) = b_0_1_13_0;
    *(localWriteB_0_0_14_0 + 0) = b_0_0_14_0;
    *(localWriteB_0_0_14_1 + 0) = b_0_1_14_0;
    *(localWriteB_0_0_15_0 + 0) = b_0_0_15_0;
    *(localWriteB_0_0_15_1 + 0) = b_0_1_15_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+1] = localReadB[4*SG1J*VECTOR_WIDTH + 1]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+1] = localReadB[5*SG1J*VECTOR_WIDTH + 1]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+1] = localReadB[6*SG1J*VECTOR_WIDTH + 1]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+1] = localReadB[7*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+1+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 1]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+1+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 1]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+1+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 1]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+1+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x16


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+1] = localReadB[4*SG1J*VECTOR_WIDTH + 1]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+1] = localReadB[5*SG1J*VECTOR_WIDTH + 1]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+1] = localReadB[6*SG1J*VECTOR_WIDTH + 1]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+1] = localReadB[7*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x16_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+1+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 1]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+1+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 1]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+1+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 1]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+1+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x16


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+1] = localReadB[4*SG1J*VECTOR_WIDTH + 1]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+1] = localReadB[5*SG1J*VECTOR_WIDTH + 1]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+1] = localReadB[6*SG1J*VECTOR_WIDTH + 1]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+1] = localReadB[7*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x16_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+1+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 1]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+1+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 1]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+1+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 1]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+1+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x16


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+1] = localReadB[4*SG1J*VECTOR_WIDTH + 1]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+1] = localReadB[5*SG1J*VECTOR_WIDTH + 1]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+1] = localReadB[6*SG1J*VECTOR_WIDTH + 1]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+1] = localReadB[7*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x16_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+1+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 1]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+1+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 1]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+1+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 1]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+1+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x16


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+1] = localReadB[4*SG1J*VECTOR_WIDTH + 1]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+1] = localReadB[5*SG1J*VECTOR_WIDTH + 1]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+1] = localReadB[6*SG1J*VECTOR_WIDTH + 1]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+1] = localReadB[7*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x16_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+1+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 1]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+1+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 1]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+1+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 1]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+1+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x16


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+1] = localReadB[4*SG1J*VECTOR_WIDTH + 1]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+1] = localReadB[5*SG1J*VECTOR_WIDTH + 1]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+1] = localReadB[6*SG1J*VECTOR_WIDTH + 1]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+1] = localReadB[7*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x16_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+1+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 1]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+1+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 1]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+1+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 1]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+1+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x16


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+1] = localReadB[4*SG1J*VECTOR_WIDTH + 1]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+1] = localReadB[5*SG1J*VECTOR_WIDTH + 1]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+1] = localReadB[6*SG1J*VECTOR_WIDTH + 1]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+1] = localReadB[7*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x16_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+1+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 1]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+1+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 1]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+1+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 1]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+1+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x16


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+1] = localReadB[4*SG1J*VECTOR_WIDTH + 1]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+1] = localReadB[5*SG1J*VECTOR_WIDTH + 1]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+1] = localReadB[6*SG1J*VECTOR_WIDTH + 1]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+1] = localReadB[7*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x16_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+1+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 1]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+1+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 1]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+1+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 1]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+1+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x16


    /* iter 15 */

    MAC_2x16_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_1_2_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 1);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_1_3_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 1);
  b_0_0_4_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_1_4_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 1);
  b_0_0_5_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_1_5_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 1);
  b_0_0_6_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_1_6_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 1);
  b_0_0_7_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);
  b_0_1_7_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 1);
  b_0_0_8_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_8_0 + 0);
  b_0_1_8_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_8_0 + 1);
  b_0_0_9_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_9_0 + 0);
  b_0_1_9_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_9_0 + 1);
  b_0_0_10_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_10_0 + 0);
  b_0_1_10_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_10_0 + 1);
  b_0_0_11_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_11_0 + 0);
  b_0_1_11_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_11_0 + 1);
  b_0_0_12_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_12_0 + 0);
  b_0_1_12_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_12_0 + 1);
  b_0_0_13_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_13_0 + 0);
  b_0_1_13_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_13_0 + 1);
  b_0_0_14_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_14_0 + 0);
  b_0_1_14_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_14_0 + 1);
  b_0_0_15_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_15_0 + 0);
  b_0_1_15_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_15_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_1);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_1);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_4_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_1);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_5_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_1);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_6_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_1);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);
  localWriteB_0_0_7_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_1);
  localWriteB_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_8_0);
  localWriteB_0_0_8_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_8_1);
  localWriteB_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_9_0);
  localWriteB_0_0_9_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_9_1);
  localWriteB_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_10_0);
  localWriteB_0_0_10_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_10_1);
  localWriteB_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_11_0);
  localWriteB_0_0_11_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_11_1);
  localWriteB_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_12_0);
  localWriteB_0_0_12_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_12_1);
  localWriteB_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_13_0);
  localWriteB_0_0_13_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_13_1);
  localWriteB_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_14_0);
  localWriteB_0_0_14_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_14_1);
  localWriteB_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_15_0);
  localWriteB_0_0_15_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_15_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_2_1 + 0) = b_0_1_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_3_1 + 0) = b_0_1_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_4_1 + 0) = b_0_1_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_5_1 + 0) = b_0_1_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_6_1 + 0) = b_0_1_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
  *(localWriteB_0_0_7_1 + 0) = b_0_1_7_0;
  *(localWriteB_0_0_8_0 + 0) = b_0_0_8_0;
  *(localWriteB_0_0_8_1 + 0) = b_0_1_8_0;
  *(localWriteB_0_0_9_0 + 0) = b_0_0_9_0;
  *(localWriteB_0_0_9_1 + 0) = b_0_1_9_0;
  *(localWriteB_0_0_10_0 + 0) = b_0_0_10_0;
  *(localWriteB_0_0_10_1 + 0) = b_0_1_10_0;
  *(localWriteB_0_0_11_0 + 0) = b_0_0_11_0;
  *(localWriteB_0_0_11_1 + 0) = b_0_1_11_0;
  *(localWriteB_0_0_12_0 + 0) = b_0_0_12_0;
  *(localWriteB_0_0_12_1 + 0) = b_0_1_12_0;
  *(localWriteB_0_0_13_0 + 0) = b_0_0_13_0;
  *(localWriteB_0_0_13_1 + 0) = b_0_1_13_0;
  *(localWriteB_0_0_14_0 + 0) = b_0_0_14_0;
  *(localWriteB_0_0_14_1 + 0) = b_0_1_14_0;
  *(localWriteB_0_0_15_0 + 0) = b_0_0_15_0;
  *(localWriteB_0_0_15_1 + 0) = b_0_1_15_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+1] = localReadB[4*SG1J*VECTOR_WIDTH + 1]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+1] = localReadB[5*SG1J*VECTOR_WIDTH + 1]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+1] = localReadB[6*SG1J*VECTOR_WIDTH + 1]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+1] = localReadB[7*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x16

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (4*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (5*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (6*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (7*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x16
#undef MAC_2x16_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT128x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;
  unsigned int flattenedOffsetA_4_0 = globalReadOffsetA0I + 0 + 4*LSPA;
  flattenedOffsetA_4_0 = (flattenedOffsetA_4_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_4_0;
  unsigned int globalReadOffsetA0I_4_0 = flattenedOffsetA_4_0;
  unsigned int flattenedOffsetA_5_0 = globalReadOffsetA0I + 0 + 5*LSPA;
  flattenedOffsetA_5_0 = (flattenedOffsetA_5_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_5_0;
  unsigned int globalReadOffsetA0I_5_0 = flattenedOffsetA_5_0;
  unsigned int flattenedOffsetA_6_0 = globalReadOffsetA0I + 0 + 6*LSPA;
  flattenedOffsetA_6_0 = (flattenedOffsetA_6_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_6_0;
  unsigned int globalReadOffsetA0I_6_0 = flattenedOffsetA_6_0;
  unsigned int flattenedOffsetA_7_0 = globalReadOffsetA0I + 0 + 7*LSPA;
  flattenedOffsetA_7_0 = (flattenedOffsetA_7_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_7_0;
  unsigned int globalReadOffsetA0I_7_0 = flattenedOffsetA_7_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 4*LSPA);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 5*LSPA);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 6*LSPA);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 7*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT128x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;
  unsigned int flattenedOffsetA_4_0 = globalReadOffsetA0I + 0 + 4*LSPA;
  flattenedOffsetA_4_0 = (flattenedOffsetA_4_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_4_0;
  unsigned int globalReadOffsetA0I_4_0 = flattenedOffsetA_4_0;
  unsigned int flattenedOffsetA_5_0 = globalReadOffsetA0I + 0 + 5*LSPA;
  flattenedOffsetA_5_0 = (flattenedOffsetA_5_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_5_0;
  unsigned int globalReadOffsetA0I_5_0 = flattenedOffsetA_5_0;
  unsigned int flattenedOffsetA_6_0 = globalReadOffsetA0I + 0 + 6*LSPA;
  flattenedOffsetA_6_0 = (flattenedOffsetA_6_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_6_0;
  unsigned int globalReadOffsetA0I_6_0 = flattenedOffsetA_6_0;
  unsigned int flattenedOffsetA_7_0 = globalReadOffsetA0I + 0 + 7*LSPA;
  flattenedOffsetA_7_0 = (flattenedOffsetA_7_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_7_0;
  unsigned int globalReadOffsetA0I_7_0 = flattenedOffsetA_7_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 4*LSPA);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 5*LSPA);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 6*LSPA);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 7*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 16
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 16
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 4096
#define LDS_NUM_ELEMENTS 4608

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 16x2 micro-tile */
#define MAC_16x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[8],rB[0],rC[8+0*TT0I] , rA[9],rB[0],rC[9+0*TT0I]); \
  TYPE_MAC(rA[10],rB[0],rC[10+0*TT0I] , rA[11],rB[0],rC[11+0*TT0I]); \
  TYPE_MAC(rA[12],rB[0],rC[12+0*TT0I] , rA[13],rB[0],rC[13+0*TT0I]); \
  TYPE_MAC(rA[14],rB[0],rC[14+0*TT0I] , rA[15],rB[0],rC[15+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[8],rB[1],rC[8+1*TT0I] , rA[9],rB[1],rC[9+1*TT0I]); \
  TYPE_MAC(rA[10],rB[1],rC[10+1*TT0I] , rA[11],rB[1],rC[11+1*TT0I]); \
  TYPE_MAC(rA[12],rB[1],rC[12+1*TT0I] , rA[13],rB[1],rC[13+1*TT0I]); \
  TYPE_MAC(rA[14],rB[1],rC[14+1*TT0I] , rA[15],rB[1],rC[15+1*TT0I]); \

#define MAC_16x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[8+TT0I],rB[0+TT1J],rC[8+0*TT0I] , rA[9+TT0I],rB[0+TT1J],rC[9+0*TT0I]); \
  TYPE_MAC(rA[10+TT0I],rB[0+TT1J],rC[10+0*TT0I] , rA[11+TT0I],rB[0+TT1J],rC[11+0*TT0I]); \
  TYPE_MAC(rA[12+TT0I],rB[0+TT1J],rC[12+0*TT0I] , rA[13+TT0I],rB[0+TT1J],rC[13+0*TT0I]); \
  TYPE_MAC(rA[14+TT0I],rB[0+TT1J],rC[14+0*TT0I] , rA[15+TT0I],rB[0+TT1J],rC[15+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[8+TT0I],rB[1+TT1J],rC[8+1*TT0I] , rA[9+TT0I],rB[1+TT1J],rC[9+1*TT0I]); \
  TYPE_MAC(rA[10+TT0I],rB[1+TT1J],rC[10+1*TT0I] , rA[11+TT0I],rB[1+TT1J],rC[11+1*TT0I]); \
  TYPE_MAC(rA[12+TT0I],rB[1+TT1J],rC[12+1*TT0I] , rA[13+TT0I],rB[1+TT1J],rC[13+1*TT0I]); \
  TYPE_MAC(rA[14+TT0I],rB[1+TT1J],rC[14+1*TT0I] , rA[15+TT0I],rB[1+TT1J],rC[15+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT256x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT16_2_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;
  unsigned int flattenedOffsetA_4_0 = globalReadOffsetA0I + 0 + 4*LSPA;
  flattenedOffsetA_4_0 = (flattenedOffsetA_4_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_4_0;
  unsigned int globalReadOffsetA0I_4_0 = flattenedOffsetA_4_0;
  unsigned int flattenedOffsetA_5_0 = globalReadOffsetA0I + 0 + 5*LSPA;
  flattenedOffsetA_5_0 = (flattenedOffsetA_5_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_5_0;
  unsigned int globalReadOffsetA0I_5_0 = flattenedOffsetA_5_0;
  unsigned int flattenedOffsetA_6_0 = globalReadOffsetA0I + 0 + 6*LSPA;
  flattenedOffsetA_6_0 = (flattenedOffsetA_6_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_6_0;
  unsigned int globalReadOffsetA0I_6_0 = flattenedOffsetA_6_0;
  unsigned int flattenedOffsetA_7_0 = globalReadOffsetA0I + 0 + 7*LSPA;
  flattenedOffsetA_7_0 = (flattenedOffsetA_7_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_7_0;
  unsigned int globalReadOffsetA0I_7_0 = flattenedOffsetA_7_0;
  unsigned int flattenedOffsetA_8_0 = globalReadOffsetA0I + 0 + 8*LSPA;
  flattenedOffsetA_8_0 = (flattenedOffsetA_8_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_8_0;
  unsigned int globalReadOffsetA0I_8_0 = flattenedOffsetA_8_0;
  unsigned int flattenedOffsetA_9_0 = globalReadOffsetA0I + 0 + 9*LSPA;
  flattenedOffsetA_9_0 = (flattenedOffsetA_9_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_9_0;
  unsigned int globalReadOffsetA0I_9_0 = flattenedOffsetA_9_0;
  unsigned int flattenedOffsetA_10_0 = globalReadOffsetA0I + 0 + 10*LSPA;
  flattenedOffsetA_10_0 = (flattenedOffsetA_10_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_10_0;
  unsigned int globalReadOffsetA0I_10_0 = flattenedOffsetA_10_0;
  unsigned int flattenedOffsetA_11_0 = globalReadOffsetA0I + 0 + 11*LSPA;
  flattenedOffsetA_11_0 = (flattenedOffsetA_11_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_11_0;
  unsigned int globalReadOffsetA0I_11_0 = flattenedOffsetA_11_0;
  unsigned int flattenedOffsetA_12_0 = globalReadOffsetA0I + 0 + 12*LSPA;
  flattenedOffsetA_12_0 = (flattenedOffsetA_12_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_12_0;
  unsigned int globalReadOffsetA0I_12_0 = flattenedOffsetA_12_0;
  unsigned int flattenedOffsetA_13_0 = globalReadOffsetA0I + 0 + 13*LSPA;
  flattenedOffsetA_13_0 = (flattenedOffsetA_13_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_13_0;
  unsigned int globalReadOffsetA0I_13_0 = flattenedOffsetA_13_0;
  unsigned int flattenedOffsetA_14_0 = globalReadOffsetA0I + 0 + 14*LSPA;
  flattenedOffsetA_14_0 = (flattenedOffsetA_14_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_14_0;
  unsigned int globalReadOffsetA0I_14_0 = flattenedOffsetA_14_0;
  unsigned int flattenedOffsetA_15_0 = globalReadOffsetA0I + 0 + 15*LSPA;
  flattenedOffsetA_15_0 = (flattenedOffsetA_15_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_15_0;
  unsigned int globalReadOffsetA0I_15_0 = flattenedOffsetA_15_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_15_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 4*LSPA);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 5*LSPA);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 6*LSPA);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 7*LSPA);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 8*LSPA);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 9*LSPA);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 10*LSPA);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 11*LSPA);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 12*LSPA);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 13*LSPA);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 14*LSPA);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 15*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_0_0_15_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_8_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_9_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_10_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_11_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_12_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_13_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_14_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_15_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_8_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_9_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_10_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_11_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_12_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_13_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_14_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_15_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0+TT0I] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0+TT0I] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0+TT0I] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0+TT0I] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0+TT0I] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0+TT0I] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0+TT0I] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0+TT0I] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0+TT0I] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0+TT0I] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0+TT0I] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0+TT0I] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0+TT0I] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0+TT0I] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0+TT0I] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0+TT0I] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0+TT0I] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0+TT0I] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0+TT0I] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0+TT0I] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0+TT0I] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0+TT0I] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0+TT0I] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0+TT0I] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0+TT0I] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0+TT0I] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0+TT0I] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0+TT0I] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0+TT0I] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0+TT0I] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0+TT0I] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0+TT0I] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0+TT0I] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0+TT0I] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0+TT0I] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0+TT0I] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0+TT0I] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0+TT0I] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0+TT0I] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0+TT0I] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0+TT0I] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0+TT0I] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0+TT0I] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0+TT0I] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0+TT0I] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0+TT0I] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0+TT0I] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0+TT0I] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0+TT0I] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0+TT0I] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0+TT0I] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0+TT0I] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0+TT0I] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0+TT0I] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0+TT0I] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0+TT0I] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0+TT0I] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0+TT0I] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0+TT0I] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0+TT0I] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0+TT0I] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0+TT0I] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0+TT0I] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0+TT0I] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_16x2


    /* iter 15 */

    MAC_16x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_0_8_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_0_0_9_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_0_0_10_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_0_0_11_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_0_0_12_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_0_0_13_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_0_0_14_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_0_0_15_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_16x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  8*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 8*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[8*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  9*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 9*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[9*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  10*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 10*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[10*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  11*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 11*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[11*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  12*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 12*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[12*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  13*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 13*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[13*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  14*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 14*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[14*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  15*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 15*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[15*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  8*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 8*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[8*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  9*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 9*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[9*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  10*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 10*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[10*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  11*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 11*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[11*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  12*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 12*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[12*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  13*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 13*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[13*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  14*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 14*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[14*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  15*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 15*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[15*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_16x2
#undef MAC_16x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 16
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 16

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 4608

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 2x16 micro-tile */
#define MAC_2x16\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[0],rB[8],rC[0+8*TT0I] , rA[1],rB[8],rC[1+8*TT0I]); \
  TYPE_MAC(rA[0],rB[9],rC[0+9*TT0I] , rA[1],rB[9],rC[1+9*TT0I]); \
  TYPE_MAC(rA[0],rB[10],rC[0+10*TT0I] , rA[1],rB[10],rC[1+10*TT0I]); \
  TYPE_MAC(rA[0],rB[11],rC[0+11*TT0I] , rA[1],rB[11],rC[1+11*TT0I]); \
  TYPE_MAC(rA[0],rB[12],rC[0+12*TT0I] , rA[1],rB[12],rC[1+12*TT0I]); \
  TYPE_MAC(rA[0],rB[13],rC[0+13*TT0I] , rA[1],rB[13],rC[1+13*TT0I]); \
  TYPE_MAC(rA[0],rB[14],rC[0+14*TT0I] , rA[1],rB[14],rC[1+14*TT0I]); \
  TYPE_MAC(rA[0],rB[15],rC[0+15*TT0I] , rA[1],rB[15],rC[1+15*TT0I]); \

#define MAC_2x16_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[8+TT1J],rC[0+8*TT0I] , rA[1+TT0I],rB[8+TT1J],rC[1+8*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[9+TT1J],rC[0+9*TT0I] , rA[1+TT0I],rB[9+TT1J],rC[1+9*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[10+TT1J],rC[0+10*TT0I] , rA[1+TT0I],rB[10+TT1J],rC[1+10*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[11+TT1J],rC[0+11*TT0I] , rA[1+TT0I],rB[11+TT1J],rC[1+11*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[12+TT1J],rC[0+12*TT0I] , rA[1+TT0I],rB[12+TT1J],rC[1+12*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[13+TT1J],rC[0+13*TT0I] , rA[1+TT0I],rB[13+TT1J],rC[1+13*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[14+TT1J],rC[0+14*TT0I] , rA[1+TT0I],rB[14+TT1J],rC[1+14*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[15+TT1J],rC[0+15*TT0I] , rA[1+TT0I],rB[15+TT1J],rC[1+15*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT32x256x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_16_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;
  DATA_TYPE b_0_0_8_0;
  DATA_TYPE b_0_0_9_0;
  DATA_TYPE b_0_0_10_0;
  DATA_TYPE b_0_0_11_0;
  DATA_TYPE b_0_0_12_0;
  DATA_TYPE b_0_0_13_0;
  DATA_TYPE b_0_0_14_0;
  DATA_TYPE b_0_0_15_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;
  unsigned int flattenedOffsetB_4_0 = globalReadOffsetB1J + 0 + 4*LSPB;
  flattenedOffsetB_4_0 = (flattenedOffsetB_4_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_4_0;
  unsigned int globalReadOffsetB1J_4_0 = flattenedOffsetB_4_0;
  unsigned int flattenedOffsetB_5_0 = globalReadOffsetB1J + 0 + 5*LSPB;
  flattenedOffsetB_5_0 = (flattenedOffsetB_5_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_5_0;
  unsigned int globalReadOffsetB1J_5_0 = flattenedOffsetB_5_0;
  unsigned int flattenedOffsetB_6_0 = globalReadOffsetB1J + 0 + 6*LSPB;
  flattenedOffsetB_6_0 = (flattenedOffsetB_6_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_6_0;
  unsigned int globalReadOffsetB1J_6_0 = flattenedOffsetB_6_0;
  unsigned int flattenedOffsetB_7_0 = globalReadOffsetB1J + 0 + 7*LSPB;
  flattenedOffsetB_7_0 = (flattenedOffsetB_7_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_7_0;
  unsigned int globalReadOffsetB1J_7_0 = flattenedOffsetB_7_0;
  unsigned int flattenedOffsetB_8_0 = globalReadOffsetB1J + 0 + 8*LSPB;
  flattenedOffsetB_8_0 = (flattenedOffsetB_8_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_8_0;
  unsigned int globalReadOffsetB1J_8_0 = flattenedOffsetB_8_0;
  unsigned int flattenedOffsetB_9_0 = globalReadOffsetB1J + 0 + 9*LSPB;
  flattenedOffsetB_9_0 = (flattenedOffsetB_9_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_9_0;
  unsigned int globalReadOffsetB1J_9_0 = flattenedOffsetB_9_0;
  unsigned int flattenedOffsetB_10_0 = globalReadOffsetB1J + 0 + 10*LSPB;
  flattenedOffsetB_10_0 = (flattenedOffsetB_10_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_10_0;
  unsigned int globalReadOffsetB1J_10_0 = flattenedOffsetB_10_0;
  unsigned int flattenedOffsetB_11_0 = globalReadOffsetB1J + 0 + 11*LSPB;
  flattenedOffsetB_11_0 = (flattenedOffsetB_11_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_11_0;
  unsigned int globalReadOffsetB1J_11_0 = flattenedOffsetB_11_0;
  unsigned int flattenedOffsetB_12_0 = globalReadOffsetB1J + 0 + 12*LSPB;
  flattenedOffsetB_12_0 = (flattenedOffsetB_12_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_12_0;
  unsigned int globalReadOffsetB1J_12_0 = flattenedOffsetB_12_0;
  unsigned int flattenedOffsetB_13_0 = globalReadOffsetB1J + 0 + 13*LSPB;
  flattenedOffsetB_13_0 = (flattenedOffsetB_13_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_13_0;
  unsigned int globalReadOffsetB1J_13_0 = flattenedOffsetB_13_0;
  unsigned int flattenedOffsetB_14_0 = globalReadOffsetB1J + 0 + 14*LSPB;
  flattenedOffsetB_14_0 = (flattenedOffsetB_14_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_14_0;
  unsigned int globalReadOffsetB1J_14_0 = flattenedOffsetB_14_0;
  unsigned int flattenedOffsetB_15_0 = globalReadOffsetB1J + 0 + 15*LSPB;
  flattenedOffsetB_15_0 = (flattenedOffsetB_15_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_15_0;
  unsigned int globalReadOffsetB1J_15_0 = flattenedOffsetB_15_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_7_0), wgK );
  int64_t globalReadOffsetB_0_0_8_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_8_0), wgK );
  int64_t globalReadOffsetB_0_0_9_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_9_0), wgK );
  int64_t globalReadOffsetB_0_0_10_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_10_0), wgK );
  int64_t globalReadOffsetB_0_0_11_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_11_0), wgK );
  int64_t globalReadOffsetB_0_0_12_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_12_0), wgK );
  int64_t globalReadOffsetB_0_0_13_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_13_0), wgK );
  int64_t globalReadOffsetB_0_0_14_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_14_0), wgK );
  int64_t globalReadOffsetB_0_0_15_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_15_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;
  DATA_TYPE const *globalReadB_0_0_8_0 = B + globalReadOffsetB_0_0_8_0;
  DATA_TYPE const *globalReadB_0_0_9_0 = B + globalReadOffsetB_0_0_9_0;
  DATA_TYPE const *globalReadB_0_0_10_0 = B + globalReadOffsetB_0_0_10_0;
  DATA_TYPE const *globalReadB_0_0_11_0 = B + globalReadOffsetB_0_0_11_0;
  DATA_TYPE const *globalReadB_0_0_12_0 = B + globalReadOffsetB_0_0_12_0;
  DATA_TYPE const *globalReadB_0_0_13_0 = B + globalReadOffsetB_0_0_13_0;
  DATA_TYPE const *globalReadB_0_0_14_0 = B + globalReadOffsetB_0_0_14_0;
  DATA_TYPE const *globalReadB_0_0_15_0 = B + globalReadOffsetB_0_0_15_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 4*LSPB);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 5*LSPB);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 6*LSPB);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 7*LSPB);
  unsigned int localWriteOffsetB_0_0_8_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 8*LSPB);
  unsigned int localWriteOffsetB_0_0_9_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 9*LSPB);
  unsigned int localWriteOffsetB_0_0_10_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 10*LSPB);
  unsigned int localWriteOffsetB_0_0_11_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 11*LSPB);
  unsigned int localWriteOffsetB_0_0_12_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 12*LSPB);
  unsigned int localWriteOffsetB_0_0_13_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 13*LSPB);
  unsigned int localWriteOffsetB_0_0_14_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 14*LSPB);
  unsigned int localWriteOffsetB_0_0_15_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 15*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;
  DATA_TYPE *localWriteB_0_0_8_0;
  DATA_TYPE *localWriteB_0_0_9_0;
  DATA_TYPE *localWriteB_0_0_10_0;
  DATA_TYPE *localWriteB_0_0_11_0;
  DATA_TYPE *localWriteB_0_0_12_0;
  DATA_TYPE *localWriteB_0_0_13_0;
  DATA_TYPE *localWriteB_0_0_14_0;
  DATA_TYPE *localWriteB_0_0_15_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);
  localWriteB_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_8_0);
  localWriteB_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_9_0);
  localWriteB_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_10_0);
  localWriteB_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_11_0);
  localWriteB_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_12_0);
  localWriteB_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_13_0);
  localWriteB_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_14_0);
  localWriteB_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_15_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_8_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_9_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_10_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_11_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_12_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_13_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_14_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_15_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);
    b_0_0_8_0 = *(globalReadB_0_0_8_0 + 0);
    b_0_0_9_0 = *(globalReadB_0_0_9_0 + 0);
    b_0_0_10_0 = *(globalReadB_0_0_10_0 + 0);
    b_0_0_11_0 = *(globalReadB_0_0_11_0 + 0);
    b_0_0_12_0 = *(globalReadB_0_0_12_0 + 0);
    b_0_0_13_0 = *(globalReadB_0_0_13_0 + 0);
    b_0_0_14_0 = *(globalReadB_0_0_14_0 + 0);
    b_0_0_15_0 = *(globalReadB_0_0_15_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_8_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_8_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_9_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_9_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_10_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_10_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_11_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_11_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_12_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_12_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_13_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_13_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_14_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_14_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_15_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_15_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
    *(localWriteB_0_0_8_0 + 0) = b_0_0_8_0;
    *(localWriteB_0_0_9_0 + 0) = b_0_0_9_0;
    *(localWriteB_0_0_10_0 + 0) = b_0_0_10_0;
    *(localWriteB_0_0_11_0 + 0) = b_0_0_11_0;
    *(localWriteB_0_0_12_0 + 0) = b_0_0_12_0;
    *(localWriteB_0_0_13_0 + 0) = b_0_0_13_0;
    *(localWriteB_0_0_14_0 + 0) = b_0_0_14_0;
    *(localWriteB_0_0_15_0 + 0) = b_0_0_15_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[8*VECTOR_WIDTH+0] = localReadB[8*SG1J*VECTOR_WIDTH + 0]; 
    rB[9*VECTOR_WIDTH+0] = localReadB[9*SG1J*VECTOR_WIDTH + 0]; 
    rB[10*VECTOR_WIDTH+0] = localReadB[10*SG1J*VECTOR_WIDTH + 0]; 
    rB[11*VECTOR_WIDTH+0] = localReadB[11*SG1J*VECTOR_WIDTH + 0]; 
    rB[12*VECTOR_WIDTH+0] = localReadB[12*SG1J*VECTOR_WIDTH + 0]; 
    rB[13*VECTOR_WIDTH+0] = localReadB[13*SG1J*VECTOR_WIDTH + 0]; 
    rB[14*VECTOR_WIDTH+0] = localReadB[14*SG1J*VECTOR_WIDTH + 0]; 
    rB[15*VECTOR_WIDTH+0] = localReadB[15*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[8*VECTOR_WIDTH+0+TT1J] = localReadB[8*SG1J*VECTOR_WIDTH + 0]; 
    rB[9*VECTOR_WIDTH+0+TT1J] = localReadB[9*SG1J*VECTOR_WIDTH + 0]; 
    rB[10*VECTOR_WIDTH+0+TT1J] = localReadB[10*SG1J*VECTOR_WIDTH + 0]; 
    rB[11*VECTOR_WIDTH+0+TT1J] = localReadB[11*SG1J*VECTOR_WIDTH + 0]; 
    rB[12*VECTOR_WIDTH+0+TT1J] = localReadB[12*SG1J*VECTOR_WIDTH + 0]; 
    rB[13*VECTOR_WIDTH+0+TT1J] = localReadB[13*SG1J*VECTOR_WIDTH + 0]; 
    rB[14*VECTOR_WIDTH+0+TT1J] = localReadB[14*SG1J*VECTOR_WIDTH + 0]; 
    rB[15*VECTOR_WIDTH+0+TT1J] = localReadB[15*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x16


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[8*VECTOR_WIDTH+0] = localReadB[8*SG1J*VECTOR_WIDTH + 0]; 
    rB[9*VECTOR_WIDTH+0] = localReadB[9*SG1J*VECTOR_WIDTH + 0]; 
    rB[10*VECTOR_WIDTH+0] = localReadB[10*SG1J*VECTOR_WIDTH + 0]; 
    rB[11*VECTOR_WIDTH+0] = localReadB[11*SG1J*VECTOR_WIDTH + 0]; 
    rB[12*VECTOR_WIDTH+0] = localReadB[12*SG1J*VECTOR_WIDTH + 0]; 
    rB[13*VECTOR_WIDTH+0] = localReadB[13*SG1J*VECTOR_WIDTH + 0]; 
    rB[14*VECTOR_WIDTH+0] = localReadB[14*SG1J*VECTOR_WIDTH + 0]; 
    rB[15*VECTOR_WIDTH+0] = localReadB[15*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x16_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[8*VECTOR_WIDTH+0+TT1J] = localReadB[8*SG1J*VECTOR_WIDTH + 0]; 
    rB[9*VECTOR_WIDTH+0+TT1J] = localReadB[9*SG1J*VECTOR_WIDTH + 0]; 
    rB[10*VECTOR_WIDTH+0+TT1J] = localReadB[10*SG1J*VECTOR_WIDTH + 0]; 
    rB[11*VECTOR_WIDTH+0+TT1J] = localReadB[11*SG1J*VECTOR_WIDTH + 0]; 
    rB[12*VECTOR_WIDTH+0+TT1J] = localReadB[12*SG1J*VECTOR_WIDTH + 0]; 
    rB[13*VECTOR_WIDTH+0+TT1J] = localReadB[13*SG1J*VECTOR_WIDTH + 0]; 
    rB[14*VECTOR_WIDTH+0+TT1J] = localReadB[14*SG1J*VECTOR_WIDTH + 0]; 
    rB[15*VECTOR_WIDTH+0+TT1J] = localReadB[15*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x16


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[8*VECTOR_WIDTH+0] = localReadB[8*SG1J*VECTOR_WIDTH + 0]; 
    rB[9*VECTOR_WIDTH+0] = localReadB[9*SG1J*VECTOR_WIDTH + 0]; 
    rB[10*VECTOR_WIDTH+0] = localReadB[10*SG1J*VECTOR_WIDTH + 0]; 
    rB[11*VECTOR_WIDTH+0] = localReadB[11*SG1J*VECTOR_WIDTH + 0]; 
    rB[12*VECTOR_WIDTH+0] = localReadB[12*SG1J*VECTOR_WIDTH + 0]; 
    rB[13*VECTOR_WIDTH+0] = localReadB[13*SG1J*VECTOR_WIDTH + 0]; 
    rB[14*VECTOR_WIDTH+0] = localReadB[14*SG1J*VECTOR_WIDTH + 0]; 
    rB[15*VECTOR_WIDTH+0] = localReadB[15*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x16_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[8*VECTOR_WIDTH+0+TT1J] = localReadB[8*SG1J*VECTOR_WIDTH + 0]; 
    rB[9*VECTOR_WIDTH+0+TT1J] = localReadB[9*SG1J*VECTOR_WIDTH + 0]; 
    rB[10*VECTOR_WIDTH+0+TT1J] = localReadB[10*SG1J*VECTOR_WIDTH + 0]; 
    rB[11*VECTOR_WIDTH+0+TT1J] = localReadB[11*SG1J*VECTOR_WIDTH + 0]; 
    rB[12*VECTOR_WIDTH+0+TT1J] = localReadB[12*SG1J*VECTOR_WIDTH + 0]; 
    rB[13*VECTOR_WIDTH+0+TT1J] = localReadB[13*SG1J*VECTOR_WIDTH + 0]; 
    rB[14*VECTOR_WIDTH+0+TT1J] = localReadB[14*SG1J*VECTOR_WIDTH + 0]; 
    rB[15*VECTOR_WIDTH+0+TT1J] = localReadB[15*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x16


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[8*VECTOR_WIDTH+0] = localReadB[8*SG1J*VECTOR_WIDTH + 0]; 
    rB[9*VECTOR_WIDTH+0] = localReadB[9*SG1J*VECTOR_WIDTH + 0]; 
    rB[10*VECTOR_WIDTH+0] = localReadB[10*SG1J*VECTOR_WIDTH + 0]; 
    rB[11*VECTOR_WIDTH+0] = localReadB[11*SG1J*VECTOR_WIDTH + 0]; 
    rB[12*VECTOR_WIDTH+0] = localReadB[12*SG1J*VECTOR_WIDTH + 0]; 
    rB[13*VECTOR_WIDTH+0] = localReadB[13*SG1J*VECTOR_WIDTH + 0]; 
    rB[14*VECTOR_WIDTH+0] = localReadB[14*SG1J*VECTOR_WIDTH + 0]; 
    rB[15*VECTOR_WIDTH+0] = localReadB[15*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x16_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[8*VECTOR_WIDTH+0+TT1J] = localReadB[8*SG1J*VECTOR_WIDTH + 0]; 
    rB[9*VECTOR_WIDTH+0+TT1J] = localReadB[9*SG1J*VECTOR_WIDTH + 0]; 
    rB[10*VECTOR_WIDTH+0+TT1J] = localReadB[10*SG1J*VECTOR_WIDTH + 0]; 
    rB[11*VECTOR_WIDTH+0+TT1J] = localReadB[11*SG1J*VECTOR_WIDTH + 0]; 
    rB[12*VECTOR_WIDTH+0+TT1J] = localReadB[12*SG1J*VECTOR_WIDTH + 0]; 
    rB[13*VECTOR_WIDTH+0+TT1J] = localReadB[13*SG1J*VECTOR_WIDTH + 0]; 
    rB[14*VECTOR_WIDTH+0+TT1J] = localReadB[14*SG1J*VECTOR_WIDTH + 0]; 
    rB[15*VECTOR_WIDTH+0+TT1J] = localReadB[15*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x16


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[8*VECTOR_WIDTH+0] = localReadB[8*SG1J*VECTOR_WIDTH + 0]; 
    rB[9*VECTOR_WIDTH+0] = localReadB[9*SG1J*VECTOR_WIDTH + 0]; 
    rB[10*VECTOR_WIDTH+0] = localReadB[10*SG1J*VECTOR_WIDTH + 0]; 
    rB[11*VECTOR_WIDTH+0] = localReadB[11*SG1J*VECTOR_WIDTH + 0]; 
    rB[12*VECTOR_WIDTH+0] = localReadB[12*SG1J*VECTOR_WIDTH + 0]; 
    rB[13*VECTOR_WIDTH+0] = localReadB[13*SG1J*VECTOR_WIDTH + 0]; 
    rB[14*VECTOR_WIDTH+0] = localReadB[14*SG1J*VECTOR_WIDTH + 0]; 
    rB[15*VECTOR_WIDTH+0] = localReadB[15*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x16_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[8*VECTOR_WIDTH+0+TT1J] = localReadB[8*SG1J*VECTOR_WIDTH + 0]; 
    rB[9*VECTOR_WIDTH+0+TT1J] = localReadB[9*SG1J*VECTOR_WIDTH + 0]; 
    rB[10*VECTOR_WIDTH+0+TT1J] = localReadB[10*SG1J*VECTOR_WIDTH + 0]; 
    rB[11*VECTOR_WIDTH+0+TT1J] = localReadB[11*SG1J*VECTOR_WIDTH + 0]; 
    rB[12*VECTOR_WIDTH+0+TT1J] = localReadB[12*SG1J*VECTOR_WIDTH + 0]; 
    rB[13*VECTOR_WIDTH+0+TT1J] = localReadB[13*SG1J*VECTOR_WIDTH + 0]; 
    rB[14*VECTOR_WIDTH+0+TT1J] = localReadB[14*SG1J*VECTOR_WIDTH + 0]; 
    rB[15*VECTOR_WIDTH+0+TT1J] = localReadB[15*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x16


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[8*VECTOR_WIDTH+0] = localReadB[8*SG1J*VECTOR_WIDTH + 0]; 
    rB[9*VECTOR_WIDTH+0] = localReadB[9*SG1J*VECTOR_WIDTH + 0]; 
    rB[10*VECTOR_WIDTH+0] = localReadB[10*SG1J*VECTOR_WIDTH + 0]; 
    rB[11*VECTOR_WIDTH+0] = localReadB[11*SG1J*VECTOR_WIDTH + 0]; 
    rB[12*VECTOR_WIDTH+0] = localReadB[12*SG1J*VECTOR_WIDTH + 0]; 
    rB[13*VECTOR_WIDTH+0] = localReadB[13*SG1J*VECTOR_WIDTH + 0]; 
    rB[14*VECTOR_WIDTH+0] = localReadB[14*SG1J*VECTOR_WIDTH + 0]; 
    rB[15*VECTOR_WIDTH+0] = localReadB[15*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x16_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[8*VECTOR_WIDTH+0+TT1J] = localReadB[8*SG1J*VECTOR_WIDTH + 0]; 
    rB[9*VECTOR_WIDTH+0+TT1J] = localReadB[9*SG1J*VECTOR_WIDTH + 0]; 
    rB[10*VECTOR_WIDTH+0+TT1J] = localReadB[10*SG1J*VECTOR_WIDTH + 0]; 
    rB[11*VECTOR_WIDTH+0+TT1J] = localReadB[11*SG1J*VECTOR_WIDTH + 0]; 
    rB[12*VECTOR_WIDTH+0+TT1J] = localReadB[12*SG1J*VECTOR_WIDTH + 0]; 
    rB[13*VECTOR_WIDTH+0+TT1J] = localReadB[13*SG1J*VECTOR_WIDTH + 0]; 
    rB[14*VECTOR_WIDTH+0+TT1J] = localReadB[14*SG1J*VECTOR_WIDTH + 0]; 
    rB[15*VECTOR_WIDTH+0+TT1J] = localReadB[15*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x16


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[8*VECTOR_WIDTH+0] = localReadB[8*SG1J*VECTOR_WIDTH + 0]; 
    rB[9*VECTOR_WIDTH+0] = localReadB[9*SG1J*VECTOR_WIDTH + 0]; 
    rB[10*VECTOR_WIDTH+0] = localReadB[10*SG1J*VECTOR_WIDTH + 0]; 
    rB[11*VECTOR_WIDTH+0] = localReadB[11*SG1J*VECTOR_WIDTH + 0]; 
    rB[12*VECTOR_WIDTH+0] = localReadB[12*SG1J*VECTOR_WIDTH + 0]; 
    rB[13*VECTOR_WIDTH+0] = localReadB[13*SG1J*VECTOR_WIDTH + 0]; 
    rB[14*VECTOR_WIDTH+0] = localReadB[14*SG1J*VECTOR_WIDTH + 0]; 
    rB[15*VECTOR_WIDTH+0] = localReadB[15*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x16_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[8*VECTOR_WIDTH+0+TT1J] = localReadB[8*SG1J*VECTOR_WIDTH + 0]; 
    rB[9*VECTOR_WIDTH+0+TT1J] = localReadB[9*SG1J*VECTOR_WIDTH + 0]; 
    rB[10*VECTOR_WIDTH+0+TT1J] = localReadB[10*SG1J*VECTOR_WIDTH + 0]; 
    rB[11*VECTOR_WIDTH+0+TT1J] = localReadB[11*SG1J*VECTOR_WIDTH + 0]; 
    rB[12*VECTOR_WIDTH+0+TT1J] = localReadB[12*SG1J*VECTOR_WIDTH + 0]; 
    rB[13*VECTOR_WIDTH+0+TT1J] = localReadB[13*SG1J*VECTOR_WIDTH + 0]; 
    rB[14*VECTOR_WIDTH+0+TT1J] = localReadB[14*SG1J*VECTOR_WIDTH + 0]; 
    rB[15*VECTOR_WIDTH+0+TT1J] = localReadB[15*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x16


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[8*VECTOR_WIDTH+0] = localReadB[8*SG1J*VECTOR_WIDTH + 0]; 
    rB[9*VECTOR_WIDTH+0] = localReadB[9*SG1J*VECTOR_WIDTH + 0]; 
    rB[10*VECTOR_WIDTH+0] = localReadB[10*SG1J*VECTOR_WIDTH + 0]; 
    rB[11*VECTOR_WIDTH+0] = localReadB[11*SG1J*VECTOR_WIDTH + 0]; 
    rB[12*VECTOR_WIDTH+0] = localReadB[12*SG1J*VECTOR_WIDTH + 0]; 
    rB[13*VECTOR_WIDTH+0] = localReadB[13*SG1J*VECTOR_WIDTH + 0]; 
    rB[14*VECTOR_WIDTH+0] = localReadB[14*SG1J*VECTOR_WIDTH + 0]; 
    rB[15*VECTOR_WIDTH+0] = localReadB[15*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x16_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[8*VECTOR_WIDTH+0+TT1J] = localReadB[8*SG1J*VECTOR_WIDTH + 0]; 
    rB[9*VECTOR_WIDTH+0+TT1J] = localReadB[9*SG1J*VECTOR_WIDTH + 0]; 
    rB[10*VECTOR_WIDTH+0+TT1J] = localReadB[10*SG1J*VECTOR_WIDTH + 0]; 
    rB[11*VECTOR_WIDTH+0+TT1J] = localReadB[11*SG1J*VECTOR_WIDTH + 0]; 
    rB[12*VECTOR_WIDTH+0+TT1J] = localReadB[12*SG1J*VECTOR_WIDTH + 0]; 
    rB[13*VECTOR_WIDTH+0+TT1J] = localReadB[13*SG1J*VECTOR_WIDTH + 0]; 
    rB[14*VECTOR_WIDTH+0+TT1J] = localReadB[14*SG1J*VECTOR_WIDTH + 0]; 
    rB[15*VECTOR_WIDTH+0+TT1J] = localReadB[15*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x16


    /* iter 15 */

    MAC_2x16_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);
  b_0_0_8_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_8_0 + 0);
  b_0_0_9_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_9_0 + 0);
  b_0_0_10_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_10_0 + 0);
  b_0_0_11_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_11_0 + 0);
  b_0_0_12_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_12_0 + 0);
  b_0_0_13_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_13_0 + 0);
  b_0_0_14_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_14_0 + 0);
  b_0_0_15_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_15_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);
  localWriteB_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_8_0);
  localWriteB_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_9_0);
  localWriteB_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_10_0);
  localWriteB_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_11_0);
  localWriteB_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_12_0);
  localWriteB_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_13_0);
  localWriteB_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_14_0);
  localWriteB_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_15_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
  *(localWriteB_0_0_8_0 + 0) = b_0_0_8_0;
  *(localWriteB_0_0_9_0 + 0) = b_0_0_9_0;
  *(localWriteB_0_0_10_0 + 0) = b_0_0_10_0;
  *(localWriteB_0_0_11_0 + 0) = b_0_0_11_0;
  *(localWriteB_0_0_12_0 + 0) = b_0_0_12_0;
  *(localWriteB_0_0_13_0 + 0) = b_0_0_13_0;
  *(localWriteB_0_0_14_0 + 0) = b_0_0_14_0;
  *(localWriteB_0_0_15_0 + 0) = b_0_0_15_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[8*VECTOR_WIDTH+0] = localReadB[8*SG1J*VECTOR_WIDTH + 0]; 
    rB[9*VECTOR_WIDTH+0] = localReadB[9*SG1J*VECTOR_WIDTH + 0]; 
    rB[10*VECTOR_WIDTH+0] = localReadB[10*SG1J*VECTOR_WIDTH + 0]; 
    rB[11*VECTOR_WIDTH+0] = localReadB[11*SG1J*VECTOR_WIDTH + 0]; 
    rB[12*VECTOR_WIDTH+0] = localReadB[12*SG1J*VECTOR_WIDTH + 0]; 
    rB[13*VECTOR_WIDTH+0] = localReadB[13*SG1J*VECTOR_WIDTH + 0]; 
    rB[14*VECTOR_WIDTH+0] = localReadB[14*SG1J*VECTOR_WIDTH + 0]; 
    rB[15*VECTOR_WIDTH+0] = localReadB[15*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x16

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 8*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 8*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (8*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 8*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 8*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (8*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 9*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 9*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (9*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 9*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 9*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (9*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 10*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 10*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (10*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 10*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 10*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (10*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 11*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 11*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (11*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 11*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 11*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (11*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 12*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 12*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (12*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 12*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 12*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (12*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 13*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 13*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (13*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 13*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 13*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (13*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 14*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 14*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (14*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 14*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 14*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (14*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 15*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 15*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (15*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 15*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 15*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (15*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x16
#undef MAC_2x16_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE double
#define DEST_DATA_TYPE double
#define COMPUTE_DATA_TYPE double
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_DB_MT32x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW2_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  double *D,
  double const * __restrict__ C,
  double const * __restrict__ A,
  double const * __restrict__ B,
  double const alpha,
  double const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (double)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE double
#define DEST_DATA_TYPE double
#define COMPUTE_DATA_TYPE double
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_DB_MT32x32x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW2_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  double *D,
  double const * __restrict__ C,
  double const * __restrict__ A,
  double const * __restrict__ B,
  double const alpha,
  double const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (double)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 819

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT64x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 7 */

    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT128x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 1) {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 2) {
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT64x16x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_1_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_1_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_1_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_1_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_1_4_0 = *(globalReadA_0_0_4_0 + 1);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_1_5_0 = *(globalReadA_0_0_5_0 + 1);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_1_6_0 = *(globalReadA_0_0_6_0 + 1);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_1_7_0 = *(globalReadA_0_0_7_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_4_0 + 1) = a_0_1_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_5_0 + 1) = a_0_1_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_6_0 + 1) = a_0_1_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_7_0 + 1) = a_0_1_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_1_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 1);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_1_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 1);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_1_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 1);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_1_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_4_0 + 1) = a_0_1_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_5_0 + 1) = a_0_1_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_6_0 + 1) = a_0_1_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_7_0 + 1) = a_0_1_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 1) {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 2) {
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT128x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 8

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x8 micro-tile */
#define MAC_2x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \

#define MAC_2x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT32x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;
  unsigned int globalReadOffsetBL_4_0 = globalReadOffsetBL + 0 + 4*LSPB;
  unsigned int globalReadOffsetBL_5_0 = globalReadOffsetBL + 0 + 5*LSPB;
  unsigned int globalReadOffsetBL_6_0 = globalReadOffsetBL + 0 + 6*LSPB;
  unsigned int globalReadOffsetBL_7_0 = globalReadOffsetBL + 0 + 7*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_7_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 4*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 5*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 6*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 7*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x8


    /* iter 15 */

    MAC_2x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x8
#undef MAC_2x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 16
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT64x16x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;
  unsigned int globalReadOffsetAL_8_0 = globalReadOffsetAL + 0 + 8*LSPA;
  unsigned int globalReadOffsetAL_9_0 = globalReadOffsetAL + 0 + 9*LSPA;
  unsigned int globalReadOffsetAL_10_0 = globalReadOffsetAL + 0 + 10*LSPA;
  unsigned int globalReadOffsetAL_11_0 = globalReadOffsetAL + 0 + 11*LSPA;
  unsigned int globalReadOffsetAL_12_0 = globalReadOffsetAL + 0 + 12*LSPA;
  unsigned int globalReadOffsetAL_13_0 = globalReadOffsetAL + 0 + 13*LSPA;
  unsigned int globalReadOffsetAL_14_0 = globalReadOffsetAL + 0 + 14*LSPA;
  unsigned int globalReadOffsetAL_15_0 = globalReadOffsetAL + 0 + 15*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_15_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 8*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 9*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 10*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 11*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 12*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 13*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 14*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 15*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_0_0_15_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_8_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_9_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_10_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_11_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_12_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_13_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_14_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_15_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_8_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_9_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_10_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_11_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_12_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_13_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_14_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_15_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_0_8_0 = ( globalReadOffsetAL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_0_0_9_0 = ( globalReadOffsetAL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_0_0_10_0 = ( globalReadOffsetAL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_0_0_11_0 = ( globalReadOffsetAL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_0_0_12_0 = ( globalReadOffsetAL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_0_0_13_0 = ( globalReadOffsetAL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_0_0_14_0 = ( globalReadOffsetAL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_0_0_15_0 = ( globalReadOffsetAL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 16

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x8 micro-tile */
#define MAC_2x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \

#define MAC_2x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT16x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;
  DATA_TYPE b_0_0_8_0;
  DATA_TYPE b_0_0_9_0;
  DATA_TYPE b_0_0_10_0;
  DATA_TYPE b_0_0_11_0;
  DATA_TYPE b_0_0_12_0;
  DATA_TYPE b_0_0_13_0;
  DATA_TYPE b_0_0_14_0;
  DATA_TYPE b_0_0_15_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;
  unsigned int globalReadOffsetBL_4_0 = globalReadOffsetBL + 0 + 4*LSPB;
  unsigned int globalReadOffsetBL_5_0 = globalReadOffsetBL + 0 + 5*LSPB;
  unsigned int globalReadOffsetBL_6_0 = globalReadOffsetBL + 0 + 6*LSPB;
  unsigned int globalReadOffsetBL_7_0 = globalReadOffsetBL + 0 + 7*LSPB;
  unsigned int globalReadOffsetBL_8_0 = globalReadOffsetBL + 0 + 8*LSPB;
  unsigned int globalReadOffsetBL_9_0 = globalReadOffsetBL + 0 + 9*LSPB;
  unsigned int globalReadOffsetBL_10_0 = globalReadOffsetBL + 0 + 10*LSPB;
  unsigned int globalReadOffsetBL_11_0 = globalReadOffsetBL + 0 + 11*LSPB;
  unsigned int globalReadOffsetBL_12_0 = globalReadOffsetBL + 0 + 12*LSPB;
  unsigned int globalReadOffsetBL_13_0 = globalReadOffsetBL + 0 + 13*LSPB;
  unsigned int globalReadOffsetBL_14_0 = globalReadOffsetBL + 0 + 14*LSPB;
  unsigned int globalReadOffsetBL_15_0 = globalReadOffsetBL + 0 + 15*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_7_0), wgK );
  int64_t globalReadOffsetB_0_0_8_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_8_0), wgK );
  int64_t globalReadOffsetB_0_0_9_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_9_0), wgK );
  int64_t globalReadOffsetB_0_0_10_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_10_0), wgK );
  int64_t globalReadOffsetB_0_0_11_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_11_0), wgK );
  int64_t globalReadOffsetB_0_0_12_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_12_0), wgK );
  int64_t globalReadOffsetB_0_0_13_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_13_0), wgK );
  int64_t globalReadOffsetB_0_0_14_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_14_0), wgK );
  int64_t globalReadOffsetB_0_0_15_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_15_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;
  DATA_TYPE const *globalReadB_0_0_8_0 = B + globalReadOffsetB_0_0_8_0;
  DATA_TYPE const *globalReadB_0_0_9_0 = B + globalReadOffsetB_0_0_9_0;
  DATA_TYPE const *globalReadB_0_0_10_0 = B + globalReadOffsetB_0_0_10_0;
  DATA_TYPE const *globalReadB_0_0_11_0 = B + globalReadOffsetB_0_0_11_0;
  DATA_TYPE const *globalReadB_0_0_12_0 = B + globalReadOffsetB_0_0_12_0;
  DATA_TYPE const *globalReadB_0_0_13_0 = B + globalReadOffsetB_0_0_13_0;
  DATA_TYPE const *globalReadB_0_0_14_0 = B + globalReadOffsetB_0_0_14_0;
  DATA_TYPE const *globalReadB_0_0_15_0 = B + globalReadOffsetB_0_0_15_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 4*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 5*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 6*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 7*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_8_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 8*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_9_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 9*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_10_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 10*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_11_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 11*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_12_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 12*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_13_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 13*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_14_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 14*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_15_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 15*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;
  DATA_TYPE *localWriteB_0_0_8_0;
  DATA_TYPE *localWriteB_0_0_9_0;
  DATA_TYPE *localWriteB_0_0_10_0;
  DATA_TYPE *localWriteB_0_0_11_0;
  DATA_TYPE *localWriteB_0_0_12_0;
  DATA_TYPE *localWriteB_0_0_13_0;
  DATA_TYPE *localWriteB_0_0_14_0;
  DATA_TYPE *localWriteB_0_0_15_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);
  localWriteB_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_8_0);
  localWriteB_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_9_0);
  localWriteB_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_10_0);
  localWriteB_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_11_0);
  localWriteB_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_12_0);
  localWriteB_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_13_0);
  localWriteB_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_14_0);
  localWriteB_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_15_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_8_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_9_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_10_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_11_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_12_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_13_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_14_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_15_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);
    b_0_0_8_0 = *(globalReadB_0_0_8_0 + 0);
    b_0_0_9_0 = *(globalReadB_0_0_9_0 + 0);
    b_0_0_10_0 = *(globalReadB_0_0_10_0 + 0);
    b_0_0_11_0 = *(globalReadB_0_0_11_0 + 0);
    b_0_0_12_0 = *(globalReadB_0_0_12_0 + 0);
    b_0_0_13_0 = *(globalReadB_0_0_13_0 + 0);
    b_0_0_14_0 = *(globalReadB_0_0_14_0 + 0);
    b_0_0_15_0 = *(globalReadB_0_0_15_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_8_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_8_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_9_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_9_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_10_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_10_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_11_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_11_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_12_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_12_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_13_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_13_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_14_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_14_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_15_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_15_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
    *(localWriteB_0_0_8_0 + 0) = b_0_0_8_0;
    *(localWriteB_0_0_9_0 + 0) = b_0_0_9_0;
    *(localWriteB_0_0_10_0 + 0) = b_0_0_10_0;
    *(localWriteB_0_0_11_0 + 0) = b_0_0_11_0;
    *(localWriteB_0_0_12_0 + 0) = b_0_0_12_0;
    *(localWriteB_0_0_13_0 + 0) = b_0_0_13_0;
    *(localWriteB_0_0_14_0 + 0) = b_0_0_14_0;
    *(localWriteB_0_0_15_0 + 0) = b_0_0_15_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x8


    /* iter 15 */

    MAC_2x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);
  b_0_0_8_0 = ( globalReadOffsetBL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_8_0 + 0);
  b_0_0_9_0 = ( globalReadOffsetBL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_9_0 + 0);
  b_0_0_10_0 = ( globalReadOffsetBL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_10_0 + 0);
  b_0_0_11_0 = ( globalReadOffsetBL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_11_0 + 0);
  b_0_0_12_0 = ( globalReadOffsetBL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_12_0 + 0);
  b_0_0_13_0 = ( globalReadOffsetBL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_13_0 + 0);
  b_0_0_14_0 = ( globalReadOffsetBL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_14_0 + 0);
  b_0_0_15_0 = ( globalReadOffsetBL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_15_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);
  localWriteB_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_8_0);
  localWriteB_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_9_0);
  localWriteB_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_10_0);
  localWriteB_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_11_0);
  localWriteB_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_12_0);
  localWriteB_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_13_0);
  localWriteB_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_14_0);
  localWriteB_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_15_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
  *(localWriteB_0_0_8_0 + 0) = b_0_0_8_0;
  *(localWriteB_0_0_9_0 + 0) = b_0_0_9_0;
  *(localWriteB_0_0_10_0 + 0) = b_0_0_10_0;
  *(localWriteB_0_0_11_0 + 0) = b_0_0_11_0;
  *(localWriteB_0_0_12_0 + 0) = b_0_0_12_0;
  *(localWriteB_0_0_13_0 + 0) = b_0_0_13_0;
  *(localWriteB_0_0_14_0 + 0) = b_0_0_14_0;
  *(localWriteB_0_0_15_0 + 0) = b_0_0_15_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x8
#undef MAC_2x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 819

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT64x32x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 7 */

    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT128x32x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 1) {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 2) {
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT64x16x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_1_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_1_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_1_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_1_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_1_4_0 = *(globalReadA_0_0_4_0 + 1);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_1_5_0 = *(globalReadA_0_0_5_0 + 1);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_1_6_0 = *(globalReadA_0_0_6_0 + 1);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_1_7_0 = *(globalReadA_0_0_7_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_4_0 + 1) = a_0_1_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_5_0 + 1) = a_0_1_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_6_0 + 1) = a_0_1_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_7_0 + 1) = a_0_1_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_1_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 1);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_1_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 1);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_1_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 1);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_1_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_4_0 + 1) = a_0_1_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_5_0 + 1) = a_0_1_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_6_0 + 1) = a_0_1_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_7_0 + 1) = a_0_1_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 1) {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 2) {
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT128x32x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 8

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x8 micro-tile */
#define MAC_2x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \

#define MAC_2x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT32x128x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;
  unsigned int globalReadOffsetBL_4_0 = globalReadOffsetBL + 0 + 4*LSPB;
  unsigned int globalReadOffsetBL_5_0 = globalReadOffsetBL + 0 + 5*LSPB;
  unsigned int globalReadOffsetBL_6_0 = globalReadOffsetBL + 0 + 6*LSPB;
  unsigned int globalReadOffsetBL_7_0 = globalReadOffsetBL + 0 + 7*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_7_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 4*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 5*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 6*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 7*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x8


    /* iter 15 */

    MAC_2x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x8
#undef MAC_2x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 16
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT64x16x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;
  unsigned int globalReadOffsetAL_8_0 = globalReadOffsetAL + 0 + 8*LSPA;
  unsigned int globalReadOffsetAL_9_0 = globalReadOffsetAL + 0 + 9*LSPA;
  unsigned int globalReadOffsetAL_10_0 = globalReadOffsetAL + 0 + 10*LSPA;
  unsigned int globalReadOffsetAL_11_0 = globalReadOffsetAL + 0 + 11*LSPA;
  unsigned int globalReadOffsetAL_12_0 = globalReadOffsetAL + 0 + 12*LSPA;
  unsigned int globalReadOffsetAL_13_0 = globalReadOffsetAL + 0 + 13*LSPA;
  unsigned int globalReadOffsetAL_14_0 = globalReadOffsetAL + 0 + 14*LSPA;
  unsigned int globalReadOffsetAL_15_0 = globalReadOffsetAL + 0 + 15*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_15_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 8*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 9*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 10*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 11*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 12*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 13*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 14*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 15*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_0_0_15_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_8_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_9_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_10_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_11_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_12_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_13_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_14_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_15_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_8_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_9_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_10_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_11_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_12_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_13_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_14_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_15_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_0_8_0 = ( globalReadOffsetAL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_0_0_9_0 = ( globalReadOffsetAL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_0_0_10_0 = ( globalReadOffsetAL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_0_0_11_0 = ( globalReadOffsetAL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_0_0_12_0 = ( globalReadOffsetAL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_0_0_13_0 = ( globalReadOffsetAL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_0_0_14_0 = ( globalReadOffsetAL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_0_0_15_0 = ( globalReadOffsetAL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 16

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x8 micro-tile */
#define MAC_2x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \

#define MAC_2x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT16x64x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;
  DATA_TYPE b_0_0_8_0;
  DATA_TYPE b_0_0_9_0;
  DATA_TYPE b_0_0_10_0;
  DATA_TYPE b_0_0_11_0;
  DATA_TYPE b_0_0_12_0;
  DATA_TYPE b_0_0_13_0;
  DATA_TYPE b_0_0_14_0;
  DATA_TYPE b_0_0_15_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;
  unsigned int globalReadOffsetBL_4_0 = globalReadOffsetBL + 0 + 4*LSPB;
  unsigned int globalReadOffsetBL_5_0 = globalReadOffsetBL + 0 + 5*LSPB;
  unsigned int globalReadOffsetBL_6_0 = globalReadOffsetBL + 0 + 6*LSPB;
  unsigned int globalReadOffsetBL_7_0 = globalReadOffsetBL + 0 + 7*LSPB;
  unsigned int globalReadOffsetBL_8_0 = globalReadOffsetBL + 0 + 8*LSPB;
  unsigned int globalReadOffsetBL_9_0 = globalReadOffsetBL + 0 + 9*LSPB;
  unsigned int globalReadOffsetBL_10_0 = globalReadOffsetBL + 0 + 10*LSPB;
  unsigned int globalReadOffsetBL_11_0 = globalReadOffsetBL + 0 + 11*LSPB;
  unsigned int globalReadOffsetBL_12_0 = globalReadOffsetBL + 0 + 12*LSPB;
  unsigned int globalReadOffsetBL_13_0 = globalReadOffsetBL + 0 + 13*LSPB;
  unsigned int globalReadOffsetBL_14_0 = globalReadOffsetBL + 0 + 14*LSPB;
  unsigned int globalReadOffsetBL_15_0 = globalReadOffsetBL + 0 + 15*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_7_0), wgK );
  int64_t globalReadOffsetB_0_0_8_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_8_0), wgK );
  int64_t globalReadOffsetB_0_0_9_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_9_0), wgK );
  int64_t globalReadOffsetB_0_0_10_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_10_0), wgK );
  int64_t globalReadOffsetB_0_0_11_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_11_0), wgK );
  int64_t globalReadOffsetB_0_0_12_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_12_0), wgK );
  int64_t globalReadOffsetB_0_0_13_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_13_0), wgK );
  int64_t globalReadOffsetB_0_0_14_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_14_0), wgK );
  int64_t globalReadOffsetB_0_0_15_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_15_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;
  DATA_TYPE const *globalReadB_0_0_8_0 = B + globalReadOffsetB_0_0_8_0;
  DATA_TYPE const *globalReadB_0_0_9_0 = B + globalReadOffsetB_0_0_9_0;
  DATA_TYPE const *globalReadB_0_0_10_0 = B + globalReadOffsetB_0_0_10_0;
  DATA_TYPE const *globalReadB_0_0_11_0 = B + globalReadOffsetB_0_0_11_0;
  DATA_TYPE const *globalReadB_0_0_12_0 = B + globalReadOffsetB_0_0_12_0;
  DATA_TYPE const *globalReadB_0_0_13_0 = B + globalReadOffsetB_0_0_13_0;
  DATA_TYPE const *globalReadB_0_0_14_0 = B + globalReadOffsetB_0_0_14_0;
  DATA_TYPE const *globalReadB_0_0_15_0 = B + globalReadOffsetB_0_0_15_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 4*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 5*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 6*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 7*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_8_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 8*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_9_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 9*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_10_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 10*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_11_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 11*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_12_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 12*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_13_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 13*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_14_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 14*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_15_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 15*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;
  DATA_TYPE *localWriteB_0_0_8_0;
  DATA_TYPE *localWriteB_0_0_9_0;
  DATA_TYPE *localWriteB_0_0_10_0;
  DATA_TYPE *localWriteB_0_0_11_0;
  DATA_TYPE *localWriteB_0_0_12_0;
  DATA_TYPE *localWriteB_0_0_13_0;
  DATA_TYPE *localWriteB_0_0_14_0;
  DATA_TYPE *localWriteB_0_0_15_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);
  localWriteB_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_8_0);
  localWriteB_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_9_0);
  localWriteB_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_10_0);
  localWriteB_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_11_0);
  localWriteB_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_12_0);
  localWriteB_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_13_0);
  localWriteB_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_14_0);
  localWriteB_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_15_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_8_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_9_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_10_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_11_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_12_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_13_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_14_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_15_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);
    b_0_0_8_0 = *(globalReadB_0_0_8_0 + 0);
    b_0_0_9_0 = *(globalReadB_0_0_9_0 + 0);
    b_0_0_10_0 = *(globalReadB_0_0_10_0 + 0);
    b_0_0_11_0 = *(globalReadB_0_0_11_0 + 0);
    b_0_0_12_0 = *(globalReadB_0_0_12_0 + 0);
    b_0_0_13_0 = *(globalReadB_0_0_13_0 + 0);
    b_0_0_14_0 = *(globalReadB_0_0_14_0 + 0);
    b_0_0_15_0 = *(globalReadB_0_0_15_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_8_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_8_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_9_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_9_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_10_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_10_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_11_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_11_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_12_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_12_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_13_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_13_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_14_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_14_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_15_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_15_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
    *(localWriteB_0_0_8_0 + 0) = b_0_0_8_0;
    *(localWriteB_0_0_9_0 + 0) = b_0_0_9_0;
    *(localWriteB_0_0_10_0 + 0) = b_0_0_10_0;
    *(localWriteB_0_0_11_0 + 0) = b_0_0_11_0;
    *(localWriteB_0_0_12_0 + 0) = b_0_0_12_0;
    *(localWriteB_0_0_13_0 + 0) = b_0_0_13_0;
    *(localWriteB_0_0_14_0 + 0) = b_0_0_14_0;
    *(localWriteB_0_0_15_0 + 0) = b_0_0_15_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x8


    /* iter 15 */

    MAC_2x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);
  b_0_0_8_0 = ( globalReadOffsetBL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_8_0 + 0);
  b_0_0_9_0 = ( globalReadOffsetBL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_9_0 + 0);
  b_0_0_10_0 = ( globalReadOffsetBL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_10_0 + 0);
  b_0_0_11_0 = ( globalReadOffsetBL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_11_0 + 0);
  b_0_0_12_0 = ( globalReadOffsetBL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_12_0 + 0);
  b_0_0_13_0 = ( globalReadOffsetBL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_13_0 + 0);
  b_0_0_14_0 = ( globalReadOffsetBL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_14_0 + 0);
  b_0_0_15_0 = ( globalReadOffsetBL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_15_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);
  localWriteB_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_8_0);
  localWriteB_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_9_0);
  localWriteB_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_10_0);
  localWriteB_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_11_0);
  localWriteB_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_12_0);
  localWriteB_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_13_0);
  localWriteB_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_14_0);
  localWriteB_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_15_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
  *(localWriteB_0_0_8_0 + 0) = b_0_0_8_0;
  *(localWriteB_0_0_9_0 + 0) = b_0_0_9_0;
  *(localWriteB_0_0_10_0 + 0) = b_0_0_10_0;
  *(localWriteB_0_0_11_0 + 0) = b_0_0_11_0;
  *(localWriteB_0_0_12_0 + 0) = b_0_0_12_0;
  *(localWriteB_0_0_13_0 + 0) = b_0_0_13_0;
  *(localWriteB_0_0_14_0 + 0) = b_0_0_14_0;
  *(localWriteB_0_0_15_0 + 0) = b_0_0_15_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x8
#undef MAC_2x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 64
#define SG1J 1
#define TT0I 1
#define TT1J 1
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 64
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 60

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 2

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 64
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 4096
#define LDS_NUM_ELEMENTS 4160

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 1x1 micro-tile */
#define MAC_1x1\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \

#define MAC_1x1_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT64x1x64_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU60_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS128_SRVW0_SVW4_SNLL0_TT1_1_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG64_1_1_WGM2(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE a_0_0_16_0;
  DATA_TYPE a_0_0_17_0;
  DATA_TYPE a_0_0_18_0;
  DATA_TYPE a_0_0_19_0;
  DATA_TYPE a_0_0_20_0;
  DATA_TYPE a_0_0_21_0;
  DATA_TYPE a_0_0_22_0;
  DATA_TYPE a_0_0_23_0;
  DATA_TYPE a_0_0_24_0;
  DATA_TYPE a_0_0_25_0;
  DATA_TYPE a_0_0_26_0;
  DATA_TYPE a_0_0_27_0;
  DATA_TYPE a_0_0_28_0;
  DATA_TYPE a_0_0_29_0;
  DATA_TYPE a_0_0_30_0;
  DATA_TYPE a_0_0_31_0;
  DATA_TYPE a_0_0_32_0;
  DATA_TYPE a_0_0_33_0;
  DATA_TYPE a_0_0_34_0;
  DATA_TYPE a_0_0_35_0;
  DATA_TYPE a_0_0_36_0;
  DATA_TYPE a_0_0_37_0;
  DATA_TYPE a_0_0_38_0;
  DATA_TYPE a_0_0_39_0;
  DATA_TYPE a_0_0_40_0;
  DATA_TYPE a_0_0_41_0;
  DATA_TYPE a_0_0_42_0;
  DATA_TYPE a_0_0_43_0;
  DATA_TYPE a_0_0_44_0;
  DATA_TYPE a_0_0_45_0;
  DATA_TYPE a_0_0_46_0;
  DATA_TYPE a_0_0_47_0;
  DATA_TYPE a_0_0_48_0;
  DATA_TYPE a_0_0_49_0;
  DATA_TYPE a_0_0_50_0;
  DATA_TYPE a_0_0_51_0;
  DATA_TYPE a_0_0_52_0;
  DATA_TYPE a_0_0_53_0;
  DATA_TYPE a_0_0_54_0;
  DATA_TYPE a_0_0_55_0;
  DATA_TYPE a_0_0_56_0;
  DATA_TYPE a_0_0_57_0;
  DATA_TYPE a_0_0_58_0;
  DATA_TYPE a_0_0_59_0;
  DATA_TYPE a_0_0_60_0;
  DATA_TYPE a_0_0_61_0;
  DATA_TYPE a_0_0_62_0;
  DATA_TYPE a_0_0_63_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;
  unsigned int globalReadOffsetAL_8_0 = globalReadOffsetAL + 0 + 8*LSPA;
  unsigned int globalReadOffsetAL_9_0 = globalReadOffsetAL + 0 + 9*LSPA;
  unsigned int globalReadOffsetAL_10_0 = globalReadOffsetAL + 0 + 10*LSPA;
  unsigned int globalReadOffsetAL_11_0 = globalReadOffsetAL + 0 + 11*LSPA;
  unsigned int globalReadOffsetAL_12_0 = globalReadOffsetAL + 0 + 12*LSPA;
  unsigned int globalReadOffsetAL_13_0 = globalReadOffsetAL + 0 + 13*LSPA;
  unsigned int globalReadOffsetAL_14_0 = globalReadOffsetAL + 0 + 14*LSPA;
  unsigned int globalReadOffsetAL_15_0 = globalReadOffsetAL + 0 + 15*LSPA;
  unsigned int globalReadOffsetAL_16_0 = globalReadOffsetAL + 0 + 16*LSPA;
  unsigned int globalReadOffsetAL_17_0 = globalReadOffsetAL + 0 + 17*LSPA;
  unsigned int globalReadOffsetAL_18_0 = globalReadOffsetAL + 0 + 18*LSPA;
  unsigned int globalReadOffsetAL_19_0 = globalReadOffsetAL + 0 + 19*LSPA;
  unsigned int globalReadOffsetAL_20_0 = globalReadOffsetAL + 0 + 20*LSPA;
  unsigned int globalReadOffsetAL_21_0 = globalReadOffsetAL + 0 + 21*LSPA;
  unsigned int globalReadOffsetAL_22_0 = globalReadOffsetAL + 0 + 22*LSPA;
  unsigned int globalReadOffsetAL_23_0 = globalReadOffsetAL + 0 + 23*LSPA;
  unsigned int globalReadOffsetAL_24_0 = globalReadOffsetAL + 0 + 24*LSPA;
  unsigned int globalReadOffsetAL_25_0 = globalReadOffsetAL + 0 + 25*LSPA;
  unsigned int globalReadOffsetAL_26_0 = globalReadOffsetAL + 0 + 26*LSPA;
  unsigned int globalReadOffsetAL_27_0 = globalReadOffsetAL + 0 + 27*LSPA;
  unsigned int globalReadOffsetAL_28_0 = globalReadOffsetAL + 0 + 28*LSPA;
  unsigned int globalReadOffsetAL_29_0 = globalReadOffsetAL + 0 + 29*LSPA;
  unsigned int globalReadOffsetAL_30_0 = globalReadOffsetAL + 0 + 30*LSPA;
  unsigned int globalReadOffsetAL_31_0 = globalReadOffsetAL + 0 + 31*LSPA;
  unsigned int globalReadOffsetAL_32_0 = globalReadOffsetAL + 0 + 32*LSPA;
  unsigned int globalReadOffsetAL_33_0 = globalReadOffsetAL + 0 + 33*LSPA;
  unsigned int globalReadOffsetAL_34_0 = globalReadOffsetAL + 0 + 34*LSPA;
  unsigned int globalReadOffsetAL_35_0 = globalReadOffsetAL + 0 + 35*LSPA;
  unsigned int globalReadOffsetAL_36_0 = globalReadOffsetAL + 0 + 36*LSPA;
  unsigned int globalReadOffsetAL_37_0 = globalReadOffsetAL + 0 + 37*LSPA;
  unsigned int globalReadOffsetAL_38_0 = globalReadOffsetAL + 0 + 38*LSPA;
  unsigned int globalReadOffsetAL_39_0 = globalReadOffsetAL + 0 + 39*LSPA;
  unsigned int globalReadOffsetAL_40_0 = globalReadOffsetAL + 0 + 40*LSPA;
  unsigned int globalReadOffsetAL_41_0 = globalReadOffsetAL + 0 + 41*LSPA;
  unsigned int globalReadOffsetAL_42_0 = globalReadOffsetAL + 0 + 42*LSPA;
  unsigned int globalReadOffsetAL_43_0 = globalReadOffsetAL + 0 + 43*LSPA;
  unsigned int globalReadOffsetAL_44_0 = globalReadOffsetAL + 0 + 44*LSPA;
  unsigned int globalReadOffsetAL_45_0 = globalReadOffsetAL + 0 + 45*LSPA;
  unsigned int globalReadOffsetAL_46_0 = globalReadOffsetAL + 0 + 46*LSPA;
  unsigned int globalReadOffsetAL_47_0 = globalReadOffsetAL + 0 + 47*LSPA;
  unsigned int globalReadOffsetAL_48_0 = globalReadOffsetAL + 0 + 48*LSPA;
  unsigned int globalReadOffsetAL_49_0 = globalReadOffsetAL + 0 + 49*LSPA;
  unsigned int globalReadOffsetAL_50_0 = globalReadOffsetAL + 0 + 50*LSPA;
  unsigned int globalReadOffsetAL_51_0 = globalReadOffsetAL + 0 + 51*LSPA;
  unsigned int globalReadOffsetAL_52_0 = globalReadOffsetAL + 0 + 52*LSPA;
  unsigned int globalReadOffsetAL_53_0 = globalReadOffsetAL + 0 + 53*LSPA;
  unsigned int globalReadOffsetAL_54_0 = globalReadOffsetAL + 0 + 54*LSPA;
  unsigned int globalReadOffsetAL_55_0 = globalReadOffsetAL + 0 + 55*LSPA;
  unsigned int globalReadOffsetAL_56_0 = globalReadOffsetAL + 0 + 56*LSPA;
  unsigned int globalReadOffsetAL_57_0 = globalReadOffsetAL + 0 + 57*LSPA;
  unsigned int globalReadOffsetAL_58_0 = globalReadOffsetAL + 0 + 58*LSPA;
  unsigned int globalReadOffsetAL_59_0 = globalReadOffsetAL + 0 + 59*LSPA;
  unsigned int globalReadOffsetAL_60_0 = globalReadOffsetAL + 0 + 60*LSPA;
  unsigned int globalReadOffsetAL_61_0 = globalReadOffsetAL + 0 + 61*LSPA;
  unsigned int globalReadOffsetAL_62_0 = globalReadOffsetAL + 0 + 62*LSPA;
  unsigned int globalReadOffsetAL_63_0 = globalReadOffsetAL + 0 + 63*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_15_0), wgK );
  int64_t globalReadOffsetA_0_0_16_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_16_0), wgK );
  int64_t globalReadOffsetA_0_0_17_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_17_0), wgK );
  int64_t globalReadOffsetA_0_0_18_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_18_0), wgK );
  int64_t globalReadOffsetA_0_0_19_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_19_0), wgK );
  int64_t globalReadOffsetA_0_0_20_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_20_0), wgK );
  int64_t globalReadOffsetA_0_0_21_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_21_0), wgK );
  int64_t globalReadOffsetA_0_0_22_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_22_0), wgK );
  int64_t globalReadOffsetA_0_0_23_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_23_0), wgK );
  int64_t globalReadOffsetA_0_0_24_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_24_0), wgK );
  int64_t globalReadOffsetA_0_0_25_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_25_0), wgK );
  int64_t globalReadOffsetA_0_0_26_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_26_0), wgK );
  int64_t globalReadOffsetA_0_0_27_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_27_0), wgK );
  int64_t globalReadOffsetA_0_0_28_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_28_0), wgK );
  int64_t globalReadOffsetA_0_0_29_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_29_0), wgK );
  int64_t globalReadOffsetA_0_0_30_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_30_0), wgK );
  int64_t globalReadOffsetA_0_0_31_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_31_0), wgK );
  int64_t globalReadOffsetA_0_0_32_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_32_0), wgK );
  int64_t globalReadOffsetA_0_0_33_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_33_0), wgK );
  int64_t globalReadOffsetA_0_0_34_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_34_0), wgK );
  int64_t globalReadOffsetA_0_0_35_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_35_0), wgK );
  int64_t globalReadOffsetA_0_0_36_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_36_0), wgK );
  int64_t globalReadOffsetA_0_0_37_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_37_0), wgK );
  int64_t globalReadOffsetA_0_0_38_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_38_0), wgK );
  int64_t globalReadOffsetA_0_0_39_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_39_0), wgK );
  int64_t globalReadOffsetA_0_0_40_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_40_0), wgK );
  int64_t globalReadOffsetA_0_0_41_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_41_0), wgK );
  int64_t globalReadOffsetA_0_0_42_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_42_0), wgK );
  int64_t globalReadOffsetA_0_0_43_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_43_0), wgK );
  int64_t globalReadOffsetA_0_0_44_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_44_0), wgK );
  int64_t globalReadOffsetA_0_0_45_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_45_0), wgK );
  int64_t globalReadOffsetA_0_0_46_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_46_0), wgK );
  int64_t globalReadOffsetA_0_0_47_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_47_0), wgK );
  int64_t globalReadOffsetA_0_0_48_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_48_0), wgK );
  int64_t globalReadOffsetA_0_0_49_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_49_0), wgK );
  int64_t globalReadOffsetA_0_0_50_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_50_0), wgK );
  int64_t globalReadOffsetA_0_0_51_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_51_0), wgK );
  int64_t globalReadOffsetA_0_0_52_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_52_0), wgK );
  int64_t globalReadOffsetA_0_0_53_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_53_0), wgK );
  int64_t globalReadOffsetA_0_0_54_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_54_0), wgK );
  int64_t globalReadOffsetA_0_0_55_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_55_0), wgK );
  int64_t globalReadOffsetA_0_0_56_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_56_0), wgK );
  int64_t globalReadOffsetA_0_0_57_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_57_0), wgK );
  int64_t globalReadOffsetA_0_0_58_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_58_0), wgK );
  int64_t globalReadOffsetA_0_0_59_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_59_0), wgK );
  int64_t globalReadOffsetA_0_0_60_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_60_0), wgK );
  int64_t globalReadOffsetA_0_0_61_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_61_0), wgK );
  int64_t globalReadOffsetA_0_0_62_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_62_0), wgK );
  int64_t globalReadOffsetA_0_0_63_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_63_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;
  DATA_TYPE const *globalReadA_0_0_16_0 = A + globalReadOffsetA_0_0_16_0;
  DATA_TYPE const *globalReadA_0_0_17_0 = A + globalReadOffsetA_0_0_17_0;
  DATA_TYPE const *globalReadA_0_0_18_0 = A + globalReadOffsetA_0_0_18_0;
  DATA_TYPE const *globalReadA_0_0_19_0 = A + globalReadOffsetA_0_0_19_0;
  DATA_TYPE const *globalReadA_0_0_20_0 = A + globalReadOffsetA_0_0_20_0;
  DATA_TYPE const *globalReadA_0_0_21_0 = A + globalReadOffsetA_0_0_21_0;
  DATA_TYPE const *globalReadA_0_0_22_0 = A + globalReadOffsetA_0_0_22_0;
  DATA_TYPE const *globalReadA_0_0_23_0 = A + globalReadOffsetA_0_0_23_0;
  DATA_TYPE const *globalReadA_0_0_24_0 = A + globalReadOffsetA_0_0_24_0;
  DATA_TYPE const *globalReadA_0_0_25_0 = A + globalReadOffsetA_0_0_25_0;
  DATA_TYPE const *globalReadA_0_0_26_0 = A + globalReadOffsetA_0_0_26_0;
  DATA_TYPE const *globalReadA_0_0_27_0 = A + globalReadOffsetA_0_0_27_0;
  DATA_TYPE const *globalReadA_0_0_28_0 = A + globalReadOffsetA_0_0_28_0;
  DATA_TYPE const *globalReadA_0_0_29_0 = A + globalReadOffsetA_0_0_29_0;
  DATA_TYPE const *globalReadA_0_0_30_0 = A + globalReadOffsetA_0_0_30_0;
  DATA_TYPE const *globalReadA_0_0_31_0 = A + globalReadOffsetA_0_0_31_0;
  DATA_TYPE const *globalReadA_0_0_32_0 = A + globalReadOffsetA_0_0_32_0;
  DATA_TYPE const *globalReadA_0_0_33_0 = A + globalReadOffsetA_0_0_33_0;
  DATA_TYPE const *globalReadA_0_0_34_0 = A + globalReadOffsetA_0_0_34_0;
  DATA_TYPE const *globalReadA_0_0_35_0 = A + globalReadOffsetA_0_0_35_0;
  DATA_TYPE const *globalReadA_0_0_36_0 = A + globalReadOffsetA_0_0_36_0;
  DATA_TYPE const *globalReadA_0_0_37_0 = A + globalReadOffsetA_0_0_37_0;
  DATA_TYPE const *globalReadA_0_0_38_0 = A + globalReadOffsetA_0_0_38_0;
  DATA_TYPE const *globalReadA_0_0_39_0 = A + globalReadOffsetA_0_0_39_0;
  DATA_TYPE const *globalReadA_0_0_40_0 = A + globalReadOffsetA_0_0_40_0;
  DATA_TYPE const *globalReadA_0_0_41_0 = A + globalReadOffsetA_0_0_41_0;
  DATA_TYPE const *globalReadA_0_0_42_0 = A + globalReadOffsetA_0_0_42_0;
  DATA_TYPE const *globalReadA_0_0_43_0 = A + globalReadOffsetA_0_0_43_0;
  DATA_TYPE const *globalReadA_0_0_44_0 = A + globalReadOffsetA_0_0_44_0;
  DATA_TYPE const *globalReadA_0_0_45_0 = A + globalReadOffsetA_0_0_45_0;
  DATA_TYPE const *globalReadA_0_0_46_0 = A + globalReadOffsetA_0_0_46_0;
  DATA_TYPE const *globalReadA_0_0_47_0 = A + globalReadOffsetA_0_0_47_0;
  DATA_TYPE const *globalReadA_0_0_48_0 = A + globalReadOffsetA_0_0_48_0;
  DATA_TYPE const *globalReadA_0_0_49_0 = A + globalReadOffsetA_0_0_49_0;
  DATA_TYPE const *globalReadA_0_0_50_0 = A + globalReadOffsetA_0_0_50_0;
  DATA_TYPE const *globalReadA_0_0_51_0 = A + globalReadOffsetA_0_0_51_0;
  DATA_TYPE const *globalReadA_0_0_52_0 = A + globalReadOffsetA_0_0_52_0;
  DATA_TYPE const *globalReadA_0_0_53_0 = A + globalReadOffsetA_0_0_53_0;
  DATA_TYPE const *globalReadA_0_0_54_0 = A + globalReadOffsetA_0_0_54_0;
  DATA_TYPE const *globalReadA_0_0_55_0 = A + globalReadOffsetA_0_0_55_0;
  DATA_TYPE const *globalReadA_0_0_56_0 = A + globalReadOffsetA_0_0_56_0;
  DATA_TYPE const *globalReadA_0_0_57_0 = A + globalReadOffsetA_0_0_57_0;
  DATA_TYPE const *globalReadA_0_0_58_0 = A + globalReadOffsetA_0_0_58_0;
  DATA_TYPE const *globalReadA_0_0_59_0 = A + globalReadOffsetA_0_0_59_0;
  DATA_TYPE const *globalReadA_0_0_60_0 = A + globalReadOffsetA_0_0_60_0;
  DATA_TYPE const *globalReadA_0_0_61_0 = A + globalReadOffsetA_0_0_61_0;
  DATA_TYPE const *globalReadA_0_0_62_0 = A + globalReadOffsetA_0_0_62_0;
  DATA_TYPE const *globalReadA_0_0_63_0 = A + globalReadOffsetA_0_0_63_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 8*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 9*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 10*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 11*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 12*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 13*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 14*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 15*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_16_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 16*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_17_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 17*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_18_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 18*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_19_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 19*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_20_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 20*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_21_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 21*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_22_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 22*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_23_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 23*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_24_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 24*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_25_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 25*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_26_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 26*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_27_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 27*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_28_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 28*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_29_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 29*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_30_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 30*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_31_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 31*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_32_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 32*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_33_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 33*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_34_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 34*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_35_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 35*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_36_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 36*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_37_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 37*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_38_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 38*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_39_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 39*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_40_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 40*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_41_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 41*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_42_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 42*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_43_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 43*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_44_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 44*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_45_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 45*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_46_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 46*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_47_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 47*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_48_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 48*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_49_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 49*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_50_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 50*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_51_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 51*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_52_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 52*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_53_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 53*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_54_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 54*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_55_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 55*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_56_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 56*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_57_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 57*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_58_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 58*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_59_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 59*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_60_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 60*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_61_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 61*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_62_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 62*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_63_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 63*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_0_0_15_0;
  DATA_TYPE *localWriteA_0_0_16_0;
  DATA_TYPE *localWriteA_0_0_17_0;
  DATA_TYPE *localWriteA_0_0_18_0;
  DATA_TYPE *localWriteA_0_0_19_0;
  DATA_TYPE *localWriteA_0_0_20_0;
  DATA_TYPE *localWriteA_0_0_21_0;
  DATA_TYPE *localWriteA_0_0_22_0;
  DATA_TYPE *localWriteA_0_0_23_0;
  DATA_TYPE *localWriteA_0_0_24_0;
  DATA_TYPE *localWriteA_0_0_25_0;
  DATA_TYPE *localWriteA_0_0_26_0;
  DATA_TYPE *localWriteA_0_0_27_0;
  DATA_TYPE *localWriteA_0_0_28_0;
  DATA_TYPE *localWriteA_0_0_29_0;
  DATA_TYPE *localWriteA_0_0_30_0;
  DATA_TYPE *localWriteA_0_0_31_0;
  DATA_TYPE *localWriteA_0_0_32_0;
  DATA_TYPE *localWriteA_0_0_33_0;
  DATA_TYPE *localWriteA_0_0_34_0;
  DATA_TYPE *localWriteA_0_0_35_0;
  DATA_TYPE *localWriteA_0_0_36_0;
  DATA_TYPE *localWriteA_0_0_37_0;
  DATA_TYPE *localWriteA_0_0_38_0;
  DATA_TYPE *localWriteA_0_0_39_0;
  DATA_TYPE *localWriteA_0_0_40_0;
  DATA_TYPE *localWriteA_0_0_41_0;
  DATA_TYPE *localWriteA_0_0_42_0;
  DATA_TYPE *localWriteA_0_0_43_0;
  DATA_TYPE *localWriteA_0_0_44_0;
  DATA_TYPE *localWriteA_0_0_45_0;
  DATA_TYPE *localWriteA_0_0_46_0;
  DATA_TYPE *localWriteA_0_0_47_0;
  DATA_TYPE *localWriteA_0_0_48_0;
  DATA_TYPE *localWriteA_0_0_49_0;
  DATA_TYPE *localWriteA_0_0_50_0;
  DATA_TYPE *localWriteA_0_0_51_0;
  DATA_TYPE *localWriteA_0_0_52_0;
  DATA_TYPE *localWriteA_0_0_53_0;
  DATA_TYPE *localWriteA_0_0_54_0;
  DATA_TYPE *localWriteA_0_0_55_0;
  DATA_TYPE *localWriteA_0_0_56_0;
  DATA_TYPE *localWriteA_0_0_57_0;
  DATA_TYPE *localWriteA_0_0_58_0;
  DATA_TYPE *localWriteA_0_0_59_0;
  DATA_TYPE *localWriteA_0_0_60_0;
  DATA_TYPE *localWriteA_0_0_61_0;
  DATA_TYPE *localWriteA_0_0_62_0;
  DATA_TYPE *localWriteA_0_0_63_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);
  localWriteA_0_0_16_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_16_0);
  localWriteA_0_0_17_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_17_0);
  localWriteA_0_0_18_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_18_0);
  localWriteA_0_0_19_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_19_0);
  localWriteA_0_0_20_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_20_0);
  localWriteA_0_0_21_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_21_0);
  localWriteA_0_0_22_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_22_0);
  localWriteA_0_0_23_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_23_0);
  localWriteA_0_0_24_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_24_0);
  localWriteA_0_0_25_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_25_0);
  localWriteA_0_0_26_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_26_0);
  localWriteA_0_0_27_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_27_0);
  localWriteA_0_0_28_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_28_0);
  localWriteA_0_0_29_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_29_0);
  localWriteA_0_0_30_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_30_0);
  localWriteA_0_0_31_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_31_0);
  localWriteA_0_0_32_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_32_0);
  localWriteA_0_0_33_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_33_0);
  localWriteA_0_0_34_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_34_0);
  localWriteA_0_0_35_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_35_0);
  localWriteA_0_0_36_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_36_0);
  localWriteA_0_0_37_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_37_0);
  localWriteA_0_0_38_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_38_0);
  localWriteA_0_0_39_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_39_0);
  localWriteA_0_0_40_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_40_0);
  localWriteA_0_0_41_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_41_0);
  localWriteA_0_0_42_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_42_0);
  localWriteA_0_0_43_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_43_0);
  localWriteA_0_0_44_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_44_0);
  localWriteA_0_0_45_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_45_0);
  localWriteA_0_0_46_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_46_0);
  localWriteA_0_0_47_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_47_0);
  localWriteA_0_0_48_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_48_0);
  localWriteA_0_0_49_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_49_0);
  localWriteA_0_0_50_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_50_0);
  localWriteA_0_0_51_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_51_0);
  localWriteA_0_0_52_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_52_0);
  localWriteA_0_0_53_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_53_0);
  localWriteA_0_0_54_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_54_0);
  localWriteA_0_0_55_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_55_0);
  localWriteA_0_0_56_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_56_0);
  localWriteA_0_0_57_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_57_0);
  localWriteA_0_0_58_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_58_0);
  localWriteA_0_0_59_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_59_0);
  localWriteA_0_0_60_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_60_0);
  localWriteA_0_0_61_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_61_0);
  localWriteA_0_0_62_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_62_0);
  localWriteA_0_0_63_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_63_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 0); // shift so each stagger has 128-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_8_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_9_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_10_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_11_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_12_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_13_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_14_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_15_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_16_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_17_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_18_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_19_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_20_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_21_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_22_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_23_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_24_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_25_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_26_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_27_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_28_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_29_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_30_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_31_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_32_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_33_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_34_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_35_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_36_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_37_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_38_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_39_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_40_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_41_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_42_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_43_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_44_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_45_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_46_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_47_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_48_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_49_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_50_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_51_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_52_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_53_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_54_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_55_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_56_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_57_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_58_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_59_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_60_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_61_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_62_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_63_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);
    a_0_0_16_0 = *(globalReadA_0_0_16_0 + 0);
    a_0_0_17_0 = *(globalReadA_0_0_17_0 + 0);
    a_0_0_18_0 = *(globalReadA_0_0_18_0 + 0);
    a_0_0_19_0 = *(globalReadA_0_0_19_0 + 0);
    a_0_0_20_0 = *(globalReadA_0_0_20_0 + 0);
    a_0_0_21_0 = *(globalReadA_0_0_21_0 + 0);
    a_0_0_22_0 = *(globalReadA_0_0_22_0 + 0);
    a_0_0_23_0 = *(globalReadA_0_0_23_0 + 0);
    a_0_0_24_0 = *(globalReadA_0_0_24_0 + 0);
    a_0_0_25_0 = *(globalReadA_0_0_25_0 + 0);
    a_0_0_26_0 = *(globalReadA_0_0_26_0 + 0);
    a_0_0_27_0 = *(globalReadA_0_0_27_0 + 0);
    a_0_0_28_0 = *(globalReadA_0_0_28_0 + 0);
    a_0_0_29_0 = *(globalReadA_0_0_29_0 + 0);
    a_0_0_30_0 = *(globalReadA_0_0_30_0 + 0);
    a_0_0_31_0 = *(globalReadA_0_0_31_0 + 0);
    a_0_0_32_0 = *(globalReadA_0_0_32_0 + 0);
    a_0_0_33_0 = *(globalReadA_0_0_33_0 + 0);
    a_0_0_34_0 = *(globalReadA_0_0_34_0 + 0);
    a_0_0_35_0 = *(globalReadA_0_0_35_0 + 0);
    a_0_0_36_0 = *(globalReadA_0_0_36_0 + 0);
    a_0_0_37_0 = *(globalReadA_0_0_37_0 + 0);
    a_0_0_38_0 = *(globalReadA_0_0_38_0 + 0);
    a_0_0_39_0 = *(globalReadA_0_0_39_0 + 0);
    a_0_0_40_0 = *(globalReadA_0_0_40_0 + 0);
    a_0_0_41_0 = *(globalReadA_0_0_41_0 + 0);
    a_0_0_42_0 = *(globalReadA_0_0_42_0 + 0);
    a_0_0_43_0 = *(globalReadA_0_0_43_0 + 0);
    a_0_0_44_0 = *(globalReadA_0_0_44_0 + 0);
    a_0_0_45_0 = *(globalReadA_0_0_45_0 + 0);
    a_0_0_46_0 = *(globalReadA_0_0_46_0 + 0);
    a_0_0_47_0 = *(globalReadA_0_0_47_0 + 0);
    a_0_0_48_0 = *(globalReadA_0_0_48_0 + 0);
    a_0_0_49_0 = *(globalReadA_0_0_49_0 + 0);
    a_0_0_50_0 = *(globalReadA_0_0_50_0 + 0);
    a_0_0_51_0 = *(globalReadA_0_0_51_0 + 0);
    a_0_0_52_0 = *(globalReadA_0_0_52_0 + 0);
    a_0_0_53_0 = *(globalReadA_0_0_53_0 + 0);
    a_0_0_54_0 = *(globalReadA_0_0_54_0 + 0);
    a_0_0_55_0 = *(globalReadA_0_0_55_0 + 0);
    a_0_0_56_0 = *(globalReadA_0_0_56_0 + 0);
    a_0_0_57_0 = *(globalReadA_0_0_57_0 + 0);
    a_0_0_58_0 = *(globalReadA_0_0_58_0 + 0);
    a_0_0_59_0 = *(globalReadA_0_0_59_0 + 0);
    a_0_0_60_0 = *(globalReadA_0_0_60_0 + 0);
    a_0_0_61_0 = *(globalReadA_0_0_61_0 + 0);
    a_0_0_62_0 = *(globalReadA_0_0_62_0 + 0);
    a_0_0_63_0 = *(globalReadA_0_0_63_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_8_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_9_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_10_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_11_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_12_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_13_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_14_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_15_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_16_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_16_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_16_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_17_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_17_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_17_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_18_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_18_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_18_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_19_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_19_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_19_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_20_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_20_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_20_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_21_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_21_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_21_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_22_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_22_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_22_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_23_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_23_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_23_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_24_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_24_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_24_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_25_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_25_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_25_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_26_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_26_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_26_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_27_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_27_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_27_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_28_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_28_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_28_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_29_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_29_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_29_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_30_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_30_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_30_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_31_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_31_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_31_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_32_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_32_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_32_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_33_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_33_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_33_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_34_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_34_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_34_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_35_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_35_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_35_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_36_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_36_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_36_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_37_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_37_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_37_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_38_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_38_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_38_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_39_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_39_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_39_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_40_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_40_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_40_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_41_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_41_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_41_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_42_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_42_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_42_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_43_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_43_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_43_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_44_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_44_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_44_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_45_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_45_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_45_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_46_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_46_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_46_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_47_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_47_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_47_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_48_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_48_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_48_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_49_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_49_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_49_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_50_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_50_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_50_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_51_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_51_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_51_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_52_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_52_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_52_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_53_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_53_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_53_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_54_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_54_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_54_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_55_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_55_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_55_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_56_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_56_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_56_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_57_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_57_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_57_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_58_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_58_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_58_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_59_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_59_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_59_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_60_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_60_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_60_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_61_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_61_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_61_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_62_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_62_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_62_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_63_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_63_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_63_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
    *(localWriteA_0_0_16_0 + 0) = a_0_0_16_0;
    *(localWriteA_0_0_17_0 + 0) = a_0_0_17_0;
    *(localWriteA_0_0_18_0 + 0) = a_0_0_18_0;
    *(localWriteA_0_0_19_0 + 0) = a_0_0_19_0;
    *(localWriteA_0_0_20_0 + 0) = a_0_0_20_0;
    *(localWriteA_0_0_21_0 + 0) = a_0_0_21_0;
    *(localWriteA_0_0_22_0 + 0) = a_0_0_22_0;
    *(localWriteA_0_0_23_0 + 0) = a_0_0_23_0;
    *(localWriteA_0_0_24_0 + 0) = a_0_0_24_0;
    *(localWriteA_0_0_25_0 + 0) = a_0_0_25_0;
    *(localWriteA_0_0_26_0 + 0) = a_0_0_26_0;
    *(localWriteA_0_0_27_0 + 0) = a_0_0_27_0;
    *(localWriteA_0_0_28_0 + 0) = a_0_0_28_0;
    *(localWriteA_0_0_29_0 + 0) = a_0_0_29_0;
    *(localWriteA_0_0_30_0 + 0) = a_0_0_30_0;
    *(localWriteA_0_0_31_0 + 0) = a_0_0_31_0;
    *(localWriteA_0_0_32_0 + 0) = a_0_0_32_0;
    *(localWriteA_0_0_33_0 + 0) = a_0_0_33_0;
    *(localWriteA_0_0_34_0 + 0) = a_0_0_34_0;
    *(localWriteA_0_0_35_0 + 0) = a_0_0_35_0;
    *(localWriteA_0_0_36_0 + 0) = a_0_0_36_0;
    *(localWriteA_0_0_37_0 + 0) = a_0_0_37_0;
    *(localWriteA_0_0_38_0 + 0) = a_0_0_38_0;
    *(localWriteA_0_0_39_0 + 0) = a_0_0_39_0;
    *(localWriteA_0_0_40_0 + 0) = a_0_0_40_0;
    *(localWriteA_0_0_41_0 + 0) = a_0_0_41_0;
    *(localWriteA_0_0_42_0 + 0) = a_0_0_42_0;
    *(localWriteA_0_0_43_0 + 0) = a_0_0_43_0;
    *(localWriteA_0_0_44_0 + 0) = a_0_0_44_0;
    *(localWriteA_0_0_45_0 + 0) = a_0_0_45_0;
    *(localWriteA_0_0_46_0 + 0) = a_0_0_46_0;
    *(localWriteA_0_0_47_0 + 0) = a_0_0_47_0;
    *(localWriteA_0_0_48_0 + 0) = a_0_0_48_0;
    *(localWriteA_0_0_49_0 + 0) = a_0_0_49_0;
    *(localWriteA_0_0_50_0 + 0) = a_0_0_50_0;
    *(localWriteA_0_0_51_0 + 0) = a_0_0_51_0;
    *(localWriteA_0_0_52_0 + 0) = a_0_0_52_0;
    *(localWriteA_0_0_53_0 + 0) = a_0_0_53_0;
    *(localWriteA_0_0_54_0 + 0) = a_0_0_54_0;
    *(localWriteA_0_0_55_0 + 0) = a_0_0_55_0;
    *(localWriteA_0_0_56_0 + 0) = a_0_0_56_0;
    *(localWriteA_0_0_57_0 + 0) = a_0_0_57_0;
    *(localWriteA_0_0_58_0 + 0) = a_0_0_58_0;
    *(localWriteA_0_0_59_0 + 0) = a_0_0_59_0;
    *(localWriteA_0_0_60_0 + 0) = a_0_0_60_0;
    *(localWriteA_0_0_61_0 + 0) = a_0_0_61_0;
    *(localWriteA_0_0_62_0 + 0) = a_0_0_62_0;
    *(localWriteA_0_0_63_0 + 0) = a_0_0_63_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 15 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 16 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 17 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 18 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 19 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 20 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 21 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 22 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 23 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 24 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 25 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 26 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 27 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 28 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 29 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 30 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 31 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 32 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 33 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 34 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 35 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 36 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 37 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 38 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 39 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 40 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 41 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 42 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 43 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 44 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 45 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 46 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 47 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 48 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 49 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 50 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 51 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 52 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 53 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 54 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 55 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 56 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 57 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 58 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 59 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 60 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 61 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 62 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_1x1


    /* iter 63 */

    MAC_1x1_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_16_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_17_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_18_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_19_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_20_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_21_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_22_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_23_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_24_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_25_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_26_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_27_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_28_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_29_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_30_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_31_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_32_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_33_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_34_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_35_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_36_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_37_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_38_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_39_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_40_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_41_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_42_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_43_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_44_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_45_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_46_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_47_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_48_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_49_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_50_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_51_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_52_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_53_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_54_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_55_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_56_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_57_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_58_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_59_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_60_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_61_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_62_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_63_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_0_8_0 = ( globalReadOffsetAL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_0_0_9_0 = ( globalReadOffsetAL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_0_0_10_0 = ( globalReadOffsetAL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_0_0_11_0 = ( globalReadOffsetAL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_0_0_12_0 = ( globalReadOffsetAL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_0_0_13_0 = ( globalReadOffsetAL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_0_0_14_0 = ( globalReadOffsetAL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_0_0_15_0 = ( globalReadOffsetAL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);
  a_0_0_16_0 = ( globalReadOffsetAL_16_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_16_0 + 0);
  a_0_0_17_0 = ( globalReadOffsetAL_17_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_17_0 + 0);
  a_0_0_18_0 = ( globalReadOffsetAL_18_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_18_0 + 0);
  a_0_0_19_0 = ( globalReadOffsetAL_19_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_19_0 + 0);
  a_0_0_20_0 = ( globalReadOffsetAL_20_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_20_0 + 0);
  a_0_0_21_0 = ( globalReadOffsetAL_21_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_21_0 + 0);
  a_0_0_22_0 = ( globalReadOffsetAL_22_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_22_0 + 0);
  a_0_0_23_0 = ( globalReadOffsetAL_23_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_23_0 + 0);
  a_0_0_24_0 = ( globalReadOffsetAL_24_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_24_0 + 0);
  a_0_0_25_0 = ( globalReadOffsetAL_25_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_25_0 + 0);
  a_0_0_26_0 = ( globalReadOffsetAL_26_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_26_0 + 0);
  a_0_0_27_0 = ( globalReadOffsetAL_27_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_27_0 + 0);
  a_0_0_28_0 = ( globalReadOffsetAL_28_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_28_0 + 0);
  a_0_0_29_0 = ( globalReadOffsetAL_29_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_29_0 + 0);
  a_0_0_30_0 = ( globalReadOffsetAL_30_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_30_0 + 0);
  a_0_0_31_0 = ( globalReadOffsetAL_31_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_31_0 + 0);
  a_0_0_32_0 = ( globalReadOffsetAL_32_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_32_0 + 0);
  a_0_0_33_0 = ( globalReadOffsetAL_33_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_33_0 + 0);
  a_0_0_34_0 = ( globalReadOffsetAL_34_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_34_0 + 0);
  a_0_0_35_0 = ( globalReadOffsetAL_35_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_35_0 + 0);
  a_0_0_36_0 = ( globalReadOffsetAL_36_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_36_0 + 0);
  a_0_0_37_0 = ( globalReadOffsetAL_37_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_37_0 + 0);
  a_0_0_38_0 = ( globalReadOffsetAL_38_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_38_0 + 0);
  a_0_0_39_0 = ( globalReadOffsetAL_39_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_39_0 + 0);
  a_0_0_40_0 = ( globalReadOffsetAL_40_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_40_0 + 0);
  a_0_0_41_0 = ( globalReadOffsetAL_41_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_41_0 + 0);
  a_0_0_42_0 = ( globalReadOffsetAL_42_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_42_0 + 0);
  a_0_0_43_0 = ( globalReadOffsetAL_43_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_43_0 + 0);
  a_0_0_44_0 = ( globalReadOffsetAL_44_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_44_0 + 0);
  a_0_0_45_0 = ( globalReadOffsetAL_45_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_45_0 + 0);
  a_0_0_46_0 = ( globalReadOffsetAL_46_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_46_0 + 0);
  a_0_0_47_0 = ( globalReadOffsetAL_47_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_47_0 + 0);
  a_0_0_48_0 = ( globalReadOffsetAL_48_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_48_0 + 0);
  a_0_0_49_0 = ( globalReadOffsetAL_49_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_49_0 + 0);
  a_0_0_50_0 = ( globalReadOffsetAL_50_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_50_0 + 0);
  a_0_0_51_0 = ( globalReadOffsetAL_51_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_51_0 + 0);
  a_0_0_52_0 = ( globalReadOffsetAL_52_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_52_0 + 0);
  a_0_0_53_0 = ( globalReadOffsetAL_53_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_53_0 + 0);
  a_0_0_54_0 = ( globalReadOffsetAL_54_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_54_0 + 0);
  a_0_0_55_0 = ( globalReadOffsetAL_55_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_55_0 + 0);
  a_0_0_56_0 = ( globalReadOffsetAL_56_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_56_0 + 0);
  a_0_0_57_0 = ( globalReadOffsetAL_57_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_57_0 + 0);
  a_0_0_58_0 = ( globalReadOffsetAL_58_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_58_0 + 0);
  a_0_0_59_0 = ( globalReadOffsetAL_59_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_59_0 + 0);
  a_0_0_60_0 = ( globalReadOffsetAL_60_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_60_0 + 0);
  a_0_0_61_0 = ( globalReadOffsetAL_61_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_61_0 + 0);
  a_0_0_62_0 = ( globalReadOffsetAL_62_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_62_0 + 0);
  a_0_0_63_0 = ( globalReadOffsetAL_63_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_63_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);
  localWriteA_0_0_16_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_16_0);
  localWriteA_0_0_17_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_17_0);
  localWriteA_0_0_18_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_18_0);
  localWriteA_0_0_19_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_19_0);
  localWriteA_0_0_20_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_20_0);
  localWriteA_0_0_21_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_21_0);
  localWriteA_0_0_22_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_22_0);
  localWriteA_0_0_23_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_23_0);
  localWriteA_0_0_24_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_24_0);
  localWriteA_0_0_25_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_25_0);
  localWriteA_0_0_26_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_26_0);
  localWriteA_0_0_27_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_27_0);
  localWriteA_0_0_28_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_28_0);
  localWriteA_0_0_29_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_29_0);
  localWriteA_0_0_30_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_30_0);
  localWriteA_0_0_31_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_31_0);
  localWriteA_0_0_32_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_32_0);
  localWriteA_0_0_33_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_33_0);
  localWriteA_0_0_34_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_34_0);
  localWriteA_0_0_35_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_35_0);
  localWriteA_0_0_36_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_36_0);
  localWriteA_0_0_37_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_37_0);
  localWriteA_0_0_38_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_38_0);
  localWriteA_0_0_39_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_39_0);
  localWriteA_0_0_40_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_40_0);
  localWriteA_0_0_41_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_41_0);
  localWriteA_0_0_42_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_42_0);
  localWriteA_0_0_43_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_43_0);
  localWriteA_0_0_44_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_44_0);
  localWriteA_0_0_45_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_45_0);
  localWriteA_0_0_46_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_46_0);
  localWriteA_0_0_47_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_47_0);
  localWriteA_0_0_48_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_48_0);
  localWriteA_0_0_49_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_49_0);
  localWriteA_0_0_50_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_50_0);
  localWriteA_0_0_51_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_51_0);
  localWriteA_0_0_52_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_52_0);
  localWriteA_0_0_53_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_53_0);
  localWriteA_0_0_54_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_54_0);
  localWriteA_0_0_55_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_55_0);
  localWriteA_0_0_56_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_56_0);
  localWriteA_0_0_57_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_57_0);
  localWriteA_0_0_58_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_58_0);
  localWriteA_0_0_59_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_59_0);
  localWriteA_0_0_60_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_60_0);
  localWriteA_0_0_61_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_61_0);
  localWriteA_0_0_62_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_62_0);
  localWriteA_0_0_63_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_63_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
  *(localWriteA_0_0_16_0 + 0) = a_0_0_16_0;
  *(localWriteA_0_0_17_0 + 0) = a_0_0_17_0;
  *(localWriteA_0_0_18_0 + 0) = a_0_0_18_0;
  *(localWriteA_0_0_19_0 + 0) = a_0_0_19_0;
  *(localWriteA_0_0_20_0 + 0) = a_0_0_20_0;
  *(localWriteA_0_0_21_0 + 0) = a_0_0_21_0;
  *(localWriteA_0_0_22_0 + 0) = a_0_0_22_0;
  *(localWriteA_0_0_23_0 + 0) = a_0_0_23_0;
  *(localWriteA_0_0_24_0 + 0) = a_0_0_24_0;
  *(localWriteA_0_0_25_0 + 0) = a_0_0_25_0;
  *(localWriteA_0_0_26_0 + 0) = a_0_0_26_0;
  *(localWriteA_0_0_27_0 + 0) = a_0_0_27_0;
  *(localWriteA_0_0_28_0 + 0) = a_0_0_28_0;
  *(localWriteA_0_0_29_0 + 0) = a_0_0_29_0;
  *(localWriteA_0_0_30_0 + 0) = a_0_0_30_0;
  *(localWriteA_0_0_31_0 + 0) = a_0_0_31_0;
  *(localWriteA_0_0_32_0 + 0) = a_0_0_32_0;
  *(localWriteA_0_0_33_0 + 0) = a_0_0_33_0;
  *(localWriteA_0_0_34_0 + 0) = a_0_0_34_0;
  *(localWriteA_0_0_35_0 + 0) = a_0_0_35_0;
  *(localWriteA_0_0_36_0 + 0) = a_0_0_36_0;
  *(localWriteA_0_0_37_0 + 0) = a_0_0_37_0;
  *(localWriteA_0_0_38_0 + 0) = a_0_0_38_0;
  *(localWriteA_0_0_39_0 + 0) = a_0_0_39_0;
  *(localWriteA_0_0_40_0 + 0) = a_0_0_40_0;
  *(localWriteA_0_0_41_0 + 0) = a_0_0_41_0;
  *(localWriteA_0_0_42_0 + 0) = a_0_0_42_0;
  *(localWriteA_0_0_43_0 + 0) = a_0_0_43_0;
  *(localWriteA_0_0_44_0 + 0) = a_0_0_44_0;
  *(localWriteA_0_0_45_0 + 0) = a_0_0_45_0;
  *(localWriteA_0_0_46_0 + 0) = a_0_0_46_0;
  *(localWriteA_0_0_47_0 + 0) = a_0_0_47_0;
  *(localWriteA_0_0_48_0 + 0) = a_0_0_48_0;
  *(localWriteA_0_0_49_0 + 0) = a_0_0_49_0;
  *(localWriteA_0_0_50_0 + 0) = a_0_0_50_0;
  *(localWriteA_0_0_51_0 + 0) = a_0_0_51_0;
  *(localWriteA_0_0_52_0 + 0) = a_0_0_52_0;
  *(localWriteA_0_0_53_0 + 0) = a_0_0_53_0;
  *(localWriteA_0_0_54_0 + 0) = a_0_0_54_0;
  *(localWriteA_0_0_55_0 + 0) = a_0_0_55_0;
  *(localWriteA_0_0_56_0 + 0) = a_0_0_56_0;
  *(localWriteA_0_0_57_0 + 0) = a_0_0_57_0;
  *(localWriteA_0_0_58_0 + 0) = a_0_0_58_0;
  *(localWriteA_0_0_59_0 + 0) = a_0_0_59_0;
  *(localWriteA_0_0_60_0 + 0) = a_0_0_60_0;
  *(localWriteA_0_0_61_0 + 0) = a_0_0_61_0;
  *(localWriteA_0_0_62_0 + 0) = a_0_0_62_0;
  *(localWriteA_0_0_63_0 + 0) = a_0_0_63_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_1x1

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_1x1
#undef MAC_1x1_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 64
#define SG1J 1
#define TT0I 1
#define TT1J 1
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 64
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 64
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 4096
#define LDS_NUM_ELEMENTS 4160

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 1x1 micro-tile */
#define MAC_1x1\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \

#define MAC_1x1_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT64x1x64_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU0_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT1_1_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG64_1_1_WGM1(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE a_0_0_16_0;
  DATA_TYPE a_0_0_17_0;
  DATA_TYPE a_0_0_18_0;
  DATA_TYPE a_0_0_19_0;
  DATA_TYPE a_0_0_20_0;
  DATA_TYPE a_0_0_21_0;
  DATA_TYPE a_0_0_22_0;
  DATA_TYPE a_0_0_23_0;
  DATA_TYPE a_0_0_24_0;
  DATA_TYPE a_0_0_25_0;
  DATA_TYPE a_0_0_26_0;
  DATA_TYPE a_0_0_27_0;
  DATA_TYPE a_0_0_28_0;
  DATA_TYPE a_0_0_29_0;
  DATA_TYPE a_0_0_30_0;
  DATA_TYPE a_0_0_31_0;
  DATA_TYPE a_0_0_32_0;
  DATA_TYPE a_0_0_33_0;
  DATA_TYPE a_0_0_34_0;
  DATA_TYPE a_0_0_35_0;
  DATA_TYPE a_0_0_36_0;
  DATA_TYPE a_0_0_37_0;
  DATA_TYPE a_0_0_38_0;
  DATA_TYPE a_0_0_39_0;
  DATA_TYPE a_0_0_40_0;
  DATA_TYPE a_0_0_41_0;
  DATA_TYPE a_0_0_42_0;
  DATA_TYPE a_0_0_43_0;
  DATA_TYPE a_0_0_44_0;
  DATA_TYPE a_0_0_45_0;
  DATA_TYPE a_0_0_46_0;
  DATA_TYPE a_0_0_47_0;
  DATA_TYPE a_0_0_48_0;
  DATA_TYPE a_0_0_49_0;
  DATA_TYPE a_0_0_50_0;
  DATA_TYPE a_0_0_51_0;
  DATA_TYPE a_0_0_52_0;
  DATA_TYPE a_0_0_53_0;
  DATA_TYPE a_0_0_54_0;
  DATA_TYPE a_0_0_55_0;
  DATA_TYPE a_0_0_56_0;
  DATA_TYPE a_0_0_57_0;
  DATA_TYPE a_0_0_58_0;
  DATA_TYPE a_0_0_59_0;
  DATA_TYPE a_0_0_60_0;
  DATA_TYPE a_0_0_61_0;
  DATA_TYPE a_0_0_62_0;
  DATA_TYPE a_0_0_63_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;
  unsigned int globalReadOffsetAL_8_0 = globalReadOffsetAL + 0 + 8*LSPA;
  unsigned int globalReadOffsetAL_9_0 = globalReadOffsetAL + 0 + 9*LSPA;
  unsigned int globalReadOffsetAL_10_0 = globalReadOffsetAL + 0 + 10*LSPA;
  unsigned int globalReadOffsetAL_11_0 = globalReadOffsetAL + 0 + 11*LSPA;
  unsigned int globalReadOffsetAL_12_0 = globalReadOffsetAL + 0 + 12*LSPA;
  unsigned int globalReadOffsetAL_13_0 = globalReadOffsetAL + 0 + 13*LSPA;
  unsigned int globalReadOffsetAL_14_0 = globalReadOffsetAL + 0 + 14*LSPA;
  unsigned int globalReadOffsetAL_15_0 = globalReadOffsetAL + 0 + 15*LSPA;
  unsigned int globalReadOffsetAL_16_0 = globalReadOffsetAL + 0 + 16*LSPA;
  unsigned int globalReadOffsetAL_17_0 = globalReadOffsetAL + 0 + 17*LSPA;
  unsigned int globalReadOffsetAL_18_0 = globalReadOffsetAL + 0 + 18*LSPA;
  unsigned int globalReadOffsetAL_19_0 = globalReadOffsetAL + 0 + 19*LSPA;
  unsigned int globalReadOffsetAL_20_0 = globalReadOffsetAL + 0 + 20*LSPA;
  unsigned int globalReadOffsetAL_21_0 = globalReadOffsetAL + 0 + 21*LSPA;
  unsigned int globalReadOffsetAL_22_0 = globalReadOffsetAL + 0 + 22*LSPA;
  unsigned int globalReadOffsetAL_23_0 = globalReadOffsetAL + 0 + 23*LSPA;
  unsigned int globalReadOffsetAL_24_0 = globalReadOffsetAL + 0 + 24*LSPA;
  unsigned int globalReadOffsetAL_25_0 = globalReadOffsetAL + 0 + 25*LSPA;
  unsigned int globalReadOffsetAL_26_0 = globalReadOffsetAL + 0 + 26*LSPA;
  unsigned int globalReadOffsetAL_27_0 = globalReadOffsetAL + 0 + 27*LSPA;
  unsigned int globalReadOffsetAL_28_0 = globalReadOffsetAL + 0 + 28*LSPA;
  unsigned int globalReadOffsetAL_29_0 = globalReadOffsetAL + 0 + 29*LSPA;
  unsigned int globalReadOffsetAL_30_0 = globalReadOffsetAL + 0 + 30*LSPA;
  unsigned int globalReadOffsetAL_31_0 = globalReadOffsetAL + 0 + 31*LSPA;
  unsigned int globalReadOffsetAL_32_0 = globalReadOffsetAL + 0 + 32*LSPA;
  unsigned int globalReadOffsetAL_33_0 = globalReadOffsetAL + 0 + 33*LSPA;
  unsigned int globalReadOffsetAL_34_0 = globalReadOffsetAL + 0 + 34*LSPA;
  unsigned int globalReadOffsetAL_35_0 = globalReadOffsetAL + 0 + 35*LSPA;
  unsigned int globalReadOffsetAL_36_0 = globalReadOffsetAL + 0 + 36*LSPA;
  unsigned int globalReadOffsetAL_37_0 = globalReadOffsetAL + 0 + 37*LSPA;
  unsigned int globalReadOffsetAL_38_0 = globalReadOffsetAL + 0 + 38*LSPA;
  unsigned int globalReadOffsetAL_39_0 = globalReadOffsetAL + 0 + 39*LSPA;
  unsigned int globalReadOffsetAL_40_0 = globalReadOffsetAL + 0 + 40*LSPA;
  unsigned int globalReadOffsetAL_41_0 = globalReadOffsetAL + 0 + 41*LSPA;
  unsigned int globalReadOffsetAL_42_0 = globalReadOffsetAL + 0 + 42*LSPA;
  unsigned int globalReadOffsetAL_43_0 = globalReadOffsetAL + 0 + 43*LSPA;
  unsigned int globalReadOffsetAL_44_0 = globalReadOffsetAL + 0 + 44*LSPA;
  unsigned int globalReadOffsetAL_45_0 = globalReadOffsetAL + 0 + 45*LSPA;
  unsigned int globalReadOffsetAL_46_0 = globalReadOffsetAL + 0 + 46*LSPA;
  unsigned int globalReadOffsetAL_47_0 = globalReadOffsetAL + 0 + 47*LSPA;
  unsigned int globalReadOffsetAL_48_0 = globalReadOffsetAL + 0 + 48*LSPA;
  unsigned int globalReadOffsetAL_49_0 = globalReadOffsetAL + 0 + 49*LSPA;
  unsigned int globalReadOffsetAL_50_0 = globalReadOffsetAL + 0 + 50*LSPA;
  unsigned int globalReadOffsetAL_51_0 = globalReadOffsetAL + 0 + 51*LSPA;
  unsigned int globalReadOffsetAL_52_0 = globalReadOffsetAL + 0 + 52*LSPA;
  unsigned int globalReadOffsetAL_53_0 = globalReadOffsetAL + 0 + 53*LSPA;
  unsigned int globalReadOffsetAL_54_0 = globalReadOffsetAL + 0 + 54*LSPA;
  unsigned int globalReadOffsetAL_55_0 = globalReadOffsetAL + 0 + 55*LSPA;
  unsigned int globalReadOffsetAL_56_0 = globalReadOffsetAL + 0 + 56*LSPA;
  unsigned int globalReadOffsetAL_57_0 = globalReadOffsetAL + 0 + 57*LSPA;
  unsigned int globalReadOffsetAL_58_0 = globalReadOffsetAL + 0 + 58*LSPA;
  unsigned int globalReadOffsetAL_59_0 = globalReadOffsetAL + 0 + 59*LSPA;
  unsigned int globalReadOffsetAL_60_0 = globalReadOffsetAL + 0 + 60*LSPA;
  unsigned int globalReadOffsetAL_61_0 = globalReadOffsetAL + 0 + 61*LSPA;
  unsigned int globalReadOffsetAL_62_0 = globalReadOffsetAL + 0 + 62*LSPA;
  unsigned int globalReadOffsetAL_63_0 = globalReadOffsetAL + 0 + 63*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_15_0), wgK );
  int64_t globalReadOffsetA_0_0_16_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_16_0), wgK );
  int64_t globalReadOffsetA_0_0_17_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_17_0), wgK );
  int64_t globalReadOffsetA_0_0_18_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_18_0), wgK );
  int64_t globalReadOffsetA_0_0_19_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_19_0), wgK );
  int64_t globalReadOffsetA_0_0_20_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_20_0), wgK );
  int64_t globalReadOffsetA_0_0_21_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_21_0), wgK );
  int64_t globalReadOffsetA_0_0_22_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_22_0), wgK );
  int64_t globalReadOffsetA_0_0_23_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_23_0), wgK );
  int64_t globalReadOffsetA_0_0_24_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_24_0), wgK );
  int64_t globalReadOffsetA_0_0_25_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_25_0), wgK );
  int64_t globalReadOffsetA_0_0_26_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_26_0), wgK );
  int64_t globalReadOffsetA_0_0_27_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_27_0), wgK );
  int64_t globalReadOffsetA_0_0_28_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_28_0), wgK );
  int64_t globalReadOffsetA_0_0_29_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_29_0), wgK );
  int64_t globalReadOffsetA_0_0_30_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_30_0), wgK );
  int64_t globalReadOffsetA_0_0_31_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_31_0), wgK );
  int64_t globalReadOffsetA_0_0_32_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_32_0), wgK );
  int64_t globalReadOffsetA_0_0_33_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_33_0), wgK );
  int64_t globalReadOffsetA_0_0_34_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_34_0), wgK );
  int64_t globalReadOffsetA_0_0_35_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_35_0), wgK );
  int64_t globalReadOffsetA_0_0_36_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_36_0), wgK );
  int64_t globalReadOffsetA_0_0_37_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_37_0), wgK );
  int64_t globalReadOffsetA_0_0_38_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_38_0), wgK );
  int64_t globalReadOffsetA_0_0_39_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_39_0), wgK );
  int64_t globalReadOffsetA_0_0_40_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_40_0), wgK );
  int64_t globalReadOffsetA_0_0_41_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_41_0), wgK );
  int64_t globalReadOffsetA_0_0_42_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_42_0), wgK );
  int64_t globalReadOffsetA_0_0_43_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_43_0), wgK );
  int64_t globalReadOffsetA_0_0_44_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_44_0), wgK );
  int64_t globalReadOffsetA_0_0_45_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_45_0), wgK );
  int64_t globalReadOffsetA_0_0_46_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_46_0), wgK );
  int64_t globalReadOffsetA_0_0_47_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_47_0), wgK );
  int64_t globalReadOffsetA_0_0_48_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_48_0), wgK );
  int64_t globalReadOffsetA_0_0_49_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_49_0), wgK );
  int64_t globalReadOffsetA_0_0_50_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_50_0), wgK );
  int64_t globalReadOffsetA_0_0_51_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_51_0), wgK );
  int64_t globalReadOffsetA_0_0_52_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_52_0), wgK );
  int64_t globalReadOffsetA_0_0_53_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_53_0), wgK );
  int64_t globalReadOffsetA_0_0_54_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_54_0), wgK );
  int64_t globalReadOffsetA_0_0_55_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_55_0), wgK );
  int64_t globalReadOffsetA_0_0_56_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_56_0), wgK );
  int64_t globalReadOffsetA_0_0_57_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_57_0), wgK );
  int64_t globalReadOffsetA_0_0_58_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_58_0), wgK );
  int64_t globalReadOffsetA_0_0_59_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_59_0), wgK );
  int64_t globalReadOffsetA_0_0_60_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_60_0), wgK );
  int64_t globalReadOffsetA_0_0_61_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_61_0), wgK );
  int64_t globalReadOffsetA_0_0_62_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_62_0), wgK );
  int64_t globalReadOffsetA_0_0_63_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_63_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;
  DATA_TYPE const *globalReadA_0_0_16_0 = A + globalReadOffsetA_0_0_16_0;
  DATA_TYPE const *globalReadA_0_0_17_0 = A + globalReadOffsetA_0_0_17_0;
  DATA_TYPE const *globalReadA_0_0_18_0 = A + globalReadOffsetA_0_0_18_0;
  DATA_TYPE const *globalReadA_0_0_19_0 = A + globalReadOffsetA_0_0_19_0;
  DATA_TYPE const *globalReadA_0_0_20_0 = A + globalReadOffsetA_0_0_20_0;
  DATA_TYPE const *globalReadA_0_0_21_0 = A + globalReadOffsetA_0_0_21_0;
  DATA_TYPE const *globalReadA_0_0_22_0 = A + globalReadOffsetA_0_0_22_0;
  DATA_TYPE const *globalReadA_0_0_23_0 = A + globalReadOffsetA_0_0_23_0;
  DATA_TYPE const *globalReadA_0_0_24_0 = A + globalReadOffsetA_0_0_24_0;
  DATA_TYPE const *globalReadA_0_0_25_0 = A + globalReadOffsetA_0_0_25_0;
  DATA_TYPE const *globalReadA_0_0_26_0 = A + globalReadOffsetA_0_0_26_0;
  DATA_TYPE const *globalReadA_0_0_27_0 = A + globalReadOffsetA_0_0_27_0;
  DATA_TYPE const *globalReadA_0_0_28_0 = A + globalReadOffsetA_0_0_28_0;
  DATA_TYPE const *globalReadA_0_0_29_0 = A + globalReadOffsetA_0_0_29_0;
  DATA_TYPE const *globalReadA_0_0_30_0 = A + globalReadOffsetA_0_0_30_0;
  DATA_TYPE const *globalReadA_0_0_31_0 = A + globalReadOffsetA_0_0_31_0;
  DATA_TYPE const *globalReadA_0_0_32_0 = A + globalReadOffsetA_0_0_32_0;
  DATA_TYPE const *globalReadA_0_0_33_0 = A + globalReadOffsetA_0_0_33_0;
  DATA_TYPE const *globalReadA_0_0_34_0 = A + globalReadOffsetA_0_0_34_0;
  DATA_TYPE const *globalReadA_0_0_35_0 = A + globalReadOffsetA_0_0_35_0;
  DATA_TYPE const *globalReadA_0_0_36_0 = A + globalReadOffsetA_0_0_36_0;
  DATA_TYPE const *globalReadA_0_0_37_0 = A + globalReadOffsetA_0_0_37_0;
  DATA_TYPE const *globalReadA_0_0_38_0 = A + globalReadOffsetA_0_0_38_0;
  DATA_TYPE const *globalReadA_0_0_39_0 = A + globalReadOffsetA_0_0_39_0;
  DATA_TYPE const *globalReadA_0_0_40_0 = A + globalReadOffsetA_0_0_40_0;
  DATA_TYPE const *globalReadA_0_0_41_0 = A + globalReadOffsetA_0_0_41_0;
  DATA_TYPE const *globalReadA_0_0_42_0 = A + globalReadOffsetA_0_0_42_0;
  DATA_TYPE const *globalReadA_0_0_43_0 = A + globalReadOffsetA_0_0_43_0;
  DATA_TYPE const *globalReadA_0_0_44_0 = A + globalReadOffsetA_0_0_44_0;
  DATA_TYPE const *globalReadA_0_0_45_0 = A + globalReadOffsetA_0_0_45_0;
  DATA_TYPE const *globalReadA_0_0_46_0 = A + globalReadOffsetA_0_0_46_0;
  DATA_TYPE const *globalReadA_0_0_47_0 = A + globalReadOffsetA_0_0_47_0;
  DATA_TYPE const *globalReadA_0_0_48_0 = A + globalReadOffsetA_0_0_48_0;
  DATA_TYPE const *globalReadA_0_0_49_0 = A + globalReadOffsetA_0_0_49_0;
  DATA_TYPE const *globalReadA_0_0_50_0 = A + globalReadOffsetA_0_0_50_0;
  DATA_TYPE const *globalReadA_0_0_51_0 = A + globalReadOffsetA_0_0_51_0;
  DATA_TYPE const *globalReadA_0_0_52_0 = A + globalReadOffsetA_0_0_52_0;
  DATA_TYPE const *globalReadA_0_0_53_0 = A + globalReadOffsetA_0_0_53_0;
  DATA_TYPE const *globalReadA_0_0_54_0 = A + globalReadOffsetA_0_0_54_0;
  DATA_TYPE const *globalReadA_0_0_55_0 = A + globalReadOffsetA_0_0_55_0;
  DATA_TYPE const *globalReadA_0_0_56_0 = A + globalReadOffsetA_0_0_56_0;
  DATA_TYPE const *globalReadA_0_0_57_0 = A + globalReadOffsetA_0_0_57_0;
  DATA_TYPE const *globalReadA_0_0_58_0 = A + globalReadOffsetA_0_0_58_0;
  DATA_TYPE const *globalReadA_0_0_59_0 = A + globalReadOffsetA_0_0_59_0;
  DATA_TYPE const *globalReadA_0_0_60_0 = A + globalReadOffsetA_0_0_60_0;
  DATA_TYPE const *globalReadA_0_0_61_0 = A + globalReadOffsetA_0_0_61_0;
  DATA_TYPE const *globalReadA_0_0_62_0 = A + globalReadOffsetA_0_0_62_0;
  DATA_TYPE const *globalReadA_0_0_63_0 = A + globalReadOffsetA_0_0_63_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 8*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 9*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 10*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 11*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 12*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 13*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 14*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 15*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_16_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 16*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_17_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 17*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_18_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 18*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_19_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 19*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_20_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 20*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_21_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 21*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_22_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 22*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_23_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 23*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_24_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 24*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_25_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 25*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_26_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 26*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_27_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 27*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_28_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 28*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_29_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 29*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_30_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 30*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_31_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 31*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_32_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 32*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_33_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 33*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_34_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 34*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_35_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 35*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_36_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 36*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_37_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 37*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_38_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 38*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_39_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 39*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_40_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 40*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_41_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 41*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_42_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 42*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_43_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 43*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_44_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 44*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_45_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 45*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_46_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 46*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_47_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 47*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_48_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 48*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_49_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 49*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_50_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 50*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_51_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 51*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_52_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 52*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_53_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 53*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_54_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 54*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_55_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 55*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_56_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 56*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_57_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 57*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_58_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 58*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_59_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 59*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_60_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 60*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_61_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 61*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_62_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 62*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_63_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 63*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_0_0_15_0;
  DATA_TYPE *localWriteA_0_0_16_0;
  DATA_TYPE *localWriteA_0_0_17_0;
  DATA_TYPE *localWriteA_0_0_18_0;
  DATA_TYPE *localWriteA_0_0_19_0;
  DATA_TYPE *localWriteA_0_0_20_0;
  DATA_TYPE *localWriteA_0_0_21_0;
  DATA_TYPE *localWriteA_0_0_22_0;
  DATA_TYPE *localWriteA_0_0_23_0;
  DATA_TYPE *localWriteA_0_0_24_0;
  DATA_TYPE *localWriteA_0_0_25_0;
  DATA_TYPE *localWriteA_0_0_26_0;
  DATA_TYPE *localWriteA_0_0_27_0;
  DATA_TYPE *localWriteA_0_0_28_0;
  DATA_TYPE *localWriteA_0_0_29_0;
  DATA_TYPE *localWriteA_0_0_30_0;
  DATA_TYPE *localWriteA_0_0_31_0;
  DATA_TYPE *localWriteA_0_0_32_0;
  DATA_TYPE *localWriteA_0_0_33_0;
  DATA_TYPE *localWriteA_0_0_34_0;
  DATA_TYPE *localWriteA_0_0_35_0;
  DATA_TYPE *localWriteA_0_0_36_0;
  DATA_TYPE *localWriteA_0_0_37_0;
  DATA_TYPE *localWriteA_0_0_38_0;
  DATA_TYPE *localWriteA_0_0_39_0;
  DATA_TYPE *localWriteA_0_0_40_0;
  DATA_TYPE *localWriteA_0_0_41_0;
  DATA_TYPE *localWriteA_0_0_42_0;
  DATA_TYPE *localWriteA_0_0_43_0;
  DATA_TYPE *localWriteA_0_0_44_0;
  DATA_TYPE *localWriteA_0_0_45_0;
  DATA_TYPE *localWriteA_0_0_46_0;
  DATA_TYPE *localWriteA_0_0_47_0;
  DATA_TYPE *localWriteA_0_0_48_0;
  DATA_TYPE *localWriteA_0_0_49_0;
  DATA_TYPE *localWriteA_0_0_50_0;
  DATA_TYPE *localWriteA_0_0_51_0;
  DATA_TYPE *localWriteA_0_0_52_0;
  DATA_TYPE *localWriteA_0_0_53_0;
  DATA_TYPE *localWriteA_0_0_54_0;
  DATA_TYPE *localWriteA_0_0_55_0;
  DATA_TYPE *localWriteA_0_0_56_0;
  DATA_TYPE *localWriteA_0_0_57_0;
  DATA_TYPE *localWriteA_0_0_58_0;
  DATA_TYPE *localWriteA_0_0_59_0;
  DATA_TYPE *localWriteA_0_0_60_0;
  DATA_TYPE *localWriteA_0_0_61_0;
  DATA_TYPE *localWriteA_0_0_62_0;
  DATA_TYPE *localWriteA_0_0_63_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);
  localWriteA_0_0_16_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_16_0);
  localWriteA_0_0_17_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_17_0);
  localWriteA_0_0_18_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_18_0);
  localWriteA_0_0_19_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_19_0);
  localWriteA_0_0_20_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_20_0);
  localWriteA_0_0_21_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_21_0);
  localWriteA_0_0_22_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_22_0);
  localWriteA_0_0_23_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_23_0);
  localWriteA_0_0_24_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_24_0);
  localWriteA_0_0_25_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_25_0);
  localWriteA_0_0_26_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_26_0);
  localWriteA_0_0_27_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_27_0);
  localWriteA_0_0_28_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_28_0);
  localWriteA_0_0_29_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_29_0);
  localWriteA_0_0_30_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_30_0);
  localWriteA_0_0_31_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_31_0);
  localWriteA_0_0_32_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_32_0);
  localWriteA_0_0_33_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_33_0);
  localWriteA_0_0_34_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_34_0);
  localWriteA_0_0_35_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_35_0);
  localWriteA_0_0_36_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_36_0);
  localWriteA_0_0_37_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_37_0);
  localWriteA_0_0_38_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_38_0);
  localWriteA_0_0_39_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_39_0);
  localWriteA_0_0_40_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_40_0);
  localWriteA_0_0_41_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_41_0);
  localWriteA_0_0_42_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_42_0);
  localWriteA_0_0_43_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_43_0);
  localWriteA_0_0_44_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_44_0);
  localWriteA_0_0_45_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_45_0);
  localWriteA_0_0_46_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_46_0);
  localWriteA_0_0_47_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_47_0);
  localWriteA_0_0_48_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_48_0);
  localWriteA_0_0_49_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_49_0);
  localWriteA_0_0_50_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_50_0);
  localWriteA_0_0_51_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_51_0);
  localWriteA_0_0_52_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_52_0);
  localWriteA_0_0_53_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_53_0);
  localWriteA_0_0_54_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_54_0);
  localWriteA_0_0_55_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_55_0);
  localWriteA_0_0_56_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_56_0);
  localWriteA_0_0_57_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_57_0);
  localWriteA_0_0_58_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_58_0);
  localWriteA_0_0_59_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_59_0);
  localWriteA_0_0_60_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_60_0);
  localWriteA_0_0_61_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_61_0);
  localWriteA_0_0_62_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_62_0);
  localWriteA_0_0_63_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_63_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);
    a_0_0_16_0 = *(globalReadA_0_0_16_0 + 0);
    a_0_0_17_0 = *(globalReadA_0_0_17_0 + 0);
    a_0_0_18_0 = *(globalReadA_0_0_18_0 + 0);
    a_0_0_19_0 = *(globalReadA_0_0_19_0 + 0);
    a_0_0_20_0 = *(globalReadA_0_0_20_0 + 0);
    a_0_0_21_0 = *(globalReadA_0_0_21_0 + 0);
    a_0_0_22_0 = *(globalReadA_0_0_22_0 + 0);
    a_0_0_23_0 = *(globalReadA_0_0_23_0 + 0);
    a_0_0_24_0 = *(globalReadA_0_0_24_0 + 0);
    a_0_0_25_0 = *(globalReadA_0_0_25_0 + 0);
    a_0_0_26_0 = *(globalReadA_0_0_26_0 + 0);
    a_0_0_27_0 = *(globalReadA_0_0_27_0 + 0);
    a_0_0_28_0 = *(globalReadA_0_0_28_0 + 0);
    a_0_0_29_0 = *(globalReadA_0_0_29_0 + 0);
    a_0_0_30_0 = *(globalReadA_0_0_30_0 + 0);
    a_0_0_31_0 = *(globalReadA_0_0_31_0 + 0);
    a_0_0_32_0 = *(globalReadA_0_0_32_0 + 0);
    a_0_0_33_0 = *(globalReadA_0_0_33_0 + 0);
    a_0_0_34_0 = *(globalReadA_0_0_34_0 + 0);
    a_0_0_35_0 = *(globalReadA_0_0_35_0 + 0);
    a_0_0_36_0 = *(globalReadA_0_0_36_0 + 0);
    a_0_0_37_0 = *(globalReadA_0_0_37_0 + 0);
    a_0_0_38_0 = *(globalReadA_0_0_38_0 + 0);
    a_0_0_39_0 = *(globalReadA_0_0_39_0 + 0);
    a_0_0_40_0 = *(globalReadA_0_0_40_0 + 0);
    a_0_0_41_0 = *(globalReadA_0_0_41_0 + 0);
    a_0_0_42_0 = *(globalReadA_0_0_42_0 + 0);
    a_0_0_43_0 = *(globalReadA_0_0_43_0 + 0);
    a_0_0_44_0 = *(globalReadA_0_0_44_0 + 0);
    a_0_0_45_0 = *(globalReadA_0_0_45_0 + 0);
    a_0_0_46_0 = *(globalReadA_0_0_46_0 + 0);
    a_0_0_47_0 = *(globalReadA_0_0_47_0 + 0);
    a_0_0_48_0 = *(globalReadA_0_0_48_0 + 0);
    a_0_0_49_0 = *(globalReadA_0_0_49_0 + 0);
    a_0_0_50_0 = *(globalReadA_0_0_50_0 + 0);
    a_0_0_51_0 = *(globalReadA_0_0_51_0 + 0);
    a_0_0_52_0 = *(globalReadA_0_0_52_0 + 0);
    a_0_0_53_0 = *(globalReadA_0_0_53_0 + 0);
    a_0_0_54_0 = *(globalReadA_0_0_54_0 + 0);
    a_0_0_55_0 = *(globalReadA_0_0_55_0 + 0);
    a_0_0_56_0 = *(globalReadA_0_0_56_0 + 0);
    a_0_0_57_0 = *(globalReadA_0_0_57_0 + 0);
    a_0_0_58_0 = *(globalReadA_0_0_58_0 + 0);
    a_0_0_59_0 = *(globalReadA_0_0_59_0 + 0);
    a_0_0_60_0 = *(globalReadA_0_0_60_0 + 0);
    a_0_0_61_0 = *(globalReadA_0_0_61_0 + 0);
    a_0_0_62_0 = *(globalReadA_0_0_62_0 + 0);
    a_0_0_63_0 = *(globalReadA_0_0_63_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    globalReadA_0_0_16_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_16_0) + 1*globalReadIncAL);
    globalReadA_0_0_17_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_17_0) + 1*globalReadIncAL);
    globalReadA_0_0_18_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_18_0) + 1*globalReadIncAL);
    globalReadA_0_0_19_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_19_0) + 1*globalReadIncAL);
    globalReadA_0_0_20_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_20_0) + 1*globalReadIncAL);
    globalReadA_0_0_21_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_21_0) + 1*globalReadIncAL);
    globalReadA_0_0_22_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_22_0) + 1*globalReadIncAL);
    globalReadA_0_0_23_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_23_0) + 1*globalReadIncAL);
    globalReadA_0_0_24_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_24_0) + 1*globalReadIncAL);
    globalReadA_0_0_25_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_25_0) + 1*globalReadIncAL);
    globalReadA_0_0_26_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_26_0) + 1*globalReadIncAL);
    globalReadA_0_0_27_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_27_0) + 1*globalReadIncAL);
    globalReadA_0_0_28_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_28_0) + 1*globalReadIncAL);
    globalReadA_0_0_29_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_29_0) + 1*globalReadIncAL);
    globalReadA_0_0_30_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_30_0) + 1*globalReadIncAL);
    globalReadA_0_0_31_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_31_0) + 1*globalReadIncAL);
    globalReadA_0_0_32_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_32_0) + 1*globalReadIncAL);
    globalReadA_0_0_33_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_33_0) + 1*globalReadIncAL);
    globalReadA_0_0_34_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_34_0) + 1*globalReadIncAL);
    globalReadA_0_0_35_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_35_0) + 1*globalReadIncAL);
    globalReadA_0_0_36_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_36_0) + 1*globalReadIncAL);
    globalReadA_0_0_37_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_37_0) + 1*globalReadIncAL);
    globalReadA_0_0_38_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_38_0) + 1*globalReadIncAL);
    globalReadA_0_0_39_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_39_0) + 1*globalReadIncAL);
    globalReadA_0_0_40_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_40_0) + 1*globalReadIncAL);
    globalReadA_0_0_41_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_41_0) + 1*globalReadIncAL);
    globalReadA_0_0_42_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_42_0) + 1*globalReadIncAL);
    globalReadA_0_0_43_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_43_0) + 1*globalReadIncAL);
    globalReadA_0_0_44_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_44_0) + 1*globalReadIncAL);
    globalReadA_0_0_45_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_45_0) + 1*globalReadIncAL);
    globalReadA_0_0_46_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_46_0) + 1*globalReadIncAL);
    globalReadA_0_0_47_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_47_0) + 1*globalReadIncAL);
    globalReadA_0_0_48_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_48_0) + 1*globalReadIncAL);
    globalReadA_0_0_49_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_49_0) + 1*globalReadIncAL);
    globalReadA_0_0_50_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_50_0) + 1*globalReadIncAL);
    globalReadA_0_0_51_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_51_0) + 1*globalReadIncAL);
    globalReadA_0_0_52_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_52_0) + 1*globalReadIncAL);
    globalReadA_0_0_53_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_53_0) + 1*globalReadIncAL);
    globalReadA_0_0_54_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_54_0) + 1*globalReadIncAL);
    globalReadA_0_0_55_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_55_0) + 1*globalReadIncAL);
    globalReadA_0_0_56_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_56_0) + 1*globalReadIncAL);
    globalReadA_0_0_57_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_57_0) + 1*globalReadIncAL);
    globalReadA_0_0_58_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_58_0) + 1*globalReadIncAL);
    globalReadA_0_0_59_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_59_0) + 1*globalReadIncAL);
    globalReadA_0_0_60_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_60_0) + 1*globalReadIncAL);
    globalReadA_0_0_61_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_61_0) + 1*globalReadIncAL);
    globalReadA_0_0_62_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_62_0) + 1*globalReadIncAL);
    globalReadA_0_0_63_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_63_0) + 1*globalReadIncAL);

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
    *(localWriteA_0_0_16_0 + 0) = a_0_0_16_0;
    *(localWriteA_0_0_17_0 + 0) = a_0_0_17_0;
    *(localWriteA_0_0_18_0 + 0) = a_0_0_18_0;
    *(localWriteA_0_0_19_0 + 0) = a_0_0_19_0;
    *(localWriteA_0_0_20_0 + 0) = a_0_0_20_0;
    *(localWriteA_0_0_21_0 + 0) = a_0_0_21_0;
    *(localWriteA_0_0_22_0 + 0) = a_0_0_22_0;
    *(localWriteA_0_0_23_0 + 0) = a_0_0_23_0;
    *(localWriteA_0_0_24_0 + 0) = a_0_0_24_0;
    *(localWriteA_0_0_25_0 + 0) = a_0_0_25_0;
    *(localWriteA_0_0_26_0 + 0) = a_0_0_26_0;
    *(localWriteA_0_0_27_0 + 0) = a_0_0_27_0;
    *(localWriteA_0_0_28_0 + 0) = a_0_0_28_0;
    *(localWriteA_0_0_29_0 + 0) = a_0_0_29_0;
    *(localWriteA_0_0_30_0 + 0) = a_0_0_30_0;
    *(localWriteA_0_0_31_0 + 0) = a_0_0_31_0;
    *(localWriteA_0_0_32_0 + 0) = a_0_0_32_0;
    *(localWriteA_0_0_33_0 + 0) = a_0_0_33_0;
    *(localWriteA_0_0_34_0 + 0) = a_0_0_34_0;
    *(localWriteA_0_0_35_0 + 0) = a_0_0_35_0;
    *(localWriteA_0_0_36_0 + 0) = a_0_0_36_0;
    *(localWriteA_0_0_37_0 + 0) = a_0_0_37_0;
    *(localWriteA_0_0_38_0 + 0) = a_0_0_38_0;
    *(localWriteA_0_0_39_0 + 0) = a_0_0_39_0;
    *(localWriteA_0_0_40_0 + 0) = a_0_0_40_0;
    *(localWriteA_0_0_41_0 + 0) = a_0_0_41_0;
    *(localWriteA_0_0_42_0 + 0) = a_0_0_42_0;
    *(localWriteA_0_0_43_0 + 0) = a_0_0_43_0;
    *(localWriteA_0_0_44_0 + 0) = a_0_0_44_0;
    *(localWriteA_0_0_45_0 + 0) = a_0_0_45_0;
    *(localWriteA_0_0_46_0 + 0) = a_0_0_46_0;
    *(localWriteA_0_0_47_0 + 0) = a_0_0_47_0;
    *(localWriteA_0_0_48_0 + 0) = a_0_0_48_0;
    *(localWriteA_0_0_49_0 + 0) = a_0_0_49_0;
    *(localWriteA_0_0_50_0 + 0) = a_0_0_50_0;
    *(localWriteA_0_0_51_0 + 0) = a_0_0_51_0;
    *(localWriteA_0_0_52_0 + 0) = a_0_0_52_0;
    *(localWriteA_0_0_53_0 + 0) = a_0_0_53_0;
    *(localWriteA_0_0_54_0 + 0) = a_0_0_54_0;
    *(localWriteA_0_0_55_0 + 0) = a_0_0_55_0;
    *(localWriteA_0_0_56_0 + 0) = a_0_0_56_0;
    *(localWriteA_0_0_57_0 + 0) = a_0_0_57_0;
    *(localWriteA_0_0_58_0 + 0) = a_0_0_58_0;
    *(localWriteA_0_0_59_0 + 0) = a_0_0_59_0;
    *(localWriteA_0_0_60_0 + 0) = a_0_0_60_0;
    *(localWriteA_0_0_61_0 + 0) = a_0_0_61_0;
    *(localWriteA_0_0_62_0 + 0) = a_0_0_62_0;
    *(localWriteA_0_0_63_0 + 0) = a_0_0_63_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 15 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 16 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 17 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 18 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 19 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 20 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 21 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 22 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 23 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 24 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 25 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 26 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 27 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 28 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 29 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 30 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 31 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 32 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 33 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 34 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 35 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 36 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 37 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 38 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 39 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 40 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 41 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 42 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 43 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 44 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 45 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 46 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 47 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 48 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 49 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 50 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 51 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 52 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 53 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 54 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 55 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 56 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 57 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 58 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 59 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 60 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 61 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 62 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_1x1


    /* iter 63 */

    MAC_1x1_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_0_8_0 = ( globalReadOffsetAL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_0_0_9_0 = ( globalReadOffsetAL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_0_0_10_0 = ( globalReadOffsetAL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_0_0_11_0 = ( globalReadOffsetAL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_0_0_12_0 = ( globalReadOffsetAL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_0_0_13_0 = ( globalReadOffsetAL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_0_0_14_0 = ( globalReadOffsetAL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_0_0_15_0 = ( globalReadOffsetAL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);
  a_0_0_16_0 = ( globalReadOffsetAL_16_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_16_0 + 0);
  a_0_0_17_0 = ( globalReadOffsetAL_17_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_17_0 + 0);
  a_0_0_18_0 = ( globalReadOffsetAL_18_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_18_0 + 0);
  a_0_0_19_0 = ( globalReadOffsetAL_19_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_19_0 + 0);
  a_0_0_20_0 = ( globalReadOffsetAL_20_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_20_0 + 0);
  a_0_0_21_0 = ( globalReadOffsetAL_21_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_21_0 + 0);
  a_0_0_22_0 = ( globalReadOffsetAL_22_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_22_0 + 0);
  a_0_0_23_0 = ( globalReadOffsetAL_23_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_23_0 + 0);
  a_0_0_24_0 = ( globalReadOffsetAL_24_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_24_0 + 0);
  a_0_0_25_0 = ( globalReadOffsetAL_25_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_25_0 + 0);
  a_0_0_26_0 = ( globalReadOffsetAL_26_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_26_0 + 0);
  a_0_0_27_0 = ( globalReadOffsetAL_27_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_27_0 + 0);
  a_0_0_28_0 = ( globalReadOffsetAL_28_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_28_0 + 0);
  a_0_0_29_0 = ( globalReadOffsetAL_29_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_29_0 + 0);
  a_0_0_30_0 = ( globalReadOffsetAL_30_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_30_0 + 0);
  a_0_0_31_0 = ( globalReadOffsetAL_31_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_31_0 + 0);
  a_0_0_32_0 = ( globalReadOffsetAL_32_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_32_0 + 0);
  a_0_0_33_0 = ( globalReadOffsetAL_33_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_33_0 + 0);
  a_0_0_34_0 = ( globalReadOffsetAL_34_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_34_0 + 0);
  a_0_0_35_0 = ( globalReadOffsetAL_35_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_35_0 + 0);
  a_0_0_36_0 = ( globalReadOffsetAL_36_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_36_0 + 0);
  a_0_0_37_0 = ( globalReadOffsetAL_37_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_37_0 + 0);
  a_0_0_38_0 = ( globalReadOffsetAL_38_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_38_0 + 0);
  a_0_0_39_0 = ( globalReadOffsetAL_39_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_39_0 + 0);
  a_0_0_40_0 = ( globalReadOffsetAL_40_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_40_0 + 0);
  a_0_0_41_0 = ( globalReadOffsetAL_41_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_41_0 + 0);
  a_0_0_42_0 = ( globalReadOffsetAL_42_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_42_0 + 0);
  a_0_0_43_0 = ( globalReadOffsetAL_43_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_43_0 + 0);
  a_0_0_44_0 = ( globalReadOffsetAL_44_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_44_0 + 0);
  a_0_0_45_0 = ( globalReadOffsetAL_45_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_45_0 + 0);
  a_0_0_46_0 = ( globalReadOffsetAL_46_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_46_0 + 0);
  a_0_0_47_0 = ( globalReadOffsetAL_47_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_47_0 + 0);
  a_0_0_48_0 = ( globalReadOffsetAL_48_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_48_0 + 0);
  a_0_0_49_0 = ( globalReadOffsetAL_49_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_49_0 + 0);
  a_0_0_50_0 = ( globalReadOffsetAL_50_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_50_0 + 0);
  a_0_0_51_0 = ( globalReadOffsetAL_51_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_51_0 + 0);
  a_0_0_52_0 = ( globalReadOffsetAL_52_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_52_0 + 0);
  a_0_0_53_0 = ( globalReadOffsetAL_53_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_53_0 + 0);
  a_0_0_54_0 = ( globalReadOffsetAL_54_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_54_0 + 0);
  a_0_0_55_0 = ( globalReadOffsetAL_55_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_55_0 + 0);
  a_0_0_56_0 = ( globalReadOffsetAL_56_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_56_0 + 0);
  a_0_0_57_0 = ( globalReadOffsetAL_57_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_57_0 + 0);
  a_0_0_58_0 = ( globalReadOffsetAL_58_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_58_0 + 0);
  a_0_0_59_0 = ( globalReadOffsetAL_59_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_59_0 + 0);
  a_0_0_60_0 = ( globalReadOffsetAL_60_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_60_0 + 0);
  a_0_0_61_0 = ( globalReadOffsetAL_61_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_61_0 + 0);
  a_0_0_62_0 = ( globalReadOffsetAL_62_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_62_0 + 0);
  a_0_0_63_0 = ( globalReadOffsetAL_63_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_63_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);
  localWriteA_0_0_16_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_16_0);
  localWriteA_0_0_17_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_17_0);
  localWriteA_0_0_18_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_18_0);
  localWriteA_0_0_19_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_19_0);
  localWriteA_0_0_20_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_20_0);
  localWriteA_0_0_21_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_21_0);
  localWriteA_0_0_22_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_22_0);
  localWriteA_0_0_23_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_23_0);
  localWriteA_0_0_24_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_24_0);
  localWriteA_0_0_25_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_25_0);
  localWriteA_0_0_26_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_26_0);
  localWriteA_0_0_27_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_27_0);
  localWriteA_0_0_28_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_28_0);
  localWriteA_0_0_29_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_29_0);
  localWriteA_0_0_30_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_30_0);
  localWriteA_0_0_31_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_31_0);
  localWriteA_0_0_32_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_32_0);
  localWriteA_0_0_33_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_33_0);
  localWriteA_0_0_34_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_34_0);
  localWriteA_0_0_35_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_35_0);
  localWriteA_0_0_36_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_36_0);
  localWriteA_0_0_37_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_37_0);
  localWriteA_0_0_38_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_38_0);
  localWriteA_0_0_39_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_39_0);
  localWriteA_0_0_40_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_40_0);
  localWriteA_0_0_41_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_41_0);
  localWriteA_0_0_42_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_42_0);
  localWriteA_0_0_43_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_43_0);
  localWriteA_0_0_44_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_44_0);
  localWriteA_0_0_45_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_45_0);
  localWriteA_0_0_46_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_46_0);
  localWriteA_0_0_47_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_47_0);
  localWriteA_0_0_48_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_48_0);
  localWriteA_0_0_49_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_49_0);
  localWriteA_0_0_50_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_50_0);
  localWriteA_0_0_51_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_51_0);
  localWriteA_0_0_52_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_52_0);
  localWriteA_0_0_53_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_53_0);
  localWriteA_0_0_54_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_54_0);
  localWriteA_0_0_55_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_55_0);
  localWriteA_0_0_56_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_56_0);
  localWriteA_0_0_57_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_57_0);
  localWriteA_0_0_58_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_58_0);
  localWriteA_0_0_59_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_59_0);
  localWriteA_0_0_60_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_60_0);
  localWriteA_0_0_61_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_61_0);
  localWriteA_0_0_62_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_62_0);
  localWriteA_0_0_63_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_63_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
  *(localWriteA_0_0_16_0 + 0) = a_0_0_16_0;
  *(localWriteA_0_0_17_0 + 0) = a_0_0_17_0;
  *(localWriteA_0_0_18_0 + 0) = a_0_0_18_0;
  *(localWriteA_0_0_19_0 + 0) = a_0_0_19_0;
  *(localWriteA_0_0_20_0 + 0) = a_0_0_20_0;
  *(localWriteA_0_0_21_0 + 0) = a_0_0_21_0;
  *(localWriteA_0_0_22_0 + 0) = a_0_0_22_0;
  *(localWriteA_0_0_23_0 + 0) = a_0_0_23_0;
  *(localWriteA_0_0_24_0 + 0) = a_0_0_24_0;
  *(localWriteA_0_0_25_0 + 0) = a_0_0_25_0;
  *(localWriteA_0_0_26_0 + 0) = a_0_0_26_0;
  *(localWriteA_0_0_27_0 + 0) = a_0_0_27_0;
  *(localWriteA_0_0_28_0 + 0) = a_0_0_28_0;
  *(localWriteA_0_0_29_0 + 0) = a_0_0_29_0;
  *(localWriteA_0_0_30_0 + 0) = a_0_0_30_0;
  *(localWriteA_0_0_31_0 + 0) = a_0_0_31_0;
  *(localWriteA_0_0_32_0 + 0) = a_0_0_32_0;
  *(localWriteA_0_0_33_0 + 0) = a_0_0_33_0;
  *(localWriteA_0_0_34_0 + 0) = a_0_0_34_0;
  *(localWriteA_0_0_35_0 + 0) = a_0_0_35_0;
  *(localWriteA_0_0_36_0 + 0) = a_0_0_36_0;
  *(localWriteA_0_0_37_0 + 0) = a_0_0_37_0;
  *(localWriteA_0_0_38_0 + 0) = a_0_0_38_0;
  *(localWriteA_0_0_39_0 + 0) = a_0_0_39_0;
  *(localWriteA_0_0_40_0 + 0) = a_0_0_40_0;
  *(localWriteA_0_0_41_0 + 0) = a_0_0_41_0;
  *(localWriteA_0_0_42_0 + 0) = a_0_0_42_0;
  *(localWriteA_0_0_43_0 + 0) = a_0_0_43_0;
  *(localWriteA_0_0_44_0 + 0) = a_0_0_44_0;
  *(localWriteA_0_0_45_0 + 0) = a_0_0_45_0;
  *(localWriteA_0_0_46_0 + 0) = a_0_0_46_0;
  *(localWriteA_0_0_47_0 + 0) = a_0_0_47_0;
  *(localWriteA_0_0_48_0 + 0) = a_0_0_48_0;
  *(localWriteA_0_0_49_0 + 0) = a_0_0_49_0;
  *(localWriteA_0_0_50_0 + 0) = a_0_0_50_0;
  *(localWriteA_0_0_51_0 + 0) = a_0_0_51_0;
  *(localWriteA_0_0_52_0 + 0) = a_0_0_52_0;
  *(localWriteA_0_0_53_0 + 0) = a_0_0_53_0;
  *(localWriteA_0_0_54_0 + 0) = a_0_0_54_0;
  *(localWriteA_0_0_55_0 + 0) = a_0_0_55_0;
  *(localWriteA_0_0_56_0 + 0) = a_0_0_56_0;
  *(localWriteA_0_0_57_0 + 0) = a_0_0_57_0;
  *(localWriteA_0_0_58_0 + 0) = a_0_0_58_0;
  *(localWriteA_0_0_59_0 + 0) = a_0_0_59_0;
  *(localWriteA_0_0_60_0 + 0) = a_0_0_60_0;
  *(localWriteA_0_0_61_0 + 0) = a_0_0_61_0;
  *(localWriteA_0_0_62_0 + 0) = a_0_0_62_0;
  *(localWriteA_0_0_63_0 + 0) = a_0_0_63_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_1x1

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_1x1
#undef MAC_1x1_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 64
#define SG1J 1
#define TT0I 1
#define TT1J 1
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 64
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 64
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 4096
#define LDS_NUM_ELEMENTS 4160

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 1x1 micro-tile */
#define MAC_1x1\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \

#define MAC_1x1_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT64x1x64_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU0_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT1_1_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG64_1_1_WGM1(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE a_0_0_16_0;
  DATA_TYPE a_0_0_17_0;
  DATA_TYPE a_0_0_18_0;
  DATA_TYPE a_0_0_19_0;
  DATA_TYPE a_0_0_20_0;
  DATA_TYPE a_0_0_21_0;
  DATA_TYPE a_0_0_22_0;
  DATA_TYPE a_0_0_23_0;
  DATA_TYPE a_0_0_24_0;
  DATA_TYPE a_0_0_25_0;
  DATA_TYPE a_0_0_26_0;
  DATA_TYPE a_0_0_27_0;
  DATA_TYPE a_0_0_28_0;
  DATA_TYPE a_0_0_29_0;
  DATA_TYPE a_0_0_30_0;
  DATA_TYPE a_0_0_31_0;
  DATA_TYPE a_0_0_32_0;
  DATA_TYPE a_0_0_33_0;
  DATA_TYPE a_0_0_34_0;
  DATA_TYPE a_0_0_35_0;
  DATA_TYPE a_0_0_36_0;
  DATA_TYPE a_0_0_37_0;
  DATA_TYPE a_0_0_38_0;
  DATA_TYPE a_0_0_39_0;
  DATA_TYPE a_0_0_40_0;
  DATA_TYPE a_0_0_41_0;
  DATA_TYPE a_0_0_42_0;
  DATA_TYPE a_0_0_43_0;
  DATA_TYPE a_0_0_44_0;
  DATA_TYPE a_0_0_45_0;
  DATA_TYPE a_0_0_46_0;
  DATA_TYPE a_0_0_47_0;
  DATA_TYPE a_0_0_48_0;
  DATA_TYPE a_0_0_49_0;
  DATA_TYPE a_0_0_50_0;
  DATA_TYPE a_0_0_51_0;
  DATA_TYPE a_0_0_52_0;
  DATA_TYPE a_0_0_53_0;
  DATA_TYPE a_0_0_54_0;
  DATA_TYPE a_0_0_55_0;
  DATA_TYPE a_0_0_56_0;
  DATA_TYPE a_0_0_57_0;
  DATA_TYPE a_0_0_58_0;
  DATA_TYPE a_0_0_59_0;
  DATA_TYPE a_0_0_60_0;
  DATA_TYPE a_0_0_61_0;
  DATA_TYPE a_0_0_62_0;
  DATA_TYPE a_0_0_63_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;
  unsigned int globalReadOffsetAL_8_0 = globalReadOffsetAL + 0 + 8*LSPA;
  unsigned int globalReadOffsetAL_9_0 = globalReadOffsetAL + 0 + 9*LSPA;
  unsigned int globalReadOffsetAL_10_0 = globalReadOffsetAL + 0 + 10*LSPA;
  unsigned int globalReadOffsetAL_11_0 = globalReadOffsetAL + 0 + 11*LSPA;
  unsigned int globalReadOffsetAL_12_0 = globalReadOffsetAL + 0 + 12*LSPA;
  unsigned int globalReadOffsetAL_13_0 = globalReadOffsetAL + 0 + 13*LSPA;
  unsigned int globalReadOffsetAL_14_0 = globalReadOffsetAL + 0 + 14*LSPA;
  unsigned int globalReadOffsetAL_15_0 = globalReadOffsetAL + 0 + 15*LSPA;
  unsigned int globalReadOffsetAL_16_0 = globalReadOffsetAL + 0 + 16*LSPA;
  unsigned int globalReadOffsetAL_17_0 = globalReadOffsetAL + 0 + 17*LSPA;
  unsigned int globalReadOffsetAL_18_0 = globalReadOffsetAL + 0 + 18*LSPA;
  unsigned int globalReadOffsetAL_19_0 = globalReadOffsetAL + 0 + 19*LSPA;
  unsigned int globalReadOffsetAL_20_0 = globalReadOffsetAL + 0 + 20*LSPA;
  unsigned int globalReadOffsetAL_21_0 = globalReadOffsetAL + 0 + 21*LSPA;
  unsigned int globalReadOffsetAL_22_0 = globalReadOffsetAL + 0 + 22*LSPA;
  unsigned int globalReadOffsetAL_23_0 = globalReadOffsetAL + 0 + 23*LSPA;
  unsigned int globalReadOffsetAL_24_0 = globalReadOffsetAL + 0 + 24*LSPA;
  unsigned int globalReadOffsetAL_25_0 = globalReadOffsetAL + 0 + 25*LSPA;
  unsigned int globalReadOffsetAL_26_0 = globalReadOffsetAL + 0 + 26*LSPA;
  unsigned int globalReadOffsetAL_27_0 = globalReadOffsetAL + 0 + 27*LSPA;
  unsigned int globalReadOffsetAL_28_0 = globalReadOffsetAL + 0 + 28*LSPA;
  unsigned int globalReadOffsetAL_29_0 = globalReadOffsetAL + 0 + 29*LSPA;
  unsigned int globalReadOffsetAL_30_0 = globalReadOffsetAL + 0 + 30*LSPA;
  unsigned int globalReadOffsetAL_31_0 = globalReadOffsetAL + 0 + 31*LSPA;
  unsigned int globalReadOffsetAL_32_0 = globalReadOffsetAL + 0 + 32*LSPA;
  unsigned int globalReadOffsetAL_33_0 = globalReadOffsetAL + 0 + 33*LSPA;
  unsigned int globalReadOffsetAL_34_0 = globalReadOffsetAL + 0 + 34*LSPA;
  unsigned int globalReadOffsetAL_35_0 = globalReadOffsetAL + 0 + 35*LSPA;
  unsigned int globalReadOffsetAL_36_0 = globalReadOffsetAL + 0 + 36*LSPA;
  unsigned int globalReadOffsetAL_37_0 = globalReadOffsetAL + 0 + 37*LSPA;
  unsigned int globalReadOffsetAL_38_0 = globalReadOffsetAL + 0 + 38*LSPA;
  unsigned int globalReadOffsetAL_39_0 = globalReadOffsetAL + 0 + 39*LSPA;
  unsigned int globalReadOffsetAL_40_0 = globalReadOffsetAL + 0 + 40*LSPA;
  unsigned int globalReadOffsetAL_41_0 = globalReadOffsetAL + 0 + 41*LSPA;
  unsigned int globalReadOffsetAL_42_0 = globalReadOffsetAL + 0 + 42*LSPA;
  unsigned int globalReadOffsetAL_43_0 = globalReadOffsetAL + 0 + 43*LSPA;
  unsigned int globalReadOffsetAL_44_0 = globalReadOffsetAL + 0 + 44*LSPA;
  unsigned int globalReadOffsetAL_45_0 = globalReadOffsetAL + 0 + 45*LSPA;
  unsigned int globalReadOffsetAL_46_0 = globalReadOffsetAL + 0 + 46*LSPA;
  unsigned int globalReadOffsetAL_47_0 = globalReadOffsetAL + 0 + 47*LSPA;
  unsigned int globalReadOffsetAL_48_0 = globalReadOffsetAL + 0 + 48*LSPA;
  unsigned int globalReadOffsetAL_49_0 = globalReadOffsetAL + 0 + 49*LSPA;
  unsigned int globalReadOffsetAL_50_0 = globalReadOffsetAL + 0 + 50*LSPA;
  unsigned int globalReadOffsetAL_51_0 = globalReadOffsetAL + 0 + 51*LSPA;
  unsigned int globalReadOffsetAL_52_0 = globalReadOffsetAL + 0 + 52*LSPA;
  unsigned int globalReadOffsetAL_53_0 = globalReadOffsetAL + 0 + 53*LSPA;
  unsigned int globalReadOffsetAL_54_0 = globalReadOffsetAL + 0 + 54*LSPA;
  unsigned int globalReadOffsetAL_55_0 = globalReadOffsetAL + 0 + 55*LSPA;
  unsigned int globalReadOffsetAL_56_0 = globalReadOffsetAL + 0 + 56*LSPA;
  unsigned int globalReadOffsetAL_57_0 = globalReadOffsetAL + 0 + 57*LSPA;
  unsigned int globalReadOffsetAL_58_0 = globalReadOffsetAL + 0 + 58*LSPA;
  unsigned int globalReadOffsetAL_59_0 = globalReadOffsetAL + 0 + 59*LSPA;
  unsigned int globalReadOffsetAL_60_0 = globalReadOffsetAL + 0 + 60*LSPA;
  unsigned int globalReadOffsetAL_61_0 = globalReadOffsetAL + 0 + 61*LSPA;
  unsigned int globalReadOffsetAL_62_0 = globalReadOffsetAL + 0 + 62*LSPA;
  unsigned int globalReadOffsetAL_63_0 = globalReadOffsetAL + 0 + 63*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_15_0), wgK );
  int64_t globalReadOffsetA_0_0_16_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_16_0), wgK );
  int64_t globalReadOffsetA_0_0_17_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_17_0), wgK );
  int64_t globalReadOffsetA_0_0_18_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_18_0), wgK );
  int64_t globalReadOffsetA_0_0_19_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_19_0), wgK );
  int64_t globalReadOffsetA_0_0_20_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_20_0), wgK );
  int64_t globalReadOffsetA_0_0_21_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_21_0), wgK );
  int64_t globalReadOffsetA_0_0_22_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_22_0), wgK );
  int64_t globalReadOffsetA_0_0_23_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_23_0), wgK );
  int64_t globalReadOffsetA_0_0_24_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_24_0), wgK );
  int64_t globalReadOffsetA_0_0_25_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_25_0), wgK );
  int64_t globalReadOffsetA_0_0_26_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_26_0), wgK );
  int64_t globalReadOffsetA_0_0_27_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_27_0), wgK );
  int64_t globalReadOffsetA_0_0_28_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_28_0), wgK );
  int64_t globalReadOffsetA_0_0_29_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_29_0), wgK );
  int64_t globalReadOffsetA_0_0_30_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_30_0), wgK );
  int64_t globalReadOffsetA_0_0_31_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_31_0), wgK );
  int64_t globalReadOffsetA_0_0_32_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_32_0), wgK );
  int64_t globalReadOffsetA_0_0_33_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_33_0), wgK );
  int64_t globalReadOffsetA_0_0_34_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_34_0), wgK );
  int64_t globalReadOffsetA_0_0_35_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_35_0), wgK );
  int64_t globalReadOffsetA_0_0_36_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_36_0), wgK );
  int64_t globalReadOffsetA_0_0_37_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_37_0), wgK );
  int64_t globalReadOffsetA_0_0_38_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_38_0), wgK );
  int64_t globalReadOffsetA_0_0_39_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_39_0), wgK );
  int64_t globalReadOffsetA_0_0_40_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_40_0), wgK );
  int64_t globalReadOffsetA_0_0_41_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_41_0), wgK );
  int64_t globalReadOffsetA_0_0_42_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_42_0), wgK );
  int64_t globalReadOffsetA_0_0_43_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_43_0), wgK );
  int64_t globalReadOffsetA_0_0_44_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_44_0), wgK );
  int64_t globalReadOffsetA_0_0_45_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_45_0), wgK );
  int64_t globalReadOffsetA_0_0_46_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_46_0), wgK );
  int64_t globalReadOffsetA_0_0_47_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_47_0), wgK );
  int64_t globalReadOffsetA_0_0_48_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_48_0), wgK );
  int64_t globalReadOffsetA_0_0_49_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_49_0), wgK );
  int64_t globalReadOffsetA_0_0_50_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_50_0), wgK );
  int64_t globalReadOffsetA_0_0_51_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_51_0), wgK );
  int64_t globalReadOffsetA_0_0_52_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_52_0), wgK );
  int64_t globalReadOffsetA_0_0_53_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_53_0), wgK );
  int64_t globalReadOffsetA_0_0_54_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_54_0), wgK );
  int64_t globalReadOffsetA_0_0_55_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_55_0), wgK );
  int64_t globalReadOffsetA_0_0_56_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_56_0), wgK );
  int64_t globalReadOffsetA_0_0_57_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_57_0), wgK );
  int64_t globalReadOffsetA_0_0_58_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_58_0), wgK );
  int64_t globalReadOffsetA_0_0_59_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_59_0), wgK );
  int64_t globalReadOffsetA_0_0_60_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_60_0), wgK );
  int64_t globalReadOffsetA_0_0_61_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_61_0), wgK );
  int64_t globalReadOffsetA_0_0_62_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_62_0), wgK );
  int64_t globalReadOffsetA_0_0_63_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_63_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;
  DATA_TYPE const *globalReadA_0_0_16_0 = A + globalReadOffsetA_0_0_16_0;
  DATA_TYPE const *globalReadA_0_0_17_0 = A + globalReadOffsetA_0_0_17_0;
  DATA_TYPE const *globalReadA_0_0_18_0 = A + globalReadOffsetA_0_0_18_0;
  DATA_TYPE const *globalReadA_0_0_19_0 = A + globalReadOffsetA_0_0_19_0;
  DATA_TYPE const *globalReadA_0_0_20_0 = A + globalReadOffsetA_0_0_20_0;
  DATA_TYPE const *globalReadA_0_0_21_0 = A + globalReadOffsetA_0_0_21_0;
  DATA_TYPE const *globalReadA_0_0_22_0 = A + globalReadOffsetA_0_0_22_0;
  DATA_TYPE const *globalReadA_0_0_23_0 = A + globalReadOffsetA_0_0_23_0;
  DATA_TYPE const *globalReadA_0_0_24_0 = A + globalReadOffsetA_0_0_24_0;
  DATA_TYPE const *globalReadA_0_0_25_0 = A + globalReadOffsetA_0_0_25_0;
  DATA_TYPE const *globalReadA_0_0_26_0 = A + globalReadOffsetA_0_0_26_0;
  DATA_TYPE const *globalReadA_0_0_27_0 = A + globalReadOffsetA_0_0_27_0;
  DATA_TYPE const *globalReadA_0_0_28_0 = A + globalReadOffsetA_0_0_28_0;
  DATA_TYPE const *globalReadA_0_0_29_0 = A + globalReadOffsetA_0_0_29_0;
  DATA_TYPE const *globalReadA_0_0_30_0 = A + globalReadOffsetA_0_0_30_0;
  DATA_TYPE const *globalReadA_0_0_31_0 = A + globalReadOffsetA_0_0_31_0;
  DATA_TYPE const *globalReadA_0_0_32_0 = A + globalReadOffsetA_0_0_32_0;
  DATA_TYPE const *globalReadA_0_0_33_0 = A + globalReadOffsetA_0_0_33_0;
  DATA_TYPE const *globalReadA_0_0_34_0 = A + globalReadOffsetA_0_0_34_0;
  DATA_TYPE const *globalReadA_0_0_35_0 = A + globalReadOffsetA_0_0_35_0;
  DATA_TYPE const *globalReadA_0_0_36_0 = A + globalReadOffsetA_0_0_36_0;
  DATA_TYPE const *globalReadA_0_0_37_0 = A + globalReadOffsetA_0_0_37_0;
  DATA_TYPE const *globalReadA_0_0_38_0 = A + globalReadOffsetA_0_0_38_0;
  DATA_TYPE const *globalReadA_0_0_39_0 = A + globalReadOffsetA_0_0_39_0;
  DATA_TYPE const *globalReadA_0_0_40_0 = A + globalReadOffsetA_0_0_40_0;
  DATA_TYPE const *globalReadA_0_0_41_0 = A + globalReadOffsetA_0_0_41_0;
  DATA_TYPE const *globalReadA_0_0_42_0 = A + globalReadOffsetA_0_0_42_0;
  DATA_TYPE const *globalReadA_0_0_43_0 = A + globalReadOffsetA_0_0_43_0;
  DATA_TYPE const *globalReadA_0_0_44_0 = A + globalReadOffsetA_0_0_44_0;
  DATA_TYPE const *globalReadA_0_0_45_0 = A + globalReadOffsetA_0_0_45_0;
  DATA_TYPE const *globalReadA_0_0_46_0 = A + globalReadOffsetA_0_0_46_0;
  DATA_TYPE const *globalReadA_0_0_47_0 = A + globalReadOffsetA_0_0_47_0;
  DATA_TYPE const *globalReadA_0_0_48_0 = A + globalReadOffsetA_0_0_48_0;
  DATA_TYPE const *globalReadA_0_0_49_0 = A + globalReadOffsetA_0_0_49_0;
  DATA_TYPE const *globalReadA_0_0_50_0 = A + globalReadOffsetA_0_0_50_0;
  DATA_TYPE const *globalReadA_0_0_51_0 = A + globalReadOffsetA_0_0_51_0;
  DATA_TYPE const *globalReadA_0_0_52_0 = A + globalReadOffsetA_0_0_52_0;
  DATA_TYPE const *globalReadA_0_0_53_0 = A + globalReadOffsetA_0_0_53_0;
  DATA_TYPE const *globalReadA_0_0_54_0 = A + globalReadOffsetA_0_0_54_0;
  DATA_TYPE const *globalReadA_0_0_55_0 = A + globalReadOffsetA_0_0_55_0;
  DATA_TYPE const *globalReadA_0_0_56_0 = A + globalReadOffsetA_0_0_56_0;
  DATA_TYPE const *globalReadA_0_0_57_0 = A + globalReadOffsetA_0_0_57_0;
  DATA_TYPE const *globalReadA_0_0_58_0 = A + globalReadOffsetA_0_0_58_0;
  DATA_TYPE const *globalReadA_0_0_59_0 = A + globalReadOffsetA_0_0_59_0;
  DATA_TYPE const *globalReadA_0_0_60_0 = A + globalReadOffsetA_0_0_60_0;
  DATA_TYPE const *globalReadA_0_0_61_0 = A + globalReadOffsetA_0_0_61_0;
  DATA_TYPE const *globalReadA_0_0_62_0 = A + globalReadOffsetA_0_0_62_0;
  DATA_TYPE const *globalReadA_0_0_63_0 = A + globalReadOffsetA_0_0_63_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 8*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 9*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 10*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 11*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 12*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 13*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 14*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 15*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_16_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 16*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_17_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 17*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_18_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 18*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_19_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 19*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_20_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 20*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_21_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 21*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_22_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 22*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_23_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 23*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_24_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 24*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_25_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 25*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_26_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 26*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_27_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 27*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_28_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 28*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_29_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 29*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_30_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 30*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_31_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 31*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_32_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 32*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_33_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 33*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_34_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 34*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_35_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 35*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_36_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 36*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_37_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 37*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_38_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 38*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_39_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 39*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_40_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 40*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_41_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 41*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_42_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 42*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_43_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 43*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_44_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 44*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_45_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 45*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_46_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 46*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_47_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 47*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_48_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 48*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_49_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 49*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_50_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 50*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_51_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 51*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_52_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 52*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_53_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 53*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_54_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 54*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_55_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 55*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_56_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 56*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_57_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 57*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_58_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 58*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_59_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 59*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_60_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 60*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_61_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 61*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_62_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 62*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_63_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 63*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_0_0_15_0;
  DATA_TYPE *localWriteA_0_0_16_0;
  DATA_TYPE *localWriteA_0_0_17_0;
  DATA_TYPE *localWriteA_0_0_18_0;
  DATA_TYPE *localWriteA_0_0_19_0;
  DATA_TYPE *localWriteA_0_0_20_0;
  DATA_TYPE *localWriteA_0_0_21_0;
  DATA_TYPE *localWriteA_0_0_22_0;
  DATA_TYPE *localWriteA_0_0_23_0;
  DATA_TYPE *localWriteA_0_0_24_0;
  DATA_TYPE *localWriteA_0_0_25_0;
  DATA_TYPE *localWriteA_0_0_26_0;
  DATA_TYPE *localWriteA_0_0_27_0;
  DATA_TYPE *localWriteA_0_0_28_0;
  DATA_TYPE *localWriteA_0_0_29_0;
  DATA_TYPE *localWriteA_0_0_30_0;
  DATA_TYPE *localWriteA_0_0_31_0;
  DATA_TYPE *localWriteA_0_0_32_0;
  DATA_TYPE *localWriteA_0_0_33_0;
  DATA_TYPE *localWriteA_0_0_34_0;
  DATA_TYPE *localWriteA_0_0_35_0;
  DATA_TYPE *localWriteA_0_0_36_0;
  DATA_TYPE *localWriteA_0_0_37_0;
  DATA_TYPE *localWriteA_0_0_38_0;
  DATA_TYPE *localWriteA_0_0_39_0;
  DATA_TYPE *localWriteA_0_0_40_0;
  DATA_TYPE *localWriteA_0_0_41_0;
  DATA_TYPE *localWriteA_0_0_42_0;
  DATA_TYPE *localWriteA_0_0_43_0;
  DATA_TYPE *localWriteA_0_0_44_0;
  DATA_TYPE *localWriteA_0_0_45_0;
  DATA_TYPE *localWriteA_0_0_46_0;
  DATA_TYPE *localWriteA_0_0_47_0;
  DATA_TYPE *localWriteA_0_0_48_0;
  DATA_TYPE *localWriteA_0_0_49_0;
  DATA_TYPE *localWriteA_0_0_50_0;
  DATA_TYPE *localWriteA_0_0_51_0;
  DATA_TYPE *localWriteA_0_0_52_0;
  DATA_TYPE *localWriteA_0_0_53_0;
  DATA_TYPE *localWriteA_0_0_54_0;
  DATA_TYPE *localWriteA_0_0_55_0;
  DATA_TYPE *localWriteA_0_0_56_0;
  DATA_TYPE *localWriteA_0_0_57_0;
  DATA_TYPE *localWriteA_0_0_58_0;
  DATA_TYPE *localWriteA_0_0_59_0;
  DATA_TYPE *localWriteA_0_0_60_0;
  DATA_TYPE *localWriteA_0_0_61_0;
  DATA_TYPE *localWriteA_0_0_62_0;
  DATA_TYPE *localWriteA_0_0_63_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);
  localWriteA_0_0_16_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_16_0);
  localWriteA_0_0_17_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_17_0);
  localWriteA_0_0_18_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_18_0);
  localWriteA_0_0_19_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_19_0);
  localWriteA_0_0_20_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_20_0);
  localWriteA_0_0_21_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_21_0);
  localWriteA_0_0_22_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_22_0);
  localWriteA_0_0_23_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_23_0);
  localWriteA_0_0_24_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_24_0);
  localWriteA_0_0_25_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_25_0);
  localWriteA_0_0_26_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_26_0);
  localWriteA_0_0_27_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_27_0);
  localWriteA_0_0_28_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_28_0);
  localWriteA_0_0_29_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_29_0);
  localWriteA_0_0_30_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_30_0);
  localWriteA_0_0_31_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_31_0);
  localWriteA_0_0_32_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_32_0);
  localWriteA_0_0_33_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_33_0);
  localWriteA_0_0_34_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_34_0);
  localWriteA_0_0_35_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_35_0);
  localWriteA_0_0_36_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_36_0);
  localWriteA_0_0_37_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_37_0);
  localWriteA_0_0_38_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_38_0);
  localWriteA_0_0_39_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_39_0);
  localWriteA_0_0_40_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_40_0);
  localWriteA_0_0_41_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_41_0);
  localWriteA_0_0_42_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_42_0);
  localWriteA_0_0_43_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_43_0);
  localWriteA_0_0_44_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_44_0);
  localWriteA_0_0_45_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_45_0);
  localWriteA_0_0_46_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_46_0);
  localWriteA_0_0_47_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_47_0);
  localWriteA_0_0_48_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_48_0);
  localWriteA_0_0_49_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_49_0);
  localWriteA_0_0_50_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_50_0);
  localWriteA_0_0_51_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_51_0);
  localWriteA_0_0_52_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_52_0);
  localWriteA_0_0_53_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_53_0);
  localWriteA_0_0_54_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_54_0);
  localWriteA_0_0_55_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_55_0);
  localWriteA_0_0_56_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_56_0);
  localWriteA_0_0_57_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_57_0);
  localWriteA_0_0_58_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_58_0);
  localWriteA_0_0_59_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_59_0);
  localWriteA_0_0_60_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_60_0);
  localWriteA_0_0_61_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_61_0);
  localWriteA_0_0_62_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_62_0);
  localWriteA_0_0_63_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_63_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);
    a_0_0_16_0 = *(globalReadA_0_0_16_0 + 0);
    a_0_0_17_0 = *(globalReadA_0_0_17_0 + 0);
    a_0_0_18_0 = *(globalReadA_0_0_18_0 + 0);
    a_0_0_19_0 = *(globalReadA_0_0_19_0 + 0);
    a_0_0_20_0 = *(globalReadA_0_0_20_0 + 0);
    a_0_0_21_0 = *(globalReadA_0_0_21_0 + 0);
    a_0_0_22_0 = *(globalReadA_0_0_22_0 + 0);
    a_0_0_23_0 = *(globalReadA_0_0_23_0 + 0);
    a_0_0_24_0 = *(globalReadA_0_0_24_0 + 0);
    a_0_0_25_0 = *(globalReadA_0_0_25_0 + 0);
    a_0_0_26_0 = *(globalReadA_0_0_26_0 + 0);
    a_0_0_27_0 = *(globalReadA_0_0_27_0 + 0);
    a_0_0_28_0 = *(globalReadA_0_0_28_0 + 0);
    a_0_0_29_0 = *(globalReadA_0_0_29_0 + 0);
    a_0_0_30_0 = *(globalReadA_0_0_30_0 + 0);
    a_0_0_31_0 = *(globalReadA_0_0_31_0 + 0);
    a_0_0_32_0 = *(globalReadA_0_0_32_0 + 0);
    a_0_0_33_0 = *(globalReadA_0_0_33_0 + 0);
    a_0_0_34_0 = *(globalReadA_0_0_34_0 + 0);
    a_0_0_35_0 = *(globalReadA_0_0_35_0 + 0);
    a_0_0_36_0 = *(globalReadA_0_0_36_0 + 0);
    a_0_0_37_0 = *(globalReadA_0_0_37_0 + 0);
    a_0_0_38_0 = *(globalReadA_0_0_38_0 + 0);
    a_0_0_39_0 = *(globalReadA_0_0_39_0 + 0);
    a_0_0_40_0 = *(globalReadA_0_0_40_0 + 0);
    a_0_0_41_0 = *(globalReadA_0_0_41_0 + 0);
    a_0_0_42_0 = *(globalReadA_0_0_42_0 + 0);
    a_0_0_43_0 = *(globalReadA_0_0_43_0 + 0);
    a_0_0_44_0 = *(globalReadA_0_0_44_0 + 0);
    a_0_0_45_0 = *(globalReadA_0_0_45_0 + 0);
    a_0_0_46_0 = *(globalReadA_0_0_46_0 + 0);
    a_0_0_47_0 = *(globalReadA_0_0_47_0 + 0);
    a_0_0_48_0 = *(globalReadA_0_0_48_0 + 0);
    a_0_0_49_0 = *(globalReadA_0_0_49_0 + 0);
    a_0_0_50_0 = *(globalReadA_0_0_50_0 + 0);
    a_0_0_51_0 = *(globalReadA_0_0_51_0 + 0);
    a_0_0_52_0 = *(globalReadA_0_0_52_0 + 0);
    a_0_0_53_0 = *(globalReadA_0_0_53_0 + 0);
    a_0_0_54_0 = *(globalReadA_0_0_54_0 + 0);
    a_0_0_55_0 = *(globalReadA_0_0_55_0 + 0);
    a_0_0_56_0 = *(globalReadA_0_0_56_0 + 0);
    a_0_0_57_0 = *(globalReadA_0_0_57_0 + 0);
    a_0_0_58_0 = *(globalReadA_0_0_58_0 + 0);
    a_0_0_59_0 = *(globalReadA_0_0_59_0 + 0);
    a_0_0_60_0 = *(globalReadA_0_0_60_0 + 0);
    a_0_0_61_0 = *(globalReadA_0_0_61_0 + 0);
    a_0_0_62_0 = *(globalReadA_0_0_62_0 + 0);
    a_0_0_63_0 = *(globalReadA_0_0_63_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    globalReadA_0_0_16_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_16_0) + 1*globalReadIncAL);
    globalReadA_0_0_17_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_17_0) + 1*globalReadIncAL);
    globalReadA_0_0_18_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_18_0) + 1*globalReadIncAL);
    globalReadA_0_0_19_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_19_0) + 1*globalReadIncAL);
    globalReadA_0_0_20_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_20_0) + 1*globalReadIncAL);
    globalReadA_0_0_21_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_21_0) + 1*globalReadIncAL);
    globalReadA_0_0_22_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_22_0) + 1*globalReadIncAL);
    globalReadA_0_0_23_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_23_0) + 1*globalReadIncAL);
    globalReadA_0_0_24_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_24_0) + 1*globalReadIncAL);
    globalReadA_0_0_25_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_25_0) + 1*globalReadIncAL);
    globalReadA_0_0_26_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_26_0) + 1*globalReadIncAL);
    globalReadA_0_0_27_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_27_0) + 1*globalReadIncAL);
    globalReadA_0_0_28_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_28_0) + 1*globalReadIncAL);
    globalReadA_0_0_29_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_29_0) + 1*globalReadIncAL);
    globalReadA_0_0_30_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_30_0) + 1*globalReadIncAL);
    globalReadA_0_0_31_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_31_0) + 1*globalReadIncAL);
    globalReadA_0_0_32_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_32_0) + 1*globalReadIncAL);
    globalReadA_0_0_33_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_33_0) + 1*globalReadIncAL);
    globalReadA_0_0_34_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_34_0) + 1*globalReadIncAL);
    globalReadA_0_0_35_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_35_0) + 1*globalReadIncAL);
    globalReadA_0_0_36_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_36_0) + 1*globalReadIncAL);
    globalReadA_0_0_37_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_37_0) + 1*globalReadIncAL);
    globalReadA_0_0_38_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_38_0) + 1*globalReadIncAL);
    globalReadA_0_0_39_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_39_0) + 1*globalReadIncAL);
    globalReadA_0_0_40_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_40_0) + 1*globalReadIncAL);
    globalReadA_0_0_41_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_41_0) + 1*globalReadIncAL);
    globalReadA_0_0_42_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_42_0) + 1*globalReadIncAL);
    globalReadA_0_0_43_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_43_0) + 1*globalReadIncAL);
    globalReadA_0_0_44_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_44_0) + 1*globalReadIncAL);
    globalReadA_0_0_45_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_45_0) + 1*globalReadIncAL);
    globalReadA_0_0_46_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_46_0) + 1*globalReadIncAL);
    globalReadA_0_0_47_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_47_0) + 1*globalReadIncAL);
    globalReadA_0_0_48_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_48_0) + 1*globalReadIncAL);
    globalReadA_0_0_49_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_49_0) + 1*globalReadIncAL);
    globalReadA_0_0_50_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_50_0) + 1*globalReadIncAL);
    globalReadA_0_0_51_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_51_0) + 1*globalReadIncAL);
    globalReadA_0_0_52_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_52_0) + 1*globalReadIncAL);
    globalReadA_0_0_53_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_53_0) + 1*globalReadIncAL);
    globalReadA_0_0_54_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_54_0) + 1*globalReadIncAL);
    globalReadA_0_0_55_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_55_0) + 1*globalReadIncAL);
    globalReadA_0_0_56_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_56_0) + 1*globalReadIncAL);
    globalReadA_0_0_57_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_57_0) + 1*globalReadIncAL);
    globalReadA_0_0_58_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_58_0) + 1*globalReadIncAL);
    globalReadA_0_0_59_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_59_0) + 1*globalReadIncAL);
    globalReadA_0_0_60_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_60_0) + 1*globalReadIncAL);
    globalReadA_0_0_61_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_61_0) + 1*globalReadIncAL);
    globalReadA_0_0_62_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_62_0) + 1*globalReadIncAL);
    globalReadA_0_0_63_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_63_0) + 1*globalReadIncAL);

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
    *(localWriteA_0_0_16_0 + 0) = a_0_0_16_0;
    *(localWriteA_0_0_17_0 + 0) = a_0_0_17_0;
    *(localWriteA_0_0_18_0 + 0) = a_0_0_18_0;
    *(localWriteA_0_0_19_0 + 0) = a_0_0_19_0;
    *(localWriteA_0_0_20_0 + 0) = a_0_0_20_0;
    *(localWriteA_0_0_21_0 + 0) = a_0_0_21_0;
    *(localWriteA_0_0_22_0 + 0) = a_0_0_22_0;
    *(localWriteA_0_0_23_0 + 0) = a_0_0_23_0;
    *(localWriteA_0_0_24_0 + 0) = a_0_0_24_0;
    *(localWriteA_0_0_25_0 + 0) = a_0_0_25_0;
    *(localWriteA_0_0_26_0 + 0) = a_0_0_26_0;
    *(localWriteA_0_0_27_0 + 0) = a_0_0_27_0;
    *(localWriteA_0_0_28_0 + 0) = a_0_0_28_0;
    *(localWriteA_0_0_29_0 + 0) = a_0_0_29_0;
    *(localWriteA_0_0_30_0 + 0) = a_0_0_30_0;
    *(localWriteA_0_0_31_0 + 0) = a_0_0_31_0;
    *(localWriteA_0_0_32_0 + 0) = a_0_0_32_0;
    *(localWriteA_0_0_33_0 + 0) = a_0_0_33_0;
    *(localWriteA_0_0_34_0 + 0) = a_0_0_34_0;
    *(localWriteA_0_0_35_0 + 0) = a_0_0_35_0;
    *(localWriteA_0_0_36_0 + 0) = a_0_0_36_0;
    *(localWriteA_0_0_37_0 + 0) = a_0_0_37_0;
    *(localWriteA_0_0_38_0 + 0) = a_0_0_38_0;
    *(localWriteA_0_0_39_0 + 0) = a_0_0_39_0;
    *(localWriteA_0_0_40_0 + 0) = a_0_0_40_0;
    *(localWriteA_0_0_41_0 + 0) = a_0_0_41_0;
    *(localWriteA_0_0_42_0 + 0) = a_0_0_42_0;
    *(localWriteA_0_0_43_0 + 0) = a_0_0_43_0;
    *(localWriteA_0_0_44_0 + 0) = a_0_0_44_0;
    *(localWriteA_0_0_45_0 + 0) = a_0_0_45_0;
    *(localWriteA_0_0_46_0 + 0) = a_0_0_46_0;
    *(localWriteA_0_0_47_0 + 0) = a_0_0_47_0;
    *(localWriteA_0_0_48_0 + 0) = a_0_0_48_0;
    *(localWriteA_0_0_49_0 + 0) = a_0_0_49_0;
    *(localWriteA_0_0_50_0 + 0) = a_0_0_50_0;
    *(localWriteA_0_0_51_0 + 0) = a_0_0_51_0;
    *(localWriteA_0_0_52_0 + 0) = a_0_0_52_0;
    *(localWriteA_0_0_53_0 + 0) = a_0_0_53_0;
    *(localWriteA_0_0_54_0 + 0) = a_0_0_54_0;
    *(localWriteA_0_0_55_0 + 0) = a_0_0_55_0;
    *(localWriteA_0_0_56_0 + 0) = a_0_0_56_0;
    *(localWriteA_0_0_57_0 + 0) = a_0_0_57_0;
    *(localWriteA_0_0_58_0 + 0) = a_0_0_58_0;
    *(localWriteA_0_0_59_0 + 0) = a_0_0_59_0;
    *(localWriteA_0_0_60_0 + 0) = a_0_0_60_0;
    *(localWriteA_0_0_61_0 + 0) = a_0_0_61_0;
    *(localWriteA_0_0_62_0 + 0) = a_0_0_62_0;
    *(localWriteA_0_0_63_0 + 0) = a_0_0_63_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 15 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 16 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 17 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 18 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 19 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 20 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 21 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 22 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 23 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 24 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 25 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 26 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 27 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 28 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 29 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 30 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 31 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 32 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 33 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 34 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 35 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 36 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 37 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 38 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 39 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 40 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 41 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 42 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 43 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 44 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 45 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 46 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 47 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 48 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 49 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 50 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 51 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 52 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 53 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 54 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 55 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 56 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 57 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 58 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 59 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 60 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 61 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 62 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_1x1


    /* iter 63 */

    MAC_1x1_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_0_8_0 = ( globalReadOffsetAL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_0_0_9_0 = ( globalReadOffsetAL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_0_0_10_0 = ( globalReadOffsetAL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_0_0_11_0 = ( globalReadOffsetAL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_0_0_12_0 = ( globalReadOffsetAL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_0_0_13_0 = ( globalReadOffsetAL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_0_0_14_0 = ( globalReadOffsetAL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_0_0_15_0 = ( globalReadOffsetAL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);
  a_0_0_16_0 = ( globalReadOffsetAL_16_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_16_0 + 0);
  a_0_0_17_0 = ( globalReadOffsetAL_17_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_17_0 + 0);
  a_0_0_18_0 = ( globalReadOffsetAL_18_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_18_0 + 0);
  a_0_0_19_0 = ( globalReadOffsetAL_19_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_19_0 + 0);
  a_0_0_20_0 = ( globalReadOffsetAL_20_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_20_0 + 0);
  a_0_0_21_0 = ( globalReadOffsetAL_21_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_21_0 + 0);
  a_0_0_22_0 = ( globalReadOffsetAL_22_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_22_0 + 0);
  a_0_0_23_0 = ( globalReadOffsetAL_23_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_23_0 + 0);
  a_0_0_24_0 = ( globalReadOffsetAL_24_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_24_0 + 0);
  a_0_0_25_0 = ( globalReadOffsetAL_25_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_25_0 + 0);
  a_0_0_26_0 = ( globalReadOffsetAL_26_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_26_0 + 0);
  a_0_0_27_0 = ( globalReadOffsetAL_27_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_27_0 + 0);
  a_0_0_28_0 = ( globalReadOffsetAL_28_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_28_0 + 0);
  a_0_0_29_0 = ( globalReadOffsetAL_29_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_29_0 + 0);
  a_0_0_30_0 = ( globalReadOffsetAL_30_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_30_0 + 0);
  a_0_0_31_0 = ( globalReadOffsetAL_31_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_31_0 + 0);
  a_0_0_32_0 = ( globalReadOffsetAL_32_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_32_0 + 0);
  a_0_0_33_0 = ( globalReadOffsetAL_33_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_33_0 + 0);
  a_0_0_34_0 = ( globalReadOffsetAL_34_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_34_0 + 0);
  a_0_0_35_0 = ( globalReadOffsetAL_35_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_35_0 + 0);
  a_0_0_36_0 = ( globalReadOffsetAL_36_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_36_0 + 0);
  a_0_0_37_0 = ( globalReadOffsetAL_37_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_37_0 + 0);
  a_0_0_38_0 = ( globalReadOffsetAL_38_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_38_0 + 0);
  a_0_0_39_0 = ( globalReadOffsetAL_39_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_39_0 + 0);
  a_0_0_40_0 = ( globalReadOffsetAL_40_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_40_0 + 0);
  a_0_0_41_0 = ( globalReadOffsetAL_41_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_41_0 + 0);
  a_0_0_42_0 = ( globalReadOffsetAL_42_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_42_0 + 0);
  a_0_0_43_0 = ( globalReadOffsetAL_43_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_43_0 + 0);
  a_0_0_44_0 = ( globalReadOffsetAL_44_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_44_0 + 0);
  a_0_0_45_0 = ( globalReadOffsetAL_45_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_45_0 + 0);
  a_0_0_46_0 = ( globalReadOffsetAL_46_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_46_0 + 0);
  a_0_0_47_0 = ( globalReadOffsetAL_47_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_47_0 + 0);
  a_0_0_48_0 = ( globalReadOffsetAL_48_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_48_0 + 0);
  a_0_0_49_0 = ( globalReadOffsetAL_49_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_49_0 + 0);
  a_0_0_50_0 = ( globalReadOffsetAL_50_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_50_0 + 0);
  a_0_0_51_0 = ( globalReadOffsetAL_51_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_51_0 + 0);
  a_0_0_52_0 = ( globalReadOffsetAL_52_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_52_0 + 0);
  a_0_0_53_0 = ( globalReadOffsetAL_53_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_53_0 + 0);
  a_0_0_54_0 = ( globalReadOffsetAL_54_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_54_0 + 0);
  a_0_0_55_0 = ( globalReadOffsetAL_55_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_55_0 + 0);
  a_0_0_56_0 = ( globalReadOffsetAL_56_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_56_0 + 0);
  a_0_0_57_0 = ( globalReadOffsetAL_57_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_57_0 + 0);
  a_0_0_58_0 = ( globalReadOffsetAL_58_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_58_0 + 0);
  a_0_0_59_0 = ( globalReadOffsetAL_59_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_59_0 + 0);
  a_0_0_60_0 = ( globalReadOffsetAL_60_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_60_0 + 0);
  a_0_0_61_0 = ( globalReadOffsetAL_61_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_61_0 + 0);
  a_0_0_62_0 = ( globalReadOffsetAL_62_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_62_0 + 0);
  a_0_0_63_0 = ( globalReadOffsetAL_63_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_63_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);
  localWriteA_0_0_16_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_16_0);
  localWriteA_0_0_17_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_17_0);
  localWriteA_0_0_18_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_18_0);
  localWriteA_0_0_19_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_19_0);
  localWriteA_0_0_20_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_20_0);
  localWriteA_0_0_21_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_21_0);
  localWriteA_0_0_22_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_22_0);
  localWriteA_0_0_23_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_23_0);
  localWriteA_0_0_24_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_24_0);
  localWriteA_0_0_25_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_25_0);
  localWriteA_0_0_26_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_26_0);
  localWriteA_0_0_27_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_27_0);
  localWriteA_0_0_28_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_28_0);
  localWriteA_0_0_29_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_29_0);
  localWriteA_0_0_30_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_30_0);
  localWriteA_0_0_31_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_31_0);
  localWriteA_0_0_32_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_32_0);
  localWriteA_0_0_33_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_33_0);
  localWriteA_0_0_34_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_34_0);
  localWriteA_0_0_35_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_35_0);
  localWriteA_0_0_36_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_36_0);
  localWriteA_0_0_37_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_37_0);
  localWriteA_0_0_38_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_38_0);
  localWriteA_0_0_39_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_39_0);
  localWriteA_0_0_40_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_40_0);
  localWriteA_0_0_41_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_41_0);
  localWriteA_0_0_42_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_42_0);
  localWriteA_0_0_43_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_43_0);
  localWriteA_0_0_44_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_44_0);
  localWriteA_0_0_45_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_45_0);
  localWriteA_0_0_46_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_46_0);
  localWriteA_0_0_47_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_47_0);
  localWriteA_0_0_48_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_48_0);
  localWriteA_0_0_49_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_49_0);
  localWriteA_0_0_50_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_50_0);
  localWriteA_0_0_51_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_51_0);
  localWriteA_0_0_52_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_52_0);
  localWriteA_0_0_53_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_53_0);
  localWriteA_0_0_54_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_54_0);
  localWriteA_0_0_55_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_55_0);
  localWriteA_0_0_56_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_56_0);
  localWriteA_0_0_57_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_57_0);
  localWriteA_0_0_58_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_58_0);
  localWriteA_0_0_59_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_59_0);
  localWriteA_0_0_60_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_60_0);
  localWriteA_0_0_61_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_61_0);
  localWriteA_0_0_62_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_62_0);
  localWriteA_0_0_63_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_63_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
  *(localWriteA_0_0_16_0 + 0) = a_0_0_16_0;
  *(localWriteA_0_0_17_0 + 0) = a_0_0_17_0;
  *(localWriteA_0_0_18_0 + 0) = a_0_0_18_0;
  *(localWriteA_0_0_19_0 + 0) = a_0_0_19_0;
  *(localWriteA_0_0_20_0 + 0) = a_0_0_20_0;
  *(localWriteA_0_0_21_0 + 0) = a_0_0_21_0;
  *(localWriteA_0_0_22_0 + 0) = a_0_0_22_0;
  *(localWriteA_0_0_23_0 + 0) = a_0_0_23_0;
  *(localWriteA_0_0_24_0 + 0) = a_0_0_24_0;
  *(localWriteA_0_0_25_0 + 0) = a_0_0_25_0;
  *(localWriteA_0_0_26_0 + 0) = a_0_0_26_0;
  *(localWriteA_0_0_27_0 + 0) = a_0_0_27_0;
  *(localWriteA_0_0_28_0 + 0) = a_0_0_28_0;
  *(localWriteA_0_0_29_0 + 0) = a_0_0_29_0;
  *(localWriteA_0_0_30_0 + 0) = a_0_0_30_0;
  *(localWriteA_0_0_31_0 + 0) = a_0_0_31_0;
  *(localWriteA_0_0_32_0 + 0) = a_0_0_32_0;
  *(localWriteA_0_0_33_0 + 0) = a_0_0_33_0;
  *(localWriteA_0_0_34_0 + 0) = a_0_0_34_0;
  *(localWriteA_0_0_35_0 + 0) = a_0_0_35_0;
  *(localWriteA_0_0_36_0 + 0) = a_0_0_36_0;
  *(localWriteA_0_0_37_0 + 0) = a_0_0_37_0;
  *(localWriteA_0_0_38_0 + 0) = a_0_0_38_0;
  *(localWriteA_0_0_39_0 + 0) = a_0_0_39_0;
  *(localWriteA_0_0_40_0 + 0) = a_0_0_40_0;
  *(localWriteA_0_0_41_0 + 0) = a_0_0_41_0;
  *(localWriteA_0_0_42_0 + 0) = a_0_0_42_0;
  *(localWriteA_0_0_43_0 + 0) = a_0_0_43_0;
  *(localWriteA_0_0_44_0 + 0) = a_0_0_44_0;
  *(localWriteA_0_0_45_0 + 0) = a_0_0_45_0;
  *(localWriteA_0_0_46_0 + 0) = a_0_0_46_0;
  *(localWriteA_0_0_47_0 + 0) = a_0_0_47_0;
  *(localWriteA_0_0_48_0 + 0) = a_0_0_48_0;
  *(localWriteA_0_0_49_0 + 0) = a_0_0_49_0;
  *(localWriteA_0_0_50_0 + 0) = a_0_0_50_0;
  *(localWriteA_0_0_51_0 + 0) = a_0_0_51_0;
  *(localWriteA_0_0_52_0 + 0) = a_0_0_52_0;
  *(localWriteA_0_0_53_0 + 0) = a_0_0_53_0;
  *(localWriteA_0_0_54_0 + 0) = a_0_0_54_0;
  *(localWriteA_0_0_55_0 + 0) = a_0_0_55_0;
  *(localWriteA_0_0_56_0 + 0) = a_0_0_56_0;
  *(localWriteA_0_0_57_0 + 0) = a_0_0_57_0;
  *(localWriteA_0_0_58_0 + 0) = a_0_0_58_0;
  *(localWriteA_0_0_59_0 + 0) = a_0_0_59_0;
  *(localWriteA_0_0_60_0 + 0) = a_0_0_60_0;
  *(localWriteA_0_0_61_0 + 0) = a_0_0_61_0;
  *(localWriteA_0_0_62_0 + 0) = a_0_0_62_0;
  *(localWriteA_0_0_63_0 + 0) = a_0_0_63_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_1x1

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_1x1
#undef MAC_1x1_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 64
#define SG1J 1
#define TT0I 1
#define TT1J 1
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 64
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 64
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 4096
#define LDS_NUM_ELEMENTS 4160

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 1x1 micro-tile */
#define MAC_1x1\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \

#define MAC_1x1_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT64x1x64_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU0_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT1_1_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG64_1_1_WGM1(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE a_0_0_16_0;
  DATA_TYPE a_0_0_17_0;
  DATA_TYPE a_0_0_18_0;
  DATA_TYPE a_0_0_19_0;
  DATA_TYPE a_0_0_20_0;
  DATA_TYPE a_0_0_21_0;
  DATA_TYPE a_0_0_22_0;
  DATA_TYPE a_0_0_23_0;
  DATA_TYPE a_0_0_24_0;
  DATA_TYPE a_0_0_25_0;
  DATA_TYPE a_0_0_26_0;
  DATA_TYPE a_0_0_27_0;
  DATA_TYPE a_0_0_28_0;
  DATA_TYPE a_0_0_29_0;
  DATA_TYPE a_0_0_30_0;
  DATA_TYPE a_0_0_31_0;
  DATA_TYPE a_0_0_32_0;
  DATA_TYPE a_0_0_33_0;
  DATA_TYPE a_0_0_34_0;
  DATA_TYPE a_0_0_35_0;
  DATA_TYPE a_0_0_36_0;
  DATA_TYPE a_0_0_37_0;
  DATA_TYPE a_0_0_38_0;
  DATA_TYPE a_0_0_39_0;
  DATA_TYPE a_0_0_40_0;
  DATA_TYPE a_0_0_41_0;
  DATA_TYPE a_0_0_42_0;
  DATA_TYPE a_0_0_43_0;
  DATA_TYPE a_0_0_44_0;
  DATA_TYPE a_0_0_45_0;
  DATA_TYPE a_0_0_46_0;
  DATA_TYPE a_0_0_47_0;
  DATA_TYPE a_0_0_48_0;
  DATA_TYPE a_0_0_49_0;
  DATA_TYPE a_0_0_50_0;
  DATA_TYPE a_0_0_51_0;
  DATA_TYPE a_0_0_52_0;
  DATA_TYPE a_0_0_53_0;
  DATA_TYPE a_0_0_54_0;
  DATA_TYPE a_0_0_55_0;
  DATA_TYPE a_0_0_56_0;
  DATA_TYPE a_0_0_57_0;
  DATA_TYPE a_0_0_58_0;
  DATA_TYPE a_0_0_59_0;
  DATA_TYPE a_0_0_60_0;
  DATA_TYPE a_0_0_61_0;
  DATA_TYPE a_0_0_62_0;
  DATA_TYPE a_0_0_63_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;
  unsigned int globalReadOffsetAL_8_0 = globalReadOffsetAL + 0 + 8*LSPA;
  unsigned int globalReadOffsetAL_9_0 = globalReadOffsetAL + 0 + 9*LSPA;
  unsigned int globalReadOffsetAL_10_0 = globalReadOffsetAL + 0 + 10*LSPA;
  unsigned int globalReadOffsetAL_11_0 = globalReadOffsetAL + 0 + 11*LSPA;
  unsigned int globalReadOffsetAL_12_0 = globalReadOffsetAL + 0 + 12*LSPA;
  unsigned int globalReadOffsetAL_13_0 = globalReadOffsetAL + 0 + 13*LSPA;
  unsigned int globalReadOffsetAL_14_0 = globalReadOffsetAL + 0 + 14*LSPA;
  unsigned int globalReadOffsetAL_15_0 = globalReadOffsetAL + 0 + 15*LSPA;
  unsigned int globalReadOffsetAL_16_0 = globalReadOffsetAL + 0 + 16*LSPA;
  unsigned int globalReadOffsetAL_17_0 = globalReadOffsetAL + 0 + 17*LSPA;
  unsigned int globalReadOffsetAL_18_0 = globalReadOffsetAL + 0 + 18*LSPA;
  unsigned int globalReadOffsetAL_19_0 = globalReadOffsetAL + 0 + 19*LSPA;
  unsigned int globalReadOffsetAL_20_0 = globalReadOffsetAL + 0 + 20*LSPA;
  unsigned int globalReadOffsetAL_21_0 = globalReadOffsetAL + 0 + 21*LSPA;
  unsigned int globalReadOffsetAL_22_0 = globalReadOffsetAL + 0 + 22*LSPA;
  unsigned int globalReadOffsetAL_23_0 = globalReadOffsetAL + 0 + 23*LSPA;
  unsigned int globalReadOffsetAL_24_0 = globalReadOffsetAL + 0 + 24*LSPA;
  unsigned int globalReadOffsetAL_25_0 = globalReadOffsetAL + 0 + 25*LSPA;
  unsigned int globalReadOffsetAL_26_0 = globalReadOffsetAL + 0 + 26*LSPA;
  unsigned int globalReadOffsetAL_27_0 = globalReadOffsetAL + 0 + 27*LSPA;
  unsigned int globalReadOffsetAL_28_0 = globalReadOffsetAL + 0 + 28*LSPA;
  unsigned int globalReadOffsetAL_29_0 = globalReadOffsetAL + 0 + 29*LSPA;
  unsigned int globalReadOffsetAL_30_0 = globalReadOffsetAL + 0 + 30*LSPA;
  unsigned int globalReadOffsetAL_31_0 = globalReadOffsetAL + 0 + 31*LSPA;
  unsigned int globalReadOffsetAL_32_0 = globalReadOffsetAL + 0 + 32*LSPA;
  unsigned int globalReadOffsetAL_33_0 = globalReadOffsetAL + 0 + 33*LSPA;
  unsigned int globalReadOffsetAL_34_0 = globalReadOffsetAL + 0 + 34*LSPA;
  unsigned int globalReadOffsetAL_35_0 = globalReadOffsetAL + 0 + 35*LSPA;
  unsigned int globalReadOffsetAL_36_0 = globalReadOffsetAL + 0 + 36*LSPA;
  unsigned int globalReadOffsetAL_37_0 = globalReadOffsetAL + 0 + 37*LSPA;
  unsigned int globalReadOffsetAL_38_0 = globalReadOffsetAL + 0 + 38*LSPA;
  unsigned int globalReadOffsetAL_39_0 = globalReadOffsetAL + 0 + 39*LSPA;
  unsigned int globalReadOffsetAL_40_0 = globalReadOffsetAL + 0 + 40*LSPA;
  unsigned int globalReadOffsetAL_41_0 = globalReadOffsetAL + 0 + 41*LSPA;
  unsigned int globalReadOffsetAL_42_0 = globalReadOffsetAL + 0 + 42*LSPA;
  unsigned int globalReadOffsetAL_43_0 = globalReadOffsetAL + 0 + 43*LSPA;
  unsigned int globalReadOffsetAL_44_0 = globalReadOffsetAL + 0 + 44*LSPA;
  unsigned int globalReadOffsetAL_45_0 = globalReadOffsetAL + 0 + 45*LSPA;
  unsigned int globalReadOffsetAL_46_0 = globalReadOffsetAL + 0 + 46*LSPA;
  unsigned int globalReadOffsetAL_47_0 = globalReadOffsetAL + 0 + 47*LSPA;
  unsigned int globalReadOffsetAL_48_0 = globalReadOffsetAL + 0 + 48*LSPA;
  unsigned int globalReadOffsetAL_49_0 = globalReadOffsetAL + 0 + 49*LSPA;
  unsigned int globalReadOffsetAL_50_0 = globalReadOffsetAL + 0 + 50*LSPA;
  unsigned int globalReadOffsetAL_51_0 = globalReadOffsetAL + 0 + 51*LSPA;
  unsigned int globalReadOffsetAL_52_0 = globalReadOffsetAL + 0 + 52*LSPA;
  unsigned int globalReadOffsetAL_53_0 = globalReadOffsetAL + 0 + 53*LSPA;
  unsigned int globalReadOffsetAL_54_0 = globalReadOffsetAL + 0 + 54*LSPA;
  unsigned int globalReadOffsetAL_55_0 = globalReadOffsetAL + 0 + 55*LSPA;
  unsigned int globalReadOffsetAL_56_0 = globalReadOffsetAL + 0 + 56*LSPA;
  unsigned int globalReadOffsetAL_57_0 = globalReadOffsetAL + 0 + 57*LSPA;
  unsigned int globalReadOffsetAL_58_0 = globalReadOffsetAL + 0 + 58*LSPA;
  unsigned int globalReadOffsetAL_59_0 = globalReadOffsetAL + 0 + 59*LSPA;
  unsigned int globalReadOffsetAL_60_0 = globalReadOffsetAL + 0 + 60*LSPA;
  unsigned int globalReadOffsetAL_61_0 = globalReadOffsetAL + 0 + 61*LSPA;
  unsigned int globalReadOffsetAL_62_0 = globalReadOffsetAL + 0 + 62*LSPA;
  unsigned int globalReadOffsetAL_63_0 = globalReadOffsetAL + 0 + 63*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_15_0), wgK );
  int64_t globalReadOffsetA_0_0_16_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_16_0), wgK );
  int64_t globalReadOffsetA_0_0_17_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_17_0), wgK );
  int64_t globalReadOffsetA_0_0_18_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_18_0), wgK );
  int64_t globalReadOffsetA_0_0_19_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_19_0), wgK );
  int64_t globalReadOffsetA_0_0_20_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_20_0), wgK );
  int64_t globalReadOffsetA_0_0_21_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_21_0), wgK );
  int64_t globalReadOffsetA_0_0_22_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_22_0), wgK );
  int64_t globalReadOffsetA_0_0_23_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_23_0), wgK );
  int64_t globalReadOffsetA_0_0_24_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_24_0), wgK );
  int64_t globalReadOffsetA_0_0_25_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_25_0), wgK );
  int64_t globalReadOffsetA_0_0_26_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_26_0), wgK );
  int64_t globalReadOffsetA_0_0_27_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_27_0), wgK );
  int64_t globalReadOffsetA_0_0_28_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_28_0), wgK );
  int64_t globalReadOffsetA_0_0_29_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_29_0), wgK );
  int64_t globalReadOffsetA_0_0_30_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_30_0), wgK );
  int64_t globalReadOffsetA_0_0_31_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_31_0), wgK );
  int64_t globalReadOffsetA_0_0_32_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_32_0), wgK );
  int64_t globalReadOffsetA_0_0_33_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_33_0), wgK );
  int64_t globalReadOffsetA_0_0_34_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_34_0), wgK );
  int64_t globalReadOffsetA_0_0_35_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_35_0), wgK );
  int64_t globalReadOffsetA_0_0_36_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_36_0), wgK );
  int64_t globalReadOffsetA_0_0_37_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_37_0), wgK );
  int64_t globalReadOffsetA_0_0_38_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_38_0), wgK );
  int64_t globalReadOffsetA_0_0_39_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_39_0), wgK );
  int64_t globalReadOffsetA_0_0_40_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_40_0), wgK );
  int64_t globalReadOffsetA_0_0_41_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_41_0), wgK );
  int64_t globalReadOffsetA_0_0_42_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_42_0), wgK );
  int64_t globalReadOffsetA_0_0_43_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_43_0), wgK );
  int64_t globalReadOffsetA_0_0_44_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_44_0), wgK );
  int64_t globalReadOffsetA_0_0_45_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_45_0), wgK );
  int64_t globalReadOffsetA_0_0_46_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_46_0), wgK );
  int64_t globalReadOffsetA_0_0_47_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_47_0), wgK );
  int64_t globalReadOffsetA_0_0_48_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_48_0), wgK );
  int64_t globalReadOffsetA_0_0_49_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_49_0), wgK );
  int64_t globalReadOffsetA_0_0_50_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_50_0), wgK );
  int64_t globalReadOffsetA_0_0_51_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_51_0), wgK );
  int64_t globalReadOffsetA_0_0_52_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_52_0), wgK );
  int64_t globalReadOffsetA_0_0_53_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_53_0), wgK );
  int64_t globalReadOffsetA_0_0_54_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_54_0), wgK );
  int64_t globalReadOffsetA_0_0_55_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_55_0), wgK );
  int64_t globalReadOffsetA_0_0_56_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_56_0), wgK );
  int64_t globalReadOffsetA_0_0_57_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_57_0), wgK );
  int64_t globalReadOffsetA_0_0_58_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_58_0), wgK );
  int64_t globalReadOffsetA_0_0_59_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_59_0), wgK );
  int64_t globalReadOffsetA_0_0_60_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_60_0), wgK );
  int64_t globalReadOffsetA_0_0_61_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_61_0), wgK );
  int64_t globalReadOffsetA_0_0_62_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_62_0), wgK );
  int64_t globalReadOffsetA_0_0_63_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_63_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;
  DATA_TYPE const *globalReadA_0_0_16_0 = A + globalReadOffsetA_0_0_16_0;
  DATA_TYPE const *globalReadA_0_0_17_0 = A + globalReadOffsetA_0_0_17_0;
  DATA_TYPE const *globalReadA_0_0_18_0 = A + globalReadOffsetA_0_0_18_0;
  DATA_TYPE const *globalReadA_0_0_19_0 = A + globalReadOffsetA_0_0_19_0;
  DATA_TYPE const *globalReadA_0_0_20_0 = A + globalReadOffsetA_0_0_20_0;
  DATA_TYPE const *globalReadA_0_0_21_0 = A + globalReadOffsetA_0_0_21_0;
  DATA_TYPE const *globalReadA_0_0_22_0 = A + globalReadOffsetA_0_0_22_0;
  DATA_TYPE const *globalReadA_0_0_23_0 = A + globalReadOffsetA_0_0_23_0;
  DATA_TYPE const *globalReadA_0_0_24_0 = A + globalReadOffsetA_0_0_24_0;
  DATA_TYPE const *globalReadA_0_0_25_0 = A + globalReadOffsetA_0_0_25_0;
  DATA_TYPE const *globalReadA_0_0_26_0 = A + globalReadOffsetA_0_0_26_0;
  DATA_TYPE const *globalReadA_0_0_27_0 = A + globalReadOffsetA_0_0_27_0;
  DATA_TYPE const *globalReadA_0_0_28_0 = A + globalReadOffsetA_0_0_28_0;
  DATA_TYPE const *globalReadA_0_0_29_0 = A + globalReadOffsetA_0_0_29_0;
  DATA_TYPE const *globalReadA_0_0_30_0 = A + globalReadOffsetA_0_0_30_0;
  DATA_TYPE const *globalReadA_0_0_31_0 = A + globalReadOffsetA_0_0_31_0;
  DATA_TYPE const *globalReadA_0_0_32_0 = A + globalReadOffsetA_0_0_32_0;
  DATA_TYPE const *globalReadA_0_0_33_0 = A + globalReadOffsetA_0_0_33_0;
  DATA_TYPE const *globalReadA_0_0_34_0 = A + globalReadOffsetA_0_0_34_0;
  DATA_TYPE const *globalReadA_0_0_35_0 = A + globalReadOffsetA_0_0_35_0;
  DATA_TYPE const *globalReadA_0_0_36_0 = A + globalReadOffsetA_0_0_36_0;
  DATA_TYPE const *globalReadA_0_0_37_0 = A + globalReadOffsetA_0_0_37_0;
  DATA_TYPE const *globalReadA_0_0_38_0 = A + globalReadOffsetA_0_0_38_0;
  DATA_TYPE const *globalReadA_0_0_39_0 = A + globalReadOffsetA_0_0_39_0;
  DATA_TYPE const *globalReadA_0_0_40_0 = A + globalReadOffsetA_0_0_40_0;
  DATA_TYPE const *globalReadA_0_0_41_0 = A + globalReadOffsetA_0_0_41_0;
  DATA_TYPE const *globalReadA_0_0_42_0 = A + globalReadOffsetA_0_0_42_0;
  DATA_TYPE const *globalReadA_0_0_43_0 = A + globalReadOffsetA_0_0_43_0;
  DATA_TYPE const *globalReadA_0_0_44_0 = A + globalReadOffsetA_0_0_44_0;
  DATA_TYPE const *globalReadA_0_0_45_0 = A + globalReadOffsetA_0_0_45_0;
  DATA_TYPE const *globalReadA_0_0_46_0 = A + globalReadOffsetA_0_0_46_0;
  DATA_TYPE const *globalReadA_0_0_47_0 = A + globalReadOffsetA_0_0_47_0;
  DATA_TYPE const *globalReadA_0_0_48_0 = A + globalReadOffsetA_0_0_48_0;
  DATA_TYPE const *globalReadA_0_0_49_0 = A + globalReadOffsetA_0_0_49_0;
  DATA_TYPE const *globalReadA_0_0_50_0 = A + globalReadOffsetA_0_0_50_0;
  DATA_TYPE const *globalReadA_0_0_51_0 = A + globalReadOffsetA_0_0_51_0;
  DATA_TYPE const *globalReadA_0_0_52_0 = A + globalReadOffsetA_0_0_52_0;
  DATA_TYPE const *globalReadA_0_0_53_0 = A + globalReadOffsetA_0_0_53_0;
  DATA_TYPE const *globalReadA_0_0_54_0 = A + globalReadOffsetA_0_0_54_0;
  DATA_TYPE const *globalReadA_0_0_55_0 = A + globalReadOffsetA_0_0_55_0;
  DATA_TYPE const *globalReadA_0_0_56_0 = A + globalReadOffsetA_0_0_56_0;
  DATA_TYPE const *globalReadA_0_0_57_0 = A + globalReadOffsetA_0_0_57_0;
  DATA_TYPE const *globalReadA_0_0_58_0 = A + globalReadOffsetA_0_0_58_0;
  DATA_TYPE const *globalReadA_0_0_59_0 = A + globalReadOffsetA_0_0_59_0;
  DATA_TYPE const *globalReadA_0_0_60_0 = A + globalReadOffsetA_0_0_60_0;
  DATA_TYPE const *globalReadA_0_0_61_0 = A + globalReadOffsetA_0_0_61_0;
  DATA_TYPE const *globalReadA_0_0_62_0 = A + globalReadOffsetA_0_0_62_0;
  DATA_TYPE const *globalReadA_0_0_63_0 = A + globalReadOffsetA_0_0_63_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 8*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 9*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 10*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 11*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 12*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 13*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 14*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 15*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_16_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 16*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_17_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 17*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_18_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 18*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_19_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 19*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_20_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 20*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_21_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 21*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_22_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 22*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_23_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 23*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_24_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 24*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_25_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 25*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_26_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 26*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_27_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 27*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_28_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 28*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_29_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 29*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_30_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 30*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_31_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 31*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_32_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 32*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_33_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 33*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_34_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 34*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_35_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 35*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_36_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 36*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_37_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 37*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_38_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 38*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_39_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 39*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_40_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 40*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_41_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 41*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_42_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 42*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_43_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 43*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_44_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 44*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_45_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 45*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_46_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 46*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_47_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 47*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_48_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 48*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_49_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 49*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_50_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 50*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_51_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 51*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_52_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 52*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_53_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 53*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_54_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 54*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_55_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 55*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_56_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 56*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_57_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 57*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_58_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 58*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_59_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 59*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_60_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 60*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_61_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 61*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_62_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 62*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_63_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 63*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_0_0_15_0;
  DATA_TYPE *localWriteA_0_0_16_0;
  DATA_TYPE *localWriteA_0_0_17_0;
  DATA_TYPE *localWriteA_0_0_18_0;
  DATA_TYPE *localWriteA_0_0_19_0;
  DATA_TYPE *localWriteA_0_0_20_0;
  DATA_TYPE *localWriteA_0_0_21_0;
  DATA_TYPE *localWriteA_0_0_22_0;
  DATA_TYPE *localWriteA_0_0_23_0;
  DATA_TYPE *localWriteA_0_0_24_0;
  DATA_TYPE *localWriteA_0_0_25_0;
  DATA_TYPE *localWriteA_0_0_26_0;
  DATA_TYPE *localWriteA_0_0_27_0;
  DATA_TYPE *localWriteA_0_0_28_0;
  DATA_TYPE *localWriteA_0_0_29_0;
  DATA_TYPE *localWriteA_0_0_30_0;
  DATA_TYPE *localWriteA_0_0_31_0;
  DATA_TYPE *localWriteA_0_0_32_0;
  DATA_TYPE *localWriteA_0_0_33_0;
  DATA_TYPE *localWriteA_0_0_34_0;
  DATA_TYPE *localWriteA_0_0_35_0;
  DATA_TYPE *localWriteA_0_0_36_0;
  DATA_TYPE *localWriteA_0_0_37_0;
  DATA_TYPE *localWriteA_0_0_38_0;
  DATA_TYPE *localWriteA_0_0_39_0;
  DATA_TYPE *localWriteA_0_0_40_0;
  DATA_TYPE *localWriteA_0_0_41_0;
  DATA_TYPE *localWriteA_0_0_42_0;
  DATA_TYPE *localWriteA_0_0_43_0;
  DATA_TYPE *localWriteA_0_0_44_0;
  DATA_TYPE *localWriteA_0_0_45_0;
  DATA_TYPE *localWriteA_0_0_46_0;
  DATA_TYPE *localWriteA_0_0_47_0;
  DATA_TYPE *localWriteA_0_0_48_0;
  DATA_TYPE *localWriteA_0_0_49_0;
  DATA_TYPE *localWriteA_0_0_50_0;
  DATA_TYPE *localWriteA_0_0_51_0;
  DATA_TYPE *localWriteA_0_0_52_0;
  DATA_TYPE *localWriteA_0_0_53_0;
  DATA_TYPE *localWriteA_0_0_54_0;
  DATA_TYPE *localWriteA_0_0_55_0;
  DATA_TYPE *localWriteA_0_0_56_0;
  DATA_TYPE *localWriteA_0_0_57_0;
  DATA_TYPE *localWriteA_0_0_58_0;
  DATA_TYPE *localWriteA_0_0_59_0;
  DATA_TYPE *localWriteA_0_0_60_0;
  DATA_TYPE *localWriteA_0_0_61_0;
  DATA_TYPE *localWriteA_0_0_62_0;
  DATA_TYPE *localWriteA_0_0_63_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);
  localWriteA_0_0_16_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_16_0);
  localWriteA_0_0_17_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_17_0);
  localWriteA_0_0_18_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_18_0);
  localWriteA_0_0_19_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_19_0);
  localWriteA_0_0_20_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_20_0);
  localWriteA_0_0_21_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_21_0);
  localWriteA_0_0_22_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_22_0);
  localWriteA_0_0_23_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_23_0);
  localWriteA_0_0_24_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_24_0);
  localWriteA_0_0_25_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_25_0);
  localWriteA_0_0_26_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_26_0);
  localWriteA_0_0_27_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_27_0);
  localWriteA_0_0_28_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_28_0);
  localWriteA_0_0_29_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_29_0);
  localWriteA_0_0_30_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_30_0);
  localWriteA_0_0_31_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_31_0);
  localWriteA_0_0_32_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_32_0);
  localWriteA_0_0_33_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_33_0);
  localWriteA_0_0_34_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_34_0);
  localWriteA_0_0_35_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_35_0);
  localWriteA_0_0_36_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_36_0);
  localWriteA_0_0_37_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_37_0);
  localWriteA_0_0_38_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_38_0);
  localWriteA_0_0_39_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_39_0);
  localWriteA_0_0_40_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_40_0);
  localWriteA_0_0_41_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_41_0);
  localWriteA_0_0_42_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_42_0);
  localWriteA_0_0_43_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_43_0);
  localWriteA_0_0_44_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_44_0);
  localWriteA_0_0_45_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_45_0);
  localWriteA_0_0_46_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_46_0);
  localWriteA_0_0_47_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_47_0);
  localWriteA_0_0_48_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_48_0);
  localWriteA_0_0_49_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_49_0);
  localWriteA_0_0_50_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_50_0);
  localWriteA_0_0_51_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_51_0);
  localWriteA_0_0_52_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_52_0);
  localWriteA_0_0_53_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_53_0);
  localWriteA_0_0_54_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_54_0);
  localWriteA_0_0_55_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_55_0);
  localWriteA_0_0_56_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_56_0);
  localWriteA_0_0_57_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_57_0);
  localWriteA_0_0_58_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_58_0);
  localWriteA_0_0_59_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_59_0);
  localWriteA_0_0_60_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_60_0);
  localWriteA_0_0_61_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_61_0);
  localWriteA_0_0_62_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_62_0);
  localWriteA_0_0_63_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_63_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);
    a_0_0_16_0 = *(globalReadA_0_0_16_0 + 0);
    a_0_0_17_0 = *(globalReadA_0_0_17_0 + 0);
    a_0_0_18_0 = *(globalReadA_0_0_18_0 + 0);
    a_0_0_19_0 = *(globalReadA_0_0_19_0 + 0);
    a_0_0_20_0 = *(globalReadA_0_0_20_0 + 0);
    a_0_0_21_0 = *(globalReadA_0_0_21_0 + 0);
    a_0_0_22_0 = *(globalReadA_0_0_22_0 + 0);
    a_0_0_23_0 = *(globalReadA_0_0_23_0 + 0);
    a_0_0_24_0 = *(globalReadA_0_0_24_0 + 0);
    a_0_0_25_0 = *(globalReadA_0_0_25_0 + 0);
    a_0_0_26_0 = *(globalReadA_0_0_26_0 + 0);
    a_0_0_27_0 = *(globalReadA_0_0_27_0 + 0);
    a_0_0_28_0 = *(globalReadA_0_0_28_0 + 0);
    a_0_0_29_0 = *(globalReadA_0_0_29_0 + 0);
    a_0_0_30_0 = *(globalReadA_0_0_30_0 + 0);
    a_0_0_31_0 = *(globalReadA_0_0_31_0 + 0);
    a_0_0_32_0 = *(globalReadA_0_0_32_0 + 0);
    a_0_0_33_0 = *(globalReadA_0_0_33_0 + 0);
    a_0_0_34_0 = *(globalReadA_0_0_34_0 + 0);
    a_0_0_35_0 = *(globalReadA_0_0_35_0 + 0);
    a_0_0_36_0 = *(globalReadA_0_0_36_0 + 0);
    a_0_0_37_0 = *(globalReadA_0_0_37_0 + 0);
    a_0_0_38_0 = *(globalReadA_0_0_38_0 + 0);
    a_0_0_39_0 = *(globalReadA_0_0_39_0 + 0);
    a_0_0_40_0 = *(globalReadA_0_0_40_0 + 0);
    a_0_0_41_0 = *(globalReadA_0_0_41_0 + 0);
    a_0_0_42_0 = *(globalReadA_0_0_42_0 + 0);
    a_0_0_43_0 = *(globalReadA_0_0_43_0 + 0);
    a_0_0_44_0 = *(globalReadA_0_0_44_0 + 0);
    a_0_0_45_0 = *(globalReadA_0_0_45_0 + 0);
    a_0_0_46_0 = *(globalReadA_0_0_46_0 + 0);
    a_0_0_47_0 = *(globalReadA_0_0_47_0 + 0);
    a_0_0_48_0 = *(globalReadA_0_0_48_0 + 0);
    a_0_0_49_0 = *(globalReadA_0_0_49_0 + 0);
    a_0_0_50_0 = *(globalReadA_0_0_50_0 + 0);
    a_0_0_51_0 = *(globalReadA_0_0_51_0 + 0);
    a_0_0_52_0 = *(globalReadA_0_0_52_0 + 0);
    a_0_0_53_0 = *(globalReadA_0_0_53_0 + 0);
    a_0_0_54_0 = *(globalReadA_0_0_54_0 + 0);
    a_0_0_55_0 = *(globalReadA_0_0_55_0 + 0);
    a_0_0_56_0 = *(globalReadA_0_0_56_0 + 0);
    a_0_0_57_0 = *(globalReadA_0_0_57_0 + 0);
    a_0_0_58_0 = *(globalReadA_0_0_58_0 + 0);
    a_0_0_59_0 = *(globalReadA_0_0_59_0 + 0);
    a_0_0_60_0 = *(globalReadA_0_0_60_0 + 0);
    a_0_0_61_0 = *(globalReadA_0_0_61_0 + 0);
    a_0_0_62_0 = *(globalReadA_0_0_62_0 + 0);
    a_0_0_63_0 = *(globalReadA_0_0_63_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    globalReadA_0_0_16_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_16_0) + 1*globalReadIncAL);
    globalReadA_0_0_17_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_17_0) + 1*globalReadIncAL);
    globalReadA_0_0_18_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_18_0) + 1*globalReadIncAL);
    globalReadA_0_0_19_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_19_0) + 1*globalReadIncAL);
    globalReadA_0_0_20_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_20_0) + 1*globalReadIncAL);
    globalReadA_0_0_21_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_21_0) + 1*globalReadIncAL);
    globalReadA_0_0_22_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_22_0) + 1*globalReadIncAL);
    globalReadA_0_0_23_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_23_0) + 1*globalReadIncAL);
    globalReadA_0_0_24_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_24_0) + 1*globalReadIncAL);
    globalReadA_0_0_25_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_25_0) + 1*globalReadIncAL);
    globalReadA_0_0_26_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_26_0) + 1*globalReadIncAL);
    globalReadA_0_0_27_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_27_0) + 1*globalReadIncAL);
    globalReadA_0_0_28_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_28_0) + 1*globalReadIncAL);
    globalReadA_0_0_29_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_29_0) + 1*globalReadIncAL);
    globalReadA_0_0_30_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_30_0) + 1*globalReadIncAL);
    globalReadA_0_0_31_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_31_0) + 1*globalReadIncAL);
    globalReadA_0_0_32_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_32_0) + 1*globalReadIncAL);
    globalReadA_0_0_33_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_33_0) + 1*globalReadIncAL);
    globalReadA_0_0_34_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_34_0) + 1*globalReadIncAL);
    globalReadA_0_0_35_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_35_0) + 1*globalReadIncAL);
    globalReadA_0_0_36_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_36_0) + 1*globalReadIncAL);
    globalReadA_0_0_37_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_37_0) + 1*globalReadIncAL);
    globalReadA_0_0_38_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_38_0) + 1*globalReadIncAL);
    globalReadA_0_0_39_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_39_0) + 1*globalReadIncAL);
    globalReadA_0_0_40_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_40_0) + 1*globalReadIncAL);
    globalReadA_0_0_41_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_41_0) + 1*globalReadIncAL);
    globalReadA_0_0_42_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_42_0) + 1*globalReadIncAL);
    globalReadA_0_0_43_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_43_0) + 1*globalReadIncAL);
    globalReadA_0_0_44_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_44_0) + 1*globalReadIncAL);
    globalReadA_0_0_45_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_45_0) + 1*globalReadIncAL);
    globalReadA_0_0_46_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_46_0) + 1*globalReadIncAL);
    globalReadA_0_0_47_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_47_0) + 1*globalReadIncAL);
    globalReadA_0_0_48_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_48_0) + 1*globalReadIncAL);
    globalReadA_0_0_49_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_49_0) + 1*globalReadIncAL);
    globalReadA_0_0_50_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_50_0) + 1*globalReadIncAL);
    globalReadA_0_0_51_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_51_0) + 1*globalReadIncAL);
    globalReadA_0_0_52_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_52_0) + 1*globalReadIncAL);
    globalReadA_0_0_53_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_53_0) + 1*globalReadIncAL);
    globalReadA_0_0_54_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_54_0) + 1*globalReadIncAL);
    globalReadA_0_0_55_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_55_0) + 1*globalReadIncAL);
    globalReadA_0_0_56_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_56_0) + 1*globalReadIncAL);
    globalReadA_0_0_57_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_57_0) + 1*globalReadIncAL);
    globalReadA_0_0_58_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_58_0) + 1*globalReadIncAL);
    globalReadA_0_0_59_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_59_0) + 1*globalReadIncAL);
    globalReadA_0_0_60_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_60_0) + 1*globalReadIncAL);
    globalReadA_0_0_61_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_61_0) + 1*globalReadIncAL);
    globalReadA_0_0_62_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_62_0) + 1*globalReadIncAL);
    globalReadA_0_0_63_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_63_0) + 1*globalReadIncAL);

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
    *(localWriteA_0_0_16_0 + 0) = a_0_0_16_0;
    *(localWriteA_0_0_17_0 + 0) = a_0_0_17_0;
    *(localWriteA_0_0_18_0 + 0) = a_0_0_18_0;
    *(localWriteA_0_0_19_0 + 0) = a_0_0_19_0;
    *(localWriteA_0_0_20_0 + 0) = a_0_0_20_0;
    *(localWriteA_0_0_21_0 + 0) = a_0_0_21_0;
    *(localWriteA_0_0_22_0 + 0) = a_0_0_22_0;
    *(localWriteA_0_0_23_0 + 0) = a_0_0_23_0;
    *(localWriteA_0_0_24_0 + 0) = a_0_0_24_0;
    *(localWriteA_0_0_25_0 + 0) = a_0_0_25_0;
    *(localWriteA_0_0_26_0 + 0) = a_0_0_26_0;
    *(localWriteA_0_0_27_0 + 0) = a_0_0_27_0;
    *(localWriteA_0_0_28_0 + 0) = a_0_0_28_0;
    *(localWriteA_0_0_29_0 + 0) = a_0_0_29_0;
    *(localWriteA_0_0_30_0 + 0) = a_0_0_30_0;
    *(localWriteA_0_0_31_0 + 0) = a_0_0_31_0;
    *(localWriteA_0_0_32_0 + 0) = a_0_0_32_0;
    *(localWriteA_0_0_33_0 + 0) = a_0_0_33_0;
    *(localWriteA_0_0_34_0 + 0) = a_0_0_34_0;
    *(localWriteA_0_0_35_0 + 0) = a_0_0_35_0;
    *(localWriteA_0_0_36_0 + 0) = a_0_0_36_0;
    *(localWriteA_0_0_37_0 + 0) = a_0_0_37_0;
    *(localWriteA_0_0_38_0 + 0) = a_0_0_38_0;
    *(localWriteA_0_0_39_0 + 0) = a_0_0_39_0;
    *(localWriteA_0_0_40_0 + 0) = a_0_0_40_0;
    *(localWriteA_0_0_41_0 + 0) = a_0_0_41_0;
    *(localWriteA_0_0_42_0 + 0) = a_0_0_42_0;
    *(localWriteA_0_0_43_0 + 0) = a_0_0_43_0;
    *(localWriteA_0_0_44_0 + 0) = a_0_0_44_0;
    *(localWriteA_0_0_45_0 + 0) = a_0_0_45_0;
    *(localWriteA_0_0_46_0 + 0) = a_0_0_46_0;
    *(localWriteA_0_0_47_0 + 0) = a_0_0_47_0;
    *(localWriteA_0_0_48_0 + 0) = a_0_0_48_0;
    *(localWriteA_0_0_49_0 + 0) = a_0_0_49_0;
    *(localWriteA_0_0_50_0 + 0) = a_0_0_50_0;
    *(localWriteA_0_0_51_0 + 0) = a_0_0_51_0;
    *(localWriteA_0_0_52_0 + 0) = a_0_0_52_0;
    *(localWriteA_0_0_53_0 + 0) = a_0_0_53_0;
    *(localWriteA_0_0_54_0 + 0) = a_0_0_54_0;
    *(localWriteA_0_0_55_0 + 0) = a_0_0_55_0;
    *(localWriteA_0_0_56_0 + 0) = a_0_0_56_0;
    *(localWriteA_0_0_57_0 + 0) = a_0_0_57_0;
    *(localWriteA_0_0_58_0 + 0) = a_0_0_58_0;
    *(localWriteA_0_0_59_0 + 0) = a_0_0_59_0;
    *(localWriteA_0_0_60_0 + 0) = a_0_0_60_0;
    *(localWriteA_0_0_61_0 + 0) = a_0_0_61_0;
    *(localWriteA_0_0_62_0 + 0) = a_0_0_62_0;
    *(localWriteA_0_0_63_0 + 0) = a_0_0_63_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 15 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 16 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 17 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 18 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 19 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 20 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 21 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 22 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 23 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 24 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 25 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 26 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 27 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 28 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 29 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 30 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 31 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 32 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 33 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 34 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 35 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 36 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 37 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 38 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 39 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 40 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 41 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 42 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 43 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 44 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 45 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 46 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 47 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 48 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 49 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 50 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 51 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 52 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 53 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 54 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 55 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 56 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 57 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 58 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 59 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 60 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 61 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 62 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_1x1


    /* iter 63 */

    MAC_1x1_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_0_8_0 = ( globalReadOffsetAL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_0_0_9_0 = ( globalReadOffsetAL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_0_0_10_0 = ( globalReadOffsetAL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_0_0_11_0 = ( globalReadOffsetAL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_0_0_12_0 = ( globalReadOffsetAL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_0_0_13_0 = ( globalReadOffsetAL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_0_0_14_0 = ( globalReadOffsetAL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_0_0_15_0 = ( globalReadOffsetAL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);
  a_0_0_16_0 = ( globalReadOffsetAL_16_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_16_0 + 0);
  a_0_0_17_0 = ( globalReadOffsetAL_17_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_17_0 + 0);
  a_0_0_18_0 = ( globalReadOffsetAL_18_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_18_0 + 0);
  a_0_0_19_0 = ( globalReadOffsetAL_19_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_19_0 + 0);
  a_0_0_20_0 = ( globalReadOffsetAL_20_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_20_0 + 0);
  a_0_0_21_0 = ( globalReadOffsetAL_21_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_21_0 + 0);
  a_0_0_22_0 = ( globalReadOffsetAL_22_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_22_0 + 0);
  a_0_0_23_0 = ( globalReadOffsetAL_23_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_23_0 + 0);
  a_0_0_24_0 = ( globalReadOffsetAL_24_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_24_0 + 0);
  a_0_0_25_0 = ( globalReadOffsetAL_25_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_25_0 + 0);
  a_0_0_26_0 = ( globalReadOffsetAL_26_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_26_0 + 0);
  a_0_0_27_0 = ( globalReadOffsetAL_27_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_27_0 + 0);
  a_0_0_28_0 = ( globalReadOffsetAL_28_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_28_0 + 0);
  a_0_0_29_0 = ( globalReadOffsetAL_29_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_29_0 + 0);
  a_0_0_30_0 = ( globalReadOffsetAL_30_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_30_0 + 0);
  a_0_0_31_0 = ( globalReadOffsetAL_31_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_31_0 + 0);
  a_0_0_32_0 = ( globalReadOffsetAL_32_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_32_0 + 0);
  a_0_0_33_0 = ( globalReadOffsetAL_33_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_33_0 + 0);
  a_0_0_34_0 = ( globalReadOffsetAL_34_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_34_0 + 0);
  a_0_0_35_0 = ( globalReadOffsetAL_35_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_35_0 + 0);
  a_0_0_36_0 = ( globalReadOffsetAL_36_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_36_0 + 0);
  a_0_0_37_0 = ( globalReadOffsetAL_37_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_37_0 + 0);
  a_0_0_38_0 = ( globalReadOffsetAL_38_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_38_0 + 0);
  a_0_0_39_0 = ( globalReadOffsetAL_39_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_39_0 + 0);
  a_0_0_40_0 = ( globalReadOffsetAL_40_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_40_0 + 0);
  a_0_0_41_0 = ( globalReadOffsetAL_41_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_41_0 + 0);
  a_0_0_42_0 = ( globalReadOffsetAL_42_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_42_0 + 0);
  a_0_0_43_0 = ( globalReadOffsetAL_43_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_43_0 + 0);
  a_0_0_44_0 = ( globalReadOffsetAL_44_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_44_0 + 0);
  a_0_0_45_0 = ( globalReadOffsetAL_45_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_45_0 + 0);
  a_0_0_46_0 = ( globalReadOffsetAL_46_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_46_0 + 0);
  a_0_0_47_0 = ( globalReadOffsetAL_47_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_47_0 + 0);
  a_0_0_48_0 = ( globalReadOffsetAL_48_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_48_0 + 0);
  a_0_0_49_0 = ( globalReadOffsetAL_49_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_49_0 + 0);
  a_0_0_50_0 = ( globalReadOffsetAL_50_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_50_0 + 0);
  a_0_0_51_0 = ( globalReadOffsetAL_51_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_51_0 + 0);
  a_0_0_52_0 = ( globalReadOffsetAL_52_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_52_0 + 0);
  a_0_0_53_0 = ( globalReadOffsetAL_53_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_53_0 + 0);
  a_0_0_54_0 = ( globalReadOffsetAL_54_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_54_0 + 0);
  a_0_0_55_0 = ( globalReadOffsetAL_55_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_55_0 + 0);
  a_0_0_56_0 = ( globalReadOffsetAL_56_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_56_0 + 0);
  a_0_0_57_0 = ( globalReadOffsetAL_57_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_57_0 + 0);
  a_0_0_58_0 = ( globalReadOffsetAL_58_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_58_0 + 0);
  a_0_0_59_0 = ( globalReadOffsetAL_59_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_59_0 + 0);
  a_0_0_60_0 = ( globalReadOffsetAL_60_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_60_0 + 0);
  a_0_0_61_0 = ( globalReadOffsetAL_61_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_61_0 + 0);
  a_0_0_62_0 = ( globalReadOffsetAL_62_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_62_0 + 0);
  a_0_0_63_0 = ( globalReadOffsetAL_63_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_63_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);
  localWriteA_0_0_16_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_16_0);
  localWriteA_0_0_17_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_17_0);
  localWriteA_0_0_18_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_18_0);
  localWriteA_0_0_19_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_19_0);
  localWriteA_0_0_20_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_20_0);
  localWriteA_0_0_21_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_21_0);
  localWriteA_0_0_22_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_22_0);
  localWriteA_0_0_23_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_23_0);
  localWriteA_0_0_24_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_24_0);
  localWriteA_0_0_25_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_25_0);
  localWriteA_0_0_26_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_26_0);
  localWriteA_0_0_27_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_27_0);
  localWriteA_0_0_28_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_28_0);
  localWriteA_0_0_29_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_29_0);
  localWriteA_0_0_30_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_30_0);
  localWriteA_0_0_31_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_31_0);
  localWriteA_0_0_32_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_32_0);
  localWriteA_0_0_33_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_33_0);
  localWriteA_0_0_34_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_34_0);
  localWriteA_0_0_35_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_35_0);
  localWriteA_0_0_36_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_36_0);
  localWriteA_0_0_37_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_37_0);
  localWriteA_0_0_38_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_38_0);
  localWriteA_0_0_39_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_39_0);
  localWriteA_0_0_40_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_40_0);
  localWriteA_0_0_41_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_41_0);
  localWriteA_0_0_42_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_42_0);
  localWriteA_0_0_43_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_43_0);
  localWriteA_0_0_44_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_44_0);
  localWriteA_0_0_45_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_45_0);
  localWriteA_0_0_46_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_46_0);
  localWriteA_0_0_47_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_47_0);
  localWriteA_0_0_48_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_48_0);
  localWriteA_0_0_49_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_49_0);
  localWriteA_0_0_50_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_50_0);
  localWriteA_0_0_51_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_51_0);
  localWriteA_0_0_52_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_52_0);
  localWriteA_0_0_53_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_53_0);
  localWriteA_0_0_54_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_54_0);
  localWriteA_0_0_55_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_55_0);
  localWriteA_0_0_56_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_56_0);
  localWriteA_0_0_57_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_57_0);
  localWriteA_0_0_58_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_58_0);
  localWriteA_0_0_59_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_59_0);
  localWriteA_0_0_60_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_60_0);
  localWriteA_0_0_61_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_61_0);
  localWriteA_0_0_62_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_62_0);
  localWriteA_0_0_63_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_63_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
  *(localWriteA_0_0_16_0 + 0) = a_0_0_16_0;
  *(localWriteA_0_0_17_0 + 0) = a_0_0_17_0;
  *(localWriteA_0_0_18_0 + 0) = a_0_0_18_0;
  *(localWriteA_0_0_19_0 + 0) = a_0_0_19_0;
  *(localWriteA_0_0_20_0 + 0) = a_0_0_20_0;
  *(localWriteA_0_0_21_0 + 0) = a_0_0_21_0;
  *(localWriteA_0_0_22_0 + 0) = a_0_0_22_0;
  *(localWriteA_0_0_23_0 + 0) = a_0_0_23_0;
  *(localWriteA_0_0_24_0 + 0) = a_0_0_24_0;
  *(localWriteA_0_0_25_0 + 0) = a_0_0_25_0;
  *(localWriteA_0_0_26_0 + 0) = a_0_0_26_0;
  *(localWriteA_0_0_27_0 + 0) = a_0_0_27_0;
  *(localWriteA_0_0_28_0 + 0) = a_0_0_28_0;
  *(localWriteA_0_0_29_0 + 0) = a_0_0_29_0;
  *(localWriteA_0_0_30_0 + 0) = a_0_0_30_0;
  *(localWriteA_0_0_31_0 + 0) = a_0_0_31_0;
  *(localWriteA_0_0_32_0 + 0) = a_0_0_32_0;
  *(localWriteA_0_0_33_0 + 0) = a_0_0_33_0;
  *(localWriteA_0_0_34_0 + 0) = a_0_0_34_0;
  *(localWriteA_0_0_35_0 + 0) = a_0_0_35_0;
  *(localWriteA_0_0_36_0 + 0) = a_0_0_36_0;
  *(localWriteA_0_0_37_0 + 0) = a_0_0_37_0;
  *(localWriteA_0_0_38_0 + 0) = a_0_0_38_0;
  *(localWriteA_0_0_39_0 + 0) = a_0_0_39_0;
  *(localWriteA_0_0_40_0 + 0) = a_0_0_40_0;
  *(localWriteA_0_0_41_0 + 0) = a_0_0_41_0;
  *(localWriteA_0_0_42_0 + 0) = a_0_0_42_0;
  *(localWriteA_0_0_43_0 + 0) = a_0_0_43_0;
  *(localWriteA_0_0_44_0 + 0) = a_0_0_44_0;
  *(localWriteA_0_0_45_0 + 0) = a_0_0_45_0;
  *(localWriteA_0_0_46_0 + 0) = a_0_0_46_0;
  *(localWriteA_0_0_47_0 + 0) = a_0_0_47_0;
  *(localWriteA_0_0_48_0 + 0) = a_0_0_48_0;
  *(localWriteA_0_0_49_0 + 0) = a_0_0_49_0;
  *(localWriteA_0_0_50_0 + 0) = a_0_0_50_0;
  *(localWriteA_0_0_51_0 + 0) = a_0_0_51_0;
  *(localWriteA_0_0_52_0 + 0) = a_0_0_52_0;
  *(localWriteA_0_0_53_0 + 0) = a_0_0_53_0;
  *(localWriteA_0_0_54_0 + 0) = a_0_0_54_0;
  *(localWriteA_0_0_55_0 + 0) = a_0_0_55_0;
  *(localWriteA_0_0_56_0 + 0) = a_0_0_56_0;
  *(localWriteA_0_0_57_0 + 0) = a_0_0_57_0;
  *(localWriteA_0_0_58_0 + 0) = a_0_0_58_0;
  *(localWriteA_0_0_59_0 + 0) = a_0_0_59_0;
  *(localWriteA_0_0_60_0 + 0) = a_0_0_60_0;
  *(localWriteA_0_0_61_0 + 0) = a_0_0_61_0;
  *(localWriteA_0_0_62_0 + 0) = a_0_0_62_0;
  *(localWriteA_0_0_63_0 + 0) = a_0_0_63_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_1x1

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_1x1
#undef MAC_1x1_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 64
#define SG1J 1
#define TT0I 1
#define TT1J 1
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 64
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 64
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 4096
#define LDS_NUM_ELEMENTS 4160

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 1x1 micro-tile */
#define MAC_1x1\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \

#define MAC_1x1_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT64x1x64_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU0_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT1_1_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG64_1_1_WGM1(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE a_0_0_16_0;
  DATA_TYPE a_0_0_17_0;
  DATA_TYPE a_0_0_18_0;
  DATA_TYPE a_0_0_19_0;
  DATA_TYPE a_0_0_20_0;
  DATA_TYPE a_0_0_21_0;
  DATA_TYPE a_0_0_22_0;
  DATA_TYPE a_0_0_23_0;
  DATA_TYPE a_0_0_24_0;
  DATA_TYPE a_0_0_25_0;
  DATA_TYPE a_0_0_26_0;
  DATA_TYPE a_0_0_27_0;
  DATA_TYPE a_0_0_28_0;
  DATA_TYPE a_0_0_29_0;
  DATA_TYPE a_0_0_30_0;
  DATA_TYPE a_0_0_31_0;
  DATA_TYPE a_0_0_32_0;
  DATA_TYPE a_0_0_33_0;
  DATA_TYPE a_0_0_34_0;
  DATA_TYPE a_0_0_35_0;
  DATA_TYPE a_0_0_36_0;
  DATA_TYPE a_0_0_37_0;
  DATA_TYPE a_0_0_38_0;
  DATA_TYPE a_0_0_39_0;
  DATA_TYPE a_0_0_40_0;
  DATA_TYPE a_0_0_41_0;
  DATA_TYPE a_0_0_42_0;
  DATA_TYPE a_0_0_43_0;
  DATA_TYPE a_0_0_44_0;
  DATA_TYPE a_0_0_45_0;
  DATA_TYPE a_0_0_46_0;
  DATA_TYPE a_0_0_47_0;
  DATA_TYPE a_0_0_48_0;
  DATA_TYPE a_0_0_49_0;
  DATA_TYPE a_0_0_50_0;
  DATA_TYPE a_0_0_51_0;
  DATA_TYPE a_0_0_52_0;
  DATA_TYPE a_0_0_53_0;
  DATA_TYPE a_0_0_54_0;
  DATA_TYPE a_0_0_55_0;
  DATA_TYPE a_0_0_56_0;
  DATA_TYPE a_0_0_57_0;
  DATA_TYPE a_0_0_58_0;
  DATA_TYPE a_0_0_59_0;
  DATA_TYPE a_0_0_60_0;
  DATA_TYPE a_0_0_61_0;
  DATA_TYPE a_0_0_62_0;
  DATA_TYPE a_0_0_63_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;
  unsigned int globalReadOffsetAL_8_0 = globalReadOffsetAL + 0 + 8*LSPA;
  unsigned int globalReadOffsetAL_9_0 = globalReadOffsetAL + 0 + 9*LSPA;
  unsigned int globalReadOffsetAL_10_0 = globalReadOffsetAL + 0 + 10*LSPA;
  unsigned int globalReadOffsetAL_11_0 = globalReadOffsetAL + 0 + 11*LSPA;
  unsigned int globalReadOffsetAL_12_0 = globalReadOffsetAL + 0 + 12*LSPA;
  unsigned int globalReadOffsetAL_13_0 = globalReadOffsetAL + 0 + 13*LSPA;
  unsigned int globalReadOffsetAL_14_0 = globalReadOffsetAL + 0 + 14*LSPA;
  unsigned int globalReadOffsetAL_15_0 = globalReadOffsetAL + 0 + 15*LSPA;
  unsigned int globalReadOffsetAL_16_0 = globalReadOffsetAL + 0 + 16*LSPA;
  unsigned int globalReadOffsetAL_17_0 = globalReadOffsetAL + 0 + 17*LSPA;
  unsigned int globalReadOffsetAL_18_0 = globalReadOffsetAL + 0 + 18*LSPA;
  unsigned int globalReadOffsetAL_19_0 = globalReadOffsetAL + 0 + 19*LSPA;
  unsigned int globalReadOffsetAL_20_0 = globalReadOffsetAL + 0 + 20*LSPA;
  unsigned int globalReadOffsetAL_21_0 = globalReadOffsetAL + 0 + 21*LSPA;
  unsigned int globalReadOffsetAL_22_0 = globalReadOffsetAL + 0 + 22*LSPA;
  unsigned int globalReadOffsetAL_23_0 = globalReadOffsetAL + 0 + 23*LSPA;
  unsigned int globalReadOffsetAL_24_0 = globalReadOffsetAL + 0 + 24*LSPA;
  unsigned int globalReadOffsetAL_25_0 = globalReadOffsetAL + 0 + 25*LSPA;
  unsigned int globalReadOffsetAL_26_0 = globalReadOffsetAL + 0 + 26*LSPA;
  unsigned int globalReadOffsetAL_27_0 = globalReadOffsetAL + 0 + 27*LSPA;
  unsigned int globalReadOffsetAL_28_0 = globalReadOffsetAL + 0 + 28*LSPA;
  unsigned int globalReadOffsetAL_29_0 = globalReadOffsetAL + 0 + 29*LSPA;
  unsigned int globalReadOffsetAL_30_0 = globalReadOffsetAL + 0 + 30*LSPA;
  unsigned int globalReadOffsetAL_31_0 = globalReadOffsetAL + 0 + 31*LSPA;
  unsigned int globalReadOffsetAL_32_0 = globalReadOffsetAL + 0 + 32*LSPA;
  unsigned int globalReadOffsetAL_33_0 = globalReadOffsetAL + 0 + 33*LSPA;
  unsigned int globalReadOffsetAL_34_0 = globalReadOffsetAL + 0 + 34*LSPA;
  unsigned int globalReadOffsetAL_35_0 = globalReadOffsetAL + 0 + 35*LSPA;
  unsigned int globalReadOffsetAL_36_0 = globalReadOffsetAL + 0 + 36*LSPA;
  unsigned int globalReadOffsetAL_37_0 = globalReadOffsetAL + 0 + 37*LSPA;
  unsigned int globalReadOffsetAL_38_0 = globalReadOffsetAL + 0 + 38*LSPA;
  unsigned int globalReadOffsetAL_39_0 = globalReadOffsetAL + 0 + 39*LSPA;
  unsigned int globalReadOffsetAL_40_0 = globalReadOffsetAL + 0 + 40*LSPA;
  unsigned int globalReadOffsetAL_41_0 = globalReadOffsetAL + 0 + 41*LSPA;
  unsigned int globalReadOffsetAL_42_0 = globalReadOffsetAL + 0 + 42*LSPA;
  unsigned int globalReadOffsetAL_43_0 = globalReadOffsetAL + 0 + 43*LSPA;
  unsigned int globalReadOffsetAL_44_0 = globalReadOffsetAL + 0 + 44*LSPA;
  unsigned int globalReadOffsetAL_45_0 = globalReadOffsetAL + 0 + 45*LSPA;
  unsigned int globalReadOffsetAL_46_0 = globalReadOffsetAL + 0 + 46*LSPA;
  unsigned int globalReadOffsetAL_47_0 = globalReadOffsetAL + 0 + 47*LSPA;
  unsigned int globalReadOffsetAL_48_0 = globalReadOffsetAL + 0 + 48*LSPA;
  unsigned int globalReadOffsetAL_49_0 = globalReadOffsetAL + 0 + 49*LSPA;
  unsigned int globalReadOffsetAL_50_0 = globalReadOffsetAL + 0 + 50*LSPA;
  unsigned int globalReadOffsetAL_51_0 = globalReadOffsetAL + 0 + 51*LSPA;
  unsigned int globalReadOffsetAL_52_0 = globalReadOffsetAL + 0 + 52*LSPA;
  unsigned int globalReadOffsetAL_53_0 = globalReadOffsetAL + 0 + 53*LSPA;
  unsigned int globalReadOffsetAL_54_0 = globalReadOffsetAL + 0 + 54*LSPA;
  unsigned int globalReadOffsetAL_55_0 = globalReadOffsetAL + 0 + 55*LSPA;
  unsigned int globalReadOffsetAL_56_0 = globalReadOffsetAL + 0 + 56*LSPA;
  unsigned int globalReadOffsetAL_57_0 = globalReadOffsetAL + 0 + 57*LSPA;
  unsigned int globalReadOffsetAL_58_0 = globalReadOffsetAL + 0 + 58*LSPA;
  unsigned int globalReadOffsetAL_59_0 = globalReadOffsetAL + 0 + 59*LSPA;
  unsigned int globalReadOffsetAL_60_0 = globalReadOffsetAL + 0 + 60*LSPA;
  unsigned int globalReadOffsetAL_61_0 = globalReadOffsetAL + 0 + 61*LSPA;
  unsigned int globalReadOffsetAL_62_0 = globalReadOffsetAL + 0 + 62*LSPA;
  unsigned int globalReadOffsetAL_63_0 = globalReadOffsetAL + 0 + 63*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_15_0), wgK );
  int64_t globalReadOffsetA_0_0_16_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_16_0), wgK );
  int64_t globalReadOffsetA_0_0_17_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_17_0), wgK );
  int64_t globalReadOffsetA_0_0_18_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_18_0), wgK );
  int64_t globalReadOffsetA_0_0_19_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_19_0), wgK );
  int64_t globalReadOffsetA_0_0_20_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_20_0), wgK );
  int64_t globalReadOffsetA_0_0_21_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_21_0), wgK );
  int64_t globalReadOffsetA_0_0_22_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_22_0), wgK );
  int64_t globalReadOffsetA_0_0_23_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_23_0), wgK );
  int64_t globalReadOffsetA_0_0_24_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_24_0), wgK );
  int64_t globalReadOffsetA_0_0_25_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_25_0), wgK );
  int64_t globalReadOffsetA_0_0_26_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_26_0), wgK );
  int64_t globalReadOffsetA_0_0_27_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_27_0), wgK );
  int64_t globalReadOffsetA_0_0_28_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_28_0), wgK );
  int64_t globalReadOffsetA_0_0_29_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_29_0), wgK );
  int64_t globalReadOffsetA_0_0_30_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_30_0), wgK );
  int64_t globalReadOffsetA_0_0_31_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_31_0), wgK );
  int64_t globalReadOffsetA_0_0_32_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_32_0), wgK );
  int64_t globalReadOffsetA_0_0_33_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_33_0), wgK );
  int64_t globalReadOffsetA_0_0_34_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_34_0), wgK );
  int64_t globalReadOffsetA_0_0_35_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_35_0), wgK );
  int64_t globalReadOffsetA_0_0_36_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_36_0), wgK );
  int64_t globalReadOffsetA_0_0_37_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_37_0), wgK );
  int64_t globalReadOffsetA_0_0_38_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_38_0), wgK );
  int64_t globalReadOffsetA_0_0_39_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_39_0), wgK );
  int64_t globalReadOffsetA_0_0_40_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_40_0), wgK );
  int64_t globalReadOffsetA_0_0_41_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_41_0), wgK );
  int64_t globalReadOffsetA_0_0_42_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_42_0), wgK );
  int64_t globalReadOffsetA_0_0_43_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_43_0), wgK );
  int64_t globalReadOffsetA_0_0_44_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_44_0), wgK );
  int64_t globalReadOffsetA_0_0_45_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_45_0), wgK );
  int64_t globalReadOffsetA_0_0_46_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_46_0), wgK );
  int64_t globalReadOffsetA_0_0_47_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_47_0), wgK );
  int64_t globalReadOffsetA_0_0_48_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_48_0), wgK );
  int64_t globalReadOffsetA_0_0_49_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_49_0), wgK );
  int64_t globalReadOffsetA_0_0_50_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_50_0), wgK );
  int64_t globalReadOffsetA_0_0_51_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_51_0), wgK );
  int64_t globalReadOffsetA_0_0_52_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_52_0), wgK );
  int64_t globalReadOffsetA_0_0_53_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_53_0), wgK );
  int64_t globalReadOffsetA_0_0_54_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_54_0), wgK );
  int64_t globalReadOffsetA_0_0_55_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_55_0), wgK );
  int64_t globalReadOffsetA_0_0_56_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_56_0), wgK );
  int64_t globalReadOffsetA_0_0_57_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_57_0), wgK );
  int64_t globalReadOffsetA_0_0_58_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_58_0), wgK );
  int64_t globalReadOffsetA_0_0_59_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_59_0), wgK );
  int64_t globalReadOffsetA_0_0_60_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_60_0), wgK );
  int64_t globalReadOffsetA_0_0_61_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_61_0), wgK );
  int64_t globalReadOffsetA_0_0_62_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_62_0), wgK );
  int64_t globalReadOffsetA_0_0_63_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_63_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;
  DATA_TYPE const *globalReadA_0_0_16_0 = A + globalReadOffsetA_0_0_16_0;
  DATA_TYPE const *globalReadA_0_0_17_0 = A + globalReadOffsetA_0_0_17_0;
  DATA_TYPE const *globalReadA_0_0_18_0 = A + globalReadOffsetA_0_0_18_0;
  DATA_TYPE const *globalReadA_0_0_19_0 = A + globalReadOffsetA_0_0_19_0;
  DATA_TYPE const *globalReadA_0_0_20_0 = A + globalReadOffsetA_0_0_20_0;
  DATA_TYPE const *globalReadA_0_0_21_0 = A + globalReadOffsetA_0_0_21_0;
  DATA_TYPE const *globalReadA_0_0_22_0 = A + globalReadOffsetA_0_0_22_0;
  DATA_TYPE const *globalReadA_0_0_23_0 = A + globalReadOffsetA_0_0_23_0;
  DATA_TYPE const *globalReadA_0_0_24_0 = A + globalReadOffsetA_0_0_24_0;
  DATA_TYPE const *globalReadA_0_0_25_0 = A + globalReadOffsetA_0_0_25_0;
  DATA_TYPE const *globalReadA_0_0_26_0 = A + globalReadOffsetA_0_0_26_0;
  DATA_TYPE const *globalReadA_0_0_27_0 = A + globalReadOffsetA_0_0_27_0;
  DATA_TYPE const *globalReadA_0_0_28_0 = A + globalReadOffsetA_0_0_28_0;
  DATA_TYPE const *globalReadA_0_0_29_0 = A + globalReadOffsetA_0_0_29_0;
  DATA_TYPE const *globalReadA_0_0_30_0 = A + globalReadOffsetA_0_0_30_0;
  DATA_TYPE const *globalReadA_0_0_31_0 = A + globalReadOffsetA_0_0_31_0;
  DATA_TYPE const *globalReadA_0_0_32_0 = A + globalReadOffsetA_0_0_32_0;
  DATA_TYPE const *globalReadA_0_0_33_0 = A + globalReadOffsetA_0_0_33_0;
  DATA_TYPE const *globalReadA_0_0_34_0 = A + globalReadOffsetA_0_0_34_0;
  DATA_TYPE const *globalReadA_0_0_35_0 = A + globalReadOffsetA_0_0_35_0;
  DATA_TYPE const *globalReadA_0_0_36_0 = A + globalReadOffsetA_0_0_36_0;
  DATA_TYPE const *globalReadA_0_0_37_0 = A + globalReadOffsetA_0_0_37_0;
  DATA_TYPE const *globalReadA_0_0_38_0 = A + globalReadOffsetA_0_0_38_0;
  DATA_TYPE const *globalReadA_0_0_39_0 = A + globalReadOffsetA_0_0_39_0;
  DATA_TYPE const *globalReadA_0_0_40_0 = A + globalReadOffsetA_0_0_40_0;
  DATA_TYPE const *globalReadA_0_0_41_0 = A + globalReadOffsetA_0_0_41_0;
  DATA_TYPE const *globalReadA_0_0_42_0 = A + globalReadOffsetA_0_0_42_0;
  DATA_TYPE const *globalReadA_0_0_43_0 = A + globalReadOffsetA_0_0_43_0;
  DATA_TYPE const *globalReadA_0_0_44_0 = A + globalReadOffsetA_0_0_44_0;
  DATA_TYPE const *globalReadA_0_0_45_0 = A + globalReadOffsetA_0_0_45_0;
  DATA_TYPE const *globalReadA_0_0_46_0 = A + globalReadOffsetA_0_0_46_0;
  DATA_TYPE const *globalReadA_0_0_47_0 = A + globalReadOffsetA_0_0_47_0;
  DATA_TYPE const *globalReadA_0_0_48_0 = A + globalReadOffsetA_0_0_48_0;
  DATA_TYPE const *globalReadA_0_0_49_0 = A + globalReadOffsetA_0_0_49_0;
  DATA_TYPE const *globalReadA_0_0_50_0 = A + globalReadOffsetA_0_0_50_0;
  DATA_TYPE const *globalReadA_0_0_51_0 = A + globalReadOffsetA_0_0_51_0;
  DATA_TYPE const *globalReadA_0_0_52_0 = A + globalReadOffsetA_0_0_52_0;
  DATA_TYPE const *globalReadA_0_0_53_0 = A + globalReadOffsetA_0_0_53_0;
  DATA_TYPE const *globalReadA_0_0_54_0 = A + globalReadOffsetA_0_0_54_0;
  DATA_TYPE const *globalReadA_0_0_55_0 = A + globalReadOffsetA_0_0_55_0;
  DATA_TYPE const *globalReadA_0_0_56_0 = A + globalReadOffsetA_0_0_56_0;
  DATA_TYPE const *globalReadA_0_0_57_0 = A + globalReadOffsetA_0_0_57_0;
  DATA_TYPE const *globalReadA_0_0_58_0 = A + globalReadOffsetA_0_0_58_0;
  DATA_TYPE const *globalReadA_0_0_59_0 = A + globalReadOffsetA_0_0_59_0;
  DATA_TYPE const *globalReadA_0_0_60_0 = A + globalReadOffsetA_0_0_60_0;
  DATA_TYPE const *globalReadA_0_0_61_0 = A + globalReadOffsetA_0_0_61_0;
  DATA_TYPE const *globalReadA_0_0_62_0 = A + globalReadOffsetA_0_0_62_0;
  DATA_TYPE const *globalReadA_0_0_63_0 = A + globalReadOffsetA_0_0_63_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 8*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 9*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 10*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 11*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 12*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 13*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 14*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 15*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_16_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 16*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_17_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 17*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_18_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 18*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_19_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 19*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_20_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 20*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_21_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 21*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_22_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 22*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_23_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 23*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_24_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 24*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_25_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 25*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_26_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 26*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_27_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 27*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_28_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 28*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_29_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 29*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_30_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 30*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_31_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 31*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_32_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 32*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_33_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 33*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_34_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 34*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_35_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 35*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_36_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 36*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_37_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 37*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_38_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 38*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_39_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 39*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_40_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 40*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_41_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 41*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_42_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 42*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_43_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 43*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_44_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 44*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_45_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 45*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_46_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 46*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_47_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 47*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_48_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 48*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_49_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 49*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_50_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 50*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_51_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 51*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_52_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 52*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_53_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 53*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_54_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 54*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_55_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 55*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_56_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 56*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_57_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 57*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_58_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 58*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_59_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 59*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_60_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 60*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_61_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 61*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_62_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 62*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_63_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 63*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_0_0_15_0;
  DATA_TYPE *localWriteA_0_0_16_0;
  DATA_TYPE *localWriteA_0_0_17_0;
  DATA_TYPE *localWriteA_0_0_18_0;
  DATA_TYPE *localWriteA_0_0_19_0;
  DATA_TYPE *localWriteA_0_0_20_0;
  DATA_TYPE *localWriteA_0_0_21_0;
  DATA_TYPE *localWriteA_0_0_22_0;
  DATA_TYPE *localWriteA_0_0_23_0;
  DATA_TYPE *localWriteA_0_0_24_0;
  DATA_TYPE *localWriteA_0_0_25_0;
  DATA_TYPE *localWriteA_0_0_26_0;
  DATA_TYPE *localWriteA_0_0_27_0;
  DATA_TYPE *localWriteA_0_0_28_0;
  DATA_TYPE *localWriteA_0_0_29_0;
  DATA_TYPE *localWriteA_0_0_30_0;
  DATA_TYPE *localWriteA_0_0_31_0;
  DATA_TYPE *localWriteA_0_0_32_0;
  DATA_TYPE *localWriteA_0_0_33_0;
  DATA_TYPE *localWriteA_0_0_34_0;
  DATA_TYPE *localWriteA_0_0_35_0;
  DATA_TYPE *localWriteA_0_0_36_0;
  DATA_TYPE *localWriteA_0_0_37_0;
  DATA_TYPE *localWriteA_0_0_38_0;
  DATA_TYPE *localWriteA_0_0_39_0;
  DATA_TYPE *localWriteA_0_0_40_0;
  DATA_TYPE *localWriteA_0_0_41_0;
  DATA_TYPE *localWriteA_0_0_42_0;
  DATA_TYPE *localWriteA_0_0_43_0;
  DATA_TYPE *localWriteA_0_0_44_0;
  DATA_TYPE *localWriteA_0_0_45_0;
  DATA_TYPE *localWriteA_0_0_46_0;
  DATA_TYPE *localWriteA_0_0_47_0;
  DATA_TYPE *localWriteA_0_0_48_0;
  DATA_TYPE *localWriteA_0_0_49_0;
  DATA_TYPE *localWriteA_0_0_50_0;
  DATA_TYPE *localWriteA_0_0_51_0;
  DATA_TYPE *localWriteA_0_0_52_0;
  DATA_TYPE *localWriteA_0_0_53_0;
  DATA_TYPE *localWriteA_0_0_54_0;
  DATA_TYPE *localWriteA_0_0_55_0;
  DATA_TYPE *localWriteA_0_0_56_0;
  DATA_TYPE *localWriteA_0_0_57_0;
  DATA_TYPE *localWriteA_0_0_58_0;
  DATA_TYPE *localWriteA_0_0_59_0;
  DATA_TYPE *localWriteA_0_0_60_0;
  DATA_TYPE *localWriteA_0_0_61_0;
  DATA_TYPE *localWriteA_0_0_62_0;
  DATA_TYPE *localWriteA_0_0_63_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);
  localWriteA_0_0_16_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_16_0);
  localWriteA_0_0_17_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_17_0);
  localWriteA_0_0_18_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_18_0);
  localWriteA_0_0_19_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_19_0);
  localWriteA_0_0_20_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_20_0);
  localWriteA_0_0_21_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_21_0);
  localWriteA_0_0_22_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_22_0);
  localWriteA_0_0_23_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_23_0);
  localWriteA_0_0_24_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_24_0);
  localWriteA_0_0_25_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_25_0);
  localWriteA_0_0_26_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_26_0);
  localWriteA_0_0_27_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_27_0);
  localWriteA_0_0_28_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_28_0);
  localWriteA_0_0_29_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_29_0);
  localWriteA_0_0_30_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_30_0);
  localWriteA_0_0_31_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_31_0);
  localWriteA_0_0_32_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_32_0);
  localWriteA_0_0_33_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_33_0);
  localWriteA_0_0_34_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_34_0);
  localWriteA_0_0_35_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_35_0);
  localWriteA_0_0_36_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_36_0);
  localWriteA_0_0_37_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_37_0);
  localWriteA_0_0_38_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_38_0);
  localWriteA_0_0_39_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_39_0);
  localWriteA_0_0_40_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_40_0);
  localWriteA_0_0_41_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_41_0);
  localWriteA_0_0_42_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_42_0);
  localWriteA_0_0_43_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_43_0);
  localWriteA_0_0_44_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_44_0);
  localWriteA_0_0_45_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_45_0);
  localWriteA_0_0_46_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_46_0);
  localWriteA_0_0_47_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_47_0);
  localWriteA_0_0_48_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_48_0);
  localWriteA_0_0_49_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_49_0);
  localWriteA_0_0_50_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_50_0);
  localWriteA_0_0_51_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_51_0);
  localWriteA_0_0_52_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_52_0);
  localWriteA_0_0_53_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_53_0);
  localWriteA_0_0_54_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_54_0);
  localWriteA_0_0_55_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_55_0);
  localWriteA_0_0_56_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_56_0);
  localWriteA_0_0_57_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_57_0);
  localWriteA_0_0_58_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_58_0);
  localWriteA_0_0_59_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_59_0);
  localWriteA_0_0_60_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_60_0);
  localWriteA_0_0_61_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_61_0);
  localWriteA_0_0_62_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_62_0);
  localWriteA_0_0_63_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_63_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);
    a_0_0_16_0 = *(globalReadA_0_0_16_0 + 0);
    a_0_0_17_0 = *(globalReadA_0_0_17_0 + 0);
    a_0_0_18_0 = *(globalReadA_0_0_18_0 + 0);
    a_0_0_19_0 = *(globalReadA_0_0_19_0 + 0);
    a_0_0_20_0 = *(globalReadA_0_0_20_0 + 0);
    a_0_0_21_0 = *(globalReadA_0_0_21_0 + 0);
    a_0_0_22_0 = *(globalReadA_0_0_22_0 + 0);
    a_0_0_23_0 = *(globalReadA_0_0_23_0 + 0);
    a_0_0_24_0 = *(globalReadA_0_0_24_0 + 0);
    a_0_0_25_0 = *(globalReadA_0_0_25_0 + 0);
    a_0_0_26_0 = *(globalReadA_0_0_26_0 + 0);
    a_0_0_27_0 = *(globalReadA_0_0_27_0 + 0);
    a_0_0_28_0 = *(globalReadA_0_0_28_0 + 0);
    a_0_0_29_0 = *(globalReadA_0_0_29_0 + 0);
    a_0_0_30_0 = *(globalReadA_0_0_30_0 + 0);
    a_0_0_31_0 = *(globalReadA_0_0_31_0 + 0);
    a_0_0_32_0 = *(globalReadA_0_0_32_0 + 0);
    a_0_0_33_0 = *(globalReadA_0_0_33_0 + 0);
    a_0_0_34_0 = *(globalReadA_0_0_34_0 + 0);
    a_0_0_35_0 = *(globalReadA_0_0_35_0 + 0);
    a_0_0_36_0 = *(globalReadA_0_0_36_0 + 0);
    a_0_0_37_0 = *(globalReadA_0_0_37_0 + 0);
    a_0_0_38_0 = *(globalReadA_0_0_38_0 + 0);
    a_0_0_39_0 = *(globalReadA_0_0_39_0 + 0);
    a_0_0_40_0 = *(globalReadA_0_0_40_0 + 0);
    a_0_0_41_0 = *(globalReadA_0_0_41_0 + 0);
    a_0_0_42_0 = *(globalReadA_0_0_42_0 + 0);
    a_0_0_43_0 = *(globalReadA_0_0_43_0 + 0);
    a_0_0_44_0 = *(globalReadA_0_0_44_0 + 0);
    a_0_0_45_0 = *(globalReadA_0_0_45_0 + 0);
    a_0_0_46_0 = *(globalReadA_0_0_46_0 + 0);
    a_0_0_47_0 = *(globalReadA_0_0_47_0 + 0);
    a_0_0_48_0 = *(globalReadA_0_0_48_0 + 0);
    a_0_0_49_0 = *(globalReadA_0_0_49_0 + 0);
    a_0_0_50_0 = *(globalReadA_0_0_50_0 + 0);
    a_0_0_51_0 = *(globalReadA_0_0_51_0 + 0);
    a_0_0_52_0 = *(globalReadA_0_0_52_0 + 0);
    a_0_0_53_0 = *(globalReadA_0_0_53_0 + 0);
    a_0_0_54_0 = *(globalReadA_0_0_54_0 + 0);
    a_0_0_55_0 = *(globalReadA_0_0_55_0 + 0);
    a_0_0_56_0 = *(globalReadA_0_0_56_0 + 0);
    a_0_0_57_0 = *(globalReadA_0_0_57_0 + 0);
    a_0_0_58_0 = *(globalReadA_0_0_58_0 + 0);
    a_0_0_59_0 = *(globalReadA_0_0_59_0 + 0);
    a_0_0_60_0 = *(globalReadA_0_0_60_0 + 0);
    a_0_0_61_0 = *(globalReadA_0_0_61_0 + 0);
    a_0_0_62_0 = *(globalReadA_0_0_62_0 + 0);
    a_0_0_63_0 = *(globalReadA_0_0_63_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    globalReadA_0_0_16_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_16_0) + 1*globalReadIncAL);
    globalReadA_0_0_17_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_17_0) + 1*globalReadIncAL);
    globalReadA_0_0_18_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_18_0) + 1*globalReadIncAL);
    globalReadA_0_0_19_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_19_0) + 1*globalReadIncAL);
    globalReadA_0_0_20_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_20_0) + 1*globalReadIncAL);
    globalReadA_0_0_21_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_21_0) + 1*globalReadIncAL);
    globalReadA_0_0_22_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_22_0) + 1*globalReadIncAL);
    globalReadA_0_0_23_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_23_0) + 1*globalReadIncAL);
    globalReadA_0_0_24_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_24_0) + 1*globalReadIncAL);
    globalReadA_0_0_25_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_25_0) + 1*globalReadIncAL);
    globalReadA_0_0_26_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_26_0) + 1*globalReadIncAL);
    globalReadA_0_0_27_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_27_0) + 1*globalReadIncAL);
    globalReadA_0_0_28_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_28_0) + 1*globalReadIncAL);
    globalReadA_0_0_29_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_29_0) + 1*globalReadIncAL);
    globalReadA_0_0_30_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_30_0) + 1*globalReadIncAL);
    globalReadA_0_0_31_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_31_0) + 1*globalReadIncAL);
    globalReadA_0_0_32_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_32_0) + 1*globalReadIncAL);
    globalReadA_0_0_33_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_33_0) + 1*globalReadIncAL);
    globalReadA_0_0_34_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_34_0) + 1*globalReadIncAL);
    globalReadA_0_0_35_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_35_0) + 1*globalReadIncAL);
    globalReadA_0_0_36_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_36_0) + 1*globalReadIncAL);
    globalReadA_0_0_37_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_37_0) + 1*globalReadIncAL);
    globalReadA_0_0_38_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_38_0) + 1*globalReadIncAL);
    globalReadA_0_0_39_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_39_0) + 1*globalReadIncAL);
    globalReadA_0_0_40_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_40_0) + 1*globalReadIncAL);
    globalReadA_0_0_41_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_41_0) + 1*globalReadIncAL);
    globalReadA_0_0_42_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_42_0) + 1*globalReadIncAL);
    globalReadA_0_0_43_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_43_0) + 1*globalReadIncAL);
    globalReadA_0_0_44_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_44_0) + 1*globalReadIncAL);
    globalReadA_0_0_45_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_45_0) + 1*globalReadIncAL);
    globalReadA_0_0_46_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_46_0) + 1*globalReadIncAL);
    globalReadA_0_0_47_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_47_0) + 1*globalReadIncAL);
    globalReadA_0_0_48_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_48_0) + 1*globalReadIncAL);
    globalReadA_0_0_49_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_49_0) + 1*globalReadIncAL);
    globalReadA_0_0_50_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_50_0) + 1*globalReadIncAL);
    globalReadA_0_0_51_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_51_0) + 1*globalReadIncAL);
    globalReadA_0_0_52_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_52_0) + 1*globalReadIncAL);
    globalReadA_0_0_53_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_53_0) + 1*globalReadIncAL);
    globalReadA_0_0_54_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_54_0) + 1*globalReadIncAL);
    globalReadA_0_0_55_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_55_0) + 1*globalReadIncAL);
    globalReadA_0_0_56_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_56_0) + 1*globalReadIncAL);
    globalReadA_0_0_57_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_57_0) + 1*globalReadIncAL);
    globalReadA_0_0_58_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_58_0) + 1*globalReadIncAL);
    globalReadA_0_0_59_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_59_0) + 1*globalReadIncAL);
    globalReadA_0_0_60_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_60_0) + 1*globalReadIncAL);
    globalReadA_0_0_61_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_61_0) + 1*globalReadIncAL);
    globalReadA_0_0_62_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_62_0) + 1*globalReadIncAL);
    globalReadA_0_0_63_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_63_0) + 1*globalReadIncAL);

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
    *(localWriteA_0_0_16_0 + 0) = a_0_0_16_0;
    *(localWriteA_0_0_17_0 + 0) = a_0_0_17_0;
    *(localWriteA_0_0_18_0 + 0) = a_0_0_18_0;
    *(localWriteA_0_0_19_0 + 0) = a_0_0_19_0;
    *(localWriteA_0_0_20_0 + 0) = a_0_0_20_0;
    *(localWriteA_0_0_21_0 + 0) = a_0_0_21_0;
    *(localWriteA_0_0_22_0 + 0) = a_0_0_22_0;
    *(localWriteA_0_0_23_0 + 0) = a_0_0_23_0;
    *(localWriteA_0_0_24_0 + 0) = a_0_0_24_0;
    *(localWriteA_0_0_25_0 + 0) = a_0_0_25_0;
    *(localWriteA_0_0_26_0 + 0) = a_0_0_26_0;
    *(localWriteA_0_0_27_0 + 0) = a_0_0_27_0;
    *(localWriteA_0_0_28_0 + 0) = a_0_0_28_0;
    *(localWriteA_0_0_29_0 + 0) = a_0_0_29_0;
    *(localWriteA_0_0_30_0 + 0) = a_0_0_30_0;
    *(localWriteA_0_0_31_0 + 0) = a_0_0_31_0;
    *(localWriteA_0_0_32_0 + 0) = a_0_0_32_0;
    *(localWriteA_0_0_33_0 + 0) = a_0_0_33_0;
    *(localWriteA_0_0_34_0 + 0) = a_0_0_34_0;
    *(localWriteA_0_0_35_0 + 0) = a_0_0_35_0;
    *(localWriteA_0_0_36_0 + 0) = a_0_0_36_0;
    *(localWriteA_0_0_37_0 + 0) = a_0_0_37_0;
    *(localWriteA_0_0_38_0 + 0) = a_0_0_38_0;
    *(localWriteA_0_0_39_0 + 0) = a_0_0_39_0;
    *(localWriteA_0_0_40_0 + 0) = a_0_0_40_0;
    *(localWriteA_0_0_41_0 + 0) = a_0_0_41_0;
    *(localWriteA_0_0_42_0 + 0) = a_0_0_42_0;
    *(localWriteA_0_0_43_0 + 0) = a_0_0_43_0;
    *(localWriteA_0_0_44_0 + 0) = a_0_0_44_0;
    *(localWriteA_0_0_45_0 + 0) = a_0_0_45_0;
    *(localWriteA_0_0_46_0 + 0) = a_0_0_46_0;
    *(localWriteA_0_0_47_0 + 0) = a_0_0_47_0;
    *(localWriteA_0_0_48_0 + 0) = a_0_0_48_0;
    *(localWriteA_0_0_49_0 + 0) = a_0_0_49_0;
    *(localWriteA_0_0_50_0 + 0) = a_0_0_50_0;
    *(localWriteA_0_0_51_0 + 0) = a_0_0_51_0;
    *(localWriteA_0_0_52_0 + 0) = a_0_0_52_0;
    *(localWriteA_0_0_53_0 + 0) = a_0_0_53_0;
    *(localWriteA_0_0_54_0 + 0) = a_0_0_54_0;
    *(localWriteA_0_0_55_0 + 0) = a_0_0_55_0;
    *(localWriteA_0_0_56_0 + 0) = a_0_0_56_0;
    *(localWriteA_0_0_57_0 + 0) = a_0_0_57_0;
    *(localWriteA_0_0_58_0 + 0) = a_0_0_58_0;
    *(localWriteA_0_0_59_0 + 0) = a_0_0_59_0;
    *(localWriteA_0_0_60_0 + 0) = a_0_0_60_0;
    *(localWriteA_0_0_61_0 + 0) = a_0_0_61_0;
    *(localWriteA_0_0_62_0 + 0) = a_0_0_62_0;
    *(localWriteA_0_0_63_0 + 0) = a_0_0_63_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 15 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 16 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 17 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 18 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 19 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 20 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 21 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 22 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 23 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 24 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 25 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 26 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 27 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 28 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 29 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 30 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 31 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 32 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 33 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 34 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 35 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 36 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 37 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 38 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 39 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 40 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 41 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 42 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 43 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 44 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 45 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 46 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 47 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 48 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 49 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 50 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 51 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 52 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 53 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 54 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 55 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 56 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 57 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 58 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 59 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 60 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 61 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 62 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_1x1


    /* iter 63 */

    MAC_1x1_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_0_8_0 = ( globalReadOffsetAL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_0_0_9_0 = ( globalReadOffsetAL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_0_0_10_0 = ( globalReadOffsetAL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_0_0_11_0 = ( globalReadOffsetAL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_0_0_12_0 = ( globalReadOffsetAL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_0_0_13_0 = ( globalReadOffsetAL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_0_0_14_0 = ( globalReadOffsetAL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_0_0_15_0 = ( globalReadOffsetAL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);
  a_0_0_16_0 = ( globalReadOffsetAL_16_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_16_0 + 0);
  a_0_0_17_0 = ( globalReadOffsetAL_17_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_17_0 + 0);
  a_0_0_18_0 = ( globalReadOffsetAL_18_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_18_0 + 0);
  a_0_0_19_0 = ( globalReadOffsetAL_19_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_19_0 + 0);
  a_0_0_20_0 = ( globalReadOffsetAL_20_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_20_0 + 0);
  a_0_0_21_0 = ( globalReadOffsetAL_21_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_21_0 + 0);
  a_0_0_22_0 = ( globalReadOffsetAL_22_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_22_0 + 0);
  a_0_0_23_0 = ( globalReadOffsetAL_23_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_23_0 + 0);
  a_0_0_24_0 = ( globalReadOffsetAL_24_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_24_0 + 0);
  a_0_0_25_0 = ( globalReadOffsetAL_25_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_25_0 + 0);
  a_0_0_26_0 = ( globalReadOffsetAL_26_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_26_0 + 0);
  a_0_0_27_0 = ( globalReadOffsetAL_27_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_27_0 + 0);
  a_0_0_28_0 = ( globalReadOffsetAL_28_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_28_0 + 0);
  a_0_0_29_0 = ( globalReadOffsetAL_29_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_29_0 + 0);
  a_0_0_30_0 = ( globalReadOffsetAL_30_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_30_0 + 0);
  a_0_0_31_0 = ( globalReadOffsetAL_31_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_31_0 + 0);
  a_0_0_32_0 = ( globalReadOffsetAL_32_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_32_0 + 0);
  a_0_0_33_0 = ( globalReadOffsetAL_33_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_33_0 + 0);
  a_0_0_34_0 = ( globalReadOffsetAL_34_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_34_0 + 0);
  a_0_0_35_0 = ( globalReadOffsetAL_35_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_35_0 + 0);
  a_0_0_36_0 = ( globalReadOffsetAL_36_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_36_0 + 0);
  a_0_0_37_0 = ( globalReadOffsetAL_37_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_37_0 + 0);
  a_0_0_38_0 = ( globalReadOffsetAL_38_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_38_0 + 0);
  a_0_0_39_0 = ( globalReadOffsetAL_39_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_39_0 + 0);
  a_0_0_40_0 = ( globalReadOffsetAL_40_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_40_0 + 0);
  a_0_0_41_0 = ( globalReadOffsetAL_41_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_41_0 + 0);
  a_0_0_42_0 = ( globalReadOffsetAL_42_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_42_0 + 0);
  a_0_0_43_0 = ( globalReadOffsetAL_43_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_43_0 + 0);
  a_0_0_44_0 = ( globalReadOffsetAL_44_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_44_0 + 0);
  a_0_0_45_0 = ( globalReadOffsetAL_45_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_45_0 + 0);
  a_0_0_46_0 = ( globalReadOffsetAL_46_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_46_0 + 0);
  a_0_0_47_0 = ( globalReadOffsetAL_47_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_47_0 + 0);
  a_0_0_48_0 = ( globalReadOffsetAL_48_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_48_0 + 0);
  a_0_0_49_0 = ( globalReadOffsetAL_49_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_49_0 + 0);
  a_0_0_50_0 = ( globalReadOffsetAL_50_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_50_0 + 0);
  a_0_0_51_0 = ( globalReadOffsetAL_51_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_51_0 + 0);
  a_0_0_52_0 = ( globalReadOffsetAL_52_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_52_0 + 0);
  a_0_0_53_0 = ( globalReadOffsetAL_53_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_53_0 + 0);
  a_0_0_54_0 = ( globalReadOffsetAL_54_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_54_0 + 0);
  a_0_0_55_0 = ( globalReadOffsetAL_55_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_55_0 + 0);
  a_0_0_56_0 = ( globalReadOffsetAL_56_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_56_0 + 0);
  a_0_0_57_0 = ( globalReadOffsetAL_57_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_57_0 + 0);
  a_0_0_58_0 = ( globalReadOffsetAL_58_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_58_0 + 0);
  a_0_0_59_0 = ( globalReadOffsetAL_59_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_59_0 + 0);
  a_0_0_60_0 = ( globalReadOffsetAL_60_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_60_0 + 0);
  a_0_0_61_0 = ( globalReadOffsetAL_61_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_61_0 + 0);
  a_0_0_62_0 = ( globalReadOffsetAL_62_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_62_0 + 0);
  a_0_0_63_0 = ( globalReadOffsetAL_63_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_63_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);
  localWriteA_0_0_16_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_16_0);
  localWriteA_0_0_17_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_17_0);
  localWriteA_0_0_18_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_18_0);
  localWriteA_0_0_19_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_19_0);
  localWriteA_0_0_20_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_20_0);
  localWriteA_0_0_21_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_21_0);
  localWriteA_0_0_22_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_22_0);
  localWriteA_0_0_23_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_23_0);
  localWriteA_0_0_24_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_24_0);
  localWriteA_0_0_25_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_25_0);
  localWriteA_0_0_26_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_26_0);
  localWriteA_0_0_27_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_27_0);
  localWriteA_0_0_28_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_28_0);
  localWriteA_0_0_29_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_29_0);
  localWriteA_0_0_30_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_30_0);
  localWriteA_0_0_31_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_31_0);
  localWriteA_0_0_32_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_32_0);
  localWriteA_0_0_33_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_33_0);
  localWriteA_0_0_34_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_34_0);
  localWriteA_0_0_35_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_35_0);
  localWriteA_0_0_36_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_36_0);
  localWriteA_0_0_37_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_37_0);
  localWriteA_0_0_38_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_38_0);
  localWriteA_0_0_39_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_39_0);
  localWriteA_0_0_40_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_40_0);
  localWriteA_0_0_41_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_41_0);
  localWriteA_0_0_42_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_42_0);
  localWriteA_0_0_43_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_43_0);
  localWriteA_0_0_44_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_44_0);
  localWriteA_0_0_45_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_45_0);
  localWriteA_0_0_46_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_46_0);
  localWriteA_0_0_47_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_47_0);
  localWriteA_0_0_48_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_48_0);
  localWriteA_0_0_49_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_49_0);
  localWriteA_0_0_50_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_50_0);
  localWriteA_0_0_51_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_51_0);
  localWriteA_0_0_52_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_52_0);
  localWriteA_0_0_53_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_53_0);
  localWriteA_0_0_54_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_54_0);
  localWriteA_0_0_55_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_55_0);
  localWriteA_0_0_56_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_56_0);
  localWriteA_0_0_57_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_57_0);
  localWriteA_0_0_58_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_58_0);
  localWriteA_0_0_59_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_59_0);
  localWriteA_0_0_60_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_60_0);
  localWriteA_0_0_61_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_61_0);
  localWriteA_0_0_62_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_62_0);
  localWriteA_0_0_63_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_63_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
  *(localWriteA_0_0_16_0 + 0) = a_0_0_16_0;
  *(localWriteA_0_0_17_0 + 0) = a_0_0_17_0;
  *(localWriteA_0_0_18_0 + 0) = a_0_0_18_0;
  *(localWriteA_0_0_19_0 + 0) = a_0_0_19_0;
  *(localWriteA_0_0_20_0 + 0) = a_0_0_20_0;
  *(localWriteA_0_0_21_0 + 0) = a_0_0_21_0;
  *(localWriteA_0_0_22_0 + 0) = a_0_0_22_0;
  *(localWriteA_0_0_23_0 + 0) = a_0_0_23_0;
  *(localWriteA_0_0_24_0 + 0) = a_0_0_24_0;
  *(localWriteA_0_0_25_0 + 0) = a_0_0_25_0;
  *(localWriteA_0_0_26_0 + 0) = a_0_0_26_0;
  *(localWriteA_0_0_27_0 + 0) = a_0_0_27_0;
  *(localWriteA_0_0_28_0 + 0) = a_0_0_28_0;
  *(localWriteA_0_0_29_0 + 0) = a_0_0_29_0;
  *(localWriteA_0_0_30_0 + 0) = a_0_0_30_0;
  *(localWriteA_0_0_31_0 + 0) = a_0_0_31_0;
  *(localWriteA_0_0_32_0 + 0) = a_0_0_32_0;
  *(localWriteA_0_0_33_0 + 0) = a_0_0_33_0;
  *(localWriteA_0_0_34_0 + 0) = a_0_0_34_0;
  *(localWriteA_0_0_35_0 + 0) = a_0_0_35_0;
  *(localWriteA_0_0_36_0 + 0) = a_0_0_36_0;
  *(localWriteA_0_0_37_0 + 0) = a_0_0_37_0;
  *(localWriteA_0_0_38_0 + 0) = a_0_0_38_0;
  *(localWriteA_0_0_39_0 + 0) = a_0_0_39_0;
  *(localWriteA_0_0_40_0 + 0) = a_0_0_40_0;
  *(localWriteA_0_0_41_0 + 0) = a_0_0_41_0;
  *(localWriteA_0_0_42_0 + 0) = a_0_0_42_0;
  *(localWriteA_0_0_43_0 + 0) = a_0_0_43_0;
  *(localWriteA_0_0_44_0 + 0) = a_0_0_44_0;
  *(localWriteA_0_0_45_0 + 0) = a_0_0_45_0;
  *(localWriteA_0_0_46_0 + 0) = a_0_0_46_0;
  *(localWriteA_0_0_47_0 + 0) = a_0_0_47_0;
  *(localWriteA_0_0_48_0 + 0) = a_0_0_48_0;
  *(localWriteA_0_0_49_0 + 0) = a_0_0_49_0;
  *(localWriteA_0_0_50_0 + 0) = a_0_0_50_0;
  *(localWriteA_0_0_51_0 + 0) = a_0_0_51_0;
  *(localWriteA_0_0_52_0 + 0) = a_0_0_52_0;
  *(localWriteA_0_0_53_0 + 0) = a_0_0_53_0;
  *(localWriteA_0_0_54_0 + 0) = a_0_0_54_0;
  *(localWriteA_0_0_55_0 + 0) = a_0_0_55_0;
  *(localWriteA_0_0_56_0 + 0) = a_0_0_56_0;
  *(localWriteA_0_0_57_0 + 0) = a_0_0_57_0;
  *(localWriteA_0_0_58_0 + 0) = a_0_0_58_0;
  *(localWriteA_0_0_59_0 + 0) = a_0_0_59_0;
  *(localWriteA_0_0_60_0 + 0) = a_0_0_60_0;
  *(localWriteA_0_0_61_0 + 0) = a_0_0_61_0;
  *(localWriteA_0_0_62_0 + 0) = a_0_0_62_0;
  *(localWriteA_0_0_63_0 + 0) = a_0_0_63_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_1x1

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_1x1
#undef MAC_1x1_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 64
#define SG1J 1
#define TT0I 1
#define TT1J 1
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 64
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 15

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 64
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 4096
#define LDS_NUM_ELEMENTS 4160

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 1x1 micro-tile */
#define MAC_1x1\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \

#define MAC_1x1_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT64x1x64_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU15_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU0_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT1_1_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG64_1_1_WGM1(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE a_0_0_16_0;
  DATA_TYPE a_0_0_17_0;
  DATA_TYPE a_0_0_18_0;
  DATA_TYPE a_0_0_19_0;
  DATA_TYPE a_0_0_20_0;
  DATA_TYPE a_0_0_21_0;
  DATA_TYPE a_0_0_22_0;
  DATA_TYPE a_0_0_23_0;
  DATA_TYPE a_0_0_24_0;
  DATA_TYPE a_0_0_25_0;
  DATA_TYPE a_0_0_26_0;
  DATA_TYPE a_0_0_27_0;
  DATA_TYPE a_0_0_28_0;
  DATA_TYPE a_0_0_29_0;
  DATA_TYPE a_0_0_30_0;
  DATA_TYPE a_0_0_31_0;
  DATA_TYPE a_0_0_32_0;
  DATA_TYPE a_0_0_33_0;
  DATA_TYPE a_0_0_34_0;
  DATA_TYPE a_0_0_35_0;
  DATA_TYPE a_0_0_36_0;
  DATA_TYPE a_0_0_37_0;
  DATA_TYPE a_0_0_38_0;
  DATA_TYPE a_0_0_39_0;
  DATA_TYPE a_0_0_40_0;
  DATA_TYPE a_0_0_41_0;
  DATA_TYPE a_0_0_42_0;
  DATA_TYPE a_0_0_43_0;
  DATA_TYPE a_0_0_44_0;
  DATA_TYPE a_0_0_45_0;
  DATA_TYPE a_0_0_46_0;
  DATA_TYPE a_0_0_47_0;
  DATA_TYPE a_0_0_48_0;
  DATA_TYPE a_0_0_49_0;
  DATA_TYPE a_0_0_50_0;
  DATA_TYPE a_0_0_51_0;
  DATA_TYPE a_0_0_52_0;
  DATA_TYPE a_0_0_53_0;
  DATA_TYPE a_0_0_54_0;
  DATA_TYPE a_0_0_55_0;
  DATA_TYPE a_0_0_56_0;
  DATA_TYPE a_0_0_57_0;
  DATA_TYPE a_0_0_58_0;
  DATA_TYPE a_0_0_59_0;
  DATA_TYPE a_0_0_60_0;
  DATA_TYPE a_0_0_61_0;
  DATA_TYPE a_0_0_62_0;
  DATA_TYPE a_0_0_63_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;
  unsigned int globalReadOffsetAL_8_0 = globalReadOffsetAL + 0 + 8*LSPA;
  unsigned int globalReadOffsetAL_9_0 = globalReadOffsetAL + 0 + 9*LSPA;
  unsigned int globalReadOffsetAL_10_0 = globalReadOffsetAL + 0 + 10*LSPA;
  unsigned int globalReadOffsetAL_11_0 = globalReadOffsetAL + 0 + 11*LSPA;
  unsigned int globalReadOffsetAL_12_0 = globalReadOffsetAL + 0 + 12*LSPA;
  unsigned int globalReadOffsetAL_13_0 = globalReadOffsetAL + 0 + 13*LSPA;
  unsigned int globalReadOffsetAL_14_0 = globalReadOffsetAL + 0 + 14*LSPA;
  unsigned int globalReadOffsetAL_15_0 = globalReadOffsetAL + 0 + 15*LSPA;
  unsigned int globalReadOffsetAL_16_0 = globalReadOffsetAL + 0 + 16*LSPA;
  unsigned int globalReadOffsetAL_17_0 = globalReadOffsetAL + 0 + 17*LSPA;
  unsigned int globalReadOffsetAL_18_0 = globalReadOffsetAL + 0 + 18*LSPA;
  unsigned int globalReadOffsetAL_19_0 = globalReadOffsetAL + 0 + 19*LSPA;
  unsigned int globalReadOffsetAL_20_0 = globalReadOffsetAL + 0 + 20*LSPA;
  unsigned int globalReadOffsetAL_21_0 = globalReadOffsetAL + 0 + 21*LSPA;
  unsigned int globalReadOffsetAL_22_0 = globalReadOffsetAL + 0 + 22*LSPA;
  unsigned int globalReadOffsetAL_23_0 = globalReadOffsetAL + 0 + 23*LSPA;
  unsigned int globalReadOffsetAL_24_0 = globalReadOffsetAL + 0 + 24*LSPA;
  unsigned int globalReadOffsetAL_25_0 = globalReadOffsetAL + 0 + 25*LSPA;
  unsigned int globalReadOffsetAL_26_0 = globalReadOffsetAL + 0 + 26*LSPA;
  unsigned int globalReadOffsetAL_27_0 = globalReadOffsetAL + 0 + 27*LSPA;
  unsigned int globalReadOffsetAL_28_0 = globalReadOffsetAL + 0 + 28*LSPA;
  unsigned int globalReadOffsetAL_29_0 = globalReadOffsetAL + 0 + 29*LSPA;
  unsigned int globalReadOffsetAL_30_0 = globalReadOffsetAL + 0 + 30*LSPA;
  unsigned int globalReadOffsetAL_31_0 = globalReadOffsetAL + 0 + 31*LSPA;
  unsigned int globalReadOffsetAL_32_0 = globalReadOffsetAL + 0 + 32*LSPA;
  unsigned int globalReadOffsetAL_33_0 = globalReadOffsetAL + 0 + 33*LSPA;
  unsigned int globalReadOffsetAL_34_0 = globalReadOffsetAL + 0 + 34*LSPA;
  unsigned int globalReadOffsetAL_35_0 = globalReadOffsetAL + 0 + 35*LSPA;
  unsigned int globalReadOffsetAL_36_0 = globalReadOffsetAL + 0 + 36*LSPA;
  unsigned int globalReadOffsetAL_37_0 = globalReadOffsetAL + 0 + 37*LSPA;
  unsigned int globalReadOffsetAL_38_0 = globalReadOffsetAL + 0 + 38*LSPA;
  unsigned int globalReadOffsetAL_39_0 = globalReadOffsetAL + 0 + 39*LSPA;
  unsigned int globalReadOffsetAL_40_0 = globalReadOffsetAL + 0 + 40*LSPA;
  unsigned int globalReadOffsetAL_41_0 = globalReadOffsetAL + 0 + 41*LSPA;
  unsigned int globalReadOffsetAL_42_0 = globalReadOffsetAL + 0 + 42*LSPA;
  unsigned int globalReadOffsetAL_43_0 = globalReadOffsetAL + 0 + 43*LSPA;
  unsigned int globalReadOffsetAL_44_0 = globalReadOffsetAL + 0 + 44*LSPA;
  unsigned int globalReadOffsetAL_45_0 = globalReadOffsetAL + 0 + 45*LSPA;
  unsigned int globalReadOffsetAL_46_0 = globalReadOffsetAL + 0 + 46*LSPA;
  unsigned int globalReadOffsetAL_47_0 = globalReadOffsetAL + 0 + 47*LSPA;
  unsigned int globalReadOffsetAL_48_0 = globalReadOffsetAL + 0 + 48*LSPA;
  unsigned int globalReadOffsetAL_49_0 = globalReadOffsetAL + 0 + 49*LSPA;
  unsigned int globalReadOffsetAL_50_0 = globalReadOffsetAL + 0 + 50*LSPA;
  unsigned int globalReadOffsetAL_51_0 = globalReadOffsetAL + 0 + 51*LSPA;
  unsigned int globalReadOffsetAL_52_0 = globalReadOffsetAL + 0 + 52*LSPA;
  unsigned int globalReadOffsetAL_53_0 = globalReadOffsetAL + 0 + 53*LSPA;
  unsigned int globalReadOffsetAL_54_0 = globalReadOffsetAL + 0 + 54*LSPA;
  unsigned int globalReadOffsetAL_55_0 = globalReadOffsetAL + 0 + 55*LSPA;
  unsigned int globalReadOffsetAL_56_0 = globalReadOffsetAL + 0 + 56*LSPA;
  unsigned int globalReadOffsetAL_57_0 = globalReadOffsetAL + 0 + 57*LSPA;
  unsigned int globalReadOffsetAL_58_0 = globalReadOffsetAL + 0 + 58*LSPA;
  unsigned int globalReadOffsetAL_59_0 = globalReadOffsetAL + 0 + 59*LSPA;
  unsigned int globalReadOffsetAL_60_0 = globalReadOffsetAL + 0 + 60*LSPA;
  unsigned int globalReadOffsetAL_61_0 = globalReadOffsetAL + 0 + 61*LSPA;
  unsigned int globalReadOffsetAL_62_0 = globalReadOffsetAL + 0 + 62*LSPA;
  unsigned int globalReadOffsetAL_63_0 = globalReadOffsetAL + 0 + 63*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_15_0), wgK );
  int64_t globalReadOffsetA_0_0_16_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_16_0), wgK );
  int64_t globalReadOffsetA_0_0_17_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_17_0), wgK );
  int64_t globalReadOffsetA_0_0_18_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_18_0), wgK );
  int64_t globalReadOffsetA_0_0_19_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_19_0), wgK );
  int64_t globalReadOffsetA_0_0_20_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_20_0), wgK );
  int64_t globalReadOffsetA_0_0_21_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_21_0), wgK );
  int64_t globalReadOffsetA_0_0_22_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_22_0), wgK );
  int64_t globalReadOffsetA_0_0_23_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_23_0), wgK );
  int64_t globalReadOffsetA_0_0_24_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_24_0), wgK );
  int64_t globalReadOffsetA_0_0_25_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_25_0), wgK );
  int64_t globalReadOffsetA_0_0_26_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_26_0), wgK );
  int64_t globalReadOffsetA_0_0_27_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_27_0), wgK );
  int64_t globalReadOffsetA_0_0_28_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_28_0), wgK );
  int64_t globalReadOffsetA_0_0_29_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_29_0), wgK );
  int64_t globalReadOffsetA_0_0_30_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_30_0), wgK );
  int64_t globalReadOffsetA_0_0_31_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_31_0), wgK );
  int64_t globalReadOffsetA_0_0_32_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_32_0), wgK );
  int64_t globalReadOffsetA_0_0_33_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_33_0), wgK );
  int64_t globalReadOffsetA_0_0_34_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_34_0), wgK );
  int64_t globalReadOffsetA_0_0_35_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_35_0), wgK );
  int64_t globalReadOffsetA_0_0_36_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_36_0), wgK );
  int64_t globalReadOffsetA_0_0_37_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_37_0), wgK );
  int64_t globalReadOffsetA_0_0_38_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_38_0), wgK );
  int64_t globalReadOffsetA_0_0_39_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_39_0), wgK );
  int64_t globalReadOffsetA_0_0_40_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_40_0), wgK );
  int64_t globalReadOffsetA_0_0_41_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_41_0), wgK );
  int64_t globalReadOffsetA_0_0_42_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_42_0), wgK );
  int64_t globalReadOffsetA_0_0_43_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_43_0), wgK );
  int64_t globalReadOffsetA_0_0_44_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_44_0), wgK );
  int64_t globalReadOffsetA_0_0_45_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_45_0), wgK );
  int64_t globalReadOffsetA_0_0_46_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_46_0), wgK );
  int64_t globalReadOffsetA_0_0_47_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_47_0), wgK );
  int64_t globalReadOffsetA_0_0_48_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_48_0), wgK );
  int64_t globalReadOffsetA_0_0_49_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_49_0), wgK );
  int64_t globalReadOffsetA_0_0_50_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_50_0), wgK );
  int64_t globalReadOffsetA_0_0_51_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_51_0), wgK );
  int64_t globalReadOffsetA_0_0_52_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_52_0), wgK );
  int64_t globalReadOffsetA_0_0_53_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_53_0), wgK );
  int64_t globalReadOffsetA_0_0_54_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_54_0), wgK );
  int64_t globalReadOffsetA_0_0_55_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_55_0), wgK );
  int64_t globalReadOffsetA_0_0_56_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_56_0), wgK );
  int64_t globalReadOffsetA_0_0_57_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_57_0), wgK );
  int64_t globalReadOffsetA_0_0_58_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_58_0), wgK );
  int64_t globalReadOffsetA_0_0_59_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_59_0), wgK );
  int64_t globalReadOffsetA_0_0_60_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_60_0), wgK );
  int64_t globalReadOffsetA_0_0_61_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_61_0), wgK );
  int64_t globalReadOffsetA_0_0_62_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_62_0), wgK );
  int64_t globalReadOffsetA_0_0_63_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_63_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;
  DATA_TYPE const *globalReadA_0_0_16_0 = A + globalReadOffsetA_0_0_16_0;
  DATA_TYPE const *globalReadA_0_0_17_0 = A + globalReadOffsetA_0_0_17_0;
  DATA_TYPE const *globalReadA_0_0_18_0 = A + globalReadOffsetA_0_0_18_0;
  DATA_TYPE const *globalReadA_0_0_19_0 = A + globalReadOffsetA_0_0_19_0;
  DATA_TYPE const *globalReadA_0_0_20_0 = A + globalReadOffsetA_0_0_20_0;
  DATA_TYPE const *globalReadA_0_0_21_0 = A + globalReadOffsetA_0_0_21_0;
  DATA_TYPE const *globalReadA_0_0_22_0 = A + globalReadOffsetA_0_0_22_0;
  DATA_TYPE const *globalReadA_0_0_23_0 = A + globalReadOffsetA_0_0_23_0;
  DATA_TYPE const *globalReadA_0_0_24_0 = A + globalReadOffsetA_0_0_24_0;
  DATA_TYPE const *globalReadA_0_0_25_0 = A + globalReadOffsetA_0_0_25_0;
  DATA_TYPE const *globalReadA_0_0_26_0 = A + globalReadOffsetA_0_0_26_0;
  DATA_TYPE const *globalReadA_0_0_27_0 = A + globalReadOffsetA_0_0_27_0;
  DATA_TYPE const *globalReadA_0_0_28_0 = A + globalReadOffsetA_0_0_28_0;
  DATA_TYPE const *globalReadA_0_0_29_0 = A + globalReadOffsetA_0_0_29_0;
  DATA_TYPE const *globalReadA_0_0_30_0 = A + globalReadOffsetA_0_0_30_0;
  DATA_TYPE const *globalReadA_0_0_31_0 = A + globalReadOffsetA_0_0_31_0;
  DATA_TYPE const *globalReadA_0_0_32_0 = A + globalReadOffsetA_0_0_32_0;
  DATA_TYPE const *globalReadA_0_0_33_0 = A + globalReadOffsetA_0_0_33_0;
  DATA_TYPE const *globalReadA_0_0_34_0 = A + globalReadOffsetA_0_0_34_0;
  DATA_TYPE const *globalReadA_0_0_35_0 = A + globalReadOffsetA_0_0_35_0;
  DATA_TYPE const *globalReadA_0_0_36_0 = A + globalReadOffsetA_0_0_36_0;
  DATA_TYPE const *globalReadA_0_0_37_0 = A + globalReadOffsetA_0_0_37_0;
  DATA_TYPE const *globalReadA_0_0_38_0 = A + globalReadOffsetA_0_0_38_0;
  DATA_TYPE const *globalReadA_0_0_39_0 = A + globalReadOffsetA_0_0_39_0;
  DATA_TYPE const *globalReadA_0_0_40_0 = A + globalReadOffsetA_0_0_40_0;
  DATA_TYPE const *globalReadA_0_0_41_0 = A + globalReadOffsetA_0_0_41_0;
  DATA_TYPE const *globalReadA_0_0_42_0 = A + globalReadOffsetA_0_0_42_0;
  DATA_TYPE const *globalReadA_0_0_43_0 = A + globalReadOffsetA_0_0_43_0;
  DATA_TYPE const *globalReadA_0_0_44_0 = A + globalReadOffsetA_0_0_44_0;
  DATA_TYPE const *globalReadA_0_0_45_0 = A + globalReadOffsetA_0_0_45_0;
  DATA_TYPE const *globalReadA_0_0_46_0 = A + globalReadOffsetA_0_0_46_0;
  DATA_TYPE const *globalReadA_0_0_47_0 = A + globalReadOffsetA_0_0_47_0;
  DATA_TYPE const *globalReadA_0_0_48_0 = A + globalReadOffsetA_0_0_48_0;
  DATA_TYPE const *globalReadA_0_0_49_0 = A + globalReadOffsetA_0_0_49_0;
  DATA_TYPE const *globalReadA_0_0_50_0 = A + globalReadOffsetA_0_0_50_0;
  DATA_TYPE const *globalReadA_0_0_51_0 = A + globalReadOffsetA_0_0_51_0;
  DATA_TYPE const *globalReadA_0_0_52_0 = A + globalReadOffsetA_0_0_52_0;
  DATA_TYPE const *globalReadA_0_0_53_0 = A + globalReadOffsetA_0_0_53_0;
  DATA_TYPE const *globalReadA_0_0_54_0 = A + globalReadOffsetA_0_0_54_0;
  DATA_TYPE const *globalReadA_0_0_55_0 = A + globalReadOffsetA_0_0_55_0;
  DATA_TYPE const *globalReadA_0_0_56_0 = A + globalReadOffsetA_0_0_56_0;
  DATA_TYPE const *globalReadA_0_0_57_0 = A + globalReadOffsetA_0_0_57_0;
  DATA_TYPE const *globalReadA_0_0_58_0 = A + globalReadOffsetA_0_0_58_0;
  DATA_TYPE const *globalReadA_0_0_59_0 = A + globalReadOffsetA_0_0_59_0;
  DATA_TYPE const *globalReadA_0_0_60_0 = A + globalReadOffsetA_0_0_60_0;
  DATA_TYPE const *globalReadA_0_0_61_0 = A + globalReadOffsetA_0_0_61_0;
  DATA_TYPE const *globalReadA_0_0_62_0 = A + globalReadOffsetA_0_0_62_0;
  DATA_TYPE const *globalReadA_0_0_63_0 = A + globalReadOffsetA_0_0_63_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 8*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 9*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 10*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 11*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 12*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 13*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 14*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 15*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_16_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 16*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_17_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 17*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_18_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 18*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_19_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 19*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_20_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 20*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_21_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 21*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_22_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 22*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_23_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 23*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_24_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 24*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_25_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 25*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_26_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 26*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_27_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 27*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_28_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 28*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_29_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 29*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_30_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 30*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_31_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 31*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_32_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 32*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_33_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 33*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_34_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 34*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_35_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 35*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_36_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 36*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_37_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 37*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_38_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 38*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_39_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 39*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_40_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 40*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_41_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 41*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_42_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 42*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_43_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 43*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_44_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 44*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_45_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 45*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_46_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 46*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_47_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 47*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_48_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 48*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_49_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 49*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_50_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 50*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_51_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 51*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_52_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 52*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_53_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 53*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_54_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 54*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_55_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 55*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_56_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 56*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_57_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 57*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_58_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 58*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_59_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 59*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_60_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 60*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_61_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 61*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_62_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 62*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_63_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 63*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_0_0_15_0;
  DATA_TYPE *localWriteA_0_0_16_0;
  DATA_TYPE *localWriteA_0_0_17_0;
  DATA_TYPE *localWriteA_0_0_18_0;
  DATA_TYPE *localWriteA_0_0_19_0;
  DATA_TYPE *localWriteA_0_0_20_0;
  DATA_TYPE *localWriteA_0_0_21_0;
  DATA_TYPE *localWriteA_0_0_22_0;
  DATA_TYPE *localWriteA_0_0_23_0;
  DATA_TYPE *localWriteA_0_0_24_0;
  DATA_TYPE *localWriteA_0_0_25_0;
  DATA_TYPE *localWriteA_0_0_26_0;
  DATA_TYPE *localWriteA_0_0_27_0;
  DATA_TYPE *localWriteA_0_0_28_0;
  DATA_TYPE *localWriteA_0_0_29_0;
  DATA_TYPE *localWriteA_0_0_30_0;
  DATA_TYPE *localWriteA_0_0_31_0;
  DATA_TYPE *localWriteA_0_0_32_0;
  DATA_TYPE *localWriteA_0_0_33_0;
  DATA_TYPE *localWriteA_0_0_34_0;
  DATA_TYPE *localWriteA_0_0_35_0;
  DATA_TYPE *localWriteA_0_0_36_0;
  DATA_TYPE *localWriteA_0_0_37_0;
  DATA_TYPE *localWriteA_0_0_38_0;
  DATA_TYPE *localWriteA_0_0_39_0;
  DATA_TYPE *localWriteA_0_0_40_0;
  DATA_TYPE *localWriteA_0_0_41_0;
  DATA_TYPE *localWriteA_0_0_42_0;
  DATA_TYPE *localWriteA_0_0_43_0;
  DATA_TYPE *localWriteA_0_0_44_0;
  DATA_TYPE *localWriteA_0_0_45_0;
  DATA_TYPE *localWriteA_0_0_46_0;
  DATA_TYPE *localWriteA_0_0_47_0;
  DATA_TYPE *localWriteA_0_0_48_0;
  DATA_TYPE *localWriteA_0_0_49_0;
  DATA_TYPE *localWriteA_0_0_50_0;
  DATA_TYPE *localWriteA_0_0_51_0;
  DATA_TYPE *localWriteA_0_0_52_0;
  DATA_TYPE *localWriteA_0_0_53_0;
  DATA_TYPE *localWriteA_0_0_54_0;
  DATA_TYPE *localWriteA_0_0_55_0;
  DATA_TYPE *localWriteA_0_0_56_0;
  DATA_TYPE *localWriteA_0_0_57_0;
  DATA_TYPE *localWriteA_0_0_58_0;
  DATA_TYPE *localWriteA_0_0_59_0;
  DATA_TYPE *localWriteA_0_0_60_0;
  DATA_TYPE *localWriteA_0_0_61_0;
  DATA_TYPE *localWriteA_0_0_62_0;
  DATA_TYPE *localWriteA_0_0_63_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);
  localWriteA_0_0_16_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_16_0);
  localWriteA_0_0_17_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_17_0);
  localWriteA_0_0_18_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_18_0);
  localWriteA_0_0_19_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_19_0);
  localWriteA_0_0_20_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_20_0);
  localWriteA_0_0_21_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_21_0);
  localWriteA_0_0_22_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_22_0);
  localWriteA_0_0_23_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_23_0);
  localWriteA_0_0_24_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_24_0);
  localWriteA_0_0_25_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_25_0);
  localWriteA_0_0_26_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_26_0);
  localWriteA_0_0_27_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_27_0);
  localWriteA_0_0_28_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_28_0);
  localWriteA_0_0_29_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_29_0);
  localWriteA_0_0_30_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_30_0);
  localWriteA_0_0_31_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_31_0);
  localWriteA_0_0_32_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_32_0);
  localWriteA_0_0_33_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_33_0);
  localWriteA_0_0_34_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_34_0);
  localWriteA_0_0_35_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_35_0);
  localWriteA_0_0_36_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_36_0);
  localWriteA_0_0_37_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_37_0);
  localWriteA_0_0_38_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_38_0);
  localWriteA_0_0_39_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_39_0);
  localWriteA_0_0_40_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_40_0);
  localWriteA_0_0_41_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_41_0);
  localWriteA_0_0_42_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_42_0);
  localWriteA_0_0_43_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_43_0);
  localWriteA_0_0_44_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_44_0);
  localWriteA_0_0_45_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_45_0);
  localWriteA_0_0_46_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_46_0);
  localWriteA_0_0_47_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_47_0);
  localWriteA_0_0_48_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_48_0);
  localWriteA_0_0_49_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_49_0);
  localWriteA_0_0_50_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_50_0);
  localWriteA_0_0_51_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_51_0);
  localWriteA_0_0_52_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_52_0);
  localWriteA_0_0_53_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_53_0);
  localWriteA_0_0_54_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_54_0);
  localWriteA_0_0_55_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_55_0);
  localWriteA_0_0_56_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_56_0);
  localWriteA_0_0_57_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_57_0);
  localWriteA_0_0_58_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_58_0);
  localWriteA_0_0_59_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_59_0);
  localWriteA_0_0_60_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_60_0);
  localWriteA_0_0_61_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_61_0);
  localWriteA_0_0_62_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_62_0);
  localWriteA_0_0_63_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_63_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);
    a_0_0_16_0 = *(globalReadA_0_0_16_0 + 0);
    a_0_0_17_0 = *(globalReadA_0_0_17_0 + 0);
    a_0_0_18_0 = *(globalReadA_0_0_18_0 + 0);
    a_0_0_19_0 = *(globalReadA_0_0_19_0 + 0);
    a_0_0_20_0 = *(globalReadA_0_0_20_0 + 0);
    a_0_0_21_0 = *(globalReadA_0_0_21_0 + 0);
    a_0_0_22_0 = *(globalReadA_0_0_22_0 + 0);
    a_0_0_23_0 = *(globalReadA_0_0_23_0 + 0);
    a_0_0_24_0 = *(globalReadA_0_0_24_0 + 0);
    a_0_0_25_0 = *(globalReadA_0_0_25_0 + 0);
    a_0_0_26_0 = *(globalReadA_0_0_26_0 + 0);
    a_0_0_27_0 = *(globalReadA_0_0_27_0 + 0);
    a_0_0_28_0 = *(globalReadA_0_0_28_0 + 0);
    a_0_0_29_0 = *(globalReadA_0_0_29_0 + 0);
    a_0_0_30_0 = *(globalReadA_0_0_30_0 + 0);
    a_0_0_31_0 = *(globalReadA_0_0_31_0 + 0);
    a_0_0_32_0 = *(globalReadA_0_0_32_0 + 0);
    a_0_0_33_0 = *(globalReadA_0_0_33_0 + 0);
    a_0_0_34_0 = *(globalReadA_0_0_34_0 + 0);
    a_0_0_35_0 = *(globalReadA_0_0_35_0 + 0);
    a_0_0_36_0 = *(globalReadA_0_0_36_0 + 0);
    a_0_0_37_0 = *(globalReadA_0_0_37_0 + 0);
    a_0_0_38_0 = *(globalReadA_0_0_38_0 + 0);
    a_0_0_39_0 = *(globalReadA_0_0_39_0 + 0);
    a_0_0_40_0 = *(globalReadA_0_0_40_0 + 0);
    a_0_0_41_0 = *(globalReadA_0_0_41_0 + 0);
    a_0_0_42_0 = *(globalReadA_0_0_42_0 + 0);
    a_0_0_43_0 = *(globalReadA_0_0_43_0 + 0);
    a_0_0_44_0 = *(globalReadA_0_0_44_0 + 0);
    a_0_0_45_0 = *(globalReadA_0_0_45_0 + 0);
    a_0_0_46_0 = *(globalReadA_0_0_46_0 + 0);
    a_0_0_47_0 = *(globalReadA_0_0_47_0 + 0);
    a_0_0_48_0 = *(globalReadA_0_0_48_0 + 0);
    a_0_0_49_0 = *(globalReadA_0_0_49_0 + 0);
    a_0_0_50_0 = *(globalReadA_0_0_50_0 + 0);
    a_0_0_51_0 = *(globalReadA_0_0_51_0 + 0);
    a_0_0_52_0 = *(globalReadA_0_0_52_0 + 0);
    a_0_0_53_0 = *(globalReadA_0_0_53_0 + 0);
    a_0_0_54_0 = *(globalReadA_0_0_54_0 + 0);
    a_0_0_55_0 = *(globalReadA_0_0_55_0 + 0);
    a_0_0_56_0 = *(globalReadA_0_0_56_0 + 0);
    a_0_0_57_0 = *(globalReadA_0_0_57_0 + 0);
    a_0_0_58_0 = *(globalReadA_0_0_58_0 + 0);
    a_0_0_59_0 = *(globalReadA_0_0_59_0 + 0);
    a_0_0_60_0 = *(globalReadA_0_0_60_0 + 0);
    a_0_0_61_0 = *(globalReadA_0_0_61_0 + 0);
    a_0_0_62_0 = *(globalReadA_0_0_62_0 + 0);
    a_0_0_63_0 = *(globalReadA_0_0_63_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    globalReadA_0_0_16_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_16_0) + 1*globalReadIncAL);
    globalReadA_0_0_17_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_17_0) + 1*globalReadIncAL);
    globalReadA_0_0_18_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_18_0) + 1*globalReadIncAL);
    globalReadA_0_0_19_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_19_0) + 1*globalReadIncAL);
    globalReadA_0_0_20_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_20_0) + 1*globalReadIncAL);
    globalReadA_0_0_21_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_21_0) + 1*globalReadIncAL);
    globalReadA_0_0_22_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_22_0) + 1*globalReadIncAL);
    globalReadA_0_0_23_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_23_0) + 1*globalReadIncAL);
    globalReadA_0_0_24_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_24_0) + 1*globalReadIncAL);
    globalReadA_0_0_25_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_25_0) + 1*globalReadIncAL);
    globalReadA_0_0_26_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_26_0) + 1*globalReadIncAL);
    globalReadA_0_0_27_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_27_0) + 1*globalReadIncAL);
    globalReadA_0_0_28_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_28_0) + 1*globalReadIncAL);
    globalReadA_0_0_29_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_29_0) + 1*globalReadIncAL);
    globalReadA_0_0_30_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_30_0) + 1*globalReadIncAL);
    globalReadA_0_0_31_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_31_0) + 1*globalReadIncAL);
    globalReadA_0_0_32_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_32_0) + 1*globalReadIncAL);
    globalReadA_0_0_33_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_33_0) + 1*globalReadIncAL);
    globalReadA_0_0_34_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_34_0) + 1*globalReadIncAL);
    globalReadA_0_0_35_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_35_0) + 1*globalReadIncAL);
    globalReadA_0_0_36_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_36_0) + 1*globalReadIncAL);
    globalReadA_0_0_37_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_37_0) + 1*globalReadIncAL);
    globalReadA_0_0_38_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_38_0) + 1*globalReadIncAL);
    globalReadA_0_0_39_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_39_0) + 1*globalReadIncAL);
    globalReadA_0_0_40_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_40_0) + 1*globalReadIncAL);
    globalReadA_0_0_41_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_41_0) + 1*globalReadIncAL);
    globalReadA_0_0_42_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_42_0) + 1*globalReadIncAL);
    globalReadA_0_0_43_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_43_0) + 1*globalReadIncAL);
    globalReadA_0_0_44_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_44_0) + 1*globalReadIncAL);
    globalReadA_0_0_45_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_45_0) + 1*globalReadIncAL);
    globalReadA_0_0_46_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_46_0) + 1*globalReadIncAL);
    globalReadA_0_0_47_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_47_0) + 1*globalReadIncAL);
    globalReadA_0_0_48_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_48_0) + 1*globalReadIncAL);
    globalReadA_0_0_49_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_49_0) + 1*globalReadIncAL);
    globalReadA_0_0_50_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_50_0) + 1*globalReadIncAL);
    globalReadA_0_0_51_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_51_0) + 1*globalReadIncAL);
    globalReadA_0_0_52_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_52_0) + 1*globalReadIncAL);
    globalReadA_0_0_53_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_53_0) + 1*globalReadIncAL);
    globalReadA_0_0_54_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_54_0) + 1*globalReadIncAL);
    globalReadA_0_0_55_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_55_0) + 1*globalReadIncAL);
    globalReadA_0_0_56_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_56_0) + 1*globalReadIncAL);
    globalReadA_0_0_57_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_57_0) + 1*globalReadIncAL);
    globalReadA_0_0_58_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_58_0) + 1*globalReadIncAL);
    globalReadA_0_0_59_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_59_0) + 1*globalReadIncAL);
    globalReadA_0_0_60_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_60_0) + 1*globalReadIncAL);
    globalReadA_0_0_61_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_61_0) + 1*globalReadIncAL);
    globalReadA_0_0_62_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_62_0) + 1*globalReadIncAL);
    globalReadA_0_0_63_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_63_0) + 1*globalReadIncAL);

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
    *(localWriteA_0_0_16_0 + 0) = a_0_0_16_0;
    *(localWriteA_0_0_17_0 + 0) = a_0_0_17_0;
    *(localWriteA_0_0_18_0 + 0) = a_0_0_18_0;
    *(localWriteA_0_0_19_0 + 0) = a_0_0_19_0;
    *(localWriteA_0_0_20_0 + 0) = a_0_0_20_0;
    *(localWriteA_0_0_21_0 + 0) = a_0_0_21_0;
    *(localWriteA_0_0_22_0 + 0) = a_0_0_22_0;
    *(localWriteA_0_0_23_0 + 0) = a_0_0_23_0;
    *(localWriteA_0_0_24_0 + 0) = a_0_0_24_0;
    *(localWriteA_0_0_25_0 + 0) = a_0_0_25_0;
    *(localWriteA_0_0_26_0 + 0) = a_0_0_26_0;
    *(localWriteA_0_0_27_0 + 0) = a_0_0_27_0;
    *(localWriteA_0_0_28_0 + 0) = a_0_0_28_0;
    *(localWriteA_0_0_29_0 + 0) = a_0_0_29_0;
    *(localWriteA_0_0_30_0 + 0) = a_0_0_30_0;
    *(localWriteA_0_0_31_0 + 0) = a_0_0_31_0;
    *(localWriteA_0_0_32_0 + 0) = a_0_0_32_0;
    *(localWriteA_0_0_33_0 + 0) = a_0_0_33_0;
    *(localWriteA_0_0_34_0 + 0) = a_0_0_34_0;
    *(localWriteA_0_0_35_0 + 0) = a_0_0_35_0;
    *(localWriteA_0_0_36_0 + 0) = a_0_0_36_0;
    *(localWriteA_0_0_37_0 + 0) = a_0_0_37_0;
    *(localWriteA_0_0_38_0 + 0) = a_0_0_38_0;
    *(localWriteA_0_0_39_0 + 0) = a_0_0_39_0;
    *(localWriteA_0_0_40_0 + 0) = a_0_0_40_0;
    *(localWriteA_0_0_41_0 + 0) = a_0_0_41_0;
    *(localWriteA_0_0_42_0 + 0) = a_0_0_42_0;
    *(localWriteA_0_0_43_0 + 0) = a_0_0_43_0;
    *(localWriteA_0_0_44_0 + 0) = a_0_0_44_0;
    *(localWriteA_0_0_45_0 + 0) = a_0_0_45_0;
    *(localWriteA_0_0_46_0 + 0) = a_0_0_46_0;
    *(localWriteA_0_0_47_0 + 0) = a_0_0_47_0;
    *(localWriteA_0_0_48_0 + 0) = a_0_0_48_0;
    *(localWriteA_0_0_49_0 + 0) = a_0_0_49_0;
    *(localWriteA_0_0_50_0 + 0) = a_0_0_50_0;
    *(localWriteA_0_0_51_0 + 0) = a_0_0_51_0;
    *(localWriteA_0_0_52_0 + 0) = a_0_0_52_0;
    *(localWriteA_0_0_53_0 + 0) = a_0_0_53_0;
    *(localWriteA_0_0_54_0 + 0) = a_0_0_54_0;
    *(localWriteA_0_0_55_0 + 0) = a_0_0_55_0;
    *(localWriteA_0_0_56_0 + 0) = a_0_0_56_0;
    *(localWriteA_0_0_57_0 + 0) = a_0_0_57_0;
    *(localWriteA_0_0_58_0 + 0) = a_0_0_58_0;
    *(localWriteA_0_0_59_0 + 0) = a_0_0_59_0;
    *(localWriteA_0_0_60_0 + 0) = a_0_0_60_0;
    *(localWriteA_0_0_61_0 + 0) = a_0_0_61_0;
    *(localWriteA_0_0_62_0 + 0) = a_0_0_62_0;
    *(localWriteA_0_0_63_0 + 0) = a_0_0_63_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 15 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 16 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 17 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 18 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 19 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 20 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 21 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 22 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 23 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 24 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 25 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 26 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 27 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 28 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 29 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 30 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 31 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 32 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 33 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 34 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 35 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 36 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 37 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 38 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 39 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 40 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 41 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 42 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 43 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 44 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 45 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 46 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 47 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 48 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 49 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 50 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 51 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 52 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 53 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 54 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 55 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 56 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 57 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 58 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 59 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 60 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 61 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 62 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_1x1


    /* iter 63 */

    MAC_1x1_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_0_8_0 = ( globalReadOffsetAL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_0_0_9_0 = ( globalReadOffsetAL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_0_0_10_0 = ( globalReadOffsetAL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_0_0_11_0 = ( globalReadOffsetAL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_0_0_12_0 = ( globalReadOffsetAL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_0_0_13_0 = ( globalReadOffsetAL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_0_0_14_0 = ( globalReadOffsetAL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_0_0_15_0 = ( globalReadOffsetAL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);
  a_0_0_16_0 = ( globalReadOffsetAL_16_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_16_0 + 0);
  a_0_0_17_0 = ( globalReadOffsetAL_17_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_17_0 + 0);
  a_0_0_18_0 = ( globalReadOffsetAL_18_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_18_0 + 0);
  a_0_0_19_0 = ( globalReadOffsetAL_19_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_19_0 + 0);
  a_0_0_20_0 = ( globalReadOffsetAL_20_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_20_0 + 0);
  a_0_0_21_0 = ( globalReadOffsetAL_21_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_21_0 + 0);
  a_0_0_22_0 = ( globalReadOffsetAL_22_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_22_0 + 0);
  a_0_0_23_0 = ( globalReadOffsetAL_23_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_23_0 + 0);
  a_0_0_24_0 = ( globalReadOffsetAL_24_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_24_0 + 0);
  a_0_0_25_0 = ( globalReadOffsetAL_25_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_25_0 + 0);
  a_0_0_26_0 = ( globalReadOffsetAL_26_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_26_0 + 0);
  a_0_0_27_0 = ( globalReadOffsetAL_27_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_27_0 + 0);
  a_0_0_28_0 = ( globalReadOffsetAL_28_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_28_0 + 0);
  a_0_0_29_0 = ( globalReadOffsetAL_29_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_29_0 + 0);
  a_0_0_30_0 = ( globalReadOffsetAL_30_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_30_0 + 0);
  a_0_0_31_0 = ( globalReadOffsetAL_31_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_31_0 + 0);
  a_0_0_32_0 = ( globalReadOffsetAL_32_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_32_0 + 0);
  a_0_0_33_0 = ( globalReadOffsetAL_33_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_33_0 + 0);
  a_0_0_34_0 = ( globalReadOffsetAL_34_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_34_0 + 0);
  a_0_0_35_0 = ( globalReadOffsetAL_35_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_35_0 + 0);
  a_0_0_36_0 = ( globalReadOffsetAL_36_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_36_0 + 0);
  a_0_0_37_0 = ( globalReadOffsetAL_37_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_37_0 + 0);
  a_0_0_38_0 = ( globalReadOffsetAL_38_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_38_0 + 0);
  a_0_0_39_0 = ( globalReadOffsetAL_39_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_39_0 + 0);
  a_0_0_40_0 = ( globalReadOffsetAL_40_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_40_0 + 0);
  a_0_0_41_0 = ( globalReadOffsetAL_41_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_41_0 + 0);
  a_0_0_42_0 = ( globalReadOffsetAL_42_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_42_0 + 0);
  a_0_0_43_0 = ( globalReadOffsetAL_43_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_43_0 + 0);
  a_0_0_44_0 = ( globalReadOffsetAL_44_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_44_0 + 0);
  a_0_0_45_0 = ( globalReadOffsetAL_45_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_45_0 + 0);
  a_0_0_46_0 = ( globalReadOffsetAL_46_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_46_0 + 0);
  a_0_0_47_0 = ( globalReadOffsetAL_47_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_47_0 + 0);
  a_0_0_48_0 = ( globalReadOffsetAL_48_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_48_0 + 0);
  a_0_0_49_0 = ( globalReadOffsetAL_49_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_49_0 + 0);
  a_0_0_50_0 = ( globalReadOffsetAL_50_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_50_0 + 0);
  a_0_0_51_0 = ( globalReadOffsetAL_51_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_51_0 + 0);
  a_0_0_52_0 = ( globalReadOffsetAL_52_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_52_0 + 0);
  a_0_0_53_0 = ( globalReadOffsetAL_53_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_53_0 + 0);
  a_0_0_54_0 = ( globalReadOffsetAL_54_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_54_0 + 0);
  a_0_0_55_0 = ( globalReadOffsetAL_55_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_55_0 + 0);
  a_0_0_56_0 = ( globalReadOffsetAL_56_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_56_0 + 0);
  a_0_0_57_0 = ( globalReadOffsetAL_57_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_57_0 + 0);
  a_0_0_58_0 = ( globalReadOffsetAL_58_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_58_0 + 0);
  a_0_0_59_0 = ( globalReadOffsetAL_59_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_59_0 + 0);
  a_0_0_60_0 = ( globalReadOffsetAL_60_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_60_0 + 0);
  a_0_0_61_0 = ( globalReadOffsetAL_61_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_61_0 + 0);
  a_0_0_62_0 = ( globalReadOffsetAL_62_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_62_0 + 0);
  a_0_0_63_0 = ( globalReadOffsetAL_63_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_63_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);
  localWriteA_0_0_16_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_16_0);
  localWriteA_0_0_17_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_17_0);
  localWriteA_0_0_18_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_18_0);
  localWriteA_0_0_19_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_19_0);
  localWriteA_0_0_20_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_20_0);
  localWriteA_0_0_21_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_21_0);
  localWriteA_0_0_22_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_22_0);
  localWriteA_0_0_23_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_23_0);
  localWriteA_0_0_24_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_24_0);
  localWriteA_0_0_25_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_25_0);
  localWriteA_0_0_26_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_26_0);
  localWriteA_0_0_27_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_27_0);
  localWriteA_0_0_28_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_28_0);
  localWriteA_0_0_29_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_29_0);
  localWriteA_0_0_30_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_30_0);
  localWriteA_0_0_31_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_31_0);
  localWriteA_0_0_32_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_32_0);
  localWriteA_0_0_33_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_33_0);
  localWriteA_0_0_34_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_34_0);
  localWriteA_0_0_35_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_35_0);
  localWriteA_0_0_36_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_36_0);
  localWriteA_0_0_37_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_37_0);
  localWriteA_0_0_38_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_38_0);
  localWriteA_0_0_39_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_39_0);
  localWriteA_0_0_40_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_40_0);
  localWriteA_0_0_41_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_41_0);
  localWriteA_0_0_42_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_42_0);
  localWriteA_0_0_43_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_43_0);
  localWriteA_0_0_44_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_44_0);
  localWriteA_0_0_45_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_45_0);
  localWriteA_0_0_46_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_46_0);
  localWriteA_0_0_47_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_47_0);
  localWriteA_0_0_48_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_48_0);
  localWriteA_0_0_49_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_49_0);
  localWriteA_0_0_50_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_50_0);
  localWriteA_0_0_51_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_51_0);
  localWriteA_0_0_52_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_52_0);
  localWriteA_0_0_53_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_53_0);
  localWriteA_0_0_54_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_54_0);
  localWriteA_0_0_55_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_55_0);
  localWriteA_0_0_56_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_56_0);
  localWriteA_0_0_57_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_57_0);
  localWriteA_0_0_58_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_58_0);
  localWriteA_0_0_59_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_59_0);
  localWriteA_0_0_60_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_60_0);
  localWriteA_0_0_61_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_61_0);
  localWriteA_0_0_62_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_62_0);
  localWriteA_0_0_63_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_63_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
  *(localWriteA_0_0_16_0 + 0) = a_0_0_16_0;
  *(localWriteA_0_0_17_0 + 0) = a_0_0_17_0;
  *(localWriteA_0_0_18_0 + 0) = a_0_0_18_0;
  *(localWriteA_0_0_19_0 + 0) = a_0_0_19_0;
  *(localWriteA_0_0_20_0 + 0) = a_0_0_20_0;
  *(localWriteA_0_0_21_0 + 0) = a_0_0_21_0;
  *(localWriteA_0_0_22_0 + 0) = a_0_0_22_0;
  *(localWriteA_0_0_23_0 + 0) = a_0_0_23_0;
  *(localWriteA_0_0_24_0 + 0) = a_0_0_24_0;
  *(localWriteA_0_0_25_0 + 0) = a_0_0_25_0;
  *(localWriteA_0_0_26_0 + 0) = a_0_0_26_0;
  *(localWriteA_0_0_27_0 + 0) = a_0_0_27_0;
  *(localWriteA_0_0_28_0 + 0) = a_0_0_28_0;
  *(localWriteA_0_0_29_0 + 0) = a_0_0_29_0;
  *(localWriteA_0_0_30_0 + 0) = a_0_0_30_0;
  *(localWriteA_0_0_31_0 + 0) = a_0_0_31_0;
  *(localWriteA_0_0_32_0 + 0) = a_0_0_32_0;
  *(localWriteA_0_0_33_0 + 0) = a_0_0_33_0;
  *(localWriteA_0_0_34_0 + 0) = a_0_0_34_0;
  *(localWriteA_0_0_35_0 + 0) = a_0_0_35_0;
  *(localWriteA_0_0_36_0 + 0) = a_0_0_36_0;
  *(localWriteA_0_0_37_0 + 0) = a_0_0_37_0;
  *(localWriteA_0_0_38_0 + 0) = a_0_0_38_0;
  *(localWriteA_0_0_39_0 + 0) = a_0_0_39_0;
  *(localWriteA_0_0_40_0 + 0) = a_0_0_40_0;
  *(localWriteA_0_0_41_0 + 0) = a_0_0_41_0;
  *(localWriteA_0_0_42_0 + 0) = a_0_0_42_0;
  *(localWriteA_0_0_43_0 + 0) = a_0_0_43_0;
  *(localWriteA_0_0_44_0 + 0) = a_0_0_44_0;
  *(localWriteA_0_0_45_0 + 0) = a_0_0_45_0;
  *(localWriteA_0_0_46_0 + 0) = a_0_0_46_0;
  *(localWriteA_0_0_47_0 + 0) = a_0_0_47_0;
  *(localWriteA_0_0_48_0 + 0) = a_0_0_48_0;
  *(localWriteA_0_0_49_0 + 0) = a_0_0_49_0;
  *(localWriteA_0_0_50_0 + 0) = a_0_0_50_0;
  *(localWriteA_0_0_51_0 + 0) = a_0_0_51_0;
  *(localWriteA_0_0_52_0 + 0) = a_0_0_52_0;
  *(localWriteA_0_0_53_0 + 0) = a_0_0_53_0;
  *(localWriteA_0_0_54_0 + 0) = a_0_0_54_0;
  *(localWriteA_0_0_55_0 + 0) = a_0_0_55_0;
  *(localWriteA_0_0_56_0 + 0) = a_0_0_56_0;
  *(localWriteA_0_0_57_0 + 0) = a_0_0_57_0;
  *(localWriteA_0_0_58_0 + 0) = a_0_0_58_0;
  *(localWriteA_0_0_59_0 + 0) = a_0_0_59_0;
  *(localWriteA_0_0_60_0 + 0) = a_0_0_60_0;
  *(localWriteA_0_0_61_0 + 0) = a_0_0_61_0;
  *(localWriteA_0_0_62_0 + 0) = a_0_0_62_0;
  *(localWriteA_0_0_63_0 + 0) = a_0_0_63_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_1x1

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_1x1
#undef MAC_1x1_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 32
#define SG1J 4
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 3200
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_4xi8BH_MT128x16x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG32_4_1_WGM1(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_2_1_0;
  DATA_TYPE a_0_3_1_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_0_2;
  DATA_TYPE *localWriteA_0_0_0_3;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;
  DATA_TYPE *localWriteA_0_0_1_2;
  DATA_TYPE *localWriteA_0_0_1_3;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
  localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_1_2 + 0) = a_0_2_1_0;
    *(localWriteA_0_0_1_3 + 0) = a_0_3_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_2 = (localWriteOffsetA_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_3 = (localWriteOffsetA_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
    localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_1_2 + 0) = a_0_2_1_0;
    *(localWriteA_0_0_1_3 + 0) = a_0_3_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_2 = (localWriteOffsetA_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_3 = (localWriteOffsetA_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
    localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 7 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_3 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_2 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_3 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_2_1_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 2);
  a_0_3_1_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
  localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
  *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
  *(localWriteA_0_0_1_2 + 0) = a_0_2_1_0;
  *(localWriteA_0_0_1_3 + 0) = a_0_3_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_4xi8BH_MT32x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM1(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1792
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x4 micro-tile */
#define MAC_2x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \

#define MAC_2x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_4xi8BH_MT32x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_4_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM1(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x4


    /* iter 7 (last unrolled loop) */

    MAC_2x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x4

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x4
#undef MAC_2x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 8
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 128
#define LDS_NUM_ELEMENTS 896
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_4xi8BH_MT16x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_16_1_WGM1(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 4

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_4xi8BH_MT32x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM4(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 4

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I]); \
  TYPE_MAC(rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I]); \
  TYPE_MAC(rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I]); \
  TYPE_MAC(rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I]); \
  TYPE_MAC(rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I]); \
  TYPE_MAC(rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I]); \
  TYPE_MAC(rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I]); \
  TYPE_MAC(rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I]); \
  TYPE_MAC(rA[7],rB[7],rC[7+7*TT0I]); \

#define MAC_8x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[4+TT1J],rC[4+4*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[4+TT1J],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[4+TT1J],rC[6+4*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[4+TT1J],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[5+TT1J],rC[4+5*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[5+TT1J],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[5+TT1J],rC[6+5*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[5+TT1J],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[6+TT1J],rC[4+6*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[6+TT1J],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[6+TT1J],rC[6+6*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[6+TT1J],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[7+TT1J],rC[4+7*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[7+TT1J],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[7+TT1J],rC[6+7*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[7+TT1J],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_4xi8BH_MT128x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM4(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_0_2;
  DATA_TYPE *localWriteA_0_0_0_3;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8


    /* iter 7 */

    MAC_8x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
  *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
  *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }





  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 2) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 3) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef MAC_8x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 64
#define LDS_NUM_ELEMENTS 409
#define LDS_OFFSET_BLK 128

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_4xi8BH_MT16x16x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 3 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 3 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 2
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 64
#define LDS_NUM_ELEMENTS 409
#define LDS_OFFSET_BLK 128

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_4xi8BH_MT32x32x2_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 1 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 1 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE TensileComplexFloat
#define DEST_DATA_TYPE TensileComplexFloat
#define COMPUTE_DATA_TYPE TensileComplexFloat
#define s0 x
#define s1 y
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) \
  DST.s0 = MAC(  MULA.s0, MULB.s0, DST.s0 ); \
  DST.s0 = MAC( -MULA.s1, MULB.s1, DST.s0 ); \
  DST.s1 = MAC(  MULA.s0, MULB.s1, DST.s1 ); \
  DST.s1 = MAC(  MULA.s1, MULB.s0, DST.s1 );
#define TYPE_MAC_WRITE( DST, SRC, ALPHA, REG, BETA ) \
  /* (1) */ \
  type_mac_tmp = REG.s0; \
  REG.s0 *= ALPHA.s0; \
  REG.s0 = MAC( -ALPHA.s1, REG.s1, REG.s0 ); \
  REG.s1 *= ALPHA.s0; \
  REG.s1 = MAC(  ALPHA.s1, type_mac_tmp, REG.s1 ); \
  /* (2) */ \
  if(BETA.s0 != 0) { \
  REG.s0 = MAC(  BETA.s0, SRC.s0, REG.s0 ); \
  REG.s1 = MAC(  BETA.s0, SRC.s1, REG.s1 ); \
  } \
  if (BETA.s1 != 0) { \
  REG.s0 = MAC( -BETA.s1, SRC.s1, REG.s0 ); \
  REG.s1 = MAC(  BETA.s1, SRC.s0, REG.s1 ); \
  } \
  /* (3) */ \
  DST = REG;

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_CB_MT32x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW2_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  TensileComplexFloat *D,
  TensileComplexFloat const * __restrict__ C,
  TensileComplexFloat const * __restrict__ A,
  TensileComplexFloat const * __restrict__ B,
  TensileComplexFloat const alpha,
  TensileComplexFloat const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (TensileComplexFloat)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == tensile_complex<float>(0.0f)) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }



  float type_mac_tmp;


  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE TensileComplexFloat
#define DEST_DATA_TYPE TensileComplexFloat
#define COMPUTE_DATA_TYPE TensileComplexFloat
#define s0 x
#define s1 y
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) \
  DST.s0 = MAC(  MULA.s0, MULB.s0, DST.s0 ); \
  DST.s0 = MAC( -MULA.s1, MULB.s1, DST.s0 ); \
  DST.s1 = MAC(  MULA.s0, MULB.s1, DST.s1 ); \
  DST.s1 = MAC(  MULA.s1, MULB.s0, DST.s1 );
#define TYPE_MAC_WRITE( DST, SRC, ALPHA, REG, BETA ) \
  /* (1) */ \
  type_mac_tmp = REG.s0; \
  REG.s0 *= ALPHA.s0; \
  REG.s0 = MAC( -ALPHA.s1, REG.s1, REG.s0 ); \
  REG.s1 *= ALPHA.s0; \
  REG.s1 = MAC(  ALPHA.s1, type_mac_tmp, REG.s1 ); \
  /* (2) */ \
  if(BETA.s0 != 0) { \
  REG.s0 = MAC(  BETA.s0, SRC.s0, REG.s0 ); \
  REG.s1 = MAC(  BETA.s0, SRC.s1, REG.s1 ); \
  } \
  if (BETA.s1 != 0) { \
  REG.s0 = MAC( -BETA.s1, SRC.s1, REG.s0 ); \
  REG.s1 = MAC(  BETA.s1, SRC.s0, REG.s1 ); \
  } \
  /* (3) */ \
  DST = REG;

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_CB_MT32x32x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW2_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  TensileComplexFloat *D,
  TensileComplexFloat const * __restrict__ C,
  TensileComplexFloat const * __restrict__ A,
  TensileComplexFloat const * __restrict__ B,
  TensileComplexFloat const alpha,
  TensileComplexFloat const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (TensileComplexFloat)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == tensile_complex<float>(0.0f)) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }



  float type_mac_tmp;


  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 128
#define LDS_NUM_ELEMENTS 512
#define LDS_OFFSET_BLK 256

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_4xi8BH_MT16x16x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 64
#define LDS_NUM_ELEMENTS 409
#define LDS_OFFSET_BLK 128

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_4xi8BH_MT16x16x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 3 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 3 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 2
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 64
#define LDS_NUM_ELEMENTS 409
#define LDS_OFFSET_BLK 128

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_4xi8BH_MT32x32x2_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 1 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 1 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE TensileComplexDouble
#define DEST_DATA_TYPE TensileComplexDouble
#define COMPUTE_DATA_TYPE TensileComplexDouble
#define s0 x
#define s1 y
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) \
  DST.s0 = MAC(  MULA.s0, MULB.s0, DST.s0 ); \
  DST.s0 = MAC(  MULA.s1, MULB.s1, DST.s0 ); \
  DST.s1 = MAC(  MULA.s0, MULB.s1, DST.s1 ); \
  DST.s1 = MAC( -MULA.s1, MULB.s0, DST.s1 );
#define TYPE_MAC_WRITE( DST, SRC, ALPHA, REG, BETA ) \
  /* (1) */ \
  type_mac_tmp = REG.s0; \
  REG.s0 *= ALPHA.s0; \
  REG.s0 = MAC( -ALPHA.s1, REG.s1, REG.s0 ); \
  REG.s1 *= ALPHA.s0; \
  REG.s1 = MAC(  ALPHA.s1, type_mac_tmp, REG.s1 ); \
  /* (2) */ \
  if(BETA.s0 != 0) { \
  REG.s0 = MAC(  BETA.s0, SRC.s0, REG.s0 ); \
  REG.s1 = MAC(  BETA.s0, SRC.s1, REG.s1 ); \
  } \
  if (BETA.s1 != 0) { \
  REG.s0 = MAC( -BETA.s1, SRC.s1, REG.s0 ); \
  REG.s1 = MAC(  BETA.s1, SRC.s0, REG.s1 ); \
  } \
  /* (3) */ \
  DST = REG;

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_AlikC_Bjlk_ZB_MT32x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW1_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  TensileComplexDouble *D,
  TensileComplexDouble const * __restrict__ C,
  TensileComplexDouble const * __restrict__ A,
  TensileComplexDouble const * __restrict__ B,
  TensileComplexDouble const alpha,
  TensileComplexDouble const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (TensileComplexDouble)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == tensile_complex<double>(0.0)) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 1); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }



  double type_mac_tmp;


  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE TensileComplexDouble
#define DEST_DATA_TYPE TensileComplexDouble
#define COMPUTE_DATA_TYPE TensileComplexDouble
#define s0 x
#define s1 y
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) \
  DST.s0 = MAC(  MULA.s0, MULB.s0, DST.s0 ); \
  DST.s0 = MAC(  MULA.s1, MULB.s1, DST.s0 ); \
  DST.s1 = MAC(  MULA.s0, MULB.s1, DST.s1 ); \
  DST.s1 = MAC( -MULA.s1, MULB.s0, DST.s1 );
#define TYPE_MAC_WRITE( DST, SRC, ALPHA, REG, BETA ) \
  /* (1) */ \
  type_mac_tmp = REG.s0; \
  REG.s0 *= ALPHA.s0; \
  REG.s0 = MAC( -ALPHA.s1, REG.s1, REG.s0 ); \
  REG.s1 *= ALPHA.s0; \
  REG.s1 = MAC(  ALPHA.s1, type_mac_tmp, REG.s1 ); \
  /* (2) */ \
  if(BETA.s0 != 0) { \
  REG.s0 = MAC(  BETA.s0, SRC.s0, REG.s0 ); \
  REG.s1 = MAC(  BETA.s0, SRC.s1, REG.s1 ); \
  } \
  if (BETA.s1 != 0) { \
  REG.s0 = MAC( -BETA.s1, SRC.s1, REG.s0 ); \
  REG.s1 = MAC(  BETA.s1, SRC.s0, REG.s1 ); \
  } \
  /* (3) */ \
  DST = REG;

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_AlikC_Bjlk_ZB_MT32x32x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW1_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  TensileComplexDouble *D,
  TensileComplexDouble const * __restrict__ C,
  TensileComplexDouble const * __restrict__ A,
  TensileComplexDouble const * __restrict__ B,
  TensileComplexDouble const alpha,
  TensileComplexDouble const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (TensileComplexDouble)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == tensile_complex<double>(0.0)) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 1); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }



  double type_mac_tmp;


  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE TensileComplexFloat
#define DEST_DATA_TYPE TensileComplexFloat
#define COMPUTE_DATA_TYPE TensileComplexFloat
#define s0 x
#define s1 y
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) \
  DST.s0 = MAC(  MULA.s0,  MULB.s0, DST.s0 ); \
  DST.s0 = MAC( -MULA.s1, -MULB.s1, DST.s0 ); \
  DST.s1 = MAC(  MULA.s0, -MULB.s1, DST.s1 ); \
  DST.s1 = MAC(  MULA.s1,  MULB.s0, DST.s1 );
#define TYPE_MAC_WRITE( DST, SRC, ALPHA, REG, BETA ) \
  /* (1) */ \
  type_mac_tmp = REG.s0; \
  REG.s0 *= ALPHA.s0; \
  REG.s0 = MAC( -ALPHA.s1, REG.s1, REG.s0 ); \
  REG.s1 *= ALPHA.s0; \
  REG.s1 = MAC(  ALPHA.s1, type_mac_tmp, REG.s1 ); \
  /* (2) */ \
  if(BETA.s0 != 0) { \
  REG.s0 = MAC(  BETA.s0, SRC.s0, REG.s0 ); \
  REG.s1 = MAC(  BETA.s0, SRC.s1, REG.s1 ); \
  } \
  if (BETA.s1 != 0) { \
  REG.s0 = MAC( -BETA.s1, SRC.s1, REG.s0 ); \
  REG.s1 = MAC(  BETA.s1, SRC.s0, REG.s1 ); \
  } \
  /* (3) */ \
  DST = REG;

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_BjlkC_CB_MT32x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW2_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  TensileComplexFloat *D,
  TensileComplexFloat const * __restrict__ C,
  TensileComplexFloat const * __restrict__ A,
  TensileComplexFloat const * __restrict__ B,
  TensileComplexFloat const alpha,
  TensileComplexFloat const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (TensileComplexFloat)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == tensile_complex<float>(0.0f)) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }



  float type_mac_tmp;


  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE TensileComplexFloat
#define DEST_DATA_TYPE TensileComplexFloat
#define COMPUTE_DATA_TYPE TensileComplexFloat
#define s0 x
#define s1 y
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) \
  DST.s0 = MAC(  MULA.s0,  MULB.s0, DST.s0 ); \
  DST.s0 = MAC( -MULA.s1, -MULB.s1, DST.s0 ); \
  DST.s1 = MAC(  MULA.s0, -MULB.s1, DST.s1 ); \
  DST.s1 = MAC(  MULA.s1,  MULB.s0, DST.s1 );
#define TYPE_MAC_WRITE( DST, SRC, ALPHA, REG, BETA ) \
  /* (1) */ \
  type_mac_tmp = REG.s0; \
  REG.s0 *= ALPHA.s0; \
  REG.s0 = MAC( -ALPHA.s1, REG.s1, REG.s0 ); \
  REG.s1 *= ALPHA.s0; \
  REG.s1 = MAC(  ALPHA.s1, type_mac_tmp, REG.s1 ); \
  /* (2) */ \
  if(BETA.s0 != 0) { \
  REG.s0 = MAC(  BETA.s0, SRC.s0, REG.s0 ); \
  REG.s1 = MAC(  BETA.s0, SRC.s1, REG.s1 ); \
  } \
  if (BETA.s1 != 0) { \
  REG.s0 = MAC( -BETA.s1, SRC.s1, REG.s0 ); \
  REG.s1 = MAC(  BETA.s1, SRC.s0, REG.s1 ); \
  } \
  /* (3) */ \
  DST = REG;

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_BjlkC_CB_MT32x32x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW2_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  TensileComplexFloat *D,
  TensileComplexFloat const * __restrict__ C,
  TensileComplexFloat const * __restrict__ A,
  TensileComplexFloat const * __restrict__ B,
  TensileComplexFloat const alpha,
  TensileComplexFloat const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (TensileComplexFloat)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == tensile_complex<float>(0.0f)) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }



  float type_mac_tmp;


  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 819

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HBH_MT64x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 7 */

    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HBH_MT128x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 1) {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 2) {
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HBH_MT64x16x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_1_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_1_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_1_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_1_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_1_4_0 = *(globalReadA_0_0_4_0 + 1);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_1_5_0 = *(globalReadA_0_0_5_0 + 1);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_1_6_0 = *(globalReadA_0_0_6_0 + 1);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_1_7_0 = *(globalReadA_0_0_7_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_4_0 + 1) = a_0_1_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_5_0 + 1) = a_0_1_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_6_0 + 1) = a_0_1_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_7_0 + 1) = a_0_1_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_1_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 1);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_1_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 1);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_1_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 1);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_1_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_4_0 + 1) = a_0_1_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_5_0 + 1) = a_0_1_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_6_0 + 1) = a_0_1_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_7_0 + 1) = a_0_1_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 1) {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 2) {
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HBH_MT128x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 8

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x8 micro-tile */
#define MAC_2x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \

#define MAC_2x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HBH_MT32x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;
  unsigned int flattenedOffsetB_4_0 = globalReadOffsetB1J + 0 + 4*LSPB;
  flattenedOffsetB_4_0 = (flattenedOffsetB_4_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_4_0;
  unsigned int globalReadOffsetB1J_4_0 = flattenedOffsetB_4_0;
  unsigned int flattenedOffsetB_5_0 = globalReadOffsetB1J + 0 + 5*LSPB;
  flattenedOffsetB_5_0 = (flattenedOffsetB_5_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_5_0;
  unsigned int globalReadOffsetB1J_5_0 = flattenedOffsetB_5_0;
  unsigned int flattenedOffsetB_6_0 = globalReadOffsetB1J + 0 + 6*LSPB;
  flattenedOffsetB_6_0 = (flattenedOffsetB_6_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_6_0;
  unsigned int globalReadOffsetB1J_6_0 = flattenedOffsetB_6_0;
  unsigned int flattenedOffsetB_7_0 = globalReadOffsetB1J + 0 + 7*LSPB;
  flattenedOffsetB_7_0 = (flattenedOffsetB_7_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_7_0;
  unsigned int globalReadOffsetB1J_7_0 = flattenedOffsetB_7_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;
  globalReadOffsetB1J_4_0 = (  globalReadOffsetB1J_4_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_4_0;
  globalReadOffsetB1J_5_0 = (  globalReadOffsetB1J_5_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_5_0;
  globalReadOffsetB1J_6_0 = (  globalReadOffsetB1J_6_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_6_0;
  globalReadOffsetB1J_7_0 = (  globalReadOffsetB1J_7_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_7_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_7_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 4*LSPB);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 5*LSPB);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 6*LSPB);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 7*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x8


    /* iter 15 */

    MAC_2x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x8
#undef MAC_2x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 16
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HBH_MT64x16x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;
  unsigned int globalReadOffsetAL_8_0 = globalReadOffsetAL + 0 + 8*LSPA;
  unsigned int globalReadOffsetAL_9_0 = globalReadOffsetAL + 0 + 9*LSPA;
  unsigned int globalReadOffsetAL_10_0 = globalReadOffsetAL + 0 + 10*LSPA;
  unsigned int globalReadOffsetAL_11_0 = globalReadOffsetAL + 0 + 11*LSPA;
  unsigned int globalReadOffsetAL_12_0 = globalReadOffsetAL + 0 + 12*LSPA;
  unsigned int globalReadOffsetAL_13_0 = globalReadOffsetAL + 0 + 13*LSPA;
  unsigned int globalReadOffsetAL_14_0 = globalReadOffsetAL + 0 + 14*LSPA;
  unsigned int globalReadOffsetAL_15_0 = globalReadOffsetAL + 0 + 15*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_15_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 8*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 9*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 10*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 11*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 12*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 13*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 14*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 15*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_0_0_15_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_8_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_9_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_10_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_11_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_12_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_13_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_14_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_15_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_8_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_9_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_10_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_11_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_12_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_13_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_14_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_15_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_0_8_0 = ( globalReadOffsetAL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_0_0_9_0 = ( globalReadOffsetAL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_0_0_10_0 = ( globalReadOffsetAL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_0_0_11_0 = ( globalReadOffsetAL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_0_0_12_0 = ( globalReadOffsetAL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_0_0_13_0 = ( globalReadOffsetAL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_0_0_14_0 = ( globalReadOffsetAL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_0_0_15_0 = ( globalReadOffsetAL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 16

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x8 micro-tile */
#define MAC_2x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \

#define MAC_2x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HBH_MT16x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;
  DATA_TYPE b_0_0_8_0;
  DATA_TYPE b_0_0_9_0;
  DATA_TYPE b_0_0_10_0;
  DATA_TYPE b_0_0_11_0;
  DATA_TYPE b_0_0_12_0;
  DATA_TYPE b_0_0_13_0;
  DATA_TYPE b_0_0_14_0;
  DATA_TYPE b_0_0_15_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;
  unsigned int flattenedOffsetB_4_0 = globalReadOffsetB1J + 0 + 4*LSPB;
  flattenedOffsetB_4_0 = (flattenedOffsetB_4_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_4_0;
  unsigned int globalReadOffsetB1J_4_0 = flattenedOffsetB_4_0;
  unsigned int flattenedOffsetB_5_0 = globalReadOffsetB1J + 0 + 5*LSPB;
  flattenedOffsetB_5_0 = (flattenedOffsetB_5_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_5_0;
  unsigned int globalReadOffsetB1J_5_0 = flattenedOffsetB_5_0;
  unsigned int flattenedOffsetB_6_0 = globalReadOffsetB1J + 0 + 6*LSPB;
  flattenedOffsetB_6_0 = (flattenedOffsetB_6_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_6_0;
  unsigned int globalReadOffsetB1J_6_0 = flattenedOffsetB_6_0;
  unsigned int flattenedOffsetB_7_0 = globalReadOffsetB1J + 0 + 7*LSPB;
  flattenedOffsetB_7_0 = (flattenedOffsetB_7_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_7_0;
  unsigned int globalReadOffsetB1J_7_0 = flattenedOffsetB_7_0;
  unsigned int flattenedOffsetB_8_0 = globalReadOffsetB1J + 0 + 8*LSPB;
  flattenedOffsetB_8_0 = (flattenedOffsetB_8_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_8_0;
  unsigned int globalReadOffsetB1J_8_0 = flattenedOffsetB_8_0;
  unsigned int flattenedOffsetB_9_0 = globalReadOffsetB1J + 0 + 9*LSPB;
  flattenedOffsetB_9_0 = (flattenedOffsetB_9_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_9_0;
  unsigned int globalReadOffsetB1J_9_0 = flattenedOffsetB_9_0;
  unsigned int flattenedOffsetB_10_0 = globalReadOffsetB1J + 0 + 10*LSPB;
  flattenedOffsetB_10_0 = (flattenedOffsetB_10_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_10_0;
  unsigned int globalReadOffsetB1J_10_0 = flattenedOffsetB_10_0;
  unsigned int flattenedOffsetB_11_0 = globalReadOffsetB1J + 0 + 11*LSPB;
  flattenedOffsetB_11_0 = (flattenedOffsetB_11_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_11_0;
  unsigned int globalReadOffsetB1J_11_0 = flattenedOffsetB_11_0;
  unsigned int flattenedOffsetB_12_0 = globalReadOffsetB1J + 0 + 12*LSPB;
  flattenedOffsetB_12_0 = (flattenedOffsetB_12_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_12_0;
  unsigned int globalReadOffsetB1J_12_0 = flattenedOffsetB_12_0;
  unsigned int flattenedOffsetB_13_0 = globalReadOffsetB1J + 0 + 13*LSPB;
  flattenedOffsetB_13_0 = (flattenedOffsetB_13_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_13_0;
  unsigned int globalReadOffsetB1J_13_0 = flattenedOffsetB_13_0;
  unsigned int flattenedOffsetB_14_0 = globalReadOffsetB1J + 0 + 14*LSPB;
  flattenedOffsetB_14_0 = (flattenedOffsetB_14_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_14_0;
  unsigned int globalReadOffsetB1J_14_0 = flattenedOffsetB_14_0;
  unsigned int flattenedOffsetB_15_0 = globalReadOffsetB1J + 0 + 15*LSPB;
  flattenedOffsetB_15_0 = (flattenedOffsetB_15_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_15_0;
  unsigned int globalReadOffsetB1J_15_0 = flattenedOffsetB_15_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;
  globalReadOffsetB1J_4_0 = (  globalReadOffsetB1J_4_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_4_0;
  globalReadOffsetB1J_5_0 = (  globalReadOffsetB1J_5_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_5_0;
  globalReadOffsetB1J_6_0 = (  globalReadOffsetB1J_6_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_6_0;
  globalReadOffsetB1J_7_0 = (  globalReadOffsetB1J_7_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_7_0;
  globalReadOffsetB1J_8_0 = (  globalReadOffsetB1J_8_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_8_0;
  globalReadOffsetB1J_9_0 = (  globalReadOffsetB1J_9_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_9_0;
  globalReadOffsetB1J_10_0 = (  globalReadOffsetB1J_10_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_10_0;
  globalReadOffsetB1J_11_0 = (  globalReadOffsetB1J_11_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_11_0;
  globalReadOffsetB1J_12_0 = (  globalReadOffsetB1J_12_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_12_0;
  globalReadOffsetB1J_13_0 = (  globalReadOffsetB1J_13_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_13_0;
  globalReadOffsetB1J_14_0 = (  globalReadOffsetB1J_14_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_14_0;
  globalReadOffsetB1J_15_0 = (  globalReadOffsetB1J_15_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_15_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_7_0), wgK );
  int64_t globalReadOffsetB_0_0_8_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_8_0), wgK );
  int64_t globalReadOffsetB_0_0_9_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_9_0), wgK );
  int64_t globalReadOffsetB_0_0_10_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_10_0), wgK );
  int64_t globalReadOffsetB_0_0_11_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_11_0), wgK );
  int64_t globalReadOffsetB_0_0_12_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_12_0), wgK );
  int64_t globalReadOffsetB_0_0_13_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_13_0), wgK );
  int64_t globalReadOffsetB_0_0_14_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_14_0), wgK );
  int64_t globalReadOffsetB_0_0_15_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_15_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;
  DATA_TYPE const *globalReadB_0_0_8_0 = B + globalReadOffsetB_0_0_8_0;
  DATA_TYPE const *globalReadB_0_0_9_0 = B + globalReadOffsetB_0_0_9_0;
  DATA_TYPE const *globalReadB_0_0_10_0 = B + globalReadOffsetB_0_0_10_0;
  DATA_TYPE const *globalReadB_0_0_11_0 = B + globalReadOffsetB_0_0_11_0;
  DATA_TYPE const *globalReadB_0_0_12_0 = B + globalReadOffsetB_0_0_12_0;
  DATA_TYPE const *globalReadB_0_0_13_0 = B + globalReadOffsetB_0_0_13_0;
  DATA_TYPE const *globalReadB_0_0_14_0 = B + globalReadOffsetB_0_0_14_0;
  DATA_TYPE const *globalReadB_0_0_15_0 = B + globalReadOffsetB_0_0_15_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 4*LSPB);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 5*LSPB);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 6*LSPB);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 7*LSPB);
  unsigned int localWriteOffsetB_0_0_8_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 8*LSPB);
  unsigned int localWriteOffsetB_0_0_9_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 9*LSPB);
  unsigned int localWriteOffsetB_0_0_10_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 10*LSPB);
  unsigned int localWriteOffsetB_0_0_11_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 11*LSPB);
  unsigned int localWriteOffsetB_0_0_12_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 12*LSPB);
  unsigned int localWriteOffsetB_0_0_13_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 13*LSPB);
  unsigned int localWriteOffsetB_0_0_14_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 14*LSPB);
  unsigned int localWriteOffsetB_0_0_15_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 15*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;
  DATA_TYPE *localWriteB_0_0_8_0;
  DATA_TYPE *localWriteB_0_0_9_0;
  DATA_TYPE *localWriteB_0_0_10_0;
  DATA_TYPE *localWriteB_0_0_11_0;
  DATA_TYPE *localWriteB_0_0_12_0;
  DATA_TYPE *localWriteB_0_0_13_0;
  DATA_TYPE *localWriteB_0_0_14_0;
  DATA_TYPE *localWriteB_0_0_15_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);
  localWriteB_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_8_0);
  localWriteB_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_9_0);
  localWriteB_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_10_0);
  localWriteB_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_11_0);
  localWriteB_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_12_0);
  localWriteB_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_13_0);
  localWriteB_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_14_0);
  localWriteB_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_15_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_8_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_9_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_10_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_11_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_12_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_13_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_14_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_15_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);
    b_0_0_8_0 = *(globalReadB_0_0_8_0 + 0);
    b_0_0_9_0 = *(globalReadB_0_0_9_0 + 0);
    b_0_0_10_0 = *(globalReadB_0_0_10_0 + 0);
    b_0_0_11_0 = *(globalReadB_0_0_11_0 + 0);
    b_0_0_12_0 = *(globalReadB_0_0_12_0 + 0);
    b_0_0_13_0 = *(globalReadB_0_0_13_0 + 0);
    b_0_0_14_0 = *(globalReadB_0_0_14_0 + 0);
    b_0_0_15_0 = *(globalReadB_0_0_15_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_8_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_8_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_9_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_9_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_10_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_10_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_11_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_11_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_12_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_12_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_13_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_13_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_14_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_14_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_15_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_15_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
    *(localWriteB_0_0_8_0 + 0) = b_0_0_8_0;
    *(localWriteB_0_0_9_0 + 0) = b_0_0_9_0;
    *(localWriteB_0_0_10_0 + 0) = b_0_0_10_0;
    *(localWriteB_0_0_11_0 + 0) = b_0_0_11_0;
    *(localWriteB_0_0_12_0 + 0) = b_0_0_12_0;
    *(localWriteB_0_0_13_0 + 0) = b_0_0_13_0;
    *(localWriteB_0_0_14_0 + 0) = b_0_0_14_0;
    *(localWriteB_0_0_15_0 + 0) = b_0_0_15_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x8


    /* iter 15 */

    MAC_2x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);
  b_0_0_8_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_8_0 + 0);
  b_0_0_9_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_9_0 + 0);
  b_0_0_10_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_10_0 + 0);
  b_0_0_11_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_11_0 + 0);
  b_0_0_12_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_12_0 + 0);
  b_0_0_13_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_13_0 + 0);
  b_0_0_14_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_14_0 + 0);
  b_0_0_15_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_15_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);
  localWriteB_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_8_0);
  localWriteB_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_9_0);
  localWriteB_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_10_0);
  localWriteB_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_11_0);
  localWriteB_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_12_0);
  localWriteB_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_13_0);
  localWriteB_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_14_0);
  localWriteB_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_15_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
  *(localWriteB_0_0_8_0 + 0) = b_0_0_8_0;
  *(localWriteB_0_0_9_0 + 0) = b_0_0_9_0;
  *(localWriteB_0_0_10_0 + 0) = b_0_0_10_0;
  *(localWriteB_0_0_11_0 + 0) = b_0_0_11_0;
  *(localWriteB_0_0_12_0 + 0) = b_0_0_12_0;
  *(localWriteB_0_0_13_0 + 0) = b_0_0_13_0;
  *(localWriteB_0_0_14_0 + 0) = b_0_0_14_0;
  *(localWriteB_0_0_15_0 + 0) = b_0_0_15_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x8
#undef MAC_2x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 819

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HBH_MT64x32x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 7 */

    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HBH_MT128x32x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 1) {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 2) {
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HBH_MT64x16x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_1_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_1_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_1_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_1_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_1_4_0 = *(globalReadA_0_0_4_0 + 1);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_1_5_0 = *(globalReadA_0_0_5_0 + 1);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_1_6_0 = *(globalReadA_0_0_6_0 + 1);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_1_7_0 = *(globalReadA_0_0_7_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_4_0 + 1) = a_0_1_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_5_0 + 1) = a_0_1_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_6_0 + 1) = a_0_1_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_7_0 + 1) = a_0_1_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_1_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 1);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_1_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 1);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_1_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 1);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_1_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_4_0 + 1) = a_0_1_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_5_0 + 1) = a_0_1_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_6_0 + 1) = a_0_1_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_7_0 + 1) = a_0_1_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 1) {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 2) {
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HBH_MT128x32x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 8

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x8 micro-tile */
#define MAC_2x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \

#define MAC_2x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HBH_MT32x128x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;
  unsigned int flattenedOffsetB_4_0 = globalReadOffsetB1J + 0 + 4*LSPB;
  flattenedOffsetB_4_0 = (flattenedOffsetB_4_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_4_0;
  unsigned int globalReadOffsetB1J_4_0 = flattenedOffsetB_4_0;
  unsigned int flattenedOffsetB_5_0 = globalReadOffsetB1J + 0 + 5*LSPB;
  flattenedOffsetB_5_0 = (flattenedOffsetB_5_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_5_0;
  unsigned int globalReadOffsetB1J_5_0 = flattenedOffsetB_5_0;
  unsigned int flattenedOffsetB_6_0 = globalReadOffsetB1J + 0 + 6*LSPB;
  flattenedOffsetB_6_0 = (flattenedOffsetB_6_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_6_0;
  unsigned int globalReadOffsetB1J_6_0 = flattenedOffsetB_6_0;
  unsigned int flattenedOffsetB_7_0 = globalReadOffsetB1J + 0 + 7*LSPB;
  flattenedOffsetB_7_0 = (flattenedOffsetB_7_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_7_0;
  unsigned int globalReadOffsetB1J_7_0 = flattenedOffsetB_7_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;
  globalReadOffsetB1J_4_0 = (  globalReadOffsetB1J_4_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_4_0;
  globalReadOffsetB1J_5_0 = (  globalReadOffsetB1J_5_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_5_0;
  globalReadOffsetB1J_6_0 = (  globalReadOffsetB1J_6_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_6_0;
  globalReadOffsetB1J_7_0 = (  globalReadOffsetB1J_7_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_7_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_7_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 4*LSPB);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 5*LSPB);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 6*LSPB);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 7*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x8


    /* iter 15 */

    MAC_2x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x8
#undef MAC_2x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 16
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HBH_MT64x16x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;
  unsigned int globalReadOffsetAL_8_0 = globalReadOffsetAL + 0 + 8*LSPA;
  unsigned int globalReadOffsetAL_9_0 = globalReadOffsetAL + 0 + 9*LSPA;
  unsigned int globalReadOffsetAL_10_0 = globalReadOffsetAL + 0 + 10*LSPA;
  unsigned int globalReadOffsetAL_11_0 = globalReadOffsetAL + 0 + 11*LSPA;
  unsigned int globalReadOffsetAL_12_0 = globalReadOffsetAL + 0 + 12*LSPA;
  unsigned int globalReadOffsetAL_13_0 = globalReadOffsetAL + 0 + 13*LSPA;
  unsigned int globalReadOffsetAL_14_0 = globalReadOffsetAL + 0 + 14*LSPA;
  unsigned int globalReadOffsetAL_15_0 = globalReadOffsetAL + 0 + 15*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_15_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 8*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 9*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 10*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 11*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 12*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 13*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 14*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 15*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_0_0_15_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_8_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_9_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_10_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_11_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_12_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_13_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_14_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_15_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_8_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_9_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_10_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_11_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_12_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_13_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_14_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_15_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_0_8_0 = ( globalReadOffsetAL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_0_0_9_0 = ( globalReadOffsetAL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_0_0_10_0 = ( globalReadOffsetAL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_0_0_11_0 = ( globalReadOffsetAL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_0_0_12_0 = ( globalReadOffsetAL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_0_0_13_0 = ( globalReadOffsetAL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_0_0_14_0 = ( globalReadOffsetAL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_0_0_15_0 = ( globalReadOffsetAL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 16

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x8 micro-tile */
#define MAC_2x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \

#define MAC_2x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HBH_MT16x64x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;
  DATA_TYPE b_0_0_8_0;
  DATA_TYPE b_0_0_9_0;
  DATA_TYPE b_0_0_10_0;
  DATA_TYPE b_0_0_11_0;
  DATA_TYPE b_0_0_12_0;
  DATA_TYPE b_0_0_13_0;
  DATA_TYPE b_0_0_14_0;
  DATA_TYPE b_0_0_15_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;
  unsigned int flattenedOffsetB_4_0 = globalReadOffsetB1J + 0 + 4*LSPB;
  flattenedOffsetB_4_0 = (flattenedOffsetB_4_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_4_0;
  unsigned int globalReadOffsetB1J_4_0 = flattenedOffsetB_4_0;
  unsigned int flattenedOffsetB_5_0 = globalReadOffsetB1J + 0 + 5*LSPB;
  flattenedOffsetB_5_0 = (flattenedOffsetB_5_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_5_0;
  unsigned int globalReadOffsetB1J_5_0 = flattenedOffsetB_5_0;
  unsigned int flattenedOffsetB_6_0 = globalReadOffsetB1J + 0 + 6*LSPB;
  flattenedOffsetB_6_0 = (flattenedOffsetB_6_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_6_0;
  unsigned int globalReadOffsetB1J_6_0 = flattenedOffsetB_6_0;
  unsigned int flattenedOffsetB_7_0 = globalReadOffsetB1J + 0 + 7*LSPB;
  flattenedOffsetB_7_0 = (flattenedOffsetB_7_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_7_0;
  unsigned int globalReadOffsetB1J_7_0 = flattenedOffsetB_7_0;
  unsigned int flattenedOffsetB_8_0 = globalReadOffsetB1J + 0 + 8*LSPB;
  flattenedOffsetB_8_0 = (flattenedOffsetB_8_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_8_0;
  unsigned int globalReadOffsetB1J_8_0 = flattenedOffsetB_8_0;
  unsigned int flattenedOffsetB_9_0 = globalReadOffsetB1J + 0 + 9*LSPB;
  flattenedOffsetB_9_0 = (flattenedOffsetB_9_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_9_0;
  unsigned int globalReadOffsetB1J_9_0 = flattenedOffsetB_9_0;
  unsigned int flattenedOffsetB_10_0 = globalReadOffsetB1J + 0 + 10*LSPB;
  flattenedOffsetB_10_0 = (flattenedOffsetB_10_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_10_0;
  unsigned int globalReadOffsetB1J_10_0 = flattenedOffsetB_10_0;
  unsigned int flattenedOffsetB_11_0 = globalReadOffsetB1J + 0 + 11*LSPB;
  flattenedOffsetB_11_0 = (flattenedOffsetB_11_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_11_0;
  unsigned int globalReadOffsetB1J_11_0 = flattenedOffsetB_11_0;
  unsigned int flattenedOffsetB_12_0 = globalReadOffsetB1J + 0 + 12*LSPB;
  flattenedOffsetB_12_0 = (flattenedOffsetB_12_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_12_0;
  unsigned int globalReadOffsetB1J_12_0 = flattenedOffsetB_12_0;
  unsigned int flattenedOffsetB_13_0 = globalReadOffsetB1J + 0 + 13*LSPB;
  flattenedOffsetB_13_0 = (flattenedOffsetB_13_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_13_0;
  unsigned int globalReadOffsetB1J_13_0 = flattenedOffsetB_13_0;
  unsigned int flattenedOffsetB_14_0 = globalReadOffsetB1J + 0 + 14*LSPB;
  flattenedOffsetB_14_0 = (flattenedOffsetB_14_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_14_0;
  unsigned int globalReadOffsetB1J_14_0 = flattenedOffsetB_14_0;
  unsigned int flattenedOffsetB_15_0 = globalReadOffsetB1J + 0 + 15*LSPB;
  flattenedOffsetB_15_0 = (flattenedOffsetB_15_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_15_0;
  unsigned int globalReadOffsetB1J_15_0 = flattenedOffsetB_15_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;
  globalReadOffsetB1J_4_0 = (  globalReadOffsetB1J_4_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_4_0;
  globalReadOffsetB1J_5_0 = (  globalReadOffsetB1J_5_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_5_0;
  globalReadOffsetB1J_6_0 = (  globalReadOffsetB1J_6_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_6_0;
  globalReadOffsetB1J_7_0 = (  globalReadOffsetB1J_7_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_7_0;
  globalReadOffsetB1J_8_0 = (  globalReadOffsetB1J_8_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_8_0;
  globalReadOffsetB1J_9_0 = (  globalReadOffsetB1J_9_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_9_0;
  globalReadOffsetB1J_10_0 = (  globalReadOffsetB1J_10_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_10_0;
  globalReadOffsetB1J_11_0 = (  globalReadOffsetB1J_11_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_11_0;
  globalReadOffsetB1J_12_0 = (  globalReadOffsetB1J_12_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_12_0;
  globalReadOffsetB1J_13_0 = (  globalReadOffsetB1J_13_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_13_0;
  globalReadOffsetB1J_14_0 = (  globalReadOffsetB1J_14_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_14_0;
  globalReadOffsetB1J_15_0 = (  globalReadOffsetB1J_15_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_15_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_7_0), wgK );
  int64_t globalReadOffsetB_0_0_8_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_8_0), wgK );
  int64_t globalReadOffsetB_0_0_9_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_9_0), wgK );
  int64_t globalReadOffsetB_0_0_10_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_10_0), wgK );
  int64_t globalReadOffsetB_0_0_11_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_11_0), wgK );
  int64_t globalReadOffsetB_0_0_12_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_12_0), wgK );
  int64_t globalReadOffsetB_0_0_13_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_13_0), wgK );
  int64_t globalReadOffsetB_0_0_14_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_14_0), wgK );
  int64_t globalReadOffsetB_0_0_15_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_15_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;
  DATA_TYPE const *globalReadB_0_0_8_0 = B + globalReadOffsetB_0_0_8_0;
  DATA_TYPE const *globalReadB_0_0_9_0 = B + globalReadOffsetB_0_0_9_0;
  DATA_TYPE const *globalReadB_0_0_10_0 = B + globalReadOffsetB_0_0_10_0;
  DATA_TYPE const *globalReadB_0_0_11_0 = B + globalReadOffsetB_0_0_11_0;
  DATA_TYPE const *globalReadB_0_0_12_0 = B + globalReadOffsetB_0_0_12_0;
  DATA_TYPE const *globalReadB_0_0_13_0 = B + globalReadOffsetB_0_0_13_0;
  DATA_TYPE const *globalReadB_0_0_14_0 = B + globalReadOffsetB_0_0_14_0;
  DATA_TYPE const *globalReadB_0_0_15_0 = B + globalReadOffsetB_0_0_15_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 4*LSPB);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 5*LSPB);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 6*LSPB);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 7*LSPB);
  unsigned int localWriteOffsetB_0_0_8_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 8*LSPB);
  unsigned int localWriteOffsetB_0_0_9_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 9*LSPB);
  unsigned int localWriteOffsetB_0_0_10_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 10*LSPB);
  unsigned int localWriteOffsetB_0_0_11_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 11*LSPB);
  unsigned int localWriteOffsetB_0_0_12_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 12*LSPB);
  unsigned int localWriteOffsetB_0_0_13_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 13*LSPB);
  unsigned int localWriteOffsetB_0_0_14_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 14*LSPB);
  unsigned int localWriteOffsetB_0_0_15_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 15*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;
  DATA_TYPE *localWriteB_0_0_8_0;
  DATA_TYPE *localWriteB_0_0_9_0;
  DATA_TYPE *localWriteB_0_0_10_0;
  DATA_TYPE *localWriteB_0_0_11_0;
  DATA_TYPE *localWriteB_0_0_12_0;
  DATA_TYPE *localWriteB_0_0_13_0;
  DATA_TYPE *localWriteB_0_0_14_0;
  DATA_TYPE *localWriteB_0_0_15_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);
  localWriteB_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_8_0);
  localWriteB_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_9_0);
  localWriteB_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_10_0);
  localWriteB_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_11_0);
  localWriteB_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_12_0);
  localWriteB_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_13_0);
  localWriteB_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_14_0);
  localWriteB_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_15_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_8_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_9_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_10_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_11_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_12_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_13_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_14_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_15_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);
    b_0_0_8_0 = *(globalReadB_0_0_8_0 + 0);
    b_0_0_9_0 = *(globalReadB_0_0_9_0 + 0);
    b_0_0_10_0 = *(globalReadB_0_0_10_0 + 0);
    b_0_0_11_0 = *(globalReadB_0_0_11_0 + 0);
    b_0_0_12_0 = *(globalReadB_0_0_12_0 + 0);
    b_0_0_13_0 = *(globalReadB_0_0_13_0 + 0);
    b_0_0_14_0 = *(globalReadB_0_0_14_0 + 0);
    b_0_0_15_0 = *(globalReadB_0_0_15_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_8_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_8_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_9_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_9_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_10_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_10_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_11_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_11_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_12_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_12_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_13_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_13_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_14_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_14_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_15_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_15_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
    *(localWriteB_0_0_8_0 + 0) = b_0_0_8_0;
    *(localWriteB_0_0_9_0 + 0) = b_0_0_9_0;
    *(localWriteB_0_0_10_0 + 0) = b_0_0_10_0;
    *(localWriteB_0_0_11_0 + 0) = b_0_0_11_0;
    *(localWriteB_0_0_12_0 + 0) = b_0_0_12_0;
    *(localWriteB_0_0_13_0 + 0) = b_0_0_13_0;
    *(localWriteB_0_0_14_0 + 0) = b_0_0_14_0;
    *(localWriteB_0_0_15_0 + 0) = b_0_0_15_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x8


    /* iter 15 */

    MAC_2x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);
  b_0_0_8_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_8_0 + 0);
  b_0_0_9_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_9_0 + 0);
  b_0_0_10_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_10_0 + 0);
  b_0_0_11_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_11_0 + 0);
  b_0_0_12_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_12_0 + 0);
  b_0_0_13_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_13_0 + 0);
  b_0_0_14_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_14_0 + 0);
  b_0_0_15_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_15_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);
  localWriteB_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_8_0);
  localWriteB_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_9_0);
  localWriteB_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_10_0);
  localWriteB_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_11_0);
  localWriteB_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_12_0);
  localWriteB_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_13_0);
  localWriteB_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_14_0);
  localWriteB_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_15_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
  *(localWriteB_0_0_8_0 + 0) = b_0_0_8_0;
  *(localWriteB_0_0_9_0 + 0) = b_0_0_9_0;
  *(localWriteB_0_0_10_0 + 0) = b_0_0_10_0;
  *(localWriteB_0_0_11_0 + 0) = b_0_0_11_0;
  *(localWriteB_0_0_12_0 + 0) = b_0_0_12_0;
  *(localWriteB_0_0_13_0 + 0) = b_0_0_13_0;
  *(localWriteB_0_0_14_0 + 0) = b_0_0_14_0;
  *(localWriteB_0_0_15_0 + 0) = b_0_0_15_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x8
#undef MAC_2x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 12
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 3
#define NLCB 3
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1536
#define LDS_NUM_ELEMENTS 6400
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x4 micro-tile */
#define MAC_8x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7],rB[3],rC[7+3*TT0I]); \

#define MAC_8x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT128x64x12_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA3_NLCB3_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_4_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_1_0_0_0;
  DATA_TYPE a_2_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_1_0_1_0;
  DATA_TYPE a_2_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_1_0_0_0;
  DATA_TYPE b_2_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSCA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSCB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_1_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_1_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_2_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_2_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_1_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_1_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_2_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_2_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_1_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_1_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_2_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_2_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_1_0_0_0 = A + globalReadOffsetA_1_0_0_0;
  DATA_TYPE const *globalReadA_2_0_0_0 = A + globalReadOffsetA_2_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_1_0_1_0 = A + globalReadOffsetA_1_0_1_0;
  DATA_TYPE const *globalReadA_2_0_1_0 = A + globalReadOffsetA_2_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_1_0_0_0 = B + globalReadOffsetB_1_0_0_0;
  DATA_TYPE const *globalReadB_2_0_0_0 = B + globalReadOffsetB_2_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_1_0_0_0 = localWriteFirstOffsetA + (0 + 1*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_2_0_0_0 = localWriteFirstOffsetA + (0 + 2*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_1_0_1_0 = localWriteFirstOffsetA + (0 + 1*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_2_0_1_0 = localWriteFirstOffsetA + (0 + 2*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_1_0_0_0 = localWriteFirstOffsetB + (0 + 1*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_2_0_0_0 = localWriteFirstOffsetB + (0 + 2*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_1_0_0_0;
  DATA_TYPE *localWriteA_2_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_1_0_1_0;
  DATA_TYPE *localWriteA_2_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_1_0_0_0;
  DATA_TYPE *localWriteB_2_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_1_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_0_0);
  localWriteA_2_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_2_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_1_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_1_0);
  localWriteA_2_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_2_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_1_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_1_0_0_0);
  localWriteB_2_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_2_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 384-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_1_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_2_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_1_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_2_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_1_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_2_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_1_0_0_0 = *(globalReadA_1_0_0_0 + 0);
    a_2_0_0_0 = *(globalReadA_2_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_1_0_1_0 = *(globalReadA_1_0_1_0 + 0);
    a_2_0_1_0 = *(globalReadA_2_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_1_0_0_0 = *(globalReadB_1_0_0_0 + 0);
    b_2_0_0_0 = *(globalReadB_2_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_1_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_1_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_1_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_2_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_2_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_2_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_1_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_1_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_1_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_2_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_2_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_2_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_1_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_1_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_1_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_2_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_2_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_2_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_1_0_0_0 + 0) = a_1_0_0_0;
    *(localWriteA_2_0_0_0 + 0) = a_2_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_1_0_1_0 + 0) = a_1_0_1_0;
    *(localWriteA_2_0_1_0 + 0) = a_2_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_1_0_0_0 + 0) = b_1_0_0_0;
    *(localWriteB_2_0_0_0 + 0) = b_2_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_1_0_0_0 = (localWriteOffsetA_1_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_2_0_0_0 = (localWriteOffsetA_2_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_1_0_1_0 = (localWriteOffsetA_1_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_2_0_1_0 = (localWriteOffsetA_2_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_1_0_0_0 = (localWriteOffsetB_1_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_2_0_0_0 = (localWriteOffsetB_2_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_1_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_0_0);
    localWriteA_2_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_2_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_1_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_1_0);
    localWriteA_2_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_2_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_1_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_1_0_0_0);
    localWriteB_2_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_2_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_1_0_0_0 = *(globalReadA_1_0_0_0 + 0);
    a_2_0_0_0 = *(globalReadA_2_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_1_0_1_0 = *(globalReadA_1_0_1_0 + 0);
    a_2_0_1_0 = *(globalReadA_2_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_1_0_0_0 = *(globalReadB_1_0_0_0 + 0);
    b_2_0_0_0 = *(globalReadB_2_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_1_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_1_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_1_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_2_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_2_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_2_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_1_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_1_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_1_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_2_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_2_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_2_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_1_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_1_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_1_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_2_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_2_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_2_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 10 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_1_0_0_0 + 0) = a_1_0_0_0;
    *(localWriteA_2_0_0_0 + 0) = a_2_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_1_0_1_0 + 0) = a_1_0_1_0;
    *(localWriteA_2_0_1_0 + 0) = a_2_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_1_0_0_0 + 0) = b_1_0_0_0;
    *(localWriteB_2_0_0_0 + 0) = b_2_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_1_0_0_0 = (localWriteOffsetA_1_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_2_0_0_0 = (localWriteOffsetA_2_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_1_0_1_0 = (localWriteOffsetA_1_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_2_0_1_0 = (localWriteOffsetA_2_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_1_0_0_0 = (localWriteOffsetB_1_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_2_0_0_0 = (localWriteOffsetB_2_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_1_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_0_0);
    localWriteA_2_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_2_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_1_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_1_0);
    localWriteA_2_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_2_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_1_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_1_0_0_0);
    localWriteB_2_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_2_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x4


    /* iter 11 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_8x4


    /* iter 11 (last unrolled loop) */

    MAC_8x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_1_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_2_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_1_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_2_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_1_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_2_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_1_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_2_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_1_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_2_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_1_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_2_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_1_0_0_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_1_0_0_0 + 0);
  a_2_0_0_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_2_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_1_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_1_0_1_0 + 0);
  a_2_0_1_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_2_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_1_0_0_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_1_0_0_0 + 0);
  b_2_0_0_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_2_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_1_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_0_0);
  localWriteA_2_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_2_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_1_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_1_0);
  localWriteA_2_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_2_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_1_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_1_0_0_0);
  localWriteB_2_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_2_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_1_0_0_0 + 0) = a_1_0_0_0;
  *(localWriteA_2_0_0_0 + 0) = a_2_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_1_0_1_0 + 0) = a_1_0_1_0;
  *(localWriteA_2_0_1_0 + 0) = a_2_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_1_0_0_0 + 0) = b_1_0_0_0;
  *(localWriteB_2_0_0_0 + 0) = b_2_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x4
#undef MAC_8x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 12
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 3
#define NLCB 3
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1536
#define LDS_NUM_ELEMENTS 6400
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x4 micro-tile */
#define MAC_8x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7],rB[3],rC[7+3*TT0I]); \

#define MAC_8x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT128x64x12_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA3_NLCB3_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_4_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_1_0_0_0;
  DATA_TYPE a_2_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_1_0_1_0;
  DATA_TYPE a_2_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_1_0_0_0;
  DATA_TYPE b_2_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSCA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSCB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_1_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_1_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_2_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_2_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_1_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_1_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_2_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_2_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_1_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_1_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_2_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_2_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_1_0_0_0 = A + globalReadOffsetA_1_0_0_0;
  DATA_TYPE const *globalReadA_2_0_0_0 = A + globalReadOffsetA_2_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_1_0_1_0 = A + globalReadOffsetA_1_0_1_0;
  DATA_TYPE const *globalReadA_2_0_1_0 = A + globalReadOffsetA_2_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_1_0_0_0 = B + globalReadOffsetB_1_0_0_0;
  DATA_TYPE const *globalReadB_2_0_0_0 = B + globalReadOffsetB_2_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_1_0_0_0 = localWriteFirstOffsetA + (0 + 1*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_2_0_0_0 = localWriteFirstOffsetA + (0 + 2*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_1_0_1_0 = localWriteFirstOffsetA + (0 + 1*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_2_0_1_0 = localWriteFirstOffsetA + (0 + 2*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_1_0_0_0 = localWriteFirstOffsetB + (0 + 1*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_2_0_0_0 = localWriteFirstOffsetB + (0 + 2*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_1_0_0_0;
  DATA_TYPE *localWriteA_2_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_1_0_1_0;
  DATA_TYPE *localWriteA_2_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_1_0_0_0;
  DATA_TYPE *localWriteB_2_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_1_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_0_0);
  localWriteA_2_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_2_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_1_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_1_0);
  localWriteA_2_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_2_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_1_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_1_0_0_0);
  localWriteB_2_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_2_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 384-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_1_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_2_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_1_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_2_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_1_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_2_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_1_0_0_0 = *(globalReadA_1_0_0_0 + 0);
    a_2_0_0_0 = *(globalReadA_2_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_1_0_1_0 = *(globalReadA_1_0_1_0 + 0);
    a_2_0_1_0 = *(globalReadA_2_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_1_0_0_0 = *(globalReadB_1_0_0_0 + 0);
    b_2_0_0_0 = *(globalReadB_2_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_1_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_1_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_1_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_2_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_2_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_2_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_1_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_1_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_1_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_2_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_2_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_2_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_1_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_1_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_1_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_2_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_2_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_2_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_1_0_0_0 + 0) = a_1_0_0_0;
    *(localWriteA_2_0_0_0 + 0) = a_2_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_1_0_1_0 + 0) = a_1_0_1_0;
    *(localWriteA_2_0_1_0 + 0) = a_2_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_1_0_0_0 + 0) = b_1_0_0_0;
    *(localWriteB_2_0_0_0 + 0) = b_2_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_1_0_0_0 = (localWriteOffsetA_1_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_2_0_0_0 = (localWriteOffsetA_2_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_1_0_1_0 = (localWriteOffsetA_1_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_2_0_1_0 = (localWriteOffsetA_2_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_1_0_0_0 = (localWriteOffsetB_1_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_2_0_0_0 = (localWriteOffsetB_2_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_1_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_0_0);
    localWriteA_2_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_2_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_1_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_1_0);
    localWriteA_2_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_2_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_1_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_1_0_0_0);
    localWriteB_2_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_2_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_1_0_0_0 = *(globalReadA_1_0_0_0 + 0);
    a_2_0_0_0 = *(globalReadA_2_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_1_0_1_0 = *(globalReadA_1_0_1_0 + 0);
    a_2_0_1_0 = *(globalReadA_2_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_1_0_0_0 = *(globalReadB_1_0_0_0 + 0);
    b_2_0_0_0 = *(globalReadB_2_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_1_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_1_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_1_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_2_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_2_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_2_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_1_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_1_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_1_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_2_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_2_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_2_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_1_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_1_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_1_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_2_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_2_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_2_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 10 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_1_0_0_0 + 0) = a_1_0_0_0;
    *(localWriteA_2_0_0_0 + 0) = a_2_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_1_0_1_0 + 0) = a_1_0_1_0;
    *(localWriteA_2_0_1_0 + 0) = a_2_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_1_0_0_0 + 0) = b_1_0_0_0;
    *(localWriteB_2_0_0_0 + 0) = b_2_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_1_0_0_0 = (localWriteOffsetA_1_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_2_0_0_0 = (localWriteOffsetA_2_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_1_0_1_0 = (localWriteOffsetA_1_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_2_0_1_0 = (localWriteOffsetA_2_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_1_0_0_0 = (localWriteOffsetB_1_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_2_0_0_0 = (localWriteOffsetB_2_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_1_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_0_0);
    localWriteA_2_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_2_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_1_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_1_0);
    localWriteA_2_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_2_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_1_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_1_0_0_0);
    localWriteB_2_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_2_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x4


    /* iter 11 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_8x4


    /* iter 11 (last unrolled loop) */

    MAC_8x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_1_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_2_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_1_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_2_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_1_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_2_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_1_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_2_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_1_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_2_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_1_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_2_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_1_0_0_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_1_0_0_0 + 0);
  a_2_0_0_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_2_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_1_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_1_0_1_0 + 0);
  a_2_0_1_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_2_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_1_0_0_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_1_0_0_0 + 0);
  b_2_0_0_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_2_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_1_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_0_0);
  localWriteA_2_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_2_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_1_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_1_0);
  localWriteA_2_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_2_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_1_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_1_0_0_0);
  localWriteB_2_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_2_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_1_0_0_0 + 0) = a_1_0_0_0;
  *(localWriteA_2_0_0_0 + 0) = a_2_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_1_0_1_0 + 0) = a_1_0_1_0;
  *(localWriteA_2_0_1_0 + 0) = a_2_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_1_0_0_0 + 0) = b_1_0_0_0;
  *(localWriteB_2_0_0_0 + 0) = b_2_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x4
#undef MAC_8x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I]); \
  TYPE_MAC(rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I]); \
  TYPE_MAC(rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I]); \
  TYPE_MAC(rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I]); \
  TYPE_MAC(rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I]); \
  TYPE_MAC(rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I]); \
  TYPE_MAC(rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I]); \
  TYPE_MAC(rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I]); \
  TYPE_MAC(rA[7],rB[7],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT128x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I]); \
  TYPE_MAC(rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I]); \
  TYPE_MAC(rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I]); \
  TYPE_MAC(rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I]); \
  TYPE_MAC(rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I]); \
  TYPE_MAC(rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I]); \
  TYPE_MAC(rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I]); \
  TYPE_MAC(rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I]); \
  TYPE_MAC(rA[7],rB[7],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT128x128x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 128
#define LDS_NUM_ELEMENTS 512
#define LDS_OFFSET_BLK 256

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_SB_MT16x16x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 128
#define LDS_NUM_ELEMENTS 896
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x4 micro-tile */
#define MAC_2x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \

#define MAC_2x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_SB_MT16x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_4_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x4


    /* iter 7 (last unrolled loop) */

    MAC_2x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x4

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x4
#undef MAC_2x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_SB_MT32x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1792
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x4 micro-tile */
#define MAC_2x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \

#define MAC_2x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_SB_MT32x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_4_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x4


    /* iter 7 (last unrolled loop) */

    MAC_2x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x4

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x4
#undef MAC_2x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I]); \
  TYPE_MAC(rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I]); \
  TYPE_MAC(rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I]); \
  TYPE_MAC(rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I]); \
  TYPE_MAC(rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I]); \
  TYPE_MAC(rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I]); \
  TYPE_MAC(rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I]); \
  TYPE_MAC(rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I]); \
  TYPE_MAC(rA[7],rB[7],rC[7+7*TT0I]); \

#define MAC_8x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[4+TT1J],rC[4+4*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[4+TT1J],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[4+TT1J],rC[6+4*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[4+TT1J],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[5+TT1J],rC[4+5*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[5+TT1J],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[5+TT1J],rC[6+5*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[5+TT1J],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[6+TT1J],rC[4+6*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[6+TT1J],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[6+TT1J],rC[6+6*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[6+TT1J],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[7+TT1J],rC[4+7*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[7+TT1J],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[7+TT1J],rC[6+7*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[7+TT1J],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_SB_MT128x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_0_2;
  DATA_TYPE *localWriteA_0_0_0_3;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_8x8


    /* iter 7 (last unrolled loop) */

    MAC_8x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_3 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
  *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
  *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 2) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 3) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef MAC_8x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 896
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_SB_MT32x16x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 2
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x4 micro-tile */
#define MAC_2x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \

#define MAC_2x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_SB_MT32x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_4_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x4


    /* iter 7 (last unrolled loop) */

    MAC_2x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x4

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x4
#undef MAC_2x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 8
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 128
#define LDS_NUM_ELEMENTS 896
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_SB_MT16x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_16_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 8
#define SG1J 16
#define TT0I 2
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 128
#define LDS_NUM_ELEMENTS 1664
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x4 micro-tile */
#define MAC_2x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \

#define MAC_2x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_SB_MT16x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_4_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_16_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x4


    /* iter 7 (last unrolled loop) */

    MAC_2x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x4

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x4
#undef MAC_2x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 4

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 128
#define LDS_NUM_ELEMENTS 512
#define LDS_OFFSET_BLK 256

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_SB_MT16x16x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM4(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 4

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 128
#define LDS_NUM_ELEMENTS 896
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x4 micro-tile */
#define MAC_2x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \

#define MAC_2x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_SB_MT16x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_4_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM4(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x4


    /* iter 7 (last unrolled loop) */

    MAC_2x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x4

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x4
#undef MAC_2x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 4

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_SB_MT32x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM4(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 4

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1792
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x4 micro-tile */
#define MAC_2x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \

#define MAC_2x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_SB_MT32x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_4_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM4(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x4


    /* iter 7 (last unrolled loop) */

    MAC_2x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x4

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x4
#undef MAC_2x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 4

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I]); \
  TYPE_MAC(rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I]); \
  TYPE_MAC(rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I]); \
  TYPE_MAC(rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I]); \
  TYPE_MAC(rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I]); \
  TYPE_MAC(rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I]); \
  TYPE_MAC(rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I]); \
  TYPE_MAC(rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I]); \
  TYPE_MAC(rA[7],rB[7],rC[7+7*TT0I]); \

#define MAC_8x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[4+TT1J],rC[4+4*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[4+TT1J],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[4+TT1J],rC[6+4*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[4+TT1J],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[5+TT1J],rC[4+5*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[5+TT1J],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[5+TT1J],rC[6+5*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[5+TT1J],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[6+TT1J],rC[4+6*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[6+TT1J],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[6+TT1J],rC[6+6*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[6+TT1J],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[7+TT1J],rC[4+7*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[7+TT1J],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[7+TT1J],rC[6+7*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[7+TT1J],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_SB_MT128x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM4(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_0_2;
  DATA_TYPE *localWriteA_0_0_0_3;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_8x8


    /* iter 7 (last unrolled loop) */

    MAC_8x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_3 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
  *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
  *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 2) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 3) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef MAC_8x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 4

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 896
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_SB_MT32x16x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_8_1_WGM4(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 2
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 4

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x4 micro-tile */
#define MAC_2x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \

#define MAC_2x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_SB_MT32x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_4_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_8_1_WGM4(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x4


    /* iter 7 (last unrolled loop) */

    MAC_2x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x4

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x4
#undef MAC_2x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 8
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 4

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 128
#define LDS_NUM_ELEMENTS 896
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_SB_MT16x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_16_1_WGM4(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I]); \
  TYPE_MAC(rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I]); \
  TYPE_MAC(rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I]); \
  TYPE_MAC(rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I]); \
  TYPE_MAC(rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I]); \
  TYPE_MAC(rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I]); \
  TYPE_MAC(rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I]); \
  TYPE_MAC(rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I]); \
  TYPE_MAC(rA[7],rB[7],rC[7+7*TT0I]); \

#define MAC_8x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[4+TT1J],rC[4+4*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[4+TT1J],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[4+TT1J],rC[6+4*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[4+TT1J],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[5+TT1J],rC[4+5*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[5+TT1J],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[5+TT1J],rC[6+5*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[5+TT1J],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[6+TT1J],rC[4+6*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[6+TT1J],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[6+TT1J],rC[6+6*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[6+TT1J],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[7+TT1J],rC[4+7*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[7+TT1J],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[7+TT1J],rC[6+7*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[7+TT1J],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_SB_MT128x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_0_2;
  DATA_TYPE *localWriteA_0_0_0_3;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8


    /* iter 7 */

    MAC_8x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
  *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
  *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }





  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 2) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 3) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef MAC_8x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 4

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I]); \
  TYPE_MAC(rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I]); \
  TYPE_MAC(rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I]); \
  TYPE_MAC(rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I]); \
  TYPE_MAC(rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I]); \
  TYPE_MAC(rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I]); \
  TYPE_MAC(rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I]); \
  TYPE_MAC(rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I]); \
  TYPE_MAC(rA[7],rB[7],rC[7+7*TT0I]); \

#define MAC_8x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[4+TT1J],rC[4+4*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[4+TT1J],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[4+TT1J],rC[6+4*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[4+TT1J],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[5+TT1J],rC[4+5*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[5+TT1J],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[5+TT1J],rC[6+5*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[5+TT1J],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[6+TT1J],rC[4+6*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[6+TT1J],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[6+TT1J],rC[6+6*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[6+TT1J],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[7+TT1J],rC[4+7*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[7+TT1J],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[7+TT1J],rC[6+7*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[7+TT1J],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_SB_MT128x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM4(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_0_2;
  DATA_TYPE *localWriteA_0_0_0_3;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8


    /* iter 7 */

    MAC_8x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
  *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
  *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }





  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 2) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 3) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef MAC_8x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 64
#define LDS_NUM_ELEMENTS 409
#define LDS_OFFSET_BLK 128

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_SB_MT16x16x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 3 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 3 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 2
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 64
#define LDS_NUM_ELEMENTS 409
#define LDS_OFFSET_BLK 128

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_SB_MT32x32x2_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 1 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 1 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 128
#define LDS_NUM_ELEMENTS 512
#define LDS_OFFSET_BLK 256

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_SB_MT16x16x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 128
#define LDS_NUM_ELEMENTS 896
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x4 micro-tile */
#define MAC_2x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \

#define MAC_2x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_SB_MT16x32x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_4_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x4


    /* iter 7 (last unrolled loop) */

    MAC_2x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x4

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x4
#undef MAC_2x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_SB_MT32x32x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1792
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x4 micro-tile */
#define MAC_2x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \

#define MAC_2x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_SB_MT32x64x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_4_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x4


    /* iter 7 (last unrolled loop) */

    MAC_2x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x4

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x4
#undef MAC_2x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I]); \
  TYPE_MAC(rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I]); \
  TYPE_MAC(rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I]); \
  TYPE_MAC(rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I]); \
  TYPE_MAC(rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I]); \
  TYPE_MAC(rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I]); \
  TYPE_MAC(rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I]); \
  TYPE_MAC(rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I]); \
  TYPE_MAC(rA[7],rB[7],rC[7+7*TT0I]); \

#define MAC_8x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[4+TT1J],rC[4+4*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[4+TT1J],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[4+TT1J],rC[6+4*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[4+TT1J],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[5+TT1J],rC[4+5*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[5+TT1J],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[5+TT1J],rC[6+5*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[5+TT1J],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[6+TT1J],rC[4+6*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[6+TT1J],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[6+TT1J],rC[6+6*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[6+TT1J],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[7+TT1J],rC[4+7*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[7+TT1J],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[7+TT1J],rC[6+7*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[7+TT1J],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_SB_MT128x128x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_0_2;
  DATA_TYPE *localWriteA_0_0_0_3;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_8x8


    /* iter 7 (last unrolled loop) */

    MAC_8x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_3 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
  *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
  *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 2) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 3) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef MAC_8x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 896
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_SB_MT32x16x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 2
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x4 micro-tile */
#define MAC_2x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \

#define MAC_2x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_SB_MT32x32x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_4_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x4


    /* iter 7 (last unrolled loop) */

    MAC_2x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x4

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x4
#undef MAC_2x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 8
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 128
#define LDS_NUM_ELEMENTS 896
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_SB_MT16x32x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_16_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 8
#define SG1J 16
#define TT0I 2
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 128
#define LDS_NUM_ELEMENTS 1664
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x4 micro-tile */
#define MAC_2x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \

#define MAC_2x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_SB_MT16x64x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_4_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_16_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x4


    /* iter 7 (last unrolled loop) */

    MAC_2x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x4

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x4
#undef MAC_2x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 4

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 128
#define LDS_NUM_ELEMENTS 512
#define LDS_OFFSET_BLK 256

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_SB_MT16x16x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM4(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 4

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 128
#define LDS_NUM_ELEMENTS 896
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x4 micro-tile */
#define MAC_2x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \

#define MAC_2x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_SB_MT16x32x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_4_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM4(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x4


    /* iter 7 (last unrolled loop) */

    MAC_2x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x4

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x4
#undef MAC_2x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 4

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_SB_MT32x32x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM4(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 4

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1792
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x4 micro-tile */
#define MAC_2x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \

#define MAC_2x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_SB_MT32x64x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_4_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM4(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x4


    /* iter 7 (last unrolled loop) */

    MAC_2x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x4

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x4
#undef MAC_2x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 4

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I]); \
  TYPE_MAC(rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I]); \
  TYPE_MAC(rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I]); \
  TYPE_MAC(rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I]); \
  TYPE_MAC(rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I]); \
  TYPE_MAC(rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I]); \
  TYPE_MAC(rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I]); \
  TYPE_MAC(rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I]); \
  TYPE_MAC(rA[7],rB[7],rC[7+7*TT0I]); \

#define MAC_8x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[4+TT1J],rC[4+4*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[4+TT1J],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[4+TT1J],rC[6+4*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[4+TT1J],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[5+TT1J],rC[4+5*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[5+TT1J],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[5+TT1J],rC[6+5*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[5+TT1J],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[6+TT1J],rC[4+6*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[6+TT1J],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[6+TT1J],rC[6+6*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[6+TT1J],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[7+TT1J],rC[4+7*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[7+TT1J],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[7+TT1J],rC[6+7*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[7+TT1J],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_SB_MT128x128x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM4(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_0_2;
  DATA_TYPE *localWriteA_0_0_0_3;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_8x8


    /* iter 7 (last unrolled loop) */

    MAC_8x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_3 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
  *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
  *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 2) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 3) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef MAC_8x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 4

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 896
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_SB_MT32x16x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_8_1_WGM4(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 2
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 4

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x4 micro-tile */
#define MAC_2x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \

#define MAC_2x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_SB_MT32x32x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_4_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_8_1_WGM4(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x4


    /* iter 7 (last unrolled loop) */

    MAC_2x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x4

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x4
#undef MAC_2x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 8
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 4

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 128
#define LDS_NUM_ELEMENTS 896
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_SB_MT16x32x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_16_1_WGM4(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I]); \
  TYPE_MAC(rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I]); \
  TYPE_MAC(rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I]); \
  TYPE_MAC(rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I]); \
  TYPE_MAC(rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I]); \
  TYPE_MAC(rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I]); \
  TYPE_MAC(rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I]); \
  TYPE_MAC(rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I]); \
  TYPE_MAC(rA[7],rB[7],rC[7+7*TT0I]); \

#define MAC_8x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[4+TT1J],rC[4+4*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[4+TT1J],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[4+TT1J],rC[6+4*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[4+TT1J],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[5+TT1J],rC[4+5*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[5+TT1J],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[5+TT1J],rC[6+5*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[5+TT1J],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[6+TT1J],rC[4+6*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[6+TT1J],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[6+TT1J],rC[6+6*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[6+TT1J],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[7+TT1J],rC[4+7*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[7+TT1J],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[7+TT1J],rC[6+7*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[7+TT1J],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_SB_MT128x128x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_0_2;
  DATA_TYPE *localWriteA_0_0_0_3;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8


    /* iter 7 */

    MAC_8x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
  *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
  *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }





  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 2) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 3) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef MAC_8x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 4

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I]); \
  TYPE_MAC(rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I]); \
  TYPE_MAC(rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I]); \
  TYPE_MAC(rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I]); \
  TYPE_MAC(rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I]); \
  TYPE_MAC(rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I]); \
  TYPE_MAC(rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I]); \
  TYPE_MAC(rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I]); \
  TYPE_MAC(rA[7],rB[7],rC[7+7*TT0I]); \

#define MAC_8x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[4+TT1J],rC[4+4*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[4+TT1J],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[4+TT1J],rC[6+4*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[4+TT1J],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[5+TT1J],rC[4+5*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[5+TT1J],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[5+TT1J],rC[6+5*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[5+TT1J],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[6+TT1J],rC[4+6*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[6+TT1J],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[6+TT1J],rC[6+6*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[6+TT1J],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[7+TT1J],rC[4+7*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[7+TT1J],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[7+TT1J],rC[6+7*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[7+TT1J],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_SB_MT128x128x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM4(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_0_2;
  DATA_TYPE *localWriteA_0_0_0_3;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8


    /* iter 7 */

    MAC_8x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
  *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
  *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }





  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 2) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 3) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef MAC_8x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 64
#define LDS_NUM_ELEMENTS 409
#define LDS_OFFSET_BLK 128

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_SB_MT16x16x4_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 3 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 3 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 2
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 64
#define LDS_NUM_ELEMENTS 409
#define LDS_OFFSET_BLK 128

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_SB_MT32x32x2_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 1 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 1 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE TensileComplexFloat
#define DEST_DATA_TYPE TensileComplexFloat
#define COMPUTE_DATA_TYPE TensileComplexFloat
#define s0 x
#define s1 y
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) \
  DST.s0 = MAC(  MULA.s0, MULB.s0, DST.s0 ); \
  DST.s0 = MAC( -MULA.s1, MULB.s1, DST.s0 ); \
  DST.s1 = MAC(  MULA.s0, MULB.s1, DST.s1 ); \
  DST.s1 = MAC(  MULA.s1, MULB.s0, DST.s1 );
#define TYPE_MAC_WRITE( DST, SRC, ALPHA, REG, BETA ) \
  /* (1) */ \
  type_mac_tmp = REG.s0; \
  REG.s0 *= ALPHA.s0; \
  REG.s0 = MAC( -ALPHA.s1, REG.s1, REG.s0 ); \
  REG.s1 *= ALPHA.s0; \
  REG.s1 = MAC(  ALPHA.s1, type_mac_tmp, REG.s1 ); \
  /* (2) */ \
  if(BETA.s0 != 0) { \
  REG.s0 = MAC(  BETA.s0, SRC.s0, REG.s0 ); \
  REG.s1 = MAC(  BETA.s0, SRC.s1, REG.s1 ); \
  } \
  if (BETA.s1 != 0) { \
  REG.s0 = MAC( -BETA.s1, SRC.s1, REG.s0 ); \
  REG.s1 = MAC(  BETA.s1, SRC.s0, REG.s1 ); \
  } \
  /* (3) */ \
  DST = REG;

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_CB_MT32x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW2_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  TensileComplexFloat *D,
  TensileComplexFloat const * __restrict__ C,
  TensileComplexFloat const * __restrict__ A,
  TensileComplexFloat const * __restrict__ B,
  TensileComplexFloat const alpha,
  TensileComplexFloat const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (TensileComplexFloat)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == tensile_complex<float>(0.0f)) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }



  float type_mac_tmp;


  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE TensileComplexFloat
#define DEST_DATA_TYPE TensileComplexFloat
#define COMPUTE_DATA_TYPE TensileComplexFloat
#define s0 x
#define s1 y
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) \
  DST.s0 = MAC(  MULA.s0, MULB.s0, DST.s0 ); \
  DST.s0 = MAC( -MULA.s1, MULB.s1, DST.s0 ); \
  DST.s1 = MAC(  MULA.s0, MULB.s1, DST.s1 ); \
  DST.s1 = MAC(  MULA.s1, MULB.s0, DST.s1 );
#define TYPE_MAC_WRITE( DST, SRC, ALPHA, REG, BETA ) \
  /* (1) */ \
  type_mac_tmp = REG.s0; \
  REG.s0 *= ALPHA.s0; \
  REG.s0 = MAC( -ALPHA.s1, REG.s1, REG.s0 ); \
  REG.s1 *= ALPHA.s0; \
  REG.s1 = MAC(  ALPHA.s1, type_mac_tmp, REG.s1 ); \
  /* (2) */ \
  if(BETA.s0 != 0) { \
  REG.s0 = MAC(  BETA.s0, SRC.s0, REG.s0 ); \
  REG.s1 = MAC(  BETA.s0, SRC.s1, REG.s1 ); \
  } \
  if (BETA.s1 != 0) { \
  REG.s0 = MAC( -BETA.s1, SRC.s1, REG.s0 ); \
  REG.s1 = MAC(  BETA.s1, SRC.s0, REG.s1 ); \
  } \
  /* (3) */ \
  DST = REG;

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_CB_MT32x32x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW2_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  TensileComplexFloat *D,
  TensileComplexFloat const * __restrict__ C,
  TensileComplexFloat const * __restrict__ A,
  TensileComplexFloat const * __restrict__ B,
  TensileComplexFloat const alpha,
  TensileComplexFloat const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (TensileComplexFloat)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == tensile_complex<float>(0.0f)) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }



  float type_mac_tmp;


  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 819

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HBH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 819

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HBH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HBH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 819

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HBH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 3 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HBH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HBH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I] , rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I] , rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I] , rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I] , rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I] , rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I] , rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I] , rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I] , rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I] , rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I] , rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I] , rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I] , rA[7],rB[7],rC[7+7*TT0I]); \

#define MAC_8x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I] , rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I] , rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I] , rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I] , rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[4+TT1J],rC[4+4*TT0I] , rA[5+TT0I],rB[4+TT1J],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[4+TT1J],rC[6+4*TT0I] , rA[7+TT0I],rB[4+TT1J],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[5+TT1J],rC[4+5*TT0I] , rA[5+TT0I],rB[5+TT1J],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[5+TT1J],rC[6+5*TT0I] , rA[7+TT0I],rB[5+TT1J],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[6+TT1J],rC[4+6*TT0I] , rA[5+TT0I],rB[6+TT1J],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[6+TT1J],rC[6+6*TT0I] , rA[7+TT0I],rB[6+TT1J],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[7+TT1J],rC[4+7*TT0I] , rA[5+TT0I],rB[7+TT1J],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[7+TT1J],rC[6+7*TT0I] , rA[7+TT0I],rB[7+TT1J],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HBH_MT128x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8


    /* iter 7 */

    MAC_8x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef MAC_8x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1536

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HBH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 7 */

    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HBH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HBH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 7 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HBH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HBH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HBH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HBH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 819

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HBH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 819

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HBH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1792
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HBH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HBH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I] , rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I] , rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I] , rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I] , rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I] , rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I] , rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I] , rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I] , rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I] , rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I] , rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I] , rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I] , rA[7],rB[7],rC[7+7*TT0I]); \

#define MAC_8x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I] , rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I] , rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I] , rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I] , rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[4+TT1J],rC[4+4*TT0I] , rA[5+TT0I],rB[4+TT1J],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[4+TT1J],rC[6+4*TT0I] , rA[7+TT0I],rB[4+TT1J],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[5+TT1J],rC[4+5*TT0I] , rA[5+TT0I],rB[5+TT1J],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[5+TT1J],rC[6+5*TT0I] , rA[7+TT0I],rB[5+TT1J],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[6+TT1J],rC[4+6*TT0I] , rA[5+TT0I],rB[6+TT1J],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[6+TT1J],rC[6+6*TT0I] , rA[7+TT0I],rB[6+TT1J],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[7+TT1J],rC[4+7*TT0I] , rA[5+TT0I],rB[7+TT1J],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[7+TT1J],rC[6+7*TT0I] , rA[7+TT0I],rB[7+TT1J],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HBH_MT128x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8


    /* iter 3 */

    MAC_8x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef MAC_8x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 819

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HBH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 3 */

    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 819

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HBH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 3 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1792
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HBH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 3 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x8


    /* iter 3 (last unrolled loop) */

    MAC_4x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HBH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 3 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 3 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I] , rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I] , rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I] , rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I] , rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I] , rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I] , rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I] , rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I] , rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I] , rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I] , rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I] , rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I] , rA[7],rB[7],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HBH_MT128x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1536

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HBH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HBH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HBH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1536

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HBH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 7 */

    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HBH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 3584
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HBH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x8


    /* iter 7 (last unrolled loop) */

    MAC_4x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HBH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 7 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HBH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HBH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 3 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 3 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 7168
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HBH_MT64x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_1_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_1_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_2_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_3_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_2_1;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_3_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_1);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_1 + 0) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_1 + 0) = b_0_1_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_1 = (localWriteOffsetB_0_0_2_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_1 = (localWriteOffsetB_0_0_3_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_1);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
    localWriteB_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_1 + 0) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_1 + 0) = b_0_1_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_1 = (localWriteOffsetB_0_0_2_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_1 = (localWriteOffsetB_0_0_3_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_1);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
    localWriteB_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x8


    /* iter 15 (last unrolled loop) */

    MAC_4x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_1_2_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 1);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_1_3_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_1);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_2_1 + 0) = b_0_1_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_3_1 + 0) = b_0_1_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE TensileComplexDouble
#define DEST_DATA_TYPE TensileComplexDouble
#define COMPUTE_DATA_TYPE TensileComplexDouble
#define s0 x
#define s1 y
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) \
  DST.s0 = MAC(  MULA.s0,  MULB.s0, DST.s0 ); \
  DST.s0 = MAC(  MULA.s1, -MULB.s1, DST.s0 ); \
  DST.s1 = MAC(  MULA.s0, -MULB.s1, DST.s1 ); \
  DST.s1 = MAC( -MULA.s1,  MULB.s0, DST.s1 );
#define TYPE_MAC_WRITE( DST, SRC, ALPHA, REG, BETA ) \
  /* (1) */ \
  type_mac_tmp = REG.s0; \
  REG.s0 *= ALPHA.s0; \
  REG.s0 = MAC( -ALPHA.s1, REG.s1, REG.s0 ); \
  REG.s1 *= ALPHA.s0; \
  REG.s1 = MAC(  ALPHA.s1, type_mac_tmp, REG.s1 ); \
  /* (2) */ \
  if(BETA.s0 != 0) { \
  REG.s0 = MAC(  BETA.s0, SRC.s0, REG.s0 ); \
  REG.s1 = MAC(  BETA.s0, SRC.s1, REG.s1 ); \
  } \
  if (BETA.s1 != 0) { \
  REG.s0 = MAC( -BETA.s1, SRC.s1, REG.s0 ); \
  REG.s1 = MAC(  BETA.s1, SRC.s0, REG.s1 ); \
  } \
  /* (3) */ \
  DST = REG;

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_AlikC_BjlkC_ZB_MT32x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW1_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  TensileComplexDouble *D,
  TensileComplexDouble const * __restrict__ C,
  TensileComplexDouble const * __restrict__ A,
  TensileComplexDouble const * __restrict__ B,
  TensileComplexDouble const alpha,
  TensileComplexDouble const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (TensileComplexDouble)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == tensile_complex<double>(0.0)) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 1); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }



  double type_mac_tmp;


  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE TensileComplexDouble
#define DEST_DATA_TYPE TensileComplexDouble
#define COMPUTE_DATA_TYPE TensileComplexDouble
#define s0 x
#define s1 y
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) \
  DST.s0 = MAC(  MULA.s0,  MULB.s0, DST.s0 ); \
  DST.s0 = MAC(  MULA.s1, -MULB.s1, DST.s0 ); \
  DST.s1 = MAC(  MULA.s0, -MULB.s1, DST.s1 ); \
  DST.s1 = MAC( -MULA.s1,  MULB.s0, DST.s1 );
#define TYPE_MAC_WRITE( DST, SRC, ALPHA, REG, BETA ) \
  /* (1) */ \
  type_mac_tmp = REG.s0; \
  REG.s0 *= ALPHA.s0; \
  REG.s0 = MAC( -ALPHA.s1, REG.s1, REG.s0 ); \
  REG.s1 *= ALPHA.s0; \
  REG.s1 = MAC(  ALPHA.s1, type_mac_tmp, REG.s1 ); \
  /* (2) */ \
  if(BETA.s0 != 0) { \
  REG.s0 = MAC(  BETA.s0, SRC.s0, REG.s0 ); \
  REG.s1 = MAC(  BETA.s0, SRC.s1, REG.s1 ); \
  } \
  if (BETA.s1 != 0) { \
  REG.s0 = MAC( -BETA.s1, SRC.s1, REG.s0 ); \
  REG.s1 = MAC(  BETA.s1, SRC.s0, REG.s1 ); \
  } \
  /* (3) */ \
  DST = REG;

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_AlikC_BjlkC_ZB_MT32x32x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW1_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  TensileComplexDouble *D,
  TensileComplexDouble const * __restrict__ C,
  TensileComplexDouble const * __restrict__ A,
  TensileComplexDouble const * __restrict__ B,
  TensileComplexDouble const alpha,
  TensileComplexDouble const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (TensileComplexDouble)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == tensile_complex<double>(0.0)) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 1); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }



  double type_mac_tmp;


  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE double
#define DEST_DATA_TYPE double
#define COMPUTE_DATA_TYPE double
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_DB_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW2_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  double *D,
  double const * __restrict__ C,
  double const * __restrict__ A,
  double const * __restrict__ B,
  double const alpha,
  double const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (double)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 7 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE double
#define DEST_DATA_TYPE double
#define COMPUTE_DATA_TYPE double
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_DB_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW2_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  double *D,
  double const * __restrict__ C,
  double const * __restrict__ A,
  double const * __restrict__ B,
  double const alpha,
  double const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (double)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 3 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 3 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE TensileComplexDouble
#define DEST_DATA_TYPE TensileComplexDouble
#define COMPUTE_DATA_TYPE TensileComplexDouble
#define s0 x
#define s1 y
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) \
  DST.s0 = MAC(  MULA.s0,  MULB.s0, DST.s0 ); \
  DST.s0 = MAC( -MULA.s1, -MULB.s1, DST.s0 ); \
  DST.s1 = MAC(  MULA.s0, -MULB.s1, DST.s1 ); \
  DST.s1 = MAC(  MULA.s1,  MULB.s0, DST.s1 );
#define TYPE_MAC_WRITE( DST, SRC, ALPHA, REG, BETA ) \
  /* (1) */ \
  type_mac_tmp = REG.s0; \
  REG.s0 *= ALPHA.s0; \
  REG.s0 = MAC( -ALPHA.s1, REG.s1, REG.s0 ); \
  REG.s1 *= ALPHA.s0; \
  REG.s1 = MAC(  ALPHA.s1, type_mac_tmp, REG.s1 ); \
  /* (2) */ \
  if(BETA.s0 != 0) { \
  REG.s0 = MAC(  BETA.s0, SRC.s0, REG.s0 ); \
  REG.s1 = MAC(  BETA.s0, SRC.s1, REG.s1 ); \
  } \
  if (BETA.s1 != 0) { \
  REG.s0 = MAC( -BETA.s1, SRC.s1, REG.s0 ); \
  REG.s1 = MAC(  BETA.s1, SRC.s0, REG.s1 ); \
  } \
  /* (3) */ \
  DST = REG;

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_BjlkC_ZB_MT32x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW1_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  TensileComplexDouble *D,
  TensileComplexDouble const * __restrict__ C,
  TensileComplexDouble const * __restrict__ A,
  TensileComplexDouble const * __restrict__ B,
  TensileComplexDouble const alpha,
  TensileComplexDouble const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (TensileComplexDouble)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == tensile_complex<double>(0.0)) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 1); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }



  double type_mac_tmp;


  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE TensileComplexDouble
#define DEST_DATA_TYPE TensileComplexDouble
#define COMPUTE_DATA_TYPE TensileComplexDouble
#define s0 x
#define s1 y
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) \
  DST.s0 = MAC(  MULA.s0,  MULB.s0, DST.s0 ); \
  DST.s0 = MAC( -MULA.s1, -MULB.s1, DST.s0 ); \
  DST.s1 = MAC(  MULA.s0, -MULB.s1, DST.s1 ); \
  DST.s1 = MAC(  MULA.s1,  MULB.s0, DST.s1 );
#define TYPE_MAC_WRITE( DST, SRC, ALPHA, REG, BETA ) \
  /* (1) */ \
  type_mac_tmp = REG.s0; \
  REG.s0 *= ALPHA.s0; \
  REG.s0 = MAC( -ALPHA.s1, REG.s1, REG.s0 ); \
  REG.s1 *= ALPHA.s0; \
  REG.s1 = MAC(  ALPHA.s1, type_mac_tmp, REG.s1 ); \
  /* (2) */ \
  if(BETA.s0 != 0) { \
  REG.s0 = MAC(  BETA.s0, SRC.s0, REG.s0 ); \
  REG.s1 = MAC(  BETA.s0, SRC.s1, REG.s1 ); \
  } \
  if (BETA.s1 != 0) { \
  REG.s0 = MAC( -BETA.s1, SRC.s1, REG.s0 ); \
  REG.s1 = MAC(  BETA.s1, SRC.s0, REG.s1 ); \
  } \
  /* (3) */ \
  DST = REG;

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_BjlkC_ZB_MT32x32x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW1_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  TensileComplexDouble *D,
  TensileComplexDouble const * __restrict__ C,
  TensileComplexDouble const * __restrict__ A,
  TensileComplexDouble const * __restrict__ B,
  TensileComplexDouble const alpha,
  TensileComplexDouble const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (TensileComplexDouble)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == tensile_complex<double>(0.0)) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 1); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }



  double type_mac_tmp;


  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 819

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 3 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 3 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 3 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? static_cast<tensile_bfloat16>((ALPHA)*(REG) + (BETA) * static_cast<float>(SRC)) : static_cast<tensile_bfloat16>((ALPHA)*(REG));

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_BBH_MT32x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_bfloat16 *D,
  tensile_bfloat16 const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 819

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? static_cast<tensile_bfloat16>((ALPHA)*(REG) + (BETA) * static_cast<float>(SRC)) : static_cast<tensile_bfloat16>((ALPHA)*(REG));

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_BBH_MT16x16x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  tensile_bfloat16 *D,
  tensile_bfloat16 const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? static_cast<tensile_bfloat16>((ALPHA)*(REG) + (BETA) * static_cast<float>(SRC)) : static_cast<tensile_bfloat16>((ALPHA)*(REG));

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_BBH_MT32x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_bfloat16 *D,
  tensile_bfloat16 const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 1536

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? static_cast<tensile_bfloat16>((ALPHA)*(REG) + (BETA) * static_cast<float>(SRC)) : static_cast<tensile_bfloat16>((ALPHA)*(REG));

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_BBH_MT64x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_bfloat16 *D,
  tensile_bfloat16 const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 819

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? static_cast<tensile_bfloat16>((ALPHA)*(REG) + (BETA) * static_cast<float>(SRC)) : static_cast<tensile_bfloat16>((ALPHA)*(REG));

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_BBH_MT16x16x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  tensile_bfloat16 *D,
  tensile_bfloat16 const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? static_cast<tensile_bfloat16>((ALPHA)*(REG) + (BETA) * static_cast<float>(SRC)) : static_cast<tensile_bfloat16>((ALPHA)*(REG));

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_BBH_MT32x32x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_bfloat16 *D,
  tensile_bfloat16 const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 819

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? static_cast<tensile_bfloat16>((ALPHA)*(REG) + (BETA) * static_cast<float>(SRC)) : static_cast<tensile_bfloat16>((ALPHA)*(REG));

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_BBH_MT16x16x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  tensile_bfloat16 *D,
  tensile_bfloat16 const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? static_cast<tensile_bfloat16>((ALPHA)*(REG) + (BETA) * static_cast<float>(SRC)) : static_cast<tensile_bfloat16>((ALPHA)*(REG));

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_BBH_MT32x32x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_bfloat16 *D,
  tensile_bfloat16 const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 1536

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? static_cast<tensile_bfloat16>((ALPHA)*(REG) + (BETA) * static_cast<float>(SRC)) : static_cast<tensile_bfloat16>((ALPHA)*(REG));

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_BBH_MT64x32x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_bfloat16 *D,
  tensile_bfloat16 const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 819

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? static_cast<tensile_bfloat16>((ALPHA)*(REG) + (BETA) * static_cast<float>(SRC)) : static_cast<tensile_bfloat16>((ALPHA)*(REG));

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_BBH_MT16x16x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  tensile_bfloat16 *D,
  tensile_bfloat16 const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 16
#define SG1J 4
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 832
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT32x8x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_4_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 128
#define LDS_NUM_ELEMENTS 512
#define LDS_OFFSET_BLK 256

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT16x16x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 896
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT32x16x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x2


    /* iter 7 (last unrolled loop) */

    MAC_4x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT32x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_0_2;
  DATA_TYPE *localWriteA_0_0_0_3;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 7 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_3 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_3 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
  *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 896
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT32x16x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 128
#define LDS_NUM_ELEMENTS 512
#define LDS_OFFSET_BLK 256

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT16x16x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT32x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_0_2;
  DATA_TYPE *localWriteA_0_0_0_3;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 7 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_3 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_3 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
  *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 896
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT32x16x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_8_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 32
#define SG1J 4
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 3200
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT64x8x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG32_4_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_2_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_3_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_2_1;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_3_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_1 + 0) = a_0_1_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_1 = (localWriteOffsetA_0_0_2_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_1 = (localWriteOffsetA_0_0_3_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
    localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_1 + 0) = a_0_1_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_1 = (localWriteOffsetA_0_0_2_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_1 = (localWriteOffsetA_0_0_3_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
    localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 15 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_1 + 0) = a_0_1_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 16
#define SG1J 4
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1664
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT32x8x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_4_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_2_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_3_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_2_1;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_3_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_1 + 0) = a_0_1_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_1 = (localWriteOffsetA_0_0_2_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_1 = (localWriteOffsetA_0_0_3_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
    localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_1 + 0) = a_0_1_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_1 = (localWriteOffsetA_0_0_2_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_1 = (localWriteOffsetA_0_0_3_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
    localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 15 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_1 + 0) = a_0_1_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT16x16x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 15 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1792
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT32x16x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_2_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_3_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_2_1;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_3_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_1 + 0) = a_0_1_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_1 = (localWriteOffsetA_0_0_2_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_1 = (localWriteOffsetA_0_0_3_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
    localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_1 + 0) = a_0_1_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_1 = (localWriteOffsetA_0_0_2_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_1 = (localWriteOffsetA_0_0_3_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
    localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x2


    /* iter 15 (last unrolled loop) */

    MAC_4x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_1 + 0) = a_0_1_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT32x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_2_1_0;
  DATA_TYPE a_0_3_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_2_1_0;
  DATA_TYPE b_0_3_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_0_2;
  DATA_TYPE *localWriteA_0_0_0_3;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;
  DATA_TYPE *localWriteA_0_0_1_2;
  DATA_TYPE *localWriteA_0_0_1_3;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;
  DATA_TYPE *localWriteB_0_0_1_2;
  DATA_TYPE *localWriteB_0_0_1_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
  localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
  localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_1_2 + 0) = a_0_2_1_0;
    *(localWriteA_0_0_1_3 + 0) = a_0_3_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
    *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_2 = (localWriteOffsetA_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_3 = (localWriteOffsetA_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_2 = (localWriteOffsetB_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_3 = (localWriteOffsetB_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
    localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
    localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
    localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_1_2 + 0) = a_0_2_1_0;
    *(localWriteA_0_0_1_3 + 0) = a_0_3_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
    *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_2 = (localWriteOffsetA_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_3 = (localWriteOffsetA_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_2 = (localWriteOffsetB_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_3 = (localWriteOffsetB_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
    localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
    localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
    localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_3 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_2 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_3 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_3 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_3 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_2_1_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 2);
  a_0_3_1_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_2_1_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 2);
  b_0_3_1_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
  localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
  localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
  *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
  *(localWriteA_0_0_1_2 + 0) = a_0_2_1_0;
  *(localWriteA_0_0_1_3 + 0) = a_0_3_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
  *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
  *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1792
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT32x16x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 15 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 3328
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT64x16x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_2_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_3_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_2_1;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_3_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_1 + 0) = a_0_1_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_1 = (localWriteOffsetA_0_0_2_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_1 = (localWriteOffsetA_0_0_3_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
    localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_1 + 0) = a_0_1_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_1 = (localWriteOffsetA_0_0_2_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_1 = (localWriteOffsetA_0_0_3_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
    localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x2


    /* iter 15 (last unrolled loop) */

    MAC_4x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_1 + 0) = a_0_1_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 3584
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT64x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_2_1_0;
  DATA_TYPE a_0_3_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_0_2;
  DATA_TYPE *localWriteA_0_0_0_3;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;
  DATA_TYPE *localWriteA_0_0_1_2;
  DATA_TYPE *localWriteA_0_0_1_3;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
  localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_1_2 + 0) = a_0_2_1_0;
    *(localWriteA_0_0_1_3 + 0) = a_0_3_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_2 = (localWriteOffsetA_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_3 = (localWriteOffsetA_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
    localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_1_2 + 0) = a_0_2_1_0;
    *(localWriteA_0_0_1_3 + 0) = a_0_3_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_2 = (localWriteOffsetA_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_3 = (localWriteOffsetA_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
    localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_3 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_2 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_3 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_3 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_2_1_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 2);
  a_0_3_1_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
  localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
  *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
  *(localWriteA_0_0_1_2 + 0) = a_0_2_1_0;
  *(localWriteA_0_0_1_3 + 0) = a_0_3_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 32
#define SG1J 4
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 3200
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT64x8x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG32_4_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_2_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_3_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_2_1;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_3_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_1 + 0) = a_0_1_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_1 = (localWriteOffsetA_0_0_2_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_1 = (localWriteOffsetA_0_0_3_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
    localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_1 + 0) = a_0_1_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_1 = (localWriteOffsetA_0_0_2_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_1 = (localWriteOffsetA_0_0_3_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
    localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 15 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_1 + 0) = a_0_1_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 16
#define SG1J 4
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1664
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT32x8x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_4_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_2_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_3_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_2_1;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_3_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_1 + 0) = a_0_1_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_1 = (localWriteOffsetA_0_0_2_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_1 = (localWriteOffsetA_0_0_3_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
    localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_1 + 0) = a_0_1_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_1 = (localWriteOffsetA_0_0_2_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_1 = (localWriteOffsetA_0_0_3_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
    localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 15 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_1 + 0) = a_0_1_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1792
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT32x16x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_8_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 15 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 3584
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT64x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_8_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_2_1_0;
  DATA_TYPE a_0_3_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_0_2;
  DATA_TYPE *localWriteA_0_0_0_3;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;
  DATA_TYPE *localWriteA_0_0_1_2;
  DATA_TYPE *localWriteA_0_0_1_3;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
  localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_1_2 + 0) = a_0_2_1_0;
    *(localWriteA_0_0_1_3 + 0) = a_0_3_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_2 = (localWriteOffsetA_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_3 = (localWriteOffsetA_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
    localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_1_2 + 0) = a_0_2_1_0;
    *(localWriteA_0_0_1_3 + 0) = a_0_3_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_2 = (localWriteOffsetA_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_3 = (localWriteOffsetA_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
    localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_3 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_2 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_3 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_3 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_2_1_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 2);
  a_0_3_1_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
  localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
  *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
  *(localWriteA_0_0_1_2 + 0) = a_0_2_1_0;
  *(localWriteA_0_0_1_3 + 0) = a_0_3_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_0_2;
  DATA_TYPE *localWriteA_0_0_0_3;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_3 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_3 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
  *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 7168
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT64x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_2_1_0;
  DATA_TYPE b_0_3_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_0_2;
  DATA_TYPE *localWriteA_0_0_0_3;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;
  DATA_TYPE *localWriteB_0_0_1_2;
  DATA_TYPE *localWriteB_0_0_1_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
  localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
    *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_2 = (localWriteOffsetB_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_3 = (localWriteOffsetB_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
    localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
    localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
    *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_2 = (localWriteOffsetB_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_3 = (localWriteOffsetB_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
    localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
    localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x8


    /* iter 15 (last unrolled loop) */

    MAC_4x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_3 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_3 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_3 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_2_1_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 2);
  b_0_3_1_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
  localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
  *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
  *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
  *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 6
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 3
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1536
#define LDS_NUM_ELEMENTS 7680
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 6x8 micro-tile */
#define MAC_6x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I]); \
  TYPE_MAC(rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I]); \
  TYPE_MAC(rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I]); \
  TYPE_MAC(rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I]); \
  TYPE_MAC(rA[5],rB[7],rC[5+7*TT0I]); \

#define MAC_6x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[4+TT1J],rC[4+4*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[4+TT1J],rC[5+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[5+TT1J],rC[4+5*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[5+TT1J],rC[5+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[6+TT1J],rC[4+6*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[6+TT1J],rC[5+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[7+TT1J],rC[4+7*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[7+TT1J],rC[5+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT96x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT6_8_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_1_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_1_3_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_2_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_2_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_3_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_2_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_2_1;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_3_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_1);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_1 + 0) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_1 + 0) = b_0_1_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_1 = (localWriteOffsetA_0_0_2_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_1 = (localWriteOffsetB_0_0_2_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_1 = (localWriteOffsetB_0_0_3_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_1);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
    localWriteB_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_1 + 0) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_1 + 0) = b_0_1_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_1 = (localWriteOffsetA_0_0_2_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_1 = (localWriteOffsetB_0_0_2_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_1 = (localWriteOffsetB_0_0_3_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_1);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
    localWriteB_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_6x8


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_6x8


    /* iter 15 (last unrolled loop) */

    MAC_6x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_1_2_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 1);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_1_3_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_1);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_2_1 + 0) = b_0_1_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_3_1 + 0) = b_0_1_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_6x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_6x8
#undef MAC_6x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 7168
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x4 micro-tile */
#define MAC_8x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7],rB[3],rC[7+3*TT0I]); \

#define MAC_8x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT128x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_2_1_0;
  DATA_TYPE a_0_3_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_0_2;
  DATA_TYPE *localWriteA_0_0_0_3;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;
  DATA_TYPE *localWriteA_0_0_1_2;
  DATA_TYPE *localWriteA_0_0_1_3;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
  localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_1_2 + 0) = a_0_2_1_0;
    *(localWriteA_0_0_1_3 + 0) = a_0_3_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_2 = (localWriteOffsetA_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_3 = (localWriteOffsetA_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
    localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_1_2 + 0) = a_0_2_1_0;
    *(localWriteA_0_0_1_3 + 0) = a_0_3_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_2 = (localWriteOffsetA_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_3 = (localWriteOffsetA_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
    localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_8x4


    /* iter 15 (last unrolled loop) */

    MAC_8x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_3 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_2 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_3 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_3 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_2_1_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 2);
  a_0_3_1_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
  localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
  *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
  *(localWriteA_0_0_1_2 + 0) = a_0_2_1_0;
  *(localWriteA_0_0_1_3 + 0) = a_0_3_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x4
#undef MAC_8x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_0_2;
  DATA_TYPE *localWriteA_0_0_0_3;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_3 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_3 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
  *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 7168
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT64x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_2_1_0;
  DATA_TYPE b_0_3_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_0_2;
  DATA_TYPE *localWriteA_0_0_0_3;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;
  DATA_TYPE *localWriteB_0_0_1_2;
  DATA_TYPE *localWriteB_0_0_1_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
  localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
    *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_2 = (localWriteOffsetB_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_3 = (localWriteOffsetB_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
    localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
    localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
    *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_2 = (localWriteOffsetB_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_3 = (localWriteOffsetB_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
    localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
    localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x8


    /* iter 15 (last unrolled loop) */

    MAC_4x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_3 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_3 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_3 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_2_1_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 2);
  b_0_3_1_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
  localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
  *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
  *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
  *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 7168
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x4 micro-tile */
#define MAC_8x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7],rB[3],rC[7+3*TT0I]); \

#define MAC_8x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT128x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_2_1_0;
  DATA_TYPE a_0_3_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_0_2;
  DATA_TYPE *localWriteA_0_0_0_3;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;
  DATA_TYPE *localWriteA_0_0_1_2;
  DATA_TYPE *localWriteA_0_0_1_3;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
  localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_1_2 + 0) = a_0_2_1_0;
    *(localWriteA_0_0_1_3 + 0) = a_0_3_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_2 = (localWriteOffsetA_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_3 = (localWriteOffsetA_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
    localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_1_2 + 0) = a_0_2_1_0;
    *(localWriteA_0_0_1_3 + 0) = a_0_3_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_2 = (localWriteOffsetA_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_3 = (localWriteOffsetA_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
    localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_8x4


    /* iter 15 (last unrolled loop) */

    MAC_8x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_3 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_2 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_3 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_3 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_2_1_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 2);
  a_0_3_1_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
  localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
  *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
  *(localWriteA_0_0_1_2 + 0) = a_0_2_1_0;
  *(localWriteA_0_0_1_3 + 0) = a_0_3_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x4
#undef MAC_8x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 32
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 8192
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT64x64x32_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_2_1_0;
  DATA_TYPE a_0_3_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_2_1_0;
  DATA_TYPE b_0_3_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_0_2;
  DATA_TYPE *localWriteA_0_0_0_3;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;
  DATA_TYPE *localWriteA_0_0_1_2;
  DATA_TYPE *localWriteA_0_0_1_3;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;
  DATA_TYPE *localWriteB_0_0_1_2;
  DATA_TYPE *localWriteB_0_0_1_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
  localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
  localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 1); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_1_2 + 0) = a_0_2_1_0;
    *(localWriteA_0_0_1_3 + 0) = a_0_3_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
    *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_2 = (localWriteOffsetA_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_3 = (localWriteOffsetA_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_2 = (localWriteOffsetB_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_3 = (localWriteOffsetB_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
    localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
    localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
    localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 16 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 17 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 18 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 19 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 20 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 21 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 22 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 23 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 24 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 25 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 26 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 27 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 28 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 29 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 30 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_1_2 + 0) = a_0_2_1_0;
    *(localWriteA_0_0_1_3 + 0) = a_0_3_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
    *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_2 = (localWriteOffsetA_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_3 = (localWriteOffsetA_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_2 = (localWriteOffsetB_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_3 = (localWriteOffsetB_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
    localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
    localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
    localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 31 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 16 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 17 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 18 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 19 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 20 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 21 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 22 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 23 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 24 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 25 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 26 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 27 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 28 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 29 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 30 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 31 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_3 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_2 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_3 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_3 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_3 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_2_1_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 2);
  a_0_3_1_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_2_1_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 2);
  b_0_3_1_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
  localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
  localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
  *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
  *(localWriteA_0_0_1_2 + 0) = a_0_2_1_0;
  *(localWriteA_0_0_1_3 + 0) = a_0_3_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
  *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
  *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 32
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 8192
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT64x64x32_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_2_1_0;
  DATA_TYPE a_0_3_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_2_1_0;
  DATA_TYPE b_0_3_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_0_2;
  DATA_TYPE *localWriteA_0_0_0_3;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;
  DATA_TYPE *localWriteA_0_0_1_2;
  DATA_TYPE *localWriteA_0_0_1_3;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;
  DATA_TYPE *localWriteB_0_0_1_2;
  DATA_TYPE *localWriteB_0_0_1_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
  localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
  localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 1); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_1_2 + 0) = a_0_2_1_0;
    *(localWriteA_0_0_1_3 + 0) = a_0_3_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
    *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_2 = (localWriteOffsetA_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_3 = (localWriteOffsetA_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_2 = (localWriteOffsetB_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_3 = (localWriteOffsetB_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
    localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
    localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
    localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 16 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 17 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 18 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 19 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 20 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 21 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 22 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 23 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 24 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 25 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 26 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 27 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 28 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 29 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 30 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_1_2 + 0) = a_0_2_1_0;
    *(localWriteA_0_0_1_3 + 0) = a_0_3_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
    *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_2 = (localWriteOffsetA_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_3 = (localWriteOffsetA_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_2 = (localWriteOffsetB_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_3 = (localWriteOffsetB_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
    localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
    localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
    localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 31 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 16 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 17 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 18 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 19 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 20 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 21 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 22 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 23 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 24 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 25 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 26 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 27 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 28 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 29 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 30 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 31 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_3 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_2 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_3 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_3 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_3 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_2_1_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 2);
  a_0_3_1_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_2_1_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 2);
  b_0_3_1_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
  localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
  localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
  *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
  *(localWriteA_0_0_1_2 + 0) = a_0_2_1_0;
  *(localWriteA_0_0_1_3 + 0) = a_0_3_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
  *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
  *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 64
#define LDS_NUM_ELEMENTS 409
#define LDS_OFFSET_BLK 128

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT16x16x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 3 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 3 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 2
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 64
#define LDS_NUM_ELEMENTS 409
#define LDS_OFFSET_BLK 128

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT32x32x2_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 1 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 1 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 16
#define SG1J 4
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 832
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT32x8x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_4_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 128
#define LDS_NUM_ELEMENTS 512
#define LDS_OFFSET_BLK 256

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT16x16x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 896
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT32x16x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x2


    /* iter 7 (last unrolled loop) */

    MAC_4x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT32x32x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_0_2;
  DATA_TYPE *localWriteA_0_0_0_3;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 7 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_3 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_3 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
  *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 896
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT32x16x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 128
#define LDS_NUM_ELEMENTS 512
#define LDS_OFFSET_BLK 256

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT16x16x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT32x32x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_0_2;
  DATA_TYPE *localWriteA_0_0_0_3;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 7 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_3 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_3 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
  *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 896
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT32x16x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_8_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 32
#define SG1J 4
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 3200
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT64x8x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG32_4_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_2_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_3_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_2_1;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_3_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_1 + 0) = a_0_1_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_1 = (localWriteOffsetA_0_0_2_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_1 = (localWriteOffsetA_0_0_3_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
    localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_1 + 0) = a_0_1_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_1 = (localWriteOffsetA_0_0_2_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_1 = (localWriteOffsetA_0_0_3_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
    localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 15 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_1 + 0) = a_0_1_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 16
#define SG1J 4
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1664
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT32x8x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_4_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_2_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_3_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_2_1;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_3_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_1 + 0) = a_0_1_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_1 = (localWriteOffsetA_0_0_2_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_1 = (localWriteOffsetA_0_0_3_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
    localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_1 + 0) = a_0_1_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_1 = (localWriteOffsetA_0_0_2_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_1 = (localWriteOffsetA_0_0_3_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
    localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 15 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_1 + 0) = a_0_1_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT16x16x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 15 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1792
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT32x16x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_2_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_3_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_2_1;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_3_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_1 + 0) = a_0_1_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_1 = (localWriteOffsetA_0_0_2_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_1 = (localWriteOffsetA_0_0_3_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
    localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_1 + 0) = a_0_1_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_1 = (localWriteOffsetA_0_0_2_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_1 = (localWriteOffsetA_0_0_3_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
    localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x2


    /* iter 15 (last unrolled loop) */

    MAC_4x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_1 + 0) = a_0_1_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT32x32x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_2_1_0;
  DATA_TYPE a_0_3_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_2_1_0;
  DATA_TYPE b_0_3_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_0_2;
  DATA_TYPE *localWriteA_0_0_0_3;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;
  DATA_TYPE *localWriteA_0_0_1_2;
  DATA_TYPE *localWriteA_0_0_1_3;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;
  DATA_TYPE *localWriteB_0_0_1_2;
  DATA_TYPE *localWriteB_0_0_1_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
  localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
  localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_1_2 + 0) = a_0_2_1_0;
    *(localWriteA_0_0_1_3 + 0) = a_0_3_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
    *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_2 = (localWriteOffsetA_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_3 = (localWriteOffsetA_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_2 = (localWriteOffsetB_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_3 = (localWriteOffsetB_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
    localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
    localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
    localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_1_2 + 0) = a_0_2_1_0;
    *(localWriteA_0_0_1_3 + 0) = a_0_3_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
    *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_2 = (localWriteOffsetA_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_3 = (localWriteOffsetA_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_2 = (localWriteOffsetB_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_3 = (localWriteOffsetB_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
    localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
    localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
    localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_3 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_2 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_3 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_3 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_3 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_2_1_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 2);
  a_0_3_1_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_2_1_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 2);
  b_0_3_1_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
  localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
  localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
  *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
  *(localWriteA_0_0_1_2 + 0) = a_0_2_1_0;
  *(localWriteA_0_0_1_3 + 0) = a_0_3_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
  *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
  *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1792
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT32x16x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 15 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 3328
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT64x16x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_2_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_3_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_2_1;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_3_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_1 + 0) = a_0_1_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_1 = (localWriteOffsetA_0_0_2_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_1 = (localWriteOffsetA_0_0_3_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
    localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_1 + 0) = a_0_1_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_1 = (localWriteOffsetA_0_0_2_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_1 = (localWriteOffsetA_0_0_3_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
    localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x2


    /* iter 15 (last unrolled loop) */

    MAC_4x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_1 + 0) = a_0_1_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 3584
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT64x32x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_2_1_0;
  DATA_TYPE a_0_3_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_0_2;
  DATA_TYPE *localWriteA_0_0_0_3;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;
  DATA_TYPE *localWriteA_0_0_1_2;
  DATA_TYPE *localWriteA_0_0_1_3;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
  localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_1_2 + 0) = a_0_2_1_0;
    *(localWriteA_0_0_1_3 + 0) = a_0_3_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_2 = (localWriteOffsetA_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_3 = (localWriteOffsetA_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
    localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_1_2 + 0) = a_0_2_1_0;
    *(localWriteA_0_0_1_3 + 0) = a_0_3_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_2 = (localWriteOffsetA_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_3 = (localWriteOffsetA_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
    localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_3 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_2 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_3 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_3 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_2_1_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 2);
  a_0_3_1_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
  localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
  *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
  *(localWriteA_0_0_1_2 + 0) = a_0_2_1_0;
  *(localWriteA_0_0_1_3 + 0) = a_0_3_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 32
#define SG1J 4
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 3200
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT64x8x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG32_4_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_2_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_3_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_2_1;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_3_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_1 + 0) = a_0_1_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_1 = (localWriteOffsetA_0_0_2_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_1 = (localWriteOffsetA_0_0_3_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
    localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_1 + 0) = a_0_1_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_1 = (localWriteOffsetA_0_0_2_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_1 = (localWriteOffsetA_0_0_3_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
    localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 15 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_1 + 0) = a_0_1_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 16
#define SG1J 4
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1664
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT32x8x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_4_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_2_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_3_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_2_1;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_3_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_1 + 0) = a_0_1_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_1 = (localWriteOffsetA_0_0_2_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_1 = (localWriteOffsetA_0_0_3_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
    localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_1 + 0) = a_0_1_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_1 = (localWriteOffsetA_0_0_2_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_1 = (localWriteOffsetA_0_0_3_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
    localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 15 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_1 + 0) = a_0_1_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1792
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT32x16x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_8_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 15 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 3584
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT64x32x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_8_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_2_1_0;
  DATA_TYPE a_0_3_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_0_2;
  DATA_TYPE *localWriteA_0_0_0_3;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;
  DATA_TYPE *localWriteA_0_0_1_2;
  DATA_TYPE *localWriteA_0_0_1_3;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
  localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_1_2 + 0) = a_0_2_1_0;
    *(localWriteA_0_0_1_3 + 0) = a_0_3_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_2 = (localWriteOffsetA_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_3 = (localWriteOffsetA_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
    localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_1_2 + 0) = a_0_2_1_0;
    *(localWriteA_0_0_1_3 + 0) = a_0_3_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_2 = (localWriteOffsetA_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_3 = (localWriteOffsetA_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
    localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_3 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_2 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_3 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_3 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_2_1_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 2);
  a_0_3_1_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
  localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
  *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
  *(localWriteA_0_0_1_2 + 0) = a_0_2_1_0;
  *(localWriteA_0_0_1_3 + 0) = a_0_3_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT64x64x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_0_2;
  DATA_TYPE *localWriteA_0_0_0_3;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_3 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_3 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
  *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 7168
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT64x128x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_2_1_0;
  DATA_TYPE b_0_3_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_0_2;
  DATA_TYPE *localWriteA_0_0_0_3;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;
  DATA_TYPE *localWriteB_0_0_1_2;
  DATA_TYPE *localWriteB_0_0_1_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
  localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
    *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_2 = (localWriteOffsetB_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_3 = (localWriteOffsetB_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
    localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
    localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
    *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_2 = (localWriteOffsetB_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_3 = (localWriteOffsetB_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
    localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
    localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x8


    /* iter 15 (last unrolled loop) */

    MAC_4x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_3 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_3 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_3 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_2_1_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 2);
  b_0_3_1_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
  localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
  *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
  *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
  *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 6
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 3
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1536
#define LDS_NUM_ELEMENTS 7680
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 6x8 micro-tile */
#define MAC_6x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I]); \
  TYPE_MAC(rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I]); \
  TYPE_MAC(rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I]); \
  TYPE_MAC(rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I]); \
  TYPE_MAC(rA[5],rB[7],rC[5+7*TT0I]); \

#define MAC_6x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[4+TT1J],rC[4+4*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[4+TT1J],rC[5+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[5+TT1J],rC[4+5*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[5+TT1J],rC[5+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[6+TT1J],rC[4+6*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[6+TT1J],rC[5+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[7+TT1J],rC[4+7*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[7+TT1J],rC[5+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT96x128x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT6_8_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_1_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_1_3_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_2_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_2_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_3_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_2_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_2_1;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_3_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_1);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_1 + 0) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_1 + 0) = b_0_1_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_1 = (localWriteOffsetA_0_0_2_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_1 = (localWriteOffsetB_0_0_2_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_1 = (localWriteOffsetB_0_0_3_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_1);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
    localWriteB_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_1 + 0) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_1 + 0) = b_0_1_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_1 = (localWriteOffsetA_0_0_2_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_1 = (localWriteOffsetB_0_0_2_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_1 = (localWriteOffsetB_0_0_3_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_1);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
    localWriteB_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_6x8


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_6x8


    /* iter 15 (last unrolled loop) */

    MAC_6x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_1_2_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 1);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_1_3_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_1);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_2_1 + 0) = b_0_1_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_3_1 + 0) = b_0_1_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_6x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_6x8
#undef MAC_6x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 7168
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x4 micro-tile */
#define MAC_8x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7],rB[3],rC[7+3*TT0I]); \

#define MAC_8x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT128x64x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_2_1_0;
  DATA_TYPE a_0_3_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_0_2;
  DATA_TYPE *localWriteA_0_0_0_3;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;
  DATA_TYPE *localWriteA_0_0_1_2;
  DATA_TYPE *localWriteA_0_0_1_3;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
  localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_1_2 + 0) = a_0_2_1_0;
    *(localWriteA_0_0_1_3 + 0) = a_0_3_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_2 = (localWriteOffsetA_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_3 = (localWriteOffsetA_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
    localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_1_2 + 0) = a_0_2_1_0;
    *(localWriteA_0_0_1_3 + 0) = a_0_3_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_2 = (localWriteOffsetA_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_3 = (localWriteOffsetA_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
    localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_8x4


    /* iter 15 (last unrolled loop) */

    MAC_8x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_3 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_2 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_3 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_3 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_2_1_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 2);
  a_0_3_1_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
  localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
  *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
  *(localWriteA_0_0_1_2 + 0) = a_0_2_1_0;
  *(localWriteA_0_0_1_3 + 0) = a_0_3_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x4
#undef MAC_8x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT64x64x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_0_2;
  DATA_TYPE *localWriteA_0_0_0_3;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_3 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_3 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
  *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 7168
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT64x128x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_2_1_0;
  DATA_TYPE b_0_3_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_0_2;
  DATA_TYPE *localWriteA_0_0_0_3;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;
  DATA_TYPE *localWriteB_0_0_1_2;
  DATA_TYPE *localWriteB_0_0_1_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
  localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
    *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_2 = (localWriteOffsetB_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_3 = (localWriteOffsetB_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
    localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
    localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
    *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_2 = (localWriteOffsetB_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_3 = (localWriteOffsetB_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
    localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
    localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x8


    /* iter 15 (last unrolled loop) */

    MAC_4x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_3 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_3 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_3 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_2_1_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 2);
  b_0_3_1_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
  localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
  *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
  *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
  *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 7168
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x4 micro-tile */
#define MAC_8x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7],rB[3],rC[7+3*TT0I]); \

#define MAC_8x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT128x64x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_2_1_0;
  DATA_TYPE a_0_3_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_0_2;
  DATA_TYPE *localWriteA_0_0_0_3;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;
  DATA_TYPE *localWriteA_0_0_1_2;
  DATA_TYPE *localWriteA_0_0_1_3;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
  localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_1_2 + 0) = a_0_2_1_0;
    *(localWriteA_0_0_1_3 + 0) = a_0_3_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_2 = (localWriteOffsetA_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_3 = (localWriteOffsetA_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
    localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_1_2 + 0) = a_0_2_1_0;
    *(localWriteA_0_0_1_3 + 0) = a_0_3_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_2 = (localWriteOffsetA_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_3 = (localWriteOffsetA_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
    localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_8x4


    /* iter 15 (last unrolled loop) */

    MAC_8x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_3 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_2 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_3 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_3 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_2_1_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 2);
  a_0_3_1_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
  localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
  *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
  *(localWriteA_0_0_1_2 + 0) = a_0_2_1_0;
  *(localWriteA_0_0_1_3 + 0) = a_0_3_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x4
#undef MAC_8x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 32
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 8192
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT64x64x32_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_2_1_0;
  DATA_TYPE a_0_3_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_2_1_0;
  DATA_TYPE b_0_3_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_0_2;
  DATA_TYPE *localWriteA_0_0_0_3;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;
  DATA_TYPE *localWriteA_0_0_1_2;
  DATA_TYPE *localWriteA_0_0_1_3;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;
  DATA_TYPE *localWriteB_0_0_1_2;
  DATA_TYPE *localWriteB_0_0_1_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
  localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
  localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 1); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_1_2 + 0) = a_0_2_1_0;
    *(localWriteA_0_0_1_3 + 0) = a_0_3_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
    *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_2 = (localWriteOffsetA_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_3 = (localWriteOffsetA_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_2 = (localWriteOffsetB_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_3 = (localWriteOffsetB_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
    localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
    localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
    localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 16 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 17 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 18 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 19 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 20 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 21 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 22 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 23 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 24 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 25 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 26 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 27 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 28 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 29 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 30 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_1_2 + 0) = a_0_2_1_0;
    *(localWriteA_0_0_1_3 + 0) = a_0_3_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
    *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_2 = (localWriteOffsetA_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_3 = (localWriteOffsetA_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_2 = (localWriteOffsetB_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_3 = (localWriteOffsetB_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
    localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
    localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
    localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 31 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 16 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 17 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 18 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 19 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 20 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 21 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 22 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 23 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 24 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 25 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 26 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 27 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 28 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 29 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 30 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 31 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_3 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_2 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_3 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_3 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_3 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_2_1_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 2);
  a_0_3_1_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_2_1_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 2);
  b_0_3_1_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
  localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
  localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
  *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
  *(localWriteA_0_0_1_2 + 0) = a_0_2_1_0;
  *(localWriteA_0_0_1_3 + 0) = a_0_3_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
  *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
  *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 32
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 8192
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT64x64x32_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_2_1_0;
  DATA_TYPE a_0_3_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_2_1_0;
  DATA_TYPE b_0_3_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_0_2;
  DATA_TYPE *localWriteA_0_0_0_3;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;
  DATA_TYPE *localWriteA_0_0_1_2;
  DATA_TYPE *localWriteA_0_0_1_3;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;
  DATA_TYPE *localWriteB_0_0_1_2;
  DATA_TYPE *localWriteB_0_0_1_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
  localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
  localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 1); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_1_2 + 0) = a_0_2_1_0;
    *(localWriteA_0_0_1_3 + 0) = a_0_3_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
    *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_2 = (localWriteOffsetA_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_3 = (localWriteOffsetA_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_2 = (localWriteOffsetB_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_3 = (localWriteOffsetB_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
    localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
    localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
    localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 16 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 17 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 18 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 19 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 20 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 21 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 22 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 23 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 24 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 25 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 26 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 27 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 28 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 29 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 30 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_1_2 + 0) = a_0_2_1_0;
    *(localWriteA_0_0_1_3 + 0) = a_0_3_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
    *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_2 = (localWriteOffsetA_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_3 = (localWriteOffsetA_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_2 = (localWriteOffsetB_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_3 = (localWriteOffsetB_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
    localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
    localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
    localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 31 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 16 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 17 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 18 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 19 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 20 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 21 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 22 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 23 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 24 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 25 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 26 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 27 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 28 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 29 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 30 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 31 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_3 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_2 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_3 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_3 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_3 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_2_1_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 2);
  a_0_3_1_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_2_1_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 2);
  b_0_3_1_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
  localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
  localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
  *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
  *(localWriteA_0_0_1_2 + 0) = a_0_2_1_0;
  *(localWriteA_0_0_1_3 + 0) = a_0_3_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
  *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
  *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 64
#define LDS_NUM_ELEMENTS 409
#define LDS_OFFSET_BLK 128

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT16x16x4_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 3 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 3 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 2
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 64
#define LDS_NUM_ELEMENTS 409
#define LDS_OFFSET_BLK 128

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT32x32x2_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 1 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 1 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE TensileComplexFloat
#define DEST_DATA_TYPE TensileComplexFloat
#define COMPUTE_DATA_TYPE TensileComplexFloat
#define s0 x
#define s1 y
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) \
  DST.s0 = MAC(  MULA.s0, MULB.s0, DST.s0 ); \
  DST.s0 = MAC(  MULA.s1, MULB.s1, DST.s0 ); \
  DST.s1 = MAC(  MULA.s0, MULB.s1, DST.s1 ); \
  DST.s1 = MAC( -MULA.s1, MULB.s0, DST.s1 );
#define TYPE_MAC_WRITE( DST, SRC, ALPHA, REG, BETA ) \
  /* (1) */ \
  type_mac_tmp = REG.s0; \
  REG.s0 *= ALPHA.s0; \
  REG.s0 = MAC( -ALPHA.s1, REG.s1, REG.s0 ); \
  REG.s1 *= ALPHA.s0; \
  REG.s1 = MAC(  ALPHA.s1, type_mac_tmp, REG.s1 ); \
  /* (2) */ \
  if(BETA.s0 != 0) { \
  REG.s0 = MAC(  BETA.s0, SRC.s0, REG.s0 ); \
  REG.s1 = MAC(  BETA.s0, SRC.s1, REG.s1 ); \
  } \
  if (BETA.s1 != 0) { \
  REG.s0 = MAC( -BETA.s1, SRC.s1, REG.s0 ); \
  REG.s1 = MAC(  BETA.s1, SRC.s0, REG.s1 ); \
  } \
  /* (3) */ \
  DST = REG;

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_AlikC_Bjlk_CB_MT32x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW2_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  TensileComplexFloat *D,
  TensileComplexFloat const * __restrict__ C,
  TensileComplexFloat const * __restrict__ A,
  TensileComplexFloat const * __restrict__ B,
  TensileComplexFloat const alpha,
  TensileComplexFloat const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (TensileComplexFloat)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == tensile_complex<float>(0.0f)) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }



  float type_mac_tmp;


  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE TensileComplexFloat
#define DEST_DATA_TYPE TensileComplexFloat
#define COMPUTE_DATA_TYPE TensileComplexFloat
#define s0 x
#define s1 y
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) \
  DST.s0 = MAC(  MULA.s0, MULB.s0, DST.s0 ); \
  DST.s0 = MAC(  MULA.s1, MULB.s1, DST.s0 ); \
  DST.s1 = MAC(  MULA.s0, MULB.s1, DST.s1 ); \
  DST.s1 = MAC( -MULA.s1, MULB.s0, DST.s1 );
#define TYPE_MAC_WRITE( DST, SRC, ALPHA, REG, BETA ) \
  /* (1) */ \
  type_mac_tmp = REG.s0; \
  REG.s0 *= ALPHA.s0; \
  REG.s0 = MAC( -ALPHA.s1, REG.s1, REG.s0 ); \
  REG.s1 *= ALPHA.s0; \
  REG.s1 = MAC(  ALPHA.s1, type_mac_tmp, REG.s1 ); \
  /* (2) */ \
  if(BETA.s0 != 0) { \
  REG.s0 = MAC(  BETA.s0, SRC.s0, REG.s0 ); \
  REG.s1 = MAC(  BETA.s0, SRC.s1, REG.s1 ); \
  } \
  if (BETA.s1 != 0) { \
  REG.s0 = MAC( -BETA.s1, SRC.s1, REG.s0 ); \
  REG.s1 = MAC(  BETA.s1, SRC.s0, REG.s1 ); \
  } \
  /* (3) */ \
  DST = REG;

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_AlikC_Bjlk_CB_MT32x32x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW2_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  TensileComplexFloat *D,
  TensileComplexFloat const * __restrict__ C,
  TensileComplexFloat const * __restrict__ A,
  TensileComplexFloat const * __restrict__ B,
  TensileComplexFloat const alpha,
  TensileComplexFloat const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (TensileComplexFloat)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == tensile_complex<float>(0.0f)) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }



  float type_mac_tmp;


  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 819

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 1) {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 2) {
        rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 819

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 819

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 3 */

    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 1) {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 2) {
        rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 819

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 3 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 3 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 3 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1536

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 1) {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 2) {
        rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1536

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 7 */

    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 1) {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 2) {
        rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 3584
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x8


    /* iter 7 (last unrolled loop) */

    MAC_4x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 1) {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 2) {
        rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 7 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 3072

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT64x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_1_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_1_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_1_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 1);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_1_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 1) {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 2) {
        rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 7168
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT64x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_1_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_1_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x8


    /* iter 15 (last unrolled loop) */

    MAC_4x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_1_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 1);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_1_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 1) {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 2) {
        rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I] , rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I] , rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I] , rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I] , rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I] , rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I] , rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I] , rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I] , rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I] , rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I] , rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I] , rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I] , rA[7],rB[7],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT128x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 819

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 819

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1792
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 819

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 3 */

    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I] , rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I] , rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I] , rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I] , rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I] , rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I] , rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I] , rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I] , rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I] , rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I] , rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I] , rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I] , rA[7],rB[7],rC[7+7*TT0I]); \

#define MAC_8x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I] , rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I] , rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I] , rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I] , rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[4+TT1J],rC[4+4*TT0I] , rA[5+TT0I],rB[4+TT1J],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[4+TT1J],rC[6+4*TT0I] , rA[7+TT0I],rB[4+TT1J],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[5+TT1J],rC[4+5*TT0I] , rA[5+TT0I],rB[5+TT1J],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[5+TT1J],rC[6+5*TT0I] , rA[7+TT0I],rB[5+TT1J],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[6+TT1J],rC[4+6*TT0I] , rA[5+TT0I],rB[6+TT1J],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[6+TT1J],rC[6+6*TT0I] , rA[7+TT0I],rB[6+TT1J],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[7+TT1J],rC[4+7*TT0I] , rA[5+TT0I],rB[7+TT1J],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[7+TT1J],rC[6+7*TT0I] , rA[7+TT0I],rB[7+TT1J],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT128x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8


    /* iter 3 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_8x8


    /* iter 3 (last unrolled loop) */

    MAC_8x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef MAC_8x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1792
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 3 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x8


    /* iter 3 (last unrolled loop) */

    MAC_4x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1536

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 7 */

    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 7 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 8

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 7168
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT64x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;
  unsigned int globalReadOffsetBL_4_0 = globalReadOffsetBL + 0 + 4*LSPB;
  unsigned int globalReadOffsetBL_5_0 = globalReadOffsetBL + 0 + 5*LSPB;
  unsigned int globalReadOffsetBL_6_0 = globalReadOffsetBL + 0 + 6*LSPB;
  unsigned int globalReadOffsetBL_7_0 = globalReadOffsetBL + 0 + 7*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_7_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 4*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 5*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 6*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 7*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_4_0 = (localWriteOffsetB_0_0_4_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_5_0 = (localWriteOffsetB_0_0_5_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_6_0 = (localWriteOffsetB_0_0_6_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_7_0 = (localWriteOffsetB_0_0_7_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
    localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
    localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
    localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
    localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_4_0 = (localWriteOffsetB_0_0_4_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_5_0 = (localWriteOffsetB_0_0_5_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_6_0 = (localWriteOffsetB_0_0_6_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_7_0 = (localWriteOffsetB_0_0_7_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
    localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
    localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
    localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
    localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x8


    /* iter 15 (last unrolled loop) */

    MAC_4x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_4_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_5_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_6_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_7_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 896
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT32x16x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x2


    /* iter 7 (last unrolled loop) */

    MAC_4x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT32x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 7 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
  *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 2) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 3) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 2) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 3) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 8
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1792
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x4 micro-tile */
#define MAC_8x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7],rB[3],rC[7+3*TT0I]); \

#define MAC_8x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT64x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_2_1_0;
  DATA_TYPE a_0_3_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
    *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
    *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_8x4


    /* iter 7 (last unrolled loop) */

    MAC_8x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_2_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 2);
  a_0_3_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
  *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
  *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 2) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 3) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 2) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 3) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x4
#undef MAC_8x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 896
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT32x16x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1664
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT64x16x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x2


    /* iter 7 (last unrolled loop) */

    MAC_4x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT32x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 7 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
  *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 2) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 3) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 2) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 3) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1664
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT64x16x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_8_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x2


    /* iter 7 (last unrolled loop) */

    MAC_4x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT16x16x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 15 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1792
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT32x16x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 15 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I]); \
  TYPE_MAC(rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I]); \
  TYPE_MAC(rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I]); \
  TYPE_MAC(rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I]); \
  TYPE_MAC(rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I]); \
  TYPE_MAC(rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I]); \
  TYPE_MAC(rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I]); \
  TYPE_MAC(rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I]); \
  TYPE_MAC(rA[7],rB[7],rC[7+7*TT0I]); \

#define MAC_8x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[4+TT1J],rC[4+4*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[4+TT1J],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[4+TT1J],rC[6+4*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[4+TT1J],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[5+TT1J],rC[4+5*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[5+TT1J],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[5+TT1J],rC[6+5*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[5+TT1J],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[6+TT1J],rC[4+6*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[6+TT1J],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[6+TT1J],rC[6+6*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[6+TT1J],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[7+TT1J],rC[4+7*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[7+TT1J],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[7+TT1J],rC[6+7*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[7+TT1J],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT128x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_8x8


    /* iter 7 (last unrolled loop) */

    MAC_8x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
  *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
    if (r0I == 2) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
    if (r0I == 3) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 2) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 3) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef MAC_8x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
  *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 2) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 3) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 2) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 3) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
  *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 2) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 3) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 2) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 3) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 7168
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT64x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_2_1_0;
  DATA_TYPE b_0_3_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
    *(localWriteB_0_0_1_0 + 2) = b_0_2_1_0;
    *(localWriteB_0_0_1_0 + 3) = b_0_3_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
    *(localWriteB_0_0_1_0 + 2) = b_0_2_1_0;
    *(localWriteB_0_0_1_0 + 3) = b_0_3_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x8


    /* iter 15 (last unrolled loop) */

    MAC_4x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_2_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 2);
  b_0_3_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
  *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
  *(localWriteB_0_0_1_0 + 2) = b_0_2_1_0;
  *(localWriteB_0_0_1_0 + 3) = b_0_3_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
    if (r0I == 2) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
    if (r0I == 3) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 2) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 3) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 64
#define LDS_NUM_ELEMENTS 409
#define LDS_OFFSET_BLK 128

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT16x16x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 3 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 3 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 2
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 64
#define LDS_NUM_ELEMENTS 409
#define LDS_OFFSET_BLK 128

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT32x32x2_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 1 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 1 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 896
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT32x16x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x2


    /* iter 7 (last unrolled loop) */

    MAC_4x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT32x32x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 7 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
  *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 2) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 3) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 2) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 3) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 8
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1792
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x4 micro-tile */
#define MAC_8x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7],rB[3],rC[7+3*TT0I]); \

#define MAC_8x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT64x32x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_2_1_0;
  DATA_TYPE a_0_3_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
    *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
    *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_8x4


    /* iter 7 (last unrolled loop) */

    MAC_8x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_2_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 2);
  a_0_3_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
  *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
  *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 2) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 3) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 2) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 3) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x4
#undef MAC_8x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 896
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT32x16x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1664
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT64x16x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x2


    /* iter 7 (last unrolled loop) */

    MAC_4x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT32x32x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 7 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
  *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 2) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 3) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 2) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 3) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1664
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT64x16x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_8_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x2


    /* iter 7 (last unrolled loop) */

    MAC_4x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT16x16x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 15 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1792
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT32x16x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 15 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I]); \
  TYPE_MAC(rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I]); \
  TYPE_MAC(rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I]); \
  TYPE_MAC(rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I]); \
  TYPE_MAC(rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I]); \
  TYPE_MAC(rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I]); \
  TYPE_MAC(rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I]); \
  TYPE_MAC(rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I]); \
  TYPE_MAC(rA[7],rB[7],rC[7+7*TT0I]); \

#define MAC_8x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[4+TT1J],rC[4+4*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[4+TT1J],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[4+TT1J],rC[6+4*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[4+TT1J],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[5+TT1J],rC[4+5*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[5+TT1J],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[5+TT1J],rC[6+5*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[5+TT1J],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[6+TT1J],rC[4+6*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[6+TT1J],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[6+TT1J],rC[6+6*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[6+TT1J],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[7+TT1J],rC[4+7*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[7+TT1J],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[7+TT1J],rC[6+7*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[7+TT1J],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT128x128x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_8x8


    /* iter 7 (last unrolled loop) */

    MAC_8x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
  *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
    if (r0I == 2) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
    if (r0I == 3) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 2) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 3) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef MAC_8x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT64x64x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
  *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 2) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 3) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 2) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 3) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT64x64x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
  *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 2) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 3) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 2) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 3) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 7168
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT64x128x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_2_1_0;
  DATA_TYPE b_0_3_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
    *(localWriteB_0_0_1_0 + 2) = b_0_2_1_0;
    *(localWriteB_0_0_1_0 + 3) = b_0_3_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
    *(localWriteB_0_0_1_0 + 2) = b_0_2_1_0;
    *(localWriteB_0_0_1_0 + 3) = b_0_3_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x8


    /* iter 15 (last unrolled loop) */

    MAC_4x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_2_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 2);
  b_0_3_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
  *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
  *(localWriteB_0_0_1_0 + 2) = b_0_2_1_0;
  *(localWriteB_0_0_1_0 + 3) = b_0_3_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
    if (r0I == 2) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
    if (r0I == 3) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 2) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 3) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 64
#define LDS_NUM_ELEMENTS 409
#define LDS_OFFSET_BLK 128

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT16x16x4_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 3 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 3 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 2
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 64
#define LDS_NUM_ELEMENTS 409
#define LDS_OFFSET_BLK 128

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT32x32x2_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 1 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 1 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 128
#define LDS_NUM_ELEMENTS 512
#define LDS_OFFSET_BLK 256

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_4xi8BH_MT16x16x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 896
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_4xi8BH_MT32x16x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_8_1_WGM1(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_4xi8BH_MT16x16x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 15 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1792
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_4xi8BH_MT32x16x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_8_1_WGM1(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 15 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 6
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 3
#define NLCB 1
#define NLPA 1
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1536
#define LDS_NUM_ELEMENTS 7680
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 6x8 micro-tile */
#define MAC_6x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I]); \
  TYPE_MAC(rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I]); \
  TYPE_MAC(rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I]); \
  TYPE_MAC(rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I]); \
  TYPE_MAC(rA[5],rB[7],rC[5+7*TT0I]); \

#define MAC_6x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[4+TT1J],rC[4+4*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[4+TT1J],rC[5+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[5+TT1J],rC[4+5*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[5+TT1J],rC[5+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[6+TT1J],rC[4+6*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[6+TT1J],rC[5+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[7+TT1J],rC[4+7*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[7+TT1J],rC[5+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_4xi8BH_MT96x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA3_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT6_8_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_1_0_0_0;
  DATA_TYPE a_1_1_0_0;
  DATA_TYPE a_2_0_0_0;
  DATA_TYPE a_2_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_1_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_1_3_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSCA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSCA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_2_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_1_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_1_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_2_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_2_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_1_0_0_0 = A + globalReadOffsetA_1_0_0_0;
  DATA_TYPE const *globalReadA_2_0_0_0 = A + globalReadOffsetA_2_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_1_0_0_0 = localWriteFirstOffsetA + (0 + 1*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_2_0_0_0 = localWriteFirstOffsetA + (0 + 2*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_1_0_0_0;
  DATA_TYPE *localWriteA_2_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_1_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_0_0);
  localWriteA_2_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_2_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_1_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_2_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_1_0_0_0 = *(globalReadA_1_0_0_0 + 0);
    a_1_1_0_0 = *(globalReadA_1_0_0_0 + 1);
    a_2_0_0_0 = *(globalReadA_2_0_0_0 + 0);
    a_2_1_0_0 = *(globalReadA_2_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_1_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_1_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_1_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_2_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_2_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_2_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_1_0_0_0 + 0) = a_1_0_0_0;
    *(localWriteA_1_0_0_0 + 1) = a_1_1_0_0;
    *(localWriteA_2_0_0_0 + 0) = a_2_0_0_0;
    *(localWriteA_2_0_0_0 + 1) = a_2_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_1_0_0_0 = (localWriteOffsetA_1_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_2_0_0_0 = (localWriteOffsetA_2_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_1_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_0_0);
    localWriteA_2_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_2_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_1_0_0_0 = *(globalReadA_1_0_0_0 + 0);
    a_1_1_0_0 = *(globalReadA_1_0_0_0 + 1);
    a_2_0_0_0 = *(globalReadA_2_0_0_0 + 0);
    a_2_1_0_0 = *(globalReadA_2_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_1_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_1_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_1_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_2_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_2_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_2_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_1_0_0_0 + 0) = a_1_0_0_0;
    *(localWriteA_1_0_0_0 + 1) = a_1_1_0_0;
    *(localWriteA_2_0_0_0 + 0) = a_2_0_0_0;
    *(localWriteA_2_0_0_0 + 1) = a_2_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_1_0_0_0 = (localWriteOffsetA_1_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_2_0_0_0 = (localWriteOffsetA_2_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_1_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_0_0);
    localWriteA_2_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_2_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_6x8


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_6x8


    /* iter 15 (last unrolled loop) */

    MAC_6x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_1_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_2_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_1_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_2_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_1_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_1_0_0_0 + 0);
  a_1_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_1_0_0_0 + 1);
  a_2_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_2_0_0_0 + 0);
  a_2_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_2_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_1_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 1);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_1_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_1_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_0_0);
  localWriteA_2_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_2_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_1_0_0_0 + 0) = a_1_0_0_0;
  *(localWriteA_1_0_0_0 + 1) = a_1_1_0_0;
  *(localWriteA_2_0_0_0 + 0) = a_2_0_0_0;
  *(localWriteA_2_0_0_0 + 1) = a_2_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_6x8

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else if (s0I == 1) {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else {
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 1) {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 2) {
        rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_6x8
#undef MAC_6x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 64
#define LDS_NUM_ELEMENTS 409
#define LDS_OFFSET_BLK 128

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_4xi8BH_MT16x16x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 3 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 3 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 2
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 64
#define LDS_NUM_ELEMENTS 409
#define LDS_OFFSET_BLK 128

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_4xi8BH_MT32x32x2_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 1 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 1 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1536

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
  a_pk_fma[0] = MULA0; \
  a_pk_fma[1] = MULA1; \
  b_pk_fma[0] = MULB0; \
  b_pk_fma[1] = MULB1; \
  c_pk_fma[0] = DST0; \
  c_pk_fma[1] = DST1; \
  c_pk_fma = tensile_fmadd_half2(a_pk_fma, b_pk_fma, c_pk_fma); \
  DST0 = c_pk_fma[0]; \
  DST1 = c_pk_fma[1]; \
 
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HB_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 819

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
  a_pk_fma[0] = MULA0; \
  a_pk_fma[1] = MULA1; \
  b_pk_fma[0] = MULB0; \
  b_pk_fma[1] = MULB1; \
  c_pk_fma[0] = DST0; \
  c_pk_fma[1] = DST1; \
  c_pk_fma = tensile_fmadd_half2(a_pk_fma, b_pk_fma, c_pk_fma); \
  DST0 = c_pk_fma[0]; \
  DST1 = c_pk_fma[1]; \
 
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HB_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
  a_pk_fma[0] = MULA0; \
  a_pk_fma[1] = MULA1; \
  b_pk_fma[0] = MULB0; \
  b_pk_fma[1] = MULB1; \
  c_pk_fma[0] = DST0; \
  c_pk_fma[1] = DST1; \
  c_pk_fma = tensile_fmadd_half2(a_pk_fma, b_pk_fma, c_pk_fma); \
  DST0 = c_pk_fma[0]; \
  DST1 = c_pk_fma[1]; \
 
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HB_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 7 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1536

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
  a_pk_fma[0] = MULA0; \
  a_pk_fma[1] = MULA1; \
  b_pk_fma[0] = MULB0; \
  b_pk_fma[1] = MULB1; \
  c_pk_fma[0] = DST0; \
  c_pk_fma[1] = DST1; \
  c_pk_fma = tensile_fmadd_half2(a_pk_fma, b_pk_fma, c_pk_fma); \
  DST0 = c_pk_fma[0]; \
  DST1 = c_pk_fma[1]; \
 
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HB_MT64x128x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 819

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
  a_pk_fma[0] = MULA0; \
  a_pk_fma[1] = MULA1; \
  b_pk_fma[0] = MULB0; \
  b_pk_fma[1] = MULB1; \
  c_pk_fma[0] = DST0; \
  c_pk_fma[1] = DST1; \
  c_pk_fma = tensile_fmadd_half2(a_pk_fma, b_pk_fma, c_pk_fma); \
  DST0 = c_pk_fma[0]; \
  DST1 = c_pk_fma[1]; \
 
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HB_MT64x128x4_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
  a_pk_fma[0] = MULA0; \
  a_pk_fma[1] = MULA1; \
  b_pk_fma[0] = MULB0; \
  b_pk_fma[1] = MULB1; \
  c_pk_fma[0] = DST0; \
  c_pk_fma[1] = DST1; \
  c_pk_fma = tensile_fmadd_half2(a_pk_fma, b_pk_fma, c_pk_fma); \
  DST0 = c_pk_fma[0]; \
  DST1 = c_pk_fma[1]; \
 
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HB_MT64x64x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 7 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 768

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_4xi8BH_MT64x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 7 */

    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_4xi8BH_MT64x16x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_1_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_1_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_1_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_1_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;
  unsigned int flattenedOffsetA_4_0 = globalReadOffsetA0I + 0 + 4*LSPA;
  flattenedOffsetA_4_0 = (flattenedOffsetA_4_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_4_0;
  unsigned int globalReadOffsetA0I_4_0 = flattenedOffsetA_4_0;
  unsigned int flattenedOffsetA_5_0 = globalReadOffsetA0I + 0 + 5*LSPA;
  flattenedOffsetA_5_0 = (flattenedOffsetA_5_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_5_0;
  unsigned int globalReadOffsetA0I_5_0 = flattenedOffsetA_5_0;
  unsigned int flattenedOffsetA_6_0 = globalReadOffsetA0I + 0 + 6*LSPA;
  flattenedOffsetA_6_0 = (flattenedOffsetA_6_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_6_0;
  unsigned int globalReadOffsetA0I_6_0 = flattenedOffsetA_6_0;
  unsigned int flattenedOffsetA_7_0 = globalReadOffsetA0I + 0 + 7*LSPA;
  flattenedOffsetA_7_0 = (flattenedOffsetA_7_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_7_0;
  unsigned int globalReadOffsetA0I_7_0 = flattenedOffsetA_7_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;
  globalReadOffsetA0I_4_0 = (  globalReadOffsetA0I_4_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_4_0;
  globalReadOffsetA0I_5_0 = (  globalReadOffsetA0I_5_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_5_0;
  globalReadOffsetA0I_6_0 = (  globalReadOffsetA0I_6_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_6_0;
  globalReadOffsetA0I_7_0 = (  globalReadOffsetA0I_7_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_7_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_2_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_3_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 4*LSPA);
  unsigned int localWriteOffsetA_0_0_4_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 4*LSPA);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 5*LSPA);
  unsigned int localWriteOffsetA_0_0_5_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 5*LSPA);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 6*LSPA);
  unsigned int localWriteOffsetA_0_0_6_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 6*LSPA);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 7*LSPA);
  unsigned int localWriteOffsetA_0_0_7_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 7*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_2_1;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_3_1;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_4_1;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_5_1;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_6_1;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_7_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_4_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_1);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_5_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_1);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_6_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_1);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_7_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_1_4_0 = *(globalReadA_0_0_4_0 + 1);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_1_5_0 = *(globalReadA_0_0_5_0 + 1);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_1_6_0 = *(globalReadA_0_0_6_0 + 1);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_1_7_0 = *(globalReadA_0_0_7_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_1 + 0) = a_0_1_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_4_1 + 0) = a_0_1_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_5_1 + 0) = a_0_1_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_6_1 + 0) = a_0_1_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_7_1 + 0) = a_0_1_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);
  a_0_0_4_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_1_4_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 1);
  a_0_0_5_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_1_5_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 1);
  a_0_0_6_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_1_6_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 1);
  a_0_0_7_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_1_7_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_4_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_1);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_5_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_1);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_6_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_1);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_7_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_1 + 0) = a_0_1_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_4_1 + 0) = a_0_1_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_5_1 + 0) = a_0_1_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_6_1 + 0) = a_0_1_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_7_1 + 0) = a_0_1_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_4xi8BH_MT128x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;
  unsigned int flattenedOffsetA_4_0 = globalReadOffsetA0I + 0 + 4*LSPA;
  flattenedOffsetA_4_0 = (flattenedOffsetA_4_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_4_0;
  unsigned int globalReadOffsetA0I_4_0 = flattenedOffsetA_4_0;
  unsigned int flattenedOffsetA_5_0 = globalReadOffsetA0I + 0 + 5*LSPA;
  flattenedOffsetA_5_0 = (flattenedOffsetA_5_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_5_0;
  unsigned int globalReadOffsetA0I_5_0 = flattenedOffsetA_5_0;
  unsigned int flattenedOffsetA_6_0 = globalReadOffsetA0I + 0 + 6*LSPA;
  flattenedOffsetA_6_0 = (flattenedOffsetA_6_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_6_0;
  unsigned int globalReadOffsetA0I_6_0 = flattenedOffsetA_6_0;
  unsigned int flattenedOffsetA_7_0 = globalReadOffsetA0I + 0 + 7*LSPA;
  flattenedOffsetA_7_0 = (flattenedOffsetA_7_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_7_0;
  unsigned int globalReadOffsetA0I_7_0 = flattenedOffsetA_7_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;
  globalReadOffsetA0I_4_0 = (  globalReadOffsetA0I_4_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_4_0;
  globalReadOffsetA0I_5_0 = (  globalReadOffsetA0I_5_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_5_0;
  globalReadOffsetA0I_6_0 = (  globalReadOffsetA0I_6_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_6_0;
  globalReadOffsetA0I_7_0 = (  globalReadOffsetA0I_7_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_7_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 4*LSPA);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 5*LSPA);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 6*LSPA);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 7*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 16
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_4xi8BH_MT64x16x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;
  unsigned int flattenedOffsetA_4_0 = globalReadOffsetA0I + 0 + 4*LSPA;
  flattenedOffsetA_4_0 = (flattenedOffsetA_4_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_4_0;
  unsigned int globalReadOffsetA0I_4_0 = flattenedOffsetA_4_0;
  unsigned int flattenedOffsetA_5_0 = globalReadOffsetA0I + 0 + 5*LSPA;
  flattenedOffsetA_5_0 = (flattenedOffsetA_5_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_5_0;
  unsigned int globalReadOffsetA0I_5_0 = flattenedOffsetA_5_0;
  unsigned int flattenedOffsetA_6_0 = globalReadOffsetA0I + 0 + 6*LSPA;
  flattenedOffsetA_6_0 = (flattenedOffsetA_6_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_6_0;
  unsigned int globalReadOffsetA0I_6_0 = flattenedOffsetA_6_0;
  unsigned int flattenedOffsetA_7_0 = globalReadOffsetA0I + 0 + 7*LSPA;
  flattenedOffsetA_7_0 = (flattenedOffsetA_7_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_7_0;
  unsigned int globalReadOffsetA0I_7_0 = flattenedOffsetA_7_0;
  unsigned int flattenedOffsetA_8_0 = globalReadOffsetA0I + 0 + 8*LSPA;
  flattenedOffsetA_8_0 = (flattenedOffsetA_8_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_8_0;
  unsigned int globalReadOffsetA0I_8_0 = flattenedOffsetA_8_0;
  unsigned int flattenedOffsetA_9_0 = globalReadOffsetA0I + 0 + 9*LSPA;
  flattenedOffsetA_9_0 = (flattenedOffsetA_9_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_9_0;
  unsigned int globalReadOffsetA0I_9_0 = flattenedOffsetA_9_0;
  unsigned int flattenedOffsetA_10_0 = globalReadOffsetA0I + 0 + 10*LSPA;
  flattenedOffsetA_10_0 = (flattenedOffsetA_10_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_10_0;
  unsigned int globalReadOffsetA0I_10_0 = flattenedOffsetA_10_0;
  unsigned int flattenedOffsetA_11_0 = globalReadOffsetA0I + 0 + 11*LSPA;
  flattenedOffsetA_11_0 = (flattenedOffsetA_11_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_11_0;
  unsigned int globalReadOffsetA0I_11_0 = flattenedOffsetA_11_0;
  unsigned int flattenedOffsetA_12_0 = globalReadOffsetA0I + 0 + 12*LSPA;
  flattenedOffsetA_12_0 = (flattenedOffsetA_12_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_12_0;
  unsigned int globalReadOffsetA0I_12_0 = flattenedOffsetA_12_0;
  unsigned int flattenedOffsetA_13_0 = globalReadOffsetA0I + 0 + 13*LSPA;
  flattenedOffsetA_13_0 = (flattenedOffsetA_13_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_13_0;
  unsigned int globalReadOffsetA0I_13_0 = flattenedOffsetA_13_0;
  unsigned int flattenedOffsetA_14_0 = globalReadOffsetA0I + 0 + 14*LSPA;
  flattenedOffsetA_14_0 = (flattenedOffsetA_14_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_14_0;
  unsigned int globalReadOffsetA0I_14_0 = flattenedOffsetA_14_0;
  unsigned int flattenedOffsetA_15_0 = globalReadOffsetA0I + 0 + 15*LSPA;
  flattenedOffsetA_15_0 = (flattenedOffsetA_15_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_15_0;
  unsigned int globalReadOffsetA0I_15_0 = flattenedOffsetA_15_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;
  globalReadOffsetA0I_4_0 = (  globalReadOffsetA0I_4_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_4_0;
  globalReadOffsetA0I_5_0 = (  globalReadOffsetA0I_5_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_5_0;
  globalReadOffsetA0I_6_0 = (  globalReadOffsetA0I_6_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_6_0;
  globalReadOffsetA0I_7_0 = (  globalReadOffsetA0I_7_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_7_0;
  globalReadOffsetA0I_8_0 = (  globalReadOffsetA0I_8_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_8_0;
  globalReadOffsetA0I_9_0 = (  globalReadOffsetA0I_9_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_9_0;
  globalReadOffsetA0I_10_0 = (  globalReadOffsetA0I_10_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_10_0;
  globalReadOffsetA0I_11_0 = (  globalReadOffsetA0I_11_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_11_0;
  globalReadOffsetA0I_12_0 = (  globalReadOffsetA0I_12_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_12_0;
  globalReadOffsetA0I_13_0 = (  globalReadOffsetA0I_13_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_13_0;
  globalReadOffsetA0I_14_0 = (  globalReadOffsetA0I_14_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_14_0;
  globalReadOffsetA0I_15_0 = (  globalReadOffsetA0I_15_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_15_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_15_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 4*LSPA);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 5*LSPA);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 6*LSPA);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 7*LSPA);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 8*LSPA);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 9*LSPA);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 10*LSPA);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 11*LSPA);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 12*LSPA);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 13*LSPA);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 14*LSPA);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 15*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_0_0_15_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_8_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_9_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_10_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_11_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_12_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_13_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_14_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_15_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_8_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_9_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_10_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_11_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_12_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_13_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_14_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_15_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_0_8_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_0_0_9_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_0_0_10_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_0_0_11_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_0_0_12_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_0_0_13_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_0_0_14_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_0_0_15_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 16

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x8 micro-tile */
#define MAC_2x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \

#define MAC_2x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_4xi8BH_MT16x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_8_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;
  DATA_TYPE b_0_0_8_0;
  DATA_TYPE b_0_0_9_0;
  DATA_TYPE b_0_0_10_0;
  DATA_TYPE b_0_0_11_0;
  DATA_TYPE b_0_0_12_0;
  DATA_TYPE b_0_0_13_0;
  DATA_TYPE b_0_0_14_0;
  DATA_TYPE b_0_0_15_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;
  unsigned int flattenedOffsetB_4_0 = globalReadOffsetB1J + 0 + 4*LSPB;
  flattenedOffsetB_4_0 = (flattenedOffsetB_4_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_4_0;
  unsigned int globalReadOffsetB1J_4_0 = flattenedOffsetB_4_0;
  unsigned int flattenedOffsetB_5_0 = globalReadOffsetB1J + 0 + 5*LSPB;
  flattenedOffsetB_5_0 = (flattenedOffsetB_5_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_5_0;
  unsigned int globalReadOffsetB1J_5_0 = flattenedOffsetB_5_0;
  unsigned int flattenedOffsetB_6_0 = globalReadOffsetB1J + 0 + 6*LSPB;
  flattenedOffsetB_6_0 = (flattenedOffsetB_6_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_6_0;
  unsigned int globalReadOffsetB1J_6_0 = flattenedOffsetB_6_0;
  unsigned int flattenedOffsetB_7_0 = globalReadOffsetB1J + 0 + 7*LSPB;
  flattenedOffsetB_7_0 = (flattenedOffsetB_7_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_7_0;
  unsigned int globalReadOffsetB1J_7_0 = flattenedOffsetB_7_0;
  unsigned int flattenedOffsetB_8_0 = globalReadOffsetB1J + 0 + 8*LSPB;
  flattenedOffsetB_8_0 = (flattenedOffsetB_8_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_8_0;
  unsigned int globalReadOffsetB1J_8_0 = flattenedOffsetB_8_0;
  unsigned int flattenedOffsetB_9_0 = globalReadOffsetB1J + 0 + 9*LSPB;
  flattenedOffsetB_9_0 = (flattenedOffsetB_9_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_9_0;
  unsigned int globalReadOffsetB1J_9_0 = flattenedOffsetB_9_0;
  unsigned int flattenedOffsetB_10_0 = globalReadOffsetB1J + 0 + 10*LSPB;
  flattenedOffsetB_10_0 = (flattenedOffsetB_10_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_10_0;
  unsigned int globalReadOffsetB1J_10_0 = flattenedOffsetB_10_0;
  unsigned int flattenedOffsetB_11_0 = globalReadOffsetB1J + 0 + 11*LSPB;
  flattenedOffsetB_11_0 = (flattenedOffsetB_11_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_11_0;
  unsigned int globalReadOffsetB1J_11_0 = flattenedOffsetB_11_0;
  unsigned int flattenedOffsetB_12_0 = globalReadOffsetB1J + 0 + 12*LSPB;
  flattenedOffsetB_12_0 = (flattenedOffsetB_12_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_12_0;
  unsigned int globalReadOffsetB1J_12_0 = flattenedOffsetB_12_0;
  unsigned int flattenedOffsetB_13_0 = globalReadOffsetB1J + 0 + 13*LSPB;
  flattenedOffsetB_13_0 = (flattenedOffsetB_13_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_13_0;
  unsigned int globalReadOffsetB1J_13_0 = flattenedOffsetB_13_0;
  unsigned int flattenedOffsetB_14_0 = globalReadOffsetB1J + 0 + 14*LSPB;
  flattenedOffsetB_14_0 = (flattenedOffsetB_14_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_14_0;
  unsigned int globalReadOffsetB1J_14_0 = flattenedOffsetB_14_0;
  unsigned int flattenedOffsetB_15_0 = globalReadOffsetB1J + 0 + 15*LSPB;
  flattenedOffsetB_15_0 = (flattenedOffsetB_15_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_15_0;
  unsigned int globalReadOffsetB1J_15_0 = flattenedOffsetB_15_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;
  globalReadOffsetB1J_4_0 = (  globalReadOffsetB1J_4_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_4_0;
  globalReadOffsetB1J_5_0 = (  globalReadOffsetB1J_5_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_5_0;
  globalReadOffsetB1J_6_0 = (  globalReadOffsetB1J_6_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_6_0;
  globalReadOffsetB1J_7_0 = (  globalReadOffsetB1J_7_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_7_0;
  globalReadOffsetB1J_8_0 = (  globalReadOffsetB1J_8_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_8_0;
  globalReadOffsetB1J_9_0 = (  globalReadOffsetB1J_9_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_9_0;
  globalReadOffsetB1J_10_0 = (  globalReadOffsetB1J_10_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_10_0;
  globalReadOffsetB1J_11_0 = (  globalReadOffsetB1J_11_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_11_0;
  globalReadOffsetB1J_12_0 = (  globalReadOffsetB1J_12_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_12_0;
  globalReadOffsetB1J_13_0 = (  globalReadOffsetB1J_13_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_13_0;
  globalReadOffsetB1J_14_0 = (  globalReadOffsetB1J_14_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_14_0;
  globalReadOffsetB1J_15_0 = (  globalReadOffsetB1J_15_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_15_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_7_0), wgK );
  int64_t globalReadOffsetB_0_0_8_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_8_0), wgK );
  int64_t globalReadOffsetB_0_0_9_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_9_0), wgK );
  int64_t globalReadOffsetB_0_0_10_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_10_0), wgK );
  int64_t globalReadOffsetB_0_0_11_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_11_0), wgK );
  int64_t globalReadOffsetB_0_0_12_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_12_0), wgK );
  int64_t globalReadOffsetB_0_0_13_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_13_0), wgK );
  int64_t globalReadOffsetB_0_0_14_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_14_0), wgK );
  int64_t globalReadOffsetB_0_0_15_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_15_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;
  DATA_TYPE const *globalReadB_0_0_8_0 = B + globalReadOffsetB_0_0_8_0;
  DATA_TYPE const *globalReadB_0_0_9_0 = B + globalReadOffsetB_0_0_9_0;
  DATA_TYPE const *globalReadB_0_0_10_0 = B + globalReadOffsetB_0_0_10_0;
  DATA_TYPE const *globalReadB_0_0_11_0 = B + globalReadOffsetB_0_0_11_0;
  DATA_TYPE const *globalReadB_0_0_12_0 = B + globalReadOffsetB_0_0_12_0;
  DATA_TYPE const *globalReadB_0_0_13_0 = B + globalReadOffsetB_0_0_13_0;
  DATA_TYPE const *globalReadB_0_0_14_0 = B + globalReadOffsetB_0_0_14_0;
  DATA_TYPE const *globalReadB_0_0_15_0 = B + globalReadOffsetB_0_0_15_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 4*LSPB);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 5*LSPB);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 6*LSPB);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 7*LSPB);
  unsigned int localWriteOffsetB_0_0_8_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 8*LSPB);
  unsigned int localWriteOffsetB_0_0_9_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 9*LSPB);
  unsigned int localWriteOffsetB_0_0_10_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 10*LSPB);
  unsigned int localWriteOffsetB_0_0_11_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 11*LSPB);
  unsigned int localWriteOffsetB_0_0_12_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 12*LSPB);
  unsigned int localWriteOffsetB_0_0_13_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 13*LSPB);
  unsigned int localWriteOffsetB_0_0_14_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 14*LSPB);
  unsigned int localWriteOffsetB_0_0_15_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 15*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;
  DATA_TYPE *localWriteB_0_0_8_0;
  DATA_TYPE *localWriteB_0_0_9_0;
  DATA_TYPE *localWriteB_0_0_10_0;
  DATA_TYPE *localWriteB_0_0_11_0;
  DATA_TYPE *localWriteB_0_0_12_0;
  DATA_TYPE *localWriteB_0_0_13_0;
  DATA_TYPE *localWriteB_0_0_14_0;
  DATA_TYPE *localWriteB_0_0_15_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);
  localWriteB_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_8_0);
  localWriteB_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_9_0);
  localWriteB_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_10_0);
  localWriteB_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_11_0);
  localWriteB_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_12_0);
  localWriteB_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_13_0);
  localWriteB_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_14_0);
  localWriteB_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_15_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_8_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_9_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_10_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_11_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_12_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_13_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_14_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_15_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);
    b_0_0_8_0 = *(globalReadB_0_0_8_0 + 0);
    b_0_0_9_0 = *(globalReadB_0_0_9_0 + 0);
    b_0_0_10_0 = *(globalReadB_0_0_10_0 + 0);
    b_0_0_11_0 = *(globalReadB_0_0_11_0 + 0);
    b_0_0_12_0 = *(globalReadB_0_0_12_0 + 0);
    b_0_0_13_0 = *(globalReadB_0_0_13_0 + 0);
    b_0_0_14_0 = *(globalReadB_0_0_14_0 + 0);
    b_0_0_15_0 = *(globalReadB_0_0_15_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_8_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_8_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_9_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_9_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_10_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_10_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_11_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_11_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_12_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_12_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_13_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_13_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_14_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_14_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_15_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_15_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
    *(localWriteB_0_0_8_0 + 0) = b_0_0_8_0;
    *(localWriteB_0_0_9_0 + 0) = b_0_0_9_0;
    *(localWriteB_0_0_10_0 + 0) = b_0_0_10_0;
    *(localWriteB_0_0_11_0 + 0) = b_0_0_11_0;
    *(localWriteB_0_0_12_0 + 0) = b_0_0_12_0;
    *(localWriteB_0_0_13_0 + 0) = b_0_0_13_0;
    *(localWriteB_0_0_14_0 + 0) = b_0_0_14_0;
    *(localWriteB_0_0_15_0 + 0) = b_0_0_15_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x8


    /* iter 15 */

    MAC_2x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);
  b_0_0_8_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_8_0 + 0);
  b_0_0_9_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_9_0 + 0);
  b_0_0_10_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_10_0 + 0);
  b_0_0_11_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_11_0 + 0);
  b_0_0_12_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_12_0 + 0);
  b_0_0_13_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_13_0 + 0);
  b_0_0_14_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_14_0 + 0);
  b_0_0_15_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_15_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);
  localWriteB_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_8_0);
  localWriteB_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_9_0);
  localWriteB_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_10_0);
  localWriteB_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_11_0);
  localWriteB_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_12_0);
  localWriteB_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_13_0);
  localWriteB_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_14_0);
  localWriteB_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_15_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
  *(localWriteB_0_0_8_0 + 0) = b_0_0_8_0;
  *(localWriteB_0_0_9_0 + 0) = b_0_0_9_0;
  *(localWriteB_0_0_10_0 + 0) = b_0_0_10_0;
  *(localWriteB_0_0_11_0 + 0) = b_0_0_11_0;
  *(localWriteB_0_0_12_0 + 0) = b_0_0_12_0;
  *(localWriteB_0_0_13_0 + 0) = b_0_0_13_0;
  *(localWriteB_0_0_14_0 + 0) = b_0_0_14_0;
  *(localWriteB_0_0_15_0 + 0) = b_0_0_15_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x8
#undef MAC_2x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 768

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_4xi8BH_MT64x32x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 7 */

    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_4xi8BH_MT64x16x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_1_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_1_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_1_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_1_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;
  unsigned int flattenedOffsetA_4_0 = globalReadOffsetA0I + 0 + 4*LSPA;
  flattenedOffsetA_4_0 = (flattenedOffsetA_4_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_4_0;
  unsigned int globalReadOffsetA0I_4_0 = flattenedOffsetA_4_0;
  unsigned int flattenedOffsetA_5_0 = globalReadOffsetA0I + 0 + 5*LSPA;
  flattenedOffsetA_5_0 = (flattenedOffsetA_5_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_5_0;
  unsigned int globalReadOffsetA0I_5_0 = flattenedOffsetA_5_0;
  unsigned int flattenedOffsetA_6_0 = globalReadOffsetA0I + 0 + 6*LSPA;
  flattenedOffsetA_6_0 = (flattenedOffsetA_6_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_6_0;
  unsigned int globalReadOffsetA0I_6_0 = flattenedOffsetA_6_0;
  unsigned int flattenedOffsetA_7_0 = globalReadOffsetA0I + 0 + 7*LSPA;
  flattenedOffsetA_7_0 = (flattenedOffsetA_7_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_7_0;
  unsigned int globalReadOffsetA0I_7_0 = flattenedOffsetA_7_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;
  globalReadOffsetA0I_4_0 = (  globalReadOffsetA0I_4_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_4_0;
  globalReadOffsetA0I_5_0 = (  globalReadOffsetA0I_5_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_5_0;
  globalReadOffsetA0I_6_0 = (  globalReadOffsetA0I_6_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_6_0;
  globalReadOffsetA0I_7_0 = (  globalReadOffsetA0I_7_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_7_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_2_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_3_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 4*LSPA);
  unsigned int localWriteOffsetA_0_0_4_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 4*LSPA);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 5*LSPA);
  unsigned int localWriteOffsetA_0_0_5_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 5*LSPA);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 6*LSPA);
  unsigned int localWriteOffsetA_0_0_6_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 6*LSPA);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 7*LSPA);
  unsigned int localWriteOffsetA_0_0_7_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 7*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_2_1;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_3_1;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_4_1;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_5_1;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_6_1;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_7_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_4_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_1);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_5_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_1);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_6_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_1);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_7_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_1_4_0 = *(globalReadA_0_0_4_0 + 1);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_1_5_0 = *(globalReadA_0_0_5_0 + 1);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_1_6_0 = *(globalReadA_0_0_6_0 + 1);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_1_7_0 = *(globalReadA_0_0_7_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_1 + 0) = a_0_1_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_4_1 + 0) = a_0_1_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_5_1 + 0) = a_0_1_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_6_1 + 0) = a_0_1_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_7_1 + 0) = a_0_1_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);
  a_0_0_4_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_1_4_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 1);
  a_0_0_5_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_1_5_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 1);
  a_0_0_6_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_1_6_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 1);
  a_0_0_7_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_1_7_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_4_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_1);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_5_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_1);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_6_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_1);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_7_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_1 + 0) = a_0_1_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_4_1 + 0) = a_0_1_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_5_1 + 0) = a_0_1_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_6_1 + 0) = a_0_1_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_7_1 + 0) = a_0_1_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_4xi8BH_MT128x32x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;
  unsigned int flattenedOffsetA_4_0 = globalReadOffsetA0I + 0 + 4*LSPA;
  flattenedOffsetA_4_0 = (flattenedOffsetA_4_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_4_0;
  unsigned int globalReadOffsetA0I_4_0 = flattenedOffsetA_4_0;
  unsigned int flattenedOffsetA_5_0 = globalReadOffsetA0I + 0 + 5*LSPA;
  flattenedOffsetA_5_0 = (flattenedOffsetA_5_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_5_0;
  unsigned int globalReadOffsetA0I_5_0 = flattenedOffsetA_5_0;
  unsigned int flattenedOffsetA_6_0 = globalReadOffsetA0I + 0 + 6*LSPA;
  flattenedOffsetA_6_0 = (flattenedOffsetA_6_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_6_0;
  unsigned int globalReadOffsetA0I_6_0 = flattenedOffsetA_6_0;
  unsigned int flattenedOffsetA_7_0 = globalReadOffsetA0I + 0 + 7*LSPA;
  flattenedOffsetA_7_0 = (flattenedOffsetA_7_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_7_0;
  unsigned int globalReadOffsetA0I_7_0 = flattenedOffsetA_7_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;
  globalReadOffsetA0I_4_0 = (  globalReadOffsetA0I_4_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_4_0;
  globalReadOffsetA0I_5_0 = (  globalReadOffsetA0I_5_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_5_0;
  globalReadOffsetA0I_6_0 = (  globalReadOffsetA0I_6_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_6_0;
  globalReadOffsetA0I_7_0 = (  globalReadOffsetA0I_7_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_7_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 4*LSPA);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 5*LSPA);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 6*LSPA);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 7*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 16
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_4xi8BH_MT64x16x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;
  unsigned int flattenedOffsetA_4_0 = globalReadOffsetA0I + 0 + 4*LSPA;
  flattenedOffsetA_4_0 = (flattenedOffsetA_4_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_4_0;
  unsigned int globalReadOffsetA0I_4_0 = flattenedOffsetA_4_0;
  unsigned int flattenedOffsetA_5_0 = globalReadOffsetA0I + 0 + 5*LSPA;
  flattenedOffsetA_5_0 = (flattenedOffsetA_5_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_5_0;
  unsigned int globalReadOffsetA0I_5_0 = flattenedOffsetA_5_0;
  unsigned int flattenedOffsetA_6_0 = globalReadOffsetA0I + 0 + 6*LSPA;
  flattenedOffsetA_6_0 = (flattenedOffsetA_6_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_6_0;
  unsigned int globalReadOffsetA0I_6_0 = flattenedOffsetA_6_0;
  unsigned int flattenedOffsetA_7_0 = globalReadOffsetA0I + 0 + 7*LSPA;
  flattenedOffsetA_7_0 = (flattenedOffsetA_7_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_7_0;
  unsigned int globalReadOffsetA0I_7_0 = flattenedOffsetA_7_0;
  unsigned int flattenedOffsetA_8_0 = globalReadOffsetA0I + 0 + 8*LSPA;
  flattenedOffsetA_8_0 = (flattenedOffsetA_8_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_8_0;
  unsigned int globalReadOffsetA0I_8_0 = flattenedOffsetA_8_0;
  unsigned int flattenedOffsetA_9_0 = globalReadOffsetA0I + 0 + 9*LSPA;
  flattenedOffsetA_9_0 = (flattenedOffsetA_9_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_9_0;
  unsigned int globalReadOffsetA0I_9_0 = flattenedOffsetA_9_0;
  unsigned int flattenedOffsetA_10_0 = globalReadOffsetA0I + 0 + 10*LSPA;
  flattenedOffsetA_10_0 = (flattenedOffsetA_10_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_10_0;
  unsigned int globalReadOffsetA0I_10_0 = flattenedOffsetA_10_0;
  unsigned int flattenedOffsetA_11_0 = globalReadOffsetA0I + 0 + 11*LSPA;
  flattenedOffsetA_11_0 = (flattenedOffsetA_11_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_11_0;
  unsigned int globalReadOffsetA0I_11_0 = flattenedOffsetA_11_0;
  unsigned int flattenedOffsetA_12_0 = globalReadOffsetA0I + 0 + 12*LSPA;
  flattenedOffsetA_12_0 = (flattenedOffsetA_12_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_12_0;
  unsigned int globalReadOffsetA0I_12_0 = flattenedOffsetA_12_0;
  unsigned int flattenedOffsetA_13_0 = globalReadOffsetA0I + 0 + 13*LSPA;
  flattenedOffsetA_13_0 = (flattenedOffsetA_13_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_13_0;
  unsigned int globalReadOffsetA0I_13_0 = flattenedOffsetA_13_0;
  unsigned int flattenedOffsetA_14_0 = globalReadOffsetA0I + 0 + 14*LSPA;
  flattenedOffsetA_14_0 = (flattenedOffsetA_14_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_14_0;
  unsigned int globalReadOffsetA0I_14_0 = flattenedOffsetA_14_0;
  unsigned int flattenedOffsetA_15_0 = globalReadOffsetA0I + 0 + 15*LSPA;
  flattenedOffsetA_15_0 = (flattenedOffsetA_15_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_15_0;
  unsigned int globalReadOffsetA0I_15_0 = flattenedOffsetA_15_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;
  globalReadOffsetA0I_4_0 = (  globalReadOffsetA0I_4_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_4_0;
  globalReadOffsetA0I_5_0 = (  globalReadOffsetA0I_5_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_5_0;
  globalReadOffsetA0I_6_0 = (  globalReadOffsetA0I_6_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_6_0;
  globalReadOffsetA0I_7_0 = (  globalReadOffsetA0I_7_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_7_0;
  globalReadOffsetA0I_8_0 = (  globalReadOffsetA0I_8_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_8_0;
  globalReadOffsetA0I_9_0 = (  globalReadOffsetA0I_9_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_9_0;
  globalReadOffsetA0I_10_0 = (  globalReadOffsetA0I_10_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_10_0;
  globalReadOffsetA0I_11_0 = (  globalReadOffsetA0I_11_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_11_0;
  globalReadOffsetA0I_12_0 = (  globalReadOffsetA0I_12_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_12_0;
  globalReadOffsetA0I_13_0 = (  globalReadOffsetA0I_13_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_13_0;
  globalReadOffsetA0I_14_0 = (  globalReadOffsetA0I_14_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_14_0;
  globalReadOffsetA0I_15_0 = (  globalReadOffsetA0I_15_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_15_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_15_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 4*LSPA);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 5*LSPA);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 6*LSPA);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 7*LSPA);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 8*LSPA);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 9*LSPA);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 10*LSPA);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 11*LSPA);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 12*LSPA);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 13*LSPA);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 14*LSPA);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 15*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_0_0_15_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_8_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_9_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_10_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_11_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_12_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_13_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_14_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_15_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_8_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_9_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_10_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_11_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_12_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_13_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_14_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_15_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_0_8_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_0_0_9_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_0_0_10_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_0_0_11_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_0_0_12_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_0_0_13_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_0_0_14_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_0_0_15_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 16

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x8 micro-tile */
#define MAC_2x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \

#define MAC_2x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_4xi8BH_MT16x64x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_8_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;
  DATA_TYPE b_0_0_8_0;
  DATA_TYPE b_0_0_9_0;
  DATA_TYPE b_0_0_10_0;
  DATA_TYPE b_0_0_11_0;
  DATA_TYPE b_0_0_12_0;
  DATA_TYPE b_0_0_13_0;
  DATA_TYPE b_0_0_14_0;
  DATA_TYPE b_0_0_15_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;
  unsigned int flattenedOffsetB_4_0 = globalReadOffsetB1J + 0 + 4*LSPB;
  flattenedOffsetB_4_0 = (flattenedOffsetB_4_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_4_0;
  unsigned int globalReadOffsetB1J_4_0 = flattenedOffsetB_4_0;
  unsigned int flattenedOffsetB_5_0 = globalReadOffsetB1J + 0 + 5*LSPB;
  flattenedOffsetB_5_0 = (flattenedOffsetB_5_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_5_0;
  unsigned int globalReadOffsetB1J_5_0 = flattenedOffsetB_5_0;
  unsigned int flattenedOffsetB_6_0 = globalReadOffsetB1J + 0 + 6*LSPB;
  flattenedOffsetB_6_0 = (flattenedOffsetB_6_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_6_0;
  unsigned int globalReadOffsetB1J_6_0 = flattenedOffsetB_6_0;
  unsigned int flattenedOffsetB_7_0 = globalReadOffsetB1J + 0 + 7*LSPB;
  flattenedOffsetB_7_0 = (flattenedOffsetB_7_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_7_0;
  unsigned int globalReadOffsetB1J_7_0 = flattenedOffsetB_7_0;
  unsigned int flattenedOffsetB_8_0 = globalReadOffsetB1J + 0 + 8*LSPB;
  flattenedOffsetB_8_0 = (flattenedOffsetB_8_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_8_0;
  unsigned int globalReadOffsetB1J_8_0 = flattenedOffsetB_8_0;
  unsigned int flattenedOffsetB_9_0 = globalReadOffsetB1J + 0 + 9*LSPB;
  flattenedOffsetB_9_0 = (flattenedOffsetB_9_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_9_0;
  unsigned int globalReadOffsetB1J_9_0 = flattenedOffsetB_9_0;
  unsigned int flattenedOffsetB_10_0 = globalReadOffsetB1J + 0 + 10*LSPB;
  flattenedOffsetB_10_0 = (flattenedOffsetB_10_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_10_0;
  unsigned int globalReadOffsetB1J_10_0 = flattenedOffsetB_10_0;
  unsigned int flattenedOffsetB_11_0 = globalReadOffsetB1J + 0 + 11*LSPB;
  flattenedOffsetB_11_0 = (flattenedOffsetB_11_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_11_0;
  unsigned int globalReadOffsetB1J_11_0 = flattenedOffsetB_11_0;
  unsigned int flattenedOffsetB_12_0 = globalReadOffsetB1J + 0 + 12*LSPB;
  flattenedOffsetB_12_0 = (flattenedOffsetB_12_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_12_0;
  unsigned int globalReadOffsetB1J_12_0 = flattenedOffsetB_12_0;
  unsigned int flattenedOffsetB_13_0 = globalReadOffsetB1J + 0 + 13*LSPB;
  flattenedOffsetB_13_0 = (flattenedOffsetB_13_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_13_0;
  unsigned int globalReadOffsetB1J_13_0 = flattenedOffsetB_13_0;
  unsigned int flattenedOffsetB_14_0 = globalReadOffsetB1J + 0 + 14*LSPB;
  flattenedOffsetB_14_0 = (flattenedOffsetB_14_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_14_0;
  unsigned int globalReadOffsetB1J_14_0 = flattenedOffsetB_14_0;
  unsigned int flattenedOffsetB_15_0 = globalReadOffsetB1J + 0 + 15*LSPB;
  flattenedOffsetB_15_0 = (flattenedOffsetB_15_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_15_0;
  unsigned int globalReadOffsetB1J_15_0 = flattenedOffsetB_15_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;
  globalReadOffsetB1J_4_0 = (  globalReadOffsetB1J_4_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_4_0;
  globalReadOffsetB1J_5_0 = (  globalReadOffsetB1J_5_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_5_0;
  globalReadOffsetB1J_6_0 = (  globalReadOffsetB1J_6_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_6_0;
  globalReadOffsetB1J_7_0 = (  globalReadOffsetB1J_7_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_7_0;
  globalReadOffsetB1J_8_0 = (  globalReadOffsetB1J_8_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_8_0;
  globalReadOffsetB1J_9_0 = (  globalReadOffsetB1J_9_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_9_0;
  globalReadOffsetB1J_10_0 = (  globalReadOffsetB1J_10_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_10_0;
  globalReadOffsetB1J_11_0 = (  globalReadOffsetB1J_11_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_11_0;
  globalReadOffsetB1J_12_0 = (  globalReadOffsetB1J_12_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_12_0;
  globalReadOffsetB1J_13_0 = (  globalReadOffsetB1J_13_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_13_0;
  globalReadOffsetB1J_14_0 = (  globalReadOffsetB1J_14_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_14_0;
  globalReadOffsetB1J_15_0 = (  globalReadOffsetB1J_15_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_15_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_7_0), wgK );
  int64_t globalReadOffsetB_0_0_8_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_8_0), wgK );
  int64_t globalReadOffsetB_0_0_9_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_9_0), wgK );
  int64_t globalReadOffsetB_0_0_10_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_10_0), wgK );
  int64_t globalReadOffsetB_0_0_11_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_11_0), wgK );
  int64_t globalReadOffsetB_0_0_12_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_12_0), wgK );
  int64_t globalReadOffsetB_0_0_13_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_13_0), wgK );
  int64_t globalReadOffsetB_0_0_14_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_14_0), wgK );
  int64_t globalReadOffsetB_0_0_15_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_15_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;
  DATA_TYPE const *globalReadB_0_0_8_0 = B + globalReadOffsetB_0_0_8_0;
  DATA_TYPE const *globalReadB_0_0_9_0 = B + globalReadOffsetB_0_0_9_0;
  DATA_TYPE const *globalReadB_0_0_10_0 = B + globalReadOffsetB_0_0_10_0;
  DATA_TYPE const *globalReadB_0_0_11_0 = B + globalReadOffsetB_0_0_11_0;
  DATA_TYPE const *globalReadB_0_0_12_0 = B + globalReadOffsetB_0_0_12_0;
  DATA_TYPE const *globalReadB_0_0_13_0 = B + globalReadOffsetB_0_0_13_0;
  DATA_TYPE const *globalReadB_0_0_14_0 = B + globalReadOffsetB_0_0_14_0;
  DATA_TYPE const *globalReadB_0_0_15_0 = B + globalReadOffsetB_0_0_15_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 4*LSPB);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 5*LSPB);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 6*LSPB);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 7*LSPB);
  unsigned int localWriteOffsetB_0_0_8_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 8*LSPB);
  unsigned int localWriteOffsetB_0_0_9_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 9*LSPB);
  unsigned int localWriteOffsetB_0_0_10_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 10*LSPB);
  unsigned int localWriteOffsetB_0_0_11_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 11*LSPB);
  unsigned int localWriteOffsetB_0_0_12_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 12*LSPB);
  unsigned int localWriteOffsetB_0_0_13_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 13*LSPB);
  unsigned int localWriteOffsetB_0_0_14_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 14*LSPB);
  unsigned int localWriteOffsetB_0_0_15_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 15*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;
  DATA_TYPE *localWriteB_0_0_8_0;
  DATA_TYPE *localWriteB_0_0_9_0;
  DATA_TYPE *localWriteB_0_0_10_0;
  DATA_TYPE *localWriteB_0_0_11_0;
  DATA_TYPE *localWriteB_0_0_12_0;
  DATA_TYPE *localWriteB_0_0_13_0;
  DATA_TYPE *localWriteB_0_0_14_0;
  DATA_TYPE *localWriteB_0_0_15_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);
  localWriteB_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_8_0);
  localWriteB_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_9_0);
  localWriteB_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_10_0);
  localWriteB_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_11_0);
  localWriteB_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_12_0);
  localWriteB_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_13_0);
  localWriteB_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_14_0);
  localWriteB_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_15_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_8_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_9_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_10_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_11_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_12_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_13_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_14_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_15_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);
    b_0_0_8_0 = *(globalReadB_0_0_8_0 + 0);
    b_0_0_9_0 = *(globalReadB_0_0_9_0 + 0);
    b_0_0_10_0 = *(globalReadB_0_0_10_0 + 0);
    b_0_0_11_0 = *(globalReadB_0_0_11_0 + 0);
    b_0_0_12_0 = *(globalReadB_0_0_12_0 + 0);
    b_0_0_13_0 = *(globalReadB_0_0_13_0 + 0);
    b_0_0_14_0 = *(globalReadB_0_0_14_0 + 0);
    b_0_0_15_0 = *(globalReadB_0_0_15_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_8_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_8_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_9_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_9_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_10_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_10_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_11_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_11_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_12_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_12_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_13_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_13_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_14_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_14_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_15_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_15_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
    *(localWriteB_0_0_8_0 + 0) = b_0_0_8_0;
    *(localWriteB_0_0_9_0 + 0) = b_0_0_9_0;
    *(localWriteB_0_0_10_0 + 0) = b_0_0_10_0;
    *(localWriteB_0_0_11_0 + 0) = b_0_0_11_0;
    *(localWriteB_0_0_12_0 + 0) = b_0_0_12_0;
    *(localWriteB_0_0_13_0 + 0) = b_0_0_13_0;
    *(localWriteB_0_0_14_0 + 0) = b_0_0_14_0;
    *(localWriteB_0_0_15_0 + 0) = b_0_0_15_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x8


    /* iter 15 */

    MAC_2x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);
  b_0_0_8_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_8_0 + 0);
  b_0_0_9_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_9_0 + 0);
  b_0_0_10_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_10_0 + 0);
  b_0_0_11_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_11_0 + 0);
  b_0_0_12_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_12_0 + 0);
  b_0_0_13_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_13_0 + 0);
  b_0_0_14_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_14_0 + 0);
  b_0_0_15_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_15_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);
  localWriteB_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_8_0);
  localWriteB_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_9_0);
  localWriteB_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_10_0);
  localWriteB_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_11_0);
  localWriteB_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_12_0);
  localWriteB_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_13_0);
  localWriteB_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_14_0);
  localWriteB_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_15_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
  *(localWriteB_0_0_8_0 + 0) = b_0_0_8_0;
  *(localWriteB_0_0_9_0 + 0) = b_0_0_9_0;
  *(localWriteB_0_0_10_0 + 0) = b_0_0_10_0;
  *(localWriteB_0_0_11_0 + 0) = b_0_0_11_0;
  *(localWriteB_0_0_12_0 + 0) = b_0_0_12_0;
  *(localWriteB_0_0_13_0 + 0) = b_0_0_13_0;
  *(localWriteB_0_0_14_0 + 0) = b_0_0_14_0;
  *(localWriteB_0_0_15_0 + 0) = b_0_0_15_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x8
#undef MAC_2x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 4

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1536

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_SB_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM4(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 4

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1536

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_SB_MT64x128x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM4(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE TensileComplexFloat
#define DEST_DATA_TYPE TensileComplexFloat
#define COMPUTE_DATA_TYPE TensileComplexFloat
#define s0 x
#define s1 y
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) \
  DST.s0 = MAC(  MULA.s0, MULB.s0, DST.s0 ); \
  DST.s0 = MAC( -MULA.s1, MULB.s1, DST.s0 ); \
  DST.s1 = MAC(  MULA.s0, MULB.s1, DST.s1 ); \
  DST.s1 = MAC(  MULA.s1, MULB.s0, DST.s1 );
#define TYPE_MAC_WRITE( DST, SRC, ALPHA, REG, BETA ) \
  /* (1) */ \
  type_mac_tmp = REG.s0; \
  REG.s0 *= ALPHA.s0; \
  REG.s0 = MAC( -ALPHA.s1, REG.s1, REG.s0 ); \
  REG.s1 *= ALPHA.s0; \
  REG.s1 = MAC(  ALPHA.s1, type_mac_tmp, REG.s1 ); \
  /* (2) */ \
  if(BETA.s0 != 0) { \
  REG.s0 = MAC(  BETA.s0, SRC.s0, REG.s0 ); \
  REG.s1 = MAC(  BETA.s0, SRC.s1, REG.s1 ); \
  } \
  if (BETA.s1 != 0) { \
  REG.s0 = MAC( -BETA.s1, SRC.s1, REG.s0 ); \
  REG.s1 = MAC(  BETA.s1, SRC.s0, REG.s1 ); \
  } \
  /* (3) */ \
  DST = REG;

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_CB_MT32x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW2_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  TensileComplexFloat *D,
  TensileComplexFloat const * __restrict__ C,
  TensileComplexFloat const * __restrict__ A,
  TensileComplexFloat const * __restrict__ B,
  TensileComplexFloat const alpha,
  TensileComplexFloat const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (TensileComplexFloat)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == tensile_complex<float>(0.0f)) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }



  float type_mac_tmp;


  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE TensileComplexFloat
#define DEST_DATA_TYPE TensileComplexFloat
#define COMPUTE_DATA_TYPE TensileComplexFloat
#define s0 x
#define s1 y
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) \
  DST.s0 = MAC(  MULA.s0, MULB.s0, DST.s0 ); \
  DST.s0 = MAC( -MULA.s1, MULB.s1, DST.s0 ); \
  DST.s1 = MAC(  MULA.s0, MULB.s1, DST.s1 ); \
  DST.s1 = MAC(  MULA.s1, MULB.s0, DST.s1 );
#define TYPE_MAC_WRITE( DST, SRC, ALPHA, REG, BETA ) \
  /* (1) */ \
  type_mac_tmp = REG.s0; \
  REG.s0 *= ALPHA.s0; \
  REG.s0 = MAC( -ALPHA.s1, REG.s1, REG.s0 ); \
  REG.s1 *= ALPHA.s0; \
  REG.s1 = MAC(  ALPHA.s1, type_mac_tmp, REG.s1 ); \
  /* (2) */ \
  if(BETA.s0 != 0) { \
  REG.s0 = MAC(  BETA.s0, SRC.s0, REG.s0 ); \
  REG.s1 = MAC(  BETA.s0, SRC.s1, REG.s1 ); \
  } \
  if (BETA.s1 != 0) { \
  REG.s0 = MAC( -BETA.s1, SRC.s1, REG.s0 ); \
  REG.s1 = MAC(  BETA.s1, SRC.s0, REG.s1 ); \
  } \
  /* (3) */ \
  DST = REG;

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_CB_MT32x32x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW2_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  TensileComplexFloat *D,
  TensileComplexFloat const * __restrict__ C,
  TensileComplexFloat const * __restrict__ A,
  TensileComplexFloat const * __restrict__ B,
  TensileComplexFloat const alpha,
  TensileComplexFloat const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (TensileComplexFloat)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == tensile_complex<float>(0.0f)) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }



  float type_mac_tmp;


  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE TensileComplexDouble
#define DEST_DATA_TYPE TensileComplexDouble
#define COMPUTE_DATA_TYPE TensileComplexDouble
#define s0 x
#define s1 y
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) \
  DST.s0 = MAC(  MULA.s0, MULB.s0, DST.s0 ); \
  DST.s0 = MAC( -MULA.s1, MULB.s1, DST.s0 ); \
  DST.s1 = MAC(  MULA.s0, MULB.s1, DST.s1 ); \
  DST.s1 = MAC(  MULA.s1, MULB.s0, DST.s1 );
#define TYPE_MAC_WRITE( DST, SRC, ALPHA, REG, BETA ) \
  /* (1) */ \
  type_mac_tmp = REG.s0; \
  REG.s0 *= ALPHA.s0; \
  REG.s0 = MAC( -ALPHA.s1, REG.s1, REG.s0 ); \
  REG.s1 *= ALPHA.s0; \
  REG.s1 = MAC(  ALPHA.s1, type_mac_tmp, REG.s1 ); \
  /* (2) */ \
  if(BETA.s0 != 0) { \
  REG.s0 = MAC(  BETA.s0, SRC.s0, REG.s0 ); \
  REG.s1 = MAC(  BETA.s0, SRC.s1, REG.s1 ); \
  } \
  if (BETA.s1 != 0) { \
  REG.s0 = MAC( -BETA.s1, SRC.s1, REG.s0 ); \
  REG.s1 = MAC(  BETA.s1, SRC.s0, REG.s1 ); \
  } \
  /* (3) */ \
  DST = REG;

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_ZB_MT32x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW1_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  TensileComplexDouble *D,
  TensileComplexDouble const * __restrict__ C,
  TensileComplexDouble const * __restrict__ A,
  TensileComplexDouble const * __restrict__ B,
  TensileComplexDouble const alpha,
  TensileComplexDouble const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (TensileComplexDouble)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == tensile_complex<double>(0.0)) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 1); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }



  double type_mac_tmp;


  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE TensileComplexDouble
#define DEST_DATA_TYPE TensileComplexDouble
#define COMPUTE_DATA_TYPE TensileComplexDouble
#define s0 x
#define s1 y
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) \
  DST.s0 = MAC(  MULA.s0, MULB.s0, DST.s0 ); \
  DST.s0 = MAC( -MULA.s1, MULB.s1, DST.s0 ); \
  DST.s1 = MAC(  MULA.s0, MULB.s1, DST.s1 ); \
  DST.s1 = MAC(  MULA.s1, MULB.s0, DST.s1 );
#define TYPE_MAC_WRITE( DST, SRC, ALPHA, REG, BETA ) \
  /* (1) */ \
  type_mac_tmp = REG.s0; \
  REG.s0 *= ALPHA.s0; \
  REG.s0 = MAC( -ALPHA.s1, REG.s1, REG.s0 ); \
  REG.s1 *= ALPHA.s0; \
  REG.s1 = MAC(  ALPHA.s1, type_mac_tmp, REG.s1 ); \
  /* (2) */ \
  if(BETA.s0 != 0) { \
  REG.s0 = MAC(  BETA.s0, SRC.s0, REG.s0 ); \
  REG.s1 = MAC(  BETA.s0, SRC.s1, REG.s1 ); \
  } \
  if (BETA.s1 != 0) { \
  REG.s0 = MAC( -BETA.s1, SRC.s1, REG.s0 ); \
  REG.s1 = MAC(  BETA.s1, SRC.s0, REG.s1 ); \
  } \
  /* (3) */ \
  DST = REG;

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_ZB_MT32x32x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW1_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  TensileComplexDouble *D,
  TensileComplexDouble const * __restrict__ C,
  TensileComplexDouble const * __restrict__ A,
  TensileComplexDouble const * __restrict__ B,
  TensileComplexDouble const alpha,
  TensileComplexDouble const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (TensileComplexDouble)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == tensile_complex<double>(0.0)) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 1); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }



  double type_mac_tmp;


  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 7168
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
  a_pk_fma[0] = MULA0; \
  a_pk_fma[1] = MULA1; \
  b_pk_fma[0] = MULB0; \
  b_pk_fma[1] = MULB1; \
  c_pk_fma[0] = DST0; \
  c_pk_fma[1] = DST1; \
  c_pk_fma = tensile_fmadd_half2(a_pk_fma, b_pk_fma, c_pk_fma); \
  DST0 = c_pk_fma[0]; \
  DST1 = c_pk_fma[1]; \
 
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HB_MT64x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_1_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_1_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_1_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 1);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_1_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 1) {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 2) {
        rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 819

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
  a_pk_fma[0] = MULA0; \
  a_pk_fma[1] = MULA1; \
  b_pk_fma[0] = MULB0; \
  b_pk_fma[1] = MULB1; \
  c_pk_fma[0] = DST0; \
  c_pk_fma[1] = DST1; \
  c_pk_fma = tensile_fmadd_half2(a_pk_fma, b_pk_fma, c_pk_fma); \
  DST0 = c_pk_fma[0]; \
  DST1 = c_pk_fma[1]; \
 
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HB_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 3 */

    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
  a_pk_fma[0] = MULA0; \
  a_pk_fma[1] = MULA1; \
  b_pk_fma[0] = MULB0; \
  b_pk_fma[1] = MULB1; \
  c_pk_fma[0] = DST0; \
  c_pk_fma[1] = DST1; \
  c_pk_fma = tensile_fmadd_half2(a_pk_fma, b_pk_fma, c_pk_fma); \
  DST0 = c_pk_fma[0]; \
  DST1 = c_pk_fma[1]; \
 
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HB_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 7 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 7168
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
  a_pk_fma[0] = MULA0; \
  a_pk_fma[1] = MULA1; \
  b_pk_fma[0] = MULB0; \
  b_pk_fma[1] = MULB1; \
  c_pk_fma[0] = DST0; \
  c_pk_fma[1] = DST1; \
  c_pk_fma = tensile_fmadd_half2(a_pk_fma, b_pk_fma, c_pk_fma); \
  DST0 = c_pk_fma[0]; \
  DST1 = c_pk_fma[1]; \
 
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HB_MT64x128x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_1_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_1_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_1_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 1);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_1_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 1) {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 2) {
        rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 819

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
  a_pk_fma[0] = MULA0; \
  a_pk_fma[1] = MULA1; \
  b_pk_fma[0] = MULB0; \
  b_pk_fma[1] = MULB1; \
  c_pk_fma[0] = DST0; \
  c_pk_fma[1] = DST1; \
  c_pk_fma = tensile_fmadd_half2(a_pk_fma, b_pk_fma, c_pk_fma); \
  DST0 = c_pk_fma[0]; \
  DST1 = c_pk_fma[1]; \
 
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HB_MT64x128x4_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 3 */

    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
  a_pk_fma[0] = MULA0; \
  a_pk_fma[1] = MULA1; \
  b_pk_fma[0] = MULB0; \
  b_pk_fma[1] = MULB1; \
  c_pk_fma[0] = DST0; \
  c_pk_fma[1] = DST1; \
  c_pk_fma = tensile_fmadd_half2(a_pk_fma, b_pk_fma, c_pk_fma); \
  DST0 = c_pk_fma[0]; \
  DST1 = c_pk_fma[1]; \
 
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HB_MT64x64x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 7 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE TensileComplexFloat
#define DEST_DATA_TYPE TensileComplexFloat
#define COMPUTE_DATA_TYPE TensileComplexFloat
#define s0 x
#define s1 y
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) \
  DST.s0 = MAC(  MULA.s0, MULB.s0, DST.s0 ); \
  DST.s0 = MAC(  MULA.s1, MULB.s1, DST.s0 ); \
  DST.s1 = MAC(  MULA.s0, MULB.s1, DST.s1 ); \
  DST.s1 = MAC( -MULA.s1, MULB.s0, DST.s1 );
#define TYPE_MAC_WRITE( DST, SRC, ALPHA, REG, BETA ) \
  /* (1) */ \
  type_mac_tmp = REG.s0; \
  REG.s0 *= ALPHA.s0; \
  REG.s0 = MAC( -ALPHA.s1, REG.s1, REG.s0 ); \
  REG.s1 *= ALPHA.s0; \
  REG.s1 = MAC(  ALPHA.s1, type_mac_tmp, REG.s1 ); \
  /* (2) */ \
  if(BETA.s0 != 0) { \
  REG.s0 = MAC(  BETA.s0, SRC.s0, REG.s0 ); \
  REG.s1 = MAC(  BETA.s0, SRC.s1, REG.s1 ); \
  } \
  if (BETA.s1 != 0) { \
  REG.s0 = MAC( -BETA.s1, SRC.s1, REG.s0 ); \
  REG.s1 = MAC(  BETA.s1, SRC.s0, REG.s1 ); \
  } \
  /* (3) */ \
  DST = REG;

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_AlikC_Bljk_CB_MT32x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW2_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  TensileComplexFloat *D,
  TensileComplexFloat const * __restrict__ C,
  TensileComplexFloat const * __restrict__ A,
  TensileComplexFloat const * __restrict__ B,
  TensileComplexFloat const alpha,
  TensileComplexFloat const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (TensileComplexFloat)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == tensile_complex<float>(0.0f)) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }



  float type_mac_tmp;


  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE TensileComplexFloat
#define DEST_DATA_TYPE TensileComplexFloat
#define COMPUTE_DATA_TYPE TensileComplexFloat
#define s0 x
#define s1 y
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) \
  DST.s0 = MAC(  MULA.s0, MULB.s0, DST.s0 ); \
  DST.s0 = MAC(  MULA.s1, MULB.s1, DST.s0 ); \
  DST.s1 = MAC(  MULA.s0, MULB.s1, DST.s1 ); \
  DST.s1 = MAC( -MULA.s1, MULB.s0, DST.s1 );
#define TYPE_MAC_WRITE( DST, SRC, ALPHA, REG, BETA ) \
  /* (1) */ \
  type_mac_tmp = REG.s0; \
  REG.s0 *= ALPHA.s0; \
  REG.s0 = MAC( -ALPHA.s1, REG.s1, REG.s0 ); \
  REG.s1 *= ALPHA.s0; \
  REG.s1 = MAC(  ALPHA.s1, type_mac_tmp, REG.s1 ); \
  /* (2) */ \
  if(BETA.s0 != 0) { \
  REG.s0 = MAC(  BETA.s0, SRC.s0, REG.s0 ); \
  REG.s1 = MAC(  BETA.s0, SRC.s1, REG.s1 ); \
  } \
  if (BETA.s1 != 0) { \
  REG.s0 = MAC( -BETA.s1, SRC.s1, REG.s0 ); \
  REG.s1 = MAC(  BETA.s1, SRC.s0, REG.s1 ); \
  } \
  /* (3) */ \
  DST = REG;

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_AlikC_Bljk_CB_MT32x32x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW2_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  TensileComplexFloat *D,
  TensileComplexFloat const * __restrict__ C,
  TensileComplexFloat const * __restrict__ A,
  TensileComplexFloat const * __restrict__ B,
  TensileComplexFloat const alpha,
  TensileComplexFloat const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (TensileComplexFloat)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == tensile_complex<float>(0.0f)) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }



  float type_mac_tmp;


  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE TensileComplexFloat
#define DEST_DATA_TYPE TensileComplexFloat
#define COMPUTE_DATA_TYPE TensileComplexFloat
#define s0 x
#define s1 y
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) \
  DST.s0 = MAC(  MULA.s0,  MULB.s0, DST.s0 ); \
  DST.s0 = MAC( -MULA.s1, -MULB.s1, DST.s0 ); \
  DST.s1 = MAC(  MULA.s0, -MULB.s1, DST.s1 ); \
  DST.s1 = MAC(  MULA.s1,  MULB.s0, DST.s1 );
#define TYPE_MAC_WRITE( DST, SRC, ALPHA, REG, BETA ) \
  /* (1) */ \
  type_mac_tmp = REG.s0; \
  REG.s0 *= ALPHA.s0; \
  REG.s0 = MAC( -ALPHA.s1, REG.s1, REG.s0 ); \
  REG.s1 *= ALPHA.s0; \
  REG.s1 = MAC(  ALPHA.s1, type_mac_tmp, REG.s1 ); \
  /* (2) */ \
  if(BETA.s0 != 0) { \
  REG.s0 = MAC(  BETA.s0, SRC.s0, REG.s0 ); \
  REG.s1 = MAC(  BETA.s0, SRC.s1, REG.s1 ); \
  } \
  if (BETA.s1 != 0) { \
  REG.s0 = MAC( -BETA.s1, SRC.s1, REG.s0 ); \
  REG.s1 = MAC(  BETA.s1, SRC.s0, REG.s1 ); \
  } \
  /* (3) */ \
  DST = REG;

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_BjlkC_CB_MT32x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW2_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  TensileComplexFloat *D,
  TensileComplexFloat const * __restrict__ C,
  TensileComplexFloat const * __restrict__ A,
  TensileComplexFloat const * __restrict__ B,
  TensileComplexFloat const alpha,
  TensileComplexFloat const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (TensileComplexFloat)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == tensile_complex<float>(0.0f)) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }



  float type_mac_tmp;


  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE TensileComplexFloat
#define DEST_DATA_TYPE TensileComplexFloat
#define COMPUTE_DATA_TYPE TensileComplexFloat
#define s0 x
#define s1 y
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) \
  DST.s0 = MAC(  MULA.s0,  MULB.s0, DST.s0 ); \
  DST.s0 = MAC( -MULA.s1, -MULB.s1, DST.s0 ); \
  DST.s1 = MAC(  MULA.s0, -MULB.s1, DST.s1 ); \
  DST.s1 = MAC(  MULA.s1,  MULB.s0, DST.s1 );
#define TYPE_MAC_WRITE( DST, SRC, ALPHA, REG, BETA ) \
  /* (1) */ \
  type_mac_tmp = REG.s0; \
  REG.s0 *= ALPHA.s0; \
  REG.s0 = MAC( -ALPHA.s1, REG.s1, REG.s0 ); \
  REG.s1 *= ALPHA.s0; \
  REG.s1 = MAC(  ALPHA.s1, type_mac_tmp, REG.s1 ); \
  /* (2) */ \
  if(BETA.s0 != 0) { \
  REG.s0 = MAC(  BETA.s0, SRC.s0, REG.s0 ); \
  REG.s1 = MAC(  BETA.s0, SRC.s1, REG.s1 ); \
  } \
  if (BETA.s1 != 0) { \
  REG.s0 = MAC( -BETA.s1, SRC.s1, REG.s0 ); \
  REG.s1 = MAC(  BETA.s1, SRC.s0, REG.s1 ); \
  } \
  /* (3) */ \
  DST = REG;

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_BjlkC_CB_MT32x32x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW2_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  TensileComplexFloat *D,
  TensileComplexFloat const * __restrict__ C,
  TensileComplexFloat const * __restrict__ A,
  TensileComplexFloat const * __restrict__ B,
  TensileComplexFloat const alpha,
  TensileComplexFloat const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (TensileComplexFloat)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == tensile_complex<float>(0.0f)) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }



  float type_mac_tmp;


  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HBH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 3 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 3 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I] , rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I] , rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I] , rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I] , rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I] , rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I] , rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I] , rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I] , rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I] , rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I] , rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I] , rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I] , rA[7],rB[7],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HBH_MT128x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I] , rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I] , rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I] , rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I] , rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I] , rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I] , rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I] , rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I] , rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I] , rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I] , rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I] , rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I] , rA[7],rB[7],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HBH_MT128x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_8x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 768

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_BSS_BH_MT64x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_bfloat16)(0)
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 7 */

    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 16
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 4096
#define LDS_NUM_ELEMENTS 4608

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 16x2 micro-tile */
#define MAC_16x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[8],rB[0],rC[8+0*TT0I]); \
  TYPE_MAC(rA[9],rB[0],rC[9+0*TT0I]); \
  TYPE_MAC(rA[10],rB[0],rC[10+0*TT0I]); \
  TYPE_MAC(rA[11],rB[0],rC[11+0*TT0I]); \
  TYPE_MAC(rA[12],rB[0],rC[12+0*TT0I]); \
  TYPE_MAC(rA[13],rB[0],rC[13+0*TT0I]); \
  TYPE_MAC(rA[14],rB[0],rC[14+0*TT0I]); \
  TYPE_MAC(rA[15],rB[0],rC[15+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[8],rB[1],rC[8+1*TT0I]); \
  TYPE_MAC(rA[9],rB[1],rC[9+1*TT0I]); \
  TYPE_MAC(rA[10],rB[1],rC[10+1*TT0I]); \
  TYPE_MAC(rA[11],rB[1],rC[11+1*TT0I]); \
  TYPE_MAC(rA[12],rB[1],rC[12+1*TT0I]); \
  TYPE_MAC(rA[13],rB[1],rC[13+1*TT0I]); \
  TYPE_MAC(rA[14],rB[1],rC[14+1*TT0I]); \
  TYPE_MAC(rA[15],rB[1],rC[15+1*TT0I]); \

#define MAC_16x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[8+TT0I],rB[0+TT1J],rC[8+0*TT0I]); \
  TYPE_MAC(rA[9+TT0I],rB[0+TT1J],rC[9+0*TT0I]); \
  TYPE_MAC(rA[10+TT0I],rB[0+TT1J],rC[10+0*TT0I]); \
  TYPE_MAC(rA[11+TT0I],rB[0+TT1J],rC[11+0*TT0I]); \
  TYPE_MAC(rA[12+TT0I],rB[0+TT1J],rC[12+0*TT0I]); \
  TYPE_MAC(rA[13+TT0I],rB[0+TT1J],rC[13+0*TT0I]); \
  TYPE_MAC(rA[14+TT0I],rB[0+TT1J],rC[14+0*TT0I]); \
  TYPE_MAC(rA[15+TT0I],rB[0+TT1J],rC[15+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[8+TT0I],rB[1+TT1J],rC[8+1*TT0I]); \
  TYPE_MAC(rA[9+TT0I],rB[1+TT1J],rC[9+1*TT0I]); \
  TYPE_MAC(rA[10+TT0I],rB[1+TT1J],rC[10+1*TT0I]); \
  TYPE_MAC(rA[11+TT0I],rB[1+TT1J],rC[11+1*TT0I]); \
  TYPE_MAC(rA[12+TT0I],rB[1+TT1J],rC[12+1*TT0I]); \
  TYPE_MAC(rA[13+TT0I],rB[1+TT1J],rC[13+1*TT0I]); \
  TYPE_MAC(rA[14+TT0I],rB[1+TT1J],rC[14+1*TT0I]); \
  TYPE_MAC(rA[15+TT0I],rB[1+TT1J],rC[15+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_BSS_BH_MT256x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT16_2_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_bfloat16)(0)
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_1_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_1_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_1_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_1_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_1_4_0 = *(globalReadA_0_0_4_0 + 1);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_1_5_0 = *(globalReadA_0_0_5_0 + 1);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_1_6_0 = *(globalReadA_0_0_6_0 + 1);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_1_7_0 = *(globalReadA_0_0_7_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_4_0 + 1) = a_0_1_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_5_0 + 1) = a_0_1_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_6_0 + 1) = a_0_1_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_7_0 + 1) = a_0_1_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_16x2


    /* iter 15 */

    MAC_16x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_1_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 1);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_1_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 1);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_1_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 1);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_1_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_4_0 + 1) = a_0_1_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_5_0 + 1) = a_0_1_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_6_0 + 1) = a_0_1_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_7_0 + 1) = a_0_1_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_16x2

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 1) {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 2) {
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 3) {
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 4) {
        rC[0 + 4*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 4*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 4*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 4*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 5) {
        rC[0 + 5*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 5*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 5*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 5*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 6) {
        rC[0 + 6*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 6*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 6*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 6*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 7*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 7*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 7*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 7*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[8 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[8 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[9 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[9 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[10 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[10 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[11 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[11 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[12 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[12 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[13 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[13 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[14 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[14 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[15 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[15 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_16x2
#undef MAC_16x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_BSS_BH_MT64x16x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_bfloat16)(0)
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_1_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_1_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_1_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_1_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_1_4_0 = *(globalReadA_0_0_4_0 + 1);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_1_5_0 = *(globalReadA_0_0_5_0 + 1);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_1_6_0 = *(globalReadA_0_0_6_0 + 1);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_1_7_0 = *(globalReadA_0_0_7_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_4_0 + 1) = a_0_1_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_5_0 + 1) = a_0_1_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_6_0 + 1) = a_0_1_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_7_0 + 1) = a_0_1_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_1_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 1);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_1_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 1);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_1_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 1);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_1_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_4_0 + 1) = a_0_1_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_5_0 + 1) = a_0_1_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_6_0 + 1) = a_0_1_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_7_0 + 1) = a_0_1_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 1) {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 2) {
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 16
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 16
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 2304

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 16x2 micro-tile */
#define MAC_16x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[8],rB[0],rC[8+0*TT0I]); \
  TYPE_MAC(rA[9],rB[0],rC[9+0*TT0I]); \
  TYPE_MAC(rA[10],rB[0],rC[10+0*TT0I]); \
  TYPE_MAC(rA[11],rB[0],rC[11+0*TT0I]); \
  TYPE_MAC(rA[12],rB[0],rC[12+0*TT0I]); \
  TYPE_MAC(rA[13],rB[0],rC[13+0*TT0I]); \
  TYPE_MAC(rA[14],rB[0],rC[14+0*TT0I]); \
  TYPE_MAC(rA[15],rB[0],rC[15+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[8],rB[1],rC[8+1*TT0I]); \
  TYPE_MAC(rA[9],rB[1],rC[9+1*TT0I]); \
  TYPE_MAC(rA[10],rB[1],rC[10+1*TT0I]); \
  TYPE_MAC(rA[11],rB[1],rC[11+1*TT0I]); \
  TYPE_MAC(rA[12],rB[1],rC[12+1*TT0I]); \
  TYPE_MAC(rA[13],rB[1],rC[13+1*TT0I]); \
  TYPE_MAC(rA[14],rB[1],rC[14+1*TT0I]); \
  TYPE_MAC(rA[15],rB[1],rC[15+1*TT0I]); \

#define MAC_16x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[8+TT0I],rB[0+TT1J],rC[8+0*TT0I]); \
  TYPE_MAC(rA[9+TT0I],rB[0+TT1J],rC[9+0*TT0I]); \
  TYPE_MAC(rA[10+TT0I],rB[0+TT1J],rC[10+0*TT0I]); \
  TYPE_MAC(rA[11+TT0I],rB[0+TT1J],rC[11+0*TT0I]); \
  TYPE_MAC(rA[12+TT0I],rB[0+TT1J],rC[12+0*TT0I]); \
  TYPE_MAC(rA[13+TT0I],rB[0+TT1J],rC[13+0*TT0I]); \
  TYPE_MAC(rA[14+TT0I],rB[0+TT1J],rC[14+0*TT0I]); \
  TYPE_MAC(rA[15+TT0I],rB[0+TT1J],rC[15+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[8+TT0I],rB[1+TT1J],rC[8+1*TT0I]); \
  TYPE_MAC(rA[9+TT0I],rB[1+TT1J],rC[9+1*TT0I]); \
  TYPE_MAC(rA[10+TT0I],rB[1+TT1J],rC[10+1*TT0I]); \
  TYPE_MAC(rA[11+TT0I],rB[1+TT1J],rC[11+1*TT0I]); \
  TYPE_MAC(rA[12+TT0I],rB[1+TT1J],rC[12+1*TT0I]); \
  TYPE_MAC(rA[13+TT0I],rB[1+TT1J],rC[13+1*TT0I]); \
  TYPE_MAC(rA[14+TT0I],rB[1+TT1J],rC[14+1*TT0I]); \
  TYPE_MAC(rA[15+TT0I],rB[1+TT1J],rC[15+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_BSS_BH_MT128x16x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT16_2_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_bfloat16)(0)
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_1_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_1_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_1_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_1_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_0_1_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_0_1_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_0_1_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_0_1_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_0_1_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_0_1_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_0_1_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE a_0_1_15_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;
  unsigned int globalReadOffsetAL_8_0 = globalReadOffsetAL + 0 + 8*LSPA;
  unsigned int globalReadOffsetAL_9_0 = globalReadOffsetAL + 0 + 9*LSPA;
  unsigned int globalReadOffsetAL_10_0 = globalReadOffsetAL + 0 + 10*LSPA;
  unsigned int globalReadOffsetAL_11_0 = globalReadOffsetAL + 0 + 11*LSPA;
  unsigned int globalReadOffsetAL_12_0 = globalReadOffsetAL + 0 + 12*LSPA;
  unsigned int globalReadOffsetAL_13_0 = globalReadOffsetAL + 0 + 13*LSPA;
  unsigned int globalReadOffsetAL_14_0 = globalReadOffsetAL + 0 + 14*LSPA;
  unsigned int globalReadOffsetAL_15_0 = globalReadOffsetAL + 0 + 15*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_15_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 8*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 9*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 10*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 11*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 12*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 13*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 14*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 15*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_0_0_15_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_8_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_9_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_10_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_11_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_12_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_13_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_14_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_15_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_1_4_0 = *(globalReadA_0_0_4_0 + 1);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_1_5_0 = *(globalReadA_0_0_5_0 + 1);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_1_6_0 = *(globalReadA_0_0_6_0 + 1);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_1_7_0 = *(globalReadA_0_0_7_0 + 1);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_0_1_8_0 = *(globalReadA_0_0_8_0 + 1);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_0_1_9_0 = *(globalReadA_0_0_9_0 + 1);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_0_1_10_0 = *(globalReadA_0_0_10_0 + 1);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_0_1_11_0 = *(globalReadA_0_0_11_0 + 1);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_0_1_12_0 = *(globalReadA_0_0_12_0 + 1);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_0_1_13_0 = *(globalReadA_0_0_13_0 + 1);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_0_1_14_0 = *(globalReadA_0_0_14_0 + 1);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);
    a_0_1_15_0 = *(globalReadA_0_0_15_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_8_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_9_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_10_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_11_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_12_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_13_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_14_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_15_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_4_0 + 1) = a_0_1_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_5_0 + 1) = a_0_1_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_6_0 + 1) = a_0_1_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_7_0 + 1) = a_0_1_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_0_0_8_0 + 1) = a_0_1_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_0_0_9_0 + 1) = a_0_1_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_0_0_10_0 + 1) = a_0_1_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_0_0_11_0 + 1) = a_0_1_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_0_0_12_0 + 1) = a_0_1_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_0_0_13_0 + 1) = a_0_1_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_0_0_14_0 + 1) = a_0_1_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
    *(localWriteA_0_0_15_0 + 1) = a_0_1_15_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_16x2


    /* iter 15 */

    MAC_16x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_1_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 1);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_1_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 1);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_1_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 1);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_1_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 1);
  a_0_0_8_0 = ( globalReadOffsetAL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_0_1_8_0 = ( globalReadOffsetAL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 1);
  a_0_0_9_0 = ( globalReadOffsetAL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_0_1_9_0 = ( globalReadOffsetAL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 1);
  a_0_0_10_0 = ( globalReadOffsetAL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_0_1_10_0 = ( globalReadOffsetAL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 1);
  a_0_0_11_0 = ( globalReadOffsetAL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_0_1_11_0 = ( globalReadOffsetAL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 1);
  a_0_0_12_0 = ( globalReadOffsetAL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_0_1_12_0 = ( globalReadOffsetAL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 1);
  a_0_0_13_0 = ( globalReadOffsetAL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_0_1_13_0 = ( globalReadOffsetAL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 1);
  a_0_0_14_0 = ( globalReadOffsetAL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_0_1_14_0 = ( globalReadOffsetAL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 1);
  a_0_0_15_0 = ( globalReadOffsetAL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);
  a_0_1_15_0 = ( globalReadOffsetAL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_4_0 + 1) = a_0_1_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_5_0 + 1) = a_0_1_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_6_0 + 1) = a_0_1_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_7_0 + 1) = a_0_1_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_0_0_8_0 + 1) = a_0_1_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_0_0_9_0 + 1) = a_0_1_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_0_0_10_0 + 1) = a_0_1_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_0_0_11_0 + 1) = a_0_1_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_0_0_12_0 + 1) = a_0_1_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_0_0_13_0 + 1) = a_0_1_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_0_0_14_0 + 1) = a_0_1_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
  *(localWriteA_0_0_15_0 + 1) = a_0_1_15_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_16x2

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 1) {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 2) {
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 3) {
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 4) {
        rC[0 + 4*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 4*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 4*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 4*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 5) {
        rC[0 + 5*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 5*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 5*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 5*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 6) {
        rC[0 + 6*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 6*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 6*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 6*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 7*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 7*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 7*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 7*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[8 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[8 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[9 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[9 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[10 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[10 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[11 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[11 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[12 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[12 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[13 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[13 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[14 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[14 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[15 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[15 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_16x2
#undef MAC_16x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_BSS_BH_MT128x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_bfloat16)(0)
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_BSS_BH_MT128x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_bfloat16)(0)
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 16
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 16
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 4096
#define LDS_NUM_ELEMENTS 4608

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 16x2 micro-tile */
#define MAC_16x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[8],rB[0],rC[8+0*TT0I]); \
  TYPE_MAC(rA[9],rB[0],rC[9+0*TT0I]); \
  TYPE_MAC(rA[10],rB[0],rC[10+0*TT0I]); \
  TYPE_MAC(rA[11],rB[0],rC[11+0*TT0I]); \
  TYPE_MAC(rA[12],rB[0],rC[12+0*TT0I]); \
  TYPE_MAC(rA[13],rB[0],rC[13+0*TT0I]); \
  TYPE_MAC(rA[14],rB[0],rC[14+0*TT0I]); \
  TYPE_MAC(rA[15],rB[0],rC[15+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[8],rB[1],rC[8+1*TT0I]); \
  TYPE_MAC(rA[9],rB[1],rC[9+1*TT0I]); \
  TYPE_MAC(rA[10],rB[1],rC[10+1*TT0I]); \
  TYPE_MAC(rA[11],rB[1],rC[11+1*TT0I]); \
  TYPE_MAC(rA[12],rB[1],rC[12+1*TT0I]); \
  TYPE_MAC(rA[13],rB[1],rC[13+1*TT0I]); \
  TYPE_MAC(rA[14],rB[1],rC[14+1*TT0I]); \
  TYPE_MAC(rA[15],rB[1],rC[15+1*TT0I]); \

#define MAC_16x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[8+TT0I],rB[0+TT1J],rC[8+0*TT0I]); \
  TYPE_MAC(rA[9+TT0I],rB[0+TT1J],rC[9+0*TT0I]); \
  TYPE_MAC(rA[10+TT0I],rB[0+TT1J],rC[10+0*TT0I]); \
  TYPE_MAC(rA[11+TT0I],rB[0+TT1J],rC[11+0*TT0I]); \
  TYPE_MAC(rA[12+TT0I],rB[0+TT1J],rC[12+0*TT0I]); \
  TYPE_MAC(rA[13+TT0I],rB[0+TT1J],rC[13+0*TT0I]); \
  TYPE_MAC(rA[14+TT0I],rB[0+TT1J],rC[14+0*TT0I]); \
  TYPE_MAC(rA[15+TT0I],rB[0+TT1J],rC[15+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[8+TT0I],rB[1+TT1J],rC[8+1*TT0I]); \
  TYPE_MAC(rA[9+TT0I],rB[1+TT1J],rC[9+1*TT0I]); \
  TYPE_MAC(rA[10+TT0I],rB[1+TT1J],rC[10+1*TT0I]); \
  TYPE_MAC(rA[11+TT0I],rB[1+TT1J],rC[11+1*TT0I]); \
  TYPE_MAC(rA[12+TT0I],rB[1+TT1J],rC[12+1*TT0I]); \
  TYPE_MAC(rA[13+TT0I],rB[1+TT1J],rC[13+1*TT0I]); \
  TYPE_MAC(rA[14+TT0I],rB[1+TT1J],rC[14+1*TT0I]); \
  TYPE_MAC(rA[15+TT0I],rB[1+TT1J],rC[15+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_BSS_BH_MT256x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT16_2_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_bfloat16)(0)
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;
  unsigned int globalReadOffsetAL_8_0 = globalReadOffsetAL + 0 + 8*LSPA;
  unsigned int globalReadOffsetAL_9_0 = globalReadOffsetAL + 0 + 9*LSPA;
  unsigned int globalReadOffsetAL_10_0 = globalReadOffsetAL + 0 + 10*LSPA;
  unsigned int globalReadOffsetAL_11_0 = globalReadOffsetAL + 0 + 11*LSPA;
  unsigned int globalReadOffsetAL_12_0 = globalReadOffsetAL + 0 + 12*LSPA;
  unsigned int globalReadOffsetAL_13_0 = globalReadOffsetAL + 0 + 13*LSPA;
  unsigned int globalReadOffsetAL_14_0 = globalReadOffsetAL + 0 + 14*LSPA;
  unsigned int globalReadOffsetAL_15_0 = globalReadOffsetAL + 0 + 15*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_15_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 8*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 9*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 10*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 11*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 12*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 13*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 14*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 15*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_0_0_15_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_8_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_9_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_10_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_11_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_12_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_13_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_14_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_15_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_8_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_9_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_10_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_11_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_12_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_13_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_14_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_15_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0+TT0I] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0+TT0I] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0+TT0I] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0+TT0I] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0+TT0I] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0+TT0I] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0+TT0I] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0+TT0I] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0+TT0I] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0+TT0I] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0+TT0I] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0+TT0I] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0+TT0I] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0+TT0I] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0+TT0I] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0+TT0I] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0+TT0I] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0+TT0I] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0+TT0I] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0+TT0I] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0+TT0I] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0+TT0I] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0+TT0I] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0+TT0I] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0+TT0I] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0+TT0I] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0+TT0I] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0+TT0I] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0+TT0I] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0+TT0I] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0+TT0I] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0+TT0I] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0+TT0I] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0+TT0I] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0+TT0I] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0+TT0I] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0+TT0I] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0+TT0I] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0+TT0I] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0+TT0I] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0+TT0I] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0+TT0I] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0+TT0I] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0+TT0I] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0+TT0I] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0+TT0I] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0+TT0I] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0+TT0I] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0+TT0I] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0+TT0I] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0+TT0I] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0+TT0I] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0+TT0I] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0+TT0I] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0+TT0I] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0+TT0I] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0+TT0I] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0+TT0I] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0+TT0I] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0+TT0I] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0+TT0I] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0+TT0I] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0+TT0I] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0+TT0I] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_16x2


    /* iter 15 */

    MAC_16x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_0_8_0 = ( globalReadOffsetAL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_0_0_9_0 = ( globalReadOffsetAL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_0_0_10_0 = ( globalReadOffsetAL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_0_0_11_0 = ( globalReadOffsetAL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_0_0_12_0 = ( globalReadOffsetAL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_0_0_13_0 = ( globalReadOffsetAL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_0_0_14_0 = ( globalReadOffsetAL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_0_0_15_0 = ( globalReadOffsetAL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_16x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  8*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 8*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[8*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  9*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 9*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[9*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  10*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 10*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[10*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  11*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 11*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[11*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  12*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 12*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[12*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  13*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 13*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[13*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  14*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 14*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[14*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  15*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 15*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[15*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  8*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 8*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[8*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  9*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 9*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[9*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  10*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 10*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[10*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  11*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 11*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[11*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  12*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 12*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[12*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  13*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 13*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[13*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  14*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 14*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[14*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  15*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 15*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[15*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_16x2
#undef MAC_16x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 16
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 2
#define NLPA 4
#define NLPB 16

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 2304

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 2x16 micro-tile */
#define MAC_2x16\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[0],rB[8],rC[0+8*TT0I]); \
  TYPE_MAC(rA[1],rB[8],rC[1+8*TT0I]); \
  TYPE_MAC(rA[0],rB[9],rC[0+9*TT0I]); \
  TYPE_MAC(rA[1],rB[9],rC[1+9*TT0I]); \
  TYPE_MAC(rA[0],rB[10],rC[0+10*TT0I]); \
  TYPE_MAC(rA[1],rB[10],rC[1+10*TT0I]); \
  TYPE_MAC(rA[0],rB[11],rC[0+11*TT0I]); \
  TYPE_MAC(rA[1],rB[11],rC[1+11*TT0I]); \
  TYPE_MAC(rA[0],rB[12],rC[0+12*TT0I]); \
  TYPE_MAC(rA[1],rB[12],rC[1+12*TT0I]); \
  TYPE_MAC(rA[0],rB[13],rC[0+13*TT0I]); \
  TYPE_MAC(rA[1],rB[13],rC[1+13*TT0I]); \
  TYPE_MAC(rA[0],rB[14],rC[0+14*TT0I]); \
  TYPE_MAC(rA[1],rB[14],rC[1+14*TT0I]); \
  TYPE_MAC(rA[0],rB[15],rC[0+15*TT0I]); \
  TYPE_MAC(rA[1],rB[15],rC[1+15*TT0I]); \

#define MAC_2x16_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[8+TT1J],rC[0+8*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[8+TT1J],rC[1+8*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[9+TT1J],rC[0+9*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[9+TT1J],rC[1+9*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[10+TT1J],rC[0+10*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[10+TT1J],rC[1+10*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[11+TT1J],rC[0+11*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[11+TT1J],rC[1+11*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[12+TT1J],rC[0+12*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[12+TT1J],rC[1+12*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[13+TT1J],rC[0+13*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[13+TT1J],rC[1+13*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[14+TT1J],rC[0+14*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[14+TT1J],rC[1+14*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[15+TT1J],rC[0+15*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[15+TT1J],rC[1+15*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_BSS_BH_MT16x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB2_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_16_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_bfloat16)(0)
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_1_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_1_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_1_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_1_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_1_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_1_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_1_0_6_0;
  DATA_TYPE b_0_0_7_0;
  DATA_TYPE b_1_0_7_0;
  DATA_TYPE b_0_0_8_0;
  DATA_TYPE b_1_0_8_0;
  DATA_TYPE b_0_0_9_0;
  DATA_TYPE b_1_0_9_0;
  DATA_TYPE b_0_0_10_0;
  DATA_TYPE b_1_0_10_0;
  DATA_TYPE b_0_0_11_0;
  DATA_TYPE b_1_0_11_0;
  DATA_TYPE b_0_0_12_0;
  DATA_TYPE b_1_0_12_0;
  DATA_TYPE b_0_0_13_0;
  DATA_TYPE b_1_0_13_0;
  DATA_TYPE b_0_0_14_0;
  DATA_TYPE b_1_0_14_0;
  DATA_TYPE b_0_0_15_0;
  DATA_TYPE b_1_0_15_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSCB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;
  unsigned int globalReadOffsetBL_4_0 = globalReadOffsetBL + 0 + 4*LSPB;
  unsigned int globalReadOffsetBL_5_0 = globalReadOffsetBL + 0 + 5*LSPB;
  unsigned int globalReadOffsetBL_6_0 = globalReadOffsetBL + 0 + 6*LSPB;
  unsigned int globalReadOffsetBL_7_0 = globalReadOffsetBL + 0 + 7*LSPB;
  unsigned int globalReadOffsetBL_8_0 = globalReadOffsetBL + 0 + 8*LSPB;
  unsigned int globalReadOffsetBL_9_0 = globalReadOffsetBL + 0 + 9*LSPB;
  unsigned int globalReadOffsetBL_10_0 = globalReadOffsetBL + 0 + 10*LSPB;
  unsigned int globalReadOffsetBL_11_0 = globalReadOffsetBL + 0 + 11*LSPB;
  unsigned int globalReadOffsetBL_12_0 = globalReadOffsetBL + 0 + 12*LSPB;
  unsigned int globalReadOffsetBL_13_0 = globalReadOffsetBL + 0 + 13*LSPB;
  unsigned int globalReadOffsetBL_14_0 = globalReadOffsetBL + 0 + 14*LSPB;
  unsigned int globalReadOffsetBL_15_0 = globalReadOffsetBL + 0 + 15*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_1_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_1_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_1_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_1_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_1_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_1_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );
  int64_t globalReadOffsetB_1_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_1_0), (globalReadOffsetBL_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_4_0), wgK );
  int64_t globalReadOffsetB_1_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_1_0), (globalReadOffsetBL_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_5_0), wgK );
  int64_t globalReadOffsetB_1_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_1_0), (globalReadOffsetBL_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_6_0), wgK );
  int64_t globalReadOffsetB_1_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_1_0), (globalReadOffsetBL_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_7_0), wgK );
  int64_t globalReadOffsetB_1_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_1_0), (globalReadOffsetBL_7_0), wgK );
  int64_t globalReadOffsetB_0_0_8_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_8_0), wgK );
  int64_t globalReadOffsetB_1_0_8_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_1_0), (globalReadOffsetBL_8_0), wgK );
  int64_t globalReadOffsetB_0_0_9_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_9_0), wgK );
  int64_t globalReadOffsetB_1_0_9_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_1_0), (globalReadOffsetBL_9_0), wgK );
  int64_t globalReadOffsetB_0_0_10_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_10_0), wgK );
  int64_t globalReadOffsetB_1_0_10_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_1_0), (globalReadOffsetBL_10_0), wgK );
  int64_t globalReadOffsetB_0_0_11_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_11_0), wgK );
  int64_t globalReadOffsetB_1_0_11_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_1_0), (globalReadOffsetBL_11_0), wgK );
  int64_t globalReadOffsetB_0_0_12_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_12_0), wgK );
  int64_t globalReadOffsetB_1_0_12_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_1_0), (globalReadOffsetBL_12_0), wgK );
  int64_t globalReadOffsetB_0_0_13_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_13_0), wgK );
  int64_t globalReadOffsetB_1_0_13_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_1_0), (globalReadOffsetBL_13_0), wgK );
  int64_t globalReadOffsetB_0_0_14_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_14_0), wgK );
  int64_t globalReadOffsetB_1_0_14_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_1_0), (globalReadOffsetBL_14_0), wgK );
  int64_t globalReadOffsetB_0_0_15_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_15_0), wgK );
  int64_t globalReadOffsetB_1_0_15_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_1_0), (globalReadOffsetBL_15_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_1_0_0_0 = B + globalReadOffsetB_1_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_1_0_1_0 = B + globalReadOffsetB_1_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_1_0_2_0 = B + globalReadOffsetB_1_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_1_0_3_0 = B + globalReadOffsetB_1_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_1_0_4_0 = B + globalReadOffsetB_1_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_1_0_5_0 = B + globalReadOffsetB_1_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_1_0_6_0 = B + globalReadOffsetB_1_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;
  DATA_TYPE const *globalReadB_1_0_7_0 = B + globalReadOffsetB_1_0_7_0;
  DATA_TYPE const *globalReadB_0_0_8_0 = B + globalReadOffsetB_0_0_8_0;
  DATA_TYPE const *globalReadB_1_0_8_0 = B + globalReadOffsetB_1_0_8_0;
  DATA_TYPE const *globalReadB_0_0_9_0 = B + globalReadOffsetB_0_0_9_0;
  DATA_TYPE const *globalReadB_1_0_9_0 = B + globalReadOffsetB_1_0_9_0;
  DATA_TYPE const *globalReadB_0_0_10_0 = B + globalReadOffsetB_0_0_10_0;
  DATA_TYPE const *globalReadB_1_0_10_0 = B + globalReadOffsetB_1_0_10_0;
  DATA_TYPE const *globalReadB_0_0_11_0 = B + globalReadOffsetB_0_0_11_0;
  DATA_TYPE const *globalReadB_1_0_11_0 = B + globalReadOffsetB_1_0_11_0;
  DATA_TYPE const *globalReadB_0_0_12_0 = B + globalReadOffsetB_0_0_12_0;
  DATA_TYPE const *globalReadB_1_0_12_0 = B + globalReadOffsetB_1_0_12_0;
  DATA_TYPE const *globalReadB_0_0_13_0 = B + globalReadOffsetB_0_0_13_0;
  DATA_TYPE const *globalReadB_1_0_13_0 = B + globalReadOffsetB_1_0_13_0;
  DATA_TYPE const *globalReadB_0_0_14_0 = B + globalReadOffsetB_0_0_14_0;
  DATA_TYPE const *globalReadB_1_0_14_0 = B + globalReadOffsetB_1_0_14_0;
  DATA_TYPE const *globalReadB_0_0_15_0 = B + globalReadOffsetB_0_0_15_0;
  DATA_TYPE const *globalReadB_1_0_15_0 = B + globalReadOffsetB_1_0_15_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_1_0_0_0 = localWriteFirstOffsetB + (0 + 1*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_1_0_1_0 = localWriteFirstOffsetB + (0 + 1*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_1_0_2_0 = localWriteFirstOffsetB + (0 + 1*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_1_0_3_0 = localWriteFirstOffsetB + (0 + 1*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 4*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_1_0_4_0 = localWriteFirstOffsetB + (0 + 1*LSCB) + (0 + 4*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 5*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_1_0_5_0 = localWriteFirstOffsetB + (0 + 1*LSCB) + (0 + 5*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 6*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_1_0_6_0 = localWriteFirstOffsetB + (0 + 1*LSCB) + (0 + 6*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 7*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_1_0_7_0 = localWriteFirstOffsetB + (0 + 1*LSCB) + (0 + 7*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_8_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 8*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_1_0_8_0 = localWriteFirstOffsetB + (0 + 1*LSCB) + (0 + 8*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_9_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 9*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_1_0_9_0 = localWriteFirstOffsetB + (0 + 1*LSCB) + (0 + 9*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_10_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 10*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_1_0_10_0 = localWriteFirstOffsetB + (0 + 1*LSCB) + (0 + 10*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_11_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 11*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_1_0_11_0 = localWriteFirstOffsetB + (0 + 1*LSCB) + (0 + 11*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_12_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 12*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_1_0_12_0 = localWriteFirstOffsetB + (0 + 1*LSCB) + (0 + 12*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_13_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 13*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_1_0_13_0 = localWriteFirstOffsetB + (0 + 1*LSCB) + (0 + 13*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_14_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 14*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_1_0_14_0 = localWriteFirstOffsetB + (0 + 1*LSCB) + (0 + 14*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_15_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 15*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_1_0_15_0 = localWriteFirstOffsetB + (0 + 1*LSCB) + (0 + 15*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_1_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_1_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_1_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_1_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_1_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_1_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_1_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;
  DATA_TYPE *localWriteB_1_0_7_0;
  DATA_TYPE *localWriteB_0_0_8_0;
  DATA_TYPE *localWriteB_1_0_8_0;
  DATA_TYPE *localWriteB_0_0_9_0;
  DATA_TYPE *localWriteB_1_0_9_0;
  DATA_TYPE *localWriteB_0_0_10_0;
  DATA_TYPE *localWriteB_1_0_10_0;
  DATA_TYPE *localWriteB_0_0_11_0;
  DATA_TYPE *localWriteB_1_0_11_0;
  DATA_TYPE *localWriteB_0_0_12_0;
  DATA_TYPE *localWriteB_1_0_12_0;
  DATA_TYPE *localWriteB_0_0_13_0;
  DATA_TYPE *localWriteB_1_0_13_0;
  DATA_TYPE *localWriteB_0_0_14_0;
  DATA_TYPE *localWriteB_1_0_14_0;
  DATA_TYPE *localWriteB_0_0_15_0;
  DATA_TYPE *localWriteB_1_0_15_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_1_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_1_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_1_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_1_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_1_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_1_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_1_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_1_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_1_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_1_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_1_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_1_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_1_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_1_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);
  localWriteB_1_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_1_0_7_0);
  localWriteB_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_8_0);
  localWriteB_1_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_1_0_8_0);
  localWriteB_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_9_0);
  localWriteB_1_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_1_0_9_0);
  localWriteB_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_10_0);
  localWriteB_1_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_1_0_10_0);
  localWriteB_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_11_0);
  localWriteB_1_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_1_0_11_0);
  localWriteB_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_12_0);
  localWriteB_1_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_1_0_12_0);
  localWriteB_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_13_0);
  localWriteB_1_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_1_0_13_0);
  localWriteB_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_14_0);
  localWriteB_1_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_1_0_14_0);
  localWriteB_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_15_0);
  localWriteB_1_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_1_0_15_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_1_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_1_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_1_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_1_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_1_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_1_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_1_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_1_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_8_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_1_0_8_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_9_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_1_0_9_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_10_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_1_0_10_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_11_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_1_0_11_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_12_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_1_0_12_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_13_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_1_0_13_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_14_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_1_0_14_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_15_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_1_0_15_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_1_0_0_0 = *(globalReadB_1_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_1_0_1_0 = *(globalReadB_1_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_1_0_2_0 = *(globalReadB_1_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_1_0_3_0 = *(globalReadB_1_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_1_0_4_0 = *(globalReadB_1_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_1_0_5_0 = *(globalReadB_1_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_1_0_6_0 = *(globalReadB_1_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);
    b_1_0_7_0 = *(globalReadB_1_0_7_0 + 0);
    b_0_0_8_0 = *(globalReadB_0_0_8_0 + 0);
    b_1_0_8_0 = *(globalReadB_1_0_8_0 + 0);
    b_0_0_9_0 = *(globalReadB_0_0_9_0 + 0);
    b_1_0_9_0 = *(globalReadB_1_0_9_0 + 0);
    b_0_0_10_0 = *(globalReadB_0_0_10_0 + 0);
    b_1_0_10_0 = *(globalReadB_1_0_10_0 + 0);
    b_0_0_11_0 = *(globalReadB_0_0_11_0 + 0);
    b_1_0_11_0 = *(globalReadB_1_0_11_0 + 0);
    b_0_0_12_0 = *(globalReadB_0_0_12_0 + 0);
    b_1_0_12_0 = *(globalReadB_1_0_12_0 + 0);
    b_0_0_13_0 = *(globalReadB_0_0_13_0 + 0);
    b_1_0_13_0 = *(globalReadB_1_0_13_0 + 0);
    b_0_0_14_0 = *(globalReadB_0_0_14_0 + 0);
    b_1_0_14_0 = *(globalReadB_1_0_14_0 + 0);
    b_0_0_15_0 = *(globalReadB_0_0_15_0 + 0);
    b_1_0_15_0 = *(globalReadB_1_0_15_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_1_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_1_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_1_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_1_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_1_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_1_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_1_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_1_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_1_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_1_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_1_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_1_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_1_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_1_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_1_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_1_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_1_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_1_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_1_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_1_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_1_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_1_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_1_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_1_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_8_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_8_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_1_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_1_0_8_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_1_0_8_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_9_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_9_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_1_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_1_0_9_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_1_0_9_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_10_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_10_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_1_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_1_0_10_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_1_0_10_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_11_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_11_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_1_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_1_0_11_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_1_0_11_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_12_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_12_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_1_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_1_0_12_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_1_0_12_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_13_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_13_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_1_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_1_0_13_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_1_0_13_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_14_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_14_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_1_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_1_0_14_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_1_0_14_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_15_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_15_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_1_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_1_0_15_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_1_0_15_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_1_0_0_0 + 0) = b_1_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_1_0_1_0 + 0) = b_1_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_1_0_2_0 + 0) = b_1_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_1_0_3_0 + 0) = b_1_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_1_0_4_0 + 0) = b_1_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_1_0_5_0 + 0) = b_1_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_1_0_6_0 + 0) = b_1_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
    *(localWriteB_1_0_7_0 + 0) = b_1_0_7_0;
    *(localWriteB_0_0_8_0 + 0) = b_0_0_8_0;
    *(localWriteB_1_0_8_0 + 0) = b_1_0_8_0;
    *(localWriteB_0_0_9_0 + 0) = b_0_0_9_0;
    *(localWriteB_1_0_9_0 + 0) = b_1_0_9_0;
    *(localWriteB_0_0_10_0 + 0) = b_0_0_10_0;
    *(localWriteB_1_0_10_0 + 0) = b_1_0_10_0;
    *(localWriteB_0_0_11_0 + 0) = b_0_0_11_0;
    *(localWriteB_1_0_11_0 + 0) = b_1_0_11_0;
    *(localWriteB_0_0_12_0 + 0) = b_0_0_12_0;
    *(localWriteB_1_0_12_0 + 0) = b_1_0_12_0;
    *(localWriteB_0_0_13_0 + 0) = b_0_0_13_0;
    *(localWriteB_1_0_13_0 + 0) = b_1_0_13_0;
    *(localWriteB_0_0_14_0 + 0) = b_0_0_14_0;
    *(localWriteB_1_0_14_0 + 0) = b_1_0_14_0;
    *(localWriteB_0_0_15_0 + 0) = b_0_0_15_0;
    *(localWriteB_1_0_15_0 + 0) = b_1_0_15_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[8*VECTOR_WIDTH+0] = localReadB[8*SG1J*VECTOR_WIDTH + 0]; 
    rB[9*VECTOR_WIDTH+0] = localReadB[9*SG1J*VECTOR_WIDTH + 0]; 
    rB[10*VECTOR_WIDTH+0] = localReadB[10*SG1J*VECTOR_WIDTH + 0]; 
    rB[11*VECTOR_WIDTH+0] = localReadB[11*SG1J*VECTOR_WIDTH + 0]; 
    rB[12*VECTOR_WIDTH+0] = localReadB[12*SG1J*VECTOR_WIDTH + 0]; 
    rB[13*VECTOR_WIDTH+0] = localReadB[13*SG1J*VECTOR_WIDTH + 0]; 
    rB[14*VECTOR_WIDTH+0] = localReadB[14*SG1J*VECTOR_WIDTH + 0]; 
    rB[15*VECTOR_WIDTH+0] = localReadB[15*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[8*VECTOR_WIDTH+0+TT1J] = localReadB[8*SG1J*VECTOR_WIDTH + 0]; 
    rB[9*VECTOR_WIDTH+0+TT1J] = localReadB[9*SG1J*VECTOR_WIDTH + 0]; 
    rB[10*VECTOR_WIDTH+0+TT1J] = localReadB[10*SG1J*VECTOR_WIDTH + 0]; 
    rB[11*VECTOR_WIDTH+0+TT1J] = localReadB[11*SG1J*VECTOR_WIDTH + 0]; 
    rB[12*VECTOR_WIDTH+0+TT1J] = localReadB[12*SG1J*VECTOR_WIDTH + 0]; 
    rB[13*VECTOR_WIDTH+0+TT1J] = localReadB[13*SG1J*VECTOR_WIDTH + 0]; 
    rB[14*VECTOR_WIDTH+0+TT1J] = localReadB[14*SG1J*VECTOR_WIDTH + 0]; 
    rB[15*VECTOR_WIDTH+0+TT1J] = localReadB[15*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x16


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[8*VECTOR_WIDTH+0] = localReadB[8*SG1J*VECTOR_WIDTH + 0]; 
    rB[9*VECTOR_WIDTH+0] = localReadB[9*SG1J*VECTOR_WIDTH + 0]; 
    rB[10*VECTOR_WIDTH+0] = localReadB[10*SG1J*VECTOR_WIDTH + 0]; 
    rB[11*VECTOR_WIDTH+0] = localReadB[11*SG1J*VECTOR_WIDTH + 0]; 
    rB[12*VECTOR_WIDTH+0] = localReadB[12*SG1J*VECTOR_WIDTH + 0]; 
    rB[13*VECTOR_WIDTH+0] = localReadB[13*SG1J*VECTOR_WIDTH + 0]; 
    rB[14*VECTOR_WIDTH+0] = localReadB[14*SG1J*VECTOR_WIDTH + 0]; 
    rB[15*VECTOR_WIDTH+0] = localReadB[15*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x16_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[8*VECTOR_WIDTH+0+TT1J] = localReadB[8*SG1J*VECTOR_WIDTH + 0]; 
    rB[9*VECTOR_WIDTH+0+TT1J] = localReadB[9*SG1J*VECTOR_WIDTH + 0]; 
    rB[10*VECTOR_WIDTH+0+TT1J] = localReadB[10*SG1J*VECTOR_WIDTH + 0]; 
    rB[11*VECTOR_WIDTH+0+TT1J] = localReadB[11*SG1J*VECTOR_WIDTH + 0]; 
    rB[12*VECTOR_WIDTH+0+TT1J] = localReadB[12*SG1J*VECTOR_WIDTH + 0]; 
    rB[13*VECTOR_WIDTH+0+TT1J] = localReadB[13*SG1J*VECTOR_WIDTH + 0]; 
    rB[14*VECTOR_WIDTH+0+TT1J] = localReadB[14*SG1J*VECTOR_WIDTH + 0]; 
    rB[15*VECTOR_WIDTH+0+TT1J] = localReadB[15*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x16


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[8*VECTOR_WIDTH+0] = localReadB[8*SG1J*VECTOR_WIDTH + 0]; 
    rB[9*VECTOR_WIDTH+0] = localReadB[9*SG1J*VECTOR_WIDTH + 0]; 
    rB[10*VECTOR_WIDTH+0] = localReadB[10*SG1J*VECTOR_WIDTH + 0]; 
    rB[11*VECTOR_WIDTH+0] = localReadB[11*SG1J*VECTOR_WIDTH + 0]; 
    rB[12*VECTOR_WIDTH+0] = localReadB[12*SG1J*VECTOR_WIDTH + 0]; 
    rB[13*VECTOR_WIDTH+0] = localReadB[13*SG1J*VECTOR_WIDTH + 0]; 
    rB[14*VECTOR_WIDTH+0] = localReadB[14*SG1J*VECTOR_WIDTH + 0]; 
    rB[15*VECTOR_WIDTH+0] = localReadB[15*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x16_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[8*VECTOR_WIDTH+0+TT1J] = localReadB[8*SG1J*VECTOR_WIDTH + 0]; 
    rB[9*VECTOR_WIDTH+0+TT1J] = localReadB[9*SG1J*VECTOR_WIDTH + 0]; 
    rB[10*VECTOR_WIDTH+0+TT1J] = localReadB[10*SG1J*VECTOR_WIDTH + 0]; 
    rB[11*VECTOR_WIDTH+0+TT1J] = localReadB[11*SG1J*VECTOR_WIDTH + 0]; 
    rB[12*VECTOR_WIDTH+0+TT1J] = localReadB[12*SG1J*VECTOR_WIDTH + 0]; 
    rB[13*VECTOR_WIDTH+0+TT1J] = localReadB[13*SG1J*VECTOR_WIDTH + 0]; 
    rB[14*VECTOR_WIDTH+0+TT1J] = localReadB[14*SG1J*VECTOR_WIDTH + 0]; 
    rB[15*VECTOR_WIDTH+0+TT1J] = localReadB[15*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x16


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[8*VECTOR_WIDTH+0] = localReadB[8*SG1J*VECTOR_WIDTH + 0]; 
    rB[9*VECTOR_WIDTH+0] = localReadB[9*SG1J*VECTOR_WIDTH + 0]; 
    rB[10*VECTOR_WIDTH+0] = localReadB[10*SG1J*VECTOR_WIDTH + 0]; 
    rB[11*VECTOR_WIDTH+0] = localReadB[11*SG1J*VECTOR_WIDTH + 0]; 
    rB[12*VECTOR_WIDTH+0] = localReadB[12*SG1J*VECTOR_WIDTH + 0]; 
    rB[13*VECTOR_WIDTH+0] = localReadB[13*SG1J*VECTOR_WIDTH + 0]; 
    rB[14*VECTOR_WIDTH+0] = localReadB[14*SG1J*VECTOR_WIDTH + 0]; 
    rB[15*VECTOR_WIDTH+0] = localReadB[15*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x16_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[8*VECTOR_WIDTH+0+TT1J] = localReadB[8*SG1J*VECTOR_WIDTH + 0]; 
    rB[9*VECTOR_WIDTH+0+TT1J] = localReadB[9*SG1J*VECTOR_WIDTH + 0]; 
    rB[10*VECTOR_WIDTH+0+TT1J] = localReadB[10*SG1J*VECTOR_WIDTH + 0]; 
    rB[11*VECTOR_WIDTH+0+TT1J] = localReadB[11*SG1J*VECTOR_WIDTH + 0]; 
    rB[12*VECTOR_WIDTH+0+TT1J] = localReadB[12*SG1J*VECTOR_WIDTH + 0]; 
    rB[13*VECTOR_WIDTH+0+TT1J] = localReadB[13*SG1J*VECTOR_WIDTH + 0]; 
    rB[14*VECTOR_WIDTH+0+TT1J] = localReadB[14*SG1J*VECTOR_WIDTH + 0]; 
    rB[15*VECTOR_WIDTH+0+TT1J] = localReadB[15*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x16


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[8*VECTOR_WIDTH+0] = localReadB[8*SG1J*VECTOR_WIDTH + 0]; 
    rB[9*VECTOR_WIDTH+0] = localReadB[9*SG1J*VECTOR_WIDTH + 0]; 
    rB[10*VECTOR_WIDTH+0] = localReadB[10*SG1J*VECTOR_WIDTH + 0]; 
    rB[11*VECTOR_WIDTH+0] = localReadB[11*SG1J*VECTOR_WIDTH + 0]; 
    rB[12*VECTOR_WIDTH+0] = localReadB[12*SG1J*VECTOR_WIDTH + 0]; 
    rB[13*VECTOR_WIDTH+0] = localReadB[13*SG1J*VECTOR_WIDTH + 0]; 
    rB[14*VECTOR_WIDTH+0] = localReadB[14*SG1J*VECTOR_WIDTH + 0]; 
    rB[15*VECTOR_WIDTH+0] = localReadB[15*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x16_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[8*VECTOR_WIDTH+0+TT1J] = localReadB[8*SG1J*VECTOR_WIDTH + 0]; 
    rB[9*VECTOR_WIDTH+0+TT1J] = localReadB[9*SG1J*VECTOR_WIDTH + 0]; 
    rB[10*VECTOR_WIDTH+0+TT1J] = localReadB[10*SG1J*VECTOR_WIDTH + 0]; 
    rB[11*VECTOR_WIDTH+0+TT1J] = localReadB[11*SG1J*VECTOR_WIDTH + 0]; 
    rB[12*VECTOR_WIDTH+0+TT1J] = localReadB[12*SG1J*VECTOR_WIDTH + 0]; 
    rB[13*VECTOR_WIDTH+0+TT1J] = localReadB[13*SG1J*VECTOR_WIDTH + 0]; 
    rB[14*VECTOR_WIDTH+0+TT1J] = localReadB[14*SG1J*VECTOR_WIDTH + 0]; 
    rB[15*VECTOR_WIDTH+0+TT1J] = localReadB[15*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x16


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[8*VECTOR_WIDTH+0] = localReadB[8*SG1J*VECTOR_WIDTH + 0]; 
    rB[9*VECTOR_WIDTH+0] = localReadB[9*SG1J*VECTOR_WIDTH + 0]; 
    rB[10*VECTOR_WIDTH+0] = localReadB[10*SG1J*VECTOR_WIDTH + 0]; 
    rB[11*VECTOR_WIDTH+0] = localReadB[11*SG1J*VECTOR_WIDTH + 0]; 
    rB[12*VECTOR_WIDTH+0] = localReadB[12*SG1J*VECTOR_WIDTH + 0]; 
    rB[13*VECTOR_WIDTH+0] = localReadB[13*SG1J*VECTOR_WIDTH + 0]; 
    rB[14*VECTOR_WIDTH+0] = localReadB[14*SG1J*VECTOR_WIDTH + 0]; 
    rB[15*VECTOR_WIDTH+0] = localReadB[15*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x16_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[8*VECTOR_WIDTH+0+TT1J] = localReadB[8*SG1J*VECTOR_WIDTH + 0]; 
    rB[9*VECTOR_WIDTH+0+TT1J] = localReadB[9*SG1J*VECTOR_WIDTH + 0]; 
    rB[10*VECTOR_WIDTH+0+TT1J] = localReadB[10*SG1J*VECTOR_WIDTH + 0]; 
    rB[11*VECTOR_WIDTH+0+TT1J] = localReadB[11*SG1J*VECTOR_WIDTH + 0]; 
    rB[12*VECTOR_WIDTH+0+TT1J] = localReadB[12*SG1J*VECTOR_WIDTH + 0]; 
    rB[13*VECTOR_WIDTH+0+TT1J] = localReadB[13*SG1J*VECTOR_WIDTH + 0]; 
    rB[14*VECTOR_WIDTH+0+TT1J] = localReadB[14*SG1J*VECTOR_WIDTH + 0]; 
    rB[15*VECTOR_WIDTH+0+TT1J] = localReadB[15*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x16


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[8*VECTOR_WIDTH+0] = localReadB[8*SG1J*VECTOR_WIDTH + 0]; 
    rB[9*VECTOR_WIDTH+0] = localReadB[9*SG1J*VECTOR_WIDTH + 0]; 
    rB[10*VECTOR_WIDTH+0] = localReadB[10*SG1J*VECTOR_WIDTH + 0]; 
    rB[11*VECTOR_WIDTH+0] = localReadB[11*SG1J*VECTOR_WIDTH + 0]; 
    rB[12*VECTOR_WIDTH+0] = localReadB[12*SG1J*VECTOR_WIDTH + 0]; 
    rB[13*VECTOR_WIDTH+0] = localReadB[13*SG1J*VECTOR_WIDTH + 0]; 
    rB[14*VECTOR_WIDTH+0] = localReadB[14*SG1J*VECTOR_WIDTH + 0]; 
    rB[15*VECTOR_WIDTH+0] = localReadB[15*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x16_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[8*VECTOR_WIDTH+0+TT1J] = localReadB[8*SG1J*VECTOR_WIDTH + 0]; 
    rB[9*VECTOR_WIDTH+0+TT1J] = localReadB[9*SG1J*VECTOR_WIDTH + 0]; 
    rB[10*VECTOR_WIDTH+0+TT1J] = localReadB[10*SG1J*VECTOR_WIDTH + 0]; 
    rB[11*VECTOR_WIDTH+0+TT1J] = localReadB[11*SG1J*VECTOR_WIDTH + 0]; 
    rB[12*VECTOR_WIDTH+0+TT1J] = localReadB[12*SG1J*VECTOR_WIDTH + 0]; 
    rB[13*VECTOR_WIDTH+0+TT1J] = localReadB[13*SG1J*VECTOR_WIDTH + 0]; 
    rB[14*VECTOR_WIDTH+0+TT1J] = localReadB[14*SG1J*VECTOR_WIDTH + 0]; 
    rB[15*VECTOR_WIDTH+0+TT1J] = localReadB[15*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x16


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[8*VECTOR_WIDTH+0] = localReadB[8*SG1J*VECTOR_WIDTH + 0]; 
    rB[9*VECTOR_WIDTH+0] = localReadB[9*SG1J*VECTOR_WIDTH + 0]; 
    rB[10*VECTOR_WIDTH+0] = localReadB[10*SG1J*VECTOR_WIDTH + 0]; 
    rB[11*VECTOR_WIDTH+0] = localReadB[11*SG1J*VECTOR_WIDTH + 0]; 
    rB[12*VECTOR_WIDTH+0] = localReadB[12*SG1J*VECTOR_WIDTH + 0]; 
    rB[13*VECTOR_WIDTH+0] = localReadB[13*SG1J*VECTOR_WIDTH + 0]; 
    rB[14*VECTOR_WIDTH+0] = localReadB[14*SG1J*VECTOR_WIDTH + 0]; 
    rB[15*VECTOR_WIDTH+0] = localReadB[15*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x16_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[8*VECTOR_WIDTH+0+TT1J] = localReadB[8*SG1J*VECTOR_WIDTH + 0]; 
    rB[9*VECTOR_WIDTH+0+TT1J] = localReadB[9*SG1J*VECTOR_WIDTH + 0]; 
    rB[10*VECTOR_WIDTH+0+TT1J] = localReadB[10*SG1J*VECTOR_WIDTH + 0]; 
    rB[11*VECTOR_WIDTH+0+TT1J] = localReadB[11*SG1J*VECTOR_WIDTH + 0]; 
    rB[12*VECTOR_WIDTH+0+TT1J] = localReadB[12*SG1J*VECTOR_WIDTH + 0]; 
    rB[13*VECTOR_WIDTH+0+TT1J] = localReadB[13*SG1J*VECTOR_WIDTH + 0]; 
    rB[14*VECTOR_WIDTH+0+TT1J] = localReadB[14*SG1J*VECTOR_WIDTH + 0]; 
    rB[15*VECTOR_WIDTH+0+TT1J] = localReadB[15*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x16


    /* iter 15 */

    MAC_2x16_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_1_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_1_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_1_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_1_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_1_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_1_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_1_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_1_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_1_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_1_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_1_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_1_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_1_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_1_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_1_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_1_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_1_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_1_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_1_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_1_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_1_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_1_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_1_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_1_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_1_0_4_0 = ( globalReadOffsetBL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_1_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_1_0_5_0 = ( globalReadOffsetBL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_1_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_1_0_6_0 = ( globalReadOffsetBL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_1_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);
  b_1_0_7_0 = ( globalReadOffsetBL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_1_0_7_0 + 0);
  b_0_0_8_0 = ( globalReadOffsetBL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_8_0 + 0);
  b_1_0_8_0 = ( globalReadOffsetBL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_1_0_8_0 + 0);
  b_0_0_9_0 = ( globalReadOffsetBL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_9_0 + 0);
  b_1_0_9_0 = ( globalReadOffsetBL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_1_0_9_0 + 0);
  b_0_0_10_0 = ( globalReadOffsetBL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_10_0 + 0);
  b_1_0_10_0 = ( globalReadOffsetBL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_1_0_10_0 + 0);
  b_0_0_11_0 = ( globalReadOffsetBL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_11_0 + 0);
  b_1_0_11_0 = ( globalReadOffsetBL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_1_0_11_0 + 0);
  b_0_0_12_0 = ( globalReadOffsetBL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_12_0 + 0);
  b_1_0_12_0 = ( globalReadOffsetBL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_1_0_12_0 + 0);
  b_0_0_13_0 = ( globalReadOffsetBL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_13_0 + 0);
  b_1_0_13_0 = ( globalReadOffsetBL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_1_0_13_0 + 0);
  b_0_0_14_0 = ( globalReadOffsetBL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_14_0 + 0);
  b_1_0_14_0 = ( globalReadOffsetBL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_1_0_14_0 + 0);
  b_0_0_15_0 = ( globalReadOffsetBL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_15_0 + 0);
  b_1_0_15_0 = ( globalReadOffsetBL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_1_0_15_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_1_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_1_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_1_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_1_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_1_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_1_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_1_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_1_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_1_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_1_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_1_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_1_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_1_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_1_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);
  localWriteB_1_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_1_0_7_0);
  localWriteB_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_8_0);
  localWriteB_1_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_1_0_8_0);
  localWriteB_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_9_0);
  localWriteB_1_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_1_0_9_0);
  localWriteB_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_10_0);
  localWriteB_1_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_1_0_10_0);
  localWriteB_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_11_0);
  localWriteB_1_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_1_0_11_0);
  localWriteB_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_12_0);
  localWriteB_1_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_1_0_12_0);
  localWriteB_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_13_0);
  localWriteB_1_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_1_0_13_0);
  localWriteB_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_14_0);
  localWriteB_1_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_1_0_14_0);
  localWriteB_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_15_0);
  localWriteB_1_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_1_0_15_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_1_0_0_0 + 0) = b_1_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_1_0_1_0 + 0) = b_1_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_1_0_2_0 + 0) = b_1_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_1_0_3_0 + 0) = b_1_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_1_0_4_0 + 0) = b_1_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_1_0_5_0 + 0) = b_1_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_1_0_6_0 + 0) = b_1_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
  *(localWriteB_1_0_7_0 + 0) = b_1_0_7_0;
  *(localWriteB_0_0_8_0 + 0) = b_0_0_8_0;
  *(localWriteB_1_0_8_0 + 0) = b_1_0_8_0;
  *(localWriteB_0_0_9_0 + 0) = b_0_0_9_0;
  *(localWriteB_1_0_9_0 + 0) = b_1_0_9_0;
  *(localWriteB_0_0_10_0 + 0) = b_0_0_10_0;
  *(localWriteB_1_0_10_0 + 0) = b_1_0_10_0;
  *(localWriteB_0_0_11_0 + 0) = b_0_0_11_0;
  *(localWriteB_1_0_11_0 + 0) = b_1_0_11_0;
  *(localWriteB_0_0_12_0 + 0) = b_0_0_12_0;
  *(localWriteB_1_0_12_0 + 0) = b_1_0_12_0;
  *(localWriteB_0_0_13_0 + 0) = b_0_0_13_0;
  *(localWriteB_1_0_13_0 + 0) = b_1_0_13_0;
  *(localWriteB_0_0_14_0 + 0) = b_0_0_14_0;
  *(localWriteB_1_0_14_0 + 0) = b_1_0_14_0;
  *(localWriteB_0_0_15_0 + 0) = b_0_0_15_0;
  *(localWriteB_1_0_15_0 + 0) = b_1_0_15_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[8*VECTOR_WIDTH+0] = localReadB[8*SG1J*VECTOR_WIDTH + 0]; 
    rB[9*VECTOR_WIDTH+0] = localReadB[9*SG1J*VECTOR_WIDTH + 0]; 
    rB[10*VECTOR_WIDTH+0] = localReadB[10*SG1J*VECTOR_WIDTH + 0]; 
    rB[11*VECTOR_WIDTH+0] = localReadB[11*SG1J*VECTOR_WIDTH + 0]; 
    rB[12*VECTOR_WIDTH+0] = localReadB[12*SG1J*VECTOR_WIDTH + 0]; 
    rB[13*VECTOR_WIDTH+0] = localReadB[13*SG1J*VECTOR_WIDTH + 0]; 
    rB[14*VECTOR_WIDTH+0] = localReadB[14*SG1J*VECTOR_WIDTH + 0]; 
    rB[15*VECTOR_WIDTH+0] = localReadB[15*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x16

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 8*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 8*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (8*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 8*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 8*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (8*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 9*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 9*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (9*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 9*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 9*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (9*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 10*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 10*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (10*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 10*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 10*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (10*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 11*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 11*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (11*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 11*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 11*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (11*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 12*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 12*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (12*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 12*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 12*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (12*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 13*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 13*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (13*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 13*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 13*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (13*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 14*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 14*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (14*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 14*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 14*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (14*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 15*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 15*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (15*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 15*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 15*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (15*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x16
#undef MAC_2x16_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 64
#define SG1J 1
#define TT0I 1
#define TT1J 1
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 64
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 60

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 64
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 4096
#define LDS_NUM_ELEMENTS 4160

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 1x1 micro-tile */
#define MAC_1x1\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_BSS_BH_MT64x1x64_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU60_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU0_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT1_1_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG64_1_1_WGM1(
  float *D,
  float const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_bfloat16)(0)
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE a_0_0_16_0;
  DATA_TYPE a_0_0_17_0;
  DATA_TYPE a_0_0_18_0;
  DATA_TYPE a_0_0_19_0;
  DATA_TYPE a_0_0_20_0;
  DATA_TYPE a_0_0_21_0;
  DATA_TYPE a_0_0_22_0;
  DATA_TYPE a_0_0_23_0;
  DATA_TYPE a_0_0_24_0;
  DATA_TYPE a_0_0_25_0;
  DATA_TYPE a_0_0_26_0;
  DATA_TYPE a_0_0_27_0;
  DATA_TYPE a_0_0_28_0;
  DATA_TYPE a_0_0_29_0;
  DATA_TYPE a_0_0_30_0;
  DATA_TYPE a_0_0_31_0;
  DATA_TYPE a_0_0_32_0;
  DATA_TYPE a_0_0_33_0;
  DATA_TYPE a_0_0_34_0;
  DATA_TYPE a_0_0_35_0;
  DATA_TYPE a_0_0_36_0;
  DATA_TYPE a_0_0_37_0;
  DATA_TYPE a_0_0_38_0;
  DATA_TYPE a_0_0_39_0;
  DATA_TYPE a_0_0_40_0;
  DATA_TYPE a_0_0_41_0;
  DATA_TYPE a_0_0_42_0;
  DATA_TYPE a_0_0_43_0;
  DATA_TYPE a_0_0_44_0;
  DATA_TYPE a_0_0_45_0;
  DATA_TYPE a_0_0_46_0;
  DATA_TYPE a_0_0_47_0;
  DATA_TYPE a_0_0_48_0;
  DATA_TYPE a_0_0_49_0;
  DATA_TYPE a_0_0_50_0;
  DATA_TYPE a_0_0_51_0;
  DATA_TYPE a_0_0_52_0;
  DATA_TYPE a_0_0_53_0;
  DATA_TYPE a_0_0_54_0;
  DATA_TYPE a_0_0_55_0;
  DATA_TYPE a_0_0_56_0;
  DATA_TYPE a_0_0_57_0;
  DATA_TYPE a_0_0_58_0;
  DATA_TYPE a_0_0_59_0;
  DATA_TYPE a_0_0_60_0;
  DATA_TYPE a_0_0_61_0;
  DATA_TYPE a_0_0_62_0;
  DATA_TYPE a_0_0_63_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;
  unsigned int globalReadOffsetAL_8_0 = globalReadOffsetAL + 0 + 8*LSPA;
  unsigned int globalReadOffsetAL_9_0 = globalReadOffsetAL + 0 + 9*LSPA;
  unsigned int globalReadOffsetAL_10_0 = globalReadOffsetAL + 0 + 10*LSPA;
  unsigned int globalReadOffsetAL_11_0 = globalReadOffsetAL + 0 + 11*LSPA;
  unsigned int globalReadOffsetAL_12_0 = globalReadOffsetAL + 0 + 12*LSPA;
  unsigned int globalReadOffsetAL_13_0 = globalReadOffsetAL + 0 + 13*LSPA;
  unsigned int globalReadOffsetAL_14_0 = globalReadOffsetAL + 0 + 14*LSPA;
  unsigned int globalReadOffsetAL_15_0 = globalReadOffsetAL + 0 + 15*LSPA;
  unsigned int globalReadOffsetAL_16_0 = globalReadOffsetAL + 0 + 16*LSPA;
  unsigned int globalReadOffsetAL_17_0 = globalReadOffsetAL + 0 + 17*LSPA;
  unsigned int globalReadOffsetAL_18_0 = globalReadOffsetAL + 0 + 18*LSPA;
  unsigned int globalReadOffsetAL_19_0 = globalReadOffsetAL + 0 + 19*LSPA;
  unsigned int globalReadOffsetAL_20_0 = globalReadOffsetAL + 0 + 20*LSPA;
  unsigned int globalReadOffsetAL_21_0 = globalReadOffsetAL + 0 + 21*LSPA;
  unsigned int globalReadOffsetAL_22_0 = globalReadOffsetAL + 0 + 22*LSPA;
  unsigned int globalReadOffsetAL_23_0 = globalReadOffsetAL + 0 + 23*LSPA;
  unsigned int globalReadOffsetAL_24_0 = globalReadOffsetAL + 0 + 24*LSPA;
  unsigned int globalReadOffsetAL_25_0 = globalReadOffsetAL + 0 + 25*LSPA;
  unsigned int globalReadOffsetAL_26_0 = globalReadOffsetAL + 0 + 26*LSPA;
  unsigned int globalReadOffsetAL_27_0 = globalReadOffsetAL + 0 + 27*LSPA;
  unsigned int globalReadOffsetAL_28_0 = globalReadOffsetAL + 0 + 28*LSPA;
  unsigned int globalReadOffsetAL_29_0 = globalReadOffsetAL + 0 + 29*LSPA;
  unsigned int globalReadOffsetAL_30_0 = globalReadOffsetAL + 0 + 30*LSPA;
  unsigned int globalReadOffsetAL_31_0 = globalReadOffsetAL + 0 + 31*LSPA;
  unsigned int globalReadOffsetAL_32_0 = globalReadOffsetAL + 0 + 32*LSPA;
  unsigned int globalReadOffsetAL_33_0 = globalReadOffsetAL + 0 + 33*LSPA;
  unsigned int globalReadOffsetAL_34_0 = globalReadOffsetAL + 0 + 34*LSPA;
  unsigned int globalReadOffsetAL_35_0 = globalReadOffsetAL + 0 + 35*LSPA;
  unsigned int globalReadOffsetAL_36_0 = globalReadOffsetAL + 0 + 36*LSPA;
  unsigned int globalReadOffsetAL_37_0 = globalReadOffsetAL + 0 + 37*LSPA;
  unsigned int globalReadOffsetAL_38_0 = globalReadOffsetAL + 0 + 38*LSPA;
  unsigned int globalReadOffsetAL_39_0 = globalReadOffsetAL + 0 + 39*LSPA;
  unsigned int globalReadOffsetAL_40_0 = globalReadOffsetAL + 0 + 40*LSPA;
  unsigned int globalReadOffsetAL_41_0 = globalReadOffsetAL + 0 + 41*LSPA;
  unsigned int globalReadOffsetAL_42_0 = globalReadOffsetAL + 0 + 42*LSPA;
  unsigned int globalReadOffsetAL_43_0 = globalReadOffsetAL + 0 + 43*LSPA;
  unsigned int globalReadOffsetAL_44_0 = globalReadOffsetAL + 0 + 44*LSPA;
  unsigned int globalReadOffsetAL_45_0 = globalReadOffsetAL + 0 + 45*LSPA;
  unsigned int globalReadOffsetAL_46_0 = globalReadOffsetAL + 0 + 46*LSPA;
  unsigned int globalReadOffsetAL_47_0 = globalReadOffsetAL + 0 + 47*LSPA;
  unsigned int globalReadOffsetAL_48_0 = globalReadOffsetAL + 0 + 48*LSPA;
  unsigned int globalReadOffsetAL_49_0 = globalReadOffsetAL + 0 + 49*LSPA;
  unsigned int globalReadOffsetAL_50_0 = globalReadOffsetAL + 0 + 50*LSPA;
  unsigned int globalReadOffsetAL_51_0 = globalReadOffsetAL + 0 + 51*LSPA;
  unsigned int globalReadOffsetAL_52_0 = globalReadOffsetAL + 0 + 52*LSPA;
  unsigned int globalReadOffsetAL_53_0 = globalReadOffsetAL + 0 + 53*LSPA;
  unsigned int globalReadOffsetAL_54_0 = globalReadOffsetAL + 0 + 54*LSPA;
  unsigned int globalReadOffsetAL_55_0 = globalReadOffsetAL + 0 + 55*LSPA;
  unsigned int globalReadOffsetAL_56_0 = globalReadOffsetAL + 0 + 56*LSPA;
  unsigned int globalReadOffsetAL_57_0 = globalReadOffsetAL + 0 + 57*LSPA;
  unsigned int globalReadOffsetAL_58_0 = globalReadOffsetAL + 0 + 58*LSPA;
  unsigned int globalReadOffsetAL_59_0 = globalReadOffsetAL + 0 + 59*LSPA;
  unsigned int globalReadOffsetAL_60_0 = globalReadOffsetAL + 0 + 60*LSPA;
  unsigned int globalReadOffsetAL_61_0 = globalReadOffsetAL + 0 + 61*LSPA;
  unsigned int globalReadOffsetAL_62_0 = globalReadOffsetAL + 0 + 62*LSPA;
  unsigned int globalReadOffsetAL_63_0 = globalReadOffsetAL + 0 + 63*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_15_0), wgK );
  int64_t globalReadOffsetA_0_0_16_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_16_0), wgK );
  int64_t globalReadOffsetA_0_0_17_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_17_0), wgK );
  int64_t globalReadOffsetA_0_0_18_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_18_0), wgK );
  int64_t globalReadOffsetA_0_0_19_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_19_0), wgK );
  int64_t globalReadOffsetA_0_0_20_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_20_0), wgK );
  int64_t globalReadOffsetA_0_0_21_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_21_0), wgK );
  int64_t globalReadOffsetA_0_0_22_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_22_0), wgK );
  int64_t globalReadOffsetA_0_0_23_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_23_0), wgK );
  int64_t globalReadOffsetA_0_0_24_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_24_0), wgK );
  int64_t globalReadOffsetA_0_0_25_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_25_0), wgK );
  int64_t globalReadOffsetA_0_0_26_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_26_0), wgK );
  int64_t globalReadOffsetA_0_0_27_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_27_0), wgK );
  int64_t globalReadOffsetA_0_0_28_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_28_0), wgK );
  int64_t globalReadOffsetA_0_0_29_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_29_0), wgK );
  int64_t globalReadOffsetA_0_0_30_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_30_0), wgK );
  int64_t globalReadOffsetA_0_0_31_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_31_0), wgK );
  int64_t globalReadOffsetA_0_0_32_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_32_0), wgK );
  int64_t globalReadOffsetA_0_0_33_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_33_0), wgK );
  int64_t globalReadOffsetA_0_0_34_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_34_0), wgK );
  int64_t globalReadOffsetA_0_0_35_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_35_0), wgK );
  int64_t globalReadOffsetA_0_0_36_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_36_0), wgK );
  int64_t globalReadOffsetA_0_0_37_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_37_0), wgK );
  int64_t globalReadOffsetA_0_0_38_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_38_0), wgK );
  int64_t globalReadOffsetA_0_0_39_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_39_0), wgK );
  int64_t globalReadOffsetA_0_0_40_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_40_0), wgK );
  int64_t globalReadOffsetA_0_0_41_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_41_0), wgK );
  int64_t globalReadOffsetA_0_0_42_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_42_0), wgK );
  int64_t globalReadOffsetA_0_0_43_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_43_0), wgK );
  int64_t globalReadOffsetA_0_0_44_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_44_0), wgK );
  int64_t globalReadOffsetA_0_0_45_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_45_0), wgK );
  int64_t globalReadOffsetA_0_0_46_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_46_0), wgK );
  int64_t globalReadOffsetA_0_0_47_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_47_0), wgK );
  int64_t globalReadOffsetA_0_0_48_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_48_0), wgK );
  int64_t globalReadOffsetA_0_0_49_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_49_0), wgK );
  int64_t globalReadOffsetA_0_0_50_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_50_0), wgK );
  int64_t globalReadOffsetA_0_0_51_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_51_0), wgK );
  int64_t globalReadOffsetA_0_0_52_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_52_0), wgK );
  int64_t globalReadOffsetA_0_0_53_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_53_0), wgK );
  int64_t globalReadOffsetA_0_0_54_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_54_0), wgK );
  int64_t globalReadOffsetA_0_0_55_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_55_0), wgK );
  int64_t globalReadOffsetA_0_0_56_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_56_0), wgK );
  int64_t globalReadOffsetA_0_0_57_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_57_0), wgK );
  int64_t globalReadOffsetA_0_0_58_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_58_0), wgK );
  int64_t globalReadOffsetA_0_0_59_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_59_0), wgK );
  int64_t globalReadOffsetA_0_0_60_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_60_0), wgK );
  int64_t globalReadOffsetA_0_0_61_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_61_0), wgK );
  int64_t globalReadOffsetA_0_0_62_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_62_0), wgK );
  int64_t globalReadOffsetA_0_0_63_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_63_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;
  DATA_TYPE const *globalReadA_0_0_16_0 = A + globalReadOffsetA_0_0_16_0;
  DATA_TYPE const *globalReadA_0_0_17_0 = A + globalReadOffsetA_0_0_17_0;
  DATA_TYPE const *globalReadA_0_0_18_0 = A + globalReadOffsetA_0_0_18_0;
  DATA_TYPE const *globalReadA_0_0_19_0 = A + globalReadOffsetA_0_0_19_0;
  DATA_TYPE const *globalReadA_0_0_20_0 = A + globalReadOffsetA_0_0_20_0;
  DATA_TYPE const *globalReadA_0_0_21_0 = A + globalReadOffsetA_0_0_21_0;
  DATA_TYPE const *globalReadA_0_0_22_0 = A + globalReadOffsetA_0_0_22_0;
  DATA_TYPE const *globalReadA_0_0_23_0 = A + globalReadOffsetA_0_0_23_0;
  DATA_TYPE const *globalReadA_0_0_24_0 = A + globalReadOffsetA_0_0_24_0;
  DATA_TYPE const *globalReadA_0_0_25_0 = A + globalReadOffsetA_0_0_25_0;
  DATA_TYPE const *globalReadA_0_0_26_0 = A + globalReadOffsetA_0_0_26_0;
  DATA_TYPE const *globalReadA_0_0_27_0 = A + globalReadOffsetA_0_0_27_0;
  DATA_TYPE const *globalReadA_0_0_28_0 = A + globalReadOffsetA_0_0_28_0;
  DATA_TYPE const *globalReadA_0_0_29_0 = A + globalReadOffsetA_0_0_29_0;
  DATA_TYPE const *globalReadA_0_0_30_0 = A + globalReadOffsetA_0_0_30_0;
  DATA_TYPE const *globalReadA_0_0_31_0 = A + globalReadOffsetA_0_0_31_0;
  DATA_TYPE const *globalReadA_0_0_32_0 = A + globalReadOffsetA_0_0_32_0;
  DATA_TYPE const *globalReadA_0_0_33_0 = A + globalReadOffsetA_0_0_33_0;
  DATA_TYPE const *globalReadA_0_0_34_0 = A + globalReadOffsetA_0_0_34_0;
  DATA_TYPE const *globalReadA_0_0_35_0 = A + globalReadOffsetA_0_0_35_0;
  DATA_TYPE const *globalReadA_0_0_36_0 = A + globalReadOffsetA_0_0_36_0;
  DATA_TYPE const *globalReadA_0_0_37_0 = A + globalReadOffsetA_0_0_37_0;
  DATA_TYPE const *globalReadA_0_0_38_0 = A + globalReadOffsetA_0_0_38_0;
  DATA_TYPE const *globalReadA_0_0_39_0 = A + globalReadOffsetA_0_0_39_0;
  DATA_TYPE const *globalReadA_0_0_40_0 = A + globalReadOffsetA_0_0_40_0;
  DATA_TYPE const *globalReadA_0_0_41_0 = A + globalReadOffsetA_0_0_41_0;
  DATA_TYPE const *globalReadA_0_0_42_0 = A + globalReadOffsetA_0_0_42_0;
  DATA_TYPE const *globalReadA_0_0_43_0 = A + globalReadOffsetA_0_0_43_0;
  DATA_TYPE const *globalReadA_0_0_44_0 = A + globalReadOffsetA_0_0_44_0;
  DATA_TYPE const *globalReadA_0_0_45_0 = A + globalReadOffsetA_0_0_45_0;
  DATA_TYPE const *globalReadA_0_0_46_0 = A + globalReadOffsetA_0_0_46_0;
  DATA_TYPE const *globalReadA_0_0_47_0 = A + globalReadOffsetA_0_0_47_0;
  DATA_TYPE const *globalReadA_0_0_48_0 = A + globalReadOffsetA_0_0_48_0;
  DATA_TYPE const *globalReadA_0_0_49_0 = A + globalReadOffsetA_0_0_49_0;
  DATA_TYPE const *globalReadA_0_0_50_0 = A + globalReadOffsetA_0_0_50_0;
  DATA_TYPE const *globalReadA_0_0_51_0 = A + globalReadOffsetA_0_0_51_0;
  DATA_TYPE const *globalReadA_0_0_52_0 = A + globalReadOffsetA_0_0_52_0;
  DATA_TYPE const *globalReadA_0_0_53_0 = A + globalReadOffsetA_0_0_53_0;
  DATA_TYPE const *globalReadA_0_0_54_0 = A + globalReadOffsetA_0_0_54_0;
  DATA_TYPE const *globalReadA_0_0_55_0 = A + globalReadOffsetA_0_0_55_0;
  DATA_TYPE const *globalReadA_0_0_56_0 = A + globalReadOffsetA_0_0_56_0;
  DATA_TYPE const *globalReadA_0_0_57_0 = A + globalReadOffsetA_0_0_57_0;
  DATA_TYPE const *globalReadA_0_0_58_0 = A + globalReadOffsetA_0_0_58_0;
  DATA_TYPE const *globalReadA_0_0_59_0 = A + globalReadOffsetA_0_0_59_0;
  DATA_TYPE const *globalReadA_0_0_60_0 = A + globalReadOffsetA_0_0_60_0;
  DATA_TYPE const *globalReadA_0_0_61_0 = A + globalReadOffsetA_0_0_61_0;
  DATA_TYPE const *globalReadA_0_0_62_0 = A + globalReadOffsetA_0_0_62_0;
  DATA_TYPE const *globalReadA_0_0_63_0 = A + globalReadOffsetA_0_0_63_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 8*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 9*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 10*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 11*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 12*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 13*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 14*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 15*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_16_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 16*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_17_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 17*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_18_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 18*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_19_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 19*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_20_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 20*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_21_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 21*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_22_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 22*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_23_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 23*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_24_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 24*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_25_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 25*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_26_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 26*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_27_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 27*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_28_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 28*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_29_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 29*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_30_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 30*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_31_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 31*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_32_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 32*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_33_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 33*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_34_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 34*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_35_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 35*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_36_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 36*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_37_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 37*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_38_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 38*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_39_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 39*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_40_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 40*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_41_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 41*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_42_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 42*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_43_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 43*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_44_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 44*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_45_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 45*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_46_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 46*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_47_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 47*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_48_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 48*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_49_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 49*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_50_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 50*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_51_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 51*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_52_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 52*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_53_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 53*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_54_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 54*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_55_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 55*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_56_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 56*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_57_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 57*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_58_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 58*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_59_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 59*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_60_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 60*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_61_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 61*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_62_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 62*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_63_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 63*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_0_0_15_0;
  DATA_TYPE *localWriteA_0_0_16_0;
  DATA_TYPE *localWriteA_0_0_17_0;
  DATA_TYPE *localWriteA_0_0_18_0;
  DATA_TYPE *localWriteA_0_0_19_0;
  DATA_TYPE *localWriteA_0_0_20_0;
  DATA_TYPE *localWriteA_0_0_21_0;
  DATA_TYPE *localWriteA_0_0_22_0;
  DATA_TYPE *localWriteA_0_0_23_0;
  DATA_TYPE *localWriteA_0_0_24_0;
  DATA_TYPE *localWriteA_0_0_25_0;
  DATA_TYPE *localWriteA_0_0_26_0;
  DATA_TYPE *localWriteA_0_0_27_0;
  DATA_TYPE *localWriteA_0_0_28_0;
  DATA_TYPE *localWriteA_0_0_29_0;
  DATA_TYPE *localWriteA_0_0_30_0;
  DATA_TYPE *localWriteA_0_0_31_0;
  DATA_TYPE *localWriteA_0_0_32_0;
  DATA_TYPE *localWriteA_0_0_33_0;
  DATA_TYPE *localWriteA_0_0_34_0;
  DATA_TYPE *localWriteA_0_0_35_0;
  DATA_TYPE *localWriteA_0_0_36_0;
  DATA_TYPE *localWriteA_0_0_37_0;
  DATA_TYPE *localWriteA_0_0_38_0;
  DATA_TYPE *localWriteA_0_0_39_0;
  DATA_TYPE *localWriteA_0_0_40_0;
  DATA_TYPE *localWriteA_0_0_41_0;
  DATA_TYPE *localWriteA_0_0_42_0;
  DATA_TYPE *localWriteA_0_0_43_0;
  DATA_TYPE *localWriteA_0_0_44_0;
  DATA_TYPE *localWriteA_0_0_45_0;
  DATA_TYPE *localWriteA_0_0_46_0;
  DATA_TYPE *localWriteA_0_0_47_0;
  DATA_TYPE *localWriteA_0_0_48_0;
  DATA_TYPE *localWriteA_0_0_49_0;
  DATA_TYPE *localWriteA_0_0_50_0;
  DATA_TYPE *localWriteA_0_0_51_0;
  DATA_TYPE *localWriteA_0_0_52_0;
  DATA_TYPE *localWriteA_0_0_53_0;
  DATA_TYPE *localWriteA_0_0_54_0;
  DATA_TYPE *localWriteA_0_0_55_0;
  DATA_TYPE *localWriteA_0_0_56_0;
  DATA_TYPE *localWriteA_0_0_57_0;
  DATA_TYPE *localWriteA_0_0_58_0;
  DATA_TYPE *localWriteA_0_0_59_0;
  DATA_TYPE *localWriteA_0_0_60_0;
  DATA_TYPE *localWriteA_0_0_61_0;
  DATA_TYPE *localWriteA_0_0_62_0;
  DATA_TYPE *localWriteA_0_0_63_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);
  localWriteA_0_0_16_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_16_0);
  localWriteA_0_0_17_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_17_0);
  localWriteA_0_0_18_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_18_0);
  localWriteA_0_0_19_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_19_0);
  localWriteA_0_0_20_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_20_0);
  localWriteA_0_0_21_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_21_0);
  localWriteA_0_0_22_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_22_0);
  localWriteA_0_0_23_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_23_0);
  localWriteA_0_0_24_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_24_0);
  localWriteA_0_0_25_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_25_0);
  localWriteA_0_0_26_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_26_0);
  localWriteA_0_0_27_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_27_0);
  localWriteA_0_0_28_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_28_0);
  localWriteA_0_0_29_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_29_0);
  localWriteA_0_0_30_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_30_0);
  localWriteA_0_0_31_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_31_0);
  localWriteA_0_0_32_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_32_0);
  localWriteA_0_0_33_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_33_0);
  localWriteA_0_0_34_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_34_0);
  localWriteA_0_0_35_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_35_0);
  localWriteA_0_0_36_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_36_0);
  localWriteA_0_0_37_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_37_0);
  localWriteA_0_0_38_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_38_0);
  localWriteA_0_0_39_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_39_0);
  localWriteA_0_0_40_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_40_0);
  localWriteA_0_0_41_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_41_0);
  localWriteA_0_0_42_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_42_0);
  localWriteA_0_0_43_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_43_0);
  localWriteA_0_0_44_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_44_0);
  localWriteA_0_0_45_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_45_0);
  localWriteA_0_0_46_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_46_0);
  localWriteA_0_0_47_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_47_0);
  localWriteA_0_0_48_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_48_0);
  localWriteA_0_0_49_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_49_0);
  localWriteA_0_0_50_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_50_0);
  localWriteA_0_0_51_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_51_0);
  localWriteA_0_0_52_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_52_0);
  localWriteA_0_0_53_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_53_0);
  localWriteA_0_0_54_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_54_0);
  localWriteA_0_0_55_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_55_0);
  localWriteA_0_0_56_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_56_0);
  localWriteA_0_0_57_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_57_0);
  localWriteA_0_0_58_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_58_0);
  localWriteA_0_0_59_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_59_0);
  localWriteA_0_0_60_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_60_0);
  localWriteA_0_0_61_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_61_0);
  localWriteA_0_0_62_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_62_0);
  localWriteA_0_0_63_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_63_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);
    a_0_0_16_0 = *(globalReadA_0_0_16_0 + 0);
    a_0_0_17_0 = *(globalReadA_0_0_17_0 + 0);
    a_0_0_18_0 = *(globalReadA_0_0_18_0 + 0);
    a_0_0_19_0 = *(globalReadA_0_0_19_0 + 0);
    a_0_0_20_0 = *(globalReadA_0_0_20_0 + 0);
    a_0_0_21_0 = *(globalReadA_0_0_21_0 + 0);
    a_0_0_22_0 = *(globalReadA_0_0_22_0 + 0);
    a_0_0_23_0 = *(globalReadA_0_0_23_0 + 0);
    a_0_0_24_0 = *(globalReadA_0_0_24_0 + 0);
    a_0_0_25_0 = *(globalReadA_0_0_25_0 + 0);
    a_0_0_26_0 = *(globalReadA_0_0_26_0 + 0);
    a_0_0_27_0 = *(globalReadA_0_0_27_0 + 0);
    a_0_0_28_0 = *(globalReadA_0_0_28_0 + 0);
    a_0_0_29_0 = *(globalReadA_0_0_29_0 + 0);
    a_0_0_30_0 = *(globalReadA_0_0_30_0 + 0);
    a_0_0_31_0 = *(globalReadA_0_0_31_0 + 0);
    a_0_0_32_0 = *(globalReadA_0_0_32_0 + 0);
    a_0_0_33_0 = *(globalReadA_0_0_33_0 + 0);
    a_0_0_34_0 = *(globalReadA_0_0_34_0 + 0);
    a_0_0_35_0 = *(globalReadA_0_0_35_0 + 0);
    a_0_0_36_0 = *(globalReadA_0_0_36_0 + 0);
    a_0_0_37_0 = *(globalReadA_0_0_37_0 + 0);
    a_0_0_38_0 = *(globalReadA_0_0_38_0 + 0);
    a_0_0_39_0 = *(globalReadA_0_0_39_0 + 0);
    a_0_0_40_0 = *(globalReadA_0_0_40_0 + 0);
    a_0_0_41_0 = *(globalReadA_0_0_41_0 + 0);
    a_0_0_42_0 = *(globalReadA_0_0_42_0 + 0);
    a_0_0_43_0 = *(globalReadA_0_0_43_0 + 0);
    a_0_0_44_0 = *(globalReadA_0_0_44_0 + 0);
    a_0_0_45_0 = *(globalReadA_0_0_45_0 + 0);
    a_0_0_46_0 = *(globalReadA_0_0_46_0 + 0);
    a_0_0_47_0 = *(globalReadA_0_0_47_0 + 0);
    a_0_0_48_0 = *(globalReadA_0_0_48_0 + 0);
    a_0_0_49_0 = *(globalReadA_0_0_49_0 + 0);
    a_0_0_50_0 = *(globalReadA_0_0_50_0 + 0);
    a_0_0_51_0 = *(globalReadA_0_0_51_0 + 0);
    a_0_0_52_0 = *(globalReadA_0_0_52_0 + 0);
    a_0_0_53_0 = *(globalReadA_0_0_53_0 + 0);
    a_0_0_54_0 = *(globalReadA_0_0_54_0 + 0);
    a_0_0_55_0 = *(globalReadA_0_0_55_0 + 0);
    a_0_0_56_0 = *(globalReadA_0_0_56_0 + 0);
    a_0_0_57_0 = *(globalReadA_0_0_57_0 + 0);
    a_0_0_58_0 = *(globalReadA_0_0_58_0 + 0);
    a_0_0_59_0 = *(globalReadA_0_0_59_0 + 0);
    a_0_0_60_0 = *(globalReadA_0_0_60_0 + 0);
    a_0_0_61_0 = *(globalReadA_0_0_61_0 + 0);
    a_0_0_62_0 = *(globalReadA_0_0_62_0 + 0);
    a_0_0_63_0 = *(globalReadA_0_0_63_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    globalReadA_0_0_16_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_16_0) + 1*globalReadIncAL);
    globalReadA_0_0_17_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_17_0) + 1*globalReadIncAL);
    globalReadA_0_0_18_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_18_0) + 1*globalReadIncAL);
    globalReadA_0_0_19_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_19_0) + 1*globalReadIncAL);
    globalReadA_0_0_20_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_20_0) + 1*globalReadIncAL);
    globalReadA_0_0_21_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_21_0) + 1*globalReadIncAL);
    globalReadA_0_0_22_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_22_0) + 1*globalReadIncAL);
    globalReadA_0_0_23_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_23_0) + 1*globalReadIncAL);
    globalReadA_0_0_24_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_24_0) + 1*globalReadIncAL);
    globalReadA_0_0_25_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_25_0) + 1*globalReadIncAL);
    globalReadA_0_0_26_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_26_0) + 1*globalReadIncAL);
    globalReadA_0_0_27_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_27_0) + 1*globalReadIncAL);
    globalReadA_0_0_28_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_28_0) + 1*globalReadIncAL);
    globalReadA_0_0_29_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_29_0) + 1*globalReadIncAL);
    globalReadA_0_0_30_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_30_0) + 1*globalReadIncAL);
    globalReadA_0_0_31_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_31_0) + 1*globalReadIncAL);
    globalReadA_0_0_32_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_32_0) + 1*globalReadIncAL);
    globalReadA_0_0_33_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_33_0) + 1*globalReadIncAL);
    globalReadA_0_0_34_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_34_0) + 1*globalReadIncAL);
    globalReadA_0_0_35_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_35_0) + 1*globalReadIncAL);
    globalReadA_0_0_36_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_36_0) + 1*globalReadIncAL);
    globalReadA_0_0_37_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_37_0) + 1*globalReadIncAL);
    globalReadA_0_0_38_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_38_0) + 1*globalReadIncAL);
    globalReadA_0_0_39_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_39_0) + 1*globalReadIncAL);
    globalReadA_0_0_40_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_40_0) + 1*globalReadIncAL);
    globalReadA_0_0_41_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_41_0) + 1*globalReadIncAL);
    globalReadA_0_0_42_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_42_0) + 1*globalReadIncAL);
    globalReadA_0_0_43_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_43_0) + 1*globalReadIncAL);
    globalReadA_0_0_44_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_44_0) + 1*globalReadIncAL);
    globalReadA_0_0_45_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_45_0) + 1*globalReadIncAL);
    globalReadA_0_0_46_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_46_0) + 1*globalReadIncAL);
    globalReadA_0_0_47_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_47_0) + 1*globalReadIncAL);
    globalReadA_0_0_48_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_48_0) + 1*globalReadIncAL);
    globalReadA_0_0_49_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_49_0) + 1*globalReadIncAL);
    globalReadA_0_0_50_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_50_0) + 1*globalReadIncAL);
    globalReadA_0_0_51_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_51_0) + 1*globalReadIncAL);
    globalReadA_0_0_52_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_52_0) + 1*globalReadIncAL);
    globalReadA_0_0_53_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_53_0) + 1*globalReadIncAL);
    globalReadA_0_0_54_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_54_0) + 1*globalReadIncAL);
    globalReadA_0_0_55_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_55_0) + 1*globalReadIncAL);
    globalReadA_0_0_56_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_56_0) + 1*globalReadIncAL);
    globalReadA_0_0_57_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_57_0) + 1*globalReadIncAL);
    globalReadA_0_0_58_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_58_0) + 1*globalReadIncAL);
    globalReadA_0_0_59_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_59_0) + 1*globalReadIncAL);
    globalReadA_0_0_60_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_60_0) + 1*globalReadIncAL);
    globalReadA_0_0_61_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_61_0) + 1*globalReadIncAL);
    globalReadA_0_0_62_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_62_0) + 1*globalReadIncAL);
    globalReadA_0_0_63_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_63_0) + 1*globalReadIncAL);

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
    *(localWriteA_0_0_16_0 + 0) = a_0_0_16_0;
    *(localWriteA_0_0_17_0 + 0) = a_0_0_17_0;
    *(localWriteA_0_0_18_0 + 0) = a_0_0_18_0;
    *(localWriteA_0_0_19_0 + 0) = a_0_0_19_0;
    *(localWriteA_0_0_20_0 + 0) = a_0_0_20_0;
    *(localWriteA_0_0_21_0 + 0) = a_0_0_21_0;
    *(localWriteA_0_0_22_0 + 0) = a_0_0_22_0;
    *(localWriteA_0_0_23_0 + 0) = a_0_0_23_0;
    *(localWriteA_0_0_24_0 + 0) = a_0_0_24_0;
    *(localWriteA_0_0_25_0 + 0) = a_0_0_25_0;
    *(localWriteA_0_0_26_0 + 0) = a_0_0_26_0;
    *(localWriteA_0_0_27_0 + 0) = a_0_0_27_0;
    *(localWriteA_0_0_28_0 + 0) = a_0_0_28_0;
    *(localWriteA_0_0_29_0 + 0) = a_0_0_29_0;
    *(localWriteA_0_0_30_0 + 0) = a_0_0_30_0;
    *(localWriteA_0_0_31_0 + 0) = a_0_0_31_0;
    *(localWriteA_0_0_32_0 + 0) = a_0_0_32_0;
    *(localWriteA_0_0_33_0 + 0) = a_0_0_33_0;
    *(localWriteA_0_0_34_0 + 0) = a_0_0_34_0;
    *(localWriteA_0_0_35_0 + 0) = a_0_0_35_0;
    *(localWriteA_0_0_36_0 + 0) = a_0_0_36_0;
    *(localWriteA_0_0_37_0 + 0) = a_0_0_37_0;
    *(localWriteA_0_0_38_0 + 0) = a_0_0_38_0;
    *(localWriteA_0_0_39_0 + 0) = a_0_0_39_0;
    *(localWriteA_0_0_40_0 + 0) = a_0_0_40_0;
    *(localWriteA_0_0_41_0 + 0) = a_0_0_41_0;
    *(localWriteA_0_0_42_0 + 0) = a_0_0_42_0;
    *(localWriteA_0_0_43_0 + 0) = a_0_0_43_0;
    *(localWriteA_0_0_44_0 + 0) = a_0_0_44_0;
    *(localWriteA_0_0_45_0 + 0) = a_0_0_45_0;
    *(localWriteA_0_0_46_0 + 0) = a_0_0_46_0;
    *(localWriteA_0_0_47_0 + 0) = a_0_0_47_0;
    *(localWriteA_0_0_48_0 + 0) = a_0_0_48_0;
    *(localWriteA_0_0_49_0 + 0) = a_0_0_49_0;
    *(localWriteA_0_0_50_0 + 0) = a_0_0_50_0;
    *(localWriteA_0_0_51_0 + 0) = a_0_0_51_0;
    *(localWriteA_0_0_52_0 + 0) = a_0_0_52_0;
    *(localWriteA_0_0_53_0 + 0) = a_0_0_53_0;
    *(localWriteA_0_0_54_0 + 0) = a_0_0_54_0;
    *(localWriteA_0_0_55_0 + 0) = a_0_0_55_0;
    *(localWriteA_0_0_56_0 + 0) = a_0_0_56_0;
    *(localWriteA_0_0_57_0 + 0) = a_0_0_57_0;
    *(localWriteA_0_0_58_0 + 0) = a_0_0_58_0;
    *(localWriteA_0_0_59_0 + 0) = a_0_0_59_0;
    *(localWriteA_0_0_60_0 + 0) = a_0_0_60_0;
    *(localWriteA_0_0_61_0 + 0) = a_0_0_61_0;
    *(localWriteA_0_0_62_0 + 0) = a_0_0_62_0;
    *(localWriteA_0_0_63_0 + 0) = a_0_0_63_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 15 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 16 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 17 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 18 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 19 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 20 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 21 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 22 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 23 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 24 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 25 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 26 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 27 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 28 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 29 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 30 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 31 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 32 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 33 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 34 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 35 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 36 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 37 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 38 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 39 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 40 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 41 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 42 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 43 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 44 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 45 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 46 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 47 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 48 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 49 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 50 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 51 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 52 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 53 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 54 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 55 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 56 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 57 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 58 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 59 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 60 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 61 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 62 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 63 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_1x1




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_0_8_0 = ( globalReadOffsetAL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_0_0_9_0 = ( globalReadOffsetAL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_0_0_10_0 = ( globalReadOffsetAL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_0_0_11_0 = ( globalReadOffsetAL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_0_0_12_0 = ( globalReadOffsetAL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_0_0_13_0 = ( globalReadOffsetAL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_0_0_14_0 = ( globalReadOffsetAL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_0_0_15_0 = ( globalReadOffsetAL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);
  a_0_0_16_0 = ( globalReadOffsetAL_16_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_16_0 + 0);
  a_0_0_17_0 = ( globalReadOffsetAL_17_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_17_0 + 0);
  a_0_0_18_0 = ( globalReadOffsetAL_18_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_18_0 + 0);
  a_0_0_19_0 = ( globalReadOffsetAL_19_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_19_0 + 0);
  a_0_0_20_0 = ( globalReadOffsetAL_20_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_20_0 + 0);
  a_0_0_21_0 = ( globalReadOffsetAL_21_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_21_0 + 0);
  a_0_0_22_0 = ( globalReadOffsetAL_22_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_22_0 + 0);
  a_0_0_23_0 = ( globalReadOffsetAL_23_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_23_0 + 0);
  a_0_0_24_0 = ( globalReadOffsetAL_24_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_24_0 + 0);
  a_0_0_25_0 = ( globalReadOffsetAL_25_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_25_0 + 0);
  a_0_0_26_0 = ( globalReadOffsetAL_26_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_26_0 + 0);
  a_0_0_27_0 = ( globalReadOffsetAL_27_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_27_0 + 0);
  a_0_0_28_0 = ( globalReadOffsetAL_28_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_28_0 + 0);
  a_0_0_29_0 = ( globalReadOffsetAL_29_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_29_0 + 0);
  a_0_0_30_0 = ( globalReadOffsetAL_30_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_30_0 + 0);
  a_0_0_31_0 = ( globalReadOffsetAL_31_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_31_0 + 0);
  a_0_0_32_0 = ( globalReadOffsetAL_32_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_32_0 + 0);
  a_0_0_33_0 = ( globalReadOffsetAL_33_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_33_0 + 0);
  a_0_0_34_0 = ( globalReadOffsetAL_34_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_34_0 + 0);
  a_0_0_35_0 = ( globalReadOffsetAL_35_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_35_0 + 0);
  a_0_0_36_0 = ( globalReadOffsetAL_36_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_36_0 + 0);
  a_0_0_37_0 = ( globalReadOffsetAL_37_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_37_0 + 0);
  a_0_0_38_0 = ( globalReadOffsetAL_38_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_38_0 + 0);
  a_0_0_39_0 = ( globalReadOffsetAL_39_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_39_0 + 0);
  a_0_0_40_0 = ( globalReadOffsetAL_40_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_40_0 + 0);
  a_0_0_41_0 = ( globalReadOffsetAL_41_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_41_0 + 0);
  a_0_0_42_0 = ( globalReadOffsetAL_42_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_42_0 + 0);
  a_0_0_43_0 = ( globalReadOffsetAL_43_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_43_0 + 0);
  a_0_0_44_0 = ( globalReadOffsetAL_44_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_44_0 + 0);
  a_0_0_45_0 = ( globalReadOffsetAL_45_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_45_0 + 0);
  a_0_0_46_0 = ( globalReadOffsetAL_46_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_46_0 + 0);
  a_0_0_47_0 = ( globalReadOffsetAL_47_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_47_0 + 0);
  a_0_0_48_0 = ( globalReadOffsetAL_48_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_48_0 + 0);
  a_0_0_49_0 = ( globalReadOffsetAL_49_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_49_0 + 0);
  a_0_0_50_0 = ( globalReadOffsetAL_50_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_50_0 + 0);
  a_0_0_51_0 = ( globalReadOffsetAL_51_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_51_0 + 0);
  a_0_0_52_0 = ( globalReadOffsetAL_52_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_52_0 + 0);
  a_0_0_53_0 = ( globalReadOffsetAL_53_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_53_0 + 0);
  a_0_0_54_0 = ( globalReadOffsetAL_54_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_54_0 + 0);
  a_0_0_55_0 = ( globalReadOffsetAL_55_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_55_0 + 0);
  a_0_0_56_0 = ( globalReadOffsetAL_56_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_56_0 + 0);
  a_0_0_57_0 = ( globalReadOffsetAL_57_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_57_0 + 0);
  a_0_0_58_0 = ( globalReadOffsetAL_58_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_58_0 + 0);
  a_0_0_59_0 = ( globalReadOffsetAL_59_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_59_0 + 0);
  a_0_0_60_0 = ( globalReadOffsetAL_60_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_60_0 + 0);
  a_0_0_61_0 = ( globalReadOffsetAL_61_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_61_0 + 0);
  a_0_0_62_0 = ( globalReadOffsetAL_62_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_62_0 + 0);
  a_0_0_63_0 = ( globalReadOffsetAL_63_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_63_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);
  localWriteA_0_0_16_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_16_0);
  localWriteA_0_0_17_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_17_0);
  localWriteA_0_0_18_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_18_0);
  localWriteA_0_0_19_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_19_0);
  localWriteA_0_0_20_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_20_0);
  localWriteA_0_0_21_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_21_0);
  localWriteA_0_0_22_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_22_0);
  localWriteA_0_0_23_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_23_0);
  localWriteA_0_0_24_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_24_0);
  localWriteA_0_0_25_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_25_0);
  localWriteA_0_0_26_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_26_0);
  localWriteA_0_0_27_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_27_0);
  localWriteA_0_0_28_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_28_0);
  localWriteA_0_0_29_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_29_0);
  localWriteA_0_0_30_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_30_0);
  localWriteA_0_0_31_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_31_0);
  localWriteA_0_0_32_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_32_0);
  localWriteA_0_0_33_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_33_0);
  localWriteA_0_0_34_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_34_0);
  localWriteA_0_0_35_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_35_0);
  localWriteA_0_0_36_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_36_0);
  localWriteA_0_0_37_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_37_0);
  localWriteA_0_0_38_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_38_0);
  localWriteA_0_0_39_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_39_0);
  localWriteA_0_0_40_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_40_0);
  localWriteA_0_0_41_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_41_0);
  localWriteA_0_0_42_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_42_0);
  localWriteA_0_0_43_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_43_0);
  localWriteA_0_0_44_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_44_0);
  localWriteA_0_0_45_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_45_0);
  localWriteA_0_0_46_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_46_0);
  localWriteA_0_0_47_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_47_0);
  localWriteA_0_0_48_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_48_0);
  localWriteA_0_0_49_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_49_0);
  localWriteA_0_0_50_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_50_0);
  localWriteA_0_0_51_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_51_0);
  localWriteA_0_0_52_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_52_0);
  localWriteA_0_0_53_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_53_0);
  localWriteA_0_0_54_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_54_0);
  localWriteA_0_0_55_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_55_0);
  localWriteA_0_0_56_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_56_0);
  localWriteA_0_0_57_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_57_0);
  localWriteA_0_0_58_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_58_0);
  localWriteA_0_0_59_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_59_0);
  localWriteA_0_0_60_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_60_0);
  localWriteA_0_0_61_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_61_0);
  localWriteA_0_0_62_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_62_0);
  localWriteA_0_0_63_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_63_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
  *(localWriteA_0_0_16_0 + 0) = a_0_0_16_0;
  *(localWriteA_0_0_17_0 + 0) = a_0_0_17_0;
  *(localWriteA_0_0_18_0 + 0) = a_0_0_18_0;
  *(localWriteA_0_0_19_0 + 0) = a_0_0_19_0;
  *(localWriteA_0_0_20_0 + 0) = a_0_0_20_0;
  *(localWriteA_0_0_21_0 + 0) = a_0_0_21_0;
  *(localWriteA_0_0_22_0 + 0) = a_0_0_22_0;
  *(localWriteA_0_0_23_0 + 0) = a_0_0_23_0;
  *(localWriteA_0_0_24_0 + 0) = a_0_0_24_0;
  *(localWriteA_0_0_25_0 + 0) = a_0_0_25_0;
  *(localWriteA_0_0_26_0 + 0) = a_0_0_26_0;
  *(localWriteA_0_0_27_0 + 0) = a_0_0_27_0;
  *(localWriteA_0_0_28_0 + 0) = a_0_0_28_0;
  *(localWriteA_0_0_29_0 + 0) = a_0_0_29_0;
  *(localWriteA_0_0_30_0 + 0) = a_0_0_30_0;
  *(localWriteA_0_0_31_0 + 0) = a_0_0_31_0;
  *(localWriteA_0_0_32_0 + 0) = a_0_0_32_0;
  *(localWriteA_0_0_33_0 + 0) = a_0_0_33_0;
  *(localWriteA_0_0_34_0 + 0) = a_0_0_34_0;
  *(localWriteA_0_0_35_0 + 0) = a_0_0_35_0;
  *(localWriteA_0_0_36_0 + 0) = a_0_0_36_0;
  *(localWriteA_0_0_37_0 + 0) = a_0_0_37_0;
  *(localWriteA_0_0_38_0 + 0) = a_0_0_38_0;
  *(localWriteA_0_0_39_0 + 0) = a_0_0_39_0;
  *(localWriteA_0_0_40_0 + 0) = a_0_0_40_0;
  *(localWriteA_0_0_41_0 + 0) = a_0_0_41_0;
  *(localWriteA_0_0_42_0 + 0) = a_0_0_42_0;
  *(localWriteA_0_0_43_0 + 0) = a_0_0_43_0;
  *(localWriteA_0_0_44_0 + 0) = a_0_0_44_0;
  *(localWriteA_0_0_45_0 + 0) = a_0_0_45_0;
  *(localWriteA_0_0_46_0 + 0) = a_0_0_46_0;
  *(localWriteA_0_0_47_0 + 0) = a_0_0_47_0;
  *(localWriteA_0_0_48_0 + 0) = a_0_0_48_0;
  *(localWriteA_0_0_49_0 + 0) = a_0_0_49_0;
  *(localWriteA_0_0_50_0 + 0) = a_0_0_50_0;
  *(localWriteA_0_0_51_0 + 0) = a_0_0_51_0;
  *(localWriteA_0_0_52_0 + 0) = a_0_0_52_0;
  *(localWriteA_0_0_53_0 + 0) = a_0_0_53_0;
  *(localWriteA_0_0_54_0 + 0) = a_0_0_54_0;
  *(localWriteA_0_0_55_0 + 0) = a_0_0_55_0;
  *(localWriteA_0_0_56_0 + 0) = a_0_0_56_0;
  *(localWriteA_0_0_57_0 + 0) = a_0_0_57_0;
  *(localWriteA_0_0_58_0 + 0) = a_0_0_58_0;
  *(localWriteA_0_0_59_0 + 0) = a_0_0_59_0;
  *(localWriteA_0_0_60_0 + 0) = a_0_0_60_0;
  *(localWriteA_0_0_61_0 + 0) = a_0_0_61_0;
  *(localWriteA_0_0_62_0 + 0) = a_0_0_62_0;
  *(localWriteA_0_0_63_0 + 0) = a_0_0_63_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_1x1

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_1x1
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 64
#define SG1J 1
#define TT0I 1
#define TT1J 1
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 64
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 64
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 4096
#define LDS_NUM_ELEMENTS 4160

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 1x1 micro-tile */
#define MAC_1x1\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_BSS_BH_MT64x1x64_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU0_SUM0_SUS128_SRVW0_SVW4_SNLL0_TT1_1_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG64_1_1_WGM1(
  float *D,
  float const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_bfloat16)(0)
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE a_0_0_16_0;
  DATA_TYPE a_0_0_17_0;
  DATA_TYPE a_0_0_18_0;
  DATA_TYPE a_0_0_19_0;
  DATA_TYPE a_0_0_20_0;
  DATA_TYPE a_0_0_21_0;
  DATA_TYPE a_0_0_22_0;
  DATA_TYPE a_0_0_23_0;
  DATA_TYPE a_0_0_24_0;
  DATA_TYPE a_0_0_25_0;
  DATA_TYPE a_0_0_26_0;
  DATA_TYPE a_0_0_27_0;
  DATA_TYPE a_0_0_28_0;
  DATA_TYPE a_0_0_29_0;
  DATA_TYPE a_0_0_30_0;
  DATA_TYPE a_0_0_31_0;
  DATA_TYPE a_0_0_32_0;
  DATA_TYPE a_0_0_33_0;
  DATA_TYPE a_0_0_34_0;
  DATA_TYPE a_0_0_35_0;
  DATA_TYPE a_0_0_36_0;
  DATA_TYPE a_0_0_37_0;
  DATA_TYPE a_0_0_38_0;
  DATA_TYPE a_0_0_39_0;
  DATA_TYPE a_0_0_40_0;
  DATA_TYPE a_0_0_41_0;
  DATA_TYPE a_0_0_42_0;
  DATA_TYPE a_0_0_43_0;
  DATA_TYPE a_0_0_44_0;
  DATA_TYPE a_0_0_45_0;
  DATA_TYPE a_0_0_46_0;
  DATA_TYPE a_0_0_47_0;
  DATA_TYPE a_0_0_48_0;
  DATA_TYPE a_0_0_49_0;
  DATA_TYPE a_0_0_50_0;
  DATA_TYPE a_0_0_51_0;
  DATA_TYPE a_0_0_52_0;
  DATA_TYPE a_0_0_53_0;
  DATA_TYPE a_0_0_54_0;
  DATA_TYPE a_0_0_55_0;
  DATA_TYPE a_0_0_56_0;
  DATA_TYPE a_0_0_57_0;
  DATA_TYPE a_0_0_58_0;
  DATA_TYPE a_0_0_59_0;
  DATA_TYPE a_0_0_60_0;
  DATA_TYPE a_0_0_61_0;
  DATA_TYPE a_0_0_62_0;
  DATA_TYPE a_0_0_63_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;
  unsigned int globalReadOffsetAL_8_0 = globalReadOffsetAL + 0 + 8*LSPA;
  unsigned int globalReadOffsetAL_9_0 = globalReadOffsetAL + 0 + 9*LSPA;
  unsigned int globalReadOffsetAL_10_0 = globalReadOffsetAL + 0 + 10*LSPA;
  unsigned int globalReadOffsetAL_11_0 = globalReadOffsetAL + 0 + 11*LSPA;
  unsigned int globalReadOffsetAL_12_0 = globalReadOffsetAL + 0 + 12*LSPA;
  unsigned int globalReadOffsetAL_13_0 = globalReadOffsetAL + 0 + 13*LSPA;
  unsigned int globalReadOffsetAL_14_0 = globalReadOffsetAL + 0 + 14*LSPA;
  unsigned int globalReadOffsetAL_15_0 = globalReadOffsetAL + 0 + 15*LSPA;
  unsigned int globalReadOffsetAL_16_0 = globalReadOffsetAL + 0 + 16*LSPA;
  unsigned int globalReadOffsetAL_17_0 = globalReadOffsetAL + 0 + 17*LSPA;
  unsigned int globalReadOffsetAL_18_0 = globalReadOffsetAL + 0 + 18*LSPA;
  unsigned int globalReadOffsetAL_19_0 = globalReadOffsetAL + 0 + 19*LSPA;
  unsigned int globalReadOffsetAL_20_0 = globalReadOffsetAL + 0 + 20*LSPA;
  unsigned int globalReadOffsetAL_21_0 = globalReadOffsetAL + 0 + 21*LSPA;
  unsigned int globalReadOffsetAL_22_0 = globalReadOffsetAL + 0 + 22*LSPA;
  unsigned int globalReadOffsetAL_23_0 = globalReadOffsetAL + 0 + 23*LSPA;
  unsigned int globalReadOffsetAL_24_0 = globalReadOffsetAL + 0 + 24*LSPA;
  unsigned int globalReadOffsetAL_25_0 = globalReadOffsetAL + 0 + 25*LSPA;
  unsigned int globalReadOffsetAL_26_0 = globalReadOffsetAL + 0 + 26*LSPA;
  unsigned int globalReadOffsetAL_27_0 = globalReadOffsetAL + 0 + 27*LSPA;
  unsigned int globalReadOffsetAL_28_0 = globalReadOffsetAL + 0 + 28*LSPA;
  unsigned int globalReadOffsetAL_29_0 = globalReadOffsetAL + 0 + 29*LSPA;
  unsigned int globalReadOffsetAL_30_0 = globalReadOffsetAL + 0 + 30*LSPA;
  unsigned int globalReadOffsetAL_31_0 = globalReadOffsetAL + 0 + 31*LSPA;
  unsigned int globalReadOffsetAL_32_0 = globalReadOffsetAL + 0 + 32*LSPA;
  unsigned int globalReadOffsetAL_33_0 = globalReadOffsetAL + 0 + 33*LSPA;
  unsigned int globalReadOffsetAL_34_0 = globalReadOffsetAL + 0 + 34*LSPA;
  unsigned int globalReadOffsetAL_35_0 = globalReadOffsetAL + 0 + 35*LSPA;
  unsigned int globalReadOffsetAL_36_0 = globalReadOffsetAL + 0 + 36*LSPA;
  unsigned int globalReadOffsetAL_37_0 = globalReadOffsetAL + 0 + 37*LSPA;
  unsigned int globalReadOffsetAL_38_0 = globalReadOffsetAL + 0 + 38*LSPA;
  unsigned int globalReadOffsetAL_39_0 = globalReadOffsetAL + 0 + 39*LSPA;
  unsigned int globalReadOffsetAL_40_0 = globalReadOffsetAL + 0 + 40*LSPA;
  unsigned int globalReadOffsetAL_41_0 = globalReadOffsetAL + 0 + 41*LSPA;
  unsigned int globalReadOffsetAL_42_0 = globalReadOffsetAL + 0 + 42*LSPA;
  unsigned int globalReadOffsetAL_43_0 = globalReadOffsetAL + 0 + 43*LSPA;
  unsigned int globalReadOffsetAL_44_0 = globalReadOffsetAL + 0 + 44*LSPA;
  unsigned int globalReadOffsetAL_45_0 = globalReadOffsetAL + 0 + 45*LSPA;
  unsigned int globalReadOffsetAL_46_0 = globalReadOffsetAL + 0 + 46*LSPA;
  unsigned int globalReadOffsetAL_47_0 = globalReadOffsetAL + 0 + 47*LSPA;
  unsigned int globalReadOffsetAL_48_0 = globalReadOffsetAL + 0 + 48*LSPA;
  unsigned int globalReadOffsetAL_49_0 = globalReadOffsetAL + 0 + 49*LSPA;
  unsigned int globalReadOffsetAL_50_0 = globalReadOffsetAL + 0 + 50*LSPA;
  unsigned int globalReadOffsetAL_51_0 = globalReadOffsetAL + 0 + 51*LSPA;
  unsigned int globalReadOffsetAL_52_0 = globalReadOffsetAL + 0 + 52*LSPA;
  unsigned int globalReadOffsetAL_53_0 = globalReadOffsetAL + 0 + 53*LSPA;
  unsigned int globalReadOffsetAL_54_0 = globalReadOffsetAL + 0 + 54*LSPA;
  unsigned int globalReadOffsetAL_55_0 = globalReadOffsetAL + 0 + 55*LSPA;
  unsigned int globalReadOffsetAL_56_0 = globalReadOffsetAL + 0 + 56*LSPA;
  unsigned int globalReadOffsetAL_57_0 = globalReadOffsetAL + 0 + 57*LSPA;
  unsigned int globalReadOffsetAL_58_0 = globalReadOffsetAL + 0 + 58*LSPA;
  unsigned int globalReadOffsetAL_59_0 = globalReadOffsetAL + 0 + 59*LSPA;
  unsigned int globalReadOffsetAL_60_0 = globalReadOffsetAL + 0 + 60*LSPA;
  unsigned int globalReadOffsetAL_61_0 = globalReadOffsetAL + 0 + 61*LSPA;
  unsigned int globalReadOffsetAL_62_0 = globalReadOffsetAL + 0 + 62*LSPA;
  unsigned int globalReadOffsetAL_63_0 = globalReadOffsetAL + 0 + 63*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_15_0), wgK );
  int64_t globalReadOffsetA_0_0_16_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_16_0), wgK );
  int64_t globalReadOffsetA_0_0_17_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_17_0), wgK );
  int64_t globalReadOffsetA_0_0_18_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_18_0), wgK );
  int64_t globalReadOffsetA_0_0_19_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_19_0), wgK );
  int64_t globalReadOffsetA_0_0_20_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_20_0), wgK );
  int64_t globalReadOffsetA_0_0_21_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_21_0), wgK );
  int64_t globalReadOffsetA_0_0_22_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_22_0), wgK );
  int64_t globalReadOffsetA_0_0_23_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_23_0), wgK );
  int64_t globalReadOffsetA_0_0_24_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_24_0), wgK );
  int64_t globalReadOffsetA_0_0_25_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_25_0), wgK );
  int64_t globalReadOffsetA_0_0_26_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_26_0), wgK );
  int64_t globalReadOffsetA_0_0_27_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_27_0), wgK );
  int64_t globalReadOffsetA_0_0_28_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_28_0), wgK );
  int64_t globalReadOffsetA_0_0_29_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_29_0), wgK );
  int64_t globalReadOffsetA_0_0_30_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_30_0), wgK );
  int64_t globalReadOffsetA_0_0_31_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_31_0), wgK );
  int64_t globalReadOffsetA_0_0_32_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_32_0), wgK );
  int64_t globalReadOffsetA_0_0_33_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_33_0), wgK );
  int64_t globalReadOffsetA_0_0_34_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_34_0), wgK );
  int64_t globalReadOffsetA_0_0_35_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_35_0), wgK );
  int64_t globalReadOffsetA_0_0_36_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_36_0), wgK );
  int64_t globalReadOffsetA_0_0_37_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_37_0), wgK );
  int64_t globalReadOffsetA_0_0_38_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_38_0), wgK );
  int64_t globalReadOffsetA_0_0_39_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_39_0), wgK );
  int64_t globalReadOffsetA_0_0_40_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_40_0), wgK );
  int64_t globalReadOffsetA_0_0_41_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_41_0), wgK );
  int64_t globalReadOffsetA_0_0_42_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_42_0), wgK );
  int64_t globalReadOffsetA_0_0_43_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_43_0), wgK );
  int64_t globalReadOffsetA_0_0_44_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_44_0), wgK );
  int64_t globalReadOffsetA_0_0_45_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_45_0), wgK );
  int64_t globalReadOffsetA_0_0_46_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_46_0), wgK );
  int64_t globalReadOffsetA_0_0_47_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_47_0), wgK );
  int64_t globalReadOffsetA_0_0_48_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_48_0), wgK );
  int64_t globalReadOffsetA_0_0_49_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_49_0), wgK );
  int64_t globalReadOffsetA_0_0_50_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_50_0), wgK );
  int64_t globalReadOffsetA_0_0_51_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_51_0), wgK );
  int64_t globalReadOffsetA_0_0_52_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_52_0), wgK );
  int64_t globalReadOffsetA_0_0_53_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_53_0), wgK );
  int64_t globalReadOffsetA_0_0_54_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_54_0), wgK );
  int64_t globalReadOffsetA_0_0_55_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_55_0), wgK );
  int64_t globalReadOffsetA_0_0_56_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_56_0), wgK );
  int64_t globalReadOffsetA_0_0_57_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_57_0), wgK );
  int64_t globalReadOffsetA_0_0_58_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_58_0), wgK );
  int64_t globalReadOffsetA_0_0_59_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_59_0), wgK );
  int64_t globalReadOffsetA_0_0_60_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_60_0), wgK );
  int64_t globalReadOffsetA_0_0_61_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_61_0), wgK );
  int64_t globalReadOffsetA_0_0_62_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_62_0), wgK );
  int64_t globalReadOffsetA_0_0_63_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_63_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;
  DATA_TYPE const *globalReadA_0_0_16_0 = A + globalReadOffsetA_0_0_16_0;
  DATA_TYPE const *globalReadA_0_0_17_0 = A + globalReadOffsetA_0_0_17_0;
  DATA_TYPE const *globalReadA_0_0_18_0 = A + globalReadOffsetA_0_0_18_0;
  DATA_TYPE const *globalReadA_0_0_19_0 = A + globalReadOffsetA_0_0_19_0;
  DATA_TYPE const *globalReadA_0_0_20_0 = A + globalReadOffsetA_0_0_20_0;
  DATA_TYPE const *globalReadA_0_0_21_0 = A + globalReadOffsetA_0_0_21_0;
  DATA_TYPE const *globalReadA_0_0_22_0 = A + globalReadOffsetA_0_0_22_0;
  DATA_TYPE const *globalReadA_0_0_23_0 = A + globalReadOffsetA_0_0_23_0;
  DATA_TYPE const *globalReadA_0_0_24_0 = A + globalReadOffsetA_0_0_24_0;
  DATA_TYPE const *globalReadA_0_0_25_0 = A + globalReadOffsetA_0_0_25_0;
  DATA_TYPE const *globalReadA_0_0_26_0 = A + globalReadOffsetA_0_0_26_0;
  DATA_TYPE const *globalReadA_0_0_27_0 = A + globalReadOffsetA_0_0_27_0;
  DATA_TYPE const *globalReadA_0_0_28_0 = A + globalReadOffsetA_0_0_28_0;
  DATA_TYPE const *globalReadA_0_0_29_0 = A + globalReadOffsetA_0_0_29_0;
  DATA_TYPE const *globalReadA_0_0_30_0 = A + globalReadOffsetA_0_0_30_0;
  DATA_TYPE const *globalReadA_0_0_31_0 = A + globalReadOffsetA_0_0_31_0;
  DATA_TYPE const *globalReadA_0_0_32_0 = A + globalReadOffsetA_0_0_32_0;
  DATA_TYPE const *globalReadA_0_0_33_0 = A + globalReadOffsetA_0_0_33_0;
  DATA_TYPE const *globalReadA_0_0_34_0 = A + globalReadOffsetA_0_0_34_0;
  DATA_TYPE const *globalReadA_0_0_35_0 = A + globalReadOffsetA_0_0_35_0;
  DATA_TYPE const *globalReadA_0_0_36_0 = A + globalReadOffsetA_0_0_36_0;
  DATA_TYPE const *globalReadA_0_0_37_0 = A + globalReadOffsetA_0_0_37_0;
  DATA_TYPE const *globalReadA_0_0_38_0 = A + globalReadOffsetA_0_0_38_0;
  DATA_TYPE const *globalReadA_0_0_39_0 = A + globalReadOffsetA_0_0_39_0;
  DATA_TYPE const *globalReadA_0_0_40_0 = A + globalReadOffsetA_0_0_40_0;
  DATA_TYPE const *globalReadA_0_0_41_0 = A + globalReadOffsetA_0_0_41_0;
  DATA_TYPE const *globalReadA_0_0_42_0 = A + globalReadOffsetA_0_0_42_0;
  DATA_TYPE const *globalReadA_0_0_43_0 = A + globalReadOffsetA_0_0_43_0;
  DATA_TYPE const *globalReadA_0_0_44_0 = A + globalReadOffsetA_0_0_44_0;
  DATA_TYPE const *globalReadA_0_0_45_0 = A + globalReadOffsetA_0_0_45_0;
  DATA_TYPE const *globalReadA_0_0_46_0 = A + globalReadOffsetA_0_0_46_0;
  DATA_TYPE const *globalReadA_0_0_47_0 = A + globalReadOffsetA_0_0_47_0;
  DATA_TYPE const *globalReadA_0_0_48_0 = A + globalReadOffsetA_0_0_48_0;
  DATA_TYPE const *globalReadA_0_0_49_0 = A + globalReadOffsetA_0_0_49_0;
  DATA_TYPE const *globalReadA_0_0_50_0 = A + globalReadOffsetA_0_0_50_0;
  DATA_TYPE const *globalReadA_0_0_51_0 = A + globalReadOffsetA_0_0_51_0;
  DATA_TYPE const *globalReadA_0_0_52_0 = A + globalReadOffsetA_0_0_52_0;
  DATA_TYPE const *globalReadA_0_0_53_0 = A + globalReadOffsetA_0_0_53_0;
  DATA_TYPE const *globalReadA_0_0_54_0 = A + globalReadOffsetA_0_0_54_0;
  DATA_TYPE const *globalReadA_0_0_55_0 = A + globalReadOffsetA_0_0_55_0;
  DATA_TYPE const *globalReadA_0_0_56_0 = A + globalReadOffsetA_0_0_56_0;
  DATA_TYPE const *globalReadA_0_0_57_0 = A + globalReadOffsetA_0_0_57_0;
  DATA_TYPE const *globalReadA_0_0_58_0 = A + globalReadOffsetA_0_0_58_0;
  DATA_TYPE const *globalReadA_0_0_59_0 = A + globalReadOffsetA_0_0_59_0;
  DATA_TYPE const *globalReadA_0_0_60_0 = A + globalReadOffsetA_0_0_60_0;
  DATA_TYPE const *globalReadA_0_0_61_0 = A + globalReadOffsetA_0_0_61_0;
  DATA_TYPE const *globalReadA_0_0_62_0 = A + globalReadOffsetA_0_0_62_0;
  DATA_TYPE const *globalReadA_0_0_63_0 = A + globalReadOffsetA_0_0_63_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 8*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 9*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 10*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 11*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 12*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 13*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 14*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 15*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_16_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 16*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_17_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 17*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_18_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 18*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_19_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 19*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_20_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 20*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_21_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 21*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_22_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 22*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_23_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 23*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_24_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 24*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_25_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 25*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_26_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 26*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_27_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 27*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_28_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 28*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_29_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 29*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_30_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 30*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_31_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 31*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_32_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 32*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_33_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 33*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_34_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 34*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_35_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 35*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_36_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 36*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_37_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 37*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_38_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 38*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_39_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 39*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_40_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 40*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_41_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 41*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_42_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 42*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_43_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 43*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_44_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 44*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_45_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 45*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_46_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 46*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_47_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 47*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_48_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 48*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_49_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 49*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_50_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 50*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_51_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 51*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_52_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 52*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_53_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 53*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_54_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 54*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_55_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 55*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_56_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 56*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_57_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 57*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_58_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 58*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_59_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 59*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_60_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 60*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_61_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 61*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_62_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 62*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_63_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 63*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_0_0_15_0;
  DATA_TYPE *localWriteA_0_0_16_0;
  DATA_TYPE *localWriteA_0_0_17_0;
  DATA_TYPE *localWriteA_0_0_18_0;
  DATA_TYPE *localWriteA_0_0_19_0;
  DATA_TYPE *localWriteA_0_0_20_0;
  DATA_TYPE *localWriteA_0_0_21_0;
  DATA_TYPE *localWriteA_0_0_22_0;
  DATA_TYPE *localWriteA_0_0_23_0;
  DATA_TYPE *localWriteA_0_0_24_0;
  DATA_TYPE *localWriteA_0_0_25_0;
  DATA_TYPE *localWriteA_0_0_26_0;
  DATA_TYPE *localWriteA_0_0_27_0;
  DATA_TYPE *localWriteA_0_0_28_0;
  DATA_TYPE *localWriteA_0_0_29_0;
  DATA_TYPE *localWriteA_0_0_30_0;
  DATA_TYPE *localWriteA_0_0_31_0;
  DATA_TYPE *localWriteA_0_0_32_0;
  DATA_TYPE *localWriteA_0_0_33_0;
  DATA_TYPE *localWriteA_0_0_34_0;
  DATA_TYPE *localWriteA_0_0_35_0;
  DATA_TYPE *localWriteA_0_0_36_0;
  DATA_TYPE *localWriteA_0_0_37_0;
  DATA_TYPE *localWriteA_0_0_38_0;
  DATA_TYPE *localWriteA_0_0_39_0;
  DATA_TYPE *localWriteA_0_0_40_0;
  DATA_TYPE *localWriteA_0_0_41_0;
  DATA_TYPE *localWriteA_0_0_42_0;
  DATA_TYPE *localWriteA_0_0_43_0;
  DATA_TYPE *localWriteA_0_0_44_0;
  DATA_TYPE *localWriteA_0_0_45_0;
  DATA_TYPE *localWriteA_0_0_46_0;
  DATA_TYPE *localWriteA_0_0_47_0;
  DATA_TYPE *localWriteA_0_0_48_0;
  DATA_TYPE *localWriteA_0_0_49_0;
  DATA_TYPE *localWriteA_0_0_50_0;
  DATA_TYPE *localWriteA_0_0_51_0;
  DATA_TYPE *localWriteA_0_0_52_0;
  DATA_TYPE *localWriteA_0_0_53_0;
  DATA_TYPE *localWriteA_0_0_54_0;
  DATA_TYPE *localWriteA_0_0_55_0;
  DATA_TYPE *localWriteA_0_0_56_0;
  DATA_TYPE *localWriteA_0_0_57_0;
  DATA_TYPE *localWriteA_0_0_58_0;
  DATA_TYPE *localWriteA_0_0_59_0;
  DATA_TYPE *localWriteA_0_0_60_0;
  DATA_TYPE *localWriteA_0_0_61_0;
  DATA_TYPE *localWriteA_0_0_62_0;
  DATA_TYPE *localWriteA_0_0_63_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);
  localWriteA_0_0_16_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_16_0);
  localWriteA_0_0_17_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_17_0);
  localWriteA_0_0_18_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_18_0);
  localWriteA_0_0_19_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_19_0);
  localWriteA_0_0_20_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_20_0);
  localWriteA_0_0_21_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_21_0);
  localWriteA_0_0_22_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_22_0);
  localWriteA_0_0_23_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_23_0);
  localWriteA_0_0_24_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_24_0);
  localWriteA_0_0_25_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_25_0);
  localWriteA_0_0_26_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_26_0);
  localWriteA_0_0_27_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_27_0);
  localWriteA_0_0_28_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_28_0);
  localWriteA_0_0_29_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_29_0);
  localWriteA_0_0_30_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_30_0);
  localWriteA_0_0_31_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_31_0);
  localWriteA_0_0_32_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_32_0);
  localWriteA_0_0_33_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_33_0);
  localWriteA_0_0_34_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_34_0);
  localWriteA_0_0_35_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_35_0);
  localWriteA_0_0_36_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_36_0);
  localWriteA_0_0_37_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_37_0);
  localWriteA_0_0_38_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_38_0);
  localWriteA_0_0_39_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_39_0);
  localWriteA_0_0_40_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_40_0);
  localWriteA_0_0_41_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_41_0);
  localWriteA_0_0_42_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_42_0);
  localWriteA_0_0_43_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_43_0);
  localWriteA_0_0_44_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_44_0);
  localWriteA_0_0_45_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_45_0);
  localWriteA_0_0_46_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_46_0);
  localWriteA_0_0_47_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_47_0);
  localWriteA_0_0_48_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_48_0);
  localWriteA_0_0_49_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_49_0);
  localWriteA_0_0_50_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_50_0);
  localWriteA_0_0_51_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_51_0);
  localWriteA_0_0_52_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_52_0);
  localWriteA_0_0_53_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_53_0);
  localWriteA_0_0_54_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_54_0);
  localWriteA_0_0_55_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_55_0);
  localWriteA_0_0_56_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_56_0);
  localWriteA_0_0_57_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_57_0);
  localWriteA_0_0_58_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_58_0);
  localWriteA_0_0_59_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_59_0);
  localWriteA_0_0_60_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_60_0);
  localWriteA_0_0_61_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_61_0);
  localWriteA_0_0_62_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_62_0);
  localWriteA_0_0_63_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_63_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);
    a_0_0_16_0 = *(globalReadA_0_0_16_0 + 0);
    a_0_0_17_0 = *(globalReadA_0_0_17_0 + 0);
    a_0_0_18_0 = *(globalReadA_0_0_18_0 + 0);
    a_0_0_19_0 = *(globalReadA_0_0_19_0 + 0);
    a_0_0_20_0 = *(globalReadA_0_0_20_0 + 0);
    a_0_0_21_0 = *(globalReadA_0_0_21_0 + 0);
    a_0_0_22_0 = *(globalReadA_0_0_22_0 + 0);
    a_0_0_23_0 = *(globalReadA_0_0_23_0 + 0);
    a_0_0_24_0 = *(globalReadA_0_0_24_0 + 0);
    a_0_0_25_0 = *(globalReadA_0_0_25_0 + 0);
    a_0_0_26_0 = *(globalReadA_0_0_26_0 + 0);
    a_0_0_27_0 = *(globalReadA_0_0_27_0 + 0);
    a_0_0_28_0 = *(globalReadA_0_0_28_0 + 0);
    a_0_0_29_0 = *(globalReadA_0_0_29_0 + 0);
    a_0_0_30_0 = *(globalReadA_0_0_30_0 + 0);
    a_0_0_31_0 = *(globalReadA_0_0_31_0 + 0);
    a_0_0_32_0 = *(globalReadA_0_0_32_0 + 0);
    a_0_0_33_0 = *(globalReadA_0_0_33_0 + 0);
    a_0_0_34_0 = *(globalReadA_0_0_34_0 + 0);
    a_0_0_35_0 = *(globalReadA_0_0_35_0 + 0);
    a_0_0_36_0 = *(globalReadA_0_0_36_0 + 0);
    a_0_0_37_0 = *(globalReadA_0_0_37_0 + 0);
    a_0_0_38_0 = *(globalReadA_0_0_38_0 + 0);
    a_0_0_39_0 = *(globalReadA_0_0_39_0 + 0);
    a_0_0_40_0 = *(globalReadA_0_0_40_0 + 0);
    a_0_0_41_0 = *(globalReadA_0_0_41_0 + 0);
    a_0_0_42_0 = *(globalReadA_0_0_42_0 + 0);
    a_0_0_43_0 = *(globalReadA_0_0_43_0 + 0);
    a_0_0_44_0 = *(globalReadA_0_0_44_0 + 0);
    a_0_0_45_0 = *(globalReadA_0_0_45_0 + 0);
    a_0_0_46_0 = *(globalReadA_0_0_46_0 + 0);
    a_0_0_47_0 = *(globalReadA_0_0_47_0 + 0);
    a_0_0_48_0 = *(globalReadA_0_0_48_0 + 0);
    a_0_0_49_0 = *(globalReadA_0_0_49_0 + 0);
    a_0_0_50_0 = *(globalReadA_0_0_50_0 + 0);
    a_0_0_51_0 = *(globalReadA_0_0_51_0 + 0);
    a_0_0_52_0 = *(globalReadA_0_0_52_0 + 0);
    a_0_0_53_0 = *(globalReadA_0_0_53_0 + 0);
    a_0_0_54_0 = *(globalReadA_0_0_54_0 + 0);
    a_0_0_55_0 = *(globalReadA_0_0_55_0 + 0);
    a_0_0_56_0 = *(globalReadA_0_0_56_0 + 0);
    a_0_0_57_0 = *(globalReadA_0_0_57_0 + 0);
    a_0_0_58_0 = *(globalReadA_0_0_58_0 + 0);
    a_0_0_59_0 = *(globalReadA_0_0_59_0 + 0);
    a_0_0_60_0 = *(globalReadA_0_0_60_0 + 0);
    a_0_0_61_0 = *(globalReadA_0_0_61_0 + 0);
    a_0_0_62_0 = *(globalReadA_0_0_62_0 + 0);
    a_0_0_63_0 = *(globalReadA_0_0_63_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    globalReadA_0_0_16_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_16_0) + 1*globalReadIncAL);
    globalReadA_0_0_17_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_17_0) + 1*globalReadIncAL);
    globalReadA_0_0_18_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_18_0) + 1*globalReadIncAL);
    globalReadA_0_0_19_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_19_0) + 1*globalReadIncAL);
    globalReadA_0_0_20_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_20_0) + 1*globalReadIncAL);
    globalReadA_0_0_21_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_21_0) + 1*globalReadIncAL);
    globalReadA_0_0_22_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_22_0) + 1*globalReadIncAL);
    globalReadA_0_0_23_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_23_0) + 1*globalReadIncAL);
    globalReadA_0_0_24_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_24_0) + 1*globalReadIncAL);
    globalReadA_0_0_25_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_25_0) + 1*globalReadIncAL);
    globalReadA_0_0_26_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_26_0) + 1*globalReadIncAL);
    globalReadA_0_0_27_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_27_0) + 1*globalReadIncAL);
    globalReadA_0_0_28_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_28_0) + 1*globalReadIncAL);
    globalReadA_0_0_29_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_29_0) + 1*globalReadIncAL);
    globalReadA_0_0_30_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_30_0) + 1*globalReadIncAL);
    globalReadA_0_0_31_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_31_0) + 1*globalReadIncAL);
    globalReadA_0_0_32_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_32_0) + 1*globalReadIncAL);
    globalReadA_0_0_33_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_33_0) + 1*globalReadIncAL);
    globalReadA_0_0_34_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_34_0) + 1*globalReadIncAL);
    globalReadA_0_0_35_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_35_0) + 1*globalReadIncAL);
    globalReadA_0_0_36_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_36_0) + 1*globalReadIncAL);
    globalReadA_0_0_37_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_37_0) + 1*globalReadIncAL);
    globalReadA_0_0_38_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_38_0) + 1*globalReadIncAL);
    globalReadA_0_0_39_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_39_0) + 1*globalReadIncAL);
    globalReadA_0_0_40_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_40_0) + 1*globalReadIncAL);
    globalReadA_0_0_41_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_41_0) + 1*globalReadIncAL);
    globalReadA_0_0_42_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_42_0) + 1*globalReadIncAL);
    globalReadA_0_0_43_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_43_0) + 1*globalReadIncAL);
    globalReadA_0_0_44_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_44_0) + 1*globalReadIncAL);
    globalReadA_0_0_45_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_45_0) + 1*globalReadIncAL);
    globalReadA_0_0_46_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_46_0) + 1*globalReadIncAL);
    globalReadA_0_0_47_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_47_0) + 1*globalReadIncAL);
    globalReadA_0_0_48_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_48_0) + 1*globalReadIncAL);
    globalReadA_0_0_49_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_49_0) + 1*globalReadIncAL);
    globalReadA_0_0_50_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_50_0) + 1*globalReadIncAL);
    globalReadA_0_0_51_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_51_0) + 1*globalReadIncAL);
    globalReadA_0_0_52_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_52_0) + 1*globalReadIncAL);
    globalReadA_0_0_53_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_53_0) + 1*globalReadIncAL);
    globalReadA_0_0_54_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_54_0) + 1*globalReadIncAL);
    globalReadA_0_0_55_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_55_0) + 1*globalReadIncAL);
    globalReadA_0_0_56_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_56_0) + 1*globalReadIncAL);
    globalReadA_0_0_57_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_57_0) + 1*globalReadIncAL);
    globalReadA_0_0_58_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_58_0) + 1*globalReadIncAL);
    globalReadA_0_0_59_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_59_0) + 1*globalReadIncAL);
    globalReadA_0_0_60_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_60_0) + 1*globalReadIncAL);
    globalReadA_0_0_61_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_61_0) + 1*globalReadIncAL);
    globalReadA_0_0_62_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_62_0) + 1*globalReadIncAL);
    globalReadA_0_0_63_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_63_0) + 1*globalReadIncAL);

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
    *(localWriteA_0_0_16_0 + 0) = a_0_0_16_0;
    *(localWriteA_0_0_17_0 + 0) = a_0_0_17_0;
    *(localWriteA_0_0_18_0 + 0) = a_0_0_18_0;
    *(localWriteA_0_0_19_0 + 0) = a_0_0_19_0;
    *(localWriteA_0_0_20_0 + 0) = a_0_0_20_0;
    *(localWriteA_0_0_21_0 + 0) = a_0_0_21_0;
    *(localWriteA_0_0_22_0 + 0) = a_0_0_22_0;
    *(localWriteA_0_0_23_0 + 0) = a_0_0_23_0;
    *(localWriteA_0_0_24_0 + 0) = a_0_0_24_0;
    *(localWriteA_0_0_25_0 + 0) = a_0_0_25_0;
    *(localWriteA_0_0_26_0 + 0) = a_0_0_26_0;
    *(localWriteA_0_0_27_0 + 0) = a_0_0_27_0;
    *(localWriteA_0_0_28_0 + 0) = a_0_0_28_0;
    *(localWriteA_0_0_29_0 + 0) = a_0_0_29_0;
    *(localWriteA_0_0_30_0 + 0) = a_0_0_30_0;
    *(localWriteA_0_0_31_0 + 0) = a_0_0_31_0;
    *(localWriteA_0_0_32_0 + 0) = a_0_0_32_0;
    *(localWriteA_0_0_33_0 + 0) = a_0_0_33_0;
    *(localWriteA_0_0_34_0 + 0) = a_0_0_34_0;
    *(localWriteA_0_0_35_0 + 0) = a_0_0_35_0;
    *(localWriteA_0_0_36_0 + 0) = a_0_0_36_0;
    *(localWriteA_0_0_37_0 + 0) = a_0_0_37_0;
    *(localWriteA_0_0_38_0 + 0) = a_0_0_38_0;
    *(localWriteA_0_0_39_0 + 0) = a_0_0_39_0;
    *(localWriteA_0_0_40_0 + 0) = a_0_0_40_0;
    *(localWriteA_0_0_41_0 + 0) = a_0_0_41_0;
    *(localWriteA_0_0_42_0 + 0) = a_0_0_42_0;
    *(localWriteA_0_0_43_0 + 0) = a_0_0_43_0;
    *(localWriteA_0_0_44_0 + 0) = a_0_0_44_0;
    *(localWriteA_0_0_45_0 + 0) = a_0_0_45_0;
    *(localWriteA_0_0_46_0 + 0) = a_0_0_46_0;
    *(localWriteA_0_0_47_0 + 0) = a_0_0_47_0;
    *(localWriteA_0_0_48_0 + 0) = a_0_0_48_0;
    *(localWriteA_0_0_49_0 + 0) = a_0_0_49_0;
    *(localWriteA_0_0_50_0 + 0) = a_0_0_50_0;
    *(localWriteA_0_0_51_0 + 0) = a_0_0_51_0;
    *(localWriteA_0_0_52_0 + 0) = a_0_0_52_0;
    *(localWriteA_0_0_53_0 + 0) = a_0_0_53_0;
    *(localWriteA_0_0_54_0 + 0) = a_0_0_54_0;
    *(localWriteA_0_0_55_0 + 0) = a_0_0_55_0;
    *(localWriteA_0_0_56_0 + 0) = a_0_0_56_0;
    *(localWriteA_0_0_57_0 + 0) = a_0_0_57_0;
    *(localWriteA_0_0_58_0 + 0) = a_0_0_58_0;
    *(localWriteA_0_0_59_0 + 0) = a_0_0_59_0;
    *(localWriteA_0_0_60_0 + 0) = a_0_0_60_0;
    *(localWriteA_0_0_61_0 + 0) = a_0_0_61_0;
    *(localWriteA_0_0_62_0 + 0) = a_0_0_62_0;
    *(localWriteA_0_0_63_0 + 0) = a_0_0_63_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 15 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 16 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 17 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 18 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 19 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 20 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 21 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 22 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 23 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 24 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 25 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 26 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 27 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 28 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 29 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 30 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 31 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 32 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 33 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 34 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 35 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 36 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 37 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 38 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 39 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 40 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 41 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 42 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 43 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 44 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 45 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 46 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 47 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 48 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 49 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 50 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 51 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 52 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 53 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 54 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 55 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 56 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 57 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 58 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 59 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 60 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 61 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 62 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 63 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_1x1




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_0_8_0 = ( globalReadOffsetAL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_0_0_9_0 = ( globalReadOffsetAL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_0_0_10_0 = ( globalReadOffsetAL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_0_0_11_0 = ( globalReadOffsetAL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_0_0_12_0 = ( globalReadOffsetAL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_0_0_13_0 = ( globalReadOffsetAL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_0_0_14_0 = ( globalReadOffsetAL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_0_0_15_0 = ( globalReadOffsetAL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);
  a_0_0_16_0 = ( globalReadOffsetAL_16_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_16_0 + 0);
  a_0_0_17_0 = ( globalReadOffsetAL_17_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_17_0 + 0);
  a_0_0_18_0 = ( globalReadOffsetAL_18_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_18_0 + 0);
  a_0_0_19_0 = ( globalReadOffsetAL_19_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_19_0 + 0);
  a_0_0_20_0 = ( globalReadOffsetAL_20_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_20_0 + 0);
  a_0_0_21_0 = ( globalReadOffsetAL_21_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_21_0 + 0);
  a_0_0_22_0 = ( globalReadOffsetAL_22_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_22_0 + 0);
  a_0_0_23_0 = ( globalReadOffsetAL_23_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_23_0 + 0);
  a_0_0_24_0 = ( globalReadOffsetAL_24_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_24_0 + 0);
  a_0_0_25_0 = ( globalReadOffsetAL_25_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_25_0 + 0);
  a_0_0_26_0 = ( globalReadOffsetAL_26_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_26_0 + 0);
  a_0_0_27_0 = ( globalReadOffsetAL_27_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_27_0 + 0);
  a_0_0_28_0 = ( globalReadOffsetAL_28_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_28_0 + 0);
  a_0_0_29_0 = ( globalReadOffsetAL_29_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_29_0 + 0);
  a_0_0_30_0 = ( globalReadOffsetAL_30_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_30_0 + 0);
  a_0_0_31_0 = ( globalReadOffsetAL_31_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_31_0 + 0);
  a_0_0_32_0 = ( globalReadOffsetAL_32_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_32_0 + 0);
  a_0_0_33_0 = ( globalReadOffsetAL_33_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_33_0 + 0);
  a_0_0_34_0 = ( globalReadOffsetAL_34_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_34_0 + 0);
  a_0_0_35_0 = ( globalReadOffsetAL_35_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_35_0 + 0);
  a_0_0_36_0 = ( globalReadOffsetAL_36_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_36_0 + 0);
  a_0_0_37_0 = ( globalReadOffsetAL_37_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_37_0 + 0);
  a_0_0_38_0 = ( globalReadOffsetAL_38_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_38_0 + 0);
  a_0_0_39_0 = ( globalReadOffsetAL_39_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_39_0 + 0);
  a_0_0_40_0 = ( globalReadOffsetAL_40_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_40_0 + 0);
  a_0_0_41_0 = ( globalReadOffsetAL_41_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_41_0 + 0);
  a_0_0_42_0 = ( globalReadOffsetAL_42_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_42_0 + 0);
  a_0_0_43_0 = ( globalReadOffsetAL_43_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_43_0 + 0);
  a_0_0_44_0 = ( globalReadOffsetAL_44_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_44_0 + 0);
  a_0_0_45_0 = ( globalReadOffsetAL_45_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_45_0 + 0);
  a_0_0_46_0 = ( globalReadOffsetAL_46_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_46_0 + 0);
  a_0_0_47_0 = ( globalReadOffsetAL_47_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_47_0 + 0);
  a_0_0_48_0 = ( globalReadOffsetAL_48_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_48_0 + 0);
  a_0_0_49_0 = ( globalReadOffsetAL_49_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_49_0 + 0);
  a_0_0_50_0 = ( globalReadOffsetAL_50_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_50_0 + 0);
  a_0_0_51_0 = ( globalReadOffsetAL_51_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_51_0 + 0);
  a_0_0_52_0 = ( globalReadOffsetAL_52_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_52_0 + 0);
  a_0_0_53_0 = ( globalReadOffsetAL_53_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_53_0 + 0);
  a_0_0_54_0 = ( globalReadOffsetAL_54_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_54_0 + 0);
  a_0_0_55_0 = ( globalReadOffsetAL_55_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_55_0 + 0);
  a_0_0_56_0 = ( globalReadOffsetAL_56_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_56_0 + 0);
  a_0_0_57_0 = ( globalReadOffsetAL_57_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_57_0 + 0);
  a_0_0_58_0 = ( globalReadOffsetAL_58_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_58_0 + 0);
  a_0_0_59_0 = ( globalReadOffsetAL_59_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_59_0 + 0);
  a_0_0_60_0 = ( globalReadOffsetAL_60_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_60_0 + 0);
  a_0_0_61_0 = ( globalReadOffsetAL_61_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_61_0 + 0);
  a_0_0_62_0 = ( globalReadOffsetAL_62_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_62_0 + 0);
  a_0_0_63_0 = ( globalReadOffsetAL_63_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_63_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);
  localWriteA_0_0_16_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_16_0);
  localWriteA_0_0_17_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_17_0);
  localWriteA_0_0_18_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_18_0);
  localWriteA_0_0_19_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_19_0);
  localWriteA_0_0_20_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_20_0);
  localWriteA_0_0_21_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_21_0);
  localWriteA_0_0_22_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_22_0);
  localWriteA_0_0_23_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_23_0);
  localWriteA_0_0_24_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_24_0);
  localWriteA_0_0_25_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_25_0);
  localWriteA_0_0_26_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_26_0);
  localWriteA_0_0_27_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_27_0);
  localWriteA_0_0_28_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_28_0);
  localWriteA_0_0_29_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_29_0);
  localWriteA_0_0_30_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_30_0);
  localWriteA_0_0_31_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_31_0);
  localWriteA_0_0_32_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_32_0);
  localWriteA_0_0_33_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_33_0);
  localWriteA_0_0_34_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_34_0);
  localWriteA_0_0_35_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_35_0);
  localWriteA_0_0_36_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_36_0);
  localWriteA_0_0_37_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_37_0);
  localWriteA_0_0_38_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_38_0);
  localWriteA_0_0_39_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_39_0);
  localWriteA_0_0_40_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_40_0);
  localWriteA_0_0_41_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_41_0);
  localWriteA_0_0_42_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_42_0);
  localWriteA_0_0_43_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_43_0);
  localWriteA_0_0_44_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_44_0);
  localWriteA_0_0_45_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_45_0);
  localWriteA_0_0_46_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_46_0);
  localWriteA_0_0_47_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_47_0);
  localWriteA_0_0_48_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_48_0);
  localWriteA_0_0_49_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_49_0);
  localWriteA_0_0_50_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_50_0);
  localWriteA_0_0_51_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_51_0);
  localWriteA_0_0_52_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_52_0);
  localWriteA_0_0_53_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_53_0);
  localWriteA_0_0_54_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_54_0);
  localWriteA_0_0_55_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_55_0);
  localWriteA_0_0_56_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_56_0);
  localWriteA_0_0_57_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_57_0);
  localWriteA_0_0_58_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_58_0);
  localWriteA_0_0_59_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_59_0);
  localWriteA_0_0_60_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_60_0);
  localWriteA_0_0_61_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_61_0);
  localWriteA_0_0_62_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_62_0);
  localWriteA_0_0_63_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_63_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
  *(localWriteA_0_0_16_0 + 0) = a_0_0_16_0;
  *(localWriteA_0_0_17_0 + 0) = a_0_0_17_0;
  *(localWriteA_0_0_18_0 + 0) = a_0_0_18_0;
  *(localWriteA_0_0_19_0 + 0) = a_0_0_19_0;
  *(localWriteA_0_0_20_0 + 0) = a_0_0_20_0;
  *(localWriteA_0_0_21_0 + 0) = a_0_0_21_0;
  *(localWriteA_0_0_22_0 + 0) = a_0_0_22_0;
  *(localWriteA_0_0_23_0 + 0) = a_0_0_23_0;
  *(localWriteA_0_0_24_0 + 0) = a_0_0_24_0;
  *(localWriteA_0_0_25_0 + 0) = a_0_0_25_0;
  *(localWriteA_0_0_26_0 + 0) = a_0_0_26_0;
  *(localWriteA_0_0_27_0 + 0) = a_0_0_27_0;
  *(localWriteA_0_0_28_0 + 0) = a_0_0_28_0;
  *(localWriteA_0_0_29_0 + 0) = a_0_0_29_0;
  *(localWriteA_0_0_30_0 + 0) = a_0_0_30_0;
  *(localWriteA_0_0_31_0 + 0) = a_0_0_31_0;
  *(localWriteA_0_0_32_0 + 0) = a_0_0_32_0;
  *(localWriteA_0_0_33_0 + 0) = a_0_0_33_0;
  *(localWriteA_0_0_34_0 + 0) = a_0_0_34_0;
  *(localWriteA_0_0_35_0 + 0) = a_0_0_35_0;
  *(localWriteA_0_0_36_0 + 0) = a_0_0_36_0;
  *(localWriteA_0_0_37_0 + 0) = a_0_0_37_0;
  *(localWriteA_0_0_38_0 + 0) = a_0_0_38_0;
  *(localWriteA_0_0_39_0 + 0) = a_0_0_39_0;
  *(localWriteA_0_0_40_0 + 0) = a_0_0_40_0;
  *(localWriteA_0_0_41_0 + 0) = a_0_0_41_0;
  *(localWriteA_0_0_42_0 + 0) = a_0_0_42_0;
  *(localWriteA_0_0_43_0 + 0) = a_0_0_43_0;
  *(localWriteA_0_0_44_0 + 0) = a_0_0_44_0;
  *(localWriteA_0_0_45_0 + 0) = a_0_0_45_0;
  *(localWriteA_0_0_46_0 + 0) = a_0_0_46_0;
  *(localWriteA_0_0_47_0 + 0) = a_0_0_47_0;
  *(localWriteA_0_0_48_0 + 0) = a_0_0_48_0;
  *(localWriteA_0_0_49_0 + 0) = a_0_0_49_0;
  *(localWriteA_0_0_50_0 + 0) = a_0_0_50_0;
  *(localWriteA_0_0_51_0 + 0) = a_0_0_51_0;
  *(localWriteA_0_0_52_0 + 0) = a_0_0_52_0;
  *(localWriteA_0_0_53_0 + 0) = a_0_0_53_0;
  *(localWriteA_0_0_54_0 + 0) = a_0_0_54_0;
  *(localWriteA_0_0_55_0 + 0) = a_0_0_55_0;
  *(localWriteA_0_0_56_0 + 0) = a_0_0_56_0;
  *(localWriteA_0_0_57_0 + 0) = a_0_0_57_0;
  *(localWriteA_0_0_58_0 + 0) = a_0_0_58_0;
  *(localWriteA_0_0_59_0 + 0) = a_0_0_59_0;
  *(localWriteA_0_0_60_0 + 0) = a_0_0_60_0;
  *(localWriteA_0_0_61_0 + 0) = a_0_0_61_0;
  *(localWriteA_0_0_62_0 + 0) = a_0_0_62_0;
  *(localWriteA_0_0_63_0 + 0) = a_0_0_63_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_1x1

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_1x1
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 64
#define SG1J 1
#define TT0I 1
#define TT1J 1
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 64
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 64
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 4096
#define LDS_NUM_ELEMENTS 4160

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 1x1 micro-tile */
#define MAC_1x1\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_BSS_BH_MT64x1x64_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU0_SUM0_SUS128_SRVW0_SVW4_SNLL0_TT1_1_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG64_1_1_WGM1(
  float *D,
  float const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_bfloat16)(0)
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE a_0_0_16_0;
  DATA_TYPE a_0_0_17_0;
  DATA_TYPE a_0_0_18_0;
  DATA_TYPE a_0_0_19_0;
  DATA_TYPE a_0_0_20_0;
  DATA_TYPE a_0_0_21_0;
  DATA_TYPE a_0_0_22_0;
  DATA_TYPE a_0_0_23_0;
  DATA_TYPE a_0_0_24_0;
  DATA_TYPE a_0_0_25_0;
  DATA_TYPE a_0_0_26_0;
  DATA_TYPE a_0_0_27_0;
  DATA_TYPE a_0_0_28_0;
  DATA_TYPE a_0_0_29_0;
  DATA_TYPE a_0_0_30_0;
  DATA_TYPE a_0_0_31_0;
  DATA_TYPE a_0_0_32_0;
  DATA_TYPE a_0_0_33_0;
  DATA_TYPE a_0_0_34_0;
  DATA_TYPE a_0_0_35_0;
  DATA_TYPE a_0_0_36_0;
  DATA_TYPE a_0_0_37_0;
  DATA_TYPE a_0_0_38_0;
  DATA_TYPE a_0_0_39_0;
  DATA_TYPE a_0_0_40_0;
  DATA_TYPE a_0_0_41_0;
  DATA_TYPE a_0_0_42_0;
  DATA_TYPE a_0_0_43_0;
  DATA_TYPE a_0_0_44_0;
  DATA_TYPE a_0_0_45_0;
  DATA_TYPE a_0_0_46_0;
  DATA_TYPE a_0_0_47_0;
  DATA_TYPE a_0_0_48_0;
  DATA_TYPE a_0_0_49_0;
  DATA_TYPE a_0_0_50_0;
  DATA_TYPE a_0_0_51_0;
  DATA_TYPE a_0_0_52_0;
  DATA_TYPE a_0_0_53_0;
  DATA_TYPE a_0_0_54_0;
  DATA_TYPE a_0_0_55_0;
  DATA_TYPE a_0_0_56_0;
  DATA_TYPE a_0_0_57_0;
  DATA_TYPE a_0_0_58_0;
  DATA_TYPE a_0_0_59_0;
  DATA_TYPE a_0_0_60_0;
  DATA_TYPE a_0_0_61_0;
  DATA_TYPE a_0_0_62_0;
  DATA_TYPE a_0_0_63_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;
  unsigned int globalReadOffsetAL_8_0 = globalReadOffsetAL + 0 + 8*LSPA;
  unsigned int globalReadOffsetAL_9_0 = globalReadOffsetAL + 0 + 9*LSPA;
  unsigned int globalReadOffsetAL_10_0 = globalReadOffsetAL + 0 + 10*LSPA;
  unsigned int globalReadOffsetAL_11_0 = globalReadOffsetAL + 0 + 11*LSPA;
  unsigned int globalReadOffsetAL_12_0 = globalReadOffsetAL + 0 + 12*LSPA;
  unsigned int globalReadOffsetAL_13_0 = globalReadOffsetAL + 0 + 13*LSPA;
  unsigned int globalReadOffsetAL_14_0 = globalReadOffsetAL + 0 + 14*LSPA;
  unsigned int globalReadOffsetAL_15_0 = globalReadOffsetAL + 0 + 15*LSPA;
  unsigned int globalReadOffsetAL_16_0 = globalReadOffsetAL + 0 + 16*LSPA;
  unsigned int globalReadOffsetAL_17_0 = globalReadOffsetAL + 0 + 17*LSPA;
  unsigned int globalReadOffsetAL_18_0 = globalReadOffsetAL + 0 + 18*LSPA;
  unsigned int globalReadOffsetAL_19_0 = globalReadOffsetAL + 0 + 19*LSPA;
  unsigned int globalReadOffsetAL_20_0 = globalReadOffsetAL + 0 + 20*LSPA;
  unsigned int globalReadOffsetAL_21_0 = globalReadOffsetAL + 0 + 21*LSPA;
  unsigned int globalReadOffsetAL_22_0 = globalReadOffsetAL + 0 + 22*LSPA;
  unsigned int globalReadOffsetAL_23_0 = globalReadOffsetAL + 0 + 23*LSPA;
  unsigned int globalReadOffsetAL_24_0 = globalReadOffsetAL + 0 + 24*LSPA;
  unsigned int globalReadOffsetAL_25_0 = globalReadOffsetAL + 0 + 25*LSPA;
  unsigned int globalReadOffsetAL_26_0 = globalReadOffsetAL + 0 + 26*LSPA;
  unsigned int globalReadOffsetAL_27_0 = globalReadOffsetAL + 0 + 27*LSPA;
  unsigned int globalReadOffsetAL_28_0 = globalReadOffsetAL + 0 + 28*LSPA;
  unsigned int globalReadOffsetAL_29_0 = globalReadOffsetAL + 0 + 29*LSPA;
  unsigned int globalReadOffsetAL_30_0 = globalReadOffsetAL + 0 + 30*LSPA;
  unsigned int globalReadOffsetAL_31_0 = globalReadOffsetAL + 0 + 31*LSPA;
  unsigned int globalReadOffsetAL_32_0 = globalReadOffsetAL + 0 + 32*LSPA;
  unsigned int globalReadOffsetAL_33_0 = globalReadOffsetAL + 0 + 33*LSPA;
  unsigned int globalReadOffsetAL_34_0 = globalReadOffsetAL + 0 + 34*LSPA;
  unsigned int globalReadOffsetAL_35_0 = globalReadOffsetAL + 0 + 35*LSPA;
  unsigned int globalReadOffsetAL_36_0 = globalReadOffsetAL + 0 + 36*LSPA;
  unsigned int globalReadOffsetAL_37_0 = globalReadOffsetAL + 0 + 37*LSPA;
  unsigned int globalReadOffsetAL_38_0 = globalReadOffsetAL + 0 + 38*LSPA;
  unsigned int globalReadOffsetAL_39_0 = globalReadOffsetAL + 0 + 39*LSPA;
  unsigned int globalReadOffsetAL_40_0 = globalReadOffsetAL + 0 + 40*LSPA;
  unsigned int globalReadOffsetAL_41_0 = globalReadOffsetAL + 0 + 41*LSPA;
  unsigned int globalReadOffsetAL_42_0 = globalReadOffsetAL + 0 + 42*LSPA;
  unsigned int globalReadOffsetAL_43_0 = globalReadOffsetAL + 0 + 43*LSPA;
  unsigned int globalReadOffsetAL_44_0 = globalReadOffsetAL + 0 + 44*LSPA;
  unsigned int globalReadOffsetAL_45_0 = globalReadOffsetAL + 0 + 45*LSPA;
  unsigned int globalReadOffsetAL_46_0 = globalReadOffsetAL + 0 + 46*LSPA;
  unsigned int globalReadOffsetAL_47_0 = globalReadOffsetAL + 0 + 47*LSPA;
  unsigned int globalReadOffsetAL_48_0 = globalReadOffsetAL + 0 + 48*LSPA;
  unsigned int globalReadOffsetAL_49_0 = globalReadOffsetAL + 0 + 49*LSPA;
  unsigned int globalReadOffsetAL_50_0 = globalReadOffsetAL + 0 + 50*LSPA;
  unsigned int globalReadOffsetAL_51_0 = globalReadOffsetAL + 0 + 51*LSPA;
  unsigned int globalReadOffsetAL_52_0 = globalReadOffsetAL + 0 + 52*LSPA;
  unsigned int globalReadOffsetAL_53_0 = globalReadOffsetAL + 0 + 53*LSPA;
  unsigned int globalReadOffsetAL_54_0 = globalReadOffsetAL + 0 + 54*LSPA;
  unsigned int globalReadOffsetAL_55_0 = globalReadOffsetAL + 0 + 55*LSPA;
  unsigned int globalReadOffsetAL_56_0 = globalReadOffsetAL + 0 + 56*LSPA;
  unsigned int globalReadOffsetAL_57_0 = globalReadOffsetAL + 0 + 57*LSPA;
  unsigned int globalReadOffsetAL_58_0 = globalReadOffsetAL + 0 + 58*LSPA;
  unsigned int globalReadOffsetAL_59_0 = globalReadOffsetAL + 0 + 59*LSPA;
  unsigned int globalReadOffsetAL_60_0 = globalReadOffsetAL + 0 + 60*LSPA;
  unsigned int globalReadOffsetAL_61_0 = globalReadOffsetAL + 0 + 61*LSPA;
  unsigned int globalReadOffsetAL_62_0 = globalReadOffsetAL + 0 + 62*LSPA;
  unsigned int globalReadOffsetAL_63_0 = globalReadOffsetAL + 0 + 63*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_15_0), wgK );
  int64_t globalReadOffsetA_0_0_16_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_16_0), wgK );
  int64_t globalReadOffsetA_0_0_17_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_17_0), wgK );
  int64_t globalReadOffsetA_0_0_18_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_18_0), wgK );
  int64_t globalReadOffsetA_0_0_19_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_19_0), wgK );
  int64_t globalReadOffsetA_0_0_20_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_20_0), wgK );
  int64_t globalReadOffsetA_0_0_21_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_21_0), wgK );
  int64_t globalReadOffsetA_0_0_22_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_22_0), wgK );
  int64_t globalReadOffsetA_0_0_23_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_23_0), wgK );
  int64_t globalReadOffsetA_0_0_24_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_24_0), wgK );
  int64_t globalReadOffsetA_0_0_25_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_25_0), wgK );
  int64_t globalReadOffsetA_0_0_26_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_26_0), wgK );
  int64_t globalReadOffsetA_0_0_27_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_27_0), wgK );
  int64_t globalReadOffsetA_0_0_28_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_28_0), wgK );
  int64_t globalReadOffsetA_0_0_29_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_29_0), wgK );
  int64_t globalReadOffsetA_0_0_30_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_30_0), wgK );
  int64_t globalReadOffsetA_0_0_31_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_31_0), wgK );
  int64_t globalReadOffsetA_0_0_32_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_32_0), wgK );
  int64_t globalReadOffsetA_0_0_33_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_33_0), wgK );
  int64_t globalReadOffsetA_0_0_34_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_34_0), wgK );
  int64_t globalReadOffsetA_0_0_35_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_35_0), wgK );
  int64_t globalReadOffsetA_0_0_36_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_36_0), wgK );
  int64_t globalReadOffsetA_0_0_37_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_37_0), wgK );
  int64_t globalReadOffsetA_0_0_38_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_38_0), wgK );
  int64_t globalReadOffsetA_0_0_39_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_39_0), wgK );
  int64_t globalReadOffsetA_0_0_40_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_40_0), wgK );
  int64_t globalReadOffsetA_0_0_41_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_41_0), wgK );
  int64_t globalReadOffsetA_0_0_42_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_42_0), wgK );
  int64_t globalReadOffsetA_0_0_43_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_43_0), wgK );
  int64_t globalReadOffsetA_0_0_44_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_44_0), wgK );
  int64_t globalReadOffsetA_0_0_45_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_45_0), wgK );
  int64_t globalReadOffsetA_0_0_46_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_46_0), wgK );
  int64_t globalReadOffsetA_0_0_47_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_47_0), wgK );
  int64_t globalReadOffsetA_0_0_48_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_48_0), wgK );
  int64_t globalReadOffsetA_0_0_49_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_49_0), wgK );
  int64_t globalReadOffsetA_0_0_50_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_50_0), wgK );
  int64_t globalReadOffsetA_0_0_51_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_51_0), wgK );
  int64_t globalReadOffsetA_0_0_52_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_52_0), wgK );
  int64_t globalReadOffsetA_0_0_53_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_53_0), wgK );
  int64_t globalReadOffsetA_0_0_54_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_54_0), wgK );
  int64_t globalReadOffsetA_0_0_55_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_55_0), wgK );
  int64_t globalReadOffsetA_0_0_56_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_56_0), wgK );
  int64_t globalReadOffsetA_0_0_57_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_57_0), wgK );
  int64_t globalReadOffsetA_0_0_58_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_58_0), wgK );
  int64_t globalReadOffsetA_0_0_59_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_59_0), wgK );
  int64_t globalReadOffsetA_0_0_60_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_60_0), wgK );
  int64_t globalReadOffsetA_0_0_61_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_61_0), wgK );
  int64_t globalReadOffsetA_0_0_62_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_62_0), wgK );
  int64_t globalReadOffsetA_0_0_63_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_63_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;
  DATA_TYPE const *globalReadA_0_0_16_0 = A + globalReadOffsetA_0_0_16_0;
  DATA_TYPE const *globalReadA_0_0_17_0 = A + globalReadOffsetA_0_0_17_0;
  DATA_TYPE const *globalReadA_0_0_18_0 = A + globalReadOffsetA_0_0_18_0;
  DATA_TYPE const *globalReadA_0_0_19_0 = A + globalReadOffsetA_0_0_19_0;
  DATA_TYPE const *globalReadA_0_0_20_0 = A + globalReadOffsetA_0_0_20_0;
  DATA_TYPE const *globalReadA_0_0_21_0 = A + globalReadOffsetA_0_0_21_0;
  DATA_TYPE const *globalReadA_0_0_22_0 = A + globalReadOffsetA_0_0_22_0;
  DATA_TYPE const *globalReadA_0_0_23_0 = A + globalReadOffsetA_0_0_23_0;
  DATA_TYPE const *globalReadA_0_0_24_0 = A + globalReadOffsetA_0_0_24_0;
  DATA_TYPE const *globalReadA_0_0_25_0 = A + globalReadOffsetA_0_0_25_0;
  DATA_TYPE const *globalReadA_0_0_26_0 = A + globalReadOffsetA_0_0_26_0;
  DATA_TYPE const *globalReadA_0_0_27_0 = A + globalReadOffsetA_0_0_27_0;
  DATA_TYPE const *globalReadA_0_0_28_0 = A + globalReadOffsetA_0_0_28_0;
  DATA_TYPE const *globalReadA_0_0_29_0 = A + globalReadOffsetA_0_0_29_0;
  DATA_TYPE const *globalReadA_0_0_30_0 = A + globalReadOffsetA_0_0_30_0;
  DATA_TYPE const *globalReadA_0_0_31_0 = A + globalReadOffsetA_0_0_31_0;
  DATA_TYPE const *globalReadA_0_0_32_0 = A + globalReadOffsetA_0_0_32_0;
  DATA_TYPE const *globalReadA_0_0_33_0 = A + globalReadOffsetA_0_0_33_0;
  DATA_TYPE const *globalReadA_0_0_34_0 = A + globalReadOffsetA_0_0_34_0;
  DATA_TYPE const *globalReadA_0_0_35_0 = A + globalReadOffsetA_0_0_35_0;
  DATA_TYPE const *globalReadA_0_0_36_0 = A + globalReadOffsetA_0_0_36_0;
  DATA_TYPE const *globalReadA_0_0_37_0 = A + globalReadOffsetA_0_0_37_0;
  DATA_TYPE const *globalReadA_0_0_38_0 = A + globalReadOffsetA_0_0_38_0;
  DATA_TYPE const *globalReadA_0_0_39_0 = A + globalReadOffsetA_0_0_39_0;
  DATA_TYPE const *globalReadA_0_0_40_0 = A + globalReadOffsetA_0_0_40_0;
  DATA_TYPE const *globalReadA_0_0_41_0 = A + globalReadOffsetA_0_0_41_0;
  DATA_TYPE const *globalReadA_0_0_42_0 = A + globalReadOffsetA_0_0_42_0;
  DATA_TYPE const *globalReadA_0_0_43_0 = A + globalReadOffsetA_0_0_43_0;
  DATA_TYPE const *globalReadA_0_0_44_0 = A + globalReadOffsetA_0_0_44_0;
  DATA_TYPE const *globalReadA_0_0_45_0 = A + globalReadOffsetA_0_0_45_0;
  DATA_TYPE const *globalReadA_0_0_46_0 = A + globalReadOffsetA_0_0_46_0;
  DATA_TYPE const *globalReadA_0_0_47_0 = A + globalReadOffsetA_0_0_47_0;
  DATA_TYPE const *globalReadA_0_0_48_0 = A + globalReadOffsetA_0_0_48_0;
  DATA_TYPE const *globalReadA_0_0_49_0 = A + globalReadOffsetA_0_0_49_0;
  DATA_TYPE const *globalReadA_0_0_50_0 = A + globalReadOffsetA_0_0_50_0;
  DATA_TYPE const *globalReadA_0_0_51_0 = A + globalReadOffsetA_0_0_51_0;
  DATA_TYPE const *globalReadA_0_0_52_0 = A + globalReadOffsetA_0_0_52_0;
  DATA_TYPE const *globalReadA_0_0_53_0 = A + globalReadOffsetA_0_0_53_0;
  DATA_TYPE const *globalReadA_0_0_54_0 = A + globalReadOffsetA_0_0_54_0;
  DATA_TYPE const *globalReadA_0_0_55_0 = A + globalReadOffsetA_0_0_55_0;
  DATA_TYPE const *globalReadA_0_0_56_0 = A + globalReadOffsetA_0_0_56_0;
  DATA_TYPE const *globalReadA_0_0_57_0 = A + globalReadOffsetA_0_0_57_0;
  DATA_TYPE const *globalReadA_0_0_58_0 = A + globalReadOffsetA_0_0_58_0;
  DATA_TYPE const *globalReadA_0_0_59_0 = A + globalReadOffsetA_0_0_59_0;
  DATA_TYPE const *globalReadA_0_0_60_0 = A + globalReadOffsetA_0_0_60_0;
  DATA_TYPE const *globalReadA_0_0_61_0 = A + globalReadOffsetA_0_0_61_0;
  DATA_TYPE const *globalReadA_0_0_62_0 = A + globalReadOffsetA_0_0_62_0;
  DATA_TYPE const *globalReadA_0_0_63_0 = A + globalReadOffsetA_0_0_63_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 8*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 9*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 10*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 11*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 12*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 13*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 14*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 15*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_16_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 16*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_17_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 17*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_18_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 18*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_19_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 19*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_20_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 20*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_21_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 21*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_22_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 22*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_23_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 23*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_24_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 24*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_25_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 25*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_26_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 26*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_27_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 27*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_28_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 28*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_29_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 29*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_30_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 30*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_31_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 31*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_32_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 32*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_33_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 33*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_34_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 34*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_35_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 35*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_36_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 36*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_37_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 37*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_38_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 38*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_39_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 39*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_40_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 40*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_41_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 41*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_42_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 42*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_43_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 43*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_44_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 44*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_45_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 45*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_46_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 46*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_47_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 47*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_48_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 48*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_49_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 49*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_50_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 50*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_51_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 51*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_52_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 52*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_53_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 53*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_54_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 54*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_55_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 55*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_56_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 56*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_57_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 57*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_58_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 58*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_59_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 59*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_60_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 60*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_61_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 61*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_62_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 62*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_63_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 63*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_0_0_15_0;
  DATA_TYPE *localWriteA_0_0_16_0;
  DATA_TYPE *localWriteA_0_0_17_0;
  DATA_TYPE *localWriteA_0_0_18_0;
  DATA_TYPE *localWriteA_0_0_19_0;
  DATA_TYPE *localWriteA_0_0_20_0;
  DATA_TYPE *localWriteA_0_0_21_0;
  DATA_TYPE *localWriteA_0_0_22_0;
  DATA_TYPE *localWriteA_0_0_23_0;
  DATA_TYPE *localWriteA_0_0_24_0;
  DATA_TYPE *localWriteA_0_0_25_0;
  DATA_TYPE *localWriteA_0_0_26_0;
  DATA_TYPE *localWriteA_0_0_27_0;
  DATA_TYPE *localWriteA_0_0_28_0;
  DATA_TYPE *localWriteA_0_0_29_0;
  DATA_TYPE *localWriteA_0_0_30_0;
  DATA_TYPE *localWriteA_0_0_31_0;
  DATA_TYPE *localWriteA_0_0_32_0;
  DATA_TYPE *localWriteA_0_0_33_0;
  DATA_TYPE *localWriteA_0_0_34_0;
  DATA_TYPE *localWriteA_0_0_35_0;
  DATA_TYPE *localWriteA_0_0_36_0;
  DATA_TYPE *localWriteA_0_0_37_0;
  DATA_TYPE *localWriteA_0_0_38_0;
  DATA_TYPE *localWriteA_0_0_39_0;
  DATA_TYPE *localWriteA_0_0_40_0;
  DATA_TYPE *localWriteA_0_0_41_0;
  DATA_TYPE *localWriteA_0_0_42_0;
  DATA_TYPE *localWriteA_0_0_43_0;
  DATA_TYPE *localWriteA_0_0_44_0;
  DATA_TYPE *localWriteA_0_0_45_0;
  DATA_TYPE *localWriteA_0_0_46_0;
  DATA_TYPE *localWriteA_0_0_47_0;
  DATA_TYPE *localWriteA_0_0_48_0;
  DATA_TYPE *localWriteA_0_0_49_0;
  DATA_TYPE *localWriteA_0_0_50_0;
  DATA_TYPE *localWriteA_0_0_51_0;
  DATA_TYPE *localWriteA_0_0_52_0;
  DATA_TYPE *localWriteA_0_0_53_0;
  DATA_TYPE *localWriteA_0_0_54_0;
  DATA_TYPE *localWriteA_0_0_55_0;
  DATA_TYPE *localWriteA_0_0_56_0;
  DATA_TYPE *localWriteA_0_0_57_0;
  DATA_TYPE *localWriteA_0_0_58_0;
  DATA_TYPE *localWriteA_0_0_59_0;
  DATA_TYPE *localWriteA_0_0_60_0;
  DATA_TYPE *localWriteA_0_0_61_0;
  DATA_TYPE *localWriteA_0_0_62_0;
  DATA_TYPE *localWriteA_0_0_63_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);
  localWriteA_0_0_16_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_16_0);
  localWriteA_0_0_17_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_17_0);
  localWriteA_0_0_18_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_18_0);
  localWriteA_0_0_19_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_19_0);
  localWriteA_0_0_20_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_20_0);
  localWriteA_0_0_21_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_21_0);
  localWriteA_0_0_22_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_22_0);
  localWriteA_0_0_23_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_23_0);
  localWriteA_0_0_24_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_24_0);
  localWriteA_0_0_25_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_25_0);
  localWriteA_0_0_26_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_26_0);
  localWriteA_0_0_27_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_27_0);
  localWriteA_0_0_28_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_28_0);
  localWriteA_0_0_29_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_29_0);
  localWriteA_0_0_30_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_30_0);
  localWriteA_0_0_31_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_31_0);
  localWriteA_0_0_32_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_32_0);
  localWriteA_0_0_33_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_33_0);
  localWriteA_0_0_34_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_34_0);
  localWriteA_0_0_35_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_35_0);
  localWriteA_0_0_36_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_36_0);
  localWriteA_0_0_37_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_37_0);
  localWriteA_0_0_38_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_38_0);
  localWriteA_0_0_39_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_39_0);
  localWriteA_0_0_40_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_40_0);
  localWriteA_0_0_41_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_41_0);
  localWriteA_0_0_42_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_42_0);
  localWriteA_0_0_43_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_43_0);
  localWriteA_0_0_44_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_44_0);
  localWriteA_0_0_45_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_45_0);
  localWriteA_0_0_46_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_46_0);
  localWriteA_0_0_47_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_47_0);
  localWriteA_0_0_48_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_48_0);
  localWriteA_0_0_49_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_49_0);
  localWriteA_0_0_50_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_50_0);
  localWriteA_0_0_51_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_51_0);
  localWriteA_0_0_52_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_52_0);
  localWriteA_0_0_53_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_53_0);
  localWriteA_0_0_54_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_54_0);
  localWriteA_0_0_55_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_55_0);
  localWriteA_0_0_56_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_56_0);
  localWriteA_0_0_57_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_57_0);
  localWriteA_0_0_58_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_58_0);
  localWriteA_0_0_59_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_59_0);
  localWriteA_0_0_60_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_60_0);
  localWriteA_0_0_61_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_61_0);
  localWriteA_0_0_62_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_62_0);
  localWriteA_0_0_63_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_63_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);
    a_0_0_16_0 = *(globalReadA_0_0_16_0 + 0);
    a_0_0_17_0 = *(globalReadA_0_0_17_0 + 0);
    a_0_0_18_0 = *(globalReadA_0_0_18_0 + 0);
    a_0_0_19_0 = *(globalReadA_0_0_19_0 + 0);
    a_0_0_20_0 = *(globalReadA_0_0_20_0 + 0);
    a_0_0_21_0 = *(globalReadA_0_0_21_0 + 0);
    a_0_0_22_0 = *(globalReadA_0_0_22_0 + 0);
    a_0_0_23_0 = *(globalReadA_0_0_23_0 + 0);
    a_0_0_24_0 = *(globalReadA_0_0_24_0 + 0);
    a_0_0_25_0 = *(globalReadA_0_0_25_0 + 0);
    a_0_0_26_0 = *(globalReadA_0_0_26_0 + 0);
    a_0_0_27_0 = *(globalReadA_0_0_27_0 + 0);
    a_0_0_28_0 = *(globalReadA_0_0_28_0 + 0);
    a_0_0_29_0 = *(globalReadA_0_0_29_0 + 0);
    a_0_0_30_0 = *(globalReadA_0_0_30_0 + 0);
    a_0_0_31_0 = *(globalReadA_0_0_31_0 + 0);
    a_0_0_32_0 = *(globalReadA_0_0_32_0 + 0);
    a_0_0_33_0 = *(globalReadA_0_0_33_0 + 0);
    a_0_0_34_0 = *(globalReadA_0_0_34_0 + 0);
    a_0_0_35_0 = *(globalReadA_0_0_35_0 + 0);
    a_0_0_36_0 = *(globalReadA_0_0_36_0 + 0);
    a_0_0_37_0 = *(globalReadA_0_0_37_0 + 0);
    a_0_0_38_0 = *(globalReadA_0_0_38_0 + 0);
    a_0_0_39_0 = *(globalReadA_0_0_39_0 + 0);
    a_0_0_40_0 = *(globalReadA_0_0_40_0 + 0);
    a_0_0_41_0 = *(globalReadA_0_0_41_0 + 0);
    a_0_0_42_0 = *(globalReadA_0_0_42_0 + 0);
    a_0_0_43_0 = *(globalReadA_0_0_43_0 + 0);
    a_0_0_44_0 = *(globalReadA_0_0_44_0 + 0);
    a_0_0_45_0 = *(globalReadA_0_0_45_0 + 0);
    a_0_0_46_0 = *(globalReadA_0_0_46_0 + 0);
    a_0_0_47_0 = *(globalReadA_0_0_47_0 + 0);
    a_0_0_48_0 = *(globalReadA_0_0_48_0 + 0);
    a_0_0_49_0 = *(globalReadA_0_0_49_0 + 0);
    a_0_0_50_0 = *(globalReadA_0_0_50_0 + 0);
    a_0_0_51_0 = *(globalReadA_0_0_51_0 + 0);
    a_0_0_52_0 = *(globalReadA_0_0_52_0 + 0);
    a_0_0_53_0 = *(globalReadA_0_0_53_0 + 0);
    a_0_0_54_0 = *(globalReadA_0_0_54_0 + 0);
    a_0_0_55_0 = *(globalReadA_0_0_55_0 + 0);
    a_0_0_56_0 = *(globalReadA_0_0_56_0 + 0);
    a_0_0_57_0 = *(globalReadA_0_0_57_0 + 0);
    a_0_0_58_0 = *(globalReadA_0_0_58_0 + 0);
    a_0_0_59_0 = *(globalReadA_0_0_59_0 + 0);
    a_0_0_60_0 = *(globalReadA_0_0_60_0 + 0);
    a_0_0_61_0 = *(globalReadA_0_0_61_0 + 0);
    a_0_0_62_0 = *(globalReadA_0_0_62_0 + 0);
    a_0_0_63_0 = *(globalReadA_0_0_63_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    globalReadA_0_0_16_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_16_0) + 1*globalReadIncAL);
    globalReadA_0_0_17_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_17_0) + 1*globalReadIncAL);
    globalReadA_0_0_18_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_18_0) + 1*globalReadIncAL);
    globalReadA_0_0_19_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_19_0) + 1*globalReadIncAL);
    globalReadA_0_0_20_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_20_0) + 1*globalReadIncAL);
    globalReadA_0_0_21_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_21_0) + 1*globalReadIncAL);
    globalReadA_0_0_22_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_22_0) + 1*globalReadIncAL);
    globalReadA_0_0_23_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_23_0) + 1*globalReadIncAL);
    globalReadA_0_0_24_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_24_0) + 1*globalReadIncAL);
    globalReadA_0_0_25_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_25_0) + 1*globalReadIncAL);
    globalReadA_0_0_26_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_26_0) + 1*globalReadIncAL);
    globalReadA_0_0_27_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_27_0) + 1*globalReadIncAL);
    globalReadA_0_0_28_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_28_0) + 1*globalReadIncAL);
    globalReadA_0_0_29_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_29_0) + 1*globalReadIncAL);
    globalReadA_0_0_30_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_30_0) + 1*globalReadIncAL);
    globalReadA_0_0_31_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_31_0) + 1*globalReadIncAL);
    globalReadA_0_0_32_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_32_0) + 1*globalReadIncAL);
    globalReadA_0_0_33_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_33_0) + 1*globalReadIncAL);
    globalReadA_0_0_34_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_34_0) + 1*globalReadIncAL);
    globalReadA_0_0_35_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_35_0) + 1*globalReadIncAL);
    globalReadA_0_0_36_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_36_0) + 1*globalReadIncAL);
    globalReadA_0_0_37_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_37_0) + 1*globalReadIncAL);
    globalReadA_0_0_38_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_38_0) + 1*globalReadIncAL);
    globalReadA_0_0_39_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_39_0) + 1*globalReadIncAL);
    globalReadA_0_0_40_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_40_0) + 1*globalReadIncAL);
    globalReadA_0_0_41_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_41_0) + 1*globalReadIncAL);
    globalReadA_0_0_42_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_42_0) + 1*globalReadIncAL);
    globalReadA_0_0_43_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_43_0) + 1*globalReadIncAL);
    globalReadA_0_0_44_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_44_0) + 1*globalReadIncAL);
    globalReadA_0_0_45_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_45_0) + 1*globalReadIncAL);
    globalReadA_0_0_46_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_46_0) + 1*globalReadIncAL);
    globalReadA_0_0_47_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_47_0) + 1*globalReadIncAL);
    globalReadA_0_0_48_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_48_0) + 1*globalReadIncAL);
    globalReadA_0_0_49_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_49_0) + 1*globalReadIncAL);
    globalReadA_0_0_50_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_50_0) + 1*globalReadIncAL);
    globalReadA_0_0_51_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_51_0) + 1*globalReadIncAL);
    globalReadA_0_0_52_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_52_0) + 1*globalReadIncAL);
    globalReadA_0_0_53_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_53_0) + 1*globalReadIncAL);
    globalReadA_0_0_54_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_54_0) + 1*globalReadIncAL);
    globalReadA_0_0_55_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_55_0) + 1*globalReadIncAL);
    globalReadA_0_0_56_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_56_0) + 1*globalReadIncAL);
    globalReadA_0_0_57_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_57_0) + 1*globalReadIncAL);
    globalReadA_0_0_58_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_58_0) + 1*globalReadIncAL);
    globalReadA_0_0_59_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_59_0) + 1*globalReadIncAL);
    globalReadA_0_0_60_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_60_0) + 1*globalReadIncAL);
    globalReadA_0_0_61_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_61_0) + 1*globalReadIncAL);
    globalReadA_0_0_62_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_62_0) + 1*globalReadIncAL);
    globalReadA_0_0_63_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_63_0) + 1*globalReadIncAL);

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
    *(localWriteA_0_0_16_0 + 0) = a_0_0_16_0;
    *(localWriteA_0_0_17_0 + 0) = a_0_0_17_0;
    *(localWriteA_0_0_18_0 + 0) = a_0_0_18_0;
    *(localWriteA_0_0_19_0 + 0) = a_0_0_19_0;
    *(localWriteA_0_0_20_0 + 0) = a_0_0_20_0;
    *(localWriteA_0_0_21_0 + 0) = a_0_0_21_0;
    *(localWriteA_0_0_22_0 + 0) = a_0_0_22_0;
    *(localWriteA_0_0_23_0 + 0) = a_0_0_23_0;
    *(localWriteA_0_0_24_0 + 0) = a_0_0_24_0;
    *(localWriteA_0_0_25_0 + 0) = a_0_0_25_0;
    *(localWriteA_0_0_26_0 + 0) = a_0_0_26_0;
    *(localWriteA_0_0_27_0 + 0) = a_0_0_27_0;
    *(localWriteA_0_0_28_0 + 0) = a_0_0_28_0;
    *(localWriteA_0_0_29_0 + 0) = a_0_0_29_0;
    *(localWriteA_0_0_30_0 + 0) = a_0_0_30_0;
    *(localWriteA_0_0_31_0 + 0) = a_0_0_31_0;
    *(localWriteA_0_0_32_0 + 0) = a_0_0_32_0;
    *(localWriteA_0_0_33_0 + 0) = a_0_0_33_0;
    *(localWriteA_0_0_34_0 + 0) = a_0_0_34_0;
    *(localWriteA_0_0_35_0 + 0) = a_0_0_35_0;
    *(localWriteA_0_0_36_0 + 0) = a_0_0_36_0;
    *(localWriteA_0_0_37_0 + 0) = a_0_0_37_0;
    *(localWriteA_0_0_38_0 + 0) = a_0_0_38_0;
    *(localWriteA_0_0_39_0 + 0) = a_0_0_39_0;
    *(localWriteA_0_0_40_0 + 0) = a_0_0_40_0;
    *(localWriteA_0_0_41_0 + 0) = a_0_0_41_0;
    *(localWriteA_0_0_42_0 + 0) = a_0_0_42_0;
    *(localWriteA_0_0_43_0 + 0) = a_0_0_43_0;
    *(localWriteA_0_0_44_0 + 0) = a_0_0_44_0;
    *(localWriteA_0_0_45_0 + 0) = a_0_0_45_0;
    *(localWriteA_0_0_46_0 + 0) = a_0_0_46_0;
    *(localWriteA_0_0_47_0 + 0) = a_0_0_47_0;
    *(localWriteA_0_0_48_0 + 0) = a_0_0_48_0;
    *(localWriteA_0_0_49_0 + 0) = a_0_0_49_0;
    *(localWriteA_0_0_50_0 + 0) = a_0_0_50_0;
    *(localWriteA_0_0_51_0 + 0) = a_0_0_51_0;
    *(localWriteA_0_0_52_0 + 0) = a_0_0_52_0;
    *(localWriteA_0_0_53_0 + 0) = a_0_0_53_0;
    *(localWriteA_0_0_54_0 + 0) = a_0_0_54_0;
    *(localWriteA_0_0_55_0 + 0) = a_0_0_55_0;
    *(localWriteA_0_0_56_0 + 0) = a_0_0_56_0;
    *(localWriteA_0_0_57_0 + 0) = a_0_0_57_0;
    *(localWriteA_0_0_58_0 + 0) = a_0_0_58_0;
    *(localWriteA_0_0_59_0 + 0) = a_0_0_59_0;
    *(localWriteA_0_0_60_0 + 0) = a_0_0_60_0;
    *(localWriteA_0_0_61_0 + 0) = a_0_0_61_0;
    *(localWriteA_0_0_62_0 + 0) = a_0_0_62_0;
    *(localWriteA_0_0_63_0 + 0) = a_0_0_63_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 15 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 16 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 17 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 18 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 19 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 20 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 21 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 22 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 23 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 24 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 25 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 26 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 27 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 28 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 29 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 30 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 31 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 32 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 33 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 34 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 35 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 36 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 37 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 38 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 39 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 40 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 41 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 42 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 43 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 44 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 45 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 46 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 47 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 48 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 49 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 50 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 51 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 52 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 53 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 54 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 55 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 56 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 57 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 58 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 59 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 60 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 61 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 62 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 63 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_1x1




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_0_8_0 = ( globalReadOffsetAL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_0_0_9_0 = ( globalReadOffsetAL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_0_0_10_0 = ( globalReadOffsetAL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_0_0_11_0 = ( globalReadOffsetAL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_0_0_12_0 = ( globalReadOffsetAL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_0_0_13_0 = ( globalReadOffsetAL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_0_0_14_0 = ( globalReadOffsetAL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_0_0_15_0 = ( globalReadOffsetAL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);
  a_0_0_16_0 = ( globalReadOffsetAL_16_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_16_0 + 0);
  a_0_0_17_0 = ( globalReadOffsetAL_17_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_17_0 + 0);
  a_0_0_18_0 = ( globalReadOffsetAL_18_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_18_0 + 0);
  a_0_0_19_0 = ( globalReadOffsetAL_19_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_19_0 + 0);
  a_0_0_20_0 = ( globalReadOffsetAL_20_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_20_0 + 0);
  a_0_0_21_0 = ( globalReadOffsetAL_21_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_21_0 + 0);
  a_0_0_22_0 = ( globalReadOffsetAL_22_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_22_0 + 0);
  a_0_0_23_0 = ( globalReadOffsetAL_23_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_23_0 + 0);
  a_0_0_24_0 = ( globalReadOffsetAL_24_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_24_0 + 0);
  a_0_0_25_0 = ( globalReadOffsetAL_25_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_25_0 + 0);
  a_0_0_26_0 = ( globalReadOffsetAL_26_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_26_0 + 0);
  a_0_0_27_0 = ( globalReadOffsetAL_27_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_27_0 + 0);
  a_0_0_28_0 = ( globalReadOffsetAL_28_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_28_0 + 0);
  a_0_0_29_0 = ( globalReadOffsetAL_29_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_29_0 + 0);
  a_0_0_30_0 = ( globalReadOffsetAL_30_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_30_0 + 0);
  a_0_0_31_0 = ( globalReadOffsetAL_31_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_31_0 + 0);
  a_0_0_32_0 = ( globalReadOffsetAL_32_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_32_0 + 0);
  a_0_0_33_0 = ( globalReadOffsetAL_33_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_33_0 + 0);
  a_0_0_34_0 = ( globalReadOffsetAL_34_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_34_0 + 0);
  a_0_0_35_0 = ( globalReadOffsetAL_35_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_35_0 + 0);
  a_0_0_36_0 = ( globalReadOffsetAL_36_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_36_0 + 0);
  a_0_0_37_0 = ( globalReadOffsetAL_37_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_37_0 + 0);
  a_0_0_38_0 = ( globalReadOffsetAL_38_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_38_0 + 0);
  a_0_0_39_0 = ( globalReadOffsetAL_39_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_39_0 + 0);
  a_0_0_40_0 = ( globalReadOffsetAL_40_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_40_0 + 0);
  a_0_0_41_0 = ( globalReadOffsetAL_41_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_41_0 + 0);
  a_0_0_42_0 = ( globalReadOffsetAL_42_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_42_0 + 0);
  a_0_0_43_0 = ( globalReadOffsetAL_43_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_43_0 + 0);
  a_0_0_44_0 = ( globalReadOffsetAL_44_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_44_0 + 0);
  a_0_0_45_0 = ( globalReadOffsetAL_45_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_45_0 + 0);
  a_0_0_46_0 = ( globalReadOffsetAL_46_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_46_0 + 0);
  a_0_0_47_0 = ( globalReadOffsetAL_47_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_47_0 + 0);
  a_0_0_48_0 = ( globalReadOffsetAL_48_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_48_0 + 0);
  a_0_0_49_0 = ( globalReadOffsetAL_49_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_49_0 + 0);
  a_0_0_50_0 = ( globalReadOffsetAL_50_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_50_0 + 0);
  a_0_0_51_0 = ( globalReadOffsetAL_51_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_51_0 + 0);
  a_0_0_52_0 = ( globalReadOffsetAL_52_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_52_0 + 0);
  a_0_0_53_0 = ( globalReadOffsetAL_53_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_53_0 + 0);
  a_0_0_54_0 = ( globalReadOffsetAL_54_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_54_0 + 0);
  a_0_0_55_0 = ( globalReadOffsetAL_55_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_55_0 + 0);
  a_0_0_56_0 = ( globalReadOffsetAL_56_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_56_0 + 0);
  a_0_0_57_0 = ( globalReadOffsetAL_57_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_57_0 + 0);
  a_0_0_58_0 = ( globalReadOffsetAL_58_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_58_0 + 0);
  a_0_0_59_0 = ( globalReadOffsetAL_59_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_59_0 + 0);
  a_0_0_60_0 = ( globalReadOffsetAL_60_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_60_0 + 0);
  a_0_0_61_0 = ( globalReadOffsetAL_61_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_61_0 + 0);
  a_0_0_62_0 = ( globalReadOffsetAL_62_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_62_0 + 0);
  a_0_0_63_0 = ( globalReadOffsetAL_63_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_63_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);
  localWriteA_0_0_16_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_16_0);
  localWriteA_0_0_17_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_17_0);
  localWriteA_0_0_18_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_18_0);
  localWriteA_0_0_19_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_19_0);
  localWriteA_0_0_20_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_20_0);
  localWriteA_0_0_21_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_21_0);
  localWriteA_0_0_22_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_22_0);
  localWriteA_0_0_23_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_23_0);
  localWriteA_0_0_24_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_24_0);
  localWriteA_0_0_25_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_25_0);
  localWriteA_0_0_26_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_26_0);
  localWriteA_0_0_27_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_27_0);
  localWriteA_0_0_28_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_28_0);
  localWriteA_0_0_29_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_29_0);
  localWriteA_0_0_30_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_30_0);
  localWriteA_0_0_31_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_31_0);
  localWriteA_0_0_32_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_32_0);
  localWriteA_0_0_33_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_33_0);
  localWriteA_0_0_34_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_34_0);
  localWriteA_0_0_35_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_35_0);
  localWriteA_0_0_36_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_36_0);
  localWriteA_0_0_37_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_37_0);
  localWriteA_0_0_38_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_38_0);
  localWriteA_0_0_39_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_39_0);
  localWriteA_0_0_40_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_40_0);
  localWriteA_0_0_41_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_41_0);
  localWriteA_0_0_42_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_42_0);
  localWriteA_0_0_43_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_43_0);
  localWriteA_0_0_44_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_44_0);
  localWriteA_0_0_45_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_45_0);
  localWriteA_0_0_46_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_46_0);
  localWriteA_0_0_47_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_47_0);
  localWriteA_0_0_48_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_48_0);
  localWriteA_0_0_49_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_49_0);
  localWriteA_0_0_50_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_50_0);
  localWriteA_0_0_51_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_51_0);
  localWriteA_0_0_52_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_52_0);
  localWriteA_0_0_53_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_53_0);
  localWriteA_0_0_54_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_54_0);
  localWriteA_0_0_55_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_55_0);
  localWriteA_0_0_56_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_56_0);
  localWriteA_0_0_57_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_57_0);
  localWriteA_0_0_58_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_58_0);
  localWriteA_0_0_59_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_59_0);
  localWriteA_0_0_60_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_60_0);
  localWriteA_0_0_61_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_61_0);
  localWriteA_0_0_62_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_62_0);
  localWriteA_0_0_63_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_63_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
  *(localWriteA_0_0_16_0 + 0) = a_0_0_16_0;
  *(localWriteA_0_0_17_0 + 0) = a_0_0_17_0;
  *(localWriteA_0_0_18_0 + 0) = a_0_0_18_0;
  *(localWriteA_0_0_19_0 + 0) = a_0_0_19_0;
  *(localWriteA_0_0_20_0 + 0) = a_0_0_20_0;
  *(localWriteA_0_0_21_0 + 0) = a_0_0_21_0;
  *(localWriteA_0_0_22_0 + 0) = a_0_0_22_0;
  *(localWriteA_0_0_23_0 + 0) = a_0_0_23_0;
  *(localWriteA_0_0_24_0 + 0) = a_0_0_24_0;
  *(localWriteA_0_0_25_0 + 0) = a_0_0_25_0;
  *(localWriteA_0_0_26_0 + 0) = a_0_0_26_0;
  *(localWriteA_0_0_27_0 + 0) = a_0_0_27_0;
  *(localWriteA_0_0_28_0 + 0) = a_0_0_28_0;
  *(localWriteA_0_0_29_0 + 0) = a_0_0_29_0;
  *(localWriteA_0_0_30_0 + 0) = a_0_0_30_0;
  *(localWriteA_0_0_31_0 + 0) = a_0_0_31_0;
  *(localWriteA_0_0_32_0 + 0) = a_0_0_32_0;
  *(localWriteA_0_0_33_0 + 0) = a_0_0_33_0;
  *(localWriteA_0_0_34_0 + 0) = a_0_0_34_0;
  *(localWriteA_0_0_35_0 + 0) = a_0_0_35_0;
  *(localWriteA_0_0_36_0 + 0) = a_0_0_36_0;
  *(localWriteA_0_0_37_0 + 0) = a_0_0_37_0;
  *(localWriteA_0_0_38_0 + 0) = a_0_0_38_0;
  *(localWriteA_0_0_39_0 + 0) = a_0_0_39_0;
  *(localWriteA_0_0_40_0 + 0) = a_0_0_40_0;
  *(localWriteA_0_0_41_0 + 0) = a_0_0_41_0;
  *(localWriteA_0_0_42_0 + 0) = a_0_0_42_0;
  *(localWriteA_0_0_43_0 + 0) = a_0_0_43_0;
  *(localWriteA_0_0_44_0 + 0) = a_0_0_44_0;
  *(localWriteA_0_0_45_0 + 0) = a_0_0_45_0;
  *(localWriteA_0_0_46_0 + 0) = a_0_0_46_0;
  *(localWriteA_0_0_47_0 + 0) = a_0_0_47_0;
  *(localWriteA_0_0_48_0 + 0) = a_0_0_48_0;
  *(localWriteA_0_0_49_0 + 0) = a_0_0_49_0;
  *(localWriteA_0_0_50_0 + 0) = a_0_0_50_0;
  *(localWriteA_0_0_51_0 + 0) = a_0_0_51_0;
  *(localWriteA_0_0_52_0 + 0) = a_0_0_52_0;
  *(localWriteA_0_0_53_0 + 0) = a_0_0_53_0;
  *(localWriteA_0_0_54_0 + 0) = a_0_0_54_0;
  *(localWriteA_0_0_55_0 + 0) = a_0_0_55_0;
  *(localWriteA_0_0_56_0 + 0) = a_0_0_56_0;
  *(localWriteA_0_0_57_0 + 0) = a_0_0_57_0;
  *(localWriteA_0_0_58_0 + 0) = a_0_0_58_0;
  *(localWriteA_0_0_59_0 + 0) = a_0_0_59_0;
  *(localWriteA_0_0_60_0 + 0) = a_0_0_60_0;
  *(localWriteA_0_0_61_0 + 0) = a_0_0_61_0;
  *(localWriteA_0_0_62_0 + 0) = a_0_0_62_0;
  *(localWriteA_0_0_63_0 + 0) = a_0_0_63_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_1x1

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_1x1
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 64
#define SG1J 1
#define TT0I 1
#define TT1J 1
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 64
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 64
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 4096
#define LDS_NUM_ELEMENTS 4160

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 1x1 micro-tile */
#define MAC_1x1\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_BSS_BH_MT64x1x64_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU0_SUM0_SUS128_SRVW0_SVW4_SNLL0_TT1_1_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG64_1_1_WGM1(
  float *D,
  float const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_bfloat16)(0)
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE a_0_0_16_0;
  DATA_TYPE a_0_0_17_0;
  DATA_TYPE a_0_0_18_0;
  DATA_TYPE a_0_0_19_0;
  DATA_TYPE a_0_0_20_0;
  DATA_TYPE a_0_0_21_0;
  DATA_TYPE a_0_0_22_0;
  DATA_TYPE a_0_0_23_0;
  DATA_TYPE a_0_0_24_0;
  DATA_TYPE a_0_0_25_0;
  DATA_TYPE a_0_0_26_0;
  DATA_TYPE a_0_0_27_0;
  DATA_TYPE a_0_0_28_0;
  DATA_TYPE a_0_0_29_0;
  DATA_TYPE a_0_0_30_0;
  DATA_TYPE a_0_0_31_0;
  DATA_TYPE a_0_0_32_0;
  DATA_TYPE a_0_0_33_0;
  DATA_TYPE a_0_0_34_0;
  DATA_TYPE a_0_0_35_0;
  DATA_TYPE a_0_0_36_0;
  DATA_TYPE a_0_0_37_0;
  DATA_TYPE a_0_0_38_0;
  DATA_TYPE a_0_0_39_0;
  DATA_TYPE a_0_0_40_0;
  DATA_TYPE a_0_0_41_0;
  DATA_TYPE a_0_0_42_0;
  DATA_TYPE a_0_0_43_0;
  DATA_TYPE a_0_0_44_0;
  DATA_TYPE a_0_0_45_0;
  DATA_TYPE a_0_0_46_0;
  DATA_TYPE a_0_0_47_0;
  DATA_TYPE a_0_0_48_0;
  DATA_TYPE a_0_0_49_0;
  DATA_TYPE a_0_0_50_0;
  DATA_TYPE a_0_0_51_0;
  DATA_TYPE a_0_0_52_0;
  DATA_TYPE a_0_0_53_0;
  DATA_TYPE a_0_0_54_0;
  DATA_TYPE a_0_0_55_0;
  DATA_TYPE a_0_0_56_0;
  DATA_TYPE a_0_0_57_0;
  DATA_TYPE a_0_0_58_0;
  DATA_TYPE a_0_0_59_0;
  DATA_TYPE a_0_0_60_0;
  DATA_TYPE a_0_0_61_0;
  DATA_TYPE a_0_0_62_0;
  DATA_TYPE a_0_0_63_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;
  unsigned int globalReadOffsetAL_8_0 = globalReadOffsetAL + 0 + 8*LSPA;
  unsigned int globalReadOffsetAL_9_0 = globalReadOffsetAL + 0 + 9*LSPA;
  unsigned int globalReadOffsetAL_10_0 = globalReadOffsetAL + 0 + 10*LSPA;
  unsigned int globalReadOffsetAL_11_0 = globalReadOffsetAL + 0 + 11*LSPA;
  unsigned int globalReadOffsetAL_12_0 = globalReadOffsetAL + 0 + 12*LSPA;
  unsigned int globalReadOffsetAL_13_0 = globalReadOffsetAL + 0 + 13*LSPA;
  unsigned int globalReadOffsetAL_14_0 = globalReadOffsetAL + 0 + 14*LSPA;
  unsigned int globalReadOffsetAL_15_0 = globalReadOffsetAL + 0 + 15*LSPA;
  unsigned int globalReadOffsetAL_16_0 = globalReadOffsetAL + 0 + 16*LSPA;
  unsigned int globalReadOffsetAL_17_0 = globalReadOffsetAL + 0 + 17*LSPA;
  unsigned int globalReadOffsetAL_18_0 = globalReadOffsetAL + 0 + 18*LSPA;
  unsigned int globalReadOffsetAL_19_0 = globalReadOffsetAL + 0 + 19*LSPA;
  unsigned int globalReadOffsetAL_20_0 = globalReadOffsetAL + 0 + 20*LSPA;
  unsigned int globalReadOffsetAL_21_0 = globalReadOffsetAL + 0 + 21*LSPA;
  unsigned int globalReadOffsetAL_22_0 = globalReadOffsetAL + 0 + 22*LSPA;
  unsigned int globalReadOffsetAL_23_0 = globalReadOffsetAL + 0 + 23*LSPA;
  unsigned int globalReadOffsetAL_24_0 = globalReadOffsetAL + 0 + 24*LSPA;
  unsigned int globalReadOffsetAL_25_0 = globalReadOffsetAL + 0 + 25*LSPA;
  unsigned int globalReadOffsetAL_26_0 = globalReadOffsetAL + 0 + 26*LSPA;
  unsigned int globalReadOffsetAL_27_0 = globalReadOffsetAL + 0 + 27*LSPA;
  unsigned int globalReadOffsetAL_28_0 = globalReadOffsetAL + 0 + 28*LSPA;
  unsigned int globalReadOffsetAL_29_0 = globalReadOffsetAL + 0 + 29*LSPA;
  unsigned int globalReadOffsetAL_30_0 = globalReadOffsetAL + 0 + 30*LSPA;
  unsigned int globalReadOffsetAL_31_0 = globalReadOffsetAL + 0 + 31*LSPA;
  unsigned int globalReadOffsetAL_32_0 = globalReadOffsetAL + 0 + 32*LSPA;
  unsigned int globalReadOffsetAL_33_0 = globalReadOffsetAL + 0 + 33*LSPA;
  unsigned int globalReadOffsetAL_34_0 = globalReadOffsetAL + 0 + 34*LSPA;
  unsigned int globalReadOffsetAL_35_0 = globalReadOffsetAL + 0 + 35*LSPA;
  unsigned int globalReadOffsetAL_36_0 = globalReadOffsetAL + 0 + 36*LSPA;
  unsigned int globalReadOffsetAL_37_0 = globalReadOffsetAL + 0 + 37*LSPA;
  unsigned int globalReadOffsetAL_38_0 = globalReadOffsetAL + 0 + 38*LSPA;
  unsigned int globalReadOffsetAL_39_0 = globalReadOffsetAL + 0 + 39*LSPA;
  unsigned int globalReadOffsetAL_40_0 = globalReadOffsetAL + 0 + 40*LSPA;
  unsigned int globalReadOffsetAL_41_0 = globalReadOffsetAL + 0 + 41*LSPA;
  unsigned int globalReadOffsetAL_42_0 = globalReadOffsetAL + 0 + 42*LSPA;
  unsigned int globalReadOffsetAL_43_0 = globalReadOffsetAL + 0 + 43*LSPA;
  unsigned int globalReadOffsetAL_44_0 = globalReadOffsetAL + 0 + 44*LSPA;
  unsigned int globalReadOffsetAL_45_0 = globalReadOffsetAL + 0 + 45*LSPA;
  unsigned int globalReadOffsetAL_46_0 = globalReadOffsetAL + 0 + 46*LSPA;
  unsigned int globalReadOffsetAL_47_0 = globalReadOffsetAL + 0 + 47*LSPA;
  unsigned int globalReadOffsetAL_48_0 = globalReadOffsetAL + 0 + 48*LSPA;
  unsigned int globalReadOffsetAL_49_0 = globalReadOffsetAL + 0 + 49*LSPA;
  unsigned int globalReadOffsetAL_50_0 = globalReadOffsetAL + 0 + 50*LSPA;
  unsigned int globalReadOffsetAL_51_0 = globalReadOffsetAL + 0 + 51*LSPA;
  unsigned int globalReadOffsetAL_52_0 = globalReadOffsetAL + 0 + 52*LSPA;
  unsigned int globalReadOffsetAL_53_0 = globalReadOffsetAL + 0 + 53*LSPA;
  unsigned int globalReadOffsetAL_54_0 = globalReadOffsetAL + 0 + 54*LSPA;
  unsigned int globalReadOffsetAL_55_0 = globalReadOffsetAL + 0 + 55*LSPA;
  unsigned int globalReadOffsetAL_56_0 = globalReadOffsetAL + 0 + 56*LSPA;
  unsigned int globalReadOffsetAL_57_0 = globalReadOffsetAL + 0 + 57*LSPA;
  unsigned int globalReadOffsetAL_58_0 = globalReadOffsetAL + 0 + 58*LSPA;
  unsigned int globalReadOffsetAL_59_0 = globalReadOffsetAL + 0 + 59*LSPA;
  unsigned int globalReadOffsetAL_60_0 = globalReadOffsetAL + 0 + 60*LSPA;
  unsigned int globalReadOffsetAL_61_0 = globalReadOffsetAL + 0 + 61*LSPA;
  unsigned int globalReadOffsetAL_62_0 = globalReadOffsetAL + 0 + 62*LSPA;
  unsigned int globalReadOffsetAL_63_0 = globalReadOffsetAL + 0 + 63*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_15_0), wgK );
  int64_t globalReadOffsetA_0_0_16_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_16_0), wgK );
  int64_t globalReadOffsetA_0_0_17_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_17_0), wgK );
  int64_t globalReadOffsetA_0_0_18_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_18_0), wgK );
  int64_t globalReadOffsetA_0_0_19_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_19_0), wgK );
  int64_t globalReadOffsetA_0_0_20_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_20_0), wgK );
  int64_t globalReadOffsetA_0_0_21_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_21_0), wgK );
  int64_t globalReadOffsetA_0_0_22_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_22_0), wgK );
  int64_t globalReadOffsetA_0_0_23_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_23_0), wgK );
  int64_t globalReadOffsetA_0_0_24_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_24_0), wgK );
  int64_t globalReadOffsetA_0_0_25_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_25_0), wgK );
  int64_t globalReadOffsetA_0_0_26_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_26_0), wgK );
  int64_t globalReadOffsetA_0_0_27_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_27_0), wgK );
  int64_t globalReadOffsetA_0_0_28_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_28_0), wgK );
  int64_t globalReadOffsetA_0_0_29_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_29_0), wgK );
  int64_t globalReadOffsetA_0_0_30_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_30_0), wgK );
  int64_t globalReadOffsetA_0_0_31_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_31_0), wgK );
  int64_t globalReadOffsetA_0_0_32_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_32_0), wgK );
  int64_t globalReadOffsetA_0_0_33_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_33_0), wgK );
  int64_t globalReadOffsetA_0_0_34_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_34_0), wgK );
  int64_t globalReadOffsetA_0_0_35_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_35_0), wgK );
  int64_t globalReadOffsetA_0_0_36_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_36_0), wgK );
  int64_t globalReadOffsetA_0_0_37_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_37_0), wgK );
  int64_t globalReadOffsetA_0_0_38_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_38_0), wgK );
  int64_t globalReadOffsetA_0_0_39_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_39_0), wgK );
  int64_t globalReadOffsetA_0_0_40_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_40_0), wgK );
  int64_t globalReadOffsetA_0_0_41_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_41_0), wgK );
  int64_t globalReadOffsetA_0_0_42_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_42_0), wgK );
  int64_t globalReadOffsetA_0_0_43_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_43_0), wgK );
  int64_t globalReadOffsetA_0_0_44_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_44_0), wgK );
  int64_t globalReadOffsetA_0_0_45_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_45_0), wgK );
  int64_t globalReadOffsetA_0_0_46_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_46_0), wgK );
  int64_t globalReadOffsetA_0_0_47_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_47_0), wgK );
  int64_t globalReadOffsetA_0_0_48_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_48_0), wgK );
  int64_t globalReadOffsetA_0_0_49_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_49_0), wgK );
  int64_t globalReadOffsetA_0_0_50_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_50_0), wgK );
  int64_t globalReadOffsetA_0_0_51_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_51_0), wgK );
  int64_t globalReadOffsetA_0_0_52_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_52_0), wgK );
  int64_t globalReadOffsetA_0_0_53_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_53_0), wgK );
  int64_t globalReadOffsetA_0_0_54_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_54_0), wgK );
  int64_t globalReadOffsetA_0_0_55_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_55_0), wgK );
  int64_t globalReadOffsetA_0_0_56_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_56_0), wgK );
  int64_t globalReadOffsetA_0_0_57_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_57_0), wgK );
  int64_t globalReadOffsetA_0_0_58_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_58_0), wgK );
  int64_t globalReadOffsetA_0_0_59_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_59_0), wgK );
  int64_t globalReadOffsetA_0_0_60_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_60_0), wgK );
  int64_t globalReadOffsetA_0_0_61_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_61_0), wgK );
  int64_t globalReadOffsetA_0_0_62_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_62_0), wgK );
  int64_t globalReadOffsetA_0_0_63_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_63_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;
  DATA_TYPE const *globalReadA_0_0_16_0 = A + globalReadOffsetA_0_0_16_0;
  DATA_TYPE const *globalReadA_0_0_17_0 = A + globalReadOffsetA_0_0_17_0;
  DATA_TYPE const *globalReadA_0_0_18_0 = A + globalReadOffsetA_0_0_18_0;
  DATA_TYPE const *globalReadA_0_0_19_0 = A + globalReadOffsetA_0_0_19_0;
  DATA_TYPE const *globalReadA_0_0_20_0 = A + globalReadOffsetA_0_0_20_0;
  DATA_TYPE const *globalReadA_0_0_21_0 = A + globalReadOffsetA_0_0_21_0;
  DATA_TYPE const *globalReadA_0_0_22_0 = A + globalReadOffsetA_0_0_22_0;
  DATA_TYPE const *globalReadA_0_0_23_0 = A + globalReadOffsetA_0_0_23_0;
  DATA_TYPE const *globalReadA_0_0_24_0 = A + globalReadOffsetA_0_0_24_0;
  DATA_TYPE const *globalReadA_0_0_25_0 = A + globalReadOffsetA_0_0_25_0;
  DATA_TYPE const *globalReadA_0_0_26_0 = A + globalReadOffsetA_0_0_26_0;
  DATA_TYPE const *globalReadA_0_0_27_0 = A + globalReadOffsetA_0_0_27_0;
  DATA_TYPE const *globalReadA_0_0_28_0 = A + globalReadOffsetA_0_0_28_0;
  DATA_TYPE const *globalReadA_0_0_29_0 = A + globalReadOffsetA_0_0_29_0;
  DATA_TYPE const *globalReadA_0_0_30_0 = A + globalReadOffsetA_0_0_30_0;
  DATA_TYPE const *globalReadA_0_0_31_0 = A + globalReadOffsetA_0_0_31_0;
  DATA_TYPE const *globalReadA_0_0_32_0 = A + globalReadOffsetA_0_0_32_0;
  DATA_TYPE const *globalReadA_0_0_33_0 = A + globalReadOffsetA_0_0_33_0;
  DATA_TYPE const *globalReadA_0_0_34_0 = A + globalReadOffsetA_0_0_34_0;
  DATA_TYPE const *globalReadA_0_0_35_0 = A + globalReadOffsetA_0_0_35_0;
  DATA_TYPE const *globalReadA_0_0_36_0 = A + globalReadOffsetA_0_0_36_0;
  DATA_TYPE const *globalReadA_0_0_37_0 = A + globalReadOffsetA_0_0_37_0;
  DATA_TYPE const *globalReadA_0_0_38_0 = A + globalReadOffsetA_0_0_38_0;
  DATA_TYPE const *globalReadA_0_0_39_0 = A + globalReadOffsetA_0_0_39_0;
  DATA_TYPE const *globalReadA_0_0_40_0 = A + globalReadOffsetA_0_0_40_0;
  DATA_TYPE const *globalReadA_0_0_41_0 = A + globalReadOffsetA_0_0_41_0;
  DATA_TYPE const *globalReadA_0_0_42_0 = A + globalReadOffsetA_0_0_42_0;
  DATA_TYPE const *globalReadA_0_0_43_0 = A + globalReadOffsetA_0_0_43_0;
  DATA_TYPE const *globalReadA_0_0_44_0 = A + globalReadOffsetA_0_0_44_0;
  DATA_TYPE const *globalReadA_0_0_45_0 = A + globalReadOffsetA_0_0_45_0;
  DATA_TYPE const *globalReadA_0_0_46_0 = A + globalReadOffsetA_0_0_46_0;
  DATA_TYPE const *globalReadA_0_0_47_0 = A + globalReadOffsetA_0_0_47_0;
  DATA_TYPE const *globalReadA_0_0_48_0 = A + globalReadOffsetA_0_0_48_0;
  DATA_TYPE const *globalReadA_0_0_49_0 = A + globalReadOffsetA_0_0_49_0;
  DATA_TYPE const *globalReadA_0_0_50_0 = A + globalReadOffsetA_0_0_50_0;
  DATA_TYPE const *globalReadA_0_0_51_0 = A + globalReadOffsetA_0_0_51_0;
  DATA_TYPE const *globalReadA_0_0_52_0 = A + globalReadOffsetA_0_0_52_0;
  DATA_TYPE const *globalReadA_0_0_53_0 = A + globalReadOffsetA_0_0_53_0;
  DATA_TYPE const *globalReadA_0_0_54_0 = A + globalReadOffsetA_0_0_54_0;
  DATA_TYPE const *globalReadA_0_0_55_0 = A + globalReadOffsetA_0_0_55_0;
  DATA_TYPE const *globalReadA_0_0_56_0 = A + globalReadOffsetA_0_0_56_0;
  DATA_TYPE const *globalReadA_0_0_57_0 = A + globalReadOffsetA_0_0_57_0;
  DATA_TYPE const *globalReadA_0_0_58_0 = A + globalReadOffsetA_0_0_58_0;
  DATA_TYPE const *globalReadA_0_0_59_0 = A + globalReadOffsetA_0_0_59_0;
  DATA_TYPE const *globalReadA_0_0_60_0 = A + globalReadOffsetA_0_0_60_0;
  DATA_TYPE const *globalReadA_0_0_61_0 = A + globalReadOffsetA_0_0_61_0;
  DATA_TYPE const *globalReadA_0_0_62_0 = A + globalReadOffsetA_0_0_62_0;
  DATA_TYPE const *globalReadA_0_0_63_0 = A + globalReadOffsetA_0_0_63_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 8*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 9*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 10*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 11*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 12*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 13*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 14*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 15*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_16_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 16*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_17_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 17*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_18_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 18*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_19_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 19*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_20_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 20*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_21_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 21*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_22_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 22*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_23_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 23*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_24_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 24*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_25_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 25*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_26_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 26*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_27_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 27*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_28_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 28*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_29_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 29*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_30_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 30*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_31_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 31*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_32_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 32*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_33_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 33*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_34_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 34*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_35_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 35*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_36_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 36*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_37_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 37*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_38_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 38*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_39_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 39*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_40_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 40*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_41_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 41*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_42_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 42*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_43_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 43*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_44_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 44*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_45_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 45*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_46_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 46*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_47_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 47*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_48_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 48*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_49_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 49*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_50_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 50*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_51_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 51*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_52_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 52*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_53_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 53*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_54_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 54*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_55_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 55*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_56_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 56*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_57_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 57*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_58_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 58*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_59_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 59*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_60_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 60*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_61_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 61*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_62_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 62*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_63_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 63*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_0_0_15_0;
  DATA_TYPE *localWriteA_0_0_16_0;
  DATA_TYPE *localWriteA_0_0_17_0;
  DATA_TYPE *localWriteA_0_0_18_0;
  DATA_TYPE *localWriteA_0_0_19_0;
  DATA_TYPE *localWriteA_0_0_20_0;
  DATA_TYPE *localWriteA_0_0_21_0;
  DATA_TYPE *localWriteA_0_0_22_0;
  DATA_TYPE *localWriteA_0_0_23_0;
  DATA_TYPE *localWriteA_0_0_24_0;
  DATA_TYPE *localWriteA_0_0_25_0;
  DATA_TYPE *localWriteA_0_0_26_0;
  DATA_TYPE *localWriteA_0_0_27_0;
  DATA_TYPE *localWriteA_0_0_28_0;
  DATA_TYPE *localWriteA_0_0_29_0;
  DATA_TYPE *localWriteA_0_0_30_0;
  DATA_TYPE *localWriteA_0_0_31_0;
  DATA_TYPE *localWriteA_0_0_32_0;
  DATA_TYPE *localWriteA_0_0_33_0;
  DATA_TYPE *localWriteA_0_0_34_0;
  DATA_TYPE *localWriteA_0_0_35_0;
  DATA_TYPE *localWriteA_0_0_36_0;
  DATA_TYPE *localWriteA_0_0_37_0;
  DATA_TYPE *localWriteA_0_0_38_0;
  DATA_TYPE *localWriteA_0_0_39_0;
  DATA_TYPE *localWriteA_0_0_40_0;
  DATA_TYPE *localWriteA_0_0_41_0;
  DATA_TYPE *localWriteA_0_0_42_0;
  DATA_TYPE *localWriteA_0_0_43_0;
  DATA_TYPE *localWriteA_0_0_44_0;
  DATA_TYPE *localWriteA_0_0_45_0;
  DATA_TYPE *localWriteA_0_0_46_0;
  DATA_TYPE *localWriteA_0_0_47_0;
  DATA_TYPE *localWriteA_0_0_48_0;
  DATA_TYPE *localWriteA_0_0_49_0;
  DATA_TYPE *localWriteA_0_0_50_0;
  DATA_TYPE *localWriteA_0_0_51_0;
  DATA_TYPE *localWriteA_0_0_52_0;
  DATA_TYPE *localWriteA_0_0_53_0;
  DATA_TYPE *localWriteA_0_0_54_0;
  DATA_TYPE *localWriteA_0_0_55_0;
  DATA_TYPE *localWriteA_0_0_56_0;
  DATA_TYPE *localWriteA_0_0_57_0;
  DATA_TYPE *localWriteA_0_0_58_0;
  DATA_TYPE *localWriteA_0_0_59_0;
  DATA_TYPE *localWriteA_0_0_60_0;
  DATA_TYPE *localWriteA_0_0_61_0;
  DATA_TYPE *localWriteA_0_0_62_0;
  DATA_TYPE *localWriteA_0_0_63_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);
  localWriteA_0_0_16_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_16_0);
  localWriteA_0_0_17_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_17_0);
  localWriteA_0_0_18_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_18_0);
  localWriteA_0_0_19_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_19_0);
  localWriteA_0_0_20_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_20_0);
  localWriteA_0_0_21_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_21_0);
  localWriteA_0_0_22_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_22_0);
  localWriteA_0_0_23_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_23_0);
  localWriteA_0_0_24_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_24_0);
  localWriteA_0_0_25_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_25_0);
  localWriteA_0_0_26_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_26_0);
  localWriteA_0_0_27_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_27_0);
  localWriteA_0_0_28_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_28_0);
  localWriteA_0_0_29_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_29_0);
  localWriteA_0_0_30_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_30_0);
  localWriteA_0_0_31_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_31_0);
  localWriteA_0_0_32_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_32_0);
  localWriteA_0_0_33_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_33_0);
  localWriteA_0_0_34_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_34_0);
  localWriteA_0_0_35_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_35_0);
  localWriteA_0_0_36_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_36_0);
  localWriteA_0_0_37_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_37_0);
  localWriteA_0_0_38_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_38_0);
  localWriteA_0_0_39_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_39_0);
  localWriteA_0_0_40_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_40_0);
  localWriteA_0_0_41_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_41_0);
  localWriteA_0_0_42_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_42_0);
  localWriteA_0_0_43_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_43_0);
  localWriteA_0_0_44_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_44_0);
  localWriteA_0_0_45_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_45_0);
  localWriteA_0_0_46_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_46_0);
  localWriteA_0_0_47_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_47_0);
  localWriteA_0_0_48_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_48_0);
  localWriteA_0_0_49_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_49_0);
  localWriteA_0_0_50_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_50_0);
  localWriteA_0_0_51_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_51_0);
  localWriteA_0_0_52_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_52_0);
  localWriteA_0_0_53_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_53_0);
  localWriteA_0_0_54_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_54_0);
  localWriteA_0_0_55_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_55_0);
  localWriteA_0_0_56_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_56_0);
  localWriteA_0_0_57_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_57_0);
  localWriteA_0_0_58_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_58_0);
  localWriteA_0_0_59_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_59_0);
  localWriteA_0_0_60_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_60_0);
  localWriteA_0_0_61_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_61_0);
  localWriteA_0_0_62_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_62_0);
  localWriteA_0_0_63_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_63_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);
    a_0_0_16_0 = *(globalReadA_0_0_16_0 + 0);
    a_0_0_17_0 = *(globalReadA_0_0_17_0 + 0);
    a_0_0_18_0 = *(globalReadA_0_0_18_0 + 0);
    a_0_0_19_0 = *(globalReadA_0_0_19_0 + 0);
    a_0_0_20_0 = *(globalReadA_0_0_20_0 + 0);
    a_0_0_21_0 = *(globalReadA_0_0_21_0 + 0);
    a_0_0_22_0 = *(globalReadA_0_0_22_0 + 0);
    a_0_0_23_0 = *(globalReadA_0_0_23_0 + 0);
    a_0_0_24_0 = *(globalReadA_0_0_24_0 + 0);
    a_0_0_25_0 = *(globalReadA_0_0_25_0 + 0);
    a_0_0_26_0 = *(globalReadA_0_0_26_0 + 0);
    a_0_0_27_0 = *(globalReadA_0_0_27_0 + 0);
    a_0_0_28_0 = *(globalReadA_0_0_28_0 + 0);
    a_0_0_29_0 = *(globalReadA_0_0_29_0 + 0);
    a_0_0_30_0 = *(globalReadA_0_0_30_0 + 0);
    a_0_0_31_0 = *(globalReadA_0_0_31_0 + 0);
    a_0_0_32_0 = *(globalReadA_0_0_32_0 + 0);
    a_0_0_33_0 = *(globalReadA_0_0_33_0 + 0);
    a_0_0_34_0 = *(globalReadA_0_0_34_0 + 0);
    a_0_0_35_0 = *(globalReadA_0_0_35_0 + 0);
    a_0_0_36_0 = *(globalReadA_0_0_36_0 + 0);
    a_0_0_37_0 = *(globalReadA_0_0_37_0 + 0);
    a_0_0_38_0 = *(globalReadA_0_0_38_0 + 0);
    a_0_0_39_0 = *(globalReadA_0_0_39_0 + 0);
    a_0_0_40_0 = *(globalReadA_0_0_40_0 + 0);
    a_0_0_41_0 = *(globalReadA_0_0_41_0 + 0);
    a_0_0_42_0 = *(globalReadA_0_0_42_0 + 0);
    a_0_0_43_0 = *(globalReadA_0_0_43_0 + 0);
    a_0_0_44_0 = *(globalReadA_0_0_44_0 + 0);
    a_0_0_45_0 = *(globalReadA_0_0_45_0 + 0);
    a_0_0_46_0 = *(globalReadA_0_0_46_0 + 0);
    a_0_0_47_0 = *(globalReadA_0_0_47_0 + 0);
    a_0_0_48_0 = *(globalReadA_0_0_48_0 + 0);
    a_0_0_49_0 = *(globalReadA_0_0_49_0 + 0);
    a_0_0_50_0 = *(globalReadA_0_0_50_0 + 0);
    a_0_0_51_0 = *(globalReadA_0_0_51_0 + 0);
    a_0_0_52_0 = *(globalReadA_0_0_52_0 + 0);
    a_0_0_53_0 = *(globalReadA_0_0_53_0 + 0);
    a_0_0_54_0 = *(globalReadA_0_0_54_0 + 0);
    a_0_0_55_0 = *(globalReadA_0_0_55_0 + 0);
    a_0_0_56_0 = *(globalReadA_0_0_56_0 + 0);
    a_0_0_57_0 = *(globalReadA_0_0_57_0 + 0);
    a_0_0_58_0 = *(globalReadA_0_0_58_0 + 0);
    a_0_0_59_0 = *(globalReadA_0_0_59_0 + 0);
    a_0_0_60_0 = *(globalReadA_0_0_60_0 + 0);
    a_0_0_61_0 = *(globalReadA_0_0_61_0 + 0);
    a_0_0_62_0 = *(globalReadA_0_0_62_0 + 0);
    a_0_0_63_0 = *(globalReadA_0_0_63_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    globalReadA_0_0_16_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_16_0) + 1*globalReadIncAL);
    globalReadA_0_0_17_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_17_0) + 1*globalReadIncAL);
    globalReadA_0_0_18_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_18_0) + 1*globalReadIncAL);
    globalReadA_0_0_19_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_19_0) + 1*globalReadIncAL);
    globalReadA_0_0_20_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_20_0) + 1*globalReadIncAL);
    globalReadA_0_0_21_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_21_0) + 1*globalReadIncAL);
    globalReadA_0_0_22_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_22_0) + 1*globalReadIncAL);
    globalReadA_0_0_23_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_23_0) + 1*globalReadIncAL);
    globalReadA_0_0_24_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_24_0) + 1*globalReadIncAL);
    globalReadA_0_0_25_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_25_0) + 1*globalReadIncAL);
    globalReadA_0_0_26_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_26_0) + 1*globalReadIncAL);
    globalReadA_0_0_27_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_27_0) + 1*globalReadIncAL);
    globalReadA_0_0_28_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_28_0) + 1*globalReadIncAL);
    globalReadA_0_0_29_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_29_0) + 1*globalReadIncAL);
    globalReadA_0_0_30_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_30_0) + 1*globalReadIncAL);
    globalReadA_0_0_31_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_31_0) + 1*globalReadIncAL);
    globalReadA_0_0_32_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_32_0) + 1*globalReadIncAL);
    globalReadA_0_0_33_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_33_0) + 1*globalReadIncAL);
    globalReadA_0_0_34_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_34_0) + 1*globalReadIncAL);
    globalReadA_0_0_35_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_35_0) + 1*globalReadIncAL);
    globalReadA_0_0_36_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_36_0) + 1*globalReadIncAL);
    globalReadA_0_0_37_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_37_0) + 1*globalReadIncAL);
    globalReadA_0_0_38_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_38_0) + 1*globalReadIncAL);
    globalReadA_0_0_39_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_39_0) + 1*globalReadIncAL);
    globalReadA_0_0_40_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_40_0) + 1*globalReadIncAL);
    globalReadA_0_0_41_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_41_0) + 1*globalReadIncAL);
    globalReadA_0_0_42_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_42_0) + 1*globalReadIncAL);
    globalReadA_0_0_43_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_43_0) + 1*globalReadIncAL);
    globalReadA_0_0_44_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_44_0) + 1*globalReadIncAL);
    globalReadA_0_0_45_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_45_0) + 1*globalReadIncAL);
    globalReadA_0_0_46_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_46_0) + 1*globalReadIncAL);
    globalReadA_0_0_47_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_47_0) + 1*globalReadIncAL);
    globalReadA_0_0_48_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_48_0) + 1*globalReadIncAL);
    globalReadA_0_0_49_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_49_0) + 1*globalReadIncAL);
    globalReadA_0_0_50_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_50_0) + 1*globalReadIncAL);
    globalReadA_0_0_51_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_51_0) + 1*globalReadIncAL);
    globalReadA_0_0_52_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_52_0) + 1*globalReadIncAL);
    globalReadA_0_0_53_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_53_0) + 1*globalReadIncAL);
    globalReadA_0_0_54_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_54_0) + 1*globalReadIncAL);
    globalReadA_0_0_55_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_55_0) + 1*globalReadIncAL);
    globalReadA_0_0_56_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_56_0) + 1*globalReadIncAL);
    globalReadA_0_0_57_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_57_0) + 1*globalReadIncAL);
    globalReadA_0_0_58_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_58_0) + 1*globalReadIncAL);
    globalReadA_0_0_59_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_59_0) + 1*globalReadIncAL);
    globalReadA_0_0_60_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_60_0) + 1*globalReadIncAL);
    globalReadA_0_0_61_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_61_0) + 1*globalReadIncAL);
    globalReadA_0_0_62_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_62_0) + 1*globalReadIncAL);
    globalReadA_0_0_63_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_63_0) + 1*globalReadIncAL);

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
    *(localWriteA_0_0_16_0 + 0) = a_0_0_16_0;
    *(localWriteA_0_0_17_0 + 0) = a_0_0_17_0;
    *(localWriteA_0_0_18_0 + 0) = a_0_0_18_0;
    *(localWriteA_0_0_19_0 + 0) = a_0_0_19_0;
    *(localWriteA_0_0_20_0 + 0) = a_0_0_20_0;
    *(localWriteA_0_0_21_0 + 0) = a_0_0_21_0;
    *(localWriteA_0_0_22_0 + 0) = a_0_0_22_0;
    *(localWriteA_0_0_23_0 + 0) = a_0_0_23_0;
    *(localWriteA_0_0_24_0 + 0) = a_0_0_24_0;
    *(localWriteA_0_0_25_0 + 0) = a_0_0_25_0;
    *(localWriteA_0_0_26_0 + 0) = a_0_0_26_0;
    *(localWriteA_0_0_27_0 + 0) = a_0_0_27_0;
    *(localWriteA_0_0_28_0 + 0) = a_0_0_28_0;
    *(localWriteA_0_0_29_0 + 0) = a_0_0_29_0;
    *(localWriteA_0_0_30_0 + 0) = a_0_0_30_0;
    *(localWriteA_0_0_31_0 + 0) = a_0_0_31_0;
    *(localWriteA_0_0_32_0 + 0) = a_0_0_32_0;
    *(localWriteA_0_0_33_0 + 0) = a_0_0_33_0;
    *(localWriteA_0_0_34_0 + 0) = a_0_0_34_0;
    *(localWriteA_0_0_35_0 + 0) = a_0_0_35_0;
    *(localWriteA_0_0_36_0 + 0) = a_0_0_36_0;
    *(localWriteA_0_0_37_0 + 0) = a_0_0_37_0;
    *(localWriteA_0_0_38_0 + 0) = a_0_0_38_0;
    *(localWriteA_0_0_39_0 + 0) = a_0_0_39_0;
    *(localWriteA_0_0_40_0 + 0) = a_0_0_40_0;
    *(localWriteA_0_0_41_0 + 0) = a_0_0_41_0;
    *(localWriteA_0_0_42_0 + 0) = a_0_0_42_0;
    *(localWriteA_0_0_43_0 + 0) = a_0_0_43_0;
    *(localWriteA_0_0_44_0 + 0) = a_0_0_44_0;
    *(localWriteA_0_0_45_0 + 0) = a_0_0_45_0;
    *(localWriteA_0_0_46_0 + 0) = a_0_0_46_0;
    *(localWriteA_0_0_47_0 + 0) = a_0_0_47_0;
    *(localWriteA_0_0_48_0 + 0) = a_0_0_48_0;
    *(localWriteA_0_0_49_0 + 0) = a_0_0_49_0;
    *(localWriteA_0_0_50_0 + 0) = a_0_0_50_0;
    *(localWriteA_0_0_51_0 + 0) = a_0_0_51_0;
    *(localWriteA_0_0_52_0 + 0) = a_0_0_52_0;
    *(localWriteA_0_0_53_0 + 0) = a_0_0_53_0;
    *(localWriteA_0_0_54_0 + 0) = a_0_0_54_0;
    *(localWriteA_0_0_55_0 + 0) = a_0_0_55_0;
    *(localWriteA_0_0_56_0 + 0) = a_0_0_56_0;
    *(localWriteA_0_0_57_0 + 0) = a_0_0_57_0;
    *(localWriteA_0_0_58_0 + 0) = a_0_0_58_0;
    *(localWriteA_0_0_59_0 + 0) = a_0_0_59_0;
    *(localWriteA_0_0_60_0 + 0) = a_0_0_60_0;
    *(localWriteA_0_0_61_0 + 0) = a_0_0_61_0;
    *(localWriteA_0_0_62_0 + 0) = a_0_0_62_0;
    *(localWriteA_0_0_63_0 + 0) = a_0_0_63_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 15 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 16 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 17 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 18 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 19 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 20 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 21 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 22 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 23 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 24 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 25 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 26 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 27 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 28 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 29 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 30 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 31 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 32 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 33 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 34 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 35 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 36 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 37 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 38 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 39 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 40 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 41 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 42 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 43 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 44 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 45 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 46 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 47 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 48 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 49 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 50 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 51 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 52 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 53 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 54 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 55 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 56 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 57 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 58 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 59 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 60 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 61 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 62 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 63 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_1x1




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_0_8_0 = ( globalReadOffsetAL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_0_0_9_0 = ( globalReadOffsetAL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_0_0_10_0 = ( globalReadOffsetAL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_0_0_11_0 = ( globalReadOffsetAL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_0_0_12_0 = ( globalReadOffsetAL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_0_0_13_0 = ( globalReadOffsetAL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_0_0_14_0 = ( globalReadOffsetAL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_0_0_15_0 = ( globalReadOffsetAL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);
  a_0_0_16_0 = ( globalReadOffsetAL_16_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_16_0 + 0);
  a_0_0_17_0 = ( globalReadOffsetAL_17_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_17_0 + 0);
  a_0_0_18_0 = ( globalReadOffsetAL_18_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_18_0 + 0);
  a_0_0_19_0 = ( globalReadOffsetAL_19_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_19_0 + 0);
  a_0_0_20_0 = ( globalReadOffsetAL_20_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_20_0 + 0);
  a_0_0_21_0 = ( globalReadOffsetAL_21_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_21_0 + 0);
  a_0_0_22_0 = ( globalReadOffsetAL_22_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_22_0 + 0);
  a_0_0_23_0 = ( globalReadOffsetAL_23_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_23_0 + 0);
  a_0_0_24_0 = ( globalReadOffsetAL_24_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_24_0 + 0);
  a_0_0_25_0 = ( globalReadOffsetAL_25_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_25_0 + 0);
  a_0_0_26_0 = ( globalReadOffsetAL_26_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_26_0 + 0);
  a_0_0_27_0 = ( globalReadOffsetAL_27_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_27_0 + 0);
  a_0_0_28_0 = ( globalReadOffsetAL_28_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_28_0 + 0);
  a_0_0_29_0 = ( globalReadOffsetAL_29_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_29_0 + 0);
  a_0_0_30_0 = ( globalReadOffsetAL_30_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_30_0 + 0);
  a_0_0_31_0 = ( globalReadOffsetAL_31_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_31_0 + 0);
  a_0_0_32_0 = ( globalReadOffsetAL_32_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_32_0 + 0);
  a_0_0_33_0 = ( globalReadOffsetAL_33_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_33_0 + 0);
  a_0_0_34_0 = ( globalReadOffsetAL_34_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_34_0 + 0);
  a_0_0_35_0 = ( globalReadOffsetAL_35_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_35_0 + 0);
  a_0_0_36_0 = ( globalReadOffsetAL_36_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_36_0 + 0);
  a_0_0_37_0 = ( globalReadOffsetAL_37_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_37_0 + 0);
  a_0_0_38_0 = ( globalReadOffsetAL_38_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_38_0 + 0);
  a_0_0_39_0 = ( globalReadOffsetAL_39_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_39_0 + 0);
  a_0_0_40_0 = ( globalReadOffsetAL_40_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_40_0 + 0);
  a_0_0_41_0 = ( globalReadOffsetAL_41_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_41_0 + 0);
  a_0_0_42_0 = ( globalReadOffsetAL_42_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_42_0 + 0);
  a_0_0_43_0 = ( globalReadOffsetAL_43_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_43_0 + 0);
  a_0_0_44_0 = ( globalReadOffsetAL_44_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_44_0 + 0);
  a_0_0_45_0 = ( globalReadOffsetAL_45_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_45_0 + 0);
  a_0_0_46_0 = ( globalReadOffsetAL_46_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_46_0 + 0);
  a_0_0_47_0 = ( globalReadOffsetAL_47_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_47_0 + 0);
  a_0_0_48_0 = ( globalReadOffsetAL_48_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_48_0 + 0);
  a_0_0_49_0 = ( globalReadOffsetAL_49_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_49_0 + 0);
  a_0_0_50_0 = ( globalReadOffsetAL_50_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_50_0 + 0);
  a_0_0_51_0 = ( globalReadOffsetAL_51_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_51_0 + 0);
  a_0_0_52_0 = ( globalReadOffsetAL_52_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_52_0 + 0);
  a_0_0_53_0 = ( globalReadOffsetAL_53_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_53_0 + 0);
  a_0_0_54_0 = ( globalReadOffsetAL_54_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_54_0 + 0);
  a_0_0_55_0 = ( globalReadOffsetAL_55_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_55_0 + 0);
  a_0_0_56_0 = ( globalReadOffsetAL_56_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_56_0 + 0);
  a_0_0_57_0 = ( globalReadOffsetAL_57_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_57_0 + 0);
  a_0_0_58_0 = ( globalReadOffsetAL_58_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_58_0 + 0);
  a_0_0_59_0 = ( globalReadOffsetAL_59_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_59_0 + 0);
  a_0_0_60_0 = ( globalReadOffsetAL_60_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_60_0 + 0);
  a_0_0_61_0 = ( globalReadOffsetAL_61_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_61_0 + 0);
  a_0_0_62_0 = ( globalReadOffsetAL_62_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_62_0 + 0);
  a_0_0_63_0 = ( globalReadOffsetAL_63_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_63_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);
  localWriteA_0_0_16_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_16_0);
  localWriteA_0_0_17_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_17_0);
  localWriteA_0_0_18_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_18_0);
  localWriteA_0_0_19_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_19_0);
  localWriteA_0_0_20_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_20_0);
  localWriteA_0_0_21_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_21_0);
  localWriteA_0_0_22_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_22_0);
  localWriteA_0_0_23_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_23_0);
  localWriteA_0_0_24_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_24_0);
  localWriteA_0_0_25_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_25_0);
  localWriteA_0_0_26_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_26_0);
  localWriteA_0_0_27_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_27_0);
  localWriteA_0_0_28_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_28_0);
  localWriteA_0_0_29_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_29_0);
  localWriteA_0_0_30_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_30_0);
  localWriteA_0_0_31_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_31_0);
  localWriteA_0_0_32_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_32_0);
  localWriteA_0_0_33_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_33_0);
  localWriteA_0_0_34_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_34_0);
  localWriteA_0_0_35_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_35_0);
  localWriteA_0_0_36_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_36_0);
  localWriteA_0_0_37_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_37_0);
  localWriteA_0_0_38_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_38_0);
  localWriteA_0_0_39_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_39_0);
  localWriteA_0_0_40_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_40_0);
  localWriteA_0_0_41_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_41_0);
  localWriteA_0_0_42_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_42_0);
  localWriteA_0_0_43_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_43_0);
  localWriteA_0_0_44_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_44_0);
  localWriteA_0_0_45_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_45_0);
  localWriteA_0_0_46_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_46_0);
  localWriteA_0_0_47_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_47_0);
  localWriteA_0_0_48_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_48_0);
  localWriteA_0_0_49_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_49_0);
  localWriteA_0_0_50_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_50_0);
  localWriteA_0_0_51_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_51_0);
  localWriteA_0_0_52_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_52_0);
  localWriteA_0_0_53_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_53_0);
  localWriteA_0_0_54_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_54_0);
  localWriteA_0_0_55_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_55_0);
  localWriteA_0_0_56_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_56_0);
  localWriteA_0_0_57_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_57_0);
  localWriteA_0_0_58_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_58_0);
  localWriteA_0_0_59_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_59_0);
  localWriteA_0_0_60_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_60_0);
  localWriteA_0_0_61_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_61_0);
  localWriteA_0_0_62_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_62_0);
  localWriteA_0_0_63_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_63_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
  *(localWriteA_0_0_16_0 + 0) = a_0_0_16_0;
  *(localWriteA_0_0_17_0 + 0) = a_0_0_17_0;
  *(localWriteA_0_0_18_0 + 0) = a_0_0_18_0;
  *(localWriteA_0_0_19_0 + 0) = a_0_0_19_0;
  *(localWriteA_0_0_20_0 + 0) = a_0_0_20_0;
  *(localWriteA_0_0_21_0 + 0) = a_0_0_21_0;
  *(localWriteA_0_0_22_0 + 0) = a_0_0_22_0;
  *(localWriteA_0_0_23_0 + 0) = a_0_0_23_0;
  *(localWriteA_0_0_24_0 + 0) = a_0_0_24_0;
  *(localWriteA_0_0_25_0 + 0) = a_0_0_25_0;
  *(localWriteA_0_0_26_0 + 0) = a_0_0_26_0;
  *(localWriteA_0_0_27_0 + 0) = a_0_0_27_0;
  *(localWriteA_0_0_28_0 + 0) = a_0_0_28_0;
  *(localWriteA_0_0_29_0 + 0) = a_0_0_29_0;
  *(localWriteA_0_0_30_0 + 0) = a_0_0_30_0;
  *(localWriteA_0_0_31_0 + 0) = a_0_0_31_0;
  *(localWriteA_0_0_32_0 + 0) = a_0_0_32_0;
  *(localWriteA_0_0_33_0 + 0) = a_0_0_33_0;
  *(localWriteA_0_0_34_0 + 0) = a_0_0_34_0;
  *(localWriteA_0_0_35_0 + 0) = a_0_0_35_0;
  *(localWriteA_0_0_36_0 + 0) = a_0_0_36_0;
  *(localWriteA_0_0_37_0 + 0) = a_0_0_37_0;
  *(localWriteA_0_0_38_0 + 0) = a_0_0_38_0;
  *(localWriteA_0_0_39_0 + 0) = a_0_0_39_0;
  *(localWriteA_0_0_40_0 + 0) = a_0_0_40_0;
  *(localWriteA_0_0_41_0 + 0) = a_0_0_41_0;
  *(localWriteA_0_0_42_0 + 0) = a_0_0_42_0;
  *(localWriteA_0_0_43_0 + 0) = a_0_0_43_0;
  *(localWriteA_0_0_44_0 + 0) = a_0_0_44_0;
  *(localWriteA_0_0_45_0 + 0) = a_0_0_45_0;
  *(localWriteA_0_0_46_0 + 0) = a_0_0_46_0;
  *(localWriteA_0_0_47_0 + 0) = a_0_0_47_0;
  *(localWriteA_0_0_48_0 + 0) = a_0_0_48_0;
  *(localWriteA_0_0_49_0 + 0) = a_0_0_49_0;
  *(localWriteA_0_0_50_0 + 0) = a_0_0_50_0;
  *(localWriteA_0_0_51_0 + 0) = a_0_0_51_0;
  *(localWriteA_0_0_52_0 + 0) = a_0_0_52_0;
  *(localWriteA_0_0_53_0 + 0) = a_0_0_53_0;
  *(localWriteA_0_0_54_0 + 0) = a_0_0_54_0;
  *(localWriteA_0_0_55_0 + 0) = a_0_0_55_0;
  *(localWriteA_0_0_56_0 + 0) = a_0_0_56_0;
  *(localWriteA_0_0_57_0 + 0) = a_0_0_57_0;
  *(localWriteA_0_0_58_0 + 0) = a_0_0_58_0;
  *(localWriteA_0_0_59_0 + 0) = a_0_0_59_0;
  *(localWriteA_0_0_60_0 + 0) = a_0_0_60_0;
  *(localWriteA_0_0_61_0 + 0) = a_0_0_61_0;
  *(localWriteA_0_0_62_0 + 0) = a_0_0_62_0;
  *(localWriteA_0_0_63_0 + 0) = a_0_0_63_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_1x1

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_1x1
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 64
#define SG1J 1
#define TT0I 1
#define TT1J 1
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 64
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 64
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 4096
#define LDS_NUM_ELEMENTS 4160

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 1x1 micro-tile */
#define MAC_1x1\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_BSS_BH_MT64x1x64_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU0_SUM0_SUS128_SRVW0_SVW4_SNLL0_TT1_1_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG64_1_1_WGM1(
  float *D,
  float const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_bfloat16)(0)
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE a_0_0_16_0;
  DATA_TYPE a_0_0_17_0;
  DATA_TYPE a_0_0_18_0;
  DATA_TYPE a_0_0_19_0;
  DATA_TYPE a_0_0_20_0;
  DATA_TYPE a_0_0_21_0;
  DATA_TYPE a_0_0_22_0;
  DATA_TYPE a_0_0_23_0;
  DATA_TYPE a_0_0_24_0;
  DATA_TYPE a_0_0_25_0;
  DATA_TYPE a_0_0_26_0;
  DATA_TYPE a_0_0_27_0;
  DATA_TYPE a_0_0_28_0;
  DATA_TYPE a_0_0_29_0;
  DATA_TYPE a_0_0_30_0;
  DATA_TYPE a_0_0_31_0;
  DATA_TYPE a_0_0_32_0;
  DATA_TYPE a_0_0_33_0;
  DATA_TYPE a_0_0_34_0;
  DATA_TYPE a_0_0_35_0;
  DATA_TYPE a_0_0_36_0;
  DATA_TYPE a_0_0_37_0;
  DATA_TYPE a_0_0_38_0;
  DATA_TYPE a_0_0_39_0;
  DATA_TYPE a_0_0_40_0;
  DATA_TYPE a_0_0_41_0;
  DATA_TYPE a_0_0_42_0;
  DATA_TYPE a_0_0_43_0;
  DATA_TYPE a_0_0_44_0;
  DATA_TYPE a_0_0_45_0;
  DATA_TYPE a_0_0_46_0;
  DATA_TYPE a_0_0_47_0;
  DATA_TYPE a_0_0_48_0;
  DATA_TYPE a_0_0_49_0;
  DATA_TYPE a_0_0_50_0;
  DATA_TYPE a_0_0_51_0;
  DATA_TYPE a_0_0_52_0;
  DATA_TYPE a_0_0_53_0;
  DATA_TYPE a_0_0_54_0;
  DATA_TYPE a_0_0_55_0;
  DATA_TYPE a_0_0_56_0;
  DATA_TYPE a_0_0_57_0;
  DATA_TYPE a_0_0_58_0;
  DATA_TYPE a_0_0_59_0;
  DATA_TYPE a_0_0_60_0;
  DATA_TYPE a_0_0_61_0;
  DATA_TYPE a_0_0_62_0;
  DATA_TYPE a_0_0_63_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;
  unsigned int globalReadOffsetAL_8_0 = globalReadOffsetAL + 0 + 8*LSPA;
  unsigned int globalReadOffsetAL_9_0 = globalReadOffsetAL + 0 + 9*LSPA;
  unsigned int globalReadOffsetAL_10_0 = globalReadOffsetAL + 0 + 10*LSPA;
  unsigned int globalReadOffsetAL_11_0 = globalReadOffsetAL + 0 + 11*LSPA;
  unsigned int globalReadOffsetAL_12_0 = globalReadOffsetAL + 0 + 12*LSPA;
  unsigned int globalReadOffsetAL_13_0 = globalReadOffsetAL + 0 + 13*LSPA;
  unsigned int globalReadOffsetAL_14_0 = globalReadOffsetAL + 0 + 14*LSPA;
  unsigned int globalReadOffsetAL_15_0 = globalReadOffsetAL + 0 + 15*LSPA;
  unsigned int globalReadOffsetAL_16_0 = globalReadOffsetAL + 0 + 16*LSPA;
  unsigned int globalReadOffsetAL_17_0 = globalReadOffsetAL + 0 + 17*LSPA;
  unsigned int globalReadOffsetAL_18_0 = globalReadOffsetAL + 0 + 18*LSPA;
  unsigned int globalReadOffsetAL_19_0 = globalReadOffsetAL + 0 + 19*LSPA;
  unsigned int globalReadOffsetAL_20_0 = globalReadOffsetAL + 0 + 20*LSPA;
  unsigned int globalReadOffsetAL_21_0 = globalReadOffsetAL + 0 + 21*LSPA;
  unsigned int globalReadOffsetAL_22_0 = globalReadOffsetAL + 0 + 22*LSPA;
  unsigned int globalReadOffsetAL_23_0 = globalReadOffsetAL + 0 + 23*LSPA;
  unsigned int globalReadOffsetAL_24_0 = globalReadOffsetAL + 0 + 24*LSPA;
  unsigned int globalReadOffsetAL_25_0 = globalReadOffsetAL + 0 + 25*LSPA;
  unsigned int globalReadOffsetAL_26_0 = globalReadOffsetAL + 0 + 26*LSPA;
  unsigned int globalReadOffsetAL_27_0 = globalReadOffsetAL + 0 + 27*LSPA;
  unsigned int globalReadOffsetAL_28_0 = globalReadOffsetAL + 0 + 28*LSPA;
  unsigned int globalReadOffsetAL_29_0 = globalReadOffsetAL + 0 + 29*LSPA;
  unsigned int globalReadOffsetAL_30_0 = globalReadOffsetAL + 0 + 30*LSPA;
  unsigned int globalReadOffsetAL_31_0 = globalReadOffsetAL + 0 + 31*LSPA;
  unsigned int globalReadOffsetAL_32_0 = globalReadOffsetAL + 0 + 32*LSPA;
  unsigned int globalReadOffsetAL_33_0 = globalReadOffsetAL + 0 + 33*LSPA;
  unsigned int globalReadOffsetAL_34_0 = globalReadOffsetAL + 0 + 34*LSPA;
  unsigned int globalReadOffsetAL_35_0 = globalReadOffsetAL + 0 + 35*LSPA;
  unsigned int globalReadOffsetAL_36_0 = globalReadOffsetAL + 0 + 36*LSPA;
  unsigned int globalReadOffsetAL_37_0 = globalReadOffsetAL + 0 + 37*LSPA;
  unsigned int globalReadOffsetAL_38_0 = globalReadOffsetAL + 0 + 38*LSPA;
  unsigned int globalReadOffsetAL_39_0 = globalReadOffsetAL + 0 + 39*LSPA;
  unsigned int globalReadOffsetAL_40_0 = globalReadOffsetAL + 0 + 40*LSPA;
  unsigned int globalReadOffsetAL_41_0 = globalReadOffsetAL + 0 + 41*LSPA;
  unsigned int globalReadOffsetAL_42_0 = globalReadOffsetAL + 0 + 42*LSPA;
  unsigned int globalReadOffsetAL_43_0 = globalReadOffsetAL + 0 + 43*LSPA;
  unsigned int globalReadOffsetAL_44_0 = globalReadOffsetAL + 0 + 44*LSPA;
  unsigned int globalReadOffsetAL_45_0 = globalReadOffsetAL + 0 + 45*LSPA;
  unsigned int globalReadOffsetAL_46_0 = globalReadOffsetAL + 0 + 46*LSPA;
  unsigned int globalReadOffsetAL_47_0 = globalReadOffsetAL + 0 + 47*LSPA;
  unsigned int globalReadOffsetAL_48_0 = globalReadOffsetAL + 0 + 48*LSPA;
  unsigned int globalReadOffsetAL_49_0 = globalReadOffsetAL + 0 + 49*LSPA;
  unsigned int globalReadOffsetAL_50_0 = globalReadOffsetAL + 0 + 50*LSPA;
  unsigned int globalReadOffsetAL_51_0 = globalReadOffsetAL + 0 + 51*LSPA;
  unsigned int globalReadOffsetAL_52_0 = globalReadOffsetAL + 0 + 52*LSPA;
  unsigned int globalReadOffsetAL_53_0 = globalReadOffsetAL + 0 + 53*LSPA;
  unsigned int globalReadOffsetAL_54_0 = globalReadOffsetAL + 0 + 54*LSPA;
  unsigned int globalReadOffsetAL_55_0 = globalReadOffsetAL + 0 + 55*LSPA;
  unsigned int globalReadOffsetAL_56_0 = globalReadOffsetAL + 0 + 56*LSPA;
  unsigned int globalReadOffsetAL_57_0 = globalReadOffsetAL + 0 + 57*LSPA;
  unsigned int globalReadOffsetAL_58_0 = globalReadOffsetAL + 0 + 58*LSPA;
  unsigned int globalReadOffsetAL_59_0 = globalReadOffsetAL + 0 + 59*LSPA;
  unsigned int globalReadOffsetAL_60_0 = globalReadOffsetAL + 0 + 60*LSPA;
  unsigned int globalReadOffsetAL_61_0 = globalReadOffsetAL + 0 + 61*LSPA;
  unsigned int globalReadOffsetAL_62_0 = globalReadOffsetAL + 0 + 62*LSPA;
  unsigned int globalReadOffsetAL_63_0 = globalReadOffsetAL + 0 + 63*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_15_0), wgK );
  int64_t globalReadOffsetA_0_0_16_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_16_0), wgK );
  int64_t globalReadOffsetA_0_0_17_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_17_0), wgK );
  int64_t globalReadOffsetA_0_0_18_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_18_0), wgK );
  int64_t globalReadOffsetA_0_0_19_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_19_0), wgK );
  int64_t globalReadOffsetA_0_0_20_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_20_0), wgK );
  int64_t globalReadOffsetA_0_0_21_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_21_0), wgK );
  int64_t globalReadOffsetA_0_0_22_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_22_0), wgK );
  int64_t globalReadOffsetA_0_0_23_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_23_0), wgK );
  int64_t globalReadOffsetA_0_0_24_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_24_0), wgK );
  int64_t globalReadOffsetA_0_0_25_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_25_0), wgK );
  int64_t globalReadOffsetA_0_0_26_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_26_0), wgK );
  int64_t globalReadOffsetA_0_0_27_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_27_0), wgK );
  int64_t globalReadOffsetA_0_0_28_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_28_0), wgK );
  int64_t globalReadOffsetA_0_0_29_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_29_0), wgK );
  int64_t globalReadOffsetA_0_0_30_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_30_0), wgK );
  int64_t globalReadOffsetA_0_0_31_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_31_0), wgK );
  int64_t globalReadOffsetA_0_0_32_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_32_0), wgK );
  int64_t globalReadOffsetA_0_0_33_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_33_0), wgK );
  int64_t globalReadOffsetA_0_0_34_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_34_0), wgK );
  int64_t globalReadOffsetA_0_0_35_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_35_0), wgK );
  int64_t globalReadOffsetA_0_0_36_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_36_0), wgK );
  int64_t globalReadOffsetA_0_0_37_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_37_0), wgK );
  int64_t globalReadOffsetA_0_0_38_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_38_0), wgK );
  int64_t globalReadOffsetA_0_0_39_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_39_0), wgK );
  int64_t globalReadOffsetA_0_0_40_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_40_0), wgK );
  int64_t globalReadOffsetA_0_0_41_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_41_0), wgK );
  int64_t globalReadOffsetA_0_0_42_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_42_0), wgK );
  int64_t globalReadOffsetA_0_0_43_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_43_0), wgK );
  int64_t globalReadOffsetA_0_0_44_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_44_0), wgK );
  int64_t globalReadOffsetA_0_0_45_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_45_0), wgK );
  int64_t globalReadOffsetA_0_0_46_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_46_0), wgK );
  int64_t globalReadOffsetA_0_0_47_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_47_0), wgK );
  int64_t globalReadOffsetA_0_0_48_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_48_0), wgK );
  int64_t globalReadOffsetA_0_0_49_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_49_0), wgK );
  int64_t globalReadOffsetA_0_0_50_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_50_0), wgK );
  int64_t globalReadOffsetA_0_0_51_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_51_0), wgK );
  int64_t globalReadOffsetA_0_0_52_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_52_0), wgK );
  int64_t globalReadOffsetA_0_0_53_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_53_0), wgK );
  int64_t globalReadOffsetA_0_0_54_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_54_0), wgK );
  int64_t globalReadOffsetA_0_0_55_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_55_0), wgK );
  int64_t globalReadOffsetA_0_0_56_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_56_0), wgK );
  int64_t globalReadOffsetA_0_0_57_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_57_0), wgK );
  int64_t globalReadOffsetA_0_0_58_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_58_0), wgK );
  int64_t globalReadOffsetA_0_0_59_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_59_0), wgK );
  int64_t globalReadOffsetA_0_0_60_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_60_0), wgK );
  int64_t globalReadOffsetA_0_0_61_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_61_0), wgK );
  int64_t globalReadOffsetA_0_0_62_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_62_0), wgK );
  int64_t globalReadOffsetA_0_0_63_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_63_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;
  DATA_TYPE const *globalReadA_0_0_16_0 = A + globalReadOffsetA_0_0_16_0;
  DATA_TYPE const *globalReadA_0_0_17_0 = A + globalReadOffsetA_0_0_17_0;
  DATA_TYPE const *globalReadA_0_0_18_0 = A + globalReadOffsetA_0_0_18_0;
  DATA_TYPE const *globalReadA_0_0_19_0 = A + globalReadOffsetA_0_0_19_0;
  DATA_TYPE const *globalReadA_0_0_20_0 = A + globalReadOffsetA_0_0_20_0;
  DATA_TYPE const *globalReadA_0_0_21_0 = A + globalReadOffsetA_0_0_21_0;
  DATA_TYPE const *globalReadA_0_0_22_0 = A + globalReadOffsetA_0_0_22_0;
  DATA_TYPE const *globalReadA_0_0_23_0 = A + globalReadOffsetA_0_0_23_0;
  DATA_TYPE const *globalReadA_0_0_24_0 = A + globalReadOffsetA_0_0_24_0;
  DATA_TYPE const *globalReadA_0_0_25_0 = A + globalReadOffsetA_0_0_25_0;
  DATA_TYPE const *globalReadA_0_0_26_0 = A + globalReadOffsetA_0_0_26_0;
  DATA_TYPE const *globalReadA_0_0_27_0 = A + globalReadOffsetA_0_0_27_0;
  DATA_TYPE const *globalReadA_0_0_28_0 = A + globalReadOffsetA_0_0_28_0;
  DATA_TYPE const *globalReadA_0_0_29_0 = A + globalReadOffsetA_0_0_29_0;
  DATA_TYPE const *globalReadA_0_0_30_0 = A + globalReadOffsetA_0_0_30_0;
  DATA_TYPE const *globalReadA_0_0_31_0 = A + globalReadOffsetA_0_0_31_0;
  DATA_TYPE const *globalReadA_0_0_32_0 = A + globalReadOffsetA_0_0_32_0;
  DATA_TYPE const *globalReadA_0_0_33_0 = A + globalReadOffsetA_0_0_33_0;
  DATA_TYPE const *globalReadA_0_0_34_0 = A + globalReadOffsetA_0_0_34_0;
  DATA_TYPE const *globalReadA_0_0_35_0 = A + globalReadOffsetA_0_0_35_0;
  DATA_TYPE const *globalReadA_0_0_36_0 = A + globalReadOffsetA_0_0_36_0;
  DATA_TYPE const *globalReadA_0_0_37_0 = A + globalReadOffsetA_0_0_37_0;
  DATA_TYPE const *globalReadA_0_0_38_0 = A + globalReadOffsetA_0_0_38_0;
  DATA_TYPE const *globalReadA_0_0_39_0 = A + globalReadOffsetA_0_0_39_0;
  DATA_TYPE const *globalReadA_0_0_40_0 = A + globalReadOffsetA_0_0_40_0;
  DATA_TYPE const *globalReadA_0_0_41_0 = A + globalReadOffsetA_0_0_41_0;
  DATA_TYPE const *globalReadA_0_0_42_0 = A + globalReadOffsetA_0_0_42_0;
  DATA_TYPE const *globalReadA_0_0_43_0 = A + globalReadOffsetA_0_0_43_0;
  DATA_TYPE const *globalReadA_0_0_44_0 = A + globalReadOffsetA_0_0_44_0;
  DATA_TYPE const *globalReadA_0_0_45_0 = A + globalReadOffsetA_0_0_45_0;
  DATA_TYPE const *globalReadA_0_0_46_0 = A + globalReadOffsetA_0_0_46_0;
  DATA_TYPE const *globalReadA_0_0_47_0 = A + globalReadOffsetA_0_0_47_0;
  DATA_TYPE const *globalReadA_0_0_48_0 = A + globalReadOffsetA_0_0_48_0;
  DATA_TYPE const *globalReadA_0_0_49_0 = A + globalReadOffsetA_0_0_49_0;
  DATA_TYPE const *globalReadA_0_0_50_0 = A + globalReadOffsetA_0_0_50_0;
  DATA_TYPE const *globalReadA_0_0_51_0 = A + globalReadOffsetA_0_0_51_0;
  DATA_TYPE const *globalReadA_0_0_52_0 = A + globalReadOffsetA_0_0_52_0;
  DATA_TYPE const *globalReadA_0_0_53_0 = A + globalReadOffsetA_0_0_53_0;
  DATA_TYPE const *globalReadA_0_0_54_0 = A + globalReadOffsetA_0_0_54_0;
  DATA_TYPE const *globalReadA_0_0_55_0 = A + globalReadOffsetA_0_0_55_0;
  DATA_TYPE const *globalReadA_0_0_56_0 = A + globalReadOffsetA_0_0_56_0;
  DATA_TYPE const *globalReadA_0_0_57_0 = A + globalReadOffsetA_0_0_57_0;
  DATA_TYPE const *globalReadA_0_0_58_0 = A + globalReadOffsetA_0_0_58_0;
  DATA_TYPE const *globalReadA_0_0_59_0 = A + globalReadOffsetA_0_0_59_0;
  DATA_TYPE const *globalReadA_0_0_60_0 = A + globalReadOffsetA_0_0_60_0;
  DATA_TYPE const *globalReadA_0_0_61_0 = A + globalReadOffsetA_0_0_61_0;
  DATA_TYPE const *globalReadA_0_0_62_0 = A + globalReadOffsetA_0_0_62_0;
  DATA_TYPE const *globalReadA_0_0_63_0 = A + globalReadOffsetA_0_0_63_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 8*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 9*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 10*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 11*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 12*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 13*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 14*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 15*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_16_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 16*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_17_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 17*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_18_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 18*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_19_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 19*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_20_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 20*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_21_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 21*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_22_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 22*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_23_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 23*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_24_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 24*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_25_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 25*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_26_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 26*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_27_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 27*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_28_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 28*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_29_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 29*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_30_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 30*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_31_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 31*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_32_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 32*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_33_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 33*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_34_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 34*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_35_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 35*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_36_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 36*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_37_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 37*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_38_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 38*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_39_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 39*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_40_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 40*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_41_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 41*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_42_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 42*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_43_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 43*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_44_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 44*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_45_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 45*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_46_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 46*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_47_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 47*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_48_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 48*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_49_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 49*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_50_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 50*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_51_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 51*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_52_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 52*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_53_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 53*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_54_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 54*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_55_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 55*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_56_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 56*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_57_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 57*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_58_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 58*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_59_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 59*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_60_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 60*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_61_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 61*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_62_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 62*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_63_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 63*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_0_0_15_0;
  DATA_TYPE *localWriteA_0_0_16_0;
  DATA_TYPE *localWriteA_0_0_17_0;
  DATA_TYPE *localWriteA_0_0_18_0;
  DATA_TYPE *localWriteA_0_0_19_0;
  DATA_TYPE *localWriteA_0_0_20_0;
  DATA_TYPE *localWriteA_0_0_21_0;
  DATA_TYPE *localWriteA_0_0_22_0;
  DATA_TYPE *localWriteA_0_0_23_0;
  DATA_TYPE *localWriteA_0_0_24_0;
  DATA_TYPE *localWriteA_0_0_25_0;
  DATA_TYPE *localWriteA_0_0_26_0;
  DATA_TYPE *localWriteA_0_0_27_0;
  DATA_TYPE *localWriteA_0_0_28_0;
  DATA_TYPE *localWriteA_0_0_29_0;
  DATA_TYPE *localWriteA_0_0_30_0;
  DATA_TYPE *localWriteA_0_0_31_0;
  DATA_TYPE *localWriteA_0_0_32_0;
  DATA_TYPE *localWriteA_0_0_33_0;
  DATA_TYPE *localWriteA_0_0_34_0;
  DATA_TYPE *localWriteA_0_0_35_0;
  DATA_TYPE *localWriteA_0_0_36_0;
  DATA_TYPE *localWriteA_0_0_37_0;
  DATA_TYPE *localWriteA_0_0_38_0;
  DATA_TYPE *localWriteA_0_0_39_0;
  DATA_TYPE *localWriteA_0_0_40_0;
  DATA_TYPE *localWriteA_0_0_41_0;
  DATA_TYPE *localWriteA_0_0_42_0;
  DATA_TYPE *localWriteA_0_0_43_0;
  DATA_TYPE *localWriteA_0_0_44_0;
  DATA_TYPE *localWriteA_0_0_45_0;
  DATA_TYPE *localWriteA_0_0_46_0;
  DATA_TYPE *localWriteA_0_0_47_0;
  DATA_TYPE *localWriteA_0_0_48_0;
  DATA_TYPE *localWriteA_0_0_49_0;
  DATA_TYPE *localWriteA_0_0_50_0;
  DATA_TYPE *localWriteA_0_0_51_0;
  DATA_TYPE *localWriteA_0_0_52_0;
  DATA_TYPE *localWriteA_0_0_53_0;
  DATA_TYPE *localWriteA_0_0_54_0;
  DATA_TYPE *localWriteA_0_0_55_0;
  DATA_TYPE *localWriteA_0_0_56_0;
  DATA_TYPE *localWriteA_0_0_57_0;
  DATA_TYPE *localWriteA_0_0_58_0;
  DATA_TYPE *localWriteA_0_0_59_0;
  DATA_TYPE *localWriteA_0_0_60_0;
  DATA_TYPE *localWriteA_0_0_61_0;
  DATA_TYPE *localWriteA_0_0_62_0;
  DATA_TYPE *localWriteA_0_0_63_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);
  localWriteA_0_0_16_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_16_0);
  localWriteA_0_0_17_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_17_0);
  localWriteA_0_0_18_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_18_0);
  localWriteA_0_0_19_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_19_0);
  localWriteA_0_0_20_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_20_0);
  localWriteA_0_0_21_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_21_0);
  localWriteA_0_0_22_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_22_0);
  localWriteA_0_0_23_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_23_0);
  localWriteA_0_0_24_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_24_0);
  localWriteA_0_0_25_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_25_0);
  localWriteA_0_0_26_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_26_0);
  localWriteA_0_0_27_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_27_0);
  localWriteA_0_0_28_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_28_0);
  localWriteA_0_0_29_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_29_0);
  localWriteA_0_0_30_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_30_0);
  localWriteA_0_0_31_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_31_0);
  localWriteA_0_0_32_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_32_0);
  localWriteA_0_0_33_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_33_0);
  localWriteA_0_0_34_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_34_0);
  localWriteA_0_0_35_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_35_0);
  localWriteA_0_0_36_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_36_0);
  localWriteA_0_0_37_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_37_0);
  localWriteA_0_0_38_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_38_0);
  localWriteA_0_0_39_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_39_0);
  localWriteA_0_0_40_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_40_0);
  localWriteA_0_0_41_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_41_0);
  localWriteA_0_0_42_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_42_0);
  localWriteA_0_0_43_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_43_0);
  localWriteA_0_0_44_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_44_0);
  localWriteA_0_0_45_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_45_0);
  localWriteA_0_0_46_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_46_0);
  localWriteA_0_0_47_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_47_0);
  localWriteA_0_0_48_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_48_0);
  localWriteA_0_0_49_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_49_0);
  localWriteA_0_0_50_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_50_0);
  localWriteA_0_0_51_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_51_0);
  localWriteA_0_0_52_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_52_0);
  localWriteA_0_0_53_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_53_0);
  localWriteA_0_0_54_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_54_0);
  localWriteA_0_0_55_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_55_0);
  localWriteA_0_0_56_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_56_0);
  localWriteA_0_0_57_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_57_0);
  localWriteA_0_0_58_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_58_0);
  localWriteA_0_0_59_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_59_0);
  localWriteA_0_0_60_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_60_0);
  localWriteA_0_0_61_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_61_0);
  localWriteA_0_0_62_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_62_0);
  localWriteA_0_0_63_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_63_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);
    a_0_0_16_0 = *(globalReadA_0_0_16_0 + 0);
    a_0_0_17_0 = *(globalReadA_0_0_17_0 + 0);
    a_0_0_18_0 = *(globalReadA_0_0_18_0 + 0);
    a_0_0_19_0 = *(globalReadA_0_0_19_0 + 0);
    a_0_0_20_0 = *(globalReadA_0_0_20_0 + 0);
    a_0_0_21_0 = *(globalReadA_0_0_21_0 + 0);
    a_0_0_22_0 = *(globalReadA_0_0_22_0 + 0);
    a_0_0_23_0 = *(globalReadA_0_0_23_0 + 0);
    a_0_0_24_0 = *(globalReadA_0_0_24_0 + 0);
    a_0_0_25_0 = *(globalReadA_0_0_25_0 + 0);
    a_0_0_26_0 = *(globalReadA_0_0_26_0 + 0);
    a_0_0_27_0 = *(globalReadA_0_0_27_0 + 0);
    a_0_0_28_0 = *(globalReadA_0_0_28_0 + 0);
    a_0_0_29_0 = *(globalReadA_0_0_29_0 + 0);
    a_0_0_30_0 = *(globalReadA_0_0_30_0 + 0);
    a_0_0_31_0 = *(globalReadA_0_0_31_0 + 0);
    a_0_0_32_0 = *(globalReadA_0_0_32_0 + 0);
    a_0_0_33_0 = *(globalReadA_0_0_33_0 + 0);
    a_0_0_34_0 = *(globalReadA_0_0_34_0 + 0);
    a_0_0_35_0 = *(globalReadA_0_0_35_0 + 0);
    a_0_0_36_0 = *(globalReadA_0_0_36_0 + 0);
    a_0_0_37_0 = *(globalReadA_0_0_37_0 + 0);
    a_0_0_38_0 = *(globalReadA_0_0_38_0 + 0);
    a_0_0_39_0 = *(globalReadA_0_0_39_0 + 0);
    a_0_0_40_0 = *(globalReadA_0_0_40_0 + 0);
    a_0_0_41_0 = *(globalReadA_0_0_41_0 + 0);
    a_0_0_42_0 = *(globalReadA_0_0_42_0 + 0);
    a_0_0_43_0 = *(globalReadA_0_0_43_0 + 0);
    a_0_0_44_0 = *(globalReadA_0_0_44_0 + 0);
    a_0_0_45_0 = *(globalReadA_0_0_45_0 + 0);
    a_0_0_46_0 = *(globalReadA_0_0_46_0 + 0);
    a_0_0_47_0 = *(globalReadA_0_0_47_0 + 0);
    a_0_0_48_0 = *(globalReadA_0_0_48_0 + 0);
    a_0_0_49_0 = *(globalReadA_0_0_49_0 + 0);
    a_0_0_50_0 = *(globalReadA_0_0_50_0 + 0);
    a_0_0_51_0 = *(globalReadA_0_0_51_0 + 0);
    a_0_0_52_0 = *(globalReadA_0_0_52_0 + 0);
    a_0_0_53_0 = *(globalReadA_0_0_53_0 + 0);
    a_0_0_54_0 = *(globalReadA_0_0_54_0 + 0);
    a_0_0_55_0 = *(globalReadA_0_0_55_0 + 0);
    a_0_0_56_0 = *(globalReadA_0_0_56_0 + 0);
    a_0_0_57_0 = *(globalReadA_0_0_57_0 + 0);
    a_0_0_58_0 = *(globalReadA_0_0_58_0 + 0);
    a_0_0_59_0 = *(globalReadA_0_0_59_0 + 0);
    a_0_0_60_0 = *(globalReadA_0_0_60_0 + 0);
    a_0_0_61_0 = *(globalReadA_0_0_61_0 + 0);
    a_0_0_62_0 = *(globalReadA_0_0_62_0 + 0);
    a_0_0_63_0 = *(globalReadA_0_0_63_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    globalReadA_0_0_16_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_16_0) + 1*globalReadIncAL);
    globalReadA_0_0_17_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_17_0) + 1*globalReadIncAL);
    globalReadA_0_0_18_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_18_0) + 1*globalReadIncAL);
    globalReadA_0_0_19_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_19_0) + 1*globalReadIncAL);
    globalReadA_0_0_20_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_20_0) + 1*globalReadIncAL);
    globalReadA_0_0_21_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_21_0) + 1*globalReadIncAL);
    globalReadA_0_0_22_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_22_0) + 1*globalReadIncAL);
    globalReadA_0_0_23_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_23_0) + 1*globalReadIncAL);
    globalReadA_0_0_24_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_24_0) + 1*globalReadIncAL);
    globalReadA_0_0_25_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_25_0) + 1*globalReadIncAL);
    globalReadA_0_0_26_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_26_0) + 1*globalReadIncAL);
    globalReadA_0_0_27_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_27_0) + 1*globalReadIncAL);
    globalReadA_0_0_28_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_28_0) + 1*globalReadIncAL);
    globalReadA_0_0_29_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_29_0) + 1*globalReadIncAL);
    globalReadA_0_0_30_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_30_0) + 1*globalReadIncAL);
    globalReadA_0_0_31_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_31_0) + 1*globalReadIncAL);
    globalReadA_0_0_32_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_32_0) + 1*globalReadIncAL);
    globalReadA_0_0_33_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_33_0) + 1*globalReadIncAL);
    globalReadA_0_0_34_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_34_0) + 1*globalReadIncAL);
    globalReadA_0_0_35_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_35_0) + 1*globalReadIncAL);
    globalReadA_0_0_36_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_36_0) + 1*globalReadIncAL);
    globalReadA_0_0_37_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_37_0) + 1*globalReadIncAL);
    globalReadA_0_0_38_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_38_0) + 1*globalReadIncAL);
    globalReadA_0_0_39_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_39_0) + 1*globalReadIncAL);
    globalReadA_0_0_40_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_40_0) + 1*globalReadIncAL);
    globalReadA_0_0_41_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_41_0) + 1*globalReadIncAL);
    globalReadA_0_0_42_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_42_0) + 1*globalReadIncAL);
    globalReadA_0_0_43_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_43_0) + 1*globalReadIncAL);
    globalReadA_0_0_44_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_44_0) + 1*globalReadIncAL);
    globalReadA_0_0_45_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_45_0) + 1*globalReadIncAL);
    globalReadA_0_0_46_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_46_0) + 1*globalReadIncAL);
    globalReadA_0_0_47_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_47_0) + 1*globalReadIncAL);
    globalReadA_0_0_48_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_48_0) + 1*globalReadIncAL);
    globalReadA_0_0_49_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_49_0) + 1*globalReadIncAL);
    globalReadA_0_0_50_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_50_0) + 1*globalReadIncAL);
    globalReadA_0_0_51_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_51_0) + 1*globalReadIncAL);
    globalReadA_0_0_52_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_52_0) + 1*globalReadIncAL);
    globalReadA_0_0_53_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_53_0) + 1*globalReadIncAL);
    globalReadA_0_0_54_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_54_0) + 1*globalReadIncAL);
    globalReadA_0_0_55_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_55_0) + 1*globalReadIncAL);
    globalReadA_0_0_56_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_56_0) + 1*globalReadIncAL);
    globalReadA_0_0_57_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_57_0) + 1*globalReadIncAL);
    globalReadA_0_0_58_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_58_0) + 1*globalReadIncAL);
    globalReadA_0_0_59_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_59_0) + 1*globalReadIncAL);
    globalReadA_0_0_60_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_60_0) + 1*globalReadIncAL);
    globalReadA_0_0_61_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_61_0) + 1*globalReadIncAL);
    globalReadA_0_0_62_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_62_0) + 1*globalReadIncAL);
    globalReadA_0_0_63_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_63_0) + 1*globalReadIncAL);

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
    *(localWriteA_0_0_16_0 + 0) = a_0_0_16_0;
    *(localWriteA_0_0_17_0 + 0) = a_0_0_17_0;
    *(localWriteA_0_0_18_0 + 0) = a_0_0_18_0;
    *(localWriteA_0_0_19_0 + 0) = a_0_0_19_0;
    *(localWriteA_0_0_20_0 + 0) = a_0_0_20_0;
    *(localWriteA_0_0_21_0 + 0) = a_0_0_21_0;
    *(localWriteA_0_0_22_0 + 0) = a_0_0_22_0;
    *(localWriteA_0_0_23_0 + 0) = a_0_0_23_0;
    *(localWriteA_0_0_24_0 + 0) = a_0_0_24_0;
    *(localWriteA_0_0_25_0 + 0) = a_0_0_25_0;
    *(localWriteA_0_0_26_0 + 0) = a_0_0_26_0;
    *(localWriteA_0_0_27_0 + 0) = a_0_0_27_0;
    *(localWriteA_0_0_28_0 + 0) = a_0_0_28_0;
    *(localWriteA_0_0_29_0 + 0) = a_0_0_29_0;
    *(localWriteA_0_0_30_0 + 0) = a_0_0_30_0;
    *(localWriteA_0_0_31_0 + 0) = a_0_0_31_0;
    *(localWriteA_0_0_32_0 + 0) = a_0_0_32_0;
    *(localWriteA_0_0_33_0 + 0) = a_0_0_33_0;
    *(localWriteA_0_0_34_0 + 0) = a_0_0_34_0;
    *(localWriteA_0_0_35_0 + 0) = a_0_0_35_0;
    *(localWriteA_0_0_36_0 + 0) = a_0_0_36_0;
    *(localWriteA_0_0_37_0 + 0) = a_0_0_37_0;
    *(localWriteA_0_0_38_0 + 0) = a_0_0_38_0;
    *(localWriteA_0_0_39_0 + 0) = a_0_0_39_0;
    *(localWriteA_0_0_40_0 + 0) = a_0_0_40_0;
    *(localWriteA_0_0_41_0 + 0) = a_0_0_41_0;
    *(localWriteA_0_0_42_0 + 0) = a_0_0_42_0;
    *(localWriteA_0_0_43_0 + 0) = a_0_0_43_0;
    *(localWriteA_0_0_44_0 + 0) = a_0_0_44_0;
    *(localWriteA_0_0_45_0 + 0) = a_0_0_45_0;
    *(localWriteA_0_0_46_0 + 0) = a_0_0_46_0;
    *(localWriteA_0_0_47_0 + 0) = a_0_0_47_0;
    *(localWriteA_0_0_48_0 + 0) = a_0_0_48_0;
    *(localWriteA_0_0_49_0 + 0) = a_0_0_49_0;
    *(localWriteA_0_0_50_0 + 0) = a_0_0_50_0;
    *(localWriteA_0_0_51_0 + 0) = a_0_0_51_0;
    *(localWriteA_0_0_52_0 + 0) = a_0_0_52_0;
    *(localWriteA_0_0_53_0 + 0) = a_0_0_53_0;
    *(localWriteA_0_0_54_0 + 0) = a_0_0_54_0;
    *(localWriteA_0_0_55_0 + 0) = a_0_0_55_0;
    *(localWriteA_0_0_56_0 + 0) = a_0_0_56_0;
    *(localWriteA_0_0_57_0 + 0) = a_0_0_57_0;
    *(localWriteA_0_0_58_0 + 0) = a_0_0_58_0;
    *(localWriteA_0_0_59_0 + 0) = a_0_0_59_0;
    *(localWriteA_0_0_60_0 + 0) = a_0_0_60_0;
    *(localWriteA_0_0_61_0 + 0) = a_0_0_61_0;
    *(localWriteA_0_0_62_0 + 0) = a_0_0_62_0;
    *(localWriteA_0_0_63_0 + 0) = a_0_0_63_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 15 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 16 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 17 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 18 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 19 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 20 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 21 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 22 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 23 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 24 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 25 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 26 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 27 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 28 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 29 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 30 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 31 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 32 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 33 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 34 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 35 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 36 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 37 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 38 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 39 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 40 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 41 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 42 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 43 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 44 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 45 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 46 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 47 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 48 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 49 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 50 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 51 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 52 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 53 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 54 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 55 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 56 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 57 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 58 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 59 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 60 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 61 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 62 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 63 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_1x1




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_0_8_0 = ( globalReadOffsetAL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_0_0_9_0 = ( globalReadOffsetAL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_0_0_10_0 = ( globalReadOffsetAL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_0_0_11_0 = ( globalReadOffsetAL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_0_0_12_0 = ( globalReadOffsetAL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_0_0_13_0 = ( globalReadOffsetAL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_0_0_14_0 = ( globalReadOffsetAL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_0_0_15_0 = ( globalReadOffsetAL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);
  a_0_0_16_0 = ( globalReadOffsetAL_16_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_16_0 + 0);
  a_0_0_17_0 = ( globalReadOffsetAL_17_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_17_0 + 0);
  a_0_0_18_0 = ( globalReadOffsetAL_18_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_18_0 + 0);
  a_0_0_19_0 = ( globalReadOffsetAL_19_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_19_0 + 0);
  a_0_0_20_0 = ( globalReadOffsetAL_20_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_20_0 + 0);
  a_0_0_21_0 = ( globalReadOffsetAL_21_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_21_0 + 0);
  a_0_0_22_0 = ( globalReadOffsetAL_22_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_22_0 + 0);
  a_0_0_23_0 = ( globalReadOffsetAL_23_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_23_0 + 0);
  a_0_0_24_0 = ( globalReadOffsetAL_24_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_24_0 + 0);
  a_0_0_25_0 = ( globalReadOffsetAL_25_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_25_0 + 0);
  a_0_0_26_0 = ( globalReadOffsetAL_26_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_26_0 + 0);
  a_0_0_27_0 = ( globalReadOffsetAL_27_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_27_0 + 0);
  a_0_0_28_0 = ( globalReadOffsetAL_28_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_28_0 + 0);
  a_0_0_29_0 = ( globalReadOffsetAL_29_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_29_0 + 0);
  a_0_0_30_0 = ( globalReadOffsetAL_30_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_30_0 + 0);
  a_0_0_31_0 = ( globalReadOffsetAL_31_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_31_0 + 0);
  a_0_0_32_0 = ( globalReadOffsetAL_32_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_32_0 + 0);
  a_0_0_33_0 = ( globalReadOffsetAL_33_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_33_0 + 0);
  a_0_0_34_0 = ( globalReadOffsetAL_34_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_34_0 + 0);
  a_0_0_35_0 = ( globalReadOffsetAL_35_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_35_0 + 0);
  a_0_0_36_0 = ( globalReadOffsetAL_36_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_36_0 + 0);
  a_0_0_37_0 = ( globalReadOffsetAL_37_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_37_0 + 0);
  a_0_0_38_0 = ( globalReadOffsetAL_38_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_38_0 + 0);
  a_0_0_39_0 = ( globalReadOffsetAL_39_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_39_0 + 0);
  a_0_0_40_0 = ( globalReadOffsetAL_40_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_40_0 + 0);
  a_0_0_41_0 = ( globalReadOffsetAL_41_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_41_0 + 0);
  a_0_0_42_0 = ( globalReadOffsetAL_42_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_42_0 + 0);
  a_0_0_43_0 = ( globalReadOffsetAL_43_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_43_0 + 0);
  a_0_0_44_0 = ( globalReadOffsetAL_44_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_44_0 + 0);
  a_0_0_45_0 = ( globalReadOffsetAL_45_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_45_0 + 0);
  a_0_0_46_0 = ( globalReadOffsetAL_46_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_46_0 + 0);
  a_0_0_47_0 = ( globalReadOffsetAL_47_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_47_0 + 0);
  a_0_0_48_0 = ( globalReadOffsetAL_48_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_48_0 + 0);
  a_0_0_49_0 = ( globalReadOffsetAL_49_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_49_0 + 0);
  a_0_0_50_0 = ( globalReadOffsetAL_50_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_50_0 + 0);
  a_0_0_51_0 = ( globalReadOffsetAL_51_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_51_0 + 0);
  a_0_0_52_0 = ( globalReadOffsetAL_52_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_52_0 + 0);
  a_0_0_53_0 = ( globalReadOffsetAL_53_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_53_0 + 0);
  a_0_0_54_0 = ( globalReadOffsetAL_54_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_54_0 + 0);
  a_0_0_55_0 = ( globalReadOffsetAL_55_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_55_0 + 0);
  a_0_0_56_0 = ( globalReadOffsetAL_56_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_56_0 + 0);
  a_0_0_57_0 = ( globalReadOffsetAL_57_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_57_0 + 0);
  a_0_0_58_0 = ( globalReadOffsetAL_58_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_58_0 + 0);
  a_0_0_59_0 = ( globalReadOffsetAL_59_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_59_0 + 0);
  a_0_0_60_0 = ( globalReadOffsetAL_60_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_60_0 + 0);
  a_0_0_61_0 = ( globalReadOffsetAL_61_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_61_0 + 0);
  a_0_0_62_0 = ( globalReadOffsetAL_62_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_62_0 + 0);
  a_0_0_63_0 = ( globalReadOffsetAL_63_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_63_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);
  localWriteA_0_0_16_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_16_0);
  localWriteA_0_0_17_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_17_0);
  localWriteA_0_0_18_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_18_0);
  localWriteA_0_0_19_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_19_0);
  localWriteA_0_0_20_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_20_0);
  localWriteA_0_0_21_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_21_0);
  localWriteA_0_0_22_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_22_0);
  localWriteA_0_0_23_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_23_0);
  localWriteA_0_0_24_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_24_0);
  localWriteA_0_0_25_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_25_0);
  localWriteA_0_0_26_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_26_0);
  localWriteA_0_0_27_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_27_0);
  localWriteA_0_0_28_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_28_0);
  localWriteA_0_0_29_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_29_0);
  localWriteA_0_0_30_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_30_0);
  localWriteA_0_0_31_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_31_0);
  localWriteA_0_0_32_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_32_0);
  localWriteA_0_0_33_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_33_0);
  localWriteA_0_0_34_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_34_0);
  localWriteA_0_0_35_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_35_0);
  localWriteA_0_0_36_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_36_0);
  localWriteA_0_0_37_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_37_0);
  localWriteA_0_0_38_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_38_0);
  localWriteA_0_0_39_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_39_0);
  localWriteA_0_0_40_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_40_0);
  localWriteA_0_0_41_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_41_0);
  localWriteA_0_0_42_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_42_0);
  localWriteA_0_0_43_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_43_0);
  localWriteA_0_0_44_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_44_0);
  localWriteA_0_0_45_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_45_0);
  localWriteA_0_0_46_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_46_0);
  localWriteA_0_0_47_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_47_0);
  localWriteA_0_0_48_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_48_0);
  localWriteA_0_0_49_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_49_0);
  localWriteA_0_0_50_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_50_0);
  localWriteA_0_0_51_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_51_0);
  localWriteA_0_0_52_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_52_0);
  localWriteA_0_0_53_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_53_0);
  localWriteA_0_0_54_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_54_0);
  localWriteA_0_0_55_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_55_0);
  localWriteA_0_0_56_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_56_0);
  localWriteA_0_0_57_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_57_0);
  localWriteA_0_0_58_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_58_0);
  localWriteA_0_0_59_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_59_0);
  localWriteA_0_0_60_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_60_0);
  localWriteA_0_0_61_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_61_0);
  localWriteA_0_0_62_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_62_0);
  localWriteA_0_0_63_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_63_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
  *(localWriteA_0_0_16_0 + 0) = a_0_0_16_0;
  *(localWriteA_0_0_17_0 + 0) = a_0_0_17_0;
  *(localWriteA_0_0_18_0 + 0) = a_0_0_18_0;
  *(localWriteA_0_0_19_0 + 0) = a_0_0_19_0;
  *(localWriteA_0_0_20_0 + 0) = a_0_0_20_0;
  *(localWriteA_0_0_21_0 + 0) = a_0_0_21_0;
  *(localWriteA_0_0_22_0 + 0) = a_0_0_22_0;
  *(localWriteA_0_0_23_0 + 0) = a_0_0_23_0;
  *(localWriteA_0_0_24_0 + 0) = a_0_0_24_0;
  *(localWriteA_0_0_25_0 + 0) = a_0_0_25_0;
  *(localWriteA_0_0_26_0 + 0) = a_0_0_26_0;
  *(localWriteA_0_0_27_0 + 0) = a_0_0_27_0;
  *(localWriteA_0_0_28_0 + 0) = a_0_0_28_0;
  *(localWriteA_0_0_29_0 + 0) = a_0_0_29_0;
  *(localWriteA_0_0_30_0 + 0) = a_0_0_30_0;
  *(localWriteA_0_0_31_0 + 0) = a_0_0_31_0;
  *(localWriteA_0_0_32_0 + 0) = a_0_0_32_0;
  *(localWriteA_0_0_33_0 + 0) = a_0_0_33_0;
  *(localWriteA_0_0_34_0 + 0) = a_0_0_34_0;
  *(localWriteA_0_0_35_0 + 0) = a_0_0_35_0;
  *(localWriteA_0_0_36_0 + 0) = a_0_0_36_0;
  *(localWriteA_0_0_37_0 + 0) = a_0_0_37_0;
  *(localWriteA_0_0_38_0 + 0) = a_0_0_38_0;
  *(localWriteA_0_0_39_0 + 0) = a_0_0_39_0;
  *(localWriteA_0_0_40_0 + 0) = a_0_0_40_0;
  *(localWriteA_0_0_41_0 + 0) = a_0_0_41_0;
  *(localWriteA_0_0_42_0 + 0) = a_0_0_42_0;
  *(localWriteA_0_0_43_0 + 0) = a_0_0_43_0;
  *(localWriteA_0_0_44_0 + 0) = a_0_0_44_0;
  *(localWriteA_0_0_45_0 + 0) = a_0_0_45_0;
  *(localWriteA_0_0_46_0 + 0) = a_0_0_46_0;
  *(localWriteA_0_0_47_0 + 0) = a_0_0_47_0;
  *(localWriteA_0_0_48_0 + 0) = a_0_0_48_0;
  *(localWriteA_0_0_49_0 + 0) = a_0_0_49_0;
  *(localWriteA_0_0_50_0 + 0) = a_0_0_50_0;
  *(localWriteA_0_0_51_0 + 0) = a_0_0_51_0;
  *(localWriteA_0_0_52_0 + 0) = a_0_0_52_0;
  *(localWriteA_0_0_53_0 + 0) = a_0_0_53_0;
  *(localWriteA_0_0_54_0 + 0) = a_0_0_54_0;
  *(localWriteA_0_0_55_0 + 0) = a_0_0_55_0;
  *(localWriteA_0_0_56_0 + 0) = a_0_0_56_0;
  *(localWriteA_0_0_57_0 + 0) = a_0_0_57_0;
  *(localWriteA_0_0_58_0 + 0) = a_0_0_58_0;
  *(localWriteA_0_0_59_0 + 0) = a_0_0_59_0;
  *(localWriteA_0_0_60_0 + 0) = a_0_0_60_0;
  *(localWriteA_0_0_61_0 + 0) = a_0_0_61_0;
  *(localWriteA_0_0_62_0 + 0) = a_0_0_62_0;
  *(localWriteA_0_0_63_0 + 0) = a_0_0_63_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_1x1

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_1x1
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 64
#define SG1J 1
#define TT0I 1
#define TT1J 1
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 64
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 15

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 64
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 4096
#define LDS_NUM_ELEMENTS 4160

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 1x1 micro-tile */
#define MAC_1x1\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_BSS_BH_MT64x1x64_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU15_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU0_SUM0_SUS128_SRVW0_SVW4_SNLL0_TT1_1_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG64_1_1_WGM1(
  float *D,
  float const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_bfloat16)(0)
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE a_0_0_16_0;
  DATA_TYPE a_0_0_17_0;
  DATA_TYPE a_0_0_18_0;
  DATA_TYPE a_0_0_19_0;
  DATA_TYPE a_0_0_20_0;
  DATA_TYPE a_0_0_21_0;
  DATA_TYPE a_0_0_22_0;
  DATA_TYPE a_0_0_23_0;
  DATA_TYPE a_0_0_24_0;
  DATA_TYPE a_0_0_25_0;
  DATA_TYPE a_0_0_26_0;
  DATA_TYPE a_0_0_27_0;
  DATA_TYPE a_0_0_28_0;
  DATA_TYPE a_0_0_29_0;
  DATA_TYPE a_0_0_30_0;
  DATA_TYPE a_0_0_31_0;
  DATA_TYPE a_0_0_32_0;
  DATA_TYPE a_0_0_33_0;
  DATA_TYPE a_0_0_34_0;
  DATA_TYPE a_0_0_35_0;
  DATA_TYPE a_0_0_36_0;
  DATA_TYPE a_0_0_37_0;
  DATA_TYPE a_0_0_38_0;
  DATA_TYPE a_0_0_39_0;
  DATA_TYPE a_0_0_40_0;
  DATA_TYPE a_0_0_41_0;
  DATA_TYPE a_0_0_42_0;
  DATA_TYPE a_0_0_43_0;
  DATA_TYPE a_0_0_44_0;
  DATA_TYPE a_0_0_45_0;
  DATA_TYPE a_0_0_46_0;
  DATA_TYPE a_0_0_47_0;
  DATA_TYPE a_0_0_48_0;
  DATA_TYPE a_0_0_49_0;
  DATA_TYPE a_0_0_50_0;
  DATA_TYPE a_0_0_51_0;
  DATA_TYPE a_0_0_52_0;
  DATA_TYPE a_0_0_53_0;
  DATA_TYPE a_0_0_54_0;
  DATA_TYPE a_0_0_55_0;
  DATA_TYPE a_0_0_56_0;
  DATA_TYPE a_0_0_57_0;
  DATA_TYPE a_0_0_58_0;
  DATA_TYPE a_0_0_59_0;
  DATA_TYPE a_0_0_60_0;
  DATA_TYPE a_0_0_61_0;
  DATA_TYPE a_0_0_62_0;
  DATA_TYPE a_0_0_63_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;
  unsigned int globalReadOffsetAL_8_0 = globalReadOffsetAL + 0 + 8*LSPA;
  unsigned int globalReadOffsetAL_9_0 = globalReadOffsetAL + 0 + 9*LSPA;
  unsigned int globalReadOffsetAL_10_0 = globalReadOffsetAL + 0 + 10*LSPA;
  unsigned int globalReadOffsetAL_11_0 = globalReadOffsetAL + 0 + 11*LSPA;
  unsigned int globalReadOffsetAL_12_0 = globalReadOffsetAL + 0 + 12*LSPA;
  unsigned int globalReadOffsetAL_13_0 = globalReadOffsetAL + 0 + 13*LSPA;
  unsigned int globalReadOffsetAL_14_0 = globalReadOffsetAL + 0 + 14*LSPA;
  unsigned int globalReadOffsetAL_15_0 = globalReadOffsetAL + 0 + 15*LSPA;
  unsigned int globalReadOffsetAL_16_0 = globalReadOffsetAL + 0 + 16*LSPA;
  unsigned int globalReadOffsetAL_17_0 = globalReadOffsetAL + 0 + 17*LSPA;
  unsigned int globalReadOffsetAL_18_0 = globalReadOffsetAL + 0 + 18*LSPA;
  unsigned int globalReadOffsetAL_19_0 = globalReadOffsetAL + 0 + 19*LSPA;
  unsigned int globalReadOffsetAL_20_0 = globalReadOffsetAL + 0 + 20*LSPA;
  unsigned int globalReadOffsetAL_21_0 = globalReadOffsetAL + 0 + 21*LSPA;
  unsigned int globalReadOffsetAL_22_0 = globalReadOffsetAL + 0 + 22*LSPA;
  unsigned int globalReadOffsetAL_23_0 = globalReadOffsetAL + 0 + 23*LSPA;
  unsigned int globalReadOffsetAL_24_0 = globalReadOffsetAL + 0 + 24*LSPA;
  unsigned int globalReadOffsetAL_25_0 = globalReadOffsetAL + 0 + 25*LSPA;
  unsigned int globalReadOffsetAL_26_0 = globalReadOffsetAL + 0 + 26*LSPA;
  unsigned int globalReadOffsetAL_27_0 = globalReadOffsetAL + 0 + 27*LSPA;
  unsigned int globalReadOffsetAL_28_0 = globalReadOffsetAL + 0 + 28*LSPA;
  unsigned int globalReadOffsetAL_29_0 = globalReadOffsetAL + 0 + 29*LSPA;
  unsigned int globalReadOffsetAL_30_0 = globalReadOffsetAL + 0 + 30*LSPA;
  unsigned int globalReadOffsetAL_31_0 = globalReadOffsetAL + 0 + 31*LSPA;
  unsigned int globalReadOffsetAL_32_0 = globalReadOffsetAL + 0 + 32*LSPA;
  unsigned int globalReadOffsetAL_33_0 = globalReadOffsetAL + 0 + 33*LSPA;
  unsigned int globalReadOffsetAL_34_0 = globalReadOffsetAL + 0 + 34*LSPA;
  unsigned int globalReadOffsetAL_35_0 = globalReadOffsetAL + 0 + 35*LSPA;
  unsigned int globalReadOffsetAL_36_0 = globalReadOffsetAL + 0 + 36*LSPA;
  unsigned int globalReadOffsetAL_37_0 = globalReadOffsetAL + 0 + 37*LSPA;
  unsigned int globalReadOffsetAL_38_0 = globalReadOffsetAL + 0 + 38*LSPA;
  unsigned int globalReadOffsetAL_39_0 = globalReadOffsetAL + 0 + 39*LSPA;
  unsigned int globalReadOffsetAL_40_0 = globalReadOffsetAL + 0 + 40*LSPA;
  unsigned int globalReadOffsetAL_41_0 = globalReadOffsetAL + 0 + 41*LSPA;
  unsigned int globalReadOffsetAL_42_0 = globalReadOffsetAL + 0 + 42*LSPA;
  unsigned int globalReadOffsetAL_43_0 = globalReadOffsetAL + 0 + 43*LSPA;
  unsigned int globalReadOffsetAL_44_0 = globalReadOffsetAL + 0 + 44*LSPA;
  unsigned int globalReadOffsetAL_45_0 = globalReadOffsetAL + 0 + 45*LSPA;
  unsigned int globalReadOffsetAL_46_0 = globalReadOffsetAL + 0 + 46*LSPA;
  unsigned int globalReadOffsetAL_47_0 = globalReadOffsetAL + 0 + 47*LSPA;
  unsigned int globalReadOffsetAL_48_0 = globalReadOffsetAL + 0 + 48*LSPA;
  unsigned int globalReadOffsetAL_49_0 = globalReadOffsetAL + 0 + 49*LSPA;
  unsigned int globalReadOffsetAL_50_0 = globalReadOffsetAL + 0 + 50*LSPA;
  unsigned int globalReadOffsetAL_51_0 = globalReadOffsetAL + 0 + 51*LSPA;
  unsigned int globalReadOffsetAL_52_0 = globalReadOffsetAL + 0 + 52*LSPA;
  unsigned int globalReadOffsetAL_53_0 = globalReadOffsetAL + 0 + 53*LSPA;
  unsigned int globalReadOffsetAL_54_0 = globalReadOffsetAL + 0 + 54*LSPA;
  unsigned int globalReadOffsetAL_55_0 = globalReadOffsetAL + 0 + 55*LSPA;
  unsigned int globalReadOffsetAL_56_0 = globalReadOffsetAL + 0 + 56*LSPA;
  unsigned int globalReadOffsetAL_57_0 = globalReadOffsetAL + 0 + 57*LSPA;
  unsigned int globalReadOffsetAL_58_0 = globalReadOffsetAL + 0 + 58*LSPA;
  unsigned int globalReadOffsetAL_59_0 = globalReadOffsetAL + 0 + 59*LSPA;
  unsigned int globalReadOffsetAL_60_0 = globalReadOffsetAL + 0 + 60*LSPA;
  unsigned int globalReadOffsetAL_61_0 = globalReadOffsetAL + 0 + 61*LSPA;
  unsigned int globalReadOffsetAL_62_0 = globalReadOffsetAL + 0 + 62*LSPA;
  unsigned int globalReadOffsetAL_63_0 = globalReadOffsetAL + 0 + 63*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_15_0), wgK );
  int64_t globalReadOffsetA_0_0_16_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_16_0), wgK );
  int64_t globalReadOffsetA_0_0_17_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_17_0), wgK );
  int64_t globalReadOffsetA_0_0_18_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_18_0), wgK );
  int64_t globalReadOffsetA_0_0_19_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_19_0), wgK );
  int64_t globalReadOffsetA_0_0_20_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_20_0), wgK );
  int64_t globalReadOffsetA_0_0_21_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_21_0), wgK );
  int64_t globalReadOffsetA_0_0_22_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_22_0), wgK );
  int64_t globalReadOffsetA_0_0_23_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_23_0), wgK );
  int64_t globalReadOffsetA_0_0_24_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_24_0), wgK );
  int64_t globalReadOffsetA_0_0_25_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_25_0), wgK );
  int64_t globalReadOffsetA_0_0_26_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_26_0), wgK );
  int64_t globalReadOffsetA_0_0_27_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_27_0), wgK );
  int64_t globalReadOffsetA_0_0_28_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_28_0), wgK );
  int64_t globalReadOffsetA_0_0_29_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_29_0), wgK );
  int64_t globalReadOffsetA_0_0_30_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_30_0), wgK );
  int64_t globalReadOffsetA_0_0_31_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_31_0), wgK );
  int64_t globalReadOffsetA_0_0_32_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_32_0), wgK );
  int64_t globalReadOffsetA_0_0_33_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_33_0), wgK );
  int64_t globalReadOffsetA_0_0_34_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_34_0), wgK );
  int64_t globalReadOffsetA_0_0_35_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_35_0), wgK );
  int64_t globalReadOffsetA_0_0_36_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_36_0), wgK );
  int64_t globalReadOffsetA_0_0_37_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_37_0), wgK );
  int64_t globalReadOffsetA_0_0_38_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_38_0), wgK );
  int64_t globalReadOffsetA_0_0_39_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_39_0), wgK );
  int64_t globalReadOffsetA_0_0_40_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_40_0), wgK );
  int64_t globalReadOffsetA_0_0_41_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_41_0), wgK );
  int64_t globalReadOffsetA_0_0_42_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_42_0), wgK );
  int64_t globalReadOffsetA_0_0_43_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_43_0), wgK );
  int64_t globalReadOffsetA_0_0_44_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_44_0), wgK );
  int64_t globalReadOffsetA_0_0_45_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_45_0), wgK );
  int64_t globalReadOffsetA_0_0_46_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_46_0), wgK );
  int64_t globalReadOffsetA_0_0_47_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_47_0), wgK );
  int64_t globalReadOffsetA_0_0_48_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_48_0), wgK );
  int64_t globalReadOffsetA_0_0_49_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_49_0), wgK );
  int64_t globalReadOffsetA_0_0_50_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_50_0), wgK );
  int64_t globalReadOffsetA_0_0_51_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_51_0), wgK );
  int64_t globalReadOffsetA_0_0_52_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_52_0), wgK );
  int64_t globalReadOffsetA_0_0_53_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_53_0), wgK );
  int64_t globalReadOffsetA_0_0_54_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_54_0), wgK );
  int64_t globalReadOffsetA_0_0_55_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_55_0), wgK );
  int64_t globalReadOffsetA_0_0_56_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_56_0), wgK );
  int64_t globalReadOffsetA_0_0_57_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_57_0), wgK );
  int64_t globalReadOffsetA_0_0_58_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_58_0), wgK );
  int64_t globalReadOffsetA_0_0_59_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_59_0), wgK );
  int64_t globalReadOffsetA_0_0_60_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_60_0), wgK );
  int64_t globalReadOffsetA_0_0_61_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_61_0), wgK );
  int64_t globalReadOffsetA_0_0_62_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_62_0), wgK );
  int64_t globalReadOffsetA_0_0_63_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_63_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;
  DATA_TYPE const *globalReadA_0_0_16_0 = A + globalReadOffsetA_0_0_16_0;
  DATA_TYPE const *globalReadA_0_0_17_0 = A + globalReadOffsetA_0_0_17_0;
  DATA_TYPE const *globalReadA_0_0_18_0 = A + globalReadOffsetA_0_0_18_0;
  DATA_TYPE const *globalReadA_0_0_19_0 = A + globalReadOffsetA_0_0_19_0;
  DATA_TYPE const *globalReadA_0_0_20_0 = A + globalReadOffsetA_0_0_20_0;
  DATA_TYPE const *globalReadA_0_0_21_0 = A + globalReadOffsetA_0_0_21_0;
  DATA_TYPE const *globalReadA_0_0_22_0 = A + globalReadOffsetA_0_0_22_0;
  DATA_TYPE const *globalReadA_0_0_23_0 = A + globalReadOffsetA_0_0_23_0;
  DATA_TYPE const *globalReadA_0_0_24_0 = A + globalReadOffsetA_0_0_24_0;
  DATA_TYPE const *globalReadA_0_0_25_0 = A + globalReadOffsetA_0_0_25_0;
  DATA_TYPE const *globalReadA_0_0_26_0 = A + globalReadOffsetA_0_0_26_0;
  DATA_TYPE const *globalReadA_0_0_27_0 = A + globalReadOffsetA_0_0_27_0;
  DATA_TYPE const *globalReadA_0_0_28_0 = A + globalReadOffsetA_0_0_28_0;
  DATA_TYPE const *globalReadA_0_0_29_0 = A + globalReadOffsetA_0_0_29_0;
  DATA_TYPE const *globalReadA_0_0_30_0 = A + globalReadOffsetA_0_0_30_0;
  DATA_TYPE const *globalReadA_0_0_31_0 = A + globalReadOffsetA_0_0_31_0;
  DATA_TYPE const *globalReadA_0_0_32_0 = A + globalReadOffsetA_0_0_32_0;
  DATA_TYPE const *globalReadA_0_0_33_0 = A + globalReadOffsetA_0_0_33_0;
  DATA_TYPE const *globalReadA_0_0_34_0 = A + globalReadOffsetA_0_0_34_0;
  DATA_TYPE const *globalReadA_0_0_35_0 = A + globalReadOffsetA_0_0_35_0;
  DATA_TYPE const *globalReadA_0_0_36_0 = A + globalReadOffsetA_0_0_36_0;
  DATA_TYPE const *globalReadA_0_0_37_0 = A + globalReadOffsetA_0_0_37_0;
  DATA_TYPE const *globalReadA_0_0_38_0 = A + globalReadOffsetA_0_0_38_0;
  DATA_TYPE const *globalReadA_0_0_39_0 = A + globalReadOffsetA_0_0_39_0;
  DATA_TYPE const *globalReadA_0_0_40_0 = A + globalReadOffsetA_0_0_40_0;
  DATA_TYPE const *globalReadA_0_0_41_0 = A + globalReadOffsetA_0_0_41_0;
  DATA_TYPE const *globalReadA_0_0_42_0 = A + globalReadOffsetA_0_0_42_0;
  DATA_TYPE const *globalReadA_0_0_43_0 = A + globalReadOffsetA_0_0_43_0;
  DATA_TYPE const *globalReadA_0_0_44_0 = A + globalReadOffsetA_0_0_44_0;
  DATA_TYPE const *globalReadA_0_0_45_0 = A + globalReadOffsetA_0_0_45_0;
  DATA_TYPE const *globalReadA_0_0_46_0 = A + globalReadOffsetA_0_0_46_0;
  DATA_TYPE const *globalReadA_0_0_47_0 = A + globalReadOffsetA_0_0_47_0;
  DATA_TYPE const *globalReadA_0_0_48_0 = A + globalReadOffsetA_0_0_48_0;
  DATA_TYPE const *globalReadA_0_0_49_0 = A + globalReadOffsetA_0_0_49_0;
  DATA_TYPE const *globalReadA_0_0_50_0 = A + globalReadOffsetA_0_0_50_0;
  DATA_TYPE const *globalReadA_0_0_51_0 = A + globalReadOffsetA_0_0_51_0;
  DATA_TYPE const *globalReadA_0_0_52_0 = A + globalReadOffsetA_0_0_52_0;
  DATA_TYPE const *globalReadA_0_0_53_0 = A + globalReadOffsetA_0_0_53_0;
  DATA_TYPE const *globalReadA_0_0_54_0 = A + globalReadOffsetA_0_0_54_0;
  DATA_TYPE const *globalReadA_0_0_55_0 = A + globalReadOffsetA_0_0_55_0;
  DATA_TYPE const *globalReadA_0_0_56_0 = A + globalReadOffsetA_0_0_56_0;
  DATA_TYPE const *globalReadA_0_0_57_0 = A + globalReadOffsetA_0_0_57_0;
  DATA_TYPE const *globalReadA_0_0_58_0 = A + globalReadOffsetA_0_0_58_0;
  DATA_TYPE const *globalReadA_0_0_59_0 = A + globalReadOffsetA_0_0_59_0;
  DATA_TYPE const *globalReadA_0_0_60_0 = A + globalReadOffsetA_0_0_60_0;
  DATA_TYPE const *globalReadA_0_0_61_0 = A + globalReadOffsetA_0_0_61_0;
  DATA_TYPE const *globalReadA_0_0_62_0 = A + globalReadOffsetA_0_0_62_0;
  DATA_TYPE const *globalReadA_0_0_63_0 = A + globalReadOffsetA_0_0_63_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 8*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 9*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 10*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 11*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 12*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 13*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 14*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 15*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_16_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 16*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_17_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 17*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_18_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 18*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_19_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 19*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_20_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 20*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_21_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 21*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_22_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 22*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_23_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 23*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_24_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 24*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_25_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 25*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_26_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 26*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_27_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 27*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_28_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 28*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_29_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 29*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_30_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 30*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_31_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 31*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_32_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 32*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_33_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 33*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_34_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 34*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_35_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 35*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_36_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 36*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_37_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 37*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_38_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 38*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_39_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 39*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_40_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 40*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_41_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 41*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_42_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 42*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_43_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 43*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_44_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 44*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_45_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 45*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_46_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 46*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_47_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 47*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_48_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 48*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_49_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 49*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_50_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 50*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_51_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 51*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_52_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 52*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_53_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 53*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_54_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 54*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_55_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 55*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_56_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 56*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_57_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 57*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_58_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 58*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_59_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 59*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_60_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 60*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_61_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 61*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_62_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 62*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_63_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 63*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_0_0_15_0;
  DATA_TYPE *localWriteA_0_0_16_0;
  DATA_TYPE *localWriteA_0_0_17_0;
  DATA_TYPE *localWriteA_0_0_18_0;
  DATA_TYPE *localWriteA_0_0_19_0;
  DATA_TYPE *localWriteA_0_0_20_0;
  DATA_TYPE *localWriteA_0_0_21_0;
  DATA_TYPE *localWriteA_0_0_22_0;
  DATA_TYPE *localWriteA_0_0_23_0;
  DATA_TYPE *localWriteA_0_0_24_0;
  DATA_TYPE *localWriteA_0_0_25_0;
  DATA_TYPE *localWriteA_0_0_26_0;
  DATA_TYPE *localWriteA_0_0_27_0;
  DATA_TYPE *localWriteA_0_0_28_0;
  DATA_TYPE *localWriteA_0_0_29_0;
  DATA_TYPE *localWriteA_0_0_30_0;
  DATA_TYPE *localWriteA_0_0_31_0;
  DATA_TYPE *localWriteA_0_0_32_0;
  DATA_TYPE *localWriteA_0_0_33_0;
  DATA_TYPE *localWriteA_0_0_34_0;
  DATA_TYPE *localWriteA_0_0_35_0;
  DATA_TYPE *localWriteA_0_0_36_0;
  DATA_TYPE *localWriteA_0_0_37_0;
  DATA_TYPE *localWriteA_0_0_38_0;
  DATA_TYPE *localWriteA_0_0_39_0;
  DATA_TYPE *localWriteA_0_0_40_0;
  DATA_TYPE *localWriteA_0_0_41_0;
  DATA_TYPE *localWriteA_0_0_42_0;
  DATA_TYPE *localWriteA_0_0_43_0;
  DATA_TYPE *localWriteA_0_0_44_0;
  DATA_TYPE *localWriteA_0_0_45_0;
  DATA_TYPE *localWriteA_0_0_46_0;
  DATA_TYPE *localWriteA_0_0_47_0;
  DATA_TYPE *localWriteA_0_0_48_0;
  DATA_TYPE *localWriteA_0_0_49_0;
  DATA_TYPE *localWriteA_0_0_50_0;
  DATA_TYPE *localWriteA_0_0_51_0;
  DATA_TYPE *localWriteA_0_0_52_0;
  DATA_TYPE *localWriteA_0_0_53_0;
  DATA_TYPE *localWriteA_0_0_54_0;
  DATA_TYPE *localWriteA_0_0_55_0;
  DATA_TYPE *localWriteA_0_0_56_0;
  DATA_TYPE *localWriteA_0_0_57_0;
  DATA_TYPE *localWriteA_0_0_58_0;
  DATA_TYPE *localWriteA_0_0_59_0;
  DATA_TYPE *localWriteA_0_0_60_0;
  DATA_TYPE *localWriteA_0_0_61_0;
  DATA_TYPE *localWriteA_0_0_62_0;
  DATA_TYPE *localWriteA_0_0_63_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);
  localWriteA_0_0_16_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_16_0);
  localWriteA_0_0_17_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_17_0);
  localWriteA_0_0_18_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_18_0);
  localWriteA_0_0_19_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_19_0);
  localWriteA_0_0_20_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_20_0);
  localWriteA_0_0_21_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_21_0);
  localWriteA_0_0_22_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_22_0);
  localWriteA_0_0_23_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_23_0);
  localWriteA_0_0_24_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_24_0);
  localWriteA_0_0_25_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_25_0);
  localWriteA_0_0_26_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_26_0);
  localWriteA_0_0_27_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_27_0);
  localWriteA_0_0_28_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_28_0);
  localWriteA_0_0_29_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_29_0);
  localWriteA_0_0_30_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_30_0);
  localWriteA_0_0_31_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_31_0);
  localWriteA_0_0_32_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_32_0);
  localWriteA_0_0_33_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_33_0);
  localWriteA_0_0_34_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_34_0);
  localWriteA_0_0_35_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_35_0);
  localWriteA_0_0_36_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_36_0);
  localWriteA_0_0_37_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_37_0);
  localWriteA_0_0_38_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_38_0);
  localWriteA_0_0_39_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_39_0);
  localWriteA_0_0_40_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_40_0);
  localWriteA_0_0_41_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_41_0);
  localWriteA_0_0_42_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_42_0);
  localWriteA_0_0_43_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_43_0);
  localWriteA_0_0_44_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_44_0);
  localWriteA_0_0_45_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_45_0);
  localWriteA_0_0_46_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_46_0);
  localWriteA_0_0_47_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_47_0);
  localWriteA_0_0_48_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_48_0);
  localWriteA_0_0_49_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_49_0);
  localWriteA_0_0_50_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_50_0);
  localWriteA_0_0_51_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_51_0);
  localWriteA_0_0_52_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_52_0);
  localWriteA_0_0_53_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_53_0);
  localWriteA_0_0_54_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_54_0);
  localWriteA_0_0_55_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_55_0);
  localWriteA_0_0_56_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_56_0);
  localWriteA_0_0_57_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_57_0);
  localWriteA_0_0_58_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_58_0);
  localWriteA_0_0_59_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_59_0);
  localWriteA_0_0_60_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_60_0);
  localWriteA_0_0_61_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_61_0);
  localWriteA_0_0_62_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_62_0);
  localWriteA_0_0_63_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_63_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);
    a_0_0_16_0 = *(globalReadA_0_0_16_0 + 0);
    a_0_0_17_0 = *(globalReadA_0_0_17_0 + 0);
    a_0_0_18_0 = *(globalReadA_0_0_18_0 + 0);
    a_0_0_19_0 = *(globalReadA_0_0_19_0 + 0);
    a_0_0_20_0 = *(globalReadA_0_0_20_0 + 0);
    a_0_0_21_0 = *(globalReadA_0_0_21_0 + 0);
    a_0_0_22_0 = *(globalReadA_0_0_22_0 + 0);
    a_0_0_23_0 = *(globalReadA_0_0_23_0 + 0);
    a_0_0_24_0 = *(globalReadA_0_0_24_0 + 0);
    a_0_0_25_0 = *(globalReadA_0_0_25_0 + 0);
    a_0_0_26_0 = *(globalReadA_0_0_26_0 + 0);
    a_0_0_27_0 = *(globalReadA_0_0_27_0 + 0);
    a_0_0_28_0 = *(globalReadA_0_0_28_0 + 0);
    a_0_0_29_0 = *(globalReadA_0_0_29_0 + 0);
    a_0_0_30_0 = *(globalReadA_0_0_30_0 + 0);
    a_0_0_31_0 = *(globalReadA_0_0_31_0 + 0);
    a_0_0_32_0 = *(globalReadA_0_0_32_0 + 0);
    a_0_0_33_0 = *(globalReadA_0_0_33_0 + 0);
    a_0_0_34_0 = *(globalReadA_0_0_34_0 + 0);
    a_0_0_35_0 = *(globalReadA_0_0_35_0 + 0);
    a_0_0_36_0 = *(globalReadA_0_0_36_0 + 0);
    a_0_0_37_0 = *(globalReadA_0_0_37_0 + 0);
    a_0_0_38_0 = *(globalReadA_0_0_38_0 + 0);
    a_0_0_39_0 = *(globalReadA_0_0_39_0 + 0);
    a_0_0_40_0 = *(globalReadA_0_0_40_0 + 0);
    a_0_0_41_0 = *(globalReadA_0_0_41_0 + 0);
    a_0_0_42_0 = *(globalReadA_0_0_42_0 + 0);
    a_0_0_43_0 = *(globalReadA_0_0_43_0 + 0);
    a_0_0_44_0 = *(globalReadA_0_0_44_0 + 0);
    a_0_0_45_0 = *(globalReadA_0_0_45_0 + 0);
    a_0_0_46_0 = *(globalReadA_0_0_46_0 + 0);
    a_0_0_47_0 = *(globalReadA_0_0_47_0 + 0);
    a_0_0_48_0 = *(globalReadA_0_0_48_0 + 0);
    a_0_0_49_0 = *(globalReadA_0_0_49_0 + 0);
    a_0_0_50_0 = *(globalReadA_0_0_50_0 + 0);
    a_0_0_51_0 = *(globalReadA_0_0_51_0 + 0);
    a_0_0_52_0 = *(globalReadA_0_0_52_0 + 0);
    a_0_0_53_0 = *(globalReadA_0_0_53_0 + 0);
    a_0_0_54_0 = *(globalReadA_0_0_54_0 + 0);
    a_0_0_55_0 = *(globalReadA_0_0_55_0 + 0);
    a_0_0_56_0 = *(globalReadA_0_0_56_0 + 0);
    a_0_0_57_0 = *(globalReadA_0_0_57_0 + 0);
    a_0_0_58_0 = *(globalReadA_0_0_58_0 + 0);
    a_0_0_59_0 = *(globalReadA_0_0_59_0 + 0);
    a_0_0_60_0 = *(globalReadA_0_0_60_0 + 0);
    a_0_0_61_0 = *(globalReadA_0_0_61_0 + 0);
    a_0_0_62_0 = *(globalReadA_0_0_62_0 + 0);
    a_0_0_63_0 = *(globalReadA_0_0_63_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    globalReadA_0_0_16_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_16_0) + 1*globalReadIncAL);
    globalReadA_0_0_17_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_17_0) + 1*globalReadIncAL);
    globalReadA_0_0_18_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_18_0) + 1*globalReadIncAL);
    globalReadA_0_0_19_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_19_0) + 1*globalReadIncAL);
    globalReadA_0_0_20_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_20_0) + 1*globalReadIncAL);
    globalReadA_0_0_21_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_21_0) + 1*globalReadIncAL);
    globalReadA_0_0_22_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_22_0) + 1*globalReadIncAL);
    globalReadA_0_0_23_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_23_0) + 1*globalReadIncAL);
    globalReadA_0_0_24_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_24_0) + 1*globalReadIncAL);
    globalReadA_0_0_25_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_25_0) + 1*globalReadIncAL);
    globalReadA_0_0_26_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_26_0) + 1*globalReadIncAL);
    globalReadA_0_0_27_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_27_0) + 1*globalReadIncAL);
    globalReadA_0_0_28_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_28_0) + 1*globalReadIncAL);
    globalReadA_0_0_29_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_29_0) + 1*globalReadIncAL);
    globalReadA_0_0_30_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_30_0) + 1*globalReadIncAL);
    globalReadA_0_0_31_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_31_0) + 1*globalReadIncAL);
    globalReadA_0_0_32_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_32_0) + 1*globalReadIncAL);
    globalReadA_0_0_33_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_33_0) + 1*globalReadIncAL);
    globalReadA_0_0_34_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_34_0) + 1*globalReadIncAL);
    globalReadA_0_0_35_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_35_0) + 1*globalReadIncAL);
    globalReadA_0_0_36_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_36_0) + 1*globalReadIncAL);
    globalReadA_0_0_37_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_37_0) + 1*globalReadIncAL);
    globalReadA_0_0_38_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_38_0) + 1*globalReadIncAL);
    globalReadA_0_0_39_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_39_0) + 1*globalReadIncAL);
    globalReadA_0_0_40_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_40_0) + 1*globalReadIncAL);
    globalReadA_0_0_41_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_41_0) + 1*globalReadIncAL);
    globalReadA_0_0_42_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_42_0) + 1*globalReadIncAL);
    globalReadA_0_0_43_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_43_0) + 1*globalReadIncAL);
    globalReadA_0_0_44_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_44_0) + 1*globalReadIncAL);
    globalReadA_0_0_45_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_45_0) + 1*globalReadIncAL);
    globalReadA_0_0_46_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_46_0) + 1*globalReadIncAL);
    globalReadA_0_0_47_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_47_0) + 1*globalReadIncAL);
    globalReadA_0_0_48_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_48_0) + 1*globalReadIncAL);
    globalReadA_0_0_49_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_49_0) + 1*globalReadIncAL);
    globalReadA_0_0_50_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_50_0) + 1*globalReadIncAL);
    globalReadA_0_0_51_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_51_0) + 1*globalReadIncAL);
    globalReadA_0_0_52_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_52_0) + 1*globalReadIncAL);
    globalReadA_0_0_53_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_53_0) + 1*globalReadIncAL);
    globalReadA_0_0_54_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_54_0) + 1*globalReadIncAL);
    globalReadA_0_0_55_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_55_0) + 1*globalReadIncAL);
    globalReadA_0_0_56_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_56_0) + 1*globalReadIncAL);
    globalReadA_0_0_57_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_57_0) + 1*globalReadIncAL);
    globalReadA_0_0_58_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_58_0) + 1*globalReadIncAL);
    globalReadA_0_0_59_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_59_0) + 1*globalReadIncAL);
    globalReadA_0_0_60_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_60_0) + 1*globalReadIncAL);
    globalReadA_0_0_61_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_61_0) + 1*globalReadIncAL);
    globalReadA_0_0_62_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_62_0) + 1*globalReadIncAL);
    globalReadA_0_0_63_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_63_0) + 1*globalReadIncAL);

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
    *(localWriteA_0_0_16_0 + 0) = a_0_0_16_0;
    *(localWriteA_0_0_17_0 + 0) = a_0_0_17_0;
    *(localWriteA_0_0_18_0 + 0) = a_0_0_18_0;
    *(localWriteA_0_0_19_0 + 0) = a_0_0_19_0;
    *(localWriteA_0_0_20_0 + 0) = a_0_0_20_0;
    *(localWriteA_0_0_21_0 + 0) = a_0_0_21_0;
    *(localWriteA_0_0_22_0 + 0) = a_0_0_22_0;
    *(localWriteA_0_0_23_0 + 0) = a_0_0_23_0;
    *(localWriteA_0_0_24_0 + 0) = a_0_0_24_0;
    *(localWriteA_0_0_25_0 + 0) = a_0_0_25_0;
    *(localWriteA_0_0_26_0 + 0) = a_0_0_26_0;
    *(localWriteA_0_0_27_0 + 0) = a_0_0_27_0;
    *(localWriteA_0_0_28_0 + 0) = a_0_0_28_0;
    *(localWriteA_0_0_29_0 + 0) = a_0_0_29_0;
    *(localWriteA_0_0_30_0 + 0) = a_0_0_30_0;
    *(localWriteA_0_0_31_0 + 0) = a_0_0_31_0;
    *(localWriteA_0_0_32_0 + 0) = a_0_0_32_0;
    *(localWriteA_0_0_33_0 + 0) = a_0_0_33_0;
    *(localWriteA_0_0_34_0 + 0) = a_0_0_34_0;
    *(localWriteA_0_0_35_0 + 0) = a_0_0_35_0;
    *(localWriteA_0_0_36_0 + 0) = a_0_0_36_0;
    *(localWriteA_0_0_37_0 + 0) = a_0_0_37_0;
    *(localWriteA_0_0_38_0 + 0) = a_0_0_38_0;
    *(localWriteA_0_0_39_0 + 0) = a_0_0_39_0;
    *(localWriteA_0_0_40_0 + 0) = a_0_0_40_0;
    *(localWriteA_0_0_41_0 + 0) = a_0_0_41_0;
    *(localWriteA_0_0_42_0 + 0) = a_0_0_42_0;
    *(localWriteA_0_0_43_0 + 0) = a_0_0_43_0;
    *(localWriteA_0_0_44_0 + 0) = a_0_0_44_0;
    *(localWriteA_0_0_45_0 + 0) = a_0_0_45_0;
    *(localWriteA_0_0_46_0 + 0) = a_0_0_46_0;
    *(localWriteA_0_0_47_0 + 0) = a_0_0_47_0;
    *(localWriteA_0_0_48_0 + 0) = a_0_0_48_0;
    *(localWriteA_0_0_49_0 + 0) = a_0_0_49_0;
    *(localWriteA_0_0_50_0 + 0) = a_0_0_50_0;
    *(localWriteA_0_0_51_0 + 0) = a_0_0_51_0;
    *(localWriteA_0_0_52_0 + 0) = a_0_0_52_0;
    *(localWriteA_0_0_53_0 + 0) = a_0_0_53_0;
    *(localWriteA_0_0_54_0 + 0) = a_0_0_54_0;
    *(localWriteA_0_0_55_0 + 0) = a_0_0_55_0;
    *(localWriteA_0_0_56_0 + 0) = a_0_0_56_0;
    *(localWriteA_0_0_57_0 + 0) = a_0_0_57_0;
    *(localWriteA_0_0_58_0 + 0) = a_0_0_58_0;
    *(localWriteA_0_0_59_0 + 0) = a_0_0_59_0;
    *(localWriteA_0_0_60_0 + 0) = a_0_0_60_0;
    *(localWriteA_0_0_61_0 + 0) = a_0_0_61_0;
    *(localWriteA_0_0_62_0 + 0) = a_0_0_62_0;
    *(localWriteA_0_0_63_0 + 0) = a_0_0_63_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 15 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 16 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 17 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 18 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 19 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 20 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 21 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 22 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 23 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 24 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 25 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 26 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 27 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 28 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 29 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 30 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 31 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 32 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 33 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 34 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 35 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 36 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 37 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 38 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 39 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 40 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 41 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 42 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 43 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 44 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 45 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 46 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 47 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 48 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 49 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 50 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 51 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 52 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 53 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 54 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 55 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 56 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 57 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 58 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 59 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 60 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 61 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 62 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 63 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_1x1




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_0_8_0 = ( globalReadOffsetAL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_0_0_9_0 = ( globalReadOffsetAL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_0_0_10_0 = ( globalReadOffsetAL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_0_0_11_0 = ( globalReadOffsetAL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_0_0_12_0 = ( globalReadOffsetAL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_0_0_13_0 = ( globalReadOffsetAL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_0_0_14_0 = ( globalReadOffsetAL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_0_0_15_0 = ( globalReadOffsetAL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);
  a_0_0_16_0 = ( globalReadOffsetAL_16_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_16_0 + 0);
  a_0_0_17_0 = ( globalReadOffsetAL_17_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_17_0 + 0);
  a_0_0_18_0 = ( globalReadOffsetAL_18_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_18_0 + 0);
  a_0_0_19_0 = ( globalReadOffsetAL_19_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_19_0 + 0);
  a_0_0_20_0 = ( globalReadOffsetAL_20_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_20_0 + 0);
  a_0_0_21_0 = ( globalReadOffsetAL_21_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_21_0 + 0);
  a_0_0_22_0 = ( globalReadOffsetAL_22_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_22_0 + 0);
  a_0_0_23_0 = ( globalReadOffsetAL_23_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_23_0 + 0);
  a_0_0_24_0 = ( globalReadOffsetAL_24_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_24_0 + 0);
  a_0_0_25_0 = ( globalReadOffsetAL_25_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_25_0 + 0);
  a_0_0_26_0 = ( globalReadOffsetAL_26_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_26_0 + 0);
  a_0_0_27_0 = ( globalReadOffsetAL_27_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_27_0 + 0);
  a_0_0_28_0 = ( globalReadOffsetAL_28_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_28_0 + 0);
  a_0_0_29_0 = ( globalReadOffsetAL_29_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_29_0 + 0);
  a_0_0_30_0 = ( globalReadOffsetAL_30_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_30_0 + 0);
  a_0_0_31_0 = ( globalReadOffsetAL_31_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_31_0 + 0);
  a_0_0_32_0 = ( globalReadOffsetAL_32_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_32_0 + 0);
  a_0_0_33_0 = ( globalReadOffsetAL_33_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_33_0 + 0);
  a_0_0_34_0 = ( globalReadOffsetAL_34_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_34_0 + 0);
  a_0_0_35_0 = ( globalReadOffsetAL_35_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_35_0 + 0);
  a_0_0_36_0 = ( globalReadOffsetAL_36_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_36_0 + 0);
  a_0_0_37_0 = ( globalReadOffsetAL_37_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_37_0 + 0);
  a_0_0_38_0 = ( globalReadOffsetAL_38_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_38_0 + 0);
  a_0_0_39_0 = ( globalReadOffsetAL_39_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_39_0 + 0);
  a_0_0_40_0 = ( globalReadOffsetAL_40_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_40_0 + 0);
  a_0_0_41_0 = ( globalReadOffsetAL_41_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_41_0 + 0);
  a_0_0_42_0 = ( globalReadOffsetAL_42_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_42_0 + 0);
  a_0_0_43_0 = ( globalReadOffsetAL_43_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_43_0 + 0);
  a_0_0_44_0 = ( globalReadOffsetAL_44_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_44_0 + 0);
  a_0_0_45_0 = ( globalReadOffsetAL_45_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_45_0 + 0);
  a_0_0_46_0 = ( globalReadOffsetAL_46_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_46_0 + 0);
  a_0_0_47_0 = ( globalReadOffsetAL_47_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_47_0 + 0);
  a_0_0_48_0 = ( globalReadOffsetAL_48_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_48_0 + 0);
  a_0_0_49_0 = ( globalReadOffsetAL_49_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_49_0 + 0);
  a_0_0_50_0 = ( globalReadOffsetAL_50_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_50_0 + 0);
  a_0_0_51_0 = ( globalReadOffsetAL_51_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_51_0 + 0);
  a_0_0_52_0 = ( globalReadOffsetAL_52_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_52_0 + 0);
  a_0_0_53_0 = ( globalReadOffsetAL_53_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_53_0 + 0);
  a_0_0_54_0 = ( globalReadOffsetAL_54_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_54_0 + 0);
  a_0_0_55_0 = ( globalReadOffsetAL_55_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_55_0 + 0);
  a_0_0_56_0 = ( globalReadOffsetAL_56_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_56_0 + 0);
  a_0_0_57_0 = ( globalReadOffsetAL_57_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_57_0 + 0);
  a_0_0_58_0 = ( globalReadOffsetAL_58_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_58_0 + 0);
  a_0_0_59_0 = ( globalReadOffsetAL_59_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_59_0 + 0);
  a_0_0_60_0 = ( globalReadOffsetAL_60_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_60_0 + 0);
  a_0_0_61_0 = ( globalReadOffsetAL_61_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_61_0 + 0);
  a_0_0_62_0 = ( globalReadOffsetAL_62_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_62_0 + 0);
  a_0_0_63_0 = ( globalReadOffsetAL_63_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_63_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);
  localWriteA_0_0_16_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_16_0);
  localWriteA_0_0_17_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_17_0);
  localWriteA_0_0_18_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_18_0);
  localWriteA_0_0_19_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_19_0);
  localWriteA_0_0_20_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_20_0);
  localWriteA_0_0_21_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_21_0);
  localWriteA_0_0_22_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_22_0);
  localWriteA_0_0_23_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_23_0);
  localWriteA_0_0_24_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_24_0);
  localWriteA_0_0_25_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_25_0);
  localWriteA_0_0_26_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_26_0);
  localWriteA_0_0_27_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_27_0);
  localWriteA_0_0_28_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_28_0);
  localWriteA_0_0_29_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_29_0);
  localWriteA_0_0_30_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_30_0);
  localWriteA_0_0_31_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_31_0);
  localWriteA_0_0_32_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_32_0);
  localWriteA_0_0_33_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_33_0);
  localWriteA_0_0_34_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_34_0);
  localWriteA_0_0_35_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_35_0);
  localWriteA_0_0_36_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_36_0);
  localWriteA_0_0_37_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_37_0);
  localWriteA_0_0_38_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_38_0);
  localWriteA_0_0_39_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_39_0);
  localWriteA_0_0_40_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_40_0);
  localWriteA_0_0_41_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_41_0);
  localWriteA_0_0_42_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_42_0);
  localWriteA_0_0_43_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_43_0);
  localWriteA_0_0_44_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_44_0);
  localWriteA_0_0_45_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_45_0);
  localWriteA_0_0_46_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_46_0);
  localWriteA_0_0_47_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_47_0);
  localWriteA_0_0_48_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_48_0);
  localWriteA_0_0_49_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_49_0);
  localWriteA_0_0_50_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_50_0);
  localWriteA_0_0_51_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_51_0);
  localWriteA_0_0_52_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_52_0);
  localWriteA_0_0_53_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_53_0);
  localWriteA_0_0_54_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_54_0);
  localWriteA_0_0_55_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_55_0);
  localWriteA_0_0_56_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_56_0);
  localWriteA_0_0_57_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_57_0);
  localWriteA_0_0_58_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_58_0);
  localWriteA_0_0_59_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_59_0);
  localWriteA_0_0_60_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_60_0);
  localWriteA_0_0_61_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_61_0);
  localWriteA_0_0_62_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_62_0);
  localWriteA_0_0_63_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_63_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
  *(localWriteA_0_0_16_0 + 0) = a_0_0_16_0;
  *(localWriteA_0_0_17_0 + 0) = a_0_0_17_0;
  *(localWriteA_0_0_18_0 + 0) = a_0_0_18_0;
  *(localWriteA_0_0_19_0 + 0) = a_0_0_19_0;
  *(localWriteA_0_0_20_0 + 0) = a_0_0_20_0;
  *(localWriteA_0_0_21_0 + 0) = a_0_0_21_0;
  *(localWriteA_0_0_22_0 + 0) = a_0_0_22_0;
  *(localWriteA_0_0_23_0 + 0) = a_0_0_23_0;
  *(localWriteA_0_0_24_0 + 0) = a_0_0_24_0;
  *(localWriteA_0_0_25_0 + 0) = a_0_0_25_0;
  *(localWriteA_0_0_26_0 + 0) = a_0_0_26_0;
  *(localWriteA_0_0_27_0 + 0) = a_0_0_27_0;
  *(localWriteA_0_0_28_0 + 0) = a_0_0_28_0;
  *(localWriteA_0_0_29_0 + 0) = a_0_0_29_0;
  *(localWriteA_0_0_30_0 + 0) = a_0_0_30_0;
  *(localWriteA_0_0_31_0 + 0) = a_0_0_31_0;
  *(localWriteA_0_0_32_0 + 0) = a_0_0_32_0;
  *(localWriteA_0_0_33_0 + 0) = a_0_0_33_0;
  *(localWriteA_0_0_34_0 + 0) = a_0_0_34_0;
  *(localWriteA_0_0_35_0 + 0) = a_0_0_35_0;
  *(localWriteA_0_0_36_0 + 0) = a_0_0_36_0;
  *(localWriteA_0_0_37_0 + 0) = a_0_0_37_0;
  *(localWriteA_0_0_38_0 + 0) = a_0_0_38_0;
  *(localWriteA_0_0_39_0 + 0) = a_0_0_39_0;
  *(localWriteA_0_0_40_0 + 0) = a_0_0_40_0;
  *(localWriteA_0_0_41_0 + 0) = a_0_0_41_0;
  *(localWriteA_0_0_42_0 + 0) = a_0_0_42_0;
  *(localWriteA_0_0_43_0 + 0) = a_0_0_43_0;
  *(localWriteA_0_0_44_0 + 0) = a_0_0_44_0;
  *(localWriteA_0_0_45_0 + 0) = a_0_0_45_0;
  *(localWriteA_0_0_46_0 + 0) = a_0_0_46_0;
  *(localWriteA_0_0_47_0 + 0) = a_0_0_47_0;
  *(localWriteA_0_0_48_0 + 0) = a_0_0_48_0;
  *(localWriteA_0_0_49_0 + 0) = a_0_0_49_0;
  *(localWriteA_0_0_50_0 + 0) = a_0_0_50_0;
  *(localWriteA_0_0_51_0 + 0) = a_0_0_51_0;
  *(localWriteA_0_0_52_0 + 0) = a_0_0_52_0;
  *(localWriteA_0_0_53_0 + 0) = a_0_0_53_0;
  *(localWriteA_0_0_54_0 + 0) = a_0_0_54_0;
  *(localWriteA_0_0_55_0 + 0) = a_0_0_55_0;
  *(localWriteA_0_0_56_0 + 0) = a_0_0_56_0;
  *(localWriteA_0_0_57_0 + 0) = a_0_0_57_0;
  *(localWriteA_0_0_58_0 + 0) = a_0_0_58_0;
  *(localWriteA_0_0_59_0 + 0) = a_0_0_59_0;
  *(localWriteA_0_0_60_0 + 0) = a_0_0_60_0;
  *(localWriteA_0_0_61_0 + 0) = a_0_0_61_0;
  *(localWriteA_0_0_62_0 + 0) = a_0_0_62_0;
  *(localWriteA_0_0_63_0 + 0) = a_0_0_63_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_1x1

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_1x1
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 768

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_4xi8BH_MT64x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 7 */

    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_4xi8BH_MT64x16x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_1_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_1_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_1_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_1_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_1_4_0 = *(globalReadA_0_0_4_0 + 1);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_1_5_0 = *(globalReadA_0_0_5_0 + 1);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_1_6_0 = *(globalReadA_0_0_6_0 + 1);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_1_7_0 = *(globalReadA_0_0_7_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_4_0 + 1) = a_0_1_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_5_0 + 1) = a_0_1_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_6_0 + 1) = a_0_1_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_7_0 + 1) = a_0_1_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_1_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 1);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_1_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 1);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_1_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 1);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_1_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_4_0 + 1) = a_0_1_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_5_0 + 1) = a_0_1_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_6_0 + 1) = a_0_1_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_7_0 + 1) = a_0_1_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 1) {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 2) {
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_4xi8BH_MT128x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 16
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_4xi8BH_MT64x16x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;
  unsigned int globalReadOffsetAL_8_0 = globalReadOffsetAL + 0 + 8*LSPA;
  unsigned int globalReadOffsetAL_9_0 = globalReadOffsetAL + 0 + 9*LSPA;
  unsigned int globalReadOffsetAL_10_0 = globalReadOffsetAL + 0 + 10*LSPA;
  unsigned int globalReadOffsetAL_11_0 = globalReadOffsetAL + 0 + 11*LSPA;
  unsigned int globalReadOffsetAL_12_0 = globalReadOffsetAL + 0 + 12*LSPA;
  unsigned int globalReadOffsetAL_13_0 = globalReadOffsetAL + 0 + 13*LSPA;
  unsigned int globalReadOffsetAL_14_0 = globalReadOffsetAL + 0 + 14*LSPA;
  unsigned int globalReadOffsetAL_15_0 = globalReadOffsetAL + 0 + 15*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_15_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 8*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 9*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 10*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 11*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 12*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 13*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 14*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 15*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_0_0_15_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_8_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_9_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_10_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_11_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_12_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_13_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_14_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_15_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_8_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_9_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_10_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_11_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_12_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_13_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_14_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_15_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_0_8_0 = ( globalReadOffsetAL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_0_0_9_0 = ( globalReadOffsetAL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_0_0_10_0 = ( globalReadOffsetAL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_0_0_11_0 = ( globalReadOffsetAL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_0_0_12_0 = ( globalReadOffsetAL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_0_0_13_0 = ( globalReadOffsetAL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_0_0_14_0 = ( globalReadOffsetAL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_0_0_15_0 = ( globalReadOffsetAL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 16

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x8 micro-tile */
#define MAC_2x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \

#define MAC_2x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_4xi8BH_MT16x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_8_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;
  DATA_TYPE b_0_0_8_0;
  DATA_TYPE b_0_0_9_0;
  DATA_TYPE b_0_0_10_0;
  DATA_TYPE b_0_0_11_0;
  DATA_TYPE b_0_0_12_0;
  DATA_TYPE b_0_0_13_0;
  DATA_TYPE b_0_0_14_0;
  DATA_TYPE b_0_0_15_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;
  unsigned int flattenedOffsetB_4_0 = globalReadOffsetB1J + 0 + 4*LSPB;
  flattenedOffsetB_4_0 = (flattenedOffsetB_4_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_4_0;
  unsigned int globalReadOffsetB1J_4_0 = flattenedOffsetB_4_0;
  unsigned int flattenedOffsetB_5_0 = globalReadOffsetB1J + 0 + 5*LSPB;
  flattenedOffsetB_5_0 = (flattenedOffsetB_5_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_5_0;
  unsigned int globalReadOffsetB1J_5_0 = flattenedOffsetB_5_0;
  unsigned int flattenedOffsetB_6_0 = globalReadOffsetB1J + 0 + 6*LSPB;
  flattenedOffsetB_6_0 = (flattenedOffsetB_6_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_6_0;
  unsigned int globalReadOffsetB1J_6_0 = flattenedOffsetB_6_0;
  unsigned int flattenedOffsetB_7_0 = globalReadOffsetB1J + 0 + 7*LSPB;
  flattenedOffsetB_7_0 = (flattenedOffsetB_7_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_7_0;
  unsigned int globalReadOffsetB1J_7_0 = flattenedOffsetB_7_0;
  unsigned int flattenedOffsetB_8_0 = globalReadOffsetB1J + 0 + 8*LSPB;
  flattenedOffsetB_8_0 = (flattenedOffsetB_8_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_8_0;
  unsigned int globalReadOffsetB1J_8_0 = flattenedOffsetB_8_0;
  unsigned int flattenedOffsetB_9_0 = globalReadOffsetB1J + 0 + 9*LSPB;
  flattenedOffsetB_9_0 = (flattenedOffsetB_9_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_9_0;
  unsigned int globalReadOffsetB1J_9_0 = flattenedOffsetB_9_0;
  unsigned int flattenedOffsetB_10_0 = globalReadOffsetB1J + 0 + 10*LSPB;
  flattenedOffsetB_10_0 = (flattenedOffsetB_10_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_10_0;
  unsigned int globalReadOffsetB1J_10_0 = flattenedOffsetB_10_0;
  unsigned int flattenedOffsetB_11_0 = globalReadOffsetB1J + 0 + 11*LSPB;
  flattenedOffsetB_11_0 = (flattenedOffsetB_11_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_11_0;
  unsigned int globalReadOffsetB1J_11_0 = flattenedOffsetB_11_0;
  unsigned int flattenedOffsetB_12_0 = globalReadOffsetB1J + 0 + 12*LSPB;
  flattenedOffsetB_12_0 = (flattenedOffsetB_12_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_12_0;
  unsigned int globalReadOffsetB1J_12_0 = flattenedOffsetB_12_0;
  unsigned int flattenedOffsetB_13_0 = globalReadOffsetB1J + 0 + 13*LSPB;
  flattenedOffsetB_13_0 = (flattenedOffsetB_13_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_13_0;
  unsigned int globalReadOffsetB1J_13_0 = flattenedOffsetB_13_0;
  unsigned int flattenedOffsetB_14_0 = globalReadOffsetB1J + 0 + 14*LSPB;
  flattenedOffsetB_14_0 = (flattenedOffsetB_14_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_14_0;
  unsigned int globalReadOffsetB1J_14_0 = flattenedOffsetB_14_0;
  unsigned int flattenedOffsetB_15_0 = globalReadOffsetB1J + 0 + 15*LSPB;
  flattenedOffsetB_15_0 = (flattenedOffsetB_15_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_15_0;
  unsigned int globalReadOffsetB1J_15_0 = flattenedOffsetB_15_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;
  globalReadOffsetB1J_4_0 = (  globalReadOffsetB1J_4_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_4_0;
  globalReadOffsetB1J_5_0 = (  globalReadOffsetB1J_5_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_5_0;
  globalReadOffsetB1J_6_0 = (  globalReadOffsetB1J_6_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_6_0;
  globalReadOffsetB1J_7_0 = (  globalReadOffsetB1J_7_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_7_0;
  globalReadOffsetB1J_8_0 = (  globalReadOffsetB1J_8_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_8_0;
  globalReadOffsetB1J_9_0 = (  globalReadOffsetB1J_9_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_9_0;
  globalReadOffsetB1J_10_0 = (  globalReadOffsetB1J_10_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_10_0;
  globalReadOffsetB1J_11_0 = (  globalReadOffsetB1J_11_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_11_0;
  globalReadOffsetB1J_12_0 = (  globalReadOffsetB1J_12_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_12_0;
  globalReadOffsetB1J_13_0 = (  globalReadOffsetB1J_13_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_13_0;
  globalReadOffsetB1J_14_0 = (  globalReadOffsetB1J_14_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_14_0;
  globalReadOffsetB1J_15_0 = (  globalReadOffsetB1J_15_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_15_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_7_0), wgK );
  int64_t globalReadOffsetB_0_0_8_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_8_0), wgK );
  int64_t globalReadOffsetB_0_0_9_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_9_0), wgK );
  int64_t globalReadOffsetB_0_0_10_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_10_0), wgK );
  int64_t globalReadOffsetB_0_0_11_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_11_0), wgK );
  int64_t globalReadOffsetB_0_0_12_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_12_0), wgK );
  int64_t globalReadOffsetB_0_0_13_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_13_0), wgK );
  int64_t globalReadOffsetB_0_0_14_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_14_0), wgK );
  int64_t globalReadOffsetB_0_0_15_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_15_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;
  DATA_TYPE const *globalReadB_0_0_8_0 = B + globalReadOffsetB_0_0_8_0;
  DATA_TYPE const *globalReadB_0_0_9_0 = B + globalReadOffsetB_0_0_9_0;
  DATA_TYPE const *globalReadB_0_0_10_0 = B + globalReadOffsetB_0_0_10_0;
  DATA_TYPE const *globalReadB_0_0_11_0 = B + globalReadOffsetB_0_0_11_0;
  DATA_TYPE const *globalReadB_0_0_12_0 = B + globalReadOffsetB_0_0_12_0;
  DATA_TYPE const *globalReadB_0_0_13_0 = B + globalReadOffsetB_0_0_13_0;
  DATA_TYPE const *globalReadB_0_0_14_0 = B + globalReadOffsetB_0_0_14_0;
  DATA_TYPE const *globalReadB_0_0_15_0 = B + globalReadOffsetB_0_0_15_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 4*LSPB);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 5*LSPB);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 6*LSPB);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 7*LSPB);
  unsigned int localWriteOffsetB_0_0_8_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 8*LSPB);
  unsigned int localWriteOffsetB_0_0_9_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 9*LSPB);
  unsigned int localWriteOffsetB_0_0_10_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 10*LSPB);
  unsigned int localWriteOffsetB_0_0_11_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 11*LSPB);
  unsigned int localWriteOffsetB_0_0_12_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 12*LSPB);
  unsigned int localWriteOffsetB_0_0_13_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 13*LSPB);
  unsigned int localWriteOffsetB_0_0_14_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 14*LSPB);
  unsigned int localWriteOffsetB_0_0_15_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 15*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;
  DATA_TYPE *localWriteB_0_0_8_0;
  DATA_TYPE *localWriteB_0_0_9_0;
  DATA_TYPE *localWriteB_0_0_10_0;
  DATA_TYPE *localWriteB_0_0_11_0;
  DATA_TYPE *localWriteB_0_0_12_0;
  DATA_TYPE *localWriteB_0_0_13_0;
  DATA_TYPE *localWriteB_0_0_14_0;
  DATA_TYPE *localWriteB_0_0_15_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);
  localWriteB_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_8_0);
  localWriteB_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_9_0);
  localWriteB_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_10_0);
  localWriteB_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_11_0);
  localWriteB_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_12_0);
  localWriteB_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_13_0);
  localWriteB_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_14_0);
  localWriteB_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_15_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_8_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_9_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_10_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_11_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_12_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_13_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_14_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_15_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);
    b_0_0_8_0 = *(globalReadB_0_0_8_0 + 0);
    b_0_0_9_0 = *(globalReadB_0_0_9_0 + 0);
    b_0_0_10_0 = *(globalReadB_0_0_10_0 + 0);
    b_0_0_11_0 = *(globalReadB_0_0_11_0 + 0);
    b_0_0_12_0 = *(globalReadB_0_0_12_0 + 0);
    b_0_0_13_0 = *(globalReadB_0_0_13_0 + 0);
    b_0_0_14_0 = *(globalReadB_0_0_14_0 + 0);
    b_0_0_15_0 = *(globalReadB_0_0_15_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_8_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_8_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_9_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_9_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_10_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_10_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_11_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_11_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_12_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_12_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_13_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_13_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_14_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_14_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_15_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_15_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
    *(localWriteB_0_0_8_0 + 0) = b_0_0_8_0;
    *(localWriteB_0_0_9_0 + 0) = b_0_0_9_0;
    *(localWriteB_0_0_10_0 + 0) = b_0_0_10_0;
    *(localWriteB_0_0_11_0 + 0) = b_0_0_11_0;
    *(localWriteB_0_0_12_0 + 0) = b_0_0_12_0;
    *(localWriteB_0_0_13_0 + 0) = b_0_0_13_0;
    *(localWriteB_0_0_14_0 + 0) = b_0_0_14_0;
    *(localWriteB_0_0_15_0 + 0) = b_0_0_15_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x8


    /* iter 15 */

    MAC_2x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);
  b_0_0_8_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_8_0 + 0);
  b_0_0_9_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_9_0 + 0);
  b_0_0_10_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_10_0 + 0);
  b_0_0_11_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_11_0 + 0);
  b_0_0_12_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_12_0 + 0);
  b_0_0_13_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_13_0 + 0);
  b_0_0_14_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_14_0 + 0);
  b_0_0_15_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_15_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);
  localWriteB_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_8_0);
  localWriteB_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_9_0);
  localWriteB_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_10_0);
  localWriteB_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_11_0);
  localWriteB_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_12_0);
  localWriteB_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_13_0);
  localWriteB_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_14_0);
  localWriteB_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_15_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
  *(localWriteB_0_0_8_0 + 0) = b_0_0_8_0;
  *(localWriteB_0_0_9_0 + 0) = b_0_0_9_0;
  *(localWriteB_0_0_10_0 + 0) = b_0_0_10_0;
  *(localWriteB_0_0_11_0 + 0) = b_0_0_11_0;
  *(localWriteB_0_0_12_0 + 0) = b_0_0_12_0;
  *(localWriteB_0_0_13_0 + 0) = b_0_0_13_0;
  *(localWriteB_0_0_14_0 + 0) = b_0_0_14_0;
  *(localWriteB_0_0_15_0 + 0) = b_0_0_15_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x8
#undef MAC_2x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 768

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_4xi8BH_MT64x32x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 7 */

    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_4xi8BH_MT64x16x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_1_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_1_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_1_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_1_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_1_4_0 = *(globalReadA_0_0_4_0 + 1);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_1_5_0 = *(globalReadA_0_0_5_0 + 1);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_1_6_0 = *(globalReadA_0_0_6_0 + 1);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_1_7_0 = *(globalReadA_0_0_7_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_4_0 + 1) = a_0_1_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_5_0 + 1) = a_0_1_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_6_0 + 1) = a_0_1_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_7_0 + 1) = a_0_1_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_1_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 1);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_1_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 1);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_1_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 1);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_1_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_4_0 + 1) = a_0_1_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_5_0 + 1) = a_0_1_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_6_0 + 1) = a_0_1_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_7_0 + 1) = a_0_1_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 1) {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 2) {
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_4xi8BH_MT128x32x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 16
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_4xi8BH_MT64x16x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;
  unsigned int globalReadOffsetAL_8_0 = globalReadOffsetAL + 0 + 8*LSPA;
  unsigned int globalReadOffsetAL_9_0 = globalReadOffsetAL + 0 + 9*LSPA;
  unsigned int globalReadOffsetAL_10_0 = globalReadOffsetAL + 0 + 10*LSPA;
  unsigned int globalReadOffsetAL_11_0 = globalReadOffsetAL + 0 + 11*LSPA;
  unsigned int globalReadOffsetAL_12_0 = globalReadOffsetAL + 0 + 12*LSPA;
  unsigned int globalReadOffsetAL_13_0 = globalReadOffsetAL + 0 + 13*LSPA;
  unsigned int globalReadOffsetAL_14_0 = globalReadOffsetAL + 0 + 14*LSPA;
  unsigned int globalReadOffsetAL_15_0 = globalReadOffsetAL + 0 + 15*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_15_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 8*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 9*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 10*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 11*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 12*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 13*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 14*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 15*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_0_0_15_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_8_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_9_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_10_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_11_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_12_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_13_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_14_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_15_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_8_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_9_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_10_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_11_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_12_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_13_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_14_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_15_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_0_8_0 = ( globalReadOffsetAL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_0_0_9_0 = ( globalReadOffsetAL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_0_0_10_0 = ( globalReadOffsetAL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_0_0_11_0 = ( globalReadOffsetAL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_0_0_12_0 = ( globalReadOffsetAL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_0_0_13_0 = ( globalReadOffsetAL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_0_0_14_0 = ( globalReadOffsetAL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_0_0_15_0 = ( globalReadOffsetAL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 16

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x8 micro-tile */
#define MAC_2x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \

#define MAC_2x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_4xi8BH_MT16x64x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_8_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;
  DATA_TYPE b_0_0_8_0;
  DATA_TYPE b_0_0_9_0;
  DATA_TYPE b_0_0_10_0;
  DATA_TYPE b_0_0_11_0;
  DATA_TYPE b_0_0_12_0;
  DATA_TYPE b_0_0_13_0;
  DATA_TYPE b_0_0_14_0;
  DATA_TYPE b_0_0_15_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;
  unsigned int flattenedOffsetB_4_0 = globalReadOffsetB1J + 0 + 4*LSPB;
  flattenedOffsetB_4_0 = (flattenedOffsetB_4_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_4_0;
  unsigned int globalReadOffsetB1J_4_0 = flattenedOffsetB_4_0;
  unsigned int flattenedOffsetB_5_0 = globalReadOffsetB1J + 0 + 5*LSPB;
  flattenedOffsetB_5_0 = (flattenedOffsetB_5_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_5_0;
  unsigned int globalReadOffsetB1J_5_0 = flattenedOffsetB_5_0;
  unsigned int flattenedOffsetB_6_0 = globalReadOffsetB1J + 0 + 6*LSPB;
  flattenedOffsetB_6_0 = (flattenedOffsetB_6_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_6_0;
  unsigned int globalReadOffsetB1J_6_0 = flattenedOffsetB_6_0;
  unsigned int flattenedOffsetB_7_0 = globalReadOffsetB1J + 0 + 7*LSPB;
  flattenedOffsetB_7_0 = (flattenedOffsetB_7_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_7_0;
  unsigned int globalReadOffsetB1J_7_0 = flattenedOffsetB_7_0;
  unsigned int flattenedOffsetB_8_0 = globalReadOffsetB1J + 0 + 8*LSPB;
  flattenedOffsetB_8_0 = (flattenedOffsetB_8_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_8_0;
  unsigned int globalReadOffsetB1J_8_0 = flattenedOffsetB_8_0;
  unsigned int flattenedOffsetB_9_0 = globalReadOffsetB1J + 0 + 9*LSPB;
  flattenedOffsetB_9_0 = (flattenedOffsetB_9_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_9_0;
  unsigned int globalReadOffsetB1J_9_0 = flattenedOffsetB_9_0;
  unsigned int flattenedOffsetB_10_0 = globalReadOffsetB1J + 0 + 10*LSPB;
  flattenedOffsetB_10_0 = (flattenedOffsetB_10_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_10_0;
  unsigned int globalReadOffsetB1J_10_0 = flattenedOffsetB_10_0;
  unsigned int flattenedOffsetB_11_0 = globalReadOffsetB1J + 0 + 11*LSPB;
  flattenedOffsetB_11_0 = (flattenedOffsetB_11_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_11_0;
  unsigned int globalReadOffsetB1J_11_0 = flattenedOffsetB_11_0;
  unsigned int flattenedOffsetB_12_0 = globalReadOffsetB1J + 0 + 12*LSPB;
  flattenedOffsetB_12_0 = (flattenedOffsetB_12_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_12_0;
  unsigned int globalReadOffsetB1J_12_0 = flattenedOffsetB_12_0;
  unsigned int flattenedOffsetB_13_0 = globalReadOffsetB1J + 0 + 13*LSPB;
  flattenedOffsetB_13_0 = (flattenedOffsetB_13_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_13_0;
  unsigned int globalReadOffsetB1J_13_0 = flattenedOffsetB_13_0;
  unsigned int flattenedOffsetB_14_0 = globalReadOffsetB1J + 0 + 14*LSPB;
  flattenedOffsetB_14_0 = (flattenedOffsetB_14_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_14_0;
  unsigned int globalReadOffsetB1J_14_0 = flattenedOffsetB_14_0;
  unsigned int flattenedOffsetB_15_0 = globalReadOffsetB1J + 0 + 15*LSPB;
  flattenedOffsetB_15_0 = (flattenedOffsetB_15_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_15_0;
  unsigned int globalReadOffsetB1J_15_0 = flattenedOffsetB_15_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;
  globalReadOffsetB1J_4_0 = (  globalReadOffsetB1J_4_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_4_0;
  globalReadOffsetB1J_5_0 = (  globalReadOffsetB1J_5_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_5_0;
  globalReadOffsetB1J_6_0 = (  globalReadOffsetB1J_6_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_6_0;
  globalReadOffsetB1J_7_0 = (  globalReadOffsetB1J_7_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_7_0;
  globalReadOffsetB1J_8_0 = (  globalReadOffsetB1J_8_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_8_0;
  globalReadOffsetB1J_9_0 = (  globalReadOffsetB1J_9_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_9_0;
  globalReadOffsetB1J_10_0 = (  globalReadOffsetB1J_10_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_10_0;
  globalReadOffsetB1J_11_0 = (  globalReadOffsetB1J_11_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_11_0;
  globalReadOffsetB1J_12_0 = (  globalReadOffsetB1J_12_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_12_0;
  globalReadOffsetB1J_13_0 = (  globalReadOffsetB1J_13_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_13_0;
  globalReadOffsetB1J_14_0 = (  globalReadOffsetB1J_14_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_14_0;
  globalReadOffsetB1J_15_0 = (  globalReadOffsetB1J_15_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_15_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_7_0), wgK );
  int64_t globalReadOffsetB_0_0_8_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_8_0), wgK );
  int64_t globalReadOffsetB_0_0_9_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_9_0), wgK );
  int64_t globalReadOffsetB_0_0_10_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_10_0), wgK );
  int64_t globalReadOffsetB_0_0_11_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_11_0), wgK );
  int64_t globalReadOffsetB_0_0_12_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_12_0), wgK );
  int64_t globalReadOffsetB_0_0_13_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_13_0), wgK );
  int64_t globalReadOffsetB_0_0_14_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_14_0), wgK );
  int64_t globalReadOffsetB_0_0_15_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_15_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;
  DATA_TYPE const *globalReadB_0_0_8_0 = B + globalReadOffsetB_0_0_8_0;
  DATA_TYPE const *globalReadB_0_0_9_0 = B + globalReadOffsetB_0_0_9_0;
  DATA_TYPE const *globalReadB_0_0_10_0 = B + globalReadOffsetB_0_0_10_0;
  DATA_TYPE const *globalReadB_0_0_11_0 = B + globalReadOffsetB_0_0_11_0;
  DATA_TYPE const *globalReadB_0_0_12_0 = B + globalReadOffsetB_0_0_12_0;
  DATA_TYPE const *globalReadB_0_0_13_0 = B + globalReadOffsetB_0_0_13_0;
  DATA_TYPE const *globalReadB_0_0_14_0 = B + globalReadOffsetB_0_0_14_0;
  DATA_TYPE const *globalReadB_0_0_15_0 = B + globalReadOffsetB_0_0_15_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 4*LSPB);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 5*LSPB);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 6*LSPB);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 7*LSPB);
  unsigned int localWriteOffsetB_0_0_8_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 8*LSPB);
  unsigned int localWriteOffsetB_0_0_9_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 9*LSPB);
  unsigned int localWriteOffsetB_0_0_10_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 10*LSPB);
  unsigned int localWriteOffsetB_0_0_11_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 11*LSPB);
  unsigned int localWriteOffsetB_0_0_12_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 12*LSPB);
  unsigned int localWriteOffsetB_0_0_13_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 13*LSPB);
  unsigned int localWriteOffsetB_0_0_14_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 14*LSPB);
  unsigned int localWriteOffsetB_0_0_15_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 15*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;
  DATA_TYPE *localWriteB_0_0_8_0;
  DATA_TYPE *localWriteB_0_0_9_0;
  DATA_TYPE *localWriteB_0_0_10_0;
  DATA_TYPE *localWriteB_0_0_11_0;
  DATA_TYPE *localWriteB_0_0_12_0;
  DATA_TYPE *localWriteB_0_0_13_0;
  DATA_TYPE *localWriteB_0_0_14_0;
  DATA_TYPE *localWriteB_0_0_15_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);
  localWriteB_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_8_0);
  localWriteB_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_9_0);
  localWriteB_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_10_0);
  localWriteB_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_11_0);
  localWriteB_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_12_0);
  localWriteB_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_13_0);
  localWriteB_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_14_0);
  localWriteB_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_15_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_8_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_9_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_10_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_11_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_12_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_13_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_14_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_15_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);
    b_0_0_8_0 = *(globalReadB_0_0_8_0 + 0);
    b_0_0_9_0 = *(globalReadB_0_0_9_0 + 0);
    b_0_0_10_0 = *(globalReadB_0_0_10_0 + 0);
    b_0_0_11_0 = *(globalReadB_0_0_11_0 + 0);
    b_0_0_12_0 = *(globalReadB_0_0_12_0 + 0);
    b_0_0_13_0 = *(globalReadB_0_0_13_0 + 0);
    b_0_0_14_0 = *(globalReadB_0_0_14_0 + 0);
    b_0_0_15_0 = *(globalReadB_0_0_15_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_8_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_8_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_9_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_9_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_10_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_10_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_11_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_11_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_12_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_12_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_13_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_13_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_14_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_14_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_15_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_15_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
    *(localWriteB_0_0_8_0 + 0) = b_0_0_8_0;
    *(localWriteB_0_0_9_0 + 0) = b_0_0_9_0;
    *(localWriteB_0_0_10_0 + 0) = b_0_0_10_0;
    *(localWriteB_0_0_11_0 + 0) = b_0_0_11_0;
    *(localWriteB_0_0_12_0 + 0) = b_0_0_12_0;
    *(localWriteB_0_0_13_0 + 0) = b_0_0_13_0;
    *(localWriteB_0_0_14_0 + 0) = b_0_0_14_0;
    *(localWriteB_0_0_15_0 + 0) = b_0_0_15_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x8


    /* iter 15 */

    MAC_2x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);
  b_0_0_8_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_8_0 + 0);
  b_0_0_9_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_9_0 + 0);
  b_0_0_10_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_10_0 + 0);
  b_0_0_11_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_11_0 + 0);
  b_0_0_12_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_12_0 + 0);
  b_0_0_13_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_13_0 + 0);
  b_0_0_14_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_14_0 + 0);
  b_0_0_15_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_15_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);
  localWriteB_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_8_0);
  localWriteB_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_9_0);
  localWriteB_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_10_0);
  localWriteB_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_11_0);
  localWriteB_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_12_0);
  localWriteB_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_13_0);
  localWriteB_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_14_0);
  localWriteB_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_15_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
  *(localWriteB_0_0_8_0 + 0) = b_0_0_8_0;
  *(localWriteB_0_0_9_0 + 0) = b_0_0_9_0;
  *(localWriteB_0_0_10_0 + 0) = b_0_0_10_0;
  *(localWriteB_0_0_11_0 + 0) = b_0_0_11_0;
  *(localWriteB_0_0_12_0 + 0) = b_0_0_12_0;
  *(localWriteB_0_0_13_0 + 0) = b_0_0_13_0;
  *(localWriteB_0_0_14_0 + 0) = b_0_0_14_0;
  *(localWriteB_0_0_15_0 + 0) = b_0_0_15_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x8
#undef MAC_2x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE double
#define DEST_DATA_TYPE double
#define COMPUTE_DATA_TYPE double
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_DB_MT32x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW2_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  double *D,
  double const * __restrict__ C,
  double const * __restrict__ A,
  double const * __restrict__ B,
  double const alpha,
  double const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (double)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE double
#define DEST_DATA_TYPE double
#define COMPUTE_DATA_TYPE double
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_DB_MT32x32x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW2_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  double *D,
  double const * __restrict__ C,
  double const * __restrict__ A,
  double const * __restrict__ B,
  double const alpha,
  double const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (double)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 768

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_BSS_BH_MT64x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_bfloat16)(0)
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 7 */

    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_BSS_BH_MT128x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_bfloat16)(0)
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 1) {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 2) {
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 2x8 micro-tile */
#define MAC_2x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \

#define MAC_2x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_BSS_BH_MT32x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_8_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_bfloat16)(0)
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_1_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_1_3_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_2_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_3_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_2_1;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_3_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_1);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_1 + 0) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_1 + 0) = b_0_1_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x8


    /* iter 15 */

    MAC_2x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_1_2_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 1);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_1_3_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_1);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_2_1 + 0) = b_0_1_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_3_1 + 0) = b_0_1_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x8

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x8
#undef MAC_2x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 16
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 4096
#define LDS_NUM_ELEMENTS 4608

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 16x2 micro-tile */
#define MAC_16x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[8],rB[0],rC[8+0*TT0I]); \
  TYPE_MAC(rA[9],rB[0],rC[9+0*TT0I]); \
  TYPE_MAC(rA[10],rB[0],rC[10+0*TT0I]); \
  TYPE_MAC(rA[11],rB[0],rC[11+0*TT0I]); \
  TYPE_MAC(rA[12],rB[0],rC[12+0*TT0I]); \
  TYPE_MAC(rA[13],rB[0],rC[13+0*TT0I]); \
  TYPE_MAC(rA[14],rB[0],rC[14+0*TT0I]); \
  TYPE_MAC(rA[15],rB[0],rC[15+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[8],rB[1],rC[8+1*TT0I]); \
  TYPE_MAC(rA[9],rB[1],rC[9+1*TT0I]); \
  TYPE_MAC(rA[10],rB[1],rC[10+1*TT0I]); \
  TYPE_MAC(rA[11],rB[1],rC[11+1*TT0I]); \
  TYPE_MAC(rA[12],rB[1],rC[12+1*TT0I]); \
  TYPE_MAC(rA[13],rB[1],rC[13+1*TT0I]); \
  TYPE_MAC(rA[14],rB[1],rC[14+1*TT0I]); \
  TYPE_MAC(rA[15],rB[1],rC[15+1*TT0I]); \

#define MAC_16x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[8+TT0I],rB[0+TT1J],rC[8+0*TT0I]); \
  TYPE_MAC(rA[9+TT0I],rB[0+TT1J],rC[9+0*TT0I]); \
  TYPE_MAC(rA[10+TT0I],rB[0+TT1J],rC[10+0*TT0I]); \
  TYPE_MAC(rA[11+TT0I],rB[0+TT1J],rC[11+0*TT0I]); \
  TYPE_MAC(rA[12+TT0I],rB[0+TT1J],rC[12+0*TT0I]); \
  TYPE_MAC(rA[13+TT0I],rB[0+TT1J],rC[13+0*TT0I]); \
  TYPE_MAC(rA[14+TT0I],rB[0+TT1J],rC[14+0*TT0I]); \
  TYPE_MAC(rA[15+TT0I],rB[0+TT1J],rC[15+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[8+TT0I],rB[1+TT1J],rC[8+1*TT0I]); \
  TYPE_MAC(rA[9+TT0I],rB[1+TT1J],rC[9+1*TT0I]); \
  TYPE_MAC(rA[10+TT0I],rB[1+TT1J],rC[10+1*TT0I]); \
  TYPE_MAC(rA[11+TT0I],rB[1+TT1J],rC[11+1*TT0I]); \
  TYPE_MAC(rA[12+TT0I],rB[1+TT1J],rC[12+1*TT0I]); \
  TYPE_MAC(rA[13+TT0I],rB[1+TT1J],rC[13+1*TT0I]); \
  TYPE_MAC(rA[14+TT0I],rB[1+TT1J],rC[14+1*TT0I]); \
  TYPE_MAC(rA[15+TT0I],rB[1+TT1J],rC[15+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_BSS_BH_MT256x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT16_2_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_bfloat16)(0)
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_1_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_1_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_1_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_1_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_1_4_0 = *(globalReadA_0_0_4_0 + 1);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_1_5_0 = *(globalReadA_0_0_5_0 + 1);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_1_6_0 = *(globalReadA_0_0_6_0 + 1);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_1_7_0 = *(globalReadA_0_0_7_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_4_0 + 1) = a_0_1_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_5_0 + 1) = a_0_1_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_6_0 + 1) = a_0_1_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_7_0 + 1) = a_0_1_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_16x2


    /* iter 15 */

    MAC_16x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_1_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 1);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_1_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 1);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_1_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 1);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_1_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_4_0 + 1) = a_0_1_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_5_0 + 1) = a_0_1_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_6_0 + 1) = a_0_1_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_7_0 + 1) = a_0_1_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_16x2

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 1) {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 2) {
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 3) {
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 4) {
        rC[0 + 4*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 4*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 4*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 4*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 5) {
        rC[0 + 5*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 5*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 5*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 5*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 6) {
        rC[0 + 6*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 6*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 6*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 6*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 7*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 7*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 7*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 7*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_16x2
#undef MAC_16x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 16
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 16
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 2304

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 16x2 micro-tile */
#define MAC_16x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[8],rB[0],rC[8+0*TT0I]); \
  TYPE_MAC(rA[9],rB[0],rC[9+0*TT0I]); \
  TYPE_MAC(rA[10],rB[0],rC[10+0*TT0I]); \
  TYPE_MAC(rA[11],rB[0],rC[11+0*TT0I]); \
  TYPE_MAC(rA[12],rB[0],rC[12+0*TT0I]); \
  TYPE_MAC(rA[13],rB[0],rC[13+0*TT0I]); \
  TYPE_MAC(rA[14],rB[0],rC[14+0*TT0I]); \
  TYPE_MAC(rA[15],rB[0],rC[15+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[8],rB[1],rC[8+1*TT0I]); \
  TYPE_MAC(rA[9],rB[1],rC[9+1*TT0I]); \
  TYPE_MAC(rA[10],rB[1],rC[10+1*TT0I]); \
  TYPE_MAC(rA[11],rB[1],rC[11+1*TT0I]); \
  TYPE_MAC(rA[12],rB[1],rC[12+1*TT0I]); \
  TYPE_MAC(rA[13],rB[1],rC[13+1*TT0I]); \
  TYPE_MAC(rA[14],rB[1],rC[14+1*TT0I]); \
  TYPE_MAC(rA[15],rB[1],rC[15+1*TT0I]); \

#define MAC_16x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[8+TT0I],rB[0+TT1J],rC[8+0*TT0I]); \
  TYPE_MAC(rA[9+TT0I],rB[0+TT1J],rC[9+0*TT0I]); \
  TYPE_MAC(rA[10+TT0I],rB[0+TT1J],rC[10+0*TT0I]); \
  TYPE_MAC(rA[11+TT0I],rB[0+TT1J],rC[11+0*TT0I]); \
  TYPE_MAC(rA[12+TT0I],rB[0+TT1J],rC[12+0*TT0I]); \
  TYPE_MAC(rA[13+TT0I],rB[0+TT1J],rC[13+0*TT0I]); \
  TYPE_MAC(rA[14+TT0I],rB[0+TT1J],rC[14+0*TT0I]); \
  TYPE_MAC(rA[15+TT0I],rB[0+TT1J],rC[15+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[8+TT0I],rB[1+TT1J],rC[8+1*TT0I]); \
  TYPE_MAC(rA[9+TT0I],rB[1+TT1J],rC[9+1*TT0I]); \
  TYPE_MAC(rA[10+TT0I],rB[1+TT1J],rC[10+1*TT0I]); \
  TYPE_MAC(rA[11+TT0I],rB[1+TT1J],rC[11+1*TT0I]); \
  TYPE_MAC(rA[12+TT0I],rB[1+TT1J],rC[12+1*TT0I]); \
  TYPE_MAC(rA[13+TT0I],rB[1+TT1J],rC[13+1*TT0I]); \
  TYPE_MAC(rA[14+TT0I],rB[1+TT1J],rC[14+1*TT0I]); \
  TYPE_MAC(rA[15+TT0I],rB[1+TT1J],rC[15+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_BSS_BH_MT128x16x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT16_2_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_bfloat16)(0)
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_1_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_1_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_1_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_1_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_0_1_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_0_1_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_0_1_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_0_1_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_0_1_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_0_1_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_0_1_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE a_0_1_15_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;
  unsigned int globalReadOffsetAL_8_0 = globalReadOffsetAL + 0 + 8*LSPA;
  unsigned int globalReadOffsetAL_9_0 = globalReadOffsetAL + 0 + 9*LSPA;
  unsigned int globalReadOffsetAL_10_0 = globalReadOffsetAL + 0 + 10*LSPA;
  unsigned int globalReadOffsetAL_11_0 = globalReadOffsetAL + 0 + 11*LSPA;
  unsigned int globalReadOffsetAL_12_0 = globalReadOffsetAL + 0 + 12*LSPA;
  unsigned int globalReadOffsetAL_13_0 = globalReadOffsetAL + 0 + 13*LSPA;
  unsigned int globalReadOffsetAL_14_0 = globalReadOffsetAL + 0 + 14*LSPA;
  unsigned int globalReadOffsetAL_15_0 = globalReadOffsetAL + 0 + 15*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_15_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 8*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 9*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 10*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 11*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 12*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 13*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 14*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 15*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_0_0_15_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_8_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_9_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_10_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_11_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_12_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_13_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_14_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_15_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_1_4_0 = *(globalReadA_0_0_4_0 + 1);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_1_5_0 = *(globalReadA_0_0_5_0 + 1);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_1_6_0 = *(globalReadA_0_0_6_0 + 1);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_1_7_0 = *(globalReadA_0_0_7_0 + 1);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_0_1_8_0 = *(globalReadA_0_0_8_0 + 1);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_0_1_9_0 = *(globalReadA_0_0_9_0 + 1);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_0_1_10_0 = *(globalReadA_0_0_10_0 + 1);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_0_1_11_0 = *(globalReadA_0_0_11_0 + 1);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_0_1_12_0 = *(globalReadA_0_0_12_0 + 1);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_0_1_13_0 = *(globalReadA_0_0_13_0 + 1);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_0_1_14_0 = *(globalReadA_0_0_14_0 + 1);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);
    a_0_1_15_0 = *(globalReadA_0_0_15_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_8_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_9_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_10_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_11_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_12_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_13_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_14_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_15_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_4_0 + 1) = a_0_1_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_5_0 + 1) = a_0_1_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_6_0 + 1) = a_0_1_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_7_0 + 1) = a_0_1_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_0_0_8_0 + 1) = a_0_1_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_0_0_9_0 + 1) = a_0_1_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_0_0_10_0 + 1) = a_0_1_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_0_0_11_0 + 1) = a_0_1_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_0_0_12_0 + 1) = a_0_1_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_0_0_13_0 + 1) = a_0_1_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_0_0_14_0 + 1) = a_0_1_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
    *(localWriteA_0_0_15_0 + 1) = a_0_1_15_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_16x2


    /* iter 15 */

    MAC_16x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_1_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 1);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_1_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 1);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_1_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 1);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_1_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 1);
  a_0_0_8_0 = ( globalReadOffsetAL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_0_1_8_0 = ( globalReadOffsetAL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 1);
  a_0_0_9_0 = ( globalReadOffsetAL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_0_1_9_0 = ( globalReadOffsetAL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 1);
  a_0_0_10_0 = ( globalReadOffsetAL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_0_1_10_0 = ( globalReadOffsetAL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 1);
  a_0_0_11_0 = ( globalReadOffsetAL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_0_1_11_0 = ( globalReadOffsetAL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 1);
  a_0_0_12_0 = ( globalReadOffsetAL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_0_1_12_0 = ( globalReadOffsetAL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 1);
  a_0_0_13_0 = ( globalReadOffsetAL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_0_1_13_0 = ( globalReadOffsetAL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 1);
  a_0_0_14_0 = ( globalReadOffsetAL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_0_1_14_0 = ( globalReadOffsetAL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 1);
  a_0_0_15_0 = ( globalReadOffsetAL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);
  a_0_1_15_0 = ( globalReadOffsetAL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_4_0 + 1) = a_0_1_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_5_0 + 1) = a_0_1_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_6_0 + 1) = a_0_1_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_7_0 + 1) = a_0_1_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_0_0_8_0 + 1) = a_0_1_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_0_0_9_0 + 1) = a_0_1_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_0_0_10_0 + 1) = a_0_1_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_0_0_11_0 + 1) = a_0_1_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_0_0_12_0 + 1) = a_0_1_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_0_0_13_0 + 1) = a_0_1_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_0_0_14_0 + 1) = a_0_1_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
  *(localWriteA_0_0_15_0 + 1) = a_0_1_15_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_16x2

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 1) {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 2) {
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 3) {
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 4) {
        rC[0 + 4*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 4*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 4*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 4*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 5) {
        rC[0 + 5*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 5*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 5*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 5*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 6) {
        rC[0 + 6*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 6*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 6*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 6*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 7*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 7*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 7*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 7*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_16x2
#undef MAC_16x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_BSS_BH_MT128x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_bfloat16)(0)
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_BSS_BH_MT128x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_bfloat16)(0)
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 16
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 2
#define NLCB 1
#define NLPA 16
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 2304

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 16x2 micro-tile */
#define MAC_16x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[8],rB[0],rC[8+0*TT0I]); \
  TYPE_MAC(rA[9],rB[0],rC[9+0*TT0I]); \
  TYPE_MAC(rA[10],rB[0],rC[10+0*TT0I]); \
  TYPE_MAC(rA[11],rB[0],rC[11+0*TT0I]); \
  TYPE_MAC(rA[12],rB[0],rC[12+0*TT0I]); \
  TYPE_MAC(rA[13],rB[0],rC[13+0*TT0I]); \
  TYPE_MAC(rA[14],rB[0],rC[14+0*TT0I]); \
  TYPE_MAC(rA[15],rB[0],rC[15+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[8],rB[1],rC[8+1*TT0I]); \
  TYPE_MAC(rA[9],rB[1],rC[9+1*TT0I]); \
  TYPE_MAC(rA[10],rB[1],rC[10+1*TT0I]); \
  TYPE_MAC(rA[11],rB[1],rC[11+1*TT0I]); \
  TYPE_MAC(rA[12],rB[1],rC[12+1*TT0I]); \
  TYPE_MAC(rA[13],rB[1],rC[13+1*TT0I]); \
  TYPE_MAC(rA[14],rB[1],rC[14+1*TT0I]); \
  TYPE_MAC(rA[15],rB[1],rC[15+1*TT0I]); \

#define MAC_16x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[8+TT0I],rB[0+TT1J],rC[8+0*TT0I]); \
  TYPE_MAC(rA[9+TT0I],rB[0+TT1J],rC[9+0*TT0I]); \
  TYPE_MAC(rA[10+TT0I],rB[0+TT1J],rC[10+0*TT0I]); \
  TYPE_MAC(rA[11+TT0I],rB[0+TT1J],rC[11+0*TT0I]); \
  TYPE_MAC(rA[12+TT0I],rB[0+TT1J],rC[12+0*TT0I]); \
  TYPE_MAC(rA[13+TT0I],rB[0+TT1J],rC[13+0*TT0I]); \
  TYPE_MAC(rA[14+TT0I],rB[0+TT1J],rC[14+0*TT0I]); \
  TYPE_MAC(rA[15+TT0I],rB[0+TT1J],rC[15+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[8+TT0I],rB[1+TT1J],rC[8+1*TT0I]); \
  TYPE_MAC(rA[9+TT0I],rB[1+TT1J],rC[9+1*TT0I]); \
  TYPE_MAC(rA[10+TT0I],rB[1+TT1J],rC[10+1*TT0I]); \
  TYPE_MAC(rA[11+TT0I],rB[1+TT1J],rC[11+1*TT0I]); \
  TYPE_MAC(rA[12+TT0I],rB[1+TT1J],rC[12+1*TT0I]); \
  TYPE_MAC(rA[13+TT0I],rB[1+TT1J],rC[13+1*TT0I]); \
  TYPE_MAC(rA[14+TT0I],rB[1+TT1J],rC[14+1*TT0I]); \
  TYPE_MAC(rA[15+TT0I],rB[1+TT1J],rC[15+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_BSS_BH_MT128x16x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA2_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT16_2_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_bfloat16)(0)
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_1_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_1_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_1_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_1_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_1_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_1_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_1_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_1_0_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_1_0_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_1_0_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_1_0_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_1_0_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_1_0_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_1_0_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_1_0_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE a_1_0_15_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSCA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;
  unsigned int globalReadOffsetAL_8_0 = globalReadOffsetAL + 0 + 8*LSPA;
  unsigned int globalReadOffsetAL_9_0 = globalReadOffsetAL + 0 + 9*LSPA;
  unsigned int globalReadOffsetAL_10_0 = globalReadOffsetAL + 0 + 10*LSPA;
  unsigned int globalReadOffsetAL_11_0 = globalReadOffsetAL + 0 + 11*LSPA;
  unsigned int globalReadOffsetAL_12_0 = globalReadOffsetAL + 0 + 12*LSPA;
  unsigned int globalReadOffsetAL_13_0 = globalReadOffsetAL + 0 + 13*LSPA;
  unsigned int globalReadOffsetAL_14_0 = globalReadOffsetAL + 0 + 14*LSPA;
  unsigned int globalReadOffsetAL_15_0 = globalReadOffsetAL + 0 + 15*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_1_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_1_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_1_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_1_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_1_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_1_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_1_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_1_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_1_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_1_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_1_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_1_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_1_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_1_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );
  int64_t globalReadOffsetA_1_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_1_0), (globalReadOffsetAL_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_8_0), wgK );
  int64_t globalReadOffsetA_1_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_1_0), (globalReadOffsetAL_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_9_0), wgK );
  int64_t globalReadOffsetA_1_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_1_0), (globalReadOffsetAL_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_10_0), wgK );
  int64_t globalReadOffsetA_1_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_1_0), (globalReadOffsetAL_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_11_0), wgK );
  int64_t globalReadOffsetA_1_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_1_0), (globalReadOffsetAL_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_12_0), wgK );
  int64_t globalReadOffsetA_1_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_1_0), (globalReadOffsetAL_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_13_0), wgK );
  int64_t globalReadOffsetA_1_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_1_0), (globalReadOffsetAL_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_14_0), wgK );
  int64_t globalReadOffsetA_1_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_1_0), (globalReadOffsetAL_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_15_0), wgK );
  int64_t globalReadOffsetA_1_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_1_0), (globalReadOffsetAL_15_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_1_0_0_0 = A + globalReadOffsetA_1_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_1_0_1_0 = A + globalReadOffsetA_1_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_1_0_2_0 = A + globalReadOffsetA_1_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_1_0_3_0 = A + globalReadOffsetA_1_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_1_0_4_0 = A + globalReadOffsetA_1_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_1_0_5_0 = A + globalReadOffsetA_1_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_1_0_6_0 = A + globalReadOffsetA_1_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_1_0_7_0 = A + globalReadOffsetA_1_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_1_0_8_0 = A + globalReadOffsetA_1_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_1_0_9_0 = A + globalReadOffsetA_1_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_1_0_10_0 = A + globalReadOffsetA_1_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_1_0_11_0 = A + globalReadOffsetA_1_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_1_0_12_0 = A + globalReadOffsetA_1_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_1_0_13_0 = A + globalReadOffsetA_1_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_1_0_14_0 = A + globalReadOffsetA_1_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;
  DATA_TYPE const *globalReadA_1_0_15_0 = A + globalReadOffsetA_1_0_15_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_1_0_0_0 = localWriteFirstOffsetA + (0 + 1*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_1_0_1_0 = localWriteFirstOffsetA + (0 + 1*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_1_0_2_0 = localWriteFirstOffsetA + (0 + 1*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_1_0_3_0 = localWriteFirstOffsetA + (0 + 1*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_1_0_4_0 = localWriteFirstOffsetA + (0 + 1*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_1_0_5_0 = localWriteFirstOffsetA + (0 + 1*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_1_0_6_0 = localWriteFirstOffsetA + (0 + 1*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_1_0_7_0 = localWriteFirstOffsetA + (0 + 1*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 8*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_1_0_8_0 = localWriteFirstOffsetA + (0 + 1*LSCA) + (0 + 8*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 9*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_1_0_9_0 = localWriteFirstOffsetA + (0 + 1*LSCA) + (0 + 9*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 10*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_1_0_10_0 = localWriteFirstOffsetA + (0 + 1*LSCA) + (0 + 10*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 11*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_1_0_11_0 = localWriteFirstOffsetA + (0 + 1*LSCA) + (0 + 11*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 12*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_1_0_12_0 = localWriteFirstOffsetA + (0 + 1*LSCA) + (0 + 12*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 13*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_1_0_13_0 = localWriteFirstOffsetA + (0 + 1*LSCA) + (0 + 13*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 14*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_1_0_14_0 = localWriteFirstOffsetA + (0 + 1*LSCA) + (0 + 14*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 15*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_1_0_15_0 = localWriteFirstOffsetA + (0 + 1*LSCA) + (0 + 15*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_1_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_1_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_1_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_1_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_1_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_1_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_1_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_1_0_7_0;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_1_0_8_0;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_1_0_9_0;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_1_0_10_0;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_1_0_11_0;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_1_0_12_0;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_1_0_13_0;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_1_0_14_0;
  DATA_TYPE *localWriteA_0_0_15_0;
  DATA_TYPE *localWriteA_1_0_15_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_1_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_1_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_1_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_1_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_1_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_1_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_1_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_1_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_1_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_1_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_1_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_1_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_1_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_1_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_1_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);
  localWriteA_1_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_1_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_1_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_1_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_1_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_1_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_1_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_1_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_1_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_8_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_1_0_8_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_9_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_1_0_9_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_10_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_1_0_10_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_11_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_1_0_11_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_12_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_1_0_12_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_13_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_1_0_13_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_14_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_1_0_14_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_15_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_1_0_15_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_1_0_0_0 = *(globalReadA_1_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_1_0_1_0 = *(globalReadA_1_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_1_0_2_0 = *(globalReadA_1_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_1_0_3_0 = *(globalReadA_1_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_1_0_4_0 = *(globalReadA_1_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_1_0_5_0 = *(globalReadA_1_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_1_0_6_0 = *(globalReadA_1_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_1_0_7_0 = *(globalReadA_1_0_7_0 + 0);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_1_0_8_0 = *(globalReadA_1_0_8_0 + 0);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_1_0_9_0 = *(globalReadA_1_0_9_0 + 0);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_1_0_10_0 = *(globalReadA_1_0_10_0 + 0);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_1_0_11_0 = *(globalReadA_1_0_11_0 + 0);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_1_0_12_0 = *(globalReadA_1_0_12_0 + 0);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_1_0_13_0 = *(globalReadA_1_0_13_0 + 0);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_1_0_14_0 = *(globalReadA_1_0_14_0 + 0);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);
    a_1_0_15_0 = *(globalReadA_1_0_15_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_1_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_1_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_1_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_1_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_1_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_1_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_1_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_1_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_1_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_1_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_1_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_1_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_1_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_1_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_1_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_1_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_1_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_1_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_1_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_1_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_1_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_1_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_1_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_1_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_8_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_1_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_1_0_8_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_1_0_8_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_9_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_1_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_1_0_9_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_1_0_9_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_10_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_1_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_1_0_10_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_1_0_10_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_11_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_1_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_1_0_11_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_1_0_11_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_12_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_1_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_1_0_12_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_1_0_12_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_13_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_1_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_1_0_13_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_1_0_13_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_14_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_1_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_1_0_14_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_1_0_14_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_15_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_1_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_1_0_15_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_1_0_15_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_1_0_0_0 + 0) = a_1_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_1_0_1_0 + 0) = a_1_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_1_0_2_0 + 0) = a_1_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_1_0_3_0 + 0) = a_1_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_1_0_4_0 + 0) = a_1_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_1_0_5_0 + 0) = a_1_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_1_0_6_0 + 0) = a_1_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_1_0_7_0 + 0) = a_1_0_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_1_0_8_0 + 0) = a_1_0_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_1_0_9_0 + 0) = a_1_0_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_1_0_10_0 + 0) = a_1_0_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_1_0_11_0 + 0) = a_1_0_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_1_0_12_0 + 0) = a_1_0_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_1_0_13_0 + 0) = a_1_0_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_1_0_14_0 + 0) = a_1_0_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
    *(localWriteA_1_0_15_0 + 0) = a_1_0_15_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0+TT0I] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0+TT0I] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0+TT0I] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0+TT0I] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0+TT0I] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0+TT0I] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0+TT0I] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0+TT0I] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0+TT0I] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0+TT0I] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0+TT0I] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0+TT0I] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0+TT0I] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0+TT0I] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0+TT0I] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0+TT0I] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0+TT0I] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0+TT0I] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0+TT0I] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0+TT0I] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0+TT0I] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0+TT0I] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0+TT0I] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0+TT0I] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0+TT0I] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0+TT0I] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0+TT0I] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0+TT0I] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0+TT0I] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0+TT0I] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0+TT0I] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0+TT0I] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0+TT0I] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0+TT0I] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0+TT0I] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0+TT0I] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0+TT0I] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0+TT0I] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0+TT0I] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0+TT0I] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0+TT0I] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0+TT0I] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0+TT0I] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0+TT0I] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0+TT0I] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0+TT0I] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0+TT0I] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0+TT0I] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0+TT0I] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0+TT0I] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0+TT0I] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0+TT0I] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0+TT0I] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0+TT0I] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0+TT0I] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0+TT0I] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0+TT0I] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0+TT0I] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0+TT0I] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0+TT0I] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0+TT0I] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0+TT0I] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0+TT0I] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0+TT0I] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_16x2


    /* iter 15 */

    MAC_16x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_1_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_1_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_1_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_1_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_1_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_1_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_1_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_1_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_1_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_1_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_1_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_1_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_1_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_1_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_1_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_1_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_1_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_1_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_1_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_1_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_1_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_1_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_1_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_1_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_1_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_1_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_1_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_1_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_1_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_1_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_1_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_1_0_7_0 + 0);
  a_0_0_8_0 = ( globalReadOffsetAL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_1_0_8_0 = ( globalReadOffsetAL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_1_0_8_0 + 0);
  a_0_0_9_0 = ( globalReadOffsetAL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_1_0_9_0 = ( globalReadOffsetAL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_1_0_9_0 + 0);
  a_0_0_10_0 = ( globalReadOffsetAL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_1_0_10_0 = ( globalReadOffsetAL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_1_0_10_0 + 0);
  a_0_0_11_0 = ( globalReadOffsetAL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_1_0_11_0 = ( globalReadOffsetAL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_1_0_11_0 + 0);
  a_0_0_12_0 = ( globalReadOffsetAL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_1_0_12_0 = ( globalReadOffsetAL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_1_0_12_0 + 0);
  a_0_0_13_0 = ( globalReadOffsetAL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_1_0_13_0 = ( globalReadOffsetAL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_1_0_13_0 + 0);
  a_0_0_14_0 = ( globalReadOffsetAL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_1_0_14_0 = ( globalReadOffsetAL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_1_0_14_0 + 0);
  a_0_0_15_0 = ( globalReadOffsetAL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);
  a_1_0_15_0 = ( globalReadOffsetAL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_1_0_15_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_1_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_1_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_1_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_1_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_1_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_1_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_1_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_1_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_1_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_1_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_1_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_1_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_1_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_1_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_1_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);
  localWriteA_1_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_1_0_0_0 + 0) = a_1_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_1_0_1_0 + 0) = a_1_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_1_0_2_0 + 0) = a_1_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_1_0_3_0 + 0) = a_1_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_1_0_4_0 + 0) = a_1_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_1_0_5_0 + 0) = a_1_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_1_0_6_0 + 0) = a_1_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_1_0_7_0 + 0) = a_1_0_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_1_0_8_0 + 0) = a_1_0_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_1_0_9_0 + 0) = a_1_0_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_1_0_10_0 + 0) = a_1_0_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_1_0_11_0 + 0) = a_1_0_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_1_0_12_0 + 0) = a_1_0_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_1_0_13_0 + 0) = a_1_0_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_1_0_14_0 + 0) = a_1_0_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
  *(localWriteA_1_0_15_0 + 0) = a_1_0_15_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_16x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  8*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 8*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[8*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  9*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 9*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[9*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  10*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 10*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[10*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  11*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 11*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[11*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  12*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 12*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[12*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  13*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 13*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[13*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  14*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 14*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[14*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  15*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 15*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[15*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  8*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 8*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[8*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  9*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 9*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[9*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  10*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 10*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[10*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  11*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 11*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[11*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  12*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 12*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[12*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  13*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 13*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[13*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  14*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 14*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[14*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  15*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 15*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[15*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_16x2
#undef MAC_16x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 128
#define LDS_NUM_ELEMENTS 512
#define LDS_OFFSET_BLK 256

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_4xi8BH_MT16x16x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 896
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_4xi8BH_MT32x16x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x2


    /* iter 7 (last unrolled loop) */

    MAC_4x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_4xi8BH_MT32x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_0_2;
  DATA_TYPE *localWriteA_0_0_0_3;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 7 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_3 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_3 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
  *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 896
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_4xi8BH_MT32x16x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_8_1_WGM1(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 896
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_4xi8BH_MT32x16x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_8_1_WGM8(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_4xi8BH_MT16x16x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 15 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1792
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_4xi8BH_MT32x16x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_8_1_WGM1(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 15 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_4xi8BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM1(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_0_2;
  DATA_TYPE *localWriteA_0_0_0_3;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_3 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_3 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
  *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_4xi8BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_0_2;
  DATA_TYPE *localWriteA_0_0_0_3;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_3 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_3 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
  *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 64
#define LDS_NUM_ELEMENTS 409
#define LDS_OFFSET_BLK 128

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_4xi8BH_MT16x16x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 3 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 3 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 2
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 64
#define LDS_NUM_ELEMENTS 409
#define LDS_OFFSET_BLK 128

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_4xi8BH_MT32x32x2_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 1 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 1 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 3072

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? static_cast<tensile_bfloat16>((ALPHA)*(REG) + (BETA) * static_cast<float>(SRC)) : static_cast<tensile_bfloat16>((ALPHA)*(REG));

/* 8x4 micro-tile */
#define MAC_8x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7],rB[3],rC[7+3*TT0I]); \

#define MAC_8x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_BBH_MT128x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_4_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_bfloat16 *D,
  tensile_bfloat16 const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;
  unsigned int flattenedOffsetA_4_0 = globalReadOffsetA0I + 0 + 4*LSPA;
  flattenedOffsetA_4_0 = (flattenedOffsetA_4_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_4_0;
  unsigned int globalReadOffsetA0I_4_0 = flattenedOffsetA_4_0;
  unsigned int flattenedOffsetA_5_0 = globalReadOffsetA0I + 0 + 5*LSPA;
  flattenedOffsetA_5_0 = (flattenedOffsetA_5_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_5_0;
  unsigned int globalReadOffsetA0I_5_0 = flattenedOffsetA_5_0;
  unsigned int flattenedOffsetA_6_0 = globalReadOffsetA0I + 0 + 6*LSPA;
  flattenedOffsetA_6_0 = (flattenedOffsetA_6_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_6_0;
  unsigned int globalReadOffsetA0I_6_0 = flattenedOffsetA_6_0;
  unsigned int flattenedOffsetA_7_0 = globalReadOffsetA0I + 0 + 7*LSPA;
  flattenedOffsetA_7_0 = (flattenedOffsetA_7_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_7_0;
  unsigned int globalReadOffsetA0I_7_0 = flattenedOffsetA_7_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;
  globalReadOffsetA0I_4_0 = (  globalReadOffsetA0I_4_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_4_0;
  globalReadOffsetA0I_5_0 = (  globalReadOffsetA0I_5_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_5_0;
  globalReadOffsetA0I_6_0 = (  globalReadOffsetA0I_6_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_6_0;
  globalReadOffsetA0I_7_0 = (  globalReadOffsetA0I_7_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_7_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 4*LSPA);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 5*LSPA);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 6*LSPA);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 7*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x4


    /* iter 15 */

    MAC_8x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x4
#undef MAC_8x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 16
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 16
#define NLPB 8

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 4096
#define LDS_NUM_ELEMENTS 6144

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? static_cast<tensile_bfloat16>((ALPHA)*(REG) + (BETA) * static_cast<float>(SRC)) : static_cast<tensile_bfloat16>((ALPHA)*(REG));

/* 16x8 micro-tile */
#define MAC_16x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[8],rB[0],rC[8+0*TT0I]); \
  TYPE_MAC(rA[9],rB[0],rC[9+0*TT0I]); \
  TYPE_MAC(rA[10],rB[0],rC[10+0*TT0I]); \
  TYPE_MAC(rA[11],rB[0],rC[11+0*TT0I]); \
  TYPE_MAC(rA[12],rB[0],rC[12+0*TT0I]); \
  TYPE_MAC(rA[13],rB[0],rC[13+0*TT0I]); \
  TYPE_MAC(rA[14],rB[0],rC[14+0*TT0I]); \
  TYPE_MAC(rA[15],rB[0],rC[15+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[8],rB[1],rC[8+1*TT0I]); \
  TYPE_MAC(rA[9],rB[1],rC[9+1*TT0I]); \
  TYPE_MAC(rA[10],rB[1],rC[10+1*TT0I]); \
  TYPE_MAC(rA[11],rB[1],rC[11+1*TT0I]); \
  TYPE_MAC(rA[12],rB[1],rC[12+1*TT0I]); \
  TYPE_MAC(rA[13],rB[1],rC[13+1*TT0I]); \
  TYPE_MAC(rA[14],rB[1],rC[14+1*TT0I]); \
  TYPE_MAC(rA[15],rB[1],rC[15+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[8],rB[2],rC[8+2*TT0I]); \
  TYPE_MAC(rA[9],rB[2],rC[9+2*TT0I]); \
  TYPE_MAC(rA[10],rB[2],rC[10+2*TT0I]); \
  TYPE_MAC(rA[11],rB[2],rC[11+2*TT0I]); \
  TYPE_MAC(rA[12],rB[2],rC[12+2*TT0I]); \
  TYPE_MAC(rA[13],rB[2],rC[13+2*TT0I]); \
  TYPE_MAC(rA[14],rB[2],rC[14+2*TT0I]); \
  TYPE_MAC(rA[15],rB[2],rC[15+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[8],rB[3],rC[8+3*TT0I]); \
  TYPE_MAC(rA[9],rB[3],rC[9+3*TT0I]); \
  TYPE_MAC(rA[10],rB[3],rC[10+3*TT0I]); \
  TYPE_MAC(rA[11],rB[3],rC[11+3*TT0I]); \
  TYPE_MAC(rA[12],rB[3],rC[12+3*TT0I]); \
  TYPE_MAC(rA[13],rB[3],rC[13+3*TT0I]); \
  TYPE_MAC(rA[14],rB[3],rC[14+3*TT0I]); \
  TYPE_MAC(rA[15],rB[3],rC[15+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I]); \
  TYPE_MAC(rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I]); \
  TYPE_MAC(rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[8],rB[4],rC[8+4*TT0I]); \
  TYPE_MAC(rA[9],rB[4],rC[9+4*TT0I]); \
  TYPE_MAC(rA[10],rB[4],rC[10+4*TT0I]); \
  TYPE_MAC(rA[11],rB[4],rC[11+4*TT0I]); \
  TYPE_MAC(rA[12],rB[4],rC[12+4*TT0I]); \
  TYPE_MAC(rA[13],rB[4],rC[13+4*TT0I]); \
  TYPE_MAC(rA[14],rB[4],rC[14+4*TT0I]); \
  TYPE_MAC(rA[15],rB[4],rC[15+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I]); \
  TYPE_MAC(rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I]); \
  TYPE_MAC(rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[8],rB[5],rC[8+5*TT0I]); \
  TYPE_MAC(rA[9],rB[5],rC[9+5*TT0I]); \
  TYPE_MAC(rA[10],rB[5],rC[10+5*TT0I]); \
  TYPE_MAC(rA[11],rB[5],rC[11+5*TT0I]); \
  TYPE_MAC(rA[12],rB[5],rC[12+5*TT0I]); \
  TYPE_MAC(rA[13],rB[5],rC[13+5*TT0I]); \
  TYPE_MAC(rA[14],rB[5],rC[14+5*TT0I]); \
  TYPE_MAC(rA[15],rB[5],rC[15+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I]); \
  TYPE_MAC(rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I]); \
  TYPE_MAC(rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[8],rB[6],rC[8+6*TT0I]); \
  TYPE_MAC(rA[9],rB[6],rC[9+6*TT0I]); \
  TYPE_MAC(rA[10],rB[6],rC[10+6*TT0I]); \
  TYPE_MAC(rA[11],rB[6],rC[11+6*TT0I]); \
  TYPE_MAC(rA[12],rB[6],rC[12+6*TT0I]); \
  TYPE_MAC(rA[13],rB[6],rC[13+6*TT0I]); \
  TYPE_MAC(rA[14],rB[6],rC[14+6*TT0I]); \
  TYPE_MAC(rA[15],rB[6],rC[15+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I]); \
  TYPE_MAC(rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I]); \
  TYPE_MAC(rA[7],rB[7],rC[7+7*TT0I]); \
  TYPE_MAC(rA[8],rB[7],rC[8+7*TT0I]); \
  TYPE_MAC(rA[9],rB[7],rC[9+7*TT0I]); \
  TYPE_MAC(rA[10],rB[7],rC[10+7*TT0I]); \
  TYPE_MAC(rA[11],rB[7],rC[11+7*TT0I]); \
  TYPE_MAC(rA[12],rB[7],rC[12+7*TT0I]); \
  TYPE_MAC(rA[13],rB[7],rC[13+7*TT0I]); \
  TYPE_MAC(rA[14],rB[7],rC[14+7*TT0I]); \
  TYPE_MAC(rA[15],rB[7],rC[15+7*TT0I]); \

#define MAC_16x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[8+TT0I],rB[0+TT1J],rC[8+0*TT0I]); \
  TYPE_MAC(rA[9+TT0I],rB[0+TT1J],rC[9+0*TT0I]); \
  TYPE_MAC(rA[10+TT0I],rB[0+TT1J],rC[10+0*TT0I]); \
  TYPE_MAC(rA[11+TT0I],rB[0+TT1J],rC[11+0*TT0I]); \
  TYPE_MAC(rA[12+TT0I],rB[0+TT1J],rC[12+0*TT0I]); \
  TYPE_MAC(rA[13+TT0I],rB[0+TT1J],rC[13+0*TT0I]); \
  TYPE_MAC(rA[14+TT0I],rB[0+TT1J],rC[14+0*TT0I]); \
  TYPE_MAC(rA[15+TT0I],rB[0+TT1J],rC[15+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[8+TT0I],rB[1+TT1J],rC[8+1*TT0I]); \
  TYPE_MAC(rA[9+TT0I],rB[1+TT1J],rC[9+1*TT0I]); \
  TYPE_MAC(rA[10+TT0I],rB[1+TT1J],rC[10+1*TT0I]); \
  TYPE_MAC(rA[11+TT0I],rB[1+TT1J],rC[11+1*TT0I]); \
  TYPE_MAC(rA[12+TT0I],rB[1+TT1J],rC[12+1*TT0I]); \
  TYPE_MAC(rA[13+TT0I],rB[1+TT1J],rC[13+1*TT0I]); \
  TYPE_MAC(rA[14+TT0I],rB[1+TT1J],rC[14+1*TT0I]); \
  TYPE_MAC(rA[15+TT0I],rB[1+TT1J],rC[15+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[8+TT0I],rB[2+TT1J],rC[8+2*TT0I]); \
  TYPE_MAC(rA[9+TT0I],rB[2+TT1J],rC[9+2*TT0I]); \
  TYPE_MAC(rA[10+TT0I],rB[2+TT1J],rC[10+2*TT0I]); \
  TYPE_MAC(rA[11+TT0I],rB[2+TT1J],rC[11+2*TT0I]); \
  TYPE_MAC(rA[12+TT0I],rB[2+TT1J],rC[12+2*TT0I]); \
  TYPE_MAC(rA[13+TT0I],rB[2+TT1J],rC[13+2*TT0I]); \
  TYPE_MAC(rA[14+TT0I],rB[2+TT1J],rC[14+2*TT0I]); \
  TYPE_MAC(rA[15+TT0I],rB[2+TT1J],rC[15+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \
  TYPE_MAC(rA[8+TT0I],rB[3+TT1J],rC[8+3*TT0I]); \
  TYPE_MAC(rA[9+TT0I],rB[3+TT1J],rC[9+3*TT0I]); \
  TYPE_MAC(rA[10+TT0I],rB[3+TT1J],rC[10+3*TT0I]); \
  TYPE_MAC(rA[11+TT0I],rB[3+TT1J],rC[11+3*TT0I]); \
  TYPE_MAC(rA[12+TT0I],rB[3+TT1J],rC[12+3*TT0I]); \
  TYPE_MAC(rA[13+TT0I],rB[3+TT1J],rC[13+3*TT0I]); \
  TYPE_MAC(rA[14+TT0I],rB[3+TT1J],rC[14+3*TT0I]); \
  TYPE_MAC(rA[15+TT0I],rB[3+TT1J],rC[15+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[4+TT1J],rC[4+4*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[4+TT1J],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[4+TT1J],rC[6+4*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[4+TT1J],rC[7+4*TT0I]); \
  TYPE_MAC(rA[8+TT0I],rB[4+TT1J],rC[8+4*TT0I]); \
  TYPE_MAC(rA[9+TT0I],rB[4+TT1J],rC[9+4*TT0I]); \
  TYPE_MAC(rA[10+TT0I],rB[4+TT1J],rC[10+4*TT0I]); \
  TYPE_MAC(rA[11+TT0I],rB[4+TT1J],rC[11+4*TT0I]); \
  TYPE_MAC(rA[12+TT0I],rB[4+TT1J],rC[12+4*TT0I]); \
  TYPE_MAC(rA[13+TT0I],rB[4+TT1J],rC[13+4*TT0I]); \
  TYPE_MAC(rA[14+TT0I],rB[4+TT1J],rC[14+4*TT0I]); \
  TYPE_MAC(rA[15+TT0I],rB[4+TT1J],rC[15+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[5+TT1J],rC[4+5*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[5+TT1J],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[5+TT1J],rC[6+5*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[5+TT1J],rC[7+5*TT0I]); \
  TYPE_MAC(rA[8+TT0I],rB[5+TT1J],rC[8+5*TT0I]); \
  TYPE_MAC(rA[9+TT0I],rB[5+TT1J],rC[9+5*TT0I]); \
  TYPE_MAC(rA[10+TT0I],rB[5+TT1J],rC[10+5*TT0I]); \
  TYPE_MAC(rA[11+TT0I],rB[5+TT1J],rC[11+5*TT0I]); \
  TYPE_MAC(rA[12+TT0I],rB[5+TT1J],rC[12+5*TT0I]); \
  TYPE_MAC(rA[13+TT0I],rB[5+TT1J],rC[13+5*TT0I]); \
  TYPE_MAC(rA[14+TT0I],rB[5+TT1J],rC[14+5*TT0I]); \
  TYPE_MAC(rA[15+TT0I],rB[5+TT1J],rC[15+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[6+TT1J],rC[4+6*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[6+TT1J],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[6+TT1J],rC[6+6*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[6+TT1J],rC[7+6*TT0I]); \
  TYPE_MAC(rA[8+TT0I],rB[6+TT1J],rC[8+6*TT0I]); \
  TYPE_MAC(rA[9+TT0I],rB[6+TT1J],rC[9+6*TT0I]); \
  TYPE_MAC(rA[10+TT0I],rB[6+TT1J],rC[10+6*TT0I]); \
  TYPE_MAC(rA[11+TT0I],rB[6+TT1J],rC[11+6*TT0I]); \
  TYPE_MAC(rA[12+TT0I],rB[6+TT1J],rC[12+6*TT0I]); \
  TYPE_MAC(rA[13+TT0I],rB[6+TT1J],rC[13+6*TT0I]); \
  TYPE_MAC(rA[14+TT0I],rB[6+TT1J],rC[14+6*TT0I]); \
  TYPE_MAC(rA[15+TT0I],rB[6+TT1J],rC[15+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[7+TT1J],rC[4+7*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[7+TT1J],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[7+TT1J],rC[6+7*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[7+TT1J],rC[7+7*TT0I]); \
  TYPE_MAC(rA[8+TT0I],rB[7+TT1J],rC[8+7*TT0I]); \
  TYPE_MAC(rA[9+TT0I],rB[7+TT1J],rC[9+7*TT0I]); \
  TYPE_MAC(rA[10+TT0I],rB[7+TT1J],rC[10+7*TT0I]); \
  TYPE_MAC(rA[11+TT0I],rB[7+TT1J],rC[11+7*TT0I]); \
  TYPE_MAC(rA[12+TT0I],rB[7+TT1J],rC[12+7*TT0I]); \
  TYPE_MAC(rA[13+TT0I],rB[7+TT1J],rC[13+7*TT0I]); \
  TYPE_MAC(rA[14+TT0I],rB[7+TT1J],rC[14+7*TT0I]); \
  TYPE_MAC(rA[15+TT0I],rB[7+TT1J],rC[15+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_BBH_MT256x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT16_8_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_bfloat16 *D,
  tensile_bfloat16 const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;
  unsigned int flattenedOffsetA_4_0 = globalReadOffsetA0I + 0 + 4*LSPA;
  flattenedOffsetA_4_0 = (flattenedOffsetA_4_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_4_0;
  unsigned int globalReadOffsetA0I_4_0 = flattenedOffsetA_4_0;
  unsigned int flattenedOffsetA_5_0 = globalReadOffsetA0I + 0 + 5*LSPA;
  flattenedOffsetA_5_0 = (flattenedOffsetA_5_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_5_0;
  unsigned int globalReadOffsetA0I_5_0 = flattenedOffsetA_5_0;
  unsigned int flattenedOffsetA_6_0 = globalReadOffsetA0I + 0 + 6*LSPA;
  flattenedOffsetA_6_0 = (flattenedOffsetA_6_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_6_0;
  unsigned int globalReadOffsetA0I_6_0 = flattenedOffsetA_6_0;
  unsigned int flattenedOffsetA_7_0 = globalReadOffsetA0I + 0 + 7*LSPA;
  flattenedOffsetA_7_0 = (flattenedOffsetA_7_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_7_0;
  unsigned int globalReadOffsetA0I_7_0 = flattenedOffsetA_7_0;
  unsigned int flattenedOffsetA_8_0 = globalReadOffsetA0I + 0 + 8*LSPA;
  flattenedOffsetA_8_0 = (flattenedOffsetA_8_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_8_0;
  unsigned int globalReadOffsetA0I_8_0 = flattenedOffsetA_8_0;
  unsigned int flattenedOffsetA_9_0 = globalReadOffsetA0I + 0 + 9*LSPA;
  flattenedOffsetA_9_0 = (flattenedOffsetA_9_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_9_0;
  unsigned int globalReadOffsetA0I_9_0 = flattenedOffsetA_9_0;
  unsigned int flattenedOffsetA_10_0 = globalReadOffsetA0I + 0 + 10*LSPA;
  flattenedOffsetA_10_0 = (flattenedOffsetA_10_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_10_0;
  unsigned int globalReadOffsetA0I_10_0 = flattenedOffsetA_10_0;
  unsigned int flattenedOffsetA_11_0 = globalReadOffsetA0I + 0 + 11*LSPA;
  flattenedOffsetA_11_0 = (flattenedOffsetA_11_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_11_0;
  unsigned int globalReadOffsetA0I_11_0 = flattenedOffsetA_11_0;
  unsigned int flattenedOffsetA_12_0 = globalReadOffsetA0I + 0 + 12*LSPA;
  flattenedOffsetA_12_0 = (flattenedOffsetA_12_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_12_0;
  unsigned int globalReadOffsetA0I_12_0 = flattenedOffsetA_12_0;
  unsigned int flattenedOffsetA_13_0 = globalReadOffsetA0I + 0 + 13*LSPA;
  flattenedOffsetA_13_0 = (flattenedOffsetA_13_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_13_0;
  unsigned int globalReadOffsetA0I_13_0 = flattenedOffsetA_13_0;
  unsigned int flattenedOffsetA_14_0 = globalReadOffsetA0I + 0 + 14*LSPA;
  flattenedOffsetA_14_0 = (flattenedOffsetA_14_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_14_0;
  unsigned int globalReadOffsetA0I_14_0 = flattenedOffsetA_14_0;
  unsigned int flattenedOffsetA_15_0 = globalReadOffsetA0I + 0 + 15*LSPA;
  flattenedOffsetA_15_0 = (flattenedOffsetA_15_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_15_0;
  unsigned int globalReadOffsetA0I_15_0 = flattenedOffsetA_15_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;
  unsigned int flattenedOffsetB_4_0 = globalReadOffsetB1J + 0 + 4*LSPB;
  flattenedOffsetB_4_0 = (flattenedOffsetB_4_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_4_0;
  unsigned int globalReadOffsetB1J_4_0 = flattenedOffsetB_4_0;
  unsigned int flattenedOffsetB_5_0 = globalReadOffsetB1J + 0 + 5*LSPB;
  flattenedOffsetB_5_0 = (flattenedOffsetB_5_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_5_0;
  unsigned int globalReadOffsetB1J_5_0 = flattenedOffsetB_5_0;
  unsigned int flattenedOffsetB_6_0 = globalReadOffsetB1J + 0 + 6*LSPB;
  flattenedOffsetB_6_0 = (flattenedOffsetB_6_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_6_0;
  unsigned int globalReadOffsetB1J_6_0 = flattenedOffsetB_6_0;
  unsigned int flattenedOffsetB_7_0 = globalReadOffsetB1J + 0 + 7*LSPB;
  flattenedOffsetB_7_0 = (flattenedOffsetB_7_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_7_0;
  unsigned int globalReadOffsetB1J_7_0 = flattenedOffsetB_7_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;
  globalReadOffsetA0I_4_0 = (  globalReadOffsetA0I_4_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_4_0;
  globalReadOffsetA0I_5_0 = (  globalReadOffsetA0I_5_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_5_0;
  globalReadOffsetA0I_6_0 = (  globalReadOffsetA0I_6_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_6_0;
  globalReadOffsetA0I_7_0 = (  globalReadOffsetA0I_7_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_7_0;
  globalReadOffsetA0I_8_0 = (  globalReadOffsetA0I_8_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_8_0;
  globalReadOffsetA0I_9_0 = (  globalReadOffsetA0I_9_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_9_0;
  globalReadOffsetA0I_10_0 = (  globalReadOffsetA0I_10_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_10_0;
  globalReadOffsetA0I_11_0 = (  globalReadOffsetA0I_11_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_11_0;
  globalReadOffsetA0I_12_0 = (  globalReadOffsetA0I_12_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_12_0;
  globalReadOffsetA0I_13_0 = (  globalReadOffsetA0I_13_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_13_0;
  globalReadOffsetA0I_14_0 = (  globalReadOffsetA0I_14_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_14_0;
  globalReadOffsetA0I_15_0 = (  globalReadOffsetA0I_15_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_15_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;
  globalReadOffsetB1J_4_0 = (  globalReadOffsetB1J_4_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_4_0;
  globalReadOffsetB1J_5_0 = (  globalReadOffsetB1J_5_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_5_0;
  globalReadOffsetB1J_6_0 = (  globalReadOffsetB1J_6_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_6_0;
  globalReadOffsetB1J_7_0 = (  globalReadOffsetB1J_7_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_7_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_15_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_7_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 4*LSPA);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 5*LSPA);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 6*LSPA);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 7*LSPA);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 8*LSPA);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 9*LSPA);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 10*LSPA);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 11*LSPA);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 12*LSPA);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 13*LSPA);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 14*LSPA);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 15*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 4*LSPB);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 5*LSPB);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 6*LSPB);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 7*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_0_0_15_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;
  rC[64] = SCALAR_ZERO;
  rC[65] = SCALAR_ZERO;
  rC[66] = SCALAR_ZERO;
  rC[67] = SCALAR_ZERO;
  rC[68] = SCALAR_ZERO;
  rC[69] = SCALAR_ZERO;
  rC[70] = SCALAR_ZERO;
  rC[71] = SCALAR_ZERO;
  rC[72] = SCALAR_ZERO;
  rC[73] = SCALAR_ZERO;
  rC[74] = SCALAR_ZERO;
  rC[75] = SCALAR_ZERO;
  rC[76] = SCALAR_ZERO;
  rC[77] = SCALAR_ZERO;
  rC[78] = SCALAR_ZERO;
  rC[79] = SCALAR_ZERO;
  rC[80] = SCALAR_ZERO;
  rC[81] = SCALAR_ZERO;
  rC[82] = SCALAR_ZERO;
  rC[83] = SCALAR_ZERO;
  rC[84] = SCALAR_ZERO;
  rC[85] = SCALAR_ZERO;
  rC[86] = SCALAR_ZERO;
  rC[87] = SCALAR_ZERO;
  rC[88] = SCALAR_ZERO;
  rC[89] = SCALAR_ZERO;
  rC[90] = SCALAR_ZERO;
  rC[91] = SCALAR_ZERO;
  rC[92] = SCALAR_ZERO;
  rC[93] = SCALAR_ZERO;
  rC[94] = SCALAR_ZERO;
  rC[95] = SCALAR_ZERO;
  rC[96] = SCALAR_ZERO;
  rC[97] = SCALAR_ZERO;
  rC[98] = SCALAR_ZERO;
  rC[99] = SCALAR_ZERO;
  rC[100] = SCALAR_ZERO;
  rC[101] = SCALAR_ZERO;
  rC[102] = SCALAR_ZERO;
  rC[103] = SCALAR_ZERO;
  rC[104] = SCALAR_ZERO;
  rC[105] = SCALAR_ZERO;
  rC[106] = SCALAR_ZERO;
  rC[107] = SCALAR_ZERO;
  rC[108] = SCALAR_ZERO;
  rC[109] = SCALAR_ZERO;
  rC[110] = SCALAR_ZERO;
  rC[111] = SCALAR_ZERO;
  rC[112] = SCALAR_ZERO;
  rC[113] = SCALAR_ZERO;
  rC[114] = SCALAR_ZERO;
  rC[115] = SCALAR_ZERO;
  rC[116] = SCALAR_ZERO;
  rC[117] = SCALAR_ZERO;
  rC[118] = SCALAR_ZERO;
  rC[119] = SCALAR_ZERO;
  rC[120] = SCALAR_ZERO;
  rC[121] = SCALAR_ZERO;
  rC[122] = SCALAR_ZERO;
  rC[123] = SCALAR_ZERO;
  rC[124] = SCALAR_ZERO;
  rC[125] = SCALAR_ZERO;
  rC[126] = SCALAR_ZERO;
  rC[127] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_8_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_9_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_10_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_11_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_12_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_13_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_14_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_15_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_8_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_9_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_10_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_11_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_12_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_13_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_14_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_15_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0+TT0I] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0+TT0I] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0+TT0I] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0+TT0I] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0+TT0I] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0+TT0I] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0+TT0I] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0+TT0I] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0+TT0I] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0+TT0I] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0+TT0I] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0+TT0I] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0+TT0I] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0+TT0I] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0+TT0I] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0+TT0I] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0+TT0I] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0+TT0I] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0+TT0I] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0+TT0I] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0+TT0I] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0+TT0I] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0+TT0I] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0+TT0I] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0+TT0I] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0+TT0I] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0+TT0I] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0+TT0I] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0+TT0I] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0+TT0I] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0+TT0I] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0+TT0I] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0+TT0I] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0+TT0I] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0+TT0I] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0+TT0I] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0+TT0I] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0+TT0I] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0+TT0I] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0+TT0I] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0+TT0I] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0+TT0I] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0+TT0I] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0+TT0I] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0+TT0I] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0+TT0I] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0+TT0I] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0+TT0I] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0+TT0I] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0+TT0I] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0+TT0I] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0+TT0I] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0+TT0I] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0+TT0I] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0+TT0I] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0+TT0I] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0+TT0I] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0+TT0I] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0+TT0I] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0+TT0I] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0+TT0I] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0+TT0I] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0+TT0I] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0+TT0I] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_16x8


    /* iter 15 */

    MAC_16x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_0_8_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_0_0_9_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_0_0_10_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_0_0_11_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_0_0_12_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_0_0_13_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_0_0_14_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_0_0_15_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_16x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  8*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 8*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[8*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  9*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 9*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[9*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  10*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 10*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[10*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  11*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 11*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[11*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  12*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 12*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[12*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  13*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 13*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[13*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  14*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 14*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[14*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  15*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 15*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[15*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  8*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 8*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[8*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  9*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 9*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[9*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  10*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 10*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[10*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  11*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 11*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[11*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  12*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 12*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[12*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  13*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 13*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[13*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  14*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 14*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[14*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  15*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 15*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[15*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  8*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 8*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[8*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  9*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 9*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[9*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  10*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 10*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[10*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  11*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 11*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[11*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  12*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 12*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[12*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  13*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 13*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[13*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  14*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 14*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[14*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  15*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 15*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[15*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  8*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 8*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[8*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  9*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 9*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[9*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  10*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 10*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[10*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  11*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 11*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[11*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  12*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 12*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[12*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  13*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 13*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[13*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  14*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 14*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[14*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  15*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 15*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[15*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  8*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 8*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[8*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  9*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 9*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[9*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  10*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 10*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[10*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  11*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 11*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[11*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  12*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 12*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[12*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  13*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 13*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[13*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  14*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 14*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[14*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  15*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 15*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[15*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  8*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 8*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[8*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  9*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 9*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[9*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  10*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 10*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[10*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  11*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 11*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[11*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  12*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 12*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[12*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  13*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 13*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[13*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  14*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 14*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[14*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  15*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 15*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[15*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  8*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 8*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[8*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  9*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 9*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[9*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  10*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 10*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[10*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  11*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 11*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[11*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  12*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 12*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[12*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  13*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 13*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[13*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  14*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 14*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[14*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  15*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 15*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[15*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  8*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 8*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[8*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  9*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 9*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[9*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  10*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 10*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[10*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  11*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 11*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[11*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  12*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 12*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[12*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  13*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 13*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[13*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  14*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 14*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[14*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  15*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 15*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[15*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_16x8
#undef MAC_16x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 3072

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? static_cast<tensile_bfloat16>((ALPHA)*(REG) + (BETA) * static_cast<float>(SRC)) : static_cast<tensile_bfloat16>((ALPHA)*(REG));

/* 8x4 micro-tile */
#define MAC_8x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7],rB[3],rC[7+3*TT0I]); \

#define MAC_8x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_BBH_MT128x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_bfloat16 *D,
  tensile_bfloat16 const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;
  unsigned int flattenedOffsetA_4_0 = globalReadOffsetA0I + 0 + 4*LSPA;
  flattenedOffsetA_4_0 = (flattenedOffsetA_4_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_4_0;
  unsigned int globalReadOffsetA0I_4_0 = flattenedOffsetA_4_0;
  unsigned int flattenedOffsetA_5_0 = globalReadOffsetA0I + 0 + 5*LSPA;
  flattenedOffsetA_5_0 = (flattenedOffsetA_5_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_5_0;
  unsigned int globalReadOffsetA0I_5_0 = flattenedOffsetA_5_0;
  unsigned int flattenedOffsetA_6_0 = globalReadOffsetA0I + 0 + 6*LSPA;
  flattenedOffsetA_6_0 = (flattenedOffsetA_6_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_6_0;
  unsigned int globalReadOffsetA0I_6_0 = flattenedOffsetA_6_0;
  unsigned int flattenedOffsetA_7_0 = globalReadOffsetA0I + 0 + 7*LSPA;
  flattenedOffsetA_7_0 = (flattenedOffsetA_7_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_7_0;
  unsigned int globalReadOffsetA0I_7_0 = flattenedOffsetA_7_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 4*LSPA);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 5*LSPA);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 6*LSPA);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 7*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x4


    /* iter 15 */

    MAC_8x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x4
#undef MAC_8x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 8

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? static_cast<tensile_bfloat16>((ALPHA)*(REG) + (BETA) * static_cast<float>(SRC)) : static_cast<tensile_bfloat16>((ALPHA)*(REG));

/* 2x8 micro-tile */
#define MAC_2x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \

#define MAC_2x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_BBH_MT32x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_8_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_bfloat16 *D,
  tensile_bfloat16 const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;
  unsigned int flattenedOffsetB_4_0 = globalReadOffsetB1J + 0 + 4*LSPB;
  flattenedOffsetB_4_0 = (flattenedOffsetB_4_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_4_0;
  unsigned int globalReadOffsetB1J_4_0 = flattenedOffsetB_4_0;
  unsigned int flattenedOffsetB_5_0 = globalReadOffsetB1J + 0 + 5*LSPB;
  flattenedOffsetB_5_0 = (flattenedOffsetB_5_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_5_0;
  unsigned int globalReadOffsetB1J_5_0 = flattenedOffsetB_5_0;
  unsigned int flattenedOffsetB_6_0 = globalReadOffsetB1J + 0 + 6*LSPB;
  flattenedOffsetB_6_0 = (flattenedOffsetB_6_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_6_0;
  unsigned int globalReadOffsetB1J_6_0 = flattenedOffsetB_6_0;
  unsigned int flattenedOffsetB_7_0 = globalReadOffsetB1J + 0 + 7*LSPB;
  flattenedOffsetB_7_0 = (flattenedOffsetB_7_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_7_0;
  unsigned int globalReadOffsetB1J_7_0 = flattenedOffsetB_7_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_7_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 4*LSPB);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 5*LSPB);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 6*LSPB);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 7*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x8


    /* iter 15 */

    MAC_2x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x8
#undef MAC_2x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 16
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? static_cast<tensile_bfloat16>((ALPHA)*(REG) + (BETA) * static_cast<float>(SRC)) : static_cast<tensile_bfloat16>((ALPHA)*(REG));

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_BBH_MT64x16x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  tensile_bfloat16 *D,
  tensile_bfloat16 const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;
  unsigned int flattenedOffsetA_4_0 = globalReadOffsetA0I + 0 + 4*LSPA;
  flattenedOffsetA_4_0 = (flattenedOffsetA_4_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_4_0;
  unsigned int globalReadOffsetA0I_4_0 = flattenedOffsetA_4_0;
  unsigned int flattenedOffsetA_5_0 = globalReadOffsetA0I + 0 + 5*LSPA;
  flattenedOffsetA_5_0 = (flattenedOffsetA_5_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_5_0;
  unsigned int globalReadOffsetA0I_5_0 = flattenedOffsetA_5_0;
  unsigned int flattenedOffsetA_6_0 = globalReadOffsetA0I + 0 + 6*LSPA;
  flattenedOffsetA_6_0 = (flattenedOffsetA_6_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_6_0;
  unsigned int globalReadOffsetA0I_6_0 = flattenedOffsetA_6_0;
  unsigned int flattenedOffsetA_7_0 = globalReadOffsetA0I + 0 + 7*LSPA;
  flattenedOffsetA_7_0 = (flattenedOffsetA_7_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_7_0;
  unsigned int globalReadOffsetA0I_7_0 = flattenedOffsetA_7_0;
  unsigned int flattenedOffsetA_8_0 = globalReadOffsetA0I + 0 + 8*LSPA;
  flattenedOffsetA_8_0 = (flattenedOffsetA_8_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_8_0;
  unsigned int globalReadOffsetA0I_8_0 = flattenedOffsetA_8_0;
  unsigned int flattenedOffsetA_9_0 = globalReadOffsetA0I + 0 + 9*LSPA;
  flattenedOffsetA_9_0 = (flattenedOffsetA_9_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_9_0;
  unsigned int globalReadOffsetA0I_9_0 = flattenedOffsetA_9_0;
  unsigned int flattenedOffsetA_10_0 = globalReadOffsetA0I + 0 + 10*LSPA;
  flattenedOffsetA_10_0 = (flattenedOffsetA_10_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_10_0;
  unsigned int globalReadOffsetA0I_10_0 = flattenedOffsetA_10_0;
  unsigned int flattenedOffsetA_11_0 = globalReadOffsetA0I + 0 + 11*LSPA;
  flattenedOffsetA_11_0 = (flattenedOffsetA_11_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_11_0;
  unsigned int globalReadOffsetA0I_11_0 = flattenedOffsetA_11_0;
  unsigned int flattenedOffsetA_12_0 = globalReadOffsetA0I + 0 + 12*LSPA;
  flattenedOffsetA_12_0 = (flattenedOffsetA_12_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_12_0;
  unsigned int globalReadOffsetA0I_12_0 = flattenedOffsetA_12_0;
  unsigned int flattenedOffsetA_13_0 = globalReadOffsetA0I + 0 + 13*LSPA;
  flattenedOffsetA_13_0 = (flattenedOffsetA_13_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_13_0;
  unsigned int globalReadOffsetA0I_13_0 = flattenedOffsetA_13_0;
  unsigned int flattenedOffsetA_14_0 = globalReadOffsetA0I + 0 + 14*LSPA;
  flattenedOffsetA_14_0 = (flattenedOffsetA_14_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_14_0;
  unsigned int globalReadOffsetA0I_14_0 = flattenedOffsetA_14_0;
  unsigned int flattenedOffsetA_15_0 = globalReadOffsetA0I + 0 + 15*LSPA;
  flattenedOffsetA_15_0 = (flattenedOffsetA_15_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_15_0;
  unsigned int globalReadOffsetA0I_15_0 = flattenedOffsetA_15_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_15_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 4*LSPA);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 5*LSPA);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 6*LSPA);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 7*LSPA);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 8*LSPA);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 9*LSPA);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 10*LSPA);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 11*LSPA);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 12*LSPA);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 13*LSPA);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 14*LSPA);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 15*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_0_0_15_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_8_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_9_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_10_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_11_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_12_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_13_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_14_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_15_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_8_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_9_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_10_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_11_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_12_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_13_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_14_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_15_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_0_8_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_0_0_9_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_0_0_10_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_0_0_11_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_0_0_12_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_0_0_13_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_0_0_14_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_0_0_15_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 8
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 16
#define NLPB 8

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 1536

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? static_cast<tensile_bfloat16>((ALPHA)*(REG) + (BETA) * static_cast<float>(SRC)) : static_cast<tensile_bfloat16>((ALPHA)*(REG));

/* 8x4 micro-tile */
#define MAC_8x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7],rB[3],rC[7+3*TT0I]); \

#define MAC_8x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_BBH_MT64x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  tensile_bfloat16 *D,
  tensile_bfloat16 const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;
  unsigned int flattenedOffsetA_4_0 = globalReadOffsetA0I + 0 + 4*LSPA;
  flattenedOffsetA_4_0 = (flattenedOffsetA_4_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_4_0;
  unsigned int globalReadOffsetA0I_4_0 = flattenedOffsetA_4_0;
  unsigned int flattenedOffsetA_5_0 = globalReadOffsetA0I + 0 + 5*LSPA;
  flattenedOffsetA_5_0 = (flattenedOffsetA_5_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_5_0;
  unsigned int globalReadOffsetA0I_5_0 = flattenedOffsetA_5_0;
  unsigned int flattenedOffsetA_6_0 = globalReadOffsetA0I + 0 + 6*LSPA;
  flattenedOffsetA_6_0 = (flattenedOffsetA_6_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_6_0;
  unsigned int globalReadOffsetA0I_6_0 = flattenedOffsetA_6_0;
  unsigned int flattenedOffsetA_7_0 = globalReadOffsetA0I + 0 + 7*LSPA;
  flattenedOffsetA_7_0 = (flattenedOffsetA_7_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_7_0;
  unsigned int globalReadOffsetA0I_7_0 = flattenedOffsetA_7_0;
  unsigned int flattenedOffsetA_8_0 = globalReadOffsetA0I + 0 + 8*LSPA;
  flattenedOffsetA_8_0 = (flattenedOffsetA_8_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_8_0;
  unsigned int globalReadOffsetA0I_8_0 = flattenedOffsetA_8_0;
  unsigned int flattenedOffsetA_9_0 = globalReadOffsetA0I + 0 + 9*LSPA;
  flattenedOffsetA_9_0 = (flattenedOffsetA_9_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_9_0;
  unsigned int globalReadOffsetA0I_9_0 = flattenedOffsetA_9_0;
  unsigned int flattenedOffsetA_10_0 = globalReadOffsetA0I + 0 + 10*LSPA;
  flattenedOffsetA_10_0 = (flattenedOffsetA_10_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_10_0;
  unsigned int globalReadOffsetA0I_10_0 = flattenedOffsetA_10_0;
  unsigned int flattenedOffsetA_11_0 = globalReadOffsetA0I + 0 + 11*LSPA;
  flattenedOffsetA_11_0 = (flattenedOffsetA_11_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_11_0;
  unsigned int globalReadOffsetA0I_11_0 = flattenedOffsetA_11_0;
  unsigned int flattenedOffsetA_12_0 = globalReadOffsetA0I + 0 + 12*LSPA;
  flattenedOffsetA_12_0 = (flattenedOffsetA_12_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_12_0;
  unsigned int globalReadOffsetA0I_12_0 = flattenedOffsetA_12_0;
  unsigned int flattenedOffsetA_13_0 = globalReadOffsetA0I + 0 + 13*LSPA;
  flattenedOffsetA_13_0 = (flattenedOffsetA_13_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_13_0;
  unsigned int globalReadOffsetA0I_13_0 = flattenedOffsetA_13_0;
  unsigned int flattenedOffsetA_14_0 = globalReadOffsetA0I + 0 + 14*LSPA;
  flattenedOffsetA_14_0 = (flattenedOffsetA_14_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_14_0;
  unsigned int globalReadOffsetA0I_14_0 = flattenedOffsetA_14_0;
  unsigned int flattenedOffsetA_15_0 = globalReadOffsetA0I + 0 + 15*LSPA;
  flattenedOffsetA_15_0 = (flattenedOffsetA_15_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_15_0;
  unsigned int globalReadOffsetA0I_15_0 = flattenedOffsetA_15_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;
  unsigned int flattenedOffsetB_4_0 = globalReadOffsetB1J + 0 + 4*LSPB;
  flattenedOffsetB_4_0 = (flattenedOffsetB_4_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_4_0;
  unsigned int globalReadOffsetB1J_4_0 = flattenedOffsetB_4_0;
  unsigned int flattenedOffsetB_5_0 = globalReadOffsetB1J + 0 + 5*LSPB;
  flattenedOffsetB_5_0 = (flattenedOffsetB_5_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_5_0;
  unsigned int globalReadOffsetB1J_5_0 = flattenedOffsetB_5_0;
  unsigned int flattenedOffsetB_6_0 = globalReadOffsetB1J + 0 + 6*LSPB;
  flattenedOffsetB_6_0 = (flattenedOffsetB_6_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_6_0;
  unsigned int globalReadOffsetB1J_6_0 = flattenedOffsetB_6_0;
  unsigned int flattenedOffsetB_7_0 = globalReadOffsetB1J + 0 + 7*LSPB;
  flattenedOffsetB_7_0 = (flattenedOffsetB_7_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_7_0;
  unsigned int globalReadOffsetB1J_7_0 = flattenedOffsetB_7_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_15_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_7_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 4*LSPA);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 5*LSPA);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 6*LSPA);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 7*LSPA);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 8*LSPA);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 9*LSPA);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 10*LSPA);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 11*LSPA);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 12*LSPA);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 13*LSPA);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 14*LSPA);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 15*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 4*LSPB);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 5*LSPB);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 6*LSPB);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 7*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_0_0_15_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_8_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_9_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_10_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_11_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_12_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_13_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_14_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_15_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_8_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_9_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_10_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_11_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_12_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_13_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_14_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_15_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x4


    /* iter 15 */

    MAC_8x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_0_8_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_0_0_9_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_0_0_10_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_0_0_11_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_0_0_12_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_0_0_13_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_0_0_14_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_0_0_15_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x4
#undef MAC_8x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 3 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I] , rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I] , rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I] , rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I] , rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I] , rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I] , rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I] , rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I] , rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I] , rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I] , rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I] , rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I] , rA[7],rB[7],rC[7+7*TT0I]); \

#define MAC_8x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I] , rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I] , rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I] , rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I] , rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[4+TT1J],rC[4+4*TT0I] , rA[5+TT0I],rB[4+TT1J],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[4+TT1J],rC[6+4*TT0I] , rA[7+TT0I],rB[4+TT1J],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[5+TT1J],rC[4+5*TT0I] , rA[5+TT0I],rB[5+TT1J],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[5+TT1J],rC[6+5*TT0I] , rA[7+TT0I],rB[5+TT1J],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[6+TT1J],rC[4+6*TT0I] , rA[5+TT0I],rB[6+TT1J],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[6+TT1J],rC[6+6*TT0I] , rA[7+TT0I],rB[6+TT1J],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[7+TT1J],rC[4+7*TT0I] , rA[5+TT0I],rB[7+TT1J],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[7+TT1J],rC[6+7*TT0I] , rA[7+TT0I],rB[7+TT1J],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT128x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8


    /* iter 3 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_8x8


    /* iter 3 (last unrolled loop) */

    MAC_8x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else if (s0I == 1) {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else if (s0I == 2) {
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else {
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 1) {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 2) {
        rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef MAC_8x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1792
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 3 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x8


    /* iter 3 (last unrolled loop) */

    MAC_4x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 1) {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 2) {
        rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 3 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 3 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 7 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I] , rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I] , rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I] , rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I] , rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I] , rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I] , rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I] , rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I] , rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I] , rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I] , rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I] , rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I] , rA[7],rB[7],rC[7+7*TT0I]); \

#define MAC_8x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I] , rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I] , rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I] , rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I] , rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[4+TT1J],rC[4+4*TT0I] , rA[5+TT0I],rB[4+TT1J],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[4+TT1J],rC[6+4*TT0I] , rA[7+TT0I],rB[4+TT1J],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[5+TT1J],rC[4+5*TT0I] , rA[5+TT0I],rB[5+TT1J],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[5+TT1J],rC[6+5*TT0I] , rA[7+TT0I],rB[5+TT1J],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[6+TT1J],rC[4+6*TT0I] , rA[5+TT0I],rB[6+TT1J],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[6+TT1J],rC[6+6*TT0I] , rA[7+TT0I],rB[6+TT1J],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[7+TT1J],rC[4+7*TT0I] , rA[5+TT0I],rB[7+TT1J],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[7+TT1J],rC[6+7*TT0I] , rA[7+TT0I],rB[7+TT1J],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT128x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_1_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_1_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8


    /* iter 15 */

    MAC_8x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_1_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 1);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_1_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else if (s0I == 1) {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else if (s0I == 2) {
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else {
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 1) {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 2) {
        rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef MAC_8x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 2

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU2_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 3072

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_1_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_1_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_1_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 1);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_1_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 1) {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 2) {
        rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 3584
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x8


    /* iter 7 (last unrolled loop) */

    MAC_4x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 1) {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 2) {
        rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 7168
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_1_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_1_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_1_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 1);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_1_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 1) {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 2) {
        rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 3072

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_1_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_1_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 15 */

    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_1_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 1);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_1_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 1) {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 2) {
        rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I] , rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I] , rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I] , rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I] , rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I] , rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I] , rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I] , rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I] , rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I] , rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I] , rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I] , rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I] , rA[7],rB[7],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT128x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 768

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I] , rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I] , rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I] , rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I] , rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I] , rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I] , rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I] , rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I] , rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I] , rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I] , rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I] , rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I] , rA[7],rB[7],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT128x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_8x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1792
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I] , rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I] , rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I] , rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I] , rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I] , rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I] , rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I] , rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I] , rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I] , rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I] , rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I] , rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I] , rA[7],rB[7],rC[7+7*TT0I]); \

#define MAC_8x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I] , rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I] , rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I] , rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I] , rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[4+TT1J],rC[4+4*TT0I] , rA[5+TT0I],rB[4+TT1J],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[4+TT1J],rC[6+4*TT0I] , rA[7+TT0I],rB[4+TT1J],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[5+TT1J],rC[4+5*TT0I] , rA[5+TT0I],rB[5+TT1J],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[5+TT1J],rC[6+5*TT0I] , rA[7+TT0I],rB[5+TT1J],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[6+TT1J],rC[4+6*TT0I] , rA[5+TT0I],rB[6+TT1J],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[6+TT1J],rC[6+6*TT0I] , rA[7+TT0I],rB[6+TT1J],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[7+TT1J],rC[4+7*TT0I] , rA[5+TT0I],rB[7+TT1J],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[7+TT1J],rC[6+7*TT0I] , rA[7+TT0I],rB[7+TT1J],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT128x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8


    /* iter 3 */

    MAC_8x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef MAC_8x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 768

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 3 */

    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 3 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I] , rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I] , rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I] , rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I] , rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I] , rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I] , rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I] , rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I] , rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I] , rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I] , rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I] , rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I] , rA[7],rB[7],rC[7+7*TT0I]); \

#define MAC_8x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I] , rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I] , rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I] , rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I] , rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[4+TT1J],rC[4+4*TT0I] , rA[5+TT0I],rB[4+TT1J],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[4+TT1J],rC[6+4*TT0I] , rA[7+TT0I],rB[4+TT1J],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[5+TT1J],rC[4+5*TT0I] , rA[5+TT0I],rB[5+TT1J],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[5+TT1J],rC[6+5*TT0I] , rA[7+TT0I],rB[5+TT1J],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[6+TT1J],rC[4+6*TT0I] , rA[5+TT0I],rB[6+TT1J],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[6+TT1J],rC[6+6*TT0I] , rA[7+TT0I],rB[6+TT1J],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[7+TT1J],rC[4+7*TT0I] , rA[5+TT0I],rB[7+TT1J],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[7+TT1J],rC[6+7*TT0I] , rA[7+TT0I],rB[7+TT1J],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT128x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8


    /* iter 3 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_8x8


    /* iter 3 (last unrolled loop) */

    MAC_8x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef MAC_8x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1792
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 3 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x8


    /* iter 3 (last unrolled loop) */

    MAC_4x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 3 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 3 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I] , rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I] , rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I] , rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I] , rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I] , rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I] , rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I] , rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I] , rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I] , rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I] , rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I] , rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I] , rA[7],rB[7],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT128x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_8x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I] , rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I] , rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I] , rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I] , rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I] , rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I] , rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I] , rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I] , rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I] , rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I] , rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I] , rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I] , rA[7],rB[7],rC[7+7*TT0I]); \

#define MAC_8x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I] , rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I] , rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I] , rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I] , rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[4+TT1J],rC[4+4*TT0I] , rA[5+TT0I],rB[4+TT1J],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[4+TT1J],rC[6+4*TT0I] , rA[7+TT0I],rB[4+TT1J],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[5+TT1J],rC[4+5*TT0I] , rA[5+TT0I],rB[5+TT1J],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[5+TT1J],rC[6+5*TT0I] , rA[7+TT0I],rB[5+TT1J],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[6+TT1J],rC[4+6*TT0I] , rA[5+TT0I],rB[6+TT1J],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[6+TT1J],rC[6+6*TT0I] , rA[7+TT0I],rB[6+TT1J],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[7+TT1J],rC[4+7*TT0I] , rA[5+TT0I],rB[7+TT1J],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[7+TT1J],rC[6+7*TT0I] , rA[7+TT0I],rB[7+TT1J],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT128x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8


    /* iter 7 */

    MAC_8x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef MAC_8x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 7 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 8

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I] , rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I] , rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I] , rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I] , rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I] , rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I] , rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I] , rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I] , rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I] , rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I] , rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I] , rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I] , rA[7],rB[7],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT128x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;
  unsigned int globalReadOffsetBL_4_0 = globalReadOffsetBL + 0 + 4*LSPB;
  unsigned int globalReadOffsetBL_5_0 = globalReadOffsetBL + 0 + 5*LSPB;
  unsigned int globalReadOffsetBL_6_0 = globalReadOffsetBL + 0 + 6*LSPB;
  unsigned int globalReadOffsetBL_7_0 = globalReadOffsetBL + 0 + 7*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_7_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 4*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 5*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 6*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 7*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 8

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 3072

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;
  unsigned int globalReadOffsetBL_4_0 = globalReadOffsetBL + 0 + 4*LSPB;
  unsigned int globalReadOffsetBL_5_0 = globalReadOffsetBL + 0 + 5*LSPB;
  unsigned int globalReadOffsetBL_6_0 = globalReadOffsetBL + 0 + 6*LSPB;
  unsigned int globalReadOffsetBL_7_0 = globalReadOffsetBL + 0 + 7*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_7_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 4*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 5*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 6*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 7*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 15 */

    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 8

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 7168
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;
  unsigned int globalReadOffsetBL_4_0 = globalReadOffsetBL + 0 + 4*LSPB;
  unsigned int globalReadOffsetBL_5_0 = globalReadOffsetBL + 0 + 5*LSPB;
  unsigned int globalReadOffsetBL_6_0 = globalReadOffsetBL + 0 + 6*LSPB;
  unsigned int globalReadOffsetBL_7_0 = globalReadOffsetBL + 0 + 7*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_7_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 4*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 5*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 6*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 7*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_4_0 = (localWriteOffsetB_0_0_4_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_5_0 = (localWriteOffsetB_0_0_5_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_6_0 = (localWriteOffsetB_0_0_6_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_7_0 = (localWriteOffsetB_0_0_7_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
    localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
    localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
    localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
    localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_4_0 = (localWriteOffsetB_0_0_4_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_5_0 = (localWriteOffsetB_0_0_5_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_6_0 = (localWriteOffsetB_0_0_6_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_7_0 = (localWriteOffsetB_0_0_7_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
    localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
    localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
    localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
    localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x8


    /* iter 15 (last unrolled loop) */

    MAC_4x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_4_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_5_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_6_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_7_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 2

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU2_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 2

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU2_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1792
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1792
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 3584
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1536

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 1) {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 2) {
        rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1536

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 7 */

    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 1) {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 2) {
        rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 3584
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x8


    /* iter 7 (last unrolled loop) */

    MAC_4x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 1) {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 2) {
        rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 3072

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_1_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_1_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_1_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 1);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_1_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 1) {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 2) {
        rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 7168
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_1_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_1_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_1_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 1);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_1_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 1) {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 2) {
        rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1536

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 3584
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1536

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 7 */

    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 3584
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1536

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1536

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 7 */

    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 8

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 3072

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;
  unsigned int globalReadOffsetBL_4_0 = globalReadOffsetBL + 0 + 4*LSPB;
  unsigned int globalReadOffsetBL_5_0 = globalReadOffsetBL + 0 + 5*LSPB;
  unsigned int globalReadOffsetBL_6_0 = globalReadOffsetBL + 0 + 6*LSPB;
  unsigned int globalReadOffsetBL_7_0 = globalReadOffsetBL + 0 + 7*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_7_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 4*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 5*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 6*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 7*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1536

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE double
#define DEST_DATA_TYPE double
#define COMPUTE_DATA_TYPE double
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_DB_MT32x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW2_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  double *D,
  double const * __restrict__ C,
  double const * __restrict__ A,
  double const * __restrict__ B,
  double const alpha,
  double const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (double)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE double
#define DEST_DATA_TYPE double
#define COMPUTE_DATA_TYPE double
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_DB_MT32x32x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW2_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  double *D,
  double const * __restrict__ C,
  double const * __restrict__ A,
  double const * __restrict__ B,
  double const alpha,
  double const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (double)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 768

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 768

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 3 */

    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 3 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1536

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1536

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 7 */

    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 7168
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_1_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_1_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_2_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_3_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_2_1;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_3_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_1);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_1 + 0) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_1 + 0) = b_0_1_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_1 = (localWriteOffsetB_0_0_2_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_1 = (localWriteOffsetB_0_0_3_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_1);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
    localWriteB_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_1);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_1 + 0) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_1 + 0) = b_0_1_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_1 = (localWriteOffsetB_0_0_2_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_1 = (localWriteOffsetB_0_0_3_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_1);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
    localWriteB_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_1_2_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 1);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_1_3_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_1);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_2_1 + 0) = b_0_1_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_3_1 + 0) = b_0_1_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I] , rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I] , rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I] , rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I] , rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I] , rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I] , rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I] , rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I] , rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I] , rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I] , rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I] , rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I] , rA[7],rB[7],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT128x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 768

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I] , rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I] , rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I] , rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I] , rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I] , rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I] , rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I] , rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I] , rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I] , rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I] , rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I] , rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I] , rA[7],rB[7],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT128x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_8x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1792
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 768

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 3 */

    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 3 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 3 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 3 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1536

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1536

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 7 */

    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 7 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 8

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 7168
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;
  unsigned int flattenedOffsetB_4_0 = globalReadOffsetB1J + 0 + 4*LSPB;
  flattenedOffsetB_4_0 = (flattenedOffsetB_4_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_4_0;
  unsigned int globalReadOffsetB1J_4_0 = flattenedOffsetB_4_0;
  unsigned int flattenedOffsetB_5_0 = globalReadOffsetB1J + 0 + 5*LSPB;
  flattenedOffsetB_5_0 = (flattenedOffsetB_5_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_5_0;
  unsigned int globalReadOffsetB1J_5_0 = flattenedOffsetB_5_0;
  unsigned int flattenedOffsetB_6_0 = globalReadOffsetB1J + 0 + 6*LSPB;
  flattenedOffsetB_6_0 = (flattenedOffsetB_6_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_6_0;
  unsigned int globalReadOffsetB1J_6_0 = flattenedOffsetB_6_0;
  unsigned int flattenedOffsetB_7_0 = globalReadOffsetB1J + 0 + 7*LSPB;
  flattenedOffsetB_7_0 = (flattenedOffsetB_7_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_7_0;
  unsigned int globalReadOffsetB1J_7_0 = flattenedOffsetB_7_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_7_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 4*LSPB);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 5*LSPB);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 6*LSPB);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 7*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_4_0 = (localWriteOffsetB_0_0_4_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_5_0 = (localWriteOffsetB_0_0_5_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_6_0 = (localWriteOffsetB_0_0_6_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_7_0 = (localWriteOffsetB_0_0_7_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
    localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
    localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
    localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
    localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_4_0 = (localWriteOffsetB_0_0_4_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_5_0 = (localWriteOffsetB_0_0_5_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_6_0 = (localWriteOffsetB_0_0_6_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_7_0 = (localWriteOffsetB_0_0_7_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
    localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
    localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
    localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
    localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_4_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_5_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_6_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_7_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1792
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1792
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 3 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x8


    /* iter 3 (last unrolled loop) */

    MAC_4x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 3 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 3 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I] , rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I] , rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I] , rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I] , rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I] , rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I] , rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I] , rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I] , rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I] , rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I] , rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I] , rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I] , rA[7],rB[7],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT128x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_8x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 7 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 7168
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_1_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_1_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_2_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_3_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_2_1;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_3_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_1);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_1 + 0) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_1 + 0) = b_0_1_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_1 = (localWriteOffsetB_0_0_2_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_1 = (localWriteOffsetB_0_0_3_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_1);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
    localWriteB_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_1 + 0) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_1 + 0) = b_0_1_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_1 = (localWriteOffsetB_0_0_2_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_1 = (localWriteOffsetB_0_0_3_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_1);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
    localWriteB_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x8


    /* iter 15 (last unrolled loop) */

    MAC_4x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_1_2_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 1);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_1_3_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_1);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_2_1 + 0) = b_0_1_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_3_1 + 0) = b_0_1_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 2

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU2_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 2

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU2_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 2

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU2_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 2

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU2_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 7 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 7 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I] , rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I] , rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I] , rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I] , rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I] , rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I] , rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I] , rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I] , rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I] , rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I] , rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I] , rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I] , rA[7],rB[7],rC[7+7*TT0I]); \

#define MAC_8x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I] , rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I] , rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I] , rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I] , rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[4+TT1J],rC[4+4*TT0I] , rA[5+TT0I],rB[4+TT1J],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[4+TT1J],rC[6+4*TT0I] , rA[7+TT0I],rB[4+TT1J],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[5+TT1J],rC[4+5*TT0I] , rA[5+TT0I],rB[5+TT1J],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[5+TT1J],rC[6+5*TT0I] , rA[7+TT0I],rB[5+TT1J],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[6+TT1J],rC[4+6*TT0I] , rA[5+TT0I],rB[6+TT1J],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[6+TT1J],rC[6+6*TT0I] , rA[7+TT0I],rB[6+TT1J],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[7+TT1J],rC[4+7*TT0I] , rA[5+TT0I],rB[7+TT1J],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[7+TT1J],rC[6+7*TT0I] , rA[7+TT0I],rB[7+TT1J],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT128x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8


    /* iter 3 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_8x8


    /* iter 3 (last unrolled loop) */

    MAC_8x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef MAC_8x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1792
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 3 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x8


    /* iter 3 (last unrolled loop) */

    MAC_4x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I] , rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I] , rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I] , rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I] , rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I] , rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I] , rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I] , rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I] , rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I] , rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I] , rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I] , rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I] , rA[7],rB[7],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT128x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_8x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I] , rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I] , rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I] , rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I] , rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I] , rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I] , rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I] , rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I] , rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I] , rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I] , rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I] , rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I] , rA[7],rB[7],rC[7+7*TT0I]); \

#define MAC_8x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I] , rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I] , rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I] , rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I] , rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[4+TT1J],rC[4+4*TT0I] , rA[5+TT0I],rB[4+TT1J],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[4+TT1J],rC[6+4*TT0I] , rA[7+TT0I],rB[4+TT1J],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[5+TT1J],rC[4+5*TT0I] , rA[5+TT0I],rB[5+TT1J],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[5+TT1J],rC[6+5*TT0I] , rA[7+TT0I],rB[5+TT1J],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[6+TT1J],rC[4+6*TT0I] , rA[5+TT0I],rB[6+TT1J],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[6+TT1J],rC[6+6*TT0I] , rA[7+TT0I],rB[6+TT1J],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[7+TT1J],rC[4+7*TT0I] , rA[5+TT0I],rB[7+TT1J],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[7+TT1J],rC[6+7*TT0I] , rA[7+TT0I],rB[7+TT1J],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT128x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_8x8


    /* iter 7 (last unrolled loop) */

    MAC_8x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef MAC_8x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 3584
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x8


    /* iter 7 (last unrolled loop) */

    MAC_4x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 8

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I] , rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I] , rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I] , rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I] , rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I] , rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I] , rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I] , rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I] , rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I] , rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I] , rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I] , rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I] , rA[7],rB[7],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT128x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;
  unsigned int flattenedOffsetA_4_0 = globalReadOffsetA0I + 0 + 4*LSPA;
  flattenedOffsetA_4_0 = (flattenedOffsetA_4_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_4_0;
  unsigned int globalReadOffsetA0I_4_0 = flattenedOffsetA_4_0;
  unsigned int flattenedOffsetA_5_0 = globalReadOffsetA0I + 0 + 5*LSPA;
  flattenedOffsetA_5_0 = (flattenedOffsetA_5_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_5_0;
  unsigned int globalReadOffsetA0I_5_0 = flattenedOffsetA_5_0;
  unsigned int flattenedOffsetA_6_0 = globalReadOffsetA0I + 0 + 6*LSPA;
  flattenedOffsetA_6_0 = (flattenedOffsetA_6_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_6_0;
  unsigned int globalReadOffsetA0I_6_0 = flattenedOffsetA_6_0;
  unsigned int flattenedOffsetA_7_0 = globalReadOffsetA0I + 0 + 7*LSPA;
  flattenedOffsetA_7_0 = (flattenedOffsetA_7_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_7_0;
  unsigned int globalReadOffsetA0I_7_0 = flattenedOffsetA_7_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;
  unsigned int flattenedOffsetB_4_0 = globalReadOffsetB1J + 0 + 4*LSPB;
  flattenedOffsetB_4_0 = (flattenedOffsetB_4_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_4_0;
  unsigned int globalReadOffsetB1J_4_0 = flattenedOffsetB_4_0;
  unsigned int flattenedOffsetB_5_0 = globalReadOffsetB1J + 0 + 5*LSPB;
  flattenedOffsetB_5_0 = (flattenedOffsetB_5_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_5_0;
  unsigned int globalReadOffsetB1J_5_0 = flattenedOffsetB_5_0;
  unsigned int flattenedOffsetB_6_0 = globalReadOffsetB1J + 0 + 6*LSPB;
  flattenedOffsetB_6_0 = (flattenedOffsetB_6_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_6_0;
  unsigned int globalReadOffsetB1J_6_0 = flattenedOffsetB_6_0;
  unsigned int flattenedOffsetB_7_0 = globalReadOffsetB1J + 0 + 7*LSPB;
  flattenedOffsetB_7_0 = (flattenedOffsetB_7_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_7_0;
  unsigned int globalReadOffsetB1J_7_0 = flattenedOffsetB_7_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_7_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 4*LSPA);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 5*LSPA);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 6*LSPA);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 7*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 4*LSPB);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 5*LSPB);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 6*LSPB);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 7*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 8

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 8192
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I] , rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I] , rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I] , rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I] , rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I] , rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I] , rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I] , rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I] , rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I] , rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I] , rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I] , rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I] , rA[7],rB[7],rC[7+7*TT0I]); \

#define MAC_8x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I] , rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I] , rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I] , rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I] , rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[4+TT1J],rC[4+4*TT0I] , rA[5+TT0I],rB[4+TT1J],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[4+TT1J],rC[6+4*TT0I] , rA[7+TT0I],rB[4+TT1J],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[5+TT1J],rC[4+5*TT0I] , rA[5+TT0I],rB[5+TT1J],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[5+TT1J],rC[6+5*TT0I] , rA[7+TT0I],rB[5+TT1J],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[6+TT1J],rC[4+6*TT0I] , rA[5+TT0I],rB[6+TT1J],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[6+TT1J],rC[6+6*TT0I] , rA[7+TT0I],rB[6+TT1J],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[7+TT1J],rC[4+7*TT0I] , rA[5+TT0I],rB[7+TT1J],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[7+TT1J],rC[6+7*TT0I] , rA[7+TT0I],rB[7+TT1J],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT128x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;
  unsigned int flattenedOffsetA_4_0 = globalReadOffsetA0I + 0 + 4*LSPA;
  flattenedOffsetA_4_0 = (flattenedOffsetA_4_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_4_0;
  unsigned int globalReadOffsetA0I_4_0 = flattenedOffsetA_4_0;
  unsigned int flattenedOffsetA_5_0 = globalReadOffsetA0I + 0 + 5*LSPA;
  flattenedOffsetA_5_0 = (flattenedOffsetA_5_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_5_0;
  unsigned int globalReadOffsetA0I_5_0 = flattenedOffsetA_5_0;
  unsigned int flattenedOffsetA_6_0 = globalReadOffsetA0I + 0 + 6*LSPA;
  flattenedOffsetA_6_0 = (flattenedOffsetA_6_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_6_0;
  unsigned int globalReadOffsetA0I_6_0 = flattenedOffsetA_6_0;
  unsigned int flattenedOffsetA_7_0 = globalReadOffsetA0I + 0 + 7*LSPA;
  flattenedOffsetA_7_0 = (flattenedOffsetA_7_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_7_0;
  unsigned int globalReadOffsetA0I_7_0 = flattenedOffsetA_7_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;
  unsigned int flattenedOffsetB_4_0 = globalReadOffsetB1J + 0 + 4*LSPB;
  flattenedOffsetB_4_0 = (flattenedOffsetB_4_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_4_0;
  unsigned int globalReadOffsetB1J_4_0 = flattenedOffsetB_4_0;
  unsigned int flattenedOffsetB_5_0 = globalReadOffsetB1J + 0 + 5*LSPB;
  flattenedOffsetB_5_0 = (flattenedOffsetB_5_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_5_0;
  unsigned int globalReadOffsetB1J_5_0 = flattenedOffsetB_5_0;
  unsigned int flattenedOffsetB_6_0 = globalReadOffsetB1J + 0 + 6*LSPB;
  flattenedOffsetB_6_0 = (flattenedOffsetB_6_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_6_0;
  unsigned int globalReadOffsetB1J_6_0 = flattenedOffsetB_6_0;
  unsigned int flattenedOffsetB_7_0 = globalReadOffsetB1J + 0 + 7*LSPB;
  flattenedOffsetB_7_0 = (flattenedOffsetB_7_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_7_0;
  unsigned int globalReadOffsetB1J_7_0 = flattenedOffsetB_7_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_7_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 4*LSPA);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 5*LSPA);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 6*LSPA);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 7*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 4*LSPB);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 5*LSPB);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 6*LSPB);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 7*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_4_0 = (localWriteOffsetA_0_0_4_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_5_0 = (localWriteOffsetA_0_0_5_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_6_0 = (localWriteOffsetA_0_0_6_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_7_0 = (localWriteOffsetA_0_0_7_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_4_0 = (localWriteOffsetB_0_0_4_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_5_0 = (localWriteOffsetB_0_0_5_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_6_0 = (localWriteOffsetB_0_0_6_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_7_0 = (localWriteOffsetB_0_0_7_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
    localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
    localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
    localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
    localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
    localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
    localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
    localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
    localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_4_0 = (localWriteOffsetA_0_0_4_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_5_0 = (localWriteOffsetA_0_0_5_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_6_0 = (localWriteOffsetA_0_0_6_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_7_0 = (localWriteOffsetA_0_0_7_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_4_0 = (localWriteOffsetB_0_0_4_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_5_0 = (localWriteOffsetB_0_0_5_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_6_0 = (localWriteOffsetB_0_0_6_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_7_0 = (localWriteOffsetB_0_0_7_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
    localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
    localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
    localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
    localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
    localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
    localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
    localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
    localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_8x8


    /* iter 15 (last unrolled loop) */

    MAC_8x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_4_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_5_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_6_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_7_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_4_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_5_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_6_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_7_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef MAC_8x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 2

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU2_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 2

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU2_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 7 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 7 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 768

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 1) {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 2) {
        rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 768

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 3 */

    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 1) {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 2) {
        rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 3 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1536

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 1) {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 2) {
        rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 3584
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 1) {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 2) {
        rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1536

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 7 */

    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 1) {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 2) {
        rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 7 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 7168
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_1_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_1_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_1_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 1);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_1_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 1) {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 2) {
        rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I] , rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I] , rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I] , rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I] , rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I] , rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I] , rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I] , rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I] , rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I] , rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I] , rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I] , rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I] , rA[7],rB[7],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT128x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 768

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1792
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I] , rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I] , rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I] , rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I] , rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I] , rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I] , rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I] , rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I] , rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I] , rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I] , rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I] , rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I] , rA[7],rB[7],rC[7+7*TT0I]); \

#define MAC_8x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I] , rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I] , rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I] , rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I] , rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[4+TT1J],rC[4+4*TT0I] , rA[5+TT0I],rB[4+TT1J],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[4+TT1J],rC[6+4*TT0I] , rA[7+TT0I],rB[4+TT1J],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[5+TT1J],rC[4+5*TT0I] , rA[5+TT0I],rB[5+TT1J],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[5+TT1J],rC[6+5*TT0I] , rA[7+TT0I],rB[5+TT1J],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[6+TT1J],rC[4+6*TT0I] , rA[5+TT0I],rB[6+TT1J],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[6+TT1J],rC[6+6*TT0I] , rA[7+TT0I],rB[6+TT1J],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[7+TT1J],rC[4+7*TT0I] , rA[5+TT0I],rB[7+TT1J],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[7+TT1J],rC[6+7*TT0I] , rA[7+TT0I],rB[7+TT1J],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT128x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8


    /* iter 3 */

    MAC_8x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef MAC_8x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 768

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 3 */

    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 3 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1792
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 3 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x8


    /* iter 3 (last unrolled loop) */

    MAC_4x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 3 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 3 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I] , rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I] , rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I] , rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I] , rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I] , rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I] , rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I] , rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I] , rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I] , rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I] , rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I] , rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I] , rA[7],rB[7],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT128x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1536

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1536

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 7 */

    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 7 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 8

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I] , rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I] , rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I] , rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I] , rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I] , rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I] , rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I] , rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I] , rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I] , rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I] , rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I] , rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I] , rA[7],rB[7],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT128x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;
  unsigned int flattenedOffsetA_4_0 = globalReadOffsetA0I + 0 + 4*LSPA;
  flattenedOffsetA_4_0 = (flattenedOffsetA_4_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_4_0;
  unsigned int globalReadOffsetA0I_4_0 = flattenedOffsetA_4_0;
  unsigned int flattenedOffsetA_5_0 = globalReadOffsetA0I + 0 + 5*LSPA;
  flattenedOffsetA_5_0 = (flattenedOffsetA_5_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_5_0;
  unsigned int globalReadOffsetA0I_5_0 = flattenedOffsetA_5_0;
  unsigned int flattenedOffsetA_6_0 = globalReadOffsetA0I + 0 + 6*LSPA;
  flattenedOffsetA_6_0 = (flattenedOffsetA_6_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_6_0;
  unsigned int globalReadOffsetA0I_6_0 = flattenedOffsetA_6_0;
  unsigned int flattenedOffsetA_7_0 = globalReadOffsetA0I + 0 + 7*LSPA;
  flattenedOffsetA_7_0 = (flattenedOffsetA_7_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_7_0;
  unsigned int globalReadOffsetA0I_7_0 = flattenedOffsetA_7_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;
  unsigned int globalReadOffsetBL_4_0 = globalReadOffsetBL + 0 + 4*LSPB;
  unsigned int globalReadOffsetBL_5_0 = globalReadOffsetBL + 0 + 5*LSPB;
  unsigned int globalReadOffsetBL_6_0 = globalReadOffsetBL + 0 + 6*LSPB;
  unsigned int globalReadOffsetBL_7_0 = globalReadOffsetBL + 0 + 7*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_7_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 4*LSPA);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 5*LSPA);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 6*LSPA);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 7*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 4*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 5*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 6*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 7*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 8

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 3072

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;
  unsigned int globalReadOffsetBL_4_0 = globalReadOffsetBL + 0 + 4*LSPB;
  unsigned int globalReadOffsetBL_5_0 = globalReadOffsetBL + 0 + 5*LSPB;
  unsigned int globalReadOffsetBL_6_0 = globalReadOffsetBL + 0 + 6*LSPB;
  unsigned int globalReadOffsetBL_7_0 = globalReadOffsetBL + 0 + 7*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_7_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 4*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 5*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 6*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 7*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 15 */

    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 3584
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1792
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 3584
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 7 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1792
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE TensileComplexDouble
#define DEST_DATA_TYPE TensileComplexDouble
#define COMPUTE_DATA_TYPE TensileComplexDouble
#define s0 x
#define s1 y
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) \
  DST.s0 = MAC(  MULA.s0, MULB.s0, DST.s0 ); \
  DST.s0 = MAC(  MULA.s1, MULB.s1, DST.s0 ); \
  DST.s1 = MAC(  MULA.s0, MULB.s1, DST.s1 ); \
  DST.s1 = MAC( -MULA.s1, MULB.s0, DST.s1 );
#define TYPE_MAC_WRITE( DST, SRC, ALPHA, REG, BETA ) \
  /* (1) */ \
  type_mac_tmp = REG.s0; \
  REG.s0 *= ALPHA.s0; \
  REG.s0 = MAC( -ALPHA.s1, REG.s1, REG.s0 ); \
  REG.s1 *= ALPHA.s0; \
  REG.s1 = MAC(  ALPHA.s1, type_mac_tmp, REG.s1 ); \
  /* (2) */ \
  if(BETA.s0 != 0) { \
  REG.s0 = MAC(  BETA.s0, SRC.s0, REG.s0 ); \
  REG.s1 = MAC(  BETA.s0, SRC.s1, REG.s1 ); \
  } \
  if (BETA.s1 != 0) { \
  REG.s0 = MAC( -BETA.s1, SRC.s1, REG.s0 ); \
  REG.s1 = MAC(  BETA.s1, SRC.s0, REG.s1 ); \
  } \
  /* (3) */ \
  DST = REG;

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_AlikC_Bljk_ZB_MT32x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW1_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  TensileComplexDouble *D,
  TensileComplexDouble const * __restrict__ C,
  TensileComplexDouble const * __restrict__ A,
  TensileComplexDouble const * __restrict__ B,
  TensileComplexDouble const alpha,
  TensileComplexDouble const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (TensileComplexDouble)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == tensile_complex<double>(0.0)) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 1); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }



  double type_mac_tmp;


  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE TensileComplexDouble
#define DEST_DATA_TYPE TensileComplexDouble
#define COMPUTE_DATA_TYPE TensileComplexDouble
#define s0 x
#define s1 y
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) \
  DST.s0 = MAC(  MULA.s0, MULB.s0, DST.s0 ); \
  DST.s0 = MAC(  MULA.s1, MULB.s1, DST.s0 ); \
  DST.s1 = MAC(  MULA.s0, MULB.s1, DST.s1 ); \
  DST.s1 = MAC( -MULA.s1, MULB.s0, DST.s1 );
#define TYPE_MAC_WRITE( DST, SRC, ALPHA, REG, BETA ) \
  /* (1) */ \
  type_mac_tmp = REG.s0; \
  REG.s0 *= ALPHA.s0; \
  REG.s0 = MAC( -ALPHA.s1, REG.s1, REG.s0 ); \
  REG.s1 *= ALPHA.s0; \
  REG.s1 = MAC(  ALPHA.s1, type_mac_tmp, REG.s1 ); \
  /* (2) */ \
  if(BETA.s0 != 0) { \
  REG.s0 = MAC(  BETA.s0, SRC.s0, REG.s0 ); \
  REG.s1 = MAC(  BETA.s0, SRC.s1, REG.s1 ); \
  } \
  if (BETA.s1 != 0) { \
  REG.s0 = MAC( -BETA.s1, SRC.s1, REG.s0 ); \
  REG.s1 = MAC(  BETA.s1, SRC.s0, REG.s1 ); \
  } \
  /* (3) */ \
  DST = REG;

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_AlikC_Bljk_ZB_MT32x32x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW1_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  TensileComplexDouble *D,
  TensileComplexDouble const * __restrict__ C,
  TensileComplexDouble const * __restrict__ A,
  TensileComplexDouble const * __restrict__ B,
  TensileComplexDouble const alpha,
  TensileComplexDouble const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (TensileComplexDouble)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == tensile_complex<double>(0.0)) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 1); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }



  double type_mac_tmp;


  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 768

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 7 */

    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 768

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 7 */

    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT128x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT128x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 16

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 2x8 micro-tile */
#define MAC_2x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \

#define MAC_2x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT16x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_8_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;
  DATA_TYPE b_0_0_8_0;
  DATA_TYPE b_0_0_9_0;
  DATA_TYPE b_0_0_10_0;
  DATA_TYPE b_0_0_11_0;
  DATA_TYPE b_0_0_12_0;
  DATA_TYPE b_0_0_13_0;
  DATA_TYPE b_0_0_14_0;
  DATA_TYPE b_0_0_15_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;
  unsigned int globalReadOffsetBL_4_0 = globalReadOffsetBL + 0 + 4*LSPB;
  unsigned int globalReadOffsetBL_5_0 = globalReadOffsetBL + 0 + 5*LSPB;
  unsigned int globalReadOffsetBL_6_0 = globalReadOffsetBL + 0 + 6*LSPB;
  unsigned int globalReadOffsetBL_7_0 = globalReadOffsetBL + 0 + 7*LSPB;
  unsigned int globalReadOffsetBL_8_0 = globalReadOffsetBL + 0 + 8*LSPB;
  unsigned int globalReadOffsetBL_9_0 = globalReadOffsetBL + 0 + 9*LSPB;
  unsigned int globalReadOffsetBL_10_0 = globalReadOffsetBL + 0 + 10*LSPB;
  unsigned int globalReadOffsetBL_11_0 = globalReadOffsetBL + 0 + 11*LSPB;
  unsigned int globalReadOffsetBL_12_0 = globalReadOffsetBL + 0 + 12*LSPB;
  unsigned int globalReadOffsetBL_13_0 = globalReadOffsetBL + 0 + 13*LSPB;
  unsigned int globalReadOffsetBL_14_0 = globalReadOffsetBL + 0 + 14*LSPB;
  unsigned int globalReadOffsetBL_15_0 = globalReadOffsetBL + 0 + 15*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_7_0), wgK );
  int64_t globalReadOffsetB_0_0_8_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_8_0), wgK );
  int64_t globalReadOffsetB_0_0_9_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_9_0), wgK );
  int64_t globalReadOffsetB_0_0_10_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_10_0), wgK );
  int64_t globalReadOffsetB_0_0_11_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_11_0), wgK );
  int64_t globalReadOffsetB_0_0_12_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_12_0), wgK );
  int64_t globalReadOffsetB_0_0_13_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_13_0), wgK );
  int64_t globalReadOffsetB_0_0_14_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_14_0), wgK );
  int64_t globalReadOffsetB_0_0_15_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_15_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;
  DATA_TYPE const *globalReadB_0_0_8_0 = B + globalReadOffsetB_0_0_8_0;
  DATA_TYPE const *globalReadB_0_0_9_0 = B + globalReadOffsetB_0_0_9_0;
  DATA_TYPE const *globalReadB_0_0_10_0 = B + globalReadOffsetB_0_0_10_0;
  DATA_TYPE const *globalReadB_0_0_11_0 = B + globalReadOffsetB_0_0_11_0;
  DATA_TYPE const *globalReadB_0_0_12_0 = B + globalReadOffsetB_0_0_12_0;
  DATA_TYPE const *globalReadB_0_0_13_0 = B + globalReadOffsetB_0_0_13_0;
  DATA_TYPE const *globalReadB_0_0_14_0 = B + globalReadOffsetB_0_0_14_0;
  DATA_TYPE const *globalReadB_0_0_15_0 = B + globalReadOffsetB_0_0_15_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 4*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 5*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 6*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 7*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_8_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 8*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_9_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 9*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_10_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 10*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_11_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 11*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_12_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 12*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_13_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 13*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_14_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 14*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_15_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 15*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;
  DATA_TYPE *localWriteB_0_0_8_0;
  DATA_TYPE *localWriteB_0_0_9_0;
  DATA_TYPE *localWriteB_0_0_10_0;
  DATA_TYPE *localWriteB_0_0_11_0;
  DATA_TYPE *localWriteB_0_0_12_0;
  DATA_TYPE *localWriteB_0_0_13_0;
  DATA_TYPE *localWriteB_0_0_14_0;
  DATA_TYPE *localWriteB_0_0_15_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);
  localWriteB_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_8_0);
  localWriteB_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_9_0);
  localWriteB_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_10_0);
  localWriteB_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_11_0);
  localWriteB_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_12_0);
  localWriteB_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_13_0);
  localWriteB_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_14_0);
  localWriteB_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_15_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_8_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_9_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_10_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_11_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_12_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_13_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_14_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_15_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);
    b_0_0_8_0 = *(globalReadB_0_0_8_0 + 0);
    b_0_0_9_0 = *(globalReadB_0_0_9_0 + 0);
    b_0_0_10_0 = *(globalReadB_0_0_10_0 + 0);
    b_0_0_11_0 = *(globalReadB_0_0_11_0 + 0);
    b_0_0_12_0 = *(globalReadB_0_0_12_0 + 0);
    b_0_0_13_0 = *(globalReadB_0_0_13_0 + 0);
    b_0_0_14_0 = *(globalReadB_0_0_14_0 + 0);
    b_0_0_15_0 = *(globalReadB_0_0_15_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_8_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_8_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_9_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_9_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_10_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_10_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_11_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_11_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_12_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_12_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_13_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_13_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_14_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_14_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_15_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_15_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
    *(localWriteB_0_0_8_0 + 0) = b_0_0_8_0;
    *(localWriteB_0_0_9_0 + 0) = b_0_0_9_0;
    *(localWriteB_0_0_10_0 + 0) = b_0_0_10_0;
    *(localWriteB_0_0_11_0 + 0) = b_0_0_11_0;
    *(localWriteB_0_0_12_0 + 0) = b_0_0_12_0;
    *(localWriteB_0_0_13_0 + 0) = b_0_0_13_0;
    *(localWriteB_0_0_14_0 + 0) = b_0_0_14_0;
    *(localWriteB_0_0_15_0 + 0) = b_0_0_15_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x8


    /* iter 15 */

    MAC_2x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);
  b_0_0_8_0 = ( globalReadOffsetBL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_8_0 + 0);
  b_0_0_9_0 = ( globalReadOffsetBL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_9_0 + 0);
  b_0_0_10_0 = ( globalReadOffsetBL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_10_0 + 0);
  b_0_0_11_0 = ( globalReadOffsetBL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_11_0 + 0);
  b_0_0_12_0 = ( globalReadOffsetBL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_12_0 + 0);
  b_0_0_13_0 = ( globalReadOffsetBL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_13_0 + 0);
  b_0_0_14_0 = ( globalReadOffsetBL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_14_0 + 0);
  b_0_0_15_0 = ( globalReadOffsetBL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_15_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);
  localWriteB_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_8_0);
  localWriteB_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_9_0);
  localWriteB_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_10_0);
  localWriteB_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_11_0);
  localWriteB_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_12_0);
  localWriteB_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_13_0);
  localWriteB_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_14_0);
  localWriteB_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_15_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
  *(localWriteB_0_0_8_0 + 0) = b_0_0_8_0;
  *(localWriteB_0_0_9_0 + 0) = b_0_0_9_0;
  *(localWriteB_0_0_10_0 + 0) = b_0_0_10_0;
  *(localWriteB_0_0_11_0 + 0) = b_0_0_11_0;
  *(localWriteB_0_0_12_0 + 0) = b_0_0_12_0;
  *(localWriteB_0_0_13_0 + 0) = b_0_0_13_0;
  *(localWriteB_0_0_14_0 + 0) = b_0_0_14_0;
  *(localWriteB_0_0_15_0 + 0) = b_0_0_15_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x8
#undef MAC_2x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 64
#define SG1J 1
#define TT0I 1
#define TT1J 1
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 64
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 60

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 64
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 4096
#define LDS_NUM_ELEMENTS 4160

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 1x1 micro-tile */
#define MAC_1x1\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \

#define MAC_1x1_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x1x64_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU60_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU0_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT1_1_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG64_1_1_WGM1(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE a_0_0_16_0;
  DATA_TYPE a_0_0_17_0;
  DATA_TYPE a_0_0_18_0;
  DATA_TYPE a_0_0_19_0;
  DATA_TYPE a_0_0_20_0;
  DATA_TYPE a_0_0_21_0;
  DATA_TYPE a_0_0_22_0;
  DATA_TYPE a_0_0_23_0;
  DATA_TYPE a_0_0_24_0;
  DATA_TYPE a_0_0_25_0;
  DATA_TYPE a_0_0_26_0;
  DATA_TYPE a_0_0_27_0;
  DATA_TYPE a_0_0_28_0;
  DATA_TYPE a_0_0_29_0;
  DATA_TYPE a_0_0_30_0;
  DATA_TYPE a_0_0_31_0;
  DATA_TYPE a_0_0_32_0;
  DATA_TYPE a_0_0_33_0;
  DATA_TYPE a_0_0_34_0;
  DATA_TYPE a_0_0_35_0;
  DATA_TYPE a_0_0_36_0;
  DATA_TYPE a_0_0_37_0;
  DATA_TYPE a_0_0_38_0;
  DATA_TYPE a_0_0_39_0;
  DATA_TYPE a_0_0_40_0;
  DATA_TYPE a_0_0_41_0;
  DATA_TYPE a_0_0_42_0;
  DATA_TYPE a_0_0_43_0;
  DATA_TYPE a_0_0_44_0;
  DATA_TYPE a_0_0_45_0;
  DATA_TYPE a_0_0_46_0;
  DATA_TYPE a_0_0_47_0;
  DATA_TYPE a_0_0_48_0;
  DATA_TYPE a_0_0_49_0;
  DATA_TYPE a_0_0_50_0;
  DATA_TYPE a_0_0_51_0;
  DATA_TYPE a_0_0_52_0;
  DATA_TYPE a_0_0_53_0;
  DATA_TYPE a_0_0_54_0;
  DATA_TYPE a_0_0_55_0;
  DATA_TYPE a_0_0_56_0;
  DATA_TYPE a_0_0_57_0;
  DATA_TYPE a_0_0_58_0;
  DATA_TYPE a_0_0_59_0;
  DATA_TYPE a_0_0_60_0;
  DATA_TYPE a_0_0_61_0;
  DATA_TYPE a_0_0_62_0;
  DATA_TYPE a_0_0_63_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;
  unsigned int globalReadOffsetAL_8_0 = globalReadOffsetAL + 0 + 8*LSPA;
  unsigned int globalReadOffsetAL_9_0 = globalReadOffsetAL + 0 + 9*LSPA;
  unsigned int globalReadOffsetAL_10_0 = globalReadOffsetAL + 0 + 10*LSPA;
  unsigned int globalReadOffsetAL_11_0 = globalReadOffsetAL + 0 + 11*LSPA;
  unsigned int globalReadOffsetAL_12_0 = globalReadOffsetAL + 0 + 12*LSPA;
  unsigned int globalReadOffsetAL_13_0 = globalReadOffsetAL + 0 + 13*LSPA;
  unsigned int globalReadOffsetAL_14_0 = globalReadOffsetAL + 0 + 14*LSPA;
  unsigned int globalReadOffsetAL_15_0 = globalReadOffsetAL + 0 + 15*LSPA;
  unsigned int globalReadOffsetAL_16_0 = globalReadOffsetAL + 0 + 16*LSPA;
  unsigned int globalReadOffsetAL_17_0 = globalReadOffsetAL + 0 + 17*LSPA;
  unsigned int globalReadOffsetAL_18_0 = globalReadOffsetAL + 0 + 18*LSPA;
  unsigned int globalReadOffsetAL_19_0 = globalReadOffsetAL + 0 + 19*LSPA;
  unsigned int globalReadOffsetAL_20_0 = globalReadOffsetAL + 0 + 20*LSPA;
  unsigned int globalReadOffsetAL_21_0 = globalReadOffsetAL + 0 + 21*LSPA;
  unsigned int globalReadOffsetAL_22_0 = globalReadOffsetAL + 0 + 22*LSPA;
  unsigned int globalReadOffsetAL_23_0 = globalReadOffsetAL + 0 + 23*LSPA;
  unsigned int globalReadOffsetAL_24_0 = globalReadOffsetAL + 0 + 24*LSPA;
  unsigned int globalReadOffsetAL_25_0 = globalReadOffsetAL + 0 + 25*LSPA;
  unsigned int globalReadOffsetAL_26_0 = globalReadOffsetAL + 0 + 26*LSPA;
  unsigned int globalReadOffsetAL_27_0 = globalReadOffsetAL + 0 + 27*LSPA;
  unsigned int globalReadOffsetAL_28_0 = globalReadOffsetAL + 0 + 28*LSPA;
  unsigned int globalReadOffsetAL_29_0 = globalReadOffsetAL + 0 + 29*LSPA;
  unsigned int globalReadOffsetAL_30_0 = globalReadOffsetAL + 0 + 30*LSPA;
  unsigned int globalReadOffsetAL_31_0 = globalReadOffsetAL + 0 + 31*LSPA;
  unsigned int globalReadOffsetAL_32_0 = globalReadOffsetAL + 0 + 32*LSPA;
  unsigned int globalReadOffsetAL_33_0 = globalReadOffsetAL + 0 + 33*LSPA;
  unsigned int globalReadOffsetAL_34_0 = globalReadOffsetAL + 0 + 34*LSPA;
  unsigned int globalReadOffsetAL_35_0 = globalReadOffsetAL + 0 + 35*LSPA;
  unsigned int globalReadOffsetAL_36_0 = globalReadOffsetAL + 0 + 36*LSPA;
  unsigned int globalReadOffsetAL_37_0 = globalReadOffsetAL + 0 + 37*LSPA;
  unsigned int globalReadOffsetAL_38_0 = globalReadOffsetAL + 0 + 38*LSPA;
  unsigned int globalReadOffsetAL_39_0 = globalReadOffsetAL + 0 + 39*LSPA;
  unsigned int globalReadOffsetAL_40_0 = globalReadOffsetAL + 0 + 40*LSPA;
  unsigned int globalReadOffsetAL_41_0 = globalReadOffsetAL + 0 + 41*LSPA;
  unsigned int globalReadOffsetAL_42_0 = globalReadOffsetAL + 0 + 42*LSPA;
  unsigned int globalReadOffsetAL_43_0 = globalReadOffsetAL + 0 + 43*LSPA;
  unsigned int globalReadOffsetAL_44_0 = globalReadOffsetAL + 0 + 44*LSPA;
  unsigned int globalReadOffsetAL_45_0 = globalReadOffsetAL + 0 + 45*LSPA;
  unsigned int globalReadOffsetAL_46_0 = globalReadOffsetAL + 0 + 46*LSPA;
  unsigned int globalReadOffsetAL_47_0 = globalReadOffsetAL + 0 + 47*LSPA;
  unsigned int globalReadOffsetAL_48_0 = globalReadOffsetAL + 0 + 48*LSPA;
  unsigned int globalReadOffsetAL_49_0 = globalReadOffsetAL + 0 + 49*LSPA;
  unsigned int globalReadOffsetAL_50_0 = globalReadOffsetAL + 0 + 50*LSPA;
  unsigned int globalReadOffsetAL_51_0 = globalReadOffsetAL + 0 + 51*LSPA;
  unsigned int globalReadOffsetAL_52_0 = globalReadOffsetAL + 0 + 52*LSPA;
  unsigned int globalReadOffsetAL_53_0 = globalReadOffsetAL + 0 + 53*LSPA;
  unsigned int globalReadOffsetAL_54_0 = globalReadOffsetAL + 0 + 54*LSPA;
  unsigned int globalReadOffsetAL_55_0 = globalReadOffsetAL + 0 + 55*LSPA;
  unsigned int globalReadOffsetAL_56_0 = globalReadOffsetAL + 0 + 56*LSPA;
  unsigned int globalReadOffsetAL_57_0 = globalReadOffsetAL + 0 + 57*LSPA;
  unsigned int globalReadOffsetAL_58_0 = globalReadOffsetAL + 0 + 58*LSPA;
  unsigned int globalReadOffsetAL_59_0 = globalReadOffsetAL + 0 + 59*LSPA;
  unsigned int globalReadOffsetAL_60_0 = globalReadOffsetAL + 0 + 60*LSPA;
  unsigned int globalReadOffsetAL_61_0 = globalReadOffsetAL + 0 + 61*LSPA;
  unsigned int globalReadOffsetAL_62_0 = globalReadOffsetAL + 0 + 62*LSPA;
  unsigned int globalReadOffsetAL_63_0 = globalReadOffsetAL + 0 + 63*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_15_0), wgK );
  int64_t globalReadOffsetA_0_0_16_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_16_0), wgK );
  int64_t globalReadOffsetA_0_0_17_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_17_0), wgK );
  int64_t globalReadOffsetA_0_0_18_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_18_0), wgK );
  int64_t globalReadOffsetA_0_0_19_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_19_0), wgK );
  int64_t globalReadOffsetA_0_0_20_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_20_0), wgK );
  int64_t globalReadOffsetA_0_0_21_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_21_0), wgK );
  int64_t globalReadOffsetA_0_0_22_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_22_0), wgK );
  int64_t globalReadOffsetA_0_0_23_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_23_0), wgK );
  int64_t globalReadOffsetA_0_0_24_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_24_0), wgK );
  int64_t globalReadOffsetA_0_0_25_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_25_0), wgK );
  int64_t globalReadOffsetA_0_0_26_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_26_0), wgK );
  int64_t globalReadOffsetA_0_0_27_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_27_0), wgK );
  int64_t globalReadOffsetA_0_0_28_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_28_0), wgK );
  int64_t globalReadOffsetA_0_0_29_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_29_0), wgK );
  int64_t globalReadOffsetA_0_0_30_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_30_0), wgK );
  int64_t globalReadOffsetA_0_0_31_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_31_0), wgK );
  int64_t globalReadOffsetA_0_0_32_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_32_0), wgK );
  int64_t globalReadOffsetA_0_0_33_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_33_0), wgK );
  int64_t globalReadOffsetA_0_0_34_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_34_0), wgK );
  int64_t globalReadOffsetA_0_0_35_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_35_0), wgK );
  int64_t globalReadOffsetA_0_0_36_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_36_0), wgK );
  int64_t globalReadOffsetA_0_0_37_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_37_0), wgK );
  int64_t globalReadOffsetA_0_0_38_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_38_0), wgK );
  int64_t globalReadOffsetA_0_0_39_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_39_0), wgK );
  int64_t globalReadOffsetA_0_0_40_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_40_0), wgK );
  int64_t globalReadOffsetA_0_0_41_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_41_0), wgK );
  int64_t globalReadOffsetA_0_0_42_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_42_0), wgK );
  int64_t globalReadOffsetA_0_0_43_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_43_0), wgK );
  int64_t globalReadOffsetA_0_0_44_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_44_0), wgK );
  int64_t globalReadOffsetA_0_0_45_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_45_0), wgK );
  int64_t globalReadOffsetA_0_0_46_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_46_0), wgK );
  int64_t globalReadOffsetA_0_0_47_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_47_0), wgK );
  int64_t globalReadOffsetA_0_0_48_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_48_0), wgK );
  int64_t globalReadOffsetA_0_0_49_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_49_0), wgK );
  int64_t globalReadOffsetA_0_0_50_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_50_0), wgK );
  int64_t globalReadOffsetA_0_0_51_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_51_0), wgK );
  int64_t globalReadOffsetA_0_0_52_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_52_0), wgK );
  int64_t globalReadOffsetA_0_0_53_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_53_0), wgK );
  int64_t globalReadOffsetA_0_0_54_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_54_0), wgK );
  int64_t globalReadOffsetA_0_0_55_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_55_0), wgK );
  int64_t globalReadOffsetA_0_0_56_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_56_0), wgK );
  int64_t globalReadOffsetA_0_0_57_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_57_0), wgK );
  int64_t globalReadOffsetA_0_0_58_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_58_0), wgK );
  int64_t globalReadOffsetA_0_0_59_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_59_0), wgK );
  int64_t globalReadOffsetA_0_0_60_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_60_0), wgK );
  int64_t globalReadOffsetA_0_0_61_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_61_0), wgK );
  int64_t globalReadOffsetA_0_0_62_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_62_0), wgK );
  int64_t globalReadOffsetA_0_0_63_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_63_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;
  DATA_TYPE const *globalReadA_0_0_16_0 = A + globalReadOffsetA_0_0_16_0;
  DATA_TYPE const *globalReadA_0_0_17_0 = A + globalReadOffsetA_0_0_17_0;
  DATA_TYPE const *globalReadA_0_0_18_0 = A + globalReadOffsetA_0_0_18_0;
  DATA_TYPE const *globalReadA_0_0_19_0 = A + globalReadOffsetA_0_0_19_0;
  DATA_TYPE const *globalReadA_0_0_20_0 = A + globalReadOffsetA_0_0_20_0;
  DATA_TYPE const *globalReadA_0_0_21_0 = A + globalReadOffsetA_0_0_21_0;
  DATA_TYPE const *globalReadA_0_0_22_0 = A + globalReadOffsetA_0_0_22_0;
  DATA_TYPE const *globalReadA_0_0_23_0 = A + globalReadOffsetA_0_0_23_0;
  DATA_TYPE const *globalReadA_0_0_24_0 = A + globalReadOffsetA_0_0_24_0;
  DATA_TYPE const *globalReadA_0_0_25_0 = A + globalReadOffsetA_0_0_25_0;
  DATA_TYPE const *globalReadA_0_0_26_0 = A + globalReadOffsetA_0_0_26_0;
  DATA_TYPE const *globalReadA_0_0_27_0 = A + globalReadOffsetA_0_0_27_0;
  DATA_TYPE const *globalReadA_0_0_28_0 = A + globalReadOffsetA_0_0_28_0;
  DATA_TYPE const *globalReadA_0_0_29_0 = A + globalReadOffsetA_0_0_29_0;
  DATA_TYPE const *globalReadA_0_0_30_0 = A + globalReadOffsetA_0_0_30_0;
  DATA_TYPE const *globalReadA_0_0_31_0 = A + globalReadOffsetA_0_0_31_0;
  DATA_TYPE const *globalReadA_0_0_32_0 = A + globalReadOffsetA_0_0_32_0;
  DATA_TYPE const *globalReadA_0_0_33_0 = A + globalReadOffsetA_0_0_33_0;
  DATA_TYPE const *globalReadA_0_0_34_0 = A + globalReadOffsetA_0_0_34_0;
  DATA_TYPE const *globalReadA_0_0_35_0 = A + globalReadOffsetA_0_0_35_0;
  DATA_TYPE const *globalReadA_0_0_36_0 = A + globalReadOffsetA_0_0_36_0;
  DATA_TYPE const *globalReadA_0_0_37_0 = A + globalReadOffsetA_0_0_37_0;
  DATA_TYPE const *globalReadA_0_0_38_0 = A + globalReadOffsetA_0_0_38_0;
  DATA_TYPE const *globalReadA_0_0_39_0 = A + globalReadOffsetA_0_0_39_0;
  DATA_TYPE const *globalReadA_0_0_40_0 = A + globalReadOffsetA_0_0_40_0;
  DATA_TYPE const *globalReadA_0_0_41_0 = A + globalReadOffsetA_0_0_41_0;
  DATA_TYPE const *globalReadA_0_0_42_0 = A + globalReadOffsetA_0_0_42_0;
  DATA_TYPE const *globalReadA_0_0_43_0 = A + globalReadOffsetA_0_0_43_0;
  DATA_TYPE const *globalReadA_0_0_44_0 = A + globalReadOffsetA_0_0_44_0;
  DATA_TYPE const *globalReadA_0_0_45_0 = A + globalReadOffsetA_0_0_45_0;
  DATA_TYPE const *globalReadA_0_0_46_0 = A + globalReadOffsetA_0_0_46_0;
  DATA_TYPE const *globalReadA_0_0_47_0 = A + globalReadOffsetA_0_0_47_0;
  DATA_TYPE const *globalReadA_0_0_48_0 = A + globalReadOffsetA_0_0_48_0;
  DATA_TYPE const *globalReadA_0_0_49_0 = A + globalReadOffsetA_0_0_49_0;
  DATA_TYPE const *globalReadA_0_0_50_0 = A + globalReadOffsetA_0_0_50_0;
  DATA_TYPE const *globalReadA_0_0_51_0 = A + globalReadOffsetA_0_0_51_0;
  DATA_TYPE const *globalReadA_0_0_52_0 = A + globalReadOffsetA_0_0_52_0;
  DATA_TYPE const *globalReadA_0_0_53_0 = A + globalReadOffsetA_0_0_53_0;
  DATA_TYPE const *globalReadA_0_0_54_0 = A + globalReadOffsetA_0_0_54_0;
  DATA_TYPE const *globalReadA_0_0_55_0 = A + globalReadOffsetA_0_0_55_0;
  DATA_TYPE const *globalReadA_0_0_56_0 = A + globalReadOffsetA_0_0_56_0;
  DATA_TYPE const *globalReadA_0_0_57_0 = A + globalReadOffsetA_0_0_57_0;
  DATA_TYPE const *globalReadA_0_0_58_0 = A + globalReadOffsetA_0_0_58_0;
  DATA_TYPE const *globalReadA_0_0_59_0 = A + globalReadOffsetA_0_0_59_0;
  DATA_TYPE const *globalReadA_0_0_60_0 = A + globalReadOffsetA_0_0_60_0;
  DATA_TYPE const *globalReadA_0_0_61_0 = A + globalReadOffsetA_0_0_61_0;
  DATA_TYPE const *globalReadA_0_0_62_0 = A + globalReadOffsetA_0_0_62_0;
  DATA_TYPE const *globalReadA_0_0_63_0 = A + globalReadOffsetA_0_0_63_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 8*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 9*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 10*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 11*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 12*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 13*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 14*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 15*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_16_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 16*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_17_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 17*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_18_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 18*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_19_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 19*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_20_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 20*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_21_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 21*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_22_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 22*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_23_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 23*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_24_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 24*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_25_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 25*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_26_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 26*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_27_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 27*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_28_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 28*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_29_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 29*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_30_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 30*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_31_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 31*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_32_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 32*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_33_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 33*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_34_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 34*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_35_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 35*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_36_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 36*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_37_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 37*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_38_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 38*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_39_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 39*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_40_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 40*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_41_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 41*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_42_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 42*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_43_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 43*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_44_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 44*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_45_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 45*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_46_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 46*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_47_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 47*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_48_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 48*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_49_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 49*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_50_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 50*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_51_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 51*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_52_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 52*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_53_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 53*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_54_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 54*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_55_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 55*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_56_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 56*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_57_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 57*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_58_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 58*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_59_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 59*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_60_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 60*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_61_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 61*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_62_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 62*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_63_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 63*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_0_0_15_0;
  DATA_TYPE *localWriteA_0_0_16_0;
  DATA_TYPE *localWriteA_0_0_17_0;
  DATA_TYPE *localWriteA_0_0_18_0;
  DATA_TYPE *localWriteA_0_0_19_0;
  DATA_TYPE *localWriteA_0_0_20_0;
  DATA_TYPE *localWriteA_0_0_21_0;
  DATA_TYPE *localWriteA_0_0_22_0;
  DATA_TYPE *localWriteA_0_0_23_0;
  DATA_TYPE *localWriteA_0_0_24_0;
  DATA_TYPE *localWriteA_0_0_25_0;
  DATA_TYPE *localWriteA_0_0_26_0;
  DATA_TYPE *localWriteA_0_0_27_0;
  DATA_TYPE *localWriteA_0_0_28_0;
  DATA_TYPE *localWriteA_0_0_29_0;
  DATA_TYPE *localWriteA_0_0_30_0;
  DATA_TYPE *localWriteA_0_0_31_0;
  DATA_TYPE *localWriteA_0_0_32_0;
  DATA_TYPE *localWriteA_0_0_33_0;
  DATA_TYPE *localWriteA_0_0_34_0;
  DATA_TYPE *localWriteA_0_0_35_0;
  DATA_TYPE *localWriteA_0_0_36_0;
  DATA_TYPE *localWriteA_0_0_37_0;
  DATA_TYPE *localWriteA_0_0_38_0;
  DATA_TYPE *localWriteA_0_0_39_0;
  DATA_TYPE *localWriteA_0_0_40_0;
  DATA_TYPE *localWriteA_0_0_41_0;
  DATA_TYPE *localWriteA_0_0_42_0;
  DATA_TYPE *localWriteA_0_0_43_0;
  DATA_TYPE *localWriteA_0_0_44_0;
  DATA_TYPE *localWriteA_0_0_45_0;
  DATA_TYPE *localWriteA_0_0_46_0;
  DATA_TYPE *localWriteA_0_0_47_0;
  DATA_TYPE *localWriteA_0_0_48_0;
  DATA_TYPE *localWriteA_0_0_49_0;
  DATA_TYPE *localWriteA_0_0_50_0;
  DATA_TYPE *localWriteA_0_0_51_0;
  DATA_TYPE *localWriteA_0_0_52_0;
  DATA_TYPE *localWriteA_0_0_53_0;
  DATA_TYPE *localWriteA_0_0_54_0;
  DATA_TYPE *localWriteA_0_0_55_0;
  DATA_TYPE *localWriteA_0_0_56_0;
  DATA_TYPE *localWriteA_0_0_57_0;
  DATA_TYPE *localWriteA_0_0_58_0;
  DATA_TYPE *localWriteA_0_0_59_0;
  DATA_TYPE *localWriteA_0_0_60_0;
  DATA_TYPE *localWriteA_0_0_61_0;
  DATA_TYPE *localWriteA_0_0_62_0;
  DATA_TYPE *localWriteA_0_0_63_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);
  localWriteA_0_0_16_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_16_0);
  localWriteA_0_0_17_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_17_0);
  localWriteA_0_0_18_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_18_0);
  localWriteA_0_0_19_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_19_0);
  localWriteA_0_0_20_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_20_0);
  localWriteA_0_0_21_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_21_0);
  localWriteA_0_0_22_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_22_0);
  localWriteA_0_0_23_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_23_0);
  localWriteA_0_0_24_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_24_0);
  localWriteA_0_0_25_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_25_0);
  localWriteA_0_0_26_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_26_0);
  localWriteA_0_0_27_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_27_0);
  localWriteA_0_0_28_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_28_0);
  localWriteA_0_0_29_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_29_0);
  localWriteA_0_0_30_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_30_0);
  localWriteA_0_0_31_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_31_0);
  localWriteA_0_0_32_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_32_0);
  localWriteA_0_0_33_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_33_0);
  localWriteA_0_0_34_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_34_0);
  localWriteA_0_0_35_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_35_0);
  localWriteA_0_0_36_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_36_0);
  localWriteA_0_0_37_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_37_0);
  localWriteA_0_0_38_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_38_0);
  localWriteA_0_0_39_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_39_0);
  localWriteA_0_0_40_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_40_0);
  localWriteA_0_0_41_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_41_0);
  localWriteA_0_0_42_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_42_0);
  localWriteA_0_0_43_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_43_0);
  localWriteA_0_0_44_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_44_0);
  localWriteA_0_0_45_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_45_0);
  localWriteA_0_0_46_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_46_0);
  localWriteA_0_0_47_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_47_0);
  localWriteA_0_0_48_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_48_0);
  localWriteA_0_0_49_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_49_0);
  localWriteA_0_0_50_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_50_0);
  localWriteA_0_0_51_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_51_0);
  localWriteA_0_0_52_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_52_0);
  localWriteA_0_0_53_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_53_0);
  localWriteA_0_0_54_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_54_0);
  localWriteA_0_0_55_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_55_0);
  localWriteA_0_0_56_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_56_0);
  localWriteA_0_0_57_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_57_0);
  localWriteA_0_0_58_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_58_0);
  localWriteA_0_0_59_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_59_0);
  localWriteA_0_0_60_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_60_0);
  localWriteA_0_0_61_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_61_0);
  localWriteA_0_0_62_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_62_0);
  localWriteA_0_0_63_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_63_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);
    a_0_0_16_0 = *(globalReadA_0_0_16_0 + 0);
    a_0_0_17_0 = *(globalReadA_0_0_17_0 + 0);
    a_0_0_18_0 = *(globalReadA_0_0_18_0 + 0);
    a_0_0_19_0 = *(globalReadA_0_0_19_0 + 0);
    a_0_0_20_0 = *(globalReadA_0_0_20_0 + 0);
    a_0_0_21_0 = *(globalReadA_0_0_21_0 + 0);
    a_0_0_22_0 = *(globalReadA_0_0_22_0 + 0);
    a_0_0_23_0 = *(globalReadA_0_0_23_0 + 0);
    a_0_0_24_0 = *(globalReadA_0_0_24_0 + 0);
    a_0_0_25_0 = *(globalReadA_0_0_25_0 + 0);
    a_0_0_26_0 = *(globalReadA_0_0_26_0 + 0);
    a_0_0_27_0 = *(globalReadA_0_0_27_0 + 0);
    a_0_0_28_0 = *(globalReadA_0_0_28_0 + 0);
    a_0_0_29_0 = *(globalReadA_0_0_29_0 + 0);
    a_0_0_30_0 = *(globalReadA_0_0_30_0 + 0);
    a_0_0_31_0 = *(globalReadA_0_0_31_0 + 0);
    a_0_0_32_0 = *(globalReadA_0_0_32_0 + 0);
    a_0_0_33_0 = *(globalReadA_0_0_33_0 + 0);
    a_0_0_34_0 = *(globalReadA_0_0_34_0 + 0);
    a_0_0_35_0 = *(globalReadA_0_0_35_0 + 0);
    a_0_0_36_0 = *(globalReadA_0_0_36_0 + 0);
    a_0_0_37_0 = *(globalReadA_0_0_37_0 + 0);
    a_0_0_38_0 = *(globalReadA_0_0_38_0 + 0);
    a_0_0_39_0 = *(globalReadA_0_0_39_0 + 0);
    a_0_0_40_0 = *(globalReadA_0_0_40_0 + 0);
    a_0_0_41_0 = *(globalReadA_0_0_41_0 + 0);
    a_0_0_42_0 = *(globalReadA_0_0_42_0 + 0);
    a_0_0_43_0 = *(globalReadA_0_0_43_0 + 0);
    a_0_0_44_0 = *(globalReadA_0_0_44_0 + 0);
    a_0_0_45_0 = *(globalReadA_0_0_45_0 + 0);
    a_0_0_46_0 = *(globalReadA_0_0_46_0 + 0);
    a_0_0_47_0 = *(globalReadA_0_0_47_0 + 0);
    a_0_0_48_0 = *(globalReadA_0_0_48_0 + 0);
    a_0_0_49_0 = *(globalReadA_0_0_49_0 + 0);
    a_0_0_50_0 = *(globalReadA_0_0_50_0 + 0);
    a_0_0_51_0 = *(globalReadA_0_0_51_0 + 0);
    a_0_0_52_0 = *(globalReadA_0_0_52_0 + 0);
    a_0_0_53_0 = *(globalReadA_0_0_53_0 + 0);
    a_0_0_54_0 = *(globalReadA_0_0_54_0 + 0);
    a_0_0_55_0 = *(globalReadA_0_0_55_0 + 0);
    a_0_0_56_0 = *(globalReadA_0_0_56_0 + 0);
    a_0_0_57_0 = *(globalReadA_0_0_57_0 + 0);
    a_0_0_58_0 = *(globalReadA_0_0_58_0 + 0);
    a_0_0_59_0 = *(globalReadA_0_0_59_0 + 0);
    a_0_0_60_0 = *(globalReadA_0_0_60_0 + 0);
    a_0_0_61_0 = *(globalReadA_0_0_61_0 + 0);
    a_0_0_62_0 = *(globalReadA_0_0_62_0 + 0);
    a_0_0_63_0 = *(globalReadA_0_0_63_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    globalReadA_0_0_16_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_16_0) + 1*globalReadIncAL);
    globalReadA_0_0_17_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_17_0) + 1*globalReadIncAL);
    globalReadA_0_0_18_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_18_0) + 1*globalReadIncAL);
    globalReadA_0_0_19_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_19_0) + 1*globalReadIncAL);
    globalReadA_0_0_20_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_20_0) + 1*globalReadIncAL);
    globalReadA_0_0_21_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_21_0) + 1*globalReadIncAL);
    globalReadA_0_0_22_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_22_0) + 1*globalReadIncAL);
    globalReadA_0_0_23_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_23_0) + 1*globalReadIncAL);
    globalReadA_0_0_24_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_24_0) + 1*globalReadIncAL);
    globalReadA_0_0_25_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_25_0) + 1*globalReadIncAL);
    globalReadA_0_0_26_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_26_0) + 1*globalReadIncAL);
    globalReadA_0_0_27_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_27_0) + 1*globalReadIncAL);
    globalReadA_0_0_28_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_28_0) + 1*globalReadIncAL);
    globalReadA_0_0_29_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_29_0) + 1*globalReadIncAL);
    globalReadA_0_0_30_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_30_0) + 1*globalReadIncAL);
    globalReadA_0_0_31_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_31_0) + 1*globalReadIncAL);
    globalReadA_0_0_32_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_32_0) + 1*globalReadIncAL);
    globalReadA_0_0_33_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_33_0) + 1*globalReadIncAL);
    globalReadA_0_0_34_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_34_0) + 1*globalReadIncAL);
    globalReadA_0_0_35_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_35_0) + 1*globalReadIncAL);
    globalReadA_0_0_36_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_36_0) + 1*globalReadIncAL);
    globalReadA_0_0_37_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_37_0) + 1*globalReadIncAL);
    globalReadA_0_0_38_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_38_0) + 1*globalReadIncAL);
    globalReadA_0_0_39_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_39_0) + 1*globalReadIncAL);
    globalReadA_0_0_40_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_40_0) + 1*globalReadIncAL);
    globalReadA_0_0_41_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_41_0) + 1*globalReadIncAL);
    globalReadA_0_0_42_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_42_0) + 1*globalReadIncAL);
    globalReadA_0_0_43_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_43_0) + 1*globalReadIncAL);
    globalReadA_0_0_44_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_44_0) + 1*globalReadIncAL);
    globalReadA_0_0_45_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_45_0) + 1*globalReadIncAL);
    globalReadA_0_0_46_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_46_0) + 1*globalReadIncAL);
    globalReadA_0_0_47_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_47_0) + 1*globalReadIncAL);
    globalReadA_0_0_48_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_48_0) + 1*globalReadIncAL);
    globalReadA_0_0_49_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_49_0) + 1*globalReadIncAL);
    globalReadA_0_0_50_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_50_0) + 1*globalReadIncAL);
    globalReadA_0_0_51_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_51_0) + 1*globalReadIncAL);
    globalReadA_0_0_52_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_52_0) + 1*globalReadIncAL);
    globalReadA_0_0_53_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_53_0) + 1*globalReadIncAL);
    globalReadA_0_0_54_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_54_0) + 1*globalReadIncAL);
    globalReadA_0_0_55_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_55_0) + 1*globalReadIncAL);
    globalReadA_0_0_56_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_56_0) + 1*globalReadIncAL);
    globalReadA_0_0_57_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_57_0) + 1*globalReadIncAL);
    globalReadA_0_0_58_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_58_0) + 1*globalReadIncAL);
    globalReadA_0_0_59_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_59_0) + 1*globalReadIncAL);
    globalReadA_0_0_60_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_60_0) + 1*globalReadIncAL);
    globalReadA_0_0_61_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_61_0) + 1*globalReadIncAL);
    globalReadA_0_0_62_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_62_0) + 1*globalReadIncAL);
    globalReadA_0_0_63_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_63_0) + 1*globalReadIncAL);

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
    *(localWriteA_0_0_16_0 + 0) = a_0_0_16_0;
    *(localWriteA_0_0_17_0 + 0) = a_0_0_17_0;
    *(localWriteA_0_0_18_0 + 0) = a_0_0_18_0;
    *(localWriteA_0_0_19_0 + 0) = a_0_0_19_0;
    *(localWriteA_0_0_20_0 + 0) = a_0_0_20_0;
    *(localWriteA_0_0_21_0 + 0) = a_0_0_21_0;
    *(localWriteA_0_0_22_0 + 0) = a_0_0_22_0;
    *(localWriteA_0_0_23_0 + 0) = a_0_0_23_0;
    *(localWriteA_0_0_24_0 + 0) = a_0_0_24_0;
    *(localWriteA_0_0_25_0 + 0) = a_0_0_25_0;
    *(localWriteA_0_0_26_0 + 0) = a_0_0_26_0;
    *(localWriteA_0_0_27_0 + 0) = a_0_0_27_0;
    *(localWriteA_0_0_28_0 + 0) = a_0_0_28_0;
    *(localWriteA_0_0_29_0 + 0) = a_0_0_29_0;
    *(localWriteA_0_0_30_0 + 0) = a_0_0_30_0;
    *(localWriteA_0_0_31_0 + 0) = a_0_0_31_0;
    *(localWriteA_0_0_32_0 + 0) = a_0_0_32_0;
    *(localWriteA_0_0_33_0 + 0) = a_0_0_33_0;
    *(localWriteA_0_0_34_0 + 0) = a_0_0_34_0;
    *(localWriteA_0_0_35_0 + 0) = a_0_0_35_0;
    *(localWriteA_0_0_36_0 + 0) = a_0_0_36_0;
    *(localWriteA_0_0_37_0 + 0) = a_0_0_37_0;
    *(localWriteA_0_0_38_0 + 0) = a_0_0_38_0;
    *(localWriteA_0_0_39_0 + 0) = a_0_0_39_0;
    *(localWriteA_0_0_40_0 + 0) = a_0_0_40_0;
    *(localWriteA_0_0_41_0 + 0) = a_0_0_41_0;
    *(localWriteA_0_0_42_0 + 0) = a_0_0_42_0;
    *(localWriteA_0_0_43_0 + 0) = a_0_0_43_0;
    *(localWriteA_0_0_44_0 + 0) = a_0_0_44_0;
    *(localWriteA_0_0_45_0 + 0) = a_0_0_45_0;
    *(localWriteA_0_0_46_0 + 0) = a_0_0_46_0;
    *(localWriteA_0_0_47_0 + 0) = a_0_0_47_0;
    *(localWriteA_0_0_48_0 + 0) = a_0_0_48_0;
    *(localWriteA_0_0_49_0 + 0) = a_0_0_49_0;
    *(localWriteA_0_0_50_0 + 0) = a_0_0_50_0;
    *(localWriteA_0_0_51_0 + 0) = a_0_0_51_0;
    *(localWriteA_0_0_52_0 + 0) = a_0_0_52_0;
    *(localWriteA_0_0_53_0 + 0) = a_0_0_53_0;
    *(localWriteA_0_0_54_0 + 0) = a_0_0_54_0;
    *(localWriteA_0_0_55_0 + 0) = a_0_0_55_0;
    *(localWriteA_0_0_56_0 + 0) = a_0_0_56_0;
    *(localWriteA_0_0_57_0 + 0) = a_0_0_57_0;
    *(localWriteA_0_0_58_0 + 0) = a_0_0_58_0;
    *(localWriteA_0_0_59_0 + 0) = a_0_0_59_0;
    *(localWriteA_0_0_60_0 + 0) = a_0_0_60_0;
    *(localWriteA_0_0_61_0 + 0) = a_0_0_61_0;
    *(localWriteA_0_0_62_0 + 0) = a_0_0_62_0;
    *(localWriteA_0_0_63_0 + 0) = a_0_0_63_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 15 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 16 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 17 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 18 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 19 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 20 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 21 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 22 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 23 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 24 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 25 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 26 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 27 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 28 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 29 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 30 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 31 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 32 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 33 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 34 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 35 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 36 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 37 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 38 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 39 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 40 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 41 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 42 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 43 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 44 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 45 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 46 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 47 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 48 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 49 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 50 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 51 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 52 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 53 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 54 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 55 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 56 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 57 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 58 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 59 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 60 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 61 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 62 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_1x1


    /* iter 63 */

    MAC_1x1_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_0_8_0 = ( globalReadOffsetAL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_0_0_9_0 = ( globalReadOffsetAL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_0_0_10_0 = ( globalReadOffsetAL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_0_0_11_0 = ( globalReadOffsetAL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_0_0_12_0 = ( globalReadOffsetAL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_0_0_13_0 = ( globalReadOffsetAL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_0_0_14_0 = ( globalReadOffsetAL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_0_0_15_0 = ( globalReadOffsetAL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);
  a_0_0_16_0 = ( globalReadOffsetAL_16_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_16_0 + 0);
  a_0_0_17_0 = ( globalReadOffsetAL_17_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_17_0 + 0);
  a_0_0_18_0 = ( globalReadOffsetAL_18_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_18_0 + 0);
  a_0_0_19_0 = ( globalReadOffsetAL_19_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_19_0 + 0);
  a_0_0_20_0 = ( globalReadOffsetAL_20_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_20_0 + 0);
  a_0_0_21_0 = ( globalReadOffsetAL_21_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_21_0 + 0);
  a_0_0_22_0 = ( globalReadOffsetAL_22_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_22_0 + 0);
  a_0_0_23_0 = ( globalReadOffsetAL_23_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_23_0 + 0);
  a_0_0_24_0 = ( globalReadOffsetAL_24_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_24_0 + 0);
  a_0_0_25_0 = ( globalReadOffsetAL_25_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_25_0 + 0);
  a_0_0_26_0 = ( globalReadOffsetAL_26_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_26_0 + 0);
  a_0_0_27_0 = ( globalReadOffsetAL_27_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_27_0 + 0);
  a_0_0_28_0 = ( globalReadOffsetAL_28_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_28_0 + 0);
  a_0_0_29_0 = ( globalReadOffsetAL_29_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_29_0 + 0);
  a_0_0_30_0 = ( globalReadOffsetAL_30_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_30_0 + 0);
  a_0_0_31_0 = ( globalReadOffsetAL_31_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_31_0 + 0);
  a_0_0_32_0 = ( globalReadOffsetAL_32_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_32_0 + 0);
  a_0_0_33_0 = ( globalReadOffsetAL_33_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_33_0 + 0);
  a_0_0_34_0 = ( globalReadOffsetAL_34_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_34_0 + 0);
  a_0_0_35_0 = ( globalReadOffsetAL_35_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_35_0 + 0);
  a_0_0_36_0 = ( globalReadOffsetAL_36_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_36_0 + 0);
  a_0_0_37_0 = ( globalReadOffsetAL_37_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_37_0 + 0);
  a_0_0_38_0 = ( globalReadOffsetAL_38_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_38_0 + 0);
  a_0_0_39_0 = ( globalReadOffsetAL_39_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_39_0 + 0);
  a_0_0_40_0 = ( globalReadOffsetAL_40_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_40_0 + 0);
  a_0_0_41_0 = ( globalReadOffsetAL_41_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_41_0 + 0);
  a_0_0_42_0 = ( globalReadOffsetAL_42_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_42_0 + 0);
  a_0_0_43_0 = ( globalReadOffsetAL_43_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_43_0 + 0);
  a_0_0_44_0 = ( globalReadOffsetAL_44_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_44_0 + 0);
  a_0_0_45_0 = ( globalReadOffsetAL_45_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_45_0 + 0);
  a_0_0_46_0 = ( globalReadOffsetAL_46_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_46_0 + 0);
  a_0_0_47_0 = ( globalReadOffsetAL_47_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_47_0 + 0);
  a_0_0_48_0 = ( globalReadOffsetAL_48_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_48_0 + 0);
  a_0_0_49_0 = ( globalReadOffsetAL_49_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_49_0 + 0);
  a_0_0_50_0 = ( globalReadOffsetAL_50_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_50_0 + 0);
  a_0_0_51_0 = ( globalReadOffsetAL_51_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_51_0 + 0);
  a_0_0_52_0 = ( globalReadOffsetAL_52_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_52_0 + 0);
  a_0_0_53_0 = ( globalReadOffsetAL_53_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_53_0 + 0);
  a_0_0_54_0 = ( globalReadOffsetAL_54_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_54_0 + 0);
  a_0_0_55_0 = ( globalReadOffsetAL_55_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_55_0 + 0);
  a_0_0_56_0 = ( globalReadOffsetAL_56_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_56_0 + 0);
  a_0_0_57_0 = ( globalReadOffsetAL_57_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_57_0 + 0);
  a_0_0_58_0 = ( globalReadOffsetAL_58_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_58_0 + 0);
  a_0_0_59_0 = ( globalReadOffsetAL_59_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_59_0 + 0);
  a_0_0_60_0 = ( globalReadOffsetAL_60_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_60_0 + 0);
  a_0_0_61_0 = ( globalReadOffsetAL_61_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_61_0 + 0);
  a_0_0_62_0 = ( globalReadOffsetAL_62_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_62_0 + 0);
  a_0_0_63_0 = ( globalReadOffsetAL_63_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_63_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);
  localWriteA_0_0_16_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_16_0);
  localWriteA_0_0_17_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_17_0);
  localWriteA_0_0_18_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_18_0);
  localWriteA_0_0_19_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_19_0);
  localWriteA_0_0_20_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_20_0);
  localWriteA_0_0_21_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_21_0);
  localWriteA_0_0_22_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_22_0);
  localWriteA_0_0_23_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_23_0);
  localWriteA_0_0_24_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_24_0);
  localWriteA_0_0_25_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_25_0);
  localWriteA_0_0_26_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_26_0);
  localWriteA_0_0_27_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_27_0);
  localWriteA_0_0_28_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_28_0);
  localWriteA_0_0_29_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_29_0);
  localWriteA_0_0_30_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_30_0);
  localWriteA_0_0_31_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_31_0);
  localWriteA_0_0_32_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_32_0);
  localWriteA_0_0_33_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_33_0);
  localWriteA_0_0_34_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_34_0);
  localWriteA_0_0_35_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_35_0);
  localWriteA_0_0_36_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_36_0);
  localWriteA_0_0_37_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_37_0);
  localWriteA_0_0_38_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_38_0);
  localWriteA_0_0_39_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_39_0);
  localWriteA_0_0_40_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_40_0);
  localWriteA_0_0_41_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_41_0);
  localWriteA_0_0_42_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_42_0);
  localWriteA_0_0_43_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_43_0);
  localWriteA_0_0_44_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_44_0);
  localWriteA_0_0_45_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_45_0);
  localWriteA_0_0_46_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_46_0);
  localWriteA_0_0_47_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_47_0);
  localWriteA_0_0_48_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_48_0);
  localWriteA_0_0_49_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_49_0);
  localWriteA_0_0_50_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_50_0);
  localWriteA_0_0_51_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_51_0);
  localWriteA_0_0_52_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_52_0);
  localWriteA_0_0_53_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_53_0);
  localWriteA_0_0_54_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_54_0);
  localWriteA_0_0_55_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_55_0);
  localWriteA_0_0_56_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_56_0);
  localWriteA_0_0_57_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_57_0);
  localWriteA_0_0_58_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_58_0);
  localWriteA_0_0_59_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_59_0);
  localWriteA_0_0_60_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_60_0);
  localWriteA_0_0_61_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_61_0);
  localWriteA_0_0_62_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_62_0);
  localWriteA_0_0_63_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_63_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
  *(localWriteA_0_0_16_0 + 0) = a_0_0_16_0;
  *(localWriteA_0_0_17_0 + 0) = a_0_0_17_0;
  *(localWriteA_0_0_18_0 + 0) = a_0_0_18_0;
  *(localWriteA_0_0_19_0 + 0) = a_0_0_19_0;
  *(localWriteA_0_0_20_0 + 0) = a_0_0_20_0;
  *(localWriteA_0_0_21_0 + 0) = a_0_0_21_0;
  *(localWriteA_0_0_22_0 + 0) = a_0_0_22_0;
  *(localWriteA_0_0_23_0 + 0) = a_0_0_23_0;
  *(localWriteA_0_0_24_0 + 0) = a_0_0_24_0;
  *(localWriteA_0_0_25_0 + 0) = a_0_0_25_0;
  *(localWriteA_0_0_26_0 + 0) = a_0_0_26_0;
  *(localWriteA_0_0_27_0 + 0) = a_0_0_27_0;
  *(localWriteA_0_0_28_0 + 0) = a_0_0_28_0;
  *(localWriteA_0_0_29_0 + 0) = a_0_0_29_0;
  *(localWriteA_0_0_30_0 + 0) = a_0_0_30_0;
  *(localWriteA_0_0_31_0 + 0) = a_0_0_31_0;
  *(localWriteA_0_0_32_0 + 0) = a_0_0_32_0;
  *(localWriteA_0_0_33_0 + 0) = a_0_0_33_0;
  *(localWriteA_0_0_34_0 + 0) = a_0_0_34_0;
  *(localWriteA_0_0_35_0 + 0) = a_0_0_35_0;
  *(localWriteA_0_0_36_0 + 0) = a_0_0_36_0;
  *(localWriteA_0_0_37_0 + 0) = a_0_0_37_0;
  *(localWriteA_0_0_38_0 + 0) = a_0_0_38_0;
  *(localWriteA_0_0_39_0 + 0) = a_0_0_39_0;
  *(localWriteA_0_0_40_0 + 0) = a_0_0_40_0;
  *(localWriteA_0_0_41_0 + 0) = a_0_0_41_0;
  *(localWriteA_0_0_42_0 + 0) = a_0_0_42_0;
  *(localWriteA_0_0_43_0 + 0) = a_0_0_43_0;
  *(localWriteA_0_0_44_0 + 0) = a_0_0_44_0;
  *(localWriteA_0_0_45_0 + 0) = a_0_0_45_0;
  *(localWriteA_0_0_46_0 + 0) = a_0_0_46_0;
  *(localWriteA_0_0_47_0 + 0) = a_0_0_47_0;
  *(localWriteA_0_0_48_0 + 0) = a_0_0_48_0;
  *(localWriteA_0_0_49_0 + 0) = a_0_0_49_0;
  *(localWriteA_0_0_50_0 + 0) = a_0_0_50_0;
  *(localWriteA_0_0_51_0 + 0) = a_0_0_51_0;
  *(localWriteA_0_0_52_0 + 0) = a_0_0_52_0;
  *(localWriteA_0_0_53_0 + 0) = a_0_0_53_0;
  *(localWriteA_0_0_54_0 + 0) = a_0_0_54_0;
  *(localWriteA_0_0_55_0 + 0) = a_0_0_55_0;
  *(localWriteA_0_0_56_0 + 0) = a_0_0_56_0;
  *(localWriteA_0_0_57_0 + 0) = a_0_0_57_0;
  *(localWriteA_0_0_58_0 + 0) = a_0_0_58_0;
  *(localWriteA_0_0_59_0 + 0) = a_0_0_59_0;
  *(localWriteA_0_0_60_0 + 0) = a_0_0_60_0;
  *(localWriteA_0_0_61_0 + 0) = a_0_0_61_0;
  *(localWriteA_0_0_62_0 + 0) = a_0_0_62_0;
  *(localWriteA_0_0_63_0 + 0) = a_0_0_63_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_1x1

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_1x1
#undef MAC_1x1_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 64
#define SG1J 1
#define TT0I 1
#define TT1J 1
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 64
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 2

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 64
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 4096
#define LDS_NUM_ELEMENTS 4160

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 1x1 micro-tile */
#define MAC_1x1\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \

#define MAC_1x1_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x1x64_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU0_SUM0_SUS128_SRVW0_SVW4_SNLL0_TT1_1_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG64_1_1_WGM2(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE a_0_0_16_0;
  DATA_TYPE a_0_0_17_0;
  DATA_TYPE a_0_0_18_0;
  DATA_TYPE a_0_0_19_0;
  DATA_TYPE a_0_0_20_0;
  DATA_TYPE a_0_0_21_0;
  DATA_TYPE a_0_0_22_0;
  DATA_TYPE a_0_0_23_0;
  DATA_TYPE a_0_0_24_0;
  DATA_TYPE a_0_0_25_0;
  DATA_TYPE a_0_0_26_0;
  DATA_TYPE a_0_0_27_0;
  DATA_TYPE a_0_0_28_0;
  DATA_TYPE a_0_0_29_0;
  DATA_TYPE a_0_0_30_0;
  DATA_TYPE a_0_0_31_0;
  DATA_TYPE a_0_0_32_0;
  DATA_TYPE a_0_0_33_0;
  DATA_TYPE a_0_0_34_0;
  DATA_TYPE a_0_0_35_0;
  DATA_TYPE a_0_0_36_0;
  DATA_TYPE a_0_0_37_0;
  DATA_TYPE a_0_0_38_0;
  DATA_TYPE a_0_0_39_0;
  DATA_TYPE a_0_0_40_0;
  DATA_TYPE a_0_0_41_0;
  DATA_TYPE a_0_0_42_0;
  DATA_TYPE a_0_0_43_0;
  DATA_TYPE a_0_0_44_0;
  DATA_TYPE a_0_0_45_0;
  DATA_TYPE a_0_0_46_0;
  DATA_TYPE a_0_0_47_0;
  DATA_TYPE a_0_0_48_0;
  DATA_TYPE a_0_0_49_0;
  DATA_TYPE a_0_0_50_0;
  DATA_TYPE a_0_0_51_0;
  DATA_TYPE a_0_0_52_0;
  DATA_TYPE a_0_0_53_0;
  DATA_TYPE a_0_0_54_0;
  DATA_TYPE a_0_0_55_0;
  DATA_TYPE a_0_0_56_0;
  DATA_TYPE a_0_0_57_0;
  DATA_TYPE a_0_0_58_0;
  DATA_TYPE a_0_0_59_0;
  DATA_TYPE a_0_0_60_0;
  DATA_TYPE a_0_0_61_0;
  DATA_TYPE a_0_0_62_0;
  DATA_TYPE a_0_0_63_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;
  unsigned int globalReadOffsetAL_8_0 = globalReadOffsetAL + 0 + 8*LSPA;
  unsigned int globalReadOffsetAL_9_0 = globalReadOffsetAL + 0 + 9*LSPA;
  unsigned int globalReadOffsetAL_10_0 = globalReadOffsetAL + 0 + 10*LSPA;
  unsigned int globalReadOffsetAL_11_0 = globalReadOffsetAL + 0 + 11*LSPA;
  unsigned int globalReadOffsetAL_12_0 = globalReadOffsetAL + 0 + 12*LSPA;
  unsigned int globalReadOffsetAL_13_0 = globalReadOffsetAL + 0 + 13*LSPA;
  unsigned int globalReadOffsetAL_14_0 = globalReadOffsetAL + 0 + 14*LSPA;
  unsigned int globalReadOffsetAL_15_0 = globalReadOffsetAL + 0 + 15*LSPA;
  unsigned int globalReadOffsetAL_16_0 = globalReadOffsetAL + 0 + 16*LSPA;
  unsigned int globalReadOffsetAL_17_0 = globalReadOffsetAL + 0 + 17*LSPA;
  unsigned int globalReadOffsetAL_18_0 = globalReadOffsetAL + 0 + 18*LSPA;
  unsigned int globalReadOffsetAL_19_0 = globalReadOffsetAL + 0 + 19*LSPA;
  unsigned int globalReadOffsetAL_20_0 = globalReadOffsetAL + 0 + 20*LSPA;
  unsigned int globalReadOffsetAL_21_0 = globalReadOffsetAL + 0 + 21*LSPA;
  unsigned int globalReadOffsetAL_22_0 = globalReadOffsetAL + 0 + 22*LSPA;
  unsigned int globalReadOffsetAL_23_0 = globalReadOffsetAL + 0 + 23*LSPA;
  unsigned int globalReadOffsetAL_24_0 = globalReadOffsetAL + 0 + 24*LSPA;
  unsigned int globalReadOffsetAL_25_0 = globalReadOffsetAL + 0 + 25*LSPA;
  unsigned int globalReadOffsetAL_26_0 = globalReadOffsetAL + 0 + 26*LSPA;
  unsigned int globalReadOffsetAL_27_0 = globalReadOffsetAL + 0 + 27*LSPA;
  unsigned int globalReadOffsetAL_28_0 = globalReadOffsetAL + 0 + 28*LSPA;
  unsigned int globalReadOffsetAL_29_0 = globalReadOffsetAL + 0 + 29*LSPA;
  unsigned int globalReadOffsetAL_30_0 = globalReadOffsetAL + 0 + 30*LSPA;
  unsigned int globalReadOffsetAL_31_0 = globalReadOffsetAL + 0 + 31*LSPA;
  unsigned int globalReadOffsetAL_32_0 = globalReadOffsetAL + 0 + 32*LSPA;
  unsigned int globalReadOffsetAL_33_0 = globalReadOffsetAL + 0 + 33*LSPA;
  unsigned int globalReadOffsetAL_34_0 = globalReadOffsetAL + 0 + 34*LSPA;
  unsigned int globalReadOffsetAL_35_0 = globalReadOffsetAL + 0 + 35*LSPA;
  unsigned int globalReadOffsetAL_36_0 = globalReadOffsetAL + 0 + 36*LSPA;
  unsigned int globalReadOffsetAL_37_0 = globalReadOffsetAL + 0 + 37*LSPA;
  unsigned int globalReadOffsetAL_38_0 = globalReadOffsetAL + 0 + 38*LSPA;
  unsigned int globalReadOffsetAL_39_0 = globalReadOffsetAL + 0 + 39*LSPA;
  unsigned int globalReadOffsetAL_40_0 = globalReadOffsetAL + 0 + 40*LSPA;
  unsigned int globalReadOffsetAL_41_0 = globalReadOffsetAL + 0 + 41*LSPA;
  unsigned int globalReadOffsetAL_42_0 = globalReadOffsetAL + 0 + 42*LSPA;
  unsigned int globalReadOffsetAL_43_0 = globalReadOffsetAL + 0 + 43*LSPA;
  unsigned int globalReadOffsetAL_44_0 = globalReadOffsetAL + 0 + 44*LSPA;
  unsigned int globalReadOffsetAL_45_0 = globalReadOffsetAL + 0 + 45*LSPA;
  unsigned int globalReadOffsetAL_46_0 = globalReadOffsetAL + 0 + 46*LSPA;
  unsigned int globalReadOffsetAL_47_0 = globalReadOffsetAL + 0 + 47*LSPA;
  unsigned int globalReadOffsetAL_48_0 = globalReadOffsetAL + 0 + 48*LSPA;
  unsigned int globalReadOffsetAL_49_0 = globalReadOffsetAL + 0 + 49*LSPA;
  unsigned int globalReadOffsetAL_50_0 = globalReadOffsetAL + 0 + 50*LSPA;
  unsigned int globalReadOffsetAL_51_0 = globalReadOffsetAL + 0 + 51*LSPA;
  unsigned int globalReadOffsetAL_52_0 = globalReadOffsetAL + 0 + 52*LSPA;
  unsigned int globalReadOffsetAL_53_0 = globalReadOffsetAL + 0 + 53*LSPA;
  unsigned int globalReadOffsetAL_54_0 = globalReadOffsetAL + 0 + 54*LSPA;
  unsigned int globalReadOffsetAL_55_0 = globalReadOffsetAL + 0 + 55*LSPA;
  unsigned int globalReadOffsetAL_56_0 = globalReadOffsetAL + 0 + 56*LSPA;
  unsigned int globalReadOffsetAL_57_0 = globalReadOffsetAL + 0 + 57*LSPA;
  unsigned int globalReadOffsetAL_58_0 = globalReadOffsetAL + 0 + 58*LSPA;
  unsigned int globalReadOffsetAL_59_0 = globalReadOffsetAL + 0 + 59*LSPA;
  unsigned int globalReadOffsetAL_60_0 = globalReadOffsetAL + 0 + 60*LSPA;
  unsigned int globalReadOffsetAL_61_0 = globalReadOffsetAL + 0 + 61*LSPA;
  unsigned int globalReadOffsetAL_62_0 = globalReadOffsetAL + 0 + 62*LSPA;
  unsigned int globalReadOffsetAL_63_0 = globalReadOffsetAL + 0 + 63*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_15_0), wgK );
  int64_t globalReadOffsetA_0_0_16_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_16_0), wgK );
  int64_t globalReadOffsetA_0_0_17_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_17_0), wgK );
  int64_t globalReadOffsetA_0_0_18_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_18_0), wgK );
  int64_t globalReadOffsetA_0_0_19_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_19_0), wgK );
  int64_t globalReadOffsetA_0_0_20_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_20_0), wgK );
  int64_t globalReadOffsetA_0_0_21_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_21_0), wgK );
  int64_t globalReadOffsetA_0_0_22_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_22_0), wgK );
  int64_t globalReadOffsetA_0_0_23_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_23_0), wgK );
  int64_t globalReadOffsetA_0_0_24_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_24_0), wgK );
  int64_t globalReadOffsetA_0_0_25_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_25_0), wgK );
  int64_t globalReadOffsetA_0_0_26_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_26_0), wgK );
  int64_t globalReadOffsetA_0_0_27_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_27_0), wgK );
  int64_t globalReadOffsetA_0_0_28_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_28_0), wgK );
  int64_t globalReadOffsetA_0_0_29_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_29_0), wgK );
  int64_t globalReadOffsetA_0_0_30_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_30_0), wgK );
  int64_t globalReadOffsetA_0_0_31_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_31_0), wgK );
  int64_t globalReadOffsetA_0_0_32_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_32_0), wgK );
  int64_t globalReadOffsetA_0_0_33_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_33_0), wgK );
  int64_t globalReadOffsetA_0_0_34_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_34_0), wgK );
  int64_t globalReadOffsetA_0_0_35_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_35_0), wgK );
  int64_t globalReadOffsetA_0_0_36_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_36_0), wgK );
  int64_t globalReadOffsetA_0_0_37_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_37_0), wgK );
  int64_t globalReadOffsetA_0_0_38_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_38_0), wgK );
  int64_t globalReadOffsetA_0_0_39_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_39_0), wgK );
  int64_t globalReadOffsetA_0_0_40_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_40_0), wgK );
  int64_t globalReadOffsetA_0_0_41_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_41_0), wgK );
  int64_t globalReadOffsetA_0_0_42_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_42_0), wgK );
  int64_t globalReadOffsetA_0_0_43_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_43_0), wgK );
  int64_t globalReadOffsetA_0_0_44_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_44_0), wgK );
  int64_t globalReadOffsetA_0_0_45_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_45_0), wgK );
  int64_t globalReadOffsetA_0_0_46_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_46_0), wgK );
  int64_t globalReadOffsetA_0_0_47_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_47_0), wgK );
  int64_t globalReadOffsetA_0_0_48_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_48_0), wgK );
  int64_t globalReadOffsetA_0_0_49_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_49_0), wgK );
  int64_t globalReadOffsetA_0_0_50_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_50_0), wgK );
  int64_t globalReadOffsetA_0_0_51_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_51_0), wgK );
  int64_t globalReadOffsetA_0_0_52_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_52_0), wgK );
  int64_t globalReadOffsetA_0_0_53_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_53_0), wgK );
  int64_t globalReadOffsetA_0_0_54_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_54_0), wgK );
  int64_t globalReadOffsetA_0_0_55_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_55_0), wgK );
  int64_t globalReadOffsetA_0_0_56_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_56_0), wgK );
  int64_t globalReadOffsetA_0_0_57_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_57_0), wgK );
  int64_t globalReadOffsetA_0_0_58_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_58_0), wgK );
  int64_t globalReadOffsetA_0_0_59_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_59_0), wgK );
  int64_t globalReadOffsetA_0_0_60_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_60_0), wgK );
  int64_t globalReadOffsetA_0_0_61_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_61_0), wgK );
  int64_t globalReadOffsetA_0_0_62_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_62_0), wgK );
  int64_t globalReadOffsetA_0_0_63_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_63_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;
  DATA_TYPE const *globalReadA_0_0_16_0 = A + globalReadOffsetA_0_0_16_0;
  DATA_TYPE const *globalReadA_0_0_17_0 = A + globalReadOffsetA_0_0_17_0;
  DATA_TYPE const *globalReadA_0_0_18_0 = A + globalReadOffsetA_0_0_18_0;
  DATA_TYPE const *globalReadA_0_0_19_0 = A + globalReadOffsetA_0_0_19_0;
  DATA_TYPE const *globalReadA_0_0_20_0 = A + globalReadOffsetA_0_0_20_0;
  DATA_TYPE const *globalReadA_0_0_21_0 = A + globalReadOffsetA_0_0_21_0;
  DATA_TYPE const *globalReadA_0_0_22_0 = A + globalReadOffsetA_0_0_22_0;
  DATA_TYPE const *globalReadA_0_0_23_0 = A + globalReadOffsetA_0_0_23_0;
  DATA_TYPE const *globalReadA_0_0_24_0 = A + globalReadOffsetA_0_0_24_0;
  DATA_TYPE const *globalReadA_0_0_25_0 = A + globalReadOffsetA_0_0_25_0;
  DATA_TYPE const *globalReadA_0_0_26_0 = A + globalReadOffsetA_0_0_26_0;
  DATA_TYPE const *globalReadA_0_0_27_0 = A + globalReadOffsetA_0_0_27_0;
  DATA_TYPE const *globalReadA_0_0_28_0 = A + globalReadOffsetA_0_0_28_0;
  DATA_TYPE const *globalReadA_0_0_29_0 = A + globalReadOffsetA_0_0_29_0;
  DATA_TYPE const *globalReadA_0_0_30_0 = A + globalReadOffsetA_0_0_30_0;
  DATA_TYPE const *globalReadA_0_0_31_0 = A + globalReadOffsetA_0_0_31_0;
  DATA_TYPE const *globalReadA_0_0_32_0 = A + globalReadOffsetA_0_0_32_0;
  DATA_TYPE const *globalReadA_0_0_33_0 = A + globalReadOffsetA_0_0_33_0;
  DATA_TYPE const *globalReadA_0_0_34_0 = A + globalReadOffsetA_0_0_34_0;
  DATA_TYPE const *globalReadA_0_0_35_0 = A + globalReadOffsetA_0_0_35_0;
  DATA_TYPE const *globalReadA_0_0_36_0 = A + globalReadOffsetA_0_0_36_0;
  DATA_TYPE const *globalReadA_0_0_37_0 = A + globalReadOffsetA_0_0_37_0;
  DATA_TYPE const *globalReadA_0_0_38_0 = A + globalReadOffsetA_0_0_38_0;
  DATA_TYPE const *globalReadA_0_0_39_0 = A + globalReadOffsetA_0_0_39_0;
  DATA_TYPE const *globalReadA_0_0_40_0 = A + globalReadOffsetA_0_0_40_0;
  DATA_TYPE const *globalReadA_0_0_41_0 = A + globalReadOffsetA_0_0_41_0;
  DATA_TYPE const *globalReadA_0_0_42_0 = A + globalReadOffsetA_0_0_42_0;
  DATA_TYPE const *globalReadA_0_0_43_0 = A + globalReadOffsetA_0_0_43_0;
  DATA_TYPE const *globalReadA_0_0_44_0 = A + globalReadOffsetA_0_0_44_0;
  DATA_TYPE const *globalReadA_0_0_45_0 = A + globalReadOffsetA_0_0_45_0;
  DATA_TYPE const *globalReadA_0_0_46_0 = A + globalReadOffsetA_0_0_46_0;
  DATA_TYPE const *globalReadA_0_0_47_0 = A + globalReadOffsetA_0_0_47_0;
  DATA_TYPE const *globalReadA_0_0_48_0 = A + globalReadOffsetA_0_0_48_0;
  DATA_TYPE const *globalReadA_0_0_49_0 = A + globalReadOffsetA_0_0_49_0;
  DATA_TYPE const *globalReadA_0_0_50_0 = A + globalReadOffsetA_0_0_50_0;
  DATA_TYPE const *globalReadA_0_0_51_0 = A + globalReadOffsetA_0_0_51_0;
  DATA_TYPE const *globalReadA_0_0_52_0 = A + globalReadOffsetA_0_0_52_0;
  DATA_TYPE const *globalReadA_0_0_53_0 = A + globalReadOffsetA_0_0_53_0;
  DATA_TYPE const *globalReadA_0_0_54_0 = A + globalReadOffsetA_0_0_54_0;
  DATA_TYPE const *globalReadA_0_0_55_0 = A + globalReadOffsetA_0_0_55_0;
  DATA_TYPE const *globalReadA_0_0_56_0 = A + globalReadOffsetA_0_0_56_0;
  DATA_TYPE const *globalReadA_0_0_57_0 = A + globalReadOffsetA_0_0_57_0;
  DATA_TYPE const *globalReadA_0_0_58_0 = A + globalReadOffsetA_0_0_58_0;
  DATA_TYPE const *globalReadA_0_0_59_0 = A + globalReadOffsetA_0_0_59_0;
  DATA_TYPE const *globalReadA_0_0_60_0 = A + globalReadOffsetA_0_0_60_0;
  DATA_TYPE const *globalReadA_0_0_61_0 = A + globalReadOffsetA_0_0_61_0;
  DATA_TYPE const *globalReadA_0_0_62_0 = A + globalReadOffsetA_0_0_62_0;
  DATA_TYPE const *globalReadA_0_0_63_0 = A + globalReadOffsetA_0_0_63_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 8*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 9*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 10*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 11*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 12*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 13*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 14*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 15*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_16_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 16*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_17_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 17*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_18_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 18*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_19_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 19*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_20_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 20*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_21_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 21*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_22_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 22*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_23_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 23*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_24_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 24*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_25_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 25*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_26_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 26*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_27_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 27*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_28_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 28*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_29_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 29*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_30_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 30*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_31_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 31*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_32_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 32*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_33_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 33*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_34_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 34*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_35_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 35*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_36_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 36*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_37_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 37*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_38_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 38*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_39_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 39*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_40_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 40*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_41_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 41*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_42_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 42*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_43_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 43*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_44_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 44*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_45_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 45*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_46_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 46*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_47_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 47*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_48_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 48*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_49_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 49*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_50_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 50*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_51_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 51*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_52_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 52*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_53_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 53*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_54_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 54*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_55_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 55*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_56_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 56*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_57_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 57*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_58_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 58*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_59_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 59*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_60_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 60*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_61_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 61*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_62_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 62*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_63_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 63*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_0_0_15_0;
  DATA_TYPE *localWriteA_0_0_16_0;
  DATA_TYPE *localWriteA_0_0_17_0;
  DATA_TYPE *localWriteA_0_0_18_0;
  DATA_TYPE *localWriteA_0_0_19_0;
  DATA_TYPE *localWriteA_0_0_20_0;
  DATA_TYPE *localWriteA_0_0_21_0;
  DATA_TYPE *localWriteA_0_0_22_0;
  DATA_TYPE *localWriteA_0_0_23_0;
  DATA_TYPE *localWriteA_0_0_24_0;
  DATA_TYPE *localWriteA_0_0_25_0;
  DATA_TYPE *localWriteA_0_0_26_0;
  DATA_TYPE *localWriteA_0_0_27_0;
  DATA_TYPE *localWriteA_0_0_28_0;
  DATA_TYPE *localWriteA_0_0_29_0;
  DATA_TYPE *localWriteA_0_0_30_0;
  DATA_TYPE *localWriteA_0_0_31_0;
  DATA_TYPE *localWriteA_0_0_32_0;
  DATA_TYPE *localWriteA_0_0_33_0;
  DATA_TYPE *localWriteA_0_0_34_0;
  DATA_TYPE *localWriteA_0_0_35_0;
  DATA_TYPE *localWriteA_0_0_36_0;
  DATA_TYPE *localWriteA_0_0_37_0;
  DATA_TYPE *localWriteA_0_0_38_0;
  DATA_TYPE *localWriteA_0_0_39_0;
  DATA_TYPE *localWriteA_0_0_40_0;
  DATA_TYPE *localWriteA_0_0_41_0;
  DATA_TYPE *localWriteA_0_0_42_0;
  DATA_TYPE *localWriteA_0_0_43_0;
  DATA_TYPE *localWriteA_0_0_44_0;
  DATA_TYPE *localWriteA_0_0_45_0;
  DATA_TYPE *localWriteA_0_0_46_0;
  DATA_TYPE *localWriteA_0_0_47_0;
  DATA_TYPE *localWriteA_0_0_48_0;
  DATA_TYPE *localWriteA_0_0_49_0;
  DATA_TYPE *localWriteA_0_0_50_0;
  DATA_TYPE *localWriteA_0_0_51_0;
  DATA_TYPE *localWriteA_0_0_52_0;
  DATA_TYPE *localWriteA_0_0_53_0;
  DATA_TYPE *localWriteA_0_0_54_0;
  DATA_TYPE *localWriteA_0_0_55_0;
  DATA_TYPE *localWriteA_0_0_56_0;
  DATA_TYPE *localWriteA_0_0_57_0;
  DATA_TYPE *localWriteA_0_0_58_0;
  DATA_TYPE *localWriteA_0_0_59_0;
  DATA_TYPE *localWriteA_0_0_60_0;
  DATA_TYPE *localWriteA_0_0_61_0;
  DATA_TYPE *localWriteA_0_0_62_0;
  DATA_TYPE *localWriteA_0_0_63_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);
  localWriteA_0_0_16_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_16_0);
  localWriteA_0_0_17_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_17_0);
  localWriteA_0_0_18_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_18_0);
  localWriteA_0_0_19_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_19_0);
  localWriteA_0_0_20_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_20_0);
  localWriteA_0_0_21_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_21_0);
  localWriteA_0_0_22_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_22_0);
  localWriteA_0_0_23_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_23_0);
  localWriteA_0_0_24_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_24_0);
  localWriteA_0_0_25_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_25_0);
  localWriteA_0_0_26_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_26_0);
  localWriteA_0_0_27_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_27_0);
  localWriteA_0_0_28_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_28_0);
  localWriteA_0_0_29_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_29_0);
  localWriteA_0_0_30_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_30_0);
  localWriteA_0_0_31_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_31_0);
  localWriteA_0_0_32_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_32_0);
  localWriteA_0_0_33_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_33_0);
  localWriteA_0_0_34_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_34_0);
  localWriteA_0_0_35_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_35_0);
  localWriteA_0_0_36_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_36_0);
  localWriteA_0_0_37_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_37_0);
  localWriteA_0_0_38_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_38_0);
  localWriteA_0_0_39_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_39_0);
  localWriteA_0_0_40_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_40_0);
  localWriteA_0_0_41_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_41_0);
  localWriteA_0_0_42_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_42_0);
  localWriteA_0_0_43_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_43_0);
  localWriteA_0_0_44_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_44_0);
  localWriteA_0_0_45_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_45_0);
  localWriteA_0_0_46_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_46_0);
  localWriteA_0_0_47_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_47_0);
  localWriteA_0_0_48_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_48_0);
  localWriteA_0_0_49_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_49_0);
  localWriteA_0_0_50_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_50_0);
  localWriteA_0_0_51_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_51_0);
  localWriteA_0_0_52_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_52_0);
  localWriteA_0_0_53_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_53_0);
  localWriteA_0_0_54_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_54_0);
  localWriteA_0_0_55_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_55_0);
  localWriteA_0_0_56_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_56_0);
  localWriteA_0_0_57_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_57_0);
  localWriteA_0_0_58_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_58_0);
  localWriteA_0_0_59_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_59_0);
  localWriteA_0_0_60_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_60_0);
  localWriteA_0_0_61_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_61_0);
  localWriteA_0_0_62_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_62_0);
  localWriteA_0_0_63_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_63_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);
    a_0_0_16_0 = *(globalReadA_0_0_16_0 + 0);
    a_0_0_17_0 = *(globalReadA_0_0_17_0 + 0);
    a_0_0_18_0 = *(globalReadA_0_0_18_0 + 0);
    a_0_0_19_0 = *(globalReadA_0_0_19_0 + 0);
    a_0_0_20_0 = *(globalReadA_0_0_20_0 + 0);
    a_0_0_21_0 = *(globalReadA_0_0_21_0 + 0);
    a_0_0_22_0 = *(globalReadA_0_0_22_0 + 0);
    a_0_0_23_0 = *(globalReadA_0_0_23_0 + 0);
    a_0_0_24_0 = *(globalReadA_0_0_24_0 + 0);
    a_0_0_25_0 = *(globalReadA_0_0_25_0 + 0);
    a_0_0_26_0 = *(globalReadA_0_0_26_0 + 0);
    a_0_0_27_0 = *(globalReadA_0_0_27_0 + 0);
    a_0_0_28_0 = *(globalReadA_0_0_28_0 + 0);
    a_0_0_29_0 = *(globalReadA_0_0_29_0 + 0);
    a_0_0_30_0 = *(globalReadA_0_0_30_0 + 0);
    a_0_0_31_0 = *(globalReadA_0_0_31_0 + 0);
    a_0_0_32_0 = *(globalReadA_0_0_32_0 + 0);
    a_0_0_33_0 = *(globalReadA_0_0_33_0 + 0);
    a_0_0_34_0 = *(globalReadA_0_0_34_0 + 0);
    a_0_0_35_0 = *(globalReadA_0_0_35_0 + 0);
    a_0_0_36_0 = *(globalReadA_0_0_36_0 + 0);
    a_0_0_37_0 = *(globalReadA_0_0_37_0 + 0);
    a_0_0_38_0 = *(globalReadA_0_0_38_0 + 0);
    a_0_0_39_0 = *(globalReadA_0_0_39_0 + 0);
    a_0_0_40_0 = *(globalReadA_0_0_40_0 + 0);
    a_0_0_41_0 = *(globalReadA_0_0_41_0 + 0);
    a_0_0_42_0 = *(globalReadA_0_0_42_0 + 0);
    a_0_0_43_0 = *(globalReadA_0_0_43_0 + 0);
    a_0_0_44_0 = *(globalReadA_0_0_44_0 + 0);
    a_0_0_45_0 = *(globalReadA_0_0_45_0 + 0);
    a_0_0_46_0 = *(globalReadA_0_0_46_0 + 0);
    a_0_0_47_0 = *(globalReadA_0_0_47_0 + 0);
    a_0_0_48_0 = *(globalReadA_0_0_48_0 + 0);
    a_0_0_49_0 = *(globalReadA_0_0_49_0 + 0);
    a_0_0_50_0 = *(globalReadA_0_0_50_0 + 0);
    a_0_0_51_0 = *(globalReadA_0_0_51_0 + 0);
    a_0_0_52_0 = *(globalReadA_0_0_52_0 + 0);
    a_0_0_53_0 = *(globalReadA_0_0_53_0 + 0);
    a_0_0_54_0 = *(globalReadA_0_0_54_0 + 0);
    a_0_0_55_0 = *(globalReadA_0_0_55_0 + 0);
    a_0_0_56_0 = *(globalReadA_0_0_56_0 + 0);
    a_0_0_57_0 = *(globalReadA_0_0_57_0 + 0);
    a_0_0_58_0 = *(globalReadA_0_0_58_0 + 0);
    a_0_0_59_0 = *(globalReadA_0_0_59_0 + 0);
    a_0_0_60_0 = *(globalReadA_0_0_60_0 + 0);
    a_0_0_61_0 = *(globalReadA_0_0_61_0 + 0);
    a_0_0_62_0 = *(globalReadA_0_0_62_0 + 0);
    a_0_0_63_0 = *(globalReadA_0_0_63_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    globalReadA_0_0_16_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_16_0) + 1*globalReadIncAL);
    globalReadA_0_0_17_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_17_0) + 1*globalReadIncAL);
    globalReadA_0_0_18_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_18_0) + 1*globalReadIncAL);
    globalReadA_0_0_19_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_19_0) + 1*globalReadIncAL);
    globalReadA_0_0_20_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_20_0) + 1*globalReadIncAL);
    globalReadA_0_0_21_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_21_0) + 1*globalReadIncAL);
    globalReadA_0_0_22_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_22_0) + 1*globalReadIncAL);
    globalReadA_0_0_23_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_23_0) + 1*globalReadIncAL);
    globalReadA_0_0_24_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_24_0) + 1*globalReadIncAL);
    globalReadA_0_0_25_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_25_0) + 1*globalReadIncAL);
    globalReadA_0_0_26_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_26_0) + 1*globalReadIncAL);
    globalReadA_0_0_27_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_27_0) + 1*globalReadIncAL);
    globalReadA_0_0_28_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_28_0) + 1*globalReadIncAL);
    globalReadA_0_0_29_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_29_0) + 1*globalReadIncAL);
    globalReadA_0_0_30_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_30_0) + 1*globalReadIncAL);
    globalReadA_0_0_31_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_31_0) + 1*globalReadIncAL);
    globalReadA_0_0_32_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_32_0) + 1*globalReadIncAL);
    globalReadA_0_0_33_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_33_0) + 1*globalReadIncAL);
    globalReadA_0_0_34_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_34_0) + 1*globalReadIncAL);
    globalReadA_0_0_35_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_35_0) + 1*globalReadIncAL);
    globalReadA_0_0_36_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_36_0) + 1*globalReadIncAL);
    globalReadA_0_0_37_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_37_0) + 1*globalReadIncAL);
    globalReadA_0_0_38_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_38_0) + 1*globalReadIncAL);
    globalReadA_0_0_39_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_39_0) + 1*globalReadIncAL);
    globalReadA_0_0_40_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_40_0) + 1*globalReadIncAL);
    globalReadA_0_0_41_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_41_0) + 1*globalReadIncAL);
    globalReadA_0_0_42_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_42_0) + 1*globalReadIncAL);
    globalReadA_0_0_43_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_43_0) + 1*globalReadIncAL);
    globalReadA_0_0_44_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_44_0) + 1*globalReadIncAL);
    globalReadA_0_0_45_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_45_0) + 1*globalReadIncAL);
    globalReadA_0_0_46_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_46_0) + 1*globalReadIncAL);
    globalReadA_0_0_47_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_47_0) + 1*globalReadIncAL);
    globalReadA_0_0_48_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_48_0) + 1*globalReadIncAL);
    globalReadA_0_0_49_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_49_0) + 1*globalReadIncAL);
    globalReadA_0_0_50_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_50_0) + 1*globalReadIncAL);
    globalReadA_0_0_51_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_51_0) + 1*globalReadIncAL);
    globalReadA_0_0_52_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_52_0) + 1*globalReadIncAL);
    globalReadA_0_0_53_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_53_0) + 1*globalReadIncAL);
    globalReadA_0_0_54_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_54_0) + 1*globalReadIncAL);
    globalReadA_0_0_55_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_55_0) + 1*globalReadIncAL);
    globalReadA_0_0_56_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_56_0) + 1*globalReadIncAL);
    globalReadA_0_0_57_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_57_0) + 1*globalReadIncAL);
    globalReadA_0_0_58_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_58_0) + 1*globalReadIncAL);
    globalReadA_0_0_59_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_59_0) + 1*globalReadIncAL);
    globalReadA_0_0_60_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_60_0) + 1*globalReadIncAL);
    globalReadA_0_0_61_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_61_0) + 1*globalReadIncAL);
    globalReadA_0_0_62_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_62_0) + 1*globalReadIncAL);
    globalReadA_0_0_63_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_63_0) + 1*globalReadIncAL);

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
    *(localWriteA_0_0_16_0 + 0) = a_0_0_16_0;
    *(localWriteA_0_0_17_0 + 0) = a_0_0_17_0;
    *(localWriteA_0_0_18_0 + 0) = a_0_0_18_0;
    *(localWriteA_0_0_19_0 + 0) = a_0_0_19_0;
    *(localWriteA_0_0_20_0 + 0) = a_0_0_20_0;
    *(localWriteA_0_0_21_0 + 0) = a_0_0_21_0;
    *(localWriteA_0_0_22_0 + 0) = a_0_0_22_0;
    *(localWriteA_0_0_23_0 + 0) = a_0_0_23_0;
    *(localWriteA_0_0_24_0 + 0) = a_0_0_24_0;
    *(localWriteA_0_0_25_0 + 0) = a_0_0_25_0;
    *(localWriteA_0_0_26_0 + 0) = a_0_0_26_0;
    *(localWriteA_0_0_27_0 + 0) = a_0_0_27_0;
    *(localWriteA_0_0_28_0 + 0) = a_0_0_28_0;
    *(localWriteA_0_0_29_0 + 0) = a_0_0_29_0;
    *(localWriteA_0_0_30_0 + 0) = a_0_0_30_0;
    *(localWriteA_0_0_31_0 + 0) = a_0_0_31_0;
    *(localWriteA_0_0_32_0 + 0) = a_0_0_32_0;
    *(localWriteA_0_0_33_0 + 0) = a_0_0_33_0;
    *(localWriteA_0_0_34_0 + 0) = a_0_0_34_0;
    *(localWriteA_0_0_35_0 + 0) = a_0_0_35_0;
    *(localWriteA_0_0_36_0 + 0) = a_0_0_36_0;
    *(localWriteA_0_0_37_0 + 0) = a_0_0_37_0;
    *(localWriteA_0_0_38_0 + 0) = a_0_0_38_0;
    *(localWriteA_0_0_39_0 + 0) = a_0_0_39_0;
    *(localWriteA_0_0_40_0 + 0) = a_0_0_40_0;
    *(localWriteA_0_0_41_0 + 0) = a_0_0_41_0;
    *(localWriteA_0_0_42_0 + 0) = a_0_0_42_0;
    *(localWriteA_0_0_43_0 + 0) = a_0_0_43_0;
    *(localWriteA_0_0_44_0 + 0) = a_0_0_44_0;
    *(localWriteA_0_0_45_0 + 0) = a_0_0_45_0;
    *(localWriteA_0_0_46_0 + 0) = a_0_0_46_0;
    *(localWriteA_0_0_47_0 + 0) = a_0_0_47_0;
    *(localWriteA_0_0_48_0 + 0) = a_0_0_48_0;
    *(localWriteA_0_0_49_0 + 0) = a_0_0_49_0;
    *(localWriteA_0_0_50_0 + 0) = a_0_0_50_0;
    *(localWriteA_0_0_51_0 + 0) = a_0_0_51_0;
    *(localWriteA_0_0_52_0 + 0) = a_0_0_52_0;
    *(localWriteA_0_0_53_0 + 0) = a_0_0_53_0;
    *(localWriteA_0_0_54_0 + 0) = a_0_0_54_0;
    *(localWriteA_0_0_55_0 + 0) = a_0_0_55_0;
    *(localWriteA_0_0_56_0 + 0) = a_0_0_56_0;
    *(localWriteA_0_0_57_0 + 0) = a_0_0_57_0;
    *(localWriteA_0_0_58_0 + 0) = a_0_0_58_0;
    *(localWriteA_0_0_59_0 + 0) = a_0_0_59_0;
    *(localWriteA_0_0_60_0 + 0) = a_0_0_60_0;
    *(localWriteA_0_0_61_0 + 0) = a_0_0_61_0;
    *(localWriteA_0_0_62_0 + 0) = a_0_0_62_0;
    *(localWriteA_0_0_63_0 + 0) = a_0_0_63_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 15 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 16 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 17 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 18 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 19 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 20 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 21 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 22 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 23 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 24 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 25 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 26 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 27 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 28 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 29 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 30 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 31 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 32 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 33 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 34 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 35 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 36 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 37 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 38 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 39 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 40 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 41 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 42 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 43 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 44 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 45 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 46 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 47 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 48 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 49 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 50 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 51 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 52 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 53 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 54 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 55 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 56 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 57 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 58 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 59 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 60 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 61 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 62 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_1x1


    /* iter 63 */

    MAC_1x1_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_0_8_0 = ( globalReadOffsetAL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_0_0_9_0 = ( globalReadOffsetAL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_0_0_10_0 = ( globalReadOffsetAL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_0_0_11_0 = ( globalReadOffsetAL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_0_0_12_0 = ( globalReadOffsetAL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_0_0_13_0 = ( globalReadOffsetAL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_0_0_14_0 = ( globalReadOffsetAL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_0_0_15_0 = ( globalReadOffsetAL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);
  a_0_0_16_0 = ( globalReadOffsetAL_16_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_16_0 + 0);
  a_0_0_17_0 = ( globalReadOffsetAL_17_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_17_0 + 0);
  a_0_0_18_0 = ( globalReadOffsetAL_18_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_18_0 + 0);
  a_0_0_19_0 = ( globalReadOffsetAL_19_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_19_0 + 0);
  a_0_0_20_0 = ( globalReadOffsetAL_20_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_20_0 + 0);
  a_0_0_21_0 = ( globalReadOffsetAL_21_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_21_0 + 0);
  a_0_0_22_0 = ( globalReadOffsetAL_22_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_22_0 + 0);
  a_0_0_23_0 = ( globalReadOffsetAL_23_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_23_0 + 0);
  a_0_0_24_0 = ( globalReadOffsetAL_24_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_24_0 + 0);
  a_0_0_25_0 = ( globalReadOffsetAL_25_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_25_0 + 0);
  a_0_0_26_0 = ( globalReadOffsetAL_26_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_26_0 + 0);
  a_0_0_27_0 = ( globalReadOffsetAL_27_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_27_0 + 0);
  a_0_0_28_0 = ( globalReadOffsetAL_28_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_28_0 + 0);
  a_0_0_29_0 = ( globalReadOffsetAL_29_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_29_0 + 0);
  a_0_0_30_0 = ( globalReadOffsetAL_30_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_30_0 + 0);
  a_0_0_31_0 = ( globalReadOffsetAL_31_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_31_0 + 0);
  a_0_0_32_0 = ( globalReadOffsetAL_32_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_32_0 + 0);
  a_0_0_33_0 = ( globalReadOffsetAL_33_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_33_0 + 0);
  a_0_0_34_0 = ( globalReadOffsetAL_34_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_34_0 + 0);
  a_0_0_35_0 = ( globalReadOffsetAL_35_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_35_0 + 0);
  a_0_0_36_0 = ( globalReadOffsetAL_36_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_36_0 + 0);
  a_0_0_37_0 = ( globalReadOffsetAL_37_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_37_0 + 0);
  a_0_0_38_0 = ( globalReadOffsetAL_38_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_38_0 + 0);
  a_0_0_39_0 = ( globalReadOffsetAL_39_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_39_0 + 0);
  a_0_0_40_0 = ( globalReadOffsetAL_40_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_40_0 + 0);
  a_0_0_41_0 = ( globalReadOffsetAL_41_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_41_0 + 0);
  a_0_0_42_0 = ( globalReadOffsetAL_42_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_42_0 + 0);
  a_0_0_43_0 = ( globalReadOffsetAL_43_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_43_0 + 0);
  a_0_0_44_0 = ( globalReadOffsetAL_44_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_44_0 + 0);
  a_0_0_45_0 = ( globalReadOffsetAL_45_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_45_0 + 0);
  a_0_0_46_0 = ( globalReadOffsetAL_46_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_46_0 + 0);
  a_0_0_47_0 = ( globalReadOffsetAL_47_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_47_0 + 0);
  a_0_0_48_0 = ( globalReadOffsetAL_48_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_48_0 + 0);
  a_0_0_49_0 = ( globalReadOffsetAL_49_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_49_0 + 0);
  a_0_0_50_0 = ( globalReadOffsetAL_50_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_50_0 + 0);
  a_0_0_51_0 = ( globalReadOffsetAL_51_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_51_0 + 0);
  a_0_0_52_0 = ( globalReadOffsetAL_52_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_52_0 + 0);
  a_0_0_53_0 = ( globalReadOffsetAL_53_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_53_0 + 0);
  a_0_0_54_0 = ( globalReadOffsetAL_54_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_54_0 + 0);
  a_0_0_55_0 = ( globalReadOffsetAL_55_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_55_0 + 0);
  a_0_0_56_0 = ( globalReadOffsetAL_56_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_56_0 + 0);
  a_0_0_57_0 = ( globalReadOffsetAL_57_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_57_0 + 0);
  a_0_0_58_0 = ( globalReadOffsetAL_58_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_58_0 + 0);
  a_0_0_59_0 = ( globalReadOffsetAL_59_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_59_0 + 0);
  a_0_0_60_0 = ( globalReadOffsetAL_60_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_60_0 + 0);
  a_0_0_61_0 = ( globalReadOffsetAL_61_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_61_0 + 0);
  a_0_0_62_0 = ( globalReadOffsetAL_62_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_62_0 + 0);
  a_0_0_63_0 = ( globalReadOffsetAL_63_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_63_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);
  localWriteA_0_0_16_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_16_0);
  localWriteA_0_0_17_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_17_0);
  localWriteA_0_0_18_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_18_0);
  localWriteA_0_0_19_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_19_0);
  localWriteA_0_0_20_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_20_0);
  localWriteA_0_0_21_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_21_0);
  localWriteA_0_0_22_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_22_0);
  localWriteA_0_0_23_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_23_0);
  localWriteA_0_0_24_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_24_0);
  localWriteA_0_0_25_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_25_0);
  localWriteA_0_0_26_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_26_0);
  localWriteA_0_0_27_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_27_0);
  localWriteA_0_0_28_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_28_0);
  localWriteA_0_0_29_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_29_0);
  localWriteA_0_0_30_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_30_0);
  localWriteA_0_0_31_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_31_0);
  localWriteA_0_0_32_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_32_0);
  localWriteA_0_0_33_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_33_0);
  localWriteA_0_0_34_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_34_0);
  localWriteA_0_0_35_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_35_0);
  localWriteA_0_0_36_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_36_0);
  localWriteA_0_0_37_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_37_0);
  localWriteA_0_0_38_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_38_0);
  localWriteA_0_0_39_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_39_0);
  localWriteA_0_0_40_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_40_0);
  localWriteA_0_0_41_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_41_0);
  localWriteA_0_0_42_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_42_0);
  localWriteA_0_0_43_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_43_0);
  localWriteA_0_0_44_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_44_0);
  localWriteA_0_0_45_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_45_0);
  localWriteA_0_0_46_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_46_0);
  localWriteA_0_0_47_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_47_0);
  localWriteA_0_0_48_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_48_0);
  localWriteA_0_0_49_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_49_0);
  localWriteA_0_0_50_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_50_0);
  localWriteA_0_0_51_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_51_0);
  localWriteA_0_0_52_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_52_0);
  localWriteA_0_0_53_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_53_0);
  localWriteA_0_0_54_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_54_0);
  localWriteA_0_0_55_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_55_0);
  localWriteA_0_0_56_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_56_0);
  localWriteA_0_0_57_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_57_0);
  localWriteA_0_0_58_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_58_0);
  localWriteA_0_0_59_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_59_0);
  localWriteA_0_0_60_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_60_0);
  localWriteA_0_0_61_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_61_0);
  localWriteA_0_0_62_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_62_0);
  localWriteA_0_0_63_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_63_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
  *(localWriteA_0_0_16_0 + 0) = a_0_0_16_0;
  *(localWriteA_0_0_17_0 + 0) = a_0_0_17_0;
  *(localWriteA_0_0_18_0 + 0) = a_0_0_18_0;
  *(localWriteA_0_0_19_0 + 0) = a_0_0_19_0;
  *(localWriteA_0_0_20_0 + 0) = a_0_0_20_0;
  *(localWriteA_0_0_21_0 + 0) = a_0_0_21_0;
  *(localWriteA_0_0_22_0 + 0) = a_0_0_22_0;
  *(localWriteA_0_0_23_0 + 0) = a_0_0_23_0;
  *(localWriteA_0_0_24_0 + 0) = a_0_0_24_0;
  *(localWriteA_0_0_25_0 + 0) = a_0_0_25_0;
  *(localWriteA_0_0_26_0 + 0) = a_0_0_26_0;
  *(localWriteA_0_0_27_0 + 0) = a_0_0_27_0;
  *(localWriteA_0_0_28_0 + 0) = a_0_0_28_0;
  *(localWriteA_0_0_29_0 + 0) = a_0_0_29_0;
  *(localWriteA_0_0_30_0 + 0) = a_0_0_30_0;
  *(localWriteA_0_0_31_0 + 0) = a_0_0_31_0;
  *(localWriteA_0_0_32_0 + 0) = a_0_0_32_0;
  *(localWriteA_0_0_33_0 + 0) = a_0_0_33_0;
  *(localWriteA_0_0_34_0 + 0) = a_0_0_34_0;
  *(localWriteA_0_0_35_0 + 0) = a_0_0_35_0;
  *(localWriteA_0_0_36_0 + 0) = a_0_0_36_0;
  *(localWriteA_0_0_37_0 + 0) = a_0_0_37_0;
  *(localWriteA_0_0_38_0 + 0) = a_0_0_38_0;
  *(localWriteA_0_0_39_0 + 0) = a_0_0_39_0;
  *(localWriteA_0_0_40_0 + 0) = a_0_0_40_0;
  *(localWriteA_0_0_41_0 + 0) = a_0_0_41_0;
  *(localWriteA_0_0_42_0 + 0) = a_0_0_42_0;
  *(localWriteA_0_0_43_0 + 0) = a_0_0_43_0;
  *(localWriteA_0_0_44_0 + 0) = a_0_0_44_0;
  *(localWriteA_0_0_45_0 + 0) = a_0_0_45_0;
  *(localWriteA_0_0_46_0 + 0) = a_0_0_46_0;
  *(localWriteA_0_0_47_0 + 0) = a_0_0_47_0;
  *(localWriteA_0_0_48_0 + 0) = a_0_0_48_0;
  *(localWriteA_0_0_49_0 + 0) = a_0_0_49_0;
  *(localWriteA_0_0_50_0 + 0) = a_0_0_50_0;
  *(localWriteA_0_0_51_0 + 0) = a_0_0_51_0;
  *(localWriteA_0_0_52_0 + 0) = a_0_0_52_0;
  *(localWriteA_0_0_53_0 + 0) = a_0_0_53_0;
  *(localWriteA_0_0_54_0 + 0) = a_0_0_54_0;
  *(localWriteA_0_0_55_0 + 0) = a_0_0_55_0;
  *(localWriteA_0_0_56_0 + 0) = a_0_0_56_0;
  *(localWriteA_0_0_57_0 + 0) = a_0_0_57_0;
  *(localWriteA_0_0_58_0 + 0) = a_0_0_58_0;
  *(localWriteA_0_0_59_0 + 0) = a_0_0_59_0;
  *(localWriteA_0_0_60_0 + 0) = a_0_0_60_0;
  *(localWriteA_0_0_61_0 + 0) = a_0_0_61_0;
  *(localWriteA_0_0_62_0 + 0) = a_0_0_62_0;
  *(localWriteA_0_0_63_0 + 0) = a_0_0_63_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_1x1

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_1x1
#undef MAC_1x1_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 64
#define SG1J 1
#define TT0I 1
#define TT1J 1
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 64
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 64
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 4096
#define LDS_NUM_ELEMENTS 4160

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 1x1 micro-tile */
#define MAC_1x1\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \

#define MAC_1x1_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x1x64_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU0_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT1_1_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG64_1_1_WGM1(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE a_0_0_16_0;
  DATA_TYPE a_0_0_17_0;
  DATA_TYPE a_0_0_18_0;
  DATA_TYPE a_0_0_19_0;
  DATA_TYPE a_0_0_20_0;
  DATA_TYPE a_0_0_21_0;
  DATA_TYPE a_0_0_22_0;
  DATA_TYPE a_0_0_23_0;
  DATA_TYPE a_0_0_24_0;
  DATA_TYPE a_0_0_25_0;
  DATA_TYPE a_0_0_26_0;
  DATA_TYPE a_0_0_27_0;
  DATA_TYPE a_0_0_28_0;
  DATA_TYPE a_0_0_29_0;
  DATA_TYPE a_0_0_30_0;
  DATA_TYPE a_0_0_31_0;
  DATA_TYPE a_0_0_32_0;
  DATA_TYPE a_0_0_33_0;
  DATA_TYPE a_0_0_34_0;
  DATA_TYPE a_0_0_35_0;
  DATA_TYPE a_0_0_36_0;
  DATA_TYPE a_0_0_37_0;
  DATA_TYPE a_0_0_38_0;
  DATA_TYPE a_0_0_39_0;
  DATA_TYPE a_0_0_40_0;
  DATA_TYPE a_0_0_41_0;
  DATA_TYPE a_0_0_42_0;
  DATA_TYPE a_0_0_43_0;
  DATA_TYPE a_0_0_44_0;
  DATA_TYPE a_0_0_45_0;
  DATA_TYPE a_0_0_46_0;
  DATA_TYPE a_0_0_47_0;
  DATA_TYPE a_0_0_48_0;
  DATA_TYPE a_0_0_49_0;
  DATA_TYPE a_0_0_50_0;
  DATA_TYPE a_0_0_51_0;
  DATA_TYPE a_0_0_52_0;
  DATA_TYPE a_0_0_53_0;
  DATA_TYPE a_0_0_54_0;
  DATA_TYPE a_0_0_55_0;
  DATA_TYPE a_0_0_56_0;
  DATA_TYPE a_0_0_57_0;
  DATA_TYPE a_0_0_58_0;
  DATA_TYPE a_0_0_59_0;
  DATA_TYPE a_0_0_60_0;
  DATA_TYPE a_0_0_61_0;
  DATA_TYPE a_0_0_62_0;
  DATA_TYPE a_0_0_63_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;
  unsigned int globalReadOffsetAL_8_0 = globalReadOffsetAL + 0 + 8*LSPA;
  unsigned int globalReadOffsetAL_9_0 = globalReadOffsetAL + 0 + 9*LSPA;
  unsigned int globalReadOffsetAL_10_0 = globalReadOffsetAL + 0 + 10*LSPA;
  unsigned int globalReadOffsetAL_11_0 = globalReadOffsetAL + 0 + 11*LSPA;
  unsigned int globalReadOffsetAL_12_0 = globalReadOffsetAL + 0 + 12*LSPA;
  unsigned int globalReadOffsetAL_13_0 = globalReadOffsetAL + 0 + 13*LSPA;
  unsigned int globalReadOffsetAL_14_0 = globalReadOffsetAL + 0 + 14*LSPA;
  unsigned int globalReadOffsetAL_15_0 = globalReadOffsetAL + 0 + 15*LSPA;
  unsigned int globalReadOffsetAL_16_0 = globalReadOffsetAL + 0 + 16*LSPA;
  unsigned int globalReadOffsetAL_17_0 = globalReadOffsetAL + 0 + 17*LSPA;
  unsigned int globalReadOffsetAL_18_0 = globalReadOffsetAL + 0 + 18*LSPA;
  unsigned int globalReadOffsetAL_19_0 = globalReadOffsetAL + 0 + 19*LSPA;
  unsigned int globalReadOffsetAL_20_0 = globalReadOffsetAL + 0 + 20*LSPA;
  unsigned int globalReadOffsetAL_21_0 = globalReadOffsetAL + 0 + 21*LSPA;
  unsigned int globalReadOffsetAL_22_0 = globalReadOffsetAL + 0 + 22*LSPA;
  unsigned int globalReadOffsetAL_23_0 = globalReadOffsetAL + 0 + 23*LSPA;
  unsigned int globalReadOffsetAL_24_0 = globalReadOffsetAL + 0 + 24*LSPA;
  unsigned int globalReadOffsetAL_25_0 = globalReadOffsetAL + 0 + 25*LSPA;
  unsigned int globalReadOffsetAL_26_0 = globalReadOffsetAL + 0 + 26*LSPA;
  unsigned int globalReadOffsetAL_27_0 = globalReadOffsetAL + 0 + 27*LSPA;
  unsigned int globalReadOffsetAL_28_0 = globalReadOffsetAL + 0 + 28*LSPA;
  unsigned int globalReadOffsetAL_29_0 = globalReadOffsetAL + 0 + 29*LSPA;
  unsigned int globalReadOffsetAL_30_0 = globalReadOffsetAL + 0 + 30*LSPA;
  unsigned int globalReadOffsetAL_31_0 = globalReadOffsetAL + 0 + 31*LSPA;
  unsigned int globalReadOffsetAL_32_0 = globalReadOffsetAL + 0 + 32*LSPA;
  unsigned int globalReadOffsetAL_33_0 = globalReadOffsetAL + 0 + 33*LSPA;
  unsigned int globalReadOffsetAL_34_0 = globalReadOffsetAL + 0 + 34*LSPA;
  unsigned int globalReadOffsetAL_35_0 = globalReadOffsetAL + 0 + 35*LSPA;
  unsigned int globalReadOffsetAL_36_0 = globalReadOffsetAL + 0 + 36*LSPA;
  unsigned int globalReadOffsetAL_37_0 = globalReadOffsetAL + 0 + 37*LSPA;
  unsigned int globalReadOffsetAL_38_0 = globalReadOffsetAL + 0 + 38*LSPA;
  unsigned int globalReadOffsetAL_39_0 = globalReadOffsetAL + 0 + 39*LSPA;
  unsigned int globalReadOffsetAL_40_0 = globalReadOffsetAL + 0 + 40*LSPA;
  unsigned int globalReadOffsetAL_41_0 = globalReadOffsetAL + 0 + 41*LSPA;
  unsigned int globalReadOffsetAL_42_0 = globalReadOffsetAL + 0 + 42*LSPA;
  unsigned int globalReadOffsetAL_43_0 = globalReadOffsetAL + 0 + 43*LSPA;
  unsigned int globalReadOffsetAL_44_0 = globalReadOffsetAL + 0 + 44*LSPA;
  unsigned int globalReadOffsetAL_45_0 = globalReadOffsetAL + 0 + 45*LSPA;
  unsigned int globalReadOffsetAL_46_0 = globalReadOffsetAL + 0 + 46*LSPA;
  unsigned int globalReadOffsetAL_47_0 = globalReadOffsetAL + 0 + 47*LSPA;
  unsigned int globalReadOffsetAL_48_0 = globalReadOffsetAL + 0 + 48*LSPA;
  unsigned int globalReadOffsetAL_49_0 = globalReadOffsetAL + 0 + 49*LSPA;
  unsigned int globalReadOffsetAL_50_0 = globalReadOffsetAL + 0 + 50*LSPA;
  unsigned int globalReadOffsetAL_51_0 = globalReadOffsetAL + 0 + 51*LSPA;
  unsigned int globalReadOffsetAL_52_0 = globalReadOffsetAL + 0 + 52*LSPA;
  unsigned int globalReadOffsetAL_53_0 = globalReadOffsetAL + 0 + 53*LSPA;
  unsigned int globalReadOffsetAL_54_0 = globalReadOffsetAL + 0 + 54*LSPA;
  unsigned int globalReadOffsetAL_55_0 = globalReadOffsetAL + 0 + 55*LSPA;
  unsigned int globalReadOffsetAL_56_0 = globalReadOffsetAL + 0 + 56*LSPA;
  unsigned int globalReadOffsetAL_57_0 = globalReadOffsetAL + 0 + 57*LSPA;
  unsigned int globalReadOffsetAL_58_0 = globalReadOffsetAL + 0 + 58*LSPA;
  unsigned int globalReadOffsetAL_59_0 = globalReadOffsetAL + 0 + 59*LSPA;
  unsigned int globalReadOffsetAL_60_0 = globalReadOffsetAL + 0 + 60*LSPA;
  unsigned int globalReadOffsetAL_61_0 = globalReadOffsetAL + 0 + 61*LSPA;
  unsigned int globalReadOffsetAL_62_0 = globalReadOffsetAL + 0 + 62*LSPA;
  unsigned int globalReadOffsetAL_63_0 = globalReadOffsetAL + 0 + 63*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_15_0), wgK );
  int64_t globalReadOffsetA_0_0_16_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_16_0), wgK );
  int64_t globalReadOffsetA_0_0_17_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_17_0), wgK );
  int64_t globalReadOffsetA_0_0_18_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_18_0), wgK );
  int64_t globalReadOffsetA_0_0_19_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_19_0), wgK );
  int64_t globalReadOffsetA_0_0_20_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_20_0), wgK );
  int64_t globalReadOffsetA_0_0_21_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_21_0), wgK );
  int64_t globalReadOffsetA_0_0_22_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_22_0), wgK );
  int64_t globalReadOffsetA_0_0_23_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_23_0), wgK );
  int64_t globalReadOffsetA_0_0_24_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_24_0), wgK );
  int64_t globalReadOffsetA_0_0_25_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_25_0), wgK );
  int64_t globalReadOffsetA_0_0_26_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_26_0), wgK );
  int64_t globalReadOffsetA_0_0_27_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_27_0), wgK );
  int64_t globalReadOffsetA_0_0_28_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_28_0), wgK );
  int64_t globalReadOffsetA_0_0_29_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_29_0), wgK );
  int64_t globalReadOffsetA_0_0_30_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_30_0), wgK );
  int64_t globalReadOffsetA_0_0_31_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_31_0), wgK );
  int64_t globalReadOffsetA_0_0_32_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_32_0), wgK );
  int64_t globalReadOffsetA_0_0_33_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_33_0), wgK );
  int64_t globalReadOffsetA_0_0_34_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_34_0), wgK );
  int64_t globalReadOffsetA_0_0_35_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_35_0), wgK );
  int64_t globalReadOffsetA_0_0_36_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_36_0), wgK );
  int64_t globalReadOffsetA_0_0_37_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_37_0), wgK );
  int64_t globalReadOffsetA_0_0_38_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_38_0), wgK );
  int64_t globalReadOffsetA_0_0_39_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_39_0), wgK );
  int64_t globalReadOffsetA_0_0_40_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_40_0), wgK );
  int64_t globalReadOffsetA_0_0_41_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_41_0), wgK );
  int64_t globalReadOffsetA_0_0_42_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_42_0), wgK );
  int64_t globalReadOffsetA_0_0_43_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_43_0), wgK );
  int64_t globalReadOffsetA_0_0_44_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_44_0), wgK );
  int64_t globalReadOffsetA_0_0_45_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_45_0), wgK );
  int64_t globalReadOffsetA_0_0_46_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_46_0), wgK );
  int64_t globalReadOffsetA_0_0_47_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_47_0), wgK );
  int64_t globalReadOffsetA_0_0_48_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_48_0), wgK );
  int64_t globalReadOffsetA_0_0_49_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_49_0), wgK );
  int64_t globalReadOffsetA_0_0_50_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_50_0), wgK );
  int64_t globalReadOffsetA_0_0_51_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_51_0), wgK );
  int64_t globalReadOffsetA_0_0_52_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_52_0), wgK );
  int64_t globalReadOffsetA_0_0_53_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_53_0), wgK );
  int64_t globalReadOffsetA_0_0_54_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_54_0), wgK );
  int64_t globalReadOffsetA_0_0_55_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_55_0), wgK );
  int64_t globalReadOffsetA_0_0_56_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_56_0), wgK );
  int64_t globalReadOffsetA_0_0_57_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_57_0), wgK );
  int64_t globalReadOffsetA_0_0_58_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_58_0), wgK );
  int64_t globalReadOffsetA_0_0_59_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_59_0), wgK );
  int64_t globalReadOffsetA_0_0_60_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_60_0), wgK );
  int64_t globalReadOffsetA_0_0_61_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_61_0), wgK );
  int64_t globalReadOffsetA_0_0_62_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_62_0), wgK );
  int64_t globalReadOffsetA_0_0_63_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_63_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;
  DATA_TYPE const *globalReadA_0_0_16_0 = A + globalReadOffsetA_0_0_16_0;
  DATA_TYPE const *globalReadA_0_0_17_0 = A + globalReadOffsetA_0_0_17_0;
  DATA_TYPE const *globalReadA_0_0_18_0 = A + globalReadOffsetA_0_0_18_0;
  DATA_TYPE const *globalReadA_0_0_19_0 = A + globalReadOffsetA_0_0_19_0;
  DATA_TYPE const *globalReadA_0_0_20_0 = A + globalReadOffsetA_0_0_20_0;
  DATA_TYPE const *globalReadA_0_0_21_0 = A + globalReadOffsetA_0_0_21_0;
  DATA_TYPE const *globalReadA_0_0_22_0 = A + globalReadOffsetA_0_0_22_0;
  DATA_TYPE const *globalReadA_0_0_23_0 = A + globalReadOffsetA_0_0_23_0;
  DATA_TYPE const *globalReadA_0_0_24_0 = A + globalReadOffsetA_0_0_24_0;
  DATA_TYPE const *globalReadA_0_0_25_0 = A + globalReadOffsetA_0_0_25_0;
  DATA_TYPE const *globalReadA_0_0_26_0 = A + globalReadOffsetA_0_0_26_0;
  DATA_TYPE const *globalReadA_0_0_27_0 = A + globalReadOffsetA_0_0_27_0;
  DATA_TYPE const *globalReadA_0_0_28_0 = A + globalReadOffsetA_0_0_28_0;
  DATA_TYPE const *globalReadA_0_0_29_0 = A + globalReadOffsetA_0_0_29_0;
  DATA_TYPE const *globalReadA_0_0_30_0 = A + globalReadOffsetA_0_0_30_0;
  DATA_TYPE const *globalReadA_0_0_31_0 = A + globalReadOffsetA_0_0_31_0;
  DATA_TYPE const *globalReadA_0_0_32_0 = A + globalReadOffsetA_0_0_32_0;
  DATA_TYPE const *globalReadA_0_0_33_0 = A + globalReadOffsetA_0_0_33_0;
  DATA_TYPE const *globalReadA_0_0_34_0 = A + globalReadOffsetA_0_0_34_0;
  DATA_TYPE const *globalReadA_0_0_35_0 = A + globalReadOffsetA_0_0_35_0;
  DATA_TYPE const *globalReadA_0_0_36_0 = A + globalReadOffsetA_0_0_36_0;
  DATA_TYPE const *globalReadA_0_0_37_0 = A + globalReadOffsetA_0_0_37_0;
  DATA_TYPE const *globalReadA_0_0_38_0 = A + globalReadOffsetA_0_0_38_0;
  DATA_TYPE const *globalReadA_0_0_39_0 = A + globalReadOffsetA_0_0_39_0;
  DATA_TYPE const *globalReadA_0_0_40_0 = A + globalReadOffsetA_0_0_40_0;
  DATA_TYPE const *globalReadA_0_0_41_0 = A + globalReadOffsetA_0_0_41_0;
  DATA_TYPE const *globalReadA_0_0_42_0 = A + globalReadOffsetA_0_0_42_0;
  DATA_TYPE const *globalReadA_0_0_43_0 = A + globalReadOffsetA_0_0_43_0;
  DATA_TYPE const *globalReadA_0_0_44_0 = A + globalReadOffsetA_0_0_44_0;
  DATA_TYPE const *globalReadA_0_0_45_0 = A + globalReadOffsetA_0_0_45_0;
  DATA_TYPE const *globalReadA_0_0_46_0 = A + globalReadOffsetA_0_0_46_0;
  DATA_TYPE const *globalReadA_0_0_47_0 = A + globalReadOffsetA_0_0_47_0;
  DATA_TYPE const *globalReadA_0_0_48_0 = A + globalReadOffsetA_0_0_48_0;
  DATA_TYPE const *globalReadA_0_0_49_0 = A + globalReadOffsetA_0_0_49_0;
  DATA_TYPE const *globalReadA_0_0_50_0 = A + globalReadOffsetA_0_0_50_0;
  DATA_TYPE const *globalReadA_0_0_51_0 = A + globalReadOffsetA_0_0_51_0;
  DATA_TYPE const *globalReadA_0_0_52_0 = A + globalReadOffsetA_0_0_52_0;
  DATA_TYPE const *globalReadA_0_0_53_0 = A + globalReadOffsetA_0_0_53_0;
  DATA_TYPE const *globalReadA_0_0_54_0 = A + globalReadOffsetA_0_0_54_0;
  DATA_TYPE const *globalReadA_0_0_55_0 = A + globalReadOffsetA_0_0_55_0;
  DATA_TYPE const *globalReadA_0_0_56_0 = A + globalReadOffsetA_0_0_56_0;
  DATA_TYPE const *globalReadA_0_0_57_0 = A + globalReadOffsetA_0_0_57_0;
  DATA_TYPE const *globalReadA_0_0_58_0 = A + globalReadOffsetA_0_0_58_0;
  DATA_TYPE const *globalReadA_0_0_59_0 = A + globalReadOffsetA_0_0_59_0;
  DATA_TYPE const *globalReadA_0_0_60_0 = A + globalReadOffsetA_0_0_60_0;
  DATA_TYPE const *globalReadA_0_0_61_0 = A + globalReadOffsetA_0_0_61_0;
  DATA_TYPE const *globalReadA_0_0_62_0 = A + globalReadOffsetA_0_0_62_0;
  DATA_TYPE const *globalReadA_0_0_63_0 = A + globalReadOffsetA_0_0_63_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 8*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 9*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 10*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 11*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 12*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 13*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 14*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 15*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_16_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 16*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_17_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 17*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_18_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 18*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_19_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 19*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_20_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 20*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_21_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 21*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_22_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 22*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_23_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 23*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_24_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 24*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_25_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 25*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_26_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 26*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_27_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 27*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_28_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 28*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_29_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 29*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_30_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 30*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_31_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 31*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_32_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 32*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_33_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 33*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_34_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 34*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_35_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 35*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_36_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 36*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_37_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 37*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_38_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 38*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_39_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 39*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_40_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 40*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_41_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 41*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_42_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 42*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_43_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 43*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_44_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 44*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_45_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 45*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_46_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 46*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_47_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 47*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_48_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 48*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_49_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 49*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_50_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 50*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_51_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 51*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_52_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 52*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_53_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 53*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_54_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 54*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_55_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 55*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_56_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 56*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_57_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 57*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_58_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 58*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_59_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 59*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_60_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 60*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_61_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 61*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_62_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 62*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_63_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 63*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_0_0_15_0;
  DATA_TYPE *localWriteA_0_0_16_0;
  DATA_TYPE *localWriteA_0_0_17_0;
  DATA_TYPE *localWriteA_0_0_18_0;
  DATA_TYPE *localWriteA_0_0_19_0;
  DATA_TYPE *localWriteA_0_0_20_0;
  DATA_TYPE *localWriteA_0_0_21_0;
  DATA_TYPE *localWriteA_0_0_22_0;
  DATA_TYPE *localWriteA_0_0_23_0;
  DATA_TYPE *localWriteA_0_0_24_0;
  DATA_TYPE *localWriteA_0_0_25_0;
  DATA_TYPE *localWriteA_0_0_26_0;
  DATA_TYPE *localWriteA_0_0_27_0;
  DATA_TYPE *localWriteA_0_0_28_0;
  DATA_TYPE *localWriteA_0_0_29_0;
  DATA_TYPE *localWriteA_0_0_30_0;
  DATA_TYPE *localWriteA_0_0_31_0;
  DATA_TYPE *localWriteA_0_0_32_0;
  DATA_TYPE *localWriteA_0_0_33_0;
  DATA_TYPE *localWriteA_0_0_34_0;
  DATA_TYPE *localWriteA_0_0_35_0;
  DATA_TYPE *localWriteA_0_0_36_0;
  DATA_TYPE *localWriteA_0_0_37_0;
  DATA_TYPE *localWriteA_0_0_38_0;
  DATA_TYPE *localWriteA_0_0_39_0;
  DATA_TYPE *localWriteA_0_0_40_0;
  DATA_TYPE *localWriteA_0_0_41_0;
  DATA_TYPE *localWriteA_0_0_42_0;
  DATA_TYPE *localWriteA_0_0_43_0;
  DATA_TYPE *localWriteA_0_0_44_0;
  DATA_TYPE *localWriteA_0_0_45_0;
  DATA_TYPE *localWriteA_0_0_46_0;
  DATA_TYPE *localWriteA_0_0_47_0;
  DATA_TYPE *localWriteA_0_0_48_0;
  DATA_TYPE *localWriteA_0_0_49_0;
  DATA_TYPE *localWriteA_0_0_50_0;
  DATA_TYPE *localWriteA_0_0_51_0;
  DATA_TYPE *localWriteA_0_0_52_0;
  DATA_TYPE *localWriteA_0_0_53_0;
  DATA_TYPE *localWriteA_0_0_54_0;
  DATA_TYPE *localWriteA_0_0_55_0;
  DATA_TYPE *localWriteA_0_0_56_0;
  DATA_TYPE *localWriteA_0_0_57_0;
  DATA_TYPE *localWriteA_0_0_58_0;
  DATA_TYPE *localWriteA_0_0_59_0;
  DATA_TYPE *localWriteA_0_0_60_0;
  DATA_TYPE *localWriteA_0_0_61_0;
  DATA_TYPE *localWriteA_0_0_62_0;
  DATA_TYPE *localWriteA_0_0_63_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);
  localWriteA_0_0_16_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_16_0);
  localWriteA_0_0_17_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_17_0);
  localWriteA_0_0_18_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_18_0);
  localWriteA_0_0_19_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_19_0);
  localWriteA_0_0_20_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_20_0);
  localWriteA_0_0_21_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_21_0);
  localWriteA_0_0_22_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_22_0);
  localWriteA_0_0_23_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_23_0);
  localWriteA_0_0_24_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_24_0);
  localWriteA_0_0_25_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_25_0);
  localWriteA_0_0_26_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_26_0);
  localWriteA_0_0_27_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_27_0);
  localWriteA_0_0_28_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_28_0);
  localWriteA_0_0_29_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_29_0);
  localWriteA_0_0_30_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_30_0);
  localWriteA_0_0_31_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_31_0);
  localWriteA_0_0_32_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_32_0);
  localWriteA_0_0_33_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_33_0);
  localWriteA_0_0_34_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_34_0);
  localWriteA_0_0_35_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_35_0);
  localWriteA_0_0_36_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_36_0);
  localWriteA_0_0_37_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_37_0);
  localWriteA_0_0_38_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_38_0);
  localWriteA_0_0_39_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_39_0);
  localWriteA_0_0_40_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_40_0);
  localWriteA_0_0_41_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_41_0);
  localWriteA_0_0_42_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_42_0);
  localWriteA_0_0_43_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_43_0);
  localWriteA_0_0_44_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_44_0);
  localWriteA_0_0_45_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_45_0);
  localWriteA_0_0_46_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_46_0);
  localWriteA_0_0_47_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_47_0);
  localWriteA_0_0_48_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_48_0);
  localWriteA_0_0_49_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_49_0);
  localWriteA_0_0_50_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_50_0);
  localWriteA_0_0_51_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_51_0);
  localWriteA_0_0_52_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_52_0);
  localWriteA_0_0_53_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_53_0);
  localWriteA_0_0_54_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_54_0);
  localWriteA_0_0_55_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_55_0);
  localWriteA_0_0_56_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_56_0);
  localWriteA_0_0_57_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_57_0);
  localWriteA_0_0_58_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_58_0);
  localWriteA_0_0_59_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_59_0);
  localWriteA_0_0_60_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_60_0);
  localWriteA_0_0_61_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_61_0);
  localWriteA_0_0_62_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_62_0);
  localWriteA_0_0_63_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_63_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);
    a_0_0_16_0 = *(globalReadA_0_0_16_0 + 0);
    a_0_0_17_0 = *(globalReadA_0_0_17_0 + 0);
    a_0_0_18_0 = *(globalReadA_0_0_18_0 + 0);
    a_0_0_19_0 = *(globalReadA_0_0_19_0 + 0);
    a_0_0_20_0 = *(globalReadA_0_0_20_0 + 0);
    a_0_0_21_0 = *(globalReadA_0_0_21_0 + 0);
    a_0_0_22_0 = *(globalReadA_0_0_22_0 + 0);
    a_0_0_23_0 = *(globalReadA_0_0_23_0 + 0);
    a_0_0_24_0 = *(globalReadA_0_0_24_0 + 0);
    a_0_0_25_0 = *(globalReadA_0_0_25_0 + 0);
    a_0_0_26_0 = *(globalReadA_0_0_26_0 + 0);
    a_0_0_27_0 = *(globalReadA_0_0_27_0 + 0);
    a_0_0_28_0 = *(globalReadA_0_0_28_0 + 0);
    a_0_0_29_0 = *(globalReadA_0_0_29_0 + 0);
    a_0_0_30_0 = *(globalReadA_0_0_30_0 + 0);
    a_0_0_31_0 = *(globalReadA_0_0_31_0 + 0);
    a_0_0_32_0 = *(globalReadA_0_0_32_0 + 0);
    a_0_0_33_0 = *(globalReadA_0_0_33_0 + 0);
    a_0_0_34_0 = *(globalReadA_0_0_34_0 + 0);
    a_0_0_35_0 = *(globalReadA_0_0_35_0 + 0);
    a_0_0_36_0 = *(globalReadA_0_0_36_0 + 0);
    a_0_0_37_0 = *(globalReadA_0_0_37_0 + 0);
    a_0_0_38_0 = *(globalReadA_0_0_38_0 + 0);
    a_0_0_39_0 = *(globalReadA_0_0_39_0 + 0);
    a_0_0_40_0 = *(globalReadA_0_0_40_0 + 0);
    a_0_0_41_0 = *(globalReadA_0_0_41_0 + 0);
    a_0_0_42_0 = *(globalReadA_0_0_42_0 + 0);
    a_0_0_43_0 = *(globalReadA_0_0_43_0 + 0);
    a_0_0_44_0 = *(globalReadA_0_0_44_0 + 0);
    a_0_0_45_0 = *(globalReadA_0_0_45_0 + 0);
    a_0_0_46_0 = *(globalReadA_0_0_46_0 + 0);
    a_0_0_47_0 = *(globalReadA_0_0_47_0 + 0);
    a_0_0_48_0 = *(globalReadA_0_0_48_0 + 0);
    a_0_0_49_0 = *(globalReadA_0_0_49_0 + 0);
    a_0_0_50_0 = *(globalReadA_0_0_50_0 + 0);
    a_0_0_51_0 = *(globalReadA_0_0_51_0 + 0);
    a_0_0_52_0 = *(globalReadA_0_0_52_0 + 0);
    a_0_0_53_0 = *(globalReadA_0_0_53_0 + 0);
    a_0_0_54_0 = *(globalReadA_0_0_54_0 + 0);
    a_0_0_55_0 = *(globalReadA_0_0_55_0 + 0);
    a_0_0_56_0 = *(globalReadA_0_0_56_0 + 0);
    a_0_0_57_0 = *(globalReadA_0_0_57_0 + 0);
    a_0_0_58_0 = *(globalReadA_0_0_58_0 + 0);
    a_0_0_59_0 = *(globalReadA_0_0_59_0 + 0);
    a_0_0_60_0 = *(globalReadA_0_0_60_0 + 0);
    a_0_0_61_0 = *(globalReadA_0_0_61_0 + 0);
    a_0_0_62_0 = *(globalReadA_0_0_62_0 + 0);
    a_0_0_63_0 = *(globalReadA_0_0_63_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    globalReadA_0_0_16_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_16_0) + 1*globalReadIncAL);
    globalReadA_0_0_17_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_17_0) + 1*globalReadIncAL);
    globalReadA_0_0_18_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_18_0) + 1*globalReadIncAL);
    globalReadA_0_0_19_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_19_0) + 1*globalReadIncAL);
    globalReadA_0_0_20_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_20_0) + 1*globalReadIncAL);
    globalReadA_0_0_21_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_21_0) + 1*globalReadIncAL);
    globalReadA_0_0_22_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_22_0) + 1*globalReadIncAL);
    globalReadA_0_0_23_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_23_0) + 1*globalReadIncAL);
    globalReadA_0_0_24_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_24_0) + 1*globalReadIncAL);
    globalReadA_0_0_25_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_25_0) + 1*globalReadIncAL);
    globalReadA_0_0_26_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_26_0) + 1*globalReadIncAL);
    globalReadA_0_0_27_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_27_0) + 1*globalReadIncAL);
    globalReadA_0_0_28_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_28_0) + 1*globalReadIncAL);
    globalReadA_0_0_29_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_29_0) + 1*globalReadIncAL);
    globalReadA_0_0_30_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_30_0) + 1*globalReadIncAL);
    globalReadA_0_0_31_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_31_0) + 1*globalReadIncAL);
    globalReadA_0_0_32_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_32_0) + 1*globalReadIncAL);
    globalReadA_0_0_33_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_33_0) + 1*globalReadIncAL);
    globalReadA_0_0_34_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_34_0) + 1*globalReadIncAL);
    globalReadA_0_0_35_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_35_0) + 1*globalReadIncAL);
    globalReadA_0_0_36_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_36_0) + 1*globalReadIncAL);
    globalReadA_0_0_37_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_37_0) + 1*globalReadIncAL);
    globalReadA_0_0_38_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_38_0) + 1*globalReadIncAL);
    globalReadA_0_0_39_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_39_0) + 1*globalReadIncAL);
    globalReadA_0_0_40_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_40_0) + 1*globalReadIncAL);
    globalReadA_0_0_41_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_41_0) + 1*globalReadIncAL);
    globalReadA_0_0_42_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_42_0) + 1*globalReadIncAL);
    globalReadA_0_0_43_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_43_0) + 1*globalReadIncAL);
    globalReadA_0_0_44_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_44_0) + 1*globalReadIncAL);
    globalReadA_0_0_45_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_45_0) + 1*globalReadIncAL);
    globalReadA_0_0_46_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_46_0) + 1*globalReadIncAL);
    globalReadA_0_0_47_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_47_0) + 1*globalReadIncAL);
    globalReadA_0_0_48_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_48_0) + 1*globalReadIncAL);
    globalReadA_0_0_49_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_49_0) + 1*globalReadIncAL);
    globalReadA_0_0_50_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_50_0) + 1*globalReadIncAL);
    globalReadA_0_0_51_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_51_0) + 1*globalReadIncAL);
    globalReadA_0_0_52_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_52_0) + 1*globalReadIncAL);
    globalReadA_0_0_53_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_53_0) + 1*globalReadIncAL);
    globalReadA_0_0_54_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_54_0) + 1*globalReadIncAL);
    globalReadA_0_0_55_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_55_0) + 1*globalReadIncAL);
    globalReadA_0_0_56_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_56_0) + 1*globalReadIncAL);
    globalReadA_0_0_57_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_57_0) + 1*globalReadIncAL);
    globalReadA_0_0_58_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_58_0) + 1*globalReadIncAL);
    globalReadA_0_0_59_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_59_0) + 1*globalReadIncAL);
    globalReadA_0_0_60_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_60_0) + 1*globalReadIncAL);
    globalReadA_0_0_61_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_61_0) + 1*globalReadIncAL);
    globalReadA_0_0_62_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_62_0) + 1*globalReadIncAL);
    globalReadA_0_0_63_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_63_0) + 1*globalReadIncAL);

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
    *(localWriteA_0_0_16_0 + 0) = a_0_0_16_0;
    *(localWriteA_0_0_17_0 + 0) = a_0_0_17_0;
    *(localWriteA_0_0_18_0 + 0) = a_0_0_18_0;
    *(localWriteA_0_0_19_0 + 0) = a_0_0_19_0;
    *(localWriteA_0_0_20_0 + 0) = a_0_0_20_0;
    *(localWriteA_0_0_21_0 + 0) = a_0_0_21_0;
    *(localWriteA_0_0_22_0 + 0) = a_0_0_22_0;
    *(localWriteA_0_0_23_0 + 0) = a_0_0_23_0;
    *(localWriteA_0_0_24_0 + 0) = a_0_0_24_0;
    *(localWriteA_0_0_25_0 + 0) = a_0_0_25_0;
    *(localWriteA_0_0_26_0 + 0) = a_0_0_26_0;
    *(localWriteA_0_0_27_0 + 0) = a_0_0_27_0;
    *(localWriteA_0_0_28_0 + 0) = a_0_0_28_0;
    *(localWriteA_0_0_29_0 + 0) = a_0_0_29_0;
    *(localWriteA_0_0_30_0 + 0) = a_0_0_30_0;
    *(localWriteA_0_0_31_0 + 0) = a_0_0_31_0;
    *(localWriteA_0_0_32_0 + 0) = a_0_0_32_0;
    *(localWriteA_0_0_33_0 + 0) = a_0_0_33_0;
    *(localWriteA_0_0_34_0 + 0) = a_0_0_34_0;
    *(localWriteA_0_0_35_0 + 0) = a_0_0_35_0;
    *(localWriteA_0_0_36_0 + 0) = a_0_0_36_0;
    *(localWriteA_0_0_37_0 + 0) = a_0_0_37_0;
    *(localWriteA_0_0_38_0 + 0) = a_0_0_38_0;
    *(localWriteA_0_0_39_0 + 0) = a_0_0_39_0;
    *(localWriteA_0_0_40_0 + 0) = a_0_0_40_0;
    *(localWriteA_0_0_41_0 + 0) = a_0_0_41_0;
    *(localWriteA_0_0_42_0 + 0) = a_0_0_42_0;
    *(localWriteA_0_0_43_0 + 0) = a_0_0_43_0;
    *(localWriteA_0_0_44_0 + 0) = a_0_0_44_0;
    *(localWriteA_0_0_45_0 + 0) = a_0_0_45_0;
    *(localWriteA_0_0_46_0 + 0) = a_0_0_46_0;
    *(localWriteA_0_0_47_0 + 0) = a_0_0_47_0;
    *(localWriteA_0_0_48_0 + 0) = a_0_0_48_0;
    *(localWriteA_0_0_49_0 + 0) = a_0_0_49_0;
    *(localWriteA_0_0_50_0 + 0) = a_0_0_50_0;
    *(localWriteA_0_0_51_0 + 0) = a_0_0_51_0;
    *(localWriteA_0_0_52_0 + 0) = a_0_0_52_0;
    *(localWriteA_0_0_53_0 + 0) = a_0_0_53_0;
    *(localWriteA_0_0_54_0 + 0) = a_0_0_54_0;
    *(localWriteA_0_0_55_0 + 0) = a_0_0_55_0;
    *(localWriteA_0_0_56_0 + 0) = a_0_0_56_0;
    *(localWriteA_0_0_57_0 + 0) = a_0_0_57_0;
    *(localWriteA_0_0_58_0 + 0) = a_0_0_58_0;
    *(localWriteA_0_0_59_0 + 0) = a_0_0_59_0;
    *(localWriteA_0_0_60_0 + 0) = a_0_0_60_0;
    *(localWriteA_0_0_61_0 + 0) = a_0_0_61_0;
    *(localWriteA_0_0_62_0 + 0) = a_0_0_62_0;
    *(localWriteA_0_0_63_0 + 0) = a_0_0_63_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 15 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 16 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 17 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 18 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 19 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 20 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 21 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 22 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 23 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 24 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 25 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 26 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 27 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 28 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 29 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 30 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 31 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 32 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 33 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 34 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 35 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 36 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 37 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 38 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 39 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 40 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 41 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 42 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 43 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 44 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 45 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 46 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 47 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 48 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 49 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 50 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 51 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 52 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 53 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 54 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 55 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 56 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 57 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 58 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 59 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 60 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 61 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 62 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_1x1


    /* iter 63 */

    MAC_1x1_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_0_8_0 = ( globalReadOffsetAL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_0_0_9_0 = ( globalReadOffsetAL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_0_0_10_0 = ( globalReadOffsetAL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_0_0_11_0 = ( globalReadOffsetAL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_0_0_12_0 = ( globalReadOffsetAL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_0_0_13_0 = ( globalReadOffsetAL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_0_0_14_0 = ( globalReadOffsetAL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_0_0_15_0 = ( globalReadOffsetAL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);
  a_0_0_16_0 = ( globalReadOffsetAL_16_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_16_0 + 0);
  a_0_0_17_0 = ( globalReadOffsetAL_17_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_17_0 + 0);
  a_0_0_18_0 = ( globalReadOffsetAL_18_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_18_0 + 0);
  a_0_0_19_0 = ( globalReadOffsetAL_19_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_19_0 + 0);
  a_0_0_20_0 = ( globalReadOffsetAL_20_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_20_0 + 0);
  a_0_0_21_0 = ( globalReadOffsetAL_21_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_21_0 + 0);
  a_0_0_22_0 = ( globalReadOffsetAL_22_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_22_0 + 0);
  a_0_0_23_0 = ( globalReadOffsetAL_23_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_23_0 + 0);
  a_0_0_24_0 = ( globalReadOffsetAL_24_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_24_0 + 0);
  a_0_0_25_0 = ( globalReadOffsetAL_25_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_25_0 + 0);
  a_0_0_26_0 = ( globalReadOffsetAL_26_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_26_0 + 0);
  a_0_0_27_0 = ( globalReadOffsetAL_27_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_27_0 + 0);
  a_0_0_28_0 = ( globalReadOffsetAL_28_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_28_0 + 0);
  a_0_0_29_0 = ( globalReadOffsetAL_29_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_29_0 + 0);
  a_0_0_30_0 = ( globalReadOffsetAL_30_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_30_0 + 0);
  a_0_0_31_0 = ( globalReadOffsetAL_31_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_31_0 + 0);
  a_0_0_32_0 = ( globalReadOffsetAL_32_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_32_0 + 0);
  a_0_0_33_0 = ( globalReadOffsetAL_33_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_33_0 + 0);
  a_0_0_34_0 = ( globalReadOffsetAL_34_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_34_0 + 0);
  a_0_0_35_0 = ( globalReadOffsetAL_35_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_35_0 + 0);
  a_0_0_36_0 = ( globalReadOffsetAL_36_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_36_0 + 0);
  a_0_0_37_0 = ( globalReadOffsetAL_37_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_37_0 + 0);
  a_0_0_38_0 = ( globalReadOffsetAL_38_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_38_0 + 0);
  a_0_0_39_0 = ( globalReadOffsetAL_39_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_39_0 + 0);
  a_0_0_40_0 = ( globalReadOffsetAL_40_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_40_0 + 0);
  a_0_0_41_0 = ( globalReadOffsetAL_41_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_41_0 + 0);
  a_0_0_42_0 = ( globalReadOffsetAL_42_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_42_0 + 0);
  a_0_0_43_0 = ( globalReadOffsetAL_43_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_43_0 + 0);
  a_0_0_44_0 = ( globalReadOffsetAL_44_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_44_0 + 0);
  a_0_0_45_0 = ( globalReadOffsetAL_45_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_45_0 + 0);
  a_0_0_46_0 = ( globalReadOffsetAL_46_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_46_0 + 0);
  a_0_0_47_0 = ( globalReadOffsetAL_47_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_47_0 + 0);
  a_0_0_48_0 = ( globalReadOffsetAL_48_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_48_0 + 0);
  a_0_0_49_0 = ( globalReadOffsetAL_49_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_49_0 + 0);
  a_0_0_50_0 = ( globalReadOffsetAL_50_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_50_0 + 0);
  a_0_0_51_0 = ( globalReadOffsetAL_51_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_51_0 + 0);
  a_0_0_52_0 = ( globalReadOffsetAL_52_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_52_0 + 0);
  a_0_0_53_0 = ( globalReadOffsetAL_53_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_53_0 + 0);
  a_0_0_54_0 = ( globalReadOffsetAL_54_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_54_0 + 0);
  a_0_0_55_0 = ( globalReadOffsetAL_55_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_55_0 + 0);
  a_0_0_56_0 = ( globalReadOffsetAL_56_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_56_0 + 0);
  a_0_0_57_0 = ( globalReadOffsetAL_57_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_57_0 + 0);
  a_0_0_58_0 = ( globalReadOffsetAL_58_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_58_0 + 0);
  a_0_0_59_0 = ( globalReadOffsetAL_59_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_59_0 + 0);
  a_0_0_60_0 = ( globalReadOffsetAL_60_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_60_0 + 0);
  a_0_0_61_0 = ( globalReadOffsetAL_61_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_61_0 + 0);
  a_0_0_62_0 = ( globalReadOffsetAL_62_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_62_0 + 0);
  a_0_0_63_0 = ( globalReadOffsetAL_63_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_63_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);
  localWriteA_0_0_16_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_16_0);
  localWriteA_0_0_17_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_17_0);
  localWriteA_0_0_18_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_18_0);
  localWriteA_0_0_19_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_19_0);
  localWriteA_0_0_20_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_20_0);
  localWriteA_0_0_21_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_21_0);
  localWriteA_0_0_22_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_22_0);
  localWriteA_0_0_23_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_23_0);
  localWriteA_0_0_24_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_24_0);
  localWriteA_0_0_25_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_25_0);
  localWriteA_0_0_26_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_26_0);
  localWriteA_0_0_27_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_27_0);
  localWriteA_0_0_28_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_28_0);
  localWriteA_0_0_29_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_29_0);
  localWriteA_0_0_30_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_30_0);
  localWriteA_0_0_31_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_31_0);
  localWriteA_0_0_32_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_32_0);
  localWriteA_0_0_33_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_33_0);
  localWriteA_0_0_34_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_34_0);
  localWriteA_0_0_35_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_35_0);
  localWriteA_0_0_36_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_36_0);
  localWriteA_0_0_37_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_37_0);
  localWriteA_0_0_38_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_38_0);
  localWriteA_0_0_39_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_39_0);
  localWriteA_0_0_40_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_40_0);
  localWriteA_0_0_41_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_41_0);
  localWriteA_0_0_42_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_42_0);
  localWriteA_0_0_43_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_43_0);
  localWriteA_0_0_44_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_44_0);
  localWriteA_0_0_45_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_45_0);
  localWriteA_0_0_46_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_46_0);
  localWriteA_0_0_47_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_47_0);
  localWriteA_0_0_48_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_48_0);
  localWriteA_0_0_49_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_49_0);
  localWriteA_0_0_50_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_50_0);
  localWriteA_0_0_51_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_51_0);
  localWriteA_0_0_52_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_52_0);
  localWriteA_0_0_53_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_53_0);
  localWriteA_0_0_54_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_54_0);
  localWriteA_0_0_55_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_55_0);
  localWriteA_0_0_56_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_56_0);
  localWriteA_0_0_57_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_57_0);
  localWriteA_0_0_58_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_58_0);
  localWriteA_0_0_59_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_59_0);
  localWriteA_0_0_60_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_60_0);
  localWriteA_0_0_61_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_61_0);
  localWriteA_0_0_62_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_62_0);
  localWriteA_0_0_63_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_63_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
  *(localWriteA_0_0_16_0 + 0) = a_0_0_16_0;
  *(localWriteA_0_0_17_0 + 0) = a_0_0_17_0;
  *(localWriteA_0_0_18_0 + 0) = a_0_0_18_0;
  *(localWriteA_0_0_19_0 + 0) = a_0_0_19_0;
  *(localWriteA_0_0_20_0 + 0) = a_0_0_20_0;
  *(localWriteA_0_0_21_0 + 0) = a_0_0_21_0;
  *(localWriteA_0_0_22_0 + 0) = a_0_0_22_0;
  *(localWriteA_0_0_23_0 + 0) = a_0_0_23_0;
  *(localWriteA_0_0_24_0 + 0) = a_0_0_24_0;
  *(localWriteA_0_0_25_0 + 0) = a_0_0_25_0;
  *(localWriteA_0_0_26_0 + 0) = a_0_0_26_0;
  *(localWriteA_0_0_27_0 + 0) = a_0_0_27_0;
  *(localWriteA_0_0_28_0 + 0) = a_0_0_28_0;
  *(localWriteA_0_0_29_0 + 0) = a_0_0_29_0;
  *(localWriteA_0_0_30_0 + 0) = a_0_0_30_0;
  *(localWriteA_0_0_31_0 + 0) = a_0_0_31_0;
  *(localWriteA_0_0_32_0 + 0) = a_0_0_32_0;
  *(localWriteA_0_0_33_0 + 0) = a_0_0_33_0;
  *(localWriteA_0_0_34_0 + 0) = a_0_0_34_0;
  *(localWriteA_0_0_35_0 + 0) = a_0_0_35_0;
  *(localWriteA_0_0_36_0 + 0) = a_0_0_36_0;
  *(localWriteA_0_0_37_0 + 0) = a_0_0_37_0;
  *(localWriteA_0_0_38_0 + 0) = a_0_0_38_0;
  *(localWriteA_0_0_39_0 + 0) = a_0_0_39_0;
  *(localWriteA_0_0_40_0 + 0) = a_0_0_40_0;
  *(localWriteA_0_0_41_0 + 0) = a_0_0_41_0;
  *(localWriteA_0_0_42_0 + 0) = a_0_0_42_0;
  *(localWriteA_0_0_43_0 + 0) = a_0_0_43_0;
  *(localWriteA_0_0_44_0 + 0) = a_0_0_44_0;
  *(localWriteA_0_0_45_0 + 0) = a_0_0_45_0;
  *(localWriteA_0_0_46_0 + 0) = a_0_0_46_0;
  *(localWriteA_0_0_47_0 + 0) = a_0_0_47_0;
  *(localWriteA_0_0_48_0 + 0) = a_0_0_48_0;
  *(localWriteA_0_0_49_0 + 0) = a_0_0_49_0;
  *(localWriteA_0_0_50_0 + 0) = a_0_0_50_0;
  *(localWriteA_0_0_51_0 + 0) = a_0_0_51_0;
  *(localWriteA_0_0_52_0 + 0) = a_0_0_52_0;
  *(localWriteA_0_0_53_0 + 0) = a_0_0_53_0;
  *(localWriteA_0_0_54_0 + 0) = a_0_0_54_0;
  *(localWriteA_0_0_55_0 + 0) = a_0_0_55_0;
  *(localWriteA_0_0_56_0 + 0) = a_0_0_56_0;
  *(localWriteA_0_0_57_0 + 0) = a_0_0_57_0;
  *(localWriteA_0_0_58_0 + 0) = a_0_0_58_0;
  *(localWriteA_0_0_59_0 + 0) = a_0_0_59_0;
  *(localWriteA_0_0_60_0 + 0) = a_0_0_60_0;
  *(localWriteA_0_0_61_0 + 0) = a_0_0_61_0;
  *(localWriteA_0_0_62_0 + 0) = a_0_0_62_0;
  *(localWriteA_0_0_63_0 + 0) = a_0_0_63_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_1x1

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_1x1
#undef MAC_1x1_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 64
#define SG1J 1
#define TT0I 1
#define TT1J 1
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 64
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 64
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 4096
#define LDS_NUM_ELEMENTS 4160

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 1x1 micro-tile */
#define MAC_1x1\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \

#define MAC_1x1_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x1x64_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU0_SUM0_SUS128_SRVW0_SVW4_SNLL0_TT1_1_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG64_1_1_WGM1(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE a_0_0_16_0;
  DATA_TYPE a_0_0_17_0;
  DATA_TYPE a_0_0_18_0;
  DATA_TYPE a_0_0_19_0;
  DATA_TYPE a_0_0_20_0;
  DATA_TYPE a_0_0_21_0;
  DATA_TYPE a_0_0_22_0;
  DATA_TYPE a_0_0_23_0;
  DATA_TYPE a_0_0_24_0;
  DATA_TYPE a_0_0_25_0;
  DATA_TYPE a_0_0_26_0;
  DATA_TYPE a_0_0_27_0;
  DATA_TYPE a_0_0_28_0;
  DATA_TYPE a_0_0_29_0;
  DATA_TYPE a_0_0_30_0;
  DATA_TYPE a_0_0_31_0;
  DATA_TYPE a_0_0_32_0;
  DATA_TYPE a_0_0_33_0;
  DATA_TYPE a_0_0_34_0;
  DATA_TYPE a_0_0_35_0;
  DATA_TYPE a_0_0_36_0;
  DATA_TYPE a_0_0_37_0;
  DATA_TYPE a_0_0_38_0;
  DATA_TYPE a_0_0_39_0;
  DATA_TYPE a_0_0_40_0;
  DATA_TYPE a_0_0_41_0;
  DATA_TYPE a_0_0_42_0;
  DATA_TYPE a_0_0_43_0;
  DATA_TYPE a_0_0_44_0;
  DATA_TYPE a_0_0_45_0;
  DATA_TYPE a_0_0_46_0;
  DATA_TYPE a_0_0_47_0;
  DATA_TYPE a_0_0_48_0;
  DATA_TYPE a_0_0_49_0;
  DATA_TYPE a_0_0_50_0;
  DATA_TYPE a_0_0_51_0;
  DATA_TYPE a_0_0_52_0;
  DATA_TYPE a_0_0_53_0;
  DATA_TYPE a_0_0_54_0;
  DATA_TYPE a_0_0_55_0;
  DATA_TYPE a_0_0_56_0;
  DATA_TYPE a_0_0_57_0;
  DATA_TYPE a_0_0_58_0;
  DATA_TYPE a_0_0_59_0;
  DATA_TYPE a_0_0_60_0;
  DATA_TYPE a_0_0_61_0;
  DATA_TYPE a_0_0_62_0;
  DATA_TYPE a_0_0_63_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;
  unsigned int globalReadOffsetAL_8_0 = globalReadOffsetAL + 0 + 8*LSPA;
  unsigned int globalReadOffsetAL_9_0 = globalReadOffsetAL + 0 + 9*LSPA;
  unsigned int globalReadOffsetAL_10_0 = globalReadOffsetAL + 0 + 10*LSPA;
  unsigned int globalReadOffsetAL_11_0 = globalReadOffsetAL + 0 + 11*LSPA;
  unsigned int globalReadOffsetAL_12_0 = globalReadOffsetAL + 0 + 12*LSPA;
  unsigned int globalReadOffsetAL_13_0 = globalReadOffsetAL + 0 + 13*LSPA;
  unsigned int globalReadOffsetAL_14_0 = globalReadOffsetAL + 0 + 14*LSPA;
  unsigned int globalReadOffsetAL_15_0 = globalReadOffsetAL + 0 + 15*LSPA;
  unsigned int globalReadOffsetAL_16_0 = globalReadOffsetAL + 0 + 16*LSPA;
  unsigned int globalReadOffsetAL_17_0 = globalReadOffsetAL + 0 + 17*LSPA;
  unsigned int globalReadOffsetAL_18_0 = globalReadOffsetAL + 0 + 18*LSPA;
  unsigned int globalReadOffsetAL_19_0 = globalReadOffsetAL + 0 + 19*LSPA;
  unsigned int globalReadOffsetAL_20_0 = globalReadOffsetAL + 0 + 20*LSPA;
  unsigned int globalReadOffsetAL_21_0 = globalReadOffsetAL + 0 + 21*LSPA;
  unsigned int globalReadOffsetAL_22_0 = globalReadOffsetAL + 0 + 22*LSPA;
  unsigned int globalReadOffsetAL_23_0 = globalReadOffsetAL + 0 + 23*LSPA;
  unsigned int globalReadOffsetAL_24_0 = globalReadOffsetAL + 0 + 24*LSPA;
  unsigned int globalReadOffsetAL_25_0 = globalReadOffsetAL + 0 + 25*LSPA;
  unsigned int globalReadOffsetAL_26_0 = globalReadOffsetAL + 0 + 26*LSPA;
  unsigned int globalReadOffsetAL_27_0 = globalReadOffsetAL + 0 + 27*LSPA;
  unsigned int globalReadOffsetAL_28_0 = globalReadOffsetAL + 0 + 28*LSPA;
  unsigned int globalReadOffsetAL_29_0 = globalReadOffsetAL + 0 + 29*LSPA;
  unsigned int globalReadOffsetAL_30_0 = globalReadOffsetAL + 0 + 30*LSPA;
  unsigned int globalReadOffsetAL_31_0 = globalReadOffsetAL + 0 + 31*LSPA;
  unsigned int globalReadOffsetAL_32_0 = globalReadOffsetAL + 0 + 32*LSPA;
  unsigned int globalReadOffsetAL_33_0 = globalReadOffsetAL + 0 + 33*LSPA;
  unsigned int globalReadOffsetAL_34_0 = globalReadOffsetAL + 0 + 34*LSPA;
  unsigned int globalReadOffsetAL_35_0 = globalReadOffsetAL + 0 + 35*LSPA;
  unsigned int globalReadOffsetAL_36_0 = globalReadOffsetAL + 0 + 36*LSPA;
  unsigned int globalReadOffsetAL_37_0 = globalReadOffsetAL + 0 + 37*LSPA;
  unsigned int globalReadOffsetAL_38_0 = globalReadOffsetAL + 0 + 38*LSPA;
  unsigned int globalReadOffsetAL_39_0 = globalReadOffsetAL + 0 + 39*LSPA;
  unsigned int globalReadOffsetAL_40_0 = globalReadOffsetAL + 0 + 40*LSPA;
  unsigned int globalReadOffsetAL_41_0 = globalReadOffsetAL + 0 + 41*LSPA;
  unsigned int globalReadOffsetAL_42_0 = globalReadOffsetAL + 0 + 42*LSPA;
  unsigned int globalReadOffsetAL_43_0 = globalReadOffsetAL + 0 + 43*LSPA;
  unsigned int globalReadOffsetAL_44_0 = globalReadOffsetAL + 0 + 44*LSPA;
  unsigned int globalReadOffsetAL_45_0 = globalReadOffsetAL + 0 + 45*LSPA;
  unsigned int globalReadOffsetAL_46_0 = globalReadOffsetAL + 0 + 46*LSPA;
  unsigned int globalReadOffsetAL_47_0 = globalReadOffsetAL + 0 + 47*LSPA;
  unsigned int globalReadOffsetAL_48_0 = globalReadOffsetAL + 0 + 48*LSPA;
  unsigned int globalReadOffsetAL_49_0 = globalReadOffsetAL + 0 + 49*LSPA;
  unsigned int globalReadOffsetAL_50_0 = globalReadOffsetAL + 0 + 50*LSPA;
  unsigned int globalReadOffsetAL_51_0 = globalReadOffsetAL + 0 + 51*LSPA;
  unsigned int globalReadOffsetAL_52_0 = globalReadOffsetAL + 0 + 52*LSPA;
  unsigned int globalReadOffsetAL_53_0 = globalReadOffsetAL + 0 + 53*LSPA;
  unsigned int globalReadOffsetAL_54_0 = globalReadOffsetAL + 0 + 54*LSPA;
  unsigned int globalReadOffsetAL_55_0 = globalReadOffsetAL + 0 + 55*LSPA;
  unsigned int globalReadOffsetAL_56_0 = globalReadOffsetAL + 0 + 56*LSPA;
  unsigned int globalReadOffsetAL_57_0 = globalReadOffsetAL + 0 + 57*LSPA;
  unsigned int globalReadOffsetAL_58_0 = globalReadOffsetAL + 0 + 58*LSPA;
  unsigned int globalReadOffsetAL_59_0 = globalReadOffsetAL + 0 + 59*LSPA;
  unsigned int globalReadOffsetAL_60_0 = globalReadOffsetAL + 0 + 60*LSPA;
  unsigned int globalReadOffsetAL_61_0 = globalReadOffsetAL + 0 + 61*LSPA;
  unsigned int globalReadOffsetAL_62_0 = globalReadOffsetAL + 0 + 62*LSPA;
  unsigned int globalReadOffsetAL_63_0 = globalReadOffsetAL + 0 + 63*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_15_0), wgK );
  int64_t globalReadOffsetA_0_0_16_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_16_0), wgK );
  int64_t globalReadOffsetA_0_0_17_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_17_0), wgK );
  int64_t globalReadOffsetA_0_0_18_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_18_0), wgK );
  int64_t globalReadOffsetA_0_0_19_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_19_0), wgK );
  int64_t globalReadOffsetA_0_0_20_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_20_0), wgK );
  int64_t globalReadOffsetA_0_0_21_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_21_0), wgK );
  int64_t globalReadOffsetA_0_0_22_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_22_0), wgK );
  int64_t globalReadOffsetA_0_0_23_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_23_0), wgK );
  int64_t globalReadOffsetA_0_0_24_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_24_0), wgK );
  int64_t globalReadOffsetA_0_0_25_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_25_0), wgK );
  int64_t globalReadOffsetA_0_0_26_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_26_0), wgK );
  int64_t globalReadOffsetA_0_0_27_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_27_0), wgK );
  int64_t globalReadOffsetA_0_0_28_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_28_0), wgK );
  int64_t globalReadOffsetA_0_0_29_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_29_0), wgK );
  int64_t globalReadOffsetA_0_0_30_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_30_0), wgK );
  int64_t globalReadOffsetA_0_0_31_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_31_0), wgK );
  int64_t globalReadOffsetA_0_0_32_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_32_0), wgK );
  int64_t globalReadOffsetA_0_0_33_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_33_0), wgK );
  int64_t globalReadOffsetA_0_0_34_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_34_0), wgK );
  int64_t globalReadOffsetA_0_0_35_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_35_0), wgK );
  int64_t globalReadOffsetA_0_0_36_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_36_0), wgK );
  int64_t globalReadOffsetA_0_0_37_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_37_0), wgK );
  int64_t globalReadOffsetA_0_0_38_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_38_0), wgK );
  int64_t globalReadOffsetA_0_0_39_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_39_0), wgK );
  int64_t globalReadOffsetA_0_0_40_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_40_0), wgK );
  int64_t globalReadOffsetA_0_0_41_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_41_0), wgK );
  int64_t globalReadOffsetA_0_0_42_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_42_0), wgK );
  int64_t globalReadOffsetA_0_0_43_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_43_0), wgK );
  int64_t globalReadOffsetA_0_0_44_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_44_0), wgK );
  int64_t globalReadOffsetA_0_0_45_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_45_0), wgK );
  int64_t globalReadOffsetA_0_0_46_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_46_0), wgK );
  int64_t globalReadOffsetA_0_0_47_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_47_0), wgK );
  int64_t globalReadOffsetA_0_0_48_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_48_0), wgK );
  int64_t globalReadOffsetA_0_0_49_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_49_0), wgK );
  int64_t globalReadOffsetA_0_0_50_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_50_0), wgK );
  int64_t globalReadOffsetA_0_0_51_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_51_0), wgK );
  int64_t globalReadOffsetA_0_0_52_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_52_0), wgK );
  int64_t globalReadOffsetA_0_0_53_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_53_0), wgK );
  int64_t globalReadOffsetA_0_0_54_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_54_0), wgK );
  int64_t globalReadOffsetA_0_0_55_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_55_0), wgK );
  int64_t globalReadOffsetA_0_0_56_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_56_0), wgK );
  int64_t globalReadOffsetA_0_0_57_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_57_0), wgK );
  int64_t globalReadOffsetA_0_0_58_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_58_0), wgK );
  int64_t globalReadOffsetA_0_0_59_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_59_0), wgK );
  int64_t globalReadOffsetA_0_0_60_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_60_0), wgK );
  int64_t globalReadOffsetA_0_0_61_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_61_0), wgK );
  int64_t globalReadOffsetA_0_0_62_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_62_0), wgK );
  int64_t globalReadOffsetA_0_0_63_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_63_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;
  DATA_TYPE const *globalReadA_0_0_16_0 = A + globalReadOffsetA_0_0_16_0;
  DATA_TYPE const *globalReadA_0_0_17_0 = A + globalReadOffsetA_0_0_17_0;
  DATA_TYPE const *globalReadA_0_0_18_0 = A + globalReadOffsetA_0_0_18_0;
  DATA_TYPE const *globalReadA_0_0_19_0 = A + globalReadOffsetA_0_0_19_0;
  DATA_TYPE const *globalReadA_0_0_20_0 = A + globalReadOffsetA_0_0_20_0;
  DATA_TYPE const *globalReadA_0_0_21_0 = A + globalReadOffsetA_0_0_21_0;
  DATA_TYPE const *globalReadA_0_0_22_0 = A + globalReadOffsetA_0_0_22_0;
  DATA_TYPE const *globalReadA_0_0_23_0 = A + globalReadOffsetA_0_0_23_0;
  DATA_TYPE const *globalReadA_0_0_24_0 = A + globalReadOffsetA_0_0_24_0;
  DATA_TYPE const *globalReadA_0_0_25_0 = A + globalReadOffsetA_0_0_25_0;
  DATA_TYPE const *globalReadA_0_0_26_0 = A + globalReadOffsetA_0_0_26_0;
  DATA_TYPE const *globalReadA_0_0_27_0 = A + globalReadOffsetA_0_0_27_0;
  DATA_TYPE const *globalReadA_0_0_28_0 = A + globalReadOffsetA_0_0_28_0;
  DATA_TYPE const *globalReadA_0_0_29_0 = A + globalReadOffsetA_0_0_29_0;
  DATA_TYPE const *globalReadA_0_0_30_0 = A + globalReadOffsetA_0_0_30_0;
  DATA_TYPE const *globalReadA_0_0_31_0 = A + globalReadOffsetA_0_0_31_0;
  DATA_TYPE const *globalReadA_0_0_32_0 = A + globalReadOffsetA_0_0_32_0;
  DATA_TYPE const *globalReadA_0_0_33_0 = A + globalReadOffsetA_0_0_33_0;
  DATA_TYPE const *globalReadA_0_0_34_0 = A + globalReadOffsetA_0_0_34_0;
  DATA_TYPE const *globalReadA_0_0_35_0 = A + globalReadOffsetA_0_0_35_0;
  DATA_TYPE const *globalReadA_0_0_36_0 = A + globalReadOffsetA_0_0_36_0;
  DATA_TYPE const *globalReadA_0_0_37_0 = A + globalReadOffsetA_0_0_37_0;
  DATA_TYPE const *globalReadA_0_0_38_0 = A + globalReadOffsetA_0_0_38_0;
  DATA_TYPE const *globalReadA_0_0_39_0 = A + globalReadOffsetA_0_0_39_0;
  DATA_TYPE const *globalReadA_0_0_40_0 = A + globalReadOffsetA_0_0_40_0;
  DATA_TYPE const *globalReadA_0_0_41_0 = A + globalReadOffsetA_0_0_41_0;
  DATA_TYPE const *globalReadA_0_0_42_0 = A + globalReadOffsetA_0_0_42_0;
  DATA_TYPE const *globalReadA_0_0_43_0 = A + globalReadOffsetA_0_0_43_0;
  DATA_TYPE const *globalReadA_0_0_44_0 = A + globalReadOffsetA_0_0_44_0;
  DATA_TYPE const *globalReadA_0_0_45_0 = A + globalReadOffsetA_0_0_45_0;
  DATA_TYPE const *globalReadA_0_0_46_0 = A + globalReadOffsetA_0_0_46_0;
  DATA_TYPE const *globalReadA_0_0_47_0 = A + globalReadOffsetA_0_0_47_0;
  DATA_TYPE const *globalReadA_0_0_48_0 = A + globalReadOffsetA_0_0_48_0;
  DATA_TYPE const *globalReadA_0_0_49_0 = A + globalReadOffsetA_0_0_49_0;
  DATA_TYPE const *globalReadA_0_0_50_0 = A + globalReadOffsetA_0_0_50_0;
  DATA_TYPE const *globalReadA_0_0_51_0 = A + globalReadOffsetA_0_0_51_0;
  DATA_TYPE const *globalReadA_0_0_52_0 = A + globalReadOffsetA_0_0_52_0;
  DATA_TYPE const *globalReadA_0_0_53_0 = A + globalReadOffsetA_0_0_53_0;
  DATA_TYPE const *globalReadA_0_0_54_0 = A + globalReadOffsetA_0_0_54_0;
  DATA_TYPE const *globalReadA_0_0_55_0 = A + globalReadOffsetA_0_0_55_0;
  DATA_TYPE const *globalReadA_0_0_56_0 = A + globalReadOffsetA_0_0_56_0;
  DATA_TYPE const *globalReadA_0_0_57_0 = A + globalReadOffsetA_0_0_57_0;
  DATA_TYPE const *globalReadA_0_0_58_0 = A + globalReadOffsetA_0_0_58_0;
  DATA_TYPE const *globalReadA_0_0_59_0 = A + globalReadOffsetA_0_0_59_0;
  DATA_TYPE const *globalReadA_0_0_60_0 = A + globalReadOffsetA_0_0_60_0;
  DATA_TYPE const *globalReadA_0_0_61_0 = A + globalReadOffsetA_0_0_61_0;
  DATA_TYPE const *globalReadA_0_0_62_0 = A + globalReadOffsetA_0_0_62_0;
  DATA_TYPE const *globalReadA_0_0_63_0 = A + globalReadOffsetA_0_0_63_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 8*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 9*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 10*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 11*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 12*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 13*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 14*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 15*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_16_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 16*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_17_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 17*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_18_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 18*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_19_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 19*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_20_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 20*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_21_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 21*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_22_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 22*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_23_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 23*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_24_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 24*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_25_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 25*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_26_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 26*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_27_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 27*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_28_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 28*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_29_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 29*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_30_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 30*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_31_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 31*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_32_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 32*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_33_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 33*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_34_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 34*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_35_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 35*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_36_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 36*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_37_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 37*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_38_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 38*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_39_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 39*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_40_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 40*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_41_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 41*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_42_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 42*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_43_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 43*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_44_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 44*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_45_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 45*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_46_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 46*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_47_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 47*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_48_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 48*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_49_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 49*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_50_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 50*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_51_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 51*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_52_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 52*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_53_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 53*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_54_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 54*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_55_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 55*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_56_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 56*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_57_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 57*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_58_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 58*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_59_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 59*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_60_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 60*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_61_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 61*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_62_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 62*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_63_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 63*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_0_0_15_0;
  DATA_TYPE *localWriteA_0_0_16_0;
  DATA_TYPE *localWriteA_0_0_17_0;
  DATA_TYPE *localWriteA_0_0_18_0;
  DATA_TYPE *localWriteA_0_0_19_0;
  DATA_TYPE *localWriteA_0_0_20_0;
  DATA_TYPE *localWriteA_0_0_21_0;
  DATA_TYPE *localWriteA_0_0_22_0;
  DATA_TYPE *localWriteA_0_0_23_0;
  DATA_TYPE *localWriteA_0_0_24_0;
  DATA_TYPE *localWriteA_0_0_25_0;
  DATA_TYPE *localWriteA_0_0_26_0;
  DATA_TYPE *localWriteA_0_0_27_0;
  DATA_TYPE *localWriteA_0_0_28_0;
  DATA_TYPE *localWriteA_0_0_29_0;
  DATA_TYPE *localWriteA_0_0_30_0;
  DATA_TYPE *localWriteA_0_0_31_0;
  DATA_TYPE *localWriteA_0_0_32_0;
  DATA_TYPE *localWriteA_0_0_33_0;
  DATA_TYPE *localWriteA_0_0_34_0;
  DATA_TYPE *localWriteA_0_0_35_0;
  DATA_TYPE *localWriteA_0_0_36_0;
  DATA_TYPE *localWriteA_0_0_37_0;
  DATA_TYPE *localWriteA_0_0_38_0;
  DATA_TYPE *localWriteA_0_0_39_0;
  DATA_TYPE *localWriteA_0_0_40_0;
  DATA_TYPE *localWriteA_0_0_41_0;
  DATA_TYPE *localWriteA_0_0_42_0;
  DATA_TYPE *localWriteA_0_0_43_0;
  DATA_TYPE *localWriteA_0_0_44_0;
  DATA_TYPE *localWriteA_0_0_45_0;
  DATA_TYPE *localWriteA_0_0_46_0;
  DATA_TYPE *localWriteA_0_0_47_0;
  DATA_TYPE *localWriteA_0_0_48_0;
  DATA_TYPE *localWriteA_0_0_49_0;
  DATA_TYPE *localWriteA_0_0_50_0;
  DATA_TYPE *localWriteA_0_0_51_0;
  DATA_TYPE *localWriteA_0_0_52_0;
  DATA_TYPE *localWriteA_0_0_53_0;
  DATA_TYPE *localWriteA_0_0_54_0;
  DATA_TYPE *localWriteA_0_0_55_0;
  DATA_TYPE *localWriteA_0_0_56_0;
  DATA_TYPE *localWriteA_0_0_57_0;
  DATA_TYPE *localWriteA_0_0_58_0;
  DATA_TYPE *localWriteA_0_0_59_0;
  DATA_TYPE *localWriteA_0_0_60_0;
  DATA_TYPE *localWriteA_0_0_61_0;
  DATA_TYPE *localWriteA_0_0_62_0;
  DATA_TYPE *localWriteA_0_0_63_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);
  localWriteA_0_0_16_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_16_0);
  localWriteA_0_0_17_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_17_0);
  localWriteA_0_0_18_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_18_0);
  localWriteA_0_0_19_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_19_0);
  localWriteA_0_0_20_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_20_0);
  localWriteA_0_0_21_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_21_0);
  localWriteA_0_0_22_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_22_0);
  localWriteA_0_0_23_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_23_0);
  localWriteA_0_0_24_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_24_0);
  localWriteA_0_0_25_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_25_0);
  localWriteA_0_0_26_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_26_0);
  localWriteA_0_0_27_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_27_0);
  localWriteA_0_0_28_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_28_0);
  localWriteA_0_0_29_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_29_0);
  localWriteA_0_0_30_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_30_0);
  localWriteA_0_0_31_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_31_0);
  localWriteA_0_0_32_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_32_0);
  localWriteA_0_0_33_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_33_0);
  localWriteA_0_0_34_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_34_0);
  localWriteA_0_0_35_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_35_0);
  localWriteA_0_0_36_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_36_0);
  localWriteA_0_0_37_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_37_0);
  localWriteA_0_0_38_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_38_0);
  localWriteA_0_0_39_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_39_0);
  localWriteA_0_0_40_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_40_0);
  localWriteA_0_0_41_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_41_0);
  localWriteA_0_0_42_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_42_0);
  localWriteA_0_0_43_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_43_0);
  localWriteA_0_0_44_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_44_0);
  localWriteA_0_0_45_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_45_0);
  localWriteA_0_0_46_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_46_0);
  localWriteA_0_0_47_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_47_0);
  localWriteA_0_0_48_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_48_0);
  localWriteA_0_0_49_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_49_0);
  localWriteA_0_0_50_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_50_0);
  localWriteA_0_0_51_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_51_0);
  localWriteA_0_0_52_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_52_0);
  localWriteA_0_0_53_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_53_0);
  localWriteA_0_0_54_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_54_0);
  localWriteA_0_0_55_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_55_0);
  localWriteA_0_0_56_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_56_0);
  localWriteA_0_0_57_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_57_0);
  localWriteA_0_0_58_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_58_0);
  localWriteA_0_0_59_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_59_0);
  localWriteA_0_0_60_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_60_0);
  localWriteA_0_0_61_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_61_0);
  localWriteA_0_0_62_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_62_0);
  localWriteA_0_0_63_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_63_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);
    a_0_0_16_0 = *(globalReadA_0_0_16_0 + 0);
    a_0_0_17_0 = *(globalReadA_0_0_17_0 + 0);
    a_0_0_18_0 = *(globalReadA_0_0_18_0 + 0);
    a_0_0_19_0 = *(globalReadA_0_0_19_0 + 0);
    a_0_0_20_0 = *(globalReadA_0_0_20_0 + 0);
    a_0_0_21_0 = *(globalReadA_0_0_21_0 + 0);
    a_0_0_22_0 = *(globalReadA_0_0_22_0 + 0);
    a_0_0_23_0 = *(globalReadA_0_0_23_0 + 0);
    a_0_0_24_0 = *(globalReadA_0_0_24_0 + 0);
    a_0_0_25_0 = *(globalReadA_0_0_25_0 + 0);
    a_0_0_26_0 = *(globalReadA_0_0_26_0 + 0);
    a_0_0_27_0 = *(globalReadA_0_0_27_0 + 0);
    a_0_0_28_0 = *(globalReadA_0_0_28_0 + 0);
    a_0_0_29_0 = *(globalReadA_0_0_29_0 + 0);
    a_0_0_30_0 = *(globalReadA_0_0_30_0 + 0);
    a_0_0_31_0 = *(globalReadA_0_0_31_0 + 0);
    a_0_0_32_0 = *(globalReadA_0_0_32_0 + 0);
    a_0_0_33_0 = *(globalReadA_0_0_33_0 + 0);
    a_0_0_34_0 = *(globalReadA_0_0_34_0 + 0);
    a_0_0_35_0 = *(globalReadA_0_0_35_0 + 0);
    a_0_0_36_0 = *(globalReadA_0_0_36_0 + 0);
    a_0_0_37_0 = *(globalReadA_0_0_37_0 + 0);
    a_0_0_38_0 = *(globalReadA_0_0_38_0 + 0);
    a_0_0_39_0 = *(globalReadA_0_0_39_0 + 0);
    a_0_0_40_0 = *(globalReadA_0_0_40_0 + 0);
    a_0_0_41_0 = *(globalReadA_0_0_41_0 + 0);
    a_0_0_42_0 = *(globalReadA_0_0_42_0 + 0);
    a_0_0_43_0 = *(globalReadA_0_0_43_0 + 0);
    a_0_0_44_0 = *(globalReadA_0_0_44_0 + 0);
    a_0_0_45_0 = *(globalReadA_0_0_45_0 + 0);
    a_0_0_46_0 = *(globalReadA_0_0_46_0 + 0);
    a_0_0_47_0 = *(globalReadA_0_0_47_0 + 0);
    a_0_0_48_0 = *(globalReadA_0_0_48_0 + 0);
    a_0_0_49_0 = *(globalReadA_0_0_49_0 + 0);
    a_0_0_50_0 = *(globalReadA_0_0_50_0 + 0);
    a_0_0_51_0 = *(globalReadA_0_0_51_0 + 0);
    a_0_0_52_0 = *(globalReadA_0_0_52_0 + 0);
    a_0_0_53_0 = *(globalReadA_0_0_53_0 + 0);
    a_0_0_54_0 = *(globalReadA_0_0_54_0 + 0);
    a_0_0_55_0 = *(globalReadA_0_0_55_0 + 0);
    a_0_0_56_0 = *(globalReadA_0_0_56_0 + 0);
    a_0_0_57_0 = *(globalReadA_0_0_57_0 + 0);
    a_0_0_58_0 = *(globalReadA_0_0_58_0 + 0);
    a_0_0_59_0 = *(globalReadA_0_0_59_0 + 0);
    a_0_0_60_0 = *(globalReadA_0_0_60_0 + 0);
    a_0_0_61_0 = *(globalReadA_0_0_61_0 + 0);
    a_0_0_62_0 = *(globalReadA_0_0_62_0 + 0);
    a_0_0_63_0 = *(globalReadA_0_0_63_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    globalReadA_0_0_16_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_16_0) + 1*globalReadIncAL);
    globalReadA_0_0_17_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_17_0) + 1*globalReadIncAL);
    globalReadA_0_0_18_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_18_0) + 1*globalReadIncAL);
    globalReadA_0_0_19_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_19_0) + 1*globalReadIncAL);
    globalReadA_0_0_20_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_20_0) + 1*globalReadIncAL);
    globalReadA_0_0_21_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_21_0) + 1*globalReadIncAL);
    globalReadA_0_0_22_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_22_0) + 1*globalReadIncAL);
    globalReadA_0_0_23_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_23_0) + 1*globalReadIncAL);
    globalReadA_0_0_24_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_24_0) + 1*globalReadIncAL);
    globalReadA_0_0_25_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_25_0) + 1*globalReadIncAL);
    globalReadA_0_0_26_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_26_0) + 1*globalReadIncAL);
    globalReadA_0_0_27_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_27_0) + 1*globalReadIncAL);
    globalReadA_0_0_28_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_28_0) + 1*globalReadIncAL);
    globalReadA_0_0_29_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_29_0) + 1*globalReadIncAL);
    globalReadA_0_0_30_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_30_0) + 1*globalReadIncAL);
    globalReadA_0_0_31_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_31_0) + 1*globalReadIncAL);
    globalReadA_0_0_32_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_32_0) + 1*globalReadIncAL);
    globalReadA_0_0_33_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_33_0) + 1*globalReadIncAL);
    globalReadA_0_0_34_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_34_0) + 1*globalReadIncAL);
    globalReadA_0_0_35_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_35_0) + 1*globalReadIncAL);
    globalReadA_0_0_36_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_36_0) + 1*globalReadIncAL);
    globalReadA_0_0_37_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_37_0) + 1*globalReadIncAL);
    globalReadA_0_0_38_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_38_0) + 1*globalReadIncAL);
    globalReadA_0_0_39_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_39_0) + 1*globalReadIncAL);
    globalReadA_0_0_40_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_40_0) + 1*globalReadIncAL);
    globalReadA_0_0_41_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_41_0) + 1*globalReadIncAL);
    globalReadA_0_0_42_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_42_0) + 1*globalReadIncAL);
    globalReadA_0_0_43_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_43_0) + 1*globalReadIncAL);
    globalReadA_0_0_44_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_44_0) + 1*globalReadIncAL);
    globalReadA_0_0_45_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_45_0) + 1*globalReadIncAL);
    globalReadA_0_0_46_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_46_0) + 1*globalReadIncAL);
    globalReadA_0_0_47_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_47_0) + 1*globalReadIncAL);
    globalReadA_0_0_48_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_48_0) + 1*globalReadIncAL);
    globalReadA_0_0_49_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_49_0) + 1*globalReadIncAL);
    globalReadA_0_0_50_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_50_0) + 1*globalReadIncAL);
    globalReadA_0_0_51_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_51_0) + 1*globalReadIncAL);
    globalReadA_0_0_52_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_52_0) + 1*globalReadIncAL);
    globalReadA_0_0_53_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_53_0) + 1*globalReadIncAL);
    globalReadA_0_0_54_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_54_0) + 1*globalReadIncAL);
    globalReadA_0_0_55_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_55_0) + 1*globalReadIncAL);
    globalReadA_0_0_56_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_56_0) + 1*globalReadIncAL);
    globalReadA_0_0_57_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_57_0) + 1*globalReadIncAL);
    globalReadA_0_0_58_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_58_0) + 1*globalReadIncAL);
    globalReadA_0_0_59_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_59_0) + 1*globalReadIncAL);
    globalReadA_0_0_60_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_60_0) + 1*globalReadIncAL);
    globalReadA_0_0_61_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_61_0) + 1*globalReadIncAL);
    globalReadA_0_0_62_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_62_0) + 1*globalReadIncAL);
    globalReadA_0_0_63_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_63_0) + 1*globalReadIncAL);

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
    *(localWriteA_0_0_16_0 + 0) = a_0_0_16_0;
    *(localWriteA_0_0_17_0 + 0) = a_0_0_17_0;
    *(localWriteA_0_0_18_0 + 0) = a_0_0_18_0;
    *(localWriteA_0_0_19_0 + 0) = a_0_0_19_0;
    *(localWriteA_0_0_20_0 + 0) = a_0_0_20_0;
    *(localWriteA_0_0_21_0 + 0) = a_0_0_21_0;
    *(localWriteA_0_0_22_0 + 0) = a_0_0_22_0;
    *(localWriteA_0_0_23_0 + 0) = a_0_0_23_0;
    *(localWriteA_0_0_24_0 + 0) = a_0_0_24_0;
    *(localWriteA_0_0_25_0 + 0) = a_0_0_25_0;
    *(localWriteA_0_0_26_0 + 0) = a_0_0_26_0;
    *(localWriteA_0_0_27_0 + 0) = a_0_0_27_0;
    *(localWriteA_0_0_28_0 + 0) = a_0_0_28_0;
    *(localWriteA_0_0_29_0 + 0) = a_0_0_29_0;
    *(localWriteA_0_0_30_0 + 0) = a_0_0_30_0;
    *(localWriteA_0_0_31_0 + 0) = a_0_0_31_0;
    *(localWriteA_0_0_32_0 + 0) = a_0_0_32_0;
    *(localWriteA_0_0_33_0 + 0) = a_0_0_33_0;
    *(localWriteA_0_0_34_0 + 0) = a_0_0_34_0;
    *(localWriteA_0_0_35_0 + 0) = a_0_0_35_0;
    *(localWriteA_0_0_36_0 + 0) = a_0_0_36_0;
    *(localWriteA_0_0_37_0 + 0) = a_0_0_37_0;
    *(localWriteA_0_0_38_0 + 0) = a_0_0_38_0;
    *(localWriteA_0_0_39_0 + 0) = a_0_0_39_0;
    *(localWriteA_0_0_40_0 + 0) = a_0_0_40_0;
    *(localWriteA_0_0_41_0 + 0) = a_0_0_41_0;
    *(localWriteA_0_0_42_0 + 0) = a_0_0_42_0;
    *(localWriteA_0_0_43_0 + 0) = a_0_0_43_0;
    *(localWriteA_0_0_44_0 + 0) = a_0_0_44_0;
    *(localWriteA_0_0_45_0 + 0) = a_0_0_45_0;
    *(localWriteA_0_0_46_0 + 0) = a_0_0_46_0;
    *(localWriteA_0_0_47_0 + 0) = a_0_0_47_0;
    *(localWriteA_0_0_48_0 + 0) = a_0_0_48_0;
    *(localWriteA_0_0_49_0 + 0) = a_0_0_49_0;
    *(localWriteA_0_0_50_0 + 0) = a_0_0_50_0;
    *(localWriteA_0_0_51_0 + 0) = a_0_0_51_0;
    *(localWriteA_0_0_52_0 + 0) = a_0_0_52_0;
    *(localWriteA_0_0_53_0 + 0) = a_0_0_53_0;
    *(localWriteA_0_0_54_0 + 0) = a_0_0_54_0;
    *(localWriteA_0_0_55_0 + 0) = a_0_0_55_0;
    *(localWriteA_0_0_56_0 + 0) = a_0_0_56_0;
    *(localWriteA_0_0_57_0 + 0) = a_0_0_57_0;
    *(localWriteA_0_0_58_0 + 0) = a_0_0_58_0;
    *(localWriteA_0_0_59_0 + 0) = a_0_0_59_0;
    *(localWriteA_0_0_60_0 + 0) = a_0_0_60_0;
    *(localWriteA_0_0_61_0 + 0) = a_0_0_61_0;
    *(localWriteA_0_0_62_0 + 0) = a_0_0_62_0;
    *(localWriteA_0_0_63_0 + 0) = a_0_0_63_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 15 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 16 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 17 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 18 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 19 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 20 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 21 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 22 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 23 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 24 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 25 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 26 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 27 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 28 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 29 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 30 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 31 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 32 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 33 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 34 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 35 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 36 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 37 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 38 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 39 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 40 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 41 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 42 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 43 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 44 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 45 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 46 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 47 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 48 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 49 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 50 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 51 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 52 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 53 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 54 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 55 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 56 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 57 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 58 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 59 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 60 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 61 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 62 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_1x1


    /* iter 63 */

    MAC_1x1_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_0_8_0 = ( globalReadOffsetAL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_0_0_9_0 = ( globalReadOffsetAL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_0_0_10_0 = ( globalReadOffsetAL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_0_0_11_0 = ( globalReadOffsetAL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_0_0_12_0 = ( globalReadOffsetAL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_0_0_13_0 = ( globalReadOffsetAL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_0_0_14_0 = ( globalReadOffsetAL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_0_0_15_0 = ( globalReadOffsetAL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);
  a_0_0_16_0 = ( globalReadOffsetAL_16_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_16_0 + 0);
  a_0_0_17_0 = ( globalReadOffsetAL_17_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_17_0 + 0);
  a_0_0_18_0 = ( globalReadOffsetAL_18_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_18_0 + 0);
  a_0_0_19_0 = ( globalReadOffsetAL_19_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_19_0 + 0);
  a_0_0_20_0 = ( globalReadOffsetAL_20_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_20_0 + 0);
  a_0_0_21_0 = ( globalReadOffsetAL_21_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_21_0 + 0);
  a_0_0_22_0 = ( globalReadOffsetAL_22_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_22_0 + 0);
  a_0_0_23_0 = ( globalReadOffsetAL_23_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_23_0 + 0);
  a_0_0_24_0 = ( globalReadOffsetAL_24_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_24_0 + 0);
  a_0_0_25_0 = ( globalReadOffsetAL_25_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_25_0 + 0);
  a_0_0_26_0 = ( globalReadOffsetAL_26_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_26_0 + 0);
  a_0_0_27_0 = ( globalReadOffsetAL_27_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_27_0 + 0);
  a_0_0_28_0 = ( globalReadOffsetAL_28_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_28_0 + 0);
  a_0_0_29_0 = ( globalReadOffsetAL_29_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_29_0 + 0);
  a_0_0_30_0 = ( globalReadOffsetAL_30_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_30_0 + 0);
  a_0_0_31_0 = ( globalReadOffsetAL_31_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_31_0 + 0);
  a_0_0_32_0 = ( globalReadOffsetAL_32_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_32_0 + 0);
  a_0_0_33_0 = ( globalReadOffsetAL_33_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_33_0 + 0);
  a_0_0_34_0 = ( globalReadOffsetAL_34_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_34_0 + 0);
  a_0_0_35_0 = ( globalReadOffsetAL_35_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_35_0 + 0);
  a_0_0_36_0 = ( globalReadOffsetAL_36_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_36_0 + 0);
  a_0_0_37_0 = ( globalReadOffsetAL_37_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_37_0 + 0);
  a_0_0_38_0 = ( globalReadOffsetAL_38_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_38_0 + 0);
  a_0_0_39_0 = ( globalReadOffsetAL_39_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_39_0 + 0);
  a_0_0_40_0 = ( globalReadOffsetAL_40_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_40_0 + 0);
  a_0_0_41_0 = ( globalReadOffsetAL_41_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_41_0 + 0);
  a_0_0_42_0 = ( globalReadOffsetAL_42_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_42_0 + 0);
  a_0_0_43_0 = ( globalReadOffsetAL_43_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_43_0 + 0);
  a_0_0_44_0 = ( globalReadOffsetAL_44_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_44_0 + 0);
  a_0_0_45_0 = ( globalReadOffsetAL_45_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_45_0 + 0);
  a_0_0_46_0 = ( globalReadOffsetAL_46_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_46_0 + 0);
  a_0_0_47_0 = ( globalReadOffsetAL_47_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_47_0 + 0);
  a_0_0_48_0 = ( globalReadOffsetAL_48_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_48_0 + 0);
  a_0_0_49_0 = ( globalReadOffsetAL_49_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_49_0 + 0);
  a_0_0_50_0 = ( globalReadOffsetAL_50_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_50_0 + 0);
  a_0_0_51_0 = ( globalReadOffsetAL_51_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_51_0 + 0);
  a_0_0_52_0 = ( globalReadOffsetAL_52_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_52_0 + 0);
  a_0_0_53_0 = ( globalReadOffsetAL_53_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_53_0 + 0);
  a_0_0_54_0 = ( globalReadOffsetAL_54_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_54_0 + 0);
  a_0_0_55_0 = ( globalReadOffsetAL_55_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_55_0 + 0);
  a_0_0_56_0 = ( globalReadOffsetAL_56_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_56_0 + 0);
  a_0_0_57_0 = ( globalReadOffsetAL_57_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_57_0 + 0);
  a_0_0_58_0 = ( globalReadOffsetAL_58_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_58_0 + 0);
  a_0_0_59_0 = ( globalReadOffsetAL_59_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_59_0 + 0);
  a_0_0_60_0 = ( globalReadOffsetAL_60_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_60_0 + 0);
  a_0_0_61_0 = ( globalReadOffsetAL_61_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_61_0 + 0);
  a_0_0_62_0 = ( globalReadOffsetAL_62_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_62_0 + 0);
  a_0_0_63_0 = ( globalReadOffsetAL_63_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_63_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);
  localWriteA_0_0_16_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_16_0);
  localWriteA_0_0_17_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_17_0);
  localWriteA_0_0_18_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_18_0);
  localWriteA_0_0_19_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_19_0);
  localWriteA_0_0_20_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_20_0);
  localWriteA_0_0_21_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_21_0);
  localWriteA_0_0_22_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_22_0);
  localWriteA_0_0_23_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_23_0);
  localWriteA_0_0_24_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_24_0);
  localWriteA_0_0_25_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_25_0);
  localWriteA_0_0_26_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_26_0);
  localWriteA_0_0_27_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_27_0);
  localWriteA_0_0_28_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_28_0);
  localWriteA_0_0_29_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_29_0);
  localWriteA_0_0_30_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_30_0);
  localWriteA_0_0_31_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_31_0);
  localWriteA_0_0_32_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_32_0);
  localWriteA_0_0_33_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_33_0);
  localWriteA_0_0_34_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_34_0);
  localWriteA_0_0_35_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_35_0);
  localWriteA_0_0_36_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_36_0);
  localWriteA_0_0_37_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_37_0);
  localWriteA_0_0_38_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_38_0);
  localWriteA_0_0_39_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_39_0);
  localWriteA_0_0_40_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_40_0);
  localWriteA_0_0_41_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_41_0);
  localWriteA_0_0_42_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_42_0);
  localWriteA_0_0_43_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_43_0);
  localWriteA_0_0_44_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_44_0);
  localWriteA_0_0_45_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_45_0);
  localWriteA_0_0_46_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_46_0);
  localWriteA_0_0_47_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_47_0);
  localWriteA_0_0_48_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_48_0);
  localWriteA_0_0_49_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_49_0);
  localWriteA_0_0_50_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_50_0);
  localWriteA_0_0_51_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_51_0);
  localWriteA_0_0_52_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_52_0);
  localWriteA_0_0_53_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_53_0);
  localWriteA_0_0_54_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_54_0);
  localWriteA_0_0_55_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_55_0);
  localWriteA_0_0_56_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_56_0);
  localWriteA_0_0_57_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_57_0);
  localWriteA_0_0_58_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_58_0);
  localWriteA_0_0_59_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_59_0);
  localWriteA_0_0_60_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_60_0);
  localWriteA_0_0_61_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_61_0);
  localWriteA_0_0_62_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_62_0);
  localWriteA_0_0_63_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_63_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
  *(localWriteA_0_0_16_0 + 0) = a_0_0_16_0;
  *(localWriteA_0_0_17_0 + 0) = a_0_0_17_0;
  *(localWriteA_0_0_18_0 + 0) = a_0_0_18_0;
  *(localWriteA_0_0_19_0 + 0) = a_0_0_19_0;
  *(localWriteA_0_0_20_0 + 0) = a_0_0_20_0;
  *(localWriteA_0_0_21_0 + 0) = a_0_0_21_0;
  *(localWriteA_0_0_22_0 + 0) = a_0_0_22_0;
  *(localWriteA_0_0_23_0 + 0) = a_0_0_23_0;
  *(localWriteA_0_0_24_0 + 0) = a_0_0_24_0;
  *(localWriteA_0_0_25_0 + 0) = a_0_0_25_0;
  *(localWriteA_0_0_26_0 + 0) = a_0_0_26_0;
  *(localWriteA_0_0_27_0 + 0) = a_0_0_27_0;
  *(localWriteA_0_0_28_0 + 0) = a_0_0_28_0;
  *(localWriteA_0_0_29_0 + 0) = a_0_0_29_0;
  *(localWriteA_0_0_30_0 + 0) = a_0_0_30_0;
  *(localWriteA_0_0_31_0 + 0) = a_0_0_31_0;
  *(localWriteA_0_0_32_0 + 0) = a_0_0_32_0;
  *(localWriteA_0_0_33_0 + 0) = a_0_0_33_0;
  *(localWriteA_0_0_34_0 + 0) = a_0_0_34_0;
  *(localWriteA_0_0_35_0 + 0) = a_0_0_35_0;
  *(localWriteA_0_0_36_0 + 0) = a_0_0_36_0;
  *(localWriteA_0_0_37_0 + 0) = a_0_0_37_0;
  *(localWriteA_0_0_38_0 + 0) = a_0_0_38_0;
  *(localWriteA_0_0_39_0 + 0) = a_0_0_39_0;
  *(localWriteA_0_0_40_0 + 0) = a_0_0_40_0;
  *(localWriteA_0_0_41_0 + 0) = a_0_0_41_0;
  *(localWriteA_0_0_42_0 + 0) = a_0_0_42_0;
  *(localWriteA_0_0_43_0 + 0) = a_0_0_43_0;
  *(localWriteA_0_0_44_0 + 0) = a_0_0_44_0;
  *(localWriteA_0_0_45_0 + 0) = a_0_0_45_0;
  *(localWriteA_0_0_46_0 + 0) = a_0_0_46_0;
  *(localWriteA_0_0_47_0 + 0) = a_0_0_47_0;
  *(localWriteA_0_0_48_0 + 0) = a_0_0_48_0;
  *(localWriteA_0_0_49_0 + 0) = a_0_0_49_0;
  *(localWriteA_0_0_50_0 + 0) = a_0_0_50_0;
  *(localWriteA_0_0_51_0 + 0) = a_0_0_51_0;
  *(localWriteA_0_0_52_0 + 0) = a_0_0_52_0;
  *(localWriteA_0_0_53_0 + 0) = a_0_0_53_0;
  *(localWriteA_0_0_54_0 + 0) = a_0_0_54_0;
  *(localWriteA_0_0_55_0 + 0) = a_0_0_55_0;
  *(localWriteA_0_0_56_0 + 0) = a_0_0_56_0;
  *(localWriteA_0_0_57_0 + 0) = a_0_0_57_0;
  *(localWriteA_0_0_58_0 + 0) = a_0_0_58_0;
  *(localWriteA_0_0_59_0 + 0) = a_0_0_59_0;
  *(localWriteA_0_0_60_0 + 0) = a_0_0_60_0;
  *(localWriteA_0_0_61_0 + 0) = a_0_0_61_0;
  *(localWriteA_0_0_62_0 + 0) = a_0_0_62_0;
  *(localWriteA_0_0_63_0 + 0) = a_0_0_63_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_1x1

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_1x1
#undef MAC_1x1_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 64
#define SG1J 1
#define TT0I 1
#define TT1J 1
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 64
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 64
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 4096
#define LDS_NUM_ELEMENTS 4160

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 1x1 micro-tile */
#define MAC_1x1\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \

#define MAC_1x1_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x1x64_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU0_SUM0_SUS128_SRVW0_SVW4_SNLL0_TT1_1_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG64_1_1_WGM1(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE a_0_0_16_0;
  DATA_TYPE a_0_0_17_0;
  DATA_TYPE a_0_0_18_0;
  DATA_TYPE a_0_0_19_0;
  DATA_TYPE a_0_0_20_0;
  DATA_TYPE a_0_0_21_0;
  DATA_TYPE a_0_0_22_0;
  DATA_TYPE a_0_0_23_0;
  DATA_TYPE a_0_0_24_0;
  DATA_TYPE a_0_0_25_0;
  DATA_TYPE a_0_0_26_0;
  DATA_TYPE a_0_0_27_0;
  DATA_TYPE a_0_0_28_0;
  DATA_TYPE a_0_0_29_0;
  DATA_TYPE a_0_0_30_0;
  DATA_TYPE a_0_0_31_0;
  DATA_TYPE a_0_0_32_0;
  DATA_TYPE a_0_0_33_0;
  DATA_TYPE a_0_0_34_0;
  DATA_TYPE a_0_0_35_0;
  DATA_TYPE a_0_0_36_0;
  DATA_TYPE a_0_0_37_0;
  DATA_TYPE a_0_0_38_0;
  DATA_TYPE a_0_0_39_0;
  DATA_TYPE a_0_0_40_0;
  DATA_TYPE a_0_0_41_0;
  DATA_TYPE a_0_0_42_0;
  DATA_TYPE a_0_0_43_0;
  DATA_TYPE a_0_0_44_0;
  DATA_TYPE a_0_0_45_0;
  DATA_TYPE a_0_0_46_0;
  DATA_TYPE a_0_0_47_0;
  DATA_TYPE a_0_0_48_0;
  DATA_TYPE a_0_0_49_0;
  DATA_TYPE a_0_0_50_0;
  DATA_TYPE a_0_0_51_0;
  DATA_TYPE a_0_0_52_0;
  DATA_TYPE a_0_0_53_0;
  DATA_TYPE a_0_0_54_0;
  DATA_TYPE a_0_0_55_0;
  DATA_TYPE a_0_0_56_0;
  DATA_TYPE a_0_0_57_0;
  DATA_TYPE a_0_0_58_0;
  DATA_TYPE a_0_0_59_0;
  DATA_TYPE a_0_0_60_0;
  DATA_TYPE a_0_0_61_0;
  DATA_TYPE a_0_0_62_0;
  DATA_TYPE a_0_0_63_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;
  unsigned int globalReadOffsetAL_8_0 = globalReadOffsetAL + 0 + 8*LSPA;
  unsigned int globalReadOffsetAL_9_0 = globalReadOffsetAL + 0 + 9*LSPA;
  unsigned int globalReadOffsetAL_10_0 = globalReadOffsetAL + 0 + 10*LSPA;
  unsigned int globalReadOffsetAL_11_0 = globalReadOffsetAL + 0 + 11*LSPA;
  unsigned int globalReadOffsetAL_12_0 = globalReadOffsetAL + 0 + 12*LSPA;
  unsigned int globalReadOffsetAL_13_0 = globalReadOffsetAL + 0 + 13*LSPA;
  unsigned int globalReadOffsetAL_14_0 = globalReadOffsetAL + 0 + 14*LSPA;
  unsigned int globalReadOffsetAL_15_0 = globalReadOffsetAL + 0 + 15*LSPA;
  unsigned int globalReadOffsetAL_16_0 = globalReadOffsetAL + 0 + 16*LSPA;
  unsigned int globalReadOffsetAL_17_0 = globalReadOffsetAL + 0 + 17*LSPA;
  unsigned int globalReadOffsetAL_18_0 = globalReadOffsetAL + 0 + 18*LSPA;
  unsigned int globalReadOffsetAL_19_0 = globalReadOffsetAL + 0 + 19*LSPA;
  unsigned int globalReadOffsetAL_20_0 = globalReadOffsetAL + 0 + 20*LSPA;
  unsigned int globalReadOffsetAL_21_0 = globalReadOffsetAL + 0 + 21*LSPA;
  unsigned int globalReadOffsetAL_22_0 = globalReadOffsetAL + 0 + 22*LSPA;
  unsigned int globalReadOffsetAL_23_0 = globalReadOffsetAL + 0 + 23*LSPA;
  unsigned int globalReadOffsetAL_24_0 = globalReadOffsetAL + 0 + 24*LSPA;
  unsigned int globalReadOffsetAL_25_0 = globalReadOffsetAL + 0 + 25*LSPA;
  unsigned int globalReadOffsetAL_26_0 = globalReadOffsetAL + 0 + 26*LSPA;
  unsigned int globalReadOffsetAL_27_0 = globalReadOffsetAL + 0 + 27*LSPA;
  unsigned int globalReadOffsetAL_28_0 = globalReadOffsetAL + 0 + 28*LSPA;
  unsigned int globalReadOffsetAL_29_0 = globalReadOffsetAL + 0 + 29*LSPA;
  unsigned int globalReadOffsetAL_30_0 = globalReadOffsetAL + 0 + 30*LSPA;
  unsigned int globalReadOffsetAL_31_0 = globalReadOffsetAL + 0 + 31*LSPA;
  unsigned int globalReadOffsetAL_32_0 = globalReadOffsetAL + 0 + 32*LSPA;
  unsigned int globalReadOffsetAL_33_0 = globalReadOffsetAL + 0 + 33*LSPA;
  unsigned int globalReadOffsetAL_34_0 = globalReadOffsetAL + 0 + 34*LSPA;
  unsigned int globalReadOffsetAL_35_0 = globalReadOffsetAL + 0 + 35*LSPA;
  unsigned int globalReadOffsetAL_36_0 = globalReadOffsetAL + 0 + 36*LSPA;
  unsigned int globalReadOffsetAL_37_0 = globalReadOffsetAL + 0 + 37*LSPA;
  unsigned int globalReadOffsetAL_38_0 = globalReadOffsetAL + 0 + 38*LSPA;
  unsigned int globalReadOffsetAL_39_0 = globalReadOffsetAL + 0 + 39*LSPA;
  unsigned int globalReadOffsetAL_40_0 = globalReadOffsetAL + 0 + 40*LSPA;
  unsigned int globalReadOffsetAL_41_0 = globalReadOffsetAL + 0 + 41*LSPA;
  unsigned int globalReadOffsetAL_42_0 = globalReadOffsetAL + 0 + 42*LSPA;
  unsigned int globalReadOffsetAL_43_0 = globalReadOffsetAL + 0 + 43*LSPA;
  unsigned int globalReadOffsetAL_44_0 = globalReadOffsetAL + 0 + 44*LSPA;
  unsigned int globalReadOffsetAL_45_0 = globalReadOffsetAL + 0 + 45*LSPA;
  unsigned int globalReadOffsetAL_46_0 = globalReadOffsetAL + 0 + 46*LSPA;
  unsigned int globalReadOffsetAL_47_0 = globalReadOffsetAL + 0 + 47*LSPA;
  unsigned int globalReadOffsetAL_48_0 = globalReadOffsetAL + 0 + 48*LSPA;
  unsigned int globalReadOffsetAL_49_0 = globalReadOffsetAL + 0 + 49*LSPA;
  unsigned int globalReadOffsetAL_50_0 = globalReadOffsetAL + 0 + 50*LSPA;
  unsigned int globalReadOffsetAL_51_0 = globalReadOffsetAL + 0 + 51*LSPA;
  unsigned int globalReadOffsetAL_52_0 = globalReadOffsetAL + 0 + 52*LSPA;
  unsigned int globalReadOffsetAL_53_0 = globalReadOffsetAL + 0 + 53*LSPA;
  unsigned int globalReadOffsetAL_54_0 = globalReadOffsetAL + 0 + 54*LSPA;
  unsigned int globalReadOffsetAL_55_0 = globalReadOffsetAL + 0 + 55*LSPA;
  unsigned int globalReadOffsetAL_56_0 = globalReadOffsetAL + 0 + 56*LSPA;
  unsigned int globalReadOffsetAL_57_0 = globalReadOffsetAL + 0 + 57*LSPA;
  unsigned int globalReadOffsetAL_58_0 = globalReadOffsetAL + 0 + 58*LSPA;
  unsigned int globalReadOffsetAL_59_0 = globalReadOffsetAL + 0 + 59*LSPA;
  unsigned int globalReadOffsetAL_60_0 = globalReadOffsetAL + 0 + 60*LSPA;
  unsigned int globalReadOffsetAL_61_0 = globalReadOffsetAL + 0 + 61*LSPA;
  unsigned int globalReadOffsetAL_62_0 = globalReadOffsetAL + 0 + 62*LSPA;
  unsigned int globalReadOffsetAL_63_0 = globalReadOffsetAL + 0 + 63*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_15_0), wgK );
  int64_t globalReadOffsetA_0_0_16_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_16_0), wgK );
  int64_t globalReadOffsetA_0_0_17_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_17_0), wgK );
  int64_t globalReadOffsetA_0_0_18_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_18_0), wgK );
  int64_t globalReadOffsetA_0_0_19_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_19_0), wgK );
  int64_t globalReadOffsetA_0_0_20_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_20_0), wgK );
  int64_t globalReadOffsetA_0_0_21_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_21_0), wgK );
  int64_t globalReadOffsetA_0_0_22_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_22_0), wgK );
  int64_t globalReadOffsetA_0_0_23_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_23_0), wgK );
  int64_t globalReadOffsetA_0_0_24_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_24_0), wgK );
  int64_t globalReadOffsetA_0_0_25_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_25_0), wgK );
  int64_t globalReadOffsetA_0_0_26_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_26_0), wgK );
  int64_t globalReadOffsetA_0_0_27_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_27_0), wgK );
  int64_t globalReadOffsetA_0_0_28_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_28_0), wgK );
  int64_t globalReadOffsetA_0_0_29_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_29_0), wgK );
  int64_t globalReadOffsetA_0_0_30_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_30_0), wgK );
  int64_t globalReadOffsetA_0_0_31_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_31_0), wgK );
  int64_t globalReadOffsetA_0_0_32_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_32_0), wgK );
  int64_t globalReadOffsetA_0_0_33_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_33_0), wgK );
  int64_t globalReadOffsetA_0_0_34_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_34_0), wgK );
  int64_t globalReadOffsetA_0_0_35_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_35_0), wgK );
  int64_t globalReadOffsetA_0_0_36_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_36_0), wgK );
  int64_t globalReadOffsetA_0_0_37_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_37_0), wgK );
  int64_t globalReadOffsetA_0_0_38_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_38_0), wgK );
  int64_t globalReadOffsetA_0_0_39_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_39_0), wgK );
  int64_t globalReadOffsetA_0_0_40_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_40_0), wgK );
  int64_t globalReadOffsetA_0_0_41_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_41_0), wgK );
  int64_t globalReadOffsetA_0_0_42_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_42_0), wgK );
  int64_t globalReadOffsetA_0_0_43_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_43_0), wgK );
  int64_t globalReadOffsetA_0_0_44_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_44_0), wgK );
  int64_t globalReadOffsetA_0_0_45_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_45_0), wgK );
  int64_t globalReadOffsetA_0_0_46_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_46_0), wgK );
  int64_t globalReadOffsetA_0_0_47_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_47_0), wgK );
  int64_t globalReadOffsetA_0_0_48_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_48_0), wgK );
  int64_t globalReadOffsetA_0_0_49_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_49_0), wgK );
  int64_t globalReadOffsetA_0_0_50_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_50_0), wgK );
  int64_t globalReadOffsetA_0_0_51_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_51_0), wgK );
  int64_t globalReadOffsetA_0_0_52_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_52_0), wgK );
  int64_t globalReadOffsetA_0_0_53_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_53_0), wgK );
  int64_t globalReadOffsetA_0_0_54_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_54_0), wgK );
  int64_t globalReadOffsetA_0_0_55_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_55_0), wgK );
  int64_t globalReadOffsetA_0_0_56_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_56_0), wgK );
  int64_t globalReadOffsetA_0_0_57_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_57_0), wgK );
  int64_t globalReadOffsetA_0_0_58_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_58_0), wgK );
  int64_t globalReadOffsetA_0_0_59_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_59_0), wgK );
  int64_t globalReadOffsetA_0_0_60_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_60_0), wgK );
  int64_t globalReadOffsetA_0_0_61_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_61_0), wgK );
  int64_t globalReadOffsetA_0_0_62_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_62_0), wgK );
  int64_t globalReadOffsetA_0_0_63_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_63_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;
  DATA_TYPE const *globalReadA_0_0_16_0 = A + globalReadOffsetA_0_0_16_0;
  DATA_TYPE const *globalReadA_0_0_17_0 = A + globalReadOffsetA_0_0_17_0;
  DATA_TYPE const *globalReadA_0_0_18_0 = A + globalReadOffsetA_0_0_18_0;
  DATA_TYPE const *globalReadA_0_0_19_0 = A + globalReadOffsetA_0_0_19_0;
  DATA_TYPE const *globalReadA_0_0_20_0 = A + globalReadOffsetA_0_0_20_0;
  DATA_TYPE const *globalReadA_0_0_21_0 = A + globalReadOffsetA_0_0_21_0;
  DATA_TYPE const *globalReadA_0_0_22_0 = A + globalReadOffsetA_0_0_22_0;
  DATA_TYPE const *globalReadA_0_0_23_0 = A + globalReadOffsetA_0_0_23_0;
  DATA_TYPE const *globalReadA_0_0_24_0 = A + globalReadOffsetA_0_0_24_0;
  DATA_TYPE const *globalReadA_0_0_25_0 = A + globalReadOffsetA_0_0_25_0;
  DATA_TYPE const *globalReadA_0_0_26_0 = A + globalReadOffsetA_0_0_26_0;
  DATA_TYPE const *globalReadA_0_0_27_0 = A + globalReadOffsetA_0_0_27_0;
  DATA_TYPE const *globalReadA_0_0_28_0 = A + globalReadOffsetA_0_0_28_0;
  DATA_TYPE const *globalReadA_0_0_29_0 = A + globalReadOffsetA_0_0_29_0;
  DATA_TYPE const *globalReadA_0_0_30_0 = A + globalReadOffsetA_0_0_30_0;
  DATA_TYPE const *globalReadA_0_0_31_0 = A + globalReadOffsetA_0_0_31_0;
  DATA_TYPE const *globalReadA_0_0_32_0 = A + globalReadOffsetA_0_0_32_0;
  DATA_TYPE const *globalReadA_0_0_33_0 = A + globalReadOffsetA_0_0_33_0;
  DATA_TYPE const *globalReadA_0_0_34_0 = A + globalReadOffsetA_0_0_34_0;
  DATA_TYPE const *globalReadA_0_0_35_0 = A + globalReadOffsetA_0_0_35_0;
  DATA_TYPE const *globalReadA_0_0_36_0 = A + globalReadOffsetA_0_0_36_0;
  DATA_TYPE const *globalReadA_0_0_37_0 = A + globalReadOffsetA_0_0_37_0;
  DATA_TYPE const *globalReadA_0_0_38_0 = A + globalReadOffsetA_0_0_38_0;
  DATA_TYPE const *globalReadA_0_0_39_0 = A + globalReadOffsetA_0_0_39_0;
  DATA_TYPE const *globalReadA_0_0_40_0 = A + globalReadOffsetA_0_0_40_0;
  DATA_TYPE const *globalReadA_0_0_41_0 = A + globalReadOffsetA_0_0_41_0;
  DATA_TYPE const *globalReadA_0_0_42_0 = A + globalReadOffsetA_0_0_42_0;
  DATA_TYPE const *globalReadA_0_0_43_0 = A + globalReadOffsetA_0_0_43_0;
  DATA_TYPE const *globalReadA_0_0_44_0 = A + globalReadOffsetA_0_0_44_0;
  DATA_TYPE const *globalReadA_0_0_45_0 = A + globalReadOffsetA_0_0_45_0;
  DATA_TYPE const *globalReadA_0_0_46_0 = A + globalReadOffsetA_0_0_46_0;
  DATA_TYPE const *globalReadA_0_0_47_0 = A + globalReadOffsetA_0_0_47_0;
  DATA_TYPE const *globalReadA_0_0_48_0 = A + globalReadOffsetA_0_0_48_0;
  DATA_TYPE const *globalReadA_0_0_49_0 = A + globalReadOffsetA_0_0_49_0;
  DATA_TYPE const *globalReadA_0_0_50_0 = A + globalReadOffsetA_0_0_50_0;
  DATA_TYPE const *globalReadA_0_0_51_0 = A + globalReadOffsetA_0_0_51_0;
  DATA_TYPE const *globalReadA_0_0_52_0 = A + globalReadOffsetA_0_0_52_0;
  DATA_TYPE const *globalReadA_0_0_53_0 = A + globalReadOffsetA_0_0_53_0;
  DATA_TYPE const *globalReadA_0_0_54_0 = A + globalReadOffsetA_0_0_54_0;
  DATA_TYPE const *globalReadA_0_0_55_0 = A + globalReadOffsetA_0_0_55_0;
  DATA_TYPE const *globalReadA_0_0_56_0 = A + globalReadOffsetA_0_0_56_0;
  DATA_TYPE const *globalReadA_0_0_57_0 = A + globalReadOffsetA_0_0_57_0;
  DATA_TYPE const *globalReadA_0_0_58_0 = A + globalReadOffsetA_0_0_58_0;
  DATA_TYPE const *globalReadA_0_0_59_0 = A + globalReadOffsetA_0_0_59_0;
  DATA_TYPE const *globalReadA_0_0_60_0 = A + globalReadOffsetA_0_0_60_0;
  DATA_TYPE const *globalReadA_0_0_61_0 = A + globalReadOffsetA_0_0_61_0;
  DATA_TYPE const *globalReadA_0_0_62_0 = A + globalReadOffsetA_0_0_62_0;
  DATA_TYPE const *globalReadA_0_0_63_0 = A + globalReadOffsetA_0_0_63_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 8*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 9*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 10*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 11*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 12*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 13*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 14*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 15*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_16_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 16*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_17_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 17*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_18_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 18*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_19_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 19*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_20_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 20*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_21_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 21*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_22_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 22*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_23_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 23*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_24_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 24*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_25_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 25*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_26_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 26*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_27_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 27*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_28_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 28*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_29_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 29*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_30_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 30*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_31_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 31*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_32_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 32*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_33_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 33*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_34_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 34*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_35_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 35*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_36_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 36*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_37_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 37*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_38_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 38*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_39_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 39*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_40_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 40*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_41_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 41*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_42_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 42*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_43_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 43*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_44_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 44*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_45_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 45*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_46_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 46*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_47_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 47*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_48_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 48*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_49_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 49*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_50_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 50*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_51_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 51*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_52_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 52*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_53_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 53*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_54_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 54*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_55_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 55*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_56_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 56*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_57_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 57*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_58_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 58*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_59_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 59*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_60_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 60*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_61_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 61*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_62_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 62*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_63_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 63*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_0_0_15_0;
  DATA_TYPE *localWriteA_0_0_16_0;
  DATA_TYPE *localWriteA_0_0_17_0;
  DATA_TYPE *localWriteA_0_0_18_0;
  DATA_TYPE *localWriteA_0_0_19_0;
  DATA_TYPE *localWriteA_0_0_20_0;
  DATA_TYPE *localWriteA_0_0_21_0;
  DATA_TYPE *localWriteA_0_0_22_0;
  DATA_TYPE *localWriteA_0_0_23_0;
  DATA_TYPE *localWriteA_0_0_24_0;
  DATA_TYPE *localWriteA_0_0_25_0;
  DATA_TYPE *localWriteA_0_0_26_0;
  DATA_TYPE *localWriteA_0_0_27_0;
  DATA_TYPE *localWriteA_0_0_28_0;
  DATA_TYPE *localWriteA_0_0_29_0;
  DATA_TYPE *localWriteA_0_0_30_0;
  DATA_TYPE *localWriteA_0_0_31_0;
  DATA_TYPE *localWriteA_0_0_32_0;
  DATA_TYPE *localWriteA_0_0_33_0;
  DATA_TYPE *localWriteA_0_0_34_0;
  DATA_TYPE *localWriteA_0_0_35_0;
  DATA_TYPE *localWriteA_0_0_36_0;
  DATA_TYPE *localWriteA_0_0_37_0;
  DATA_TYPE *localWriteA_0_0_38_0;
  DATA_TYPE *localWriteA_0_0_39_0;
  DATA_TYPE *localWriteA_0_0_40_0;
  DATA_TYPE *localWriteA_0_0_41_0;
  DATA_TYPE *localWriteA_0_0_42_0;
  DATA_TYPE *localWriteA_0_0_43_0;
  DATA_TYPE *localWriteA_0_0_44_0;
  DATA_TYPE *localWriteA_0_0_45_0;
  DATA_TYPE *localWriteA_0_0_46_0;
  DATA_TYPE *localWriteA_0_0_47_0;
  DATA_TYPE *localWriteA_0_0_48_0;
  DATA_TYPE *localWriteA_0_0_49_0;
  DATA_TYPE *localWriteA_0_0_50_0;
  DATA_TYPE *localWriteA_0_0_51_0;
  DATA_TYPE *localWriteA_0_0_52_0;
  DATA_TYPE *localWriteA_0_0_53_0;
  DATA_TYPE *localWriteA_0_0_54_0;
  DATA_TYPE *localWriteA_0_0_55_0;
  DATA_TYPE *localWriteA_0_0_56_0;
  DATA_TYPE *localWriteA_0_0_57_0;
  DATA_TYPE *localWriteA_0_0_58_0;
  DATA_TYPE *localWriteA_0_0_59_0;
  DATA_TYPE *localWriteA_0_0_60_0;
  DATA_TYPE *localWriteA_0_0_61_0;
  DATA_TYPE *localWriteA_0_0_62_0;
  DATA_TYPE *localWriteA_0_0_63_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);
  localWriteA_0_0_16_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_16_0);
  localWriteA_0_0_17_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_17_0);
  localWriteA_0_0_18_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_18_0);
  localWriteA_0_0_19_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_19_0);
  localWriteA_0_0_20_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_20_0);
  localWriteA_0_0_21_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_21_0);
  localWriteA_0_0_22_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_22_0);
  localWriteA_0_0_23_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_23_0);
  localWriteA_0_0_24_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_24_0);
  localWriteA_0_0_25_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_25_0);
  localWriteA_0_0_26_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_26_0);
  localWriteA_0_0_27_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_27_0);
  localWriteA_0_0_28_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_28_0);
  localWriteA_0_0_29_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_29_0);
  localWriteA_0_0_30_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_30_0);
  localWriteA_0_0_31_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_31_0);
  localWriteA_0_0_32_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_32_0);
  localWriteA_0_0_33_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_33_0);
  localWriteA_0_0_34_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_34_0);
  localWriteA_0_0_35_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_35_0);
  localWriteA_0_0_36_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_36_0);
  localWriteA_0_0_37_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_37_0);
  localWriteA_0_0_38_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_38_0);
  localWriteA_0_0_39_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_39_0);
  localWriteA_0_0_40_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_40_0);
  localWriteA_0_0_41_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_41_0);
  localWriteA_0_0_42_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_42_0);
  localWriteA_0_0_43_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_43_0);
  localWriteA_0_0_44_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_44_0);
  localWriteA_0_0_45_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_45_0);
  localWriteA_0_0_46_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_46_0);
  localWriteA_0_0_47_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_47_0);
  localWriteA_0_0_48_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_48_0);
  localWriteA_0_0_49_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_49_0);
  localWriteA_0_0_50_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_50_0);
  localWriteA_0_0_51_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_51_0);
  localWriteA_0_0_52_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_52_0);
  localWriteA_0_0_53_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_53_0);
  localWriteA_0_0_54_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_54_0);
  localWriteA_0_0_55_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_55_0);
  localWriteA_0_0_56_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_56_0);
  localWriteA_0_0_57_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_57_0);
  localWriteA_0_0_58_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_58_0);
  localWriteA_0_0_59_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_59_0);
  localWriteA_0_0_60_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_60_0);
  localWriteA_0_0_61_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_61_0);
  localWriteA_0_0_62_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_62_0);
  localWriteA_0_0_63_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_63_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);
    a_0_0_16_0 = *(globalReadA_0_0_16_0 + 0);
    a_0_0_17_0 = *(globalReadA_0_0_17_0 + 0);
    a_0_0_18_0 = *(globalReadA_0_0_18_0 + 0);
    a_0_0_19_0 = *(globalReadA_0_0_19_0 + 0);
    a_0_0_20_0 = *(globalReadA_0_0_20_0 + 0);
    a_0_0_21_0 = *(globalReadA_0_0_21_0 + 0);
    a_0_0_22_0 = *(globalReadA_0_0_22_0 + 0);
    a_0_0_23_0 = *(globalReadA_0_0_23_0 + 0);
    a_0_0_24_0 = *(globalReadA_0_0_24_0 + 0);
    a_0_0_25_0 = *(globalReadA_0_0_25_0 + 0);
    a_0_0_26_0 = *(globalReadA_0_0_26_0 + 0);
    a_0_0_27_0 = *(globalReadA_0_0_27_0 + 0);
    a_0_0_28_0 = *(globalReadA_0_0_28_0 + 0);
    a_0_0_29_0 = *(globalReadA_0_0_29_0 + 0);
    a_0_0_30_0 = *(globalReadA_0_0_30_0 + 0);
    a_0_0_31_0 = *(globalReadA_0_0_31_0 + 0);
    a_0_0_32_0 = *(globalReadA_0_0_32_0 + 0);
    a_0_0_33_0 = *(globalReadA_0_0_33_0 + 0);
    a_0_0_34_0 = *(globalReadA_0_0_34_0 + 0);
    a_0_0_35_0 = *(globalReadA_0_0_35_0 + 0);
    a_0_0_36_0 = *(globalReadA_0_0_36_0 + 0);
    a_0_0_37_0 = *(globalReadA_0_0_37_0 + 0);
    a_0_0_38_0 = *(globalReadA_0_0_38_0 + 0);
    a_0_0_39_0 = *(globalReadA_0_0_39_0 + 0);
    a_0_0_40_0 = *(globalReadA_0_0_40_0 + 0);
    a_0_0_41_0 = *(globalReadA_0_0_41_0 + 0);
    a_0_0_42_0 = *(globalReadA_0_0_42_0 + 0);
    a_0_0_43_0 = *(globalReadA_0_0_43_0 + 0);
    a_0_0_44_0 = *(globalReadA_0_0_44_0 + 0);
    a_0_0_45_0 = *(globalReadA_0_0_45_0 + 0);
    a_0_0_46_0 = *(globalReadA_0_0_46_0 + 0);
    a_0_0_47_0 = *(globalReadA_0_0_47_0 + 0);
    a_0_0_48_0 = *(globalReadA_0_0_48_0 + 0);
    a_0_0_49_0 = *(globalReadA_0_0_49_0 + 0);
    a_0_0_50_0 = *(globalReadA_0_0_50_0 + 0);
    a_0_0_51_0 = *(globalReadA_0_0_51_0 + 0);
    a_0_0_52_0 = *(globalReadA_0_0_52_0 + 0);
    a_0_0_53_0 = *(globalReadA_0_0_53_0 + 0);
    a_0_0_54_0 = *(globalReadA_0_0_54_0 + 0);
    a_0_0_55_0 = *(globalReadA_0_0_55_0 + 0);
    a_0_0_56_0 = *(globalReadA_0_0_56_0 + 0);
    a_0_0_57_0 = *(globalReadA_0_0_57_0 + 0);
    a_0_0_58_0 = *(globalReadA_0_0_58_0 + 0);
    a_0_0_59_0 = *(globalReadA_0_0_59_0 + 0);
    a_0_0_60_0 = *(globalReadA_0_0_60_0 + 0);
    a_0_0_61_0 = *(globalReadA_0_0_61_0 + 0);
    a_0_0_62_0 = *(globalReadA_0_0_62_0 + 0);
    a_0_0_63_0 = *(globalReadA_0_0_63_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    globalReadA_0_0_16_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_16_0) + 1*globalReadIncAL);
    globalReadA_0_0_17_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_17_0) + 1*globalReadIncAL);
    globalReadA_0_0_18_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_18_0) + 1*globalReadIncAL);
    globalReadA_0_0_19_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_19_0) + 1*globalReadIncAL);
    globalReadA_0_0_20_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_20_0) + 1*globalReadIncAL);
    globalReadA_0_0_21_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_21_0) + 1*globalReadIncAL);
    globalReadA_0_0_22_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_22_0) + 1*globalReadIncAL);
    globalReadA_0_0_23_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_23_0) + 1*globalReadIncAL);
    globalReadA_0_0_24_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_24_0) + 1*globalReadIncAL);
    globalReadA_0_0_25_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_25_0) + 1*globalReadIncAL);
    globalReadA_0_0_26_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_26_0) + 1*globalReadIncAL);
    globalReadA_0_0_27_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_27_0) + 1*globalReadIncAL);
    globalReadA_0_0_28_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_28_0) + 1*globalReadIncAL);
    globalReadA_0_0_29_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_29_0) + 1*globalReadIncAL);
    globalReadA_0_0_30_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_30_0) + 1*globalReadIncAL);
    globalReadA_0_0_31_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_31_0) + 1*globalReadIncAL);
    globalReadA_0_0_32_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_32_0) + 1*globalReadIncAL);
    globalReadA_0_0_33_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_33_0) + 1*globalReadIncAL);
    globalReadA_0_0_34_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_34_0) + 1*globalReadIncAL);
    globalReadA_0_0_35_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_35_0) + 1*globalReadIncAL);
    globalReadA_0_0_36_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_36_0) + 1*globalReadIncAL);
    globalReadA_0_0_37_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_37_0) + 1*globalReadIncAL);
    globalReadA_0_0_38_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_38_0) + 1*globalReadIncAL);
    globalReadA_0_0_39_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_39_0) + 1*globalReadIncAL);
    globalReadA_0_0_40_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_40_0) + 1*globalReadIncAL);
    globalReadA_0_0_41_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_41_0) + 1*globalReadIncAL);
    globalReadA_0_0_42_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_42_0) + 1*globalReadIncAL);
    globalReadA_0_0_43_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_43_0) + 1*globalReadIncAL);
    globalReadA_0_0_44_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_44_0) + 1*globalReadIncAL);
    globalReadA_0_0_45_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_45_0) + 1*globalReadIncAL);
    globalReadA_0_0_46_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_46_0) + 1*globalReadIncAL);
    globalReadA_0_0_47_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_47_0) + 1*globalReadIncAL);
    globalReadA_0_0_48_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_48_0) + 1*globalReadIncAL);
    globalReadA_0_0_49_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_49_0) + 1*globalReadIncAL);
    globalReadA_0_0_50_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_50_0) + 1*globalReadIncAL);
    globalReadA_0_0_51_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_51_0) + 1*globalReadIncAL);
    globalReadA_0_0_52_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_52_0) + 1*globalReadIncAL);
    globalReadA_0_0_53_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_53_0) + 1*globalReadIncAL);
    globalReadA_0_0_54_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_54_0) + 1*globalReadIncAL);
    globalReadA_0_0_55_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_55_0) + 1*globalReadIncAL);
    globalReadA_0_0_56_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_56_0) + 1*globalReadIncAL);
    globalReadA_0_0_57_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_57_0) + 1*globalReadIncAL);
    globalReadA_0_0_58_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_58_0) + 1*globalReadIncAL);
    globalReadA_0_0_59_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_59_0) + 1*globalReadIncAL);
    globalReadA_0_0_60_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_60_0) + 1*globalReadIncAL);
    globalReadA_0_0_61_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_61_0) + 1*globalReadIncAL);
    globalReadA_0_0_62_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_62_0) + 1*globalReadIncAL);
    globalReadA_0_0_63_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_63_0) + 1*globalReadIncAL);

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
    *(localWriteA_0_0_16_0 + 0) = a_0_0_16_0;
    *(localWriteA_0_0_17_0 + 0) = a_0_0_17_0;
    *(localWriteA_0_0_18_0 + 0) = a_0_0_18_0;
    *(localWriteA_0_0_19_0 + 0) = a_0_0_19_0;
    *(localWriteA_0_0_20_0 + 0) = a_0_0_20_0;
    *(localWriteA_0_0_21_0 + 0) = a_0_0_21_0;
    *(localWriteA_0_0_22_0 + 0) = a_0_0_22_0;
    *(localWriteA_0_0_23_0 + 0) = a_0_0_23_0;
    *(localWriteA_0_0_24_0 + 0) = a_0_0_24_0;
    *(localWriteA_0_0_25_0 + 0) = a_0_0_25_0;
    *(localWriteA_0_0_26_0 + 0) = a_0_0_26_0;
    *(localWriteA_0_0_27_0 + 0) = a_0_0_27_0;
    *(localWriteA_0_0_28_0 + 0) = a_0_0_28_0;
    *(localWriteA_0_0_29_0 + 0) = a_0_0_29_0;
    *(localWriteA_0_0_30_0 + 0) = a_0_0_30_0;
    *(localWriteA_0_0_31_0 + 0) = a_0_0_31_0;
    *(localWriteA_0_0_32_0 + 0) = a_0_0_32_0;
    *(localWriteA_0_0_33_0 + 0) = a_0_0_33_0;
    *(localWriteA_0_0_34_0 + 0) = a_0_0_34_0;
    *(localWriteA_0_0_35_0 + 0) = a_0_0_35_0;
    *(localWriteA_0_0_36_0 + 0) = a_0_0_36_0;
    *(localWriteA_0_0_37_0 + 0) = a_0_0_37_0;
    *(localWriteA_0_0_38_0 + 0) = a_0_0_38_0;
    *(localWriteA_0_0_39_0 + 0) = a_0_0_39_0;
    *(localWriteA_0_0_40_0 + 0) = a_0_0_40_0;
    *(localWriteA_0_0_41_0 + 0) = a_0_0_41_0;
    *(localWriteA_0_0_42_0 + 0) = a_0_0_42_0;
    *(localWriteA_0_0_43_0 + 0) = a_0_0_43_0;
    *(localWriteA_0_0_44_0 + 0) = a_0_0_44_0;
    *(localWriteA_0_0_45_0 + 0) = a_0_0_45_0;
    *(localWriteA_0_0_46_0 + 0) = a_0_0_46_0;
    *(localWriteA_0_0_47_0 + 0) = a_0_0_47_0;
    *(localWriteA_0_0_48_0 + 0) = a_0_0_48_0;
    *(localWriteA_0_0_49_0 + 0) = a_0_0_49_0;
    *(localWriteA_0_0_50_0 + 0) = a_0_0_50_0;
    *(localWriteA_0_0_51_0 + 0) = a_0_0_51_0;
    *(localWriteA_0_0_52_0 + 0) = a_0_0_52_0;
    *(localWriteA_0_0_53_0 + 0) = a_0_0_53_0;
    *(localWriteA_0_0_54_0 + 0) = a_0_0_54_0;
    *(localWriteA_0_0_55_0 + 0) = a_0_0_55_0;
    *(localWriteA_0_0_56_0 + 0) = a_0_0_56_0;
    *(localWriteA_0_0_57_0 + 0) = a_0_0_57_0;
    *(localWriteA_0_0_58_0 + 0) = a_0_0_58_0;
    *(localWriteA_0_0_59_0 + 0) = a_0_0_59_0;
    *(localWriteA_0_0_60_0 + 0) = a_0_0_60_0;
    *(localWriteA_0_0_61_0 + 0) = a_0_0_61_0;
    *(localWriteA_0_0_62_0 + 0) = a_0_0_62_0;
    *(localWriteA_0_0_63_0 + 0) = a_0_0_63_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 15 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 16 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 17 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 18 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 19 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 20 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 21 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 22 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 23 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 24 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 25 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 26 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 27 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 28 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 29 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 30 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 31 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 32 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 33 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 34 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 35 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 36 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 37 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 38 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 39 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 40 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 41 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 42 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 43 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 44 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 45 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 46 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 47 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 48 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 49 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 50 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 51 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 52 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 53 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 54 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 55 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 56 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 57 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 58 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 59 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 60 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 61 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 62 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_1x1


    /* iter 63 */

    MAC_1x1_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_0_8_0 = ( globalReadOffsetAL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_0_0_9_0 = ( globalReadOffsetAL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_0_0_10_0 = ( globalReadOffsetAL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_0_0_11_0 = ( globalReadOffsetAL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_0_0_12_0 = ( globalReadOffsetAL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_0_0_13_0 = ( globalReadOffsetAL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_0_0_14_0 = ( globalReadOffsetAL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_0_0_15_0 = ( globalReadOffsetAL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);
  a_0_0_16_0 = ( globalReadOffsetAL_16_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_16_0 + 0);
  a_0_0_17_0 = ( globalReadOffsetAL_17_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_17_0 + 0);
  a_0_0_18_0 = ( globalReadOffsetAL_18_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_18_0 + 0);
  a_0_0_19_0 = ( globalReadOffsetAL_19_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_19_0 + 0);
  a_0_0_20_0 = ( globalReadOffsetAL_20_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_20_0 + 0);
  a_0_0_21_0 = ( globalReadOffsetAL_21_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_21_0 + 0);
  a_0_0_22_0 = ( globalReadOffsetAL_22_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_22_0 + 0);
  a_0_0_23_0 = ( globalReadOffsetAL_23_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_23_0 + 0);
  a_0_0_24_0 = ( globalReadOffsetAL_24_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_24_0 + 0);
  a_0_0_25_0 = ( globalReadOffsetAL_25_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_25_0 + 0);
  a_0_0_26_0 = ( globalReadOffsetAL_26_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_26_0 + 0);
  a_0_0_27_0 = ( globalReadOffsetAL_27_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_27_0 + 0);
  a_0_0_28_0 = ( globalReadOffsetAL_28_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_28_0 + 0);
  a_0_0_29_0 = ( globalReadOffsetAL_29_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_29_0 + 0);
  a_0_0_30_0 = ( globalReadOffsetAL_30_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_30_0 + 0);
  a_0_0_31_0 = ( globalReadOffsetAL_31_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_31_0 + 0);
  a_0_0_32_0 = ( globalReadOffsetAL_32_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_32_0 + 0);
  a_0_0_33_0 = ( globalReadOffsetAL_33_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_33_0 + 0);
  a_0_0_34_0 = ( globalReadOffsetAL_34_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_34_0 + 0);
  a_0_0_35_0 = ( globalReadOffsetAL_35_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_35_0 + 0);
  a_0_0_36_0 = ( globalReadOffsetAL_36_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_36_0 + 0);
  a_0_0_37_0 = ( globalReadOffsetAL_37_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_37_0 + 0);
  a_0_0_38_0 = ( globalReadOffsetAL_38_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_38_0 + 0);
  a_0_0_39_0 = ( globalReadOffsetAL_39_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_39_0 + 0);
  a_0_0_40_0 = ( globalReadOffsetAL_40_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_40_0 + 0);
  a_0_0_41_0 = ( globalReadOffsetAL_41_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_41_0 + 0);
  a_0_0_42_0 = ( globalReadOffsetAL_42_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_42_0 + 0);
  a_0_0_43_0 = ( globalReadOffsetAL_43_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_43_0 + 0);
  a_0_0_44_0 = ( globalReadOffsetAL_44_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_44_0 + 0);
  a_0_0_45_0 = ( globalReadOffsetAL_45_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_45_0 + 0);
  a_0_0_46_0 = ( globalReadOffsetAL_46_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_46_0 + 0);
  a_0_0_47_0 = ( globalReadOffsetAL_47_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_47_0 + 0);
  a_0_0_48_0 = ( globalReadOffsetAL_48_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_48_0 + 0);
  a_0_0_49_0 = ( globalReadOffsetAL_49_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_49_0 + 0);
  a_0_0_50_0 = ( globalReadOffsetAL_50_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_50_0 + 0);
  a_0_0_51_0 = ( globalReadOffsetAL_51_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_51_0 + 0);
  a_0_0_52_0 = ( globalReadOffsetAL_52_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_52_0 + 0);
  a_0_0_53_0 = ( globalReadOffsetAL_53_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_53_0 + 0);
  a_0_0_54_0 = ( globalReadOffsetAL_54_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_54_0 + 0);
  a_0_0_55_0 = ( globalReadOffsetAL_55_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_55_0 + 0);
  a_0_0_56_0 = ( globalReadOffsetAL_56_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_56_0 + 0);
  a_0_0_57_0 = ( globalReadOffsetAL_57_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_57_0 + 0);
  a_0_0_58_0 = ( globalReadOffsetAL_58_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_58_0 + 0);
  a_0_0_59_0 = ( globalReadOffsetAL_59_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_59_0 + 0);
  a_0_0_60_0 = ( globalReadOffsetAL_60_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_60_0 + 0);
  a_0_0_61_0 = ( globalReadOffsetAL_61_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_61_0 + 0);
  a_0_0_62_0 = ( globalReadOffsetAL_62_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_62_0 + 0);
  a_0_0_63_0 = ( globalReadOffsetAL_63_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_63_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);
  localWriteA_0_0_16_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_16_0);
  localWriteA_0_0_17_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_17_0);
  localWriteA_0_0_18_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_18_0);
  localWriteA_0_0_19_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_19_0);
  localWriteA_0_0_20_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_20_0);
  localWriteA_0_0_21_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_21_0);
  localWriteA_0_0_22_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_22_0);
  localWriteA_0_0_23_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_23_0);
  localWriteA_0_0_24_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_24_0);
  localWriteA_0_0_25_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_25_0);
  localWriteA_0_0_26_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_26_0);
  localWriteA_0_0_27_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_27_0);
  localWriteA_0_0_28_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_28_0);
  localWriteA_0_0_29_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_29_0);
  localWriteA_0_0_30_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_30_0);
  localWriteA_0_0_31_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_31_0);
  localWriteA_0_0_32_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_32_0);
  localWriteA_0_0_33_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_33_0);
  localWriteA_0_0_34_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_34_0);
  localWriteA_0_0_35_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_35_0);
  localWriteA_0_0_36_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_36_0);
  localWriteA_0_0_37_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_37_0);
  localWriteA_0_0_38_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_38_0);
  localWriteA_0_0_39_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_39_0);
  localWriteA_0_0_40_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_40_0);
  localWriteA_0_0_41_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_41_0);
  localWriteA_0_0_42_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_42_0);
  localWriteA_0_0_43_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_43_0);
  localWriteA_0_0_44_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_44_0);
  localWriteA_0_0_45_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_45_0);
  localWriteA_0_0_46_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_46_0);
  localWriteA_0_0_47_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_47_0);
  localWriteA_0_0_48_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_48_0);
  localWriteA_0_0_49_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_49_0);
  localWriteA_0_0_50_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_50_0);
  localWriteA_0_0_51_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_51_0);
  localWriteA_0_0_52_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_52_0);
  localWriteA_0_0_53_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_53_0);
  localWriteA_0_0_54_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_54_0);
  localWriteA_0_0_55_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_55_0);
  localWriteA_0_0_56_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_56_0);
  localWriteA_0_0_57_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_57_0);
  localWriteA_0_0_58_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_58_0);
  localWriteA_0_0_59_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_59_0);
  localWriteA_0_0_60_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_60_0);
  localWriteA_0_0_61_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_61_0);
  localWriteA_0_0_62_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_62_0);
  localWriteA_0_0_63_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_63_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
  *(localWriteA_0_0_16_0 + 0) = a_0_0_16_0;
  *(localWriteA_0_0_17_0 + 0) = a_0_0_17_0;
  *(localWriteA_0_0_18_0 + 0) = a_0_0_18_0;
  *(localWriteA_0_0_19_0 + 0) = a_0_0_19_0;
  *(localWriteA_0_0_20_0 + 0) = a_0_0_20_0;
  *(localWriteA_0_0_21_0 + 0) = a_0_0_21_0;
  *(localWriteA_0_0_22_0 + 0) = a_0_0_22_0;
  *(localWriteA_0_0_23_0 + 0) = a_0_0_23_0;
  *(localWriteA_0_0_24_0 + 0) = a_0_0_24_0;
  *(localWriteA_0_0_25_0 + 0) = a_0_0_25_0;
  *(localWriteA_0_0_26_0 + 0) = a_0_0_26_0;
  *(localWriteA_0_0_27_0 + 0) = a_0_0_27_0;
  *(localWriteA_0_0_28_0 + 0) = a_0_0_28_0;
  *(localWriteA_0_0_29_0 + 0) = a_0_0_29_0;
  *(localWriteA_0_0_30_0 + 0) = a_0_0_30_0;
  *(localWriteA_0_0_31_0 + 0) = a_0_0_31_0;
  *(localWriteA_0_0_32_0 + 0) = a_0_0_32_0;
  *(localWriteA_0_0_33_0 + 0) = a_0_0_33_0;
  *(localWriteA_0_0_34_0 + 0) = a_0_0_34_0;
  *(localWriteA_0_0_35_0 + 0) = a_0_0_35_0;
  *(localWriteA_0_0_36_0 + 0) = a_0_0_36_0;
  *(localWriteA_0_0_37_0 + 0) = a_0_0_37_0;
  *(localWriteA_0_0_38_0 + 0) = a_0_0_38_0;
  *(localWriteA_0_0_39_0 + 0) = a_0_0_39_0;
  *(localWriteA_0_0_40_0 + 0) = a_0_0_40_0;
  *(localWriteA_0_0_41_0 + 0) = a_0_0_41_0;
  *(localWriteA_0_0_42_0 + 0) = a_0_0_42_0;
  *(localWriteA_0_0_43_0 + 0) = a_0_0_43_0;
  *(localWriteA_0_0_44_0 + 0) = a_0_0_44_0;
  *(localWriteA_0_0_45_0 + 0) = a_0_0_45_0;
  *(localWriteA_0_0_46_0 + 0) = a_0_0_46_0;
  *(localWriteA_0_0_47_0 + 0) = a_0_0_47_0;
  *(localWriteA_0_0_48_0 + 0) = a_0_0_48_0;
  *(localWriteA_0_0_49_0 + 0) = a_0_0_49_0;
  *(localWriteA_0_0_50_0 + 0) = a_0_0_50_0;
  *(localWriteA_0_0_51_0 + 0) = a_0_0_51_0;
  *(localWriteA_0_0_52_0 + 0) = a_0_0_52_0;
  *(localWriteA_0_0_53_0 + 0) = a_0_0_53_0;
  *(localWriteA_0_0_54_0 + 0) = a_0_0_54_0;
  *(localWriteA_0_0_55_0 + 0) = a_0_0_55_0;
  *(localWriteA_0_0_56_0 + 0) = a_0_0_56_0;
  *(localWriteA_0_0_57_0 + 0) = a_0_0_57_0;
  *(localWriteA_0_0_58_0 + 0) = a_0_0_58_0;
  *(localWriteA_0_0_59_0 + 0) = a_0_0_59_0;
  *(localWriteA_0_0_60_0 + 0) = a_0_0_60_0;
  *(localWriteA_0_0_61_0 + 0) = a_0_0_61_0;
  *(localWriteA_0_0_62_0 + 0) = a_0_0_62_0;
  *(localWriteA_0_0_63_0 + 0) = a_0_0_63_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_1x1

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_1x1
#undef MAC_1x1_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 64
#define SG1J 1
#define TT0I 1
#define TT1J 1
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 64
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 64
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 4096
#define LDS_NUM_ELEMENTS 4160

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 1x1 micro-tile */
#define MAC_1x1\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \

#define MAC_1x1_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x1x64_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU0_SUM0_SUS128_SRVW0_SVW4_SNLL0_TT1_1_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG64_1_1_WGM1(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE a_0_0_16_0;
  DATA_TYPE a_0_0_17_0;
  DATA_TYPE a_0_0_18_0;
  DATA_TYPE a_0_0_19_0;
  DATA_TYPE a_0_0_20_0;
  DATA_TYPE a_0_0_21_0;
  DATA_TYPE a_0_0_22_0;
  DATA_TYPE a_0_0_23_0;
  DATA_TYPE a_0_0_24_0;
  DATA_TYPE a_0_0_25_0;
  DATA_TYPE a_0_0_26_0;
  DATA_TYPE a_0_0_27_0;
  DATA_TYPE a_0_0_28_0;
  DATA_TYPE a_0_0_29_0;
  DATA_TYPE a_0_0_30_0;
  DATA_TYPE a_0_0_31_0;
  DATA_TYPE a_0_0_32_0;
  DATA_TYPE a_0_0_33_0;
  DATA_TYPE a_0_0_34_0;
  DATA_TYPE a_0_0_35_0;
  DATA_TYPE a_0_0_36_0;
  DATA_TYPE a_0_0_37_0;
  DATA_TYPE a_0_0_38_0;
  DATA_TYPE a_0_0_39_0;
  DATA_TYPE a_0_0_40_0;
  DATA_TYPE a_0_0_41_0;
  DATA_TYPE a_0_0_42_0;
  DATA_TYPE a_0_0_43_0;
  DATA_TYPE a_0_0_44_0;
  DATA_TYPE a_0_0_45_0;
  DATA_TYPE a_0_0_46_0;
  DATA_TYPE a_0_0_47_0;
  DATA_TYPE a_0_0_48_0;
  DATA_TYPE a_0_0_49_0;
  DATA_TYPE a_0_0_50_0;
  DATA_TYPE a_0_0_51_0;
  DATA_TYPE a_0_0_52_0;
  DATA_TYPE a_0_0_53_0;
  DATA_TYPE a_0_0_54_0;
  DATA_TYPE a_0_0_55_0;
  DATA_TYPE a_0_0_56_0;
  DATA_TYPE a_0_0_57_0;
  DATA_TYPE a_0_0_58_0;
  DATA_TYPE a_0_0_59_0;
  DATA_TYPE a_0_0_60_0;
  DATA_TYPE a_0_0_61_0;
  DATA_TYPE a_0_0_62_0;
  DATA_TYPE a_0_0_63_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;
  unsigned int globalReadOffsetAL_8_0 = globalReadOffsetAL + 0 + 8*LSPA;
  unsigned int globalReadOffsetAL_9_0 = globalReadOffsetAL + 0 + 9*LSPA;
  unsigned int globalReadOffsetAL_10_0 = globalReadOffsetAL + 0 + 10*LSPA;
  unsigned int globalReadOffsetAL_11_0 = globalReadOffsetAL + 0 + 11*LSPA;
  unsigned int globalReadOffsetAL_12_0 = globalReadOffsetAL + 0 + 12*LSPA;
  unsigned int globalReadOffsetAL_13_0 = globalReadOffsetAL + 0 + 13*LSPA;
  unsigned int globalReadOffsetAL_14_0 = globalReadOffsetAL + 0 + 14*LSPA;
  unsigned int globalReadOffsetAL_15_0 = globalReadOffsetAL + 0 + 15*LSPA;
  unsigned int globalReadOffsetAL_16_0 = globalReadOffsetAL + 0 + 16*LSPA;
  unsigned int globalReadOffsetAL_17_0 = globalReadOffsetAL + 0 + 17*LSPA;
  unsigned int globalReadOffsetAL_18_0 = globalReadOffsetAL + 0 + 18*LSPA;
  unsigned int globalReadOffsetAL_19_0 = globalReadOffsetAL + 0 + 19*LSPA;
  unsigned int globalReadOffsetAL_20_0 = globalReadOffsetAL + 0 + 20*LSPA;
  unsigned int globalReadOffsetAL_21_0 = globalReadOffsetAL + 0 + 21*LSPA;
  unsigned int globalReadOffsetAL_22_0 = globalReadOffsetAL + 0 + 22*LSPA;
  unsigned int globalReadOffsetAL_23_0 = globalReadOffsetAL + 0 + 23*LSPA;
  unsigned int globalReadOffsetAL_24_0 = globalReadOffsetAL + 0 + 24*LSPA;
  unsigned int globalReadOffsetAL_25_0 = globalReadOffsetAL + 0 + 25*LSPA;
  unsigned int globalReadOffsetAL_26_0 = globalReadOffsetAL + 0 + 26*LSPA;
  unsigned int globalReadOffsetAL_27_0 = globalReadOffsetAL + 0 + 27*LSPA;
  unsigned int globalReadOffsetAL_28_0 = globalReadOffsetAL + 0 + 28*LSPA;
  unsigned int globalReadOffsetAL_29_0 = globalReadOffsetAL + 0 + 29*LSPA;
  unsigned int globalReadOffsetAL_30_0 = globalReadOffsetAL + 0 + 30*LSPA;
  unsigned int globalReadOffsetAL_31_0 = globalReadOffsetAL + 0 + 31*LSPA;
  unsigned int globalReadOffsetAL_32_0 = globalReadOffsetAL + 0 + 32*LSPA;
  unsigned int globalReadOffsetAL_33_0 = globalReadOffsetAL + 0 + 33*LSPA;
  unsigned int globalReadOffsetAL_34_0 = globalReadOffsetAL + 0 + 34*LSPA;
  unsigned int globalReadOffsetAL_35_0 = globalReadOffsetAL + 0 + 35*LSPA;
  unsigned int globalReadOffsetAL_36_0 = globalReadOffsetAL + 0 + 36*LSPA;
  unsigned int globalReadOffsetAL_37_0 = globalReadOffsetAL + 0 + 37*LSPA;
  unsigned int globalReadOffsetAL_38_0 = globalReadOffsetAL + 0 + 38*LSPA;
  unsigned int globalReadOffsetAL_39_0 = globalReadOffsetAL + 0 + 39*LSPA;
  unsigned int globalReadOffsetAL_40_0 = globalReadOffsetAL + 0 + 40*LSPA;
  unsigned int globalReadOffsetAL_41_0 = globalReadOffsetAL + 0 + 41*LSPA;
  unsigned int globalReadOffsetAL_42_0 = globalReadOffsetAL + 0 + 42*LSPA;
  unsigned int globalReadOffsetAL_43_0 = globalReadOffsetAL + 0 + 43*LSPA;
  unsigned int globalReadOffsetAL_44_0 = globalReadOffsetAL + 0 + 44*LSPA;
  unsigned int globalReadOffsetAL_45_0 = globalReadOffsetAL + 0 + 45*LSPA;
  unsigned int globalReadOffsetAL_46_0 = globalReadOffsetAL + 0 + 46*LSPA;
  unsigned int globalReadOffsetAL_47_0 = globalReadOffsetAL + 0 + 47*LSPA;
  unsigned int globalReadOffsetAL_48_0 = globalReadOffsetAL + 0 + 48*LSPA;
  unsigned int globalReadOffsetAL_49_0 = globalReadOffsetAL + 0 + 49*LSPA;
  unsigned int globalReadOffsetAL_50_0 = globalReadOffsetAL + 0 + 50*LSPA;
  unsigned int globalReadOffsetAL_51_0 = globalReadOffsetAL + 0 + 51*LSPA;
  unsigned int globalReadOffsetAL_52_0 = globalReadOffsetAL + 0 + 52*LSPA;
  unsigned int globalReadOffsetAL_53_0 = globalReadOffsetAL + 0 + 53*LSPA;
  unsigned int globalReadOffsetAL_54_0 = globalReadOffsetAL + 0 + 54*LSPA;
  unsigned int globalReadOffsetAL_55_0 = globalReadOffsetAL + 0 + 55*LSPA;
  unsigned int globalReadOffsetAL_56_0 = globalReadOffsetAL + 0 + 56*LSPA;
  unsigned int globalReadOffsetAL_57_0 = globalReadOffsetAL + 0 + 57*LSPA;
  unsigned int globalReadOffsetAL_58_0 = globalReadOffsetAL + 0 + 58*LSPA;
  unsigned int globalReadOffsetAL_59_0 = globalReadOffsetAL + 0 + 59*LSPA;
  unsigned int globalReadOffsetAL_60_0 = globalReadOffsetAL + 0 + 60*LSPA;
  unsigned int globalReadOffsetAL_61_0 = globalReadOffsetAL + 0 + 61*LSPA;
  unsigned int globalReadOffsetAL_62_0 = globalReadOffsetAL + 0 + 62*LSPA;
  unsigned int globalReadOffsetAL_63_0 = globalReadOffsetAL + 0 + 63*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_15_0), wgK );
  int64_t globalReadOffsetA_0_0_16_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_16_0), wgK );
  int64_t globalReadOffsetA_0_0_17_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_17_0), wgK );
  int64_t globalReadOffsetA_0_0_18_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_18_0), wgK );
  int64_t globalReadOffsetA_0_0_19_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_19_0), wgK );
  int64_t globalReadOffsetA_0_0_20_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_20_0), wgK );
  int64_t globalReadOffsetA_0_0_21_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_21_0), wgK );
  int64_t globalReadOffsetA_0_0_22_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_22_0), wgK );
  int64_t globalReadOffsetA_0_0_23_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_23_0), wgK );
  int64_t globalReadOffsetA_0_0_24_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_24_0), wgK );
  int64_t globalReadOffsetA_0_0_25_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_25_0), wgK );
  int64_t globalReadOffsetA_0_0_26_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_26_0), wgK );
  int64_t globalReadOffsetA_0_0_27_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_27_0), wgK );
  int64_t globalReadOffsetA_0_0_28_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_28_0), wgK );
  int64_t globalReadOffsetA_0_0_29_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_29_0), wgK );
  int64_t globalReadOffsetA_0_0_30_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_30_0), wgK );
  int64_t globalReadOffsetA_0_0_31_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_31_0), wgK );
  int64_t globalReadOffsetA_0_0_32_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_32_0), wgK );
  int64_t globalReadOffsetA_0_0_33_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_33_0), wgK );
  int64_t globalReadOffsetA_0_0_34_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_34_0), wgK );
  int64_t globalReadOffsetA_0_0_35_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_35_0), wgK );
  int64_t globalReadOffsetA_0_0_36_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_36_0), wgK );
  int64_t globalReadOffsetA_0_0_37_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_37_0), wgK );
  int64_t globalReadOffsetA_0_0_38_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_38_0), wgK );
  int64_t globalReadOffsetA_0_0_39_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_39_0), wgK );
  int64_t globalReadOffsetA_0_0_40_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_40_0), wgK );
  int64_t globalReadOffsetA_0_0_41_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_41_0), wgK );
  int64_t globalReadOffsetA_0_0_42_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_42_0), wgK );
  int64_t globalReadOffsetA_0_0_43_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_43_0), wgK );
  int64_t globalReadOffsetA_0_0_44_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_44_0), wgK );
  int64_t globalReadOffsetA_0_0_45_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_45_0), wgK );
  int64_t globalReadOffsetA_0_0_46_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_46_0), wgK );
  int64_t globalReadOffsetA_0_0_47_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_47_0), wgK );
  int64_t globalReadOffsetA_0_0_48_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_48_0), wgK );
  int64_t globalReadOffsetA_0_0_49_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_49_0), wgK );
  int64_t globalReadOffsetA_0_0_50_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_50_0), wgK );
  int64_t globalReadOffsetA_0_0_51_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_51_0), wgK );
  int64_t globalReadOffsetA_0_0_52_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_52_0), wgK );
  int64_t globalReadOffsetA_0_0_53_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_53_0), wgK );
  int64_t globalReadOffsetA_0_0_54_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_54_0), wgK );
  int64_t globalReadOffsetA_0_0_55_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_55_0), wgK );
  int64_t globalReadOffsetA_0_0_56_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_56_0), wgK );
  int64_t globalReadOffsetA_0_0_57_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_57_0), wgK );
  int64_t globalReadOffsetA_0_0_58_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_58_0), wgK );
  int64_t globalReadOffsetA_0_0_59_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_59_0), wgK );
  int64_t globalReadOffsetA_0_0_60_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_60_0), wgK );
  int64_t globalReadOffsetA_0_0_61_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_61_0), wgK );
  int64_t globalReadOffsetA_0_0_62_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_62_0), wgK );
  int64_t globalReadOffsetA_0_0_63_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_63_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;
  DATA_TYPE const *globalReadA_0_0_16_0 = A + globalReadOffsetA_0_0_16_0;
  DATA_TYPE const *globalReadA_0_0_17_0 = A + globalReadOffsetA_0_0_17_0;
  DATA_TYPE const *globalReadA_0_0_18_0 = A + globalReadOffsetA_0_0_18_0;
  DATA_TYPE const *globalReadA_0_0_19_0 = A + globalReadOffsetA_0_0_19_0;
  DATA_TYPE const *globalReadA_0_0_20_0 = A + globalReadOffsetA_0_0_20_0;
  DATA_TYPE const *globalReadA_0_0_21_0 = A + globalReadOffsetA_0_0_21_0;
  DATA_TYPE const *globalReadA_0_0_22_0 = A + globalReadOffsetA_0_0_22_0;
  DATA_TYPE const *globalReadA_0_0_23_0 = A + globalReadOffsetA_0_0_23_0;
  DATA_TYPE const *globalReadA_0_0_24_0 = A + globalReadOffsetA_0_0_24_0;
  DATA_TYPE const *globalReadA_0_0_25_0 = A + globalReadOffsetA_0_0_25_0;
  DATA_TYPE const *globalReadA_0_0_26_0 = A + globalReadOffsetA_0_0_26_0;
  DATA_TYPE const *globalReadA_0_0_27_0 = A + globalReadOffsetA_0_0_27_0;
  DATA_TYPE const *globalReadA_0_0_28_0 = A + globalReadOffsetA_0_0_28_0;
  DATA_TYPE const *globalReadA_0_0_29_0 = A + globalReadOffsetA_0_0_29_0;
  DATA_TYPE const *globalReadA_0_0_30_0 = A + globalReadOffsetA_0_0_30_0;
  DATA_TYPE const *globalReadA_0_0_31_0 = A + globalReadOffsetA_0_0_31_0;
  DATA_TYPE const *globalReadA_0_0_32_0 = A + globalReadOffsetA_0_0_32_0;
  DATA_TYPE const *globalReadA_0_0_33_0 = A + globalReadOffsetA_0_0_33_0;
  DATA_TYPE const *globalReadA_0_0_34_0 = A + globalReadOffsetA_0_0_34_0;
  DATA_TYPE const *globalReadA_0_0_35_0 = A + globalReadOffsetA_0_0_35_0;
  DATA_TYPE const *globalReadA_0_0_36_0 = A + globalReadOffsetA_0_0_36_0;
  DATA_TYPE const *globalReadA_0_0_37_0 = A + globalReadOffsetA_0_0_37_0;
  DATA_TYPE const *globalReadA_0_0_38_0 = A + globalReadOffsetA_0_0_38_0;
  DATA_TYPE const *globalReadA_0_0_39_0 = A + globalReadOffsetA_0_0_39_0;
  DATA_TYPE const *globalReadA_0_0_40_0 = A + globalReadOffsetA_0_0_40_0;
  DATA_TYPE const *globalReadA_0_0_41_0 = A + globalReadOffsetA_0_0_41_0;
  DATA_TYPE const *globalReadA_0_0_42_0 = A + globalReadOffsetA_0_0_42_0;
  DATA_TYPE const *globalReadA_0_0_43_0 = A + globalReadOffsetA_0_0_43_0;
  DATA_TYPE const *globalReadA_0_0_44_0 = A + globalReadOffsetA_0_0_44_0;
  DATA_TYPE const *globalReadA_0_0_45_0 = A + globalReadOffsetA_0_0_45_0;
  DATA_TYPE const *globalReadA_0_0_46_0 = A + globalReadOffsetA_0_0_46_0;
  DATA_TYPE const *globalReadA_0_0_47_0 = A + globalReadOffsetA_0_0_47_0;
  DATA_TYPE const *globalReadA_0_0_48_0 = A + globalReadOffsetA_0_0_48_0;
  DATA_TYPE const *globalReadA_0_0_49_0 = A + globalReadOffsetA_0_0_49_0;
  DATA_TYPE const *globalReadA_0_0_50_0 = A + globalReadOffsetA_0_0_50_0;
  DATA_TYPE const *globalReadA_0_0_51_0 = A + globalReadOffsetA_0_0_51_0;
  DATA_TYPE const *globalReadA_0_0_52_0 = A + globalReadOffsetA_0_0_52_0;
  DATA_TYPE const *globalReadA_0_0_53_0 = A + globalReadOffsetA_0_0_53_0;
  DATA_TYPE const *globalReadA_0_0_54_0 = A + globalReadOffsetA_0_0_54_0;
  DATA_TYPE const *globalReadA_0_0_55_0 = A + globalReadOffsetA_0_0_55_0;
  DATA_TYPE const *globalReadA_0_0_56_0 = A + globalReadOffsetA_0_0_56_0;
  DATA_TYPE const *globalReadA_0_0_57_0 = A + globalReadOffsetA_0_0_57_0;
  DATA_TYPE const *globalReadA_0_0_58_0 = A + globalReadOffsetA_0_0_58_0;
  DATA_TYPE const *globalReadA_0_0_59_0 = A + globalReadOffsetA_0_0_59_0;
  DATA_TYPE const *globalReadA_0_0_60_0 = A + globalReadOffsetA_0_0_60_0;
  DATA_TYPE const *globalReadA_0_0_61_0 = A + globalReadOffsetA_0_0_61_0;
  DATA_TYPE const *globalReadA_0_0_62_0 = A + globalReadOffsetA_0_0_62_0;
  DATA_TYPE const *globalReadA_0_0_63_0 = A + globalReadOffsetA_0_0_63_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 8*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 9*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 10*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 11*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 12*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 13*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 14*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 15*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_16_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 16*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_17_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 17*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_18_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 18*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_19_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 19*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_20_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 20*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_21_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 21*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_22_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 22*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_23_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 23*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_24_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 24*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_25_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 25*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_26_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 26*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_27_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 27*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_28_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 28*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_29_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 29*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_30_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 30*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_31_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 31*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_32_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 32*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_33_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 33*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_34_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 34*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_35_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 35*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_36_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 36*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_37_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 37*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_38_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 38*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_39_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 39*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_40_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 40*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_41_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 41*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_42_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 42*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_43_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 43*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_44_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 44*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_45_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 45*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_46_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 46*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_47_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 47*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_48_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 48*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_49_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 49*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_50_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 50*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_51_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 51*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_52_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 52*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_53_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 53*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_54_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 54*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_55_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 55*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_56_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 56*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_57_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 57*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_58_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 58*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_59_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 59*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_60_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 60*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_61_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 61*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_62_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 62*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_63_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 63*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_0_0_15_0;
  DATA_TYPE *localWriteA_0_0_16_0;
  DATA_TYPE *localWriteA_0_0_17_0;
  DATA_TYPE *localWriteA_0_0_18_0;
  DATA_TYPE *localWriteA_0_0_19_0;
  DATA_TYPE *localWriteA_0_0_20_0;
  DATA_TYPE *localWriteA_0_0_21_0;
  DATA_TYPE *localWriteA_0_0_22_0;
  DATA_TYPE *localWriteA_0_0_23_0;
  DATA_TYPE *localWriteA_0_0_24_0;
  DATA_TYPE *localWriteA_0_0_25_0;
  DATA_TYPE *localWriteA_0_0_26_0;
  DATA_TYPE *localWriteA_0_0_27_0;
  DATA_TYPE *localWriteA_0_0_28_0;
  DATA_TYPE *localWriteA_0_0_29_0;
  DATA_TYPE *localWriteA_0_0_30_0;
  DATA_TYPE *localWriteA_0_0_31_0;
  DATA_TYPE *localWriteA_0_0_32_0;
  DATA_TYPE *localWriteA_0_0_33_0;
  DATA_TYPE *localWriteA_0_0_34_0;
  DATA_TYPE *localWriteA_0_0_35_0;
  DATA_TYPE *localWriteA_0_0_36_0;
  DATA_TYPE *localWriteA_0_0_37_0;
  DATA_TYPE *localWriteA_0_0_38_0;
  DATA_TYPE *localWriteA_0_0_39_0;
  DATA_TYPE *localWriteA_0_0_40_0;
  DATA_TYPE *localWriteA_0_0_41_0;
  DATA_TYPE *localWriteA_0_0_42_0;
  DATA_TYPE *localWriteA_0_0_43_0;
  DATA_TYPE *localWriteA_0_0_44_0;
  DATA_TYPE *localWriteA_0_0_45_0;
  DATA_TYPE *localWriteA_0_0_46_0;
  DATA_TYPE *localWriteA_0_0_47_0;
  DATA_TYPE *localWriteA_0_0_48_0;
  DATA_TYPE *localWriteA_0_0_49_0;
  DATA_TYPE *localWriteA_0_0_50_0;
  DATA_TYPE *localWriteA_0_0_51_0;
  DATA_TYPE *localWriteA_0_0_52_0;
  DATA_TYPE *localWriteA_0_0_53_0;
  DATA_TYPE *localWriteA_0_0_54_0;
  DATA_TYPE *localWriteA_0_0_55_0;
  DATA_TYPE *localWriteA_0_0_56_0;
  DATA_TYPE *localWriteA_0_0_57_0;
  DATA_TYPE *localWriteA_0_0_58_0;
  DATA_TYPE *localWriteA_0_0_59_0;
  DATA_TYPE *localWriteA_0_0_60_0;
  DATA_TYPE *localWriteA_0_0_61_0;
  DATA_TYPE *localWriteA_0_0_62_0;
  DATA_TYPE *localWriteA_0_0_63_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);
  localWriteA_0_0_16_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_16_0);
  localWriteA_0_0_17_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_17_0);
  localWriteA_0_0_18_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_18_0);
  localWriteA_0_0_19_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_19_0);
  localWriteA_0_0_20_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_20_0);
  localWriteA_0_0_21_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_21_0);
  localWriteA_0_0_22_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_22_0);
  localWriteA_0_0_23_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_23_0);
  localWriteA_0_0_24_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_24_0);
  localWriteA_0_0_25_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_25_0);
  localWriteA_0_0_26_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_26_0);
  localWriteA_0_0_27_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_27_0);
  localWriteA_0_0_28_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_28_0);
  localWriteA_0_0_29_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_29_0);
  localWriteA_0_0_30_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_30_0);
  localWriteA_0_0_31_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_31_0);
  localWriteA_0_0_32_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_32_0);
  localWriteA_0_0_33_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_33_0);
  localWriteA_0_0_34_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_34_0);
  localWriteA_0_0_35_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_35_0);
  localWriteA_0_0_36_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_36_0);
  localWriteA_0_0_37_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_37_0);
  localWriteA_0_0_38_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_38_0);
  localWriteA_0_0_39_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_39_0);
  localWriteA_0_0_40_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_40_0);
  localWriteA_0_0_41_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_41_0);
  localWriteA_0_0_42_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_42_0);
  localWriteA_0_0_43_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_43_0);
  localWriteA_0_0_44_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_44_0);
  localWriteA_0_0_45_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_45_0);
  localWriteA_0_0_46_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_46_0);
  localWriteA_0_0_47_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_47_0);
  localWriteA_0_0_48_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_48_0);
  localWriteA_0_0_49_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_49_0);
  localWriteA_0_0_50_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_50_0);
  localWriteA_0_0_51_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_51_0);
  localWriteA_0_0_52_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_52_0);
  localWriteA_0_0_53_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_53_0);
  localWriteA_0_0_54_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_54_0);
  localWriteA_0_0_55_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_55_0);
  localWriteA_0_0_56_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_56_0);
  localWriteA_0_0_57_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_57_0);
  localWriteA_0_0_58_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_58_0);
  localWriteA_0_0_59_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_59_0);
  localWriteA_0_0_60_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_60_0);
  localWriteA_0_0_61_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_61_0);
  localWriteA_0_0_62_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_62_0);
  localWriteA_0_0_63_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_63_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);
    a_0_0_16_0 = *(globalReadA_0_0_16_0 + 0);
    a_0_0_17_0 = *(globalReadA_0_0_17_0 + 0);
    a_0_0_18_0 = *(globalReadA_0_0_18_0 + 0);
    a_0_0_19_0 = *(globalReadA_0_0_19_0 + 0);
    a_0_0_20_0 = *(globalReadA_0_0_20_0 + 0);
    a_0_0_21_0 = *(globalReadA_0_0_21_0 + 0);
    a_0_0_22_0 = *(globalReadA_0_0_22_0 + 0);
    a_0_0_23_0 = *(globalReadA_0_0_23_0 + 0);
    a_0_0_24_0 = *(globalReadA_0_0_24_0 + 0);
    a_0_0_25_0 = *(globalReadA_0_0_25_0 + 0);
    a_0_0_26_0 = *(globalReadA_0_0_26_0 + 0);
    a_0_0_27_0 = *(globalReadA_0_0_27_0 + 0);
    a_0_0_28_0 = *(globalReadA_0_0_28_0 + 0);
    a_0_0_29_0 = *(globalReadA_0_0_29_0 + 0);
    a_0_0_30_0 = *(globalReadA_0_0_30_0 + 0);
    a_0_0_31_0 = *(globalReadA_0_0_31_0 + 0);
    a_0_0_32_0 = *(globalReadA_0_0_32_0 + 0);
    a_0_0_33_0 = *(globalReadA_0_0_33_0 + 0);
    a_0_0_34_0 = *(globalReadA_0_0_34_0 + 0);
    a_0_0_35_0 = *(globalReadA_0_0_35_0 + 0);
    a_0_0_36_0 = *(globalReadA_0_0_36_0 + 0);
    a_0_0_37_0 = *(globalReadA_0_0_37_0 + 0);
    a_0_0_38_0 = *(globalReadA_0_0_38_0 + 0);
    a_0_0_39_0 = *(globalReadA_0_0_39_0 + 0);
    a_0_0_40_0 = *(globalReadA_0_0_40_0 + 0);
    a_0_0_41_0 = *(globalReadA_0_0_41_0 + 0);
    a_0_0_42_0 = *(globalReadA_0_0_42_0 + 0);
    a_0_0_43_0 = *(globalReadA_0_0_43_0 + 0);
    a_0_0_44_0 = *(globalReadA_0_0_44_0 + 0);
    a_0_0_45_0 = *(globalReadA_0_0_45_0 + 0);
    a_0_0_46_0 = *(globalReadA_0_0_46_0 + 0);
    a_0_0_47_0 = *(globalReadA_0_0_47_0 + 0);
    a_0_0_48_0 = *(globalReadA_0_0_48_0 + 0);
    a_0_0_49_0 = *(globalReadA_0_0_49_0 + 0);
    a_0_0_50_0 = *(globalReadA_0_0_50_0 + 0);
    a_0_0_51_0 = *(globalReadA_0_0_51_0 + 0);
    a_0_0_52_0 = *(globalReadA_0_0_52_0 + 0);
    a_0_0_53_0 = *(globalReadA_0_0_53_0 + 0);
    a_0_0_54_0 = *(globalReadA_0_0_54_0 + 0);
    a_0_0_55_0 = *(globalReadA_0_0_55_0 + 0);
    a_0_0_56_0 = *(globalReadA_0_0_56_0 + 0);
    a_0_0_57_0 = *(globalReadA_0_0_57_0 + 0);
    a_0_0_58_0 = *(globalReadA_0_0_58_0 + 0);
    a_0_0_59_0 = *(globalReadA_0_0_59_0 + 0);
    a_0_0_60_0 = *(globalReadA_0_0_60_0 + 0);
    a_0_0_61_0 = *(globalReadA_0_0_61_0 + 0);
    a_0_0_62_0 = *(globalReadA_0_0_62_0 + 0);
    a_0_0_63_0 = *(globalReadA_0_0_63_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    globalReadA_0_0_16_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_16_0) + 1*globalReadIncAL);
    globalReadA_0_0_17_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_17_0) + 1*globalReadIncAL);
    globalReadA_0_0_18_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_18_0) + 1*globalReadIncAL);
    globalReadA_0_0_19_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_19_0) + 1*globalReadIncAL);
    globalReadA_0_0_20_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_20_0) + 1*globalReadIncAL);
    globalReadA_0_0_21_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_21_0) + 1*globalReadIncAL);
    globalReadA_0_0_22_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_22_0) + 1*globalReadIncAL);
    globalReadA_0_0_23_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_23_0) + 1*globalReadIncAL);
    globalReadA_0_0_24_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_24_0) + 1*globalReadIncAL);
    globalReadA_0_0_25_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_25_0) + 1*globalReadIncAL);
    globalReadA_0_0_26_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_26_0) + 1*globalReadIncAL);
    globalReadA_0_0_27_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_27_0) + 1*globalReadIncAL);
    globalReadA_0_0_28_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_28_0) + 1*globalReadIncAL);
    globalReadA_0_0_29_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_29_0) + 1*globalReadIncAL);
    globalReadA_0_0_30_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_30_0) + 1*globalReadIncAL);
    globalReadA_0_0_31_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_31_0) + 1*globalReadIncAL);
    globalReadA_0_0_32_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_32_0) + 1*globalReadIncAL);
    globalReadA_0_0_33_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_33_0) + 1*globalReadIncAL);
    globalReadA_0_0_34_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_34_0) + 1*globalReadIncAL);
    globalReadA_0_0_35_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_35_0) + 1*globalReadIncAL);
    globalReadA_0_0_36_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_36_0) + 1*globalReadIncAL);
    globalReadA_0_0_37_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_37_0) + 1*globalReadIncAL);
    globalReadA_0_0_38_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_38_0) + 1*globalReadIncAL);
    globalReadA_0_0_39_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_39_0) + 1*globalReadIncAL);
    globalReadA_0_0_40_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_40_0) + 1*globalReadIncAL);
    globalReadA_0_0_41_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_41_0) + 1*globalReadIncAL);
    globalReadA_0_0_42_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_42_0) + 1*globalReadIncAL);
    globalReadA_0_0_43_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_43_0) + 1*globalReadIncAL);
    globalReadA_0_0_44_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_44_0) + 1*globalReadIncAL);
    globalReadA_0_0_45_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_45_0) + 1*globalReadIncAL);
    globalReadA_0_0_46_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_46_0) + 1*globalReadIncAL);
    globalReadA_0_0_47_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_47_0) + 1*globalReadIncAL);
    globalReadA_0_0_48_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_48_0) + 1*globalReadIncAL);
    globalReadA_0_0_49_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_49_0) + 1*globalReadIncAL);
    globalReadA_0_0_50_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_50_0) + 1*globalReadIncAL);
    globalReadA_0_0_51_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_51_0) + 1*globalReadIncAL);
    globalReadA_0_0_52_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_52_0) + 1*globalReadIncAL);
    globalReadA_0_0_53_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_53_0) + 1*globalReadIncAL);
    globalReadA_0_0_54_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_54_0) + 1*globalReadIncAL);
    globalReadA_0_0_55_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_55_0) + 1*globalReadIncAL);
    globalReadA_0_0_56_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_56_0) + 1*globalReadIncAL);
    globalReadA_0_0_57_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_57_0) + 1*globalReadIncAL);
    globalReadA_0_0_58_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_58_0) + 1*globalReadIncAL);
    globalReadA_0_0_59_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_59_0) + 1*globalReadIncAL);
    globalReadA_0_0_60_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_60_0) + 1*globalReadIncAL);
    globalReadA_0_0_61_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_61_0) + 1*globalReadIncAL);
    globalReadA_0_0_62_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_62_0) + 1*globalReadIncAL);
    globalReadA_0_0_63_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_63_0) + 1*globalReadIncAL);

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
    *(localWriteA_0_0_16_0 + 0) = a_0_0_16_0;
    *(localWriteA_0_0_17_0 + 0) = a_0_0_17_0;
    *(localWriteA_0_0_18_0 + 0) = a_0_0_18_0;
    *(localWriteA_0_0_19_0 + 0) = a_0_0_19_0;
    *(localWriteA_0_0_20_0 + 0) = a_0_0_20_0;
    *(localWriteA_0_0_21_0 + 0) = a_0_0_21_0;
    *(localWriteA_0_0_22_0 + 0) = a_0_0_22_0;
    *(localWriteA_0_0_23_0 + 0) = a_0_0_23_0;
    *(localWriteA_0_0_24_0 + 0) = a_0_0_24_0;
    *(localWriteA_0_0_25_0 + 0) = a_0_0_25_0;
    *(localWriteA_0_0_26_0 + 0) = a_0_0_26_0;
    *(localWriteA_0_0_27_0 + 0) = a_0_0_27_0;
    *(localWriteA_0_0_28_0 + 0) = a_0_0_28_0;
    *(localWriteA_0_0_29_0 + 0) = a_0_0_29_0;
    *(localWriteA_0_0_30_0 + 0) = a_0_0_30_0;
    *(localWriteA_0_0_31_0 + 0) = a_0_0_31_0;
    *(localWriteA_0_0_32_0 + 0) = a_0_0_32_0;
    *(localWriteA_0_0_33_0 + 0) = a_0_0_33_0;
    *(localWriteA_0_0_34_0 + 0) = a_0_0_34_0;
    *(localWriteA_0_0_35_0 + 0) = a_0_0_35_0;
    *(localWriteA_0_0_36_0 + 0) = a_0_0_36_0;
    *(localWriteA_0_0_37_0 + 0) = a_0_0_37_0;
    *(localWriteA_0_0_38_0 + 0) = a_0_0_38_0;
    *(localWriteA_0_0_39_0 + 0) = a_0_0_39_0;
    *(localWriteA_0_0_40_0 + 0) = a_0_0_40_0;
    *(localWriteA_0_0_41_0 + 0) = a_0_0_41_0;
    *(localWriteA_0_0_42_0 + 0) = a_0_0_42_0;
    *(localWriteA_0_0_43_0 + 0) = a_0_0_43_0;
    *(localWriteA_0_0_44_0 + 0) = a_0_0_44_0;
    *(localWriteA_0_0_45_0 + 0) = a_0_0_45_0;
    *(localWriteA_0_0_46_0 + 0) = a_0_0_46_0;
    *(localWriteA_0_0_47_0 + 0) = a_0_0_47_0;
    *(localWriteA_0_0_48_0 + 0) = a_0_0_48_0;
    *(localWriteA_0_0_49_0 + 0) = a_0_0_49_0;
    *(localWriteA_0_0_50_0 + 0) = a_0_0_50_0;
    *(localWriteA_0_0_51_0 + 0) = a_0_0_51_0;
    *(localWriteA_0_0_52_0 + 0) = a_0_0_52_0;
    *(localWriteA_0_0_53_0 + 0) = a_0_0_53_0;
    *(localWriteA_0_0_54_0 + 0) = a_0_0_54_0;
    *(localWriteA_0_0_55_0 + 0) = a_0_0_55_0;
    *(localWriteA_0_0_56_0 + 0) = a_0_0_56_0;
    *(localWriteA_0_0_57_0 + 0) = a_0_0_57_0;
    *(localWriteA_0_0_58_0 + 0) = a_0_0_58_0;
    *(localWriteA_0_0_59_0 + 0) = a_0_0_59_0;
    *(localWriteA_0_0_60_0 + 0) = a_0_0_60_0;
    *(localWriteA_0_0_61_0 + 0) = a_0_0_61_0;
    *(localWriteA_0_0_62_0 + 0) = a_0_0_62_0;
    *(localWriteA_0_0_63_0 + 0) = a_0_0_63_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 15 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 16 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 17 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 18 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 19 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 20 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 21 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 22 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 23 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 24 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 25 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 26 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 27 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 28 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 29 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 30 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 31 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 32 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 33 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 34 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 35 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 36 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 37 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 38 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 39 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 40 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 41 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 42 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 43 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 44 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 45 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 46 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 47 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 48 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 49 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 50 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 51 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 52 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 53 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 54 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 55 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 56 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 57 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 58 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 59 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 60 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 61 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 62 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_1x1


    /* iter 63 */

    MAC_1x1_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_0_8_0 = ( globalReadOffsetAL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_0_0_9_0 = ( globalReadOffsetAL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_0_0_10_0 = ( globalReadOffsetAL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_0_0_11_0 = ( globalReadOffsetAL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_0_0_12_0 = ( globalReadOffsetAL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_0_0_13_0 = ( globalReadOffsetAL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_0_0_14_0 = ( globalReadOffsetAL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_0_0_15_0 = ( globalReadOffsetAL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);
  a_0_0_16_0 = ( globalReadOffsetAL_16_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_16_0 + 0);
  a_0_0_17_0 = ( globalReadOffsetAL_17_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_17_0 + 0);
  a_0_0_18_0 = ( globalReadOffsetAL_18_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_18_0 + 0);
  a_0_0_19_0 = ( globalReadOffsetAL_19_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_19_0 + 0);
  a_0_0_20_0 = ( globalReadOffsetAL_20_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_20_0 + 0);
  a_0_0_21_0 = ( globalReadOffsetAL_21_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_21_0 + 0);
  a_0_0_22_0 = ( globalReadOffsetAL_22_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_22_0 + 0);
  a_0_0_23_0 = ( globalReadOffsetAL_23_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_23_0 + 0);
  a_0_0_24_0 = ( globalReadOffsetAL_24_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_24_0 + 0);
  a_0_0_25_0 = ( globalReadOffsetAL_25_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_25_0 + 0);
  a_0_0_26_0 = ( globalReadOffsetAL_26_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_26_0 + 0);
  a_0_0_27_0 = ( globalReadOffsetAL_27_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_27_0 + 0);
  a_0_0_28_0 = ( globalReadOffsetAL_28_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_28_0 + 0);
  a_0_0_29_0 = ( globalReadOffsetAL_29_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_29_0 + 0);
  a_0_0_30_0 = ( globalReadOffsetAL_30_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_30_0 + 0);
  a_0_0_31_0 = ( globalReadOffsetAL_31_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_31_0 + 0);
  a_0_0_32_0 = ( globalReadOffsetAL_32_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_32_0 + 0);
  a_0_0_33_0 = ( globalReadOffsetAL_33_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_33_0 + 0);
  a_0_0_34_0 = ( globalReadOffsetAL_34_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_34_0 + 0);
  a_0_0_35_0 = ( globalReadOffsetAL_35_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_35_0 + 0);
  a_0_0_36_0 = ( globalReadOffsetAL_36_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_36_0 + 0);
  a_0_0_37_0 = ( globalReadOffsetAL_37_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_37_0 + 0);
  a_0_0_38_0 = ( globalReadOffsetAL_38_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_38_0 + 0);
  a_0_0_39_0 = ( globalReadOffsetAL_39_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_39_0 + 0);
  a_0_0_40_0 = ( globalReadOffsetAL_40_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_40_0 + 0);
  a_0_0_41_0 = ( globalReadOffsetAL_41_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_41_0 + 0);
  a_0_0_42_0 = ( globalReadOffsetAL_42_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_42_0 + 0);
  a_0_0_43_0 = ( globalReadOffsetAL_43_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_43_0 + 0);
  a_0_0_44_0 = ( globalReadOffsetAL_44_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_44_0 + 0);
  a_0_0_45_0 = ( globalReadOffsetAL_45_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_45_0 + 0);
  a_0_0_46_0 = ( globalReadOffsetAL_46_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_46_0 + 0);
  a_0_0_47_0 = ( globalReadOffsetAL_47_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_47_0 + 0);
  a_0_0_48_0 = ( globalReadOffsetAL_48_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_48_0 + 0);
  a_0_0_49_0 = ( globalReadOffsetAL_49_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_49_0 + 0);
  a_0_0_50_0 = ( globalReadOffsetAL_50_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_50_0 + 0);
  a_0_0_51_0 = ( globalReadOffsetAL_51_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_51_0 + 0);
  a_0_0_52_0 = ( globalReadOffsetAL_52_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_52_0 + 0);
  a_0_0_53_0 = ( globalReadOffsetAL_53_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_53_0 + 0);
  a_0_0_54_0 = ( globalReadOffsetAL_54_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_54_0 + 0);
  a_0_0_55_0 = ( globalReadOffsetAL_55_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_55_0 + 0);
  a_0_0_56_0 = ( globalReadOffsetAL_56_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_56_0 + 0);
  a_0_0_57_0 = ( globalReadOffsetAL_57_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_57_0 + 0);
  a_0_0_58_0 = ( globalReadOffsetAL_58_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_58_0 + 0);
  a_0_0_59_0 = ( globalReadOffsetAL_59_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_59_0 + 0);
  a_0_0_60_0 = ( globalReadOffsetAL_60_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_60_0 + 0);
  a_0_0_61_0 = ( globalReadOffsetAL_61_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_61_0 + 0);
  a_0_0_62_0 = ( globalReadOffsetAL_62_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_62_0 + 0);
  a_0_0_63_0 = ( globalReadOffsetAL_63_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_63_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);
  localWriteA_0_0_16_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_16_0);
  localWriteA_0_0_17_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_17_0);
  localWriteA_0_0_18_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_18_0);
  localWriteA_0_0_19_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_19_0);
  localWriteA_0_0_20_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_20_0);
  localWriteA_0_0_21_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_21_0);
  localWriteA_0_0_22_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_22_0);
  localWriteA_0_0_23_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_23_0);
  localWriteA_0_0_24_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_24_0);
  localWriteA_0_0_25_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_25_0);
  localWriteA_0_0_26_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_26_0);
  localWriteA_0_0_27_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_27_0);
  localWriteA_0_0_28_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_28_0);
  localWriteA_0_0_29_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_29_0);
  localWriteA_0_0_30_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_30_0);
  localWriteA_0_0_31_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_31_0);
  localWriteA_0_0_32_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_32_0);
  localWriteA_0_0_33_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_33_0);
  localWriteA_0_0_34_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_34_0);
  localWriteA_0_0_35_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_35_0);
  localWriteA_0_0_36_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_36_0);
  localWriteA_0_0_37_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_37_0);
  localWriteA_0_0_38_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_38_0);
  localWriteA_0_0_39_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_39_0);
  localWriteA_0_0_40_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_40_0);
  localWriteA_0_0_41_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_41_0);
  localWriteA_0_0_42_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_42_0);
  localWriteA_0_0_43_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_43_0);
  localWriteA_0_0_44_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_44_0);
  localWriteA_0_0_45_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_45_0);
  localWriteA_0_0_46_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_46_0);
  localWriteA_0_0_47_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_47_0);
  localWriteA_0_0_48_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_48_0);
  localWriteA_0_0_49_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_49_0);
  localWriteA_0_0_50_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_50_0);
  localWriteA_0_0_51_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_51_0);
  localWriteA_0_0_52_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_52_0);
  localWriteA_0_0_53_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_53_0);
  localWriteA_0_0_54_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_54_0);
  localWriteA_0_0_55_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_55_0);
  localWriteA_0_0_56_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_56_0);
  localWriteA_0_0_57_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_57_0);
  localWriteA_0_0_58_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_58_0);
  localWriteA_0_0_59_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_59_0);
  localWriteA_0_0_60_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_60_0);
  localWriteA_0_0_61_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_61_0);
  localWriteA_0_0_62_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_62_0);
  localWriteA_0_0_63_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_63_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
  *(localWriteA_0_0_16_0 + 0) = a_0_0_16_0;
  *(localWriteA_0_0_17_0 + 0) = a_0_0_17_0;
  *(localWriteA_0_0_18_0 + 0) = a_0_0_18_0;
  *(localWriteA_0_0_19_0 + 0) = a_0_0_19_0;
  *(localWriteA_0_0_20_0 + 0) = a_0_0_20_0;
  *(localWriteA_0_0_21_0 + 0) = a_0_0_21_0;
  *(localWriteA_0_0_22_0 + 0) = a_0_0_22_0;
  *(localWriteA_0_0_23_0 + 0) = a_0_0_23_0;
  *(localWriteA_0_0_24_0 + 0) = a_0_0_24_0;
  *(localWriteA_0_0_25_0 + 0) = a_0_0_25_0;
  *(localWriteA_0_0_26_0 + 0) = a_0_0_26_0;
  *(localWriteA_0_0_27_0 + 0) = a_0_0_27_0;
  *(localWriteA_0_0_28_0 + 0) = a_0_0_28_0;
  *(localWriteA_0_0_29_0 + 0) = a_0_0_29_0;
  *(localWriteA_0_0_30_0 + 0) = a_0_0_30_0;
  *(localWriteA_0_0_31_0 + 0) = a_0_0_31_0;
  *(localWriteA_0_0_32_0 + 0) = a_0_0_32_0;
  *(localWriteA_0_0_33_0 + 0) = a_0_0_33_0;
  *(localWriteA_0_0_34_0 + 0) = a_0_0_34_0;
  *(localWriteA_0_0_35_0 + 0) = a_0_0_35_0;
  *(localWriteA_0_0_36_0 + 0) = a_0_0_36_0;
  *(localWriteA_0_0_37_0 + 0) = a_0_0_37_0;
  *(localWriteA_0_0_38_0 + 0) = a_0_0_38_0;
  *(localWriteA_0_0_39_0 + 0) = a_0_0_39_0;
  *(localWriteA_0_0_40_0 + 0) = a_0_0_40_0;
  *(localWriteA_0_0_41_0 + 0) = a_0_0_41_0;
  *(localWriteA_0_0_42_0 + 0) = a_0_0_42_0;
  *(localWriteA_0_0_43_0 + 0) = a_0_0_43_0;
  *(localWriteA_0_0_44_0 + 0) = a_0_0_44_0;
  *(localWriteA_0_0_45_0 + 0) = a_0_0_45_0;
  *(localWriteA_0_0_46_0 + 0) = a_0_0_46_0;
  *(localWriteA_0_0_47_0 + 0) = a_0_0_47_0;
  *(localWriteA_0_0_48_0 + 0) = a_0_0_48_0;
  *(localWriteA_0_0_49_0 + 0) = a_0_0_49_0;
  *(localWriteA_0_0_50_0 + 0) = a_0_0_50_0;
  *(localWriteA_0_0_51_0 + 0) = a_0_0_51_0;
  *(localWriteA_0_0_52_0 + 0) = a_0_0_52_0;
  *(localWriteA_0_0_53_0 + 0) = a_0_0_53_0;
  *(localWriteA_0_0_54_0 + 0) = a_0_0_54_0;
  *(localWriteA_0_0_55_0 + 0) = a_0_0_55_0;
  *(localWriteA_0_0_56_0 + 0) = a_0_0_56_0;
  *(localWriteA_0_0_57_0 + 0) = a_0_0_57_0;
  *(localWriteA_0_0_58_0 + 0) = a_0_0_58_0;
  *(localWriteA_0_0_59_0 + 0) = a_0_0_59_0;
  *(localWriteA_0_0_60_0 + 0) = a_0_0_60_0;
  *(localWriteA_0_0_61_0 + 0) = a_0_0_61_0;
  *(localWriteA_0_0_62_0 + 0) = a_0_0_62_0;
  *(localWriteA_0_0_63_0 + 0) = a_0_0_63_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_1x1

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_1x1
#undef MAC_1x1_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 64
#define SG1J 1
#define TT0I 1
#define TT1J 1
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 64
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 15

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 64
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 4096
#define LDS_NUM_ELEMENTS 4160

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 1x1 micro-tile */
#define MAC_1x1\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \

#define MAC_1x1_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x1x64_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU15_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU0_SUM0_SUS128_SRVW0_SVW4_SNLL0_TT1_1_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG64_1_1_WGM1(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE a_0_0_16_0;
  DATA_TYPE a_0_0_17_0;
  DATA_TYPE a_0_0_18_0;
  DATA_TYPE a_0_0_19_0;
  DATA_TYPE a_0_0_20_0;
  DATA_TYPE a_0_0_21_0;
  DATA_TYPE a_0_0_22_0;
  DATA_TYPE a_0_0_23_0;
  DATA_TYPE a_0_0_24_0;
  DATA_TYPE a_0_0_25_0;
  DATA_TYPE a_0_0_26_0;
  DATA_TYPE a_0_0_27_0;
  DATA_TYPE a_0_0_28_0;
  DATA_TYPE a_0_0_29_0;
  DATA_TYPE a_0_0_30_0;
  DATA_TYPE a_0_0_31_0;
  DATA_TYPE a_0_0_32_0;
  DATA_TYPE a_0_0_33_0;
  DATA_TYPE a_0_0_34_0;
  DATA_TYPE a_0_0_35_0;
  DATA_TYPE a_0_0_36_0;
  DATA_TYPE a_0_0_37_0;
  DATA_TYPE a_0_0_38_0;
  DATA_TYPE a_0_0_39_0;
  DATA_TYPE a_0_0_40_0;
  DATA_TYPE a_0_0_41_0;
  DATA_TYPE a_0_0_42_0;
  DATA_TYPE a_0_0_43_0;
  DATA_TYPE a_0_0_44_0;
  DATA_TYPE a_0_0_45_0;
  DATA_TYPE a_0_0_46_0;
  DATA_TYPE a_0_0_47_0;
  DATA_TYPE a_0_0_48_0;
  DATA_TYPE a_0_0_49_0;
  DATA_TYPE a_0_0_50_0;
  DATA_TYPE a_0_0_51_0;
  DATA_TYPE a_0_0_52_0;
  DATA_TYPE a_0_0_53_0;
  DATA_TYPE a_0_0_54_0;
  DATA_TYPE a_0_0_55_0;
  DATA_TYPE a_0_0_56_0;
  DATA_TYPE a_0_0_57_0;
  DATA_TYPE a_0_0_58_0;
  DATA_TYPE a_0_0_59_0;
  DATA_TYPE a_0_0_60_0;
  DATA_TYPE a_0_0_61_0;
  DATA_TYPE a_0_0_62_0;
  DATA_TYPE a_0_0_63_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;
  unsigned int globalReadOffsetAL_8_0 = globalReadOffsetAL + 0 + 8*LSPA;
  unsigned int globalReadOffsetAL_9_0 = globalReadOffsetAL + 0 + 9*LSPA;
  unsigned int globalReadOffsetAL_10_0 = globalReadOffsetAL + 0 + 10*LSPA;
  unsigned int globalReadOffsetAL_11_0 = globalReadOffsetAL + 0 + 11*LSPA;
  unsigned int globalReadOffsetAL_12_0 = globalReadOffsetAL + 0 + 12*LSPA;
  unsigned int globalReadOffsetAL_13_0 = globalReadOffsetAL + 0 + 13*LSPA;
  unsigned int globalReadOffsetAL_14_0 = globalReadOffsetAL + 0 + 14*LSPA;
  unsigned int globalReadOffsetAL_15_0 = globalReadOffsetAL + 0 + 15*LSPA;
  unsigned int globalReadOffsetAL_16_0 = globalReadOffsetAL + 0 + 16*LSPA;
  unsigned int globalReadOffsetAL_17_0 = globalReadOffsetAL + 0 + 17*LSPA;
  unsigned int globalReadOffsetAL_18_0 = globalReadOffsetAL + 0 + 18*LSPA;
  unsigned int globalReadOffsetAL_19_0 = globalReadOffsetAL + 0 + 19*LSPA;
  unsigned int globalReadOffsetAL_20_0 = globalReadOffsetAL + 0 + 20*LSPA;
  unsigned int globalReadOffsetAL_21_0 = globalReadOffsetAL + 0 + 21*LSPA;
  unsigned int globalReadOffsetAL_22_0 = globalReadOffsetAL + 0 + 22*LSPA;
  unsigned int globalReadOffsetAL_23_0 = globalReadOffsetAL + 0 + 23*LSPA;
  unsigned int globalReadOffsetAL_24_0 = globalReadOffsetAL + 0 + 24*LSPA;
  unsigned int globalReadOffsetAL_25_0 = globalReadOffsetAL + 0 + 25*LSPA;
  unsigned int globalReadOffsetAL_26_0 = globalReadOffsetAL + 0 + 26*LSPA;
  unsigned int globalReadOffsetAL_27_0 = globalReadOffsetAL + 0 + 27*LSPA;
  unsigned int globalReadOffsetAL_28_0 = globalReadOffsetAL + 0 + 28*LSPA;
  unsigned int globalReadOffsetAL_29_0 = globalReadOffsetAL + 0 + 29*LSPA;
  unsigned int globalReadOffsetAL_30_0 = globalReadOffsetAL + 0 + 30*LSPA;
  unsigned int globalReadOffsetAL_31_0 = globalReadOffsetAL + 0 + 31*LSPA;
  unsigned int globalReadOffsetAL_32_0 = globalReadOffsetAL + 0 + 32*LSPA;
  unsigned int globalReadOffsetAL_33_0 = globalReadOffsetAL + 0 + 33*LSPA;
  unsigned int globalReadOffsetAL_34_0 = globalReadOffsetAL + 0 + 34*LSPA;
  unsigned int globalReadOffsetAL_35_0 = globalReadOffsetAL + 0 + 35*LSPA;
  unsigned int globalReadOffsetAL_36_0 = globalReadOffsetAL + 0 + 36*LSPA;
  unsigned int globalReadOffsetAL_37_0 = globalReadOffsetAL + 0 + 37*LSPA;
  unsigned int globalReadOffsetAL_38_0 = globalReadOffsetAL + 0 + 38*LSPA;
  unsigned int globalReadOffsetAL_39_0 = globalReadOffsetAL + 0 + 39*LSPA;
  unsigned int globalReadOffsetAL_40_0 = globalReadOffsetAL + 0 + 40*LSPA;
  unsigned int globalReadOffsetAL_41_0 = globalReadOffsetAL + 0 + 41*LSPA;
  unsigned int globalReadOffsetAL_42_0 = globalReadOffsetAL + 0 + 42*LSPA;
  unsigned int globalReadOffsetAL_43_0 = globalReadOffsetAL + 0 + 43*LSPA;
  unsigned int globalReadOffsetAL_44_0 = globalReadOffsetAL + 0 + 44*LSPA;
  unsigned int globalReadOffsetAL_45_0 = globalReadOffsetAL + 0 + 45*LSPA;
  unsigned int globalReadOffsetAL_46_0 = globalReadOffsetAL + 0 + 46*LSPA;
  unsigned int globalReadOffsetAL_47_0 = globalReadOffsetAL + 0 + 47*LSPA;
  unsigned int globalReadOffsetAL_48_0 = globalReadOffsetAL + 0 + 48*LSPA;
  unsigned int globalReadOffsetAL_49_0 = globalReadOffsetAL + 0 + 49*LSPA;
  unsigned int globalReadOffsetAL_50_0 = globalReadOffsetAL + 0 + 50*LSPA;
  unsigned int globalReadOffsetAL_51_0 = globalReadOffsetAL + 0 + 51*LSPA;
  unsigned int globalReadOffsetAL_52_0 = globalReadOffsetAL + 0 + 52*LSPA;
  unsigned int globalReadOffsetAL_53_0 = globalReadOffsetAL + 0 + 53*LSPA;
  unsigned int globalReadOffsetAL_54_0 = globalReadOffsetAL + 0 + 54*LSPA;
  unsigned int globalReadOffsetAL_55_0 = globalReadOffsetAL + 0 + 55*LSPA;
  unsigned int globalReadOffsetAL_56_0 = globalReadOffsetAL + 0 + 56*LSPA;
  unsigned int globalReadOffsetAL_57_0 = globalReadOffsetAL + 0 + 57*LSPA;
  unsigned int globalReadOffsetAL_58_0 = globalReadOffsetAL + 0 + 58*LSPA;
  unsigned int globalReadOffsetAL_59_0 = globalReadOffsetAL + 0 + 59*LSPA;
  unsigned int globalReadOffsetAL_60_0 = globalReadOffsetAL + 0 + 60*LSPA;
  unsigned int globalReadOffsetAL_61_0 = globalReadOffsetAL + 0 + 61*LSPA;
  unsigned int globalReadOffsetAL_62_0 = globalReadOffsetAL + 0 + 62*LSPA;
  unsigned int globalReadOffsetAL_63_0 = globalReadOffsetAL + 0 + 63*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_15_0), wgK );
  int64_t globalReadOffsetA_0_0_16_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_16_0), wgK );
  int64_t globalReadOffsetA_0_0_17_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_17_0), wgK );
  int64_t globalReadOffsetA_0_0_18_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_18_0), wgK );
  int64_t globalReadOffsetA_0_0_19_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_19_0), wgK );
  int64_t globalReadOffsetA_0_0_20_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_20_0), wgK );
  int64_t globalReadOffsetA_0_0_21_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_21_0), wgK );
  int64_t globalReadOffsetA_0_0_22_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_22_0), wgK );
  int64_t globalReadOffsetA_0_0_23_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_23_0), wgK );
  int64_t globalReadOffsetA_0_0_24_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_24_0), wgK );
  int64_t globalReadOffsetA_0_0_25_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_25_0), wgK );
  int64_t globalReadOffsetA_0_0_26_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_26_0), wgK );
  int64_t globalReadOffsetA_0_0_27_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_27_0), wgK );
  int64_t globalReadOffsetA_0_0_28_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_28_0), wgK );
  int64_t globalReadOffsetA_0_0_29_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_29_0), wgK );
  int64_t globalReadOffsetA_0_0_30_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_30_0), wgK );
  int64_t globalReadOffsetA_0_0_31_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_31_0), wgK );
  int64_t globalReadOffsetA_0_0_32_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_32_0), wgK );
  int64_t globalReadOffsetA_0_0_33_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_33_0), wgK );
  int64_t globalReadOffsetA_0_0_34_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_34_0), wgK );
  int64_t globalReadOffsetA_0_0_35_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_35_0), wgK );
  int64_t globalReadOffsetA_0_0_36_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_36_0), wgK );
  int64_t globalReadOffsetA_0_0_37_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_37_0), wgK );
  int64_t globalReadOffsetA_0_0_38_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_38_0), wgK );
  int64_t globalReadOffsetA_0_0_39_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_39_0), wgK );
  int64_t globalReadOffsetA_0_0_40_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_40_0), wgK );
  int64_t globalReadOffsetA_0_0_41_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_41_0), wgK );
  int64_t globalReadOffsetA_0_0_42_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_42_0), wgK );
  int64_t globalReadOffsetA_0_0_43_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_43_0), wgK );
  int64_t globalReadOffsetA_0_0_44_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_44_0), wgK );
  int64_t globalReadOffsetA_0_0_45_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_45_0), wgK );
  int64_t globalReadOffsetA_0_0_46_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_46_0), wgK );
  int64_t globalReadOffsetA_0_0_47_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_47_0), wgK );
  int64_t globalReadOffsetA_0_0_48_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_48_0), wgK );
  int64_t globalReadOffsetA_0_0_49_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_49_0), wgK );
  int64_t globalReadOffsetA_0_0_50_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_50_0), wgK );
  int64_t globalReadOffsetA_0_0_51_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_51_0), wgK );
  int64_t globalReadOffsetA_0_0_52_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_52_0), wgK );
  int64_t globalReadOffsetA_0_0_53_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_53_0), wgK );
  int64_t globalReadOffsetA_0_0_54_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_54_0), wgK );
  int64_t globalReadOffsetA_0_0_55_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_55_0), wgK );
  int64_t globalReadOffsetA_0_0_56_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_56_0), wgK );
  int64_t globalReadOffsetA_0_0_57_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_57_0), wgK );
  int64_t globalReadOffsetA_0_0_58_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_58_0), wgK );
  int64_t globalReadOffsetA_0_0_59_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_59_0), wgK );
  int64_t globalReadOffsetA_0_0_60_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_60_0), wgK );
  int64_t globalReadOffsetA_0_0_61_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_61_0), wgK );
  int64_t globalReadOffsetA_0_0_62_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_62_0), wgK );
  int64_t globalReadOffsetA_0_0_63_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_63_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;
  DATA_TYPE const *globalReadA_0_0_16_0 = A + globalReadOffsetA_0_0_16_0;
  DATA_TYPE const *globalReadA_0_0_17_0 = A + globalReadOffsetA_0_0_17_0;
  DATA_TYPE const *globalReadA_0_0_18_0 = A + globalReadOffsetA_0_0_18_0;
  DATA_TYPE const *globalReadA_0_0_19_0 = A + globalReadOffsetA_0_0_19_0;
  DATA_TYPE const *globalReadA_0_0_20_0 = A + globalReadOffsetA_0_0_20_0;
  DATA_TYPE const *globalReadA_0_0_21_0 = A + globalReadOffsetA_0_0_21_0;
  DATA_TYPE const *globalReadA_0_0_22_0 = A + globalReadOffsetA_0_0_22_0;
  DATA_TYPE const *globalReadA_0_0_23_0 = A + globalReadOffsetA_0_0_23_0;
  DATA_TYPE const *globalReadA_0_0_24_0 = A + globalReadOffsetA_0_0_24_0;
  DATA_TYPE const *globalReadA_0_0_25_0 = A + globalReadOffsetA_0_0_25_0;
  DATA_TYPE const *globalReadA_0_0_26_0 = A + globalReadOffsetA_0_0_26_0;
  DATA_TYPE const *globalReadA_0_0_27_0 = A + globalReadOffsetA_0_0_27_0;
  DATA_TYPE const *globalReadA_0_0_28_0 = A + globalReadOffsetA_0_0_28_0;
  DATA_TYPE const *globalReadA_0_0_29_0 = A + globalReadOffsetA_0_0_29_0;
  DATA_TYPE const *globalReadA_0_0_30_0 = A + globalReadOffsetA_0_0_30_0;
  DATA_TYPE const *globalReadA_0_0_31_0 = A + globalReadOffsetA_0_0_31_0;
  DATA_TYPE const *globalReadA_0_0_32_0 = A + globalReadOffsetA_0_0_32_0;
  DATA_TYPE const *globalReadA_0_0_33_0 = A + globalReadOffsetA_0_0_33_0;
  DATA_TYPE const *globalReadA_0_0_34_0 = A + globalReadOffsetA_0_0_34_0;
  DATA_TYPE const *globalReadA_0_0_35_0 = A + globalReadOffsetA_0_0_35_0;
  DATA_TYPE const *globalReadA_0_0_36_0 = A + globalReadOffsetA_0_0_36_0;
  DATA_TYPE const *globalReadA_0_0_37_0 = A + globalReadOffsetA_0_0_37_0;
  DATA_TYPE const *globalReadA_0_0_38_0 = A + globalReadOffsetA_0_0_38_0;
  DATA_TYPE const *globalReadA_0_0_39_0 = A + globalReadOffsetA_0_0_39_0;
  DATA_TYPE const *globalReadA_0_0_40_0 = A + globalReadOffsetA_0_0_40_0;
  DATA_TYPE const *globalReadA_0_0_41_0 = A + globalReadOffsetA_0_0_41_0;
  DATA_TYPE const *globalReadA_0_0_42_0 = A + globalReadOffsetA_0_0_42_0;
  DATA_TYPE const *globalReadA_0_0_43_0 = A + globalReadOffsetA_0_0_43_0;
  DATA_TYPE const *globalReadA_0_0_44_0 = A + globalReadOffsetA_0_0_44_0;
  DATA_TYPE const *globalReadA_0_0_45_0 = A + globalReadOffsetA_0_0_45_0;
  DATA_TYPE const *globalReadA_0_0_46_0 = A + globalReadOffsetA_0_0_46_0;
  DATA_TYPE const *globalReadA_0_0_47_0 = A + globalReadOffsetA_0_0_47_0;
  DATA_TYPE const *globalReadA_0_0_48_0 = A + globalReadOffsetA_0_0_48_0;
  DATA_TYPE const *globalReadA_0_0_49_0 = A + globalReadOffsetA_0_0_49_0;
  DATA_TYPE const *globalReadA_0_0_50_0 = A + globalReadOffsetA_0_0_50_0;
  DATA_TYPE const *globalReadA_0_0_51_0 = A + globalReadOffsetA_0_0_51_0;
  DATA_TYPE const *globalReadA_0_0_52_0 = A + globalReadOffsetA_0_0_52_0;
  DATA_TYPE const *globalReadA_0_0_53_0 = A + globalReadOffsetA_0_0_53_0;
  DATA_TYPE const *globalReadA_0_0_54_0 = A + globalReadOffsetA_0_0_54_0;
  DATA_TYPE const *globalReadA_0_0_55_0 = A + globalReadOffsetA_0_0_55_0;
  DATA_TYPE const *globalReadA_0_0_56_0 = A + globalReadOffsetA_0_0_56_0;
  DATA_TYPE const *globalReadA_0_0_57_0 = A + globalReadOffsetA_0_0_57_0;
  DATA_TYPE const *globalReadA_0_0_58_0 = A + globalReadOffsetA_0_0_58_0;
  DATA_TYPE const *globalReadA_0_0_59_0 = A + globalReadOffsetA_0_0_59_0;
  DATA_TYPE const *globalReadA_0_0_60_0 = A + globalReadOffsetA_0_0_60_0;
  DATA_TYPE const *globalReadA_0_0_61_0 = A + globalReadOffsetA_0_0_61_0;
  DATA_TYPE const *globalReadA_0_0_62_0 = A + globalReadOffsetA_0_0_62_0;
  DATA_TYPE const *globalReadA_0_0_63_0 = A + globalReadOffsetA_0_0_63_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 8*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 9*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 10*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 11*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 12*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 13*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 14*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 15*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_16_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 16*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_17_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 17*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_18_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 18*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_19_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 19*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_20_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 20*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_21_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 21*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_22_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 22*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_23_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 23*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_24_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 24*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_25_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 25*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_26_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 26*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_27_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 27*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_28_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 28*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_29_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 29*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_30_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 30*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_31_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 31*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_32_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 32*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_33_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 33*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_34_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 34*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_35_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 35*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_36_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 36*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_37_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 37*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_38_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 38*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_39_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 39*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_40_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 40*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_41_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 41*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_42_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 42*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_43_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 43*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_44_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 44*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_45_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 45*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_46_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 46*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_47_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 47*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_48_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 48*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_49_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 49*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_50_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 50*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_51_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 51*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_52_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 52*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_53_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 53*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_54_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 54*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_55_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 55*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_56_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 56*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_57_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 57*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_58_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 58*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_59_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 59*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_60_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 60*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_61_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 61*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_62_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 62*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_63_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 63*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_0_0_15_0;
  DATA_TYPE *localWriteA_0_0_16_0;
  DATA_TYPE *localWriteA_0_0_17_0;
  DATA_TYPE *localWriteA_0_0_18_0;
  DATA_TYPE *localWriteA_0_0_19_0;
  DATA_TYPE *localWriteA_0_0_20_0;
  DATA_TYPE *localWriteA_0_0_21_0;
  DATA_TYPE *localWriteA_0_0_22_0;
  DATA_TYPE *localWriteA_0_0_23_0;
  DATA_TYPE *localWriteA_0_0_24_0;
  DATA_TYPE *localWriteA_0_0_25_0;
  DATA_TYPE *localWriteA_0_0_26_0;
  DATA_TYPE *localWriteA_0_0_27_0;
  DATA_TYPE *localWriteA_0_0_28_0;
  DATA_TYPE *localWriteA_0_0_29_0;
  DATA_TYPE *localWriteA_0_0_30_0;
  DATA_TYPE *localWriteA_0_0_31_0;
  DATA_TYPE *localWriteA_0_0_32_0;
  DATA_TYPE *localWriteA_0_0_33_0;
  DATA_TYPE *localWriteA_0_0_34_0;
  DATA_TYPE *localWriteA_0_0_35_0;
  DATA_TYPE *localWriteA_0_0_36_0;
  DATA_TYPE *localWriteA_0_0_37_0;
  DATA_TYPE *localWriteA_0_0_38_0;
  DATA_TYPE *localWriteA_0_0_39_0;
  DATA_TYPE *localWriteA_0_0_40_0;
  DATA_TYPE *localWriteA_0_0_41_0;
  DATA_TYPE *localWriteA_0_0_42_0;
  DATA_TYPE *localWriteA_0_0_43_0;
  DATA_TYPE *localWriteA_0_0_44_0;
  DATA_TYPE *localWriteA_0_0_45_0;
  DATA_TYPE *localWriteA_0_0_46_0;
  DATA_TYPE *localWriteA_0_0_47_0;
  DATA_TYPE *localWriteA_0_0_48_0;
  DATA_TYPE *localWriteA_0_0_49_0;
  DATA_TYPE *localWriteA_0_0_50_0;
  DATA_TYPE *localWriteA_0_0_51_0;
  DATA_TYPE *localWriteA_0_0_52_0;
  DATA_TYPE *localWriteA_0_0_53_0;
  DATA_TYPE *localWriteA_0_0_54_0;
  DATA_TYPE *localWriteA_0_0_55_0;
  DATA_TYPE *localWriteA_0_0_56_0;
  DATA_TYPE *localWriteA_0_0_57_0;
  DATA_TYPE *localWriteA_0_0_58_0;
  DATA_TYPE *localWriteA_0_0_59_0;
  DATA_TYPE *localWriteA_0_0_60_0;
  DATA_TYPE *localWriteA_0_0_61_0;
  DATA_TYPE *localWriteA_0_0_62_0;
  DATA_TYPE *localWriteA_0_0_63_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);
  localWriteA_0_0_16_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_16_0);
  localWriteA_0_0_17_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_17_0);
  localWriteA_0_0_18_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_18_0);
  localWriteA_0_0_19_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_19_0);
  localWriteA_0_0_20_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_20_0);
  localWriteA_0_0_21_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_21_0);
  localWriteA_0_0_22_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_22_0);
  localWriteA_0_0_23_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_23_0);
  localWriteA_0_0_24_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_24_0);
  localWriteA_0_0_25_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_25_0);
  localWriteA_0_0_26_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_26_0);
  localWriteA_0_0_27_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_27_0);
  localWriteA_0_0_28_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_28_0);
  localWriteA_0_0_29_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_29_0);
  localWriteA_0_0_30_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_30_0);
  localWriteA_0_0_31_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_31_0);
  localWriteA_0_0_32_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_32_0);
  localWriteA_0_0_33_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_33_0);
  localWriteA_0_0_34_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_34_0);
  localWriteA_0_0_35_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_35_0);
  localWriteA_0_0_36_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_36_0);
  localWriteA_0_0_37_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_37_0);
  localWriteA_0_0_38_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_38_0);
  localWriteA_0_0_39_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_39_0);
  localWriteA_0_0_40_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_40_0);
  localWriteA_0_0_41_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_41_0);
  localWriteA_0_0_42_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_42_0);
  localWriteA_0_0_43_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_43_0);
  localWriteA_0_0_44_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_44_0);
  localWriteA_0_0_45_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_45_0);
  localWriteA_0_0_46_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_46_0);
  localWriteA_0_0_47_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_47_0);
  localWriteA_0_0_48_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_48_0);
  localWriteA_0_0_49_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_49_0);
  localWriteA_0_0_50_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_50_0);
  localWriteA_0_0_51_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_51_0);
  localWriteA_0_0_52_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_52_0);
  localWriteA_0_0_53_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_53_0);
  localWriteA_0_0_54_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_54_0);
  localWriteA_0_0_55_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_55_0);
  localWriteA_0_0_56_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_56_0);
  localWriteA_0_0_57_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_57_0);
  localWriteA_0_0_58_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_58_0);
  localWriteA_0_0_59_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_59_0);
  localWriteA_0_0_60_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_60_0);
  localWriteA_0_0_61_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_61_0);
  localWriteA_0_0_62_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_62_0);
  localWriteA_0_0_63_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_63_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);
    a_0_0_16_0 = *(globalReadA_0_0_16_0 + 0);
    a_0_0_17_0 = *(globalReadA_0_0_17_0 + 0);
    a_0_0_18_0 = *(globalReadA_0_0_18_0 + 0);
    a_0_0_19_0 = *(globalReadA_0_0_19_0 + 0);
    a_0_0_20_0 = *(globalReadA_0_0_20_0 + 0);
    a_0_0_21_0 = *(globalReadA_0_0_21_0 + 0);
    a_0_0_22_0 = *(globalReadA_0_0_22_0 + 0);
    a_0_0_23_0 = *(globalReadA_0_0_23_0 + 0);
    a_0_0_24_0 = *(globalReadA_0_0_24_0 + 0);
    a_0_0_25_0 = *(globalReadA_0_0_25_0 + 0);
    a_0_0_26_0 = *(globalReadA_0_0_26_0 + 0);
    a_0_0_27_0 = *(globalReadA_0_0_27_0 + 0);
    a_0_0_28_0 = *(globalReadA_0_0_28_0 + 0);
    a_0_0_29_0 = *(globalReadA_0_0_29_0 + 0);
    a_0_0_30_0 = *(globalReadA_0_0_30_0 + 0);
    a_0_0_31_0 = *(globalReadA_0_0_31_0 + 0);
    a_0_0_32_0 = *(globalReadA_0_0_32_0 + 0);
    a_0_0_33_0 = *(globalReadA_0_0_33_0 + 0);
    a_0_0_34_0 = *(globalReadA_0_0_34_0 + 0);
    a_0_0_35_0 = *(globalReadA_0_0_35_0 + 0);
    a_0_0_36_0 = *(globalReadA_0_0_36_0 + 0);
    a_0_0_37_0 = *(globalReadA_0_0_37_0 + 0);
    a_0_0_38_0 = *(globalReadA_0_0_38_0 + 0);
    a_0_0_39_0 = *(globalReadA_0_0_39_0 + 0);
    a_0_0_40_0 = *(globalReadA_0_0_40_0 + 0);
    a_0_0_41_0 = *(globalReadA_0_0_41_0 + 0);
    a_0_0_42_0 = *(globalReadA_0_0_42_0 + 0);
    a_0_0_43_0 = *(globalReadA_0_0_43_0 + 0);
    a_0_0_44_0 = *(globalReadA_0_0_44_0 + 0);
    a_0_0_45_0 = *(globalReadA_0_0_45_0 + 0);
    a_0_0_46_0 = *(globalReadA_0_0_46_0 + 0);
    a_0_0_47_0 = *(globalReadA_0_0_47_0 + 0);
    a_0_0_48_0 = *(globalReadA_0_0_48_0 + 0);
    a_0_0_49_0 = *(globalReadA_0_0_49_0 + 0);
    a_0_0_50_0 = *(globalReadA_0_0_50_0 + 0);
    a_0_0_51_0 = *(globalReadA_0_0_51_0 + 0);
    a_0_0_52_0 = *(globalReadA_0_0_52_0 + 0);
    a_0_0_53_0 = *(globalReadA_0_0_53_0 + 0);
    a_0_0_54_0 = *(globalReadA_0_0_54_0 + 0);
    a_0_0_55_0 = *(globalReadA_0_0_55_0 + 0);
    a_0_0_56_0 = *(globalReadA_0_0_56_0 + 0);
    a_0_0_57_0 = *(globalReadA_0_0_57_0 + 0);
    a_0_0_58_0 = *(globalReadA_0_0_58_0 + 0);
    a_0_0_59_0 = *(globalReadA_0_0_59_0 + 0);
    a_0_0_60_0 = *(globalReadA_0_0_60_0 + 0);
    a_0_0_61_0 = *(globalReadA_0_0_61_0 + 0);
    a_0_0_62_0 = *(globalReadA_0_0_62_0 + 0);
    a_0_0_63_0 = *(globalReadA_0_0_63_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    globalReadA_0_0_16_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_16_0) + 1*globalReadIncAL);
    globalReadA_0_0_17_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_17_0) + 1*globalReadIncAL);
    globalReadA_0_0_18_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_18_0) + 1*globalReadIncAL);
    globalReadA_0_0_19_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_19_0) + 1*globalReadIncAL);
    globalReadA_0_0_20_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_20_0) + 1*globalReadIncAL);
    globalReadA_0_0_21_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_21_0) + 1*globalReadIncAL);
    globalReadA_0_0_22_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_22_0) + 1*globalReadIncAL);
    globalReadA_0_0_23_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_23_0) + 1*globalReadIncAL);
    globalReadA_0_0_24_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_24_0) + 1*globalReadIncAL);
    globalReadA_0_0_25_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_25_0) + 1*globalReadIncAL);
    globalReadA_0_0_26_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_26_0) + 1*globalReadIncAL);
    globalReadA_0_0_27_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_27_0) + 1*globalReadIncAL);
    globalReadA_0_0_28_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_28_0) + 1*globalReadIncAL);
    globalReadA_0_0_29_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_29_0) + 1*globalReadIncAL);
    globalReadA_0_0_30_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_30_0) + 1*globalReadIncAL);
    globalReadA_0_0_31_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_31_0) + 1*globalReadIncAL);
    globalReadA_0_0_32_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_32_0) + 1*globalReadIncAL);
    globalReadA_0_0_33_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_33_0) + 1*globalReadIncAL);
    globalReadA_0_0_34_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_34_0) + 1*globalReadIncAL);
    globalReadA_0_0_35_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_35_0) + 1*globalReadIncAL);
    globalReadA_0_0_36_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_36_0) + 1*globalReadIncAL);
    globalReadA_0_0_37_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_37_0) + 1*globalReadIncAL);
    globalReadA_0_0_38_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_38_0) + 1*globalReadIncAL);
    globalReadA_0_0_39_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_39_0) + 1*globalReadIncAL);
    globalReadA_0_0_40_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_40_0) + 1*globalReadIncAL);
    globalReadA_0_0_41_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_41_0) + 1*globalReadIncAL);
    globalReadA_0_0_42_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_42_0) + 1*globalReadIncAL);
    globalReadA_0_0_43_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_43_0) + 1*globalReadIncAL);
    globalReadA_0_0_44_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_44_0) + 1*globalReadIncAL);
    globalReadA_0_0_45_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_45_0) + 1*globalReadIncAL);
    globalReadA_0_0_46_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_46_0) + 1*globalReadIncAL);
    globalReadA_0_0_47_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_47_0) + 1*globalReadIncAL);
    globalReadA_0_0_48_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_48_0) + 1*globalReadIncAL);
    globalReadA_0_0_49_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_49_0) + 1*globalReadIncAL);
    globalReadA_0_0_50_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_50_0) + 1*globalReadIncAL);
    globalReadA_0_0_51_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_51_0) + 1*globalReadIncAL);
    globalReadA_0_0_52_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_52_0) + 1*globalReadIncAL);
    globalReadA_0_0_53_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_53_0) + 1*globalReadIncAL);
    globalReadA_0_0_54_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_54_0) + 1*globalReadIncAL);
    globalReadA_0_0_55_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_55_0) + 1*globalReadIncAL);
    globalReadA_0_0_56_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_56_0) + 1*globalReadIncAL);
    globalReadA_0_0_57_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_57_0) + 1*globalReadIncAL);
    globalReadA_0_0_58_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_58_0) + 1*globalReadIncAL);
    globalReadA_0_0_59_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_59_0) + 1*globalReadIncAL);
    globalReadA_0_0_60_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_60_0) + 1*globalReadIncAL);
    globalReadA_0_0_61_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_61_0) + 1*globalReadIncAL);
    globalReadA_0_0_62_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_62_0) + 1*globalReadIncAL);
    globalReadA_0_0_63_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_63_0) + 1*globalReadIncAL);

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
    *(localWriteA_0_0_16_0 + 0) = a_0_0_16_0;
    *(localWriteA_0_0_17_0 + 0) = a_0_0_17_0;
    *(localWriteA_0_0_18_0 + 0) = a_0_0_18_0;
    *(localWriteA_0_0_19_0 + 0) = a_0_0_19_0;
    *(localWriteA_0_0_20_0 + 0) = a_0_0_20_0;
    *(localWriteA_0_0_21_0 + 0) = a_0_0_21_0;
    *(localWriteA_0_0_22_0 + 0) = a_0_0_22_0;
    *(localWriteA_0_0_23_0 + 0) = a_0_0_23_0;
    *(localWriteA_0_0_24_0 + 0) = a_0_0_24_0;
    *(localWriteA_0_0_25_0 + 0) = a_0_0_25_0;
    *(localWriteA_0_0_26_0 + 0) = a_0_0_26_0;
    *(localWriteA_0_0_27_0 + 0) = a_0_0_27_0;
    *(localWriteA_0_0_28_0 + 0) = a_0_0_28_0;
    *(localWriteA_0_0_29_0 + 0) = a_0_0_29_0;
    *(localWriteA_0_0_30_0 + 0) = a_0_0_30_0;
    *(localWriteA_0_0_31_0 + 0) = a_0_0_31_0;
    *(localWriteA_0_0_32_0 + 0) = a_0_0_32_0;
    *(localWriteA_0_0_33_0 + 0) = a_0_0_33_0;
    *(localWriteA_0_0_34_0 + 0) = a_0_0_34_0;
    *(localWriteA_0_0_35_0 + 0) = a_0_0_35_0;
    *(localWriteA_0_0_36_0 + 0) = a_0_0_36_0;
    *(localWriteA_0_0_37_0 + 0) = a_0_0_37_0;
    *(localWriteA_0_0_38_0 + 0) = a_0_0_38_0;
    *(localWriteA_0_0_39_0 + 0) = a_0_0_39_0;
    *(localWriteA_0_0_40_0 + 0) = a_0_0_40_0;
    *(localWriteA_0_0_41_0 + 0) = a_0_0_41_0;
    *(localWriteA_0_0_42_0 + 0) = a_0_0_42_0;
    *(localWriteA_0_0_43_0 + 0) = a_0_0_43_0;
    *(localWriteA_0_0_44_0 + 0) = a_0_0_44_0;
    *(localWriteA_0_0_45_0 + 0) = a_0_0_45_0;
    *(localWriteA_0_0_46_0 + 0) = a_0_0_46_0;
    *(localWriteA_0_0_47_0 + 0) = a_0_0_47_0;
    *(localWriteA_0_0_48_0 + 0) = a_0_0_48_0;
    *(localWriteA_0_0_49_0 + 0) = a_0_0_49_0;
    *(localWriteA_0_0_50_0 + 0) = a_0_0_50_0;
    *(localWriteA_0_0_51_0 + 0) = a_0_0_51_0;
    *(localWriteA_0_0_52_0 + 0) = a_0_0_52_0;
    *(localWriteA_0_0_53_0 + 0) = a_0_0_53_0;
    *(localWriteA_0_0_54_0 + 0) = a_0_0_54_0;
    *(localWriteA_0_0_55_0 + 0) = a_0_0_55_0;
    *(localWriteA_0_0_56_0 + 0) = a_0_0_56_0;
    *(localWriteA_0_0_57_0 + 0) = a_0_0_57_0;
    *(localWriteA_0_0_58_0 + 0) = a_0_0_58_0;
    *(localWriteA_0_0_59_0 + 0) = a_0_0_59_0;
    *(localWriteA_0_0_60_0 + 0) = a_0_0_60_0;
    *(localWriteA_0_0_61_0 + 0) = a_0_0_61_0;
    *(localWriteA_0_0_62_0 + 0) = a_0_0_62_0;
    *(localWriteA_0_0_63_0 + 0) = a_0_0_63_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 15 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 16 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 17 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 18 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 19 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 20 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 21 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 22 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 23 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 24 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 25 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 26 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 27 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 28 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 29 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 30 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 31 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 32 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 33 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 34 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 35 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 36 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 37 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 38 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 39 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 40 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 41 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 42 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 43 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 44 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 45 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 46 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 47 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 48 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 49 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 50 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 51 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 52 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 53 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 54 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 55 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 56 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 57 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 58 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 59 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 60 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 61 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 62 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_1x1


    /* iter 63 */

    MAC_1x1_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_0_8_0 = ( globalReadOffsetAL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_0_0_9_0 = ( globalReadOffsetAL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_0_0_10_0 = ( globalReadOffsetAL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_0_0_11_0 = ( globalReadOffsetAL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_0_0_12_0 = ( globalReadOffsetAL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_0_0_13_0 = ( globalReadOffsetAL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_0_0_14_0 = ( globalReadOffsetAL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_0_0_15_0 = ( globalReadOffsetAL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);
  a_0_0_16_0 = ( globalReadOffsetAL_16_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_16_0 + 0);
  a_0_0_17_0 = ( globalReadOffsetAL_17_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_17_0 + 0);
  a_0_0_18_0 = ( globalReadOffsetAL_18_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_18_0 + 0);
  a_0_0_19_0 = ( globalReadOffsetAL_19_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_19_0 + 0);
  a_0_0_20_0 = ( globalReadOffsetAL_20_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_20_0 + 0);
  a_0_0_21_0 = ( globalReadOffsetAL_21_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_21_0 + 0);
  a_0_0_22_0 = ( globalReadOffsetAL_22_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_22_0 + 0);
  a_0_0_23_0 = ( globalReadOffsetAL_23_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_23_0 + 0);
  a_0_0_24_0 = ( globalReadOffsetAL_24_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_24_0 + 0);
  a_0_0_25_0 = ( globalReadOffsetAL_25_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_25_0 + 0);
  a_0_0_26_0 = ( globalReadOffsetAL_26_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_26_0 + 0);
  a_0_0_27_0 = ( globalReadOffsetAL_27_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_27_0 + 0);
  a_0_0_28_0 = ( globalReadOffsetAL_28_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_28_0 + 0);
  a_0_0_29_0 = ( globalReadOffsetAL_29_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_29_0 + 0);
  a_0_0_30_0 = ( globalReadOffsetAL_30_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_30_0 + 0);
  a_0_0_31_0 = ( globalReadOffsetAL_31_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_31_0 + 0);
  a_0_0_32_0 = ( globalReadOffsetAL_32_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_32_0 + 0);
  a_0_0_33_0 = ( globalReadOffsetAL_33_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_33_0 + 0);
  a_0_0_34_0 = ( globalReadOffsetAL_34_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_34_0 + 0);
  a_0_0_35_0 = ( globalReadOffsetAL_35_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_35_0 + 0);
  a_0_0_36_0 = ( globalReadOffsetAL_36_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_36_0 + 0);
  a_0_0_37_0 = ( globalReadOffsetAL_37_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_37_0 + 0);
  a_0_0_38_0 = ( globalReadOffsetAL_38_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_38_0 + 0);
  a_0_0_39_0 = ( globalReadOffsetAL_39_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_39_0 + 0);
  a_0_0_40_0 = ( globalReadOffsetAL_40_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_40_0 + 0);
  a_0_0_41_0 = ( globalReadOffsetAL_41_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_41_0 + 0);
  a_0_0_42_0 = ( globalReadOffsetAL_42_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_42_0 + 0);
  a_0_0_43_0 = ( globalReadOffsetAL_43_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_43_0 + 0);
  a_0_0_44_0 = ( globalReadOffsetAL_44_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_44_0 + 0);
  a_0_0_45_0 = ( globalReadOffsetAL_45_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_45_0 + 0);
  a_0_0_46_0 = ( globalReadOffsetAL_46_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_46_0 + 0);
  a_0_0_47_0 = ( globalReadOffsetAL_47_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_47_0 + 0);
  a_0_0_48_0 = ( globalReadOffsetAL_48_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_48_0 + 0);
  a_0_0_49_0 = ( globalReadOffsetAL_49_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_49_0 + 0);
  a_0_0_50_0 = ( globalReadOffsetAL_50_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_50_0 + 0);
  a_0_0_51_0 = ( globalReadOffsetAL_51_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_51_0 + 0);
  a_0_0_52_0 = ( globalReadOffsetAL_52_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_52_0 + 0);
  a_0_0_53_0 = ( globalReadOffsetAL_53_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_53_0 + 0);
  a_0_0_54_0 = ( globalReadOffsetAL_54_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_54_0 + 0);
  a_0_0_55_0 = ( globalReadOffsetAL_55_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_55_0 + 0);
  a_0_0_56_0 = ( globalReadOffsetAL_56_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_56_0 + 0);
  a_0_0_57_0 = ( globalReadOffsetAL_57_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_57_0 + 0);
  a_0_0_58_0 = ( globalReadOffsetAL_58_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_58_0 + 0);
  a_0_0_59_0 = ( globalReadOffsetAL_59_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_59_0 + 0);
  a_0_0_60_0 = ( globalReadOffsetAL_60_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_60_0 + 0);
  a_0_0_61_0 = ( globalReadOffsetAL_61_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_61_0 + 0);
  a_0_0_62_0 = ( globalReadOffsetAL_62_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_62_0 + 0);
  a_0_0_63_0 = ( globalReadOffsetAL_63_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_63_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);
  localWriteA_0_0_16_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_16_0);
  localWriteA_0_0_17_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_17_0);
  localWriteA_0_0_18_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_18_0);
  localWriteA_0_0_19_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_19_0);
  localWriteA_0_0_20_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_20_0);
  localWriteA_0_0_21_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_21_0);
  localWriteA_0_0_22_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_22_0);
  localWriteA_0_0_23_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_23_0);
  localWriteA_0_0_24_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_24_0);
  localWriteA_0_0_25_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_25_0);
  localWriteA_0_0_26_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_26_0);
  localWriteA_0_0_27_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_27_0);
  localWriteA_0_0_28_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_28_0);
  localWriteA_0_0_29_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_29_0);
  localWriteA_0_0_30_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_30_0);
  localWriteA_0_0_31_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_31_0);
  localWriteA_0_0_32_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_32_0);
  localWriteA_0_0_33_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_33_0);
  localWriteA_0_0_34_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_34_0);
  localWriteA_0_0_35_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_35_0);
  localWriteA_0_0_36_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_36_0);
  localWriteA_0_0_37_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_37_0);
  localWriteA_0_0_38_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_38_0);
  localWriteA_0_0_39_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_39_0);
  localWriteA_0_0_40_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_40_0);
  localWriteA_0_0_41_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_41_0);
  localWriteA_0_0_42_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_42_0);
  localWriteA_0_0_43_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_43_0);
  localWriteA_0_0_44_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_44_0);
  localWriteA_0_0_45_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_45_0);
  localWriteA_0_0_46_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_46_0);
  localWriteA_0_0_47_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_47_0);
  localWriteA_0_0_48_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_48_0);
  localWriteA_0_0_49_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_49_0);
  localWriteA_0_0_50_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_50_0);
  localWriteA_0_0_51_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_51_0);
  localWriteA_0_0_52_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_52_0);
  localWriteA_0_0_53_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_53_0);
  localWriteA_0_0_54_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_54_0);
  localWriteA_0_0_55_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_55_0);
  localWriteA_0_0_56_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_56_0);
  localWriteA_0_0_57_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_57_0);
  localWriteA_0_0_58_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_58_0);
  localWriteA_0_0_59_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_59_0);
  localWriteA_0_0_60_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_60_0);
  localWriteA_0_0_61_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_61_0);
  localWriteA_0_0_62_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_62_0);
  localWriteA_0_0_63_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_63_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
  *(localWriteA_0_0_16_0 + 0) = a_0_0_16_0;
  *(localWriteA_0_0_17_0 + 0) = a_0_0_17_0;
  *(localWriteA_0_0_18_0 + 0) = a_0_0_18_0;
  *(localWriteA_0_0_19_0 + 0) = a_0_0_19_0;
  *(localWriteA_0_0_20_0 + 0) = a_0_0_20_0;
  *(localWriteA_0_0_21_0 + 0) = a_0_0_21_0;
  *(localWriteA_0_0_22_0 + 0) = a_0_0_22_0;
  *(localWriteA_0_0_23_0 + 0) = a_0_0_23_0;
  *(localWriteA_0_0_24_0 + 0) = a_0_0_24_0;
  *(localWriteA_0_0_25_0 + 0) = a_0_0_25_0;
  *(localWriteA_0_0_26_0 + 0) = a_0_0_26_0;
  *(localWriteA_0_0_27_0 + 0) = a_0_0_27_0;
  *(localWriteA_0_0_28_0 + 0) = a_0_0_28_0;
  *(localWriteA_0_0_29_0 + 0) = a_0_0_29_0;
  *(localWriteA_0_0_30_0 + 0) = a_0_0_30_0;
  *(localWriteA_0_0_31_0 + 0) = a_0_0_31_0;
  *(localWriteA_0_0_32_0 + 0) = a_0_0_32_0;
  *(localWriteA_0_0_33_0 + 0) = a_0_0_33_0;
  *(localWriteA_0_0_34_0 + 0) = a_0_0_34_0;
  *(localWriteA_0_0_35_0 + 0) = a_0_0_35_0;
  *(localWriteA_0_0_36_0 + 0) = a_0_0_36_0;
  *(localWriteA_0_0_37_0 + 0) = a_0_0_37_0;
  *(localWriteA_0_0_38_0 + 0) = a_0_0_38_0;
  *(localWriteA_0_0_39_0 + 0) = a_0_0_39_0;
  *(localWriteA_0_0_40_0 + 0) = a_0_0_40_0;
  *(localWriteA_0_0_41_0 + 0) = a_0_0_41_0;
  *(localWriteA_0_0_42_0 + 0) = a_0_0_42_0;
  *(localWriteA_0_0_43_0 + 0) = a_0_0_43_0;
  *(localWriteA_0_0_44_0 + 0) = a_0_0_44_0;
  *(localWriteA_0_0_45_0 + 0) = a_0_0_45_0;
  *(localWriteA_0_0_46_0 + 0) = a_0_0_46_0;
  *(localWriteA_0_0_47_0 + 0) = a_0_0_47_0;
  *(localWriteA_0_0_48_0 + 0) = a_0_0_48_0;
  *(localWriteA_0_0_49_0 + 0) = a_0_0_49_0;
  *(localWriteA_0_0_50_0 + 0) = a_0_0_50_0;
  *(localWriteA_0_0_51_0 + 0) = a_0_0_51_0;
  *(localWriteA_0_0_52_0 + 0) = a_0_0_52_0;
  *(localWriteA_0_0_53_0 + 0) = a_0_0_53_0;
  *(localWriteA_0_0_54_0 + 0) = a_0_0_54_0;
  *(localWriteA_0_0_55_0 + 0) = a_0_0_55_0;
  *(localWriteA_0_0_56_0 + 0) = a_0_0_56_0;
  *(localWriteA_0_0_57_0 + 0) = a_0_0_57_0;
  *(localWriteA_0_0_58_0 + 0) = a_0_0_58_0;
  *(localWriteA_0_0_59_0 + 0) = a_0_0_59_0;
  *(localWriteA_0_0_60_0 + 0) = a_0_0_60_0;
  *(localWriteA_0_0_61_0 + 0) = a_0_0_61_0;
  *(localWriteA_0_0_62_0 + 0) = a_0_0_62_0;
  *(localWriteA_0_0_63_0 + 0) = a_0_0_63_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_1x1

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_1x1
#undef MAC_1x1_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I] , rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I] , rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I] , rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I] , rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I] , rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I] , rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I] , rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I] , rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I] , rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I] , rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I] , rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I] , rA[7],rB[7],rC[7+7*TT0I]); \

#define MAC_8x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I] , rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I] , rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I] , rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I] , rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[4+TT1J],rC[4+4*TT0I] , rA[5+TT0I],rB[4+TT1J],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[4+TT1J],rC[6+4*TT0I] , rA[7+TT0I],rB[4+TT1J],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[5+TT1J],rC[4+5*TT0I] , rA[5+TT0I],rB[5+TT1J],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[5+TT1J],rC[6+5*TT0I] , rA[7+TT0I],rB[5+TT1J],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[6+TT1J],rC[4+6*TT0I] , rA[5+TT0I],rB[6+TT1J],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[6+TT1J],rC[6+6*TT0I] , rA[7+TT0I],rB[6+TT1J],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[7+TT1J],rC[4+7*TT0I] , rA[5+TT0I],rB[7+TT1J],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[7+TT1J],rC[6+7*TT0I] , rA[7+TT0I],rB[7+TT1J],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT128x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8


    /* iter 3 */

    MAC_8x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef MAC_8x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 3584
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x8


    /* iter 7 (last unrolled loop) */

    MAC_4x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 8

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 3072

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HBH_MT64x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;
  unsigned int globalReadOffsetBL_4_0 = globalReadOffsetBL + 0 + 4*LSPB;
  unsigned int globalReadOffsetBL_5_0 = globalReadOffsetBL + 0 + 5*LSPB;
  unsigned int globalReadOffsetBL_6_0 = globalReadOffsetBL + 0 + 6*LSPB;
  unsigned int globalReadOffsetBL_7_0 = globalReadOffsetBL + 0 + 7*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_7_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 4*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 5*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 6*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 7*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 15 */

    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE double
#define DEST_DATA_TYPE double
#define COMPUTE_DATA_TYPE double
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_DB_MT32x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW2_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  double *D,
  double const * __restrict__ C,
  double const * __restrict__ A,
  double const * __restrict__ B,
  double const alpha,
  double const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (double)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE double
#define DEST_DATA_TYPE double
#define COMPUTE_DATA_TYPE double
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_DB_MT32x32x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW2_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  double *D,
  double const * __restrict__ C,
  double const * __restrict__ A,
  double const * __restrict__ B,
  double const alpha,
  double const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (double)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 819

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HBH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1792
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HBH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HBH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 3 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 3 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1536

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HBH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HBH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HBH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1536

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HBH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 7 */

    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HBH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 3584
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HBH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x8


    /* iter 7 (last unrolled loop) */

    MAC_4x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 3072

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HBH_MT64x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_1_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_1_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_2_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_3_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_2_1;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_3_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_1);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_1 + 0) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_1 + 0) = b_0_1_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_1_2_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 1);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_1_3_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_1);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_2_1 + 0) = b_0_1_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_3_1 + 0) = b_0_1_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HBH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HBH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 3072

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HBH_MT64x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_1_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_1_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_2_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_3_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_2_1;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_3_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_1);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_1 + 0) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_1 + 0) = b_0_1_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 15 */

    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_1_2_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 1);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_1_3_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_1);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_2_1 + 0) = b_0_1_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_3_1 + 0) = b_0_1_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 7168
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HBH_MT64x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_1_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_1_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_2_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_3_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_2_1;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_3_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_1);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_1 + 0) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_1 + 0) = b_0_1_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_1 = (localWriteOffsetB_0_0_2_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_1 = (localWriteOffsetB_0_0_3_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_1);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
    localWriteB_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_1 + 0) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_1 + 0) = b_0_1_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_1 = (localWriteOffsetB_0_0_2_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_1 = (localWriteOffsetB_0_0_3_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_1);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
    localWriteB_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x8


    /* iter 15 (last unrolled loop) */

    MAC_4x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_1_2_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 1);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_1_3_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_1);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_2_1 + 0) = b_0_1_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_3_1 + 0) = b_0_1_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 819

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HBH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1792
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HBH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HBH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I] , rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I] , rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I] , rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I] , rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I] , rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I] , rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I] , rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I] , rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I] , rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I] , rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I] , rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I] , rA[7],rB[7],rC[7+7*TT0I]); \

#define MAC_8x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I] , rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I] , rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I] , rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I] , rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[4+TT1J],rC[4+4*TT0I] , rA[5+TT0I],rB[4+TT1J],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[4+TT1J],rC[6+4*TT0I] , rA[7+TT0I],rB[4+TT1J],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[5+TT1J],rC[4+5*TT0I] , rA[5+TT0I],rB[5+TT1J],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[5+TT1J],rC[6+5*TT0I] , rA[7+TT0I],rB[5+TT1J],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[6+TT1J],rC[4+6*TT0I] , rA[5+TT0I],rB[6+TT1J],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[6+TT1J],rC[6+6*TT0I] , rA[7+TT0I],rB[6+TT1J],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[7+TT1J],rC[4+7*TT0I] , rA[5+TT0I],rB[7+TT1J],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[7+TT1J],rC[6+7*TT0I] , rA[7+TT0I],rB[7+TT1J],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HBH_MT128x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8


    /* iter 3 */

    MAC_8x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef MAC_8x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 819

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HBH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 3 */

    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1792
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HBH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 3 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x8


    /* iter 3 (last unrolled loop) */

    MAC_4x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1536

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HBH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HBH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HBH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1536

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HBH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 7 */

    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HBH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I] , rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I] , rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I] , rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I] , rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I] , rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I] , rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I] , rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I] , rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I] , rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I] , rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I] , rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I] , rA[7],rB[7],rC[7+7*TT0I]); \

#define MAC_8x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I] , rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I] , rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I] , rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I] , rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[4+TT1J],rC[4+4*TT0I] , rA[5+TT0I],rB[4+TT1J],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[4+TT1J],rC[6+4*TT0I] , rA[7+TT0I],rB[4+TT1J],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[5+TT1J],rC[4+5*TT0I] , rA[5+TT0I],rB[5+TT1J],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[5+TT1J],rC[6+5*TT0I] , rA[7+TT0I],rB[5+TT1J],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[6+TT1J],rC[4+6*TT0I] , rA[5+TT0I],rB[6+TT1J],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[6+TT1J],rC[6+6*TT0I] , rA[7+TT0I],rB[6+TT1J],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[7+TT1J],rC[4+7*TT0I] , rA[5+TT0I],rB[7+TT1J],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[7+TT1J],rC[6+7*TT0I] , rA[7+TT0I],rB[7+TT1J],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HBH_MT128x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_8x8


    /* iter 7 (last unrolled loop) */

    MAC_8x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef MAC_8x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HBH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HBH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 768

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_4xi8BH_MT64x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 7 */

    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 8

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x8 micro-tile */
#define MAC_2x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \

#define MAC_2x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_4xi8BH_MT16x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_8_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_1_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_1_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_1_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_1_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_1_6_0;
  DATA_TYPE b_0_0_7_0;
  DATA_TYPE b_0_1_7_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;
  unsigned int globalReadOffsetBL_4_0 = globalReadOffsetBL + 0 + 4*LSPB;
  unsigned int globalReadOffsetBL_5_0 = globalReadOffsetBL + 0 + 5*LSPB;
  unsigned int globalReadOffsetBL_6_0 = globalReadOffsetBL + 0 + 6*LSPB;
  unsigned int globalReadOffsetBL_7_0 = globalReadOffsetBL + 0 + 7*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_7_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 4*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 5*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 6*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 7*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_1_4_0 = *(globalReadB_0_0_4_0 + 1);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_1_5_0 = *(globalReadB_0_0_5_0 + 1);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_1_6_0 = *(globalReadB_0_0_6_0 + 1);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);
    b_0_1_7_0 = *(globalReadB_0_0_7_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_4_0 + 1) = b_0_1_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_5_0 + 1) = b_0_1_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_6_0 + 1) = b_0_1_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
    *(localWriteB_0_0_7_0 + 1) = b_0_1_7_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x8


    /* iter 15 */

    MAC_2x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_1_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 1);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_1_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 1);
  b_0_0_4_0 = ( globalReadOffsetBL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_1_4_0 = ( globalReadOffsetBL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 1);
  b_0_0_5_0 = ( globalReadOffsetBL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_1_5_0 = ( globalReadOffsetBL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 1);
  b_0_0_6_0 = ( globalReadOffsetBL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_1_6_0 = ( globalReadOffsetBL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 1);
  b_0_0_7_0 = ( globalReadOffsetBL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);
  b_0_1_7_0 = ( globalReadOffsetBL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_4_0 + 1) = b_0_1_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_5_0 + 1) = b_0_1_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_6_0 + 1) = b_0_1_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
  *(localWriteB_0_0_7_0 + 1) = b_0_1_7_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x8

  }





  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 1) {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 2) {
        rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x8
#undef MAC_2x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_4xi8BH_MT128x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;
  unsigned int flattenedOffsetA_4_0 = globalReadOffsetA0I + 0 + 4*LSPA;
  flattenedOffsetA_4_0 = (flattenedOffsetA_4_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_4_0;
  unsigned int globalReadOffsetA0I_4_0 = flattenedOffsetA_4_0;
  unsigned int flattenedOffsetA_5_0 = globalReadOffsetA0I + 0 + 5*LSPA;
  flattenedOffsetA_5_0 = (flattenedOffsetA_5_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_5_0;
  unsigned int globalReadOffsetA0I_5_0 = flattenedOffsetA_5_0;
  unsigned int flattenedOffsetA_6_0 = globalReadOffsetA0I + 0 + 6*LSPA;
  flattenedOffsetA_6_0 = (flattenedOffsetA_6_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_6_0;
  unsigned int globalReadOffsetA0I_6_0 = flattenedOffsetA_6_0;
  unsigned int flattenedOffsetA_7_0 = globalReadOffsetA0I + 0 + 7*LSPA;
  flattenedOffsetA_7_0 = (flattenedOffsetA_7_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_7_0;
  unsigned int globalReadOffsetA0I_7_0 = flattenedOffsetA_7_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;
  globalReadOffsetA0I_4_0 = (  globalReadOffsetA0I_4_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_4_0;
  globalReadOffsetA0I_5_0 = (  globalReadOffsetA0I_5_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_5_0;
  globalReadOffsetA0I_6_0 = (  globalReadOffsetA0I_6_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_6_0;
  globalReadOffsetA0I_7_0 = (  globalReadOffsetA0I_7_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_7_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 4*LSPA);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 5*LSPA);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 6*LSPA);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 7*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 8

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x8 micro-tile */
#define MAC_2x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \

#define MAC_2x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_4xi8BH_MT32x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_8_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;
  unsigned int globalReadOffsetBL_4_0 = globalReadOffsetBL + 0 + 4*LSPB;
  unsigned int globalReadOffsetBL_5_0 = globalReadOffsetBL + 0 + 5*LSPB;
  unsigned int globalReadOffsetBL_6_0 = globalReadOffsetBL + 0 + 6*LSPB;
  unsigned int globalReadOffsetBL_7_0 = globalReadOffsetBL + 0 + 7*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_7_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 4*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 5*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 6*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 7*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x8


    /* iter 15 */

    MAC_2x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x8
#undef MAC_2x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 16
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_4xi8BH_MT64x16x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;
  unsigned int flattenedOffsetA_4_0 = globalReadOffsetA0I + 0 + 4*LSPA;
  flattenedOffsetA_4_0 = (flattenedOffsetA_4_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_4_0;
  unsigned int globalReadOffsetA0I_4_0 = flattenedOffsetA_4_0;
  unsigned int flattenedOffsetA_5_0 = globalReadOffsetA0I + 0 + 5*LSPA;
  flattenedOffsetA_5_0 = (flattenedOffsetA_5_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_5_0;
  unsigned int globalReadOffsetA0I_5_0 = flattenedOffsetA_5_0;
  unsigned int flattenedOffsetA_6_0 = globalReadOffsetA0I + 0 + 6*LSPA;
  flattenedOffsetA_6_0 = (flattenedOffsetA_6_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_6_0;
  unsigned int globalReadOffsetA0I_6_0 = flattenedOffsetA_6_0;
  unsigned int flattenedOffsetA_7_0 = globalReadOffsetA0I + 0 + 7*LSPA;
  flattenedOffsetA_7_0 = (flattenedOffsetA_7_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_7_0;
  unsigned int globalReadOffsetA0I_7_0 = flattenedOffsetA_7_0;
  unsigned int flattenedOffsetA_8_0 = globalReadOffsetA0I + 0 + 8*LSPA;
  flattenedOffsetA_8_0 = (flattenedOffsetA_8_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_8_0;
  unsigned int globalReadOffsetA0I_8_0 = flattenedOffsetA_8_0;
  unsigned int flattenedOffsetA_9_0 = globalReadOffsetA0I + 0 + 9*LSPA;
  flattenedOffsetA_9_0 = (flattenedOffsetA_9_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_9_0;
  unsigned int globalReadOffsetA0I_9_0 = flattenedOffsetA_9_0;
  unsigned int flattenedOffsetA_10_0 = globalReadOffsetA0I + 0 + 10*LSPA;
  flattenedOffsetA_10_0 = (flattenedOffsetA_10_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_10_0;
  unsigned int globalReadOffsetA0I_10_0 = flattenedOffsetA_10_0;
  unsigned int flattenedOffsetA_11_0 = globalReadOffsetA0I + 0 + 11*LSPA;
  flattenedOffsetA_11_0 = (flattenedOffsetA_11_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_11_0;
  unsigned int globalReadOffsetA0I_11_0 = flattenedOffsetA_11_0;
  unsigned int flattenedOffsetA_12_0 = globalReadOffsetA0I + 0 + 12*LSPA;
  flattenedOffsetA_12_0 = (flattenedOffsetA_12_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_12_0;
  unsigned int globalReadOffsetA0I_12_0 = flattenedOffsetA_12_0;
  unsigned int flattenedOffsetA_13_0 = globalReadOffsetA0I + 0 + 13*LSPA;
  flattenedOffsetA_13_0 = (flattenedOffsetA_13_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_13_0;
  unsigned int globalReadOffsetA0I_13_0 = flattenedOffsetA_13_0;
  unsigned int flattenedOffsetA_14_0 = globalReadOffsetA0I + 0 + 14*LSPA;
  flattenedOffsetA_14_0 = (flattenedOffsetA_14_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_14_0;
  unsigned int globalReadOffsetA0I_14_0 = flattenedOffsetA_14_0;
  unsigned int flattenedOffsetA_15_0 = globalReadOffsetA0I + 0 + 15*LSPA;
  flattenedOffsetA_15_0 = (flattenedOffsetA_15_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_15_0;
  unsigned int globalReadOffsetA0I_15_0 = flattenedOffsetA_15_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;
  globalReadOffsetA0I_4_0 = (  globalReadOffsetA0I_4_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_4_0;
  globalReadOffsetA0I_5_0 = (  globalReadOffsetA0I_5_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_5_0;
  globalReadOffsetA0I_6_0 = (  globalReadOffsetA0I_6_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_6_0;
  globalReadOffsetA0I_7_0 = (  globalReadOffsetA0I_7_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_7_0;
  globalReadOffsetA0I_8_0 = (  globalReadOffsetA0I_8_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_8_0;
  globalReadOffsetA0I_9_0 = (  globalReadOffsetA0I_9_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_9_0;
  globalReadOffsetA0I_10_0 = (  globalReadOffsetA0I_10_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_10_0;
  globalReadOffsetA0I_11_0 = (  globalReadOffsetA0I_11_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_11_0;
  globalReadOffsetA0I_12_0 = (  globalReadOffsetA0I_12_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_12_0;
  globalReadOffsetA0I_13_0 = (  globalReadOffsetA0I_13_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_13_0;
  globalReadOffsetA0I_14_0 = (  globalReadOffsetA0I_14_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_14_0;
  globalReadOffsetA0I_15_0 = (  globalReadOffsetA0I_15_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_15_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_15_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 4*LSPA);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 5*LSPA);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 6*LSPA);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 7*LSPA);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 8*LSPA);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 9*LSPA);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 10*LSPA);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 11*LSPA);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 12*LSPA);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 13*LSPA);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 14*LSPA);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 15*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_0_0_15_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_8_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_9_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_10_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_11_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_12_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_13_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_14_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_15_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_8_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_9_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_10_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_11_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_12_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_13_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_14_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_15_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_0_8_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_0_0_9_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_0_0_10_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_0_0_11_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_0_0_12_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_0_0_13_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_0_0_14_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_0_0_15_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 16

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x8 micro-tile */
#define MAC_2x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \

#define MAC_2x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_4xi8BH_MT16x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_8_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;
  DATA_TYPE b_0_0_8_0;
  DATA_TYPE b_0_0_9_0;
  DATA_TYPE b_0_0_10_0;
  DATA_TYPE b_0_0_11_0;
  DATA_TYPE b_0_0_12_0;
  DATA_TYPE b_0_0_13_0;
  DATA_TYPE b_0_0_14_0;
  DATA_TYPE b_0_0_15_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;
  unsigned int globalReadOffsetBL_4_0 = globalReadOffsetBL + 0 + 4*LSPB;
  unsigned int globalReadOffsetBL_5_0 = globalReadOffsetBL + 0 + 5*LSPB;
  unsigned int globalReadOffsetBL_6_0 = globalReadOffsetBL + 0 + 6*LSPB;
  unsigned int globalReadOffsetBL_7_0 = globalReadOffsetBL + 0 + 7*LSPB;
  unsigned int globalReadOffsetBL_8_0 = globalReadOffsetBL + 0 + 8*LSPB;
  unsigned int globalReadOffsetBL_9_0 = globalReadOffsetBL + 0 + 9*LSPB;
  unsigned int globalReadOffsetBL_10_0 = globalReadOffsetBL + 0 + 10*LSPB;
  unsigned int globalReadOffsetBL_11_0 = globalReadOffsetBL + 0 + 11*LSPB;
  unsigned int globalReadOffsetBL_12_0 = globalReadOffsetBL + 0 + 12*LSPB;
  unsigned int globalReadOffsetBL_13_0 = globalReadOffsetBL + 0 + 13*LSPB;
  unsigned int globalReadOffsetBL_14_0 = globalReadOffsetBL + 0 + 14*LSPB;
  unsigned int globalReadOffsetBL_15_0 = globalReadOffsetBL + 0 + 15*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_7_0), wgK );
  int64_t globalReadOffsetB_0_0_8_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_8_0), wgK );
  int64_t globalReadOffsetB_0_0_9_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_9_0), wgK );
  int64_t globalReadOffsetB_0_0_10_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_10_0), wgK );
  int64_t globalReadOffsetB_0_0_11_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_11_0), wgK );
  int64_t globalReadOffsetB_0_0_12_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_12_0), wgK );
  int64_t globalReadOffsetB_0_0_13_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_13_0), wgK );
  int64_t globalReadOffsetB_0_0_14_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_14_0), wgK );
  int64_t globalReadOffsetB_0_0_15_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_15_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;
  DATA_TYPE const *globalReadB_0_0_8_0 = B + globalReadOffsetB_0_0_8_0;
  DATA_TYPE const *globalReadB_0_0_9_0 = B + globalReadOffsetB_0_0_9_0;
  DATA_TYPE const *globalReadB_0_0_10_0 = B + globalReadOffsetB_0_0_10_0;
  DATA_TYPE const *globalReadB_0_0_11_0 = B + globalReadOffsetB_0_0_11_0;
  DATA_TYPE const *globalReadB_0_0_12_0 = B + globalReadOffsetB_0_0_12_0;
  DATA_TYPE const *globalReadB_0_0_13_0 = B + globalReadOffsetB_0_0_13_0;
  DATA_TYPE const *globalReadB_0_0_14_0 = B + globalReadOffsetB_0_0_14_0;
  DATA_TYPE const *globalReadB_0_0_15_0 = B + globalReadOffsetB_0_0_15_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 4*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 5*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 6*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 7*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_8_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 8*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_9_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 9*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_10_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 10*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_11_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 11*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_12_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 12*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_13_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 13*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_14_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 14*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_15_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 15*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;
  DATA_TYPE *localWriteB_0_0_8_0;
  DATA_TYPE *localWriteB_0_0_9_0;
  DATA_TYPE *localWriteB_0_0_10_0;
  DATA_TYPE *localWriteB_0_0_11_0;
  DATA_TYPE *localWriteB_0_0_12_0;
  DATA_TYPE *localWriteB_0_0_13_0;
  DATA_TYPE *localWriteB_0_0_14_0;
  DATA_TYPE *localWriteB_0_0_15_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);
  localWriteB_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_8_0);
  localWriteB_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_9_0);
  localWriteB_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_10_0);
  localWriteB_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_11_0);
  localWriteB_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_12_0);
  localWriteB_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_13_0);
  localWriteB_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_14_0);
  localWriteB_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_15_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_8_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_9_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_10_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_11_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_12_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_13_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_14_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_15_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);
    b_0_0_8_0 = *(globalReadB_0_0_8_0 + 0);
    b_0_0_9_0 = *(globalReadB_0_0_9_0 + 0);
    b_0_0_10_0 = *(globalReadB_0_0_10_0 + 0);
    b_0_0_11_0 = *(globalReadB_0_0_11_0 + 0);
    b_0_0_12_0 = *(globalReadB_0_0_12_0 + 0);
    b_0_0_13_0 = *(globalReadB_0_0_13_0 + 0);
    b_0_0_14_0 = *(globalReadB_0_0_14_0 + 0);
    b_0_0_15_0 = *(globalReadB_0_0_15_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_8_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_8_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_9_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_9_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_10_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_10_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_11_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_11_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_12_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_12_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_13_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_13_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_14_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_14_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_15_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_15_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
    *(localWriteB_0_0_8_0 + 0) = b_0_0_8_0;
    *(localWriteB_0_0_9_0 + 0) = b_0_0_9_0;
    *(localWriteB_0_0_10_0 + 0) = b_0_0_10_0;
    *(localWriteB_0_0_11_0 + 0) = b_0_0_11_0;
    *(localWriteB_0_0_12_0 + 0) = b_0_0_12_0;
    *(localWriteB_0_0_13_0 + 0) = b_0_0_13_0;
    *(localWriteB_0_0_14_0 + 0) = b_0_0_14_0;
    *(localWriteB_0_0_15_0 + 0) = b_0_0_15_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x8


    /* iter 15 */

    MAC_2x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);
  b_0_0_8_0 = ( globalReadOffsetBL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_8_0 + 0);
  b_0_0_9_0 = ( globalReadOffsetBL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_9_0 + 0);
  b_0_0_10_0 = ( globalReadOffsetBL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_10_0 + 0);
  b_0_0_11_0 = ( globalReadOffsetBL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_11_0 + 0);
  b_0_0_12_0 = ( globalReadOffsetBL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_12_0 + 0);
  b_0_0_13_0 = ( globalReadOffsetBL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_13_0 + 0);
  b_0_0_14_0 = ( globalReadOffsetBL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_14_0 + 0);
  b_0_0_15_0 = ( globalReadOffsetBL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_15_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);
  localWriteB_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_8_0);
  localWriteB_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_9_0);
  localWriteB_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_10_0);
  localWriteB_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_11_0);
  localWriteB_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_12_0);
  localWriteB_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_13_0);
  localWriteB_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_14_0);
  localWriteB_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_15_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
  *(localWriteB_0_0_8_0 + 0) = b_0_0_8_0;
  *(localWriteB_0_0_9_0 + 0) = b_0_0_9_0;
  *(localWriteB_0_0_10_0 + 0) = b_0_0_10_0;
  *(localWriteB_0_0_11_0 + 0) = b_0_0_11_0;
  *(localWriteB_0_0_12_0 + 0) = b_0_0_12_0;
  *(localWriteB_0_0_13_0 + 0) = b_0_0_13_0;
  *(localWriteB_0_0_14_0 + 0) = b_0_0_14_0;
  *(localWriteB_0_0_15_0 + 0) = b_0_0_15_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x8
#undef MAC_2x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 768

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_4xi8BH_MT64x32x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 7 */

    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 8

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x8 micro-tile */
#define MAC_2x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \

#define MAC_2x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_4xi8BH_MT16x64x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_8_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_1_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_1_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_1_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_1_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_1_6_0;
  DATA_TYPE b_0_0_7_0;
  DATA_TYPE b_0_1_7_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;
  unsigned int globalReadOffsetBL_4_0 = globalReadOffsetBL + 0 + 4*LSPB;
  unsigned int globalReadOffsetBL_5_0 = globalReadOffsetBL + 0 + 5*LSPB;
  unsigned int globalReadOffsetBL_6_0 = globalReadOffsetBL + 0 + 6*LSPB;
  unsigned int globalReadOffsetBL_7_0 = globalReadOffsetBL + 0 + 7*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_7_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 4*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 5*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 6*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 7*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_1_4_0 = *(globalReadB_0_0_4_0 + 1);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_1_5_0 = *(globalReadB_0_0_5_0 + 1);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_1_6_0 = *(globalReadB_0_0_6_0 + 1);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);
    b_0_1_7_0 = *(globalReadB_0_0_7_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_4_0 + 1) = b_0_1_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_5_0 + 1) = b_0_1_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_6_0 + 1) = b_0_1_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
    *(localWriteB_0_0_7_0 + 1) = b_0_1_7_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x8


    /* iter 15 */

    MAC_2x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_1_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 1);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_1_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 1);
  b_0_0_4_0 = ( globalReadOffsetBL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_1_4_0 = ( globalReadOffsetBL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 1);
  b_0_0_5_0 = ( globalReadOffsetBL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_1_5_0 = ( globalReadOffsetBL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 1);
  b_0_0_6_0 = ( globalReadOffsetBL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_1_6_0 = ( globalReadOffsetBL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 1);
  b_0_0_7_0 = ( globalReadOffsetBL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);
  b_0_1_7_0 = ( globalReadOffsetBL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_4_0 + 1) = b_0_1_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_5_0 + 1) = b_0_1_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_6_0 + 1) = b_0_1_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
  *(localWriteB_0_0_7_0 + 1) = b_0_1_7_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x8

  }





  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 1) {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 2) {
        rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x8
#undef MAC_2x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_4xi8BH_MT128x32x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;
  unsigned int flattenedOffsetA_4_0 = globalReadOffsetA0I + 0 + 4*LSPA;
  flattenedOffsetA_4_0 = (flattenedOffsetA_4_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_4_0;
  unsigned int globalReadOffsetA0I_4_0 = flattenedOffsetA_4_0;
  unsigned int flattenedOffsetA_5_0 = globalReadOffsetA0I + 0 + 5*LSPA;
  flattenedOffsetA_5_0 = (flattenedOffsetA_5_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_5_0;
  unsigned int globalReadOffsetA0I_5_0 = flattenedOffsetA_5_0;
  unsigned int flattenedOffsetA_6_0 = globalReadOffsetA0I + 0 + 6*LSPA;
  flattenedOffsetA_6_0 = (flattenedOffsetA_6_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_6_0;
  unsigned int globalReadOffsetA0I_6_0 = flattenedOffsetA_6_0;
  unsigned int flattenedOffsetA_7_0 = globalReadOffsetA0I + 0 + 7*LSPA;
  flattenedOffsetA_7_0 = (flattenedOffsetA_7_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_7_0;
  unsigned int globalReadOffsetA0I_7_0 = flattenedOffsetA_7_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;
  globalReadOffsetA0I_4_0 = (  globalReadOffsetA0I_4_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_4_0;
  globalReadOffsetA0I_5_0 = (  globalReadOffsetA0I_5_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_5_0;
  globalReadOffsetA0I_6_0 = (  globalReadOffsetA0I_6_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_6_0;
  globalReadOffsetA0I_7_0 = (  globalReadOffsetA0I_7_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_7_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 4*LSPA);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 5*LSPA);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 6*LSPA);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 7*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 8

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x8 micro-tile */
#define MAC_2x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \

#define MAC_2x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_4xi8BH_MT32x128x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_8_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;
  unsigned int globalReadOffsetBL_4_0 = globalReadOffsetBL + 0 + 4*LSPB;
  unsigned int globalReadOffsetBL_5_0 = globalReadOffsetBL + 0 + 5*LSPB;
  unsigned int globalReadOffsetBL_6_0 = globalReadOffsetBL + 0 + 6*LSPB;
  unsigned int globalReadOffsetBL_7_0 = globalReadOffsetBL + 0 + 7*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_7_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 4*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 5*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 6*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 7*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x8


    /* iter 15 */

    MAC_2x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x8
#undef MAC_2x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 16
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_4xi8BH_MT64x16x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;
  unsigned int flattenedOffsetA_4_0 = globalReadOffsetA0I + 0 + 4*LSPA;
  flattenedOffsetA_4_0 = (flattenedOffsetA_4_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_4_0;
  unsigned int globalReadOffsetA0I_4_0 = flattenedOffsetA_4_0;
  unsigned int flattenedOffsetA_5_0 = globalReadOffsetA0I + 0 + 5*LSPA;
  flattenedOffsetA_5_0 = (flattenedOffsetA_5_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_5_0;
  unsigned int globalReadOffsetA0I_5_0 = flattenedOffsetA_5_0;
  unsigned int flattenedOffsetA_6_0 = globalReadOffsetA0I + 0 + 6*LSPA;
  flattenedOffsetA_6_0 = (flattenedOffsetA_6_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_6_0;
  unsigned int globalReadOffsetA0I_6_0 = flattenedOffsetA_6_0;
  unsigned int flattenedOffsetA_7_0 = globalReadOffsetA0I + 0 + 7*LSPA;
  flattenedOffsetA_7_0 = (flattenedOffsetA_7_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_7_0;
  unsigned int globalReadOffsetA0I_7_0 = flattenedOffsetA_7_0;
  unsigned int flattenedOffsetA_8_0 = globalReadOffsetA0I + 0 + 8*LSPA;
  flattenedOffsetA_8_0 = (flattenedOffsetA_8_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_8_0;
  unsigned int globalReadOffsetA0I_8_0 = flattenedOffsetA_8_0;
  unsigned int flattenedOffsetA_9_0 = globalReadOffsetA0I + 0 + 9*LSPA;
  flattenedOffsetA_9_0 = (flattenedOffsetA_9_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_9_0;
  unsigned int globalReadOffsetA0I_9_0 = flattenedOffsetA_9_0;
  unsigned int flattenedOffsetA_10_0 = globalReadOffsetA0I + 0 + 10*LSPA;
  flattenedOffsetA_10_0 = (flattenedOffsetA_10_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_10_0;
  unsigned int globalReadOffsetA0I_10_0 = flattenedOffsetA_10_0;
  unsigned int flattenedOffsetA_11_0 = globalReadOffsetA0I + 0 + 11*LSPA;
  flattenedOffsetA_11_0 = (flattenedOffsetA_11_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_11_0;
  unsigned int globalReadOffsetA0I_11_0 = flattenedOffsetA_11_0;
  unsigned int flattenedOffsetA_12_0 = globalReadOffsetA0I + 0 + 12*LSPA;
  flattenedOffsetA_12_0 = (flattenedOffsetA_12_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_12_0;
  unsigned int globalReadOffsetA0I_12_0 = flattenedOffsetA_12_0;
  unsigned int flattenedOffsetA_13_0 = globalReadOffsetA0I + 0 + 13*LSPA;
  flattenedOffsetA_13_0 = (flattenedOffsetA_13_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_13_0;
  unsigned int globalReadOffsetA0I_13_0 = flattenedOffsetA_13_0;
  unsigned int flattenedOffsetA_14_0 = globalReadOffsetA0I + 0 + 14*LSPA;
  flattenedOffsetA_14_0 = (flattenedOffsetA_14_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_14_0;
  unsigned int globalReadOffsetA0I_14_0 = flattenedOffsetA_14_0;
  unsigned int flattenedOffsetA_15_0 = globalReadOffsetA0I + 0 + 15*LSPA;
  flattenedOffsetA_15_0 = (flattenedOffsetA_15_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_15_0;
  unsigned int globalReadOffsetA0I_15_0 = flattenedOffsetA_15_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;
  globalReadOffsetA0I_4_0 = (  globalReadOffsetA0I_4_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_4_0;
  globalReadOffsetA0I_5_0 = (  globalReadOffsetA0I_5_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_5_0;
  globalReadOffsetA0I_6_0 = (  globalReadOffsetA0I_6_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_6_0;
  globalReadOffsetA0I_7_0 = (  globalReadOffsetA0I_7_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_7_0;
  globalReadOffsetA0I_8_0 = (  globalReadOffsetA0I_8_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_8_0;
  globalReadOffsetA0I_9_0 = (  globalReadOffsetA0I_9_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_9_0;
  globalReadOffsetA0I_10_0 = (  globalReadOffsetA0I_10_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_10_0;
  globalReadOffsetA0I_11_0 = (  globalReadOffsetA0I_11_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_11_0;
  globalReadOffsetA0I_12_0 = (  globalReadOffsetA0I_12_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_12_0;
  globalReadOffsetA0I_13_0 = (  globalReadOffsetA0I_13_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_13_0;
  globalReadOffsetA0I_14_0 = (  globalReadOffsetA0I_14_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_14_0;
  globalReadOffsetA0I_15_0 = (  globalReadOffsetA0I_15_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_15_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_15_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 4*LSPA);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 5*LSPA);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 6*LSPA);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 7*LSPA);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 8*LSPA);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 9*LSPA);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 10*LSPA);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 11*LSPA);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 12*LSPA);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 13*LSPA);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 14*LSPA);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 15*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_0_0_15_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_8_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_9_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_10_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_11_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_12_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_13_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_14_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_15_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_8_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_9_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_10_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_11_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_12_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_13_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_14_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_15_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_0_8_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_0_0_9_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_0_0_10_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_0_0_11_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_0_0_12_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_0_0_13_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_0_0_14_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_0_0_15_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 16

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x8 micro-tile */
#define MAC_2x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \

#define MAC_2x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_4xi8BH_MT16x64x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_8_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;
  DATA_TYPE b_0_0_8_0;
  DATA_TYPE b_0_0_9_0;
  DATA_TYPE b_0_0_10_0;
  DATA_TYPE b_0_0_11_0;
  DATA_TYPE b_0_0_12_0;
  DATA_TYPE b_0_0_13_0;
  DATA_TYPE b_0_0_14_0;
  DATA_TYPE b_0_0_15_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;
  unsigned int globalReadOffsetBL_4_0 = globalReadOffsetBL + 0 + 4*LSPB;
  unsigned int globalReadOffsetBL_5_0 = globalReadOffsetBL + 0 + 5*LSPB;
  unsigned int globalReadOffsetBL_6_0 = globalReadOffsetBL + 0 + 6*LSPB;
  unsigned int globalReadOffsetBL_7_0 = globalReadOffsetBL + 0 + 7*LSPB;
  unsigned int globalReadOffsetBL_8_0 = globalReadOffsetBL + 0 + 8*LSPB;
  unsigned int globalReadOffsetBL_9_0 = globalReadOffsetBL + 0 + 9*LSPB;
  unsigned int globalReadOffsetBL_10_0 = globalReadOffsetBL + 0 + 10*LSPB;
  unsigned int globalReadOffsetBL_11_0 = globalReadOffsetBL + 0 + 11*LSPB;
  unsigned int globalReadOffsetBL_12_0 = globalReadOffsetBL + 0 + 12*LSPB;
  unsigned int globalReadOffsetBL_13_0 = globalReadOffsetBL + 0 + 13*LSPB;
  unsigned int globalReadOffsetBL_14_0 = globalReadOffsetBL + 0 + 14*LSPB;
  unsigned int globalReadOffsetBL_15_0 = globalReadOffsetBL + 0 + 15*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_7_0), wgK );
  int64_t globalReadOffsetB_0_0_8_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_8_0), wgK );
  int64_t globalReadOffsetB_0_0_9_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_9_0), wgK );
  int64_t globalReadOffsetB_0_0_10_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_10_0), wgK );
  int64_t globalReadOffsetB_0_0_11_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_11_0), wgK );
  int64_t globalReadOffsetB_0_0_12_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_12_0), wgK );
  int64_t globalReadOffsetB_0_0_13_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_13_0), wgK );
  int64_t globalReadOffsetB_0_0_14_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_14_0), wgK );
  int64_t globalReadOffsetB_0_0_15_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_15_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;
  DATA_TYPE const *globalReadB_0_0_8_0 = B + globalReadOffsetB_0_0_8_0;
  DATA_TYPE const *globalReadB_0_0_9_0 = B + globalReadOffsetB_0_0_9_0;
  DATA_TYPE const *globalReadB_0_0_10_0 = B + globalReadOffsetB_0_0_10_0;
  DATA_TYPE const *globalReadB_0_0_11_0 = B + globalReadOffsetB_0_0_11_0;
  DATA_TYPE const *globalReadB_0_0_12_0 = B + globalReadOffsetB_0_0_12_0;
  DATA_TYPE const *globalReadB_0_0_13_0 = B + globalReadOffsetB_0_0_13_0;
  DATA_TYPE const *globalReadB_0_0_14_0 = B + globalReadOffsetB_0_0_14_0;
  DATA_TYPE const *globalReadB_0_0_15_0 = B + globalReadOffsetB_0_0_15_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 4*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 5*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 6*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 7*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_8_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 8*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_9_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 9*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_10_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 10*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_11_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 11*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_12_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 12*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_13_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 13*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_14_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 14*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_15_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 15*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;
  DATA_TYPE *localWriteB_0_0_8_0;
  DATA_TYPE *localWriteB_0_0_9_0;
  DATA_TYPE *localWriteB_0_0_10_0;
  DATA_TYPE *localWriteB_0_0_11_0;
  DATA_TYPE *localWriteB_0_0_12_0;
  DATA_TYPE *localWriteB_0_0_13_0;
  DATA_TYPE *localWriteB_0_0_14_0;
  DATA_TYPE *localWriteB_0_0_15_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);
  localWriteB_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_8_0);
  localWriteB_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_9_0);
  localWriteB_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_10_0);
  localWriteB_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_11_0);
  localWriteB_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_12_0);
  localWriteB_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_13_0);
  localWriteB_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_14_0);
  localWriteB_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_15_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_8_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_9_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_10_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_11_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_12_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_13_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_14_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_15_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);
    b_0_0_8_0 = *(globalReadB_0_0_8_0 + 0);
    b_0_0_9_0 = *(globalReadB_0_0_9_0 + 0);
    b_0_0_10_0 = *(globalReadB_0_0_10_0 + 0);
    b_0_0_11_0 = *(globalReadB_0_0_11_0 + 0);
    b_0_0_12_0 = *(globalReadB_0_0_12_0 + 0);
    b_0_0_13_0 = *(globalReadB_0_0_13_0 + 0);
    b_0_0_14_0 = *(globalReadB_0_0_14_0 + 0);
    b_0_0_15_0 = *(globalReadB_0_0_15_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_8_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_8_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_9_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_9_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_10_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_10_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_11_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_11_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_12_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_12_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_13_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_13_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_14_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_14_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_15_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_15_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
    *(localWriteB_0_0_8_0 + 0) = b_0_0_8_0;
    *(localWriteB_0_0_9_0 + 0) = b_0_0_9_0;
    *(localWriteB_0_0_10_0 + 0) = b_0_0_10_0;
    *(localWriteB_0_0_11_0 + 0) = b_0_0_11_0;
    *(localWriteB_0_0_12_0 + 0) = b_0_0_12_0;
    *(localWriteB_0_0_13_0 + 0) = b_0_0_13_0;
    *(localWriteB_0_0_14_0 + 0) = b_0_0_14_0;
    *(localWriteB_0_0_15_0 + 0) = b_0_0_15_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x8


    /* iter 15 */

    MAC_2x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);
  b_0_0_8_0 = ( globalReadOffsetBL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_8_0 + 0);
  b_0_0_9_0 = ( globalReadOffsetBL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_9_0 + 0);
  b_0_0_10_0 = ( globalReadOffsetBL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_10_0 + 0);
  b_0_0_11_0 = ( globalReadOffsetBL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_11_0 + 0);
  b_0_0_12_0 = ( globalReadOffsetBL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_12_0 + 0);
  b_0_0_13_0 = ( globalReadOffsetBL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_13_0 + 0);
  b_0_0_14_0 = ( globalReadOffsetBL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_14_0 + 0);
  b_0_0_15_0 = ( globalReadOffsetBL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_15_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);
  localWriteB_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_8_0);
  localWriteB_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_9_0);
  localWriteB_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_10_0);
  localWriteB_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_11_0);
  localWriteB_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_12_0);
  localWriteB_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_13_0);
  localWriteB_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_14_0);
  localWriteB_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_15_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
  *(localWriteB_0_0_8_0 + 0) = b_0_0_8_0;
  *(localWriteB_0_0_9_0 + 0) = b_0_0_9_0;
  *(localWriteB_0_0_10_0 + 0) = b_0_0_10_0;
  *(localWriteB_0_0_11_0 + 0) = b_0_0_11_0;
  *(localWriteB_0_0_12_0 + 0) = b_0_0_12_0;
  *(localWriteB_0_0_13_0 + 0) = b_0_0_13_0;
  *(localWriteB_0_0_14_0 + 0) = b_0_0_14_0;
  *(localWriteB_0_0_15_0 + 0) = b_0_0_15_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x8
#undef MAC_2x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 768

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 768

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 3 */

    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 3 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I] , rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I] , rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I] , rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I] , rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I] , rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I] , rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I] , rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I] , rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I] , rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I] , rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I] , rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I] , rA[7],rB[7],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT128x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_8x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else if (s0I == 1) {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else if (s0I == 2) {
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else {
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I] , rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I] , rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I] , rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I] , rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I] , rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I] , rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I] , rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I] , rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I] , rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I] , rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I] , rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I] , rA[7],rB[7],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT128x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 768

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1792
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I] , rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I] , rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I] , rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I] , rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I] , rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I] , rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I] , rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I] , rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I] , rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I] , rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I] , rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I] , rA[7],rB[7],rC[7+7*TT0I]); \

#define MAC_8x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I] , rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I] , rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I] , rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I] , rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[4+TT1J],rC[4+4*TT0I] , rA[5+TT0I],rB[4+TT1J],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[4+TT1J],rC[6+4*TT0I] , rA[7+TT0I],rB[4+TT1J],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[5+TT1J],rC[4+5*TT0I] , rA[5+TT0I],rB[5+TT1J],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[5+TT1J],rC[6+5*TT0I] , rA[7+TT0I],rB[5+TT1J],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[6+TT1J],rC[4+6*TT0I] , rA[5+TT0I],rB[6+TT1J],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[6+TT1J],rC[6+6*TT0I] , rA[7+TT0I],rB[6+TT1J],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[7+TT1J],rC[4+7*TT0I] , rA[5+TT0I],rB[7+TT1J],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[7+TT1J],rC[6+7*TT0I] , rA[7+TT0I],rB[7+TT1J],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT128x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8


    /* iter 3 */

    MAC_8x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef MAC_8x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 768

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 3 */

    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 3 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I] , rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I] , rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I] , rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I] , rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I] , rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I] , rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I] , rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I] , rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I] , rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I] , rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I] , rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I] , rA[7],rB[7],rC[7+7*TT0I]); \

#define MAC_8x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I] , rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I] , rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I] , rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I] , rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[4+TT1J],rC[4+4*TT0I] , rA[5+TT0I],rB[4+TT1J],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[4+TT1J],rC[6+4*TT0I] , rA[7+TT0I],rB[4+TT1J],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[5+TT1J],rC[4+5*TT0I] , rA[5+TT0I],rB[5+TT1J],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[5+TT1J],rC[6+5*TT0I] , rA[7+TT0I],rB[5+TT1J],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[6+TT1J],rC[4+6*TT0I] , rA[5+TT0I],rB[6+TT1J],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[6+TT1J],rC[6+6*TT0I] , rA[7+TT0I],rB[6+TT1J],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[7+TT1J],rC[4+7*TT0I] , rA[5+TT0I],rB[7+TT1J],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[7+TT1J],rC[6+7*TT0I] , rA[7+TT0I],rB[7+TT1J],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT128x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8


    /* iter 3 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_8x8


    /* iter 3 (last unrolled loop) */

    MAC_8x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef MAC_8x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 3 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 3 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1536

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1536

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 7 */

    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 7 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE double
#define DEST_DATA_TYPE double
#define COMPUTE_DATA_TYPE double
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_DB_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW2_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  double *D,
  double const * __restrict__ C,
  double const * __restrict__ A,
  double const * __restrict__ B,
  double const alpha,
  double const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (double)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 7 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE double
#define DEST_DATA_TYPE double
#define COMPUTE_DATA_TYPE double
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_DB_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW2_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  double *D,
  double const * __restrict__ C,
  double const * __restrict__ A,
  double const * __restrict__ B,
  double const alpha,
  double const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (double)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 3 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 3 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 768

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 1) {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 2) {
        rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 3072

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HSS_BH_MT64x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_1_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_1_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 15 */

    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_1_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 1);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_1_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 1) {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 2) {
        rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 16
#define SG1J 4
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 832
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT32x8x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_4_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 128
#define LDS_NUM_ELEMENTS 512
#define LDS_OFFSET_BLK 256

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT16x16x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 128
#define LDS_NUM_ELEMENTS 512
#define LDS_OFFSET_BLK 256

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT16x16x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 896
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT32x16x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x2


    /* iter 7 (last unrolled loop) */

    MAC_4x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 896
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT32x16x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_8_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 32
#define SG1J 4
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 3200
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT64x8x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG32_4_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 15 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1792
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT32x16x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x2


    /* iter 15 (last unrolled loop) */

    MAC_4x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT32x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_2_1_0;
  DATA_TYPE a_0_3_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_2_1_0;
  DATA_TYPE b_0_3_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
    *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
    *(localWriteB_0_0_1_0 + 2) = b_0_2_1_0;
    *(localWriteB_0_0_1_0 + 3) = b_0_3_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
    *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
    *(localWriteB_0_0_1_0 + 2) = b_0_2_1_0;
    *(localWriteB_0_0_1_0 + 3) = b_0_3_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_2_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 2);
  a_0_3_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_2_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 2);
  b_0_3_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
  *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
  *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
  *(localWriteB_0_0_1_0 + 2) = b_0_2_1_0;
  *(localWriteB_0_0_1_0 + 3) = b_0_3_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 2) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 3) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 2) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 3) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 3328
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT64x16x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x2


    /* iter 15 (last unrolled loop) */

    MAC_4x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 3584
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT64x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_2_1_0;
  DATA_TYPE a_0_3_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
    *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
    *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_2_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 2);
  a_0_3_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
  *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
  *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 2) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 3) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 2) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 3) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 32
#define SG1J 4
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 3200
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT64x8x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG32_4_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 15 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 16
#define SG1J 4
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1664
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT32x8x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_4_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 15 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT16x16x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 15 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 24
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 3
#define NLPB 3

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 384
#define LDS_NUM_ELEMENTS 1792
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT16x16x24_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_1_2_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 384-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 15 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 16 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 17 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 18 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 19 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 20 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 21 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 22 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 23 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 16 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 17 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 18 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 19 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 20 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 21 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 22 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 23 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_1_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 7168
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT64x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_2_1_0;
  DATA_TYPE b_0_3_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
    *(localWriteB_0_0_1_0 + 2) = b_0_2_1_0;
    *(localWriteB_0_0_1_0 + 3) = b_0_3_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
    *(localWriteB_0_0_1_0 + 2) = b_0_2_1_0;
    *(localWriteB_0_0_1_0 + 3) = b_0_3_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x8


    /* iter 15 (last unrolled loop) */

    MAC_4x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_2_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 2);
  b_0_3_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
  *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
  *(localWriteB_0_0_1_0 + 2) = b_0_2_1_0;
  *(localWriteB_0_0_1_0 + 3) = b_0_3_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
    if (r0I == 2) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
    if (r0I == 3) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 2) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 3) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 7168
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x4 micro-tile */
#define MAC_8x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7],rB[3],rC[7+3*TT0I]); \

#define MAC_8x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT128x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_2_1_0;
  DATA_TYPE a_0_3_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
    *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
    *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_8x4


    /* iter 15 (last unrolled loop) */

    MAC_8x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_2_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 2);
  a_0_3_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
  *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
  *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 2) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 3) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 2) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 3) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x4
#undef MAC_8x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 7168
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x4 micro-tile */
#define MAC_8x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7],rB[3],rC[7+3*TT0I]); \

#define MAC_8x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT128x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_2_1_0;
  DATA_TYPE a_0_3_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
    *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
    *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_8x4


    /* iter 15 (last unrolled loop) */

    MAC_8x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_2_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 2);
  a_0_3_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
  *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
  *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 2) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 3) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 2) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 3) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x4
#undef MAC_8x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 16
#define SG1J 4
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 832
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT32x8x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_4_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 128
#define LDS_NUM_ELEMENTS 512
#define LDS_OFFSET_BLK 256

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT16x16x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 128
#define LDS_NUM_ELEMENTS 512
#define LDS_OFFSET_BLK 256

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT16x16x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 896
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT32x16x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x2


    /* iter 7 (last unrolled loop) */

    MAC_4x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 896
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT32x16x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_8_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 32
#define SG1J 4
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 3200
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT64x8x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG32_4_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 15 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1792
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT32x16x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x2


    /* iter 15 (last unrolled loop) */

    MAC_4x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT32x32x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_2_1_0;
  DATA_TYPE a_0_3_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_2_1_0;
  DATA_TYPE b_0_3_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
    *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
    *(localWriteB_0_0_1_0 + 2) = b_0_2_1_0;
    *(localWriteB_0_0_1_0 + 3) = b_0_3_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
    *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
    *(localWriteB_0_0_1_0 + 2) = b_0_2_1_0;
    *(localWriteB_0_0_1_0 + 3) = b_0_3_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_2_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 2);
  a_0_3_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_2_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 2);
  b_0_3_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
  *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
  *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
  *(localWriteB_0_0_1_0 + 2) = b_0_2_1_0;
  *(localWriteB_0_0_1_0 + 3) = b_0_3_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 2) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 3) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 2) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 3) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 3328
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT64x16x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x2


    /* iter 15 (last unrolled loop) */

    MAC_4x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 3584
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT64x32x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_2_1_0;
  DATA_TYPE a_0_3_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
    *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
    *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_2_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 2);
  a_0_3_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
  *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
  *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 2) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 3) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 2) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 3) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 32
#define SG1J 4
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 3200
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT64x8x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG32_4_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 15 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 16
#define SG1J 4
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1664
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT32x8x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_4_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 15 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT16x16x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 15 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 24
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 3
#define NLPB 3

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 384
#define LDS_NUM_ELEMENTS 1792
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT16x16x24_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_1_2_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 384-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 15 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 16 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 17 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 18 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 19 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 20 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 21 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 22 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 23 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 16 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 17 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 18 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 19 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 20 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 21 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 22 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 23 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_1_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 7168
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT64x128x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_2_1_0;
  DATA_TYPE b_0_3_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
    *(localWriteB_0_0_1_0 + 2) = b_0_2_1_0;
    *(localWriteB_0_0_1_0 + 3) = b_0_3_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
    *(localWriteB_0_0_1_0 + 2) = b_0_2_1_0;
    *(localWriteB_0_0_1_0 + 3) = b_0_3_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x8


    /* iter 15 (last unrolled loop) */

    MAC_4x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_2_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 2);
  b_0_3_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
  *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
  *(localWriteB_0_0_1_0 + 2) = b_0_2_1_0;
  *(localWriteB_0_0_1_0 + 3) = b_0_3_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
    if (r0I == 2) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
    if (r0I == 3) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 2) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 3) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 7168
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x4 micro-tile */
#define MAC_8x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7],rB[3],rC[7+3*TT0I]); \

#define MAC_8x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT128x64x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_2_1_0;
  DATA_TYPE a_0_3_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
    *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
    *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_8x4


    /* iter 15 (last unrolled loop) */

    MAC_8x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_2_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 2);
  a_0_3_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
  *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
  *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 2) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 3) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 2) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 3) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x4
#undef MAC_8x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 7168
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x4 micro-tile */
#define MAC_8x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7],rB[3],rC[7+3*TT0I]); \

#define MAC_8x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_SB_MT128x64x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_2_1_0;
  DATA_TYPE a_0_3_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
    *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
    *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
    *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_8x4


    /* iter 15 (last unrolled loop) */

    MAC_8x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_2_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 2);
  a_0_3_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
  *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_0_0 + 2) = b_0_2_0_0;
  *(localWriteB_0_0_0_0 + 3) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 2) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 3) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 2) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
    if (r1J == 3) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 2*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x4
#undef MAC_8x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1664
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT64x16x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x2


    /* iter 7 (last unrolled loop) */

    MAC_4x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 896
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT32x16x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x2


    /* iter 7 (last unrolled loop) */

    MAC_4x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT16x16x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 15 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT32x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_2_1_0;
  DATA_TYPE a_0_3_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_2_1_0;
  DATA_TYPE b_0_3_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_0_2;
  DATA_TYPE *localWriteA_0_0_0_3;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;
  DATA_TYPE *localWriteA_0_0_1_2;
  DATA_TYPE *localWriteA_0_0_1_3;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;
  DATA_TYPE *localWriteB_0_0_1_2;
  DATA_TYPE *localWriteB_0_0_1_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
  localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
  localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_1_2 + 0) = a_0_2_1_0;
    *(localWriteA_0_0_1_3 + 0) = a_0_3_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
    *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_2 = (localWriteOffsetA_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_3 = (localWriteOffsetA_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_2 = (localWriteOffsetB_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_3 = (localWriteOffsetB_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
    localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
    localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
    localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_1_2 + 0) = a_0_2_1_0;
    *(localWriteA_0_0_1_3 + 0) = a_0_3_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
    *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_2 = (localWriteOffsetA_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_3 = (localWriteOffsetA_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_2 = (localWriteOffsetB_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_3 = (localWriteOffsetB_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
    localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
    localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
    localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_3 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_2 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_3 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_3 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_3 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_2_1_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 2);
  a_0_3_1_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_2_1_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 2);
  b_0_3_1_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
  localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
  localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
  *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
  *(localWriteA_0_0_1_2 + 0) = a_0_2_1_0;
  *(localWriteA_0_0_1_3 + 0) = a_0_3_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
  *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
  *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1664
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT64x16x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x2


    /* iter 7 (last unrolled loop) */

    MAC_4x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 896
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT32x16x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x2


    /* iter 7 (last unrolled loop) */

    MAC_4x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT16x16x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 15 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT32x32x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_2_1_0;
  DATA_TYPE a_0_3_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_2_1_0;
  DATA_TYPE b_0_3_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_2 = localWriteFirstOffsetA + (2 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_3 = localWriteFirstOffsetA + (3 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_0_2;
  DATA_TYPE *localWriteA_0_0_0_3;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;
  DATA_TYPE *localWriteA_0_0_1_2;
  DATA_TYPE *localWriteA_0_0_1_3;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;
  DATA_TYPE *localWriteB_0_0_1_2;
  DATA_TYPE *localWriteB_0_0_1_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
  localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
  localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_1_2 + 0) = a_0_2_1_0;
    *(localWriteA_0_0_1_3 + 0) = a_0_3_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
    *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_2 = (localWriteOffsetA_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_3 = (localWriteOffsetA_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_2 = (localWriteOffsetB_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_3 = (localWriteOffsetB_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
    localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
    localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
    localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
    *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_1_2 + 0) = a_0_2_1_0;
    *(localWriteA_0_0_1_3 + 0) = a_0_3_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
    *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_2 = (localWriteOffsetA_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_3 = (localWriteOffsetA_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_2 = (localWriteOffsetA_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_3 = (localWriteOffsetA_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_2 = (localWriteOffsetB_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_3 = (localWriteOffsetB_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
    localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
    localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
    localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
    localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
    localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_3 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_2 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_3 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_3 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_3 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_2_1_0 = ( globalReadOffsetAL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 2);
  a_0_3_1_0 = ( globalReadOffsetAL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_2_1_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 2);
  b_0_3_1_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_2);
  localWriteA_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_3);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_2);
  localWriteA_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_3);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
  localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_0_2 + 0) = a_0_2_0_0;
  *(localWriteA_0_0_0_3 + 0) = a_0_3_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
  *(localWriteA_0_0_1_2 + 0) = a_0_2_1_0;
  *(localWriteA_0_0_1_3 + 0) = a_0_3_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
  *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
  *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1792
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1792
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 3 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x8


    /* iter 3 (last unrolled loop) */

    MAC_4x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 3 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 3 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1536

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 7 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 2

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU2_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 2

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU2_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 2

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU2_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 7 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 7 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I] , rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I] , rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I] , rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I] , rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I] , rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I] , rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I] , rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I] , rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I] , rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I] , rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I] , rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I] , rA[7],rB[7],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT128x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_8x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 3584
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 3584
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x8


    /* iter 7 (last unrolled loop) */

    MAC_4x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 8

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I] , rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I] , rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I] , rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I] , rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I] , rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I] , rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I] , rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I] , rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I] , rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I] , rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I] , rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I] , rA[7],rB[7],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT128x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;
  unsigned int flattenedOffsetB_4_0 = globalReadOffsetB1J + 0 + 4*LSPB;
  flattenedOffsetB_4_0 = (flattenedOffsetB_4_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_4_0;
  unsigned int globalReadOffsetB1J_4_0 = flattenedOffsetB_4_0;
  unsigned int flattenedOffsetB_5_0 = globalReadOffsetB1J + 0 + 5*LSPB;
  flattenedOffsetB_5_0 = (flattenedOffsetB_5_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_5_0;
  unsigned int globalReadOffsetB1J_5_0 = flattenedOffsetB_5_0;
  unsigned int flattenedOffsetB_6_0 = globalReadOffsetB1J + 0 + 6*LSPB;
  flattenedOffsetB_6_0 = (flattenedOffsetB_6_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_6_0;
  unsigned int globalReadOffsetB1J_6_0 = flattenedOffsetB_6_0;
  unsigned int flattenedOffsetB_7_0 = globalReadOffsetB1J + 0 + 7*LSPB;
  flattenedOffsetB_7_0 = (flattenedOffsetB_7_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_7_0;
  unsigned int globalReadOffsetB1J_7_0 = flattenedOffsetB_7_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_7_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 4*LSPB);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 5*LSPB);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 6*LSPB);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 7*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 2

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU2_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 2

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU2_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 3584
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 2

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU2_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 2

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU2_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 3584
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1536

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 7 */

    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 3584
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x8


    /* iter 7 (last unrolled loop) */

    MAC_4x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 3584
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 3584
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HSS_BH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 8
#define SG1J 16
#define TT0I 2
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 4

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 128
#define LDS_NUM_ELEMENTS 1664
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x4 micro-tile */
#define MAC_2x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \

#define MAC_2x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_SB_MT16x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_4_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_16_1_WGM4(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x4


    /* iter 7 (last unrolled loop) */

    MAC_2x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x4

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x4
#undef MAC_2x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 3072

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float

  /* atomic add float */
#ifndef ATOMIC_FLOAT_FUNCTION
#define ATOMIC_FLOAT_FUNCTION

__device__ inline int atomicAddType(int *fPtr, int operand)
{
  return atomicAdd(fPtr,operand);
}

__device__ inline unsigned int atomicAddType(unsigned int *fPtr, unsigned int operand)
{
  return atomicAdd(fPtr,operand);
}

__device__ inline unsigned long long int atomicAddType(unsigned long long int *fPtr, unsigned long long int operand)
{
  return atomicAdd(fPtr,operand);
}

__device__ inline float atomicAddType(float *fPtr, float operand)
{
  return atomicAdd(fPtr,operand);
}

__device__ inline double atomicAddType(double *fPtr, double operand)
{
  return atomicAdd(fPtr,operand);
}

#endif
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) atomicAddType(&(DST), (ALPHA)*(REG));
/* 8x4 micro-tile */
#define MAC_8x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7],rB[3],rC[7+3*TT0I]); \

#define MAC_8x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT128x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_4_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;
  unsigned int flattenedOffsetA_4_0 = globalReadOffsetA0I + 0 + 4*LSPA;
  flattenedOffsetA_4_0 = (flattenedOffsetA_4_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_4_0;
  unsigned int globalReadOffsetA0I_4_0 = flattenedOffsetA_4_0;
  unsigned int flattenedOffsetA_5_0 = globalReadOffsetA0I + 0 + 5*LSPA;
  flattenedOffsetA_5_0 = (flattenedOffsetA_5_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_5_0;
  unsigned int globalReadOffsetA0I_5_0 = flattenedOffsetA_5_0;
  unsigned int flattenedOffsetA_6_0 = globalReadOffsetA0I + 0 + 6*LSPA;
  flattenedOffsetA_6_0 = (flattenedOffsetA_6_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_6_0;
  unsigned int globalReadOffsetA0I_6_0 = flattenedOffsetA_6_0;
  unsigned int flattenedOffsetA_7_0 = globalReadOffsetA0I + 0 + 7*LSPA;
  flattenedOffsetA_7_0 = (flattenedOffsetA_7_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_7_0;
  unsigned int globalReadOffsetA0I_7_0 = flattenedOffsetA_7_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;
  globalReadOffsetA0I_4_0 = (  globalReadOffsetA0I_4_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_4_0;
  globalReadOffsetA0I_5_0 = (  globalReadOffsetA0I_5_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_5_0;
  globalReadOffsetA0I_6_0 = (  globalReadOffsetA0I_6_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_6_0;
  globalReadOffsetA0I_7_0 = (  globalReadOffsetA0I_7_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_7_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 4*LSPA);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 5*LSPA);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 6*LSPA);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 7*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x4


    /* iter 15 */

    MAC_8x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x4
#undef MAC_8x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 3072

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x4 micro-tile */
#define MAC_8x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7],rB[3],rC[7+3*TT0I]); \

#define MAC_8x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT128x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_4_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;
  unsigned int flattenedOffsetA_4_0 = globalReadOffsetA0I + 0 + 4*LSPA;
  flattenedOffsetA_4_0 = (flattenedOffsetA_4_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_4_0;
  unsigned int globalReadOffsetA0I_4_0 = flattenedOffsetA_4_0;
  unsigned int flattenedOffsetA_5_0 = globalReadOffsetA0I + 0 + 5*LSPA;
  flattenedOffsetA_5_0 = (flattenedOffsetA_5_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_5_0;
  unsigned int globalReadOffsetA0I_5_0 = flattenedOffsetA_5_0;
  unsigned int flattenedOffsetA_6_0 = globalReadOffsetA0I + 0 + 6*LSPA;
  flattenedOffsetA_6_0 = (flattenedOffsetA_6_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_6_0;
  unsigned int globalReadOffsetA0I_6_0 = flattenedOffsetA_6_0;
  unsigned int flattenedOffsetA_7_0 = globalReadOffsetA0I + 0 + 7*LSPA;
  flattenedOffsetA_7_0 = (flattenedOffsetA_7_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_7_0;
  unsigned int globalReadOffsetA0I_7_0 = flattenedOffsetA_7_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;
  globalReadOffsetA0I_4_0 = (  globalReadOffsetA0I_4_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_4_0;
  globalReadOffsetA0I_5_0 = (  globalReadOffsetA0I_5_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_5_0;
  globalReadOffsetA0I_6_0 = (  globalReadOffsetA0I_6_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_6_0;
  globalReadOffsetA0I_7_0 = (  globalReadOffsetA0I_7_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_7_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 4*LSPA);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 5*LSPA);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 6*LSPA);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 7*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x4


    /* iter 15 */

    MAC_8x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x4
#undef MAC_8x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 16
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 16
#define NLPB 8

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 4096
#define LDS_NUM_ELEMENTS 6144

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float

  /* atomic add float */
#ifndef ATOMIC_FLOAT_FUNCTION
#define ATOMIC_FLOAT_FUNCTION

__device__ inline int atomicAddType(int *fPtr, int operand)
{
  return atomicAdd(fPtr,operand);
}

__device__ inline unsigned int atomicAddType(unsigned int *fPtr, unsigned int operand)
{
  return atomicAdd(fPtr,operand);
}

__device__ inline unsigned long long int atomicAddType(unsigned long long int *fPtr, unsigned long long int operand)
{
  return atomicAdd(fPtr,operand);
}

__device__ inline float atomicAddType(float *fPtr, float operand)
{
  return atomicAdd(fPtr,operand);
}

__device__ inline double atomicAddType(double *fPtr, double operand)
{
  return atomicAdd(fPtr,operand);
}

#endif
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) atomicAddType(&(DST), (ALPHA)*(REG));
/* 16x8 micro-tile */
#define MAC_16x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[8],rB[0],rC[8+0*TT0I]); \
  TYPE_MAC(rA[9],rB[0],rC[9+0*TT0I]); \
  TYPE_MAC(rA[10],rB[0],rC[10+0*TT0I]); \
  TYPE_MAC(rA[11],rB[0],rC[11+0*TT0I]); \
  TYPE_MAC(rA[12],rB[0],rC[12+0*TT0I]); \
  TYPE_MAC(rA[13],rB[0],rC[13+0*TT0I]); \
  TYPE_MAC(rA[14],rB[0],rC[14+0*TT0I]); \
  TYPE_MAC(rA[15],rB[0],rC[15+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[8],rB[1],rC[8+1*TT0I]); \
  TYPE_MAC(rA[9],rB[1],rC[9+1*TT0I]); \
  TYPE_MAC(rA[10],rB[1],rC[10+1*TT0I]); \
  TYPE_MAC(rA[11],rB[1],rC[11+1*TT0I]); \
  TYPE_MAC(rA[12],rB[1],rC[12+1*TT0I]); \
  TYPE_MAC(rA[13],rB[1],rC[13+1*TT0I]); \
  TYPE_MAC(rA[14],rB[1],rC[14+1*TT0I]); \
  TYPE_MAC(rA[15],rB[1],rC[15+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[8],rB[2],rC[8+2*TT0I]); \
  TYPE_MAC(rA[9],rB[2],rC[9+2*TT0I]); \
  TYPE_MAC(rA[10],rB[2],rC[10+2*TT0I]); \
  TYPE_MAC(rA[11],rB[2],rC[11+2*TT0I]); \
  TYPE_MAC(rA[12],rB[2],rC[12+2*TT0I]); \
  TYPE_MAC(rA[13],rB[2],rC[13+2*TT0I]); \
  TYPE_MAC(rA[14],rB[2],rC[14+2*TT0I]); \
  TYPE_MAC(rA[15],rB[2],rC[15+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[8],rB[3],rC[8+3*TT0I]); \
  TYPE_MAC(rA[9],rB[3],rC[9+3*TT0I]); \
  TYPE_MAC(rA[10],rB[3],rC[10+3*TT0I]); \
  TYPE_MAC(rA[11],rB[3],rC[11+3*TT0I]); \
  TYPE_MAC(rA[12],rB[3],rC[12+3*TT0I]); \
  TYPE_MAC(rA[13],rB[3],rC[13+3*TT0I]); \
  TYPE_MAC(rA[14],rB[3],rC[14+3*TT0I]); \
  TYPE_MAC(rA[15],rB[3],rC[15+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I]); \
  TYPE_MAC(rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I]); \
  TYPE_MAC(rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[8],rB[4],rC[8+4*TT0I]); \
  TYPE_MAC(rA[9],rB[4],rC[9+4*TT0I]); \
  TYPE_MAC(rA[10],rB[4],rC[10+4*TT0I]); \
  TYPE_MAC(rA[11],rB[4],rC[11+4*TT0I]); \
  TYPE_MAC(rA[12],rB[4],rC[12+4*TT0I]); \
  TYPE_MAC(rA[13],rB[4],rC[13+4*TT0I]); \
  TYPE_MAC(rA[14],rB[4],rC[14+4*TT0I]); \
  TYPE_MAC(rA[15],rB[4],rC[15+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I]); \
  TYPE_MAC(rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I]); \
  TYPE_MAC(rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[8],rB[5],rC[8+5*TT0I]); \
  TYPE_MAC(rA[9],rB[5],rC[9+5*TT0I]); \
  TYPE_MAC(rA[10],rB[5],rC[10+5*TT0I]); \
  TYPE_MAC(rA[11],rB[5],rC[11+5*TT0I]); \
  TYPE_MAC(rA[12],rB[5],rC[12+5*TT0I]); \
  TYPE_MAC(rA[13],rB[5],rC[13+5*TT0I]); \
  TYPE_MAC(rA[14],rB[5],rC[14+5*TT0I]); \
  TYPE_MAC(rA[15],rB[5],rC[15+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I]); \
  TYPE_MAC(rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I]); \
  TYPE_MAC(rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[8],rB[6],rC[8+6*TT0I]); \
  TYPE_MAC(rA[9],rB[6],rC[9+6*TT0I]); \
  TYPE_MAC(rA[10],rB[6],rC[10+6*TT0I]); \
  TYPE_MAC(rA[11],rB[6],rC[11+6*TT0I]); \
  TYPE_MAC(rA[12],rB[6],rC[12+6*TT0I]); \
  TYPE_MAC(rA[13],rB[6],rC[13+6*TT0I]); \
  TYPE_MAC(rA[14],rB[6],rC[14+6*TT0I]); \
  TYPE_MAC(rA[15],rB[6],rC[15+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I]); \
  TYPE_MAC(rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I]); \
  TYPE_MAC(rA[7],rB[7],rC[7+7*TT0I]); \
  TYPE_MAC(rA[8],rB[7],rC[8+7*TT0I]); \
  TYPE_MAC(rA[9],rB[7],rC[9+7*TT0I]); \
  TYPE_MAC(rA[10],rB[7],rC[10+7*TT0I]); \
  TYPE_MAC(rA[11],rB[7],rC[11+7*TT0I]); \
  TYPE_MAC(rA[12],rB[7],rC[12+7*TT0I]); \
  TYPE_MAC(rA[13],rB[7],rC[13+7*TT0I]); \
  TYPE_MAC(rA[14],rB[7],rC[14+7*TT0I]); \
  TYPE_MAC(rA[15],rB[7],rC[15+7*TT0I]); \

#define MAC_16x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[8+TT0I],rB[0+TT1J],rC[8+0*TT0I]); \
  TYPE_MAC(rA[9+TT0I],rB[0+TT1J],rC[9+0*TT0I]); \
  TYPE_MAC(rA[10+TT0I],rB[0+TT1J],rC[10+0*TT0I]); \
  TYPE_MAC(rA[11+TT0I],rB[0+TT1J],rC[11+0*TT0I]); \
  TYPE_MAC(rA[12+TT0I],rB[0+TT1J],rC[12+0*TT0I]); \
  TYPE_MAC(rA[13+TT0I],rB[0+TT1J],rC[13+0*TT0I]); \
  TYPE_MAC(rA[14+TT0I],rB[0+TT1J],rC[14+0*TT0I]); \
  TYPE_MAC(rA[15+TT0I],rB[0+TT1J],rC[15+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[8+TT0I],rB[1+TT1J],rC[8+1*TT0I]); \
  TYPE_MAC(rA[9+TT0I],rB[1+TT1J],rC[9+1*TT0I]); \
  TYPE_MAC(rA[10+TT0I],rB[1+TT1J],rC[10+1*TT0I]); \
  TYPE_MAC(rA[11+TT0I],rB[1+TT1J],rC[11+1*TT0I]); \
  TYPE_MAC(rA[12+TT0I],rB[1+TT1J],rC[12+1*TT0I]); \
  TYPE_MAC(rA[13+TT0I],rB[1+TT1J],rC[13+1*TT0I]); \
  TYPE_MAC(rA[14+TT0I],rB[1+TT1J],rC[14+1*TT0I]); \
  TYPE_MAC(rA[15+TT0I],rB[1+TT1J],rC[15+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[8+TT0I],rB[2+TT1J],rC[8+2*TT0I]); \
  TYPE_MAC(rA[9+TT0I],rB[2+TT1J],rC[9+2*TT0I]); \
  TYPE_MAC(rA[10+TT0I],rB[2+TT1J],rC[10+2*TT0I]); \
  TYPE_MAC(rA[11+TT0I],rB[2+TT1J],rC[11+2*TT0I]); \
  TYPE_MAC(rA[12+TT0I],rB[2+TT1J],rC[12+2*TT0I]); \
  TYPE_MAC(rA[13+TT0I],rB[2+TT1J],rC[13+2*TT0I]); \
  TYPE_MAC(rA[14+TT0I],rB[2+TT1J],rC[14+2*TT0I]); \
  TYPE_MAC(rA[15+TT0I],rB[2+TT1J],rC[15+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \
  TYPE_MAC(rA[8+TT0I],rB[3+TT1J],rC[8+3*TT0I]); \
  TYPE_MAC(rA[9+TT0I],rB[3+TT1J],rC[9+3*TT0I]); \
  TYPE_MAC(rA[10+TT0I],rB[3+TT1J],rC[10+3*TT0I]); \
  TYPE_MAC(rA[11+TT0I],rB[3+TT1J],rC[11+3*TT0I]); \
  TYPE_MAC(rA[12+TT0I],rB[3+TT1J],rC[12+3*TT0I]); \
  TYPE_MAC(rA[13+TT0I],rB[3+TT1J],rC[13+3*TT0I]); \
  TYPE_MAC(rA[14+TT0I],rB[3+TT1J],rC[14+3*TT0I]); \
  TYPE_MAC(rA[15+TT0I],rB[3+TT1J],rC[15+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[4+TT1J],rC[4+4*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[4+TT1J],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[4+TT1J],rC[6+4*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[4+TT1J],rC[7+4*TT0I]); \
  TYPE_MAC(rA[8+TT0I],rB[4+TT1J],rC[8+4*TT0I]); \
  TYPE_MAC(rA[9+TT0I],rB[4+TT1J],rC[9+4*TT0I]); \
  TYPE_MAC(rA[10+TT0I],rB[4+TT1J],rC[10+4*TT0I]); \
  TYPE_MAC(rA[11+TT0I],rB[4+TT1J],rC[11+4*TT0I]); \
  TYPE_MAC(rA[12+TT0I],rB[4+TT1J],rC[12+4*TT0I]); \
  TYPE_MAC(rA[13+TT0I],rB[4+TT1J],rC[13+4*TT0I]); \
  TYPE_MAC(rA[14+TT0I],rB[4+TT1J],rC[14+4*TT0I]); \
  TYPE_MAC(rA[15+TT0I],rB[4+TT1J],rC[15+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[5+TT1J],rC[4+5*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[5+TT1J],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[5+TT1J],rC[6+5*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[5+TT1J],rC[7+5*TT0I]); \
  TYPE_MAC(rA[8+TT0I],rB[5+TT1J],rC[8+5*TT0I]); \
  TYPE_MAC(rA[9+TT0I],rB[5+TT1J],rC[9+5*TT0I]); \
  TYPE_MAC(rA[10+TT0I],rB[5+TT1J],rC[10+5*TT0I]); \
  TYPE_MAC(rA[11+TT0I],rB[5+TT1J],rC[11+5*TT0I]); \
  TYPE_MAC(rA[12+TT0I],rB[5+TT1J],rC[12+5*TT0I]); \
  TYPE_MAC(rA[13+TT0I],rB[5+TT1J],rC[13+5*TT0I]); \
  TYPE_MAC(rA[14+TT0I],rB[5+TT1J],rC[14+5*TT0I]); \
  TYPE_MAC(rA[15+TT0I],rB[5+TT1J],rC[15+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[6+TT1J],rC[4+6*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[6+TT1J],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[6+TT1J],rC[6+6*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[6+TT1J],rC[7+6*TT0I]); \
  TYPE_MAC(rA[8+TT0I],rB[6+TT1J],rC[8+6*TT0I]); \
  TYPE_MAC(rA[9+TT0I],rB[6+TT1J],rC[9+6*TT0I]); \
  TYPE_MAC(rA[10+TT0I],rB[6+TT1J],rC[10+6*TT0I]); \
  TYPE_MAC(rA[11+TT0I],rB[6+TT1J],rC[11+6*TT0I]); \
  TYPE_MAC(rA[12+TT0I],rB[6+TT1J],rC[12+6*TT0I]); \
  TYPE_MAC(rA[13+TT0I],rB[6+TT1J],rC[13+6*TT0I]); \
  TYPE_MAC(rA[14+TT0I],rB[6+TT1J],rC[14+6*TT0I]); \
  TYPE_MAC(rA[15+TT0I],rB[6+TT1J],rC[15+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[7+TT1J],rC[4+7*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[7+TT1J],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[7+TT1J],rC[6+7*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[7+TT1J],rC[7+7*TT0I]); \
  TYPE_MAC(rA[8+TT0I],rB[7+TT1J],rC[8+7*TT0I]); \
  TYPE_MAC(rA[9+TT0I],rB[7+TT1J],rC[9+7*TT0I]); \
  TYPE_MAC(rA[10+TT0I],rB[7+TT1J],rC[10+7*TT0I]); \
  TYPE_MAC(rA[11+TT0I],rB[7+TT1J],rC[11+7*TT0I]); \
  TYPE_MAC(rA[12+TT0I],rB[7+TT1J],rC[12+7*TT0I]); \
  TYPE_MAC(rA[13+TT0I],rB[7+TT1J],rC[13+7*TT0I]); \
  TYPE_MAC(rA[14+TT0I],rB[7+TT1J],rC[14+7*TT0I]); \
  TYPE_MAC(rA[15+TT0I],rB[7+TT1J],rC[15+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT256x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT16_8_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;
  unsigned int flattenedOffsetA_4_0 = globalReadOffsetA0I + 0 + 4*LSPA;
  flattenedOffsetA_4_0 = (flattenedOffsetA_4_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_4_0;
  unsigned int globalReadOffsetA0I_4_0 = flattenedOffsetA_4_0;
  unsigned int flattenedOffsetA_5_0 = globalReadOffsetA0I + 0 + 5*LSPA;
  flattenedOffsetA_5_0 = (flattenedOffsetA_5_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_5_0;
  unsigned int globalReadOffsetA0I_5_0 = flattenedOffsetA_5_0;
  unsigned int flattenedOffsetA_6_0 = globalReadOffsetA0I + 0 + 6*LSPA;
  flattenedOffsetA_6_0 = (flattenedOffsetA_6_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_6_0;
  unsigned int globalReadOffsetA0I_6_0 = flattenedOffsetA_6_0;
  unsigned int flattenedOffsetA_7_0 = globalReadOffsetA0I + 0 + 7*LSPA;
  flattenedOffsetA_7_0 = (flattenedOffsetA_7_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_7_0;
  unsigned int globalReadOffsetA0I_7_0 = flattenedOffsetA_7_0;
  unsigned int flattenedOffsetA_8_0 = globalReadOffsetA0I + 0 + 8*LSPA;
  flattenedOffsetA_8_0 = (flattenedOffsetA_8_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_8_0;
  unsigned int globalReadOffsetA0I_8_0 = flattenedOffsetA_8_0;
  unsigned int flattenedOffsetA_9_0 = globalReadOffsetA0I + 0 + 9*LSPA;
  flattenedOffsetA_9_0 = (flattenedOffsetA_9_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_9_0;
  unsigned int globalReadOffsetA0I_9_0 = flattenedOffsetA_9_0;
  unsigned int flattenedOffsetA_10_0 = globalReadOffsetA0I + 0 + 10*LSPA;
  flattenedOffsetA_10_0 = (flattenedOffsetA_10_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_10_0;
  unsigned int globalReadOffsetA0I_10_0 = flattenedOffsetA_10_0;
  unsigned int flattenedOffsetA_11_0 = globalReadOffsetA0I + 0 + 11*LSPA;
  flattenedOffsetA_11_0 = (flattenedOffsetA_11_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_11_0;
  unsigned int globalReadOffsetA0I_11_0 = flattenedOffsetA_11_0;
  unsigned int flattenedOffsetA_12_0 = globalReadOffsetA0I + 0 + 12*LSPA;
  flattenedOffsetA_12_0 = (flattenedOffsetA_12_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_12_0;
  unsigned int globalReadOffsetA0I_12_0 = flattenedOffsetA_12_0;
  unsigned int flattenedOffsetA_13_0 = globalReadOffsetA0I + 0 + 13*LSPA;
  flattenedOffsetA_13_0 = (flattenedOffsetA_13_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_13_0;
  unsigned int globalReadOffsetA0I_13_0 = flattenedOffsetA_13_0;
  unsigned int flattenedOffsetA_14_0 = globalReadOffsetA0I + 0 + 14*LSPA;
  flattenedOffsetA_14_0 = (flattenedOffsetA_14_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_14_0;
  unsigned int globalReadOffsetA0I_14_0 = flattenedOffsetA_14_0;
  unsigned int flattenedOffsetA_15_0 = globalReadOffsetA0I + 0 + 15*LSPA;
  flattenedOffsetA_15_0 = (flattenedOffsetA_15_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_15_0;
  unsigned int globalReadOffsetA0I_15_0 = flattenedOffsetA_15_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;
  unsigned int flattenedOffsetB_4_0 = globalReadOffsetB1J + 0 + 4*LSPB;
  flattenedOffsetB_4_0 = (flattenedOffsetB_4_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_4_0;
  unsigned int globalReadOffsetB1J_4_0 = flattenedOffsetB_4_0;
  unsigned int flattenedOffsetB_5_0 = globalReadOffsetB1J + 0 + 5*LSPB;
  flattenedOffsetB_5_0 = (flattenedOffsetB_5_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_5_0;
  unsigned int globalReadOffsetB1J_5_0 = flattenedOffsetB_5_0;
  unsigned int flattenedOffsetB_6_0 = globalReadOffsetB1J + 0 + 6*LSPB;
  flattenedOffsetB_6_0 = (flattenedOffsetB_6_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_6_0;
  unsigned int globalReadOffsetB1J_6_0 = flattenedOffsetB_6_0;
  unsigned int flattenedOffsetB_7_0 = globalReadOffsetB1J + 0 + 7*LSPB;
  flattenedOffsetB_7_0 = (flattenedOffsetB_7_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_7_0;
  unsigned int globalReadOffsetB1J_7_0 = flattenedOffsetB_7_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;
  globalReadOffsetA0I_4_0 = (  globalReadOffsetA0I_4_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_4_0;
  globalReadOffsetA0I_5_0 = (  globalReadOffsetA0I_5_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_5_0;
  globalReadOffsetA0I_6_0 = (  globalReadOffsetA0I_6_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_6_0;
  globalReadOffsetA0I_7_0 = (  globalReadOffsetA0I_7_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_7_0;
  globalReadOffsetA0I_8_0 = (  globalReadOffsetA0I_8_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_8_0;
  globalReadOffsetA0I_9_0 = (  globalReadOffsetA0I_9_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_9_0;
  globalReadOffsetA0I_10_0 = (  globalReadOffsetA0I_10_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_10_0;
  globalReadOffsetA0I_11_0 = (  globalReadOffsetA0I_11_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_11_0;
  globalReadOffsetA0I_12_0 = (  globalReadOffsetA0I_12_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_12_0;
  globalReadOffsetA0I_13_0 = (  globalReadOffsetA0I_13_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_13_0;
  globalReadOffsetA0I_14_0 = (  globalReadOffsetA0I_14_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_14_0;
  globalReadOffsetA0I_15_0 = (  globalReadOffsetA0I_15_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_15_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;
  globalReadOffsetB1J_4_0 = (  globalReadOffsetB1J_4_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_4_0;
  globalReadOffsetB1J_5_0 = (  globalReadOffsetB1J_5_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_5_0;
  globalReadOffsetB1J_6_0 = (  globalReadOffsetB1J_6_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_6_0;
  globalReadOffsetB1J_7_0 = (  globalReadOffsetB1J_7_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_7_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_15_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_7_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 4*LSPA);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 5*LSPA);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 6*LSPA);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 7*LSPA);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 8*LSPA);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 9*LSPA);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 10*LSPA);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 11*LSPA);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 12*LSPA);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 13*LSPA);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 14*LSPA);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 15*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 4*LSPB);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 5*LSPB);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 6*LSPB);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 7*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_0_0_15_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;
  rC[64] = SCALAR_ZERO;
  rC[65] = SCALAR_ZERO;
  rC[66] = SCALAR_ZERO;
  rC[67] = SCALAR_ZERO;
  rC[68] = SCALAR_ZERO;
  rC[69] = SCALAR_ZERO;
  rC[70] = SCALAR_ZERO;
  rC[71] = SCALAR_ZERO;
  rC[72] = SCALAR_ZERO;
  rC[73] = SCALAR_ZERO;
  rC[74] = SCALAR_ZERO;
  rC[75] = SCALAR_ZERO;
  rC[76] = SCALAR_ZERO;
  rC[77] = SCALAR_ZERO;
  rC[78] = SCALAR_ZERO;
  rC[79] = SCALAR_ZERO;
  rC[80] = SCALAR_ZERO;
  rC[81] = SCALAR_ZERO;
  rC[82] = SCALAR_ZERO;
  rC[83] = SCALAR_ZERO;
  rC[84] = SCALAR_ZERO;
  rC[85] = SCALAR_ZERO;
  rC[86] = SCALAR_ZERO;
  rC[87] = SCALAR_ZERO;
  rC[88] = SCALAR_ZERO;
  rC[89] = SCALAR_ZERO;
  rC[90] = SCALAR_ZERO;
  rC[91] = SCALAR_ZERO;
  rC[92] = SCALAR_ZERO;
  rC[93] = SCALAR_ZERO;
  rC[94] = SCALAR_ZERO;
  rC[95] = SCALAR_ZERO;
  rC[96] = SCALAR_ZERO;
  rC[97] = SCALAR_ZERO;
  rC[98] = SCALAR_ZERO;
  rC[99] = SCALAR_ZERO;
  rC[100] = SCALAR_ZERO;
  rC[101] = SCALAR_ZERO;
  rC[102] = SCALAR_ZERO;
  rC[103] = SCALAR_ZERO;
  rC[104] = SCALAR_ZERO;
  rC[105] = SCALAR_ZERO;
  rC[106] = SCALAR_ZERO;
  rC[107] = SCALAR_ZERO;
  rC[108] = SCALAR_ZERO;
  rC[109] = SCALAR_ZERO;
  rC[110] = SCALAR_ZERO;
  rC[111] = SCALAR_ZERO;
  rC[112] = SCALAR_ZERO;
  rC[113] = SCALAR_ZERO;
  rC[114] = SCALAR_ZERO;
  rC[115] = SCALAR_ZERO;
  rC[116] = SCALAR_ZERO;
  rC[117] = SCALAR_ZERO;
  rC[118] = SCALAR_ZERO;
  rC[119] = SCALAR_ZERO;
  rC[120] = SCALAR_ZERO;
  rC[121] = SCALAR_ZERO;
  rC[122] = SCALAR_ZERO;
  rC[123] = SCALAR_ZERO;
  rC[124] = SCALAR_ZERO;
  rC[125] = SCALAR_ZERO;
  rC[126] = SCALAR_ZERO;
  rC[127] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_8_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_9_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_10_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_11_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_12_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_13_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_14_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_15_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_8_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_9_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_10_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_11_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_12_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_13_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_14_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_15_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0+TT0I] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0+TT0I] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0+TT0I] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0+TT0I] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0+TT0I] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0+TT0I] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0+TT0I] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0+TT0I] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0+TT0I] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0+TT0I] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0+TT0I] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0+TT0I] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0+TT0I] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0+TT0I] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0+TT0I] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0+TT0I] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0+TT0I] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0+TT0I] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0+TT0I] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0+TT0I] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0+TT0I] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0+TT0I] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0+TT0I] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0+TT0I] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0+TT0I] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0+TT0I] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0+TT0I] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0+TT0I] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0+TT0I] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0+TT0I] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0+TT0I] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0+TT0I] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0+TT0I] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0+TT0I] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0+TT0I] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0+TT0I] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0+TT0I] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0+TT0I] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0+TT0I] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0+TT0I] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0+TT0I] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0+TT0I] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0+TT0I] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0+TT0I] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0+TT0I] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0+TT0I] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0+TT0I] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0+TT0I] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0+TT0I] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0+TT0I] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0+TT0I] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0+TT0I] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0+TT0I] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0+TT0I] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0+TT0I] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0+TT0I] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0+TT0I] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0+TT0I] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0+TT0I] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0+TT0I] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0+TT0I] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0+TT0I] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0+TT0I] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0+TT0I] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_16x8


    /* iter 15 */

    MAC_16x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_0_8_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_0_0_9_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_0_0_10_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_0_0_11_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_0_0_12_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_0_0_13_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_0_0_14_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_0_0_15_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_16x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  8*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 8*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[8*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  9*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 9*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[9*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  10*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 10*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[10*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  11*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 11*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[11*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  12*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 12*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[12*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  13*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 13*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[13*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  14*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 14*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[14*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  15*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 15*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[15*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  8*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 8*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[8*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  9*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 9*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[9*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  10*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 10*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[10*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  11*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 11*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[11*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  12*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 12*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[12*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  13*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 13*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[13*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  14*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 14*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[14*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  15*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 15*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[15*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  8*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 8*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[8*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  9*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 9*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[9*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  10*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 10*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[10*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  11*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 11*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[11*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  12*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 12*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[12*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  13*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 13*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[13*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  14*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 14*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[14*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  15*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 15*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[15*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  8*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 8*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[8*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  9*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 9*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[9*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  10*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 10*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[10*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  11*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 11*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[11*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  12*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 12*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[12*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  13*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 13*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[13*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  14*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 14*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[14*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  15*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 15*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[15*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  8*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 8*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[8*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  9*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 9*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[9*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  10*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 10*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[10*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  11*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 11*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[11*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  12*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 12*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[12*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  13*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 13*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[13*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  14*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 14*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[14*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  15*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 15*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[15*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  8*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 8*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[8*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  9*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 9*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[9*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  10*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 10*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[10*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  11*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 11*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[11*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  12*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 12*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[12*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  13*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 13*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[13*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  14*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 14*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[14*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  15*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 15*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[15*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  8*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 8*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[8*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  9*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 9*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[9*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  10*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 10*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[10*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  11*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 11*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[11*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  12*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 12*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[12*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  13*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 13*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[13*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  14*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 14*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[14*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  15*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 15*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[15*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  8*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 8*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[8*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  9*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 9*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[9*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  10*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 10*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[10*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  11*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 11*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[11*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  12*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 12*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[12*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  13*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 13*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[13*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  14*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 14*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[14*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  15*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 15*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[15*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_16x8
#undef MAC_16x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 16
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 16
#define NLPB 8

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 4096
#define LDS_NUM_ELEMENTS 6144

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 16x8 micro-tile */
#define MAC_16x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[8],rB[0],rC[8+0*TT0I]); \
  TYPE_MAC(rA[9],rB[0],rC[9+0*TT0I]); \
  TYPE_MAC(rA[10],rB[0],rC[10+0*TT0I]); \
  TYPE_MAC(rA[11],rB[0],rC[11+0*TT0I]); \
  TYPE_MAC(rA[12],rB[0],rC[12+0*TT0I]); \
  TYPE_MAC(rA[13],rB[0],rC[13+0*TT0I]); \
  TYPE_MAC(rA[14],rB[0],rC[14+0*TT0I]); \
  TYPE_MAC(rA[15],rB[0],rC[15+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[8],rB[1],rC[8+1*TT0I]); \
  TYPE_MAC(rA[9],rB[1],rC[9+1*TT0I]); \
  TYPE_MAC(rA[10],rB[1],rC[10+1*TT0I]); \
  TYPE_MAC(rA[11],rB[1],rC[11+1*TT0I]); \
  TYPE_MAC(rA[12],rB[1],rC[12+1*TT0I]); \
  TYPE_MAC(rA[13],rB[1],rC[13+1*TT0I]); \
  TYPE_MAC(rA[14],rB[1],rC[14+1*TT0I]); \
  TYPE_MAC(rA[15],rB[1],rC[15+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[8],rB[2],rC[8+2*TT0I]); \
  TYPE_MAC(rA[9],rB[2],rC[9+2*TT0I]); \
  TYPE_MAC(rA[10],rB[2],rC[10+2*TT0I]); \
  TYPE_MAC(rA[11],rB[2],rC[11+2*TT0I]); \
  TYPE_MAC(rA[12],rB[2],rC[12+2*TT0I]); \
  TYPE_MAC(rA[13],rB[2],rC[13+2*TT0I]); \
  TYPE_MAC(rA[14],rB[2],rC[14+2*TT0I]); \
  TYPE_MAC(rA[15],rB[2],rC[15+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[8],rB[3],rC[8+3*TT0I]); \
  TYPE_MAC(rA[9],rB[3],rC[9+3*TT0I]); \
  TYPE_MAC(rA[10],rB[3],rC[10+3*TT0I]); \
  TYPE_MAC(rA[11],rB[3],rC[11+3*TT0I]); \
  TYPE_MAC(rA[12],rB[3],rC[12+3*TT0I]); \
  TYPE_MAC(rA[13],rB[3],rC[13+3*TT0I]); \
  TYPE_MAC(rA[14],rB[3],rC[14+3*TT0I]); \
  TYPE_MAC(rA[15],rB[3],rC[15+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I]); \
  TYPE_MAC(rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I]); \
  TYPE_MAC(rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[8],rB[4],rC[8+4*TT0I]); \
  TYPE_MAC(rA[9],rB[4],rC[9+4*TT0I]); \
  TYPE_MAC(rA[10],rB[4],rC[10+4*TT0I]); \
  TYPE_MAC(rA[11],rB[4],rC[11+4*TT0I]); \
  TYPE_MAC(rA[12],rB[4],rC[12+4*TT0I]); \
  TYPE_MAC(rA[13],rB[4],rC[13+4*TT0I]); \
  TYPE_MAC(rA[14],rB[4],rC[14+4*TT0I]); \
  TYPE_MAC(rA[15],rB[4],rC[15+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I]); \
  TYPE_MAC(rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I]); \
  TYPE_MAC(rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[8],rB[5],rC[8+5*TT0I]); \
  TYPE_MAC(rA[9],rB[5],rC[9+5*TT0I]); \
  TYPE_MAC(rA[10],rB[5],rC[10+5*TT0I]); \
  TYPE_MAC(rA[11],rB[5],rC[11+5*TT0I]); \
  TYPE_MAC(rA[12],rB[5],rC[12+5*TT0I]); \
  TYPE_MAC(rA[13],rB[5],rC[13+5*TT0I]); \
  TYPE_MAC(rA[14],rB[5],rC[14+5*TT0I]); \
  TYPE_MAC(rA[15],rB[5],rC[15+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I]); \
  TYPE_MAC(rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I]); \
  TYPE_MAC(rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[8],rB[6],rC[8+6*TT0I]); \
  TYPE_MAC(rA[9],rB[6],rC[9+6*TT0I]); \
  TYPE_MAC(rA[10],rB[6],rC[10+6*TT0I]); \
  TYPE_MAC(rA[11],rB[6],rC[11+6*TT0I]); \
  TYPE_MAC(rA[12],rB[6],rC[12+6*TT0I]); \
  TYPE_MAC(rA[13],rB[6],rC[13+6*TT0I]); \
  TYPE_MAC(rA[14],rB[6],rC[14+6*TT0I]); \
  TYPE_MAC(rA[15],rB[6],rC[15+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I]); \
  TYPE_MAC(rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I]); \
  TYPE_MAC(rA[7],rB[7],rC[7+7*TT0I]); \
  TYPE_MAC(rA[8],rB[7],rC[8+7*TT0I]); \
  TYPE_MAC(rA[9],rB[7],rC[9+7*TT0I]); \
  TYPE_MAC(rA[10],rB[7],rC[10+7*TT0I]); \
  TYPE_MAC(rA[11],rB[7],rC[11+7*TT0I]); \
  TYPE_MAC(rA[12],rB[7],rC[12+7*TT0I]); \
  TYPE_MAC(rA[13],rB[7],rC[13+7*TT0I]); \
  TYPE_MAC(rA[14],rB[7],rC[14+7*TT0I]); \
  TYPE_MAC(rA[15],rB[7],rC[15+7*TT0I]); \

#define MAC_16x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[8+TT0I],rB[0+TT1J],rC[8+0*TT0I]); \
  TYPE_MAC(rA[9+TT0I],rB[0+TT1J],rC[9+0*TT0I]); \
  TYPE_MAC(rA[10+TT0I],rB[0+TT1J],rC[10+0*TT0I]); \
  TYPE_MAC(rA[11+TT0I],rB[0+TT1J],rC[11+0*TT0I]); \
  TYPE_MAC(rA[12+TT0I],rB[0+TT1J],rC[12+0*TT0I]); \
  TYPE_MAC(rA[13+TT0I],rB[0+TT1J],rC[13+0*TT0I]); \
  TYPE_MAC(rA[14+TT0I],rB[0+TT1J],rC[14+0*TT0I]); \
  TYPE_MAC(rA[15+TT0I],rB[0+TT1J],rC[15+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[8+TT0I],rB[1+TT1J],rC[8+1*TT0I]); \
  TYPE_MAC(rA[9+TT0I],rB[1+TT1J],rC[9+1*TT0I]); \
  TYPE_MAC(rA[10+TT0I],rB[1+TT1J],rC[10+1*TT0I]); \
  TYPE_MAC(rA[11+TT0I],rB[1+TT1J],rC[11+1*TT0I]); \
  TYPE_MAC(rA[12+TT0I],rB[1+TT1J],rC[12+1*TT0I]); \
  TYPE_MAC(rA[13+TT0I],rB[1+TT1J],rC[13+1*TT0I]); \
  TYPE_MAC(rA[14+TT0I],rB[1+TT1J],rC[14+1*TT0I]); \
  TYPE_MAC(rA[15+TT0I],rB[1+TT1J],rC[15+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[8+TT0I],rB[2+TT1J],rC[8+2*TT0I]); \
  TYPE_MAC(rA[9+TT0I],rB[2+TT1J],rC[9+2*TT0I]); \
  TYPE_MAC(rA[10+TT0I],rB[2+TT1J],rC[10+2*TT0I]); \
  TYPE_MAC(rA[11+TT0I],rB[2+TT1J],rC[11+2*TT0I]); \
  TYPE_MAC(rA[12+TT0I],rB[2+TT1J],rC[12+2*TT0I]); \
  TYPE_MAC(rA[13+TT0I],rB[2+TT1J],rC[13+2*TT0I]); \
  TYPE_MAC(rA[14+TT0I],rB[2+TT1J],rC[14+2*TT0I]); \
  TYPE_MAC(rA[15+TT0I],rB[2+TT1J],rC[15+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \
  TYPE_MAC(rA[8+TT0I],rB[3+TT1J],rC[8+3*TT0I]); \
  TYPE_MAC(rA[9+TT0I],rB[3+TT1J],rC[9+3*TT0I]); \
  TYPE_MAC(rA[10+TT0I],rB[3+TT1J],rC[10+3*TT0I]); \
  TYPE_MAC(rA[11+TT0I],rB[3+TT1J],rC[11+3*TT0I]); \
  TYPE_MAC(rA[12+TT0I],rB[3+TT1J],rC[12+3*TT0I]); \
  TYPE_MAC(rA[13+TT0I],rB[3+TT1J],rC[13+3*TT0I]); \
  TYPE_MAC(rA[14+TT0I],rB[3+TT1J],rC[14+3*TT0I]); \
  TYPE_MAC(rA[15+TT0I],rB[3+TT1J],rC[15+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[4+TT1J],rC[4+4*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[4+TT1J],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[4+TT1J],rC[6+4*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[4+TT1J],rC[7+4*TT0I]); \
  TYPE_MAC(rA[8+TT0I],rB[4+TT1J],rC[8+4*TT0I]); \
  TYPE_MAC(rA[9+TT0I],rB[4+TT1J],rC[9+4*TT0I]); \
  TYPE_MAC(rA[10+TT0I],rB[4+TT1J],rC[10+4*TT0I]); \
  TYPE_MAC(rA[11+TT0I],rB[4+TT1J],rC[11+4*TT0I]); \
  TYPE_MAC(rA[12+TT0I],rB[4+TT1J],rC[12+4*TT0I]); \
  TYPE_MAC(rA[13+TT0I],rB[4+TT1J],rC[13+4*TT0I]); \
  TYPE_MAC(rA[14+TT0I],rB[4+TT1J],rC[14+4*TT0I]); \
  TYPE_MAC(rA[15+TT0I],rB[4+TT1J],rC[15+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[5+TT1J],rC[4+5*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[5+TT1J],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[5+TT1J],rC[6+5*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[5+TT1J],rC[7+5*TT0I]); \
  TYPE_MAC(rA[8+TT0I],rB[5+TT1J],rC[8+5*TT0I]); \
  TYPE_MAC(rA[9+TT0I],rB[5+TT1J],rC[9+5*TT0I]); \
  TYPE_MAC(rA[10+TT0I],rB[5+TT1J],rC[10+5*TT0I]); \
  TYPE_MAC(rA[11+TT0I],rB[5+TT1J],rC[11+5*TT0I]); \
  TYPE_MAC(rA[12+TT0I],rB[5+TT1J],rC[12+5*TT0I]); \
  TYPE_MAC(rA[13+TT0I],rB[5+TT1J],rC[13+5*TT0I]); \
  TYPE_MAC(rA[14+TT0I],rB[5+TT1J],rC[14+5*TT0I]); \
  TYPE_MAC(rA[15+TT0I],rB[5+TT1J],rC[15+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[6+TT1J],rC[4+6*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[6+TT1J],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[6+TT1J],rC[6+6*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[6+TT1J],rC[7+6*TT0I]); \
  TYPE_MAC(rA[8+TT0I],rB[6+TT1J],rC[8+6*TT0I]); \
  TYPE_MAC(rA[9+TT0I],rB[6+TT1J],rC[9+6*TT0I]); \
  TYPE_MAC(rA[10+TT0I],rB[6+TT1J],rC[10+6*TT0I]); \
  TYPE_MAC(rA[11+TT0I],rB[6+TT1J],rC[11+6*TT0I]); \
  TYPE_MAC(rA[12+TT0I],rB[6+TT1J],rC[12+6*TT0I]); \
  TYPE_MAC(rA[13+TT0I],rB[6+TT1J],rC[13+6*TT0I]); \
  TYPE_MAC(rA[14+TT0I],rB[6+TT1J],rC[14+6*TT0I]); \
  TYPE_MAC(rA[15+TT0I],rB[6+TT1J],rC[15+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[7+TT1J],rC[4+7*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[7+TT1J],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[7+TT1J],rC[6+7*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[7+TT1J],rC[7+7*TT0I]); \
  TYPE_MAC(rA[8+TT0I],rB[7+TT1J],rC[8+7*TT0I]); \
  TYPE_MAC(rA[9+TT0I],rB[7+TT1J],rC[9+7*TT0I]); \
  TYPE_MAC(rA[10+TT0I],rB[7+TT1J],rC[10+7*TT0I]); \
  TYPE_MAC(rA[11+TT0I],rB[7+TT1J],rC[11+7*TT0I]); \
  TYPE_MAC(rA[12+TT0I],rB[7+TT1J],rC[12+7*TT0I]); \
  TYPE_MAC(rA[13+TT0I],rB[7+TT1J],rC[13+7*TT0I]); \
  TYPE_MAC(rA[14+TT0I],rB[7+TT1J],rC[14+7*TT0I]); \
  TYPE_MAC(rA[15+TT0I],rB[7+TT1J],rC[15+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT256x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT16_8_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;
  unsigned int flattenedOffsetA_4_0 = globalReadOffsetA0I + 0 + 4*LSPA;
  flattenedOffsetA_4_0 = (flattenedOffsetA_4_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_4_0;
  unsigned int globalReadOffsetA0I_4_0 = flattenedOffsetA_4_0;
  unsigned int flattenedOffsetA_5_0 = globalReadOffsetA0I + 0 + 5*LSPA;
  flattenedOffsetA_5_0 = (flattenedOffsetA_5_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_5_0;
  unsigned int globalReadOffsetA0I_5_0 = flattenedOffsetA_5_0;
  unsigned int flattenedOffsetA_6_0 = globalReadOffsetA0I + 0 + 6*LSPA;
  flattenedOffsetA_6_0 = (flattenedOffsetA_6_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_6_0;
  unsigned int globalReadOffsetA0I_6_0 = flattenedOffsetA_6_0;
  unsigned int flattenedOffsetA_7_0 = globalReadOffsetA0I + 0 + 7*LSPA;
  flattenedOffsetA_7_0 = (flattenedOffsetA_7_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_7_0;
  unsigned int globalReadOffsetA0I_7_0 = flattenedOffsetA_7_0;
  unsigned int flattenedOffsetA_8_0 = globalReadOffsetA0I + 0 + 8*LSPA;
  flattenedOffsetA_8_0 = (flattenedOffsetA_8_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_8_0;
  unsigned int globalReadOffsetA0I_8_0 = flattenedOffsetA_8_0;
  unsigned int flattenedOffsetA_9_0 = globalReadOffsetA0I + 0 + 9*LSPA;
  flattenedOffsetA_9_0 = (flattenedOffsetA_9_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_9_0;
  unsigned int globalReadOffsetA0I_9_0 = flattenedOffsetA_9_0;
  unsigned int flattenedOffsetA_10_0 = globalReadOffsetA0I + 0 + 10*LSPA;
  flattenedOffsetA_10_0 = (flattenedOffsetA_10_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_10_0;
  unsigned int globalReadOffsetA0I_10_0 = flattenedOffsetA_10_0;
  unsigned int flattenedOffsetA_11_0 = globalReadOffsetA0I + 0 + 11*LSPA;
  flattenedOffsetA_11_0 = (flattenedOffsetA_11_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_11_0;
  unsigned int globalReadOffsetA0I_11_0 = flattenedOffsetA_11_0;
  unsigned int flattenedOffsetA_12_0 = globalReadOffsetA0I + 0 + 12*LSPA;
  flattenedOffsetA_12_0 = (flattenedOffsetA_12_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_12_0;
  unsigned int globalReadOffsetA0I_12_0 = flattenedOffsetA_12_0;
  unsigned int flattenedOffsetA_13_0 = globalReadOffsetA0I + 0 + 13*LSPA;
  flattenedOffsetA_13_0 = (flattenedOffsetA_13_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_13_0;
  unsigned int globalReadOffsetA0I_13_0 = flattenedOffsetA_13_0;
  unsigned int flattenedOffsetA_14_0 = globalReadOffsetA0I + 0 + 14*LSPA;
  flattenedOffsetA_14_0 = (flattenedOffsetA_14_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_14_0;
  unsigned int globalReadOffsetA0I_14_0 = flattenedOffsetA_14_0;
  unsigned int flattenedOffsetA_15_0 = globalReadOffsetA0I + 0 + 15*LSPA;
  flattenedOffsetA_15_0 = (flattenedOffsetA_15_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_15_0;
  unsigned int globalReadOffsetA0I_15_0 = flattenedOffsetA_15_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;
  unsigned int flattenedOffsetB_4_0 = globalReadOffsetB1J + 0 + 4*LSPB;
  flattenedOffsetB_4_0 = (flattenedOffsetB_4_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_4_0;
  unsigned int globalReadOffsetB1J_4_0 = flattenedOffsetB_4_0;
  unsigned int flattenedOffsetB_5_0 = globalReadOffsetB1J + 0 + 5*LSPB;
  flattenedOffsetB_5_0 = (flattenedOffsetB_5_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_5_0;
  unsigned int globalReadOffsetB1J_5_0 = flattenedOffsetB_5_0;
  unsigned int flattenedOffsetB_6_0 = globalReadOffsetB1J + 0 + 6*LSPB;
  flattenedOffsetB_6_0 = (flattenedOffsetB_6_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_6_0;
  unsigned int globalReadOffsetB1J_6_0 = flattenedOffsetB_6_0;
  unsigned int flattenedOffsetB_7_0 = globalReadOffsetB1J + 0 + 7*LSPB;
  flattenedOffsetB_7_0 = (flattenedOffsetB_7_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_7_0;
  unsigned int globalReadOffsetB1J_7_0 = flattenedOffsetB_7_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;
  globalReadOffsetA0I_4_0 = (  globalReadOffsetA0I_4_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_4_0;
  globalReadOffsetA0I_5_0 = (  globalReadOffsetA0I_5_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_5_0;
  globalReadOffsetA0I_6_0 = (  globalReadOffsetA0I_6_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_6_0;
  globalReadOffsetA0I_7_0 = (  globalReadOffsetA0I_7_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_7_0;
  globalReadOffsetA0I_8_0 = (  globalReadOffsetA0I_8_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_8_0;
  globalReadOffsetA0I_9_0 = (  globalReadOffsetA0I_9_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_9_0;
  globalReadOffsetA0I_10_0 = (  globalReadOffsetA0I_10_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_10_0;
  globalReadOffsetA0I_11_0 = (  globalReadOffsetA0I_11_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_11_0;
  globalReadOffsetA0I_12_0 = (  globalReadOffsetA0I_12_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_12_0;
  globalReadOffsetA0I_13_0 = (  globalReadOffsetA0I_13_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_13_0;
  globalReadOffsetA0I_14_0 = (  globalReadOffsetA0I_14_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_14_0;
  globalReadOffsetA0I_15_0 = (  globalReadOffsetA0I_15_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_15_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;
  globalReadOffsetB1J_4_0 = (  globalReadOffsetB1J_4_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_4_0;
  globalReadOffsetB1J_5_0 = (  globalReadOffsetB1J_5_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_5_0;
  globalReadOffsetB1J_6_0 = (  globalReadOffsetB1J_6_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_6_0;
  globalReadOffsetB1J_7_0 = (  globalReadOffsetB1J_7_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_7_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_15_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_7_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 4*LSPA);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 5*LSPA);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 6*LSPA);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 7*LSPA);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 8*LSPA);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 9*LSPA);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 10*LSPA);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 11*LSPA);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 12*LSPA);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 13*LSPA);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 14*LSPA);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 15*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 4*LSPB);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 5*LSPB);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 6*LSPB);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 7*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_0_0_15_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;
  rC[64] = SCALAR_ZERO;
  rC[65] = SCALAR_ZERO;
  rC[66] = SCALAR_ZERO;
  rC[67] = SCALAR_ZERO;
  rC[68] = SCALAR_ZERO;
  rC[69] = SCALAR_ZERO;
  rC[70] = SCALAR_ZERO;
  rC[71] = SCALAR_ZERO;
  rC[72] = SCALAR_ZERO;
  rC[73] = SCALAR_ZERO;
  rC[74] = SCALAR_ZERO;
  rC[75] = SCALAR_ZERO;
  rC[76] = SCALAR_ZERO;
  rC[77] = SCALAR_ZERO;
  rC[78] = SCALAR_ZERO;
  rC[79] = SCALAR_ZERO;
  rC[80] = SCALAR_ZERO;
  rC[81] = SCALAR_ZERO;
  rC[82] = SCALAR_ZERO;
  rC[83] = SCALAR_ZERO;
  rC[84] = SCALAR_ZERO;
  rC[85] = SCALAR_ZERO;
  rC[86] = SCALAR_ZERO;
  rC[87] = SCALAR_ZERO;
  rC[88] = SCALAR_ZERO;
  rC[89] = SCALAR_ZERO;
  rC[90] = SCALAR_ZERO;
  rC[91] = SCALAR_ZERO;
  rC[92] = SCALAR_ZERO;
  rC[93] = SCALAR_ZERO;
  rC[94] = SCALAR_ZERO;
  rC[95] = SCALAR_ZERO;
  rC[96] = SCALAR_ZERO;
  rC[97] = SCALAR_ZERO;
  rC[98] = SCALAR_ZERO;
  rC[99] = SCALAR_ZERO;
  rC[100] = SCALAR_ZERO;
  rC[101] = SCALAR_ZERO;
  rC[102] = SCALAR_ZERO;
  rC[103] = SCALAR_ZERO;
  rC[104] = SCALAR_ZERO;
  rC[105] = SCALAR_ZERO;
  rC[106] = SCALAR_ZERO;
  rC[107] = SCALAR_ZERO;
  rC[108] = SCALAR_ZERO;
  rC[109] = SCALAR_ZERO;
  rC[110] = SCALAR_ZERO;
  rC[111] = SCALAR_ZERO;
  rC[112] = SCALAR_ZERO;
  rC[113] = SCALAR_ZERO;
  rC[114] = SCALAR_ZERO;
  rC[115] = SCALAR_ZERO;
  rC[116] = SCALAR_ZERO;
  rC[117] = SCALAR_ZERO;
  rC[118] = SCALAR_ZERO;
  rC[119] = SCALAR_ZERO;
  rC[120] = SCALAR_ZERO;
  rC[121] = SCALAR_ZERO;
  rC[122] = SCALAR_ZERO;
  rC[123] = SCALAR_ZERO;
  rC[124] = SCALAR_ZERO;
  rC[125] = SCALAR_ZERO;
  rC[126] = SCALAR_ZERO;
  rC[127] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_8_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_9_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_10_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_11_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_12_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_13_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_14_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_15_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_8_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_9_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_10_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_11_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_12_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_13_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_14_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_15_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0+TT0I] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0+TT0I] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0+TT0I] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0+TT0I] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0+TT0I] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0+TT0I] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0+TT0I] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0+TT0I] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0+TT0I] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0+TT0I] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0+TT0I] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0+TT0I] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0+TT0I] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0+TT0I] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0+TT0I] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0+TT0I] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0+TT0I] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0+TT0I] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0+TT0I] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0+TT0I] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0+TT0I] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0+TT0I] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0+TT0I] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0+TT0I] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0+TT0I] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0+TT0I] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0+TT0I] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0+TT0I] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0+TT0I] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0+TT0I] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0+TT0I] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0+TT0I] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0+TT0I] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0+TT0I] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0+TT0I] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0+TT0I] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0+TT0I] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0+TT0I] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0+TT0I] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0+TT0I] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0+TT0I] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0+TT0I] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0+TT0I] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0+TT0I] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0+TT0I] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0+TT0I] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0+TT0I] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0+TT0I] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0+TT0I] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0+TT0I] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0+TT0I] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0+TT0I] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0+TT0I] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0+TT0I] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0+TT0I] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0+TT0I] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0+TT0I] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0+TT0I] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0+TT0I] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0+TT0I] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0+TT0I] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0+TT0I] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0+TT0I] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0+TT0I] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_16x8


    /* iter 15 */

    MAC_16x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_0_8_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_0_0_9_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_0_0_10_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_0_0_11_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_0_0_12_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_0_0_13_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_0_0_14_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_0_0_15_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_16x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  8*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 8*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[8*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  9*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 9*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[9*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  10*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 10*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[10*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  11*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 11*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[11*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  12*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 12*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[12*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  13*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 13*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[13*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  14*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 14*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[14*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  15*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 15*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[15*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  8*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 8*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[8*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  9*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 9*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[9*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  10*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 10*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[10*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  11*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 11*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[11*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  12*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 12*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[12*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  13*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 13*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[13*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  14*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 14*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[14*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  15*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 15*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[15*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  8*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 8*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[8*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  9*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 9*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[9*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  10*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 10*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[10*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  11*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 11*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[11*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  12*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 12*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[12*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  13*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 13*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[13*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  14*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 14*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[14*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  15*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 15*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[15*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  8*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 8*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[8*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  9*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 9*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[9*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  10*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 10*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[10*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  11*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 11*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[11*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  12*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 12*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[12*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  13*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 13*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[13*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  14*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 14*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[14*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  15*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 15*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[15*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  8*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 8*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[8*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  9*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 9*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[9*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  10*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 10*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[10*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  11*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 11*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[11*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  12*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 12*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[12*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  13*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 13*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[13*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  14*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 14*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[14*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  15*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 15*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[15*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  8*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 8*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[8*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  9*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 9*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[9*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  10*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 10*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[10*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  11*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 11*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[11*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  12*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 12*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[12*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  13*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 13*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[13*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  14*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 14*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[14*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  15*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 15*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[15*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  8*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 8*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[8*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  9*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 9*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[9*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  10*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 10*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[10*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  11*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 11*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[11*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  12*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 12*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[12*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  13*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 13*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[13*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  14*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 14*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[14*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  15*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 15*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[15*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  8*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 8*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[8*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  9*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 9*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[9*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  10*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 10*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[10*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  11*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 11*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[11*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  12*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 12*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[12*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  13*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 13*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[13*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  14*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 14*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[14*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  15*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 15*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[15*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_16x8
#undef MAC_16x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 8

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 3072

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float

  /* atomic add float */
#ifndef ATOMIC_FLOAT_FUNCTION
#define ATOMIC_FLOAT_FUNCTION

__device__ inline int atomicAddType(int *fPtr, int operand)
{
  return atomicAdd(fPtr,operand);
}

__device__ inline unsigned int atomicAddType(unsigned int *fPtr, unsigned int operand)
{
  return atomicAdd(fPtr,operand);
}

__device__ inline unsigned long long int atomicAddType(unsigned long long int *fPtr, unsigned long long int operand)
{
  return atomicAdd(fPtr,operand);
}

__device__ inline float atomicAddType(float *fPtr, float operand)
{
  return atomicAdd(fPtr,operand);
}

__device__ inline double atomicAddType(double *fPtr, double operand)
{
  return atomicAdd(fPtr,operand);
}

#endif
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) atomicAddType(&(DST), (ALPHA)*(REG));
/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT64x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;
  unsigned int flattenedOffsetB_4_0 = globalReadOffsetB1J + 0 + 4*LSPB;
  flattenedOffsetB_4_0 = (flattenedOffsetB_4_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_4_0;
  unsigned int globalReadOffsetB1J_4_0 = flattenedOffsetB_4_0;
  unsigned int flattenedOffsetB_5_0 = globalReadOffsetB1J + 0 + 5*LSPB;
  flattenedOffsetB_5_0 = (flattenedOffsetB_5_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_5_0;
  unsigned int globalReadOffsetB1J_5_0 = flattenedOffsetB_5_0;
  unsigned int flattenedOffsetB_6_0 = globalReadOffsetB1J + 0 + 6*LSPB;
  flattenedOffsetB_6_0 = (flattenedOffsetB_6_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_6_0;
  unsigned int globalReadOffsetB1J_6_0 = flattenedOffsetB_6_0;
  unsigned int flattenedOffsetB_7_0 = globalReadOffsetB1J + 0 + 7*LSPB;
  flattenedOffsetB_7_0 = (flattenedOffsetB_7_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_7_0;
  unsigned int globalReadOffsetB1J_7_0 = flattenedOffsetB_7_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;
  globalReadOffsetB1J_4_0 = (  globalReadOffsetB1J_4_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_4_0;
  globalReadOffsetB1J_5_0 = (  globalReadOffsetB1J_5_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_5_0;
  globalReadOffsetB1J_6_0 = (  globalReadOffsetB1J_6_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_6_0;
  globalReadOffsetB1J_7_0 = (  globalReadOffsetB1J_7_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_7_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_7_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 4*LSPB);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 5*LSPB);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 6*LSPB);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 7*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 15 */

    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 8

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 3072

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT64x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;
  unsigned int flattenedOffsetB_4_0 = globalReadOffsetB1J + 0 + 4*LSPB;
  flattenedOffsetB_4_0 = (flattenedOffsetB_4_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_4_0;
  unsigned int globalReadOffsetB1J_4_0 = flattenedOffsetB_4_0;
  unsigned int flattenedOffsetB_5_0 = globalReadOffsetB1J + 0 + 5*LSPB;
  flattenedOffsetB_5_0 = (flattenedOffsetB_5_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_5_0;
  unsigned int globalReadOffsetB1J_5_0 = flattenedOffsetB_5_0;
  unsigned int flattenedOffsetB_6_0 = globalReadOffsetB1J + 0 + 6*LSPB;
  flattenedOffsetB_6_0 = (flattenedOffsetB_6_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_6_0;
  unsigned int globalReadOffsetB1J_6_0 = flattenedOffsetB_6_0;
  unsigned int flattenedOffsetB_7_0 = globalReadOffsetB1J + 0 + 7*LSPB;
  flattenedOffsetB_7_0 = (flattenedOffsetB_7_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_7_0;
  unsigned int globalReadOffsetB1J_7_0 = flattenedOffsetB_7_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;
  globalReadOffsetB1J_4_0 = (  globalReadOffsetB1J_4_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_4_0;
  globalReadOffsetB1J_5_0 = (  globalReadOffsetB1J_5_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_5_0;
  globalReadOffsetB1J_6_0 = (  globalReadOffsetB1J_6_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_6_0;
  globalReadOffsetB1J_7_0 = (  globalReadOffsetB1J_7_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_7_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_7_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 4*LSPB);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 5*LSPB);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 6*LSPB);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 7*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 15 */

    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float

  /* atomic add float */
#ifndef ATOMIC_FLOAT_FUNCTION
#define ATOMIC_FLOAT_FUNCTION

__device__ inline int atomicAddType(int *fPtr, int operand)
{
  return atomicAdd(fPtr,operand);
}

__device__ inline unsigned int atomicAddType(unsigned int *fPtr, unsigned int operand)
{
  return atomicAdd(fPtr,operand);
}

__device__ inline unsigned long long int atomicAddType(unsigned long long int *fPtr, unsigned long long int operand)
{
  return atomicAdd(fPtr,operand);
}

__device__ inline float atomicAddType(float *fPtr, float operand)
{
  return atomicAdd(fPtr,operand);
}

__device__ inline double atomicAddType(double *fPtr, double operand)
{
  return atomicAdd(fPtr,operand);
}

#endif
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) atomicAddType(&(DST), (ALPHA)*(REG));
/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT128x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;
  unsigned int flattenedOffsetA_4_0 = globalReadOffsetA0I + 0 + 4*LSPA;
  flattenedOffsetA_4_0 = (flattenedOffsetA_4_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_4_0;
  unsigned int globalReadOffsetA0I_4_0 = flattenedOffsetA_4_0;
  unsigned int flattenedOffsetA_5_0 = globalReadOffsetA0I + 0 + 5*LSPA;
  flattenedOffsetA_5_0 = (flattenedOffsetA_5_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_5_0;
  unsigned int globalReadOffsetA0I_5_0 = flattenedOffsetA_5_0;
  unsigned int flattenedOffsetA_6_0 = globalReadOffsetA0I + 0 + 6*LSPA;
  flattenedOffsetA_6_0 = (flattenedOffsetA_6_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_6_0;
  unsigned int globalReadOffsetA0I_6_0 = flattenedOffsetA_6_0;
  unsigned int flattenedOffsetA_7_0 = globalReadOffsetA0I + 0 + 7*LSPA;
  flattenedOffsetA_7_0 = (flattenedOffsetA_7_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_7_0;
  unsigned int globalReadOffsetA0I_7_0 = flattenedOffsetA_7_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;
  globalReadOffsetA0I_4_0 = (  globalReadOffsetA0I_4_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_4_0;
  globalReadOffsetA0I_5_0 = (  globalReadOffsetA0I_5_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_5_0;
  globalReadOffsetA0I_6_0 = (  globalReadOffsetA0I_6_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_6_0;
  globalReadOffsetA0I_7_0 = (  globalReadOffsetA0I_7_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_7_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 4*LSPA);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 5*LSPA);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 6*LSPA);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 7*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 8

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float

  /* atomic add float */
#ifndef ATOMIC_FLOAT_FUNCTION
#define ATOMIC_FLOAT_FUNCTION

__device__ inline int atomicAddType(int *fPtr, int operand)
{
  return atomicAdd(fPtr,operand);
}

__device__ inline unsigned int atomicAddType(unsigned int *fPtr, unsigned int operand)
{
  return atomicAdd(fPtr,operand);
}

__device__ inline unsigned long long int atomicAddType(unsigned long long int *fPtr, unsigned long long int operand)
{
  return atomicAdd(fPtr,operand);
}

__device__ inline float atomicAddType(float *fPtr, float operand)
{
  return atomicAdd(fPtr,operand);
}

__device__ inline double atomicAddType(double *fPtr, double operand)
{
  return atomicAdd(fPtr,operand);
}

#endif
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) atomicAddType(&(DST), (ALPHA)*(REG));
/* 2x8 micro-tile */
#define MAC_2x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \

#define MAC_2x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_SB_MT32x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_8_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;
  unsigned int flattenedOffsetB_4_0 = globalReadOffsetB1J + 0 + 4*LSPB;
  flattenedOffsetB_4_0 = (flattenedOffsetB_4_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_4_0;
  unsigned int globalReadOffsetB1J_4_0 = flattenedOffsetB_4_0;
  unsigned int flattenedOffsetB_5_0 = globalReadOffsetB1J + 0 + 5*LSPB;
  flattenedOffsetB_5_0 = (flattenedOffsetB_5_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_5_0;
  unsigned int globalReadOffsetB1J_5_0 = flattenedOffsetB_5_0;
  unsigned int flattenedOffsetB_6_0 = globalReadOffsetB1J + 0 + 6*LSPB;
  flattenedOffsetB_6_0 = (flattenedOffsetB_6_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_6_0;
  unsigned int globalReadOffsetB1J_6_0 = flattenedOffsetB_6_0;
  unsigned int flattenedOffsetB_7_0 = globalReadOffsetB1J + 0 + 7*LSPB;
  flattenedOffsetB_7_0 = (flattenedOffsetB_7_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_7_0;
  unsigned int globalReadOffsetB1J_7_0 = flattenedOffsetB_7_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;
  globalReadOffsetB1J_4_0 = (  globalReadOffsetB1J_4_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_4_0;
  globalReadOffsetB1J_5_0 = (  globalReadOffsetB1J_5_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_5_0;
  globalReadOffsetB1J_6_0 = (  globalReadOffsetB1J_6_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_6_0;
  globalReadOffsetB1J_7_0 = (  globalReadOffsetB1J_7_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_7_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_7_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 4*LSPB);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 5*LSPB);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 6*LSPB);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 7*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x8


    /* iter 15 */

    MAC_2x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x8
#undef MAC_2x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE TensileComplexDouble
#define DEST_DATA_TYPE TensileComplexDouble
#define COMPUTE_DATA_TYPE TensileComplexDouble
#define s0 x
#define s1 y
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) \
  DST.s0 = MAC(  MULA.s0, MULB.s0, DST.s0 ); \
  DST.s0 = MAC( -MULA.s1, MULB.s1, DST.s0 ); \
  DST.s1 = MAC(  MULA.s0, MULB.s1, DST.s1 ); \
  DST.s1 = MAC(  MULA.s1, MULB.s0, DST.s1 );
#define TYPE_MAC_WRITE( DST, SRC, ALPHA, REG, BETA ) \
  /* (1) */ \
  type_mac_tmp = REG.s0; \
  REG.s0 *= ALPHA.s0; \
  REG.s0 = MAC( -ALPHA.s1, REG.s1, REG.s0 ); \
  REG.s1 *= ALPHA.s0; \
  REG.s1 = MAC(  ALPHA.s1, type_mac_tmp, REG.s1 ); \
  /* (2) */ \
  if(BETA.s0 != 0) { \
  REG.s0 = MAC(  BETA.s0, SRC.s0, REG.s0 ); \
  REG.s1 = MAC(  BETA.s0, SRC.s1, REG.s1 ); \
  } \
  if (BETA.s1 != 0) { \
  REG.s0 = MAC( -BETA.s1, SRC.s1, REG.s0 ); \
  REG.s1 = MAC(  BETA.s1, SRC.s0, REG.s1 ); \
  } \
  /* (3) */ \
  DST = REG;

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_ZB_MT32x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW1_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  TensileComplexDouble *D,
  TensileComplexDouble const * __restrict__ C,
  TensileComplexDouble const * __restrict__ A,
  TensileComplexDouble const * __restrict__ B,
  TensileComplexDouble const alpha,
  TensileComplexDouble const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (TensileComplexDouble)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == tensile_complex<double>(0.0)) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 1); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }



  double type_mac_tmp;


  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE TensileComplexDouble
#define DEST_DATA_TYPE TensileComplexDouble
#define COMPUTE_DATA_TYPE TensileComplexDouble
#define s0 x
#define s1 y
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) \
  DST.s0 = MAC(  MULA.s0, MULB.s0, DST.s0 ); \
  DST.s0 = MAC( -MULA.s1, MULB.s1, DST.s0 ); \
  DST.s1 = MAC(  MULA.s0, MULB.s1, DST.s1 ); \
  DST.s1 = MAC(  MULA.s1, MULB.s0, DST.s1 );
#define TYPE_MAC_WRITE( DST, SRC, ALPHA, REG, BETA ) \
  /* (1) */ \
  type_mac_tmp = REG.s0; \
  REG.s0 *= ALPHA.s0; \
  REG.s0 = MAC( -ALPHA.s1, REG.s1, REG.s0 ); \
  REG.s1 *= ALPHA.s0; \
  REG.s1 = MAC(  ALPHA.s1, type_mac_tmp, REG.s1 ); \
  /* (2) */ \
  if(BETA.s0 != 0) { \
  REG.s0 = MAC(  BETA.s0, SRC.s0, REG.s0 ); \
  REG.s1 = MAC(  BETA.s0, SRC.s1, REG.s1 ); \
  } \
  if (BETA.s1 != 0) { \
  REG.s0 = MAC( -BETA.s1, SRC.s1, REG.s0 ); \
  REG.s1 = MAC(  BETA.s1, SRC.s0, REG.s1 ); \
  } \
  /* (3) */ \
  DST = REG;

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_ZB_MT32x32x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW1_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  TensileComplexDouble *D,
  TensileComplexDouble const * __restrict__ C,
  TensileComplexDouble const * __restrict__ A,
  TensileComplexDouble const * __restrict__ B,
  TensileComplexDouble const alpha,
  TensileComplexDouble const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (TensileComplexDouble)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == tensile_complex<double>(0.0)) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 1); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }



  double type_mac_tmp;


  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 32
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 4096
#define LDS_NUM_ELEMENTS 6144

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x4 micro-tile */
#define MAC_8x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7],rB[3],rC[7+3*TT0I]); \

#define MAC_8x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT128x64x32_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_2_1_0;
  DATA_TYPE a_0_3_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_2_2_0;
  DATA_TYPE a_0_3_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE a_0_2_3_0;
  DATA_TYPE a_0_3_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_2_1_0;
  DATA_TYPE b_0_3_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;
  DATA_TYPE *localWriteB_0_0_1_2;
  DATA_TYPE *localWriteB_0_0_1_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
  localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 1); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_2_2_0 = *(globalReadA_0_0_2_0 + 2);
    a_0_3_2_0 = *(globalReadA_0_0_2_0 + 3);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);
    a_0_2_3_0 = *(globalReadA_0_0_3_0 + 2);
    a_0_3_3_0 = *(globalReadA_0_0_3_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
    *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_2_0 + 2) = a_0_2_2_0;
    *(localWriteA_0_0_2_0 + 3) = a_0_3_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
    *(localWriteA_0_0_3_0 + 2) = a_0_2_3_0;
    *(localWriteA_0_0_3_0 + 3) = a_0_3_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
    *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 15 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 16 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 17 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 18 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 19 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 20 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 21 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 22 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 23 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 24 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 25 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 26 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 27 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 28 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 29 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 30 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x4


    /* iter 31 */

    MAC_8x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_2_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 2);
  a_0_3_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 3);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_2_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 2);
  a_0_3_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 3);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);
  a_0_2_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 2);
  a_0_3_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_2_1_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 2);
  b_0_3_1_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
  localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
  *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
  *(localWriteA_0_0_2_0 + 2) = a_0_2_2_0;
  *(localWriteA_0_0_2_0 + 3) = a_0_3_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
  *(localWriteA_0_0_3_0 + 2) = a_0_2_3_0;
  *(localWriteA_0_0_3_0 + 3) = a_0_3_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
  *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
  *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x4

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 2) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 3) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x4
#undef MAC_8x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 32
#define SG1J 4
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 3200
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT64x8x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG32_4_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 15 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I]); \
  TYPE_MAC(rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I]); \
  TYPE_MAC(rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I]); \
  TYPE_MAC(rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I]); \
  TYPE_MAC(rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I]); \
  TYPE_MAC(rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I]); \
  TYPE_MAC(rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I]); \
  TYPE_MAC(rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I]); \
  TYPE_MAC(rA[7],rB[7],rC[7+7*TT0I]); \

#define MAC_8x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[4+TT1J],rC[4+4*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[4+TT1J],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[4+TT1J],rC[6+4*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[4+TT1J],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[5+TT1J],rC[4+5*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[5+TT1J],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[5+TT1J],rC[6+5*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[5+TT1J],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[6+TT1J],rC[4+6*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[6+TT1J],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[6+TT1J],rC[6+6*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[6+TT1J],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[7+TT1J],rC[4+7*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[7+TT1J],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[7+TT1J],rC[6+7*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[7+TT1J],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT128x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8


    /* iter 7 */

    MAC_8x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
    if (r0I == 2) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
    if (r0I == 3) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef MAC_8x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT32x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 7 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_3 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 2) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 3) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 7168
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x4 micro-tile */
#define MAC_8x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7],rB[3],rC[7+3*TT0I]); \

#define MAC_8x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT128x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_2_1_0;
  DATA_TYPE a_0_3_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
    *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
    *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_8x4


    /* iter 15 (last unrolled loop) */

    MAC_8x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_3 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_2_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 2);
  a_0_3_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
  *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 2) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 3) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x4
#undef MAC_8x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 128
#define LDS_NUM_ELEMENTS 512
#define LDS_OFFSET_BLK 256

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT16x16x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 32
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT64x64x32_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_2_1_0;
  DATA_TYPE a_0_3_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_2_1_0;
  DATA_TYPE b_0_3_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;
  DATA_TYPE *localWriteB_0_0_1_2;
  DATA_TYPE *localWriteB_0_0_1_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
  localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 1); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
    *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
    *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 16 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 17 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 18 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 19 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 20 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 21 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 22 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 23 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 24 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 25 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 26 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 27 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 28 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 29 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 30 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 31 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_2_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 2);
  a_0_3_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_2_1_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 2);
  b_0_3_1_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
  localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
  *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
  *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
  *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 2) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 3) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 16
#define SG1J 4
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 832
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT32x8x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_4_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 6
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 3
#define NLCB 1
#define NLPA 1
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1536
#define LDS_NUM_ELEMENTS 3584

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 6x8 micro-tile */
#define MAC_6x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I]); \
  TYPE_MAC(rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I]); \
  TYPE_MAC(rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I]); \
  TYPE_MAC(rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I]); \
  TYPE_MAC(rA[5],rB[7],rC[5+7*TT0I]); \

#define MAC_6x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[4+TT1J],rC[4+4*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[4+TT1J],rC[5+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[5+TT1J],rC[4+5*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[5+TT1J],rC[5+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[6+TT1J],rC[4+6*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[6+TT1J],rC[5+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[7+TT1J],rC[4+7*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[7+TT1J],rC[5+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT96x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA3_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT6_8_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_1_0_0_0;
  DATA_TYPE a_1_1_0_0;
  DATA_TYPE a_2_0_0_0;
  DATA_TYPE a_2_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_1_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_1_3_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSCA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSCA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_2_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_1_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_1_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_2_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_2_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_1_0_0_0 = A + globalReadOffsetA_1_0_0_0;
  DATA_TYPE const *globalReadA_2_0_0_0 = A + globalReadOffsetA_2_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_1_0_0_0 = localWriteFirstOffsetA + (0 + 1*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_2_0_0_0 = localWriteFirstOffsetA + (0 + 2*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_2_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_3_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_1_0_0_0;
  DATA_TYPE *localWriteA_2_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_2_1;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_3_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_1_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_0_0);
  localWriteA_2_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_2_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_1);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_1_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_2_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_1_0_0_0 = *(globalReadA_1_0_0_0 + 0);
    a_1_1_0_0 = *(globalReadA_1_0_0_0 + 1);
    a_2_0_0_0 = *(globalReadA_2_0_0_0 + 0);
    a_2_1_0_0 = *(globalReadA_2_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_1_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_1_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_1_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_2_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_2_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_2_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_1_0_0_0 + 0) = a_1_0_0_0;
    *(localWriteA_1_0_0_0 + 1) = a_1_1_0_0;
    *(localWriteA_2_0_0_0 + 0) = a_2_0_0_0;
    *(localWriteA_2_0_0_0 + 1) = a_2_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_1 + 0) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_1 + 0) = b_0_1_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_6x8


    /* iter 15 */

    MAC_6x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_1_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_2_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_1_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_1_0_0_0 + 0);
  a_1_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_1_0_0_0 + 1);
  a_2_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_2_0_0_0 + 0);
  a_2_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_2_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_1_2_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 1);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_1_3_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_1_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_0_0);
  localWriteA_2_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_2_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_1);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_1_0_0_0 + 0) = a_1_0_0_0;
  *(localWriteA_1_0_0_0 + 1) = a_1_1_0_0;
  *(localWriteA_2_0_0_0 + 0) = a_2_0_0_0;
  *(localWriteA_2_0_0_0 + 1) = a_2_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_2_1 + 0) = b_0_1_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_3_1 + 0) = b_0_1_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_6x8

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else if (s0I == 1) {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else {
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_6x8
#undef MAC_6x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 32
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 6144

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT64x128x32_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_2_1_0;
  DATA_TYPE a_0_3_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_2_1_0;
  DATA_TYPE b_0_3_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_1_2_0;
  DATA_TYPE b_0_2_2_0;
  DATA_TYPE b_0_3_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_1_3_0;
  DATA_TYPE b_0_2_3_0;
  DATA_TYPE b_0_3_3_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_2_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_2_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_2_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_3_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_3_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_3_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;
  DATA_TYPE *localWriteB_0_0_1_2;
  DATA_TYPE *localWriteB_0_0_1_3;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_2_1;
  DATA_TYPE *localWriteB_0_0_2_2;
  DATA_TYPE *localWriteB_0_0_2_3;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_3_1;
  DATA_TYPE *localWriteB_0_0_3_2;
  DATA_TYPE *localWriteB_0_0_3_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
  localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_1);
  localWriteB_0_0_2_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_2);
  localWriteB_0_0_2_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_3);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_1);
  localWriteB_0_0_3_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_2);
  localWriteB_0_0_3_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_3);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 1); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_2_2_0 = *(globalReadB_0_0_2_0 + 2);
    b_0_3_2_0 = *(globalReadB_0_0_2_0 + 3);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);
    b_0_2_3_0 = *(globalReadB_0_0_3_0 + 2);
    b_0_3_3_0 = *(globalReadB_0_0_3_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
    *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
    *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_1 + 0) = b_0_1_2_0;
    *(localWriteB_0_0_2_2 + 0) = b_0_2_2_0;
    *(localWriteB_0_0_2_3 + 0) = b_0_3_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_1 + 0) = b_0_1_3_0;
    *(localWriteB_0_0_3_2 + 0) = b_0_2_3_0;
    *(localWriteB_0_0_3_3 + 0) = b_0_3_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 15 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 16 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 17 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 18 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 19 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 20 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 21 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 22 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 23 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 24 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 25 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 26 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 27 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 28 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 29 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 30 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 31 */

    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_2_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 2);
  a_0_3_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_2_1_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 2);
  b_0_3_1_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 3);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_1_2_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 1);
  b_0_2_2_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 2);
  b_0_3_2_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 3);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_1_3_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 1);
  b_0_2_3_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 2);
  b_0_3_3_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
  localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_1);
  localWriteB_0_0_2_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_2);
  localWriteB_0_0_2_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_3);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_1);
  localWriteB_0_0_3_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_2);
  localWriteB_0_0_3_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
  *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
  *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
  *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_2_1 + 0) = b_0_1_2_0;
  *(localWriteB_0_0_2_2 + 0) = b_0_2_2_0;
  *(localWriteB_0_0_2_3 + 0) = b_0_3_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_3_1 + 0) = b_0_1_3_0;
  *(localWriteB_0_0_3_2 + 0) = b_0_2_3_0;
  *(localWriteB_0_0_3_3 + 0) = b_0_3_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
    if (r0I == 2) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
    if (r0I == 3) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 3072

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x4 micro-tile */
#define MAC_8x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7],rB[3],rC[7+3*TT0I]); \

#define MAC_8x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT128x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_2_1_0;
  DATA_TYPE a_0_3_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
    *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x4


    /* iter 15 */

    MAC_8x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_2_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 2);
  a_0_3_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
  *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x4

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 2) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 3) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x4
#undef MAC_8x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 32
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT64x64x32_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_2_1_0;
  DATA_TYPE a_0_3_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_2_1_0;
  DATA_TYPE b_0_3_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;
  DATA_TYPE *localWriteB_0_0_1_2;
  DATA_TYPE *localWriteB_0_0_1_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
  localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 1); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
    *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
    *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 16 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 17 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 18 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 19 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 20 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 21 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 22 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 23 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 24 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 25 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 26 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 27 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 28 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 29 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 30 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 31 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_2_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 2);
  a_0_3_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_2_1_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 2);
  b_0_3_1_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
  localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
  *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
  *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
  *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 2) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 3) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 7168
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x4 micro-tile */
#define MAC_8x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7],rB[3],rC[7+3*TT0I]); \

#define MAC_8x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT128x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_2_1_0;
  DATA_TYPE a_0_3_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
    *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
    *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_8x4


    /* iter 15 (last unrolled loop) */

    MAC_8x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_3 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_2_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 2);
  a_0_3_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
  *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 2) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 3) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x4
#undef MAC_8x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1792
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT32x16x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x2


    /* iter 15 (last unrolled loop) */

    MAC_4x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 24
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 3
#define NLPA 6
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 768
#define LDS_NUM_ELEMENTS 3200
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT32x16x24_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB3_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_1_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_1_5_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_1_0_0_0;
  DATA_TYPE b_1_1_0_0;
  DATA_TYPE b_2_0_0_0;
  DATA_TYPE b_2_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSCB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_1_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_1_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_2_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_2_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_1_0_0_0 = B + globalReadOffsetB_1_0_0_0;
  DATA_TYPE const *globalReadB_2_0_0_0 = B + globalReadOffsetB_2_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_1_0_0_0 = localWriteFirstOffsetB + (0 + 1*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_2_0_0_0 = localWriteFirstOffsetB + (0 + 2*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_1_0_0_1 = localWriteFirstOffsetB + (1 + 1*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_2_0_0_1 = localWriteFirstOffsetB + (1 + 2*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_1_0_0_0;
  DATA_TYPE *localWriteB_2_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_1_0_0_1;
  DATA_TYPE *localWriteB_2_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_1_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_1_0_0_0);
  localWriteB_2_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_2_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_1_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_1_0_0_1);
  localWriteB_2_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_2_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 384-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_1_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_2_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_1_4_0 = *(globalReadA_0_0_4_0 + 1);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_1_5_0 = *(globalReadA_0_0_5_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_1_0_0_0 = *(globalReadB_1_0_0_0 + 0);
    b_1_1_0_0 = *(globalReadB_1_0_0_0 + 1);
    b_2_0_0_0 = *(globalReadB_2_0_0_0 + 0);
    b_2_1_0_0 = *(globalReadB_2_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_1_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_1_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_1_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_2_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_2_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_2_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_4_0 + 1) = a_0_1_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_5_0 + 1) = a_0_1_5_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_1_0_0_0 + 0) = b_1_0_0_0;
    *(localWriteB_2_0_0_0 + 0) = b_2_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_1_0_0_1 + 0) = b_1_1_0_0;
    *(localWriteB_2_0_0_1 + 0) = b_2_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_4_0 = (localWriteOffsetA_0_0_4_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_5_0 = (localWriteOffsetA_0_0_5_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_1_0_0_0 = (localWriteOffsetB_1_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_2_0_0_0 = (localWriteOffsetB_2_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_1_0_0_1 = (localWriteOffsetB_1_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_2_0_0_1 = (localWriteOffsetB_2_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
    localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
    localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_1_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_1_0_0_0);
    localWriteB_2_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_2_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_1_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_1_0_0_1);
    localWriteB_2_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_2_0_0_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_1_4_0 = *(globalReadA_0_0_4_0 + 1);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_1_5_0 = *(globalReadA_0_0_5_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_1_0_0_0 = *(globalReadB_1_0_0_0 + 0);
    b_1_1_0_0 = *(globalReadB_1_0_0_0 + 1);
    b_2_0_0_0 = *(globalReadB_2_0_0_0 + 0);
    b_2_1_0_0 = *(globalReadB_2_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_1_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_1_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_1_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_2_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_2_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_2_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 15 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 16 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 17 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 18 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 19 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 20 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 21 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 22 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_4_0 + 1) = a_0_1_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_5_0 + 1) = a_0_1_5_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_1_0_0_0 + 0) = b_1_0_0_0;
    *(localWriteB_2_0_0_0 + 0) = b_2_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_1_0_0_1 + 0) = b_1_1_0_0;
    *(localWriteB_2_0_0_1 + 0) = b_2_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_4_0 = (localWriteOffsetA_0_0_4_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_5_0 = (localWriteOffsetA_0_0_5_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_1_0_0_0 = (localWriteOffsetB_1_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_2_0_0_0 = (localWriteOffsetB_2_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_1_0_0_1 = (localWriteOffsetB_1_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_2_0_0_1 = (localWriteOffsetB_2_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
    localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
    localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_1_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_1_0_0_0);
    localWriteB_2_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_2_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_1_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_1_0_0_1);
    localWriteB_2_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_2_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 23 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 16 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 17 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 18 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 19 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 20 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 21 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 22 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x2


    /* iter 23 (last unrolled loop) */

    MAC_4x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_4_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_5_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_1_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_2_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_1_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_2_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_1_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_2_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_1_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 1);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_1_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_1_0_0_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_1_0_0_0 + 0);
  b_1_1_0_0 = ( globalReadOffsetBL_1_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_1_0_0_0 + 1);
  b_2_0_0_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_2_0_0_0 + 0);
  b_2_1_0_0 = ( globalReadOffsetBL_2_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_2_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_1_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_1_0_0_0);
  localWriteB_2_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_2_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_1_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_1_0_0_1);
  localWriteB_2_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_2_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_4_0 + 1) = a_0_1_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_5_0 + 1) = a_0_1_5_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_1_0_0_0 + 0) = b_1_0_0_0;
  *(localWriteB_2_0_0_0 + 0) = b_2_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_1_0_0_1 + 0) = b_1_1_0_0;
  *(localWriteB_2_0_0_1 + 0) = b_2_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 32
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 8192
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT64x64x32_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_2_1_0;
  DATA_TYPE a_0_3_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_2_1_0;
  DATA_TYPE b_0_3_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;
  DATA_TYPE *localWriteB_0_0_1_2;
  DATA_TYPE *localWriteB_0_0_1_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
  localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 1); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
    *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
    *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_2 = (localWriteOffsetB_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_3 = (localWriteOffsetB_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
    localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
    localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 16 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 17 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 18 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 19 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 20 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 21 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 22 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 23 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 24 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 25 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 26 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 27 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 28 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 29 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 30 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
    *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
    *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_2 = (localWriteOffsetB_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_3 = (localWriteOffsetB_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
    localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
    localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 31 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 16 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 17 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 18 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 19 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 20 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 21 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 22 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 23 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 24 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 25 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 26 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 27 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 28 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 29 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 30 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 31 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_3 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_3 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_2_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 2);
  a_0_3_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_2_1_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 2);
  b_0_3_1_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
  localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
  *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
  *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
  *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 2) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 3) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 16
#define SG1J 4
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 32
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 3328
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT32x8x32_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_4_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_1_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_1_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_1_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_1_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 1); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_1_4_0 = *(globalReadA_0_0_4_0 + 1);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_1_5_0 = *(globalReadA_0_0_5_0 + 1);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_1_6_0 = *(globalReadA_0_0_6_0 + 1);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_1_7_0 = *(globalReadA_0_0_7_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_4_0 + 1) = a_0_1_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_5_0 + 1) = a_0_1_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_6_0 + 1) = a_0_1_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_7_0 + 1) = a_0_1_7_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_4_0 = (localWriteOffsetA_0_0_4_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_5_0 = (localWriteOffsetA_0_0_5_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_6_0 = (localWriteOffsetA_0_0_6_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_7_0 = (localWriteOffsetA_0_0_7_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
    localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
    localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
    localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
    localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_1_4_0 = *(globalReadA_0_0_4_0 + 1);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_1_5_0 = *(globalReadA_0_0_5_0 + 1);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_1_6_0 = *(globalReadA_0_0_6_0 + 1);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_1_7_0 = *(globalReadA_0_0_7_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 15 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 16 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 17 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 18 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 19 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 20 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 21 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 22 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 23 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 24 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 25 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 26 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 27 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 28 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 29 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 30 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_4_0 + 1) = a_0_1_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_5_0 + 1) = a_0_1_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_6_0 + 1) = a_0_1_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_7_0 + 1) = a_0_1_7_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_4_0 = (localWriteOffsetA_0_0_4_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_5_0 = (localWriteOffsetA_0_0_5_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_6_0 = (localWriteOffsetA_0_0_6_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_7_0 = (localWriteOffsetA_0_0_7_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
    localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
    localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
    localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
    localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 31 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 16 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 17 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 18 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 19 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 20 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 21 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 22 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 23 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 24 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 25 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 26 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 27 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 28 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 29 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 30 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 31 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_4_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_5_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_6_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_7_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_1_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 1);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_1_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 1);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_1_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 1);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_1_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_4_0 + 1) = a_0_1_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_5_0 + 1) = a_0_1_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_6_0 + 1) = a_0_1_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_7_0 + 1) = a_0_1_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 3584
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT64x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_2_1_0;
  DATA_TYPE a_0_3_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
    *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
    *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_3 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_2_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 2);
  a_0_3_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
  *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 2) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 3) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT32x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_2_1_0;
  DATA_TYPE a_0_3_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_2_1_0;
  DATA_TYPE b_0_3_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;
  DATA_TYPE *localWriteB_0_0_1_2;
  DATA_TYPE *localWriteB_0_0_1_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
  localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
    *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
    *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_2 = (localWriteOffsetB_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_3 = (localWriteOffsetB_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
    localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
    localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
    *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
    *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_2 = (localWriteOffsetB_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_3 = (localWriteOffsetB_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
    localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
    localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_3 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_3 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_2_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 2);
  a_0_3_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_2_1_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 2);
  b_0_3_1_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
  localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
  *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
  *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
  *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 2) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 3) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE TensileComplexDouble
#define DEST_DATA_TYPE TensileComplexDouble
#define COMPUTE_DATA_TYPE TensileComplexDouble
#define s0 x
#define s1 y
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) \
  DST.s0 = MAC(  MULA.s0, MULB.s0, DST.s0 ); \
  DST.s0 = MAC( -MULA.s1, MULB.s1, DST.s0 ); \
  DST.s1 = MAC(  MULA.s0, MULB.s1, DST.s1 ); \
  DST.s1 = MAC(  MULA.s1, MULB.s0, DST.s1 );
#define TYPE_MAC_WRITE( DST, SRC, ALPHA, REG, BETA ) \
  /* (1) */ \
  type_mac_tmp = REG.s0; \
  REG.s0 *= ALPHA.s0; \
  REG.s0 = MAC( -ALPHA.s1, REG.s1, REG.s0 ); \
  REG.s1 *= ALPHA.s0; \
  REG.s1 = MAC(  ALPHA.s1, type_mac_tmp, REG.s1 ); \
  /* (2) */ \
  if(BETA.s0 != 0) { \
  REG.s0 = MAC(  BETA.s0, SRC.s0, REG.s0 ); \
  REG.s1 = MAC(  BETA.s0, SRC.s1, REG.s1 ); \
  } \
  if (BETA.s1 != 0) { \
  REG.s0 = MAC( -BETA.s1, SRC.s1, REG.s0 ); \
  REG.s1 = MAC(  BETA.s1, SRC.s0, REG.s1 ); \
  } \
  /* (3) */ \
  DST = REG;

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_ZB_MT32x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW1_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  TensileComplexDouble *D,
  TensileComplexDouble const * __restrict__ C,
  TensileComplexDouble const * __restrict__ A,
  TensileComplexDouble const * __restrict__ B,
  TensileComplexDouble const alpha,
  TensileComplexDouble const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (TensileComplexDouble)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == tensile_complex<double>(0.0)) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 1); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }



  double type_mac_tmp;


  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE TensileComplexDouble
#define DEST_DATA_TYPE TensileComplexDouble
#define COMPUTE_DATA_TYPE TensileComplexDouble
#define s0 x
#define s1 y
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) \
  DST.s0 = MAC(  MULA.s0, MULB.s0, DST.s0 ); \
  DST.s0 = MAC( -MULA.s1, MULB.s1, DST.s0 ); \
  DST.s1 = MAC(  MULA.s0, MULB.s1, DST.s1 ); \
  DST.s1 = MAC(  MULA.s1, MULB.s0, DST.s1 );
#define TYPE_MAC_WRITE( DST, SRC, ALPHA, REG, BETA ) \
  /* (1) */ \
  type_mac_tmp = REG.s0; \
  REG.s0 *= ALPHA.s0; \
  REG.s0 = MAC( -ALPHA.s1, REG.s1, REG.s0 ); \
  REG.s1 *= ALPHA.s0; \
  REG.s1 = MAC(  ALPHA.s1, type_mac_tmp, REG.s1 ); \
  /* (2) */ \
  if(BETA.s0 != 0) { \
  REG.s0 = MAC(  BETA.s0, SRC.s0, REG.s0 ); \
  REG.s1 = MAC(  BETA.s0, SRC.s1, REG.s1 ); \
  } \
  if (BETA.s1 != 0) { \
  REG.s0 = MAC( -BETA.s1, SRC.s1, REG.s0 ); \
  REG.s1 = MAC(  BETA.s1, SRC.s0, REG.s1 ); \
  } \
  /* (3) */ \
  DST = REG;

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_ZB_MT32x32x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW1_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  TensileComplexDouble *D,
  TensileComplexDouble const * __restrict__ C,
  TensileComplexDouble const * __restrict__ A,
  TensileComplexDouble const * __restrict__ B,
  TensileComplexDouble const alpha,
  TensileComplexDouble const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (TensileComplexDouble)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == tensile_complex<double>(0.0)) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 1); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }



  double type_mac_tmp;


  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 768

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_4xi8BH_MT64x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 7 */

    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_4xi8BH_MT64x16x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_1_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_1_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_1_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_1_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_1_4_0 = *(globalReadA_0_0_4_0 + 1);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_1_5_0 = *(globalReadA_0_0_5_0 + 1);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_1_6_0 = *(globalReadA_0_0_6_0 + 1);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_1_7_0 = *(globalReadA_0_0_7_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_4_0 + 1) = a_0_1_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_5_0 + 1) = a_0_1_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_6_0 + 1) = a_0_1_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_7_0 + 1) = a_0_1_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_1_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 1);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_1_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 1);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_1_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 1);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_1_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_4_0 + 1) = a_0_1_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_5_0 + 1) = a_0_1_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_6_0 + 1) = a_0_1_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_7_0 + 1) = a_0_1_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 1) {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 2) {
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_4xi8BH_MT128x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 8

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x8 micro-tile */
#define MAC_2x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \

#define MAC_2x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_4xi8BH_MT32x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_8_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;
  unsigned int globalReadOffsetBL_4_0 = globalReadOffsetBL + 0 + 4*LSPB;
  unsigned int globalReadOffsetBL_5_0 = globalReadOffsetBL + 0 + 5*LSPB;
  unsigned int globalReadOffsetBL_6_0 = globalReadOffsetBL + 0 + 6*LSPB;
  unsigned int globalReadOffsetBL_7_0 = globalReadOffsetBL + 0 + 7*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_7_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 4*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 5*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 6*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 7*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x8


    /* iter 15 */

    MAC_2x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x8
#undef MAC_2x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 16
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_4xi8BH_MT64x16x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;
  unsigned int globalReadOffsetAL_8_0 = globalReadOffsetAL + 0 + 8*LSPA;
  unsigned int globalReadOffsetAL_9_0 = globalReadOffsetAL + 0 + 9*LSPA;
  unsigned int globalReadOffsetAL_10_0 = globalReadOffsetAL + 0 + 10*LSPA;
  unsigned int globalReadOffsetAL_11_0 = globalReadOffsetAL + 0 + 11*LSPA;
  unsigned int globalReadOffsetAL_12_0 = globalReadOffsetAL + 0 + 12*LSPA;
  unsigned int globalReadOffsetAL_13_0 = globalReadOffsetAL + 0 + 13*LSPA;
  unsigned int globalReadOffsetAL_14_0 = globalReadOffsetAL + 0 + 14*LSPA;
  unsigned int globalReadOffsetAL_15_0 = globalReadOffsetAL + 0 + 15*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_15_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 8*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 9*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 10*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 11*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 12*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 13*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 14*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 15*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_0_0_15_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_8_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_9_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_10_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_11_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_12_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_13_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_14_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_15_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_8_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_9_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_10_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_11_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_12_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_13_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_14_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_15_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_0_8_0 = ( globalReadOffsetAL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_0_0_9_0 = ( globalReadOffsetAL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_0_0_10_0 = ( globalReadOffsetAL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_0_0_11_0 = ( globalReadOffsetAL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_0_0_12_0 = ( globalReadOffsetAL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_0_0_13_0 = ( globalReadOffsetAL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_0_0_14_0 = ( globalReadOffsetAL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_0_0_15_0 = ( globalReadOffsetAL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 16

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x8 micro-tile */
#define MAC_2x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \

#define MAC_2x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_4xi8BH_MT16x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_8_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;
  DATA_TYPE b_0_0_8_0;
  DATA_TYPE b_0_0_9_0;
  DATA_TYPE b_0_0_10_0;
  DATA_TYPE b_0_0_11_0;
  DATA_TYPE b_0_0_12_0;
  DATA_TYPE b_0_0_13_0;
  DATA_TYPE b_0_0_14_0;
  DATA_TYPE b_0_0_15_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;
  unsigned int globalReadOffsetBL_4_0 = globalReadOffsetBL + 0 + 4*LSPB;
  unsigned int globalReadOffsetBL_5_0 = globalReadOffsetBL + 0 + 5*LSPB;
  unsigned int globalReadOffsetBL_6_0 = globalReadOffsetBL + 0 + 6*LSPB;
  unsigned int globalReadOffsetBL_7_0 = globalReadOffsetBL + 0 + 7*LSPB;
  unsigned int globalReadOffsetBL_8_0 = globalReadOffsetBL + 0 + 8*LSPB;
  unsigned int globalReadOffsetBL_9_0 = globalReadOffsetBL + 0 + 9*LSPB;
  unsigned int globalReadOffsetBL_10_0 = globalReadOffsetBL + 0 + 10*LSPB;
  unsigned int globalReadOffsetBL_11_0 = globalReadOffsetBL + 0 + 11*LSPB;
  unsigned int globalReadOffsetBL_12_0 = globalReadOffsetBL + 0 + 12*LSPB;
  unsigned int globalReadOffsetBL_13_0 = globalReadOffsetBL + 0 + 13*LSPB;
  unsigned int globalReadOffsetBL_14_0 = globalReadOffsetBL + 0 + 14*LSPB;
  unsigned int globalReadOffsetBL_15_0 = globalReadOffsetBL + 0 + 15*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_7_0), wgK );
  int64_t globalReadOffsetB_0_0_8_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_8_0), wgK );
  int64_t globalReadOffsetB_0_0_9_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_9_0), wgK );
  int64_t globalReadOffsetB_0_0_10_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_10_0), wgK );
  int64_t globalReadOffsetB_0_0_11_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_11_0), wgK );
  int64_t globalReadOffsetB_0_0_12_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_12_0), wgK );
  int64_t globalReadOffsetB_0_0_13_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_13_0), wgK );
  int64_t globalReadOffsetB_0_0_14_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_14_0), wgK );
  int64_t globalReadOffsetB_0_0_15_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_15_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;
  DATA_TYPE const *globalReadB_0_0_8_0 = B + globalReadOffsetB_0_0_8_0;
  DATA_TYPE const *globalReadB_0_0_9_0 = B + globalReadOffsetB_0_0_9_0;
  DATA_TYPE const *globalReadB_0_0_10_0 = B + globalReadOffsetB_0_0_10_0;
  DATA_TYPE const *globalReadB_0_0_11_0 = B + globalReadOffsetB_0_0_11_0;
  DATA_TYPE const *globalReadB_0_0_12_0 = B + globalReadOffsetB_0_0_12_0;
  DATA_TYPE const *globalReadB_0_0_13_0 = B + globalReadOffsetB_0_0_13_0;
  DATA_TYPE const *globalReadB_0_0_14_0 = B + globalReadOffsetB_0_0_14_0;
  DATA_TYPE const *globalReadB_0_0_15_0 = B + globalReadOffsetB_0_0_15_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 4*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 5*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 6*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 7*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_8_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 8*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_9_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 9*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_10_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 10*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_11_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 11*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_12_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 12*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_13_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 13*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_14_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 14*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_15_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 15*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;
  DATA_TYPE *localWriteB_0_0_8_0;
  DATA_TYPE *localWriteB_0_0_9_0;
  DATA_TYPE *localWriteB_0_0_10_0;
  DATA_TYPE *localWriteB_0_0_11_0;
  DATA_TYPE *localWriteB_0_0_12_0;
  DATA_TYPE *localWriteB_0_0_13_0;
  DATA_TYPE *localWriteB_0_0_14_0;
  DATA_TYPE *localWriteB_0_0_15_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);
  localWriteB_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_8_0);
  localWriteB_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_9_0);
  localWriteB_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_10_0);
  localWriteB_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_11_0);
  localWriteB_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_12_0);
  localWriteB_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_13_0);
  localWriteB_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_14_0);
  localWriteB_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_15_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_8_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_9_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_10_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_11_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_12_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_13_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_14_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_15_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);
    b_0_0_8_0 = *(globalReadB_0_0_8_0 + 0);
    b_0_0_9_0 = *(globalReadB_0_0_9_0 + 0);
    b_0_0_10_0 = *(globalReadB_0_0_10_0 + 0);
    b_0_0_11_0 = *(globalReadB_0_0_11_0 + 0);
    b_0_0_12_0 = *(globalReadB_0_0_12_0 + 0);
    b_0_0_13_0 = *(globalReadB_0_0_13_0 + 0);
    b_0_0_14_0 = *(globalReadB_0_0_14_0 + 0);
    b_0_0_15_0 = *(globalReadB_0_0_15_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_8_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_8_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_9_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_9_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_10_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_10_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_11_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_11_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_12_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_12_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_13_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_13_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_14_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_14_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_15_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_15_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
    *(localWriteB_0_0_8_0 + 0) = b_0_0_8_0;
    *(localWriteB_0_0_9_0 + 0) = b_0_0_9_0;
    *(localWriteB_0_0_10_0 + 0) = b_0_0_10_0;
    *(localWriteB_0_0_11_0 + 0) = b_0_0_11_0;
    *(localWriteB_0_0_12_0 + 0) = b_0_0_12_0;
    *(localWriteB_0_0_13_0 + 0) = b_0_0_13_0;
    *(localWriteB_0_0_14_0 + 0) = b_0_0_14_0;
    *(localWriteB_0_0_15_0 + 0) = b_0_0_15_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x8


    /* iter 15 */

    MAC_2x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);
  b_0_0_8_0 = ( globalReadOffsetBL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_8_0 + 0);
  b_0_0_9_0 = ( globalReadOffsetBL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_9_0 + 0);
  b_0_0_10_0 = ( globalReadOffsetBL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_10_0 + 0);
  b_0_0_11_0 = ( globalReadOffsetBL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_11_0 + 0);
  b_0_0_12_0 = ( globalReadOffsetBL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_12_0 + 0);
  b_0_0_13_0 = ( globalReadOffsetBL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_13_0 + 0);
  b_0_0_14_0 = ( globalReadOffsetBL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_14_0 + 0);
  b_0_0_15_0 = ( globalReadOffsetBL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_15_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);
  localWriteB_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_8_0);
  localWriteB_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_9_0);
  localWriteB_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_10_0);
  localWriteB_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_11_0);
  localWriteB_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_12_0);
  localWriteB_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_13_0);
  localWriteB_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_14_0);
  localWriteB_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_15_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
  *(localWriteB_0_0_8_0 + 0) = b_0_0_8_0;
  *(localWriteB_0_0_9_0 + 0) = b_0_0_9_0;
  *(localWriteB_0_0_10_0 + 0) = b_0_0_10_0;
  *(localWriteB_0_0_11_0 + 0) = b_0_0_11_0;
  *(localWriteB_0_0_12_0 + 0) = b_0_0_12_0;
  *(localWriteB_0_0_13_0 + 0) = b_0_0_13_0;
  *(localWriteB_0_0_14_0 + 0) = b_0_0_14_0;
  *(localWriteB_0_0_15_0 + 0) = b_0_0_15_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x8
#undef MAC_2x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 768

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_4xi8BH_MT64x32x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 7 */

    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_4xi8BH_MT64x16x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_1_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_1_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_1_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_1_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_1_4_0 = *(globalReadA_0_0_4_0 + 1);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_1_5_0 = *(globalReadA_0_0_5_0 + 1);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_1_6_0 = *(globalReadA_0_0_6_0 + 1);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_1_7_0 = *(globalReadA_0_0_7_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_4_0 + 1) = a_0_1_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_5_0 + 1) = a_0_1_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_6_0 + 1) = a_0_1_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_7_0 + 1) = a_0_1_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_1_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 1);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_1_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 1);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_1_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 1);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_1_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_4_0 + 1) = a_0_1_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_5_0 + 1) = a_0_1_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_6_0 + 1) = a_0_1_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_7_0 + 1) = a_0_1_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 1) {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 2) {
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_4xi8BH_MT128x32x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 8

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x8 micro-tile */
#define MAC_2x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \

#define MAC_2x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_4xi8BH_MT32x128x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_8_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;
  unsigned int globalReadOffsetBL_4_0 = globalReadOffsetBL + 0 + 4*LSPB;
  unsigned int globalReadOffsetBL_5_0 = globalReadOffsetBL + 0 + 5*LSPB;
  unsigned int globalReadOffsetBL_6_0 = globalReadOffsetBL + 0 + 6*LSPB;
  unsigned int globalReadOffsetBL_7_0 = globalReadOffsetBL + 0 + 7*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_7_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 4*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 5*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 6*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 7*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x8


    /* iter 15 */

    MAC_2x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x8
#undef MAC_2x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 16
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_4xi8BH_MT64x16x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;
  unsigned int globalReadOffsetAL_8_0 = globalReadOffsetAL + 0 + 8*LSPA;
  unsigned int globalReadOffsetAL_9_0 = globalReadOffsetAL + 0 + 9*LSPA;
  unsigned int globalReadOffsetAL_10_0 = globalReadOffsetAL + 0 + 10*LSPA;
  unsigned int globalReadOffsetAL_11_0 = globalReadOffsetAL + 0 + 11*LSPA;
  unsigned int globalReadOffsetAL_12_0 = globalReadOffsetAL + 0 + 12*LSPA;
  unsigned int globalReadOffsetAL_13_0 = globalReadOffsetAL + 0 + 13*LSPA;
  unsigned int globalReadOffsetAL_14_0 = globalReadOffsetAL + 0 + 14*LSPA;
  unsigned int globalReadOffsetAL_15_0 = globalReadOffsetAL + 0 + 15*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_15_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 8*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 9*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 10*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 11*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 12*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 13*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 14*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 15*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_0_0_15_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_8_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_9_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_10_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_11_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_12_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_13_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_14_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_15_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_8_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_9_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_10_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_11_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_12_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_13_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_14_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_15_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_0_8_0 = ( globalReadOffsetAL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_0_0_9_0 = ( globalReadOffsetAL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_0_0_10_0 = ( globalReadOffsetAL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_0_0_11_0 = ( globalReadOffsetAL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_0_0_12_0 = ( globalReadOffsetAL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_0_0_13_0 = ( globalReadOffsetAL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_0_0_14_0 = ( globalReadOffsetAL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_0_0_15_0 = ( globalReadOffsetAL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 16

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE uint32_t
#define DEST_DATA_TYPE int32_t
#define COMPUTE_DATA_TYPE int32_t
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST = GenDot4(static_cast<int>(A), static_cast<int>(B), static_cast<int>(DST))
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x8 micro-tile */
#define MAC_2x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \

#define MAC_2x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_4xi8BH_MT16x64x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_8_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  int32_t *D,
  int32_t const * __restrict__ C,
  uint32_t const * __restrict__ A,
  uint32_t const * __restrict__ B,
  int32_t const alpha,
  int32_t const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (uint32_t)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;
  DATA_TYPE b_0_0_8_0;
  DATA_TYPE b_0_0_9_0;
  DATA_TYPE b_0_0_10_0;
  DATA_TYPE b_0_0_11_0;
  DATA_TYPE b_0_0_12_0;
  DATA_TYPE b_0_0_13_0;
  DATA_TYPE b_0_0_14_0;
  DATA_TYPE b_0_0_15_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;
  unsigned int globalReadOffsetBL_4_0 = globalReadOffsetBL + 0 + 4*LSPB;
  unsigned int globalReadOffsetBL_5_0 = globalReadOffsetBL + 0 + 5*LSPB;
  unsigned int globalReadOffsetBL_6_0 = globalReadOffsetBL + 0 + 6*LSPB;
  unsigned int globalReadOffsetBL_7_0 = globalReadOffsetBL + 0 + 7*LSPB;
  unsigned int globalReadOffsetBL_8_0 = globalReadOffsetBL + 0 + 8*LSPB;
  unsigned int globalReadOffsetBL_9_0 = globalReadOffsetBL + 0 + 9*LSPB;
  unsigned int globalReadOffsetBL_10_0 = globalReadOffsetBL + 0 + 10*LSPB;
  unsigned int globalReadOffsetBL_11_0 = globalReadOffsetBL + 0 + 11*LSPB;
  unsigned int globalReadOffsetBL_12_0 = globalReadOffsetBL + 0 + 12*LSPB;
  unsigned int globalReadOffsetBL_13_0 = globalReadOffsetBL + 0 + 13*LSPB;
  unsigned int globalReadOffsetBL_14_0 = globalReadOffsetBL + 0 + 14*LSPB;
  unsigned int globalReadOffsetBL_15_0 = globalReadOffsetBL + 0 + 15*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_7_0), wgK );
  int64_t globalReadOffsetB_0_0_8_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_8_0), wgK );
  int64_t globalReadOffsetB_0_0_9_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_9_0), wgK );
  int64_t globalReadOffsetB_0_0_10_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_10_0), wgK );
  int64_t globalReadOffsetB_0_0_11_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_11_0), wgK );
  int64_t globalReadOffsetB_0_0_12_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_12_0), wgK );
  int64_t globalReadOffsetB_0_0_13_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_13_0), wgK );
  int64_t globalReadOffsetB_0_0_14_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_14_0), wgK );
  int64_t globalReadOffsetB_0_0_15_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_15_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;
  DATA_TYPE const *globalReadB_0_0_8_0 = B + globalReadOffsetB_0_0_8_0;
  DATA_TYPE const *globalReadB_0_0_9_0 = B + globalReadOffsetB_0_0_9_0;
  DATA_TYPE const *globalReadB_0_0_10_0 = B + globalReadOffsetB_0_0_10_0;
  DATA_TYPE const *globalReadB_0_0_11_0 = B + globalReadOffsetB_0_0_11_0;
  DATA_TYPE const *globalReadB_0_0_12_0 = B + globalReadOffsetB_0_0_12_0;
  DATA_TYPE const *globalReadB_0_0_13_0 = B + globalReadOffsetB_0_0_13_0;
  DATA_TYPE const *globalReadB_0_0_14_0 = B + globalReadOffsetB_0_0_14_0;
  DATA_TYPE const *globalReadB_0_0_15_0 = B + globalReadOffsetB_0_0_15_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 4*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 5*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 6*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 7*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_8_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 8*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_9_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 9*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_10_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 10*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_11_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 11*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_12_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 12*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_13_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 13*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_14_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 14*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_15_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 15*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;
  DATA_TYPE *localWriteB_0_0_8_0;
  DATA_TYPE *localWriteB_0_0_9_0;
  DATA_TYPE *localWriteB_0_0_10_0;
  DATA_TYPE *localWriteB_0_0_11_0;
  DATA_TYPE *localWriteB_0_0_12_0;
  DATA_TYPE *localWriteB_0_0_13_0;
  DATA_TYPE *localWriteB_0_0_14_0;
  DATA_TYPE *localWriteB_0_0_15_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);
  localWriteB_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_8_0);
  localWriteB_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_9_0);
  localWriteB_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_10_0);
  localWriteB_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_11_0);
  localWriteB_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_12_0);
  localWriteB_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_13_0);
  localWriteB_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_14_0);
  localWriteB_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_15_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_8_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_9_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_10_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_11_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_12_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_13_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_14_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_15_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);
    b_0_0_8_0 = *(globalReadB_0_0_8_0 + 0);
    b_0_0_9_0 = *(globalReadB_0_0_9_0 + 0);
    b_0_0_10_0 = *(globalReadB_0_0_10_0 + 0);
    b_0_0_11_0 = *(globalReadB_0_0_11_0 + 0);
    b_0_0_12_0 = *(globalReadB_0_0_12_0 + 0);
    b_0_0_13_0 = *(globalReadB_0_0_13_0 + 0);
    b_0_0_14_0 = *(globalReadB_0_0_14_0 + 0);
    b_0_0_15_0 = *(globalReadB_0_0_15_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_8_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_8_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_9_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_9_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_10_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_10_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_11_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_11_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_12_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_12_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_13_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_13_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_14_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_14_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_15_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_15_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
    *(localWriteB_0_0_8_0 + 0) = b_0_0_8_0;
    *(localWriteB_0_0_9_0 + 0) = b_0_0_9_0;
    *(localWriteB_0_0_10_0 + 0) = b_0_0_10_0;
    *(localWriteB_0_0_11_0 + 0) = b_0_0_11_0;
    *(localWriteB_0_0_12_0 + 0) = b_0_0_12_0;
    *(localWriteB_0_0_13_0 + 0) = b_0_0_13_0;
    *(localWriteB_0_0_14_0 + 0) = b_0_0_14_0;
    *(localWriteB_0_0_15_0 + 0) = b_0_0_15_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x8


    /* iter 15 */

    MAC_2x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);
  b_0_0_8_0 = ( globalReadOffsetBL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_8_0 + 0);
  b_0_0_9_0 = ( globalReadOffsetBL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_9_0 + 0);
  b_0_0_10_0 = ( globalReadOffsetBL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_10_0 + 0);
  b_0_0_11_0 = ( globalReadOffsetBL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_11_0 + 0);
  b_0_0_12_0 = ( globalReadOffsetBL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_12_0 + 0);
  b_0_0_13_0 = ( globalReadOffsetBL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_13_0 + 0);
  b_0_0_14_0 = ( globalReadOffsetBL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_14_0 + 0);
  b_0_0_15_0 = ( globalReadOffsetBL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_15_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);
  localWriteB_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_8_0);
  localWriteB_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_9_0);
  localWriteB_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_10_0);
  localWriteB_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_11_0);
  localWriteB_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_12_0);
  localWriteB_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_13_0);
  localWriteB_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_14_0);
  localWriteB_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_15_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
  *(localWriteB_0_0_8_0 + 0) = b_0_0_8_0;
  *(localWriteB_0_0_9_0 + 0) = b_0_0_9_0;
  *(localWriteB_0_0_10_0 + 0) = b_0_0_10_0;
  *(localWriteB_0_0_11_0 + 0) = b_0_0_11_0;
  *(localWriteB_0_0_12_0 + 0) = b_0_0_12_0;
  *(localWriteB_0_0_13_0 + 0) = b_0_0_13_0;
  *(localWriteB_0_0_14_0 + 0) = b_0_0_14_0;
  *(localWriteB_0_0_15_0 + 0) = b_0_0_15_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x8
#undef MAC_2x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 819

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? static_cast<tensile_bfloat16>((ALPHA)*(REG) + (BETA) * static_cast<float>(SRC)) : static_cast<tensile_bfloat16>((ALPHA)*(REG));

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_BBH_MT64x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_bfloat16 *D,
  tensile_bfloat16 const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 7 */

    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? static_cast<tensile_bfloat16>((ALPHA)*(REG) + (BETA) * static_cast<float>(SRC)) : static_cast<tensile_bfloat16>((ALPHA)*(REG));

/* 2x8 micro-tile */
#define MAC_2x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \

#define MAC_2x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_BBH_MT32x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_8_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_bfloat16 *D,
  tensile_bfloat16 const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_1_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_1_3_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x8


    /* iter 15 */

    MAC_2x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_1_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 1);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_1_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x8

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 1) {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 2) {
        rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x8
#undef MAC_2x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 8

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? static_cast<tensile_bfloat16>((ALPHA)*(REG) + (BETA) * static_cast<float>(SRC)) : static_cast<tensile_bfloat16>((ALPHA)*(REG));

/* 2x8 micro-tile */
#define MAC_2x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \

#define MAC_2x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_BBH_MT16x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_8_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  tensile_bfloat16 *D,
  tensile_bfloat16 const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_1_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_1_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_1_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_1_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_1_6_0;
  DATA_TYPE b_0_0_7_0;
  DATA_TYPE b_0_1_7_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;
  unsigned int globalReadOffsetBL_4_0 = globalReadOffsetBL + 0 + 4*LSPB;
  unsigned int globalReadOffsetBL_5_0 = globalReadOffsetBL + 0 + 5*LSPB;
  unsigned int globalReadOffsetBL_6_0 = globalReadOffsetBL + 0 + 6*LSPB;
  unsigned int globalReadOffsetBL_7_0 = globalReadOffsetBL + 0 + 7*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_7_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 4*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 5*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 6*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 7*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_1_4_0 = *(globalReadB_0_0_4_0 + 1);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_1_5_0 = *(globalReadB_0_0_5_0 + 1);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_1_6_0 = *(globalReadB_0_0_6_0 + 1);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);
    b_0_1_7_0 = *(globalReadB_0_0_7_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_4_0 + 1) = b_0_1_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_5_0 + 1) = b_0_1_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_6_0 + 1) = b_0_1_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
    *(localWriteB_0_0_7_0 + 1) = b_0_1_7_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x8


    /* iter 15 */

    MAC_2x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_1_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 1);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_1_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 1);
  b_0_0_4_0 = ( globalReadOffsetBL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_1_4_0 = ( globalReadOffsetBL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 1);
  b_0_0_5_0 = ( globalReadOffsetBL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_1_5_0 = ( globalReadOffsetBL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 1);
  b_0_0_6_0 = ( globalReadOffsetBL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_1_6_0 = ( globalReadOffsetBL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 1);
  b_0_0_7_0 = ( globalReadOffsetBL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);
  b_0_1_7_0 = ( globalReadOffsetBL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_4_0 + 1) = b_0_1_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_5_0 + 1) = b_0_1_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_6_0 + 1) = b_0_1_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
  *(localWriteB_0_0_7_0 + 1) = b_0_1_7_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x8

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 1) {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 2) {
        rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x8
#undef MAC_2x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? static_cast<tensile_bfloat16>((ALPHA)*(REG) + (BETA) * static_cast<float>(SRC)) : static_cast<tensile_bfloat16>((ALPHA)*(REG));

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_BBH_MT128x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_bfloat16 *D,
  tensile_bfloat16 const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 8

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? static_cast<tensile_bfloat16>((ALPHA)*(REG) + (BETA) * static_cast<float>(SRC)) : static_cast<tensile_bfloat16>((ALPHA)*(REG));

/* 2x8 micro-tile */
#define MAC_2x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \

#define MAC_2x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_BBH_MT32x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_8_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_bfloat16 *D,
  tensile_bfloat16 const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;
  unsigned int globalReadOffsetBL_4_0 = globalReadOffsetBL + 0 + 4*LSPB;
  unsigned int globalReadOffsetBL_5_0 = globalReadOffsetBL + 0 + 5*LSPB;
  unsigned int globalReadOffsetBL_6_0 = globalReadOffsetBL + 0 + 6*LSPB;
  unsigned int globalReadOffsetBL_7_0 = globalReadOffsetBL + 0 + 7*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_7_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 4*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 5*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 6*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 7*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x8


    /* iter 15 */

    MAC_2x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x8
#undef MAC_2x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 16
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? static_cast<tensile_bfloat16>((ALPHA)*(REG) + (BETA) * static_cast<float>(SRC)) : static_cast<tensile_bfloat16>((ALPHA)*(REG));

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_BBH_MT64x16x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  tensile_bfloat16 *D,
  tensile_bfloat16 const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;
  unsigned int globalReadOffsetAL_8_0 = globalReadOffsetAL + 0 + 8*LSPA;
  unsigned int globalReadOffsetAL_9_0 = globalReadOffsetAL + 0 + 9*LSPA;
  unsigned int globalReadOffsetAL_10_0 = globalReadOffsetAL + 0 + 10*LSPA;
  unsigned int globalReadOffsetAL_11_0 = globalReadOffsetAL + 0 + 11*LSPA;
  unsigned int globalReadOffsetAL_12_0 = globalReadOffsetAL + 0 + 12*LSPA;
  unsigned int globalReadOffsetAL_13_0 = globalReadOffsetAL + 0 + 13*LSPA;
  unsigned int globalReadOffsetAL_14_0 = globalReadOffsetAL + 0 + 14*LSPA;
  unsigned int globalReadOffsetAL_15_0 = globalReadOffsetAL + 0 + 15*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_15_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 8*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 9*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 10*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 11*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 12*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 13*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 14*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 15*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_0_0_15_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_8_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_9_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_10_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_11_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_12_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_13_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_14_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_15_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_8_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_9_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_10_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_11_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_12_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_13_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_14_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_15_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_0_8_0 = ( globalReadOffsetAL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_0_0_9_0 = ( globalReadOffsetAL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_0_0_10_0 = ( globalReadOffsetAL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_0_0_11_0 = ( globalReadOffsetAL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_0_0_12_0 = ( globalReadOffsetAL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_0_0_13_0 = ( globalReadOffsetAL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_0_0_14_0 = ( globalReadOffsetAL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_0_0_15_0 = ( globalReadOffsetAL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? static_cast<tensile_bfloat16>((ALPHA)*(REG) + (BETA) * static_cast<float>(SRC)) : static_cast<tensile_bfloat16>((ALPHA)*(REG));

/* 2x8 micro-tile */
#define MAC_2x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \

#define MAC_2x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_BBH_MT32x128x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_8_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_bfloat16 *D,
  tensile_bfloat16 const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_1_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_1_3_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x8


    /* iter 15 */

    MAC_2x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_1_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 1);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_1_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x8

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 1) {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 2) {
        rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x8
#undef MAC_2x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? static_cast<tensile_bfloat16>((ALPHA)*(REG) + (BETA) * static_cast<float>(SRC)) : static_cast<tensile_bfloat16>((ALPHA)*(REG));

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_BBH_MT128x32x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_bfloat16 *D,
  tensile_bfloat16 const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 16
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 1280

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? static_cast<tensile_bfloat16>((ALPHA)*(REG) + (BETA) * static_cast<float>(SRC)) : static_cast<tensile_bfloat16>((ALPHA)*(REG));

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_BBH_MT64x16x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  tensile_bfloat16 *D,
  tensile_bfloat16 const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;
  unsigned int globalReadOffsetAL_8_0 = globalReadOffsetAL + 0 + 8*LSPA;
  unsigned int globalReadOffsetAL_9_0 = globalReadOffsetAL + 0 + 9*LSPA;
  unsigned int globalReadOffsetAL_10_0 = globalReadOffsetAL + 0 + 10*LSPA;
  unsigned int globalReadOffsetAL_11_0 = globalReadOffsetAL + 0 + 11*LSPA;
  unsigned int globalReadOffsetAL_12_0 = globalReadOffsetAL + 0 + 12*LSPA;
  unsigned int globalReadOffsetAL_13_0 = globalReadOffsetAL + 0 + 13*LSPA;
  unsigned int globalReadOffsetAL_14_0 = globalReadOffsetAL + 0 + 14*LSPA;
  unsigned int globalReadOffsetAL_15_0 = globalReadOffsetAL + 0 + 15*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_15_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 8*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 9*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 10*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 11*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 12*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 13*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 14*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 15*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_0_0_15_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_8_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_9_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_10_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_11_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_12_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_13_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_14_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_15_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_8_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_9_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_10_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_11_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_12_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_13_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_14_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_15_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_0_8_0 = ( globalReadOffsetAL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_0_0_9_0 = ( globalReadOffsetAL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_0_0_10_0 = ( globalReadOffsetAL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_0_0_11_0 = ( globalReadOffsetAL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_0_0_12_0 = ( globalReadOffsetAL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_0_0_13_0 = ( globalReadOffsetAL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_0_0_14_0 = ( globalReadOffsetAL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_0_0_15_0 = ( globalReadOffsetAL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 64
#define SG1J 1
#define TT0I 1
#define TT1J 1
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 64
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 60

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 64
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 4096
#define LDS_NUM_ELEMENTS 4160

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 1x1 micro-tile */
#define MAC_1x1\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \

#define MAC_1x1_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_BBH_MT64x1x64_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU60_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU0_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT1_1_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG64_1_1_WGM1(
  float *D,
  float const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE a_0_0_16_0;
  DATA_TYPE a_0_0_17_0;
  DATA_TYPE a_0_0_18_0;
  DATA_TYPE a_0_0_19_0;
  DATA_TYPE a_0_0_20_0;
  DATA_TYPE a_0_0_21_0;
  DATA_TYPE a_0_0_22_0;
  DATA_TYPE a_0_0_23_0;
  DATA_TYPE a_0_0_24_0;
  DATA_TYPE a_0_0_25_0;
  DATA_TYPE a_0_0_26_0;
  DATA_TYPE a_0_0_27_0;
  DATA_TYPE a_0_0_28_0;
  DATA_TYPE a_0_0_29_0;
  DATA_TYPE a_0_0_30_0;
  DATA_TYPE a_0_0_31_0;
  DATA_TYPE a_0_0_32_0;
  DATA_TYPE a_0_0_33_0;
  DATA_TYPE a_0_0_34_0;
  DATA_TYPE a_0_0_35_0;
  DATA_TYPE a_0_0_36_0;
  DATA_TYPE a_0_0_37_0;
  DATA_TYPE a_0_0_38_0;
  DATA_TYPE a_0_0_39_0;
  DATA_TYPE a_0_0_40_0;
  DATA_TYPE a_0_0_41_0;
  DATA_TYPE a_0_0_42_0;
  DATA_TYPE a_0_0_43_0;
  DATA_TYPE a_0_0_44_0;
  DATA_TYPE a_0_0_45_0;
  DATA_TYPE a_0_0_46_0;
  DATA_TYPE a_0_0_47_0;
  DATA_TYPE a_0_0_48_0;
  DATA_TYPE a_0_0_49_0;
  DATA_TYPE a_0_0_50_0;
  DATA_TYPE a_0_0_51_0;
  DATA_TYPE a_0_0_52_0;
  DATA_TYPE a_0_0_53_0;
  DATA_TYPE a_0_0_54_0;
  DATA_TYPE a_0_0_55_0;
  DATA_TYPE a_0_0_56_0;
  DATA_TYPE a_0_0_57_0;
  DATA_TYPE a_0_0_58_0;
  DATA_TYPE a_0_0_59_0;
  DATA_TYPE a_0_0_60_0;
  DATA_TYPE a_0_0_61_0;
  DATA_TYPE a_0_0_62_0;
  DATA_TYPE a_0_0_63_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;
  unsigned int globalReadOffsetAL_8_0 = globalReadOffsetAL + 0 + 8*LSPA;
  unsigned int globalReadOffsetAL_9_0 = globalReadOffsetAL + 0 + 9*LSPA;
  unsigned int globalReadOffsetAL_10_0 = globalReadOffsetAL + 0 + 10*LSPA;
  unsigned int globalReadOffsetAL_11_0 = globalReadOffsetAL + 0 + 11*LSPA;
  unsigned int globalReadOffsetAL_12_0 = globalReadOffsetAL + 0 + 12*LSPA;
  unsigned int globalReadOffsetAL_13_0 = globalReadOffsetAL + 0 + 13*LSPA;
  unsigned int globalReadOffsetAL_14_0 = globalReadOffsetAL + 0 + 14*LSPA;
  unsigned int globalReadOffsetAL_15_0 = globalReadOffsetAL + 0 + 15*LSPA;
  unsigned int globalReadOffsetAL_16_0 = globalReadOffsetAL + 0 + 16*LSPA;
  unsigned int globalReadOffsetAL_17_0 = globalReadOffsetAL + 0 + 17*LSPA;
  unsigned int globalReadOffsetAL_18_0 = globalReadOffsetAL + 0 + 18*LSPA;
  unsigned int globalReadOffsetAL_19_0 = globalReadOffsetAL + 0 + 19*LSPA;
  unsigned int globalReadOffsetAL_20_0 = globalReadOffsetAL + 0 + 20*LSPA;
  unsigned int globalReadOffsetAL_21_0 = globalReadOffsetAL + 0 + 21*LSPA;
  unsigned int globalReadOffsetAL_22_0 = globalReadOffsetAL + 0 + 22*LSPA;
  unsigned int globalReadOffsetAL_23_0 = globalReadOffsetAL + 0 + 23*LSPA;
  unsigned int globalReadOffsetAL_24_0 = globalReadOffsetAL + 0 + 24*LSPA;
  unsigned int globalReadOffsetAL_25_0 = globalReadOffsetAL + 0 + 25*LSPA;
  unsigned int globalReadOffsetAL_26_0 = globalReadOffsetAL + 0 + 26*LSPA;
  unsigned int globalReadOffsetAL_27_0 = globalReadOffsetAL + 0 + 27*LSPA;
  unsigned int globalReadOffsetAL_28_0 = globalReadOffsetAL + 0 + 28*LSPA;
  unsigned int globalReadOffsetAL_29_0 = globalReadOffsetAL + 0 + 29*LSPA;
  unsigned int globalReadOffsetAL_30_0 = globalReadOffsetAL + 0 + 30*LSPA;
  unsigned int globalReadOffsetAL_31_0 = globalReadOffsetAL + 0 + 31*LSPA;
  unsigned int globalReadOffsetAL_32_0 = globalReadOffsetAL + 0 + 32*LSPA;
  unsigned int globalReadOffsetAL_33_0 = globalReadOffsetAL + 0 + 33*LSPA;
  unsigned int globalReadOffsetAL_34_0 = globalReadOffsetAL + 0 + 34*LSPA;
  unsigned int globalReadOffsetAL_35_0 = globalReadOffsetAL + 0 + 35*LSPA;
  unsigned int globalReadOffsetAL_36_0 = globalReadOffsetAL + 0 + 36*LSPA;
  unsigned int globalReadOffsetAL_37_0 = globalReadOffsetAL + 0 + 37*LSPA;
  unsigned int globalReadOffsetAL_38_0 = globalReadOffsetAL + 0 + 38*LSPA;
  unsigned int globalReadOffsetAL_39_0 = globalReadOffsetAL + 0 + 39*LSPA;
  unsigned int globalReadOffsetAL_40_0 = globalReadOffsetAL + 0 + 40*LSPA;
  unsigned int globalReadOffsetAL_41_0 = globalReadOffsetAL + 0 + 41*LSPA;
  unsigned int globalReadOffsetAL_42_0 = globalReadOffsetAL + 0 + 42*LSPA;
  unsigned int globalReadOffsetAL_43_0 = globalReadOffsetAL + 0 + 43*LSPA;
  unsigned int globalReadOffsetAL_44_0 = globalReadOffsetAL + 0 + 44*LSPA;
  unsigned int globalReadOffsetAL_45_0 = globalReadOffsetAL + 0 + 45*LSPA;
  unsigned int globalReadOffsetAL_46_0 = globalReadOffsetAL + 0 + 46*LSPA;
  unsigned int globalReadOffsetAL_47_0 = globalReadOffsetAL + 0 + 47*LSPA;
  unsigned int globalReadOffsetAL_48_0 = globalReadOffsetAL + 0 + 48*LSPA;
  unsigned int globalReadOffsetAL_49_0 = globalReadOffsetAL + 0 + 49*LSPA;
  unsigned int globalReadOffsetAL_50_0 = globalReadOffsetAL + 0 + 50*LSPA;
  unsigned int globalReadOffsetAL_51_0 = globalReadOffsetAL + 0 + 51*LSPA;
  unsigned int globalReadOffsetAL_52_0 = globalReadOffsetAL + 0 + 52*LSPA;
  unsigned int globalReadOffsetAL_53_0 = globalReadOffsetAL + 0 + 53*LSPA;
  unsigned int globalReadOffsetAL_54_0 = globalReadOffsetAL + 0 + 54*LSPA;
  unsigned int globalReadOffsetAL_55_0 = globalReadOffsetAL + 0 + 55*LSPA;
  unsigned int globalReadOffsetAL_56_0 = globalReadOffsetAL + 0 + 56*LSPA;
  unsigned int globalReadOffsetAL_57_0 = globalReadOffsetAL + 0 + 57*LSPA;
  unsigned int globalReadOffsetAL_58_0 = globalReadOffsetAL + 0 + 58*LSPA;
  unsigned int globalReadOffsetAL_59_0 = globalReadOffsetAL + 0 + 59*LSPA;
  unsigned int globalReadOffsetAL_60_0 = globalReadOffsetAL + 0 + 60*LSPA;
  unsigned int globalReadOffsetAL_61_0 = globalReadOffsetAL + 0 + 61*LSPA;
  unsigned int globalReadOffsetAL_62_0 = globalReadOffsetAL + 0 + 62*LSPA;
  unsigned int globalReadOffsetAL_63_0 = globalReadOffsetAL + 0 + 63*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_15_0), wgK );
  int64_t globalReadOffsetA_0_0_16_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_16_0), wgK );
  int64_t globalReadOffsetA_0_0_17_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_17_0), wgK );
  int64_t globalReadOffsetA_0_0_18_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_18_0), wgK );
  int64_t globalReadOffsetA_0_0_19_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_19_0), wgK );
  int64_t globalReadOffsetA_0_0_20_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_20_0), wgK );
  int64_t globalReadOffsetA_0_0_21_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_21_0), wgK );
  int64_t globalReadOffsetA_0_0_22_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_22_0), wgK );
  int64_t globalReadOffsetA_0_0_23_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_23_0), wgK );
  int64_t globalReadOffsetA_0_0_24_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_24_0), wgK );
  int64_t globalReadOffsetA_0_0_25_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_25_0), wgK );
  int64_t globalReadOffsetA_0_0_26_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_26_0), wgK );
  int64_t globalReadOffsetA_0_0_27_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_27_0), wgK );
  int64_t globalReadOffsetA_0_0_28_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_28_0), wgK );
  int64_t globalReadOffsetA_0_0_29_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_29_0), wgK );
  int64_t globalReadOffsetA_0_0_30_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_30_0), wgK );
  int64_t globalReadOffsetA_0_0_31_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_31_0), wgK );
  int64_t globalReadOffsetA_0_0_32_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_32_0), wgK );
  int64_t globalReadOffsetA_0_0_33_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_33_0), wgK );
  int64_t globalReadOffsetA_0_0_34_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_34_0), wgK );
  int64_t globalReadOffsetA_0_0_35_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_35_0), wgK );
  int64_t globalReadOffsetA_0_0_36_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_36_0), wgK );
  int64_t globalReadOffsetA_0_0_37_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_37_0), wgK );
  int64_t globalReadOffsetA_0_0_38_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_38_0), wgK );
  int64_t globalReadOffsetA_0_0_39_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_39_0), wgK );
  int64_t globalReadOffsetA_0_0_40_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_40_0), wgK );
  int64_t globalReadOffsetA_0_0_41_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_41_0), wgK );
  int64_t globalReadOffsetA_0_0_42_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_42_0), wgK );
  int64_t globalReadOffsetA_0_0_43_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_43_0), wgK );
  int64_t globalReadOffsetA_0_0_44_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_44_0), wgK );
  int64_t globalReadOffsetA_0_0_45_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_45_0), wgK );
  int64_t globalReadOffsetA_0_0_46_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_46_0), wgK );
  int64_t globalReadOffsetA_0_0_47_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_47_0), wgK );
  int64_t globalReadOffsetA_0_0_48_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_48_0), wgK );
  int64_t globalReadOffsetA_0_0_49_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_49_0), wgK );
  int64_t globalReadOffsetA_0_0_50_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_50_0), wgK );
  int64_t globalReadOffsetA_0_0_51_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_51_0), wgK );
  int64_t globalReadOffsetA_0_0_52_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_52_0), wgK );
  int64_t globalReadOffsetA_0_0_53_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_53_0), wgK );
  int64_t globalReadOffsetA_0_0_54_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_54_0), wgK );
  int64_t globalReadOffsetA_0_0_55_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_55_0), wgK );
  int64_t globalReadOffsetA_0_0_56_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_56_0), wgK );
  int64_t globalReadOffsetA_0_0_57_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_57_0), wgK );
  int64_t globalReadOffsetA_0_0_58_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_58_0), wgK );
  int64_t globalReadOffsetA_0_0_59_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_59_0), wgK );
  int64_t globalReadOffsetA_0_0_60_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_60_0), wgK );
  int64_t globalReadOffsetA_0_0_61_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_61_0), wgK );
  int64_t globalReadOffsetA_0_0_62_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_62_0), wgK );
  int64_t globalReadOffsetA_0_0_63_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_63_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;
  DATA_TYPE const *globalReadA_0_0_16_0 = A + globalReadOffsetA_0_0_16_0;
  DATA_TYPE const *globalReadA_0_0_17_0 = A + globalReadOffsetA_0_0_17_0;
  DATA_TYPE const *globalReadA_0_0_18_0 = A + globalReadOffsetA_0_0_18_0;
  DATA_TYPE const *globalReadA_0_0_19_0 = A + globalReadOffsetA_0_0_19_0;
  DATA_TYPE const *globalReadA_0_0_20_0 = A + globalReadOffsetA_0_0_20_0;
  DATA_TYPE const *globalReadA_0_0_21_0 = A + globalReadOffsetA_0_0_21_0;
  DATA_TYPE const *globalReadA_0_0_22_0 = A + globalReadOffsetA_0_0_22_0;
  DATA_TYPE const *globalReadA_0_0_23_0 = A + globalReadOffsetA_0_0_23_0;
  DATA_TYPE const *globalReadA_0_0_24_0 = A + globalReadOffsetA_0_0_24_0;
  DATA_TYPE const *globalReadA_0_0_25_0 = A + globalReadOffsetA_0_0_25_0;
  DATA_TYPE const *globalReadA_0_0_26_0 = A + globalReadOffsetA_0_0_26_0;
  DATA_TYPE const *globalReadA_0_0_27_0 = A + globalReadOffsetA_0_0_27_0;
  DATA_TYPE const *globalReadA_0_0_28_0 = A + globalReadOffsetA_0_0_28_0;
  DATA_TYPE const *globalReadA_0_0_29_0 = A + globalReadOffsetA_0_0_29_0;
  DATA_TYPE const *globalReadA_0_0_30_0 = A + globalReadOffsetA_0_0_30_0;
  DATA_TYPE const *globalReadA_0_0_31_0 = A + globalReadOffsetA_0_0_31_0;
  DATA_TYPE const *globalReadA_0_0_32_0 = A + globalReadOffsetA_0_0_32_0;
  DATA_TYPE const *globalReadA_0_0_33_0 = A + globalReadOffsetA_0_0_33_0;
  DATA_TYPE const *globalReadA_0_0_34_0 = A + globalReadOffsetA_0_0_34_0;
  DATA_TYPE const *globalReadA_0_0_35_0 = A + globalReadOffsetA_0_0_35_0;
  DATA_TYPE const *globalReadA_0_0_36_0 = A + globalReadOffsetA_0_0_36_0;
  DATA_TYPE const *globalReadA_0_0_37_0 = A + globalReadOffsetA_0_0_37_0;
  DATA_TYPE const *globalReadA_0_0_38_0 = A + globalReadOffsetA_0_0_38_0;
  DATA_TYPE const *globalReadA_0_0_39_0 = A + globalReadOffsetA_0_0_39_0;
  DATA_TYPE const *globalReadA_0_0_40_0 = A + globalReadOffsetA_0_0_40_0;
  DATA_TYPE const *globalReadA_0_0_41_0 = A + globalReadOffsetA_0_0_41_0;
  DATA_TYPE const *globalReadA_0_0_42_0 = A + globalReadOffsetA_0_0_42_0;
  DATA_TYPE const *globalReadA_0_0_43_0 = A + globalReadOffsetA_0_0_43_0;
  DATA_TYPE const *globalReadA_0_0_44_0 = A + globalReadOffsetA_0_0_44_0;
  DATA_TYPE const *globalReadA_0_0_45_0 = A + globalReadOffsetA_0_0_45_0;
  DATA_TYPE const *globalReadA_0_0_46_0 = A + globalReadOffsetA_0_0_46_0;
  DATA_TYPE const *globalReadA_0_0_47_0 = A + globalReadOffsetA_0_0_47_0;
  DATA_TYPE const *globalReadA_0_0_48_0 = A + globalReadOffsetA_0_0_48_0;
  DATA_TYPE const *globalReadA_0_0_49_0 = A + globalReadOffsetA_0_0_49_0;
  DATA_TYPE const *globalReadA_0_0_50_0 = A + globalReadOffsetA_0_0_50_0;
  DATA_TYPE const *globalReadA_0_0_51_0 = A + globalReadOffsetA_0_0_51_0;
  DATA_TYPE const *globalReadA_0_0_52_0 = A + globalReadOffsetA_0_0_52_0;
  DATA_TYPE const *globalReadA_0_0_53_0 = A + globalReadOffsetA_0_0_53_0;
  DATA_TYPE const *globalReadA_0_0_54_0 = A + globalReadOffsetA_0_0_54_0;
  DATA_TYPE const *globalReadA_0_0_55_0 = A + globalReadOffsetA_0_0_55_0;
  DATA_TYPE const *globalReadA_0_0_56_0 = A + globalReadOffsetA_0_0_56_0;
  DATA_TYPE const *globalReadA_0_0_57_0 = A + globalReadOffsetA_0_0_57_0;
  DATA_TYPE const *globalReadA_0_0_58_0 = A + globalReadOffsetA_0_0_58_0;
  DATA_TYPE const *globalReadA_0_0_59_0 = A + globalReadOffsetA_0_0_59_0;
  DATA_TYPE const *globalReadA_0_0_60_0 = A + globalReadOffsetA_0_0_60_0;
  DATA_TYPE const *globalReadA_0_0_61_0 = A + globalReadOffsetA_0_0_61_0;
  DATA_TYPE const *globalReadA_0_0_62_0 = A + globalReadOffsetA_0_0_62_0;
  DATA_TYPE const *globalReadA_0_0_63_0 = A + globalReadOffsetA_0_0_63_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 8*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 9*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 10*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 11*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 12*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 13*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 14*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 15*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_16_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 16*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_17_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 17*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_18_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 18*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_19_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 19*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_20_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 20*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_21_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 21*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_22_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 22*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_23_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 23*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_24_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 24*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_25_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 25*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_26_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 26*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_27_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 27*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_28_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 28*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_29_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 29*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_30_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 30*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_31_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 31*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_32_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 32*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_33_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 33*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_34_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 34*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_35_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 35*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_36_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 36*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_37_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 37*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_38_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 38*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_39_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 39*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_40_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 40*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_41_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 41*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_42_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 42*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_43_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 43*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_44_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 44*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_45_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 45*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_46_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 46*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_47_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 47*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_48_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 48*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_49_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 49*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_50_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 50*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_51_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 51*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_52_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 52*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_53_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 53*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_54_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 54*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_55_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 55*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_56_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 56*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_57_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 57*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_58_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 58*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_59_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 59*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_60_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 60*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_61_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 61*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_62_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 62*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_63_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 63*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_0_0_15_0;
  DATA_TYPE *localWriteA_0_0_16_0;
  DATA_TYPE *localWriteA_0_0_17_0;
  DATA_TYPE *localWriteA_0_0_18_0;
  DATA_TYPE *localWriteA_0_0_19_0;
  DATA_TYPE *localWriteA_0_0_20_0;
  DATA_TYPE *localWriteA_0_0_21_0;
  DATA_TYPE *localWriteA_0_0_22_0;
  DATA_TYPE *localWriteA_0_0_23_0;
  DATA_TYPE *localWriteA_0_0_24_0;
  DATA_TYPE *localWriteA_0_0_25_0;
  DATA_TYPE *localWriteA_0_0_26_0;
  DATA_TYPE *localWriteA_0_0_27_0;
  DATA_TYPE *localWriteA_0_0_28_0;
  DATA_TYPE *localWriteA_0_0_29_0;
  DATA_TYPE *localWriteA_0_0_30_0;
  DATA_TYPE *localWriteA_0_0_31_0;
  DATA_TYPE *localWriteA_0_0_32_0;
  DATA_TYPE *localWriteA_0_0_33_0;
  DATA_TYPE *localWriteA_0_0_34_0;
  DATA_TYPE *localWriteA_0_0_35_0;
  DATA_TYPE *localWriteA_0_0_36_0;
  DATA_TYPE *localWriteA_0_0_37_0;
  DATA_TYPE *localWriteA_0_0_38_0;
  DATA_TYPE *localWriteA_0_0_39_0;
  DATA_TYPE *localWriteA_0_0_40_0;
  DATA_TYPE *localWriteA_0_0_41_0;
  DATA_TYPE *localWriteA_0_0_42_0;
  DATA_TYPE *localWriteA_0_0_43_0;
  DATA_TYPE *localWriteA_0_0_44_0;
  DATA_TYPE *localWriteA_0_0_45_0;
  DATA_TYPE *localWriteA_0_0_46_0;
  DATA_TYPE *localWriteA_0_0_47_0;
  DATA_TYPE *localWriteA_0_0_48_0;
  DATA_TYPE *localWriteA_0_0_49_0;
  DATA_TYPE *localWriteA_0_0_50_0;
  DATA_TYPE *localWriteA_0_0_51_0;
  DATA_TYPE *localWriteA_0_0_52_0;
  DATA_TYPE *localWriteA_0_0_53_0;
  DATA_TYPE *localWriteA_0_0_54_0;
  DATA_TYPE *localWriteA_0_0_55_0;
  DATA_TYPE *localWriteA_0_0_56_0;
  DATA_TYPE *localWriteA_0_0_57_0;
  DATA_TYPE *localWriteA_0_0_58_0;
  DATA_TYPE *localWriteA_0_0_59_0;
  DATA_TYPE *localWriteA_0_0_60_0;
  DATA_TYPE *localWriteA_0_0_61_0;
  DATA_TYPE *localWriteA_0_0_62_0;
  DATA_TYPE *localWriteA_0_0_63_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);
  localWriteA_0_0_16_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_16_0);
  localWriteA_0_0_17_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_17_0);
  localWriteA_0_0_18_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_18_0);
  localWriteA_0_0_19_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_19_0);
  localWriteA_0_0_20_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_20_0);
  localWriteA_0_0_21_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_21_0);
  localWriteA_0_0_22_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_22_0);
  localWriteA_0_0_23_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_23_0);
  localWriteA_0_0_24_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_24_0);
  localWriteA_0_0_25_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_25_0);
  localWriteA_0_0_26_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_26_0);
  localWriteA_0_0_27_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_27_0);
  localWriteA_0_0_28_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_28_0);
  localWriteA_0_0_29_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_29_0);
  localWriteA_0_0_30_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_30_0);
  localWriteA_0_0_31_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_31_0);
  localWriteA_0_0_32_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_32_0);
  localWriteA_0_0_33_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_33_0);
  localWriteA_0_0_34_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_34_0);
  localWriteA_0_0_35_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_35_0);
  localWriteA_0_0_36_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_36_0);
  localWriteA_0_0_37_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_37_0);
  localWriteA_0_0_38_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_38_0);
  localWriteA_0_0_39_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_39_0);
  localWriteA_0_0_40_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_40_0);
  localWriteA_0_0_41_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_41_0);
  localWriteA_0_0_42_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_42_0);
  localWriteA_0_0_43_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_43_0);
  localWriteA_0_0_44_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_44_0);
  localWriteA_0_0_45_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_45_0);
  localWriteA_0_0_46_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_46_0);
  localWriteA_0_0_47_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_47_0);
  localWriteA_0_0_48_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_48_0);
  localWriteA_0_0_49_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_49_0);
  localWriteA_0_0_50_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_50_0);
  localWriteA_0_0_51_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_51_0);
  localWriteA_0_0_52_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_52_0);
  localWriteA_0_0_53_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_53_0);
  localWriteA_0_0_54_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_54_0);
  localWriteA_0_0_55_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_55_0);
  localWriteA_0_0_56_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_56_0);
  localWriteA_0_0_57_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_57_0);
  localWriteA_0_0_58_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_58_0);
  localWriteA_0_0_59_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_59_0);
  localWriteA_0_0_60_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_60_0);
  localWriteA_0_0_61_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_61_0);
  localWriteA_0_0_62_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_62_0);
  localWriteA_0_0_63_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_63_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);
    a_0_0_16_0 = *(globalReadA_0_0_16_0 + 0);
    a_0_0_17_0 = *(globalReadA_0_0_17_0 + 0);
    a_0_0_18_0 = *(globalReadA_0_0_18_0 + 0);
    a_0_0_19_0 = *(globalReadA_0_0_19_0 + 0);
    a_0_0_20_0 = *(globalReadA_0_0_20_0 + 0);
    a_0_0_21_0 = *(globalReadA_0_0_21_0 + 0);
    a_0_0_22_0 = *(globalReadA_0_0_22_0 + 0);
    a_0_0_23_0 = *(globalReadA_0_0_23_0 + 0);
    a_0_0_24_0 = *(globalReadA_0_0_24_0 + 0);
    a_0_0_25_0 = *(globalReadA_0_0_25_0 + 0);
    a_0_0_26_0 = *(globalReadA_0_0_26_0 + 0);
    a_0_0_27_0 = *(globalReadA_0_0_27_0 + 0);
    a_0_0_28_0 = *(globalReadA_0_0_28_0 + 0);
    a_0_0_29_0 = *(globalReadA_0_0_29_0 + 0);
    a_0_0_30_0 = *(globalReadA_0_0_30_0 + 0);
    a_0_0_31_0 = *(globalReadA_0_0_31_0 + 0);
    a_0_0_32_0 = *(globalReadA_0_0_32_0 + 0);
    a_0_0_33_0 = *(globalReadA_0_0_33_0 + 0);
    a_0_0_34_0 = *(globalReadA_0_0_34_0 + 0);
    a_0_0_35_0 = *(globalReadA_0_0_35_0 + 0);
    a_0_0_36_0 = *(globalReadA_0_0_36_0 + 0);
    a_0_0_37_0 = *(globalReadA_0_0_37_0 + 0);
    a_0_0_38_0 = *(globalReadA_0_0_38_0 + 0);
    a_0_0_39_0 = *(globalReadA_0_0_39_0 + 0);
    a_0_0_40_0 = *(globalReadA_0_0_40_0 + 0);
    a_0_0_41_0 = *(globalReadA_0_0_41_0 + 0);
    a_0_0_42_0 = *(globalReadA_0_0_42_0 + 0);
    a_0_0_43_0 = *(globalReadA_0_0_43_0 + 0);
    a_0_0_44_0 = *(globalReadA_0_0_44_0 + 0);
    a_0_0_45_0 = *(globalReadA_0_0_45_0 + 0);
    a_0_0_46_0 = *(globalReadA_0_0_46_0 + 0);
    a_0_0_47_0 = *(globalReadA_0_0_47_0 + 0);
    a_0_0_48_0 = *(globalReadA_0_0_48_0 + 0);
    a_0_0_49_0 = *(globalReadA_0_0_49_0 + 0);
    a_0_0_50_0 = *(globalReadA_0_0_50_0 + 0);
    a_0_0_51_0 = *(globalReadA_0_0_51_0 + 0);
    a_0_0_52_0 = *(globalReadA_0_0_52_0 + 0);
    a_0_0_53_0 = *(globalReadA_0_0_53_0 + 0);
    a_0_0_54_0 = *(globalReadA_0_0_54_0 + 0);
    a_0_0_55_0 = *(globalReadA_0_0_55_0 + 0);
    a_0_0_56_0 = *(globalReadA_0_0_56_0 + 0);
    a_0_0_57_0 = *(globalReadA_0_0_57_0 + 0);
    a_0_0_58_0 = *(globalReadA_0_0_58_0 + 0);
    a_0_0_59_0 = *(globalReadA_0_0_59_0 + 0);
    a_0_0_60_0 = *(globalReadA_0_0_60_0 + 0);
    a_0_0_61_0 = *(globalReadA_0_0_61_0 + 0);
    a_0_0_62_0 = *(globalReadA_0_0_62_0 + 0);
    a_0_0_63_0 = *(globalReadA_0_0_63_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    globalReadA_0_0_16_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_16_0) + 1*globalReadIncAL);
    globalReadA_0_0_17_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_17_0) + 1*globalReadIncAL);
    globalReadA_0_0_18_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_18_0) + 1*globalReadIncAL);
    globalReadA_0_0_19_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_19_0) + 1*globalReadIncAL);
    globalReadA_0_0_20_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_20_0) + 1*globalReadIncAL);
    globalReadA_0_0_21_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_21_0) + 1*globalReadIncAL);
    globalReadA_0_0_22_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_22_0) + 1*globalReadIncAL);
    globalReadA_0_0_23_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_23_0) + 1*globalReadIncAL);
    globalReadA_0_0_24_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_24_0) + 1*globalReadIncAL);
    globalReadA_0_0_25_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_25_0) + 1*globalReadIncAL);
    globalReadA_0_0_26_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_26_0) + 1*globalReadIncAL);
    globalReadA_0_0_27_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_27_0) + 1*globalReadIncAL);
    globalReadA_0_0_28_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_28_0) + 1*globalReadIncAL);
    globalReadA_0_0_29_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_29_0) + 1*globalReadIncAL);
    globalReadA_0_0_30_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_30_0) + 1*globalReadIncAL);
    globalReadA_0_0_31_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_31_0) + 1*globalReadIncAL);
    globalReadA_0_0_32_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_32_0) + 1*globalReadIncAL);
    globalReadA_0_0_33_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_33_0) + 1*globalReadIncAL);
    globalReadA_0_0_34_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_34_0) + 1*globalReadIncAL);
    globalReadA_0_0_35_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_35_0) + 1*globalReadIncAL);
    globalReadA_0_0_36_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_36_0) + 1*globalReadIncAL);
    globalReadA_0_0_37_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_37_0) + 1*globalReadIncAL);
    globalReadA_0_0_38_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_38_0) + 1*globalReadIncAL);
    globalReadA_0_0_39_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_39_0) + 1*globalReadIncAL);
    globalReadA_0_0_40_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_40_0) + 1*globalReadIncAL);
    globalReadA_0_0_41_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_41_0) + 1*globalReadIncAL);
    globalReadA_0_0_42_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_42_0) + 1*globalReadIncAL);
    globalReadA_0_0_43_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_43_0) + 1*globalReadIncAL);
    globalReadA_0_0_44_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_44_0) + 1*globalReadIncAL);
    globalReadA_0_0_45_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_45_0) + 1*globalReadIncAL);
    globalReadA_0_0_46_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_46_0) + 1*globalReadIncAL);
    globalReadA_0_0_47_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_47_0) + 1*globalReadIncAL);
    globalReadA_0_0_48_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_48_0) + 1*globalReadIncAL);
    globalReadA_0_0_49_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_49_0) + 1*globalReadIncAL);
    globalReadA_0_0_50_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_50_0) + 1*globalReadIncAL);
    globalReadA_0_0_51_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_51_0) + 1*globalReadIncAL);
    globalReadA_0_0_52_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_52_0) + 1*globalReadIncAL);
    globalReadA_0_0_53_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_53_0) + 1*globalReadIncAL);
    globalReadA_0_0_54_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_54_0) + 1*globalReadIncAL);
    globalReadA_0_0_55_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_55_0) + 1*globalReadIncAL);
    globalReadA_0_0_56_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_56_0) + 1*globalReadIncAL);
    globalReadA_0_0_57_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_57_0) + 1*globalReadIncAL);
    globalReadA_0_0_58_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_58_0) + 1*globalReadIncAL);
    globalReadA_0_0_59_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_59_0) + 1*globalReadIncAL);
    globalReadA_0_0_60_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_60_0) + 1*globalReadIncAL);
    globalReadA_0_0_61_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_61_0) + 1*globalReadIncAL);
    globalReadA_0_0_62_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_62_0) + 1*globalReadIncAL);
    globalReadA_0_0_63_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_63_0) + 1*globalReadIncAL);

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
    *(localWriteA_0_0_16_0 + 0) = a_0_0_16_0;
    *(localWriteA_0_0_17_0 + 0) = a_0_0_17_0;
    *(localWriteA_0_0_18_0 + 0) = a_0_0_18_0;
    *(localWriteA_0_0_19_0 + 0) = a_0_0_19_0;
    *(localWriteA_0_0_20_0 + 0) = a_0_0_20_0;
    *(localWriteA_0_0_21_0 + 0) = a_0_0_21_0;
    *(localWriteA_0_0_22_0 + 0) = a_0_0_22_0;
    *(localWriteA_0_0_23_0 + 0) = a_0_0_23_0;
    *(localWriteA_0_0_24_0 + 0) = a_0_0_24_0;
    *(localWriteA_0_0_25_0 + 0) = a_0_0_25_0;
    *(localWriteA_0_0_26_0 + 0) = a_0_0_26_0;
    *(localWriteA_0_0_27_0 + 0) = a_0_0_27_0;
    *(localWriteA_0_0_28_0 + 0) = a_0_0_28_0;
    *(localWriteA_0_0_29_0 + 0) = a_0_0_29_0;
    *(localWriteA_0_0_30_0 + 0) = a_0_0_30_0;
    *(localWriteA_0_0_31_0 + 0) = a_0_0_31_0;
    *(localWriteA_0_0_32_0 + 0) = a_0_0_32_0;
    *(localWriteA_0_0_33_0 + 0) = a_0_0_33_0;
    *(localWriteA_0_0_34_0 + 0) = a_0_0_34_0;
    *(localWriteA_0_0_35_0 + 0) = a_0_0_35_0;
    *(localWriteA_0_0_36_0 + 0) = a_0_0_36_0;
    *(localWriteA_0_0_37_0 + 0) = a_0_0_37_0;
    *(localWriteA_0_0_38_0 + 0) = a_0_0_38_0;
    *(localWriteA_0_0_39_0 + 0) = a_0_0_39_0;
    *(localWriteA_0_0_40_0 + 0) = a_0_0_40_0;
    *(localWriteA_0_0_41_0 + 0) = a_0_0_41_0;
    *(localWriteA_0_0_42_0 + 0) = a_0_0_42_0;
    *(localWriteA_0_0_43_0 + 0) = a_0_0_43_0;
    *(localWriteA_0_0_44_0 + 0) = a_0_0_44_0;
    *(localWriteA_0_0_45_0 + 0) = a_0_0_45_0;
    *(localWriteA_0_0_46_0 + 0) = a_0_0_46_0;
    *(localWriteA_0_0_47_0 + 0) = a_0_0_47_0;
    *(localWriteA_0_0_48_0 + 0) = a_0_0_48_0;
    *(localWriteA_0_0_49_0 + 0) = a_0_0_49_0;
    *(localWriteA_0_0_50_0 + 0) = a_0_0_50_0;
    *(localWriteA_0_0_51_0 + 0) = a_0_0_51_0;
    *(localWriteA_0_0_52_0 + 0) = a_0_0_52_0;
    *(localWriteA_0_0_53_0 + 0) = a_0_0_53_0;
    *(localWriteA_0_0_54_0 + 0) = a_0_0_54_0;
    *(localWriteA_0_0_55_0 + 0) = a_0_0_55_0;
    *(localWriteA_0_0_56_0 + 0) = a_0_0_56_0;
    *(localWriteA_0_0_57_0 + 0) = a_0_0_57_0;
    *(localWriteA_0_0_58_0 + 0) = a_0_0_58_0;
    *(localWriteA_0_0_59_0 + 0) = a_0_0_59_0;
    *(localWriteA_0_0_60_0 + 0) = a_0_0_60_0;
    *(localWriteA_0_0_61_0 + 0) = a_0_0_61_0;
    *(localWriteA_0_0_62_0 + 0) = a_0_0_62_0;
    *(localWriteA_0_0_63_0 + 0) = a_0_0_63_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 15 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 16 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 17 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 18 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 19 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 20 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 21 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 22 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 23 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 24 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 25 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 26 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 27 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 28 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 29 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 30 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 31 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 32 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 33 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 34 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 35 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 36 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 37 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 38 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 39 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 40 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 41 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 42 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 43 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 44 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 45 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 46 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 47 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 48 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 49 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 50 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 51 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 52 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 53 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 54 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 55 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 56 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 57 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 58 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 59 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 60 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 61 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 62 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_1x1


    /* iter 63 */

    MAC_1x1_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_0_8_0 = ( globalReadOffsetAL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_0_0_9_0 = ( globalReadOffsetAL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_0_0_10_0 = ( globalReadOffsetAL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_0_0_11_0 = ( globalReadOffsetAL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_0_0_12_0 = ( globalReadOffsetAL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_0_0_13_0 = ( globalReadOffsetAL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_0_0_14_0 = ( globalReadOffsetAL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_0_0_15_0 = ( globalReadOffsetAL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);
  a_0_0_16_0 = ( globalReadOffsetAL_16_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_16_0 + 0);
  a_0_0_17_0 = ( globalReadOffsetAL_17_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_17_0 + 0);
  a_0_0_18_0 = ( globalReadOffsetAL_18_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_18_0 + 0);
  a_0_0_19_0 = ( globalReadOffsetAL_19_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_19_0 + 0);
  a_0_0_20_0 = ( globalReadOffsetAL_20_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_20_0 + 0);
  a_0_0_21_0 = ( globalReadOffsetAL_21_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_21_0 + 0);
  a_0_0_22_0 = ( globalReadOffsetAL_22_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_22_0 + 0);
  a_0_0_23_0 = ( globalReadOffsetAL_23_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_23_0 + 0);
  a_0_0_24_0 = ( globalReadOffsetAL_24_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_24_0 + 0);
  a_0_0_25_0 = ( globalReadOffsetAL_25_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_25_0 + 0);
  a_0_0_26_0 = ( globalReadOffsetAL_26_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_26_0 + 0);
  a_0_0_27_0 = ( globalReadOffsetAL_27_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_27_0 + 0);
  a_0_0_28_0 = ( globalReadOffsetAL_28_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_28_0 + 0);
  a_0_0_29_0 = ( globalReadOffsetAL_29_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_29_0 + 0);
  a_0_0_30_0 = ( globalReadOffsetAL_30_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_30_0 + 0);
  a_0_0_31_0 = ( globalReadOffsetAL_31_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_31_0 + 0);
  a_0_0_32_0 = ( globalReadOffsetAL_32_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_32_0 + 0);
  a_0_0_33_0 = ( globalReadOffsetAL_33_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_33_0 + 0);
  a_0_0_34_0 = ( globalReadOffsetAL_34_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_34_0 + 0);
  a_0_0_35_0 = ( globalReadOffsetAL_35_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_35_0 + 0);
  a_0_0_36_0 = ( globalReadOffsetAL_36_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_36_0 + 0);
  a_0_0_37_0 = ( globalReadOffsetAL_37_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_37_0 + 0);
  a_0_0_38_0 = ( globalReadOffsetAL_38_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_38_0 + 0);
  a_0_0_39_0 = ( globalReadOffsetAL_39_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_39_0 + 0);
  a_0_0_40_0 = ( globalReadOffsetAL_40_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_40_0 + 0);
  a_0_0_41_0 = ( globalReadOffsetAL_41_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_41_0 + 0);
  a_0_0_42_0 = ( globalReadOffsetAL_42_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_42_0 + 0);
  a_0_0_43_0 = ( globalReadOffsetAL_43_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_43_0 + 0);
  a_0_0_44_0 = ( globalReadOffsetAL_44_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_44_0 + 0);
  a_0_0_45_0 = ( globalReadOffsetAL_45_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_45_0 + 0);
  a_0_0_46_0 = ( globalReadOffsetAL_46_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_46_0 + 0);
  a_0_0_47_0 = ( globalReadOffsetAL_47_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_47_0 + 0);
  a_0_0_48_0 = ( globalReadOffsetAL_48_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_48_0 + 0);
  a_0_0_49_0 = ( globalReadOffsetAL_49_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_49_0 + 0);
  a_0_0_50_0 = ( globalReadOffsetAL_50_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_50_0 + 0);
  a_0_0_51_0 = ( globalReadOffsetAL_51_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_51_0 + 0);
  a_0_0_52_0 = ( globalReadOffsetAL_52_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_52_0 + 0);
  a_0_0_53_0 = ( globalReadOffsetAL_53_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_53_0 + 0);
  a_0_0_54_0 = ( globalReadOffsetAL_54_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_54_0 + 0);
  a_0_0_55_0 = ( globalReadOffsetAL_55_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_55_0 + 0);
  a_0_0_56_0 = ( globalReadOffsetAL_56_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_56_0 + 0);
  a_0_0_57_0 = ( globalReadOffsetAL_57_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_57_0 + 0);
  a_0_0_58_0 = ( globalReadOffsetAL_58_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_58_0 + 0);
  a_0_0_59_0 = ( globalReadOffsetAL_59_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_59_0 + 0);
  a_0_0_60_0 = ( globalReadOffsetAL_60_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_60_0 + 0);
  a_0_0_61_0 = ( globalReadOffsetAL_61_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_61_0 + 0);
  a_0_0_62_0 = ( globalReadOffsetAL_62_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_62_0 + 0);
  a_0_0_63_0 = ( globalReadOffsetAL_63_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_63_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);
  localWriteA_0_0_16_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_16_0);
  localWriteA_0_0_17_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_17_0);
  localWriteA_0_0_18_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_18_0);
  localWriteA_0_0_19_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_19_0);
  localWriteA_0_0_20_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_20_0);
  localWriteA_0_0_21_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_21_0);
  localWriteA_0_0_22_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_22_0);
  localWriteA_0_0_23_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_23_0);
  localWriteA_0_0_24_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_24_0);
  localWriteA_0_0_25_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_25_0);
  localWriteA_0_0_26_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_26_0);
  localWriteA_0_0_27_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_27_0);
  localWriteA_0_0_28_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_28_0);
  localWriteA_0_0_29_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_29_0);
  localWriteA_0_0_30_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_30_0);
  localWriteA_0_0_31_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_31_0);
  localWriteA_0_0_32_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_32_0);
  localWriteA_0_0_33_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_33_0);
  localWriteA_0_0_34_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_34_0);
  localWriteA_0_0_35_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_35_0);
  localWriteA_0_0_36_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_36_0);
  localWriteA_0_0_37_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_37_0);
  localWriteA_0_0_38_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_38_0);
  localWriteA_0_0_39_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_39_0);
  localWriteA_0_0_40_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_40_0);
  localWriteA_0_0_41_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_41_0);
  localWriteA_0_0_42_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_42_0);
  localWriteA_0_0_43_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_43_0);
  localWriteA_0_0_44_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_44_0);
  localWriteA_0_0_45_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_45_0);
  localWriteA_0_0_46_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_46_0);
  localWriteA_0_0_47_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_47_0);
  localWriteA_0_0_48_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_48_0);
  localWriteA_0_0_49_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_49_0);
  localWriteA_0_0_50_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_50_0);
  localWriteA_0_0_51_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_51_0);
  localWriteA_0_0_52_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_52_0);
  localWriteA_0_0_53_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_53_0);
  localWriteA_0_0_54_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_54_0);
  localWriteA_0_0_55_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_55_0);
  localWriteA_0_0_56_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_56_0);
  localWriteA_0_0_57_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_57_0);
  localWriteA_0_0_58_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_58_0);
  localWriteA_0_0_59_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_59_0);
  localWriteA_0_0_60_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_60_0);
  localWriteA_0_0_61_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_61_0);
  localWriteA_0_0_62_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_62_0);
  localWriteA_0_0_63_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_63_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
  *(localWriteA_0_0_16_0 + 0) = a_0_0_16_0;
  *(localWriteA_0_0_17_0 + 0) = a_0_0_17_0;
  *(localWriteA_0_0_18_0 + 0) = a_0_0_18_0;
  *(localWriteA_0_0_19_0 + 0) = a_0_0_19_0;
  *(localWriteA_0_0_20_0 + 0) = a_0_0_20_0;
  *(localWriteA_0_0_21_0 + 0) = a_0_0_21_0;
  *(localWriteA_0_0_22_0 + 0) = a_0_0_22_0;
  *(localWriteA_0_0_23_0 + 0) = a_0_0_23_0;
  *(localWriteA_0_0_24_0 + 0) = a_0_0_24_0;
  *(localWriteA_0_0_25_0 + 0) = a_0_0_25_0;
  *(localWriteA_0_0_26_0 + 0) = a_0_0_26_0;
  *(localWriteA_0_0_27_0 + 0) = a_0_0_27_0;
  *(localWriteA_0_0_28_0 + 0) = a_0_0_28_0;
  *(localWriteA_0_0_29_0 + 0) = a_0_0_29_0;
  *(localWriteA_0_0_30_0 + 0) = a_0_0_30_0;
  *(localWriteA_0_0_31_0 + 0) = a_0_0_31_0;
  *(localWriteA_0_0_32_0 + 0) = a_0_0_32_0;
  *(localWriteA_0_0_33_0 + 0) = a_0_0_33_0;
  *(localWriteA_0_0_34_0 + 0) = a_0_0_34_0;
  *(localWriteA_0_0_35_0 + 0) = a_0_0_35_0;
  *(localWriteA_0_0_36_0 + 0) = a_0_0_36_0;
  *(localWriteA_0_0_37_0 + 0) = a_0_0_37_0;
  *(localWriteA_0_0_38_0 + 0) = a_0_0_38_0;
  *(localWriteA_0_0_39_0 + 0) = a_0_0_39_0;
  *(localWriteA_0_0_40_0 + 0) = a_0_0_40_0;
  *(localWriteA_0_0_41_0 + 0) = a_0_0_41_0;
  *(localWriteA_0_0_42_0 + 0) = a_0_0_42_0;
  *(localWriteA_0_0_43_0 + 0) = a_0_0_43_0;
  *(localWriteA_0_0_44_0 + 0) = a_0_0_44_0;
  *(localWriteA_0_0_45_0 + 0) = a_0_0_45_0;
  *(localWriteA_0_0_46_0 + 0) = a_0_0_46_0;
  *(localWriteA_0_0_47_0 + 0) = a_0_0_47_0;
  *(localWriteA_0_0_48_0 + 0) = a_0_0_48_0;
  *(localWriteA_0_0_49_0 + 0) = a_0_0_49_0;
  *(localWriteA_0_0_50_0 + 0) = a_0_0_50_0;
  *(localWriteA_0_0_51_0 + 0) = a_0_0_51_0;
  *(localWriteA_0_0_52_0 + 0) = a_0_0_52_0;
  *(localWriteA_0_0_53_0 + 0) = a_0_0_53_0;
  *(localWriteA_0_0_54_0 + 0) = a_0_0_54_0;
  *(localWriteA_0_0_55_0 + 0) = a_0_0_55_0;
  *(localWriteA_0_0_56_0 + 0) = a_0_0_56_0;
  *(localWriteA_0_0_57_0 + 0) = a_0_0_57_0;
  *(localWriteA_0_0_58_0 + 0) = a_0_0_58_0;
  *(localWriteA_0_0_59_0 + 0) = a_0_0_59_0;
  *(localWriteA_0_0_60_0 + 0) = a_0_0_60_0;
  *(localWriteA_0_0_61_0 + 0) = a_0_0_61_0;
  *(localWriteA_0_0_62_0 + 0) = a_0_0_62_0;
  *(localWriteA_0_0_63_0 + 0) = a_0_0_63_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_1x1

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_1x1
#undef MAC_1x1_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 64
#define SG1J 1
#define TT0I 1
#define TT1J 1
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 64
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 64
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 4096
#define LDS_NUM_ELEMENTS 4160

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? static_cast<tensile_bfloat16>((ALPHA)*(REG) + (BETA) * static_cast<float>(SRC)) : static_cast<tensile_bfloat16>((ALPHA)*(REG));

/* 1x1 micro-tile */
#define MAC_1x1\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_BBH_MT64x1x64_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU0_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT1_1_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG64_1_1_WGM1(
  tensile_bfloat16 *D,
  tensile_bfloat16 const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE a_0_0_16_0;
  DATA_TYPE a_0_0_17_0;
  DATA_TYPE a_0_0_18_0;
  DATA_TYPE a_0_0_19_0;
  DATA_TYPE a_0_0_20_0;
  DATA_TYPE a_0_0_21_0;
  DATA_TYPE a_0_0_22_0;
  DATA_TYPE a_0_0_23_0;
  DATA_TYPE a_0_0_24_0;
  DATA_TYPE a_0_0_25_0;
  DATA_TYPE a_0_0_26_0;
  DATA_TYPE a_0_0_27_0;
  DATA_TYPE a_0_0_28_0;
  DATA_TYPE a_0_0_29_0;
  DATA_TYPE a_0_0_30_0;
  DATA_TYPE a_0_0_31_0;
  DATA_TYPE a_0_0_32_0;
  DATA_TYPE a_0_0_33_0;
  DATA_TYPE a_0_0_34_0;
  DATA_TYPE a_0_0_35_0;
  DATA_TYPE a_0_0_36_0;
  DATA_TYPE a_0_0_37_0;
  DATA_TYPE a_0_0_38_0;
  DATA_TYPE a_0_0_39_0;
  DATA_TYPE a_0_0_40_0;
  DATA_TYPE a_0_0_41_0;
  DATA_TYPE a_0_0_42_0;
  DATA_TYPE a_0_0_43_0;
  DATA_TYPE a_0_0_44_0;
  DATA_TYPE a_0_0_45_0;
  DATA_TYPE a_0_0_46_0;
  DATA_TYPE a_0_0_47_0;
  DATA_TYPE a_0_0_48_0;
  DATA_TYPE a_0_0_49_0;
  DATA_TYPE a_0_0_50_0;
  DATA_TYPE a_0_0_51_0;
  DATA_TYPE a_0_0_52_0;
  DATA_TYPE a_0_0_53_0;
  DATA_TYPE a_0_0_54_0;
  DATA_TYPE a_0_0_55_0;
  DATA_TYPE a_0_0_56_0;
  DATA_TYPE a_0_0_57_0;
  DATA_TYPE a_0_0_58_0;
  DATA_TYPE a_0_0_59_0;
  DATA_TYPE a_0_0_60_0;
  DATA_TYPE a_0_0_61_0;
  DATA_TYPE a_0_0_62_0;
  DATA_TYPE a_0_0_63_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;
  unsigned int globalReadOffsetAL_8_0 = globalReadOffsetAL + 0 + 8*LSPA;
  unsigned int globalReadOffsetAL_9_0 = globalReadOffsetAL + 0 + 9*LSPA;
  unsigned int globalReadOffsetAL_10_0 = globalReadOffsetAL + 0 + 10*LSPA;
  unsigned int globalReadOffsetAL_11_0 = globalReadOffsetAL + 0 + 11*LSPA;
  unsigned int globalReadOffsetAL_12_0 = globalReadOffsetAL + 0 + 12*LSPA;
  unsigned int globalReadOffsetAL_13_0 = globalReadOffsetAL + 0 + 13*LSPA;
  unsigned int globalReadOffsetAL_14_0 = globalReadOffsetAL + 0 + 14*LSPA;
  unsigned int globalReadOffsetAL_15_0 = globalReadOffsetAL + 0 + 15*LSPA;
  unsigned int globalReadOffsetAL_16_0 = globalReadOffsetAL + 0 + 16*LSPA;
  unsigned int globalReadOffsetAL_17_0 = globalReadOffsetAL + 0 + 17*LSPA;
  unsigned int globalReadOffsetAL_18_0 = globalReadOffsetAL + 0 + 18*LSPA;
  unsigned int globalReadOffsetAL_19_0 = globalReadOffsetAL + 0 + 19*LSPA;
  unsigned int globalReadOffsetAL_20_0 = globalReadOffsetAL + 0 + 20*LSPA;
  unsigned int globalReadOffsetAL_21_0 = globalReadOffsetAL + 0 + 21*LSPA;
  unsigned int globalReadOffsetAL_22_0 = globalReadOffsetAL + 0 + 22*LSPA;
  unsigned int globalReadOffsetAL_23_0 = globalReadOffsetAL + 0 + 23*LSPA;
  unsigned int globalReadOffsetAL_24_0 = globalReadOffsetAL + 0 + 24*LSPA;
  unsigned int globalReadOffsetAL_25_0 = globalReadOffsetAL + 0 + 25*LSPA;
  unsigned int globalReadOffsetAL_26_0 = globalReadOffsetAL + 0 + 26*LSPA;
  unsigned int globalReadOffsetAL_27_0 = globalReadOffsetAL + 0 + 27*LSPA;
  unsigned int globalReadOffsetAL_28_0 = globalReadOffsetAL + 0 + 28*LSPA;
  unsigned int globalReadOffsetAL_29_0 = globalReadOffsetAL + 0 + 29*LSPA;
  unsigned int globalReadOffsetAL_30_0 = globalReadOffsetAL + 0 + 30*LSPA;
  unsigned int globalReadOffsetAL_31_0 = globalReadOffsetAL + 0 + 31*LSPA;
  unsigned int globalReadOffsetAL_32_0 = globalReadOffsetAL + 0 + 32*LSPA;
  unsigned int globalReadOffsetAL_33_0 = globalReadOffsetAL + 0 + 33*LSPA;
  unsigned int globalReadOffsetAL_34_0 = globalReadOffsetAL + 0 + 34*LSPA;
  unsigned int globalReadOffsetAL_35_0 = globalReadOffsetAL + 0 + 35*LSPA;
  unsigned int globalReadOffsetAL_36_0 = globalReadOffsetAL + 0 + 36*LSPA;
  unsigned int globalReadOffsetAL_37_0 = globalReadOffsetAL + 0 + 37*LSPA;
  unsigned int globalReadOffsetAL_38_0 = globalReadOffsetAL + 0 + 38*LSPA;
  unsigned int globalReadOffsetAL_39_0 = globalReadOffsetAL + 0 + 39*LSPA;
  unsigned int globalReadOffsetAL_40_0 = globalReadOffsetAL + 0 + 40*LSPA;
  unsigned int globalReadOffsetAL_41_0 = globalReadOffsetAL + 0 + 41*LSPA;
  unsigned int globalReadOffsetAL_42_0 = globalReadOffsetAL + 0 + 42*LSPA;
  unsigned int globalReadOffsetAL_43_0 = globalReadOffsetAL + 0 + 43*LSPA;
  unsigned int globalReadOffsetAL_44_0 = globalReadOffsetAL + 0 + 44*LSPA;
  unsigned int globalReadOffsetAL_45_0 = globalReadOffsetAL + 0 + 45*LSPA;
  unsigned int globalReadOffsetAL_46_0 = globalReadOffsetAL + 0 + 46*LSPA;
  unsigned int globalReadOffsetAL_47_0 = globalReadOffsetAL + 0 + 47*LSPA;
  unsigned int globalReadOffsetAL_48_0 = globalReadOffsetAL + 0 + 48*LSPA;
  unsigned int globalReadOffsetAL_49_0 = globalReadOffsetAL + 0 + 49*LSPA;
  unsigned int globalReadOffsetAL_50_0 = globalReadOffsetAL + 0 + 50*LSPA;
  unsigned int globalReadOffsetAL_51_0 = globalReadOffsetAL + 0 + 51*LSPA;
  unsigned int globalReadOffsetAL_52_0 = globalReadOffsetAL + 0 + 52*LSPA;
  unsigned int globalReadOffsetAL_53_0 = globalReadOffsetAL + 0 + 53*LSPA;
  unsigned int globalReadOffsetAL_54_0 = globalReadOffsetAL + 0 + 54*LSPA;
  unsigned int globalReadOffsetAL_55_0 = globalReadOffsetAL + 0 + 55*LSPA;
  unsigned int globalReadOffsetAL_56_0 = globalReadOffsetAL + 0 + 56*LSPA;
  unsigned int globalReadOffsetAL_57_0 = globalReadOffsetAL + 0 + 57*LSPA;
  unsigned int globalReadOffsetAL_58_0 = globalReadOffsetAL + 0 + 58*LSPA;
  unsigned int globalReadOffsetAL_59_0 = globalReadOffsetAL + 0 + 59*LSPA;
  unsigned int globalReadOffsetAL_60_0 = globalReadOffsetAL + 0 + 60*LSPA;
  unsigned int globalReadOffsetAL_61_0 = globalReadOffsetAL + 0 + 61*LSPA;
  unsigned int globalReadOffsetAL_62_0 = globalReadOffsetAL + 0 + 62*LSPA;
  unsigned int globalReadOffsetAL_63_0 = globalReadOffsetAL + 0 + 63*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_15_0), wgK );
  int64_t globalReadOffsetA_0_0_16_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_16_0), wgK );
  int64_t globalReadOffsetA_0_0_17_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_17_0), wgK );
  int64_t globalReadOffsetA_0_0_18_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_18_0), wgK );
  int64_t globalReadOffsetA_0_0_19_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_19_0), wgK );
  int64_t globalReadOffsetA_0_0_20_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_20_0), wgK );
  int64_t globalReadOffsetA_0_0_21_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_21_0), wgK );
  int64_t globalReadOffsetA_0_0_22_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_22_0), wgK );
  int64_t globalReadOffsetA_0_0_23_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_23_0), wgK );
  int64_t globalReadOffsetA_0_0_24_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_24_0), wgK );
  int64_t globalReadOffsetA_0_0_25_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_25_0), wgK );
  int64_t globalReadOffsetA_0_0_26_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_26_0), wgK );
  int64_t globalReadOffsetA_0_0_27_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_27_0), wgK );
  int64_t globalReadOffsetA_0_0_28_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_28_0), wgK );
  int64_t globalReadOffsetA_0_0_29_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_29_0), wgK );
  int64_t globalReadOffsetA_0_0_30_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_30_0), wgK );
  int64_t globalReadOffsetA_0_0_31_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_31_0), wgK );
  int64_t globalReadOffsetA_0_0_32_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_32_0), wgK );
  int64_t globalReadOffsetA_0_0_33_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_33_0), wgK );
  int64_t globalReadOffsetA_0_0_34_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_34_0), wgK );
  int64_t globalReadOffsetA_0_0_35_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_35_0), wgK );
  int64_t globalReadOffsetA_0_0_36_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_36_0), wgK );
  int64_t globalReadOffsetA_0_0_37_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_37_0), wgK );
  int64_t globalReadOffsetA_0_0_38_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_38_0), wgK );
  int64_t globalReadOffsetA_0_0_39_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_39_0), wgK );
  int64_t globalReadOffsetA_0_0_40_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_40_0), wgK );
  int64_t globalReadOffsetA_0_0_41_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_41_0), wgK );
  int64_t globalReadOffsetA_0_0_42_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_42_0), wgK );
  int64_t globalReadOffsetA_0_0_43_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_43_0), wgK );
  int64_t globalReadOffsetA_0_0_44_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_44_0), wgK );
  int64_t globalReadOffsetA_0_0_45_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_45_0), wgK );
  int64_t globalReadOffsetA_0_0_46_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_46_0), wgK );
  int64_t globalReadOffsetA_0_0_47_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_47_0), wgK );
  int64_t globalReadOffsetA_0_0_48_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_48_0), wgK );
  int64_t globalReadOffsetA_0_0_49_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_49_0), wgK );
  int64_t globalReadOffsetA_0_0_50_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_50_0), wgK );
  int64_t globalReadOffsetA_0_0_51_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_51_0), wgK );
  int64_t globalReadOffsetA_0_0_52_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_52_0), wgK );
  int64_t globalReadOffsetA_0_0_53_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_53_0), wgK );
  int64_t globalReadOffsetA_0_0_54_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_54_0), wgK );
  int64_t globalReadOffsetA_0_0_55_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_55_0), wgK );
  int64_t globalReadOffsetA_0_0_56_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_56_0), wgK );
  int64_t globalReadOffsetA_0_0_57_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_57_0), wgK );
  int64_t globalReadOffsetA_0_0_58_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_58_0), wgK );
  int64_t globalReadOffsetA_0_0_59_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_59_0), wgK );
  int64_t globalReadOffsetA_0_0_60_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_60_0), wgK );
  int64_t globalReadOffsetA_0_0_61_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_61_0), wgK );
  int64_t globalReadOffsetA_0_0_62_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_62_0), wgK );
  int64_t globalReadOffsetA_0_0_63_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_63_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;
  DATA_TYPE const *globalReadA_0_0_16_0 = A + globalReadOffsetA_0_0_16_0;
  DATA_TYPE const *globalReadA_0_0_17_0 = A + globalReadOffsetA_0_0_17_0;
  DATA_TYPE const *globalReadA_0_0_18_0 = A + globalReadOffsetA_0_0_18_0;
  DATA_TYPE const *globalReadA_0_0_19_0 = A + globalReadOffsetA_0_0_19_0;
  DATA_TYPE const *globalReadA_0_0_20_0 = A + globalReadOffsetA_0_0_20_0;
  DATA_TYPE const *globalReadA_0_0_21_0 = A + globalReadOffsetA_0_0_21_0;
  DATA_TYPE const *globalReadA_0_0_22_0 = A + globalReadOffsetA_0_0_22_0;
  DATA_TYPE const *globalReadA_0_0_23_0 = A + globalReadOffsetA_0_0_23_0;
  DATA_TYPE const *globalReadA_0_0_24_0 = A + globalReadOffsetA_0_0_24_0;
  DATA_TYPE const *globalReadA_0_0_25_0 = A + globalReadOffsetA_0_0_25_0;
  DATA_TYPE const *globalReadA_0_0_26_0 = A + globalReadOffsetA_0_0_26_0;
  DATA_TYPE const *globalReadA_0_0_27_0 = A + globalReadOffsetA_0_0_27_0;
  DATA_TYPE const *globalReadA_0_0_28_0 = A + globalReadOffsetA_0_0_28_0;
  DATA_TYPE const *globalReadA_0_0_29_0 = A + globalReadOffsetA_0_0_29_0;
  DATA_TYPE const *globalReadA_0_0_30_0 = A + globalReadOffsetA_0_0_30_0;
  DATA_TYPE const *globalReadA_0_0_31_0 = A + globalReadOffsetA_0_0_31_0;
  DATA_TYPE const *globalReadA_0_0_32_0 = A + globalReadOffsetA_0_0_32_0;
  DATA_TYPE const *globalReadA_0_0_33_0 = A + globalReadOffsetA_0_0_33_0;
  DATA_TYPE const *globalReadA_0_0_34_0 = A + globalReadOffsetA_0_0_34_0;
  DATA_TYPE const *globalReadA_0_0_35_0 = A + globalReadOffsetA_0_0_35_0;
  DATA_TYPE const *globalReadA_0_0_36_0 = A + globalReadOffsetA_0_0_36_0;
  DATA_TYPE const *globalReadA_0_0_37_0 = A + globalReadOffsetA_0_0_37_0;
  DATA_TYPE const *globalReadA_0_0_38_0 = A + globalReadOffsetA_0_0_38_0;
  DATA_TYPE const *globalReadA_0_0_39_0 = A + globalReadOffsetA_0_0_39_0;
  DATA_TYPE const *globalReadA_0_0_40_0 = A + globalReadOffsetA_0_0_40_0;
  DATA_TYPE const *globalReadA_0_0_41_0 = A + globalReadOffsetA_0_0_41_0;
  DATA_TYPE const *globalReadA_0_0_42_0 = A + globalReadOffsetA_0_0_42_0;
  DATA_TYPE const *globalReadA_0_0_43_0 = A + globalReadOffsetA_0_0_43_0;
  DATA_TYPE const *globalReadA_0_0_44_0 = A + globalReadOffsetA_0_0_44_0;
  DATA_TYPE const *globalReadA_0_0_45_0 = A + globalReadOffsetA_0_0_45_0;
  DATA_TYPE const *globalReadA_0_0_46_0 = A + globalReadOffsetA_0_0_46_0;
  DATA_TYPE const *globalReadA_0_0_47_0 = A + globalReadOffsetA_0_0_47_0;
  DATA_TYPE const *globalReadA_0_0_48_0 = A + globalReadOffsetA_0_0_48_0;
  DATA_TYPE const *globalReadA_0_0_49_0 = A + globalReadOffsetA_0_0_49_0;
  DATA_TYPE const *globalReadA_0_0_50_0 = A + globalReadOffsetA_0_0_50_0;
  DATA_TYPE const *globalReadA_0_0_51_0 = A + globalReadOffsetA_0_0_51_0;
  DATA_TYPE const *globalReadA_0_0_52_0 = A + globalReadOffsetA_0_0_52_0;
  DATA_TYPE const *globalReadA_0_0_53_0 = A + globalReadOffsetA_0_0_53_0;
  DATA_TYPE const *globalReadA_0_0_54_0 = A + globalReadOffsetA_0_0_54_0;
  DATA_TYPE const *globalReadA_0_0_55_0 = A + globalReadOffsetA_0_0_55_0;
  DATA_TYPE const *globalReadA_0_0_56_0 = A + globalReadOffsetA_0_0_56_0;
  DATA_TYPE const *globalReadA_0_0_57_0 = A + globalReadOffsetA_0_0_57_0;
  DATA_TYPE const *globalReadA_0_0_58_0 = A + globalReadOffsetA_0_0_58_0;
  DATA_TYPE const *globalReadA_0_0_59_0 = A + globalReadOffsetA_0_0_59_0;
  DATA_TYPE const *globalReadA_0_0_60_0 = A + globalReadOffsetA_0_0_60_0;
  DATA_TYPE const *globalReadA_0_0_61_0 = A + globalReadOffsetA_0_0_61_0;
  DATA_TYPE const *globalReadA_0_0_62_0 = A + globalReadOffsetA_0_0_62_0;
  DATA_TYPE const *globalReadA_0_0_63_0 = A + globalReadOffsetA_0_0_63_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 8*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 9*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 10*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 11*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 12*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 13*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 14*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 15*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_16_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 16*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_17_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 17*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_18_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 18*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_19_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 19*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_20_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 20*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_21_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 21*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_22_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 22*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_23_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 23*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_24_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 24*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_25_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 25*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_26_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 26*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_27_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 27*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_28_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 28*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_29_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 29*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_30_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 30*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_31_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 31*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_32_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 32*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_33_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 33*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_34_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 34*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_35_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 35*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_36_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 36*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_37_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 37*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_38_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 38*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_39_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 39*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_40_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 40*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_41_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 41*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_42_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 42*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_43_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 43*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_44_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 44*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_45_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 45*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_46_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 46*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_47_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 47*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_48_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 48*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_49_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 49*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_50_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 50*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_51_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 51*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_52_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 52*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_53_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 53*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_54_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 54*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_55_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 55*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_56_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 56*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_57_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 57*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_58_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 58*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_59_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 59*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_60_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 60*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_61_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 61*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_62_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 62*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_63_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 63*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_0_0_15_0;
  DATA_TYPE *localWriteA_0_0_16_0;
  DATA_TYPE *localWriteA_0_0_17_0;
  DATA_TYPE *localWriteA_0_0_18_0;
  DATA_TYPE *localWriteA_0_0_19_0;
  DATA_TYPE *localWriteA_0_0_20_0;
  DATA_TYPE *localWriteA_0_0_21_0;
  DATA_TYPE *localWriteA_0_0_22_0;
  DATA_TYPE *localWriteA_0_0_23_0;
  DATA_TYPE *localWriteA_0_0_24_0;
  DATA_TYPE *localWriteA_0_0_25_0;
  DATA_TYPE *localWriteA_0_0_26_0;
  DATA_TYPE *localWriteA_0_0_27_0;
  DATA_TYPE *localWriteA_0_0_28_0;
  DATA_TYPE *localWriteA_0_0_29_0;
  DATA_TYPE *localWriteA_0_0_30_0;
  DATA_TYPE *localWriteA_0_0_31_0;
  DATA_TYPE *localWriteA_0_0_32_0;
  DATA_TYPE *localWriteA_0_0_33_0;
  DATA_TYPE *localWriteA_0_0_34_0;
  DATA_TYPE *localWriteA_0_0_35_0;
  DATA_TYPE *localWriteA_0_0_36_0;
  DATA_TYPE *localWriteA_0_0_37_0;
  DATA_TYPE *localWriteA_0_0_38_0;
  DATA_TYPE *localWriteA_0_0_39_0;
  DATA_TYPE *localWriteA_0_0_40_0;
  DATA_TYPE *localWriteA_0_0_41_0;
  DATA_TYPE *localWriteA_0_0_42_0;
  DATA_TYPE *localWriteA_0_0_43_0;
  DATA_TYPE *localWriteA_0_0_44_0;
  DATA_TYPE *localWriteA_0_0_45_0;
  DATA_TYPE *localWriteA_0_0_46_0;
  DATA_TYPE *localWriteA_0_0_47_0;
  DATA_TYPE *localWriteA_0_0_48_0;
  DATA_TYPE *localWriteA_0_0_49_0;
  DATA_TYPE *localWriteA_0_0_50_0;
  DATA_TYPE *localWriteA_0_0_51_0;
  DATA_TYPE *localWriteA_0_0_52_0;
  DATA_TYPE *localWriteA_0_0_53_0;
  DATA_TYPE *localWriteA_0_0_54_0;
  DATA_TYPE *localWriteA_0_0_55_0;
  DATA_TYPE *localWriteA_0_0_56_0;
  DATA_TYPE *localWriteA_0_0_57_0;
  DATA_TYPE *localWriteA_0_0_58_0;
  DATA_TYPE *localWriteA_0_0_59_0;
  DATA_TYPE *localWriteA_0_0_60_0;
  DATA_TYPE *localWriteA_0_0_61_0;
  DATA_TYPE *localWriteA_0_0_62_0;
  DATA_TYPE *localWriteA_0_0_63_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);
  localWriteA_0_0_16_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_16_0);
  localWriteA_0_0_17_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_17_0);
  localWriteA_0_0_18_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_18_0);
  localWriteA_0_0_19_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_19_0);
  localWriteA_0_0_20_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_20_0);
  localWriteA_0_0_21_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_21_0);
  localWriteA_0_0_22_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_22_0);
  localWriteA_0_0_23_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_23_0);
  localWriteA_0_0_24_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_24_0);
  localWriteA_0_0_25_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_25_0);
  localWriteA_0_0_26_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_26_0);
  localWriteA_0_0_27_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_27_0);
  localWriteA_0_0_28_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_28_0);
  localWriteA_0_0_29_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_29_0);
  localWriteA_0_0_30_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_30_0);
  localWriteA_0_0_31_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_31_0);
  localWriteA_0_0_32_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_32_0);
  localWriteA_0_0_33_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_33_0);
  localWriteA_0_0_34_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_34_0);
  localWriteA_0_0_35_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_35_0);
  localWriteA_0_0_36_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_36_0);
  localWriteA_0_0_37_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_37_0);
  localWriteA_0_0_38_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_38_0);
  localWriteA_0_0_39_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_39_0);
  localWriteA_0_0_40_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_40_0);
  localWriteA_0_0_41_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_41_0);
  localWriteA_0_0_42_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_42_0);
  localWriteA_0_0_43_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_43_0);
  localWriteA_0_0_44_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_44_0);
  localWriteA_0_0_45_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_45_0);
  localWriteA_0_0_46_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_46_0);
  localWriteA_0_0_47_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_47_0);
  localWriteA_0_0_48_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_48_0);
  localWriteA_0_0_49_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_49_0);
  localWriteA_0_0_50_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_50_0);
  localWriteA_0_0_51_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_51_0);
  localWriteA_0_0_52_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_52_0);
  localWriteA_0_0_53_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_53_0);
  localWriteA_0_0_54_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_54_0);
  localWriteA_0_0_55_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_55_0);
  localWriteA_0_0_56_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_56_0);
  localWriteA_0_0_57_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_57_0);
  localWriteA_0_0_58_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_58_0);
  localWriteA_0_0_59_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_59_0);
  localWriteA_0_0_60_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_60_0);
  localWriteA_0_0_61_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_61_0);
  localWriteA_0_0_62_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_62_0);
  localWriteA_0_0_63_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_63_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);
    a_0_0_16_0 = *(globalReadA_0_0_16_0 + 0);
    a_0_0_17_0 = *(globalReadA_0_0_17_0 + 0);
    a_0_0_18_0 = *(globalReadA_0_0_18_0 + 0);
    a_0_0_19_0 = *(globalReadA_0_0_19_0 + 0);
    a_0_0_20_0 = *(globalReadA_0_0_20_0 + 0);
    a_0_0_21_0 = *(globalReadA_0_0_21_0 + 0);
    a_0_0_22_0 = *(globalReadA_0_0_22_0 + 0);
    a_0_0_23_0 = *(globalReadA_0_0_23_0 + 0);
    a_0_0_24_0 = *(globalReadA_0_0_24_0 + 0);
    a_0_0_25_0 = *(globalReadA_0_0_25_0 + 0);
    a_0_0_26_0 = *(globalReadA_0_0_26_0 + 0);
    a_0_0_27_0 = *(globalReadA_0_0_27_0 + 0);
    a_0_0_28_0 = *(globalReadA_0_0_28_0 + 0);
    a_0_0_29_0 = *(globalReadA_0_0_29_0 + 0);
    a_0_0_30_0 = *(globalReadA_0_0_30_0 + 0);
    a_0_0_31_0 = *(globalReadA_0_0_31_0 + 0);
    a_0_0_32_0 = *(globalReadA_0_0_32_0 + 0);
    a_0_0_33_0 = *(globalReadA_0_0_33_0 + 0);
    a_0_0_34_0 = *(globalReadA_0_0_34_0 + 0);
    a_0_0_35_0 = *(globalReadA_0_0_35_0 + 0);
    a_0_0_36_0 = *(globalReadA_0_0_36_0 + 0);
    a_0_0_37_0 = *(globalReadA_0_0_37_0 + 0);
    a_0_0_38_0 = *(globalReadA_0_0_38_0 + 0);
    a_0_0_39_0 = *(globalReadA_0_0_39_0 + 0);
    a_0_0_40_0 = *(globalReadA_0_0_40_0 + 0);
    a_0_0_41_0 = *(globalReadA_0_0_41_0 + 0);
    a_0_0_42_0 = *(globalReadA_0_0_42_0 + 0);
    a_0_0_43_0 = *(globalReadA_0_0_43_0 + 0);
    a_0_0_44_0 = *(globalReadA_0_0_44_0 + 0);
    a_0_0_45_0 = *(globalReadA_0_0_45_0 + 0);
    a_0_0_46_0 = *(globalReadA_0_0_46_0 + 0);
    a_0_0_47_0 = *(globalReadA_0_0_47_0 + 0);
    a_0_0_48_0 = *(globalReadA_0_0_48_0 + 0);
    a_0_0_49_0 = *(globalReadA_0_0_49_0 + 0);
    a_0_0_50_0 = *(globalReadA_0_0_50_0 + 0);
    a_0_0_51_0 = *(globalReadA_0_0_51_0 + 0);
    a_0_0_52_0 = *(globalReadA_0_0_52_0 + 0);
    a_0_0_53_0 = *(globalReadA_0_0_53_0 + 0);
    a_0_0_54_0 = *(globalReadA_0_0_54_0 + 0);
    a_0_0_55_0 = *(globalReadA_0_0_55_0 + 0);
    a_0_0_56_0 = *(globalReadA_0_0_56_0 + 0);
    a_0_0_57_0 = *(globalReadA_0_0_57_0 + 0);
    a_0_0_58_0 = *(globalReadA_0_0_58_0 + 0);
    a_0_0_59_0 = *(globalReadA_0_0_59_0 + 0);
    a_0_0_60_0 = *(globalReadA_0_0_60_0 + 0);
    a_0_0_61_0 = *(globalReadA_0_0_61_0 + 0);
    a_0_0_62_0 = *(globalReadA_0_0_62_0 + 0);
    a_0_0_63_0 = *(globalReadA_0_0_63_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    globalReadA_0_0_16_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_16_0) + 1*globalReadIncAL);
    globalReadA_0_0_17_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_17_0) + 1*globalReadIncAL);
    globalReadA_0_0_18_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_18_0) + 1*globalReadIncAL);
    globalReadA_0_0_19_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_19_0) + 1*globalReadIncAL);
    globalReadA_0_0_20_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_20_0) + 1*globalReadIncAL);
    globalReadA_0_0_21_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_21_0) + 1*globalReadIncAL);
    globalReadA_0_0_22_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_22_0) + 1*globalReadIncAL);
    globalReadA_0_0_23_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_23_0) + 1*globalReadIncAL);
    globalReadA_0_0_24_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_24_0) + 1*globalReadIncAL);
    globalReadA_0_0_25_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_25_0) + 1*globalReadIncAL);
    globalReadA_0_0_26_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_26_0) + 1*globalReadIncAL);
    globalReadA_0_0_27_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_27_0) + 1*globalReadIncAL);
    globalReadA_0_0_28_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_28_0) + 1*globalReadIncAL);
    globalReadA_0_0_29_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_29_0) + 1*globalReadIncAL);
    globalReadA_0_0_30_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_30_0) + 1*globalReadIncAL);
    globalReadA_0_0_31_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_31_0) + 1*globalReadIncAL);
    globalReadA_0_0_32_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_32_0) + 1*globalReadIncAL);
    globalReadA_0_0_33_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_33_0) + 1*globalReadIncAL);
    globalReadA_0_0_34_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_34_0) + 1*globalReadIncAL);
    globalReadA_0_0_35_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_35_0) + 1*globalReadIncAL);
    globalReadA_0_0_36_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_36_0) + 1*globalReadIncAL);
    globalReadA_0_0_37_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_37_0) + 1*globalReadIncAL);
    globalReadA_0_0_38_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_38_0) + 1*globalReadIncAL);
    globalReadA_0_0_39_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_39_0) + 1*globalReadIncAL);
    globalReadA_0_0_40_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_40_0) + 1*globalReadIncAL);
    globalReadA_0_0_41_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_41_0) + 1*globalReadIncAL);
    globalReadA_0_0_42_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_42_0) + 1*globalReadIncAL);
    globalReadA_0_0_43_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_43_0) + 1*globalReadIncAL);
    globalReadA_0_0_44_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_44_0) + 1*globalReadIncAL);
    globalReadA_0_0_45_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_45_0) + 1*globalReadIncAL);
    globalReadA_0_0_46_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_46_0) + 1*globalReadIncAL);
    globalReadA_0_0_47_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_47_0) + 1*globalReadIncAL);
    globalReadA_0_0_48_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_48_0) + 1*globalReadIncAL);
    globalReadA_0_0_49_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_49_0) + 1*globalReadIncAL);
    globalReadA_0_0_50_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_50_0) + 1*globalReadIncAL);
    globalReadA_0_0_51_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_51_0) + 1*globalReadIncAL);
    globalReadA_0_0_52_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_52_0) + 1*globalReadIncAL);
    globalReadA_0_0_53_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_53_0) + 1*globalReadIncAL);
    globalReadA_0_0_54_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_54_0) + 1*globalReadIncAL);
    globalReadA_0_0_55_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_55_0) + 1*globalReadIncAL);
    globalReadA_0_0_56_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_56_0) + 1*globalReadIncAL);
    globalReadA_0_0_57_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_57_0) + 1*globalReadIncAL);
    globalReadA_0_0_58_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_58_0) + 1*globalReadIncAL);
    globalReadA_0_0_59_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_59_0) + 1*globalReadIncAL);
    globalReadA_0_0_60_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_60_0) + 1*globalReadIncAL);
    globalReadA_0_0_61_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_61_0) + 1*globalReadIncAL);
    globalReadA_0_0_62_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_62_0) + 1*globalReadIncAL);
    globalReadA_0_0_63_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_63_0) + 1*globalReadIncAL);

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
    *(localWriteA_0_0_16_0 + 0) = a_0_0_16_0;
    *(localWriteA_0_0_17_0 + 0) = a_0_0_17_0;
    *(localWriteA_0_0_18_0 + 0) = a_0_0_18_0;
    *(localWriteA_0_0_19_0 + 0) = a_0_0_19_0;
    *(localWriteA_0_0_20_0 + 0) = a_0_0_20_0;
    *(localWriteA_0_0_21_0 + 0) = a_0_0_21_0;
    *(localWriteA_0_0_22_0 + 0) = a_0_0_22_0;
    *(localWriteA_0_0_23_0 + 0) = a_0_0_23_0;
    *(localWriteA_0_0_24_0 + 0) = a_0_0_24_0;
    *(localWriteA_0_0_25_0 + 0) = a_0_0_25_0;
    *(localWriteA_0_0_26_0 + 0) = a_0_0_26_0;
    *(localWriteA_0_0_27_0 + 0) = a_0_0_27_0;
    *(localWriteA_0_0_28_0 + 0) = a_0_0_28_0;
    *(localWriteA_0_0_29_0 + 0) = a_0_0_29_0;
    *(localWriteA_0_0_30_0 + 0) = a_0_0_30_0;
    *(localWriteA_0_0_31_0 + 0) = a_0_0_31_0;
    *(localWriteA_0_0_32_0 + 0) = a_0_0_32_0;
    *(localWriteA_0_0_33_0 + 0) = a_0_0_33_0;
    *(localWriteA_0_0_34_0 + 0) = a_0_0_34_0;
    *(localWriteA_0_0_35_0 + 0) = a_0_0_35_0;
    *(localWriteA_0_0_36_0 + 0) = a_0_0_36_0;
    *(localWriteA_0_0_37_0 + 0) = a_0_0_37_0;
    *(localWriteA_0_0_38_0 + 0) = a_0_0_38_0;
    *(localWriteA_0_0_39_0 + 0) = a_0_0_39_0;
    *(localWriteA_0_0_40_0 + 0) = a_0_0_40_0;
    *(localWriteA_0_0_41_0 + 0) = a_0_0_41_0;
    *(localWriteA_0_0_42_0 + 0) = a_0_0_42_0;
    *(localWriteA_0_0_43_0 + 0) = a_0_0_43_0;
    *(localWriteA_0_0_44_0 + 0) = a_0_0_44_0;
    *(localWriteA_0_0_45_0 + 0) = a_0_0_45_0;
    *(localWriteA_0_0_46_0 + 0) = a_0_0_46_0;
    *(localWriteA_0_0_47_0 + 0) = a_0_0_47_0;
    *(localWriteA_0_0_48_0 + 0) = a_0_0_48_0;
    *(localWriteA_0_0_49_0 + 0) = a_0_0_49_0;
    *(localWriteA_0_0_50_0 + 0) = a_0_0_50_0;
    *(localWriteA_0_0_51_0 + 0) = a_0_0_51_0;
    *(localWriteA_0_0_52_0 + 0) = a_0_0_52_0;
    *(localWriteA_0_0_53_0 + 0) = a_0_0_53_0;
    *(localWriteA_0_0_54_0 + 0) = a_0_0_54_0;
    *(localWriteA_0_0_55_0 + 0) = a_0_0_55_0;
    *(localWriteA_0_0_56_0 + 0) = a_0_0_56_0;
    *(localWriteA_0_0_57_0 + 0) = a_0_0_57_0;
    *(localWriteA_0_0_58_0 + 0) = a_0_0_58_0;
    *(localWriteA_0_0_59_0 + 0) = a_0_0_59_0;
    *(localWriteA_0_0_60_0 + 0) = a_0_0_60_0;
    *(localWriteA_0_0_61_0 + 0) = a_0_0_61_0;
    *(localWriteA_0_0_62_0 + 0) = a_0_0_62_0;
    *(localWriteA_0_0_63_0 + 0) = a_0_0_63_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 15 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 16 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 17 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 18 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 19 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 20 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 21 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 22 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 23 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 24 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 25 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 26 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 27 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 28 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 29 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 30 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 31 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 32 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 33 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 34 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 35 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 36 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 37 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 38 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 39 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 40 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 41 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 42 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 43 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 44 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 45 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 46 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 47 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 48 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 49 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 50 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 51 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 52 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 53 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 54 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 55 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 56 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 57 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 58 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 59 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 60 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 61 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 62 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 63 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_1x1




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_0_8_0 = ( globalReadOffsetAL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_0_0_9_0 = ( globalReadOffsetAL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_0_0_10_0 = ( globalReadOffsetAL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_0_0_11_0 = ( globalReadOffsetAL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_0_0_12_0 = ( globalReadOffsetAL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_0_0_13_0 = ( globalReadOffsetAL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_0_0_14_0 = ( globalReadOffsetAL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_0_0_15_0 = ( globalReadOffsetAL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);
  a_0_0_16_0 = ( globalReadOffsetAL_16_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_16_0 + 0);
  a_0_0_17_0 = ( globalReadOffsetAL_17_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_17_0 + 0);
  a_0_0_18_0 = ( globalReadOffsetAL_18_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_18_0 + 0);
  a_0_0_19_0 = ( globalReadOffsetAL_19_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_19_0 + 0);
  a_0_0_20_0 = ( globalReadOffsetAL_20_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_20_0 + 0);
  a_0_0_21_0 = ( globalReadOffsetAL_21_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_21_0 + 0);
  a_0_0_22_0 = ( globalReadOffsetAL_22_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_22_0 + 0);
  a_0_0_23_0 = ( globalReadOffsetAL_23_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_23_0 + 0);
  a_0_0_24_0 = ( globalReadOffsetAL_24_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_24_0 + 0);
  a_0_0_25_0 = ( globalReadOffsetAL_25_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_25_0 + 0);
  a_0_0_26_0 = ( globalReadOffsetAL_26_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_26_0 + 0);
  a_0_0_27_0 = ( globalReadOffsetAL_27_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_27_0 + 0);
  a_0_0_28_0 = ( globalReadOffsetAL_28_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_28_0 + 0);
  a_0_0_29_0 = ( globalReadOffsetAL_29_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_29_0 + 0);
  a_0_0_30_0 = ( globalReadOffsetAL_30_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_30_0 + 0);
  a_0_0_31_0 = ( globalReadOffsetAL_31_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_31_0 + 0);
  a_0_0_32_0 = ( globalReadOffsetAL_32_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_32_0 + 0);
  a_0_0_33_0 = ( globalReadOffsetAL_33_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_33_0 + 0);
  a_0_0_34_0 = ( globalReadOffsetAL_34_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_34_0 + 0);
  a_0_0_35_0 = ( globalReadOffsetAL_35_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_35_0 + 0);
  a_0_0_36_0 = ( globalReadOffsetAL_36_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_36_0 + 0);
  a_0_0_37_0 = ( globalReadOffsetAL_37_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_37_0 + 0);
  a_0_0_38_0 = ( globalReadOffsetAL_38_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_38_0 + 0);
  a_0_0_39_0 = ( globalReadOffsetAL_39_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_39_0 + 0);
  a_0_0_40_0 = ( globalReadOffsetAL_40_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_40_0 + 0);
  a_0_0_41_0 = ( globalReadOffsetAL_41_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_41_0 + 0);
  a_0_0_42_0 = ( globalReadOffsetAL_42_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_42_0 + 0);
  a_0_0_43_0 = ( globalReadOffsetAL_43_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_43_0 + 0);
  a_0_0_44_0 = ( globalReadOffsetAL_44_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_44_0 + 0);
  a_0_0_45_0 = ( globalReadOffsetAL_45_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_45_0 + 0);
  a_0_0_46_0 = ( globalReadOffsetAL_46_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_46_0 + 0);
  a_0_0_47_0 = ( globalReadOffsetAL_47_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_47_0 + 0);
  a_0_0_48_0 = ( globalReadOffsetAL_48_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_48_0 + 0);
  a_0_0_49_0 = ( globalReadOffsetAL_49_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_49_0 + 0);
  a_0_0_50_0 = ( globalReadOffsetAL_50_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_50_0 + 0);
  a_0_0_51_0 = ( globalReadOffsetAL_51_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_51_0 + 0);
  a_0_0_52_0 = ( globalReadOffsetAL_52_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_52_0 + 0);
  a_0_0_53_0 = ( globalReadOffsetAL_53_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_53_0 + 0);
  a_0_0_54_0 = ( globalReadOffsetAL_54_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_54_0 + 0);
  a_0_0_55_0 = ( globalReadOffsetAL_55_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_55_0 + 0);
  a_0_0_56_0 = ( globalReadOffsetAL_56_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_56_0 + 0);
  a_0_0_57_0 = ( globalReadOffsetAL_57_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_57_0 + 0);
  a_0_0_58_0 = ( globalReadOffsetAL_58_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_58_0 + 0);
  a_0_0_59_0 = ( globalReadOffsetAL_59_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_59_0 + 0);
  a_0_0_60_0 = ( globalReadOffsetAL_60_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_60_0 + 0);
  a_0_0_61_0 = ( globalReadOffsetAL_61_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_61_0 + 0);
  a_0_0_62_0 = ( globalReadOffsetAL_62_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_62_0 + 0);
  a_0_0_63_0 = ( globalReadOffsetAL_63_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_63_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);
  localWriteA_0_0_16_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_16_0);
  localWriteA_0_0_17_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_17_0);
  localWriteA_0_0_18_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_18_0);
  localWriteA_0_0_19_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_19_0);
  localWriteA_0_0_20_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_20_0);
  localWriteA_0_0_21_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_21_0);
  localWriteA_0_0_22_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_22_0);
  localWriteA_0_0_23_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_23_0);
  localWriteA_0_0_24_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_24_0);
  localWriteA_0_0_25_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_25_0);
  localWriteA_0_0_26_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_26_0);
  localWriteA_0_0_27_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_27_0);
  localWriteA_0_0_28_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_28_0);
  localWriteA_0_0_29_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_29_0);
  localWriteA_0_0_30_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_30_0);
  localWriteA_0_0_31_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_31_0);
  localWriteA_0_0_32_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_32_0);
  localWriteA_0_0_33_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_33_0);
  localWriteA_0_0_34_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_34_0);
  localWriteA_0_0_35_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_35_0);
  localWriteA_0_0_36_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_36_0);
  localWriteA_0_0_37_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_37_0);
  localWriteA_0_0_38_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_38_0);
  localWriteA_0_0_39_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_39_0);
  localWriteA_0_0_40_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_40_0);
  localWriteA_0_0_41_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_41_0);
  localWriteA_0_0_42_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_42_0);
  localWriteA_0_0_43_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_43_0);
  localWriteA_0_0_44_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_44_0);
  localWriteA_0_0_45_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_45_0);
  localWriteA_0_0_46_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_46_0);
  localWriteA_0_0_47_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_47_0);
  localWriteA_0_0_48_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_48_0);
  localWriteA_0_0_49_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_49_0);
  localWriteA_0_0_50_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_50_0);
  localWriteA_0_0_51_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_51_0);
  localWriteA_0_0_52_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_52_0);
  localWriteA_0_0_53_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_53_0);
  localWriteA_0_0_54_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_54_0);
  localWriteA_0_0_55_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_55_0);
  localWriteA_0_0_56_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_56_0);
  localWriteA_0_0_57_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_57_0);
  localWriteA_0_0_58_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_58_0);
  localWriteA_0_0_59_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_59_0);
  localWriteA_0_0_60_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_60_0);
  localWriteA_0_0_61_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_61_0);
  localWriteA_0_0_62_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_62_0);
  localWriteA_0_0_63_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_63_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
  *(localWriteA_0_0_16_0 + 0) = a_0_0_16_0;
  *(localWriteA_0_0_17_0 + 0) = a_0_0_17_0;
  *(localWriteA_0_0_18_0 + 0) = a_0_0_18_0;
  *(localWriteA_0_0_19_0 + 0) = a_0_0_19_0;
  *(localWriteA_0_0_20_0 + 0) = a_0_0_20_0;
  *(localWriteA_0_0_21_0 + 0) = a_0_0_21_0;
  *(localWriteA_0_0_22_0 + 0) = a_0_0_22_0;
  *(localWriteA_0_0_23_0 + 0) = a_0_0_23_0;
  *(localWriteA_0_0_24_0 + 0) = a_0_0_24_0;
  *(localWriteA_0_0_25_0 + 0) = a_0_0_25_0;
  *(localWriteA_0_0_26_0 + 0) = a_0_0_26_0;
  *(localWriteA_0_0_27_0 + 0) = a_0_0_27_0;
  *(localWriteA_0_0_28_0 + 0) = a_0_0_28_0;
  *(localWriteA_0_0_29_0 + 0) = a_0_0_29_0;
  *(localWriteA_0_0_30_0 + 0) = a_0_0_30_0;
  *(localWriteA_0_0_31_0 + 0) = a_0_0_31_0;
  *(localWriteA_0_0_32_0 + 0) = a_0_0_32_0;
  *(localWriteA_0_0_33_0 + 0) = a_0_0_33_0;
  *(localWriteA_0_0_34_0 + 0) = a_0_0_34_0;
  *(localWriteA_0_0_35_0 + 0) = a_0_0_35_0;
  *(localWriteA_0_0_36_0 + 0) = a_0_0_36_0;
  *(localWriteA_0_0_37_0 + 0) = a_0_0_37_0;
  *(localWriteA_0_0_38_0 + 0) = a_0_0_38_0;
  *(localWriteA_0_0_39_0 + 0) = a_0_0_39_0;
  *(localWriteA_0_0_40_0 + 0) = a_0_0_40_0;
  *(localWriteA_0_0_41_0 + 0) = a_0_0_41_0;
  *(localWriteA_0_0_42_0 + 0) = a_0_0_42_0;
  *(localWriteA_0_0_43_0 + 0) = a_0_0_43_0;
  *(localWriteA_0_0_44_0 + 0) = a_0_0_44_0;
  *(localWriteA_0_0_45_0 + 0) = a_0_0_45_0;
  *(localWriteA_0_0_46_0 + 0) = a_0_0_46_0;
  *(localWriteA_0_0_47_0 + 0) = a_0_0_47_0;
  *(localWriteA_0_0_48_0 + 0) = a_0_0_48_0;
  *(localWriteA_0_0_49_0 + 0) = a_0_0_49_0;
  *(localWriteA_0_0_50_0 + 0) = a_0_0_50_0;
  *(localWriteA_0_0_51_0 + 0) = a_0_0_51_0;
  *(localWriteA_0_0_52_0 + 0) = a_0_0_52_0;
  *(localWriteA_0_0_53_0 + 0) = a_0_0_53_0;
  *(localWriteA_0_0_54_0 + 0) = a_0_0_54_0;
  *(localWriteA_0_0_55_0 + 0) = a_0_0_55_0;
  *(localWriteA_0_0_56_0 + 0) = a_0_0_56_0;
  *(localWriteA_0_0_57_0 + 0) = a_0_0_57_0;
  *(localWriteA_0_0_58_0 + 0) = a_0_0_58_0;
  *(localWriteA_0_0_59_0 + 0) = a_0_0_59_0;
  *(localWriteA_0_0_60_0 + 0) = a_0_0_60_0;
  *(localWriteA_0_0_61_0 + 0) = a_0_0_61_0;
  *(localWriteA_0_0_62_0 + 0) = a_0_0_62_0;
  *(localWriteA_0_0_63_0 + 0) = a_0_0_63_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_1x1

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_1x1
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 64
#define SG1J 1
#define TT0I 1
#define TT1J 1
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 64
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 64
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 4096
#define LDS_NUM_ELEMENTS 4160

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 1x1 micro-tile */
#define MAC_1x1\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_BBH_MT64x1x64_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU0_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT1_1_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG64_1_1_WGM1(
  float *D,
  float const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE a_0_0_16_0;
  DATA_TYPE a_0_0_17_0;
  DATA_TYPE a_0_0_18_0;
  DATA_TYPE a_0_0_19_0;
  DATA_TYPE a_0_0_20_0;
  DATA_TYPE a_0_0_21_0;
  DATA_TYPE a_0_0_22_0;
  DATA_TYPE a_0_0_23_0;
  DATA_TYPE a_0_0_24_0;
  DATA_TYPE a_0_0_25_0;
  DATA_TYPE a_0_0_26_0;
  DATA_TYPE a_0_0_27_0;
  DATA_TYPE a_0_0_28_0;
  DATA_TYPE a_0_0_29_0;
  DATA_TYPE a_0_0_30_0;
  DATA_TYPE a_0_0_31_0;
  DATA_TYPE a_0_0_32_0;
  DATA_TYPE a_0_0_33_0;
  DATA_TYPE a_0_0_34_0;
  DATA_TYPE a_0_0_35_0;
  DATA_TYPE a_0_0_36_0;
  DATA_TYPE a_0_0_37_0;
  DATA_TYPE a_0_0_38_0;
  DATA_TYPE a_0_0_39_0;
  DATA_TYPE a_0_0_40_0;
  DATA_TYPE a_0_0_41_0;
  DATA_TYPE a_0_0_42_0;
  DATA_TYPE a_0_0_43_0;
  DATA_TYPE a_0_0_44_0;
  DATA_TYPE a_0_0_45_0;
  DATA_TYPE a_0_0_46_0;
  DATA_TYPE a_0_0_47_0;
  DATA_TYPE a_0_0_48_0;
  DATA_TYPE a_0_0_49_0;
  DATA_TYPE a_0_0_50_0;
  DATA_TYPE a_0_0_51_0;
  DATA_TYPE a_0_0_52_0;
  DATA_TYPE a_0_0_53_0;
  DATA_TYPE a_0_0_54_0;
  DATA_TYPE a_0_0_55_0;
  DATA_TYPE a_0_0_56_0;
  DATA_TYPE a_0_0_57_0;
  DATA_TYPE a_0_0_58_0;
  DATA_TYPE a_0_0_59_0;
  DATA_TYPE a_0_0_60_0;
  DATA_TYPE a_0_0_61_0;
  DATA_TYPE a_0_0_62_0;
  DATA_TYPE a_0_0_63_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;
  unsigned int globalReadOffsetAL_8_0 = globalReadOffsetAL + 0 + 8*LSPA;
  unsigned int globalReadOffsetAL_9_0 = globalReadOffsetAL + 0 + 9*LSPA;
  unsigned int globalReadOffsetAL_10_0 = globalReadOffsetAL + 0 + 10*LSPA;
  unsigned int globalReadOffsetAL_11_0 = globalReadOffsetAL + 0 + 11*LSPA;
  unsigned int globalReadOffsetAL_12_0 = globalReadOffsetAL + 0 + 12*LSPA;
  unsigned int globalReadOffsetAL_13_0 = globalReadOffsetAL + 0 + 13*LSPA;
  unsigned int globalReadOffsetAL_14_0 = globalReadOffsetAL + 0 + 14*LSPA;
  unsigned int globalReadOffsetAL_15_0 = globalReadOffsetAL + 0 + 15*LSPA;
  unsigned int globalReadOffsetAL_16_0 = globalReadOffsetAL + 0 + 16*LSPA;
  unsigned int globalReadOffsetAL_17_0 = globalReadOffsetAL + 0 + 17*LSPA;
  unsigned int globalReadOffsetAL_18_0 = globalReadOffsetAL + 0 + 18*LSPA;
  unsigned int globalReadOffsetAL_19_0 = globalReadOffsetAL + 0 + 19*LSPA;
  unsigned int globalReadOffsetAL_20_0 = globalReadOffsetAL + 0 + 20*LSPA;
  unsigned int globalReadOffsetAL_21_0 = globalReadOffsetAL + 0 + 21*LSPA;
  unsigned int globalReadOffsetAL_22_0 = globalReadOffsetAL + 0 + 22*LSPA;
  unsigned int globalReadOffsetAL_23_0 = globalReadOffsetAL + 0 + 23*LSPA;
  unsigned int globalReadOffsetAL_24_0 = globalReadOffsetAL + 0 + 24*LSPA;
  unsigned int globalReadOffsetAL_25_0 = globalReadOffsetAL + 0 + 25*LSPA;
  unsigned int globalReadOffsetAL_26_0 = globalReadOffsetAL + 0 + 26*LSPA;
  unsigned int globalReadOffsetAL_27_0 = globalReadOffsetAL + 0 + 27*LSPA;
  unsigned int globalReadOffsetAL_28_0 = globalReadOffsetAL + 0 + 28*LSPA;
  unsigned int globalReadOffsetAL_29_0 = globalReadOffsetAL + 0 + 29*LSPA;
  unsigned int globalReadOffsetAL_30_0 = globalReadOffsetAL + 0 + 30*LSPA;
  unsigned int globalReadOffsetAL_31_0 = globalReadOffsetAL + 0 + 31*LSPA;
  unsigned int globalReadOffsetAL_32_0 = globalReadOffsetAL + 0 + 32*LSPA;
  unsigned int globalReadOffsetAL_33_0 = globalReadOffsetAL + 0 + 33*LSPA;
  unsigned int globalReadOffsetAL_34_0 = globalReadOffsetAL + 0 + 34*LSPA;
  unsigned int globalReadOffsetAL_35_0 = globalReadOffsetAL + 0 + 35*LSPA;
  unsigned int globalReadOffsetAL_36_0 = globalReadOffsetAL + 0 + 36*LSPA;
  unsigned int globalReadOffsetAL_37_0 = globalReadOffsetAL + 0 + 37*LSPA;
  unsigned int globalReadOffsetAL_38_0 = globalReadOffsetAL + 0 + 38*LSPA;
  unsigned int globalReadOffsetAL_39_0 = globalReadOffsetAL + 0 + 39*LSPA;
  unsigned int globalReadOffsetAL_40_0 = globalReadOffsetAL + 0 + 40*LSPA;
  unsigned int globalReadOffsetAL_41_0 = globalReadOffsetAL + 0 + 41*LSPA;
  unsigned int globalReadOffsetAL_42_0 = globalReadOffsetAL + 0 + 42*LSPA;
  unsigned int globalReadOffsetAL_43_0 = globalReadOffsetAL + 0 + 43*LSPA;
  unsigned int globalReadOffsetAL_44_0 = globalReadOffsetAL + 0 + 44*LSPA;
  unsigned int globalReadOffsetAL_45_0 = globalReadOffsetAL + 0 + 45*LSPA;
  unsigned int globalReadOffsetAL_46_0 = globalReadOffsetAL + 0 + 46*LSPA;
  unsigned int globalReadOffsetAL_47_0 = globalReadOffsetAL + 0 + 47*LSPA;
  unsigned int globalReadOffsetAL_48_0 = globalReadOffsetAL + 0 + 48*LSPA;
  unsigned int globalReadOffsetAL_49_0 = globalReadOffsetAL + 0 + 49*LSPA;
  unsigned int globalReadOffsetAL_50_0 = globalReadOffsetAL + 0 + 50*LSPA;
  unsigned int globalReadOffsetAL_51_0 = globalReadOffsetAL + 0 + 51*LSPA;
  unsigned int globalReadOffsetAL_52_0 = globalReadOffsetAL + 0 + 52*LSPA;
  unsigned int globalReadOffsetAL_53_0 = globalReadOffsetAL + 0 + 53*LSPA;
  unsigned int globalReadOffsetAL_54_0 = globalReadOffsetAL + 0 + 54*LSPA;
  unsigned int globalReadOffsetAL_55_0 = globalReadOffsetAL + 0 + 55*LSPA;
  unsigned int globalReadOffsetAL_56_0 = globalReadOffsetAL + 0 + 56*LSPA;
  unsigned int globalReadOffsetAL_57_0 = globalReadOffsetAL + 0 + 57*LSPA;
  unsigned int globalReadOffsetAL_58_0 = globalReadOffsetAL + 0 + 58*LSPA;
  unsigned int globalReadOffsetAL_59_0 = globalReadOffsetAL + 0 + 59*LSPA;
  unsigned int globalReadOffsetAL_60_0 = globalReadOffsetAL + 0 + 60*LSPA;
  unsigned int globalReadOffsetAL_61_0 = globalReadOffsetAL + 0 + 61*LSPA;
  unsigned int globalReadOffsetAL_62_0 = globalReadOffsetAL + 0 + 62*LSPA;
  unsigned int globalReadOffsetAL_63_0 = globalReadOffsetAL + 0 + 63*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_15_0), wgK );
  int64_t globalReadOffsetA_0_0_16_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_16_0), wgK );
  int64_t globalReadOffsetA_0_0_17_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_17_0), wgK );
  int64_t globalReadOffsetA_0_0_18_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_18_0), wgK );
  int64_t globalReadOffsetA_0_0_19_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_19_0), wgK );
  int64_t globalReadOffsetA_0_0_20_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_20_0), wgK );
  int64_t globalReadOffsetA_0_0_21_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_21_0), wgK );
  int64_t globalReadOffsetA_0_0_22_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_22_0), wgK );
  int64_t globalReadOffsetA_0_0_23_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_23_0), wgK );
  int64_t globalReadOffsetA_0_0_24_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_24_0), wgK );
  int64_t globalReadOffsetA_0_0_25_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_25_0), wgK );
  int64_t globalReadOffsetA_0_0_26_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_26_0), wgK );
  int64_t globalReadOffsetA_0_0_27_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_27_0), wgK );
  int64_t globalReadOffsetA_0_0_28_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_28_0), wgK );
  int64_t globalReadOffsetA_0_0_29_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_29_0), wgK );
  int64_t globalReadOffsetA_0_0_30_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_30_0), wgK );
  int64_t globalReadOffsetA_0_0_31_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_31_0), wgK );
  int64_t globalReadOffsetA_0_0_32_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_32_0), wgK );
  int64_t globalReadOffsetA_0_0_33_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_33_0), wgK );
  int64_t globalReadOffsetA_0_0_34_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_34_0), wgK );
  int64_t globalReadOffsetA_0_0_35_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_35_0), wgK );
  int64_t globalReadOffsetA_0_0_36_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_36_0), wgK );
  int64_t globalReadOffsetA_0_0_37_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_37_0), wgK );
  int64_t globalReadOffsetA_0_0_38_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_38_0), wgK );
  int64_t globalReadOffsetA_0_0_39_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_39_0), wgK );
  int64_t globalReadOffsetA_0_0_40_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_40_0), wgK );
  int64_t globalReadOffsetA_0_0_41_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_41_0), wgK );
  int64_t globalReadOffsetA_0_0_42_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_42_0), wgK );
  int64_t globalReadOffsetA_0_0_43_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_43_0), wgK );
  int64_t globalReadOffsetA_0_0_44_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_44_0), wgK );
  int64_t globalReadOffsetA_0_0_45_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_45_0), wgK );
  int64_t globalReadOffsetA_0_0_46_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_46_0), wgK );
  int64_t globalReadOffsetA_0_0_47_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_47_0), wgK );
  int64_t globalReadOffsetA_0_0_48_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_48_0), wgK );
  int64_t globalReadOffsetA_0_0_49_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_49_0), wgK );
  int64_t globalReadOffsetA_0_0_50_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_50_0), wgK );
  int64_t globalReadOffsetA_0_0_51_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_51_0), wgK );
  int64_t globalReadOffsetA_0_0_52_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_52_0), wgK );
  int64_t globalReadOffsetA_0_0_53_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_53_0), wgK );
  int64_t globalReadOffsetA_0_0_54_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_54_0), wgK );
  int64_t globalReadOffsetA_0_0_55_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_55_0), wgK );
  int64_t globalReadOffsetA_0_0_56_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_56_0), wgK );
  int64_t globalReadOffsetA_0_0_57_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_57_0), wgK );
  int64_t globalReadOffsetA_0_0_58_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_58_0), wgK );
  int64_t globalReadOffsetA_0_0_59_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_59_0), wgK );
  int64_t globalReadOffsetA_0_0_60_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_60_0), wgK );
  int64_t globalReadOffsetA_0_0_61_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_61_0), wgK );
  int64_t globalReadOffsetA_0_0_62_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_62_0), wgK );
  int64_t globalReadOffsetA_0_0_63_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_63_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;
  DATA_TYPE const *globalReadA_0_0_16_0 = A + globalReadOffsetA_0_0_16_0;
  DATA_TYPE const *globalReadA_0_0_17_0 = A + globalReadOffsetA_0_0_17_0;
  DATA_TYPE const *globalReadA_0_0_18_0 = A + globalReadOffsetA_0_0_18_0;
  DATA_TYPE const *globalReadA_0_0_19_0 = A + globalReadOffsetA_0_0_19_0;
  DATA_TYPE const *globalReadA_0_0_20_0 = A + globalReadOffsetA_0_0_20_0;
  DATA_TYPE const *globalReadA_0_0_21_0 = A + globalReadOffsetA_0_0_21_0;
  DATA_TYPE const *globalReadA_0_0_22_0 = A + globalReadOffsetA_0_0_22_0;
  DATA_TYPE const *globalReadA_0_0_23_0 = A + globalReadOffsetA_0_0_23_0;
  DATA_TYPE const *globalReadA_0_0_24_0 = A + globalReadOffsetA_0_0_24_0;
  DATA_TYPE const *globalReadA_0_0_25_0 = A + globalReadOffsetA_0_0_25_0;
  DATA_TYPE const *globalReadA_0_0_26_0 = A + globalReadOffsetA_0_0_26_0;
  DATA_TYPE const *globalReadA_0_0_27_0 = A + globalReadOffsetA_0_0_27_0;
  DATA_TYPE const *globalReadA_0_0_28_0 = A + globalReadOffsetA_0_0_28_0;
  DATA_TYPE const *globalReadA_0_0_29_0 = A + globalReadOffsetA_0_0_29_0;
  DATA_TYPE const *globalReadA_0_0_30_0 = A + globalReadOffsetA_0_0_30_0;
  DATA_TYPE const *globalReadA_0_0_31_0 = A + globalReadOffsetA_0_0_31_0;
  DATA_TYPE const *globalReadA_0_0_32_0 = A + globalReadOffsetA_0_0_32_0;
  DATA_TYPE const *globalReadA_0_0_33_0 = A + globalReadOffsetA_0_0_33_0;
  DATA_TYPE const *globalReadA_0_0_34_0 = A + globalReadOffsetA_0_0_34_0;
  DATA_TYPE const *globalReadA_0_0_35_0 = A + globalReadOffsetA_0_0_35_0;
  DATA_TYPE const *globalReadA_0_0_36_0 = A + globalReadOffsetA_0_0_36_0;
  DATA_TYPE const *globalReadA_0_0_37_0 = A + globalReadOffsetA_0_0_37_0;
  DATA_TYPE const *globalReadA_0_0_38_0 = A + globalReadOffsetA_0_0_38_0;
  DATA_TYPE const *globalReadA_0_0_39_0 = A + globalReadOffsetA_0_0_39_0;
  DATA_TYPE const *globalReadA_0_0_40_0 = A + globalReadOffsetA_0_0_40_0;
  DATA_TYPE const *globalReadA_0_0_41_0 = A + globalReadOffsetA_0_0_41_0;
  DATA_TYPE const *globalReadA_0_0_42_0 = A + globalReadOffsetA_0_0_42_0;
  DATA_TYPE const *globalReadA_0_0_43_0 = A + globalReadOffsetA_0_0_43_0;
  DATA_TYPE const *globalReadA_0_0_44_0 = A + globalReadOffsetA_0_0_44_0;
  DATA_TYPE const *globalReadA_0_0_45_0 = A + globalReadOffsetA_0_0_45_0;
  DATA_TYPE const *globalReadA_0_0_46_0 = A + globalReadOffsetA_0_0_46_0;
  DATA_TYPE const *globalReadA_0_0_47_0 = A + globalReadOffsetA_0_0_47_0;
  DATA_TYPE const *globalReadA_0_0_48_0 = A + globalReadOffsetA_0_0_48_0;
  DATA_TYPE const *globalReadA_0_0_49_0 = A + globalReadOffsetA_0_0_49_0;
  DATA_TYPE const *globalReadA_0_0_50_0 = A + globalReadOffsetA_0_0_50_0;
  DATA_TYPE const *globalReadA_0_0_51_0 = A + globalReadOffsetA_0_0_51_0;
  DATA_TYPE const *globalReadA_0_0_52_0 = A + globalReadOffsetA_0_0_52_0;
  DATA_TYPE const *globalReadA_0_0_53_0 = A + globalReadOffsetA_0_0_53_0;
  DATA_TYPE const *globalReadA_0_0_54_0 = A + globalReadOffsetA_0_0_54_0;
  DATA_TYPE const *globalReadA_0_0_55_0 = A + globalReadOffsetA_0_0_55_0;
  DATA_TYPE const *globalReadA_0_0_56_0 = A + globalReadOffsetA_0_0_56_0;
  DATA_TYPE const *globalReadA_0_0_57_0 = A + globalReadOffsetA_0_0_57_0;
  DATA_TYPE const *globalReadA_0_0_58_0 = A + globalReadOffsetA_0_0_58_0;
  DATA_TYPE const *globalReadA_0_0_59_0 = A + globalReadOffsetA_0_0_59_0;
  DATA_TYPE const *globalReadA_0_0_60_0 = A + globalReadOffsetA_0_0_60_0;
  DATA_TYPE const *globalReadA_0_0_61_0 = A + globalReadOffsetA_0_0_61_0;
  DATA_TYPE const *globalReadA_0_0_62_0 = A + globalReadOffsetA_0_0_62_0;
  DATA_TYPE const *globalReadA_0_0_63_0 = A + globalReadOffsetA_0_0_63_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 8*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 9*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 10*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 11*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 12*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 13*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 14*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 15*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_16_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 16*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_17_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 17*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_18_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 18*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_19_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 19*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_20_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 20*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_21_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 21*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_22_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 22*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_23_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 23*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_24_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 24*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_25_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 25*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_26_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 26*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_27_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 27*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_28_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 28*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_29_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 29*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_30_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 30*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_31_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 31*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_32_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 32*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_33_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 33*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_34_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 34*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_35_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 35*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_36_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 36*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_37_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 37*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_38_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 38*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_39_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 39*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_40_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 40*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_41_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 41*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_42_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 42*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_43_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 43*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_44_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 44*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_45_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 45*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_46_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 46*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_47_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 47*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_48_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 48*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_49_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 49*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_50_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 50*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_51_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 51*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_52_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 52*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_53_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 53*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_54_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 54*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_55_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 55*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_56_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 56*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_57_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 57*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_58_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 58*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_59_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 59*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_60_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 60*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_61_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 61*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_62_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 62*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_63_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 63*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_0_0_15_0;
  DATA_TYPE *localWriteA_0_0_16_0;
  DATA_TYPE *localWriteA_0_0_17_0;
  DATA_TYPE *localWriteA_0_0_18_0;
  DATA_TYPE *localWriteA_0_0_19_0;
  DATA_TYPE *localWriteA_0_0_20_0;
  DATA_TYPE *localWriteA_0_0_21_0;
  DATA_TYPE *localWriteA_0_0_22_0;
  DATA_TYPE *localWriteA_0_0_23_0;
  DATA_TYPE *localWriteA_0_0_24_0;
  DATA_TYPE *localWriteA_0_0_25_0;
  DATA_TYPE *localWriteA_0_0_26_0;
  DATA_TYPE *localWriteA_0_0_27_0;
  DATA_TYPE *localWriteA_0_0_28_0;
  DATA_TYPE *localWriteA_0_0_29_0;
  DATA_TYPE *localWriteA_0_0_30_0;
  DATA_TYPE *localWriteA_0_0_31_0;
  DATA_TYPE *localWriteA_0_0_32_0;
  DATA_TYPE *localWriteA_0_0_33_0;
  DATA_TYPE *localWriteA_0_0_34_0;
  DATA_TYPE *localWriteA_0_0_35_0;
  DATA_TYPE *localWriteA_0_0_36_0;
  DATA_TYPE *localWriteA_0_0_37_0;
  DATA_TYPE *localWriteA_0_0_38_0;
  DATA_TYPE *localWriteA_0_0_39_0;
  DATA_TYPE *localWriteA_0_0_40_0;
  DATA_TYPE *localWriteA_0_0_41_0;
  DATA_TYPE *localWriteA_0_0_42_0;
  DATA_TYPE *localWriteA_0_0_43_0;
  DATA_TYPE *localWriteA_0_0_44_0;
  DATA_TYPE *localWriteA_0_0_45_0;
  DATA_TYPE *localWriteA_0_0_46_0;
  DATA_TYPE *localWriteA_0_0_47_0;
  DATA_TYPE *localWriteA_0_0_48_0;
  DATA_TYPE *localWriteA_0_0_49_0;
  DATA_TYPE *localWriteA_0_0_50_0;
  DATA_TYPE *localWriteA_0_0_51_0;
  DATA_TYPE *localWriteA_0_0_52_0;
  DATA_TYPE *localWriteA_0_0_53_0;
  DATA_TYPE *localWriteA_0_0_54_0;
  DATA_TYPE *localWriteA_0_0_55_0;
  DATA_TYPE *localWriteA_0_0_56_0;
  DATA_TYPE *localWriteA_0_0_57_0;
  DATA_TYPE *localWriteA_0_0_58_0;
  DATA_TYPE *localWriteA_0_0_59_0;
  DATA_TYPE *localWriteA_0_0_60_0;
  DATA_TYPE *localWriteA_0_0_61_0;
  DATA_TYPE *localWriteA_0_0_62_0;
  DATA_TYPE *localWriteA_0_0_63_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);
  localWriteA_0_0_16_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_16_0);
  localWriteA_0_0_17_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_17_0);
  localWriteA_0_0_18_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_18_0);
  localWriteA_0_0_19_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_19_0);
  localWriteA_0_0_20_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_20_0);
  localWriteA_0_0_21_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_21_0);
  localWriteA_0_0_22_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_22_0);
  localWriteA_0_0_23_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_23_0);
  localWriteA_0_0_24_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_24_0);
  localWriteA_0_0_25_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_25_0);
  localWriteA_0_0_26_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_26_0);
  localWriteA_0_0_27_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_27_0);
  localWriteA_0_0_28_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_28_0);
  localWriteA_0_0_29_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_29_0);
  localWriteA_0_0_30_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_30_0);
  localWriteA_0_0_31_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_31_0);
  localWriteA_0_0_32_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_32_0);
  localWriteA_0_0_33_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_33_0);
  localWriteA_0_0_34_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_34_0);
  localWriteA_0_0_35_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_35_0);
  localWriteA_0_0_36_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_36_0);
  localWriteA_0_0_37_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_37_0);
  localWriteA_0_0_38_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_38_0);
  localWriteA_0_0_39_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_39_0);
  localWriteA_0_0_40_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_40_0);
  localWriteA_0_0_41_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_41_0);
  localWriteA_0_0_42_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_42_0);
  localWriteA_0_0_43_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_43_0);
  localWriteA_0_0_44_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_44_0);
  localWriteA_0_0_45_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_45_0);
  localWriteA_0_0_46_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_46_0);
  localWriteA_0_0_47_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_47_0);
  localWriteA_0_0_48_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_48_0);
  localWriteA_0_0_49_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_49_0);
  localWriteA_0_0_50_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_50_0);
  localWriteA_0_0_51_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_51_0);
  localWriteA_0_0_52_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_52_0);
  localWriteA_0_0_53_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_53_0);
  localWriteA_0_0_54_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_54_0);
  localWriteA_0_0_55_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_55_0);
  localWriteA_0_0_56_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_56_0);
  localWriteA_0_0_57_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_57_0);
  localWriteA_0_0_58_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_58_0);
  localWriteA_0_0_59_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_59_0);
  localWriteA_0_0_60_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_60_0);
  localWriteA_0_0_61_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_61_0);
  localWriteA_0_0_62_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_62_0);
  localWriteA_0_0_63_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_63_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);
    a_0_0_16_0 = *(globalReadA_0_0_16_0 + 0);
    a_0_0_17_0 = *(globalReadA_0_0_17_0 + 0);
    a_0_0_18_0 = *(globalReadA_0_0_18_0 + 0);
    a_0_0_19_0 = *(globalReadA_0_0_19_0 + 0);
    a_0_0_20_0 = *(globalReadA_0_0_20_0 + 0);
    a_0_0_21_0 = *(globalReadA_0_0_21_0 + 0);
    a_0_0_22_0 = *(globalReadA_0_0_22_0 + 0);
    a_0_0_23_0 = *(globalReadA_0_0_23_0 + 0);
    a_0_0_24_0 = *(globalReadA_0_0_24_0 + 0);
    a_0_0_25_0 = *(globalReadA_0_0_25_0 + 0);
    a_0_0_26_0 = *(globalReadA_0_0_26_0 + 0);
    a_0_0_27_0 = *(globalReadA_0_0_27_0 + 0);
    a_0_0_28_0 = *(globalReadA_0_0_28_0 + 0);
    a_0_0_29_0 = *(globalReadA_0_0_29_0 + 0);
    a_0_0_30_0 = *(globalReadA_0_0_30_0 + 0);
    a_0_0_31_0 = *(globalReadA_0_0_31_0 + 0);
    a_0_0_32_0 = *(globalReadA_0_0_32_0 + 0);
    a_0_0_33_0 = *(globalReadA_0_0_33_0 + 0);
    a_0_0_34_0 = *(globalReadA_0_0_34_0 + 0);
    a_0_0_35_0 = *(globalReadA_0_0_35_0 + 0);
    a_0_0_36_0 = *(globalReadA_0_0_36_0 + 0);
    a_0_0_37_0 = *(globalReadA_0_0_37_0 + 0);
    a_0_0_38_0 = *(globalReadA_0_0_38_0 + 0);
    a_0_0_39_0 = *(globalReadA_0_0_39_0 + 0);
    a_0_0_40_0 = *(globalReadA_0_0_40_0 + 0);
    a_0_0_41_0 = *(globalReadA_0_0_41_0 + 0);
    a_0_0_42_0 = *(globalReadA_0_0_42_0 + 0);
    a_0_0_43_0 = *(globalReadA_0_0_43_0 + 0);
    a_0_0_44_0 = *(globalReadA_0_0_44_0 + 0);
    a_0_0_45_0 = *(globalReadA_0_0_45_0 + 0);
    a_0_0_46_0 = *(globalReadA_0_0_46_0 + 0);
    a_0_0_47_0 = *(globalReadA_0_0_47_0 + 0);
    a_0_0_48_0 = *(globalReadA_0_0_48_0 + 0);
    a_0_0_49_0 = *(globalReadA_0_0_49_0 + 0);
    a_0_0_50_0 = *(globalReadA_0_0_50_0 + 0);
    a_0_0_51_0 = *(globalReadA_0_0_51_0 + 0);
    a_0_0_52_0 = *(globalReadA_0_0_52_0 + 0);
    a_0_0_53_0 = *(globalReadA_0_0_53_0 + 0);
    a_0_0_54_0 = *(globalReadA_0_0_54_0 + 0);
    a_0_0_55_0 = *(globalReadA_0_0_55_0 + 0);
    a_0_0_56_0 = *(globalReadA_0_0_56_0 + 0);
    a_0_0_57_0 = *(globalReadA_0_0_57_0 + 0);
    a_0_0_58_0 = *(globalReadA_0_0_58_0 + 0);
    a_0_0_59_0 = *(globalReadA_0_0_59_0 + 0);
    a_0_0_60_0 = *(globalReadA_0_0_60_0 + 0);
    a_0_0_61_0 = *(globalReadA_0_0_61_0 + 0);
    a_0_0_62_0 = *(globalReadA_0_0_62_0 + 0);
    a_0_0_63_0 = *(globalReadA_0_0_63_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    globalReadA_0_0_16_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_16_0) + 1*globalReadIncAL);
    globalReadA_0_0_17_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_17_0) + 1*globalReadIncAL);
    globalReadA_0_0_18_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_18_0) + 1*globalReadIncAL);
    globalReadA_0_0_19_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_19_0) + 1*globalReadIncAL);
    globalReadA_0_0_20_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_20_0) + 1*globalReadIncAL);
    globalReadA_0_0_21_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_21_0) + 1*globalReadIncAL);
    globalReadA_0_0_22_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_22_0) + 1*globalReadIncAL);
    globalReadA_0_0_23_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_23_0) + 1*globalReadIncAL);
    globalReadA_0_0_24_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_24_0) + 1*globalReadIncAL);
    globalReadA_0_0_25_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_25_0) + 1*globalReadIncAL);
    globalReadA_0_0_26_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_26_0) + 1*globalReadIncAL);
    globalReadA_0_0_27_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_27_0) + 1*globalReadIncAL);
    globalReadA_0_0_28_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_28_0) + 1*globalReadIncAL);
    globalReadA_0_0_29_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_29_0) + 1*globalReadIncAL);
    globalReadA_0_0_30_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_30_0) + 1*globalReadIncAL);
    globalReadA_0_0_31_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_31_0) + 1*globalReadIncAL);
    globalReadA_0_0_32_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_32_0) + 1*globalReadIncAL);
    globalReadA_0_0_33_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_33_0) + 1*globalReadIncAL);
    globalReadA_0_0_34_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_34_0) + 1*globalReadIncAL);
    globalReadA_0_0_35_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_35_0) + 1*globalReadIncAL);
    globalReadA_0_0_36_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_36_0) + 1*globalReadIncAL);
    globalReadA_0_0_37_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_37_0) + 1*globalReadIncAL);
    globalReadA_0_0_38_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_38_0) + 1*globalReadIncAL);
    globalReadA_0_0_39_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_39_0) + 1*globalReadIncAL);
    globalReadA_0_0_40_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_40_0) + 1*globalReadIncAL);
    globalReadA_0_0_41_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_41_0) + 1*globalReadIncAL);
    globalReadA_0_0_42_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_42_0) + 1*globalReadIncAL);
    globalReadA_0_0_43_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_43_0) + 1*globalReadIncAL);
    globalReadA_0_0_44_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_44_0) + 1*globalReadIncAL);
    globalReadA_0_0_45_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_45_0) + 1*globalReadIncAL);
    globalReadA_0_0_46_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_46_0) + 1*globalReadIncAL);
    globalReadA_0_0_47_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_47_0) + 1*globalReadIncAL);
    globalReadA_0_0_48_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_48_0) + 1*globalReadIncAL);
    globalReadA_0_0_49_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_49_0) + 1*globalReadIncAL);
    globalReadA_0_0_50_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_50_0) + 1*globalReadIncAL);
    globalReadA_0_0_51_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_51_0) + 1*globalReadIncAL);
    globalReadA_0_0_52_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_52_0) + 1*globalReadIncAL);
    globalReadA_0_0_53_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_53_0) + 1*globalReadIncAL);
    globalReadA_0_0_54_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_54_0) + 1*globalReadIncAL);
    globalReadA_0_0_55_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_55_0) + 1*globalReadIncAL);
    globalReadA_0_0_56_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_56_0) + 1*globalReadIncAL);
    globalReadA_0_0_57_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_57_0) + 1*globalReadIncAL);
    globalReadA_0_0_58_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_58_0) + 1*globalReadIncAL);
    globalReadA_0_0_59_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_59_0) + 1*globalReadIncAL);
    globalReadA_0_0_60_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_60_0) + 1*globalReadIncAL);
    globalReadA_0_0_61_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_61_0) + 1*globalReadIncAL);
    globalReadA_0_0_62_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_62_0) + 1*globalReadIncAL);
    globalReadA_0_0_63_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_63_0) + 1*globalReadIncAL);

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
    *(localWriteA_0_0_16_0 + 0) = a_0_0_16_0;
    *(localWriteA_0_0_17_0 + 0) = a_0_0_17_0;
    *(localWriteA_0_0_18_0 + 0) = a_0_0_18_0;
    *(localWriteA_0_0_19_0 + 0) = a_0_0_19_0;
    *(localWriteA_0_0_20_0 + 0) = a_0_0_20_0;
    *(localWriteA_0_0_21_0 + 0) = a_0_0_21_0;
    *(localWriteA_0_0_22_0 + 0) = a_0_0_22_0;
    *(localWriteA_0_0_23_0 + 0) = a_0_0_23_0;
    *(localWriteA_0_0_24_0 + 0) = a_0_0_24_0;
    *(localWriteA_0_0_25_0 + 0) = a_0_0_25_0;
    *(localWriteA_0_0_26_0 + 0) = a_0_0_26_0;
    *(localWriteA_0_0_27_0 + 0) = a_0_0_27_0;
    *(localWriteA_0_0_28_0 + 0) = a_0_0_28_0;
    *(localWriteA_0_0_29_0 + 0) = a_0_0_29_0;
    *(localWriteA_0_0_30_0 + 0) = a_0_0_30_0;
    *(localWriteA_0_0_31_0 + 0) = a_0_0_31_0;
    *(localWriteA_0_0_32_0 + 0) = a_0_0_32_0;
    *(localWriteA_0_0_33_0 + 0) = a_0_0_33_0;
    *(localWriteA_0_0_34_0 + 0) = a_0_0_34_0;
    *(localWriteA_0_0_35_0 + 0) = a_0_0_35_0;
    *(localWriteA_0_0_36_0 + 0) = a_0_0_36_0;
    *(localWriteA_0_0_37_0 + 0) = a_0_0_37_0;
    *(localWriteA_0_0_38_0 + 0) = a_0_0_38_0;
    *(localWriteA_0_0_39_0 + 0) = a_0_0_39_0;
    *(localWriteA_0_0_40_0 + 0) = a_0_0_40_0;
    *(localWriteA_0_0_41_0 + 0) = a_0_0_41_0;
    *(localWriteA_0_0_42_0 + 0) = a_0_0_42_0;
    *(localWriteA_0_0_43_0 + 0) = a_0_0_43_0;
    *(localWriteA_0_0_44_0 + 0) = a_0_0_44_0;
    *(localWriteA_0_0_45_0 + 0) = a_0_0_45_0;
    *(localWriteA_0_0_46_0 + 0) = a_0_0_46_0;
    *(localWriteA_0_0_47_0 + 0) = a_0_0_47_0;
    *(localWriteA_0_0_48_0 + 0) = a_0_0_48_0;
    *(localWriteA_0_0_49_0 + 0) = a_0_0_49_0;
    *(localWriteA_0_0_50_0 + 0) = a_0_0_50_0;
    *(localWriteA_0_0_51_0 + 0) = a_0_0_51_0;
    *(localWriteA_0_0_52_0 + 0) = a_0_0_52_0;
    *(localWriteA_0_0_53_0 + 0) = a_0_0_53_0;
    *(localWriteA_0_0_54_0 + 0) = a_0_0_54_0;
    *(localWriteA_0_0_55_0 + 0) = a_0_0_55_0;
    *(localWriteA_0_0_56_0 + 0) = a_0_0_56_0;
    *(localWriteA_0_0_57_0 + 0) = a_0_0_57_0;
    *(localWriteA_0_0_58_0 + 0) = a_0_0_58_0;
    *(localWriteA_0_0_59_0 + 0) = a_0_0_59_0;
    *(localWriteA_0_0_60_0 + 0) = a_0_0_60_0;
    *(localWriteA_0_0_61_0 + 0) = a_0_0_61_0;
    *(localWriteA_0_0_62_0 + 0) = a_0_0_62_0;
    *(localWriteA_0_0_63_0 + 0) = a_0_0_63_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 15 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 16 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 17 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 18 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 19 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 20 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 21 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 22 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 23 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 24 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 25 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 26 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 27 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 28 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 29 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 30 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 31 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 32 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 33 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 34 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 35 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 36 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 37 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 38 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 39 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 40 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 41 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 42 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 43 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 44 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 45 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 46 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 47 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 48 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 49 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 50 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 51 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 52 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 53 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 54 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 55 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 56 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 57 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 58 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 59 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 60 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 61 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 62 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 63 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_1x1




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_0_8_0 = ( globalReadOffsetAL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_0_0_9_0 = ( globalReadOffsetAL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_0_0_10_0 = ( globalReadOffsetAL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_0_0_11_0 = ( globalReadOffsetAL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_0_0_12_0 = ( globalReadOffsetAL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_0_0_13_0 = ( globalReadOffsetAL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_0_0_14_0 = ( globalReadOffsetAL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_0_0_15_0 = ( globalReadOffsetAL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);
  a_0_0_16_0 = ( globalReadOffsetAL_16_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_16_0 + 0);
  a_0_0_17_0 = ( globalReadOffsetAL_17_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_17_0 + 0);
  a_0_0_18_0 = ( globalReadOffsetAL_18_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_18_0 + 0);
  a_0_0_19_0 = ( globalReadOffsetAL_19_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_19_0 + 0);
  a_0_0_20_0 = ( globalReadOffsetAL_20_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_20_0 + 0);
  a_0_0_21_0 = ( globalReadOffsetAL_21_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_21_0 + 0);
  a_0_0_22_0 = ( globalReadOffsetAL_22_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_22_0 + 0);
  a_0_0_23_0 = ( globalReadOffsetAL_23_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_23_0 + 0);
  a_0_0_24_0 = ( globalReadOffsetAL_24_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_24_0 + 0);
  a_0_0_25_0 = ( globalReadOffsetAL_25_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_25_0 + 0);
  a_0_0_26_0 = ( globalReadOffsetAL_26_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_26_0 + 0);
  a_0_0_27_0 = ( globalReadOffsetAL_27_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_27_0 + 0);
  a_0_0_28_0 = ( globalReadOffsetAL_28_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_28_0 + 0);
  a_0_0_29_0 = ( globalReadOffsetAL_29_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_29_0 + 0);
  a_0_0_30_0 = ( globalReadOffsetAL_30_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_30_0 + 0);
  a_0_0_31_0 = ( globalReadOffsetAL_31_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_31_0 + 0);
  a_0_0_32_0 = ( globalReadOffsetAL_32_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_32_0 + 0);
  a_0_0_33_0 = ( globalReadOffsetAL_33_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_33_0 + 0);
  a_0_0_34_0 = ( globalReadOffsetAL_34_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_34_0 + 0);
  a_0_0_35_0 = ( globalReadOffsetAL_35_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_35_0 + 0);
  a_0_0_36_0 = ( globalReadOffsetAL_36_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_36_0 + 0);
  a_0_0_37_0 = ( globalReadOffsetAL_37_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_37_0 + 0);
  a_0_0_38_0 = ( globalReadOffsetAL_38_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_38_0 + 0);
  a_0_0_39_0 = ( globalReadOffsetAL_39_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_39_0 + 0);
  a_0_0_40_0 = ( globalReadOffsetAL_40_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_40_0 + 0);
  a_0_0_41_0 = ( globalReadOffsetAL_41_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_41_0 + 0);
  a_0_0_42_0 = ( globalReadOffsetAL_42_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_42_0 + 0);
  a_0_0_43_0 = ( globalReadOffsetAL_43_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_43_0 + 0);
  a_0_0_44_0 = ( globalReadOffsetAL_44_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_44_0 + 0);
  a_0_0_45_0 = ( globalReadOffsetAL_45_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_45_0 + 0);
  a_0_0_46_0 = ( globalReadOffsetAL_46_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_46_0 + 0);
  a_0_0_47_0 = ( globalReadOffsetAL_47_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_47_0 + 0);
  a_0_0_48_0 = ( globalReadOffsetAL_48_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_48_0 + 0);
  a_0_0_49_0 = ( globalReadOffsetAL_49_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_49_0 + 0);
  a_0_0_50_0 = ( globalReadOffsetAL_50_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_50_0 + 0);
  a_0_0_51_0 = ( globalReadOffsetAL_51_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_51_0 + 0);
  a_0_0_52_0 = ( globalReadOffsetAL_52_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_52_0 + 0);
  a_0_0_53_0 = ( globalReadOffsetAL_53_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_53_0 + 0);
  a_0_0_54_0 = ( globalReadOffsetAL_54_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_54_0 + 0);
  a_0_0_55_0 = ( globalReadOffsetAL_55_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_55_0 + 0);
  a_0_0_56_0 = ( globalReadOffsetAL_56_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_56_0 + 0);
  a_0_0_57_0 = ( globalReadOffsetAL_57_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_57_0 + 0);
  a_0_0_58_0 = ( globalReadOffsetAL_58_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_58_0 + 0);
  a_0_0_59_0 = ( globalReadOffsetAL_59_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_59_0 + 0);
  a_0_0_60_0 = ( globalReadOffsetAL_60_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_60_0 + 0);
  a_0_0_61_0 = ( globalReadOffsetAL_61_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_61_0 + 0);
  a_0_0_62_0 = ( globalReadOffsetAL_62_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_62_0 + 0);
  a_0_0_63_0 = ( globalReadOffsetAL_63_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_63_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);
  localWriteA_0_0_16_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_16_0);
  localWriteA_0_0_17_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_17_0);
  localWriteA_0_0_18_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_18_0);
  localWriteA_0_0_19_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_19_0);
  localWriteA_0_0_20_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_20_0);
  localWriteA_0_0_21_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_21_0);
  localWriteA_0_0_22_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_22_0);
  localWriteA_0_0_23_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_23_0);
  localWriteA_0_0_24_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_24_0);
  localWriteA_0_0_25_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_25_0);
  localWriteA_0_0_26_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_26_0);
  localWriteA_0_0_27_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_27_0);
  localWriteA_0_0_28_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_28_0);
  localWriteA_0_0_29_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_29_0);
  localWriteA_0_0_30_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_30_0);
  localWriteA_0_0_31_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_31_0);
  localWriteA_0_0_32_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_32_0);
  localWriteA_0_0_33_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_33_0);
  localWriteA_0_0_34_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_34_0);
  localWriteA_0_0_35_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_35_0);
  localWriteA_0_0_36_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_36_0);
  localWriteA_0_0_37_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_37_0);
  localWriteA_0_0_38_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_38_0);
  localWriteA_0_0_39_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_39_0);
  localWriteA_0_0_40_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_40_0);
  localWriteA_0_0_41_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_41_0);
  localWriteA_0_0_42_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_42_0);
  localWriteA_0_0_43_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_43_0);
  localWriteA_0_0_44_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_44_0);
  localWriteA_0_0_45_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_45_0);
  localWriteA_0_0_46_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_46_0);
  localWriteA_0_0_47_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_47_0);
  localWriteA_0_0_48_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_48_0);
  localWriteA_0_0_49_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_49_0);
  localWriteA_0_0_50_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_50_0);
  localWriteA_0_0_51_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_51_0);
  localWriteA_0_0_52_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_52_0);
  localWriteA_0_0_53_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_53_0);
  localWriteA_0_0_54_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_54_0);
  localWriteA_0_0_55_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_55_0);
  localWriteA_0_0_56_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_56_0);
  localWriteA_0_0_57_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_57_0);
  localWriteA_0_0_58_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_58_0);
  localWriteA_0_0_59_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_59_0);
  localWriteA_0_0_60_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_60_0);
  localWriteA_0_0_61_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_61_0);
  localWriteA_0_0_62_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_62_0);
  localWriteA_0_0_63_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_63_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
  *(localWriteA_0_0_16_0 + 0) = a_0_0_16_0;
  *(localWriteA_0_0_17_0 + 0) = a_0_0_17_0;
  *(localWriteA_0_0_18_0 + 0) = a_0_0_18_0;
  *(localWriteA_0_0_19_0 + 0) = a_0_0_19_0;
  *(localWriteA_0_0_20_0 + 0) = a_0_0_20_0;
  *(localWriteA_0_0_21_0 + 0) = a_0_0_21_0;
  *(localWriteA_0_0_22_0 + 0) = a_0_0_22_0;
  *(localWriteA_0_0_23_0 + 0) = a_0_0_23_0;
  *(localWriteA_0_0_24_0 + 0) = a_0_0_24_0;
  *(localWriteA_0_0_25_0 + 0) = a_0_0_25_0;
  *(localWriteA_0_0_26_0 + 0) = a_0_0_26_0;
  *(localWriteA_0_0_27_0 + 0) = a_0_0_27_0;
  *(localWriteA_0_0_28_0 + 0) = a_0_0_28_0;
  *(localWriteA_0_0_29_0 + 0) = a_0_0_29_0;
  *(localWriteA_0_0_30_0 + 0) = a_0_0_30_0;
  *(localWriteA_0_0_31_0 + 0) = a_0_0_31_0;
  *(localWriteA_0_0_32_0 + 0) = a_0_0_32_0;
  *(localWriteA_0_0_33_0 + 0) = a_0_0_33_0;
  *(localWriteA_0_0_34_0 + 0) = a_0_0_34_0;
  *(localWriteA_0_0_35_0 + 0) = a_0_0_35_0;
  *(localWriteA_0_0_36_0 + 0) = a_0_0_36_0;
  *(localWriteA_0_0_37_0 + 0) = a_0_0_37_0;
  *(localWriteA_0_0_38_0 + 0) = a_0_0_38_0;
  *(localWriteA_0_0_39_0 + 0) = a_0_0_39_0;
  *(localWriteA_0_0_40_0 + 0) = a_0_0_40_0;
  *(localWriteA_0_0_41_0 + 0) = a_0_0_41_0;
  *(localWriteA_0_0_42_0 + 0) = a_0_0_42_0;
  *(localWriteA_0_0_43_0 + 0) = a_0_0_43_0;
  *(localWriteA_0_0_44_0 + 0) = a_0_0_44_0;
  *(localWriteA_0_0_45_0 + 0) = a_0_0_45_0;
  *(localWriteA_0_0_46_0 + 0) = a_0_0_46_0;
  *(localWriteA_0_0_47_0 + 0) = a_0_0_47_0;
  *(localWriteA_0_0_48_0 + 0) = a_0_0_48_0;
  *(localWriteA_0_0_49_0 + 0) = a_0_0_49_0;
  *(localWriteA_0_0_50_0 + 0) = a_0_0_50_0;
  *(localWriteA_0_0_51_0 + 0) = a_0_0_51_0;
  *(localWriteA_0_0_52_0 + 0) = a_0_0_52_0;
  *(localWriteA_0_0_53_0 + 0) = a_0_0_53_0;
  *(localWriteA_0_0_54_0 + 0) = a_0_0_54_0;
  *(localWriteA_0_0_55_0 + 0) = a_0_0_55_0;
  *(localWriteA_0_0_56_0 + 0) = a_0_0_56_0;
  *(localWriteA_0_0_57_0 + 0) = a_0_0_57_0;
  *(localWriteA_0_0_58_0 + 0) = a_0_0_58_0;
  *(localWriteA_0_0_59_0 + 0) = a_0_0_59_0;
  *(localWriteA_0_0_60_0 + 0) = a_0_0_60_0;
  *(localWriteA_0_0_61_0 + 0) = a_0_0_61_0;
  *(localWriteA_0_0_62_0 + 0) = a_0_0_62_0;
  *(localWriteA_0_0_63_0 + 0) = a_0_0_63_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_1x1

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_1x1
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 64
#define SG1J 1
#define TT0I 1
#define TT1J 1
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 64
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 64
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 4096
#define LDS_NUM_ELEMENTS 4160

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 1x1 micro-tile */
#define MAC_1x1\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \

#define MAC_1x1_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_BBH_MT64x1x64_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU0_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT1_1_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG64_1_1_WGM1(
  float *D,
  float const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE a_0_0_16_0;
  DATA_TYPE a_0_0_17_0;
  DATA_TYPE a_0_0_18_0;
  DATA_TYPE a_0_0_19_0;
  DATA_TYPE a_0_0_20_0;
  DATA_TYPE a_0_0_21_0;
  DATA_TYPE a_0_0_22_0;
  DATA_TYPE a_0_0_23_0;
  DATA_TYPE a_0_0_24_0;
  DATA_TYPE a_0_0_25_0;
  DATA_TYPE a_0_0_26_0;
  DATA_TYPE a_0_0_27_0;
  DATA_TYPE a_0_0_28_0;
  DATA_TYPE a_0_0_29_0;
  DATA_TYPE a_0_0_30_0;
  DATA_TYPE a_0_0_31_0;
  DATA_TYPE a_0_0_32_0;
  DATA_TYPE a_0_0_33_0;
  DATA_TYPE a_0_0_34_0;
  DATA_TYPE a_0_0_35_0;
  DATA_TYPE a_0_0_36_0;
  DATA_TYPE a_0_0_37_0;
  DATA_TYPE a_0_0_38_0;
  DATA_TYPE a_0_0_39_0;
  DATA_TYPE a_0_0_40_0;
  DATA_TYPE a_0_0_41_0;
  DATA_TYPE a_0_0_42_0;
  DATA_TYPE a_0_0_43_0;
  DATA_TYPE a_0_0_44_0;
  DATA_TYPE a_0_0_45_0;
  DATA_TYPE a_0_0_46_0;
  DATA_TYPE a_0_0_47_0;
  DATA_TYPE a_0_0_48_0;
  DATA_TYPE a_0_0_49_0;
  DATA_TYPE a_0_0_50_0;
  DATA_TYPE a_0_0_51_0;
  DATA_TYPE a_0_0_52_0;
  DATA_TYPE a_0_0_53_0;
  DATA_TYPE a_0_0_54_0;
  DATA_TYPE a_0_0_55_0;
  DATA_TYPE a_0_0_56_0;
  DATA_TYPE a_0_0_57_0;
  DATA_TYPE a_0_0_58_0;
  DATA_TYPE a_0_0_59_0;
  DATA_TYPE a_0_0_60_0;
  DATA_TYPE a_0_0_61_0;
  DATA_TYPE a_0_0_62_0;
  DATA_TYPE a_0_0_63_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;
  unsigned int globalReadOffsetAL_8_0 = globalReadOffsetAL + 0 + 8*LSPA;
  unsigned int globalReadOffsetAL_9_0 = globalReadOffsetAL + 0 + 9*LSPA;
  unsigned int globalReadOffsetAL_10_0 = globalReadOffsetAL + 0 + 10*LSPA;
  unsigned int globalReadOffsetAL_11_0 = globalReadOffsetAL + 0 + 11*LSPA;
  unsigned int globalReadOffsetAL_12_0 = globalReadOffsetAL + 0 + 12*LSPA;
  unsigned int globalReadOffsetAL_13_0 = globalReadOffsetAL + 0 + 13*LSPA;
  unsigned int globalReadOffsetAL_14_0 = globalReadOffsetAL + 0 + 14*LSPA;
  unsigned int globalReadOffsetAL_15_0 = globalReadOffsetAL + 0 + 15*LSPA;
  unsigned int globalReadOffsetAL_16_0 = globalReadOffsetAL + 0 + 16*LSPA;
  unsigned int globalReadOffsetAL_17_0 = globalReadOffsetAL + 0 + 17*LSPA;
  unsigned int globalReadOffsetAL_18_0 = globalReadOffsetAL + 0 + 18*LSPA;
  unsigned int globalReadOffsetAL_19_0 = globalReadOffsetAL + 0 + 19*LSPA;
  unsigned int globalReadOffsetAL_20_0 = globalReadOffsetAL + 0 + 20*LSPA;
  unsigned int globalReadOffsetAL_21_0 = globalReadOffsetAL + 0 + 21*LSPA;
  unsigned int globalReadOffsetAL_22_0 = globalReadOffsetAL + 0 + 22*LSPA;
  unsigned int globalReadOffsetAL_23_0 = globalReadOffsetAL + 0 + 23*LSPA;
  unsigned int globalReadOffsetAL_24_0 = globalReadOffsetAL + 0 + 24*LSPA;
  unsigned int globalReadOffsetAL_25_0 = globalReadOffsetAL + 0 + 25*LSPA;
  unsigned int globalReadOffsetAL_26_0 = globalReadOffsetAL + 0 + 26*LSPA;
  unsigned int globalReadOffsetAL_27_0 = globalReadOffsetAL + 0 + 27*LSPA;
  unsigned int globalReadOffsetAL_28_0 = globalReadOffsetAL + 0 + 28*LSPA;
  unsigned int globalReadOffsetAL_29_0 = globalReadOffsetAL + 0 + 29*LSPA;
  unsigned int globalReadOffsetAL_30_0 = globalReadOffsetAL + 0 + 30*LSPA;
  unsigned int globalReadOffsetAL_31_0 = globalReadOffsetAL + 0 + 31*LSPA;
  unsigned int globalReadOffsetAL_32_0 = globalReadOffsetAL + 0 + 32*LSPA;
  unsigned int globalReadOffsetAL_33_0 = globalReadOffsetAL + 0 + 33*LSPA;
  unsigned int globalReadOffsetAL_34_0 = globalReadOffsetAL + 0 + 34*LSPA;
  unsigned int globalReadOffsetAL_35_0 = globalReadOffsetAL + 0 + 35*LSPA;
  unsigned int globalReadOffsetAL_36_0 = globalReadOffsetAL + 0 + 36*LSPA;
  unsigned int globalReadOffsetAL_37_0 = globalReadOffsetAL + 0 + 37*LSPA;
  unsigned int globalReadOffsetAL_38_0 = globalReadOffsetAL + 0 + 38*LSPA;
  unsigned int globalReadOffsetAL_39_0 = globalReadOffsetAL + 0 + 39*LSPA;
  unsigned int globalReadOffsetAL_40_0 = globalReadOffsetAL + 0 + 40*LSPA;
  unsigned int globalReadOffsetAL_41_0 = globalReadOffsetAL + 0 + 41*LSPA;
  unsigned int globalReadOffsetAL_42_0 = globalReadOffsetAL + 0 + 42*LSPA;
  unsigned int globalReadOffsetAL_43_0 = globalReadOffsetAL + 0 + 43*LSPA;
  unsigned int globalReadOffsetAL_44_0 = globalReadOffsetAL + 0 + 44*LSPA;
  unsigned int globalReadOffsetAL_45_0 = globalReadOffsetAL + 0 + 45*LSPA;
  unsigned int globalReadOffsetAL_46_0 = globalReadOffsetAL + 0 + 46*LSPA;
  unsigned int globalReadOffsetAL_47_0 = globalReadOffsetAL + 0 + 47*LSPA;
  unsigned int globalReadOffsetAL_48_0 = globalReadOffsetAL + 0 + 48*LSPA;
  unsigned int globalReadOffsetAL_49_0 = globalReadOffsetAL + 0 + 49*LSPA;
  unsigned int globalReadOffsetAL_50_0 = globalReadOffsetAL + 0 + 50*LSPA;
  unsigned int globalReadOffsetAL_51_0 = globalReadOffsetAL + 0 + 51*LSPA;
  unsigned int globalReadOffsetAL_52_0 = globalReadOffsetAL + 0 + 52*LSPA;
  unsigned int globalReadOffsetAL_53_0 = globalReadOffsetAL + 0 + 53*LSPA;
  unsigned int globalReadOffsetAL_54_0 = globalReadOffsetAL + 0 + 54*LSPA;
  unsigned int globalReadOffsetAL_55_0 = globalReadOffsetAL + 0 + 55*LSPA;
  unsigned int globalReadOffsetAL_56_0 = globalReadOffsetAL + 0 + 56*LSPA;
  unsigned int globalReadOffsetAL_57_0 = globalReadOffsetAL + 0 + 57*LSPA;
  unsigned int globalReadOffsetAL_58_0 = globalReadOffsetAL + 0 + 58*LSPA;
  unsigned int globalReadOffsetAL_59_0 = globalReadOffsetAL + 0 + 59*LSPA;
  unsigned int globalReadOffsetAL_60_0 = globalReadOffsetAL + 0 + 60*LSPA;
  unsigned int globalReadOffsetAL_61_0 = globalReadOffsetAL + 0 + 61*LSPA;
  unsigned int globalReadOffsetAL_62_0 = globalReadOffsetAL + 0 + 62*LSPA;
  unsigned int globalReadOffsetAL_63_0 = globalReadOffsetAL + 0 + 63*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_15_0), wgK );
  int64_t globalReadOffsetA_0_0_16_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_16_0), wgK );
  int64_t globalReadOffsetA_0_0_17_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_17_0), wgK );
  int64_t globalReadOffsetA_0_0_18_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_18_0), wgK );
  int64_t globalReadOffsetA_0_0_19_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_19_0), wgK );
  int64_t globalReadOffsetA_0_0_20_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_20_0), wgK );
  int64_t globalReadOffsetA_0_0_21_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_21_0), wgK );
  int64_t globalReadOffsetA_0_0_22_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_22_0), wgK );
  int64_t globalReadOffsetA_0_0_23_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_23_0), wgK );
  int64_t globalReadOffsetA_0_0_24_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_24_0), wgK );
  int64_t globalReadOffsetA_0_0_25_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_25_0), wgK );
  int64_t globalReadOffsetA_0_0_26_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_26_0), wgK );
  int64_t globalReadOffsetA_0_0_27_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_27_0), wgK );
  int64_t globalReadOffsetA_0_0_28_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_28_0), wgK );
  int64_t globalReadOffsetA_0_0_29_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_29_0), wgK );
  int64_t globalReadOffsetA_0_0_30_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_30_0), wgK );
  int64_t globalReadOffsetA_0_0_31_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_31_0), wgK );
  int64_t globalReadOffsetA_0_0_32_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_32_0), wgK );
  int64_t globalReadOffsetA_0_0_33_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_33_0), wgK );
  int64_t globalReadOffsetA_0_0_34_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_34_0), wgK );
  int64_t globalReadOffsetA_0_0_35_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_35_0), wgK );
  int64_t globalReadOffsetA_0_0_36_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_36_0), wgK );
  int64_t globalReadOffsetA_0_0_37_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_37_0), wgK );
  int64_t globalReadOffsetA_0_0_38_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_38_0), wgK );
  int64_t globalReadOffsetA_0_0_39_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_39_0), wgK );
  int64_t globalReadOffsetA_0_0_40_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_40_0), wgK );
  int64_t globalReadOffsetA_0_0_41_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_41_0), wgK );
  int64_t globalReadOffsetA_0_0_42_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_42_0), wgK );
  int64_t globalReadOffsetA_0_0_43_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_43_0), wgK );
  int64_t globalReadOffsetA_0_0_44_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_44_0), wgK );
  int64_t globalReadOffsetA_0_0_45_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_45_0), wgK );
  int64_t globalReadOffsetA_0_0_46_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_46_0), wgK );
  int64_t globalReadOffsetA_0_0_47_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_47_0), wgK );
  int64_t globalReadOffsetA_0_0_48_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_48_0), wgK );
  int64_t globalReadOffsetA_0_0_49_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_49_0), wgK );
  int64_t globalReadOffsetA_0_0_50_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_50_0), wgK );
  int64_t globalReadOffsetA_0_0_51_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_51_0), wgK );
  int64_t globalReadOffsetA_0_0_52_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_52_0), wgK );
  int64_t globalReadOffsetA_0_0_53_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_53_0), wgK );
  int64_t globalReadOffsetA_0_0_54_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_54_0), wgK );
  int64_t globalReadOffsetA_0_0_55_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_55_0), wgK );
  int64_t globalReadOffsetA_0_0_56_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_56_0), wgK );
  int64_t globalReadOffsetA_0_0_57_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_57_0), wgK );
  int64_t globalReadOffsetA_0_0_58_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_58_0), wgK );
  int64_t globalReadOffsetA_0_0_59_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_59_0), wgK );
  int64_t globalReadOffsetA_0_0_60_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_60_0), wgK );
  int64_t globalReadOffsetA_0_0_61_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_61_0), wgK );
  int64_t globalReadOffsetA_0_0_62_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_62_0), wgK );
  int64_t globalReadOffsetA_0_0_63_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_63_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;
  DATA_TYPE const *globalReadA_0_0_16_0 = A + globalReadOffsetA_0_0_16_0;
  DATA_TYPE const *globalReadA_0_0_17_0 = A + globalReadOffsetA_0_0_17_0;
  DATA_TYPE const *globalReadA_0_0_18_0 = A + globalReadOffsetA_0_0_18_0;
  DATA_TYPE const *globalReadA_0_0_19_0 = A + globalReadOffsetA_0_0_19_0;
  DATA_TYPE const *globalReadA_0_0_20_0 = A + globalReadOffsetA_0_0_20_0;
  DATA_TYPE const *globalReadA_0_0_21_0 = A + globalReadOffsetA_0_0_21_0;
  DATA_TYPE const *globalReadA_0_0_22_0 = A + globalReadOffsetA_0_0_22_0;
  DATA_TYPE const *globalReadA_0_0_23_0 = A + globalReadOffsetA_0_0_23_0;
  DATA_TYPE const *globalReadA_0_0_24_0 = A + globalReadOffsetA_0_0_24_0;
  DATA_TYPE const *globalReadA_0_0_25_0 = A + globalReadOffsetA_0_0_25_0;
  DATA_TYPE const *globalReadA_0_0_26_0 = A + globalReadOffsetA_0_0_26_0;
  DATA_TYPE const *globalReadA_0_0_27_0 = A + globalReadOffsetA_0_0_27_0;
  DATA_TYPE const *globalReadA_0_0_28_0 = A + globalReadOffsetA_0_0_28_0;
  DATA_TYPE const *globalReadA_0_0_29_0 = A + globalReadOffsetA_0_0_29_0;
  DATA_TYPE const *globalReadA_0_0_30_0 = A + globalReadOffsetA_0_0_30_0;
  DATA_TYPE const *globalReadA_0_0_31_0 = A + globalReadOffsetA_0_0_31_0;
  DATA_TYPE const *globalReadA_0_0_32_0 = A + globalReadOffsetA_0_0_32_0;
  DATA_TYPE const *globalReadA_0_0_33_0 = A + globalReadOffsetA_0_0_33_0;
  DATA_TYPE const *globalReadA_0_0_34_0 = A + globalReadOffsetA_0_0_34_0;
  DATA_TYPE const *globalReadA_0_0_35_0 = A + globalReadOffsetA_0_0_35_0;
  DATA_TYPE const *globalReadA_0_0_36_0 = A + globalReadOffsetA_0_0_36_0;
  DATA_TYPE const *globalReadA_0_0_37_0 = A + globalReadOffsetA_0_0_37_0;
  DATA_TYPE const *globalReadA_0_0_38_0 = A + globalReadOffsetA_0_0_38_0;
  DATA_TYPE const *globalReadA_0_0_39_0 = A + globalReadOffsetA_0_0_39_0;
  DATA_TYPE const *globalReadA_0_0_40_0 = A + globalReadOffsetA_0_0_40_0;
  DATA_TYPE const *globalReadA_0_0_41_0 = A + globalReadOffsetA_0_0_41_0;
  DATA_TYPE const *globalReadA_0_0_42_0 = A + globalReadOffsetA_0_0_42_0;
  DATA_TYPE const *globalReadA_0_0_43_0 = A + globalReadOffsetA_0_0_43_0;
  DATA_TYPE const *globalReadA_0_0_44_0 = A + globalReadOffsetA_0_0_44_0;
  DATA_TYPE const *globalReadA_0_0_45_0 = A + globalReadOffsetA_0_0_45_0;
  DATA_TYPE const *globalReadA_0_0_46_0 = A + globalReadOffsetA_0_0_46_0;
  DATA_TYPE const *globalReadA_0_0_47_0 = A + globalReadOffsetA_0_0_47_0;
  DATA_TYPE const *globalReadA_0_0_48_0 = A + globalReadOffsetA_0_0_48_0;
  DATA_TYPE const *globalReadA_0_0_49_0 = A + globalReadOffsetA_0_0_49_0;
  DATA_TYPE const *globalReadA_0_0_50_0 = A + globalReadOffsetA_0_0_50_0;
  DATA_TYPE const *globalReadA_0_0_51_0 = A + globalReadOffsetA_0_0_51_0;
  DATA_TYPE const *globalReadA_0_0_52_0 = A + globalReadOffsetA_0_0_52_0;
  DATA_TYPE const *globalReadA_0_0_53_0 = A + globalReadOffsetA_0_0_53_0;
  DATA_TYPE const *globalReadA_0_0_54_0 = A + globalReadOffsetA_0_0_54_0;
  DATA_TYPE const *globalReadA_0_0_55_0 = A + globalReadOffsetA_0_0_55_0;
  DATA_TYPE const *globalReadA_0_0_56_0 = A + globalReadOffsetA_0_0_56_0;
  DATA_TYPE const *globalReadA_0_0_57_0 = A + globalReadOffsetA_0_0_57_0;
  DATA_TYPE const *globalReadA_0_0_58_0 = A + globalReadOffsetA_0_0_58_0;
  DATA_TYPE const *globalReadA_0_0_59_0 = A + globalReadOffsetA_0_0_59_0;
  DATA_TYPE const *globalReadA_0_0_60_0 = A + globalReadOffsetA_0_0_60_0;
  DATA_TYPE const *globalReadA_0_0_61_0 = A + globalReadOffsetA_0_0_61_0;
  DATA_TYPE const *globalReadA_0_0_62_0 = A + globalReadOffsetA_0_0_62_0;
  DATA_TYPE const *globalReadA_0_0_63_0 = A + globalReadOffsetA_0_0_63_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 8*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 9*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 10*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 11*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 12*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 13*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 14*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 15*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_16_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 16*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_17_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 17*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_18_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 18*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_19_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 19*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_20_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 20*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_21_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 21*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_22_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 22*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_23_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 23*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_24_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 24*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_25_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 25*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_26_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 26*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_27_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 27*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_28_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 28*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_29_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 29*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_30_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 30*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_31_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 31*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_32_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 32*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_33_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 33*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_34_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 34*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_35_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 35*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_36_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 36*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_37_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 37*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_38_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 38*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_39_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 39*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_40_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 40*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_41_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 41*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_42_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 42*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_43_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 43*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_44_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 44*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_45_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 45*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_46_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 46*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_47_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 47*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_48_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 48*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_49_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 49*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_50_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 50*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_51_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 51*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_52_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 52*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_53_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 53*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_54_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 54*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_55_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 55*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_56_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 56*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_57_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 57*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_58_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 58*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_59_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 59*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_60_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 60*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_61_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 61*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_62_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 62*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_63_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 63*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_0_0_15_0;
  DATA_TYPE *localWriteA_0_0_16_0;
  DATA_TYPE *localWriteA_0_0_17_0;
  DATA_TYPE *localWriteA_0_0_18_0;
  DATA_TYPE *localWriteA_0_0_19_0;
  DATA_TYPE *localWriteA_0_0_20_0;
  DATA_TYPE *localWriteA_0_0_21_0;
  DATA_TYPE *localWriteA_0_0_22_0;
  DATA_TYPE *localWriteA_0_0_23_0;
  DATA_TYPE *localWriteA_0_0_24_0;
  DATA_TYPE *localWriteA_0_0_25_0;
  DATA_TYPE *localWriteA_0_0_26_0;
  DATA_TYPE *localWriteA_0_0_27_0;
  DATA_TYPE *localWriteA_0_0_28_0;
  DATA_TYPE *localWriteA_0_0_29_0;
  DATA_TYPE *localWriteA_0_0_30_0;
  DATA_TYPE *localWriteA_0_0_31_0;
  DATA_TYPE *localWriteA_0_0_32_0;
  DATA_TYPE *localWriteA_0_0_33_0;
  DATA_TYPE *localWriteA_0_0_34_0;
  DATA_TYPE *localWriteA_0_0_35_0;
  DATA_TYPE *localWriteA_0_0_36_0;
  DATA_TYPE *localWriteA_0_0_37_0;
  DATA_TYPE *localWriteA_0_0_38_0;
  DATA_TYPE *localWriteA_0_0_39_0;
  DATA_TYPE *localWriteA_0_0_40_0;
  DATA_TYPE *localWriteA_0_0_41_0;
  DATA_TYPE *localWriteA_0_0_42_0;
  DATA_TYPE *localWriteA_0_0_43_0;
  DATA_TYPE *localWriteA_0_0_44_0;
  DATA_TYPE *localWriteA_0_0_45_0;
  DATA_TYPE *localWriteA_0_0_46_0;
  DATA_TYPE *localWriteA_0_0_47_0;
  DATA_TYPE *localWriteA_0_0_48_0;
  DATA_TYPE *localWriteA_0_0_49_0;
  DATA_TYPE *localWriteA_0_0_50_0;
  DATA_TYPE *localWriteA_0_0_51_0;
  DATA_TYPE *localWriteA_0_0_52_0;
  DATA_TYPE *localWriteA_0_0_53_0;
  DATA_TYPE *localWriteA_0_0_54_0;
  DATA_TYPE *localWriteA_0_0_55_0;
  DATA_TYPE *localWriteA_0_0_56_0;
  DATA_TYPE *localWriteA_0_0_57_0;
  DATA_TYPE *localWriteA_0_0_58_0;
  DATA_TYPE *localWriteA_0_0_59_0;
  DATA_TYPE *localWriteA_0_0_60_0;
  DATA_TYPE *localWriteA_0_0_61_0;
  DATA_TYPE *localWriteA_0_0_62_0;
  DATA_TYPE *localWriteA_0_0_63_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);
  localWriteA_0_0_16_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_16_0);
  localWriteA_0_0_17_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_17_0);
  localWriteA_0_0_18_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_18_0);
  localWriteA_0_0_19_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_19_0);
  localWriteA_0_0_20_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_20_0);
  localWriteA_0_0_21_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_21_0);
  localWriteA_0_0_22_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_22_0);
  localWriteA_0_0_23_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_23_0);
  localWriteA_0_0_24_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_24_0);
  localWriteA_0_0_25_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_25_0);
  localWriteA_0_0_26_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_26_0);
  localWriteA_0_0_27_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_27_0);
  localWriteA_0_0_28_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_28_0);
  localWriteA_0_0_29_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_29_0);
  localWriteA_0_0_30_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_30_0);
  localWriteA_0_0_31_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_31_0);
  localWriteA_0_0_32_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_32_0);
  localWriteA_0_0_33_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_33_0);
  localWriteA_0_0_34_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_34_0);
  localWriteA_0_0_35_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_35_0);
  localWriteA_0_0_36_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_36_0);
  localWriteA_0_0_37_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_37_0);
  localWriteA_0_0_38_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_38_0);
  localWriteA_0_0_39_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_39_0);
  localWriteA_0_0_40_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_40_0);
  localWriteA_0_0_41_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_41_0);
  localWriteA_0_0_42_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_42_0);
  localWriteA_0_0_43_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_43_0);
  localWriteA_0_0_44_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_44_0);
  localWriteA_0_0_45_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_45_0);
  localWriteA_0_0_46_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_46_0);
  localWriteA_0_0_47_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_47_0);
  localWriteA_0_0_48_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_48_0);
  localWriteA_0_0_49_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_49_0);
  localWriteA_0_0_50_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_50_0);
  localWriteA_0_0_51_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_51_0);
  localWriteA_0_0_52_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_52_0);
  localWriteA_0_0_53_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_53_0);
  localWriteA_0_0_54_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_54_0);
  localWriteA_0_0_55_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_55_0);
  localWriteA_0_0_56_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_56_0);
  localWriteA_0_0_57_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_57_0);
  localWriteA_0_0_58_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_58_0);
  localWriteA_0_0_59_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_59_0);
  localWriteA_0_0_60_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_60_0);
  localWriteA_0_0_61_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_61_0);
  localWriteA_0_0_62_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_62_0);
  localWriteA_0_0_63_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_63_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);
    a_0_0_16_0 = *(globalReadA_0_0_16_0 + 0);
    a_0_0_17_0 = *(globalReadA_0_0_17_0 + 0);
    a_0_0_18_0 = *(globalReadA_0_0_18_0 + 0);
    a_0_0_19_0 = *(globalReadA_0_0_19_0 + 0);
    a_0_0_20_0 = *(globalReadA_0_0_20_0 + 0);
    a_0_0_21_0 = *(globalReadA_0_0_21_0 + 0);
    a_0_0_22_0 = *(globalReadA_0_0_22_0 + 0);
    a_0_0_23_0 = *(globalReadA_0_0_23_0 + 0);
    a_0_0_24_0 = *(globalReadA_0_0_24_0 + 0);
    a_0_0_25_0 = *(globalReadA_0_0_25_0 + 0);
    a_0_0_26_0 = *(globalReadA_0_0_26_0 + 0);
    a_0_0_27_0 = *(globalReadA_0_0_27_0 + 0);
    a_0_0_28_0 = *(globalReadA_0_0_28_0 + 0);
    a_0_0_29_0 = *(globalReadA_0_0_29_0 + 0);
    a_0_0_30_0 = *(globalReadA_0_0_30_0 + 0);
    a_0_0_31_0 = *(globalReadA_0_0_31_0 + 0);
    a_0_0_32_0 = *(globalReadA_0_0_32_0 + 0);
    a_0_0_33_0 = *(globalReadA_0_0_33_0 + 0);
    a_0_0_34_0 = *(globalReadA_0_0_34_0 + 0);
    a_0_0_35_0 = *(globalReadA_0_0_35_0 + 0);
    a_0_0_36_0 = *(globalReadA_0_0_36_0 + 0);
    a_0_0_37_0 = *(globalReadA_0_0_37_0 + 0);
    a_0_0_38_0 = *(globalReadA_0_0_38_0 + 0);
    a_0_0_39_0 = *(globalReadA_0_0_39_0 + 0);
    a_0_0_40_0 = *(globalReadA_0_0_40_0 + 0);
    a_0_0_41_0 = *(globalReadA_0_0_41_0 + 0);
    a_0_0_42_0 = *(globalReadA_0_0_42_0 + 0);
    a_0_0_43_0 = *(globalReadA_0_0_43_0 + 0);
    a_0_0_44_0 = *(globalReadA_0_0_44_0 + 0);
    a_0_0_45_0 = *(globalReadA_0_0_45_0 + 0);
    a_0_0_46_0 = *(globalReadA_0_0_46_0 + 0);
    a_0_0_47_0 = *(globalReadA_0_0_47_0 + 0);
    a_0_0_48_0 = *(globalReadA_0_0_48_0 + 0);
    a_0_0_49_0 = *(globalReadA_0_0_49_0 + 0);
    a_0_0_50_0 = *(globalReadA_0_0_50_0 + 0);
    a_0_0_51_0 = *(globalReadA_0_0_51_0 + 0);
    a_0_0_52_0 = *(globalReadA_0_0_52_0 + 0);
    a_0_0_53_0 = *(globalReadA_0_0_53_0 + 0);
    a_0_0_54_0 = *(globalReadA_0_0_54_0 + 0);
    a_0_0_55_0 = *(globalReadA_0_0_55_0 + 0);
    a_0_0_56_0 = *(globalReadA_0_0_56_0 + 0);
    a_0_0_57_0 = *(globalReadA_0_0_57_0 + 0);
    a_0_0_58_0 = *(globalReadA_0_0_58_0 + 0);
    a_0_0_59_0 = *(globalReadA_0_0_59_0 + 0);
    a_0_0_60_0 = *(globalReadA_0_0_60_0 + 0);
    a_0_0_61_0 = *(globalReadA_0_0_61_0 + 0);
    a_0_0_62_0 = *(globalReadA_0_0_62_0 + 0);
    a_0_0_63_0 = *(globalReadA_0_0_63_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    globalReadA_0_0_16_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_16_0) + 1*globalReadIncAL);
    globalReadA_0_0_17_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_17_0) + 1*globalReadIncAL);
    globalReadA_0_0_18_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_18_0) + 1*globalReadIncAL);
    globalReadA_0_0_19_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_19_0) + 1*globalReadIncAL);
    globalReadA_0_0_20_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_20_0) + 1*globalReadIncAL);
    globalReadA_0_0_21_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_21_0) + 1*globalReadIncAL);
    globalReadA_0_0_22_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_22_0) + 1*globalReadIncAL);
    globalReadA_0_0_23_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_23_0) + 1*globalReadIncAL);
    globalReadA_0_0_24_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_24_0) + 1*globalReadIncAL);
    globalReadA_0_0_25_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_25_0) + 1*globalReadIncAL);
    globalReadA_0_0_26_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_26_0) + 1*globalReadIncAL);
    globalReadA_0_0_27_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_27_0) + 1*globalReadIncAL);
    globalReadA_0_0_28_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_28_0) + 1*globalReadIncAL);
    globalReadA_0_0_29_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_29_0) + 1*globalReadIncAL);
    globalReadA_0_0_30_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_30_0) + 1*globalReadIncAL);
    globalReadA_0_0_31_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_31_0) + 1*globalReadIncAL);
    globalReadA_0_0_32_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_32_0) + 1*globalReadIncAL);
    globalReadA_0_0_33_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_33_0) + 1*globalReadIncAL);
    globalReadA_0_0_34_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_34_0) + 1*globalReadIncAL);
    globalReadA_0_0_35_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_35_0) + 1*globalReadIncAL);
    globalReadA_0_0_36_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_36_0) + 1*globalReadIncAL);
    globalReadA_0_0_37_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_37_0) + 1*globalReadIncAL);
    globalReadA_0_0_38_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_38_0) + 1*globalReadIncAL);
    globalReadA_0_0_39_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_39_0) + 1*globalReadIncAL);
    globalReadA_0_0_40_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_40_0) + 1*globalReadIncAL);
    globalReadA_0_0_41_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_41_0) + 1*globalReadIncAL);
    globalReadA_0_0_42_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_42_0) + 1*globalReadIncAL);
    globalReadA_0_0_43_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_43_0) + 1*globalReadIncAL);
    globalReadA_0_0_44_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_44_0) + 1*globalReadIncAL);
    globalReadA_0_0_45_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_45_0) + 1*globalReadIncAL);
    globalReadA_0_0_46_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_46_0) + 1*globalReadIncAL);
    globalReadA_0_0_47_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_47_0) + 1*globalReadIncAL);
    globalReadA_0_0_48_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_48_0) + 1*globalReadIncAL);
    globalReadA_0_0_49_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_49_0) + 1*globalReadIncAL);
    globalReadA_0_0_50_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_50_0) + 1*globalReadIncAL);
    globalReadA_0_0_51_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_51_0) + 1*globalReadIncAL);
    globalReadA_0_0_52_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_52_0) + 1*globalReadIncAL);
    globalReadA_0_0_53_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_53_0) + 1*globalReadIncAL);
    globalReadA_0_0_54_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_54_0) + 1*globalReadIncAL);
    globalReadA_0_0_55_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_55_0) + 1*globalReadIncAL);
    globalReadA_0_0_56_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_56_0) + 1*globalReadIncAL);
    globalReadA_0_0_57_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_57_0) + 1*globalReadIncAL);
    globalReadA_0_0_58_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_58_0) + 1*globalReadIncAL);
    globalReadA_0_0_59_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_59_0) + 1*globalReadIncAL);
    globalReadA_0_0_60_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_60_0) + 1*globalReadIncAL);
    globalReadA_0_0_61_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_61_0) + 1*globalReadIncAL);
    globalReadA_0_0_62_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_62_0) + 1*globalReadIncAL);
    globalReadA_0_0_63_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_63_0) + 1*globalReadIncAL);

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
    *(localWriteA_0_0_16_0 + 0) = a_0_0_16_0;
    *(localWriteA_0_0_17_0 + 0) = a_0_0_17_0;
    *(localWriteA_0_0_18_0 + 0) = a_0_0_18_0;
    *(localWriteA_0_0_19_0 + 0) = a_0_0_19_0;
    *(localWriteA_0_0_20_0 + 0) = a_0_0_20_0;
    *(localWriteA_0_0_21_0 + 0) = a_0_0_21_0;
    *(localWriteA_0_0_22_0 + 0) = a_0_0_22_0;
    *(localWriteA_0_0_23_0 + 0) = a_0_0_23_0;
    *(localWriteA_0_0_24_0 + 0) = a_0_0_24_0;
    *(localWriteA_0_0_25_0 + 0) = a_0_0_25_0;
    *(localWriteA_0_0_26_0 + 0) = a_0_0_26_0;
    *(localWriteA_0_0_27_0 + 0) = a_0_0_27_0;
    *(localWriteA_0_0_28_0 + 0) = a_0_0_28_0;
    *(localWriteA_0_0_29_0 + 0) = a_0_0_29_0;
    *(localWriteA_0_0_30_0 + 0) = a_0_0_30_0;
    *(localWriteA_0_0_31_0 + 0) = a_0_0_31_0;
    *(localWriteA_0_0_32_0 + 0) = a_0_0_32_0;
    *(localWriteA_0_0_33_0 + 0) = a_0_0_33_0;
    *(localWriteA_0_0_34_0 + 0) = a_0_0_34_0;
    *(localWriteA_0_0_35_0 + 0) = a_0_0_35_0;
    *(localWriteA_0_0_36_0 + 0) = a_0_0_36_0;
    *(localWriteA_0_0_37_0 + 0) = a_0_0_37_0;
    *(localWriteA_0_0_38_0 + 0) = a_0_0_38_0;
    *(localWriteA_0_0_39_0 + 0) = a_0_0_39_0;
    *(localWriteA_0_0_40_0 + 0) = a_0_0_40_0;
    *(localWriteA_0_0_41_0 + 0) = a_0_0_41_0;
    *(localWriteA_0_0_42_0 + 0) = a_0_0_42_0;
    *(localWriteA_0_0_43_0 + 0) = a_0_0_43_0;
    *(localWriteA_0_0_44_0 + 0) = a_0_0_44_0;
    *(localWriteA_0_0_45_0 + 0) = a_0_0_45_0;
    *(localWriteA_0_0_46_0 + 0) = a_0_0_46_0;
    *(localWriteA_0_0_47_0 + 0) = a_0_0_47_0;
    *(localWriteA_0_0_48_0 + 0) = a_0_0_48_0;
    *(localWriteA_0_0_49_0 + 0) = a_0_0_49_0;
    *(localWriteA_0_0_50_0 + 0) = a_0_0_50_0;
    *(localWriteA_0_0_51_0 + 0) = a_0_0_51_0;
    *(localWriteA_0_0_52_0 + 0) = a_0_0_52_0;
    *(localWriteA_0_0_53_0 + 0) = a_0_0_53_0;
    *(localWriteA_0_0_54_0 + 0) = a_0_0_54_0;
    *(localWriteA_0_0_55_0 + 0) = a_0_0_55_0;
    *(localWriteA_0_0_56_0 + 0) = a_0_0_56_0;
    *(localWriteA_0_0_57_0 + 0) = a_0_0_57_0;
    *(localWriteA_0_0_58_0 + 0) = a_0_0_58_0;
    *(localWriteA_0_0_59_0 + 0) = a_0_0_59_0;
    *(localWriteA_0_0_60_0 + 0) = a_0_0_60_0;
    *(localWriteA_0_0_61_0 + 0) = a_0_0_61_0;
    *(localWriteA_0_0_62_0 + 0) = a_0_0_62_0;
    *(localWriteA_0_0_63_0 + 0) = a_0_0_63_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 15 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 16 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 17 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 18 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 19 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 20 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 21 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 22 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 23 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 24 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 25 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 26 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 27 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 28 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 29 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 30 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 31 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 32 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 33 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 34 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 35 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 36 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 37 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 38 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 39 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 40 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 41 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 42 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 43 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 44 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 45 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 46 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 47 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 48 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 49 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 50 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 51 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 52 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 53 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 54 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 55 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 56 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 57 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 58 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 59 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 60 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 61 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1_BLK


    /* iter 62 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_1x1


    /* iter 63 */

    MAC_1x1_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_0_8_0 = ( globalReadOffsetAL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_0_0_9_0 = ( globalReadOffsetAL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_0_0_10_0 = ( globalReadOffsetAL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_0_0_11_0 = ( globalReadOffsetAL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_0_0_12_0 = ( globalReadOffsetAL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_0_0_13_0 = ( globalReadOffsetAL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_0_0_14_0 = ( globalReadOffsetAL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_0_0_15_0 = ( globalReadOffsetAL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);
  a_0_0_16_0 = ( globalReadOffsetAL_16_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_16_0 + 0);
  a_0_0_17_0 = ( globalReadOffsetAL_17_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_17_0 + 0);
  a_0_0_18_0 = ( globalReadOffsetAL_18_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_18_0 + 0);
  a_0_0_19_0 = ( globalReadOffsetAL_19_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_19_0 + 0);
  a_0_0_20_0 = ( globalReadOffsetAL_20_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_20_0 + 0);
  a_0_0_21_0 = ( globalReadOffsetAL_21_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_21_0 + 0);
  a_0_0_22_0 = ( globalReadOffsetAL_22_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_22_0 + 0);
  a_0_0_23_0 = ( globalReadOffsetAL_23_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_23_0 + 0);
  a_0_0_24_0 = ( globalReadOffsetAL_24_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_24_0 + 0);
  a_0_0_25_0 = ( globalReadOffsetAL_25_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_25_0 + 0);
  a_0_0_26_0 = ( globalReadOffsetAL_26_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_26_0 + 0);
  a_0_0_27_0 = ( globalReadOffsetAL_27_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_27_0 + 0);
  a_0_0_28_0 = ( globalReadOffsetAL_28_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_28_0 + 0);
  a_0_0_29_0 = ( globalReadOffsetAL_29_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_29_0 + 0);
  a_0_0_30_0 = ( globalReadOffsetAL_30_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_30_0 + 0);
  a_0_0_31_0 = ( globalReadOffsetAL_31_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_31_0 + 0);
  a_0_0_32_0 = ( globalReadOffsetAL_32_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_32_0 + 0);
  a_0_0_33_0 = ( globalReadOffsetAL_33_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_33_0 + 0);
  a_0_0_34_0 = ( globalReadOffsetAL_34_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_34_0 + 0);
  a_0_0_35_0 = ( globalReadOffsetAL_35_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_35_0 + 0);
  a_0_0_36_0 = ( globalReadOffsetAL_36_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_36_0 + 0);
  a_0_0_37_0 = ( globalReadOffsetAL_37_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_37_0 + 0);
  a_0_0_38_0 = ( globalReadOffsetAL_38_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_38_0 + 0);
  a_0_0_39_0 = ( globalReadOffsetAL_39_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_39_0 + 0);
  a_0_0_40_0 = ( globalReadOffsetAL_40_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_40_0 + 0);
  a_0_0_41_0 = ( globalReadOffsetAL_41_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_41_0 + 0);
  a_0_0_42_0 = ( globalReadOffsetAL_42_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_42_0 + 0);
  a_0_0_43_0 = ( globalReadOffsetAL_43_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_43_0 + 0);
  a_0_0_44_0 = ( globalReadOffsetAL_44_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_44_0 + 0);
  a_0_0_45_0 = ( globalReadOffsetAL_45_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_45_0 + 0);
  a_0_0_46_0 = ( globalReadOffsetAL_46_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_46_0 + 0);
  a_0_0_47_0 = ( globalReadOffsetAL_47_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_47_0 + 0);
  a_0_0_48_0 = ( globalReadOffsetAL_48_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_48_0 + 0);
  a_0_0_49_0 = ( globalReadOffsetAL_49_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_49_0 + 0);
  a_0_0_50_0 = ( globalReadOffsetAL_50_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_50_0 + 0);
  a_0_0_51_0 = ( globalReadOffsetAL_51_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_51_0 + 0);
  a_0_0_52_0 = ( globalReadOffsetAL_52_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_52_0 + 0);
  a_0_0_53_0 = ( globalReadOffsetAL_53_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_53_0 + 0);
  a_0_0_54_0 = ( globalReadOffsetAL_54_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_54_0 + 0);
  a_0_0_55_0 = ( globalReadOffsetAL_55_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_55_0 + 0);
  a_0_0_56_0 = ( globalReadOffsetAL_56_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_56_0 + 0);
  a_0_0_57_0 = ( globalReadOffsetAL_57_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_57_0 + 0);
  a_0_0_58_0 = ( globalReadOffsetAL_58_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_58_0 + 0);
  a_0_0_59_0 = ( globalReadOffsetAL_59_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_59_0 + 0);
  a_0_0_60_0 = ( globalReadOffsetAL_60_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_60_0 + 0);
  a_0_0_61_0 = ( globalReadOffsetAL_61_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_61_0 + 0);
  a_0_0_62_0 = ( globalReadOffsetAL_62_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_62_0 + 0);
  a_0_0_63_0 = ( globalReadOffsetAL_63_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_63_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);
  localWriteA_0_0_16_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_16_0);
  localWriteA_0_0_17_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_17_0);
  localWriteA_0_0_18_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_18_0);
  localWriteA_0_0_19_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_19_0);
  localWriteA_0_0_20_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_20_0);
  localWriteA_0_0_21_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_21_0);
  localWriteA_0_0_22_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_22_0);
  localWriteA_0_0_23_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_23_0);
  localWriteA_0_0_24_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_24_0);
  localWriteA_0_0_25_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_25_0);
  localWriteA_0_0_26_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_26_0);
  localWriteA_0_0_27_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_27_0);
  localWriteA_0_0_28_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_28_0);
  localWriteA_0_0_29_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_29_0);
  localWriteA_0_0_30_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_30_0);
  localWriteA_0_0_31_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_31_0);
  localWriteA_0_0_32_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_32_0);
  localWriteA_0_0_33_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_33_0);
  localWriteA_0_0_34_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_34_0);
  localWriteA_0_0_35_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_35_0);
  localWriteA_0_0_36_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_36_0);
  localWriteA_0_0_37_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_37_0);
  localWriteA_0_0_38_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_38_0);
  localWriteA_0_0_39_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_39_0);
  localWriteA_0_0_40_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_40_0);
  localWriteA_0_0_41_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_41_0);
  localWriteA_0_0_42_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_42_0);
  localWriteA_0_0_43_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_43_0);
  localWriteA_0_0_44_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_44_0);
  localWriteA_0_0_45_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_45_0);
  localWriteA_0_0_46_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_46_0);
  localWriteA_0_0_47_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_47_0);
  localWriteA_0_0_48_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_48_0);
  localWriteA_0_0_49_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_49_0);
  localWriteA_0_0_50_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_50_0);
  localWriteA_0_0_51_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_51_0);
  localWriteA_0_0_52_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_52_0);
  localWriteA_0_0_53_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_53_0);
  localWriteA_0_0_54_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_54_0);
  localWriteA_0_0_55_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_55_0);
  localWriteA_0_0_56_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_56_0);
  localWriteA_0_0_57_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_57_0);
  localWriteA_0_0_58_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_58_0);
  localWriteA_0_0_59_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_59_0);
  localWriteA_0_0_60_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_60_0);
  localWriteA_0_0_61_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_61_0);
  localWriteA_0_0_62_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_62_0);
  localWriteA_0_0_63_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_63_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
  *(localWriteA_0_0_16_0 + 0) = a_0_0_16_0;
  *(localWriteA_0_0_17_0 + 0) = a_0_0_17_0;
  *(localWriteA_0_0_18_0 + 0) = a_0_0_18_0;
  *(localWriteA_0_0_19_0 + 0) = a_0_0_19_0;
  *(localWriteA_0_0_20_0 + 0) = a_0_0_20_0;
  *(localWriteA_0_0_21_0 + 0) = a_0_0_21_0;
  *(localWriteA_0_0_22_0 + 0) = a_0_0_22_0;
  *(localWriteA_0_0_23_0 + 0) = a_0_0_23_0;
  *(localWriteA_0_0_24_0 + 0) = a_0_0_24_0;
  *(localWriteA_0_0_25_0 + 0) = a_0_0_25_0;
  *(localWriteA_0_0_26_0 + 0) = a_0_0_26_0;
  *(localWriteA_0_0_27_0 + 0) = a_0_0_27_0;
  *(localWriteA_0_0_28_0 + 0) = a_0_0_28_0;
  *(localWriteA_0_0_29_0 + 0) = a_0_0_29_0;
  *(localWriteA_0_0_30_0 + 0) = a_0_0_30_0;
  *(localWriteA_0_0_31_0 + 0) = a_0_0_31_0;
  *(localWriteA_0_0_32_0 + 0) = a_0_0_32_0;
  *(localWriteA_0_0_33_0 + 0) = a_0_0_33_0;
  *(localWriteA_0_0_34_0 + 0) = a_0_0_34_0;
  *(localWriteA_0_0_35_0 + 0) = a_0_0_35_0;
  *(localWriteA_0_0_36_0 + 0) = a_0_0_36_0;
  *(localWriteA_0_0_37_0 + 0) = a_0_0_37_0;
  *(localWriteA_0_0_38_0 + 0) = a_0_0_38_0;
  *(localWriteA_0_0_39_0 + 0) = a_0_0_39_0;
  *(localWriteA_0_0_40_0 + 0) = a_0_0_40_0;
  *(localWriteA_0_0_41_0 + 0) = a_0_0_41_0;
  *(localWriteA_0_0_42_0 + 0) = a_0_0_42_0;
  *(localWriteA_0_0_43_0 + 0) = a_0_0_43_0;
  *(localWriteA_0_0_44_0 + 0) = a_0_0_44_0;
  *(localWriteA_0_0_45_0 + 0) = a_0_0_45_0;
  *(localWriteA_0_0_46_0 + 0) = a_0_0_46_0;
  *(localWriteA_0_0_47_0 + 0) = a_0_0_47_0;
  *(localWriteA_0_0_48_0 + 0) = a_0_0_48_0;
  *(localWriteA_0_0_49_0 + 0) = a_0_0_49_0;
  *(localWriteA_0_0_50_0 + 0) = a_0_0_50_0;
  *(localWriteA_0_0_51_0 + 0) = a_0_0_51_0;
  *(localWriteA_0_0_52_0 + 0) = a_0_0_52_0;
  *(localWriteA_0_0_53_0 + 0) = a_0_0_53_0;
  *(localWriteA_0_0_54_0 + 0) = a_0_0_54_0;
  *(localWriteA_0_0_55_0 + 0) = a_0_0_55_0;
  *(localWriteA_0_0_56_0 + 0) = a_0_0_56_0;
  *(localWriteA_0_0_57_0 + 0) = a_0_0_57_0;
  *(localWriteA_0_0_58_0 + 0) = a_0_0_58_0;
  *(localWriteA_0_0_59_0 + 0) = a_0_0_59_0;
  *(localWriteA_0_0_60_0 + 0) = a_0_0_60_0;
  *(localWriteA_0_0_61_0 + 0) = a_0_0_61_0;
  *(localWriteA_0_0_62_0 + 0) = a_0_0_62_0;
  *(localWriteA_0_0_63_0 + 0) = a_0_0_63_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_1x1

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_1x1
#undef MAC_1x1_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 64
#define SG1J 1
#define TT0I 1
#define TT1J 1
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 64
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 64
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 4096
#define LDS_NUM_ELEMENTS 4160

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 1x1 micro-tile */
#define MAC_1x1\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_BBH_MT64x1x64_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU0_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT1_1_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG64_1_1_WGM1(
  float *D,
  float const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE a_0_0_16_0;
  DATA_TYPE a_0_0_17_0;
  DATA_TYPE a_0_0_18_0;
  DATA_TYPE a_0_0_19_0;
  DATA_TYPE a_0_0_20_0;
  DATA_TYPE a_0_0_21_0;
  DATA_TYPE a_0_0_22_0;
  DATA_TYPE a_0_0_23_0;
  DATA_TYPE a_0_0_24_0;
  DATA_TYPE a_0_0_25_0;
  DATA_TYPE a_0_0_26_0;
  DATA_TYPE a_0_0_27_0;
  DATA_TYPE a_0_0_28_0;
  DATA_TYPE a_0_0_29_0;
  DATA_TYPE a_0_0_30_0;
  DATA_TYPE a_0_0_31_0;
  DATA_TYPE a_0_0_32_0;
  DATA_TYPE a_0_0_33_0;
  DATA_TYPE a_0_0_34_0;
  DATA_TYPE a_0_0_35_0;
  DATA_TYPE a_0_0_36_0;
  DATA_TYPE a_0_0_37_0;
  DATA_TYPE a_0_0_38_0;
  DATA_TYPE a_0_0_39_0;
  DATA_TYPE a_0_0_40_0;
  DATA_TYPE a_0_0_41_0;
  DATA_TYPE a_0_0_42_0;
  DATA_TYPE a_0_0_43_0;
  DATA_TYPE a_0_0_44_0;
  DATA_TYPE a_0_0_45_0;
  DATA_TYPE a_0_0_46_0;
  DATA_TYPE a_0_0_47_0;
  DATA_TYPE a_0_0_48_0;
  DATA_TYPE a_0_0_49_0;
  DATA_TYPE a_0_0_50_0;
  DATA_TYPE a_0_0_51_0;
  DATA_TYPE a_0_0_52_0;
  DATA_TYPE a_0_0_53_0;
  DATA_TYPE a_0_0_54_0;
  DATA_TYPE a_0_0_55_0;
  DATA_TYPE a_0_0_56_0;
  DATA_TYPE a_0_0_57_0;
  DATA_TYPE a_0_0_58_0;
  DATA_TYPE a_0_0_59_0;
  DATA_TYPE a_0_0_60_0;
  DATA_TYPE a_0_0_61_0;
  DATA_TYPE a_0_0_62_0;
  DATA_TYPE a_0_0_63_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;
  unsigned int globalReadOffsetAL_8_0 = globalReadOffsetAL + 0 + 8*LSPA;
  unsigned int globalReadOffsetAL_9_0 = globalReadOffsetAL + 0 + 9*LSPA;
  unsigned int globalReadOffsetAL_10_0 = globalReadOffsetAL + 0 + 10*LSPA;
  unsigned int globalReadOffsetAL_11_0 = globalReadOffsetAL + 0 + 11*LSPA;
  unsigned int globalReadOffsetAL_12_0 = globalReadOffsetAL + 0 + 12*LSPA;
  unsigned int globalReadOffsetAL_13_0 = globalReadOffsetAL + 0 + 13*LSPA;
  unsigned int globalReadOffsetAL_14_0 = globalReadOffsetAL + 0 + 14*LSPA;
  unsigned int globalReadOffsetAL_15_0 = globalReadOffsetAL + 0 + 15*LSPA;
  unsigned int globalReadOffsetAL_16_0 = globalReadOffsetAL + 0 + 16*LSPA;
  unsigned int globalReadOffsetAL_17_0 = globalReadOffsetAL + 0 + 17*LSPA;
  unsigned int globalReadOffsetAL_18_0 = globalReadOffsetAL + 0 + 18*LSPA;
  unsigned int globalReadOffsetAL_19_0 = globalReadOffsetAL + 0 + 19*LSPA;
  unsigned int globalReadOffsetAL_20_0 = globalReadOffsetAL + 0 + 20*LSPA;
  unsigned int globalReadOffsetAL_21_0 = globalReadOffsetAL + 0 + 21*LSPA;
  unsigned int globalReadOffsetAL_22_0 = globalReadOffsetAL + 0 + 22*LSPA;
  unsigned int globalReadOffsetAL_23_0 = globalReadOffsetAL + 0 + 23*LSPA;
  unsigned int globalReadOffsetAL_24_0 = globalReadOffsetAL + 0 + 24*LSPA;
  unsigned int globalReadOffsetAL_25_0 = globalReadOffsetAL + 0 + 25*LSPA;
  unsigned int globalReadOffsetAL_26_0 = globalReadOffsetAL + 0 + 26*LSPA;
  unsigned int globalReadOffsetAL_27_0 = globalReadOffsetAL + 0 + 27*LSPA;
  unsigned int globalReadOffsetAL_28_0 = globalReadOffsetAL + 0 + 28*LSPA;
  unsigned int globalReadOffsetAL_29_0 = globalReadOffsetAL + 0 + 29*LSPA;
  unsigned int globalReadOffsetAL_30_0 = globalReadOffsetAL + 0 + 30*LSPA;
  unsigned int globalReadOffsetAL_31_0 = globalReadOffsetAL + 0 + 31*LSPA;
  unsigned int globalReadOffsetAL_32_0 = globalReadOffsetAL + 0 + 32*LSPA;
  unsigned int globalReadOffsetAL_33_0 = globalReadOffsetAL + 0 + 33*LSPA;
  unsigned int globalReadOffsetAL_34_0 = globalReadOffsetAL + 0 + 34*LSPA;
  unsigned int globalReadOffsetAL_35_0 = globalReadOffsetAL + 0 + 35*LSPA;
  unsigned int globalReadOffsetAL_36_0 = globalReadOffsetAL + 0 + 36*LSPA;
  unsigned int globalReadOffsetAL_37_0 = globalReadOffsetAL + 0 + 37*LSPA;
  unsigned int globalReadOffsetAL_38_0 = globalReadOffsetAL + 0 + 38*LSPA;
  unsigned int globalReadOffsetAL_39_0 = globalReadOffsetAL + 0 + 39*LSPA;
  unsigned int globalReadOffsetAL_40_0 = globalReadOffsetAL + 0 + 40*LSPA;
  unsigned int globalReadOffsetAL_41_0 = globalReadOffsetAL + 0 + 41*LSPA;
  unsigned int globalReadOffsetAL_42_0 = globalReadOffsetAL + 0 + 42*LSPA;
  unsigned int globalReadOffsetAL_43_0 = globalReadOffsetAL + 0 + 43*LSPA;
  unsigned int globalReadOffsetAL_44_0 = globalReadOffsetAL + 0 + 44*LSPA;
  unsigned int globalReadOffsetAL_45_0 = globalReadOffsetAL + 0 + 45*LSPA;
  unsigned int globalReadOffsetAL_46_0 = globalReadOffsetAL + 0 + 46*LSPA;
  unsigned int globalReadOffsetAL_47_0 = globalReadOffsetAL + 0 + 47*LSPA;
  unsigned int globalReadOffsetAL_48_0 = globalReadOffsetAL + 0 + 48*LSPA;
  unsigned int globalReadOffsetAL_49_0 = globalReadOffsetAL + 0 + 49*LSPA;
  unsigned int globalReadOffsetAL_50_0 = globalReadOffsetAL + 0 + 50*LSPA;
  unsigned int globalReadOffsetAL_51_0 = globalReadOffsetAL + 0 + 51*LSPA;
  unsigned int globalReadOffsetAL_52_0 = globalReadOffsetAL + 0 + 52*LSPA;
  unsigned int globalReadOffsetAL_53_0 = globalReadOffsetAL + 0 + 53*LSPA;
  unsigned int globalReadOffsetAL_54_0 = globalReadOffsetAL + 0 + 54*LSPA;
  unsigned int globalReadOffsetAL_55_0 = globalReadOffsetAL + 0 + 55*LSPA;
  unsigned int globalReadOffsetAL_56_0 = globalReadOffsetAL + 0 + 56*LSPA;
  unsigned int globalReadOffsetAL_57_0 = globalReadOffsetAL + 0 + 57*LSPA;
  unsigned int globalReadOffsetAL_58_0 = globalReadOffsetAL + 0 + 58*LSPA;
  unsigned int globalReadOffsetAL_59_0 = globalReadOffsetAL + 0 + 59*LSPA;
  unsigned int globalReadOffsetAL_60_0 = globalReadOffsetAL + 0 + 60*LSPA;
  unsigned int globalReadOffsetAL_61_0 = globalReadOffsetAL + 0 + 61*LSPA;
  unsigned int globalReadOffsetAL_62_0 = globalReadOffsetAL + 0 + 62*LSPA;
  unsigned int globalReadOffsetAL_63_0 = globalReadOffsetAL + 0 + 63*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_15_0), wgK );
  int64_t globalReadOffsetA_0_0_16_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_16_0), wgK );
  int64_t globalReadOffsetA_0_0_17_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_17_0), wgK );
  int64_t globalReadOffsetA_0_0_18_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_18_0), wgK );
  int64_t globalReadOffsetA_0_0_19_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_19_0), wgK );
  int64_t globalReadOffsetA_0_0_20_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_20_0), wgK );
  int64_t globalReadOffsetA_0_0_21_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_21_0), wgK );
  int64_t globalReadOffsetA_0_0_22_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_22_0), wgK );
  int64_t globalReadOffsetA_0_0_23_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_23_0), wgK );
  int64_t globalReadOffsetA_0_0_24_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_24_0), wgK );
  int64_t globalReadOffsetA_0_0_25_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_25_0), wgK );
  int64_t globalReadOffsetA_0_0_26_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_26_0), wgK );
  int64_t globalReadOffsetA_0_0_27_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_27_0), wgK );
  int64_t globalReadOffsetA_0_0_28_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_28_0), wgK );
  int64_t globalReadOffsetA_0_0_29_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_29_0), wgK );
  int64_t globalReadOffsetA_0_0_30_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_30_0), wgK );
  int64_t globalReadOffsetA_0_0_31_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_31_0), wgK );
  int64_t globalReadOffsetA_0_0_32_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_32_0), wgK );
  int64_t globalReadOffsetA_0_0_33_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_33_0), wgK );
  int64_t globalReadOffsetA_0_0_34_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_34_0), wgK );
  int64_t globalReadOffsetA_0_0_35_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_35_0), wgK );
  int64_t globalReadOffsetA_0_0_36_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_36_0), wgK );
  int64_t globalReadOffsetA_0_0_37_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_37_0), wgK );
  int64_t globalReadOffsetA_0_0_38_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_38_0), wgK );
  int64_t globalReadOffsetA_0_0_39_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_39_0), wgK );
  int64_t globalReadOffsetA_0_0_40_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_40_0), wgK );
  int64_t globalReadOffsetA_0_0_41_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_41_0), wgK );
  int64_t globalReadOffsetA_0_0_42_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_42_0), wgK );
  int64_t globalReadOffsetA_0_0_43_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_43_0), wgK );
  int64_t globalReadOffsetA_0_0_44_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_44_0), wgK );
  int64_t globalReadOffsetA_0_0_45_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_45_0), wgK );
  int64_t globalReadOffsetA_0_0_46_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_46_0), wgK );
  int64_t globalReadOffsetA_0_0_47_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_47_0), wgK );
  int64_t globalReadOffsetA_0_0_48_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_48_0), wgK );
  int64_t globalReadOffsetA_0_0_49_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_49_0), wgK );
  int64_t globalReadOffsetA_0_0_50_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_50_0), wgK );
  int64_t globalReadOffsetA_0_0_51_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_51_0), wgK );
  int64_t globalReadOffsetA_0_0_52_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_52_0), wgK );
  int64_t globalReadOffsetA_0_0_53_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_53_0), wgK );
  int64_t globalReadOffsetA_0_0_54_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_54_0), wgK );
  int64_t globalReadOffsetA_0_0_55_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_55_0), wgK );
  int64_t globalReadOffsetA_0_0_56_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_56_0), wgK );
  int64_t globalReadOffsetA_0_0_57_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_57_0), wgK );
  int64_t globalReadOffsetA_0_0_58_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_58_0), wgK );
  int64_t globalReadOffsetA_0_0_59_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_59_0), wgK );
  int64_t globalReadOffsetA_0_0_60_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_60_0), wgK );
  int64_t globalReadOffsetA_0_0_61_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_61_0), wgK );
  int64_t globalReadOffsetA_0_0_62_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_62_0), wgK );
  int64_t globalReadOffsetA_0_0_63_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_63_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;
  DATA_TYPE const *globalReadA_0_0_16_0 = A + globalReadOffsetA_0_0_16_0;
  DATA_TYPE const *globalReadA_0_0_17_0 = A + globalReadOffsetA_0_0_17_0;
  DATA_TYPE const *globalReadA_0_0_18_0 = A + globalReadOffsetA_0_0_18_0;
  DATA_TYPE const *globalReadA_0_0_19_0 = A + globalReadOffsetA_0_0_19_0;
  DATA_TYPE const *globalReadA_0_0_20_0 = A + globalReadOffsetA_0_0_20_0;
  DATA_TYPE const *globalReadA_0_0_21_0 = A + globalReadOffsetA_0_0_21_0;
  DATA_TYPE const *globalReadA_0_0_22_0 = A + globalReadOffsetA_0_0_22_0;
  DATA_TYPE const *globalReadA_0_0_23_0 = A + globalReadOffsetA_0_0_23_0;
  DATA_TYPE const *globalReadA_0_0_24_0 = A + globalReadOffsetA_0_0_24_0;
  DATA_TYPE const *globalReadA_0_0_25_0 = A + globalReadOffsetA_0_0_25_0;
  DATA_TYPE const *globalReadA_0_0_26_0 = A + globalReadOffsetA_0_0_26_0;
  DATA_TYPE const *globalReadA_0_0_27_0 = A + globalReadOffsetA_0_0_27_0;
  DATA_TYPE const *globalReadA_0_0_28_0 = A + globalReadOffsetA_0_0_28_0;
  DATA_TYPE const *globalReadA_0_0_29_0 = A + globalReadOffsetA_0_0_29_0;
  DATA_TYPE const *globalReadA_0_0_30_0 = A + globalReadOffsetA_0_0_30_0;
  DATA_TYPE const *globalReadA_0_0_31_0 = A + globalReadOffsetA_0_0_31_0;
  DATA_TYPE const *globalReadA_0_0_32_0 = A + globalReadOffsetA_0_0_32_0;
  DATA_TYPE const *globalReadA_0_0_33_0 = A + globalReadOffsetA_0_0_33_0;
  DATA_TYPE const *globalReadA_0_0_34_0 = A + globalReadOffsetA_0_0_34_0;
  DATA_TYPE const *globalReadA_0_0_35_0 = A + globalReadOffsetA_0_0_35_0;
  DATA_TYPE const *globalReadA_0_0_36_0 = A + globalReadOffsetA_0_0_36_0;
  DATA_TYPE const *globalReadA_0_0_37_0 = A + globalReadOffsetA_0_0_37_0;
  DATA_TYPE const *globalReadA_0_0_38_0 = A + globalReadOffsetA_0_0_38_0;
  DATA_TYPE const *globalReadA_0_0_39_0 = A + globalReadOffsetA_0_0_39_0;
  DATA_TYPE const *globalReadA_0_0_40_0 = A + globalReadOffsetA_0_0_40_0;
  DATA_TYPE const *globalReadA_0_0_41_0 = A + globalReadOffsetA_0_0_41_0;
  DATA_TYPE const *globalReadA_0_0_42_0 = A + globalReadOffsetA_0_0_42_0;
  DATA_TYPE const *globalReadA_0_0_43_0 = A + globalReadOffsetA_0_0_43_0;
  DATA_TYPE const *globalReadA_0_0_44_0 = A + globalReadOffsetA_0_0_44_0;
  DATA_TYPE const *globalReadA_0_0_45_0 = A + globalReadOffsetA_0_0_45_0;
  DATA_TYPE const *globalReadA_0_0_46_0 = A + globalReadOffsetA_0_0_46_0;
  DATA_TYPE const *globalReadA_0_0_47_0 = A + globalReadOffsetA_0_0_47_0;
  DATA_TYPE const *globalReadA_0_0_48_0 = A + globalReadOffsetA_0_0_48_0;
  DATA_TYPE const *globalReadA_0_0_49_0 = A + globalReadOffsetA_0_0_49_0;
  DATA_TYPE const *globalReadA_0_0_50_0 = A + globalReadOffsetA_0_0_50_0;
  DATA_TYPE const *globalReadA_0_0_51_0 = A + globalReadOffsetA_0_0_51_0;
  DATA_TYPE const *globalReadA_0_0_52_0 = A + globalReadOffsetA_0_0_52_0;
  DATA_TYPE const *globalReadA_0_0_53_0 = A + globalReadOffsetA_0_0_53_0;
  DATA_TYPE const *globalReadA_0_0_54_0 = A + globalReadOffsetA_0_0_54_0;
  DATA_TYPE const *globalReadA_0_0_55_0 = A + globalReadOffsetA_0_0_55_0;
  DATA_TYPE const *globalReadA_0_0_56_0 = A + globalReadOffsetA_0_0_56_0;
  DATA_TYPE const *globalReadA_0_0_57_0 = A + globalReadOffsetA_0_0_57_0;
  DATA_TYPE const *globalReadA_0_0_58_0 = A + globalReadOffsetA_0_0_58_0;
  DATA_TYPE const *globalReadA_0_0_59_0 = A + globalReadOffsetA_0_0_59_0;
  DATA_TYPE const *globalReadA_0_0_60_0 = A + globalReadOffsetA_0_0_60_0;
  DATA_TYPE const *globalReadA_0_0_61_0 = A + globalReadOffsetA_0_0_61_0;
  DATA_TYPE const *globalReadA_0_0_62_0 = A + globalReadOffsetA_0_0_62_0;
  DATA_TYPE const *globalReadA_0_0_63_0 = A + globalReadOffsetA_0_0_63_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 8*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 9*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 10*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 11*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 12*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 13*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 14*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 15*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_16_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 16*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_17_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 17*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_18_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 18*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_19_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 19*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_20_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 20*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_21_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 21*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_22_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 22*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_23_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 23*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_24_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 24*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_25_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 25*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_26_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 26*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_27_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 27*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_28_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 28*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_29_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 29*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_30_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 30*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_31_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 31*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_32_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 32*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_33_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 33*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_34_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 34*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_35_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 35*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_36_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 36*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_37_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 37*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_38_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 38*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_39_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 39*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_40_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 40*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_41_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 41*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_42_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 42*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_43_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 43*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_44_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 44*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_45_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 45*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_46_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 46*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_47_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 47*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_48_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 48*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_49_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 49*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_50_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 50*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_51_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 51*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_52_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 52*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_53_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 53*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_54_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 54*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_55_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 55*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_56_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 56*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_57_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 57*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_58_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 58*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_59_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 59*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_60_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 60*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_61_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 61*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_62_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 62*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_63_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 63*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_0_0_15_0;
  DATA_TYPE *localWriteA_0_0_16_0;
  DATA_TYPE *localWriteA_0_0_17_0;
  DATA_TYPE *localWriteA_0_0_18_0;
  DATA_TYPE *localWriteA_0_0_19_0;
  DATA_TYPE *localWriteA_0_0_20_0;
  DATA_TYPE *localWriteA_0_0_21_0;
  DATA_TYPE *localWriteA_0_0_22_0;
  DATA_TYPE *localWriteA_0_0_23_0;
  DATA_TYPE *localWriteA_0_0_24_0;
  DATA_TYPE *localWriteA_0_0_25_0;
  DATA_TYPE *localWriteA_0_0_26_0;
  DATA_TYPE *localWriteA_0_0_27_0;
  DATA_TYPE *localWriteA_0_0_28_0;
  DATA_TYPE *localWriteA_0_0_29_0;
  DATA_TYPE *localWriteA_0_0_30_0;
  DATA_TYPE *localWriteA_0_0_31_0;
  DATA_TYPE *localWriteA_0_0_32_0;
  DATA_TYPE *localWriteA_0_0_33_0;
  DATA_TYPE *localWriteA_0_0_34_0;
  DATA_TYPE *localWriteA_0_0_35_0;
  DATA_TYPE *localWriteA_0_0_36_0;
  DATA_TYPE *localWriteA_0_0_37_0;
  DATA_TYPE *localWriteA_0_0_38_0;
  DATA_TYPE *localWriteA_0_0_39_0;
  DATA_TYPE *localWriteA_0_0_40_0;
  DATA_TYPE *localWriteA_0_0_41_0;
  DATA_TYPE *localWriteA_0_0_42_0;
  DATA_TYPE *localWriteA_0_0_43_0;
  DATA_TYPE *localWriteA_0_0_44_0;
  DATA_TYPE *localWriteA_0_0_45_0;
  DATA_TYPE *localWriteA_0_0_46_0;
  DATA_TYPE *localWriteA_0_0_47_0;
  DATA_TYPE *localWriteA_0_0_48_0;
  DATA_TYPE *localWriteA_0_0_49_0;
  DATA_TYPE *localWriteA_0_0_50_0;
  DATA_TYPE *localWriteA_0_0_51_0;
  DATA_TYPE *localWriteA_0_0_52_0;
  DATA_TYPE *localWriteA_0_0_53_0;
  DATA_TYPE *localWriteA_0_0_54_0;
  DATA_TYPE *localWriteA_0_0_55_0;
  DATA_TYPE *localWriteA_0_0_56_0;
  DATA_TYPE *localWriteA_0_0_57_0;
  DATA_TYPE *localWriteA_0_0_58_0;
  DATA_TYPE *localWriteA_0_0_59_0;
  DATA_TYPE *localWriteA_0_0_60_0;
  DATA_TYPE *localWriteA_0_0_61_0;
  DATA_TYPE *localWriteA_0_0_62_0;
  DATA_TYPE *localWriteA_0_0_63_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);
  localWriteA_0_0_16_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_16_0);
  localWriteA_0_0_17_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_17_0);
  localWriteA_0_0_18_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_18_0);
  localWriteA_0_0_19_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_19_0);
  localWriteA_0_0_20_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_20_0);
  localWriteA_0_0_21_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_21_0);
  localWriteA_0_0_22_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_22_0);
  localWriteA_0_0_23_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_23_0);
  localWriteA_0_0_24_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_24_0);
  localWriteA_0_0_25_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_25_0);
  localWriteA_0_0_26_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_26_0);
  localWriteA_0_0_27_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_27_0);
  localWriteA_0_0_28_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_28_0);
  localWriteA_0_0_29_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_29_0);
  localWriteA_0_0_30_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_30_0);
  localWriteA_0_0_31_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_31_0);
  localWriteA_0_0_32_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_32_0);
  localWriteA_0_0_33_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_33_0);
  localWriteA_0_0_34_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_34_0);
  localWriteA_0_0_35_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_35_0);
  localWriteA_0_0_36_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_36_0);
  localWriteA_0_0_37_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_37_0);
  localWriteA_0_0_38_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_38_0);
  localWriteA_0_0_39_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_39_0);
  localWriteA_0_0_40_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_40_0);
  localWriteA_0_0_41_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_41_0);
  localWriteA_0_0_42_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_42_0);
  localWriteA_0_0_43_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_43_0);
  localWriteA_0_0_44_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_44_0);
  localWriteA_0_0_45_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_45_0);
  localWriteA_0_0_46_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_46_0);
  localWriteA_0_0_47_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_47_0);
  localWriteA_0_0_48_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_48_0);
  localWriteA_0_0_49_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_49_0);
  localWriteA_0_0_50_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_50_0);
  localWriteA_0_0_51_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_51_0);
  localWriteA_0_0_52_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_52_0);
  localWriteA_0_0_53_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_53_0);
  localWriteA_0_0_54_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_54_0);
  localWriteA_0_0_55_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_55_0);
  localWriteA_0_0_56_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_56_0);
  localWriteA_0_0_57_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_57_0);
  localWriteA_0_0_58_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_58_0);
  localWriteA_0_0_59_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_59_0);
  localWriteA_0_0_60_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_60_0);
  localWriteA_0_0_61_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_61_0);
  localWriteA_0_0_62_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_62_0);
  localWriteA_0_0_63_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_63_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);
    a_0_0_16_0 = *(globalReadA_0_0_16_0 + 0);
    a_0_0_17_0 = *(globalReadA_0_0_17_0 + 0);
    a_0_0_18_0 = *(globalReadA_0_0_18_0 + 0);
    a_0_0_19_0 = *(globalReadA_0_0_19_0 + 0);
    a_0_0_20_0 = *(globalReadA_0_0_20_0 + 0);
    a_0_0_21_0 = *(globalReadA_0_0_21_0 + 0);
    a_0_0_22_0 = *(globalReadA_0_0_22_0 + 0);
    a_0_0_23_0 = *(globalReadA_0_0_23_0 + 0);
    a_0_0_24_0 = *(globalReadA_0_0_24_0 + 0);
    a_0_0_25_0 = *(globalReadA_0_0_25_0 + 0);
    a_0_0_26_0 = *(globalReadA_0_0_26_0 + 0);
    a_0_0_27_0 = *(globalReadA_0_0_27_0 + 0);
    a_0_0_28_0 = *(globalReadA_0_0_28_0 + 0);
    a_0_0_29_0 = *(globalReadA_0_0_29_0 + 0);
    a_0_0_30_0 = *(globalReadA_0_0_30_0 + 0);
    a_0_0_31_0 = *(globalReadA_0_0_31_0 + 0);
    a_0_0_32_0 = *(globalReadA_0_0_32_0 + 0);
    a_0_0_33_0 = *(globalReadA_0_0_33_0 + 0);
    a_0_0_34_0 = *(globalReadA_0_0_34_0 + 0);
    a_0_0_35_0 = *(globalReadA_0_0_35_0 + 0);
    a_0_0_36_0 = *(globalReadA_0_0_36_0 + 0);
    a_0_0_37_0 = *(globalReadA_0_0_37_0 + 0);
    a_0_0_38_0 = *(globalReadA_0_0_38_0 + 0);
    a_0_0_39_0 = *(globalReadA_0_0_39_0 + 0);
    a_0_0_40_0 = *(globalReadA_0_0_40_0 + 0);
    a_0_0_41_0 = *(globalReadA_0_0_41_0 + 0);
    a_0_0_42_0 = *(globalReadA_0_0_42_0 + 0);
    a_0_0_43_0 = *(globalReadA_0_0_43_0 + 0);
    a_0_0_44_0 = *(globalReadA_0_0_44_0 + 0);
    a_0_0_45_0 = *(globalReadA_0_0_45_0 + 0);
    a_0_0_46_0 = *(globalReadA_0_0_46_0 + 0);
    a_0_0_47_0 = *(globalReadA_0_0_47_0 + 0);
    a_0_0_48_0 = *(globalReadA_0_0_48_0 + 0);
    a_0_0_49_0 = *(globalReadA_0_0_49_0 + 0);
    a_0_0_50_0 = *(globalReadA_0_0_50_0 + 0);
    a_0_0_51_0 = *(globalReadA_0_0_51_0 + 0);
    a_0_0_52_0 = *(globalReadA_0_0_52_0 + 0);
    a_0_0_53_0 = *(globalReadA_0_0_53_0 + 0);
    a_0_0_54_0 = *(globalReadA_0_0_54_0 + 0);
    a_0_0_55_0 = *(globalReadA_0_0_55_0 + 0);
    a_0_0_56_0 = *(globalReadA_0_0_56_0 + 0);
    a_0_0_57_0 = *(globalReadA_0_0_57_0 + 0);
    a_0_0_58_0 = *(globalReadA_0_0_58_0 + 0);
    a_0_0_59_0 = *(globalReadA_0_0_59_0 + 0);
    a_0_0_60_0 = *(globalReadA_0_0_60_0 + 0);
    a_0_0_61_0 = *(globalReadA_0_0_61_0 + 0);
    a_0_0_62_0 = *(globalReadA_0_0_62_0 + 0);
    a_0_0_63_0 = *(globalReadA_0_0_63_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    globalReadA_0_0_16_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_16_0) + 1*globalReadIncAL);
    globalReadA_0_0_17_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_17_0) + 1*globalReadIncAL);
    globalReadA_0_0_18_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_18_0) + 1*globalReadIncAL);
    globalReadA_0_0_19_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_19_0) + 1*globalReadIncAL);
    globalReadA_0_0_20_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_20_0) + 1*globalReadIncAL);
    globalReadA_0_0_21_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_21_0) + 1*globalReadIncAL);
    globalReadA_0_0_22_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_22_0) + 1*globalReadIncAL);
    globalReadA_0_0_23_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_23_0) + 1*globalReadIncAL);
    globalReadA_0_0_24_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_24_0) + 1*globalReadIncAL);
    globalReadA_0_0_25_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_25_0) + 1*globalReadIncAL);
    globalReadA_0_0_26_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_26_0) + 1*globalReadIncAL);
    globalReadA_0_0_27_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_27_0) + 1*globalReadIncAL);
    globalReadA_0_0_28_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_28_0) + 1*globalReadIncAL);
    globalReadA_0_0_29_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_29_0) + 1*globalReadIncAL);
    globalReadA_0_0_30_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_30_0) + 1*globalReadIncAL);
    globalReadA_0_0_31_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_31_0) + 1*globalReadIncAL);
    globalReadA_0_0_32_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_32_0) + 1*globalReadIncAL);
    globalReadA_0_0_33_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_33_0) + 1*globalReadIncAL);
    globalReadA_0_0_34_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_34_0) + 1*globalReadIncAL);
    globalReadA_0_0_35_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_35_0) + 1*globalReadIncAL);
    globalReadA_0_0_36_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_36_0) + 1*globalReadIncAL);
    globalReadA_0_0_37_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_37_0) + 1*globalReadIncAL);
    globalReadA_0_0_38_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_38_0) + 1*globalReadIncAL);
    globalReadA_0_0_39_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_39_0) + 1*globalReadIncAL);
    globalReadA_0_0_40_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_40_0) + 1*globalReadIncAL);
    globalReadA_0_0_41_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_41_0) + 1*globalReadIncAL);
    globalReadA_0_0_42_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_42_0) + 1*globalReadIncAL);
    globalReadA_0_0_43_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_43_0) + 1*globalReadIncAL);
    globalReadA_0_0_44_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_44_0) + 1*globalReadIncAL);
    globalReadA_0_0_45_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_45_0) + 1*globalReadIncAL);
    globalReadA_0_0_46_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_46_0) + 1*globalReadIncAL);
    globalReadA_0_0_47_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_47_0) + 1*globalReadIncAL);
    globalReadA_0_0_48_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_48_0) + 1*globalReadIncAL);
    globalReadA_0_0_49_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_49_0) + 1*globalReadIncAL);
    globalReadA_0_0_50_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_50_0) + 1*globalReadIncAL);
    globalReadA_0_0_51_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_51_0) + 1*globalReadIncAL);
    globalReadA_0_0_52_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_52_0) + 1*globalReadIncAL);
    globalReadA_0_0_53_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_53_0) + 1*globalReadIncAL);
    globalReadA_0_0_54_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_54_0) + 1*globalReadIncAL);
    globalReadA_0_0_55_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_55_0) + 1*globalReadIncAL);
    globalReadA_0_0_56_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_56_0) + 1*globalReadIncAL);
    globalReadA_0_0_57_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_57_0) + 1*globalReadIncAL);
    globalReadA_0_0_58_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_58_0) + 1*globalReadIncAL);
    globalReadA_0_0_59_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_59_0) + 1*globalReadIncAL);
    globalReadA_0_0_60_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_60_0) + 1*globalReadIncAL);
    globalReadA_0_0_61_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_61_0) + 1*globalReadIncAL);
    globalReadA_0_0_62_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_62_0) + 1*globalReadIncAL);
    globalReadA_0_0_63_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_63_0) + 1*globalReadIncAL);

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
    *(localWriteA_0_0_16_0 + 0) = a_0_0_16_0;
    *(localWriteA_0_0_17_0 + 0) = a_0_0_17_0;
    *(localWriteA_0_0_18_0 + 0) = a_0_0_18_0;
    *(localWriteA_0_0_19_0 + 0) = a_0_0_19_0;
    *(localWriteA_0_0_20_0 + 0) = a_0_0_20_0;
    *(localWriteA_0_0_21_0 + 0) = a_0_0_21_0;
    *(localWriteA_0_0_22_0 + 0) = a_0_0_22_0;
    *(localWriteA_0_0_23_0 + 0) = a_0_0_23_0;
    *(localWriteA_0_0_24_0 + 0) = a_0_0_24_0;
    *(localWriteA_0_0_25_0 + 0) = a_0_0_25_0;
    *(localWriteA_0_0_26_0 + 0) = a_0_0_26_0;
    *(localWriteA_0_0_27_0 + 0) = a_0_0_27_0;
    *(localWriteA_0_0_28_0 + 0) = a_0_0_28_0;
    *(localWriteA_0_0_29_0 + 0) = a_0_0_29_0;
    *(localWriteA_0_0_30_0 + 0) = a_0_0_30_0;
    *(localWriteA_0_0_31_0 + 0) = a_0_0_31_0;
    *(localWriteA_0_0_32_0 + 0) = a_0_0_32_0;
    *(localWriteA_0_0_33_0 + 0) = a_0_0_33_0;
    *(localWriteA_0_0_34_0 + 0) = a_0_0_34_0;
    *(localWriteA_0_0_35_0 + 0) = a_0_0_35_0;
    *(localWriteA_0_0_36_0 + 0) = a_0_0_36_0;
    *(localWriteA_0_0_37_0 + 0) = a_0_0_37_0;
    *(localWriteA_0_0_38_0 + 0) = a_0_0_38_0;
    *(localWriteA_0_0_39_0 + 0) = a_0_0_39_0;
    *(localWriteA_0_0_40_0 + 0) = a_0_0_40_0;
    *(localWriteA_0_0_41_0 + 0) = a_0_0_41_0;
    *(localWriteA_0_0_42_0 + 0) = a_0_0_42_0;
    *(localWriteA_0_0_43_0 + 0) = a_0_0_43_0;
    *(localWriteA_0_0_44_0 + 0) = a_0_0_44_0;
    *(localWriteA_0_0_45_0 + 0) = a_0_0_45_0;
    *(localWriteA_0_0_46_0 + 0) = a_0_0_46_0;
    *(localWriteA_0_0_47_0 + 0) = a_0_0_47_0;
    *(localWriteA_0_0_48_0 + 0) = a_0_0_48_0;
    *(localWriteA_0_0_49_0 + 0) = a_0_0_49_0;
    *(localWriteA_0_0_50_0 + 0) = a_0_0_50_0;
    *(localWriteA_0_0_51_0 + 0) = a_0_0_51_0;
    *(localWriteA_0_0_52_0 + 0) = a_0_0_52_0;
    *(localWriteA_0_0_53_0 + 0) = a_0_0_53_0;
    *(localWriteA_0_0_54_0 + 0) = a_0_0_54_0;
    *(localWriteA_0_0_55_0 + 0) = a_0_0_55_0;
    *(localWriteA_0_0_56_0 + 0) = a_0_0_56_0;
    *(localWriteA_0_0_57_0 + 0) = a_0_0_57_0;
    *(localWriteA_0_0_58_0 + 0) = a_0_0_58_0;
    *(localWriteA_0_0_59_0 + 0) = a_0_0_59_0;
    *(localWriteA_0_0_60_0 + 0) = a_0_0_60_0;
    *(localWriteA_0_0_61_0 + 0) = a_0_0_61_0;
    *(localWriteA_0_0_62_0 + 0) = a_0_0_62_0;
    *(localWriteA_0_0_63_0 + 0) = a_0_0_63_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 15 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 16 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 17 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 18 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 19 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 20 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 21 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 22 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 23 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 24 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 25 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 26 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 27 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 28 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 29 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 30 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 31 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 32 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 33 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 34 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 35 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 36 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 37 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 38 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 39 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 40 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 41 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 42 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 43 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 44 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 45 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 46 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 47 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 48 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 49 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 50 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 51 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 52 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 53 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 54 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 55 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 56 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 57 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 58 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 59 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 60 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 61 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 62 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 63 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_1x1




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_0_8_0 = ( globalReadOffsetAL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_0_0_9_0 = ( globalReadOffsetAL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_0_0_10_0 = ( globalReadOffsetAL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_0_0_11_0 = ( globalReadOffsetAL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_0_0_12_0 = ( globalReadOffsetAL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_0_0_13_0 = ( globalReadOffsetAL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_0_0_14_0 = ( globalReadOffsetAL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_0_0_15_0 = ( globalReadOffsetAL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);
  a_0_0_16_0 = ( globalReadOffsetAL_16_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_16_0 + 0);
  a_0_0_17_0 = ( globalReadOffsetAL_17_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_17_0 + 0);
  a_0_0_18_0 = ( globalReadOffsetAL_18_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_18_0 + 0);
  a_0_0_19_0 = ( globalReadOffsetAL_19_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_19_0 + 0);
  a_0_0_20_0 = ( globalReadOffsetAL_20_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_20_0 + 0);
  a_0_0_21_0 = ( globalReadOffsetAL_21_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_21_0 + 0);
  a_0_0_22_0 = ( globalReadOffsetAL_22_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_22_0 + 0);
  a_0_0_23_0 = ( globalReadOffsetAL_23_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_23_0 + 0);
  a_0_0_24_0 = ( globalReadOffsetAL_24_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_24_0 + 0);
  a_0_0_25_0 = ( globalReadOffsetAL_25_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_25_0 + 0);
  a_0_0_26_0 = ( globalReadOffsetAL_26_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_26_0 + 0);
  a_0_0_27_0 = ( globalReadOffsetAL_27_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_27_0 + 0);
  a_0_0_28_0 = ( globalReadOffsetAL_28_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_28_0 + 0);
  a_0_0_29_0 = ( globalReadOffsetAL_29_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_29_0 + 0);
  a_0_0_30_0 = ( globalReadOffsetAL_30_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_30_0 + 0);
  a_0_0_31_0 = ( globalReadOffsetAL_31_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_31_0 + 0);
  a_0_0_32_0 = ( globalReadOffsetAL_32_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_32_0 + 0);
  a_0_0_33_0 = ( globalReadOffsetAL_33_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_33_0 + 0);
  a_0_0_34_0 = ( globalReadOffsetAL_34_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_34_0 + 0);
  a_0_0_35_0 = ( globalReadOffsetAL_35_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_35_0 + 0);
  a_0_0_36_0 = ( globalReadOffsetAL_36_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_36_0 + 0);
  a_0_0_37_0 = ( globalReadOffsetAL_37_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_37_0 + 0);
  a_0_0_38_0 = ( globalReadOffsetAL_38_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_38_0 + 0);
  a_0_0_39_0 = ( globalReadOffsetAL_39_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_39_0 + 0);
  a_0_0_40_0 = ( globalReadOffsetAL_40_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_40_0 + 0);
  a_0_0_41_0 = ( globalReadOffsetAL_41_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_41_0 + 0);
  a_0_0_42_0 = ( globalReadOffsetAL_42_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_42_0 + 0);
  a_0_0_43_0 = ( globalReadOffsetAL_43_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_43_0 + 0);
  a_0_0_44_0 = ( globalReadOffsetAL_44_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_44_0 + 0);
  a_0_0_45_0 = ( globalReadOffsetAL_45_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_45_0 + 0);
  a_0_0_46_0 = ( globalReadOffsetAL_46_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_46_0 + 0);
  a_0_0_47_0 = ( globalReadOffsetAL_47_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_47_0 + 0);
  a_0_0_48_0 = ( globalReadOffsetAL_48_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_48_0 + 0);
  a_0_0_49_0 = ( globalReadOffsetAL_49_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_49_0 + 0);
  a_0_0_50_0 = ( globalReadOffsetAL_50_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_50_0 + 0);
  a_0_0_51_0 = ( globalReadOffsetAL_51_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_51_0 + 0);
  a_0_0_52_0 = ( globalReadOffsetAL_52_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_52_0 + 0);
  a_0_0_53_0 = ( globalReadOffsetAL_53_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_53_0 + 0);
  a_0_0_54_0 = ( globalReadOffsetAL_54_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_54_0 + 0);
  a_0_0_55_0 = ( globalReadOffsetAL_55_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_55_0 + 0);
  a_0_0_56_0 = ( globalReadOffsetAL_56_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_56_0 + 0);
  a_0_0_57_0 = ( globalReadOffsetAL_57_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_57_0 + 0);
  a_0_0_58_0 = ( globalReadOffsetAL_58_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_58_0 + 0);
  a_0_0_59_0 = ( globalReadOffsetAL_59_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_59_0 + 0);
  a_0_0_60_0 = ( globalReadOffsetAL_60_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_60_0 + 0);
  a_0_0_61_0 = ( globalReadOffsetAL_61_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_61_0 + 0);
  a_0_0_62_0 = ( globalReadOffsetAL_62_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_62_0 + 0);
  a_0_0_63_0 = ( globalReadOffsetAL_63_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_63_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);
  localWriteA_0_0_16_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_16_0);
  localWriteA_0_0_17_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_17_0);
  localWriteA_0_0_18_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_18_0);
  localWriteA_0_0_19_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_19_0);
  localWriteA_0_0_20_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_20_0);
  localWriteA_0_0_21_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_21_0);
  localWriteA_0_0_22_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_22_0);
  localWriteA_0_0_23_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_23_0);
  localWriteA_0_0_24_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_24_0);
  localWriteA_0_0_25_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_25_0);
  localWriteA_0_0_26_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_26_0);
  localWriteA_0_0_27_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_27_0);
  localWriteA_0_0_28_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_28_0);
  localWriteA_0_0_29_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_29_0);
  localWriteA_0_0_30_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_30_0);
  localWriteA_0_0_31_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_31_0);
  localWriteA_0_0_32_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_32_0);
  localWriteA_0_0_33_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_33_0);
  localWriteA_0_0_34_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_34_0);
  localWriteA_0_0_35_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_35_0);
  localWriteA_0_0_36_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_36_0);
  localWriteA_0_0_37_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_37_0);
  localWriteA_0_0_38_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_38_0);
  localWriteA_0_0_39_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_39_0);
  localWriteA_0_0_40_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_40_0);
  localWriteA_0_0_41_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_41_0);
  localWriteA_0_0_42_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_42_0);
  localWriteA_0_0_43_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_43_0);
  localWriteA_0_0_44_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_44_0);
  localWriteA_0_0_45_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_45_0);
  localWriteA_0_0_46_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_46_0);
  localWriteA_0_0_47_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_47_0);
  localWriteA_0_0_48_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_48_0);
  localWriteA_0_0_49_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_49_0);
  localWriteA_0_0_50_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_50_0);
  localWriteA_0_0_51_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_51_0);
  localWriteA_0_0_52_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_52_0);
  localWriteA_0_0_53_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_53_0);
  localWriteA_0_0_54_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_54_0);
  localWriteA_0_0_55_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_55_0);
  localWriteA_0_0_56_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_56_0);
  localWriteA_0_0_57_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_57_0);
  localWriteA_0_0_58_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_58_0);
  localWriteA_0_0_59_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_59_0);
  localWriteA_0_0_60_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_60_0);
  localWriteA_0_0_61_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_61_0);
  localWriteA_0_0_62_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_62_0);
  localWriteA_0_0_63_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_63_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
  *(localWriteA_0_0_16_0 + 0) = a_0_0_16_0;
  *(localWriteA_0_0_17_0 + 0) = a_0_0_17_0;
  *(localWriteA_0_0_18_0 + 0) = a_0_0_18_0;
  *(localWriteA_0_0_19_0 + 0) = a_0_0_19_0;
  *(localWriteA_0_0_20_0 + 0) = a_0_0_20_0;
  *(localWriteA_0_0_21_0 + 0) = a_0_0_21_0;
  *(localWriteA_0_0_22_0 + 0) = a_0_0_22_0;
  *(localWriteA_0_0_23_0 + 0) = a_0_0_23_0;
  *(localWriteA_0_0_24_0 + 0) = a_0_0_24_0;
  *(localWriteA_0_0_25_0 + 0) = a_0_0_25_0;
  *(localWriteA_0_0_26_0 + 0) = a_0_0_26_0;
  *(localWriteA_0_0_27_0 + 0) = a_0_0_27_0;
  *(localWriteA_0_0_28_0 + 0) = a_0_0_28_0;
  *(localWriteA_0_0_29_0 + 0) = a_0_0_29_0;
  *(localWriteA_0_0_30_0 + 0) = a_0_0_30_0;
  *(localWriteA_0_0_31_0 + 0) = a_0_0_31_0;
  *(localWriteA_0_0_32_0 + 0) = a_0_0_32_0;
  *(localWriteA_0_0_33_0 + 0) = a_0_0_33_0;
  *(localWriteA_0_0_34_0 + 0) = a_0_0_34_0;
  *(localWriteA_0_0_35_0 + 0) = a_0_0_35_0;
  *(localWriteA_0_0_36_0 + 0) = a_0_0_36_0;
  *(localWriteA_0_0_37_0 + 0) = a_0_0_37_0;
  *(localWriteA_0_0_38_0 + 0) = a_0_0_38_0;
  *(localWriteA_0_0_39_0 + 0) = a_0_0_39_0;
  *(localWriteA_0_0_40_0 + 0) = a_0_0_40_0;
  *(localWriteA_0_0_41_0 + 0) = a_0_0_41_0;
  *(localWriteA_0_0_42_0 + 0) = a_0_0_42_0;
  *(localWriteA_0_0_43_0 + 0) = a_0_0_43_0;
  *(localWriteA_0_0_44_0 + 0) = a_0_0_44_0;
  *(localWriteA_0_0_45_0 + 0) = a_0_0_45_0;
  *(localWriteA_0_0_46_0 + 0) = a_0_0_46_0;
  *(localWriteA_0_0_47_0 + 0) = a_0_0_47_0;
  *(localWriteA_0_0_48_0 + 0) = a_0_0_48_0;
  *(localWriteA_0_0_49_0 + 0) = a_0_0_49_0;
  *(localWriteA_0_0_50_0 + 0) = a_0_0_50_0;
  *(localWriteA_0_0_51_0 + 0) = a_0_0_51_0;
  *(localWriteA_0_0_52_0 + 0) = a_0_0_52_0;
  *(localWriteA_0_0_53_0 + 0) = a_0_0_53_0;
  *(localWriteA_0_0_54_0 + 0) = a_0_0_54_0;
  *(localWriteA_0_0_55_0 + 0) = a_0_0_55_0;
  *(localWriteA_0_0_56_0 + 0) = a_0_0_56_0;
  *(localWriteA_0_0_57_0 + 0) = a_0_0_57_0;
  *(localWriteA_0_0_58_0 + 0) = a_0_0_58_0;
  *(localWriteA_0_0_59_0 + 0) = a_0_0_59_0;
  *(localWriteA_0_0_60_0 + 0) = a_0_0_60_0;
  *(localWriteA_0_0_61_0 + 0) = a_0_0_61_0;
  *(localWriteA_0_0_62_0 + 0) = a_0_0_62_0;
  *(localWriteA_0_0_63_0 + 0) = a_0_0_63_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_1x1

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_1x1
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 64
#define SG1J 1
#define TT0I 1
#define TT1J 1
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 64
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 15

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 64
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 4096
#define LDS_NUM_ELEMENTS 4160

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE tensile_bfloat16
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 1x1 micro-tile */
#define MAC_1x1\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_BBH_MT64x1x64_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU15_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU0_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT1_1_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG64_1_1_WGM1(
  float *D,
  float const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0.0f
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE a_0_0_16_0;
  DATA_TYPE a_0_0_17_0;
  DATA_TYPE a_0_0_18_0;
  DATA_TYPE a_0_0_19_0;
  DATA_TYPE a_0_0_20_0;
  DATA_TYPE a_0_0_21_0;
  DATA_TYPE a_0_0_22_0;
  DATA_TYPE a_0_0_23_0;
  DATA_TYPE a_0_0_24_0;
  DATA_TYPE a_0_0_25_0;
  DATA_TYPE a_0_0_26_0;
  DATA_TYPE a_0_0_27_0;
  DATA_TYPE a_0_0_28_0;
  DATA_TYPE a_0_0_29_0;
  DATA_TYPE a_0_0_30_0;
  DATA_TYPE a_0_0_31_0;
  DATA_TYPE a_0_0_32_0;
  DATA_TYPE a_0_0_33_0;
  DATA_TYPE a_0_0_34_0;
  DATA_TYPE a_0_0_35_0;
  DATA_TYPE a_0_0_36_0;
  DATA_TYPE a_0_0_37_0;
  DATA_TYPE a_0_0_38_0;
  DATA_TYPE a_0_0_39_0;
  DATA_TYPE a_0_0_40_0;
  DATA_TYPE a_0_0_41_0;
  DATA_TYPE a_0_0_42_0;
  DATA_TYPE a_0_0_43_0;
  DATA_TYPE a_0_0_44_0;
  DATA_TYPE a_0_0_45_0;
  DATA_TYPE a_0_0_46_0;
  DATA_TYPE a_0_0_47_0;
  DATA_TYPE a_0_0_48_0;
  DATA_TYPE a_0_0_49_0;
  DATA_TYPE a_0_0_50_0;
  DATA_TYPE a_0_0_51_0;
  DATA_TYPE a_0_0_52_0;
  DATA_TYPE a_0_0_53_0;
  DATA_TYPE a_0_0_54_0;
  DATA_TYPE a_0_0_55_0;
  DATA_TYPE a_0_0_56_0;
  DATA_TYPE a_0_0_57_0;
  DATA_TYPE a_0_0_58_0;
  DATA_TYPE a_0_0_59_0;
  DATA_TYPE a_0_0_60_0;
  DATA_TYPE a_0_0_61_0;
  DATA_TYPE a_0_0_62_0;
  DATA_TYPE a_0_0_63_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;
  unsigned int globalReadOffsetAL_8_0 = globalReadOffsetAL + 0 + 8*LSPA;
  unsigned int globalReadOffsetAL_9_0 = globalReadOffsetAL + 0 + 9*LSPA;
  unsigned int globalReadOffsetAL_10_0 = globalReadOffsetAL + 0 + 10*LSPA;
  unsigned int globalReadOffsetAL_11_0 = globalReadOffsetAL + 0 + 11*LSPA;
  unsigned int globalReadOffsetAL_12_0 = globalReadOffsetAL + 0 + 12*LSPA;
  unsigned int globalReadOffsetAL_13_0 = globalReadOffsetAL + 0 + 13*LSPA;
  unsigned int globalReadOffsetAL_14_0 = globalReadOffsetAL + 0 + 14*LSPA;
  unsigned int globalReadOffsetAL_15_0 = globalReadOffsetAL + 0 + 15*LSPA;
  unsigned int globalReadOffsetAL_16_0 = globalReadOffsetAL + 0 + 16*LSPA;
  unsigned int globalReadOffsetAL_17_0 = globalReadOffsetAL + 0 + 17*LSPA;
  unsigned int globalReadOffsetAL_18_0 = globalReadOffsetAL + 0 + 18*LSPA;
  unsigned int globalReadOffsetAL_19_0 = globalReadOffsetAL + 0 + 19*LSPA;
  unsigned int globalReadOffsetAL_20_0 = globalReadOffsetAL + 0 + 20*LSPA;
  unsigned int globalReadOffsetAL_21_0 = globalReadOffsetAL + 0 + 21*LSPA;
  unsigned int globalReadOffsetAL_22_0 = globalReadOffsetAL + 0 + 22*LSPA;
  unsigned int globalReadOffsetAL_23_0 = globalReadOffsetAL + 0 + 23*LSPA;
  unsigned int globalReadOffsetAL_24_0 = globalReadOffsetAL + 0 + 24*LSPA;
  unsigned int globalReadOffsetAL_25_0 = globalReadOffsetAL + 0 + 25*LSPA;
  unsigned int globalReadOffsetAL_26_0 = globalReadOffsetAL + 0 + 26*LSPA;
  unsigned int globalReadOffsetAL_27_0 = globalReadOffsetAL + 0 + 27*LSPA;
  unsigned int globalReadOffsetAL_28_0 = globalReadOffsetAL + 0 + 28*LSPA;
  unsigned int globalReadOffsetAL_29_0 = globalReadOffsetAL + 0 + 29*LSPA;
  unsigned int globalReadOffsetAL_30_0 = globalReadOffsetAL + 0 + 30*LSPA;
  unsigned int globalReadOffsetAL_31_0 = globalReadOffsetAL + 0 + 31*LSPA;
  unsigned int globalReadOffsetAL_32_0 = globalReadOffsetAL + 0 + 32*LSPA;
  unsigned int globalReadOffsetAL_33_0 = globalReadOffsetAL + 0 + 33*LSPA;
  unsigned int globalReadOffsetAL_34_0 = globalReadOffsetAL + 0 + 34*LSPA;
  unsigned int globalReadOffsetAL_35_0 = globalReadOffsetAL + 0 + 35*LSPA;
  unsigned int globalReadOffsetAL_36_0 = globalReadOffsetAL + 0 + 36*LSPA;
  unsigned int globalReadOffsetAL_37_0 = globalReadOffsetAL + 0 + 37*LSPA;
  unsigned int globalReadOffsetAL_38_0 = globalReadOffsetAL + 0 + 38*LSPA;
  unsigned int globalReadOffsetAL_39_0 = globalReadOffsetAL + 0 + 39*LSPA;
  unsigned int globalReadOffsetAL_40_0 = globalReadOffsetAL + 0 + 40*LSPA;
  unsigned int globalReadOffsetAL_41_0 = globalReadOffsetAL + 0 + 41*LSPA;
  unsigned int globalReadOffsetAL_42_0 = globalReadOffsetAL + 0 + 42*LSPA;
  unsigned int globalReadOffsetAL_43_0 = globalReadOffsetAL + 0 + 43*LSPA;
  unsigned int globalReadOffsetAL_44_0 = globalReadOffsetAL + 0 + 44*LSPA;
  unsigned int globalReadOffsetAL_45_0 = globalReadOffsetAL + 0 + 45*LSPA;
  unsigned int globalReadOffsetAL_46_0 = globalReadOffsetAL + 0 + 46*LSPA;
  unsigned int globalReadOffsetAL_47_0 = globalReadOffsetAL + 0 + 47*LSPA;
  unsigned int globalReadOffsetAL_48_0 = globalReadOffsetAL + 0 + 48*LSPA;
  unsigned int globalReadOffsetAL_49_0 = globalReadOffsetAL + 0 + 49*LSPA;
  unsigned int globalReadOffsetAL_50_0 = globalReadOffsetAL + 0 + 50*LSPA;
  unsigned int globalReadOffsetAL_51_0 = globalReadOffsetAL + 0 + 51*LSPA;
  unsigned int globalReadOffsetAL_52_0 = globalReadOffsetAL + 0 + 52*LSPA;
  unsigned int globalReadOffsetAL_53_0 = globalReadOffsetAL + 0 + 53*LSPA;
  unsigned int globalReadOffsetAL_54_0 = globalReadOffsetAL + 0 + 54*LSPA;
  unsigned int globalReadOffsetAL_55_0 = globalReadOffsetAL + 0 + 55*LSPA;
  unsigned int globalReadOffsetAL_56_0 = globalReadOffsetAL + 0 + 56*LSPA;
  unsigned int globalReadOffsetAL_57_0 = globalReadOffsetAL + 0 + 57*LSPA;
  unsigned int globalReadOffsetAL_58_0 = globalReadOffsetAL + 0 + 58*LSPA;
  unsigned int globalReadOffsetAL_59_0 = globalReadOffsetAL + 0 + 59*LSPA;
  unsigned int globalReadOffsetAL_60_0 = globalReadOffsetAL + 0 + 60*LSPA;
  unsigned int globalReadOffsetAL_61_0 = globalReadOffsetAL + 0 + 61*LSPA;
  unsigned int globalReadOffsetAL_62_0 = globalReadOffsetAL + 0 + 62*LSPA;
  unsigned int globalReadOffsetAL_63_0 = globalReadOffsetAL + 0 + 63*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_15_0), wgK );
  int64_t globalReadOffsetA_0_0_16_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_16_0), wgK );
  int64_t globalReadOffsetA_0_0_17_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_17_0), wgK );
  int64_t globalReadOffsetA_0_0_18_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_18_0), wgK );
  int64_t globalReadOffsetA_0_0_19_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_19_0), wgK );
  int64_t globalReadOffsetA_0_0_20_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_20_0), wgK );
  int64_t globalReadOffsetA_0_0_21_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_21_0), wgK );
  int64_t globalReadOffsetA_0_0_22_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_22_0), wgK );
  int64_t globalReadOffsetA_0_0_23_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_23_0), wgK );
  int64_t globalReadOffsetA_0_0_24_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_24_0), wgK );
  int64_t globalReadOffsetA_0_0_25_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_25_0), wgK );
  int64_t globalReadOffsetA_0_0_26_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_26_0), wgK );
  int64_t globalReadOffsetA_0_0_27_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_27_0), wgK );
  int64_t globalReadOffsetA_0_0_28_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_28_0), wgK );
  int64_t globalReadOffsetA_0_0_29_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_29_0), wgK );
  int64_t globalReadOffsetA_0_0_30_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_30_0), wgK );
  int64_t globalReadOffsetA_0_0_31_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_31_0), wgK );
  int64_t globalReadOffsetA_0_0_32_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_32_0), wgK );
  int64_t globalReadOffsetA_0_0_33_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_33_0), wgK );
  int64_t globalReadOffsetA_0_0_34_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_34_0), wgK );
  int64_t globalReadOffsetA_0_0_35_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_35_0), wgK );
  int64_t globalReadOffsetA_0_0_36_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_36_0), wgK );
  int64_t globalReadOffsetA_0_0_37_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_37_0), wgK );
  int64_t globalReadOffsetA_0_0_38_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_38_0), wgK );
  int64_t globalReadOffsetA_0_0_39_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_39_0), wgK );
  int64_t globalReadOffsetA_0_0_40_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_40_0), wgK );
  int64_t globalReadOffsetA_0_0_41_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_41_0), wgK );
  int64_t globalReadOffsetA_0_0_42_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_42_0), wgK );
  int64_t globalReadOffsetA_0_0_43_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_43_0), wgK );
  int64_t globalReadOffsetA_0_0_44_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_44_0), wgK );
  int64_t globalReadOffsetA_0_0_45_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_45_0), wgK );
  int64_t globalReadOffsetA_0_0_46_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_46_0), wgK );
  int64_t globalReadOffsetA_0_0_47_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_47_0), wgK );
  int64_t globalReadOffsetA_0_0_48_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_48_0), wgK );
  int64_t globalReadOffsetA_0_0_49_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_49_0), wgK );
  int64_t globalReadOffsetA_0_0_50_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_50_0), wgK );
  int64_t globalReadOffsetA_0_0_51_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_51_0), wgK );
  int64_t globalReadOffsetA_0_0_52_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_52_0), wgK );
  int64_t globalReadOffsetA_0_0_53_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_53_0), wgK );
  int64_t globalReadOffsetA_0_0_54_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_54_0), wgK );
  int64_t globalReadOffsetA_0_0_55_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_55_0), wgK );
  int64_t globalReadOffsetA_0_0_56_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_56_0), wgK );
  int64_t globalReadOffsetA_0_0_57_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_57_0), wgK );
  int64_t globalReadOffsetA_0_0_58_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_58_0), wgK );
  int64_t globalReadOffsetA_0_0_59_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_59_0), wgK );
  int64_t globalReadOffsetA_0_0_60_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_60_0), wgK );
  int64_t globalReadOffsetA_0_0_61_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_61_0), wgK );
  int64_t globalReadOffsetA_0_0_62_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_62_0), wgK );
  int64_t globalReadOffsetA_0_0_63_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_63_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;
  DATA_TYPE const *globalReadA_0_0_16_0 = A + globalReadOffsetA_0_0_16_0;
  DATA_TYPE const *globalReadA_0_0_17_0 = A + globalReadOffsetA_0_0_17_0;
  DATA_TYPE const *globalReadA_0_0_18_0 = A + globalReadOffsetA_0_0_18_0;
  DATA_TYPE const *globalReadA_0_0_19_0 = A + globalReadOffsetA_0_0_19_0;
  DATA_TYPE const *globalReadA_0_0_20_0 = A + globalReadOffsetA_0_0_20_0;
  DATA_TYPE const *globalReadA_0_0_21_0 = A + globalReadOffsetA_0_0_21_0;
  DATA_TYPE const *globalReadA_0_0_22_0 = A + globalReadOffsetA_0_0_22_0;
  DATA_TYPE const *globalReadA_0_0_23_0 = A + globalReadOffsetA_0_0_23_0;
  DATA_TYPE const *globalReadA_0_0_24_0 = A + globalReadOffsetA_0_0_24_0;
  DATA_TYPE const *globalReadA_0_0_25_0 = A + globalReadOffsetA_0_0_25_0;
  DATA_TYPE const *globalReadA_0_0_26_0 = A + globalReadOffsetA_0_0_26_0;
  DATA_TYPE const *globalReadA_0_0_27_0 = A + globalReadOffsetA_0_0_27_0;
  DATA_TYPE const *globalReadA_0_0_28_0 = A + globalReadOffsetA_0_0_28_0;
  DATA_TYPE const *globalReadA_0_0_29_0 = A + globalReadOffsetA_0_0_29_0;
  DATA_TYPE const *globalReadA_0_0_30_0 = A + globalReadOffsetA_0_0_30_0;
  DATA_TYPE const *globalReadA_0_0_31_0 = A + globalReadOffsetA_0_0_31_0;
  DATA_TYPE const *globalReadA_0_0_32_0 = A + globalReadOffsetA_0_0_32_0;
  DATA_TYPE const *globalReadA_0_0_33_0 = A + globalReadOffsetA_0_0_33_0;
  DATA_TYPE const *globalReadA_0_0_34_0 = A + globalReadOffsetA_0_0_34_0;
  DATA_TYPE const *globalReadA_0_0_35_0 = A + globalReadOffsetA_0_0_35_0;
  DATA_TYPE const *globalReadA_0_0_36_0 = A + globalReadOffsetA_0_0_36_0;
  DATA_TYPE const *globalReadA_0_0_37_0 = A + globalReadOffsetA_0_0_37_0;
  DATA_TYPE const *globalReadA_0_0_38_0 = A + globalReadOffsetA_0_0_38_0;
  DATA_TYPE const *globalReadA_0_0_39_0 = A + globalReadOffsetA_0_0_39_0;
  DATA_TYPE const *globalReadA_0_0_40_0 = A + globalReadOffsetA_0_0_40_0;
  DATA_TYPE const *globalReadA_0_0_41_0 = A + globalReadOffsetA_0_0_41_0;
  DATA_TYPE const *globalReadA_0_0_42_0 = A + globalReadOffsetA_0_0_42_0;
  DATA_TYPE const *globalReadA_0_0_43_0 = A + globalReadOffsetA_0_0_43_0;
  DATA_TYPE const *globalReadA_0_0_44_0 = A + globalReadOffsetA_0_0_44_0;
  DATA_TYPE const *globalReadA_0_0_45_0 = A + globalReadOffsetA_0_0_45_0;
  DATA_TYPE const *globalReadA_0_0_46_0 = A + globalReadOffsetA_0_0_46_0;
  DATA_TYPE const *globalReadA_0_0_47_0 = A + globalReadOffsetA_0_0_47_0;
  DATA_TYPE const *globalReadA_0_0_48_0 = A + globalReadOffsetA_0_0_48_0;
  DATA_TYPE const *globalReadA_0_0_49_0 = A + globalReadOffsetA_0_0_49_0;
  DATA_TYPE const *globalReadA_0_0_50_0 = A + globalReadOffsetA_0_0_50_0;
  DATA_TYPE const *globalReadA_0_0_51_0 = A + globalReadOffsetA_0_0_51_0;
  DATA_TYPE const *globalReadA_0_0_52_0 = A + globalReadOffsetA_0_0_52_0;
  DATA_TYPE const *globalReadA_0_0_53_0 = A + globalReadOffsetA_0_0_53_0;
  DATA_TYPE const *globalReadA_0_0_54_0 = A + globalReadOffsetA_0_0_54_0;
  DATA_TYPE const *globalReadA_0_0_55_0 = A + globalReadOffsetA_0_0_55_0;
  DATA_TYPE const *globalReadA_0_0_56_0 = A + globalReadOffsetA_0_0_56_0;
  DATA_TYPE const *globalReadA_0_0_57_0 = A + globalReadOffsetA_0_0_57_0;
  DATA_TYPE const *globalReadA_0_0_58_0 = A + globalReadOffsetA_0_0_58_0;
  DATA_TYPE const *globalReadA_0_0_59_0 = A + globalReadOffsetA_0_0_59_0;
  DATA_TYPE const *globalReadA_0_0_60_0 = A + globalReadOffsetA_0_0_60_0;
  DATA_TYPE const *globalReadA_0_0_61_0 = A + globalReadOffsetA_0_0_61_0;
  DATA_TYPE const *globalReadA_0_0_62_0 = A + globalReadOffsetA_0_0_62_0;
  DATA_TYPE const *globalReadA_0_0_63_0 = A + globalReadOffsetA_0_0_63_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 8*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 9*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 10*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 11*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 12*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 13*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 14*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 15*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_16_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 16*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_17_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 17*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_18_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 18*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_19_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 19*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_20_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 20*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_21_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 21*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_22_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 22*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_23_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 23*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_24_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 24*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_25_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 25*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_26_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 26*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_27_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 27*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_28_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 28*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_29_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 29*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_30_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 30*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_31_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 31*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_32_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 32*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_33_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 33*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_34_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 34*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_35_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 35*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_36_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 36*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_37_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 37*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_38_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 38*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_39_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 39*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_40_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 40*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_41_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 41*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_42_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 42*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_43_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 43*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_44_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 44*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_45_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 45*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_46_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 46*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_47_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 47*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_48_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 48*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_49_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 49*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_50_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 50*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_51_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 51*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_52_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 52*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_53_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 53*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_54_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 54*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_55_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 55*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_56_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 56*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_57_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 57*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_58_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 58*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_59_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 59*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_60_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 60*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_61_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 61*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_62_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 62*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_63_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 63*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_0_0_15_0;
  DATA_TYPE *localWriteA_0_0_16_0;
  DATA_TYPE *localWriteA_0_0_17_0;
  DATA_TYPE *localWriteA_0_0_18_0;
  DATA_TYPE *localWriteA_0_0_19_0;
  DATA_TYPE *localWriteA_0_0_20_0;
  DATA_TYPE *localWriteA_0_0_21_0;
  DATA_TYPE *localWriteA_0_0_22_0;
  DATA_TYPE *localWriteA_0_0_23_0;
  DATA_TYPE *localWriteA_0_0_24_0;
  DATA_TYPE *localWriteA_0_0_25_0;
  DATA_TYPE *localWriteA_0_0_26_0;
  DATA_TYPE *localWriteA_0_0_27_0;
  DATA_TYPE *localWriteA_0_0_28_0;
  DATA_TYPE *localWriteA_0_0_29_0;
  DATA_TYPE *localWriteA_0_0_30_0;
  DATA_TYPE *localWriteA_0_0_31_0;
  DATA_TYPE *localWriteA_0_0_32_0;
  DATA_TYPE *localWriteA_0_0_33_0;
  DATA_TYPE *localWriteA_0_0_34_0;
  DATA_TYPE *localWriteA_0_0_35_0;
  DATA_TYPE *localWriteA_0_0_36_0;
  DATA_TYPE *localWriteA_0_0_37_0;
  DATA_TYPE *localWriteA_0_0_38_0;
  DATA_TYPE *localWriteA_0_0_39_0;
  DATA_TYPE *localWriteA_0_0_40_0;
  DATA_TYPE *localWriteA_0_0_41_0;
  DATA_TYPE *localWriteA_0_0_42_0;
  DATA_TYPE *localWriteA_0_0_43_0;
  DATA_TYPE *localWriteA_0_0_44_0;
  DATA_TYPE *localWriteA_0_0_45_0;
  DATA_TYPE *localWriteA_0_0_46_0;
  DATA_TYPE *localWriteA_0_0_47_0;
  DATA_TYPE *localWriteA_0_0_48_0;
  DATA_TYPE *localWriteA_0_0_49_0;
  DATA_TYPE *localWriteA_0_0_50_0;
  DATA_TYPE *localWriteA_0_0_51_0;
  DATA_TYPE *localWriteA_0_0_52_0;
  DATA_TYPE *localWriteA_0_0_53_0;
  DATA_TYPE *localWriteA_0_0_54_0;
  DATA_TYPE *localWriteA_0_0_55_0;
  DATA_TYPE *localWriteA_0_0_56_0;
  DATA_TYPE *localWriteA_0_0_57_0;
  DATA_TYPE *localWriteA_0_0_58_0;
  DATA_TYPE *localWriteA_0_0_59_0;
  DATA_TYPE *localWriteA_0_0_60_0;
  DATA_TYPE *localWriteA_0_0_61_0;
  DATA_TYPE *localWriteA_0_0_62_0;
  DATA_TYPE *localWriteA_0_0_63_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);
  localWriteA_0_0_16_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_16_0);
  localWriteA_0_0_17_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_17_0);
  localWriteA_0_0_18_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_18_0);
  localWriteA_0_0_19_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_19_0);
  localWriteA_0_0_20_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_20_0);
  localWriteA_0_0_21_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_21_0);
  localWriteA_0_0_22_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_22_0);
  localWriteA_0_0_23_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_23_0);
  localWriteA_0_0_24_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_24_0);
  localWriteA_0_0_25_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_25_0);
  localWriteA_0_0_26_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_26_0);
  localWriteA_0_0_27_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_27_0);
  localWriteA_0_0_28_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_28_0);
  localWriteA_0_0_29_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_29_0);
  localWriteA_0_0_30_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_30_0);
  localWriteA_0_0_31_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_31_0);
  localWriteA_0_0_32_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_32_0);
  localWriteA_0_0_33_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_33_0);
  localWriteA_0_0_34_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_34_0);
  localWriteA_0_0_35_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_35_0);
  localWriteA_0_0_36_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_36_0);
  localWriteA_0_0_37_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_37_0);
  localWriteA_0_0_38_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_38_0);
  localWriteA_0_0_39_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_39_0);
  localWriteA_0_0_40_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_40_0);
  localWriteA_0_0_41_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_41_0);
  localWriteA_0_0_42_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_42_0);
  localWriteA_0_0_43_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_43_0);
  localWriteA_0_0_44_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_44_0);
  localWriteA_0_0_45_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_45_0);
  localWriteA_0_0_46_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_46_0);
  localWriteA_0_0_47_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_47_0);
  localWriteA_0_0_48_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_48_0);
  localWriteA_0_0_49_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_49_0);
  localWriteA_0_0_50_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_50_0);
  localWriteA_0_0_51_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_51_0);
  localWriteA_0_0_52_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_52_0);
  localWriteA_0_0_53_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_53_0);
  localWriteA_0_0_54_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_54_0);
  localWriteA_0_0_55_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_55_0);
  localWriteA_0_0_56_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_56_0);
  localWriteA_0_0_57_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_57_0);
  localWriteA_0_0_58_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_58_0);
  localWriteA_0_0_59_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_59_0);
  localWriteA_0_0_60_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_60_0);
  localWriteA_0_0_61_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_61_0);
  localWriteA_0_0_62_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_62_0);
  localWriteA_0_0_63_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_63_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);
    a_0_0_16_0 = *(globalReadA_0_0_16_0 + 0);
    a_0_0_17_0 = *(globalReadA_0_0_17_0 + 0);
    a_0_0_18_0 = *(globalReadA_0_0_18_0 + 0);
    a_0_0_19_0 = *(globalReadA_0_0_19_0 + 0);
    a_0_0_20_0 = *(globalReadA_0_0_20_0 + 0);
    a_0_0_21_0 = *(globalReadA_0_0_21_0 + 0);
    a_0_0_22_0 = *(globalReadA_0_0_22_0 + 0);
    a_0_0_23_0 = *(globalReadA_0_0_23_0 + 0);
    a_0_0_24_0 = *(globalReadA_0_0_24_0 + 0);
    a_0_0_25_0 = *(globalReadA_0_0_25_0 + 0);
    a_0_0_26_0 = *(globalReadA_0_0_26_0 + 0);
    a_0_0_27_0 = *(globalReadA_0_0_27_0 + 0);
    a_0_0_28_0 = *(globalReadA_0_0_28_0 + 0);
    a_0_0_29_0 = *(globalReadA_0_0_29_0 + 0);
    a_0_0_30_0 = *(globalReadA_0_0_30_0 + 0);
    a_0_0_31_0 = *(globalReadA_0_0_31_0 + 0);
    a_0_0_32_0 = *(globalReadA_0_0_32_0 + 0);
    a_0_0_33_0 = *(globalReadA_0_0_33_0 + 0);
    a_0_0_34_0 = *(globalReadA_0_0_34_0 + 0);
    a_0_0_35_0 = *(globalReadA_0_0_35_0 + 0);
    a_0_0_36_0 = *(globalReadA_0_0_36_0 + 0);
    a_0_0_37_0 = *(globalReadA_0_0_37_0 + 0);
    a_0_0_38_0 = *(globalReadA_0_0_38_0 + 0);
    a_0_0_39_0 = *(globalReadA_0_0_39_0 + 0);
    a_0_0_40_0 = *(globalReadA_0_0_40_0 + 0);
    a_0_0_41_0 = *(globalReadA_0_0_41_0 + 0);
    a_0_0_42_0 = *(globalReadA_0_0_42_0 + 0);
    a_0_0_43_0 = *(globalReadA_0_0_43_0 + 0);
    a_0_0_44_0 = *(globalReadA_0_0_44_0 + 0);
    a_0_0_45_0 = *(globalReadA_0_0_45_0 + 0);
    a_0_0_46_0 = *(globalReadA_0_0_46_0 + 0);
    a_0_0_47_0 = *(globalReadA_0_0_47_0 + 0);
    a_0_0_48_0 = *(globalReadA_0_0_48_0 + 0);
    a_0_0_49_0 = *(globalReadA_0_0_49_0 + 0);
    a_0_0_50_0 = *(globalReadA_0_0_50_0 + 0);
    a_0_0_51_0 = *(globalReadA_0_0_51_0 + 0);
    a_0_0_52_0 = *(globalReadA_0_0_52_0 + 0);
    a_0_0_53_0 = *(globalReadA_0_0_53_0 + 0);
    a_0_0_54_0 = *(globalReadA_0_0_54_0 + 0);
    a_0_0_55_0 = *(globalReadA_0_0_55_0 + 0);
    a_0_0_56_0 = *(globalReadA_0_0_56_0 + 0);
    a_0_0_57_0 = *(globalReadA_0_0_57_0 + 0);
    a_0_0_58_0 = *(globalReadA_0_0_58_0 + 0);
    a_0_0_59_0 = *(globalReadA_0_0_59_0 + 0);
    a_0_0_60_0 = *(globalReadA_0_0_60_0 + 0);
    a_0_0_61_0 = *(globalReadA_0_0_61_0 + 0);
    a_0_0_62_0 = *(globalReadA_0_0_62_0 + 0);
    a_0_0_63_0 = *(globalReadA_0_0_63_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    globalReadA_0_0_16_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_16_0) + 1*globalReadIncAL);
    globalReadA_0_0_17_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_17_0) + 1*globalReadIncAL);
    globalReadA_0_0_18_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_18_0) + 1*globalReadIncAL);
    globalReadA_0_0_19_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_19_0) + 1*globalReadIncAL);
    globalReadA_0_0_20_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_20_0) + 1*globalReadIncAL);
    globalReadA_0_0_21_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_21_0) + 1*globalReadIncAL);
    globalReadA_0_0_22_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_22_0) + 1*globalReadIncAL);
    globalReadA_0_0_23_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_23_0) + 1*globalReadIncAL);
    globalReadA_0_0_24_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_24_0) + 1*globalReadIncAL);
    globalReadA_0_0_25_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_25_0) + 1*globalReadIncAL);
    globalReadA_0_0_26_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_26_0) + 1*globalReadIncAL);
    globalReadA_0_0_27_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_27_0) + 1*globalReadIncAL);
    globalReadA_0_0_28_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_28_0) + 1*globalReadIncAL);
    globalReadA_0_0_29_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_29_0) + 1*globalReadIncAL);
    globalReadA_0_0_30_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_30_0) + 1*globalReadIncAL);
    globalReadA_0_0_31_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_31_0) + 1*globalReadIncAL);
    globalReadA_0_0_32_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_32_0) + 1*globalReadIncAL);
    globalReadA_0_0_33_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_33_0) + 1*globalReadIncAL);
    globalReadA_0_0_34_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_34_0) + 1*globalReadIncAL);
    globalReadA_0_0_35_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_35_0) + 1*globalReadIncAL);
    globalReadA_0_0_36_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_36_0) + 1*globalReadIncAL);
    globalReadA_0_0_37_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_37_0) + 1*globalReadIncAL);
    globalReadA_0_0_38_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_38_0) + 1*globalReadIncAL);
    globalReadA_0_0_39_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_39_0) + 1*globalReadIncAL);
    globalReadA_0_0_40_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_40_0) + 1*globalReadIncAL);
    globalReadA_0_0_41_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_41_0) + 1*globalReadIncAL);
    globalReadA_0_0_42_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_42_0) + 1*globalReadIncAL);
    globalReadA_0_0_43_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_43_0) + 1*globalReadIncAL);
    globalReadA_0_0_44_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_44_0) + 1*globalReadIncAL);
    globalReadA_0_0_45_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_45_0) + 1*globalReadIncAL);
    globalReadA_0_0_46_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_46_0) + 1*globalReadIncAL);
    globalReadA_0_0_47_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_47_0) + 1*globalReadIncAL);
    globalReadA_0_0_48_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_48_0) + 1*globalReadIncAL);
    globalReadA_0_0_49_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_49_0) + 1*globalReadIncAL);
    globalReadA_0_0_50_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_50_0) + 1*globalReadIncAL);
    globalReadA_0_0_51_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_51_0) + 1*globalReadIncAL);
    globalReadA_0_0_52_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_52_0) + 1*globalReadIncAL);
    globalReadA_0_0_53_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_53_0) + 1*globalReadIncAL);
    globalReadA_0_0_54_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_54_0) + 1*globalReadIncAL);
    globalReadA_0_0_55_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_55_0) + 1*globalReadIncAL);
    globalReadA_0_0_56_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_56_0) + 1*globalReadIncAL);
    globalReadA_0_0_57_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_57_0) + 1*globalReadIncAL);
    globalReadA_0_0_58_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_58_0) + 1*globalReadIncAL);
    globalReadA_0_0_59_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_59_0) + 1*globalReadIncAL);
    globalReadA_0_0_60_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_60_0) + 1*globalReadIncAL);
    globalReadA_0_0_61_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_61_0) + 1*globalReadIncAL);
    globalReadA_0_0_62_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_62_0) + 1*globalReadIncAL);
    globalReadA_0_0_63_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_63_0) + 1*globalReadIncAL);

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
    *(localWriteA_0_0_16_0 + 0) = a_0_0_16_0;
    *(localWriteA_0_0_17_0 + 0) = a_0_0_17_0;
    *(localWriteA_0_0_18_0 + 0) = a_0_0_18_0;
    *(localWriteA_0_0_19_0 + 0) = a_0_0_19_0;
    *(localWriteA_0_0_20_0 + 0) = a_0_0_20_0;
    *(localWriteA_0_0_21_0 + 0) = a_0_0_21_0;
    *(localWriteA_0_0_22_0 + 0) = a_0_0_22_0;
    *(localWriteA_0_0_23_0 + 0) = a_0_0_23_0;
    *(localWriteA_0_0_24_0 + 0) = a_0_0_24_0;
    *(localWriteA_0_0_25_0 + 0) = a_0_0_25_0;
    *(localWriteA_0_0_26_0 + 0) = a_0_0_26_0;
    *(localWriteA_0_0_27_0 + 0) = a_0_0_27_0;
    *(localWriteA_0_0_28_0 + 0) = a_0_0_28_0;
    *(localWriteA_0_0_29_0 + 0) = a_0_0_29_0;
    *(localWriteA_0_0_30_0 + 0) = a_0_0_30_0;
    *(localWriteA_0_0_31_0 + 0) = a_0_0_31_0;
    *(localWriteA_0_0_32_0 + 0) = a_0_0_32_0;
    *(localWriteA_0_0_33_0 + 0) = a_0_0_33_0;
    *(localWriteA_0_0_34_0 + 0) = a_0_0_34_0;
    *(localWriteA_0_0_35_0 + 0) = a_0_0_35_0;
    *(localWriteA_0_0_36_0 + 0) = a_0_0_36_0;
    *(localWriteA_0_0_37_0 + 0) = a_0_0_37_0;
    *(localWriteA_0_0_38_0 + 0) = a_0_0_38_0;
    *(localWriteA_0_0_39_0 + 0) = a_0_0_39_0;
    *(localWriteA_0_0_40_0 + 0) = a_0_0_40_0;
    *(localWriteA_0_0_41_0 + 0) = a_0_0_41_0;
    *(localWriteA_0_0_42_0 + 0) = a_0_0_42_0;
    *(localWriteA_0_0_43_0 + 0) = a_0_0_43_0;
    *(localWriteA_0_0_44_0 + 0) = a_0_0_44_0;
    *(localWriteA_0_0_45_0 + 0) = a_0_0_45_0;
    *(localWriteA_0_0_46_0 + 0) = a_0_0_46_0;
    *(localWriteA_0_0_47_0 + 0) = a_0_0_47_0;
    *(localWriteA_0_0_48_0 + 0) = a_0_0_48_0;
    *(localWriteA_0_0_49_0 + 0) = a_0_0_49_0;
    *(localWriteA_0_0_50_0 + 0) = a_0_0_50_0;
    *(localWriteA_0_0_51_0 + 0) = a_0_0_51_0;
    *(localWriteA_0_0_52_0 + 0) = a_0_0_52_0;
    *(localWriteA_0_0_53_0 + 0) = a_0_0_53_0;
    *(localWriteA_0_0_54_0 + 0) = a_0_0_54_0;
    *(localWriteA_0_0_55_0 + 0) = a_0_0_55_0;
    *(localWriteA_0_0_56_0 + 0) = a_0_0_56_0;
    *(localWriteA_0_0_57_0 + 0) = a_0_0_57_0;
    *(localWriteA_0_0_58_0 + 0) = a_0_0_58_0;
    *(localWriteA_0_0_59_0 + 0) = a_0_0_59_0;
    *(localWriteA_0_0_60_0 + 0) = a_0_0_60_0;
    *(localWriteA_0_0_61_0 + 0) = a_0_0_61_0;
    *(localWriteA_0_0_62_0 + 0) = a_0_0_62_0;
    *(localWriteA_0_0_63_0 + 0) = a_0_0_63_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 15 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 16 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 17 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 18 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 19 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 20 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 21 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 22 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 23 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 24 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 25 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 26 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 27 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 28 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 29 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 30 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 31 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 32 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 33 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 34 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 35 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 36 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 37 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 38 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 39 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 40 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 41 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 42 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 43 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 44 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 45 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 46 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 47 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 48 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 49 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 50 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 51 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 52 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 53 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 54 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 55 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 56 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 57 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 58 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 59 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 60 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 61 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 62 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_1x1


    /* iter 63 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_1x1




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_0_8_0 = ( globalReadOffsetAL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_0_0_9_0 = ( globalReadOffsetAL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_0_0_10_0 = ( globalReadOffsetAL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_0_0_11_0 = ( globalReadOffsetAL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_0_0_12_0 = ( globalReadOffsetAL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_0_0_13_0 = ( globalReadOffsetAL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_0_0_14_0 = ( globalReadOffsetAL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_0_0_15_0 = ( globalReadOffsetAL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);
  a_0_0_16_0 = ( globalReadOffsetAL_16_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_16_0 + 0);
  a_0_0_17_0 = ( globalReadOffsetAL_17_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_17_0 + 0);
  a_0_0_18_0 = ( globalReadOffsetAL_18_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_18_0 + 0);
  a_0_0_19_0 = ( globalReadOffsetAL_19_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_19_0 + 0);
  a_0_0_20_0 = ( globalReadOffsetAL_20_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_20_0 + 0);
  a_0_0_21_0 = ( globalReadOffsetAL_21_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_21_0 + 0);
  a_0_0_22_0 = ( globalReadOffsetAL_22_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_22_0 + 0);
  a_0_0_23_0 = ( globalReadOffsetAL_23_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_23_0 + 0);
  a_0_0_24_0 = ( globalReadOffsetAL_24_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_24_0 + 0);
  a_0_0_25_0 = ( globalReadOffsetAL_25_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_25_0 + 0);
  a_0_0_26_0 = ( globalReadOffsetAL_26_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_26_0 + 0);
  a_0_0_27_0 = ( globalReadOffsetAL_27_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_27_0 + 0);
  a_0_0_28_0 = ( globalReadOffsetAL_28_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_28_0 + 0);
  a_0_0_29_0 = ( globalReadOffsetAL_29_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_29_0 + 0);
  a_0_0_30_0 = ( globalReadOffsetAL_30_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_30_0 + 0);
  a_0_0_31_0 = ( globalReadOffsetAL_31_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_31_0 + 0);
  a_0_0_32_0 = ( globalReadOffsetAL_32_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_32_0 + 0);
  a_0_0_33_0 = ( globalReadOffsetAL_33_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_33_0 + 0);
  a_0_0_34_0 = ( globalReadOffsetAL_34_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_34_0 + 0);
  a_0_0_35_0 = ( globalReadOffsetAL_35_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_35_0 + 0);
  a_0_0_36_0 = ( globalReadOffsetAL_36_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_36_0 + 0);
  a_0_0_37_0 = ( globalReadOffsetAL_37_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_37_0 + 0);
  a_0_0_38_0 = ( globalReadOffsetAL_38_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_38_0 + 0);
  a_0_0_39_0 = ( globalReadOffsetAL_39_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_39_0 + 0);
  a_0_0_40_0 = ( globalReadOffsetAL_40_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_40_0 + 0);
  a_0_0_41_0 = ( globalReadOffsetAL_41_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_41_0 + 0);
  a_0_0_42_0 = ( globalReadOffsetAL_42_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_42_0 + 0);
  a_0_0_43_0 = ( globalReadOffsetAL_43_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_43_0 + 0);
  a_0_0_44_0 = ( globalReadOffsetAL_44_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_44_0 + 0);
  a_0_0_45_0 = ( globalReadOffsetAL_45_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_45_0 + 0);
  a_0_0_46_0 = ( globalReadOffsetAL_46_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_46_0 + 0);
  a_0_0_47_0 = ( globalReadOffsetAL_47_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_47_0 + 0);
  a_0_0_48_0 = ( globalReadOffsetAL_48_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_48_0 + 0);
  a_0_0_49_0 = ( globalReadOffsetAL_49_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_49_0 + 0);
  a_0_0_50_0 = ( globalReadOffsetAL_50_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_50_0 + 0);
  a_0_0_51_0 = ( globalReadOffsetAL_51_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_51_0 + 0);
  a_0_0_52_0 = ( globalReadOffsetAL_52_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_52_0 + 0);
  a_0_0_53_0 = ( globalReadOffsetAL_53_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_53_0 + 0);
  a_0_0_54_0 = ( globalReadOffsetAL_54_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_54_0 + 0);
  a_0_0_55_0 = ( globalReadOffsetAL_55_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_55_0 + 0);
  a_0_0_56_0 = ( globalReadOffsetAL_56_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_56_0 + 0);
  a_0_0_57_0 = ( globalReadOffsetAL_57_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_57_0 + 0);
  a_0_0_58_0 = ( globalReadOffsetAL_58_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_58_0 + 0);
  a_0_0_59_0 = ( globalReadOffsetAL_59_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_59_0 + 0);
  a_0_0_60_0 = ( globalReadOffsetAL_60_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_60_0 + 0);
  a_0_0_61_0 = ( globalReadOffsetAL_61_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_61_0 + 0);
  a_0_0_62_0 = ( globalReadOffsetAL_62_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_62_0 + 0);
  a_0_0_63_0 = ( globalReadOffsetAL_63_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_63_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);
  localWriteA_0_0_16_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_16_0);
  localWriteA_0_0_17_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_17_0);
  localWriteA_0_0_18_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_18_0);
  localWriteA_0_0_19_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_19_0);
  localWriteA_0_0_20_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_20_0);
  localWriteA_0_0_21_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_21_0);
  localWriteA_0_0_22_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_22_0);
  localWriteA_0_0_23_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_23_0);
  localWriteA_0_0_24_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_24_0);
  localWriteA_0_0_25_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_25_0);
  localWriteA_0_0_26_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_26_0);
  localWriteA_0_0_27_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_27_0);
  localWriteA_0_0_28_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_28_0);
  localWriteA_0_0_29_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_29_0);
  localWriteA_0_0_30_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_30_0);
  localWriteA_0_0_31_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_31_0);
  localWriteA_0_0_32_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_32_0);
  localWriteA_0_0_33_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_33_0);
  localWriteA_0_0_34_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_34_0);
  localWriteA_0_0_35_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_35_0);
  localWriteA_0_0_36_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_36_0);
  localWriteA_0_0_37_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_37_0);
  localWriteA_0_0_38_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_38_0);
  localWriteA_0_0_39_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_39_0);
  localWriteA_0_0_40_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_40_0);
  localWriteA_0_0_41_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_41_0);
  localWriteA_0_0_42_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_42_0);
  localWriteA_0_0_43_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_43_0);
  localWriteA_0_0_44_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_44_0);
  localWriteA_0_0_45_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_45_0);
  localWriteA_0_0_46_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_46_0);
  localWriteA_0_0_47_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_47_0);
  localWriteA_0_0_48_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_48_0);
  localWriteA_0_0_49_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_49_0);
  localWriteA_0_0_50_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_50_0);
  localWriteA_0_0_51_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_51_0);
  localWriteA_0_0_52_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_52_0);
  localWriteA_0_0_53_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_53_0);
  localWriteA_0_0_54_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_54_0);
  localWriteA_0_0_55_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_55_0);
  localWriteA_0_0_56_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_56_0);
  localWriteA_0_0_57_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_57_0);
  localWriteA_0_0_58_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_58_0);
  localWriteA_0_0_59_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_59_0);
  localWriteA_0_0_60_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_60_0);
  localWriteA_0_0_61_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_61_0);
  localWriteA_0_0_62_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_62_0);
  localWriteA_0_0_63_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_63_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
  *(localWriteA_0_0_16_0 + 0) = a_0_0_16_0;
  *(localWriteA_0_0_17_0 + 0) = a_0_0_17_0;
  *(localWriteA_0_0_18_0 + 0) = a_0_0_18_0;
  *(localWriteA_0_0_19_0 + 0) = a_0_0_19_0;
  *(localWriteA_0_0_20_0 + 0) = a_0_0_20_0;
  *(localWriteA_0_0_21_0 + 0) = a_0_0_21_0;
  *(localWriteA_0_0_22_0 + 0) = a_0_0_22_0;
  *(localWriteA_0_0_23_0 + 0) = a_0_0_23_0;
  *(localWriteA_0_0_24_0 + 0) = a_0_0_24_0;
  *(localWriteA_0_0_25_0 + 0) = a_0_0_25_0;
  *(localWriteA_0_0_26_0 + 0) = a_0_0_26_0;
  *(localWriteA_0_0_27_0 + 0) = a_0_0_27_0;
  *(localWriteA_0_0_28_0 + 0) = a_0_0_28_0;
  *(localWriteA_0_0_29_0 + 0) = a_0_0_29_0;
  *(localWriteA_0_0_30_0 + 0) = a_0_0_30_0;
  *(localWriteA_0_0_31_0 + 0) = a_0_0_31_0;
  *(localWriteA_0_0_32_0 + 0) = a_0_0_32_0;
  *(localWriteA_0_0_33_0 + 0) = a_0_0_33_0;
  *(localWriteA_0_0_34_0 + 0) = a_0_0_34_0;
  *(localWriteA_0_0_35_0 + 0) = a_0_0_35_0;
  *(localWriteA_0_0_36_0 + 0) = a_0_0_36_0;
  *(localWriteA_0_0_37_0 + 0) = a_0_0_37_0;
  *(localWriteA_0_0_38_0 + 0) = a_0_0_38_0;
  *(localWriteA_0_0_39_0 + 0) = a_0_0_39_0;
  *(localWriteA_0_0_40_0 + 0) = a_0_0_40_0;
  *(localWriteA_0_0_41_0 + 0) = a_0_0_41_0;
  *(localWriteA_0_0_42_0 + 0) = a_0_0_42_0;
  *(localWriteA_0_0_43_0 + 0) = a_0_0_43_0;
  *(localWriteA_0_0_44_0 + 0) = a_0_0_44_0;
  *(localWriteA_0_0_45_0 + 0) = a_0_0_45_0;
  *(localWriteA_0_0_46_0 + 0) = a_0_0_46_0;
  *(localWriteA_0_0_47_0 + 0) = a_0_0_47_0;
  *(localWriteA_0_0_48_0 + 0) = a_0_0_48_0;
  *(localWriteA_0_0_49_0 + 0) = a_0_0_49_0;
  *(localWriteA_0_0_50_0 + 0) = a_0_0_50_0;
  *(localWriteA_0_0_51_0 + 0) = a_0_0_51_0;
  *(localWriteA_0_0_52_0 + 0) = a_0_0_52_0;
  *(localWriteA_0_0_53_0 + 0) = a_0_0_53_0;
  *(localWriteA_0_0_54_0 + 0) = a_0_0_54_0;
  *(localWriteA_0_0_55_0 + 0) = a_0_0_55_0;
  *(localWriteA_0_0_56_0 + 0) = a_0_0_56_0;
  *(localWriteA_0_0_57_0 + 0) = a_0_0_57_0;
  *(localWriteA_0_0_58_0 + 0) = a_0_0_58_0;
  *(localWriteA_0_0_59_0 + 0) = a_0_0_59_0;
  *(localWriteA_0_0_60_0 + 0) = a_0_0_60_0;
  *(localWriteA_0_0_61_0 + 0) = a_0_0_61_0;
  *(localWriteA_0_0_62_0 + 0) = a_0_0_62_0;
  *(localWriteA_0_0_63_0 + 0) = a_0_0_63_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_1x1

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_1x1
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT32x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT128x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;
  unsigned int flattenedOffsetA_4_0 = globalReadOffsetA0I + 0 + 4*LSPA;
  flattenedOffsetA_4_0 = (flattenedOffsetA_4_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_4_0;
  unsigned int globalReadOffsetA0I_4_0 = flattenedOffsetA_4_0;
  unsigned int flattenedOffsetA_5_0 = globalReadOffsetA0I + 0 + 5*LSPA;
  flattenedOffsetA_5_0 = (flattenedOffsetA_5_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_5_0;
  unsigned int globalReadOffsetA0I_5_0 = flattenedOffsetA_5_0;
  unsigned int flattenedOffsetA_6_0 = globalReadOffsetA0I + 0 + 6*LSPA;
  flattenedOffsetA_6_0 = (flattenedOffsetA_6_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_6_0;
  unsigned int globalReadOffsetA0I_6_0 = flattenedOffsetA_6_0;
  unsigned int flattenedOffsetA_7_0 = globalReadOffsetA0I + 0 + 7*LSPA;
  flattenedOffsetA_7_0 = (flattenedOffsetA_7_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_7_0;
  unsigned int globalReadOffsetA0I_7_0 = flattenedOffsetA_7_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 4*LSPA);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 5*LSPA);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 6*LSPA);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 7*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT32x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 1536

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT16x16x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 8192
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
  a_pk_fma[0] = MULA0; \
  a_pk_fma[1] = MULA1; \
  b_pk_fma[0] = MULB0; \
  b_pk_fma[1] = MULB1; \
  c_pk_fma[0] = DST0; \
  c_pk_fma[1] = DST1; \
  c_pk_fma = tensile_fmadd_half2(a_pk_fma, b_pk_fma, c_pk_fma); \
  DST0 = c_pk_fma[0]; \
  DST1 = c_pk_fma[1]; \
 
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I] , rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I] , rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I] , rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I] , rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I] , rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I] , rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I] , rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I] , rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I] , rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I] , rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I] , rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I] , rA[7],rB[7],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HB_MT128x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_1_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_1_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_8x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_1_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 1);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_1_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else if (s0I == 1) {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else if (s0I == 2) {
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else {
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 1) {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 2) {
        rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 819

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
  a_pk_fma[0] = MULA0; \
  a_pk_fma[1] = MULA1; \
  b_pk_fma[0] = MULB0; \
  b_pk_fma[1] = MULB1; \
  c_pk_fma[0] = DST0; \
  c_pk_fma[1] = DST1; \
  c_pk_fma = tensile_fmadd_half2(a_pk_fma, b_pk_fma, c_pk_fma); \
  DST0 = c_pk_fma[0]; \
  DST1 = c_pk_fma[1]; \
 
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HB_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 3 */

    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
  a_pk_fma[0] = MULA0; \
  a_pk_fma[1] = MULA1; \
  b_pk_fma[0] = MULB0; \
  b_pk_fma[1] = MULB1; \
  c_pk_fma[0] = DST0; \
  c_pk_fma[1] = DST1; \
  c_pk_fma = tensile_fmadd_half2(a_pk_fma, b_pk_fma, c_pk_fma); \
  DST0 = c_pk_fma[0]; \
  DST1 = c_pk_fma[1]; \
 
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HB_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 32
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 16

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 6144

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
  a_pk_fma[0] = MULA0; \
  a_pk_fma[1] = MULA1; \
  b_pk_fma[0] = MULB0; \
  b_pk_fma[1] = MULB1; \
  c_pk_fma[0] = DST0; \
  c_pk_fma[1] = DST1; \
  c_pk_fma = tensile_fmadd_half2(a_pk_fma, b_pk_fma, c_pk_fma); \
  DST0 = c_pk_fma[0]; \
  DST1 = c_pk_fma[1]; \
 
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HB_MT64x128x32_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;
  DATA_TYPE b_0_0_8_0;
  DATA_TYPE b_0_0_9_0;
  DATA_TYPE b_0_0_10_0;
  DATA_TYPE b_0_0_11_0;
  DATA_TYPE b_0_0_12_0;
  DATA_TYPE b_0_0_13_0;
  DATA_TYPE b_0_0_14_0;
  DATA_TYPE b_0_0_15_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;
  unsigned int globalReadOffsetBL_4_0 = globalReadOffsetBL + 0 + 4*LSPB;
  unsigned int globalReadOffsetBL_5_0 = globalReadOffsetBL + 0 + 5*LSPB;
  unsigned int globalReadOffsetBL_6_0 = globalReadOffsetBL + 0 + 6*LSPB;
  unsigned int globalReadOffsetBL_7_0 = globalReadOffsetBL + 0 + 7*LSPB;
  unsigned int globalReadOffsetBL_8_0 = globalReadOffsetBL + 0 + 8*LSPB;
  unsigned int globalReadOffsetBL_9_0 = globalReadOffsetBL + 0 + 9*LSPB;
  unsigned int globalReadOffsetBL_10_0 = globalReadOffsetBL + 0 + 10*LSPB;
  unsigned int globalReadOffsetBL_11_0 = globalReadOffsetBL + 0 + 11*LSPB;
  unsigned int globalReadOffsetBL_12_0 = globalReadOffsetBL + 0 + 12*LSPB;
  unsigned int globalReadOffsetBL_13_0 = globalReadOffsetBL + 0 + 13*LSPB;
  unsigned int globalReadOffsetBL_14_0 = globalReadOffsetBL + 0 + 14*LSPB;
  unsigned int globalReadOffsetBL_15_0 = globalReadOffsetBL + 0 + 15*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_7_0), wgK );
  int64_t globalReadOffsetB_0_0_8_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_8_0), wgK );
  int64_t globalReadOffsetB_0_0_9_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_9_0), wgK );
  int64_t globalReadOffsetB_0_0_10_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_10_0), wgK );
  int64_t globalReadOffsetB_0_0_11_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_11_0), wgK );
  int64_t globalReadOffsetB_0_0_12_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_12_0), wgK );
  int64_t globalReadOffsetB_0_0_13_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_13_0), wgK );
  int64_t globalReadOffsetB_0_0_14_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_14_0), wgK );
  int64_t globalReadOffsetB_0_0_15_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_15_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;
  DATA_TYPE const *globalReadB_0_0_8_0 = B + globalReadOffsetB_0_0_8_0;
  DATA_TYPE const *globalReadB_0_0_9_0 = B + globalReadOffsetB_0_0_9_0;
  DATA_TYPE const *globalReadB_0_0_10_0 = B + globalReadOffsetB_0_0_10_0;
  DATA_TYPE const *globalReadB_0_0_11_0 = B + globalReadOffsetB_0_0_11_0;
  DATA_TYPE const *globalReadB_0_0_12_0 = B + globalReadOffsetB_0_0_12_0;
  DATA_TYPE const *globalReadB_0_0_13_0 = B + globalReadOffsetB_0_0_13_0;
  DATA_TYPE const *globalReadB_0_0_14_0 = B + globalReadOffsetB_0_0_14_0;
  DATA_TYPE const *globalReadB_0_0_15_0 = B + globalReadOffsetB_0_0_15_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 4*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 5*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 6*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 7*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_8_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 8*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_9_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 9*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_10_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 10*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_11_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 11*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_12_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 12*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_13_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 13*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_14_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 14*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_15_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 15*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;
  DATA_TYPE *localWriteB_0_0_8_0;
  DATA_TYPE *localWriteB_0_0_9_0;
  DATA_TYPE *localWriteB_0_0_10_0;
  DATA_TYPE *localWriteB_0_0_11_0;
  DATA_TYPE *localWriteB_0_0_12_0;
  DATA_TYPE *localWriteB_0_0_13_0;
  DATA_TYPE *localWriteB_0_0_14_0;
  DATA_TYPE *localWriteB_0_0_15_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);
  localWriteB_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_8_0);
  localWriteB_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_9_0);
  localWriteB_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_10_0);
  localWriteB_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_11_0);
  localWriteB_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_12_0);
  localWriteB_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_13_0);
  localWriteB_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_14_0);
  localWriteB_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_15_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_8_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_9_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_10_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_11_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_12_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_13_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_14_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_15_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);
    b_0_0_8_0 = *(globalReadB_0_0_8_0 + 0);
    b_0_0_9_0 = *(globalReadB_0_0_9_0 + 0);
    b_0_0_10_0 = *(globalReadB_0_0_10_0 + 0);
    b_0_0_11_0 = *(globalReadB_0_0_11_0 + 0);
    b_0_0_12_0 = *(globalReadB_0_0_12_0 + 0);
    b_0_0_13_0 = *(globalReadB_0_0_13_0 + 0);
    b_0_0_14_0 = *(globalReadB_0_0_14_0 + 0);
    b_0_0_15_0 = *(globalReadB_0_0_15_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_8_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_8_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_9_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_9_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_10_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_10_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_11_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_11_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_12_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_12_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_13_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_13_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_14_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_14_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_15_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_15_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
    *(localWriteB_0_0_8_0 + 0) = b_0_0_8_0;
    *(localWriteB_0_0_9_0 + 0) = b_0_0_9_0;
    *(localWriteB_0_0_10_0 + 0) = b_0_0_10_0;
    *(localWriteB_0_0_11_0 + 0) = b_0_0_11_0;
    *(localWriteB_0_0_12_0 + 0) = b_0_0_12_0;
    *(localWriteB_0_0_13_0 + 0) = b_0_0_13_0;
    *(localWriteB_0_0_14_0 + 0) = b_0_0_14_0;
    *(localWriteB_0_0_15_0 + 0) = b_0_0_15_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 15 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 16 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 17 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 18 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 19 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 20 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 21 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 22 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 23 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 24 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 25 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 26 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 27 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 28 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 29 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 30 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 31 */

    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);
  b_0_0_8_0 = ( globalReadOffsetBL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_8_0 + 0);
  b_0_0_9_0 = ( globalReadOffsetBL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_9_0 + 0);
  b_0_0_10_0 = ( globalReadOffsetBL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_10_0 + 0);
  b_0_0_11_0 = ( globalReadOffsetBL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_11_0 + 0);
  b_0_0_12_0 = ( globalReadOffsetBL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_12_0 + 0);
  b_0_0_13_0 = ( globalReadOffsetBL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_13_0 + 0);
  b_0_0_14_0 = ( globalReadOffsetBL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_14_0 + 0);
  b_0_0_15_0 = ( globalReadOffsetBL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_15_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);
  localWriteB_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_8_0);
  localWriteB_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_9_0);
  localWriteB_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_10_0);
  localWriteB_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_11_0);
  localWriteB_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_12_0);
  localWriteB_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_13_0);
  localWriteB_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_14_0);
  localWriteB_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_15_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
  *(localWriteB_0_0_8_0 + 0) = b_0_0_8_0;
  *(localWriteB_0_0_9_0 + 0) = b_0_0_9_0;
  *(localWriteB_0_0_10_0 + 0) = b_0_0_10_0;
  *(localWriteB_0_0_11_0 + 0) = b_0_0_11_0;
  *(localWriteB_0_0_12_0 + 0) = b_0_0_12_0;
  *(localWriteB_0_0_13_0 + 0) = b_0_0_13_0;
  *(localWriteB_0_0_14_0 + 0) = b_0_0_14_0;
  *(localWriteB_0_0_15_0 + 0) = b_0_0_15_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 8192
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
  a_pk_fma[0] = MULA0; \
  a_pk_fma[1] = MULA1; \
  b_pk_fma[0] = MULB0; \
  b_pk_fma[1] = MULB1; \
  c_pk_fma[0] = DST0; \
  c_pk_fma[1] = DST1; \
  c_pk_fma = tensile_fmadd_half2(a_pk_fma, b_pk_fma, c_pk_fma); \
  DST0 = c_pk_fma[0]; \
  DST1 = c_pk_fma[1]; \
 
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I] , rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I] , rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I] , rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I] , rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I] , rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I] , rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I] , rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I] , rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I] , rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I] , rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I] , rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I] , rA[7],rB[7],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HB_MT128x128x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_1_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_1_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_8x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_1_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 1);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_1_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else if (s0I == 1) {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else if (s0I == 2) {
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else {
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 1) {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 2) {
        rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 819

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
  a_pk_fma[0] = MULA0; \
  a_pk_fma[1] = MULA1; \
  b_pk_fma[0] = MULB0; \
  b_pk_fma[1] = MULB1; \
  c_pk_fma[0] = DST0; \
  c_pk_fma[1] = DST1; \
  c_pk_fma = tensile_fmadd_half2(a_pk_fma, b_pk_fma, c_pk_fma); \
  DST0 = c_pk_fma[0]; \
  DST1 = c_pk_fma[1]; \
 
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HB_MT64x128x4_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 3 */

    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
  a_pk_fma[0] = MULA0; \
  a_pk_fma[1] = MULA1; \
  b_pk_fma[0] = MULB0; \
  b_pk_fma[1] = MULB1; \
  c_pk_fma[0] = DST0; \
  c_pk_fma[1] = DST1; \
  c_pk_fma = tensile_fmadd_half2(a_pk_fma, b_pk_fma, c_pk_fma); \
  DST0 = c_pk_fma[0]; \
  DST1 = c_pk_fma[1]; \
 
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HB_MT64x64x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 32
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 16

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 6144

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
  a_pk_fma[0] = MULA0; \
  a_pk_fma[1] = MULA1; \
  b_pk_fma[0] = MULB0; \
  b_pk_fma[1] = MULB1; \
  c_pk_fma[0] = DST0; \
  c_pk_fma[1] = DST1; \
  c_pk_fma = tensile_fmadd_half2(a_pk_fma, b_pk_fma, c_pk_fma); \
  DST0 = c_pk_fma[0]; \
  DST1 = c_pk_fma[1]; \
 
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_HB_MT64x128x32_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;
  DATA_TYPE b_0_0_8_0;
  DATA_TYPE b_0_0_9_0;
  DATA_TYPE b_0_0_10_0;
  DATA_TYPE b_0_0_11_0;
  DATA_TYPE b_0_0_12_0;
  DATA_TYPE b_0_0_13_0;
  DATA_TYPE b_0_0_14_0;
  DATA_TYPE b_0_0_15_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;
  unsigned int globalReadOffsetBL_4_0 = globalReadOffsetBL + 0 + 4*LSPB;
  unsigned int globalReadOffsetBL_5_0 = globalReadOffsetBL + 0 + 5*LSPB;
  unsigned int globalReadOffsetBL_6_0 = globalReadOffsetBL + 0 + 6*LSPB;
  unsigned int globalReadOffsetBL_7_0 = globalReadOffsetBL + 0 + 7*LSPB;
  unsigned int globalReadOffsetBL_8_0 = globalReadOffsetBL + 0 + 8*LSPB;
  unsigned int globalReadOffsetBL_9_0 = globalReadOffsetBL + 0 + 9*LSPB;
  unsigned int globalReadOffsetBL_10_0 = globalReadOffsetBL + 0 + 10*LSPB;
  unsigned int globalReadOffsetBL_11_0 = globalReadOffsetBL + 0 + 11*LSPB;
  unsigned int globalReadOffsetBL_12_0 = globalReadOffsetBL + 0 + 12*LSPB;
  unsigned int globalReadOffsetBL_13_0 = globalReadOffsetBL + 0 + 13*LSPB;
  unsigned int globalReadOffsetBL_14_0 = globalReadOffsetBL + 0 + 14*LSPB;
  unsigned int globalReadOffsetBL_15_0 = globalReadOffsetBL + 0 + 15*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_7_0), wgK );
  int64_t globalReadOffsetB_0_0_8_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_8_0), wgK );
  int64_t globalReadOffsetB_0_0_9_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_9_0), wgK );
  int64_t globalReadOffsetB_0_0_10_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_10_0), wgK );
  int64_t globalReadOffsetB_0_0_11_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_11_0), wgK );
  int64_t globalReadOffsetB_0_0_12_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_12_0), wgK );
  int64_t globalReadOffsetB_0_0_13_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_13_0), wgK );
  int64_t globalReadOffsetB_0_0_14_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_14_0), wgK );
  int64_t globalReadOffsetB_0_0_15_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_15_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;
  DATA_TYPE const *globalReadB_0_0_8_0 = B + globalReadOffsetB_0_0_8_0;
  DATA_TYPE const *globalReadB_0_0_9_0 = B + globalReadOffsetB_0_0_9_0;
  DATA_TYPE const *globalReadB_0_0_10_0 = B + globalReadOffsetB_0_0_10_0;
  DATA_TYPE const *globalReadB_0_0_11_0 = B + globalReadOffsetB_0_0_11_0;
  DATA_TYPE const *globalReadB_0_0_12_0 = B + globalReadOffsetB_0_0_12_0;
  DATA_TYPE const *globalReadB_0_0_13_0 = B + globalReadOffsetB_0_0_13_0;
  DATA_TYPE const *globalReadB_0_0_14_0 = B + globalReadOffsetB_0_0_14_0;
  DATA_TYPE const *globalReadB_0_0_15_0 = B + globalReadOffsetB_0_0_15_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 4*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 5*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 6*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 7*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_8_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 8*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_9_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 9*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_10_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 10*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_11_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 11*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_12_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 12*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_13_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 13*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_14_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 14*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_15_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 15*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;
  DATA_TYPE *localWriteB_0_0_8_0;
  DATA_TYPE *localWriteB_0_0_9_0;
  DATA_TYPE *localWriteB_0_0_10_0;
  DATA_TYPE *localWriteB_0_0_11_0;
  DATA_TYPE *localWriteB_0_0_12_0;
  DATA_TYPE *localWriteB_0_0_13_0;
  DATA_TYPE *localWriteB_0_0_14_0;
  DATA_TYPE *localWriteB_0_0_15_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);
  localWriteB_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_8_0);
  localWriteB_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_9_0);
  localWriteB_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_10_0);
  localWriteB_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_11_0);
  localWriteB_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_12_0);
  localWriteB_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_13_0);
  localWriteB_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_14_0);
  localWriteB_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_15_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_8_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_9_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_10_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_11_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_12_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_13_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_14_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_15_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);
    b_0_0_8_0 = *(globalReadB_0_0_8_0 + 0);
    b_0_0_9_0 = *(globalReadB_0_0_9_0 + 0);
    b_0_0_10_0 = *(globalReadB_0_0_10_0 + 0);
    b_0_0_11_0 = *(globalReadB_0_0_11_0 + 0);
    b_0_0_12_0 = *(globalReadB_0_0_12_0 + 0);
    b_0_0_13_0 = *(globalReadB_0_0_13_0 + 0);
    b_0_0_14_0 = *(globalReadB_0_0_14_0 + 0);
    b_0_0_15_0 = *(globalReadB_0_0_15_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_8_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_8_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_9_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_9_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_10_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_10_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_11_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_11_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_12_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_12_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_13_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_13_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_14_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_14_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_15_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_15_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
    *(localWriteB_0_0_8_0 + 0) = b_0_0_8_0;
    *(localWriteB_0_0_9_0 + 0) = b_0_0_9_0;
    *(localWriteB_0_0_10_0 + 0) = b_0_0_10_0;
    *(localWriteB_0_0_11_0 + 0) = b_0_0_11_0;
    *(localWriteB_0_0_12_0 + 0) = b_0_0_12_0;
    *(localWriteB_0_0_13_0 + 0) = b_0_0_13_0;
    *(localWriteB_0_0_14_0 + 0) = b_0_0_14_0;
    *(localWriteB_0_0_15_0 + 0) = b_0_0_15_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 15 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 16 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 17 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 18 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 19 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 20 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 21 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 22 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 23 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 24 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 25 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 26 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 27 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 28 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 29 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 30 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 31 */

    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);
  b_0_0_8_0 = ( globalReadOffsetBL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_8_0 + 0);
  b_0_0_9_0 = ( globalReadOffsetBL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_9_0 + 0);
  b_0_0_10_0 = ( globalReadOffsetBL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_10_0 + 0);
  b_0_0_11_0 = ( globalReadOffsetBL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_11_0 + 0);
  b_0_0_12_0 = ( globalReadOffsetBL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_12_0 + 0);
  b_0_0_13_0 = ( globalReadOffsetBL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_13_0 + 0);
  b_0_0_14_0 = ( globalReadOffsetBL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_14_0 + 0);
  b_0_0_15_0 = ( globalReadOffsetBL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_15_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);
  localWriteB_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_8_0);
  localWriteB_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_9_0);
  localWriteB_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_10_0);
  localWriteB_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_11_0);
  localWriteB_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_12_0);
  localWriteB_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_13_0);
  localWriteB_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_14_0);
  localWriteB_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_15_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
  *(localWriteB_0_0_8_0 + 0) = b_0_0_8_0;
  *(localWriteB_0_0_9_0 + 0) = b_0_0_9_0;
  *(localWriteB_0_0_10_0 + 0) = b_0_0_10_0;
  *(localWriteB_0_0_11_0 + 0) = b_0_0_11_0;
  *(localWriteB_0_0_12_0 + 0) = b_0_0_12_0;
  *(localWriteB_0_0_13_0 + 0) = b_0_0_13_0;
  *(localWriteB_0_0_14_0 + 0) = b_0_0_14_0;
  *(localWriteB_0_0_15_0 + 0) = b_0_0_15_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1536

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
  a_pk_fma[0] = MULA0; \
  a_pk_fma[1] = MULA1; \
  b_pk_fma[0] = MULB0; \
  b_pk_fma[1] = MULB1; \
  c_pk_fma[0] = DST0; \
  c_pk_fma[1] = DST1; \
  c_pk_fma = tensile_fmadd_half2(a_pk_fma, b_pk_fma, c_pk_fma); \
  DST0 = c_pk_fma[0]; \
  DST1 = c_pk_fma[1]; \
 
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HB_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 7 */

    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 819

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
  a_pk_fma[0] = MULA0; \
  a_pk_fma[1] = MULA1; \
  b_pk_fma[0] = MULB0; \
  b_pk_fma[1] = MULB1; \
  c_pk_fma[0] = DST0; \
  c_pk_fma[1] = DST1; \
  c_pk_fma = tensile_fmadd_half2(a_pk_fma, b_pk_fma, c_pk_fma); \
  DST0 = c_pk_fma[0]; \
  DST1 = c_pk_fma[1]; \
 
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HB_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
  a_pk_fma[0] = MULA0; \
  a_pk_fma[1] = MULA1; \
  b_pk_fma[0] = MULB0; \
  b_pk_fma[1] = MULB1; \
  c_pk_fma[0] = DST0; \
  c_pk_fma[1] = DST1; \
  c_pk_fma = tensile_fmadd_half2(a_pk_fma, b_pk_fma, c_pk_fma); \
  DST0 = c_pk_fma[0]; \
  DST1 = c_pk_fma[1]; \
 
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HB_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 7 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1536

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
  a_pk_fma[0] = MULA0; \
  a_pk_fma[1] = MULA1; \
  b_pk_fma[0] = MULB0; \
  b_pk_fma[1] = MULB1; \
  c_pk_fma[0] = DST0; \
  c_pk_fma[1] = DST1; \
  c_pk_fma = tensile_fmadd_half2(a_pk_fma, b_pk_fma, c_pk_fma); \
  DST0 = c_pk_fma[0]; \
  DST1 = c_pk_fma[1]; \
 
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HB_MT64x128x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 7 */

    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 819

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
  a_pk_fma[0] = MULA0; \
  a_pk_fma[1] = MULA1; \
  b_pk_fma[0] = MULB0; \
  b_pk_fma[1] = MULB1; \
  c_pk_fma[0] = DST0; \
  c_pk_fma[1] = DST1; \
  c_pk_fma = tensile_fmadd_half2(a_pk_fma, b_pk_fma, c_pk_fma); \
  DST0 = c_pk_fma[0]; \
  DST1 = c_pk_fma[1]; \
 
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HB_MT64x128x4_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
  a_pk_fma[0] = MULA0; \
  a_pk_fma[1] = MULA1; \
  b_pk_fma[0] = MULB0; \
  b_pk_fma[1] = MULB1; \
  c_pk_fma[0] = DST0; \
  c_pk_fma[1] = DST1; \
  c_pk_fma = tensile_fmadd_half2(a_pk_fma, b_pk_fma, c_pk_fma); \
  DST0 = c_pk_fma[0]; \
  DST1 = c_pk_fma[1]; \
 
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HB_MT64x64x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 7 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE double
#define DEST_DATA_TYPE double
#define COMPUTE_DATA_TYPE double
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_DB_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW2_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  double *D,
  double const * __restrict__ C,
  double const * __restrict__ A,
  double const * __restrict__ B,
  double const alpha,
  double const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (double)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 7 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE double
#define DEST_DATA_TYPE double
#define COMPUTE_DATA_TYPE double
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_DB_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW2_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  double *D,
  double const * __restrict__ C,
  double const * __restrict__ A,
  double const * __restrict__ B,
  double const alpha,
  double const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (double)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 3 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 3 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 768

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_BSS_BH_MT64x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_bfloat16)(0)
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 7 */

    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 768

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_BSS_BH_MT64x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_bfloat16)(0)
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 7 */

    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_BSS_BH_MT128x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_bfloat16)(0)
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_2_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_3_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_2_1;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_3_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_1 + 0) = a_0_1_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_1 + 0) = a_0_1_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 16
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 16
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 2304

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 16x2 micro-tile */
#define MAC_16x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[8],rB[0],rC[8+0*TT0I]); \
  TYPE_MAC(rA[9],rB[0],rC[9+0*TT0I]); \
  TYPE_MAC(rA[10],rB[0],rC[10+0*TT0I]); \
  TYPE_MAC(rA[11],rB[0],rC[11+0*TT0I]); \
  TYPE_MAC(rA[12],rB[0],rC[12+0*TT0I]); \
  TYPE_MAC(rA[13],rB[0],rC[13+0*TT0I]); \
  TYPE_MAC(rA[14],rB[0],rC[14+0*TT0I]); \
  TYPE_MAC(rA[15],rB[0],rC[15+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[8],rB[1],rC[8+1*TT0I]); \
  TYPE_MAC(rA[9],rB[1],rC[9+1*TT0I]); \
  TYPE_MAC(rA[10],rB[1],rC[10+1*TT0I]); \
  TYPE_MAC(rA[11],rB[1],rC[11+1*TT0I]); \
  TYPE_MAC(rA[12],rB[1],rC[12+1*TT0I]); \
  TYPE_MAC(rA[13],rB[1],rC[13+1*TT0I]); \
  TYPE_MAC(rA[14],rB[1],rC[14+1*TT0I]); \
  TYPE_MAC(rA[15],rB[1],rC[15+1*TT0I]); \

#define MAC_16x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[8+TT0I],rB[0+TT1J],rC[8+0*TT0I]); \
  TYPE_MAC(rA[9+TT0I],rB[0+TT1J],rC[9+0*TT0I]); \
  TYPE_MAC(rA[10+TT0I],rB[0+TT1J],rC[10+0*TT0I]); \
  TYPE_MAC(rA[11+TT0I],rB[0+TT1J],rC[11+0*TT0I]); \
  TYPE_MAC(rA[12+TT0I],rB[0+TT1J],rC[12+0*TT0I]); \
  TYPE_MAC(rA[13+TT0I],rB[0+TT1J],rC[13+0*TT0I]); \
  TYPE_MAC(rA[14+TT0I],rB[0+TT1J],rC[14+0*TT0I]); \
  TYPE_MAC(rA[15+TT0I],rB[0+TT1J],rC[15+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[8+TT0I],rB[1+TT1J],rC[8+1*TT0I]); \
  TYPE_MAC(rA[9+TT0I],rB[1+TT1J],rC[9+1*TT0I]); \
  TYPE_MAC(rA[10+TT0I],rB[1+TT1J],rC[10+1*TT0I]); \
  TYPE_MAC(rA[11+TT0I],rB[1+TT1J],rC[11+1*TT0I]); \
  TYPE_MAC(rA[12+TT0I],rB[1+TT1J],rC[12+1*TT0I]); \
  TYPE_MAC(rA[13+TT0I],rB[1+TT1J],rC[13+1*TT0I]); \
  TYPE_MAC(rA[14+TT0I],rB[1+TT1J],rC[14+1*TT0I]); \
  TYPE_MAC(rA[15+TT0I],rB[1+TT1J],rC[15+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_BSS_BH_MT128x16x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT16_2_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_bfloat16)(0)
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_1_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_1_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_1_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_1_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_0_1_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_0_1_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_0_1_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_0_1_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_0_1_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_0_1_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_0_1_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE a_0_1_15_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;
  unsigned int flattenedOffsetA_4_0 = globalReadOffsetA0I + 0 + 4*LSPA;
  flattenedOffsetA_4_0 = (flattenedOffsetA_4_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_4_0;
  unsigned int globalReadOffsetA0I_4_0 = flattenedOffsetA_4_0;
  unsigned int flattenedOffsetA_5_0 = globalReadOffsetA0I + 0 + 5*LSPA;
  flattenedOffsetA_5_0 = (flattenedOffsetA_5_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_5_0;
  unsigned int globalReadOffsetA0I_5_0 = flattenedOffsetA_5_0;
  unsigned int flattenedOffsetA_6_0 = globalReadOffsetA0I + 0 + 6*LSPA;
  flattenedOffsetA_6_0 = (flattenedOffsetA_6_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_6_0;
  unsigned int globalReadOffsetA0I_6_0 = flattenedOffsetA_6_0;
  unsigned int flattenedOffsetA_7_0 = globalReadOffsetA0I + 0 + 7*LSPA;
  flattenedOffsetA_7_0 = (flattenedOffsetA_7_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_7_0;
  unsigned int globalReadOffsetA0I_7_0 = flattenedOffsetA_7_0;
  unsigned int flattenedOffsetA_8_0 = globalReadOffsetA0I + 0 + 8*LSPA;
  flattenedOffsetA_8_0 = (flattenedOffsetA_8_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_8_0;
  unsigned int globalReadOffsetA0I_8_0 = flattenedOffsetA_8_0;
  unsigned int flattenedOffsetA_9_0 = globalReadOffsetA0I + 0 + 9*LSPA;
  flattenedOffsetA_9_0 = (flattenedOffsetA_9_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_9_0;
  unsigned int globalReadOffsetA0I_9_0 = flattenedOffsetA_9_0;
  unsigned int flattenedOffsetA_10_0 = globalReadOffsetA0I + 0 + 10*LSPA;
  flattenedOffsetA_10_0 = (flattenedOffsetA_10_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_10_0;
  unsigned int globalReadOffsetA0I_10_0 = flattenedOffsetA_10_0;
  unsigned int flattenedOffsetA_11_0 = globalReadOffsetA0I + 0 + 11*LSPA;
  flattenedOffsetA_11_0 = (flattenedOffsetA_11_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_11_0;
  unsigned int globalReadOffsetA0I_11_0 = flattenedOffsetA_11_0;
  unsigned int flattenedOffsetA_12_0 = globalReadOffsetA0I + 0 + 12*LSPA;
  flattenedOffsetA_12_0 = (flattenedOffsetA_12_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_12_0;
  unsigned int globalReadOffsetA0I_12_0 = flattenedOffsetA_12_0;
  unsigned int flattenedOffsetA_13_0 = globalReadOffsetA0I + 0 + 13*LSPA;
  flattenedOffsetA_13_0 = (flattenedOffsetA_13_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_13_0;
  unsigned int globalReadOffsetA0I_13_0 = flattenedOffsetA_13_0;
  unsigned int flattenedOffsetA_14_0 = globalReadOffsetA0I + 0 + 14*LSPA;
  flattenedOffsetA_14_0 = (flattenedOffsetA_14_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_14_0;
  unsigned int globalReadOffsetA0I_14_0 = flattenedOffsetA_14_0;
  unsigned int flattenedOffsetA_15_0 = globalReadOffsetA0I + 0 + 15*LSPA;
  flattenedOffsetA_15_0 = (flattenedOffsetA_15_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_15_0;
  unsigned int globalReadOffsetA0I_15_0 = flattenedOffsetA_15_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_15_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_2_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_3_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 4*LSPA);
  unsigned int localWriteOffsetA_0_0_4_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 4*LSPA);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 5*LSPA);
  unsigned int localWriteOffsetA_0_0_5_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 5*LSPA);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 6*LSPA);
  unsigned int localWriteOffsetA_0_0_6_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 6*LSPA);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 7*LSPA);
  unsigned int localWriteOffsetA_0_0_7_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 7*LSPA);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 8*LSPA);
  unsigned int localWriteOffsetA_0_0_8_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 8*LSPA);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 9*LSPA);
  unsigned int localWriteOffsetA_0_0_9_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 9*LSPA);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 10*LSPA);
  unsigned int localWriteOffsetA_0_0_10_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 10*LSPA);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 11*LSPA);
  unsigned int localWriteOffsetA_0_0_11_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 11*LSPA);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 12*LSPA);
  unsigned int localWriteOffsetA_0_0_12_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 12*LSPA);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 13*LSPA);
  unsigned int localWriteOffsetA_0_0_13_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 13*LSPA);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 14*LSPA);
  unsigned int localWriteOffsetA_0_0_14_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 14*LSPA);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 15*LSPA);
  unsigned int localWriteOffsetA_0_0_15_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 15*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_2_1;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_3_1;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_4_1;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_5_1;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_6_1;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_0_0_7_1;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_0_0_8_1;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_0_0_9_1;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_0_0_10_1;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_0_0_11_1;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_0_0_12_1;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_0_0_13_1;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_0_0_14_1;
  DATA_TYPE *localWriteA_0_0_15_0;
  DATA_TYPE *localWriteA_0_0_15_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_4_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_1);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_5_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_1);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_6_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_1);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_7_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_1);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_8_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_1);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_9_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_1);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_10_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_1);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_11_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_1);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_12_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_1);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_13_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_1);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_14_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_1);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);
  localWriteA_0_0_15_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_8_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_9_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_10_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_11_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_12_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_13_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_14_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_15_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_1_4_0 = *(globalReadA_0_0_4_0 + 1);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_1_5_0 = *(globalReadA_0_0_5_0 + 1);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_1_6_0 = *(globalReadA_0_0_6_0 + 1);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_1_7_0 = *(globalReadA_0_0_7_0 + 1);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_0_1_8_0 = *(globalReadA_0_0_8_0 + 1);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_0_1_9_0 = *(globalReadA_0_0_9_0 + 1);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_0_1_10_0 = *(globalReadA_0_0_10_0 + 1);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_0_1_11_0 = *(globalReadA_0_0_11_0 + 1);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_0_1_12_0 = *(globalReadA_0_0_12_0 + 1);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_0_1_13_0 = *(globalReadA_0_0_13_0 + 1);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_0_1_14_0 = *(globalReadA_0_0_14_0 + 1);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);
    a_0_1_15_0 = *(globalReadA_0_0_15_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_8_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_9_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_10_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_11_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_12_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_13_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_14_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_15_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_1 + 0) = a_0_1_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_4_1 + 0) = a_0_1_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_5_1 + 0) = a_0_1_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_6_1 + 0) = a_0_1_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_7_1 + 0) = a_0_1_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_0_0_8_1 + 0) = a_0_1_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_0_0_9_1 + 0) = a_0_1_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_0_0_10_1 + 0) = a_0_1_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_0_0_11_1 + 0) = a_0_1_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_0_0_12_1 + 0) = a_0_1_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_0_0_13_1 + 0) = a_0_1_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_0_0_14_1 + 0) = a_0_1_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
    *(localWriteA_0_0_15_1 + 0) = a_0_1_15_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_16x2


    /* iter 15 */

    MAC_16x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);
  a_0_0_4_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_1_4_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 1);
  a_0_0_5_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_1_5_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 1);
  a_0_0_6_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_1_6_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 1);
  a_0_0_7_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_1_7_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 1);
  a_0_0_8_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_0_1_8_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 1);
  a_0_0_9_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_0_1_9_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 1);
  a_0_0_10_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_0_1_10_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 1);
  a_0_0_11_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_0_1_11_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 1);
  a_0_0_12_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_0_1_12_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 1);
  a_0_0_13_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_0_1_13_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 1);
  a_0_0_14_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_0_1_14_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 1);
  a_0_0_15_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);
  a_0_1_15_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_1);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_1);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_4_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_1);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_5_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_1);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_6_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_1);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_0_0_7_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_1);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_0_0_8_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_1);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_0_0_9_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_1);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_0_0_10_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_1);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_0_0_11_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_1);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_0_0_12_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_1);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_0_0_13_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_1);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_0_0_14_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_1);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);
  localWriteA_0_0_15_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_1 + 0) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_1 + 0) = a_0_1_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_4_1 + 0) = a_0_1_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_5_1 + 0) = a_0_1_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_6_1 + 0) = a_0_1_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_7_1 + 0) = a_0_1_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_0_0_8_1 + 0) = a_0_1_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_0_0_9_1 + 0) = a_0_1_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_0_0_10_1 + 0) = a_0_1_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_0_0_11_1 + 0) = a_0_1_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_0_0_12_1 + 0) = a_0_1_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_0_0_13_1 + 0) = a_0_1_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_0_0_14_1 + 0) = a_0_1_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
  *(localWriteA_0_0_15_1 + 0) = a_0_1_15_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_16x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_16x2
#undef MAC_16x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_BSS_BH_MT128x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_bfloat16)(0)
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;
  unsigned int flattenedOffsetA_4_0 = globalReadOffsetA0I + 0 + 4*LSPA;
  flattenedOffsetA_4_0 = (flattenedOffsetA_4_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_4_0;
  unsigned int globalReadOffsetA0I_4_0 = flattenedOffsetA_4_0;
  unsigned int flattenedOffsetA_5_0 = globalReadOffsetA0I + 0 + 5*LSPA;
  flattenedOffsetA_5_0 = (flattenedOffsetA_5_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_5_0;
  unsigned int globalReadOffsetA0I_5_0 = flattenedOffsetA_5_0;
  unsigned int flattenedOffsetA_6_0 = globalReadOffsetA0I + 0 + 6*LSPA;
  flattenedOffsetA_6_0 = (flattenedOffsetA_6_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_6_0;
  unsigned int globalReadOffsetA0I_6_0 = flattenedOffsetA_6_0;
  unsigned int flattenedOffsetA_7_0 = globalReadOffsetA0I + 0 + 7*LSPA;
  flattenedOffsetA_7_0 = (flattenedOffsetA_7_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_7_0;
  unsigned int globalReadOffsetA0I_7_0 = flattenedOffsetA_7_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 4*LSPA);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 5*LSPA);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 6*LSPA);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 7*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_BSS_BH_MT128x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_bfloat16)(0)
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;
  unsigned int flattenedOffsetA_4_0 = globalReadOffsetA0I + 0 + 4*LSPA;
  flattenedOffsetA_4_0 = (flattenedOffsetA_4_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_4_0;
  unsigned int globalReadOffsetA0I_4_0 = flattenedOffsetA_4_0;
  unsigned int flattenedOffsetA_5_0 = globalReadOffsetA0I + 0 + 5*LSPA;
  flattenedOffsetA_5_0 = (flattenedOffsetA_5_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_5_0;
  unsigned int globalReadOffsetA0I_5_0 = flattenedOffsetA_5_0;
  unsigned int flattenedOffsetA_6_0 = globalReadOffsetA0I + 0 + 6*LSPA;
  flattenedOffsetA_6_0 = (flattenedOffsetA_6_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_6_0;
  unsigned int globalReadOffsetA0I_6_0 = flattenedOffsetA_6_0;
  unsigned int flattenedOffsetA_7_0 = globalReadOffsetA0I + 0 + 7*LSPA;
  flattenedOffsetA_7_0 = (flattenedOffsetA_7_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_7_0;
  unsigned int globalReadOffsetA0I_7_0 = flattenedOffsetA_7_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 4*LSPA);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 5*LSPA);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 6*LSPA);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 7*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 16
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 32

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 2304

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_bfloat16
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B);
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 2x16 micro-tile */
#define MAC_2x16\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[0],rB[8],rC[0+8*TT0I]); \
  TYPE_MAC(rA[1],rB[8],rC[1+8*TT0I]); \
  TYPE_MAC(rA[0],rB[9],rC[0+9*TT0I]); \
  TYPE_MAC(rA[1],rB[9],rC[1+9*TT0I]); \
  TYPE_MAC(rA[0],rB[10],rC[0+10*TT0I]); \
  TYPE_MAC(rA[1],rB[10],rC[1+10*TT0I]); \
  TYPE_MAC(rA[0],rB[11],rC[0+11*TT0I]); \
  TYPE_MAC(rA[1],rB[11],rC[1+11*TT0I]); \
  TYPE_MAC(rA[0],rB[12],rC[0+12*TT0I]); \
  TYPE_MAC(rA[1],rB[12],rC[1+12*TT0I]); \
  TYPE_MAC(rA[0],rB[13],rC[0+13*TT0I]); \
  TYPE_MAC(rA[1],rB[13],rC[1+13*TT0I]); \
  TYPE_MAC(rA[0],rB[14],rC[0+14*TT0I]); \
  TYPE_MAC(rA[1],rB[14],rC[1+14*TT0I]); \
  TYPE_MAC(rA[0],rB[15],rC[0+15*TT0I]); \
  TYPE_MAC(rA[1],rB[15],rC[1+15*TT0I]); \

#define MAC_2x16_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[8+TT1J],rC[0+8*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[8+TT1J],rC[1+8*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[9+TT1J],rC[0+9*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[9+TT1J],rC[1+9*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[10+TT1J],rC[0+10*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[10+TT1J],rC[1+10*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[11+TT1J],rC[0+11*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[11+TT1J],rC[1+11*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[12+TT1J],rC[0+12*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[12+TT1J],rC[1+12*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[13+TT1J],rC[0+13*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[13+TT1J],rC[1+13*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[14+TT1J],rC[0+14*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[14+TT1J],rC[1+14*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[15+TT1J],rC[0+15*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[15+TT1J],rC[1+15*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_BSS_BH_MT16x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_16_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_bfloat16 const * __restrict__ A,
  tensile_bfloat16 const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_bfloat16)(0)
#define SCALAR_OOB_DATA static_cast<tensile_bfloat16>(0.0f)
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;
  DATA_TYPE b_0_0_8_0;
  DATA_TYPE b_0_0_9_0;
  DATA_TYPE b_0_0_10_0;
  DATA_TYPE b_0_0_11_0;
  DATA_TYPE b_0_0_12_0;
  DATA_TYPE b_0_0_13_0;
  DATA_TYPE b_0_0_14_0;
  DATA_TYPE b_0_0_15_0;
  DATA_TYPE b_0_0_16_0;
  DATA_TYPE b_0_0_17_0;
  DATA_TYPE b_0_0_18_0;
  DATA_TYPE b_0_0_19_0;
  DATA_TYPE b_0_0_20_0;
  DATA_TYPE b_0_0_21_0;
  DATA_TYPE b_0_0_22_0;
  DATA_TYPE b_0_0_23_0;
  DATA_TYPE b_0_0_24_0;
  DATA_TYPE b_0_0_25_0;
  DATA_TYPE b_0_0_26_0;
  DATA_TYPE b_0_0_27_0;
  DATA_TYPE b_0_0_28_0;
  DATA_TYPE b_0_0_29_0;
  DATA_TYPE b_0_0_30_0;
  DATA_TYPE b_0_0_31_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;
  unsigned int flattenedOffsetB_4_0 = globalReadOffsetB1J + 0 + 4*LSPB;
  flattenedOffsetB_4_0 = (flattenedOffsetB_4_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_4_0;
  unsigned int globalReadOffsetB1J_4_0 = flattenedOffsetB_4_0;
  unsigned int flattenedOffsetB_5_0 = globalReadOffsetB1J + 0 + 5*LSPB;
  flattenedOffsetB_5_0 = (flattenedOffsetB_5_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_5_0;
  unsigned int globalReadOffsetB1J_5_0 = flattenedOffsetB_5_0;
  unsigned int flattenedOffsetB_6_0 = globalReadOffsetB1J + 0 + 6*LSPB;
  flattenedOffsetB_6_0 = (flattenedOffsetB_6_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_6_0;
  unsigned int globalReadOffsetB1J_6_0 = flattenedOffsetB_6_0;
  unsigned int flattenedOffsetB_7_0 = globalReadOffsetB1J + 0 + 7*LSPB;
  flattenedOffsetB_7_0 = (flattenedOffsetB_7_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_7_0;
  unsigned int globalReadOffsetB1J_7_0 = flattenedOffsetB_7_0;
  unsigned int flattenedOffsetB_8_0 = globalReadOffsetB1J + 0 + 8*LSPB;
  flattenedOffsetB_8_0 = (flattenedOffsetB_8_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_8_0;
  unsigned int globalReadOffsetB1J_8_0 = flattenedOffsetB_8_0;
  unsigned int flattenedOffsetB_9_0 = globalReadOffsetB1J + 0 + 9*LSPB;
  flattenedOffsetB_9_0 = (flattenedOffsetB_9_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_9_0;
  unsigned int globalReadOffsetB1J_9_0 = flattenedOffsetB_9_0;
  unsigned int flattenedOffsetB_10_0 = globalReadOffsetB1J + 0 + 10*LSPB;
  flattenedOffsetB_10_0 = (flattenedOffsetB_10_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_10_0;
  unsigned int globalReadOffsetB1J_10_0 = flattenedOffsetB_10_0;
  unsigned int flattenedOffsetB_11_0 = globalReadOffsetB1J + 0 + 11*LSPB;
  flattenedOffsetB_11_0 = (flattenedOffsetB_11_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_11_0;
  unsigned int globalReadOffsetB1J_11_0 = flattenedOffsetB_11_0;
  unsigned int flattenedOffsetB_12_0 = globalReadOffsetB1J + 0 + 12*LSPB;
  flattenedOffsetB_12_0 = (flattenedOffsetB_12_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_12_0;
  unsigned int globalReadOffsetB1J_12_0 = flattenedOffsetB_12_0;
  unsigned int flattenedOffsetB_13_0 = globalReadOffsetB1J + 0 + 13*LSPB;
  flattenedOffsetB_13_0 = (flattenedOffsetB_13_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_13_0;
  unsigned int globalReadOffsetB1J_13_0 = flattenedOffsetB_13_0;
  unsigned int flattenedOffsetB_14_0 = globalReadOffsetB1J + 0 + 14*LSPB;
  flattenedOffsetB_14_0 = (flattenedOffsetB_14_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_14_0;
  unsigned int globalReadOffsetB1J_14_0 = flattenedOffsetB_14_0;
  unsigned int flattenedOffsetB_15_0 = globalReadOffsetB1J + 0 + 15*LSPB;
  flattenedOffsetB_15_0 = (flattenedOffsetB_15_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_15_0;
  unsigned int globalReadOffsetB1J_15_0 = flattenedOffsetB_15_0;
  unsigned int flattenedOffsetB_16_0 = globalReadOffsetB1J + 0 + 16*LSPB;
  flattenedOffsetB_16_0 = (flattenedOffsetB_16_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_16_0;
  unsigned int globalReadOffsetB1J_16_0 = flattenedOffsetB_16_0;
  unsigned int flattenedOffsetB_17_0 = globalReadOffsetB1J + 0 + 17*LSPB;
  flattenedOffsetB_17_0 = (flattenedOffsetB_17_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_17_0;
  unsigned int globalReadOffsetB1J_17_0 = flattenedOffsetB_17_0;
  unsigned int flattenedOffsetB_18_0 = globalReadOffsetB1J + 0 + 18*LSPB;
  flattenedOffsetB_18_0 = (flattenedOffsetB_18_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_18_0;
  unsigned int globalReadOffsetB1J_18_0 = flattenedOffsetB_18_0;
  unsigned int flattenedOffsetB_19_0 = globalReadOffsetB1J + 0 + 19*LSPB;
  flattenedOffsetB_19_0 = (flattenedOffsetB_19_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_19_0;
  unsigned int globalReadOffsetB1J_19_0 = flattenedOffsetB_19_0;
  unsigned int flattenedOffsetB_20_0 = globalReadOffsetB1J + 0 + 20*LSPB;
  flattenedOffsetB_20_0 = (flattenedOffsetB_20_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_20_0;
  unsigned int globalReadOffsetB1J_20_0 = flattenedOffsetB_20_0;
  unsigned int flattenedOffsetB_21_0 = globalReadOffsetB1J + 0 + 21*LSPB;
  flattenedOffsetB_21_0 = (flattenedOffsetB_21_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_21_0;
  unsigned int globalReadOffsetB1J_21_0 = flattenedOffsetB_21_0;
  unsigned int flattenedOffsetB_22_0 = globalReadOffsetB1J + 0 + 22*LSPB;
  flattenedOffsetB_22_0 = (flattenedOffsetB_22_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_22_0;
  unsigned int globalReadOffsetB1J_22_0 = flattenedOffsetB_22_0;
  unsigned int flattenedOffsetB_23_0 = globalReadOffsetB1J + 0 + 23*LSPB;
  flattenedOffsetB_23_0 = (flattenedOffsetB_23_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_23_0;
  unsigned int globalReadOffsetB1J_23_0 = flattenedOffsetB_23_0;
  unsigned int flattenedOffsetB_24_0 = globalReadOffsetB1J + 0 + 24*LSPB;
  flattenedOffsetB_24_0 = (flattenedOffsetB_24_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_24_0;
  unsigned int globalReadOffsetB1J_24_0 = flattenedOffsetB_24_0;
  unsigned int flattenedOffsetB_25_0 = globalReadOffsetB1J + 0 + 25*LSPB;
  flattenedOffsetB_25_0 = (flattenedOffsetB_25_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_25_0;
  unsigned int globalReadOffsetB1J_25_0 = flattenedOffsetB_25_0;
  unsigned int flattenedOffsetB_26_0 = globalReadOffsetB1J + 0 + 26*LSPB;
  flattenedOffsetB_26_0 = (flattenedOffsetB_26_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_26_0;
  unsigned int globalReadOffsetB1J_26_0 = flattenedOffsetB_26_0;
  unsigned int flattenedOffsetB_27_0 = globalReadOffsetB1J + 0 + 27*LSPB;
  flattenedOffsetB_27_0 = (flattenedOffsetB_27_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_27_0;
  unsigned int globalReadOffsetB1J_27_0 = flattenedOffsetB_27_0;
  unsigned int flattenedOffsetB_28_0 = globalReadOffsetB1J + 0 + 28*LSPB;
  flattenedOffsetB_28_0 = (flattenedOffsetB_28_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_28_0;
  unsigned int globalReadOffsetB1J_28_0 = flattenedOffsetB_28_0;
  unsigned int flattenedOffsetB_29_0 = globalReadOffsetB1J + 0 + 29*LSPB;
  flattenedOffsetB_29_0 = (flattenedOffsetB_29_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_29_0;
  unsigned int globalReadOffsetB1J_29_0 = flattenedOffsetB_29_0;
  unsigned int flattenedOffsetB_30_0 = globalReadOffsetB1J + 0 + 30*LSPB;
  flattenedOffsetB_30_0 = (flattenedOffsetB_30_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_30_0;
  unsigned int globalReadOffsetB1J_30_0 = flattenedOffsetB_30_0;
  unsigned int flattenedOffsetB_31_0 = globalReadOffsetB1J + 0 + 31*LSPB;
  flattenedOffsetB_31_0 = (flattenedOffsetB_31_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_31_0;
  unsigned int globalReadOffsetB1J_31_0 = flattenedOffsetB_31_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_7_0), wgK );
  int64_t globalReadOffsetB_0_0_8_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_8_0), wgK );
  int64_t globalReadOffsetB_0_0_9_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_9_0), wgK );
  int64_t globalReadOffsetB_0_0_10_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_10_0), wgK );
  int64_t globalReadOffsetB_0_0_11_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_11_0), wgK );
  int64_t globalReadOffsetB_0_0_12_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_12_0), wgK );
  int64_t globalReadOffsetB_0_0_13_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_13_0), wgK );
  int64_t globalReadOffsetB_0_0_14_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_14_0), wgK );
  int64_t globalReadOffsetB_0_0_15_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_15_0), wgK );
  int64_t globalReadOffsetB_0_0_16_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_16_0), wgK );
  int64_t globalReadOffsetB_0_0_17_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_17_0), wgK );
  int64_t globalReadOffsetB_0_0_18_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_18_0), wgK );
  int64_t globalReadOffsetB_0_0_19_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_19_0), wgK );
  int64_t globalReadOffsetB_0_0_20_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_20_0), wgK );
  int64_t globalReadOffsetB_0_0_21_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_21_0), wgK );
  int64_t globalReadOffsetB_0_0_22_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_22_0), wgK );
  int64_t globalReadOffsetB_0_0_23_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_23_0), wgK );
  int64_t globalReadOffsetB_0_0_24_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_24_0), wgK );
  int64_t globalReadOffsetB_0_0_25_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_25_0), wgK );
  int64_t globalReadOffsetB_0_0_26_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_26_0), wgK );
  int64_t globalReadOffsetB_0_0_27_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_27_0), wgK );
  int64_t globalReadOffsetB_0_0_28_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_28_0), wgK );
  int64_t globalReadOffsetB_0_0_29_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_29_0), wgK );
  int64_t globalReadOffsetB_0_0_30_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_30_0), wgK );
  int64_t globalReadOffsetB_0_0_31_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_31_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;
  DATA_TYPE const *globalReadB_0_0_8_0 = B + globalReadOffsetB_0_0_8_0;
  DATA_TYPE const *globalReadB_0_0_9_0 = B + globalReadOffsetB_0_0_9_0;
  DATA_TYPE const *globalReadB_0_0_10_0 = B + globalReadOffsetB_0_0_10_0;
  DATA_TYPE const *globalReadB_0_0_11_0 = B + globalReadOffsetB_0_0_11_0;
  DATA_TYPE const *globalReadB_0_0_12_0 = B + globalReadOffsetB_0_0_12_0;
  DATA_TYPE const *globalReadB_0_0_13_0 = B + globalReadOffsetB_0_0_13_0;
  DATA_TYPE const *globalReadB_0_0_14_0 = B + globalReadOffsetB_0_0_14_0;
  DATA_TYPE const *globalReadB_0_0_15_0 = B + globalReadOffsetB_0_0_15_0;
  DATA_TYPE const *globalReadB_0_0_16_0 = B + globalReadOffsetB_0_0_16_0;
  DATA_TYPE const *globalReadB_0_0_17_0 = B + globalReadOffsetB_0_0_17_0;
  DATA_TYPE const *globalReadB_0_0_18_0 = B + globalReadOffsetB_0_0_18_0;
  DATA_TYPE const *globalReadB_0_0_19_0 = B + globalReadOffsetB_0_0_19_0;
  DATA_TYPE const *globalReadB_0_0_20_0 = B + globalReadOffsetB_0_0_20_0;
  DATA_TYPE const *globalReadB_0_0_21_0 = B + globalReadOffsetB_0_0_21_0;
  DATA_TYPE const *globalReadB_0_0_22_0 = B + globalReadOffsetB_0_0_22_0;
  DATA_TYPE const *globalReadB_0_0_23_0 = B + globalReadOffsetB_0_0_23_0;
  DATA_TYPE const *globalReadB_0_0_24_0 = B + globalReadOffsetB_0_0_24_0;
  DATA_TYPE const *globalReadB_0_0_25_0 = B + globalReadOffsetB_0_0_25_0;
  DATA_TYPE const *globalReadB_0_0_26_0 = B + globalReadOffsetB_0_0_26_0;
  DATA_TYPE const *globalReadB_0_0_27_0 = B + globalReadOffsetB_0_0_27_0;
  DATA_TYPE const *globalReadB_0_0_28_0 = B + globalReadOffsetB_0_0_28_0;
  DATA_TYPE const *globalReadB_0_0_29_0 = B + globalReadOffsetB_0_0_29_0;
  DATA_TYPE const *globalReadB_0_0_30_0 = B + globalReadOffsetB_0_0_30_0;
  DATA_TYPE const *globalReadB_0_0_31_0 = B + globalReadOffsetB_0_0_31_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 4*LSPB);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 5*LSPB);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 6*LSPB);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 7*LSPB);
  unsigned int localWriteOffsetB_0_0_8_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 8*LSPB);
  unsigned int localWriteOffsetB_0_0_9_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 9*LSPB);
  unsigned int localWriteOffsetB_0_0_10_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 10*LSPB);
  unsigned int localWriteOffsetB_0_0_11_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 11*LSPB);
  unsigned int localWriteOffsetB_0_0_12_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 12*LSPB);
  unsigned int localWriteOffsetB_0_0_13_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 13*LSPB);
  unsigned int localWriteOffsetB_0_0_14_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 14*LSPB);
  unsigned int localWriteOffsetB_0_0_15_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 15*LSPB);
  unsigned int localWriteOffsetB_0_0_16_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 16*LSPB);
  unsigned int localWriteOffsetB_0_0_17_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 17*LSPB);
  unsigned int localWriteOffsetB_0_0_18_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 18*LSPB);
  unsigned int localWriteOffsetB_0_0_19_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 19*LSPB);
  unsigned int localWriteOffsetB_0_0_20_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 20*LSPB);
  unsigned int localWriteOffsetB_0_0_21_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 21*LSPB);
  unsigned int localWriteOffsetB_0_0_22_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 22*LSPB);
  unsigned int localWriteOffsetB_0_0_23_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 23*LSPB);
  unsigned int localWriteOffsetB_0_0_24_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 24*LSPB);
  unsigned int localWriteOffsetB_0_0_25_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 25*LSPB);
  unsigned int localWriteOffsetB_0_0_26_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 26*LSPB);
  unsigned int localWriteOffsetB_0_0_27_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 27*LSPB);
  unsigned int localWriteOffsetB_0_0_28_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 28*LSPB);
  unsigned int localWriteOffsetB_0_0_29_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 29*LSPB);
  unsigned int localWriteOffsetB_0_0_30_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 30*LSPB);
  unsigned int localWriteOffsetB_0_0_31_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 31*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;
  DATA_TYPE *localWriteB_0_0_8_0;
  DATA_TYPE *localWriteB_0_0_9_0;
  DATA_TYPE *localWriteB_0_0_10_0;
  DATA_TYPE *localWriteB_0_0_11_0;
  DATA_TYPE *localWriteB_0_0_12_0;
  DATA_TYPE *localWriteB_0_0_13_0;
  DATA_TYPE *localWriteB_0_0_14_0;
  DATA_TYPE *localWriteB_0_0_15_0;
  DATA_TYPE *localWriteB_0_0_16_0;
  DATA_TYPE *localWriteB_0_0_17_0;
  DATA_TYPE *localWriteB_0_0_18_0;
  DATA_TYPE *localWriteB_0_0_19_0;
  DATA_TYPE *localWriteB_0_0_20_0;
  DATA_TYPE *localWriteB_0_0_21_0;
  DATA_TYPE *localWriteB_0_0_22_0;
  DATA_TYPE *localWriteB_0_0_23_0;
  DATA_TYPE *localWriteB_0_0_24_0;
  DATA_TYPE *localWriteB_0_0_25_0;
  DATA_TYPE *localWriteB_0_0_26_0;
  DATA_TYPE *localWriteB_0_0_27_0;
  DATA_TYPE *localWriteB_0_0_28_0;
  DATA_TYPE *localWriteB_0_0_29_0;
  DATA_TYPE *localWriteB_0_0_30_0;
  DATA_TYPE *localWriteB_0_0_31_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);
  localWriteB_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_8_0);
  localWriteB_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_9_0);
  localWriteB_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_10_0);
  localWriteB_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_11_0);
  localWriteB_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_12_0);
  localWriteB_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_13_0);
  localWriteB_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_14_0);
  localWriteB_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_15_0);
  localWriteB_0_0_16_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_16_0);
  localWriteB_0_0_17_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_17_0);
  localWriteB_0_0_18_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_18_0);
  localWriteB_0_0_19_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_19_0);
  localWriteB_0_0_20_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_20_0);
  localWriteB_0_0_21_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_21_0);
  localWriteB_0_0_22_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_22_0);
  localWriteB_0_0_23_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_23_0);
  localWriteB_0_0_24_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_24_0);
  localWriteB_0_0_25_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_25_0);
  localWriteB_0_0_26_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_26_0);
  localWriteB_0_0_27_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_27_0);
  localWriteB_0_0_28_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_28_0);
  localWriteB_0_0_29_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_29_0);
  localWriteB_0_0_30_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_30_0);
  localWriteB_0_0_31_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_31_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_8_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_9_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_10_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_11_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_12_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_13_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_14_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_15_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_16_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_17_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_18_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_19_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_20_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_21_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_22_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_23_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_24_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_25_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_26_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_27_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_28_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_29_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_30_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_31_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);
    b_0_0_8_0 = *(globalReadB_0_0_8_0 + 0);
    b_0_0_9_0 = *(globalReadB_0_0_9_0 + 0);
    b_0_0_10_0 = *(globalReadB_0_0_10_0 + 0);
    b_0_0_11_0 = *(globalReadB_0_0_11_0 + 0);
    b_0_0_12_0 = *(globalReadB_0_0_12_0 + 0);
    b_0_0_13_0 = *(globalReadB_0_0_13_0 + 0);
    b_0_0_14_0 = *(globalReadB_0_0_14_0 + 0);
    b_0_0_15_0 = *(globalReadB_0_0_15_0 + 0);
    b_0_0_16_0 = *(globalReadB_0_0_16_0 + 0);
    b_0_0_17_0 = *(globalReadB_0_0_17_0 + 0);
    b_0_0_18_0 = *(globalReadB_0_0_18_0 + 0);
    b_0_0_19_0 = *(globalReadB_0_0_19_0 + 0);
    b_0_0_20_0 = *(globalReadB_0_0_20_0 + 0);
    b_0_0_21_0 = *(globalReadB_0_0_21_0 + 0);
    b_0_0_22_0 = *(globalReadB_0_0_22_0 + 0);
    b_0_0_23_0 = *(globalReadB_0_0_23_0 + 0);
    b_0_0_24_0 = *(globalReadB_0_0_24_0 + 0);
    b_0_0_25_0 = *(globalReadB_0_0_25_0 + 0);
    b_0_0_26_0 = *(globalReadB_0_0_26_0 + 0);
    b_0_0_27_0 = *(globalReadB_0_0_27_0 + 0);
    b_0_0_28_0 = *(globalReadB_0_0_28_0 + 0);
    b_0_0_29_0 = *(globalReadB_0_0_29_0 + 0);
    b_0_0_30_0 = *(globalReadB_0_0_30_0 + 0);
    b_0_0_31_0 = *(globalReadB_0_0_31_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_8_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_8_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_9_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_9_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_10_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_10_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_11_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_11_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_12_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_12_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_13_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_13_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_14_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_14_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_15_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_15_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_16_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_16_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_16_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_17_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_17_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_17_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_18_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_18_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_18_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_19_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_19_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_19_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_20_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_20_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_20_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_21_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_21_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_21_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_22_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_22_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_22_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_23_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_23_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_23_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_24_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_24_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_24_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_25_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_25_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_25_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_26_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_26_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_26_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_27_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_27_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_27_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_28_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_28_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_28_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_29_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_29_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_29_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_30_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_30_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_30_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_31_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_31_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_31_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
    *(localWriteB_0_0_8_0 + 0) = b_0_0_8_0;
    *(localWriteB_0_0_9_0 + 0) = b_0_0_9_0;
    *(localWriteB_0_0_10_0 + 0) = b_0_0_10_0;
    *(localWriteB_0_0_11_0 + 0) = b_0_0_11_0;
    *(localWriteB_0_0_12_0 + 0) = b_0_0_12_0;
    *(localWriteB_0_0_13_0 + 0) = b_0_0_13_0;
    *(localWriteB_0_0_14_0 + 0) = b_0_0_14_0;
    *(localWriteB_0_0_15_0 + 0) = b_0_0_15_0;
    *(localWriteB_0_0_16_0 + 0) = b_0_0_16_0;
    *(localWriteB_0_0_17_0 + 0) = b_0_0_17_0;
    *(localWriteB_0_0_18_0 + 0) = b_0_0_18_0;
    *(localWriteB_0_0_19_0 + 0) = b_0_0_19_0;
    *(localWriteB_0_0_20_0 + 0) = b_0_0_20_0;
    *(localWriteB_0_0_21_0 + 0) = b_0_0_21_0;
    *(localWriteB_0_0_22_0 + 0) = b_0_0_22_0;
    *(localWriteB_0_0_23_0 + 0) = b_0_0_23_0;
    *(localWriteB_0_0_24_0 + 0) = b_0_0_24_0;
    *(localWriteB_0_0_25_0 + 0) = b_0_0_25_0;
    *(localWriteB_0_0_26_0 + 0) = b_0_0_26_0;
    *(localWriteB_0_0_27_0 + 0) = b_0_0_27_0;
    *(localWriteB_0_0_28_0 + 0) = b_0_0_28_0;
    *(localWriteB_0_0_29_0 + 0) = b_0_0_29_0;
    *(localWriteB_0_0_30_0 + 0) = b_0_0_30_0;
    *(localWriteB_0_0_31_0 + 0) = b_0_0_31_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[8*VECTOR_WIDTH+0] = localReadB[8*SG1J*VECTOR_WIDTH + 0]; 
    rB[9*VECTOR_WIDTH+0] = localReadB[9*SG1J*VECTOR_WIDTH + 0]; 
    rB[10*VECTOR_WIDTH+0] = localReadB[10*SG1J*VECTOR_WIDTH + 0]; 
    rB[11*VECTOR_WIDTH+0] = localReadB[11*SG1J*VECTOR_WIDTH + 0]; 
    rB[12*VECTOR_WIDTH+0] = localReadB[12*SG1J*VECTOR_WIDTH + 0]; 
    rB[13*VECTOR_WIDTH+0] = localReadB[13*SG1J*VECTOR_WIDTH + 0]; 
    rB[14*VECTOR_WIDTH+0] = localReadB[14*SG1J*VECTOR_WIDTH + 0]; 
    rB[15*VECTOR_WIDTH+0] = localReadB[15*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[8*VECTOR_WIDTH+0+TT1J] = localReadB[8*SG1J*VECTOR_WIDTH + 0]; 
    rB[9*VECTOR_WIDTH+0+TT1J] = localReadB[9*SG1J*VECTOR_WIDTH + 0]; 
    rB[10*VECTOR_WIDTH+0+TT1J] = localReadB[10*SG1J*VECTOR_WIDTH + 0]; 
    rB[11*VECTOR_WIDTH+0+TT1J] = localReadB[11*SG1J*VECTOR_WIDTH + 0]; 
    rB[12*VECTOR_WIDTH+0+TT1J] = localReadB[12*SG1J*VECTOR_WIDTH + 0]; 
    rB[13*VECTOR_WIDTH+0+TT1J] = localReadB[13*SG1J*VECTOR_WIDTH + 0]; 
    rB[14*VECTOR_WIDTH+0+TT1J] = localReadB[14*SG1J*VECTOR_WIDTH + 0]; 
    rB[15*VECTOR_WIDTH+0+TT1J] = localReadB[15*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x16


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[8*VECTOR_WIDTH+0] = localReadB[8*SG1J*VECTOR_WIDTH + 0]; 
    rB[9*VECTOR_WIDTH+0] = localReadB[9*SG1J*VECTOR_WIDTH + 0]; 
    rB[10*VECTOR_WIDTH+0] = localReadB[10*SG1J*VECTOR_WIDTH + 0]; 
    rB[11*VECTOR_WIDTH+0] = localReadB[11*SG1J*VECTOR_WIDTH + 0]; 
    rB[12*VECTOR_WIDTH+0] = localReadB[12*SG1J*VECTOR_WIDTH + 0]; 
    rB[13*VECTOR_WIDTH+0] = localReadB[13*SG1J*VECTOR_WIDTH + 0]; 
    rB[14*VECTOR_WIDTH+0] = localReadB[14*SG1J*VECTOR_WIDTH + 0]; 
    rB[15*VECTOR_WIDTH+0] = localReadB[15*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x16_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[8*VECTOR_WIDTH+0+TT1J] = localReadB[8*SG1J*VECTOR_WIDTH + 0]; 
    rB[9*VECTOR_WIDTH+0+TT1J] = localReadB[9*SG1J*VECTOR_WIDTH + 0]; 
    rB[10*VECTOR_WIDTH+0+TT1J] = localReadB[10*SG1J*VECTOR_WIDTH + 0]; 
    rB[11*VECTOR_WIDTH+0+TT1J] = localReadB[11*SG1J*VECTOR_WIDTH + 0]; 
    rB[12*VECTOR_WIDTH+0+TT1J] = localReadB[12*SG1J*VECTOR_WIDTH + 0]; 
    rB[13*VECTOR_WIDTH+0+TT1J] = localReadB[13*SG1J*VECTOR_WIDTH + 0]; 
    rB[14*VECTOR_WIDTH+0+TT1J] = localReadB[14*SG1J*VECTOR_WIDTH + 0]; 
    rB[15*VECTOR_WIDTH+0+TT1J] = localReadB[15*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x16


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[8*VECTOR_WIDTH+0] = localReadB[8*SG1J*VECTOR_WIDTH + 0]; 
    rB[9*VECTOR_WIDTH+0] = localReadB[9*SG1J*VECTOR_WIDTH + 0]; 
    rB[10*VECTOR_WIDTH+0] = localReadB[10*SG1J*VECTOR_WIDTH + 0]; 
    rB[11*VECTOR_WIDTH+0] = localReadB[11*SG1J*VECTOR_WIDTH + 0]; 
    rB[12*VECTOR_WIDTH+0] = localReadB[12*SG1J*VECTOR_WIDTH + 0]; 
    rB[13*VECTOR_WIDTH+0] = localReadB[13*SG1J*VECTOR_WIDTH + 0]; 
    rB[14*VECTOR_WIDTH+0] = localReadB[14*SG1J*VECTOR_WIDTH + 0]; 
    rB[15*VECTOR_WIDTH+0] = localReadB[15*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x16_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[8*VECTOR_WIDTH+0+TT1J] = localReadB[8*SG1J*VECTOR_WIDTH + 0]; 
    rB[9*VECTOR_WIDTH+0+TT1J] = localReadB[9*SG1J*VECTOR_WIDTH + 0]; 
    rB[10*VECTOR_WIDTH+0+TT1J] = localReadB[10*SG1J*VECTOR_WIDTH + 0]; 
    rB[11*VECTOR_WIDTH+0+TT1J] = localReadB[11*SG1J*VECTOR_WIDTH + 0]; 
    rB[12*VECTOR_WIDTH+0+TT1J] = localReadB[12*SG1J*VECTOR_WIDTH + 0]; 
    rB[13*VECTOR_WIDTH+0+TT1J] = localReadB[13*SG1J*VECTOR_WIDTH + 0]; 
    rB[14*VECTOR_WIDTH+0+TT1J] = localReadB[14*SG1J*VECTOR_WIDTH + 0]; 
    rB[15*VECTOR_WIDTH+0+TT1J] = localReadB[15*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x16


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[8*VECTOR_WIDTH+0] = localReadB[8*SG1J*VECTOR_WIDTH + 0]; 
    rB[9*VECTOR_WIDTH+0] = localReadB[9*SG1J*VECTOR_WIDTH + 0]; 
    rB[10*VECTOR_WIDTH+0] = localReadB[10*SG1J*VECTOR_WIDTH + 0]; 
    rB[11*VECTOR_WIDTH+0] = localReadB[11*SG1J*VECTOR_WIDTH + 0]; 
    rB[12*VECTOR_WIDTH+0] = localReadB[12*SG1J*VECTOR_WIDTH + 0]; 
    rB[13*VECTOR_WIDTH+0] = localReadB[13*SG1J*VECTOR_WIDTH + 0]; 
    rB[14*VECTOR_WIDTH+0] = localReadB[14*SG1J*VECTOR_WIDTH + 0]; 
    rB[15*VECTOR_WIDTH+0] = localReadB[15*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x16_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[8*VECTOR_WIDTH+0+TT1J] = localReadB[8*SG1J*VECTOR_WIDTH + 0]; 
    rB[9*VECTOR_WIDTH+0+TT1J] = localReadB[9*SG1J*VECTOR_WIDTH + 0]; 
    rB[10*VECTOR_WIDTH+0+TT1J] = localReadB[10*SG1J*VECTOR_WIDTH + 0]; 
    rB[11*VECTOR_WIDTH+0+TT1J] = localReadB[11*SG1J*VECTOR_WIDTH + 0]; 
    rB[12*VECTOR_WIDTH+0+TT1J] = localReadB[12*SG1J*VECTOR_WIDTH + 0]; 
    rB[13*VECTOR_WIDTH+0+TT1J] = localReadB[13*SG1J*VECTOR_WIDTH + 0]; 
    rB[14*VECTOR_WIDTH+0+TT1J] = localReadB[14*SG1J*VECTOR_WIDTH + 0]; 
    rB[15*VECTOR_WIDTH+0+TT1J] = localReadB[15*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x16


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[8*VECTOR_WIDTH+0] = localReadB[8*SG1J*VECTOR_WIDTH + 0]; 
    rB[9*VECTOR_WIDTH+0] = localReadB[9*SG1J*VECTOR_WIDTH + 0]; 
    rB[10*VECTOR_WIDTH+0] = localReadB[10*SG1J*VECTOR_WIDTH + 0]; 
    rB[11*VECTOR_WIDTH+0] = localReadB[11*SG1J*VECTOR_WIDTH + 0]; 
    rB[12*VECTOR_WIDTH+0] = localReadB[12*SG1J*VECTOR_WIDTH + 0]; 
    rB[13*VECTOR_WIDTH+0] = localReadB[13*SG1J*VECTOR_WIDTH + 0]; 
    rB[14*VECTOR_WIDTH+0] = localReadB[14*SG1J*VECTOR_WIDTH + 0]; 
    rB[15*VECTOR_WIDTH+0] = localReadB[15*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x16_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[8*VECTOR_WIDTH+0+TT1J] = localReadB[8*SG1J*VECTOR_WIDTH + 0]; 
    rB[9*VECTOR_WIDTH+0+TT1J] = localReadB[9*SG1J*VECTOR_WIDTH + 0]; 
    rB[10*VECTOR_WIDTH+0+TT1J] = localReadB[10*SG1J*VECTOR_WIDTH + 0]; 
    rB[11*VECTOR_WIDTH+0+TT1J] = localReadB[11*SG1J*VECTOR_WIDTH + 0]; 
    rB[12*VECTOR_WIDTH+0+TT1J] = localReadB[12*SG1J*VECTOR_WIDTH + 0]; 
    rB[13*VECTOR_WIDTH+0+TT1J] = localReadB[13*SG1J*VECTOR_WIDTH + 0]; 
    rB[14*VECTOR_WIDTH+0+TT1J] = localReadB[14*SG1J*VECTOR_WIDTH + 0]; 
    rB[15*VECTOR_WIDTH+0+TT1J] = localReadB[15*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x16


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[8*VECTOR_WIDTH+0] = localReadB[8*SG1J*VECTOR_WIDTH + 0]; 
    rB[9*VECTOR_WIDTH+0] = localReadB[9*SG1J*VECTOR_WIDTH + 0]; 
    rB[10*VECTOR_WIDTH+0] = localReadB[10*SG1J*VECTOR_WIDTH + 0]; 
    rB[11*VECTOR_WIDTH+0] = localReadB[11*SG1J*VECTOR_WIDTH + 0]; 
    rB[12*VECTOR_WIDTH+0] = localReadB[12*SG1J*VECTOR_WIDTH + 0]; 
    rB[13*VECTOR_WIDTH+0] = localReadB[13*SG1J*VECTOR_WIDTH + 0]; 
    rB[14*VECTOR_WIDTH+0] = localReadB[14*SG1J*VECTOR_WIDTH + 0]; 
    rB[15*VECTOR_WIDTH+0] = localReadB[15*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x16_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[8*VECTOR_WIDTH+0+TT1J] = localReadB[8*SG1J*VECTOR_WIDTH + 0]; 
    rB[9*VECTOR_WIDTH+0+TT1J] = localReadB[9*SG1J*VECTOR_WIDTH + 0]; 
    rB[10*VECTOR_WIDTH+0+TT1J] = localReadB[10*SG1J*VECTOR_WIDTH + 0]; 
    rB[11*VECTOR_WIDTH+0+TT1J] = localReadB[11*SG1J*VECTOR_WIDTH + 0]; 
    rB[12*VECTOR_WIDTH+0+TT1J] = localReadB[12*SG1J*VECTOR_WIDTH + 0]; 
    rB[13*VECTOR_WIDTH+0+TT1J] = localReadB[13*SG1J*VECTOR_WIDTH + 0]; 
    rB[14*VECTOR_WIDTH+0+TT1J] = localReadB[14*SG1J*VECTOR_WIDTH + 0]; 
    rB[15*VECTOR_WIDTH+0+TT1J] = localReadB[15*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x16


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[8*VECTOR_WIDTH+0] = localReadB[8*SG1J*VECTOR_WIDTH + 0]; 
    rB[9*VECTOR_WIDTH+0] = localReadB[9*SG1J*VECTOR_WIDTH + 0]; 
    rB[10*VECTOR_WIDTH+0] = localReadB[10*SG1J*VECTOR_WIDTH + 0]; 
    rB[11*VECTOR_WIDTH+0] = localReadB[11*SG1J*VECTOR_WIDTH + 0]; 
    rB[12*VECTOR_WIDTH+0] = localReadB[12*SG1J*VECTOR_WIDTH + 0]; 
    rB[13*VECTOR_WIDTH+0] = localReadB[13*SG1J*VECTOR_WIDTH + 0]; 
    rB[14*VECTOR_WIDTH+0] = localReadB[14*SG1J*VECTOR_WIDTH + 0]; 
    rB[15*VECTOR_WIDTH+0] = localReadB[15*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x16_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[8*VECTOR_WIDTH+0+TT1J] = localReadB[8*SG1J*VECTOR_WIDTH + 0]; 
    rB[9*VECTOR_WIDTH+0+TT1J] = localReadB[9*SG1J*VECTOR_WIDTH + 0]; 
    rB[10*VECTOR_WIDTH+0+TT1J] = localReadB[10*SG1J*VECTOR_WIDTH + 0]; 
    rB[11*VECTOR_WIDTH+0+TT1J] = localReadB[11*SG1J*VECTOR_WIDTH + 0]; 
    rB[12*VECTOR_WIDTH+0+TT1J] = localReadB[12*SG1J*VECTOR_WIDTH + 0]; 
    rB[13*VECTOR_WIDTH+0+TT1J] = localReadB[13*SG1J*VECTOR_WIDTH + 0]; 
    rB[14*VECTOR_WIDTH+0+TT1J] = localReadB[14*SG1J*VECTOR_WIDTH + 0]; 
    rB[15*VECTOR_WIDTH+0+TT1J] = localReadB[15*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x16


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[8*VECTOR_WIDTH+0] = localReadB[8*SG1J*VECTOR_WIDTH + 0]; 
    rB[9*VECTOR_WIDTH+0] = localReadB[9*SG1J*VECTOR_WIDTH + 0]; 
    rB[10*VECTOR_WIDTH+0] = localReadB[10*SG1J*VECTOR_WIDTH + 0]; 
    rB[11*VECTOR_WIDTH+0] = localReadB[11*SG1J*VECTOR_WIDTH + 0]; 
    rB[12*VECTOR_WIDTH+0] = localReadB[12*SG1J*VECTOR_WIDTH + 0]; 
    rB[13*VECTOR_WIDTH+0] = localReadB[13*SG1J*VECTOR_WIDTH + 0]; 
    rB[14*VECTOR_WIDTH+0] = localReadB[14*SG1J*VECTOR_WIDTH + 0]; 
    rB[15*VECTOR_WIDTH+0] = localReadB[15*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x16_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[8*VECTOR_WIDTH+0+TT1J] = localReadB[8*SG1J*VECTOR_WIDTH + 0]; 
    rB[9*VECTOR_WIDTH+0+TT1J] = localReadB[9*SG1J*VECTOR_WIDTH + 0]; 
    rB[10*VECTOR_WIDTH+0+TT1J] = localReadB[10*SG1J*VECTOR_WIDTH + 0]; 
    rB[11*VECTOR_WIDTH+0+TT1J] = localReadB[11*SG1J*VECTOR_WIDTH + 0]; 
    rB[12*VECTOR_WIDTH+0+TT1J] = localReadB[12*SG1J*VECTOR_WIDTH + 0]; 
    rB[13*VECTOR_WIDTH+0+TT1J] = localReadB[13*SG1J*VECTOR_WIDTH + 0]; 
    rB[14*VECTOR_WIDTH+0+TT1J] = localReadB[14*SG1J*VECTOR_WIDTH + 0]; 
    rB[15*VECTOR_WIDTH+0+TT1J] = localReadB[15*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x16


    /* iter 15 */

    MAC_2x16_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_16_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_17_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_18_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_19_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_20_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_21_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_22_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_23_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_24_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_25_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_26_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_27_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_28_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_29_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_30_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_31_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);
  b_0_0_8_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_8_0 + 0);
  b_0_0_9_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_9_0 + 0);
  b_0_0_10_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_10_0 + 0);
  b_0_0_11_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_11_0 + 0);
  b_0_0_12_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_12_0 + 0);
  b_0_0_13_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_13_0 + 0);
  b_0_0_14_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_14_0 + 0);
  b_0_0_15_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_15_0 + 0);
  b_0_0_16_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_16_0 + 0);
  b_0_0_17_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_17_0 + 0);
  b_0_0_18_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_18_0 + 0);
  b_0_0_19_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_19_0 + 0);
  b_0_0_20_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_20_0 + 0);
  b_0_0_21_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_21_0 + 0);
  b_0_0_22_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_22_0 + 0);
  b_0_0_23_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_23_0 + 0);
  b_0_0_24_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_24_0 + 0);
  b_0_0_25_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_25_0 + 0);
  b_0_0_26_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_26_0 + 0);
  b_0_0_27_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_27_0 + 0);
  b_0_0_28_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_28_0 + 0);
  b_0_0_29_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_29_0 + 0);
  b_0_0_30_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_30_0 + 0);
  b_0_0_31_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_31_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);
  localWriteB_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_8_0);
  localWriteB_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_9_0);
  localWriteB_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_10_0);
  localWriteB_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_11_0);
  localWriteB_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_12_0);
  localWriteB_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_13_0);
  localWriteB_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_14_0);
  localWriteB_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_15_0);
  localWriteB_0_0_16_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_16_0);
  localWriteB_0_0_17_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_17_0);
  localWriteB_0_0_18_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_18_0);
  localWriteB_0_0_19_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_19_0);
  localWriteB_0_0_20_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_20_0);
  localWriteB_0_0_21_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_21_0);
  localWriteB_0_0_22_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_22_0);
  localWriteB_0_0_23_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_23_0);
  localWriteB_0_0_24_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_24_0);
  localWriteB_0_0_25_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_25_0);
  localWriteB_0_0_26_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_26_0);
  localWriteB_0_0_27_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_27_0);
  localWriteB_0_0_28_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_28_0);
  localWriteB_0_0_29_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_29_0);
  localWriteB_0_0_30_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_30_0);
  localWriteB_0_0_31_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_31_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
  *(localWriteB_0_0_8_0 + 0) = b_0_0_8_0;
  *(localWriteB_0_0_9_0 + 0) = b_0_0_9_0;
  *(localWriteB_0_0_10_0 + 0) = b_0_0_10_0;
  *(localWriteB_0_0_11_0 + 0) = b_0_0_11_0;
  *(localWriteB_0_0_12_0 + 0) = b_0_0_12_0;
  *(localWriteB_0_0_13_0 + 0) = b_0_0_13_0;
  *(localWriteB_0_0_14_0 + 0) = b_0_0_14_0;
  *(localWriteB_0_0_15_0 + 0) = b_0_0_15_0;
  *(localWriteB_0_0_16_0 + 0) = b_0_0_16_0;
  *(localWriteB_0_0_17_0 + 0) = b_0_0_17_0;
  *(localWriteB_0_0_18_0 + 0) = b_0_0_18_0;
  *(localWriteB_0_0_19_0 + 0) = b_0_0_19_0;
  *(localWriteB_0_0_20_0 + 0) = b_0_0_20_0;
  *(localWriteB_0_0_21_0 + 0) = b_0_0_21_0;
  *(localWriteB_0_0_22_0 + 0) = b_0_0_22_0;
  *(localWriteB_0_0_23_0 + 0) = b_0_0_23_0;
  *(localWriteB_0_0_24_0 + 0) = b_0_0_24_0;
  *(localWriteB_0_0_25_0 + 0) = b_0_0_25_0;
  *(localWriteB_0_0_26_0 + 0) = b_0_0_26_0;
  *(localWriteB_0_0_27_0 + 0) = b_0_0_27_0;
  *(localWriteB_0_0_28_0 + 0) = b_0_0_28_0;
  *(localWriteB_0_0_29_0 + 0) = b_0_0_29_0;
  *(localWriteB_0_0_30_0 + 0) = b_0_0_30_0;
  *(localWriteB_0_0_31_0 + 0) = b_0_0_31_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 
    rB[8*VECTOR_WIDTH+0] = localReadB[8*SG1J*VECTOR_WIDTH + 0]; 
    rB[9*VECTOR_WIDTH+0] = localReadB[9*SG1J*VECTOR_WIDTH + 0]; 
    rB[10*VECTOR_WIDTH+0] = localReadB[10*SG1J*VECTOR_WIDTH + 0]; 
    rB[11*VECTOR_WIDTH+0] = localReadB[11*SG1J*VECTOR_WIDTH + 0]; 
    rB[12*VECTOR_WIDTH+0] = localReadB[12*SG1J*VECTOR_WIDTH + 0]; 
    rB[13*VECTOR_WIDTH+0] = localReadB[13*SG1J*VECTOR_WIDTH + 0]; 
    rB[14*VECTOR_WIDTH+0] = localReadB[14*SG1J*VECTOR_WIDTH + 0]; 
    rB[15*VECTOR_WIDTH+0] = localReadB[15*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x16

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 8*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 8*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (8*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 8*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 8*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (8*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 9*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 9*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (9*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 9*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 9*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (9*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 10*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 10*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (10*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 10*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 10*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (10*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 11*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 11*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (11*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 11*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 11*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (11*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 12*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 12*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (12*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 12*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 12*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (12*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 13*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 13*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (13*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 13*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 13*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (13*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 14*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 14*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (14*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 14*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 14*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (14*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 15*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 15*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (15*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 15*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 15*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (15*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x16
#undef MAC_2x16_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE double
#define DEST_DATA_TYPE double
#define COMPUTE_DATA_TYPE double
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_DB_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW2_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  double *D,
  double const * __restrict__ C,
  double const * __restrict__ A,
  double const * __restrict__ B,
  double const alpha,
  double const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (double)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 7 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }


  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE double
#define DEST_DATA_TYPE double
#define COMPUTE_DATA_TYPE double
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bjlk_DB_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW2_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  double *D,
  double const * __restrict__ C,
  double const * __restrict__ A,
  double const * __restrict__ B,
  double const alpha,
  double const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (double)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 3 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 3 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 3584
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1536

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 7 */

    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1792
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 3 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x8


    /* iter 3 (last unrolled loop) */

    MAC_4x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 8

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 7168
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;
  unsigned int flattenedOffsetB_4_0 = globalReadOffsetB1J + 0 + 4*LSPB;
  flattenedOffsetB_4_0 = (flattenedOffsetB_4_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_4_0;
  unsigned int globalReadOffsetB1J_4_0 = flattenedOffsetB_4_0;
  unsigned int flattenedOffsetB_5_0 = globalReadOffsetB1J + 0 + 5*LSPB;
  flattenedOffsetB_5_0 = (flattenedOffsetB_5_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_5_0;
  unsigned int globalReadOffsetB1J_5_0 = flattenedOffsetB_5_0;
  unsigned int flattenedOffsetB_6_0 = globalReadOffsetB1J + 0 + 6*LSPB;
  flattenedOffsetB_6_0 = (flattenedOffsetB_6_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_6_0;
  unsigned int globalReadOffsetB1J_6_0 = flattenedOffsetB_6_0;
  unsigned int flattenedOffsetB_7_0 = globalReadOffsetB1J + 0 + 7*LSPB;
  flattenedOffsetB_7_0 = (flattenedOffsetB_7_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_7_0;
  unsigned int globalReadOffsetB1J_7_0 = flattenedOffsetB_7_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_7_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 4*LSPB);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 5*LSPB);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 6*LSPB);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 7*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_4_0 = (localWriteOffsetB_0_0_4_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_5_0 = (localWriteOffsetB_0_0_5_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_6_0 = (localWriteOffsetB_0_0_6_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_7_0 = (localWriteOffsetB_0_0_7_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
    localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
    localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
    localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
    localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_4_0 = (localWriteOffsetB_0_0_4_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_5_0 = (localWriteOffsetB_0_0_5_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_6_0 = (localWriteOffsetB_0_0_6_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_7_0 = (localWriteOffsetB_0_0_7_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
    localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
    localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
    localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
    localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_4_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_5_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_6_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_7_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1792
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 3584
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 7 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE TensileComplexDouble
#define DEST_DATA_TYPE TensileComplexDouble
#define COMPUTE_DATA_TYPE TensileComplexDouble
#define s0 x
#define s1 y
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) \
  DST.s0 = MAC(  MULA.s0,  MULB.s0, DST.s0 ); \
  DST.s0 = MAC( -MULA.s1, -MULB.s1, DST.s0 ); \
  DST.s1 = MAC(  MULA.s0, -MULB.s1, DST.s1 ); \
  DST.s1 = MAC(  MULA.s1,  MULB.s0, DST.s1 );
#define TYPE_MAC_WRITE( DST, SRC, ALPHA, REG, BETA ) \
  /* (1) */ \
  type_mac_tmp = REG.s0; \
  REG.s0 *= ALPHA.s0; \
  REG.s0 = MAC( -ALPHA.s1, REG.s1, REG.s0 ); \
  REG.s1 *= ALPHA.s0; \
  REG.s1 = MAC(  ALPHA.s1, type_mac_tmp, REG.s1 ); \
  /* (2) */ \
  if(BETA.s0 != 0) { \
  REG.s0 = MAC(  BETA.s0, SRC.s0, REG.s0 ); \
  REG.s1 = MAC(  BETA.s0, SRC.s1, REG.s1 ); \
  } \
  if (BETA.s1 != 0) { \
  REG.s0 = MAC( -BETA.s1, SRC.s1, REG.s0 ); \
  REG.s1 = MAC(  BETA.s1, SRC.s0, REG.s1 ); \
  } \
  /* (3) */ \
  DST = REG;

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_BjlkC_ZB_MT32x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW1_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  TensileComplexDouble *D,
  TensileComplexDouble const * __restrict__ C,
  TensileComplexDouble const * __restrict__ A,
  TensileComplexDouble const * __restrict__ B,
  TensileComplexDouble const alpha,
  TensileComplexDouble const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (TensileComplexDouble)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == tensile_complex<double>(0.0)) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 1); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }



  double type_mac_tmp;


  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE TensileComplexDouble
#define DEST_DATA_TYPE TensileComplexDouble
#define COMPUTE_DATA_TYPE TensileComplexDouble
#define s0 x
#define s1 y
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) \
  DST.s0 = MAC(  MULA.s0,  MULB.s0, DST.s0 ); \
  DST.s0 = MAC( -MULA.s1, -MULB.s1, DST.s0 ); \
  DST.s1 = MAC(  MULA.s0, -MULB.s1, DST.s1 ); \
  DST.s1 = MAC(  MULA.s1,  MULB.s0, DST.s1 );
#define TYPE_MAC_WRITE( DST, SRC, ALPHA, REG, BETA ) \
  /* (1) */ \
  type_mac_tmp = REG.s0; \
  REG.s0 *= ALPHA.s0; \
  REG.s0 = MAC( -ALPHA.s1, REG.s1, REG.s0 ); \
  REG.s1 *= ALPHA.s0; \
  REG.s1 = MAC(  ALPHA.s1, type_mac_tmp, REG.s1 ); \
  /* (2) */ \
  if(BETA.s0 != 0) { \
  REG.s0 = MAC(  BETA.s0, SRC.s0, REG.s0 ); \
  REG.s1 = MAC(  BETA.s0, SRC.s1, REG.s1 ); \
  } \
  if (BETA.s1 != 0) { \
  REG.s0 = MAC( -BETA.s1, SRC.s1, REG.s0 ); \
  REG.s1 = MAC(  BETA.s1, SRC.s0, REG.s1 ); \
  } \
  /* (3) */ \
  DST = REG;

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_BjlkC_ZB_MT32x32x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW1_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  TensileComplexDouble *D,
  TensileComplexDouble const * __restrict__ C,
  TensileComplexDouble const * __restrict__ A,
  TensileComplexDouble const * __restrict__ B,
  TensileComplexDouble const alpha,
  TensileComplexDouble const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (TensileComplexDouble)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == tensile_complex<double>(0.0)) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 1); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }



  double type_mac_tmp;


  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE TensileComplexFloat
#define DEST_DATA_TYPE TensileComplexFloat
#define COMPUTE_DATA_TYPE TensileComplexFloat
#define s0 x
#define s1 y
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) \
  DST.s0 = MAC(  MULA.s0, MULB.s0, DST.s0 ); \
  DST.s0 = MAC( -MULA.s1, MULB.s1, DST.s0 ); \
  DST.s1 = MAC(  MULA.s0, MULB.s1, DST.s1 ); \
  DST.s1 = MAC(  MULA.s1, MULB.s0, DST.s1 );
#define TYPE_MAC_WRITE( DST, SRC, ALPHA, REG, BETA ) \
  /* (1) */ \
  type_mac_tmp = REG.s0; \
  REG.s0 *= ALPHA.s0; \
  REG.s0 = MAC( -ALPHA.s1, REG.s1, REG.s0 ); \
  REG.s1 *= ALPHA.s0; \
  REG.s1 = MAC(  ALPHA.s1, type_mac_tmp, REG.s1 ); \
  /* (2) */ \
  if(BETA.s0 != 0) { \
  REG.s0 = MAC(  BETA.s0, SRC.s0, REG.s0 ); \
  REG.s1 = MAC(  BETA.s0, SRC.s1, REG.s1 ); \
  } \
  if (BETA.s1 != 0) { \
  REG.s0 = MAC( -BETA.s1, SRC.s1, REG.s0 ); \
  REG.s1 = MAC(  BETA.s1, SRC.s0, REG.s1 ); \
  } \
  /* (3) */ \
  DST = REG;

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_CB_MT32x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW2_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  TensileComplexFloat *D,
  TensileComplexFloat const * __restrict__ C,
  TensileComplexFloat const * __restrict__ A,
  TensileComplexFloat const * __restrict__ B,
  TensileComplexFloat const alpha,
  TensileComplexFloat const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (TensileComplexFloat)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == tensile_complex<float>(0.0f)) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }



  float type_mac_tmp;


  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE TensileComplexFloat
#define DEST_DATA_TYPE TensileComplexFloat
#define COMPUTE_DATA_TYPE TensileComplexFloat
#define s0 x
#define s1 y
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) \
  DST.s0 = MAC(  MULA.s0, MULB.s0, DST.s0 ); \
  DST.s0 = MAC( -MULA.s1, MULB.s1, DST.s0 ); \
  DST.s1 = MAC(  MULA.s0, MULB.s1, DST.s1 ); \
  DST.s1 = MAC(  MULA.s1, MULB.s0, DST.s1 );
#define TYPE_MAC_WRITE( DST, SRC, ALPHA, REG, BETA ) \
  /* (1) */ \
  type_mac_tmp = REG.s0; \
  REG.s0 *= ALPHA.s0; \
  REG.s0 = MAC( -ALPHA.s1, REG.s1, REG.s0 ); \
  REG.s1 *= ALPHA.s0; \
  REG.s1 = MAC(  ALPHA.s1, type_mac_tmp, REG.s1 ); \
  /* (2) */ \
  if(BETA.s0 != 0) { \
  REG.s0 = MAC(  BETA.s0, SRC.s0, REG.s0 ); \
  REG.s1 = MAC(  BETA.s0, SRC.s1, REG.s1 ); \
  } \
  if (BETA.s1 != 0) { \
  REG.s0 = MAC( -BETA.s1, SRC.s1, REG.s0 ); \
  REG.s1 = MAC(  BETA.s1, SRC.s0, REG.s1 ); \
  } \
  /* (3) */ \
  DST = REG;

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_CB_MT32x32x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW2_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  TensileComplexFloat *D,
  TensileComplexFloat const * __restrict__ C,
  TensileComplexFloat const * __restrict__ A,
  TensileComplexFloat const * __restrict__ B,
  TensileComplexFloat const alpha,
  TensileComplexFloat const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (TensileComplexFloat)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == tensile_complex<float>(0.0f)) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }



  float type_mac_tmp;


  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I] , rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I] , rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I] , rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I] , rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I] , rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I] , rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I] , rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I] , rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I] , rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I] , rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I] , rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I] , rA[7],rB[7],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT128x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_8x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 1) {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 2) {
        rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1792
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 1) {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 2) {
        rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 3584
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x8


    /* iter 7 (last unrolled loop) */

    MAC_4x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 1) {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 2) {
        rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 7168
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_1_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_1_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x8


    /* iter 15 (last unrolled loop) */

    MAC_4x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_1_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 1);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_1_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 1) {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 2) {
        rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1792
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 3 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x8


    /* iter 3 (last unrolled loop) */

    MAC_4x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 1) {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 2) {
        rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 3 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 3 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I] , rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I] , rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I] , rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I] , rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I] , rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I] , rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I] , rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I] , rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I] , rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I] , rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I] , rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I] , rA[7],rB[7],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT128x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_8x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 1) {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 2) {
        rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[4 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[4 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[5 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[5 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[6 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[6 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[7 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[7 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 2

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU2_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 2

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 7168
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU2_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_1_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_1_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_1_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 1);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_1_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 1) {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 2) {
        rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 2

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU2_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 3584
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x8


    /* iter 7 (last unrolled loop) */

    MAC_4x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 1) {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 2) {
        rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 7168
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_1_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_1_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_1_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 1);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_1_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 1) {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 2) {
        rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 3584
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x8


    /* iter 7 (last unrolled loop) */

    MAC_4x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 1) {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 2) {
        rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 7168
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_1_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_1_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_1_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 1);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_1_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_2_0 + 1) = b_0_1_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_3_0 + 1) = b_0_1_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 1) {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else if (s1J == 2) {
        rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 2*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 3*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_1 = (localWriteOffsetA_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      if (s1J == 0) {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      } else {
        rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 1*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I] , rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I] , rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I] , rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I] , rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I] , rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I] , rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I] , rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I] , rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I] , rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I] , rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I] , rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I] , rA[7],rB[7],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT128x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_8x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I] , rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I] , rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I] , rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I] , rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I] , rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I] , rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I] , rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I] , rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I] , rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I] , rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I] , rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I] , rA[7],rB[7],rC[7+7*TT0I]); \

#define MAC_8x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I] , rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I] , rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I] , rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I] , rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[4+TT1J],rC[4+4*TT0I] , rA[5+TT0I],rB[4+TT1J],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[4+TT1J],rC[6+4*TT0I] , rA[7+TT0I],rB[4+TT1J],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[5+TT1J],rC[4+5*TT0I] , rA[5+TT0I],rB[5+TT1J],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[5+TT1J],rC[6+5*TT0I] , rA[7+TT0I],rB[5+TT1J],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[6+TT1J],rC[4+6*TT0I] , rA[5+TT0I],rB[6+TT1J],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[6+TT1J],rC[6+6*TT0I] , rA[7+TT0I],rB[6+TT1J],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[7+TT1J],rC[4+7*TT0I] , rA[5+TT0I],rB[7+TT1J],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[7+TT1J],rC[6+7*TT0I] , rA[7+TT0I],rB[7+TT1J],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT128x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8


    /* iter 3 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_8x8


    /* iter 3 (last unrolled loop) */

    MAC_8x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef MAC_8x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 3584
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I] , rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I] , rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I] , rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I] , rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I] , rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I] , rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I] , rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I] , rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I] , rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I] , rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I] , rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I] , rA[7],rB[7],rC[7+7*TT0I]); \

#define MAC_8x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I] , rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I] , rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I] , rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I] , rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[4+TT1J],rC[4+4*TT0I] , rA[5+TT0I],rB[4+TT1J],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[4+TT1J],rC[6+4*TT0I] , rA[7+TT0I],rB[4+TT1J],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[5+TT1J],rC[4+5*TT0I] , rA[5+TT0I],rB[5+TT1J],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[5+TT1J],rC[6+5*TT0I] , rA[7+TT0I],rB[5+TT1J],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[6+TT1J],rC[4+6*TT0I] , rA[5+TT0I],rB[6+TT1J],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[6+TT1J],rC[6+6*TT0I] , rA[7+TT0I],rB[6+TT1J],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[7+TT1J],rC[4+7*TT0I] , rA[5+TT0I],rB[7+TT1J],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[7+TT1J],rC[6+7*TT0I] , rA[7+TT0I],rB[7+TT1J],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT128x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8


    /* iter 7 */

    MAC_8x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef MAC_8x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 8

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 7168
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;
  unsigned int globalReadOffsetBL_4_0 = globalReadOffsetBL + 0 + 4*LSPB;
  unsigned int globalReadOffsetBL_5_0 = globalReadOffsetBL + 0 + 5*LSPB;
  unsigned int globalReadOffsetBL_6_0 = globalReadOffsetBL + 0 + 6*LSPB;
  unsigned int globalReadOffsetBL_7_0 = globalReadOffsetBL + 0 + 7*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_7_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 4*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 5*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 6*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 7*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_4_0 = (localWriteOffsetB_0_0_4_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_5_0 = (localWriteOffsetB_0_0_5_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_6_0 = (localWriteOffsetB_0_0_6_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_7_0 = (localWriteOffsetB_0_0_7_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
    localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
    localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
    localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
    localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_4_0 = (localWriteOffsetB_0_0_4_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_5_0 = (localWriteOffsetB_0_0_5_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_6_0 = (localWriteOffsetB_0_0_6_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_7_0 = (localWriteOffsetB_0_0_7_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
    localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
    localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
    localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
    localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_4_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_5_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_6_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_7_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 8

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I] , rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I] , rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I] , rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I] , rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I] , rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I] , rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I] , rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I] , rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I] , rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I] , rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I] , rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I] , rA[7],rB[7],rC[7+7*TT0I]); \

#define MAC_8x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I] , rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I] , rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I] , rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I] , rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[4+TT1J],rC[4+4*TT0I] , rA[5+TT0I],rB[4+TT1J],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[4+TT1J],rC[6+4*TT0I] , rA[7+TT0I],rB[4+TT1J],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[5+TT1J],rC[4+5*TT0I] , rA[5+TT0I],rB[5+TT1J],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[5+TT1J],rC[6+5*TT0I] , rA[7+TT0I],rB[5+TT1J],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[6+TT1J],rC[4+6*TT0I] , rA[5+TT0I],rB[6+TT1J],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[6+TT1J],rC[6+6*TT0I] , rA[7+TT0I],rB[6+TT1J],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[7+TT1J],rC[4+7*TT0I] , rA[5+TT0I],rB[7+TT1J],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[7+TT1J],rC[6+7*TT0I] , rA[7+TT0I],rB[7+TT1J],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT128x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;
  unsigned int flattenedOffsetA_4_0 = globalReadOffsetA0I + 0 + 4*LSPA;
  flattenedOffsetA_4_0 = (flattenedOffsetA_4_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_4_0;
  unsigned int globalReadOffsetA0I_4_0 = flattenedOffsetA_4_0;
  unsigned int flattenedOffsetA_5_0 = globalReadOffsetA0I + 0 + 5*LSPA;
  flattenedOffsetA_5_0 = (flattenedOffsetA_5_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_5_0;
  unsigned int globalReadOffsetA0I_5_0 = flattenedOffsetA_5_0;
  unsigned int flattenedOffsetA_6_0 = globalReadOffsetA0I + 0 + 6*LSPA;
  flattenedOffsetA_6_0 = (flattenedOffsetA_6_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_6_0;
  unsigned int globalReadOffsetA0I_6_0 = flattenedOffsetA_6_0;
  unsigned int flattenedOffsetA_7_0 = globalReadOffsetA0I + 0 + 7*LSPA;
  flattenedOffsetA_7_0 = (flattenedOffsetA_7_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_7_0;
  unsigned int globalReadOffsetA0I_7_0 = flattenedOffsetA_7_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;
  unsigned int globalReadOffsetBL_4_0 = globalReadOffsetBL + 0 + 4*LSPB;
  unsigned int globalReadOffsetBL_5_0 = globalReadOffsetBL + 0 + 5*LSPB;
  unsigned int globalReadOffsetBL_6_0 = globalReadOffsetBL + 0 + 6*LSPB;
  unsigned int globalReadOffsetBL_7_0 = globalReadOffsetBL + 0 + 7*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_7_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 4*LSPA);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 5*LSPA);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 6*LSPA);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 7*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 4*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 5*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 6*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 7*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8


    /* iter 15 */

    MAC_8x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef MAC_8x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 8

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 8192
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I] , rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I] , rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I] , rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I] , rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I] , rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I] , rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I] , rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I] , rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I] , rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I] , rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I] , rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I] , rA[7],rB[7],rC[7+7*TT0I]); \

#define MAC_8x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I] , rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I] , rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I] , rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I] , rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[4+TT1J],rC[4+4*TT0I] , rA[5+TT0I],rB[4+TT1J],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[4+TT1J],rC[6+4*TT0I] , rA[7+TT0I],rB[4+TT1J],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[5+TT1J],rC[4+5*TT0I] , rA[5+TT0I],rB[5+TT1J],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[5+TT1J],rC[6+5*TT0I] , rA[7+TT0I],rB[5+TT1J],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[6+TT1J],rC[4+6*TT0I] , rA[5+TT0I],rB[6+TT1J],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[6+TT1J],rC[6+6*TT0I] , rA[7+TT0I],rB[6+TT1J],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[7+TT1J],rC[4+7*TT0I] , rA[5+TT0I],rB[7+TT1J],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[7+TT1J],rC[6+7*TT0I] , rA[7+TT0I],rB[7+TT1J],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT128x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;
  unsigned int flattenedOffsetA_4_0 = globalReadOffsetA0I + 0 + 4*LSPA;
  flattenedOffsetA_4_0 = (flattenedOffsetA_4_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_4_0;
  unsigned int globalReadOffsetA0I_4_0 = flattenedOffsetA_4_0;
  unsigned int flattenedOffsetA_5_0 = globalReadOffsetA0I + 0 + 5*LSPA;
  flattenedOffsetA_5_0 = (flattenedOffsetA_5_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_5_0;
  unsigned int globalReadOffsetA0I_5_0 = flattenedOffsetA_5_0;
  unsigned int flattenedOffsetA_6_0 = globalReadOffsetA0I + 0 + 6*LSPA;
  flattenedOffsetA_6_0 = (flattenedOffsetA_6_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_6_0;
  unsigned int globalReadOffsetA0I_6_0 = flattenedOffsetA_6_0;
  unsigned int flattenedOffsetA_7_0 = globalReadOffsetA0I + 0 + 7*LSPA;
  flattenedOffsetA_7_0 = (flattenedOffsetA_7_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_7_0;
  unsigned int globalReadOffsetA0I_7_0 = flattenedOffsetA_7_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;
  unsigned int globalReadOffsetBL_4_0 = globalReadOffsetBL + 0 + 4*LSPB;
  unsigned int globalReadOffsetBL_5_0 = globalReadOffsetBL + 0 + 5*LSPB;
  unsigned int globalReadOffsetBL_6_0 = globalReadOffsetBL + 0 + 6*LSPB;
  unsigned int globalReadOffsetBL_7_0 = globalReadOffsetBL + 0 + 7*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_7_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 4*LSPA);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 5*LSPA);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 6*LSPA);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 7*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 4*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 5*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 6*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 7*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_4_0 = (localWriteOffsetA_0_0_4_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_5_0 = (localWriteOffsetA_0_0_5_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_6_0 = (localWriteOffsetA_0_0_6_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_7_0 = (localWriteOffsetA_0_0_7_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_4_0 = (localWriteOffsetB_0_0_4_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_5_0 = (localWriteOffsetB_0_0_5_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_6_0 = (localWriteOffsetB_0_0_6_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_7_0 = (localWriteOffsetB_0_0_7_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
    localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
    localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
    localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
    localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
    localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
    localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
    localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
    localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_4_0 = (localWriteOffsetA_0_0_4_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_5_0 = (localWriteOffsetA_0_0_5_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_6_0 = (localWriteOffsetA_0_0_6_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_7_0 = (localWriteOffsetA_0_0_7_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_4_0 = (localWriteOffsetB_0_0_4_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_5_0 = (localWriteOffsetB_0_0_5_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_6_0 = (localWriteOffsetB_0_0_6_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_7_0 = (localWriteOffsetB_0_0_7_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
    localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
    localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
    localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
    localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
    localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
    localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
    localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
    localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_8x8


    /* iter 15 (last unrolled loop) */

    MAC_8x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_4_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_5_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_6_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_7_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_4_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_5_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_6_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_7_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef MAC_8x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 8

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 7168
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;
  unsigned int globalReadOffsetBL_4_0 = globalReadOffsetBL + 0 + 4*LSPB;
  unsigned int globalReadOffsetBL_5_0 = globalReadOffsetBL + 0 + 5*LSPB;
  unsigned int globalReadOffsetBL_6_0 = globalReadOffsetBL + 0 + 6*LSPB;
  unsigned int globalReadOffsetBL_7_0 = globalReadOffsetBL + 0 + 7*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_7_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 4*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 5*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 6*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 7*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_4_0 = (localWriteOffsetB_0_0_4_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_5_0 = (localWriteOffsetB_0_0_5_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_6_0 = (localWriteOffsetB_0_0_6_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_7_0 = (localWriteOffsetB_0_0_7_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
    localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
    localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
    localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
    localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_4_0 = (localWriteOffsetB_0_0_4_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_5_0 = (localWriteOffsetB_0_0_5_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_6_0 = (localWriteOffsetB_0_0_6_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_7_0 = (localWriteOffsetB_0_0_7_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
    localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
    localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
    localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
    localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x8


    /* iter 15 (last unrolled loop) */

    MAC_4x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_4_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_5_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_6_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_7_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 2

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU2_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 2

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU2_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 2

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU2_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x64x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 5

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU5_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 7 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 7

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 4096
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HSS_BH_MT64x64x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU7_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (numIterMyWg >= 1) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_2_0 = (localWriteOffsetB_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_3_0 = (localWriteOffsetB_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
    localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (numIterMyWg >= 1) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_4x4

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_3_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 1536

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x4 micro-tile */
#define MAC_8x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7],rB[3],rC[7+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT128x64x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_4_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 1536

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x4 micro-tile */
#define MAC_8x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7],rB[3],rC[7+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT128x64x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_4_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x4




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x4

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x4
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 16
#define SG1J 4
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 832
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT32x8x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_4_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT32x32x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 7 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_3 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 2) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 3) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 128
#define LDS_NUM_ELEMENTS 512
#define LDS_OFFSET_BLK 256

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT16x16x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 32
#define SG1J 4
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 3200
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT64x8x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG32_4_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 15 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1792
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT32x16x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x2


    /* iter 15 (last unrolled loop) */

    MAC_4x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2048
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT32x32x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_2_1_0;
  DATA_TYPE a_0_3_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_2_1_0;
  DATA_TYPE b_0_3_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;
  DATA_TYPE *localWriteB_0_0_1_2;
  DATA_TYPE *localWriteB_0_0_1_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
  localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
    *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
    *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_2 = (localWriteOffsetB_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_3 = (localWriteOffsetB_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
    localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
    localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
    *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
    *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_2 = (localWriteOffsetB_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_3 = (localWriteOffsetB_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
    localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
    localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_3 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_3 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_2_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 2);
  a_0_3_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_2_1_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 2);
  b_0_3_1_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
  localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
  *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
  *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
  *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 2) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 3) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 128
#define SG0I 16
#define SG1J 8
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 3584
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT64x32x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_2_1_0;
  DATA_TYPE a_0_3_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
    *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
    *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 15 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_3 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_2_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 2);
  a_0_3_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
  *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 2) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 3) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 24
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 3
#define NLPA 6
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 768
#define LDS_NUM_ELEMENTS 3200
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT32x16x24_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB3_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_1_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_1_5_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_1_0_0_0;
  DATA_TYPE b_1_1_0_0;
  DATA_TYPE b_2_0_0_0;
  DATA_TYPE b_2_1_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSCB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_1_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_1_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_2_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_2_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_1_0_0_0 = B + globalReadOffsetB_1_0_0_0;
  DATA_TYPE const *globalReadB_2_0_0_0 = B + globalReadOffsetB_2_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_1_0_0_0 = localWriteFirstOffsetB + (0 + 1*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_2_0_0_0 = localWriteFirstOffsetB + (0 + 2*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_1_0_0_1 = localWriteFirstOffsetB + (1 + 1*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_2_0_0_1 = localWriteFirstOffsetB + (1 + 2*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_1_0_0_0;
  DATA_TYPE *localWriteB_2_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_1_0_0_1;
  DATA_TYPE *localWriteB_2_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_1_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_1_0_0_0);
  localWriteB_2_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_2_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_1_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_1_0_0_1);
  localWriteB_2_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_2_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 384-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_1_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_2_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_1_4_0 = *(globalReadA_0_0_4_0 + 1);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_1_5_0 = *(globalReadA_0_0_5_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_1_0_0_0 = *(globalReadB_1_0_0_0 + 0);
    b_1_1_0_0 = *(globalReadB_1_0_0_0 + 1);
    b_2_0_0_0 = *(globalReadB_2_0_0_0 + 0);
    b_2_1_0_0 = *(globalReadB_2_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_1_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_1_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_1_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_2_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_2_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_2_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_4_0 + 1) = a_0_1_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_5_0 + 1) = a_0_1_5_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_1_0_0_0 + 0) = b_1_0_0_0;
    *(localWriteB_2_0_0_0 + 0) = b_2_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_1_0_0_1 + 0) = b_1_1_0_0;
    *(localWriteB_2_0_0_1 + 0) = b_2_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_4_0 = (localWriteOffsetA_0_0_4_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_5_0 = (localWriteOffsetA_0_0_5_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_1_0_0_0 = (localWriteOffsetB_1_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_2_0_0_0 = (localWriteOffsetB_2_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_1_0_0_1 = (localWriteOffsetB_1_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_2_0_0_1 = (localWriteOffsetB_2_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
    localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
    localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_1_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_1_0_0_0);
    localWriteB_2_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_2_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_1_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_1_0_0_1);
    localWriteB_2_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_2_0_0_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_1_4_0 = *(globalReadA_0_0_4_0 + 1);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_1_5_0 = *(globalReadA_0_0_5_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_1_0_0_0 = *(globalReadB_1_0_0_0 + 0);
    b_1_1_0_0 = *(globalReadB_1_0_0_0 + 1);
    b_2_0_0_0 = *(globalReadB_2_0_0_0 + 0);
    b_2_1_0_0 = *(globalReadB_2_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_1_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_1_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_1_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_2_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_2_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_2_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 15 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 16 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 17 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 18 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 19 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 20 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 21 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 22 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_4_0 + 1) = a_0_1_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_5_0 + 1) = a_0_1_5_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_1_0_0_0 + 0) = b_1_0_0_0;
    *(localWriteB_2_0_0_0 + 0) = b_2_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_1_0_0_1 + 0) = b_1_1_0_0;
    *(localWriteB_2_0_0_1 + 0) = b_2_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_4_0 = (localWriteOffsetA_0_0_4_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_5_0 = (localWriteOffsetA_0_0_5_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_1_0_0_0 = (localWriteOffsetB_1_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_2_0_0_0 = (localWriteOffsetB_2_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_1_0_0_1 = (localWriteOffsetB_1_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_2_0_0_1 = (localWriteOffsetB_2_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
    localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
    localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_1_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_1_0_0_0);
    localWriteB_2_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_2_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_1_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_1_0_0_1);
    localWriteB_2_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_2_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 23 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 16 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 17 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 18 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 19 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 20 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 21 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 22 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x2


    /* iter 23 (last unrolled loop) */

    MAC_4x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_4_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_5_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_1_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_2_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_1_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_2_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_1_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_2_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_1_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 1);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_1_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_1_0_0_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_1_0_0_0 + 0);
  b_1_1_0_0 = ( globalReadOffsetBL_1_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_1_0_0_0 + 1);
  b_2_0_0_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_2_0_0_0 + 0);
  b_2_1_0_0 = ( globalReadOffsetBL_2_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_2_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_1_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_1_0_0_0);
  localWriteB_2_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_2_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_1_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_1_0_0_1);
  localWriteB_2_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_2_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_4_0 + 1) = a_0_1_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_5_0 + 1) = a_0_1_5_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_1_0_0_0 + 0) = b_1_0_0_0;
  *(localWriteB_2_0_0_0 + 0) = b_2_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_1_0_0_1 + 0) = b_1_1_0_0;
  *(localWriteB_2_0_0_1 + 0) = b_2_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 16
#define SG1J 4
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 32
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 3328
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT32x8x32_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_4_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_1_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_1_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_1_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_1_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 1); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_1_4_0 = *(globalReadA_0_0_4_0 + 1);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_1_5_0 = *(globalReadA_0_0_5_0 + 1);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_1_6_0 = *(globalReadA_0_0_6_0 + 1);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_1_7_0 = *(globalReadA_0_0_7_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_4_0 + 1) = a_0_1_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_5_0 + 1) = a_0_1_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_6_0 + 1) = a_0_1_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_7_0 + 1) = a_0_1_7_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_4_0 = (localWriteOffsetA_0_0_4_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_5_0 = (localWriteOffsetA_0_0_5_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_6_0 = (localWriteOffsetA_0_0_6_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_7_0 = (localWriteOffsetA_0_0_7_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
    localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
    localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
    localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
    localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_1_4_0 = *(globalReadA_0_0_4_0 + 1);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_1_5_0 = *(globalReadA_0_0_5_0 + 1);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_1_6_0 = *(globalReadA_0_0_6_0 + 1);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_1_7_0 = *(globalReadA_0_0_7_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 15 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 16 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 17 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 18 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 19 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 20 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 21 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 22 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 23 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 24 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 25 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 26 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 27 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 28 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 29 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 30 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_4_0 + 1) = a_0_1_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_5_0 + 1) = a_0_1_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_6_0 + 1) = a_0_1_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_7_0 + 1) = a_0_1_7_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_2_0 = (localWriteOffsetA_0_0_2_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_3_0 = (localWriteOffsetA_0_0_3_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_4_0 = (localWriteOffsetA_0_0_4_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_5_0 = (localWriteOffsetA_0_0_5_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_6_0 = (localWriteOffsetA_0_0_6_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_7_0 = (localWriteOffsetA_0_0_7_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
    localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
    localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
    localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
    localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
    localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
    localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 31 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 16 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 17 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 18 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 19 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 20 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 21 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 22 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 23 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 24 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 25 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 26 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 27 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 28 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 29 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 30 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 31 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_2_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_3_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_4_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_5_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_6_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_7_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_1_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 1);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_1_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 1);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_1_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 1);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_1_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_4_0 + 1) = a_0_1_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_5_0 + 1) = a_0_1_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_6_0 + 1) = a_0_1_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_7_0 + 1) = a_0_1_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I]); \
  TYPE_MAC(rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I]); \
  TYPE_MAC(rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I]); \
  TYPE_MAC(rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I]); \
  TYPE_MAC(rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I]); \
  TYPE_MAC(rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I]); \
  TYPE_MAC(rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I]); \
  TYPE_MAC(rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I]); \
  TYPE_MAC(rA[7],rB[7],rC[7+7*TT0I]); \

#define MAC_8x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[4+TT1J],rC[4+4*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[4+TT1J],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[4+TT1J],rC[6+4*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[4+TT1J],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[5+TT1J],rC[4+5*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[5+TT1J],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[5+TT1J],rC[6+5*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[5+TT1J],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[6+TT1J],rC[4+6*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[6+TT1J],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[6+TT1J],rC[6+6*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[6+TT1J],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[7+TT1J],rC[4+7*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[7+TT1J],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[7+TT1J],rC[6+7*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[7+TT1J],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT128x128x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8


    /* iter 7 */

    MAC_8x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
    if (r0I == 2) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
    if (r0I == 3) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef MAC_8x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 3072

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x4 micro-tile */
#define MAC_8x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7],rB[3],rC[7+3*TT0I]); \

#define MAC_8x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT128x64x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_2_1_0;
  DATA_TYPE a_0_3_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
    *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x4


    /* iter 15 */

    MAC_8x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_2_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 2);
  a_0_3_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
  *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x4

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 2) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 3) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x4
#undef MAC_8x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 7168
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x4 micro-tile */
#define MAC_8x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7],rB[3],rC[7+3*TT0I]); \

#define MAC_8x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT128x64x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_2_1_0;
  DATA_TYPE a_0_3_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
    *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
    *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_8x4


    /* iter 15 (last unrolled loop) */

    MAC_8x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_3 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_2_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 2);
  a_0_3_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
  *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 2) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 3) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x4
#undef MAC_8x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 6
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 3
#define NLCB 1
#define NLPA 1
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1536
#define LDS_NUM_ELEMENTS 3584

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 6x8 micro-tile */
#define MAC_6x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I]); \
  TYPE_MAC(rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I]); \
  TYPE_MAC(rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I]); \
  TYPE_MAC(rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I]); \
  TYPE_MAC(rA[5],rB[7],rC[5+7*TT0I]); \

#define MAC_6x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[4+TT1J],rC[4+4*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[4+TT1J],rC[5+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[5+TT1J],rC[4+5*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[5+TT1J],rC[5+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[6+TT1J],rC[4+6*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[6+TT1J],rC[5+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[7+TT1J],rC[4+7*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[7+TT1J],rC[5+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT96x128x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA3_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT6_8_TLDS0_USFGRO0_VAW1_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_1_0_0_0;
  DATA_TYPE a_1_1_0_0;
  DATA_TYPE a_2_0_0_0;
  DATA_TYPE a_2_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_1_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_1_3_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSCA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSCA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_2_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_1_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_1_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_2_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_2_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_1_0_0_0 = A + globalReadOffsetA_1_0_0_0;
  DATA_TYPE const *globalReadA_2_0_0_0 = A + globalReadOffsetA_2_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_1_0_0_0 = localWriteFirstOffsetA + (0 + 1*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_2_0_0_0 = localWriteFirstOffsetA + (0 + 2*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_2_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_3_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_1_0_0_0;
  DATA_TYPE *localWriteA_2_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_2_1;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_3_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_1_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_0_0);
  localWriteA_2_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_2_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_1);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_1_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_2_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_1_0_0_0 = *(globalReadA_1_0_0_0 + 0);
    a_1_1_0_0 = *(globalReadA_1_0_0_0 + 1);
    a_2_0_0_0 = *(globalReadA_2_0_0_0 + 0);
    a_2_1_0_0 = *(globalReadA_2_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_1_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_1_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_1_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_2_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_2_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_2_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_1_0_0_0 + 0) = a_1_0_0_0;
    *(localWriteA_1_0_0_0 + 1) = a_1_1_0_0;
    *(localWriteA_2_0_0_0 + 0) = a_2_0_0_0;
    *(localWriteA_2_0_0_0 + 1) = a_2_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_1 + 0) = b_0_1_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_1 + 0) = b_0_1_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_6x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_6x8


    /* iter 15 */

    MAC_6x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_1_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_2_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_1_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_1_0_0_0 + 0);
  a_1_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_1_0_0_0 + 1);
  a_2_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_2_0_0_0 + 0);
  a_2_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_2_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_1_2_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 1);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_1_3_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_1_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_0_0);
  localWriteA_2_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_2_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_1);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_1_0_0_0 + 0) = a_1_0_0_0;
  *(localWriteA_1_0_0_0 + 1) = a_1_1_0_0;
  *(localWriteA_2_0_0_0 + 0) = a_2_0_0_0;
  *(localWriteA_2_0_0_0 + 1) = a_2_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_2_1 + 0) = b_0_1_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_3_1 + 0) = b_0_1_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_6x8

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else if (s0I == 1) {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      } else {
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_6x8
#undef MAC_6x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 7168
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x4 micro-tile */
#define MAC_8x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7],rB[3],rC[7+3*TT0I]); \

#define MAC_8x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT128x64x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_2_1_0;
  DATA_TYPE a_0_3_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 2); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
    *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
    *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x4


    /* iter 15 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_8x4


    /* iter 15 (last unrolled loop) */

    MAC_8x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_3 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_2_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 2);
  a_0_3_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
  *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 2) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 3) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x4
#undef MAC_8x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 32
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT64x64x32_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_2_1_0;
  DATA_TYPE a_0_3_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_2_1_0;
  DATA_TYPE b_0_3_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;
  DATA_TYPE *localWriteB_0_0_1_2;
  DATA_TYPE *localWriteB_0_0_1_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
  localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 1); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
    *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
    *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 16 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 17 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 18 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 19 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 20 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 21 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 22 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 23 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 24 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 25 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 26 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 27 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 28 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 29 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 30 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 31 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_2_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 2);
  a_0_3_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_2_1_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 2);
  b_0_3_1_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
  localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
  *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
  *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
  *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 2) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 3) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 32
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 6144

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT64x128x32_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_2_1_0;
  DATA_TYPE a_0_3_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_2_1_0;
  DATA_TYPE b_0_3_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_1_2_0;
  DATA_TYPE b_0_2_2_0;
  DATA_TYPE b_0_3_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_1_3_0;
  DATA_TYPE b_0_2_3_0;
  DATA_TYPE b_0_3_3_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;
  globalReadOffsetB1J_2_0 = (  globalReadOffsetB1J_2_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_2_0;
  globalReadOffsetB1J_3_0 = (  globalReadOffsetB1J_3_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_3_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_2_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_2_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_2_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_3_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_3_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_3_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;
  DATA_TYPE *localWriteB_0_0_1_2;
  DATA_TYPE *localWriteB_0_0_1_3;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_2_1;
  DATA_TYPE *localWriteB_0_0_2_2;
  DATA_TYPE *localWriteB_0_0_2_3;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_3_1;
  DATA_TYPE *localWriteB_0_0_3_2;
  DATA_TYPE *localWriteB_0_0_3_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
  localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_1);
  localWriteB_0_0_2_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_2);
  localWriteB_0_0_2_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_3);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_1);
  localWriteB_0_0_3_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_2);
  localWriteB_0_0_3_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_3);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 1); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_1_2_0 = *(globalReadB_0_0_2_0 + 1);
    b_0_2_2_0 = *(globalReadB_0_0_2_0 + 2);
    b_0_3_2_0 = *(globalReadB_0_0_2_0 + 3);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_1_3_0 = *(globalReadB_0_0_3_0 + 1);
    b_0_2_3_0 = *(globalReadB_0_0_3_0 + 2);
    b_0_3_3_0 = *(globalReadB_0_0_3_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
    *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
    *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_2_1 + 0) = b_0_1_2_0;
    *(localWriteB_0_0_2_2 + 0) = b_0_2_2_0;
    *(localWriteB_0_0_2_3 + 0) = b_0_3_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_3_1 + 0) = b_0_1_3_0;
    *(localWriteB_0_0_3_2 + 0) = b_0_2_3_0;
    *(localWriteB_0_0_3_3 + 0) = b_0_3_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 15 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 16 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 17 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 18 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 19 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 20 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 21 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 22 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 23 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 24 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 25 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 26 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 27 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 28 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 29 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 30 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 31 */

    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_2_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 2);
  a_0_3_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_2_1_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 2);
  b_0_3_1_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 3);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_1_2_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 1);
  b_0_2_2_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 2);
  b_0_3_2_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 3);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_1_3_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 1);
  b_0_2_3_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 2);
  b_0_3_3_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
  localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_2_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_1);
  localWriteB_0_0_2_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_2);
  localWriteB_0_0_2_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_3);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_3_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_1);
  localWriteB_0_0_3_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_2);
  localWriteB_0_0_3_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
  *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
  *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
  *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_2_1 + 0) = b_0_1_2_0;
  *(localWriteB_0_0_2_2 + 0) = b_0_2_2_0;
  *(localWriteB_0_0_2_3 + 0) = b_0_3_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_3_1 + 0) = b_0_1_3_0;
  *(localWriteB_0_0_3_2 + 0) = b_0_2_3_0;
  *(localWriteB_0_0_3_3 + 0) = b_0_3_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+2] = localReadB[1*SG1J*VECTOR_WIDTH + 2]; 
    rB[1*VECTOR_WIDTH+3] = localReadB[1*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
    if (r0I == 2) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
    if (r0I == 3) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 4*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 5*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 6*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 7*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (1*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 32
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 1

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 4096
#define LDS_NUM_ELEMENTS 6144

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x4 micro-tile */
#define MAC_8x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7],rB[3],rC[7+3*TT0I]); \

#define MAC_8x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[2+TT1J],rC[4+2*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[2+TT1J],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[2+TT1J],rC[6+2*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[2+TT1J],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[3+TT1J],rC[4+3*TT0I]); \
  TYPE_MAC(rA[5+TT0I],rB[3+TT1J],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[3+TT1J],rC[6+3*TT0I]); \
  TYPE_MAC(rA[7+TT0I],rB[3+TT1J],rC[7+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT128x64x32_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM1(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_2_1_0;
  DATA_TYPE a_0_3_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_2_2_0;
  DATA_TYPE a_0_3_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE a_0_2_3_0;
  DATA_TYPE a_0_3_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_2_1_0;
  DATA_TYPE b_0_3_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;
  DATA_TYPE *localWriteB_0_0_1_2;
  DATA_TYPE *localWriteB_0_0_1_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
  localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 1); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_2_2_0 = *(globalReadA_0_0_2_0 + 2);
    a_0_3_2_0 = *(globalReadA_0_0_2_0 + 3);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);
    a_0_2_3_0 = *(globalReadA_0_0_3_0 + 2);
    a_0_3_3_0 = *(globalReadA_0_0_3_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
    *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_2_0 + 2) = a_0_2_2_0;
    *(localWriteA_0_0_2_0 + 3) = a_0_3_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
    *(localWriteA_0_0_3_0 + 2) = a_0_2_3_0;
    *(localWriteA_0_0_3_0 + 3) = a_0_3_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
    *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 15 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 16 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 17 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 18 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 19 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 20 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 21 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 22 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 23 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 24 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 25 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 26 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 27 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 28 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4


    /* iter 29 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x4_BLK


    /* iter 30 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x4


    /* iter 31 */

    MAC_8x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_2_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 2);
  a_0_3_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 3);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_2_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 2);
  a_0_3_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 3);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);
  a_0_2_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 2);
  a_0_3_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_2_1_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 2);
  b_0_3_1_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
  localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
  *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
  *(localWriteA_0_0_2_0 + 2) = a_0_2_2_0;
  *(localWriteA_0_0_2_0 + 3) = a_0_3_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
  *(localWriteA_0_0_3_0 + 2) = a_0_2_3_0;
  *(localWriteA_0_0_3_0 + 3) = a_0_3_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
  *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
  *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+2] = localReadA[1*SG0I*VECTOR_WIDTH + 2]; 
    rA[1*VECTOR_WIDTH+3] = localReadA[1*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x4

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 2) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 3) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x4
#undef MAC_8x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 32
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT64x64x32_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_2_1_0;
  DATA_TYPE a_0_3_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_2_1_0;
  DATA_TYPE b_0_3_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;
  DATA_TYPE *localWriteB_0_0_1_2;
  DATA_TYPE *localWriteB_0_0_1_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
  localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 1); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
    *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
    *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 16 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 17 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 18 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 19 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 20 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 21 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 22 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 23 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 24 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 25 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 26 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 27 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 28 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 29 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 30 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 31 */

    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_2_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 2);
  a_0_3_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_2_1_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 2);
  b_0_3_1_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
  localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
  *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
  *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
  *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 2) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 3) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 4
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 4
#define GLOBAL_LOAD_VECTOR_WIDTH_A 4
#define GLOBAL_LOAD_VECTOR_WIDTH_B 4
#define GLOBAL_WRITE_VECTOR_WIDTH 4

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 32
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 8192
#define LDS_OFFSET_BLK 4096

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE float
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) DST = MAC(MULA,MULB,DST);
#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x4 micro-tile */
#define MAC_4x4\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3],rB[3],rC[3+3*TT0I]); \

#define MAC_4x4_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I]); \
  TYPE_MAC(rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_SB_MT64x64x32_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW4_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW4_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_4_TLDS0_USFGRO0_VAW1_VS1_VW4_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  float const * __restrict__ A,
  float const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (float)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_2_0_0;
  DATA_TYPE a_0_3_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_2_1_0;
  DATA_TYPE a_0_3_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_2_0_0;
  DATA_TYPE b_0_3_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;
  DATA_TYPE b_0_2_1_0;
  DATA_TYPE b_0_3_1_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_2 = localWriteFirstOffsetB + (2 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_3 = localWriteFirstOffsetB + (3 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_0_2;
  DATA_TYPE *localWriteB_0_0_0_3;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;
  DATA_TYPE *localWriteB_0_0_1_2;
  DATA_TYPE *localWriteB_0_0_1_3;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
  localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 1); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
    *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
    *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_2 = (localWriteOffsetB_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_3 = (localWriteOffsetB_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
    localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
    localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_2_0_0 = *(globalReadA_0_0_0_0 + 2);
    a_0_3_0_0 = *(globalReadA_0_0_0_0 + 3);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_2_1_0 = *(globalReadA_0_0_1_0 + 2);
    a_0_3_1_0 = *(globalReadA_0_0_1_0 + 3);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_2_0_0 = *(globalReadB_0_0_0_0 + 2);
    b_0_3_0_0 = *(globalReadB_0_0_0_0 + 3);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);
    b_0_2_1_0 = *(globalReadB_0_0_1_0 + 2);
    b_0_3_1_0 = *(globalReadB_0_0_1_0 + 3);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 16 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 17 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 18 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 19 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 20 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 21 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 22 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 23 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 24 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 25 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 26 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 27 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 28 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 29 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 30 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
    *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
    *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
    *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
    *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
    *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_1_0 = (localWriteOffsetA_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_2 = (localWriteOffsetB_0_0_0_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_3 = (localWriteOffsetB_0_0_0_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_2 = (localWriteOffsetB_0_0_1_2 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_3 = (localWriteOffsetB_0_0_1_3 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
    localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
    localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
    localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x4


    /* iter 31 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 8 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 9 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 10 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 11 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 12 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 13 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 14 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 15 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 16 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 17 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 18 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 19 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 20 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 21 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 22 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 23 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 24 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 25 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 26 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 27 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 28 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4


    /* iter 29 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x4_BLK


    /* iter 30 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 

/* local write A */

/* local write B */
    MAC_4x4


    /* iter 31 (last unrolled loop) */

    MAC_4x4_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_1_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_3 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_2 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_3 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_2_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 2);
  a_0_3_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 3);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_2_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 2);
  a_0_3_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 3);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_2_0_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 2);
  b_0_3_0_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 3);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);
  b_0_2_1_0 = ( globalReadOffsetBL_0_0  + 2 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 2);
  b_0_3_1_0 = ( globalReadOffsetBL_0_0  + 3 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 3);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_0_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_2);
  localWriteB_0_0_0_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_3);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);
  localWriteB_0_0_1_2 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_2);
  localWriteB_0_0_1_3 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_3);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_0_0 + 2) = a_0_2_0_0;
  *(localWriteA_0_0_0_0 + 3) = a_0_3_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_1_0 + 2) = a_0_2_1_0;
  *(localWriteA_0_0_1_0 + 3) = a_0_3_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_0_2 + 0) = b_0_2_0_0;
  *(localWriteB_0_0_0_3 + 0) = b_0_3_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
  *(localWriteB_0_0_1_2 + 0) = b_0_2_1_0;
  *(localWriteB_0_0_1_3 + 0) = b_0_3_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[0*VECTOR_WIDTH+2] = localReadA[0*SG0I*VECTOR_WIDTH + 2]; 
    rA[0*VECTOR_WIDTH+3] = localReadA[0*SG0I*VECTOR_WIDTH + 3]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[0*VECTOR_WIDTH+2] = localReadB[0*SG1J*VECTOR_WIDTH + 2]; 
    rB[0*VECTOR_WIDTH+3] = localReadB[0*SG1J*VECTOR_WIDTH + 3]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x4

  }




  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 2) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
    if (r0I == 3) {
      {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 2*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
        rC[2 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I] = rC[3 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 3*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+2)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 2 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 2, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+2 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }
  if (flattenedGlobalC0 + 3 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 3, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+3 + (0*VECTOR_WIDTH+3)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x4
#undef MAC_4x4_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HBH_MT32x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 1536

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HBH_MT64x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }





  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 819

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HBH_MT16x16x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HBH_MT32x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 1536

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HBH_MT64x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 819

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HBH_MT16x16x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HBH_MT32x32x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 1536

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HBH_MT64x32x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }





  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[2 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[3 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 819

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HBH_MT16x16x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_1_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_1_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_1 + 0) = a_0_1_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_0 + 1) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_0 + 1) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }





  /* shift vector components d1 */

  unsigned int wgMT1J = size1J - wg1J*MT1J;
  if (wgMT1J > MT1J) wgMT1J = MT1J;
  unsigned int r1J = wgMT1J % GLOBAL_LOAD_VECTOR_WIDTH_B;
  if (r1J > 0 && ((wgMT1J/VECTOR_WIDTH) % SG1J) == (serial / SG0I) % SG1J ) {
    unsigned int s1J = (wgMT1J%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_B + (wgMT1J/(SG1J*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_B);
    if (r1J == 1) {
      {
        rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[0 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
        rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 0*TT0I] = rC[1 + 0*TT0I*GLOBAL_LOAD_VECTOR_WIDTH_B + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HBH_MT32x32x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 1024
#define LDS_NUM_ELEMENTS 1536

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HBH_MT64x32x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 4
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 819

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_HBH_MT16x16x16_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_2_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;
  unsigned int flattenedOffsetA_2_0 = globalReadOffsetA0I + 0 + 2*LSPA;
  flattenedOffsetA_2_0 = (flattenedOffsetA_2_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_2_0;
  unsigned int globalReadOffsetA0I_2_0 = flattenedOffsetA_2_0;
  unsigned int flattenedOffsetA_3_0 = globalReadOffsetA0I + 0 + 3*LSPA;
  flattenedOffsetA_3_0 = (flattenedOffsetA_3_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_3_0;
  unsigned int globalReadOffsetA0I_3_0 = flattenedOffsetA_3_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;
  unsigned int globalReadOffsetBL_1_0 = globalReadOffsetBL + 0 + 1*LSPB;
  unsigned int globalReadOffsetBL_2_0 = globalReadOffsetBL + 0 + 2*LSPB;
  unsigned int globalReadOffsetBL_3_0 = globalReadOffsetBL + 0 + 3*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;
  globalReadOffsetA0I_2_0 = (  globalReadOffsetA0I_2_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_2_0;
  globalReadOffsetA0I_3_0 = (  globalReadOffsetA0I_3_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_3_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_3_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 2*LSPA);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 3*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 1*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 2*LSPB)*(MT1J+PAD);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 3*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 14 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 15 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE TensileComplexDouble
#define DEST_DATA_TYPE TensileComplexDouble
#define COMPUTE_DATA_TYPE TensileComplexDouble
#define s0 x
#define s1 y
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) \
  DST.s0 = MAC(  MULA.s0, MULB.s0, DST.s0 ); \
  DST.s0 = MAC( -MULA.s1, MULB.s1, DST.s0 ); \
  DST.s1 = MAC(  MULA.s0, MULB.s1, DST.s1 ); \
  DST.s1 = MAC(  MULA.s1, MULB.s0, DST.s1 );
#define TYPE_MAC_WRITE( DST, SRC, ALPHA, REG, BETA ) \
  /* (1) */ \
  type_mac_tmp = REG.s0; \
  REG.s0 *= ALPHA.s0; \
  REG.s0 = MAC( -ALPHA.s1, REG.s1, REG.s0 ); \
  REG.s1 *= ALPHA.s0; \
  REG.s1 = MAC(  ALPHA.s1, type_mac_tmp, REG.s1 ); \
  /* (2) */ \
  if(BETA.s0 != 0) { \
  REG.s0 = MAC(  BETA.s0, SRC.s0, REG.s0 ); \
  REG.s1 = MAC(  BETA.s0, SRC.s1, REG.s1 ); \
  } \
  if (BETA.s1 != 0) { \
  REG.s0 = MAC( -BETA.s1, SRC.s1, REG.s0 ); \
  REG.s1 = MAC(  BETA.s1, SRC.s0, REG.s1 ); \
  } \
  /* (3) */ \
  DST = REG;

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_ZB_MT32x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW1_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  TensileComplexDouble *D,
  TensileComplexDouble const * __restrict__ C,
  TensileComplexDouble const * __restrict__ A,
  TensileComplexDouble const * __restrict__ B,
  TensileComplexDouble const alpha,
  TensileComplexDouble const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (TensileComplexDouble)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == tensile_complex<double>(0.0)) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 1); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }



  double type_mac_tmp;


  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (MT1J/NLCB)
#define LSPB (LOCAL_DEPTHU/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1024
#define LDS_OFFSET_BLK 512

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDX1J, IDXL, IDXK) (( (IDX1J)*strideB1J + (IDXL)*strideBL + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE TensileComplexDouble
#define DEST_DATA_TYPE TensileComplexDouble
#define COMPUTE_DATA_TYPE TensileComplexDouble
#define s0 x
#define s1 y
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += A*B
#define TYPE_MAC(MULA,MULB,DST) \
  DST.s0 = MAC(  MULA.s0, MULB.s0, DST.s0 ); \
  DST.s0 = MAC( -MULA.s1, MULB.s1, DST.s0 ); \
  DST.s1 = MAC(  MULA.s0, MULB.s1, DST.s1 ); \
  DST.s1 = MAC(  MULA.s1, MULB.s0, DST.s1 );
#define TYPE_MAC_WRITE( DST, SRC, ALPHA, REG, BETA ) \
  /* (1) */ \
  type_mac_tmp = REG.s0; \
  REG.s0 *= ALPHA.s0; \
  REG.s0 = MAC( -ALPHA.s1, REG.s1, REG.s0 ); \
  REG.s1 *= ALPHA.s0; \
  REG.s1 = MAC(  ALPHA.s1, type_mac_tmp, REG.s1 ); \
  /* (2) */ \
  if(BETA.s0 != 0) { \
  REG.s0 = MAC(  BETA.s0, SRC.s0, REG.s0 ); \
  REG.s1 = MAC(  BETA.s0, SRC.s1, REG.s1 ); \
  } \
  if (BETA.s1 != 0) { \
  REG.s0 = MAC( -BETA.s1, SRC.s1, REG.s0 ); \
  REG.s1 = MAC(  BETA.s1, SRC.s0, REG.s1 ); \
  } \
  /* (3) */ \
  DST = REG;

/* 2x2 micro-tile */
#define MAC_2x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1],rB[1],rC[1+1*TT0I]); \

#define MAC_2x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I]); \
  TYPE_MAC(rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideB1J 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bjlk_ZB_MT32x32x8_SN_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW1_SNLL0_TT2_2_TLDS0_USFGRO0_VAW1_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  TensileComplexDouble *D,
  TensileComplexDouble const * __restrict__ C,
  TensileComplexDouble const * __restrict__ A,
  TensileComplexDouble const * __restrict__ B,
  TensileComplexDouble const alpha,
  TensileComplexDouble const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideBL,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (TensileComplexDouble)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  DEST_DATA_TYPE rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial/LVCB);


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSCB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSPB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0) ? size1J-GLOBAL_LOAD_VECTOR_WIDTH_B+0 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetB1J_0_0), (globalReadOffsetBL_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial/LVCB);


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB) + (0 + 0*LSPB)*(MT1J+PAD);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == tensile_complex<double>(0.0)) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 1); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x2


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x2_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

/* local write A */

/* local write B */
    MAC_2x2


    /* iter 7 (last unrolled loop) */

    MAC_2x2_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x2

  }



  double type_mac_tmp;


  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x2
#undef MAC_2x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideB1J






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 1792
#define LDS_OFFSET_BLK 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HBH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 256
#define LDS_NUM_ELEMENTS 819

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HBH_MT64x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 3 */

    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1536

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HBH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 3584
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HBH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    __syncthreads(); //4sync for global read


    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {


    __syncthreads(); //



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 7 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x8

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 3584
#define LDS_OFFSET_BLK 2048

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x8 micro-tile */
#define MAC_4x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \

#define MAC_4x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[2+TT1J],rC[2+2*TT0I] , rA[3+TT0I],rB[2+TT1J],rC[3+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[3+TT1J],rC[2+3*TT0I] , rA[3+TT0I],rB[3+TT1J],rC[3+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[4+TT1J],rC[2+4*TT0I] , rA[3+TT0I],rB[4+TT1J],rC[3+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[5+TT1J],rC[2+5*TT0I] , rA[3+TT0I],rB[5+TT1J],rC[3+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[6+TT1J],rC[2+6*TT0I] , rA[3+TT0I],rB[6+TT1J],rC[3+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[7+TT1J],rC[2+7*TT0I] , rA[3+TT0I],rB[7+TT1J],rC[3+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HBH_MT64x128x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR1_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_8_TLDS0_USFGRO0_VAW2_VS1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_1_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_0_1 = localWriteFirstOffsetA + (1 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_1_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_0_1;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_1_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

  /* declare loop num iterations */

  int numIterL;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 1; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* prefetch: global -> local */

  if (sizeL >= LOCAL_DEPTHU) {



    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);



    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);


    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /******************************************/
    /* End setupNewTile, isPap=False             */
    /******************************************/




  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;




    /* local write a */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

    /* local write b */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


    /* local write swap a */

    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write swap b */

    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);


    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


    __syncthreads(); //


    /* local read prefetch a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read prefetch b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);

  } // end PrefetchGlobalRead
  else { // still need to initC even if skipped prefetch

  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  }


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 1) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */




    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_1_1_0 = *(globalReadB_0_0_1_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop

/* local write B */
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop

    /* local write swap offsets a */
    localWriteOffsetA_0_0_0_0 = (localWriteOffsetA_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetA_0_0_0_1 = (localWriteOffsetA_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write swap offsets b */
    localWriteOffsetB_0_0_0_0 = (localWriteOffsetB_0_0_0_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_0_1 = (localWriteOffsetB_0_0_0_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_0 = (localWriteOffsetB_0_0_1_0 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);
    localWriteOffsetB_0_0_1_1 = (localWriteOffsetB_0_0_1_1 + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local write init pointers A */
    localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
    localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);

    /* local write init pointers B */
    localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
    localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
    localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
    localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);

    /* local read swap offsets a */
    localReadOffsetA = (localReadOffsetA + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read swap offsets b */
    localReadOffsetB = (localReadOffsetB + LDS_OFFSET_BLK)%(LDS_OFFSET_BLK*2);

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x8


    /* iter 7 */

    __syncthreads(); //

    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Ord. NoLoadLoop - Begin                  */
  /******************************************/

  if (sizeL >= LOCAL_DEPTHU) {



    /* iter 0 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 1 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 2 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 3 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 4 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8


    /* iter 5 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x8_BLK


    /* iter 6 (last unrolled loop) */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 

/* local write A */

/* local write B */
    MAC_4x8


    /* iter 7 (last unrolled loop) */

    MAC_4x8_BLK

  } // end unroll


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* local write reset offsets a */

  localWriteOffsetA_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetA_0_0_0_1 %= LDS_OFFSET_BLK;


  /* local write reset offsets b */

  localWriteOffsetB_0_0_0_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_0_1 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_0 %= LDS_OFFSET_BLK;
  localWriteOffsetB_0_0_1_1 %= LDS_OFFSET_BLK;


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 1)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_1_1_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_1);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_1_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_1 + 0) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_1_1 + 0) = b_0_1_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* local read reset offsets a */

  localReadOffsetA %= LDS_OFFSET_BLK;


  /* local read reset offsets b */

  localReadOffsetB %= LDS_OFFSET_BLK;


  /* local read init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);


  /* local read init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+1] = localReadB[1*SG1J*VECTOR_WIDTH + 1]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+1] = localReadB[2*SG1J*VECTOR_WIDTH + 1]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+1] = localReadB[3*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x8

  }





  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (1*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (2*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (3*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x8
#undef MAC_4x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 4
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (LOCAL_DEPTHU/NLCA)
#define LSPA (MT0I/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 1024

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDXL, IDX0I, IDXK) (( (IDXL)*strideAL + (IDX0I)*strideA0I + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE tensile_half
#define COMPUTE_DATA_TYPE tensile_half
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x8 micro-tile */
#define MAC_8x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[2],rB[2],rC[2+2*TT0I] , rA[3],rB[2],rC[3+2*TT0I]); \
  TYPE_MAC(rA[4],rB[2],rC[4+2*TT0I] , rA[5],rB[2],rC[5+2*TT0I]); \
  TYPE_MAC(rA[6],rB[2],rC[6+2*TT0I] , rA[7],rB[2],rC[7+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[2],rB[3],rC[2+3*TT0I] , rA[3],rB[3],rC[3+3*TT0I]); \
  TYPE_MAC(rA[4],rB[3],rC[4+3*TT0I] , rA[5],rB[3],rC[5+3*TT0I]); \
  TYPE_MAC(rA[6],rB[3],rC[6+3*TT0I] , rA[7],rB[3],rC[7+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[2],rB[4],rC[2+4*TT0I] , rA[3],rB[4],rC[3+4*TT0I]); \
  TYPE_MAC(rA[4],rB[4],rC[4+4*TT0I] , rA[5],rB[4],rC[5+4*TT0I]); \
  TYPE_MAC(rA[6],rB[4],rC[6+4*TT0I] , rA[7],rB[4],rC[7+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[2],rB[5],rC[2+5*TT0I] , rA[3],rB[5],rC[3+5*TT0I]); \
  TYPE_MAC(rA[4],rB[5],rC[4+5*TT0I] , rA[5],rB[5],rC[5+5*TT0I]); \
  TYPE_MAC(rA[6],rB[5],rC[6+5*TT0I] , rA[7],rB[5],rC[7+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[2],rB[6],rC[2+6*TT0I] , rA[3],rB[6],rC[3+6*TT0I]); \
  TYPE_MAC(rA[4],rB[6],rC[4+6*TT0I] , rA[5],rB[6],rC[5+6*TT0I]); \
  TYPE_MAC(rA[6],rB[6],rC[6+6*TT0I] , rA[7],rB[6],rC[7+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \
  TYPE_MAC(rA[2],rB[7],rC[2+7*TT0I] , rA[3],rB[7],rC[3+7*TT0I]); \
  TYPE_MAC(rA[4],rB[7],rC[4+7*TT0I] , rA[5],rB[7],rC[5+7*TT0I]); \
  TYPE_MAC(rA[6],rB[7],rC[6+7*TT0I] , rA[7],rB[7],rC[7+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideAL 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Alik_Bljk_HBH_MT128x128x4_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL0_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR0_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_8_TLDS0_USFGRO0_VAW2_VS1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideA0I,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I];
  DATA_TYPE rB[TT1J];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial/LVCA) + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSPA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSPA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSCA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_0_0;
  globalReadOffsetA0I_1_0 = (  globalReadOffsetA0I_1_0 > size0I-1) ? size0I-1 : globalReadOffsetA0I_1_0;


  /* global read addresses: shift b */

  globalReadOffsetB1J_0_0 = (  globalReadOffsetB1J_0_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_0_0;
  globalReadOffsetB1J_1_0 = (  globalReadOffsetB1J_1_0 > size1J-1) ? size1J-1 : globalReadOffsetB1J_1_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetAL_0_0), (globalReadOffsetA0I_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial/LVCA);


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 0*LSPA);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA)*(MT0I+PAD) + (0 + 1*LSPA);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;
  rC[32] = SCALAR_ZERO;
  rC[33] = SCALAR_ZERO;
  rC[34] = SCALAR_ZERO;
  rC[35] = SCALAR_ZERO;
  rC[36] = SCALAR_ZERO;
  rC[37] = SCALAR_ZERO;
  rC[38] = SCALAR_ZERO;
  rC[39] = SCALAR_ZERO;
  rC[40] = SCALAR_ZERO;
  rC[41] = SCALAR_ZERO;
  rC[42] = SCALAR_ZERO;
  rC[43] = SCALAR_ZERO;
  rC[44] = SCALAR_ZERO;
  rC[45] = SCALAR_ZERO;
  rC[46] = SCALAR_ZERO;
  rC[47] = SCALAR_ZERO;
  rC[48] = SCALAR_ZERO;
  rC[49] = SCALAR_ZERO;
  rC[50] = SCALAR_ZERO;
  rC[51] = SCALAR_ZERO;
  rC[52] = SCALAR_ZERO;
  rC[53] = SCALAR_ZERO;
  rC[54] = SCALAR_ZERO;
  rC[55] = SCALAR_ZERO;
  rC[56] = SCALAR_ZERO;
  rC[57] = SCALAR_ZERO;
  rC[58] = SCALAR_ZERO;
  rC[59] = SCALAR_ZERO;
  rC[60] = SCALAR_ZERO;
  rC[61] = SCALAR_ZERO;
  rC[62] = SCALAR_ZERO;
  rC[63] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 5); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x8


    /* iter 3 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x8




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x8
#undef strideD0I
#undef strideC0I
#undef strideAL
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 768

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGROn1_VAW2_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 7 */

    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 4
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 8
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 1
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 768

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 4x2 micro-tile */
#define MAC_4x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \

#define MAC_4x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT64x32x8_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT4_2_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE b_0_0_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 4); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_4x2_BLK


    /* iter 6 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_4x2


    /* iter 7 */

    MAC_4x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_4x2

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_4x2
#undef MAC_4x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 16
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 2
#define GLOBAL_LOAD_VECTOR_WIDTH_A 2
#define GLOBAL_LOAD_VECTOR_WIDTH_B 2
#define GLOBAL_WRITE_VECTOR_WIDTH 2

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 1

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 4096
#define LDS_NUM_ELEMENTS 4608

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 16x2 micro-tile */
#define MAC_16x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[8],rB[0],rC[8+0*TT0I] , rA[9],rB[0],rC[9+0*TT0I]); \
  TYPE_MAC(rA[10],rB[0],rC[10+0*TT0I] , rA[11],rB[0],rC[11+0*TT0I]); \
  TYPE_MAC(rA[12],rB[0],rC[12+0*TT0I] , rA[13],rB[0],rC[13+0*TT0I]); \
  TYPE_MAC(rA[14],rB[0],rC[14+0*TT0I] , rA[15],rB[0],rC[15+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[8],rB[1],rC[8+1*TT0I] , rA[9],rB[1],rC[9+1*TT0I]); \
  TYPE_MAC(rA[10],rB[1],rC[10+1*TT0I] , rA[11],rB[1],rC[11+1*TT0I]); \
  TYPE_MAC(rA[12],rB[1],rC[12+1*TT0I] , rA[13],rB[1],rC[13+1*TT0I]); \
  TYPE_MAC(rA[14],rB[1],rC[14+1*TT0I] , rA[15],rB[1],rC[15+1*TT0I]); \

#define MAC_16x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[8+TT0I],rB[0+TT1J],rC[8+0*TT0I] , rA[9+TT0I],rB[0+TT1J],rC[9+0*TT0I]); \
  TYPE_MAC(rA[10+TT0I],rB[0+TT1J],rC[10+0*TT0I] , rA[11+TT0I],rB[0+TT1J],rC[11+0*TT0I]); \
  TYPE_MAC(rA[12+TT0I],rB[0+TT1J],rC[12+0*TT0I] , rA[13+TT0I],rB[0+TT1J],rC[13+0*TT0I]); \
  TYPE_MAC(rA[14+TT0I],rB[0+TT1J],rC[14+0*TT0I] , rA[15+TT0I],rB[0+TT1J],rC[15+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[8+TT0I],rB[1+TT1J],rC[8+1*TT0I] , rA[9+TT0I],rB[1+TT1J],rC[9+1*TT0I]); \
  TYPE_MAC(rA[10+TT0I],rB[1+TT1J],rC[10+1*TT0I] , rA[11+TT0I],rB[1+TT1J],rC[11+1*TT0I]); \
  TYPE_MAC(rA[12+TT0I],rB[1+TT1J],rC[12+1*TT0I] , rA[13+TT0I],rB[1+TT1J],rC[13+1*TT0I]); \
  TYPE_MAC(rA[14+TT0I],rB[1+TT1J],rC[14+1*TT0I] , rA[15+TT0I],rB[1+TT1J],rC[15+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT256x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS2_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW2_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW2_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT16_2_TLDS0_USFGROn1_VAW1_VSn1_VW2_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO 0
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_1_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_1_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_1_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_1_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_1_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_1_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_1_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_0_1_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_1_0_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: shift a */

  globalReadOffsetA0I_0_0 = (  globalReadOffsetA0I_0_0 > size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0) ? size0I-GLOBAL_LOAD_VECTOR_WIDTH_A+0 : globalReadOffsetA0I_0_0;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_0_1 = localWriteFirstOffsetB + (1 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_0_1;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_1_0_0 = *(globalReadA_0_0_0_0 + 1);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_1_1_0 = *(globalReadA_0_0_1_0 + 1);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_1_2_0 = *(globalReadA_0_0_2_0 + 1);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_1_3_0 = *(globalReadA_0_0_3_0 + 1);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_1_4_0 = *(globalReadA_0_0_4_0 + 1);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_1_5_0 = *(globalReadA_0_0_5_0 + 1);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_1_6_0 = *(globalReadA_0_0_6_0 + 1);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_0_1_7_0 = *(globalReadA_0_0_7_0 + 1);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_1_0_0 = *(globalReadB_0_0_0_0 + 1);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_4_0 + 1) = a_0_1_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_5_0 + 1) = a_0_1_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_6_0 + 1) = a_0_1_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_0_0_7_0 + 1) = a_0_1_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_16x2


    /* iter 15 */

    MAC_16x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_1_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 1);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_1_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 1);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_1_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 1);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_1_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 1);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_1_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 1);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_1_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 1);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_1_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 1);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_0_1_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 1);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_1_0_0 = ( globalReadOffsetBL_0_0  + 1 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 1);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_0_1 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_1);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_0_0 + 1) = a_0_1_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_1_0 + 1) = a_0_1_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_2_0 + 1) = a_0_1_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_3_0 + 1) = a_0_1_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_4_0 + 1) = a_0_1_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_5_0 + 1) = a_0_1_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_6_0 + 1) = a_0_1_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_0_0_7_0 + 1) = a_0_1_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_0_1 + 0) = b_0_1_0_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[0*VECTOR_WIDTH+1] = localReadA[0*SG0I*VECTOR_WIDTH + 1]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+1] = localReadA[1*SG0I*VECTOR_WIDTH + 1]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+1] = localReadA[2*SG0I*VECTOR_WIDTH + 1]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+1] = localReadA[3*SG0I*VECTOR_WIDTH + 1]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+1] = localReadA[4*SG0I*VECTOR_WIDTH + 1]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+1] = localReadA[5*SG0I*VECTOR_WIDTH + 1]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+1] = localReadA[6*SG0I*VECTOR_WIDTH + 1]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+1] = localReadA[7*SG0I*VECTOR_WIDTH + 1]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[0*VECTOR_WIDTH+1] = localReadB[0*SG1J*VECTOR_WIDTH + 1]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_16x2

  }





  /* shift vector components d0 */

  unsigned int wgMT0I = size0I - wg0I*MT0I;
  if (wgMT0I > MT0I) wgMT0I = MT0I;
  unsigned int r0I = wgMT0I % GLOBAL_LOAD_VECTOR_WIDTH_A;
  if (r0I > 0 && ((wgMT0I/VECTOR_WIDTH) % SG0I) == (serial % SG0I) ) {
    unsigned int s0I = (wgMT0I%VECTOR_WIDTH)/GLOBAL_LOAD_VECTOR_WIDTH_A + (wgMT0I/(SG0I*VECTOR_WIDTH))*(VECTOR_WIDTH/GLOBAL_LOAD_VECTOR_WIDTH_A);
    if (r0I == 1) {
      if (s0I == 0) {
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 0*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 1) {
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 1*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 2) {
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 2*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 3) {
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 3*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 4) {
        rC[0 + 4*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 4*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 4*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 4*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 5) {
        rC[0 + 5*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 5*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 5*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 5*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else if (s0I == 6) {
        rC[0 + 6*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 6*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 6*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 6*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      } else {
        rC[0 + 7*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I] = rC[1 + 7*GLOBAL_LOAD_VECTOR_WIDTH_A + 0*TT0I];
        rC[0 + 7*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I] = rC[1 + 7*GLOBAL_LOAD_VECTOR_WIDTH_A + 1*TT0I];
      }
    }
  }



  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 0, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }
  if (flattenedGlobalC0 + 1 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I + 1, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+1 + (0*VECTOR_WIDTH+1)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_16x2
#undef MAC_16x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = 0 != (BETA) ? (ALPHA)*(REG) + (BETA)*(SRC) : (ALPHA)*(REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT128x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU1_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGROn1_VAW2_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA);


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  numIterL = sizeL / LOCAL_DEPTHU;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU) ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 8
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 8
#define NLPB 2

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 8x2 micro-tile */
#define MAC_8x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \

#define MAC_8x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT128x32x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT8_2_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_8x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_8x2


    /* iter 15 */

    MAC_8x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_8x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_8x2
#undef MAC_8x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS 256
#define SG0I 16
#define SG1J 16
#define TT0I 2
#define TT1J 8
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 1
#define NLCB 1
#define NLPA 2
#define NLPB 8

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 512
#define LDS_NUM_ELEMENTS 2560

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 2x8 micro-tile */
#define MAC_2x8\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0],rB[2],rC[0+2*TT0I] , rA[1],rB[2],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0],rB[3],rC[0+3*TT0I] , rA[1],rB[3],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0],rB[4],rC[0+4*TT0I] , rA[1],rB[4],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0],rB[5],rC[0+5*TT0I] , rA[1],rB[5],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0],rB[6],rC[0+6*TT0I] , rA[1],rB[6],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0],rB[7],rC[0+7*TT0I] , rA[1],rB[7],rC[1+7*TT0I]); \

#define MAC_2x8_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[2+TT1J],rC[0+2*TT0I] , rA[1+TT0I],rB[2+TT1J],rC[1+2*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[3+TT1J],rC[0+3*TT0I] , rA[1+TT0I],rB[3+TT1J],rC[1+3*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[4+TT1J],rC[0+4*TT0I] , rA[1+TT0I],rB[4+TT1J],rC[1+4*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[5+TT1J],rC[0+5*TT0I] , rA[1+TT0I],rB[5+TT1J],rC[1+5*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[6+TT1J],rC[0+6*TT0I] , rA[1+TT0I],rB[6+TT1J],rC[1+6*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[7+TT1J],rC[0+7*TT0I] , rA[1+TT0I],rB[7+TT1J],rC[1+7*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT32x128x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA1_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT2_8_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG16_16_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;
  DATA_TYPE b_0_0_4_0;
  DATA_TYPE b_0_0_5_0;
  DATA_TYPE b_0_0_6_0;
  DATA_TYPE b_0_0_7_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;
  unsigned int flattenedOffsetB_4_0 = globalReadOffsetB1J + 0 + 4*LSPB;
  flattenedOffsetB_4_0 = (flattenedOffsetB_4_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_4_0;
  unsigned int globalReadOffsetB1J_4_0 = flattenedOffsetB_4_0;
  unsigned int flattenedOffsetB_5_0 = globalReadOffsetB1J + 0 + 5*LSPB;
  flattenedOffsetB_5_0 = (flattenedOffsetB_5_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_5_0;
  unsigned int globalReadOffsetB1J_5_0 = flattenedOffsetB_5_0;
  unsigned int flattenedOffsetB_6_0 = globalReadOffsetB1J + 0 + 6*LSPB;
  flattenedOffsetB_6_0 = (flattenedOffsetB_6_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_6_0;
  unsigned int globalReadOffsetB1J_6_0 = flattenedOffsetB_6_0;
  unsigned int flattenedOffsetB_7_0 = globalReadOffsetB1J + 0 + 7*LSPB;
  flattenedOffsetB_7_0 = (flattenedOffsetB_7_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_7_0;
  unsigned int globalReadOffsetB1J_7_0 = flattenedOffsetB_7_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );
  int64_t globalReadOffsetB_0_0_4_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_4_0), wgK );
  int64_t globalReadOffsetB_0_0_5_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_5_0), wgK );
  int64_t globalReadOffsetB_0_0_6_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_6_0), wgK );
  int64_t globalReadOffsetB_0_0_7_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_7_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;
  DATA_TYPE const *globalReadB_0_0_4_0 = B + globalReadOffsetB_0_0_4_0;
  DATA_TYPE const *globalReadB_0_0_5_0 = B + globalReadOffsetB_0_0_5_0;
  DATA_TYPE const *globalReadB_0_0_6_0 = B + globalReadOffsetB_0_0_6_0;
  DATA_TYPE const *globalReadB_0_0_7_0 = B + globalReadOffsetB_0_0_7_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);
  unsigned int localWriteOffsetB_0_0_4_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 4*LSPB);
  unsigned int localWriteOffsetB_0_0_5_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 5*LSPB);
  unsigned int localWriteOffsetB_0_0_6_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 6*LSPB);
  unsigned int localWriteOffsetB_0_0_7_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 7*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;
  DATA_TYPE *localWriteB_0_0_4_0;
  DATA_TYPE *localWriteB_0_0_5_0;
  DATA_TYPE *localWriteB_0_0_6_0;
  DATA_TYPE *localWriteB_0_0_7_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_4_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_5_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_6_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_7_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);
    b_0_0_4_0 = *(globalReadB_0_0_4_0 + 0);
    b_0_0_5_0 = *(globalReadB_0_0_5_0 + 0);
    b_0_0_6_0 = *(globalReadB_0_0_6_0 + 0);
    b_0_0_7_0 = *(globalReadB_0_0_7_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_4_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_4_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_5_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_5_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_6_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_6_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_7_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_7_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
    *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
    *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
    *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
    *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_2x8_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0+TT1J] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0+TT1J] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0+TT1J] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0+TT1J] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0+TT1J] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0+TT1J] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_2x8


    /* iter 15 */

    MAC_2x8_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);
  b_0_0_4_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_4_0 + 0);
  b_0_0_5_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_5_0 + 0);
  b_0_0_6_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_6_0 + 0);
  b_0_0_7_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_7_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);
  localWriteB_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_4_0);
  localWriteB_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_5_0);
  localWriteB_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_6_0);
  localWriteB_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_7_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
  *(localWriteB_0_0_4_0 + 0) = b_0_0_4_0;
  *(localWriteB_0_0_5_0 + 0) = b_0_0_5_0;
  *(localWriteB_0_0_6_0 + 0) = b_0_0_6_0;
  *(localWriteB_0_0_7_0 + 0) = b_0_0_7_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 
    rB[2*VECTOR_WIDTH+0] = localReadB[2*SG1J*VECTOR_WIDTH + 0]; 
    rB[3*VECTOR_WIDTH+0] = localReadB[3*SG1J*VECTOR_WIDTH + 0]; 
    rB[4*VECTOR_WIDTH+0] = localReadB[4*SG1J*VECTOR_WIDTH + 0]; 
    rB[5*VECTOR_WIDTH+0] = localReadB[5*SG1J*VECTOR_WIDTH + 0]; 
    rB[6*VECTOR_WIDTH+0] = localReadB[6*SG1J*VECTOR_WIDTH + 0]; 
    rB[7*VECTOR_WIDTH+0] = localReadB[7*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_2x8

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 2*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 2*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (2*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 3*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 3*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (3*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 4*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 4*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (4*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 5*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 5*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (5*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 6*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 6*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (6*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 7*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 7*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (7*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_2x8
#undef MAC_2x8_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL






  /******************************************/
  /* Function Prefix                        */
  /******************************************/



/* tile parameters */
#define NUM_THREADS  64
#define SG0I 8
#define SG1J 8
#define TT0I 16
#define TT1J 2
#define MT0I (SG0I*TT0I)
#define MT1J (SG1J*TT1J)
#define VECTOR_WIDTH 1
#define GLOBAL_LOAD_VECTOR_WIDTH_A 1
#define GLOBAL_LOAD_VECTOR_WIDTH_B 1
#define GLOBAL_WRITE_VECTOR_WIDTH 1

/* DepthU parameters*/
#define CPSV (NUM_THREADS / MT0I * VECTOR_WIDTH)
#define LOCAL_SPLITU 1
#define UNROLL 16
#define LOCAL_DEPTHU (LOCAL_SPLITU*UNROLL)
#define GLOBAL_SPLITU 3

/* other */
#define PAD 0
#define WORK_GROUP_MAPPING 8

/* num loads parallel and perpendicular to coalesced */
#define NLCA 2
#define NLCB 1
#define NLPA 16
#define NLPB 4

/* load sizes parallel and perpendicular to coalesced */
#define LSCA (MT0I/NLCA)
#define LSPA (LOCAL_DEPTHU/NLPA)
#define LSCB (LOCAL_DEPTHU/NLCB)
#define LSPB (MT1J/NLPB)
#define LVCA (LSCA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVCB (LSCB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LVPA (LSPA/GLOBAL_LOAD_VECTOR_WIDTH_A)
#define LVPB (LSPB/GLOBAL_LOAD_VECTOR_WIDTH_B)
#define LDS_OFFSET_B 2048
#define LDS_NUM_ELEMENTS 2304

/* global memory indices */
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK + (gsuSumIdx)*strideW ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
#define GLOBAL_OFFSET_A(IDX0I, IDXL, IDXK) (( (IDX0I)*strideA0I + (IDXL)*strideAL + (IDXK)*strideAK ))
#define GLOBAL_OFFSET_B(IDXL, IDX1J, IDXK) (( (IDXL)*strideBL + (IDX1J)*strideB1J + (IDXK)*strideBK ))

/* data types */
#define DATA_TYPE tensile_half
#define DEST_DATA_TYPE float
#define COMPUTE_DATA_TYPE float
#define MAGIC_DIV1(dividend, magicNumber, magicShift) ((uint64_t)(dividend) * magicNumber >> magicShift)

/* MAC's */
#define MAC(A,B,DST) DST += static_cast<float>(A) * static_cast<float>(B)
#define TYPE_MAC(MULA0,MULB0,DST0,MULA1,MULB1,DST1) \
 DST0 = MAC(MULA0,MULB0,DST0);\
 DST1 = MAC(MULA1,MULB1,DST1);\

#define TYPE_MAC_WRITE(DST,SRC,ALPHA,REG,BETA) DST = (REG);

/* 16x2 micro-tile */
#define MAC_16x2\
  TYPE_MAC(rA[0],rB[0],rC[0+0*TT0I] , rA[1],rB[0],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2],rB[0],rC[2+0*TT0I] , rA[3],rB[0],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4],rB[0],rC[4+0*TT0I] , rA[5],rB[0],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6],rB[0],rC[6+0*TT0I] , rA[7],rB[0],rC[7+0*TT0I]); \
  TYPE_MAC(rA[8],rB[0],rC[8+0*TT0I] , rA[9],rB[0],rC[9+0*TT0I]); \
  TYPE_MAC(rA[10],rB[0],rC[10+0*TT0I] , rA[11],rB[0],rC[11+0*TT0I]); \
  TYPE_MAC(rA[12],rB[0],rC[12+0*TT0I] , rA[13],rB[0],rC[13+0*TT0I]); \
  TYPE_MAC(rA[14],rB[0],rC[14+0*TT0I] , rA[15],rB[0],rC[15+0*TT0I]); \
  TYPE_MAC(rA[0],rB[1],rC[0+1*TT0I] , rA[1],rB[1],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2],rB[1],rC[2+1*TT0I] , rA[3],rB[1],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4],rB[1],rC[4+1*TT0I] , rA[5],rB[1],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6],rB[1],rC[6+1*TT0I] , rA[7],rB[1],rC[7+1*TT0I]); \
  TYPE_MAC(rA[8],rB[1],rC[8+1*TT0I] , rA[9],rB[1],rC[9+1*TT0I]); \
  TYPE_MAC(rA[10],rB[1],rC[10+1*TT0I] , rA[11],rB[1],rC[11+1*TT0I]); \
  TYPE_MAC(rA[12],rB[1],rC[12+1*TT0I] , rA[13],rB[1],rC[13+1*TT0I]); \
  TYPE_MAC(rA[14],rB[1],rC[14+1*TT0I] , rA[15],rB[1],rC[15+1*TT0I]); \

#define MAC_16x2_BLK\
  TYPE_MAC(rA[0+TT0I],rB[0+TT1J],rC[0+0*TT0I] , rA[1+TT0I],rB[0+TT1J],rC[1+0*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[0+TT1J],rC[2+0*TT0I] , rA[3+TT0I],rB[0+TT1J],rC[3+0*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[0+TT1J],rC[4+0*TT0I] , rA[5+TT0I],rB[0+TT1J],rC[5+0*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[0+TT1J],rC[6+0*TT0I] , rA[7+TT0I],rB[0+TT1J],rC[7+0*TT0I]); \
  TYPE_MAC(rA[8+TT0I],rB[0+TT1J],rC[8+0*TT0I] , rA[9+TT0I],rB[0+TT1J],rC[9+0*TT0I]); \
  TYPE_MAC(rA[10+TT0I],rB[0+TT1J],rC[10+0*TT0I] , rA[11+TT0I],rB[0+TT1J],rC[11+0*TT0I]); \
  TYPE_MAC(rA[12+TT0I],rB[0+TT1J],rC[12+0*TT0I] , rA[13+TT0I],rB[0+TT1J],rC[13+0*TT0I]); \
  TYPE_MAC(rA[14+TT0I],rB[0+TT1J],rC[14+0*TT0I] , rA[15+TT0I],rB[0+TT1J],rC[15+0*TT0I]); \
  TYPE_MAC(rA[0+TT0I],rB[1+TT1J],rC[0+1*TT0I] , rA[1+TT0I],rB[1+TT1J],rC[1+1*TT0I]); \
  TYPE_MAC(rA[2+TT0I],rB[1+TT1J],rC[2+1*TT0I] , rA[3+TT0I],rB[1+TT1J],rC[3+1*TT0I]); \
  TYPE_MAC(rA[4+TT0I],rB[1+TT1J],rC[4+1*TT0I] , rA[5+TT0I],rB[1+TT1J],rC[5+1*TT0I]); \
  TYPE_MAC(rA[6+TT0I],rB[1+TT1J],rC[6+1*TT0I] , rA[7+TT0I],rB[1+TT1J],rC[7+1*TT0I]); \
  TYPE_MAC(rA[8+TT0I],rB[1+TT1J],rC[8+1*TT0I] , rA[9+TT0I],rB[1+TT1J],rC[9+1*TT0I]); \
  TYPE_MAC(rA[10+TT0I],rB[1+TT1J],rC[10+1*TT0I] , rA[11+TT0I],rB[1+TT1J],rC[11+1*TT0I]); \
  TYPE_MAC(rA[12+TT0I],rB[1+TT1J],rC[12+1*TT0I] , rA[13+TT0I],rB[1+TT1J],rC[13+1*TT0I]); \
  TYPE_MAC(rA[14+TT0I],rB[1+TT1J],rC[14+1*TT0I] , rA[15+TT0I],rB[1+TT1J],rC[15+1*TT0I]); \

/* hard-coded initial strides CD*/
/* hard-coded initial strides AB */
#define strideD0I 1
#define strideC0I 1
#define strideA0I 1
#define strideBL 1


  /******************************************/
  /* Begin Kernel                           */
  /******************************************/

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-parameter"
extern "C"
__global__ void Cijk_Ailk_Bljk_HSS_BH_MT128x16x16_SE_1LDSB0_APM1_AF0EM1_AF1EM1_AMAS0_ASAE01_ASBE01_ASCE01_ASEM1_BL1_DTL0_DVO0_EPS0_FL0_GRVW1_GSU3_ISA000_IU1_K1_KLS_LBSPP0_LPA0_LPB0_LDL1_LRVW1_MDA2_NLCA2_NLCB1_ONLL1_PBD0_PK0_PGR0_PLR1_RK0_SIA1_SU32_SUM0_SUS256_SRVW0_SVW4_SNLL0_TT16_2_TLDS0_USFGROn1_VAW1_VSn1_VW1_WSGRA0_WSGRB0_WG8_8_1_WGM8(
  float *D,
  float const * __restrict__ C,
  tensile_half const * __restrict__ A,
  tensile_half const * __restrict__ B,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const strideAL,
  unsigned int const strideAK,
  unsigned int const strideB1J,
  unsigned int const strideBK,
  unsigned int size0I,
  unsigned int size1J,
  unsigned int sizeK,
  unsigned int sizeL,
  unsigned int staggerUIterParm,
  unsigned int problemNumGroupTiles0,
  unsigned int problemNumGroupTiles1,
  unsigned int magicNumberProblemNumGroupTiles0 )
#pragma clang diagnostic pop

 {


  /******************************************/
  /* Allocate Resources                     */
  /******************************************/

  unsigned int serial = hc_get_workitem_id(0);
  unsigned int sgId = serial / (SG0I*SG1J);
#define SCALAR_ZERO (tensile_half)(0)
#define SCALAR_OOB_DATA SCALAR_ZERO
  /* registers for MAC's */
  float rC[TT0I*TT1J];
  DATA_TYPE rA[TT0I*2];
  DATA_TYPE rB[TT1J*2];

  /* registers for global->local */
  DATA_TYPE a_0_0_0_0;
  DATA_TYPE a_1_0_0_0;
  DATA_TYPE a_0_0_1_0;
  DATA_TYPE a_1_0_1_0;
  DATA_TYPE a_0_0_2_0;
  DATA_TYPE a_1_0_2_0;
  DATA_TYPE a_0_0_3_0;
  DATA_TYPE a_1_0_3_0;
  DATA_TYPE a_0_0_4_0;
  DATA_TYPE a_1_0_4_0;
  DATA_TYPE a_0_0_5_0;
  DATA_TYPE a_1_0_5_0;
  DATA_TYPE a_0_0_6_0;
  DATA_TYPE a_1_0_6_0;
  DATA_TYPE a_0_0_7_0;
  DATA_TYPE a_1_0_7_0;
  DATA_TYPE a_0_0_8_0;
  DATA_TYPE a_1_0_8_0;
  DATA_TYPE a_0_0_9_0;
  DATA_TYPE a_1_0_9_0;
  DATA_TYPE a_0_0_10_0;
  DATA_TYPE a_1_0_10_0;
  DATA_TYPE a_0_0_11_0;
  DATA_TYPE a_1_0_11_0;
  DATA_TYPE a_0_0_12_0;
  DATA_TYPE a_1_0_12_0;
  DATA_TYPE a_0_0_13_0;
  DATA_TYPE a_1_0_13_0;
  DATA_TYPE a_0_0_14_0;
  DATA_TYPE a_1_0_14_0;
  DATA_TYPE a_0_0_15_0;
  DATA_TYPE a_1_0_15_0;
  DATA_TYPE b_0_0_0_0;
  DATA_TYPE b_0_0_1_0;
  DATA_TYPE b_0_0_2_0;
  DATA_TYPE b_0_0_3_0;

  /* allocate tensile_half2 memory */
  tensile_half2 a_pk_fma;
  tensile_half2 b_pk_fma;
  tensile_half2 c_pk_fma;

  /* allocate local memory */
  __shared__ DATA_TYPE localMemory[LDS_NUM_ELEMENTS];
typedef struct MagicStruct {unsigned M; int a; int s;} MagicStruct;
const unsigned MAGIC_STRUCT_A = 0x80000000; // for extracting a-bit from shift kernarg
#define MAGIC_DIV2(dividend, magic) (((((uint64_t)(dividend) * magic.M) >> 32) + dividend*magic.a) >> magic.s)


  /******************************************/
  /* Local Read Addresses                   */
  /******************************************/


  /* local read addresses: tile assignments a */

  unsigned int lr0I = (serial % SG0I);


  /* local read addresses: tile assignments b */

  unsigned int lr1J = (serial / SG0I) % SG1J;


  /* local read addresses: final offsets a */

  unsigned int localReadOffsetA = lr0I*VECTOR_WIDTH + sgId*(MT0I+PAD);


  /* local read addresses: final offsets b */

  unsigned int localReadOffsetB = lr1J*VECTOR_WIDTH + sgId*(MT1J+PAD) + LDS_OFFSET_B;


  /* local read addresses: declare addresses a */

  DATA_TYPE *localReadA;


  /* local read addresses: declare addresses b */

  DATA_TYPE *localReadB;



  /******************************************/
  /* Begin setupNewTile, isPap=False           */
  /******************************************/


  /* global read addresses: work-group */

  unsigned int wg0I = hc_get_group_id(0);
  unsigned int wg1J = hc_get_group_id(1);
  unsigned int nwg0I = hc_get_num_groups(0);
  unsigned int nwg1J = hc_get_num_groups(1);
  nwg1J /= GLOBAL_SPLITU;
  unsigned int gsuSumIdx;
  gsuSumIdx = wg1J % GLOBAL_SPLITU;
  wg1J = wg1J / GLOBAL_SPLITU;

  uint64_t wgSerial = wg0I + (wg1J % WORK_GROUP_MAPPING) * nwg0I;// within block
  unsigned int block = wg1J / WORK_GROUP_MAPPING;
  unsigned int blockRemainder = (wg1J < nwg1J-(nwg1J % WORK_GROUP_MAPPING) ) ? 0 : nwg1J % WORK_GROUP_MAPPING;
  if ( blockRemainder == 0) {
    wg0I = wgSerial / 8;
    wg1J = wgSerial % 8 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 1) {
    wg0I = wgSerial / 1;
    wg1J = wgSerial % 1 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 2) {
    wg0I = wgSerial / 2;
    wg1J = wgSerial % 2 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 3) {
    wg0I = wgSerial / 3;
    wg1J = wgSerial % 3 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 4) {
    wg0I = wgSerial / 4;
    wg1J = wgSerial % 4 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 5) {
    wg0I = wgSerial / 5;
    wg1J = wgSerial % 5 + block*WORK_GROUP_MAPPING;
  } else if ( blockRemainder == 6) {
    wg0I = wgSerial / 6;
    wg1J = wgSerial % 6 + block*WORK_GROUP_MAPPING;
  } else {
    wg0I = wgSerial / 7;
    wg1J = wgSerial % 7 + block*WORK_GROUP_MAPPING;
  }


  /* global read addresses: tile offset assignment a */

  unsigned int globalReadOffsetA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A + (wg0I)*MT0I;


  /* global read addresses: tile offset assignment b */

  unsigned int globalReadOffsetB1J = (serial/LVCB) + (wg1J)*MT1J;


  /* global read addresses: unroll assignment a */

  unsigned int globalReadOffsetAL = (serial/LVCA) + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: unroll assignment b */

  unsigned int globalReadOffsetBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B + LOCAL_DEPTHU*gsuSumIdx;


  /* global read addresses: other free assignments */

  unsigned int wgK = ( hc_get_group_id(2) ) % sizeK;


  /* global read addresses: tile offsets a */

  unsigned int flattenedOffsetA_0_0 = globalReadOffsetA0I + 0 + 0*LSCA;
  flattenedOffsetA_0_0 = (flattenedOffsetA_0_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_0_0;
  unsigned int globalReadOffsetA0I_0_0 = flattenedOffsetA_0_0;
  unsigned int flattenedOffsetA_1_0 = globalReadOffsetA0I + 0 + 1*LSCA;
  flattenedOffsetA_1_0 = (flattenedOffsetA_1_0 > (size0I-1)) ? (size0I-1):flattenedOffsetA_1_0;
  unsigned int globalReadOffsetA0I_1_0 = flattenedOffsetA_1_0;


  /* global read addresses: tile offsets b */

  unsigned int flattenedOffsetB_0_0 = globalReadOffsetB1J + 0 + 0*LSPB;
  flattenedOffsetB_0_0 = (flattenedOffsetB_0_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_0_0;
  unsigned int globalReadOffsetB1J_0_0 = flattenedOffsetB_0_0;
  unsigned int flattenedOffsetB_1_0 = globalReadOffsetB1J + 0 + 1*LSPB;
  flattenedOffsetB_1_0 = (flattenedOffsetB_1_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_1_0;
  unsigned int globalReadOffsetB1J_1_0 = flattenedOffsetB_1_0;
  unsigned int flattenedOffsetB_2_0 = globalReadOffsetB1J + 0 + 2*LSPB;
  flattenedOffsetB_2_0 = (flattenedOffsetB_2_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_2_0;
  unsigned int globalReadOffsetB1J_2_0 = flattenedOffsetB_2_0;
  unsigned int flattenedOffsetB_3_0 = globalReadOffsetB1J + 0 + 3*LSPB;
  flattenedOffsetB_3_0 = (flattenedOffsetB_3_0 > (size1J-1)) ? (size1J-1):flattenedOffsetB_3_0;
  unsigned int globalReadOffsetB1J_3_0 = flattenedOffsetB_3_0;


  /* global read addresses: unroll offsets a */

  unsigned int globalReadOffsetAL_0_0 = globalReadOffsetAL + 0 + 0*LSPA;
  unsigned int globalReadOffsetAL_1_0 = globalReadOffsetAL + 0 + 1*LSPA;
  unsigned int globalReadOffsetAL_2_0 = globalReadOffsetAL + 0 + 2*LSPA;
  unsigned int globalReadOffsetAL_3_0 = globalReadOffsetAL + 0 + 3*LSPA;
  unsigned int globalReadOffsetAL_4_0 = globalReadOffsetAL + 0 + 4*LSPA;
  unsigned int globalReadOffsetAL_5_0 = globalReadOffsetAL + 0 + 5*LSPA;
  unsigned int globalReadOffsetAL_6_0 = globalReadOffsetAL + 0 + 6*LSPA;
  unsigned int globalReadOffsetAL_7_0 = globalReadOffsetAL + 0 + 7*LSPA;
  unsigned int globalReadOffsetAL_8_0 = globalReadOffsetAL + 0 + 8*LSPA;
  unsigned int globalReadOffsetAL_9_0 = globalReadOffsetAL + 0 + 9*LSPA;
  unsigned int globalReadOffsetAL_10_0 = globalReadOffsetAL + 0 + 10*LSPA;
  unsigned int globalReadOffsetAL_11_0 = globalReadOffsetAL + 0 + 11*LSPA;
  unsigned int globalReadOffsetAL_12_0 = globalReadOffsetAL + 0 + 12*LSPA;
  unsigned int globalReadOffsetAL_13_0 = globalReadOffsetAL + 0 + 13*LSPA;
  unsigned int globalReadOffsetAL_14_0 = globalReadOffsetAL + 0 + 14*LSPA;
  unsigned int globalReadOffsetAL_15_0 = globalReadOffsetAL + 0 + 15*LSPA;


  /* global read addresses: unroll offsets b */

  unsigned int globalReadOffsetBL_0_0 = globalReadOffsetBL + 0 + 0*LSCB;


  /* global read addresses: final offsets a */

  int64_t globalReadOffsetA_0_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_1_0_0_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_1_0), (globalReadOffsetAL_0_0), wgK );
  int64_t globalReadOffsetA_0_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_1_0_1_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_1_0), (globalReadOffsetAL_1_0), wgK );
  int64_t globalReadOffsetA_0_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_1_0_2_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_1_0), (globalReadOffsetAL_2_0), wgK );
  int64_t globalReadOffsetA_0_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_1_0_3_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_1_0), (globalReadOffsetAL_3_0), wgK );
  int64_t globalReadOffsetA_0_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_1_0_4_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_1_0), (globalReadOffsetAL_4_0), wgK );
  int64_t globalReadOffsetA_0_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_1_0_5_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_1_0), (globalReadOffsetAL_5_0), wgK );
  int64_t globalReadOffsetA_0_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_1_0_6_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_1_0), (globalReadOffsetAL_6_0), wgK );
  int64_t globalReadOffsetA_0_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_7_0), wgK );
  int64_t globalReadOffsetA_1_0_7_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_1_0), (globalReadOffsetAL_7_0), wgK );
  int64_t globalReadOffsetA_0_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_8_0), wgK );
  int64_t globalReadOffsetA_1_0_8_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_1_0), (globalReadOffsetAL_8_0), wgK );
  int64_t globalReadOffsetA_0_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_9_0), wgK );
  int64_t globalReadOffsetA_1_0_9_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_1_0), (globalReadOffsetAL_9_0), wgK );
  int64_t globalReadOffsetA_0_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_10_0), wgK );
  int64_t globalReadOffsetA_1_0_10_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_1_0), (globalReadOffsetAL_10_0), wgK );
  int64_t globalReadOffsetA_0_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_11_0), wgK );
  int64_t globalReadOffsetA_1_0_11_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_1_0), (globalReadOffsetAL_11_0), wgK );
  int64_t globalReadOffsetA_0_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_12_0), wgK );
  int64_t globalReadOffsetA_1_0_12_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_1_0), (globalReadOffsetAL_12_0), wgK );
  int64_t globalReadOffsetA_0_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_13_0), wgK );
  int64_t globalReadOffsetA_1_0_13_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_1_0), (globalReadOffsetAL_13_0), wgK );
  int64_t globalReadOffsetA_0_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_14_0), wgK );
  int64_t globalReadOffsetA_1_0_14_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_1_0), (globalReadOffsetAL_14_0), wgK );
  int64_t globalReadOffsetA_0_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_0_0), (globalReadOffsetAL_15_0), wgK );
  int64_t globalReadOffsetA_1_0_15_0 = GLOBAL_OFFSET_A( (globalReadOffsetA0I_1_0), (globalReadOffsetAL_15_0), wgK );


  /* global read addresses: final offsets b */

  int64_t globalReadOffsetB_0_0_0_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_0_0), wgK );
  int64_t globalReadOffsetB_0_0_1_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_1_0), wgK );
  int64_t globalReadOffsetB_0_0_2_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_2_0), wgK );
  int64_t globalReadOffsetB_0_0_3_0 = GLOBAL_OFFSET_B( (globalReadOffsetBL_0_0), (globalReadOffsetB1J_3_0), wgK );


  /* global read addresses: addresses a */

  DATA_TYPE const *globalReadA_0_0_0_0 = A + globalReadOffsetA_0_0_0_0;
  DATA_TYPE const *globalReadA_1_0_0_0 = A + globalReadOffsetA_1_0_0_0;
  DATA_TYPE const *globalReadA_0_0_1_0 = A + globalReadOffsetA_0_0_1_0;
  DATA_TYPE const *globalReadA_1_0_1_0 = A + globalReadOffsetA_1_0_1_0;
  DATA_TYPE const *globalReadA_0_0_2_0 = A + globalReadOffsetA_0_0_2_0;
  DATA_TYPE const *globalReadA_1_0_2_0 = A + globalReadOffsetA_1_0_2_0;
  DATA_TYPE const *globalReadA_0_0_3_0 = A + globalReadOffsetA_0_0_3_0;
  DATA_TYPE const *globalReadA_1_0_3_0 = A + globalReadOffsetA_1_0_3_0;
  DATA_TYPE const *globalReadA_0_0_4_0 = A + globalReadOffsetA_0_0_4_0;
  DATA_TYPE const *globalReadA_1_0_4_0 = A + globalReadOffsetA_1_0_4_0;
  DATA_TYPE const *globalReadA_0_0_5_0 = A + globalReadOffsetA_0_0_5_0;
  DATA_TYPE const *globalReadA_1_0_5_0 = A + globalReadOffsetA_1_0_5_0;
  DATA_TYPE const *globalReadA_0_0_6_0 = A + globalReadOffsetA_0_0_6_0;
  DATA_TYPE const *globalReadA_1_0_6_0 = A + globalReadOffsetA_1_0_6_0;
  DATA_TYPE const *globalReadA_0_0_7_0 = A + globalReadOffsetA_0_0_7_0;
  DATA_TYPE const *globalReadA_1_0_7_0 = A + globalReadOffsetA_1_0_7_0;
  DATA_TYPE const *globalReadA_0_0_8_0 = A + globalReadOffsetA_0_0_8_0;
  DATA_TYPE const *globalReadA_1_0_8_0 = A + globalReadOffsetA_1_0_8_0;
  DATA_TYPE const *globalReadA_0_0_9_0 = A + globalReadOffsetA_0_0_9_0;
  DATA_TYPE const *globalReadA_1_0_9_0 = A + globalReadOffsetA_1_0_9_0;
  DATA_TYPE const *globalReadA_0_0_10_0 = A + globalReadOffsetA_0_0_10_0;
  DATA_TYPE const *globalReadA_1_0_10_0 = A + globalReadOffsetA_1_0_10_0;
  DATA_TYPE const *globalReadA_0_0_11_0 = A + globalReadOffsetA_0_0_11_0;
  DATA_TYPE const *globalReadA_1_0_11_0 = A + globalReadOffsetA_1_0_11_0;
  DATA_TYPE const *globalReadA_0_0_12_0 = A + globalReadOffsetA_0_0_12_0;
  DATA_TYPE const *globalReadA_1_0_12_0 = A + globalReadOffsetA_1_0_12_0;
  DATA_TYPE const *globalReadA_0_0_13_0 = A + globalReadOffsetA_0_0_13_0;
  DATA_TYPE const *globalReadA_1_0_13_0 = A + globalReadOffsetA_1_0_13_0;
  DATA_TYPE const *globalReadA_0_0_14_0 = A + globalReadOffsetA_0_0_14_0;
  DATA_TYPE const *globalReadA_1_0_14_0 = A + globalReadOffsetA_1_0_14_0;
  DATA_TYPE const *globalReadA_0_0_15_0 = A + globalReadOffsetA_0_0_15_0;
  DATA_TYPE const *globalReadA_1_0_15_0 = A + globalReadOffsetA_1_0_15_0;


  /* global read addresses: addresses b */

  DATA_TYPE const *globalReadB_0_0_0_0 = B + globalReadOffsetB_0_0_0_0;
  DATA_TYPE const *globalReadB_0_0_1_0 = B + globalReadOffsetB_0_0_1_0;
  DATA_TYPE const *globalReadB_0_0_2_0 = B + globalReadOffsetB_0_0_2_0;
  DATA_TYPE const *globalReadB_0_0_3_0 = B + globalReadOffsetB_0_0_3_0;


  /* global read addresses: increments a */

  int64_t globalReadIncAL = (int64_t)strideAL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /* global read addresses: increments b */

  int64_t globalReadIncBL = (int64_t)strideBL*LOCAL_DEPTHU*GLOBAL_SPLITU;


  /******************************************/
  /* Local Write Addresses                  */
  /******************************************/


  /* local write addresses: tile assignment A */
  unsigned int lwA0I = (serial%LVCA)*GLOBAL_LOAD_VECTOR_WIDTH_A;


  /* local write addresses: tile assignment B */
  unsigned int lwB1J = (serial/LVCB);


  /* local write addresses: unroll assignment A */
  unsigned int lwAL = (serial/LVCA);


  /* local write addresses: unroll assignment B */
  unsigned int lwBL = (serial%LVCB)*GLOBAL_LOAD_VECTOR_WIDTH_B;


  /* local write addresses: first offset a */

  unsigned int localWriteFirstOffsetA = lwA0I + lwAL*(MT0I+PAD);


  /* local write addresses: first offset b */

  unsigned int localWriteFirstOffsetB = lwB1J + lwBL*(MT1J+PAD) + LDS_OFFSET_B;


  /* local write addresses: final offsets A */
  unsigned int localWriteOffsetA_0_0_0_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_1_0_0_0 = localWriteFirstOffsetA + (0 + 1*LSCA) + (0 + 0*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_1_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_1_0_1_0 = localWriteFirstOffsetA + (0 + 1*LSCA) + (0 + 1*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_2_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_1_0_2_0 = localWriteFirstOffsetA + (0 + 1*LSCA) + (0 + 2*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_3_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_1_0_3_0 = localWriteFirstOffsetA + (0 + 1*LSCA) + (0 + 3*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_4_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_1_0_4_0 = localWriteFirstOffsetA + (0 + 1*LSCA) + (0 + 4*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_5_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_1_0_5_0 = localWriteFirstOffsetA + (0 + 1*LSCA) + (0 + 5*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_6_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_1_0_6_0 = localWriteFirstOffsetA + (0 + 1*LSCA) + (0 + 6*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_7_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_1_0_7_0 = localWriteFirstOffsetA + (0 + 1*LSCA) + (0 + 7*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_8_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 8*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_1_0_8_0 = localWriteFirstOffsetA + (0 + 1*LSCA) + (0 + 8*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_9_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 9*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_1_0_9_0 = localWriteFirstOffsetA + (0 + 1*LSCA) + (0 + 9*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_10_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 10*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_1_0_10_0 = localWriteFirstOffsetA + (0 + 1*LSCA) + (0 + 10*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_11_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 11*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_1_0_11_0 = localWriteFirstOffsetA + (0 + 1*LSCA) + (0 + 11*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_12_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 12*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_1_0_12_0 = localWriteFirstOffsetA + (0 + 1*LSCA) + (0 + 12*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_13_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 13*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_1_0_13_0 = localWriteFirstOffsetA + (0 + 1*LSCA) + (0 + 13*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_14_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 14*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_1_0_14_0 = localWriteFirstOffsetA + (0 + 1*LSCA) + (0 + 14*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_0_0_15_0 = localWriteFirstOffsetA + (0 + 0*LSCA) + (0 + 15*LSPA)*(MT0I+PAD);
  unsigned int localWriteOffsetA_1_0_15_0 = localWriteFirstOffsetA + (0 + 1*LSCA) + (0 + 15*LSPA)*(MT0I+PAD);


  /* local write addresses: final offsets B */
  unsigned int localWriteOffsetB_0_0_0_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 0*LSPB);
  unsigned int localWriteOffsetB_0_0_1_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 1*LSPB);
  unsigned int localWriteOffsetB_0_0_2_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 2*LSPB);
  unsigned int localWriteOffsetB_0_0_3_0 = localWriteFirstOffsetB + (0 + 0*LSCB)*(MT1J+PAD) + (0 + 3*LSPB);


  /* local write addresses: declare addresses A */
  DATA_TYPE *localWriteA_0_0_0_0;
  DATA_TYPE *localWriteA_1_0_0_0;
  DATA_TYPE *localWriteA_0_0_1_0;
  DATA_TYPE *localWriteA_1_0_1_0;
  DATA_TYPE *localWriteA_0_0_2_0;
  DATA_TYPE *localWriteA_1_0_2_0;
  DATA_TYPE *localWriteA_0_0_3_0;
  DATA_TYPE *localWriteA_1_0_3_0;
  DATA_TYPE *localWriteA_0_0_4_0;
  DATA_TYPE *localWriteA_1_0_4_0;
  DATA_TYPE *localWriteA_0_0_5_0;
  DATA_TYPE *localWriteA_1_0_5_0;
  DATA_TYPE *localWriteA_0_0_6_0;
  DATA_TYPE *localWriteA_1_0_6_0;
  DATA_TYPE *localWriteA_0_0_7_0;
  DATA_TYPE *localWriteA_1_0_7_0;
  DATA_TYPE *localWriteA_0_0_8_0;
  DATA_TYPE *localWriteA_1_0_8_0;
  DATA_TYPE *localWriteA_0_0_9_0;
  DATA_TYPE *localWriteA_1_0_9_0;
  DATA_TYPE *localWriteA_0_0_10_0;
  DATA_TYPE *localWriteA_1_0_10_0;
  DATA_TYPE *localWriteA_0_0_11_0;
  DATA_TYPE *localWriteA_1_0_11_0;
  DATA_TYPE *localWriteA_0_0_12_0;
  DATA_TYPE *localWriteA_1_0_12_0;
  DATA_TYPE *localWriteA_0_0_13_0;
  DATA_TYPE *localWriteA_1_0_13_0;
  DATA_TYPE *localWriteA_0_0_14_0;
  DATA_TYPE *localWriteA_1_0_14_0;
  DATA_TYPE *localWriteA_0_0_15_0;
  DATA_TYPE *localWriteA_1_0_15_0;


  /* local write addresses: declare addresses B */
  DATA_TYPE *localWriteB_0_0_0_0;
  DATA_TYPE *localWriteB_0_0_1_0;
  DATA_TYPE *localWriteB_0_0_2_0;
  DATA_TYPE *localWriteB_0_0_3_0;


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_1_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_1_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_1_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_1_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_1_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_1_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_1_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_1_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_1_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_1_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_1_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_1_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_1_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_1_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_1_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);
  localWriteA_1_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);

  /* declare loop num iterations */

  int numIterL;


  rC[0] = SCALAR_ZERO;
  rC[1] = SCALAR_ZERO;
  rC[2] = SCALAR_ZERO;
  rC[3] = SCALAR_ZERO;
  rC[4] = SCALAR_ZERO;
  rC[5] = SCALAR_ZERO;
  rC[6] = SCALAR_ZERO;
  rC[7] = SCALAR_ZERO;
  rC[8] = SCALAR_ZERO;
  rC[9] = SCALAR_ZERO;
  rC[10] = SCALAR_ZERO;
  rC[11] = SCALAR_ZERO;
  rC[12] = SCALAR_ZERO;
  rC[13] = SCALAR_ZERO;
  rC[14] = SCALAR_ZERO;
  rC[15] = SCALAR_ZERO;
  rC[16] = SCALAR_ZERO;
  rC[17] = SCALAR_ZERO;
  rC[18] = SCALAR_ZERO;
  rC[19] = SCALAR_ZERO;
  rC[20] = SCALAR_ZERO;
  rC[21] = SCALAR_ZERO;
  rC[22] = SCALAR_ZERO;
  rC[23] = SCALAR_ZERO;
  rC[24] = SCALAR_ZERO;
  rC[25] = SCALAR_ZERO;
  rC[26] = SCALAR_ZERO;
  rC[27] = SCALAR_ZERO;
  rC[28] = SCALAR_ZERO;
  rC[29] = SCALAR_ZERO;
  rC[30] = SCALAR_ZERO;
  rC[31] = SCALAR_ZERO;


  /* Compute summation loop num iter */
  if(alpha == 0.0) sizeL = 0;  // Short circuit check alpha=0, skip A*B 
  unsigned int numIterMyWg = (sizeL / LOCAL_DEPTHU) / GLOBAL_SPLITU;
  unsigned int numIterPerWgRemainder = (sizeL / LOCAL_DEPTHU) % GLOBAL_SPLITU;
  if (gsuSumIdx < numIterPerWgRemainder) {
    numIterMyWg ++;
  }
  numIterL = numIterMyWg;

  const unsigned origNumIter = numIterL;
  unsigned staggerUIter = (wg0I & staggerUIterParm);
  staggerUIter = (staggerUIter << 3); // shift so each stagger has 256-byte stride

  globalReadA_0_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_1_0_0_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_1_0_1_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_1_0_2_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_1_0_3_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_1_0_4_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_1_0_5_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_1_0_6_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_1_0_7_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_8_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_1_0_8_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_9_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_1_0_9_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_10_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_1_0_10_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_11_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_1_0_11_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_12_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_1_0_12_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_13_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_1_0_13_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_14_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_1_0_14_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_0_0_15_0 += (staggerUIter * globalReadIncAL); // apply stagger offset
  globalReadA_1_0_15_0 += (staggerUIter * globalReadIncAL); // apply stagger offset


  globalReadB_0_0_0_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_1_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_2_0 += (staggerUIter * globalReadIncBL); // apply stagger offset
  globalReadB_0_0_3_0 += (staggerUIter * globalReadIncBL); // apply stagger offset

  staggerUIter += 0; // add PrefetchGlobalRead

  /* local read addresses: init pointers a */

  localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

  /* local read addresses: init pointers b */

  localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);


  /******************************************/
  /* End setupNewTile, isPap=False             */
  /******************************************/


  /******************************************/
  /* Unrolled Loop(s) - Begin               */
  /******************************************/

  while (numIterL-- > 0) {


    /******************************************/
    /* Unrolled Loop 1/1 - Begin              */
    /******************************************/



    /* Begin Each Unroll: Check VGPR.checkin for INT8 LW */


    /* global read A */
    a_0_0_0_0 = *(globalReadA_0_0_0_0 + 0);
    a_1_0_0_0 = *(globalReadA_1_0_0_0 + 0);
    a_0_0_1_0 = *(globalReadA_0_0_1_0 + 0);
    a_1_0_1_0 = *(globalReadA_1_0_1_0 + 0);
    a_0_0_2_0 = *(globalReadA_0_0_2_0 + 0);
    a_1_0_2_0 = *(globalReadA_1_0_2_0 + 0);
    a_0_0_3_0 = *(globalReadA_0_0_3_0 + 0);
    a_1_0_3_0 = *(globalReadA_1_0_3_0 + 0);
    a_0_0_4_0 = *(globalReadA_0_0_4_0 + 0);
    a_1_0_4_0 = *(globalReadA_1_0_4_0 + 0);
    a_0_0_5_0 = *(globalReadA_0_0_5_0 + 0);
    a_1_0_5_0 = *(globalReadA_1_0_5_0 + 0);
    a_0_0_6_0 = *(globalReadA_0_0_6_0 + 0);
    a_1_0_6_0 = *(globalReadA_1_0_6_0 + 0);
    a_0_0_7_0 = *(globalReadA_0_0_7_0 + 0);
    a_1_0_7_0 = *(globalReadA_1_0_7_0 + 0);
    a_0_0_8_0 = *(globalReadA_0_0_8_0 + 0);
    a_1_0_8_0 = *(globalReadA_1_0_8_0 + 0);
    a_0_0_9_0 = *(globalReadA_0_0_9_0 + 0);
    a_1_0_9_0 = *(globalReadA_1_0_9_0 + 0);
    a_0_0_10_0 = *(globalReadA_0_0_10_0 + 0);
    a_1_0_10_0 = *(globalReadA_1_0_10_0 + 0);
    a_0_0_11_0 = *(globalReadA_0_0_11_0 + 0);
    a_1_0_11_0 = *(globalReadA_1_0_11_0 + 0);
    a_0_0_12_0 = *(globalReadA_0_0_12_0 + 0);
    a_1_0_12_0 = *(globalReadA_1_0_12_0 + 0);
    a_0_0_13_0 = *(globalReadA_0_0_13_0 + 0);
    a_1_0_13_0 = *(globalReadA_1_0_13_0 + 0);
    a_0_0_14_0 = *(globalReadA_0_0_14_0 + 0);
    a_1_0_14_0 = *(globalReadA_1_0_14_0 + 0);
    a_0_0_15_0 = *(globalReadA_0_0_15_0 + 0);
    a_1_0_15_0 = *(globalReadA_1_0_15_0 + 0);

    /* global read B */
    b_0_0_0_0 = *(globalReadB_0_0_0_0 + 0);
    b_0_0_1_0 = *(globalReadB_0_0_1_0 + 0);
    b_0_0_2_0 = *(globalReadB_0_0_2_0 + 0);
    b_0_0_3_0 = *(globalReadB_0_0_3_0 + 0);

    /* global read inc A for sumL */
    globalReadA_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_1_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_1_0_0_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_1_0_0_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_1_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_1_0_1_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_1_0_1_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_1_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_1_0_2_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_1_0_2_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_1_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_1_0_3_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_1_0_3_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_1_0_4_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_1_0_4_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_1_0_4_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_1_0_5_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_1_0_5_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_1_0_5_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_1_0_6_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_1_0_6_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_1_0_6_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_1_0_7_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_1_0_7_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_1_0_7_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_8_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_8_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_1_0_8_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_1_0_8_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_1_0_8_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_9_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_9_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_1_0_9_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_1_0_9_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_1_0_9_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_10_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_10_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_1_0_10_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_1_0_10_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_1_0_10_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_11_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_11_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_1_0_11_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_1_0_11_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_1_0_11_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_12_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_12_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_1_0_12_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_1_0_12_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_1_0_12_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_13_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_13_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_1_0_13_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_1_0_13_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_1_0_13_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_14_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_14_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_1_0_14_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_1_0_14_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_1_0_14_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_0_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_0_0_15_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_0_0_15_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }
    globalReadA_1_0_15_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadA_1_0_15_0) + 1*globalReadIncAL);
    if ((numIterL) == staggerUIter) {
      globalReadA_1_0_15_0 -= (origNumIter * globalReadIncAL); // wrap staggered offset back to row start
    }

    /* global read inc B for sumL */
    globalReadB_0_0_0_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_0_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_0_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_1_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_1_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_1_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_2_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_2_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_2_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }
    globalReadB_0_0_3_0 = (DATA_TYPE const *)( ((DATA_TYPE const *)globalReadB_0_0_3_0) + 1*globalReadIncBL);
    if ((numIterL) == staggerUIter) {
      globalReadB_0_0_3_0 -= (origNumIter * globalReadIncBL); // wrap staggered offset back to row start
    }


    __syncthreads(); //PGR=0, prior iter done reading lds


    /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
    *(localWriteA_1_0_0_0 + 0) = a_1_0_0_0;
    *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
    *(localWriteA_1_0_1_0 + 0) = a_1_0_1_0;
    *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
    *(localWriteA_1_0_2_0 + 0) = a_1_0_2_0;
    *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
    *(localWriteA_1_0_3_0 + 0) = a_1_0_3_0;
    *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
    *(localWriteA_1_0_4_0 + 0) = a_1_0_4_0;
    *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
    *(localWriteA_1_0_5_0 + 0) = a_1_0_5_0;
    *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
    *(localWriteA_1_0_6_0 + 0) = a_1_0_6_0;
    *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
    *(localWriteA_1_0_7_0 + 0) = a_1_0_7_0;
    *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
    *(localWriteA_1_0_8_0 + 0) = a_1_0_8_0;
    *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
    *(localWriteA_1_0_9_0 + 0) = a_1_0_9_0;
    *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
    *(localWriteA_1_0_10_0 + 0) = a_1_0_10_0;
    *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
    *(localWriteA_1_0_11_0 + 0) = a_1_0_11_0;
    *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
    *(localWriteA_1_0_12_0 + 0) = a_1_0_12_0;
    *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
    *(localWriteA_1_0_13_0 + 0) = a_1_0_13_0;
    *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
    *(localWriteA_1_0_14_0 + 0) = a_1_0_14_0;
    *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
    *(localWriteA_1_0_15_0 + 0) = a_1_0_15_0;
#pragma clang diagnostic pop


    /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
    *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
    *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
    *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
    *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


    __syncthreads(); //


    /* prefetch local a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 


    /* prefetch local b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);




    /* iter 0 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0+TT0I] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0+TT0I] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0+TT0I] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0+TT0I] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0+TT0I] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0+TT0I] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0+TT0I] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0+TT0I] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 1 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 2 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0+TT0I] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0+TT0I] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0+TT0I] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0+TT0I] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0+TT0I] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0+TT0I] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0+TT0I] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0+TT0I] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 3 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 4 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0+TT0I] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0+TT0I] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0+TT0I] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0+TT0I] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0+TT0I] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0+TT0I] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0+TT0I] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0+TT0I] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 5 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 6 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0+TT0I] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0+TT0I] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0+TT0I] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0+TT0I] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0+TT0I] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0+TT0I] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0+TT0I] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0+TT0I] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 7 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 8 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0+TT0I] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0+TT0I] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0+TT0I] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0+TT0I] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0+TT0I] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0+TT0I] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0+TT0I] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0+TT0I] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 9 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 10 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0+TT0I] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0+TT0I] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0+TT0I] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0+TT0I] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0+TT0I] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0+TT0I] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0+TT0I] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0+TT0I] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 11 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 12 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0+TT0I] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0+TT0I] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0+TT0I] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0+TT0I] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0+TT0I] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0+TT0I] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0+TT0I] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0+TT0I] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2


    /* iter 13 */


    /* local read a */
    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read increment a */
    localReadA += LOCAL_SPLITU*(MT0I+PAD);

    /* local read increment b */
    localReadB += LOCAL_SPLITU*(MT1J+PAD);
    MAC_16x2_BLK


    /* iter 14 (reset local read pointers iteration)  (swap and reset local write pointers iteration)  (swap local read pointers iteration)  */


    /* local read a */
    rA[0*VECTOR_WIDTH+0+TT0I] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0+TT0I] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0+TT0I] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0+TT0I] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0+TT0I] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0+TT0I] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0+TT0I] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0+TT0I] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0+TT0I] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0+TT0I] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0+TT0I] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0+TT0I] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0+TT0I] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0+TT0I] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0+TT0I] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0+TT0I] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 

    /* local read b */
    rB[0*VECTOR_WIDTH+0+TT1J] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0+TT1J] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 

    /* local read init pointers a */
    localReadA = (DATA_TYPE *)(localMemory + localReadOffsetA);

    /* local read init pointers b */
    localReadB = (DATA_TYPE *)(localMemory + localReadOffsetB);
    MAC_16x2


    /* iter 15 */

    MAC_16x2_BLK




    /******************************************/
    /* Unrolled Loop - End                    */
    /******************************************/

  }


  /* Before NLL: Check VGPR.checkin for INT8 LW */


  /******************************************/
  /* Tail Loop                              */
  /******************************************/


  /* Compute tail loop num iter */
  numIterL = (((sizeL % LOCAL_DEPTHU) + LOCAL_SPLITU - 1) / LOCAL_SPLITU);
  if (gsuSumIdx != numIterPerWgRemainder) {
    numIterL = 0;
  }


  /* remove stagger offsets for tail loop */

  globalReadA_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_1_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_1_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_1_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_1_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_1_0_4_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_1_0_5_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_1_0_6_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_1_0_7_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_1_0_8_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_1_0_9_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_1_0_10_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_1_0_11_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_1_0_12_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_1_0_13_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_1_0_14_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_0_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset
  globalReadA_1_0_15_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncAL); // remove stagger offset

  globalReadB_0_0_0_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_1_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_2_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset
  globalReadB_0_0_3_0 += ((origNumIter - (staggerUIter - 0)) * globalReadIncBL); // remove stagger offset


  /* Update M0 for DTLDS */



  /* global read a */


  /* global read A */
  a_0_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_0_0 + 0);
  a_1_0_0_0 = ( globalReadOffsetAL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_1_0_0_0 + 0);
  a_0_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_1_0 + 0);
  a_1_0_1_0 = ( globalReadOffsetAL_1_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_1_0_1_0 + 0);
  a_0_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_2_0 + 0);
  a_1_0_2_0 = ( globalReadOffsetAL_2_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_1_0_2_0 + 0);
  a_0_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_3_0 + 0);
  a_1_0_3_0 = ( globalReadOffsetAL_3_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_1_0_3_0 + 0);
  a_0_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_4_0 + 0);
  a_1_0_4_0 = ( globalReadOffsetAL_4_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_1_0_4_0 + 0);
  a_0_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_5_0 + 0);
  a_1_0_5_0 = ( globalReadOffsetAL_5_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_1_0_5_0 + 0);
  a_0_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_6_0 + 0);
  a_1_0_6_0 = ( globalReadOffsetAL_6_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_1_0_6_0 + 0);
  a_0_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_7_0 + 0);
  a_1_0_7_0 = ( globalReadOffsetAL_7_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_1_0_7_0 + 0);
  a_0_0_8_0 = ( globalReadOffsetAL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_8_0 + 0);
  a_1_0_8_0 = ( globalReadOffsetAL_8_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_1_0_8_0 + 0);
  a_0_0_9_0 = ( globalReadOffsetAL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_9_0 + 0);
  a_1_0_9_0 = ( globalReadOffsetAL_9_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_1_0_9_0 + 0);
  a_0_0_10_0 = ( globalReadOffsetAL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_10_0 + 0);
  a_1_0_10_0 = ( globalReadOffsetAL_10_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_1_0_10_0 + 0);
  a_0_0_11_0 = ( globalReadOffsetAL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_11_0 + 0);
  a_1_0_11_0 = ( globalReadOffsetAL_11_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_1_0_11_0 + 0);
  a_0_0_12_0 = ( globalReadOffsetAL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_12_0 + 0);
  a_1_0_12_0 = ( globalReadOffsetAL_12_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_1_0_12_0 + 0);
  a_0_0_13_0 = ( globalReadOffsetAL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_13_0 + 0);
  a_1_0_13_0 = ( globalReadOffsetAL_13_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_1_0_13_0 + 0);
  a_0_0_14_0 = ( globalReadOffsetAL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_14_0 + 0);
  a_1_0_14_0 = ( globalReadOffsetAL_14_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_1_0_14_0 + 0);
  a_0_0_15_0 = ( globalReadOffsetAL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_0_0_15_0 + 0);
  a_1_0_15_0 = ( globalReadOffsetAL_15_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadA_1_0_15_0 + 0);


  /* Update M0 for DTLDS */



  /* global read b */


  /* global read B */
  b_0_0_0_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_0_0 + 0);
  b_0_0_1_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_1_0 + 0);
  b_0_0_2_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_2_0 + 0);
  b_0_0_3_0 = ( globalReadOffsetBL_0_0  + 0 >= (sizeL % LOCAL_DEPTHU + LOCAL_DEPTHU*gsuSumIdx) || !numIterL ) ? SCALAR_OOB_DATA : *(globalReadB_0_0_3_0 + 0);


  __syncthreads(); //


  /* local write init pointers A */
  localWriteA_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_0_0);
  localWriteA_1_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_0_0);
  localWriteA_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_1_0);
  localWriteA_1_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_1_0);
  localWriteA_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_2_0);
  localWriteA_1_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_2_0);
  localWriteA_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_3_0);
  localWriteA_1_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_3_0);
  localWriteA_0_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_4_0);
  localWriteA_1_0_4_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_4_0);
  localWriteA_0_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_5_0);
  localWriteA_1_0_5_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_5_0);
  localWriteA_0_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_6_0);
  localWriteA_1_0_6_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_6_0);
  localWriteA_0_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_7_0);
  localWriteA_1_0_7_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_7_0);
  localWriteA_0_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_8_0);
  localWriteA_1_0_8_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_8_0);
  localWriteA_0_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_9_0);
  localWriteA_1_0_9_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_9_0);
  localWriteA_0_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_10_0);
  localWriteA_1_0_10_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_10_0);
  localWriteA_0_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_11_0);
  localWriteA_1_0_11_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_11_0);
  localWriteA_0_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_12_0);
  localWriteA_1_0_12_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_12_0);
  localWriteA_0_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_13_0);
  localWriteA_1_0_13_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_13_0);
  localWriteA_0_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_14_0);
  localWriteA_1_0_14_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_14_0);
  localWriteA_0_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_0_0_15_0);
  localWriteA_1_0_15_0 = (DATA_TYPE *)(localMemory + localWriteOffsetA_1_0_15_0);


  /* local write init pointers B */
  localWriteB_0_0_0_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_0_0);
  localWriteB_0_0_1_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_1_0);
  localWriteB_0_0_2_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_2_0);
  localWriteB_0_0_3_0 = (DATA_TYPE *)(localMemory + localWriteOffsetB_0_0_3_0);


  /* local write a */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteA_0_0_0_0 + 0) = a_0_0_0_0;
  *(localWriteA_1_0_0_0 + 0) = a_1_0_0_0;
  *(localWriteA_0_0_1_0 + 0) = a_0_0_1_0;
  *(localWriteA_1_0_1_0 + 0) = a_1_0_1_0;
  *(localWriteA_0_0_2_0 + 0) = a_0_0_2_0;
  *(localWriteA_1_0_2_0 + 0) = a_1_0_2_0;
  *(localWriteA_0_0_3_0 + 0) = a_0_0_3_0;
  *(localWriteA_1_0_3_0 + 0) = a_1_0_3_0;
  *(localWriteA_0_0_4_0 + 0) = a_0_0_4_0;
  *(localWriteA_1_0_4_0 + 0) = a_1_0_4_0;
  *(localWriteA_0_0_5_0 + 0) = a_0_0_5_0;
  *(localWriteA_1_0_5_0 + 0) = a_1_0_5_0;
  *(localWriteA_0_0_6_0 + 0) = a_0_0_6_0;
  *(localWriteA_1_0_6_0 + 0) = a_1_0_6_0;
  *(localWriteA_0_0_7_0 + 0) = a_0_0_7_0;
  *(localWriteA_1_0_7_0 + 0) = a_1_0_7_0;
  *(localWriteA_0_0_8_0 + 0) = a_0_0_8_0;
  *(localWriteA_1_0_8_0 + 0) = a_1_0_8_0;
  *(localWriteA_0_0_9_0 + 0) = a_0_0_9_0;
  *(localWriteA_1_0_9_0 + 0) = a_1_0_9_0;
  *(localWriteA_0_0_10_0 + 0) = a_0_0_10_0;
  *(localWriteA_1_0_10_0 + 0) = a_1_0_10_0;
  *(localWriteA_0_0_11_0 + 0) = a_0_0_11_0;
  *(localWriteA_1_0_11_0 + 0) = a_1_0_11_0;
  *(localWriteA_0_0_12_0 + 0) = a_0_0_12_0;
  *(localWriteA_1_0_12_0 + 0) = a_1_0_12_0;
  *(localWriteA_0_0_13_0 + 0) = a_0_0_13_0;
  *(localWriteA_1_0_13_0 + 0) = a_1_0_13_0;
  *(localWriteA_0_0_14_0 + 0) = a_0_0_14_0;
  *(localWriteA_1_0_14_0 + 0) = a_1_0_14_0;
  *(localWriteA_0_0_15_0 + 0) = a_0_0_15_0;
  *(localWriteA_1_0_15_0 + 0) = a_1_0_15_0;
#pragma clang diagnostic pop


  /* local write b */

#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wconditional-uninitialized"
  *(localWriteB_0_0_0_0 + 0) = b_0_0_0_0;
  *(localWriteB_0_0_1_0 + 0) = b_0_0_1_0;
  *(localWriteB_0_0_2_0 + 0) = b_0_0_2_0;
  *(localWriteB_0_0_3_0 + 0) = b_0_0_3_0;
#pragma clang diagnostic pop


  /* Recalc local read offsets */



  __syncthreads(); //


  /* TailLoop: Check VGPR.checkin for INT8 LW */


  /* tail loop: macs */

  while (numIterL-- > 0) {


    /* local read a */

    rA[0*VECTOR_WIDTH+0] = localReadA[0*SG0I*VECTOR_WIDTH + 0]; 
    rA[1*VECTOR_WIDTH+0] = localReadA[1*SG0I*VECTOR_WIDTH + 0]; 
    rA[2*VECTOR_WIDTH+0] = localReadA[2*SG0I*VECTOR_WIDTH + 0]; 
    rA[3*VECTOR_WIDTH+0] = localReadA[3*SG0I*VECTOR_WIDTH + 0]; 
    rA[4*VECTOR_WIDTH+0] = localReadA[4*SG0I*VECTOR_WIDTH + 0]; 
    rA[5*VECTOR_WIDTH+0] = localReadA[5*SG0I*VECTOR_WIDTH + 0]; 
    rA[6*VECTOR_WIDTH+0] = localReadA[6*SG0I*VECTOR_WIDTH + 0]; 
    rA[7*VECTOR_WIDTH+0] = localReadA[7*SG0I*VECTOR_WIDTH + 0]; 
    rA[8*VECTOR_WIDTH+0] = localReadA[8*SG0I*VECTOR_WIDTH + 0]; 
    rA[9*VECTOR_WIDTH+0] = localReadA[9*SG0I*VECTOR_WIDTH + 0]; 
    rA[10*VECTOR_WIDTH+0] = localReadA[10*SG0I*VECTOR_WIDTH + 0]; 
    rA[11*VECTOR_WIDTH+0] = localReadA[11*SG0I*VECTOR_WIDTH + 0]; 
    rA[12*VECTOR_WIDTH+0] = localReadA[12*SG0I*VECTOR_WIDTH + 0]; 
    rA[13*VECTOR_WIDTH+0] = localReadA[13*SG0I*VECTOR_WIDTH + 0]; 
    rA[14*VECTOR_WIDTH+0] = localReadA[14*SG0I*VECTOR_WIDTH + 0]; 
    rA[15*VECTOR_WIDTH+0] = localReadA[15*SG0I*VECTOR_WIDTH + 0]; 


    /* local read b */

    rB[0*VECTOR_WIDTH+0] = localReadB[0*SG1J*VECTOR_WIDTH + 0]; 
    rB[1*VECTOR_WIDTH+0] = localReadB[1*SG1J*VECTOR_WIDTH + 0]; 


    /* local read inc a */

    localReadA += LOCAL_SPLITU*(MT0I+PAD);


    /* local read inc b */

    localReadB += LOCAL_SPLITU*(MT1J+PAD);


    MAC_16x2

  }






  /* not-LocalSplitU: global write indices */

  unsigned int flattenedGlobalC0 = (wg0I)*MT0I + (serial % SG0I)*VECTOR_WIDTH;
  unsigned int flattenedGlobalC1 = (wg1J)*MT1J + (serial / SG0I)*VECTOR_WIDTH;
  unsigned int globalC0I = flattenedGlobalC0;
  unsigned int globalC1J = flattenedGlobalC1;
  unsigned int globalCK = (wgK);
  uint64_t strideW = 1 + (size0I - 1)  + (size1J - 1) * 1 * size0I + (sizeK - 1) * 1 * size0I * size1J;


  /* not-LocalSplitU: global write */


  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  8*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 8*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[8*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  9*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 9*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[9*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  10*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 10*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[10*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  11*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 11*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[11*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  12*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 12*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[12*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  13*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 13*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[13*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  14*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 14*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[14*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  15*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 0*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 15*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 0*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[15*VECTOR_WIDTH+0 + (0*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  0*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 0*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[0*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  1*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 1*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[1*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  2*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 2*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[2*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  3*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 3*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[3*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  4*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 4*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[4*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  5*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 5*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[5*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  6*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 6*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[6*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  7*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 7*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[7*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  8*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 8*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[8*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  9*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 9*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[9*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  10*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 10*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[10*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  11*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 11*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[11*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  12*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 12*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[12*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  13*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 13*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[13*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  14*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 14*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[14*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }

  /* new vw0 offset - inc and extract tensor dims */
  globalC0I =   flattenedGlobalC0 +  15*SG0I*VECTOR_WIDTH;
  /* new vw1 offset - inc and extract tensor dims */
  globalC1J =   flattenedGlobalC1 + 0 + 1*SG1J*VECTOR_WIDTH;
  if (flattenedGlobalC0 + 15*SG0I*VECTOR_WIDTH < size0I) {  if (flattenedGlobalC1 + 1*SG1J*VECTOR_WIDTH < size1J) {  TYPE_MAC_WRITE( D[ GLOBAL_D( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], C[ GLOBAL_C( (uint64_t) globalC0I, (uint64_t) globalC1J, (uint64_t) globalCK) ], alpha, rC[15*VECTOR_WIDTH+0 + (1*VECTOR_WIDTH+0)*TT0I], beta) } }


}

#undef UNROLL
#undef LOCAL_SPLITU
#undef LOCAL_DEPTHU
#undef SG0I
#undef SG1J
#undef TT0I
#undef TT1J
#undef MT0I
#undef MT1J
#undef NLCA
#undef NLCB
#undef NLPA
#undef NLPB
#undef LSCA
#undef LSPA
#undef LSCB
#undef LSPB
#undef GLOBAL_D
#undef GLOBAL_C
#undef GLOBAL_OFFSET_A
#undef GLOBAL_OFFSET_B
#undef DATA_TYPE
#undef DEST_DATA_TYPE
#undef COMPUTE_DATA_TYPE
#undef LDS_OFFSET_B
#undef LDS_OFFSET_BLK
#undef LDS_NUM_ELEMENTS
#undef NUM_THREADS
#undef WORK_GROUP_MAPPING
#undef VECTOR_WIDTH
#undef GLOBAL_LOAD_VECTOR_WIDTH_A
#undef GLOBAL_LOAD_VECTOR_WIDTH_B
#undef GLOBAL_WRITE_VECTOR_WIDTH
#undef MAC
#undef TYPE_MAC
#undef TYPE_MAC_WRITE
#undef GLOBAL_SPLITU
#undef SCALAR_ZERO
#undef SCALAR_OOB_DATA
#undef MAC_16x2
#undef MAC_16x2_BLK
#undef strideD0I
#undef strideC0I
#undef strideA0I
#undef strideBL





extern "C"
__global__ void Cijk_S(
  float *D,
  float const * __restrict__ C,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const size0I,
  unsigned int const size1J,
  unsigned int const sizeK,
  float const beta)
{
/* hard-coded initial strides */
#define strideD0I 1
#define strideC0I 1
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
  uint64_t id = hc_get_workitem_absolute_id(0);
  if (id >= (size0I*size1J*sizeK))
    return;
  uint64_t id0, id1, id2;
  id0 = id % size0I;
  id  = id / size0I;
  id1 = id % size1J;
  id  = id / size1J;
  id2 = id % sizeK;
  id  = id / sizeK;

  uint64_t idxD = GLOBAL_D( (uint64_t)id0, id1, id2);
  uint64_t idxC = GLOBAL_C( (uint64_t)id0, id1, id2);
#define SCALAR_ZERO ((float)(0))
  if(beta == SCALAR_ZERO) {
    D[idxD] = SCALAR_ZERO;
  } else {
    D[idxD] = ((float)(C[idxC])) * beta;
  }
}
#undef strideD0I
#undef strideC0I
#undef GLOBAL_D
#undef GLOBAL_C
#undef SCALAR_ZERO

extern "C"
__global__ void Cijk_S_GA(
  float *D,
  float const * __restrict__ C,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const size0I,
  unsigned int const size1J,
  unsigned int const sizeK,
  float const beta)
{
/* hard-coded initial strides */
#define strideD0I 1
#define strideC0I 1
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
  uint64_t id = hc_get_workitem_absolute_id(0);
  if (id >= (size0I*size1J*sizeK))
    return;
  uint64_t id0, id1, id2;
  id0 = id % size0I;
  id  = id / size0I;
  id1 = id % size1J;
  id  = id / size1J;
  id2 = id % sizeK;
  id  = id / sizeK;

  uint64_t idxD = GLOBAL_D( (uint64_t)id0, id1, id2);
  uint64_t idxC = GLOBAL_C( (uint64_t)id0, id1, id2);
#define SCALAR_ZERO ((float)(0))
  if(beta == SCALAR_ZERO) {
    D[idxD] = SCALAR_ZERO;
  } else {
    D[idxD] = ((float)(C[idxC])) * beta;
  }
}
#undef strideD0I
#undef strideC0I
#undef GLOBAL_D
#undef GLOBAL_C
#undef SCALAR_ZERO

extern "C"
__global__ void Cijk_S_PostGSU(
  float * D,
  float * W,
  float const * __restrict__ C,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideW1J,
  unsigned int const strideWK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const size0I,
  unsigned int const size1J,
  unsigned int const sizeK,
  unsigned int const gsu)
{
/* hard-coded initial strides */
#define strideD0I 1
#define strideW0I 1
#define strideC0I 1
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_W(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideW0I + (IDX1J)*strideW1J + (IDXK)*strideWK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
  uint64_t id = hc_get_workitem_absolute_id(0);
  if (id >= (size0I*size1J*sizeK))
    return;
  uint64_t id0, id1, id2;
  id0 = id % size0I;
  id  = id / size0I;
  id1 = id % size1J;
  id  = id / size1J;
  id2 = id % sizeK;
  id  = id / sizeK;

  uint64_t idxD = GLOBAL_D( (uint64_t)id0, id1, id2);
  uint64_t idxW = GLOBAL_W( (uint64_t)id0, id1, id2);
  uint64_t idxC = GLOBAL_C( (uint64_t)id0, id1, id2);
  uint64_t strideW = 1 + (size0I - 1) * strideW0I + (size1J - 1) * strideW1J + (sizeK - 1) * strideWK;
  float accum = 0.0f;
  for (int i=0; i<gsu; i++) {
    accum += W[idxW];
    idxW  += strideW;
  }
  if( beta == (float)0)
    accum = ((float)alpha) * accum;
  else
    accum = (((float)alpha) * accum + ((float)beta) * ((float)C[idxC]));
  D[idxD] = (float)accum;
}
#undef strideD0I
#undef strideW0I
#undef strideC0I
#undef GLOBAL_D
#undef GLOBAL_W
#undef GLOBAL_C

extern "C"
__global__ void Cijk_H_GA(
  float *D,
  tensile_half const * __restrict__ C,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const size0I,
  unsigned int const size1J,
  unsigned int const sizeK,
  tensile_half const beta)
{
/* hard-coded initial strides */
#define strideD0I 1
#define strideC0I 1
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
  uint64_t id = hc_get_workitem_absolute_id(0);
  if (id >= (size0I*size1J*sizeK))
    return;
  uint64_t id0, id1, id2;
  id0 = id % size0I;
  id  = id / size0I;
  id1 = id % size1J;
  id  = id / size1J;
  id2 = id % sizeK;
  id  = id / sizeK;

  uint64_t idxD = GLOBAL_D( (uint64_t)id0, id1, id2);
  uint64_t idxC = GLOBAL_C( (uint64_t)id0, id1, id2);
#define SCALAR_ZERO ((float)(0))
  if(beta == SCALAR_ZERO) {
    D[idxD] = SCALAR_ZERO;
  } else {
    D[idxD] = ((tensile_half)(C[idxC])) * beta;
  }
}
#undef strideD0I
#undef strideC0I
#undef GLOBAL_D
#undef GLOBAL_C
#undef SCALAR_ZERO

extern "C"
__global__ void Cijk_H_PostGSU(
  tensile_half * D,
  float * W,
  tensile_half const * __restrict__ C,
  tensile_half const alpha,
  tensile_half const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideW1J,
  unsigned int const strideWK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const size0I,
  unsigned int const size1J,
  unsigned int const sizeK,
  unsigned int const gsu)
{
/* hard-coded initial strides */
#define strideD0I 1
#define strideW0I 1
#define strideC0I 1
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_W(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideW0I + (IDX1J)*strideW1J + (IDXK)*strideWK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
  uint64_t id = hc_get_workitem_absolute_id(0);
  if (id >= (size0I*size1J*sizeK))
    return;
  uint64_t id0, id1, id2;
  id0 = id % size0I;
  id  = id / size0I;
  id1 = id % size1J;
  id  = id / size1J;
  id2 = id % sizeK;
  id  = id / sizeK;

  uint64_t idxD = GLOBAL_D( (uint64_t)id0, id1, id2);
  uint64_t idxW = GLOBAL_W( (uint64_t)id0, id1, id2);
  uint64_t idxC = GLOBAL_C( (uint64_t)id0, id1, id2);
  uint64_t strideW = 1 + (size0I - 1) * strideW0I + (size1J - 1) * strideW1J + (sizeK - 1) * strideWK;
  float accum = 0.0f;
  for (int i=0; i<gsu; i++) {
    accum += W[idxW];
    idxW  += strideW;
  }
  if( beta == (tensile_half)0)
    accum = ((float)alpha) * accum;
  else
    accum = (((float)alpha) * accum + ((float)beta) * ((float)C[idxC]));
  D[idxD] = (tensile_half)accum;
}
#undef strideD0I
#undef strideW0I
#undef strideC0I
#undef GLOBAL_D
#undef GLOBAL_W
#undef GLOBAL_C

extern "C"
__global__ void Cijk_I(
  int32_t *D,
  int32_t const * __restrict__ C,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const size0I,
  unsigned int const size1J,
  unsigned int const sizeK,
  int32_t const beta)
{
/* hard-coded initial strides */
#define strideD0I 1
#define strideC0I 1
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
  uint64_t id = hc_get_workitem_absolute_id(0);
  if (id >= (size0I*size1J*sizeK))
    return;
  uint64_t id0, id1, id2;
  id0 = id % size0I;
  id  = id / size0I;
  id1 = id % size1J;
  id  = id / size1J;
  id2 = id % sizeK;
  id  = id / sizeK;

  uint64_t idxD = GLOBAL_D( (uint64_t)id0, id1, id2);
  uint64_t idxC = GLOBAL_C( (uint64_t)id0, id1, id2);
#define SCALAR_ZERO ((uint32_t)(0))
  if(beta == SCALAR_ZERO) {
    D[idxD] = SCALAR_ZERO;
  } else {
    D[idxD] = ((int32_t)(C[idxC])) * beta;
  }
}
#undef strideD0I
#undef strideC0I
#undef GLOBAL_D
#undef GLOBAL_C
#undef SCALAR_ZERO

extern "C"
__global__ void Cijk_H(
  tensile_half *D,
  tensile_half const * __restrict__ C,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const size0I,
  unsigned int const size1J,
  unsigned int const sizeK,
  tensile_half const beta)
{
/* hard-coded initial strides */
#define strideD0I 1
#define strideC0I 1
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
  uint64_t id = hc_get_workitem_absolute_id(0);
  if (id >= (size0I*size1J*sizeK))
    return;
  uint64_t id0, id1, id2;
  id0 = id % size0I;
  id  = id / size0I;
  id1 = id % size1J;
  id  = id / size1J;
  id2 = id % sizeK;
  id  = id / sizeK;

  uint64_t idxD = GLOBAL_D( (uint64_t)id0, id1, id2);
  uint64_t idxC = GLOBAL_C( (uint64_t)id0, id1, id2);
#define SCALAR_ZERO ((tensile_half)(0))
  if(beta == SCALAR_ZERO) {
    D[idxD] = SCALAR_ZERO;
  } else {
    D[idxD] = ((tensile_half)(C[idxC])) * beta;
  }
}
#undef strideD0I
#undef strideC0I
#undef GLOBAL_D
#undef GLOBAL_C
#undef SCALAR_ZERO

extern "C"
__global__ void Cijk_B_GA(
  float *D,
  tensile_bfloat16 const * __restrict__ C,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const size0I,
  unsigned int const size1J,
  unsigned int const sizeK,
  float const beta)
{
/* hard-coded initial strides */
#define strideD0I 1
#define strideC0I 1
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
  uint64_t id = hc_get_workitem_absolute_id(0);
  if (id >= (size0I*size1J*sizeK))
    return;
  uint64_t id0, id1, id2;
  id0 = id % size0I;
  id  = id / size0I;
  id1 = id % size1J;
  id  = id / size1J;
  id2 = id % sizeK;
  id  = id / sizeK;

  uint64_t idxD = GLOBAL_D( (uint64_t)id0, id1, id2);
  uint64_t idxC = GLOBAL_C( (uint64_t)id0, id1, id2);
#define SCALAR_ZERO ((float)(0))
  if(beta == SCALAR_ZERO) {
    D[idxD] = SCALAR_ZERO;
  } else {
    D[idxD] = ((float)(C[idxC])) * beta;
  }
}
#undef strideD0I
#undef strideC0I
#undef GLOBAL_D
#undef GLOBAL_C
#undef SCALAR_ZERO

extern "C"
__global__ void Cijk_B_PostGSU(
  tensile_bfloat16 * D,
  float * W,
  tensile_bfloat16 const * __restrict__ C,
  float const alpha,
  float const beta,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideW1J,
  unsigned int const strideWK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const size0I,
  unsigned int const size1J,
  unsigned int const sizeK,
  unsigned int const gsu)
{
/* hard-coded initial strides */
#define strideD0I 1
#define strideW0I 1
#define strideC0I 1
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_W(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideW0I + (IDX1J)*strideW1J + (IDXK)*strideWK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
  uint64_t id = hc_get_workitem_absolute_id(0);
  if (id >= (size0I*size1J*sizeK))
    return;
  uint64_t id0, id1, id2;
  id0 = id % size0I;
  id  = id / size0I;
  id1 = id % size1J;
  id  = id / size1J;
  id2 = id % sizeK;
  id  = id / sizeK;

  uint64_t idxD = GLOBAL_D( (uint64_t)id0, id1, id2);
  uint64_t idxW = GLOBAL_W( (uint64_t)id0, id1, id2);
  uint64_t idxC = GLOBAL_C( (uint64_t)id0, id1, id2);
  uint64_t strideW = 1 + (size0I - 1) * strideW0I + (size1J - 1) * strideW1J + (sizeK - 1) * strideWK;
  float accum = 0.0f;
  for (int i=0; i<gsu; i++) {
    accum += W[idxW];
    idxW  += strideW;
  }
  if( beta == (float)0)
    accum = ((float)alpha) * accum;
  else
    accum = (((float)alpha) * accum + ((float)beta) * ((float)C[idxC]));
  D[idxD] = (tensile_bfloat16)accum;
}
#undef strideD0I
#undef strideW0I
#undef strideC0I
#undef GLOBAL_D
#undef GLOBAL_W
#undef GLOBAL_C

extern "C"
__global__ void Cijk_D(
  double *D,
  double const * __restrict__ C,
  unsigned int const strideD1J,
  unsigned int const strideDK,
  unsigned int const strideC1J,
  unsigned int const strideCK,
  unsigned int const size0I,
  unsigned int const size1J,
  unsigned int const sizeK,
  double const beta)
{
/* hard-coded initial strides */
#define strideD0I 1
#define strideC0I 1
#define GLOBAL_D(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideD0I + (IDX1J)*strideD1J + (IDXK)*strideDK ))
#define GLOBAL_C(IDX0I, IDX1J, IDXK) (( (IDX0I)*strideC0I + (IDX1J)*strideC1J + (IDXK)*strideCK ))
  uint64_t id = hc_get_workitem_absolute_id(0);
  if (id >= (size0I*size1J*sizeK))
    return;
  uint64_t id0, id1, id2;
  id0 = id % size0I;
  id  = id / size0I;
  id1 = id % size1J;
  id  = id / size1J;
  id2 = id % sizeK;
  id  = id / sizeK;

  uint64_t idxD = GLOBAL_D( (uint64_t)id0, id1, id2);
  uint64_t idxC = GLOBAL_C( (uint64_t)id0, id1, id2);
#define SCALAR_ZERO ((double)(0))
  if(beta == SCALAR_ZERO) {
    D[idxD] = SCALAR_ZERO;
  } else {
    D[idxD] = ((double)(C[idxC])) * beta;
  }
}
#undef strideD0I
#undef strideC0I
#undef GLOBAL_D
#undef GLOBAL_C
#undef SCALAR_ZERO
